(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.salaxy = global.salaxy || {}, global.salaxy.core = {})));
}(this, (function (exports) { 'use strict';

    /**
     * Class for reading Environment specific configuration for Salaxy API's and JavaScript in general
     */
    class Configs {
        static getGlobalConfig() {
            const g = this.getGlobal();
            return (g.salaxy && g.salaxy.config) ? g.salaxy.config : null;
        }
        static setGlobalConfig(config) {
            const g = this.getGlobal();
            g.salaxy = g.salaxy || {};
            g.salaxy.config = config;
        }
        static getGlobal() {
            if (typeof globalThis !== "undefined") {
                return globalThis;
            }
            if (typeof self !== "undefined") {
                return self;
            }
            if (typeof window !== "undefined") {
                return window;
            }
            if (typeof global !== "undefined") {
                return global;
            }
            throw new Error("unable to locate global object");
        }
        /**
         * Returns current configuration.
         */
        static get current() {
            return this.getGlobalConfig();
        }
        /**
         * Set current configuration.
         */
        static set current(config) {
            this.setGlobalConfig(config);
        }
        /** Get global, globalThis object */
        static get global() {
            return this.getGlobal();
        }
    }

    /**
     * Very simple cookie implementation - at the moment for token storage only.
     * Expand if needed for other use cases (see todo section in the docs).
     *
     * @todo Expand this class as necessary e.g. something like this: https://developer.mozilla.org/en-US/docs/Web/API/Document/cookie/Simple_document.cookie_framework
     * or this https://github.com/Booyanach/cookie-wrapper ... which unfortunately cannot be used at the moment, because they are GPL license.
     */
    class Cookies {
        /**
         * Gets a cookie value by a key
         * @param key - CName / key to look for.
         */
        get(key) {
            const cookie = document.cookie;
            const cookieParts = cookie.split(";");
            for (const item of cookieParts) {
                const cookiePart = decodeURIComponent(item).trim();
                if (cookiePart.indexOf(key + "=") === 0) {
                    return cookiePart.substring(key.length + 1);
                }
            }
            return null;
        }
        /**
         * Sets a cookie value for the specified CName
         * @param cname - CName for the cookie value. Name is uri encoded as a fallback, but generally, you should make sure this is a valid cname.
         * @param value - Value to set for the cookie. Value is uri encoded.
         * @param expirationDays - Days until the cookie expires.
         * Set this to null or 0 to not set the expiration date at all.
         * This defaults the behavior where cookie is deleted when browser is closed.
         */
        setCookie(cname, value, expirationDays) {
            cname = encodeURIComponent(cname);
            value = encodeURIComponent(value);
            if (expirationDays && expirationDays > 0) {
                const d = new Date();
                d.setTime(d.getTime() + (expirationDays * 24 * 60 * 60 * 1000));
                const expires = "expires=" + d.toUTCString();
                document.cookie = cname + "=" + value + ";" + expires + ";path=/";
            }
            else {
                document.cookie = cname + "=" + value + ";path=/";
            }
        }
    }

    /**
     * Provides wrapper methods for communicating with the Palkkaus.fi API.
     * The raw Ajax-access to the server methods: GET, POST and DELETE
     * with different return types and authentication / error events.
     * This is the JQuery based ajax implementation.
     */
    class AjaxJQuery {
        /**
         * Creates a new instance of AjaxJQuery
         */
        constructor() {
            /**
             * By default (true) the token is set to salaxy-token -cookie.
             * Disable cookies with this flag.
             */
            this.useCookie = true;
            /**
             * By default credentials are not used in http-calls.
             * Enable credentials with this flag.
             */
            this.useCredentials = false;
            /**
             * The server address - root of the server. This is settable field.
             * Will probably be changed to a configuration object in the final version.
             */
            this.serverAddress = "https://test-api.salaxy.com";
            const config = Configs.current;
            if (config) {
                // apiServer
                if (config.apiServer) {
                    this.serverAddress = config.apiServer;
                }
                // useCredentials
                if (config.useCredentials != null) {
                    this.useCredentials = config.useCredentials;
                }
                // useCookie
                if (config.useCookie != null) {
                    this.useCookie = config.useCookie;
                }
            }
        }
        /** Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api' */
        getApiAddress() {
            return this.serverAddress + "/v02/api";
        }
        /** Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com' */
        getServerAddress() {
            return this.serverAddress;
        }
        /**
         * Gets a JSON-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        getJSON(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                dataType: "json",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Gets a HTML-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result html. Standard Promise rejection to be used for error handling.
         */
        getHTML(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                dataType: "html",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * POSTS data to server and receives back a JSON-message.
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        postJSON(method, data) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getUrl(method),
                data,
                dataType: "json",
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * POSTS data to server and receives back HTML.
         *
         * @param method - The API method is the url starting from api version, e.g. '/v02/api'. E.g. '/calculator/new'
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        postHTML(method, data) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getUrl(method),
                data,
                dataType: "html",
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Sends a DELETE-message to server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        remove(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "DELETE",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Gets the current token.
         * Will check the salaxy-token cookie if the token is persisted there
         */
        getCurrentToken() {
            if (!this.token && this.useCookie) {
                this.token = new Cookies().get("salaxy-token") || "";
            }
            return this.token;
        }
        /**
         * Sets the current token. The token is set to cookie called "salaxy-token" or
         * if the HTML page is running from local computer, it is set to local storage.
         *
         * @param token - the authentication token to store.
         */
        setCurrentToken(token) {
            if (this.useCookie) {
                new Cookies().setCookie("salaxy-token", token || "");
            }
            this.token = token;
        }
        /**
         * Implements the OAuth2 "Resource Owner Password Credentials Grant" flow (RFC6749 4.3).
         * This method is not typically used in production web sites - use Implicit Grant instead for client side JavScript (SPAs).
         * However, it is very useful in development, testing, trusted helpers and server-side scenarios.
         */
        resourceOwnerLogin(username, password) {
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getServerAddress() + "/oauth2/token",
                data: {
                    grant_type: "password",
                    username,
                    password,
                },
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                success: (data, textStatus, request) => {
                    this.setCurrentToken(data.access_token);
                },
                dataType: "json",
            });
        }
        getJQuery() {
            const jQuery = Configs.global.jQuery;
            if (jQuery) {
                return jQuery;
            }
            else {
                throw new Error("ERROR: No jQuery. AjaxJQuery requires jQuery to be referenced.");
            }
        }
        /** If missing, append the API server address to the given url method string */
        getUrl(method) {
            if (!method || method.trim() === "") {
                return null;
            }
            if (method.toLowerCase().startsWith("http")) {
                return method;
            }
            if (method.toLowerCase().startsWith("/v")) {
                return this.getServerAddress() + method;
            }
            return this.getApiAddress() + method;
        }
    }

    /**
     * Provides wrapper methods for communicating with the Palkkaus.fi API.
     * The raw Ajax-access to the server methods: GET, POST and DELETE
     * with different return types and authentication / error events.
     * This is the XMLHttpRequest based Ajax implementation.
     */
    class AjaxXHR {
        /**
         * Creates a new instance of AjaxXHR
         */
        constructor() {
            /**
             * By default (true) the token is set to salaxy-token -cookie.
             * Disable cookies with this flag.
             */
            this.useCookie = true;
            /**
             * The server address - root of the server. This is settable field.
             * Will probably be changed to a configuration object in the final version.
             */
            this.serverAddress = "https://test-api.salaxy.com";
            const config = Configs.current;
            if (config) {
                // apiServer
                if (config.apiServer) {
                    this.serverAddress = config.apiServer;
                }
                // useCredentials
                if (config.useCredentials != null) {
                    this.useCredentials = config.useCredentials;
                }
                // useCookie
                if (config.useCookie != null) {
                    this.useCookie = config.useCookie;
                }
            }
        }
        /** Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api' */
        getApiAddress() {
            return this.serverAddress + "/v02/api";
        }
        /** Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com' */
        getServerAddress() {
            return this.serverAddress;
        }
        /**
         * Gets a JSON-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        getJSON(method) {
            return new Promise((resolve, reject) => {
                const xhr = this.createCORSRequest("GET", this.getUrl(method));
                if (!xhr) {
                    return reject(new Error("CORS not supported!"));
                }
                xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
                xhr.setRequestHeader("Accept", "application/json; charset=UTF-8");
                // xhr.setRequestHeader("Cache-Control", "max-age=0");
                const token = this.getCurrentToken();
                if (token) {
                    xhr.setRequestHeader("Authorization", "Bearer " + token);
                }
                xhr.withCredentials = (token) ? false : this.useCredentials;
                xhr.onreadystatechange = () => {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            try {
                                const response = JSON.parse(xhr.responseText);
                                return resolve(response);
                            }
                            catch (error) {
                                return reject(error);
                            }
                        }
                        else {
                            return reject(new Error("There was a problem with the request."));
                        }
                    }
                };
                xhr.onerror = () => {
                    return reject(new Error("There was a problem with the request."));
                };
                xhr.send();
            });
        }
        /**
         * Gets a HTML-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result html. Standard Promise rejection to be used for error handling.
         */
        getHTML(method) {
            return new Promise((resolve, reject) => {
                const xhr = this.createCORSRequest("GET", this.getUrl(method));
                if (!xhr) {
                    return reject(new Error("CORS not supported!"));
                }
                xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
                xhr.setRequestHeader("Accept", "application/json; charset=UTF-8");
                // xhr.setRequestHeader("Cache-Control", "max-age=0");
                const token = this.getCurrentToken();
                if (token) {
                    xhr.setRequestHeader("Authorization", "Bearer " + token);
                }
                xhr.withCredentials = (token) ? false : this.useCredentials;
                xhr.onreadystatechange = () => {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            return resolve(xhr.responseText);
                        }
                        else {
                            return reject(new Error("There was a problem with the request."));
                        }
                    }
                };
                xhr.onerror = () => {
                    return reject(new Error("There was a problem with the request."));
                };
                xhr.send();
            });
        }
        /**
         * POSTS data to server and receives back a JSON-message.
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result. Standard Promise rejection to be used for error handling.
         */
        postJSON(method, data) {
            return new Promise((resolve, reject) => {
                const xhr = this.createCORSRequest("POST", this.getUrl(method));
                if (!xhr) {
                    return reject(new Error("CORS not supported!"));
                }
                xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
                xhr.setRequestHeader("Accept", "application/json; charset=UTF-8");
                // xhr.setRequestHeader("Cache-Control", "max-age=0");
                const token = this.getCurrentToken();
                if (token) {
                    xhr.setRequestHeader("Authorization", "Bearer " + token);
                }
                xhr.withCredentials = (token) ? false : this.useCredentials;
                xhr.onreadystatechange = () => {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            try {
                                const response = JSON.parse(xhr.responseText);
                                return resolve(response);
                            }
                            catch (error) {
                                return reject(error);
                            }
                        }
                        else {
                            return reject(new Error("There was a problem with the request."));
                        }
                    }
                };
                xhr.onerror = () => {
                    return reject(new Error("There was a problem with the request."));
                };
                const params = (!data || typeof data === "string") ? data
                    : Object.keys(data).map((k) => encodeURIComponent(k) + "=" + encodeURIComponent(data[k])).join("&");
                xhr.send(params);
            });
        }
        /**
         * POSTS data to server and receives back HTML.
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result. Standard Promise rejection to be used for error handling.
         */
        postHTML(method, data) {
            return new Promise((resolve, reject) => {
                const xhr = this.createCORSRequest("POST", this.getUrl(method));
                if (!xhr) {
                    return reject(new Error("CORS not supported!"));
                }
                xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
                xhr.setRequestHeader("Accept", "application/json; charset=UTF-8");
                // xhr.setRequestHeader("Cache-Control", "max-age=0");
                const token = this.getCurrentToken();
                if (token) {
                    xhr.setRequestHeader("Authorization", "Bearer " + token);
                }
                xhr.withCredentials = (token) ? false : this.useCredentials;
                xhr.onreadystatechange = () => {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            return resolve(xhr.responseText);
                        }
                        else {
                            return reject(new Error("There was a problem with the request."));
                        }
                    }
                };
                xhr.onerror = () => {
                    return reject(new Error("There was a problem with the request."));
                };
                const params = (!data || typeof data === "string") ? data
                    : Object.keys(data).map((k) => encodeURIComponent(k) + "=" + encodeURIComponent(data[k])).join("&");
                xhr.send(params);
            });
        }
        /**
         * Sends a DELETE-message to server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result. Standard Promise rejection to be used for error handling.
         */
        remove(method) {
            return new Promise((resolve, reject) => {
                const xhr = this.createCORSRequest("DELETE", this.getUrl(method));
                if (!xhr) {
                    return reject(new Error("CORS not supported!"));
                }
                xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
                xhr.setRequestHeader("Accept", "application/json; charset=UTF-8");
                // xhr.setRequestHeader("Cache-Control", "max-age=0");
                const token = this.getCurrentToken();
                if (token) {
                    xhr.setRequestHeader("Authorization", "Bearer " + token);
                }
                xhr.withCredentials = (token) ? false : this.useCredentials;
                xhr.onreadystatechange = () => {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            try {
                                const response = JSON.parse(xhr.responseText);
                                return resolve(response);
                            }
                            catch (error) {
                                return reject(error);
                            }
                        }
                        else {
                            return reject(new Error("There was a problem with the request."));
                        }
                    }
                };
                xhr.onerror = () => {
                    return reject(new Error("There was a problem with the request."));
                };
                xhr.send();
            });
        }
        /**
         * Gets the current token.
         * Will check the salaxy-token cookie if the token is persisted there
         */
        getCurrentToken() {
            if (!this.token && this.useCookie) {
                this.token = new Cookies().get("salaxy-token") || "";
            }
            return this.token;
        }
        /**
         * Sets the current token. The token is set to cookie called "salaxy-token" or
         * if the HTML page is running from local computer, it is set to local storage.
         *
         * @param token - the authentication token to store.
         */
        setCurrentToken(token) {
            if (this.useCookie) {
                new Cookies().setCookie("salaxy-token", token || "");
            }
            this.token = token;
        }
        /** If missing, append the API server address to the given url method string */
        getUrl(method) {
            if (!method || method.trim() === "") {
                return null;
            }
            if (method.toLowerCase().startsWith("http")) {
                return method;
            }
            if (method.toLowerCase().startsWith("/v")) {
                return this.getServerAddress() + method;
            }
            return this.getApiAddress() + method;
        }
        createCORSRequest(method, url) {
            let xhr = new XMLHttpRequest();
            if ("withCredentials" in xhr) {
                // XHR for Chrome/Firefox/Opera/Safari.
                xhr.open(method, url, true);
            }
            else if (typeof XDomainRequest !== "undefined") {
                // XDomainRequest for IE8/9.
                xhr = new XDomainRequest();
                xhr.open(method, url);
            }
            else {
                // CORS not supported.
                xhr = null;
            }
            return xhr;
        }
    }

    /**
     * Provides framework independent helpers for creating HTML forms
     * based on JSON schema and other metadata.
     */
    class FormHelpers {
        /**
         * Gets a valid input type, format and other input properties from schema.
         * @param schema The property schema based on which the input is created.
         * @param propertyName Name of property or null if the input is not created from a property, but from model directly.
         * @param dataBindingPath Path for the data model.
         * Default is "form": The current form. Could be a a longer path e.g. "form.result.employerCalc"
         */
        static getInputMetadata(schema, propertyName, dataBindingPath = "form") {
            var _a;
            const result = {
                name: propertyName,
                path: null,
                type: schema.type,
                format: schema.format,
                isEnum: false,
            };
            propertyName = propertyName || "";
            result.path = result.name ? (dataBindingPath ? (dataBindingPath + "." + result.name) : result.name) : dataBindingPath;
            switch (result.type) {
                case "number":
                case "integer": // TODO: Add enum support to numbers.
                case "boolean":
                case "object":
                case "array":
                    break;
                case "string":
                    result.isEnum = ((_a = schema.enum) === null || _a === void 0 ? void 0 : _a.length) > 0;
                    break;
                default:
                    result.type = "error";
                    result.content = `Unhandled data type '${result.type}', format '${result.format}' in '${propertyName || "no property name"}'.`;
                    return result;
            }
            return result;
        }
        /**
         * Gets the necessary metadata for generating the inputs for the specified model of type "object".
         * If the schema is not of type "object", returns null.
         * @param schema Schema for which the user interface is generated.
         * @param dataBindingPath Path for the data model.
         * Default is "form": The current form. Could be a a longer path e.g. "form.result.employerCalc"
         */
        static getInputsForObject(schema, dataBindingPath = "form") {
            if (schema.type === "object") {
                return Object.keys(schema.properties).map((key) => this.getInputMetadata(schema, key, dataBindingPath));
            }
            else {
                return null;
            }
        }
        /**
         * Gets input for the schema itself.
         * @param schema Schema for which the input component is generated.
         * @param propertyName Property name in relation to schema. This may be used in language versioning / texts.
         * @param dataBindingPath Path for the data binding within the form.
         */
        static getInputForSelf(schema, propertyName, dataBindingPath) {
            return this.getInputMetadata(schema, propertyName, dataBindingPath);
        }
        /**
         * Gets the necessary metadata for generating the inputs for the array items.
         * If the schema is not of type "array", returns null.
         * @param schema Schema for which the user interface is generated.
         * @param dataBindingPath Path for the data binding within the form.
         */
        static getInputsForArray(schema, dataBindingPath) {
            if (schema.type === "array") {
                return FormHelpers.getInputsForObject(schema.items, dataBindingPath)
                    || [FormHelpers.getInputForSelf(schema.items, null, dataBindingPath)];
            }
            return null;
        }
    }

    /** Static utilities for JSON Schemas an Open API documents. */
    class JsonSchemaUtils {
        /**
         * Gets a property from a schema document. Supports single property names or longer property paths.
         * @param schema The root type from which the property is found.
         * @param path Property path starting with the root schema.
         */
        static getProperty(schema, path) {
            var _a;
            if (!schema || !(path === null || path === void 0 ? void 0 : path.trim())) {
                return null;
            }
            const pathArr = path.split(".");
            let member = schema;
            let parentName = null;
            let propertyName = null;
            let isRequired = false;
            while (pathArr.length) {
                propertyName = pathArr.shift().trim();
                const memberOrRef = member.properties[propertyName];
                parentName = member.format;
                isRequired = (((_a = member.required) === null || _a === void 0 ? void 0 : _a.indexOf(propertyName)) >= 0) || false;
                if (JsonSchemaUtils.isReferenceObject(memberOrRef)) {
                    throw new Error(`Unresolved reference in ${propertyName}`);
                }
                member = memberOrRef;
                if (!member) {
                    return null;
                }
            }
            return {
                parentName,
                propertyName,
                schema: member,
                isRequired,
            };
        }
        /** Typeguard for OpenAPIV2 */
        static isOpenAPIV3(doc) {
            var _a, _b;
            return ((_b = (_a = doc) === null || _a === void 0 ? void 0 : _a.openapi) === null || _b === void 0 ? void 0 : _b.startsWith("3.")) || false;
        }
        /** Typeguard for OpenAPIV3 */
        static isOpenAPIV2(doc) {
            var _a, _b;
            return ((_b = (_a = doc) === null || _a === void 0 ? void 0 : _a.swagger) === null || _b === void 0 ? void 0 : _b.startsWith("2.")) || false;
        }
        /** Typeguard for JSON schema (TODO: consider adding more generic Schema typing).) */
        static isSchemaDocument(doc) {
            var _a;
            return !!((_a = doc) === null || _a === void 0 ? void 0 : _a.$schema);
        }
        /** Typeguard for OpenAPIV3.ReferenceObject  */
        static isReferenceObject(schema) {
            var _a;
            return !!((_a = schema) === null || _a === void 0 ? void 0 : _a.$ref);
        }
        /** Typeguard for OpenAPIV3.NonArraySchemaObject  */
        static isNonArraySchemaObject(schema) {
            return ["boolean", "object", "number", "string", "integer"].indexOf(schema.type) >= 0;
        }
        /** Typeguard for OpenAPIV3.ArraySchemaObject  */
        static isArraySchemaObject(schema) {
            return schema.type === "array";
        }
    }

    /**
     * Implemenatation of cache of resolved schemas and methods on reading their structure.
     */
    class JsonSchemaCache {
        constructor(ajax) {
            this.ajax = ajax;
            /** Cache for loaded schema elements. */
            this.schemaCache = [];
        }
        /**
         * Adds a local schema document and derefernces it.
         * @param url URL that identifies the schema
         * @param doc The schema document as object.
         * @returns A promise of the object.
         */
        addSchemaDocument(url, doc) {
            url = url.toLowerCase().trim();
            this.schemaCache = this.schemaCache.filter((x) => x.url !== url);
            if (JsonSchemaUtils.isOpenAPIV2(doc)) {
                const v3doc = {
                    "openapi": "3.0.1",
                    "info": doc.info,
                    paths: doc.paths,
                    components: {
                        // HACK: There are differneces in two schema objects. Go through what they are exactly and migrate/mitigate if possible.
                        schemas: doc.definitions,
                    }
                };
                doc = v3doc;
            }
            if (JsonSchemaUtils.isSchemaDocument(doc)) {
                const v3doc = {
                    openapi: "3.0.1",
                    info: {
                        title: doc.title,
                        version: "1.0",
                        description: doc.description,
                    },
                    paths: {},
                    components: {
                        // TODO: There may be differneces in two schema objects (depending on the version). Go through what they are exactly and migrate/mitigate if possible.
                        schemas: doc.definitions,
                    }
                };
                doc = v3doc;
            }
            if (JsonSchemaUtils.isOpenAPIV3(doc)) {
                this.prepareSchema(doc);
                // Add to cache
                this.schemaCache.push({
                    url: url.toLowerCase(),
                    doc,
                    type: "OpenAPIV3",
                });
                return doc;
            }
            throw new Error("Unsupported JSON schema / open api: " + url);
        }
        /**
         * Prepares the scehma for our use:
         * Dereferences the internal references ($ref-attribute) as JavaScript references.
         * Also adds the schema names to the format attribute of schema.
         * External references are currently not handled.
         */
        prepareSchema(doc) {
            Object.keys(doc.components.schemas).forEach((schemaName) => {
                const schema = doc.components.schemas[schemaName];
                // TODO: Consider adding as a new property instead of recycling format attribute.
                schema.format = schemaName;
                if (JsonSchemaUtils.isNonArraySchemaObject(schema)) {
                    // JsonSchemaCache.sortSchemaProperties(schema);
                    if (schema.type === "object") {
                        if (schema.properties) {
                            // First resolve references as JavaScript references
                            Object.keys(schema.properties).forEach((propName) => {
                                const prop = schema.properties[propName];
                                let ref = prop.$ref;
                                if (!ref && prop.oneOf) {
                                    // HACK: Just a quick hack => Go through in more detail and make more generic (oneOf, allOf etc.)
                                    // Also add support for JsonSchema style oneOf["string", "null"] for nullable items.
                                    ref = prop.oneOf[0].$ref;
                                }
                                if (ref) {
                                    if (ref.startsWith("#/components/schemas/")) {
                                        schema.properties[propName] = doc.components.schemas[ref.replace("#/components/schemas/", "")];
                                    }
                                    else if (ref.startsWith("#/definitions/")) {
                                        schema.properties[propName] = doc.components.schemas[ref.replace("#/definitions/", "")];
                                    }
                                    else {
                                        console.error(`Unhandled $ref: ${ref}`);
                                    }
                                }
                                else if (JsonSchemaUtils.isArraySchemaObject(prop)) {
                                    const arrayRef = prop.items.$ref;
                                    if (arrayRef) {
                                        if (arrayRef.startsWith("#/components/schemas/")) {
                                            prop.items = doc.components.schemas[arrayRef.replace("#/components/schemas/", "")];
                                        }
                                        else if (arrayRef.startsWith("#/definitions/")) {
                                            prop.items = doc.components.schemas[arrayRef.replace("#/definitions/", "")];
                                        }
                                        else {
                                            console.error(`Unhandled array items $ref: ${ref}`);
                                        }
                                    }
                                }
                            });
                            // Handle differences in Swagger/OpenApi 2, OpenApi 3 and Json Schema
                            Object.keys(schema.properties).forEach((propName) => {
                                const prop = schema.properties[propName];
                                if (prop.readonly != null) {
                                    prop.readOnly = prop.readonly;
                                }
                                else if (prop["x-readonly"] != null) {
                                    prop.readOnly = prop["x-readonly"];
                                }
                                // TODO: Consider other cases, at least nullability.
                                // See logic in NJsonSchema: https://github.com/RicoSuter/NJsonSchema/blob/master/src/NJsonSchema/JsonSchema.Serialization.cs
                            });
                        }
                    }
                }
            });
        }
        /**
         * Sorts the properties in JSON schema according to some misc. logic rules
         * (e.g. id first, objects and arrays lower and unknown last).
         * TODO: We could potentially put this in another place: Server-side? UI framework?
         */
        static sortSchemaProperties(schema) {
            if (schema.properties) {
                const orderedArray = Object.keys(schema.properties).map((key) => {
                    const prop = schema.properties[key];
                    const orderResult = {
                        key,
                        prop,
                        sort: 50,
                    };
                    switch (prop.type) {
                        case "number":
                        case "integer":
                        case "boolean":
                            break;
                        case "string":
                            if (key.endsWith("At")) {
                                orderResult.sort = 20;
                            }
                            else if (key === "id") {
                                orderResult.sort = 10;
                            }
                            break;
                        case "array":
                            orderResult.sort = 70;
                            break;
                        case "object":
                            {
                                switch (prop.format) {
                                    case "DateRange":
                                    case "Avatar":
                                        orderResult.sort = 50;
                                        break;
                                    default:
                                        orderResult.sort = 60;
                                        break;
                                }
                            }
                            break;
                        default:
                            orderResult.sort = 80;
                            break;
                    }
                    return orderResult;
                });
                const ordered = {};
                orderedArray.sort((a, b) => (a.sort.toString() + a.key).localeCompare(b.sort.toString() + b.key))
                    .forEach((x) => {
                    ordered[x.key] = x.prop;
                });
                schema.properties = ordered;
            }
        }
        /**
         * Assures that a schema document identified by URL is loaded from server to the cache.
         * Currently, supports only OpenApi 3, but we may later support OpenApi 2 and/or plain JSON schema.
         * @param openApiUrl URL to the open API document.
         * @returns A promise that resolves when the document has been loaded and you can call other methods on it.
         */
        assureSchemaDocument(openApiUrl) {
            const cachedSchema = this.schemaCache.find((x) => x.url === openApiUrl);
            if (cachedSchema) {
                if (cachedSchema.loadingDonePromise != null) {
                    return cachedSchema.loadingDonePromise;
                }
                return Promise.resolve(cachedSchema.doc);
            }
            const loadingDonePromise = new Promise((resolve, reject) => {
                this.ajax.getJSON(openApiUrl).then((data) => {
                    if (!data) {
                        throw new Error("No data from the schema: " + openApiUrl);
                    }
                    resolve(this.addSchemaDocument(openApiUrl, data));
                }).catch((reason) => {
                    reject(reason);
                });
            });
            this.schemaCache.push({
                url: openApiUrl,
                type: "OpenAPIV3",
                doc: null,
                loadingDonePromise,
            });
            return loadingDonePromise;
        }
        /**
         * Finds a schema document from the cache.
         * This method is syncronous base method for other schema operations
         * Make sure that the schme document has been loaded before calling as this will not load the document,
         * but will fail instead if the schema document is not there.
         * @param openApiUrl URL to the open API document.
         * @param throwIfNotFound If true, will throw an error if the document is not found in the cache.
         */
        findSchemaDoc(openApiUrl, throwIfNotFound) {
            openApiUrl = openApiUrl.toLowerCase().trim();
            if (!openApiUrl) {
                throw new Error("Empty Open API URL.");
            }
            const result = this.schemaCache.find((x) => x.url === openApiUrl);
            if (!result && throwIfNotFound) {
                throw new Error("Schema document not loaded: " + openApiUrl);
            }
            return result;
        }
        /**
         * Finds a single schema (Data model) within a schema document
         * Make sure that the schme document has been loaded before calling as this will not load the document,
         * but will fail instead if the schema document is not there.
         * @param openApiUrl URL to the open API document.
         * @param name Name of the schema (data model)
         */
        findSchema(openApiUrl, name) {
            var _a, _b;
            const schema = (_b = (_a = this.findSchemaDoc(openApiUrl, true)) === null || _a === void 0 ? void 0 : _a.doc.components) === null || _b === void 0 ? void 0 : _b.schemas[name];
            if (!schema) {
                return null;
            }
            if (JsonSchemaUtils.isReferenceObject(schema)) {
                throw new Error(`Unresolved reference in ${openApiUrl} - ${name}`);
            }
            return schema;
        }
    }

    /**
     * Helpers for handling arrays.
     */
    class Arrays {
        /**
         * Takes in an array or string or null and always returns an array.
         * If the incoming value is a string, splits it by commas. This is especaially useful for HTML attributes.
         * Note that null (or other falsy objects) are returned as an empty array.
         *
         * @param arrayCandidate - A string array or string that is split by comma.
         */
        static assureArray(arrayCandidate) {
            if (!arrayCandidate) {
                return [];
            }
            if (Array.isArray(arrayCandidate)) {
                return arrayCandidate;
            }
            return arrayCandidate.split(",");
        }
        /**
         * Calculates a sum of array items.
         * Null items will be added as zero.
         * @param array Array to sum
         * @param selector Selector for a number to sum.
         */
        static sum(array, selector) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return array.reduce((prev, current, ix) => current ? (prev + (selector(current) || 0)) : 0, 0);
        }
        /**
         * Sequence generator function gets an array between two numbers (inclusive).
         * (commonly referred to as "range", e.g. Clojure, PHP etc)
         * @param start Start number of the sequence
         * @param stop End number of the sequence
         * @param step Step between the numbers, default is 1.
         * @example
         * Arrays.getRange(2019, 2023);
         * // [2019, 2020, 2021, 2022, 2023]
         */
        static getRange(start, stop, step = 1) {
            const range = (start, stop, step) => Array.from({ length: (stop - start) / step + 1 }, (_, i) => start + (i * step));
            return range(start, stop, step);
        }
    }

    var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

    function commonjsRequire (path) {
    	throw new Error('Could not dynamically require "' + path + '". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.');
    }

    var moment$1 = {exports: {}};

    (function (module, exports) {
    (function (global, factory) {
        module.exports = factory() ;
    }(commonjsGlobal, (function () {
        var hookCallback;

        function hooks() {
            return hookCallback.apply(null, arguments);
        }

        // This is done to register the method called with moment()
        // without creating circular dependencies.
        function setHookCallback(callback) {
            hookCallback = callback;
        }

        function isArray(input) {
            return (
                input instanceof Array ||
                Object.prototype.toString.call(input) === '[object Array]'
            );
        }

        function isObject(input) {
            // IE8 will treat undefined and null as object if it wasn't for
            // input != null
            return (
                input != null &&
                Object.prototype.toString.call(input) === '[object Object]'
            );
        }

        function hasOwnProp(a, b) {
            return Object.prototype.hasOwnProperty.call(a, b);
        }

        function isObjectEmpty(obj) {
            if (Object.getOwnPropertyNames) {
                return Object.getOwnPropertyNames(obj).length === 0;
            } else {
                var k;
                for (k in obj) {
                    if (hasOwnProp(obj, k)) {
                        return false;
                    }
                }
                return true;
            }
        }

        function isUndefined(input) {
            return input === void 0;
        }

        function isNumber(input) {
            return (
                typeof input === 'number' ||
                Object.prototype.toString.call(input) === '[object Number]'
            );
        }

        function isDate(input) {
            return (
                input instanceof Date ||
                Object.prototype.toString.call(input) === '[object Date]'
            );
        }

        function map(arr, fn) {
            var res = [],
                i;
            for (i = 0; i < arr.length; ++i) {
                res.push(fn(arr[i], i));
            }
            return res;
        }

        function extend(a, b) {
            for (var i in b) {
                if (hasOwnProp(b, i)) {
                    a[i] = b[i];
                }
            }

            if (hasOwnProp(b, 'toString')) {
                a.toString = b.toString;
            }

            if (hasOwnProp(b, 'valueOf')) {
                a.valueOf = b.valueOf;
            }

            return a;
        }

        function createUTC(input, format, locale, strict) {
            return createLocalOrUTC(input, format, locale, strict, true).utc();
        }

        function defaultParsingFlags() {
            // We need to deep clone this object.
            return {
                empty: false,
                unusedTokens: [],
                unusedInput: [],
                overflow: -2,
                charsLeftOver: 0,
                nullInput: false,
                invalidEra: null,
                invalidMonth: null,
                invalidFormat: false,
                userInvalidated: false,
                iso: false,
                parsedDateParts: [],
                era: null,
                meridiem: null,
                rfc2822: false,
                weekdayMismatch: false,
            };
        }

        function getParsingFlags(m) {
            if (m._pf == null) {
                m._pf = defaultParsingFlags();
            }
            return m._pf;
        }

        var some;
        if (Array.prototype.some) {
            some = Array.prototype.some;
        } else {
            some = function (fun) {
                var t = Object(this),
                    len = t.length >>> 0,
                    i;

                for (i = 0; i < len; i++) {
                    if (i in t && fun.call(this, t[i], i, t)) {
                        return true;
                    }
                }

                return false;
            };
        }

        function isValid(m) {
            if (m._isValid == null) {
                var flags = getParsingFlags(m),
                    parsedParts = some.call(flags.parsedDateParts, function (i) {
                        return i != null;
                    }),
                    isNowValid =
                        !isNaN(m._d.getTime()) &&
                        flags.overflow < 0 &&
                        !flags.empty &&
                        !flags.invalidEra &&
                        !flags.invalidMonth &&
                        !flags.invalidWeekday &&
                        !flags.weekdayMismatch &&
                        !flags.nullInput &&
                        !flags.invalidFormat &&
                        !flags.userInvalidated &&
                        (!flags.meridiem || (flags.meridiem && parsedParts));

                if (m._strict) {
                    isNowValid =
                        isNowValid &&
                        flags.charsLeftOver === 0 &&
                        flags.unusedTokens.length === 0 &&
                        flags.bigHour === undefined;
                }

                if (Object.isFrozen == null || !Object.isFrozen(m)) {
                    m._isValid = isNowValid;
                } else {
                    return isNowValid;
                }
            }
            return m._isValid;
        }

        function createInvalid(flags) {
            var m = createUTC(NaN);
            if (flags != null) {
                extend(getParsingFlags(m), flags);
            } else {
                getParsingFlags(m).userInvalidated = true;
            }

            return m;
        }

        // Plugins that add properties should also add the key here (null value),
        // so we can properly clone ourselves.
        var momentProperties = (hooks.momentProperties = []),
            updateInProgress = false;

        function copyConfig(to, from) {
            var i, prop, val;

            if (!isUndefined(from._isAMomentObject)) {
                to._isAMomentObject = from._isAMomentObject;
            }
            if (!isUndefined(from._i)) {
                to._i = from._i;
            }
            if (!isUndefined(from._f)) {
                to._f = from._f;
            }
            if (!isUndefined(from._l)) {
                to._l = from._l;
            }
            if (!isUndefined(from._strict)) {
                to._strict = from._strict;
            }
            if (!isUndefined(from._tzm)) {
                to._tzm = from._tzm;
            }
            if (!isUndefined(from._isUTC)) {
                to._isUTC = from._isUTC;
            }
            if (!isUndefined(from._offset)) {
                to._offset = from._offset;
            }
            if (!isUndefined(from._pf)) {
                to._pf = getParsingFlags(from);
            }
            if (!isUndefined(from._locale)) {
                to._locale = from._locale;
            }

            if (momentProperties.length > 0) {
                for (i = 0; i < momentProperties.length; i++) {
                    prop = momentProperties[i];
                    val = from[prop];
                    if (!isUndefined(val)) {
                        to[prop] = val;
                    }
                }
            }

            return to;
        }

        // Moment prototype object
        function Moment(config) {
            copyConfig(this, config);
            this._d = new Date(config._d != null ? config._d.getTime() : NaN);
            if (!this.isValid()) {
                this._d = new Date(NaN);
            }
            // Prevent infinite loop in case updateOffset creates new moment
            // objects.
            if (updateInProgress === false) {
                updateInProgress = true;
                hooks.updateOffset(this);
                updateInProgress = false;
            }
        }

        function isMoment(obj) {
            return (
                obj instanceof Moment || (obj != null && obj._isAMomentObject != null)
            );
        }

        function warn(msg) {
            if (
                hooks.suppressDeprecationWarnings === false &&
                typeof console !== 'undefined' &&
                console.warn
            ) {
                console.warn('Deprecation warning: ' + msg);
            }
        }

        function deprecate(msg, fn) {
            var firstTime = true;

            return extend(function () {
                if (hooks.deprecationHandler != null) {
                    hooks.deprecationHandler(null, msg);
                }
                if (firstTime) {
                    var args = [],
                        arg,
                        i,
                        key;
                    for (i = 0; i < arguments.length; i++) {
                        arg = '';
                        if (typeof arguments[i] === 'object') {
                            arg += '\n[' + i + '] ';
                            for (key in arguments[0]) {
                                if (hasOwnProp(arguments[0], key)) {
                                    arg += key + ': ' + arguments[0][key] + ', ';
                                }
                            }
                            arg = arg.slice(0, -2); // Remove trailing comma and space
                        } else {
                            arg = arguments[i];
                        }
                        args.push(arg);
                    }
                    warn(
                        msg +
                            '\nArguments: ' +
                            Array.prototype.slice.call(args).join('') +
                            '\n' +
                            new Error().stack
                    );
                    firstTime = false;
                }
                return fn.apply(this, arguments);
            }, fn);
        }

        var deprecations = {};

        function deprecateSimple(name, msg) {
            if (hooks.deprecationHandler != null) {
                hooks.deprecationHandler(name, msg);
            }
            if (!deprecations[name]) {
                warn(msg);
                deprecations[name] = true;
            }
        }

        hooks.suppressDeprecationWarnings = false;
        hooks.deprecationHandler = null;

        function isFunction(input) {
            return (
                (typeof Function !== 'undefined' && input instanceof Function) ||
                Object.prototype.toString.call(input) === '[object Function]'
            );
        }

        function set(config) {
            var prop, i;
            for (i in config) {
                if (hasOwnProp(config, i)) {
                    prop = config[i];
                    if (isFunction(prop)) {
                        this[i] = prop;
                    } else {
                        this['_' + i] = prop;
                    }
                }
            }
            this._config = config;
            // Lenient ordinal parsing accepts just a number in addition to
            // number + (possibly) stuff coming from _dayOfMonthOrdinalParse.
            // TODO: Remove "ordinalParse" fallback in next major release.
            this._dayOfMonthOrdinalParseLenient = new RegExp(
                (this._dayOfMonthOrdinalParse.source || this._ordinalParse.source) +
                    '|' +
                    /\d{1,2}/.source
            );
        }

        function mergeConfigs(parentConfig, childConfig) {
            var res = extend({}, parentConfig),
                prop;
            for (prop in childConfig) {
                if (hasOwnProp(childConfig, prop)) {
                    if (isObject(parentConfig[prop]) && isObject(childConfig[prop])) {
                        res[prop] = {};
                        extend(res[prop], parentConfig[prop]);
                        extend(res[prop], childConfig[prop]);
                    } else if (childConfig[prop] != null) {
                        res[prop] = childConfig[prop];
                    } else {
                        delete res[prop];
                    }
                }
            }
            for (prop in parentConfig) {
                if (
                    hasOwnProp(parentConfig, prop) &&
                    !hasOwnProp(childConfig, prop) &&
                    isObject(parentConfig[prop])
                ) {
                    // make sure changes to properties don't modify parent config
                    res[prop] = extend({}, res[prop]);
                }
            }
            return res;
        }

        function Locale(config) {
            if (config != null) {
                this.set(config);
            }
        }

        var keys;

        if (Object.keys) {
            keys = Object.keys;
        } else {
            keys = function (obj) {
                var i,
                    res = [];
                for (i in obj) {
                    if (hasOwnProp(obj, i)) {
                        res.push(i);
                    }
                }
                return res;
            };
        }

        var defaultCalendar = {
            sameDay: '[Today at] LT',
            nextDay: '[Tomorrow at] LT',
            nextWeek: 'dddd [at] LT',
            lastDay: '[Yesterday at] LT',
            lastWeek: '[Last] dddd [at] LT',
            sameElse: 'L',
        };

        function calendar(key, mom, now) {
            var output = this._calendar[key] || this._calendar['sameElse'];
            return isFunction(output) ? output.call(mom, now) : output;
        }

        function zeroFill(number, targetLength, forceSign) {
            var absNumber = '' + Math.abs(number),
                zerosToFill = targetLength - absNumber.length,
                sign = number >= 0;
            return (
                (sign ? (forceSign ? '+' : '') : '-') +
                Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) +
                absNumber
            );
        }

        var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,
            localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,
            formatFunctions = {},
            formatTokenFunctions = {};

        // token:    'M'
        // padded:   ['MM', 2]
        // ordinal:  'Mo'
        // callback: function () { this.month() + 1 }
        function addFormatToken(token, padded, ordinal, callback) {
            var func = callback;
            if (typeof callback === 'string') {
                func = function () {
                    return this[callback]();
                };
            }
            if (token) {
                formatTokenFunctions[token] = func;
            }
            if (padded) {
                formatTokenFunctions[padded[0]] = function () {
                    return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
                };
            }
            if (ordinal) {
                formatTokenFunctions[ordinal] = function () {
                    return this.localeData().ordinal(
                        func.apply(this, arguments),
                        token
                    );
                };
            }
        }

        function removeFormattingTokens(input) {
            if (input.match(/\[[\s\S]/)) {
                return input.replace(/^\[|\]$/g, '');
            }
            return input.replace(/\\/g, '');
        }

        function makeFormatFunction(format) {
            var array = format.match(formattingTokens),
                i,
                length;

            for (i = 0, length = array.length; i < length; i++) {
                if (formatTokenFunctions[array[i]]) {
                    array[i] = formatTokenFunctions[array[i]];
                } else {
                    array[i] = removeFormattingTokens(array[i]);
                }
            }

            return function (mom) {
                var output = '',
                    i;
                for (i = 0; i < length; i++) {
                    output += isFunction(array[i])
                        ? array[i].call(mom, format)
                        : array[i];
                }
                return output;
            };
        }

        // format date using native date object
        function formatMoment(m, format) {
            if (!m.isValid()) {
                return m.localeData().invalidDate();
            }

            format = expandFormat(format, m.localeData());
            formatFunctions[format] =
                formatFunctions[format] || makeFormatFunction(format);

            return formatFunctions[format](m);
        }

        function expandFormat(format, locale) {
            var i = 5;

            function replaceLongDateFormatTokens(input) {
                return locale.longDateFormat(input) || input;
            }

            localFormattingTokens.lastIndex = 0;
            while (i >= 0 && localFormattingTokens.test(format)) {
                format = format.replace(
                    localFormattingTokens,
                    replaceLongDateFormatTokens
                );
                localFormattingTokens.lastIndex = 0;
                i -= 1;
            }

            return format;
        }

        var defaultLongDateFormat = {
            LTS: 'h:mm:ss A',
            LT: 'h:mm A',
            L: 'MM/DD/YYYY',
            LL: 'MMMM D, YYYY',
            LLL: 'MMMM D, YYYY h:mm A',
            LLLL: 'dddd, MMMM D, YYYY h:mm A',
        };

        function longDateFormat(key) {
            var format = this._longDateFormat[key],
                formatUpper = this._longDateFormat[key.toUpperCase()];

            if (format || !formatUpper) {
                return format;
            }

            this._longDateFormat[key] = formatUpper
                .match(formattingTokens)
                .map(function (tok) {
                    if (
                        tok === 'MMMM' ||
                        tok === 'MM' ||
                        tok === 'DD' ||
                        tok === 'dddd'
                    ) {
                        return tok.slice(1);
                    }
                    return tok;
                })
                .join('');

            return this._longDateFormat[key];
        }

        var defaultInvalidDate = 'Invalid date';

        function invalidDate() {
            return this._invalidDate;
        }

        var defaultOrdinal = '%d',
            defaultDayOfMonthOrdinalParse = /\d{1,2}/;

        function ordinal(number) {
            return this._ordinal.replace('%d', number);
        }

        var defaultRelativeTime = {
            future: 'in %s',
            past: '%s ago',
            s: 'a few seconds',
            ss: '%d seconds',
            m: 'a minute',
            mm: '%d minutes',
            h: 'an hour',
            hh: '%d hours',
            d: 'a day',
            dd: '%d days',
            w: 'a week',
            ww: '%d weeks',
            M: 'a month',
            MM: '%d months',
            y: 'a year',
            yy: '%d years',
        };

        function relativeTime(number, withoutSuffix, string, isFuture) {
            var output = this._relativeTime[string];
            return isFunction(output)
                ? output(number, withoutSuffix, string, isFuture)
                : output.replace(/%d/i, number);
        }

        function pastFuture(diff, output) {
            var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
            return isFunction(format) ? format(output) : format.replace(/%s/i, output);
        }

        var aliases = {};

        function addUnitAlias(unit, shorthand) {
            var lowerCase = unit.toLowerCase();
            aliases[lowerCase] = aliases[lowerCase + 's'] = aliases[shorthand] = unit;
        }

        function normalizeUnits(units) {
            return typeof units === 'string'
                ? aliases[units] || aliases[units.toLowerCase()]
                : undefined;
        }

        function normalizeObjectUnits(inputObject) {
            var normalizedInput = {},
                normalizedProp,
                prop;

            for (prop in inputObject) {
                if (hasOwnProp(inputObject, prop)) {
                    normalizedProp = normalizeUnits(prop);
                    if (normalizedProp) {
                        normalizedInput[normalizedProp] = inputObject[prop];
                    }
                }
            }

            return normalizedInput;
        }

        var priorities = {};

        function addUnitPriority(unit, priority) {
            priorities[unit] = priority;
        }

        function getPrioritizedUnits(unitsObj) {
            var units = [],
                u;
            for (u in unitsObj) {
                if (hasOwnProp(unitsObj, u)) {
                    units.push({ unit: u, priority: priorities[u] });
                }
            }
            units.sort(function (a, b) {
                return a.priority - b.priority;
            });
            return units;
        }

        function isLeapYear(year) {
            return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;
        }

        function absFloor(number) {
            if (number < 0) {
                // -0 -> 0
                return Math.ceil(number) || 0;
            } else {
                return Math.floor(number);
            }
        }

        function toInt(argumentForCoercion) {
            var coercedNumber = +argumentForCoercion,
                value = 0;

            if (coercedNumber !== 0 && isFinite(coercedNumber)) {
                value = absFloor(coercedNumber);
            }

            return value;
        }

        function makeGetSet(unit, keepTime) {
            return function (value) {
                if (value != null) {
                    set$1(this, unit, value);
                    hooks.updateOffset(this, keepTime);
                    return this;
                } else {
                    return get(this, unit);
                }
            };
        }

        function get(mom, unit) {
            return mom.isValid()
                ? mom._d['get' + (mom._isUTC ? 'UTC' : '') + unit]()
                : NaN;
        }

        function set$1(mom, unit, value) {
            if (mom.isValid() && !isNaN(value)) {
                if (
                    unit === 'FullYear' &&
                    isLeapYear(mom.year()) &&
                    mom.month() === 1 &&
                    mom.date() === 29
                ) {
                    value = toInt(value);
                    mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](
                        value,
                        mom.month(),
                        daysInMonth(value, mom.month())
                    );
                } else {
                    mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value);
                }
            }
        }

        // MOMENTS

        function stringGet(units) {
            units = normalizeUnits(units);
            if (isFunction(this[units])) {
                return this[units]();
            }
            return this;
        }

        function stringSet(units, value) {
            if (typeof units === 'object') {
                units = normalizeObjectUnits(units);
                var prioritized = getPrioritizedUnits(units),
                    i;
                for (i = 0; i < prioritized.length; i++) {
                    this[prioritized[i].unit](units[prioritized[i].unit]);
                }
            } else {
                units = normalizeUnits(units);
                if (isFunction(this[units])) {
                    return this[units](value);
                }
            }
            return this;
        }

        var match1 = /\d/, //       0 - 9
            match2 = /\d\d/, //      00 - 99
            match3 = /\d{3}/, //     000 - 999
            match4 = /\d{4}/, //    0000 - 9999
            match6 = /[+-]?\d{6}/, // -999999 - 999999
            match1to2 = /\d\d?/, //       0 - 99
            match3to4 = /\d\d\d\d?/, //     999 - 9999
            match5to6 = /\d\d\d\d\d\d?/, //   99999 - 999999
            match1to3 = /\d{1,3}/, //       0 - 999
            match1to4 = /\d{1,4}/, //       0 - 9999
            match1to6 = /[+-]?\d{1,6}/, // -999999 - 999999
            matchUnsigned = /\d+/, //       0 - inf
            matchSigned = /[+-]?\d+/, //    -inf - inf
            matchOffset = /Z|[+-]\d\d:?\d\d/gi, // +00:00 -00:00 +0000 -0000 or Z
            matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi, // +00 -00 +00:00 -00:00 +0000 -0000 or Z
            matchTimestamp = /[+-]?\d+(\.\d{1,3})?/, // 123456789 123456789.123
            // any word (or two) characters or numbers including two/three word month in arabic.
            // includes scottish gaelic two word and hyphenated months
            matchWord = /[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,
            regexes;

        regexes = {};

        function addRegexToken(token, regex, strictRegex) {
            regexes[token] = isFunction(regex)
                ? regex
                : function (isStrict, localeData) {
                      return isStrict && strictRegex ? strictRegex : regex;
                  };
        }

        function getParseRegexForToken(token, config) {
            if (!hasOwnProp(regexes, token)) {
                return new RegExp(unescapeFormat(token));
            }

            return regexes[token](config._strict, config._locale);
        }

        // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
        function unescapeFormat(s) {
            return regexEscape(
                s
                    .replace('\\', '')
                    .replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (
                        matched,
                        p1,
                        p2,
                        p3,
                        p4
                    ) {
                        return p1 || p2 || p3 || p4;
                    })
            );
        }

        function regexEscape(s) {
            return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
        }

        var tokens = {};

        function addParseToken(token, callback) {
            var i,
                func = callback;
            if (typeof token === 'string') {
                token = [token];
            }
            if (isNumber(callback)) {
                func = function (input, array) {
                    array[callback] = toInt(input);
                };
            }
            for (i = 0; i < token.length; i++) {
                tokens[token[i]] = func;
            }
        }

        function addWeekParseToken(token, callback) {
            addParseToken(token, function (input, array, config, token) {
                config._w = config._w || {};
                callback(input, config._w, config, token);
            });
        }

        function addTimeToArrayFromToken(token, input, config) {
            if (input != null && hasOwnProp(tokens, token)) {
                tokens[token](input, config._a, config, token);
            }
        }

        var YEAR = 0,
            MONTH = 1,
            DATE = 2,
            HOUR = 3,
            MINUTE = 4,
            SECOND = 5,
            MILLISECOND = 6,
            WEEK = 7,
            WEEKDAY = 8;

        function mod(n, x) {
            return ((n % x) + x) % x;
        }

        var indexOf;

        if (Array.prototype.indexOf) {
            indexOf = Array.prototype.indexOf;
        } else {
            indexOf = function (o) {
                // I know
                var i;
                for (i = 0; i < this.length; ++i) {
                    if (this[i] === o) {
                        return i;
                    }
                }
                return -1;
            };
        }

        function daysInMonth(year, month) {
            if (isNaN(year) || isNaN(month)) {
                return NaN;
            }
            var modMonth = mod(month, 12);
            year += (month - modMonth) / 12;
            return modMonth === 1
                ? isLeapYear(year)
                    ? 29
                    : 28
                : 31 - ((modMonth % 7) % 2);
        }

        // FORMATTING

        addFormatToken('M', ['MM', 2], 'Mo', function () {
            return this.month() + 1;
        });

        addFormatToken('MMM', 0, 0, function (format) {
            return this.localeData().monthsShort(this, format);
        });

        addFormatToken('MMMM', 0, 0, function (format) {
            return this.localeData().months(this, format);
        });

        // ALIASES

        addUnitAlias('month', 'M');

        // PRIORITY

        addUnitPriority('month', 8);

        // PARSING

        addRegexToken('M', match1to2);
        addRegexToken('MM', match1to2, match2);
        addRegexToken('MMM', function (isStrict, locale) {
            return locale.monthsShortRegex(isStrict);
        });
        addRegexToken('MMMM', function (isStrict, locale) {
            return locale.monthsRegex(isStrict);
        });

        addParseToken(['M', 'MM'], function (input, array) {
            array[MONTH] = toInt(input) - 1;
        });

        addParseToken(['MMM', 'MMMM'], function (input, array, config, token) {
            var month = config._locale.monthsParse(input, token, config._strict);
            // if we didn't find a month name, mark the date as invalid.
            if (month != null) {
                array[MONTH] = month;
            } else {
                getParsingFlags(config).invalidMonth = input;
            }
        });

        // LOCALES

        var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split(
                '_'
            ),
            defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split(
                '_'
            ),
            MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,
            defaultMonthsShortRegex = matchWord,
            defaultMonthsRegex = matchWord;

        function localeMonths(m, format) {
            if (!m) {
                return isArray(this._months)
                    ? this._months
                    : this._months['standalone'];
            }
            return isArray(this._months)
                ? this._months[m.month()]
                : this._months[
                      (this._months.isFormat || MONTHS_IN_FORMAT).test(format)
                          ? 'format'
                          : 'standalone'
                  ][m.month()];
        }

        function localeMonthsShort(m, format) {
            if (!m) {
                return isArray(this._monthsShort)
                    ? this._monthsShort
                    : this._monthsShort['standalone'];
            }
            return isArray(this._monthsShort)
                ? this._monthsShort[m.month()]
                : this._monthsShort[
                      MONTHS_IN_FORMAT.test(format) ? 'format' : 'standalone'
                  ][m.month()];
        }

        function handleStrictParse(monthName, format, strict) {
            var i,
                ii,
                mom,
                llc = monthName.toLocaleLowerCase();
            if (!this._monthsParse) {
                // this is not used
                this._monthsParse = [];
                this._longMonthsParse = [];
                this._shortMonthsParse = [];
                for (i = 0; i < 12; ++i) {
                    mom = createUTC([2000, i]);
                    this._shortMonthsParse[i] = this.monthsShort(
                        mom,
                        ''
                    ).toLocaleLowerCase();
                    this._longMonthsParse[i] = this.months(mom, '').toLocaleLowerCase();
                }
            }

            if (strict) {
                if (format === 'MMM') {
                    ii = indexOf.call(this._shortMonthsParse, llc);
                    return ii !== -1 ? ii : null;
                } else {
                    ii = indexOf.call(this._longMonthsParse, llc);
                    return ii !== -1 ? ii : null;
                }
            } else {
                if (format === 'MMM') {
                    ii = indexOf.call(this._shortMonthsParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._longMonthsParse, llc);
                    return ii !== -1 ? ii : null;
                } else {
                    ii = indexOf.call(this._longMonthsParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._shortMonthsParse, llc);
                    return ii !== -1 ? ii : null;
                }
            }
        }

        function localeMonthsParse(monthName, format, strict) {
            var i, mom, regex;

            if (this._monthsParseExact) {
                return handleStrictParse.call(this, monthName, format, strict);
            }

            if (!this._monthsParse) {
                this._monthsParse = [];
                this._longMonthsParse = [];
                this._shortMonthsParse = [];
            }

            // TODO: add sorting
            // Sorting makes sure if one month (or abbr) is a prefix of another
            // see sorting in computeMonthsParse
            for (i = 0; i < 12; i++) {
                // make the regex if we don't have it already
                mom = createUTC([2000, i]);
                if (strict && !this._longMonthsParse[i]) {
                    this._longMonthsParse[i] = new RegExp(
                        '^' + this.months(mom, '').replace('.', '') + '$',
                        'i'
                    );
                    this._shortMonthsParse[i] = new RegExp(
                        '^' + this.monthsShort(mom, '').replace('.', '') + '$',
                        'i'
                    );
                }
                if (!strict && !this._monthsParse[i]) {
                    regex =
                        '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
                    this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
                }
                // test the regex
                if (
                    strict &&
                    format === 'MMMM' &&
                    this._longMonthsParse[i].test(monthName)
                ) {
                    return i;
                } else if (
                    strict &&
                    format === 'MMM' &&
                    this._shortMonthsParse[i].test(monthName)
                ) {
                    return i;
                } else if (!strict && this._monthsParse[i].test(monthName)) {
                    return i;
                }
            }
        }

        // MOMENTS

        function setMonth(mom, value) {
            var dayOfMonth;

            if (!mom.isValid()) {
                // No op
                return mom;
            }

            if (typeof value === 'string') {
                if (/^\d+$/.test(value)) {
                    value = toInt(value);
                } else {
                    value = mom.localeData().monthsParse(value);
                    // TODO: Another silent failure?
                    if (!isNumber(value)) {
                        return mom;
                    }
                }
            }

            dayOfMonth = Math.min(mom.date(), daysInMonth(mom.year(), value));
            mom._d['set' + (mom._isUTC ? 'UTC' : '') + 'Month'](value, dayOfMonth);
            return mom;
        }

        function getSetMonth(value) {
            if (value != null) {
                setMonth(this, value);
                hooks.updateOffset(this, true);
                return this;
            } else {
                return get(this, 'Month');
            }
        }

        function getDaysInMonth() {
            return daysInMonth(this.year(), this.month());
        }

        function monthsShortRegex(isStrict) {
            if (this._monthsParseExact) {
                if (!hasOwnProp(this, '_monthsRegex')) {
                    computeMonthsParse.call(this);
                }
                if (isStrict) {
                    return this._monthsShortStrictRegex;
                } else {
                    return this._monthsShortRegex;
                }
            } else {
                if (!hasOwnProp(this, '_monthsShortRegex')) {
                    this._monthsShortRegex = defaultMonthsShortRegex;
                }
                return this._monthsShortStrictRegex && isStrict
                    ? this._monthsShortStrictRegex
                    : this._monthsShortRegex;
            }
        }

        function monthsRegex(isStrict) {
            if (this._monthsParseExact) {
                if (!hasOwnProp(this, '_monthsRegex')) {
                    computeMonthsParse.call(this);
                }
                if (isStrict) {
                    return this._monthsStrictRegex;
                } else {
                    return this._monthsRegex;
                }
            } else {
                if (!hasOwnProp(this, '_monthsRegex')) {
                    this._monthsRegex = defaultMonthsRegex;
                }
                return this._monthsStrictRegex && isStrict
                    ? this._monthsStrictRegex
                    : this._monthsRegex;
            }
        }

        function computeMonthsParse() {
            function cmpLenRev(a, b) {
                return b.length - a.length;
            }

            var shortPieces = [],
                longPieces = [],
                mixedPieces = [],
                i,
                mom;
            for (i = 0; i < 12; i++) {
                // make the regex if we don't have it already
                mom = createUTC([2000, i]);
                shortPieces.push(this.monthsShort(mom, ''));
                longPieces.push(this.months(mom, ''));
                mixedPieces.push(this.months(mom, ''));
                mixedPieces.push(this.monthsShort(mom, ''));
            }
            // Sorting makes sure if one month (or abbr) is a prefix of another it
            // will match the longer piece.
            shortPieces.sort(cmpLenRev);
            longPieces.sort(cmpLenRev);
            mixedPieces.sort(cmpLenRev);
            for (i = 0; i < 12; i++) {
                shortPieces[i] = regexEscape(shortPieces[i]);
                longPieces[i] = regexEscape(longPieces[i]);
            }
            for (i = 0; i < 24; i++) {
                mixedPieces[i] = regexEscape(mixedPieces[i]);
            }

            this._monthsRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
            this._monthsShortRegex = this._monthsRegex;
            this._monthsStrictRegex = new RegExp(
                '^(' + longPieces.join('|') + ')',
                'i'
            );
            this._monthsShortStrictRegex = new RegExp(
                '^(' + shortPieces.join('|') + ')',
                'i'
            );
        }

        // FORMATTING

        addFormatToken('Y', 0, 0, function () {
            var y = this.year();
            return y <= 9999 ? zeroFill(y, 4) : '+' + y;
        });

        addFormatToken(0, ['YY', 2], 0, function () {
            return this.year() % 100;
        });

        addFormatToken(0, ['YYYY', 4], 0, 'year');
        addFormatToken(0, ['YYYYY', 5], 0, 'year');
        addFormatToken(0, ['YYYYYY', 6, true], 0, 'year');

        // ALIASES

        addUnitAlias('year', 'y');

        // PRIORITIES

        addUnitPriority('year', 1);

        // PARSING

        addRegexToken('Y', matchSigned);
        addRegexToken('YY', match1to2, match2);
        addRegexToken('YYYY', match1to4, match4);
        addRegexToken('YYYYY', match1to6, match6);
        addRegexToken('YYYYYY', match1to6, match6);

        addParseToken(['YYYYY', 'YYYYYY'], YEAR);
        addParseToken('YYYY', function (input, array) {
            array[YEAR] =
                input.length === 2 ? hooks.parseTwoDigitYear(input) : toInt(input);
        });
        addParseToken('YY', function (input, array) {
            array[YEAR] = hooks.parseTwoDigitYear(input);
        });
        addParseToken('Y', function (input, array) {
            array[YEAR] = parseInt(input, 10);
        });

        // HELPERS

        function daysInYear(year) {
            return isLeapYear(year) ? 366 : 365;
        }

        // HOOKS

        hooks.parseTwoDigitYear = function (input) {
            return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
        };

        // MOMENTS

        var getSetYear = makeGetSet('FullYear', true);

        function getIsLeapYear() {
            return isLeapYear(this.year());
        }

        function createDate(y, m, d, h, M, s, ms) {
            // can't just apply() to create a date:
            // https://stackoverflow.com/q/181348
            var date;
            // the date constructor remaps years 0-99 to 1900-1999
            if (y < 100 && y >= 0) {
                // preserve leap years using a full 400 year cycle, then reset
                date = new Date(y + 400, m, d, h, M, s, ms);
                if (isFinite(date.getFullYear())) {
                    date.setFullYear(y);
                }
            } else {
                date = new Date(y, m, d, h, M, s, ms);
            }

            return date;
        }

        function createUTCDate(y) {
            var date, args;
            // the Date.UTC function remaps years 0-99 to 1900-1999
            if (y < 100 && y >= 0) {
                args = Array.prototype.slice.call(arguments);
                // preserve leap years using a full 400 year cycle, then reset
                args[0] = y + 400;
                date = new Date(Date.UTC.apply(null, args));
                if (isFinite(date.getUTCFullYear())) {
                    date.setUTCFullYear(y);
                }
            } else {
                date = new Date(Date.UTC.apply(null, arguments));
            }

            return date;
        }

        // start-of-first-week - start-of-year
        function firstWeekOffset(year, dow, doy) {
            var // first-week day -- which january is always in the first week (4 for iso, 1 for other)
                fwd = 7 + dow - doy,
                // first-week day local weekday -- which local weekday is fwd
                fwdlw = (7 + createUTCDate(year, 0, fwd).getUTCDay() - dow) % 7;

            return -fwdlw + fwd - 1;
        }

        // https://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
        function dayOfYearFromWeeks(year, week, weekday, dow, doy) {
            var localWeekday = (7 + weekday - dow) % 7,
                weekOffset = firstWeekOffset(year, dow, doy),
                dayOfYear = 1 + 7 * (week - 1) + localWeekday + weekOffset,
                resYear,
                resDayOfYear;

            if (dayOfYear <= 0) {
                resYear = year - 1;
                resDayOfYear = daysInYear(resYear) + dayOfYear;
            } else if (dayOfYear > daysInYear(year)) {
                resYear = year + 1;
                resDayOfYear = dayOfYear - daysInYear(year);
            } else {
                resYear = year;
                resDayOfYear = dayOfYear;
            }

            return {
                year: resYear,
                dayOfYear: resDayOfYear,
            };
        }

        function weekOfYear(mom, dow, doy) {
            var weekOffset = firstWeekOffset(mom.year(), dow, doy),
                week = Math.floor((mom.dayOfYear() - weekOffset - 1) / 7) + 1,
                resWeek,
                resYear;

            if (week < 1) {
                resYear = mom.year() - 1;
                resWeek = week + weeksInYear(resYear, dow, doy);
            } else if (week > weeksInYear(mom.year(), dow, doy)) {
                resWeek = week - weeksInYear(mom.year(), dow, doy);
                resYear = mom.year() + 1;
            } else {
                resYear = mom.year();
                resWeek = week;
            }

            return {
                week: resWeek,
                year: resYear,
            };
        }

        function weeksInYear(year, dow, doy) {
            var weekOffset = firstWeekOffset(year, dow, doy),
                weekOffsetNext = firstWeekOffset(year + 1, dow, doy);
            return (daysInYear(year) - weekOffset + weekOffsetNext) / 7;
        }

        // FORMATTING

        addFormatToken('w', ['ww', 2], 'wo', 'week');
        addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek');

        // ALIASES

        addUnitAlias('week', 'w');
        addUnitAlias('isoWeek', 'W');

        // PRIORITIES

        addUnitPriority('week', 5);
        addUnitPriority('isoWeek', 5);

        // PARSING

        addRegexToken('w', match1to2);
        addRegexToken('ww', match1to2, match2);
        addRegexToken('W', match1to2);
        addRegexToken('WW', match1to2, match2);

        addWeekParseToken(['w', 'ww', 'W', 'WW'], function (
            input,
            week,
            config,
            token
        ) {
            week[token.substr(0, 1)] = toInt(input);
        });

        // HELPERS

        // LOCALES

        function localeWeek(mom) {
            return weekOfYear(mom, this._week.dow, this._week.doy).week;
        }

        var defaultLocaleWeek = {
            dow: 0, // Sunday is the first day of the week.
            doy: 6, // The week that contains Jan 6th is the first week of the year.
        };

        function localeFirstDayOfWeek() {
            return this._week.dow;
        }

        function localeFirstDayOfYear() {
            return this._week.doy;
        }

        // MOMENTS

        function getSetWeek(input) {
            var week = this.localeData().week(this);
            return input == null ? week : this.add((input - week) * 7, 'd');
        }

        function getSetISOWeek(input) {
            var week = weekOfYear(this, 1, 4).week;
            return input == null ? week : this.add((input - week) * 7, 'd');
        }

        // FORMATTING

        addFormatToken('d', 0, 'do', 'day');

        addFormatToken('dd', 0, 0, function (format) {
            return this.localeData().weekdaysMin(this, format);
        });

        addFormatToken('ddd', 0, 0, function (format) {
            return this.localeData().weekdaysShort(this, format);
        });

        addFormatToken('dddd', 0, 0, function (format) {
            return this.localeData().weekdays(this, format);
        });

        addFormatToken('e', 0, 0, 'weekday');
        addFormatToken('E', 0, 0, 'isoWeekday');

        // ALIASES

        addUnitAlias('day', 'd');
        addUnitAlias('weekday', 'e');
        addUnitAlias('isoWeekday', 'E');

        // PRIORITY
        addUnitPriority('day', 11);
        addUnitPriority('weekday', 11);
        addUnitPriority('isoWeekday', 11);

        // PARSING

        addRegexToken('d', match1to2);
        addRegexToken('e', match1to2);
        addRegexToken('E', match1to2);
        addRegexToken('dd', function (isStrict, locale) {
            return locale.weekdaysMinRegex(isStrict);
        });
        addRegexToken('ddd', function (isStrict, locale) {
            return locale.weekdaysShortRegex(isStrict);
        });
        addRegexToken('dddd', function (isStrict, locale) {
            return locale.weekdaysRegex(isStrict);
        });

        addWeekParseToken(['dd', 'ddd', 'dddd'], function (input, week, config, token) {
            var weekday = config._locale.weekdaysParse(input, token, config._strict);
            // if we didn't get a weekday name, mark the date as invalid
            if (weekday != null) {
                week.d = weekday;
            } else {
                getParsingFlags(config).invalidWeekday = input;
            }
        });

        addWeekParseToken(['d', 'e', 'E'], function (input, week, config, token) {
            week[token] = toInt(input);
        });

        // HELPERS

        function parseWeekday(input, locale) {
            if (typeof input !== 'string') {
                return input;
            }

            if (!isNaN(input)) {
                return parseInt(input, 10);
            }

            input = locale.weekdaysParse(input);
            if (typeof input === 'number') {
                return input;
            }

            return null;
        }

        function parseIsoWeekday(input, locale) {
            if (typeof input === 'string') {
                return locale.weekdaysParse(input) % 7 || 7;
            }
            return isNaN(input) ? null : input;
        }

        // LOCALES
        function shiftWeekdays(ws, n) {
            return ws.slice(n, 7).concat(ws.slice(0, n));
        }

        var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split(
                '_'
            ),
            defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
            defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
            defaultWeekdaysRegex = matchWord,
            defaultWeekdaysShortRegex = matchWord,
            defaultWeekdaysMinRegex = matchWord;

        function localeWeekdays(m, format) {
            var weekdays = isArray(this._weekdays)
                ? this._weekdays
                : this._weekdays[
                      m && m !== true && this._weekdays.isFormat.test(format)
                          ? 'format'
                          : 'standalone'
                  ];
            return m === true
                ? shiftWeekdays(weekdays, this._week.dow)
                : m
                ? weekdays[m.day()]
                : weekdays;
        }

        function localeWeekdaysShort(m) {
            return m === true
                ? shiftWeekdays(this._weekdaysShort, this._week.dow)
                : m
                ? this._weekdaysShort[m.day()]
                : this._weekdaysShort;
        }

        function localeWeekdaysMin(m) {
            return m === true
                ? shiftWeekdays(this._weekdaysMin, this._week.dow)
                : m
                ? this._weekdaysMin[m.day()]
                : this._weekdaysMin;
        }

        function handleStrictParse$1(weekdayName, format, strict) {
            var i,
                ii,
                mom,
                llc = weekdayName.toLocaleLowerCase();
            if (!this._weekdaysParse) {
                this._weekdaysParse = [];
                this._shortWeekdaysParse = [];
                this._minWeekdaysParse = [];

                for (i = 0; i < 7; ++i) {
                    mom = createUTC([2000, 1]).day(i);
                    this._minWeekdaysParse[i] = this.weekdaysMin(
                        mom,
                        ''
                    ).toLocaleLowerCase();
                    this._shortWeekdaysParse[i] = this.weekdaysShort(
                        mom,
                        ''
                    ).toLocaleLowerCase();
                    this._weekdaysParse[i] = this.weekdays(mom, '').toLocaleLowerCase();
                }
            }

            if (strict) {
                if (format === 'dddd') {
                    ii = indexOf.call(this._weekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                } else if (format === 'ddd') {
                    ii = indexOf.call(this._shortWeekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                } else {
                    ii = indexOf.call(this._minWeekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                }
            } else {
                if (format === 'dddd') {
                    ii = indexOf.call(this._weekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._shortWeekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._minWeekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                } else if (format === 'ddd') {
                    ii = indexOf.call(this._shortWeekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._weekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._minWeekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                } else {
                    ii = indexOf.call(this._minWeekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._weekdaysParse, llc);
                    if (ii !== -1) {
                        return ii;
                    }
                    ii = indexOf.call(this._shortWeekdaysParse, llc);
                    return ii !== -1 ? ii : null;
                }
            }
        }

        function localeWeekdaysParse(weekdayName, format, strict) {
            var i, mom, regex;

            if (this._weekdaysParseExact) {
                return handleStrictParse$1.call(this, weekdayName, format, strict);
            }

            if (!this._weekdaysParse) {
                this._weekdaysParse = [];
                this._minWeekdaysParse = [];
                this._shortWeekdaysParse = [];
                this._fullWeekdaysParse = [];
            }

            for (i = 0; i < 7; i++) {
                // make the regex if we don't have it already

                mom = createUTC([2000, 1]).day(i);
                if (strict && !this._fullWeekdaysParse[i]) {
                    this._fullWeekdaysParse[i] = new RegExp(
                        '^' + this.weekdays(mom, '').replace('.', '\\.?') + '$',
                        'i'
                    );
                    this._shortWeekdaysParse[i] = new RegExp(
                        '^' + this.weekdaysShort(mom, '').replace('.', '\\.?') + '$',
                        'i'
                    );
                    this._minWeekdaysParse[i] = new RegExp(
                        '^' + this.weekdaysMin(mom, '').replace('.', '\\.?') + '$',
                        'i'
                    );
                }
                if (!this._weekdaysParse[i]) {
                    regex =
                        '^' +
                        this.weekdays(mom, '') +
                        '|^' +
                        this.weekdaysShort(mom, '') +
                        '|^' +
                        this.weekdaysMin(mom, '');
                    this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
                }
                // test the regex
                if (
                    strict &&
                    format === 'dddd' &&
                    this._fullWeekdaysParse[i].test(weekdayName)
                ) {
                    return i;
                } else if (
                    strict &&
                    format === 'ddd' &&
                    this._shortWeekdaysParse[i].test(weekdayName)
                ) {
                    return i;
                } else if (
                    strict &&
                    format === 'dd' &&
                    this._minWeekdaysParse[i].test(weekdayName)
                ) {
                    return i;
                } else if (!strict && this._weekdaysParse[i].test(weekdayName)) {
                    return i;
                }
            }
        }

        // MOMENTS

        function getSetDayOfWeek(input) {
            if (!this.isValid()) {
                return input != null ? this : NaN;
            }
            var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();
            if (input != null) {
                input = parseWeekday(input, this.localeData());
                return this.add(input - day, 'd');
            } else {
                return day;
            }
        }

        function getSetLocaleDayOfWeek(input) {
            if (!this.isValid()) {
                return input != null ? this : NaN;
            }
            var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
            return input == null ? weekday : this.add(input - weekday, 'd');
        }

        function getSetISODayOfWeek(input) {
            if (!this.isValid()) {
                return input != null ? this : NaN;
            }

            // behaves the same as moment#day except
            // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
            // as a setter, sunday should belong to the previous week.

            if (input != null) {
                var weekday = parseIsoWeekday(input, this.localeData());
                return this.day(this.day() % 7 ? weekday : weekday - 7);
            } else {
                return this.day() || 7;
            }
        }

        function weekdaysRegex(isStrict) {
            if (this._weekdaysParseExact) {
                if (!hasOwnProp(this, '_weekdaysRegex')) {
                    computeWeekdaysParse.call(this);
                }
                if (isStrict) {
                    return this._weekdaysStrictRegex;
                } else {
                    return this._weekdaysRegex;
                }
            } else {
                if (!hasOwnProp(this, '_weekdaysRegex')) {
                    this._weekdaysRegex = defaultWeekdaysRegex;
                }
                return this._weekdaysStrictRegex && isStrict
                    ? this._weekdaysStrictRegex
                    : this._weekdaysRegex;
            }
        }

        function weekdaysShortRegex(isStrict) {
            if (this._weekdaysParseExact) {
                if (!hasOwnProp(this, '_weekdaysRegex')) {
                    computeWeekdaysParse.call(this);
                }
                if (isStrict) {
                    return this._weekdaysShortStrictRegex;
                } else {
                    return this._weekdaysShortRegex;
                }
            } else {
                if (!hasOwnProp(this, '_weekdaysShortRegex')) {
                    this._weekdaysShortRegex = defaultWeekdaysShortRegex;
                }
                return this._weekdaysShortStrictRegex && isStrict
                    ? this._weekdaysShortStrictRegex
                    : this._weekdaysShortRegex;
            }
        }

        function weekdaysMinRegex(isStrict) {
            if (this._weekdaysParseExact) {
                if (!hasOwnProp(this, '_weekdaysRegex')) {
                    computeWeekdaysParse.call(this);
                }
                if (isStrict) {
                    return this._weekdaysMinStrictRegex;
                } else {
                    return this._weekdaysMinRegex;
                }
            } else {
                if (!hasOwnProp(this, '_weekdaysMinRegex')) {
                    this._weekdaysMinRegex = defaultWeekdaysMinRegex;
                }
                return this._weekdaysMinStrictRegex && isStrict
                    ? this._weekdaysMinStrictRegex
                    : this._weekdaysMinRegex;
            }
        }

        function computeWeekdaysParse() {
            function cmpLenRev(a, b) {
                return b.length - a.length;
            }

            var minPieces = [],
                shortPieces = [],
                longPieces = [],
                mixedPieces = [],
                i,
                mom,
                minp,
                shortp,
                longp;
            for (i = 0; i < 7; i++) {
                // make the regex if we don't have it already
                mom = createUTC([2000, 1]).day(i);
                minp = regexEscape(this.weekdaysMin(mom, ''));
                shortp = regexEscape(this.weekdaysShort(mom, ''));
                longp = regexEscape(this.weekdays(mom, ''));
                minPieces.push(minp);
                shortPieces.push(shortp);
                longPieces.push(longp);
                mixedPieces.push(minp);
                mixedPieces.push(shortp);
                mixedPieces.push(longp);
            }
            // Sorting makes sure if one weekday (or abbr) is a prefix of another it
            // will match the longer piece.
            minPieces.sort(cmpLenRev);
            shortPieces.sort(cmpLenRev);
            longPieces.sort(cmpLenRev);
            mixedPieces.sort(cmpLenRev);

            this._weekdaysRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
            this._weekdaysShortRegex = this._weekdaysRegex;
            this._weekdaysMinRegex = this._weekdaysRegex;

            this._weekdaysStrictRegex = new RegExp(
                '^(' + longPieces.join('|') + ')',
                'i'
            );
            this._weekdaysShortStrictRegex = new RegExp(
                '^(' + shortPieces.join('|') + ')',
                'i'
            );
            this._weekdaysMinStrictRegex = new RegExp(
                '^(' + minPieces.join('|') + ')',
                'i'
            );
        }

        // FORMATTING

        function hFormat() {
            return this.hours() % 12 || 12;
        }

        function kFormat() {
            return this.hours() || 24;
        }

        addFormatToken('H', ['HH', 2], 0, 'hour');
        addFormatToken('h', ['hh', 2], 0, hFormat);
        addFormatToken('k', ['kk', 2], 0, kFormat);

        addFormatToken('hmm', 0, 0, function () {
            return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2);
        });

        addFormatToken('hmmss', 0, 0, function () {
            return (
                '' +
                hFormat.apply(this) +
                zeroFill(this.minutes(), 2) +
                zeroFill(this.seconds(), 2)
            );
        });

        addFormatToken('Hmm', 0, 0, function () {
            return '' + this.hours() + zeroFill(this.minutes(), 2);
        });

        addFormatToken('Hmmss', 0, 0, function () {
            return (
                '' +
                this.hours() +
                zeroFill(this.minutes(), 2) +
                zeroFill(this.seconds(), 2)
            );
        });

        function meridiem(token, lowercase) {
            addFormatToken(token, 0, 0, function () {
                return this.localeData().meridiem(
                    this.hours(),
                    this.minutes(),
                    lowercase
                );
            });
        }

        meridiem('a', true);
        meridiem('A', false);

        // ALIASES

        addUnitAlias('hour', 'h');

        // PRIORITY
        addUnitPriority('hour', 13);

        // PARSING

        function matchMeridiem(isStrict, locale) {
            return locale._meridiemParse;
        }

        addRegexToken('a', matchMeridiem);
        addRegexToken('A', matchMeridiem);
        addRegexToken('H', match1to2);
        addRegexToken('h', match1to2);
        addRegexToken('k', match1to2);
        addRegexToken('HH', match1to2, match2);
        addRegexToken('hh', match1to2, match2);
        addRegexToken('kk', match1to2, match2);

        addRegexToken('hmm', match3to4);
        addRegexToken('hmmss', match5to6);
        addRegexToken('Hmm', match3to4);
        addRegexToken('Hmmss', match5to6);

        addParseToken(['H', 'HH'], HOUR);
        addParseToken(['k', 'kk'], function (input, array, config) {
            var kInput = toInt(input);
            array[HOUR] = kInput === 24 ? 0 : kInput;
        });
        addParseToken(['a', 'A'], function (input, array, config) {
            config._isPm = config._locale.isPM(input);
            config._meridiem = input;
        });
        addParseToken(['h', 'hh'], function (input, array, config) {
            array[HOUR] = toInt(input);
            getParsingFlags(config).bigHour = true;
        });
        addParseToken('hmm', function (input, array, config) {
            var pos = input.length - 2;
            array[HOUR] = toInt(input.substr(0, pos));
            array[MINUTE] = toInt(input.substr(pos));
            getParsingFlags(config).bigHour = true;
        });
        addParseToken('hmmss', function (input, array, config) {
            var pos1 = input.length - 4,
                pos2 = input.length - 2;
            array[HOUR] = toInt(input.substr(0, pos1));
            array[MINUTE] = toInt(input.substr(pos1, 2));
            array[SECOND] = toInt(input.substr(pos2));
            getParsingFlags(config).bigHour = true;
        });
        addParseToken('Hmm', function (input, array, config) {
            var pos = input.length - 2;
            array[HOUR] = toInt(input.substr(0, pos));
            array[MINUTE] = toInt(input.substr(pos));
        });
        addParseToken('Hmmss', function (input, array, config) {
            var pos1 = input.length - 4,
                pos2 = input.length - 2;
            array[HOUR] = toInt(input.substr(0, pos1));
            array[MINUTE] = toInt(input.substr(pos1, 2));
            array[SECOND] = toInt(input.substr(pos2));
        });

        // LOCALES

        function localeIsPM(input) {
            // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
            // Using charAt should be more compatible.
            return (input + '').toLowerCase().charAt(0) === 'p';
        }

        var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i,
            // Setting the hour should keep the time, because the user explicitly
            // specified which hour they want. So trying to maintain the same hour (in
            // a new timezone) makes sense. Adding/subtracting hours does not follow
            // this rule.
            getSetHour = makeGetSet('Hours', true);

        function localeMeridiem(hours, minutes, isLower) {
            if (hours > 11) {
                return isLower ? 'pm' : 'PM';
            } else {
                return isLower ? 'am' : 'AM';
            }
        }

        var baseConfig = {
            calendar: defaultCalendar,
            longDateFormat: defaultLongDateFormat,
            invalidDate: defaultInvalidDate,
            ordinal: defaultOrdinal,
            dayOfMonthOrdinalParse: defaultDayOfMonthOrdinalParse,
            relativeTime: defaultRelativeTime,

            months: defaultLocaleMonths,
            monthsShort: defaultLocaleMonthsShort,

            week: defaultLocaleWeek,

            weekdays: defaultLocaleWeekdays,
            weekdaysMin: defaultLocaleWeekdaysMin,
            weekdaysShort: defaultLocaleWeekdaysShort,

            meridiemParse: defaultLocaleMeridiemParse,
        };

        // internal storage for locale config files
        var locales = {},
            localeFamilies = {},
            globalLocale;

        function commonPrefix(arr1, arr2) {
            var i,
                minl = Math.min(arr1.length, arr2.length);
            for (i = 0; i < minl; i += 1) {
                if (arr1[i] !== arr2[i]) {
                    return i;
                }
            }
            return minl;
        }

        function normalizeLocale(key) {
            return key ? key.toLowerCase().replace('_', '-') : key;
        }

        // pick the locale from the array
        // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
        // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
        function chooseLocale(names) {
            var i = 0,
                j,
                next,
                locale,
                split;

            while (i < names.length) {
                split = normalizeLocale(names[i]).split('-');
                j = split.length;
                next = normalizeLocale(names[i + 1]);
                next = next ? next.split('-') : null;
                while (j > 0) {
                    locale = loadLocale(split.slice(0, j).join('-'));
                    if (locale) {
                        return locale;
                    }
                    if (
                        next &&
                        next.length >= j &&
                        commonPrefix(split, next) >= j - 1
                    ) {
                        //the next array item is better than a shallower substring of this one
                        break;
                    }
                    j--;
                }
                i++;
            }
            return globalLocale;
        }

        function loadLocale(name) {
            var oldLocale = null,
                aliasedRequire;
            // TODO: Find a better way to register and load all the locales in Node
            if (
                locales[name] === undefined &&
                'object' !== 'undefined' &&
                module &&
                module.exports
            ) {
                try {
                    oldLocale = globalLocale._abbr;
                    aliasedRequire = commonjsRequire;
                    aliasedRequire('./locale/' + name);
                    getSetGlobalLocale(oldLocale);
                } catch (e) {
                    // mark as not found to avoid repeating expensive file require call causing high CPU
                    // when trying to find en-US, en_US, en-us for every format call
                    locales[name] = null; // null means not found
                }
            }
            return locales[name];
        }

        // This function will load locale and then set the global locale.  If
        // no arguments are passed in, it will simply return the current global
        // locale key.
        function getSetGlobalLocale(key, values) {
            var data;
            if (key) {
                if (isUndefined(values)) {
                    data = getLocale(key);
                } else {
                    data = defineLocale(key, values);
                }

                if (data) {
                    // moment.duration._locale = moment._locale = data;
                    globalLocale = data;
                } else {
                    if (typeof console !== 'undefined' && console.warn) {
                        //warn user if arguments are passed but the locale could not be set
                        console.warn(
                            'Locale ' + key + ' not found. Did you forget to load it?'
                        );
                    }
                }
            }

            return globalLocale._abbr;
        }

        function defineLocale(name, config) {
            if (config !== null) {
                var locale,
                    parentConfig = baseConfig;
                config.abbr = name;
                if (locales[name] != null) {
                    deprecateSimple(
                        'defineLocaleOverride',
                        'use moment.updateLocale(localeName, config) to change ' +
                            'an existing locale. moment.defineLocale(localeName, ' +
                            'config) should only be used for creating a new locale ' +
                            'See http://momentjs.com/guides/#/warnings/define-locale/ for more info.'
                    );
                    parentConfig = locales[name]._config;
                } else if (config.parentLocale != null) {
                    if (locales[config.parentLocale] != null) {
                        parentConfig = locales[config.parentLocale]._config;
                    } else {
                        locale = loadLocale(config.parentLocale);
                        if (locale != null) {
                            parentConfig = locale._config;
                        } else {
                            if (!localeFamilies[config.parentLocale]) {
                                localeFamilies[config.parentLocale] = [];
                            }
                            localeFamilies[config.parentLocale].push({
                                name: name,
                                config: config,
                            });
                            return null;
                        }
                    }
                }
                locales[name] = new Locale(mergeConfigs(parentConfig, config));

                if (localeFamilies[name]) {
                    localeFamilies[name].forEach(function (x) {
                        defineLocale(x.name, x.config);
                    });
                }

                // backwards compat for now: also set the locale
                // make sure we set the locale AFTER all child locales have been
                // created, so we won't end up with the child locale set.
                getSetGlobalLocale(name);

                return locales[name];
            } else {
                // useful for testing
                delete locales[name];
                return null;
            }
        }

        function updateLocale(name, config) {
            if (config != null) {
                var locale,
                    tmpLocale,
                    parentConfig = baseConfig;

                if (locales[name] != null && locales[name].parentLocale != null) {
                    // Update existing child locale in-place to avoid memory-leaks
                    locales[name].set(mergeConfigs(locales[name]._config, config));
                } else {
                    // MERGE
                    tmpLocale = loadLocale(name);
                    if (tmpLocale != null) {
                        parentConfig = tmpLocale._config;
                    }
                    config = mergeConfigs(parentConfig, config);
                    if (tmpLocale == null) {
                        // updateLocale is called for creating a new locale
                        // Set abbr so it will have a name (getters return
                        // undefined otherwise).
                        config.abbr = name;
                    }
                    locale = new Locale(config);
                    locale.parentLocale = locales[name];
                    locales[name] = locale;
                }

                // backwards compat for now: also set the locale
                getSetGlobalLocale(name);
            } else {
                // pass null for config to unupdate, useful for tests
                if (locales[name] != null) {
                    if (locales[name].parentLocale != null) {
                        locales[name] = locales[name].parentLocale;
                        if (name === getSetGlobalLocale()) {
                            getSetGlobalLocale(name);
                        }
                    } else if (locales[name] != null) {
                        delete locales[name];
                    }
                }
            }
            return locales[name];
        }

        // returns locale data
        function getLocale(key) {
            var locale;

            if (key && key._locale && key._locale._abbr) {
                key = key._locale._abbr;
            }

            if (!key) {
                return globalLocale;
            }

            if (!isArray(key)) {
                //short-circuit everything else
                locale = loadLocale(key);
                if (locale) {
                    return locale;
                }
                key = [key];
            }

            return chooseLocale(key);
        }

        function listLocales() {
            return keys(locales);
        }

        function checkOverflow(m) {
            var overflow,
                a = m._a;

            if (a && getParsingFlags(m).overflow === -2) {
                overflow =
                    a[MONTH] < 0 || a[MONTH] > 11
                        ? MONTH
                        : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH])
                        ? DATE
                        : a[HOUR] < 0 ||
                          a[HOUR] > 24 ||
                          (a[HOUR] === 24 &&
                              (a[MINUTE] !== 0 ||
                                  a[SECOND] !== 0 ||
                                  a[MILLISECOND] !== 0))
                        ? HOUR
                        : a[MINUTE] < 0 || a[MINUTE] > 59
                        ? MINUTE
                        : a[SECOND] < 0 || a[SECOND] > 59
                        ? SECOND
                        : a[MILLISECOND] < 0 || a[MILLISECOND] > 999
                        ? MILLISECOND
                        : -1;

                if (
                    getParsingFlags(m)._overflowDayOfYear &&
                    (overflow < YEAR || overflow > DATE)
                ) {
                    overflow = DATE;
                }
                if (getParsingFlags(m)._overflowWeeks && overflow === -1) {
                    overflow = WEEK;
                }
                if (getParsingFlags(m)._overflowWeekday && overflow === -1) {
                    overflow = WEEKDAY;
                }

                getParsingFlags(m).overflow = overflow;
            }

            return m;
        }

        // iso 8601 regex
        // 0000-00-00 0000-W00 or 0000-W00-0 + T + 00 or 00:00 or 00:00:00 or 00:00:00.000 + +00:00 or +0000 or +00)
        var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
            basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
            tzRegex = /Z|[+-]\d\d(?::?\d\d)?/,
            isoDates = [
                ['YYYYYY-MM-DD', /[+-]\d{6}-\d\d-\d\d/],
                ['YYYY-MM-DD', /\d{4}-\d\d-\d\d/],
                ['GGGG-[W]WW-E', /\d{4}-W\d\d-\d/],
                ['GGGG-[W]WW', /\d{4}-W\d\d/, false],
                ['YYYY-DDD', /\d{4}-\d{3}/],
                ['YYYY-MM', /\d{4}-\d\d/, false],
                ['YYYYYYMMDD', /[+-]\d{10}/],
                ['YYYYMMDD', /\d{8}/],
                ['GGGG[W]WWE', /\d{4}W\d{3}/],
                ['GGGG[W]WW', /\d{4}W\d{2}/, false],
                ['YYYYDDD', /\d{7}/],
                ['YYYYMM', /\d{6}/, false],
                ['YYYY', /\d{4}/, false],
            ],
            // iso time formats and regexes
            isoTimes = [
                ['HH:mm:ss.SSSS', /\d\d:\d\d:\d\d\.\d+/],
                ['HH:mm:ss,SSSS', /\d\d:\d\d:\d\d,\d+/],
                ['HH:mm:ss', /\d\d:\d\d:\d\d/],
                ['HH:mm', /\d\d:\d\d/],
                ['HHmmss.SSSS', /\d\d\d\d\d\d\.\d+/],
                ['HHmmss,SSSS', /\d\d\d\d\d\d,\d+/],
                ['HHmmss', /\d\d\d\d\d\d/],
                ['HHmm', /\d\d\d\d/],
                ['HH', /\d\d/],
            ],
            aspNetJsonRegex = /^\/?Date\((-?\d+)/i,
            // RFC 2822 regex: For details see https://tools.ietf.org/html/rfc2822#section-3.3
            rfc2822 = /^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,
            obsOffsets = {
                UT: 0,
                GMT: 0,
                EDT: -4 * 60,
                EST: -5 * 60,
                CDT: -5 * 60,
                CST: -6 * 60,
                MDT: -6 * 60,
                MST: -7 * 60,
                PDT: -7 * 60,
                PST: -8 * 60,
            };

        // date from iso format
        function configFromISO(config) {
            var i,
                l,
                string = config._i,
                match = extendedIsoRegex.exec(string) || basicIsoRegex.exec(string),
                allowTime,
                dateFormat,
                timeFormat,
                tzFormat;

            if (match) {
                getParsingFlags(config).iso = true;

                for (i = 0, l = isoDates.length; i < l; i++) {
                    if (isoDates[i][1].exec(match[1])) {
                        dateFormat = isoDates[i][0];
                        allowTime = isoDates[i][2] !== false;
                        break;
                    }
                }
                if (dateFormat == null) {
                    config._isValid = false;
                    return;
                }
                if (match[3]) {
                    for (i = 0, l = isoTimes.length; i < l; i++) {
                        if (isoTimes[i][1].exec(match[3])) {
                            // match[2] should be 'T' or space
                            timeFormat = (match[2] || ' ') + isoTimes[i][0];
                            break;
                        }
                    }
                    if (timeFormat == null) {
                        config._isValid = false;
                        return;
                    }
                }
                if (!allowTime && timeFormat != null) {
                    config._isValid = false;
                    return;
                }
                if (match[4]) {
                    if (tzRegex.exec(match[4])) {
                        tzFormat = 'Z';
                    } else {
                        config._isValid = false;
                        return;
                    }
                }
                config._f = dateFormat + (timeFormat || '') + (tzFormat || '');
                configFromStringAndFormat(config);
            } else {
                config._isValid = false;
            }
        }

        function extractFromRFC2822Strings(
            yearStr,
            monthStr,
            dayStr,
            hourStr,
            minuteStr,
            secondStr
        ) {
            var result = [
                untruncateYear(yearStr),
                defaultLocaleMonthsShort.indexOf(monthStr),
                parseInt(dayStr, 10),
                parseInt(hourStr, 10),
                parseInt(minuteStr, 10),
            ];

            if (secondStr) {
                result.push(parseInt(secondStr, 10));
            }

            return result;
        }

        function untruncateYear(yearStr) {
            var year = parseInt(yearStr, 10);
            if (year <= 49) {
                return 2000 + year;
            } else if (year <= 999) {
                return 1900 + year;
            }
            return year;
        }

        function preprocessRFC2822(s) {
            // Remove comments and folding whitespace and replace multiple-spaces with a single space
            return s
                .replace(/\([^)]*\)|[\n\t]/g, ' ')
                .replace(/(\s\s+)/g, ' ')
                .replace(/^\s\s*/, '')
                .replace(/\s\s*$/, '');
        }

        function checkWeekday(weekdayStr, parsedInput, config) {
            if (weekdayStr) {
                // TODO: Replace the vanilla JS Date object with an independent day-of-week check.
                var weekdayProvided = defaultLocaleWeekdaysShort.indexOf(weekdayStr),
                    weekdayActual = new Date(
                        parsedInput[0],
                        parsedInput[1],
                        parsedInput[2]
                    ).getDay();
                if (weekdayProvided !== weekdayActual) {
                    getParsingFlags(config).weekdayMismatch = true;
                    config._isValid = false;
                    return false;
                }
            }
            return true;
        }

        function calculateOffset(obsOffset, militaryOffset, numOffset) {
            if (obsOffset) {
                return obsOffsets[obsOffset];
            } else if (militaryOffset) {
                // the only allowed military tz is Z
                return 0;
            } else {
                var hm = parseInt(numOffset, 10),
                    m = hm % 100,
                    h = (hm - m) / 100;
                return h * 60 + m;
            }
        }

        // date and time from ref 2822 format
        function configFromRFC2822(config) {
            var match = rfc2822.exec(preprocessRFC2822(config._i)),
                parsedArray;
            if (match) {
                parsedArray = extractFromRFC2822Strings(
                    match[4],
                    match[3],
                    match[2],
                    match[5],
                    match[6],
                    match[7]
                );
                if (!checkWeekday(match[1], parsedArray, config)) {
                    return;
                }

                config._a = parsedArray;
                config._tzm = calculateOffset(match[8], match[9], match[10]);

                config._d = createUTCDate.apply(null, config._a);
                config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);

                getParsingFlags(config).rfc2822 = true;
            } else {
                config._isValid = false;
            }
        }

        // date from 1) ASP.NET, 2) ISO, 3) RFC 2822 formats, or 4) optional fallback if parsing isn't strict
        function configFromString(config) {
            var matched = aspNetJsonRegex.exec(config._i);
            if (matched !== null) {
                config._d = new Date(+matched[1]);
                return;
            }

            configFromISO(config);
            if (config._isValid === false) {
                delete config._isValid;
            } else {
                return;
            }

            configFromRFC2822(config);
            if (config._isValid === false) {
                delete config._isValid;
            } else {
                return;
            }

            if (config._strict) {
                config._isValid = false;
            } else {
                // Final attempt, use Input Fallback
                hooks.createFromInputFallback(config);
            }
        }

        hooks.createFromInputFallback = deprecate(
            'value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), ' +
                'which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are ' +
                'discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.',
            function (config) {
                config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
            }
        );

        // Pick the first defined of two or three arguments.
        function defaults(a, b, c) {
            if (a != null) {
                return a;
            }
            if (b != null) {
                return b;
            }
            return c;
        }

        function currentDateArray(config) {
            // hooks is actually the exported moment object
            var nowValue = new Date(hooks.now());
            if (config._useUTC) {
                return [
                    nowValue.getUTCFullYear(),
                    nowValue.getUTCMonth(),
                    nowValue.getUTCDate(),
                ];
            }
            return [nowValue.getFullYear(), nowValue.getMonth(), nowValue.getDate()];
        }

        // convert an array to a date.
        // the array should mirror the parameters below
        // note: all values past the year are optional and will default to the lowest possible value.
        // [year, month, day , hour, minute, second, millisecond]
        function configFromArray(config) {
            var i,
                date,
                input = [],
                currentDate,
                expectedWeekday,
                yearToUse;

            if (config._d) {
                return;
            }

            currentDate = currentDateArray(config);

            //compute day of the year from weeks and weekdays
            if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
                dayOfYearFromWeekInfo(config);
            }

            //if the day of the year is set, figure out what it is
            if (config._dayOfYear != null) {
                yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);

                if (
                    config._dayOfYear > daysInYear(yearToUse) ||
                    config._dayOfYear === 0
                ) {
                    getParsingFlags(config)._overflowDayOfYear = true;
                }

                date = createUTCDate(yearToUse, 0, config._dayOfYear);
                config._a[MONTH] = date.getUTCMonth();
                config._a[DATE] = date.getUTCDate();
            }

            // Default to current date.
            // * if no year, month, day of month are given, default to today
            // * if day of month is given, default month and year
            // * if month is given, default only year
            // * if year is given, don't default anything
            for (i = 0; i < 3 && config._a[i] == null; ++i) {
                config._a[i] = input[i] = currentDate[i];
            }

            // Zero out whatever was not defaulted, including time
            for (; i < 7; i++) {
                config._a[i] = input[i] =
                    config._a[i] == null ? (i === 2 ? 1 : 0) : config._a[i];
            }

            // Check for 24:00:00.000
            if (
                config._a[HOUR] === 24 &&
                config._a[MINUTE] === 0 &&
                config._a[SECOND] === 0 &&
                config._a[MILLISECOND] === 0
            ) {
                config._nextDay = true;
                config._a[HOUR] = 0;
            }

            config._d = (config._useUTC ? createUTCDate : createDate).apply(
                null,
                input
            );
            expectedWeekday = config._useUTC
                ? config._d.getUTCDay()
                : config._d.getDay();

            // Apply timezone offset from input. The actual utcOffset can be changed
            // with parseZone.
            if (config._tzm != null) {
                config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
            }

            if (config._nextDay) {
                config._a[HOUR] = 24;
            }

            // check for mismatching day of week
            if (
                config._w &&
                typeof config._w.d !== 'undefined' &&
                config._w.d !== expectedWeekday
            ) {
                getParsingFlags(config).weekdayMismatch = true;
            }
        }

        function dayOfYearFromWeekInfo(config) {
            var w, weekYear, week, weekday, dow, doy, temp, weekdayOverflow, curWeek;

            w = config._w;
            if (w.GG != null || w.W != null || w.E != null) {
                dow = 1;
                doy = 4;

                // TODO: We need to take the current isoWeekYear, but that depends on
                // how we interpret now (local, utc, fixed offset). So create
                // a now version of current config (take local/utc/offset flags, and
                // create now).
                weekYear = defaults(
                    w.GG,
                    config._a[YEAR],
                    weekOfYear(createLocal(), 1, 4).year
                );
                week = defaults(w.W, 1);
                weekday = defaults(w.E, 1);
                if (weekday < 1 || weekday > 7) {
                    weekdayOverflow = true;
                }
            } else {
                dow = config._locale._week.dow;
                doy = config._locale._week.doy;

                curWeek = weekOfYear(createLocal(), dow, doy);

                weekYear = defaults(w.gg, config._a[YEAR], curWeek.year);

                // Default to current week.
                week = defaults(w.w, curWeek.week);

                if (w.d != null) {
                    // weekday -- low day numbers are considered next week
                    weekday = w.d;
                    if (weekday < 0 || weekday > 6) {
                        weekdayOverflow = true;
                    }
                } else if (w.e != null) {
                    // local weekday -- counting starts from beginning of week
                    weekday = w.e + dow;
                    if (w.e < 0 || w.e > 6) {
                        weekdayOverflow = true;
                    }
                } else {
                    // default to beginning of week
                    weekday = dow;
                }
            }
            if (week < 1 || week > weeksInYear(weekYear, dow, doy)) {
                getParsingFlags(config)._overflowWeeks = true;
            } else if (weekdayOverflow != null) {
                getParsingFlags(config)._overflowWeekday = true;
            } else {
                temp = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy);
                config._a[YEAR] = temp.year;
                config._dayOfYear = temp.dayOfYear;
            }
        }

        // constant that refers to the ISO standard
        hooks.ISO_8601 = function () {};

        // constant that refers to the RFC 2822 form
        hooks.RFC_2822 = function () {};

        // date from string and format string
        function configFromStringAndFormat(config) {
            // TODO: Move this to another part of the creation flow to prevent circular deps
            if (config._f === hooks.ISO_8601) {
                configFromISO(config);
                return;
            }
            if (config._f === hooks.RFC_2822) {
                configFromRFC2822(config);
                return;
            }
            config._a = [];
            getParsingFlags(config).empty = true;

            // This array is used to make a Date, either with `new Date` or `Date.UTC`
            var string = '' + config._i,
                i,
                parsedInput,
                tokens,
                token,
                skipped,
                stringLength = string.length,
                totalParsedInputLength = 0,
                era;

            tokens =
                expandFormat(config._f, config._locale).match(formattingTokens) || [];

            for (i = 0; i < tokens.length; i++) {
                token = tokens[i];
                parsedInput = (string.match(getParseRegexForToken(token, config)) ||
                    [])[0];
                if (parsedInput) {
                    skipped = string.substr(0, string.indexOf(parsedInput));
                    if (skipped.length > 0) {
                        getParsingFlags(config).unusedInput.push(skipped);
                    }
                    string = string.slice(
                        string.indexOf(parsedInput) + parsedInput.length
                    );
                    totalParsedInputLength += parsedInput.length;
                }
                // don't parse if it's not a known token
                if (formatTokenFunctions[token]) {
                    if (parsedInput) {
                        getParsingFlags(config).empty = false;
                    } else {
                        getParsingFlags(config).unusedTokens.push(token);
                    }
                    addTimeToArrayFromToken(token, parsedInput, config);
                } else if (config._strict && !parsedInput) {
                    getParsingFlags(config).unusedTokens.push(token);
                }
            }

            // add remaining unparsed input length to the string
            getParsingFlags(config).charsLeftOver =
                stringLength - totalParsedInputLength;
            if (string.length > 0) {
                getParsingFlags(config).unusedInput.push(string);
            }

            // clear _12h flag if hour is <= 12
            if (
                config._a[HOUR] <= 12 &&
                getParsingFlags(config).bigHour === true &&
                config._a[HOUR] > 0
            ) {
                getParsingFlags(config).bigHour = undefined;
            }

            getParsingFlags(config).parsedDateParts = config._a.slice(0);
            getParsingFlags(config).meridiem = config._meridiem;
            // handle meridiem
            config._a[HOUR] = meridiemFixWrap(
                config._locale,
                config._a[HOUR],
                config._meridiem
            );

            // handle era
            era = getParsingFlags(config).era;
            if (era !== null) {
                config._a[YEAR] = config._locale.erasConvertYear(era, config._a[YEAR]);
            }

            configFromArray(config);
            checkOverflow(config);
        }

        function meridiemFixWrap(locale, hour, meridiem) {
            var isPm;

            if (meridiem == null) {
                // nothing to do
                return hour;
            }
            if (locale.meridiemHour != null) {
                return locale.meridiemHour(hour, meridiem);
            } else if (locale.isPM != null) {
                // Fallback
                isPm = locale.isPM(meridiem);
                if (isPm && hour < 12) {
                    hour += 12;
                }
                if (!isPm && hour === 12) {
                    hour = 0;
                }
                return hour;
            } else {
                // this is not supposed to happen
                return hour;
            }
        }

        // date from string and array of format strings
        function configFromStringAndArray(config) {
            var tempConfig,
                bestMoment,
                scoreToBeat,
                i,
                currentScore,
                validFormatFound,
                bestFormatIsValid = false;

            if (config._f.length === 0) {
                getParsingFlags(config).invalidFormat = true;
                config._d = new Date(NaN);
                return;
            }

            for (i = 0; i < config._f.length; i++) {
                currentScore = 0;
                validFormatFound = false;
                tempConfig = copyConfig({}, config);
                if (config._useUTC != null) {
                    tempConfig._useUTC = config._useUTC;
                }
                tempConfig._f = config._f[i];
                configFromStringAndFormat(tempConfig);

                if (isValid(tempConfig)) {
                    validFormatFound = true;
                }

                // if there is any input that was not parsed add a penalty for that format
                currentScore += getParsingFlags(tempConfig).charsLeftOver;

                //or tokens
                currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;

                getParsingFlags(tempConfig).score = currentScore;

                if (!bestFormatIsValid) {
                    if (
                        scoreToBeat == null ||
                        currentScore < scoreToBeat ||
                        validFormatFound
                    ) {
                        scoreToBeat = currentScore;
                        bestMoment = tempConfig;
                        if (validFormatFound) {
                            bestFormatIsValid = true;
                        }
                    }
                } else {
                    if (currentScore < scoreToBeat) {
                        scoreToBeat = currentScore;
                        bestMoment = tempConfig;
                    }
                }
            }

            extend(config, bestMoment || tempConfig);
        }

        function configFromObject(config) {
            if (config._d) {
                return;
            }

            var i = normalizeObjectUnits(config._i),
                dayOrDate = i.day === undefined ? i.date : i.day;
            config._a = map(
                [i.year, i.month, dayOrDate, i.hour, i.minute, i.second, i.millisecond],
                function (obj) {
                    return obj && parseInt(obj, 10);
                }
            );

            configFromArray(config);
        }

        function createFromConfig(config) {
            var res = new Moment(checkOverflow(prepareConfig(config)));
            if (res._nextDay) {
                // Adding is smart enough around DST
                res.add(1, 'd');
                res._nextDay = undefined;
            }

            return res;
        }

        function prepareConfig(config) {
            var input = config._i,
                format = config._f;

            config._locale = config._locale || getLocale(config._l);

            if (input === null || (format === undefined && input === '')) {
                return createInvalid({ nullInput: true });
            }

            if (typeof input === 'string') {
                config._i = input = config._locale.preparse(input);
            }

            if (isMoment(input)) {
                return new Moment(checkOverflow(input));
            } else if (isDate(input)) {
                config._d = input;
            } else if (isArray(format)) {
                configFromStringAndArray(config);
            } else if (format) {
                configFromStringAndFormat(config);
            } else {
                configFromInput(config);
            }

            if (!isValid(config)) {
                config._d = null;
            }

            return config;
        }

        function configFromInput(config) {
            var input = config._i;
            if (isUndefined(input)) {
                config._d = new Date(hooks.now());
            } else if (isDate(input)) {
                config._d = new Date(input.valueOf());
            } else if (typeof input === 'string') {
                configFromString(config);
            } else if (isArray(input)) {
                config._a = map(input.slice(0), function (obj) {
                    return parseInt(obj, 10);
                });
                configFromArray(config);
            } else if (isObject(input)) {
                configFromObject(config);
            } else if (isNumber(input)) {
                // from milliseconds
                config._d = new Date(input);
            } else {
                hooks.createFromInputFallback(config);
            }
        }

        function createLocalOrUTC(input, format, locale, strict, isUTC) {
            var c = {};

            if (format === true || format === false) {
                strict = format;
                format = undefined;
            }

            if (locale === true || locale === false) {
                strict = locale;
                locale = undefined;
            }

            if (
                (isObject(input) && isObjectEmpty(input)) ||
                (isArray(input) && input.length === 0)
            ) {
                input = undefined;
            }
            // object construction must be done this way.
            // https://github.com/moment/moment/issues/1423
            c._isAMomentObject = true;
            c._useUTC = c._isUTC = isUTC;
            c._l = locale;
            c._i = input;
            c._f = format;
            c._strict = strict;

            return createFromConfig(c);
        }

        function createLocal(input, format, locale, strict) {
            return createLocalOrUTC(input, format, locale, strict, false);
        }

        var prototypeMin = deprecate(
                'moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/',
                function () {
                    var other = createLocal.apply(null, arguments);
                    if (this.isValid() && other.isValid()) {
                        return other < this ? this : other;
                    } else {
                        return createInvalid();
                    }
                }
            ),
            prototypeMax = deprecate(
                'moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/',
                function () {
                    var other = createLocal.apply(null, arguments);
                    if (this.isValid() && other.isValid()) {
                        return other > this ? this : other;
                    } else {
                        return createInvalid();
                    }
                }
            );

        // Pick a moment m from moments so that m[fn](other) is true for all
        // other. This relies on the function fn to be transitive.
        //
        // moments should either be an array of moment objects or an array, whose
        // first element is an array of moment objects.
        function pickBy(fn, moments) {
            var res, i;
            if (moments.length === 1 && isArray(moments[0])) {
                moments = moments[0];
            }
            if (!moments.length) {
                return createLocal();
            }
            res = moments[0];
            for (i = 1; i < moments.length; ++i) {
                if (!moments[i].isValid() || moments[i][fn](res)) {
                    res = moments[i];
                }
            }
            return res;
        }

        // TODO: Use [].sort instead?
        function min() {
            var args = [].slice.call(arguments, 0);

            return pickBy('isBefore', args);
        }

        function max() {
            var args = [].slice.call(arguments, 0);

            return pickBy('isAfter', args);
        }

        var now = function () {
            return Date.now ? Date.now() : +new Date();
        };

        var ordering = [
            'year',
            'quarter',
            'month',
            'week',
            'day',
            'hour',
            'minute',
            'second',
            'millisecond',
        ];

        function isDurationValid(m) {
            var key,
                unitHasDecimal = false,
                i;
            for (key in m) {
                if (
                    hasOwnProp(m, key) &&
                    !(
                        indexOf.call(ordering, key) !== -1 &&
                        (m[key] == null || !isNaN(m[key]))
                    )
                ) {
                    return false;
                }
            }

            for (i = 0; i < ordering.length; ++i) {
                if (m[ordering[i]]) {
                    if (unitHasDecimal) {
                        return false; // only allow non-integers for smallest unit
                    }
                    if (parseFloat(m[ordering[i]]) !== toInt(m[ordering[i]])) {
                        unitHasDecimal = true;
                    }
                }
            }

            return true;
        }

        function isValid$1() {
            return this._isValid;
        }

        function createInvalid$1() {
            return createDuration(NaN);
        }

        function Duration(duration) {
            var normalizedInput = normalizeObjectUnits(duration),
                years = normalizedInput.year || 0,
                quarters = normalizedInput.quarter || 0,
                months = normalizedInput.month || 0,
                weeks = normalizedInput.week || normalizedInput.isoWeek || 0,
                days = normalizedInput.day || 0,
                hours = normalizedInput.hour || 0,
                minutes = normalizedInput.minute || 0,
                seconds = normalizedInput.second || 0,
                milliseconds = normalizedInput.millisecond || 0;

            this._isValid = isDurationValid(normalizedInput);

            // representation for dateAddRemove
            this._milliseconds =
                +milliseconds +
                seconds * 1e3 + // 1000
                minutes * 6e4 + // 1000 * 60
                hours * 1000 * 60 * 60; //using 1000 * 60 * 60 instead of 36e5 to avoid floating point rounding errors https://github.com/moment/moment/issues/2978
            // Because of dateAddRemove treats 24 hours as different from a
            // day when working around DST, we need to store them separately
            this._days = +days + weeks * 7;
            // It is impossible to translate months into days without knowing
            // which months you are are talking about, so we have to store
            // it separately.
            this._months = +months + quarters * 3 + years * 12;

            this._data = {};

            this._locale = getLocale();

            this._bubble();
        }

        function isDuration(obj) {
            return obj instanceof Duration;
        }

        function absRound(number) {
            if (number < 0) {
                return Math.round(-1 * number) * -1;
            } else {
                return Math.round(number);
            }
        }

        // compare two arrays, return the number of differences
        function compareArrays(array1, array2, dontConvert) {
            var len = Math.min(array1.length, array2.length),
                lengthDiff = Math.abs(array1.length - array2.length),
                diffs = 0,
                i;
            for (i = 0; i < len; i++) {
                if (
                    (dontConvert && array1[i] !== array2[i]) ||
                    (!dontConvert && toInt(array1[i]) !== toInt(array2[i]))
                ) {
                    diffs++;
                }
            }
            return diffs + lengthDiff;
        }

        // FORMATTING

        function offset(token, separator) {
            addFormatToken(token, 0, 0, function () {
                var offset = this.utcOffset(),
                    sign = '+';
                if (offset < 0) {
                    offset = -offset;
                    sign = '-';
                }
                return (
                    sign +
                    zeroFill(~~(offset / 60), 2) +
                    separator +
                    zeroFill(~~offset % 60, 2)
                );
            });
        }

        offset('Z', ':');
        offset('ZZ', '');

        // PARSING

        addRegexToken('Z', matchShortOffset);
        addRegexToken('ZZ', matchShortOffset);
        addParseToken(['Z', 'ZZ'], function (input, array, config) {
            config._useUTC = true;
            config._tzm = offsetFromString(matchShortOffset, input);
        });

        // HELPERS

        // timezone chunker
        // '+10:00' > ['10',  '00']
        // '-1530'  > ['-15', '30']
        var chunkOffset = /([\+\-]|\d\d)/gi;

        function offsetFromString(matcher, string) {
            var matches = (string || '').match(matcher),
                chunk,
                parts,
                minutes;

            if (matches === null) {
                return null;
            }

            chunk = matches[matches.length - 1] || [];
            parts = (chunk + '').match(chunkOffset) || ['-', 0, 0];
            minutes = +(parts[1] * 60) + toInt(parts[2]);

            return minutes === 0 ? 0 : parts[0] === '+' ? minutes : -minutes;
        }

        // Return a moment from input, that is local/utc/zone equivalent to model.
        function cloneWithOffset(input, model) {
            var res, diff;
            if (model._isUTC) {
                res = model.clone();
                diff =
                    (isMoment(input) || isDate(input)
                        ? input.valueOf()
                        : createLocal(input).valueOf()) - res.valueOf();
                // Use low-level api, because this fn is low-level api.
                res._d.setTime(res._d.valueOf() + diff);
                hooks.updateOffset(res, false);
                return res;
            } else {
                return createLocal(input).local();
            }
        }

        function getDateOffset(m) {
            // On Firefox.24 Date#getTimezoneOffset returns a floating point.
            // https://github.com/moment/moment/pull/1871
            return -Math.round(m._d.getTimezoneOffset());
        }

        // HOOKS

        // This function will be called whenever a moment is mutated.
        // It is intended to keep the offset in sync with the timezone.
        hooks.updateOffset = function () {};

        // MOMENTS

        // keepLocalTime = true means only change the timezone, without
        // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
        // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
        // +0200, so we adjust the time as needed, to be valid.
        //
        // Keeping the time actually adds/subtracts (one hour)
        // from the actual represented time. That is why we call updateOffset
        // a second time. In case it wants us to change the offset again
        // _changeInProgress == true case, then we have to adjust, because
        // there is no such time in the given timezone.
        function getSetOffset(input, keepLocalTime, keepMinutes) {
            var offset = this._offset || 0,
                localAdjust;
            if (!this.isValid()) {
                return input != null ? this : NaN;
            }
            if (input != null) {
                if (typeof input === 'string') {
                    input = offsetFromString(matchShortOffset, input);
                    if (input === null) {
                        return this;
                    }
                } else if (Math.abs(input) < 16 && !keepMinutes) {
                    input = input * 60;
                }
                if (!this._isUTC && keepLocalTime) {
                    localAdjust = getDateOffset(this);
                }
                this._offset = input;
                this._isUTC = true;
                if (localAdjust != null) {
                    this.add(localAdjust, 'm');
                }
                if (offset !== input) {
                    if (!keepLocalTime || this._changeInProgress) {
                        addSubtract(
                            this,
                            createDuration(input - offset, 'm'),
                            1,
                            false
                        );
                    } else if (!this._changeInProgress) {
                        this._changeInProgress = true;
                        hooks.updateOffset(this, true);
                        this._changeInProgress = null;
                    }
                }
                return this;
            } else {
                return this._isUTC ? offset : getDateOffset(this);
            }
        }

        function getSetZone(input, keepLocalTime) {
            if (input != null) {
                if (typeof input !== 'string') {
                    input = -input;
                }

                this.utcOffset(input, keepLocalTime);

                return this;
            } else {
                return -this.utcOffset();
            }
        }

        function setOffsetToUTC(keepLocalTime) {
            return this.utcOffset(0, keepLocalTime);
        }

        function setOffsetToLocal(keepLocalTime) {
            if (this._isUTC) {
                this.utcOffset(0, keepLocalTime);
                this._isUTC = false;

                if (keepLocalTime) {
                    this.subtract(getDateOffset(this), 'm');
                }
            }
            return this;
        }

        function setOffsetToParsedOffset() {
            if (this._tzm != null) {
                this.utcOffset(this._tzm, false, true);
            } else if (typeof this._i === 'string') {
                var tZone = offsetFromString(matchOffset, this._i);
                if (tZone != null) {
                    this.utcOffset(tZone);
                } else {
                    this.utcOffset(0, true);
                }
            }
            return this;
        }

        function hasAlignedHourOffset(input) {
            if (!this.isValid()) {
                return false;
            }
            input = input ? createLocal(input).utcOffset() : 0;

            return (this.utcOffset() - input) % 60 === 0;
        }

        function isDaylightSavingTime() {
            return (
                this.utcOffset() > this.clone().month(0).utcOffset() ||
                this.utcOffset() > this.clone().month(5).utcOffset()
            );
        }

        function isDaylightSavingTimeShifted() {
            if (!isUndefined(this._isDSTShifted)) {
                return this._isDSTShifted;
            }

            var c = {},
                other;

            copyConfig(c, this);
            c = prepareConfig(c);

            if (c._a) {
                other = c._isUTC ? createUTC(c._a) : createLocal(c._a);
                this._isDSTShifted =
                    this.isValid() && compareArrays(c._a, other.toArray()) > 0;
            } else {
                this._isDSTShifted = false;
            }

            return this._isDSTShifted;
        }

        function isLocal() {
            return this.isValid() ? !this._isUTC : false;
        }

        function isUtcOffset() {
            return this.isValid() ? this._isUTC : false;
        }

        function isUtc() {
            return this.isValid() ? this._isUTC && this._offset === 0 : false;
        }

        // ASP.NET json date format regex
        var aspNetRegex = /^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,
            // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
            // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
            // and further modified to allow for strings containing both week and day
            isoRegex = /^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;

        function createDuration(input, key) {
            var duration = input,
                // matching against regexp is expensive, do it on demand
                match = null,
                sign,
                ret,
                diffRes;

            if (isDuration(input)) {
                duration = {
                    ms: input._milliseconds,
                    d: input._days,
                    M: input._months,
                };
            } else if (isNumber(input) || !isNaN(+input)) {
                duration = {};
                if (key) {
                    duration[key] = +input;
                } else {
                    duration.milliseconds = +input;
                }
            } else if ((match = aspNetRegex.exec(input))) {
                sign = match[1] === '-' ? -1 : 1;
                duration = {
                    y: 0,
                    d: toInt(match[DATE]) * sign,
                    h: toInt(match[HOUR]) * sign,
                    m: toInt(match[MINUTE]) * sign,
                    s: toInt(match[SECOND]) * sign,
                    ms: toInt(absRound(match[MILLISECOND] * 1000)) * sign, // the millisecond decimal point is included in the match
                };
            } else if ((match = isoRegex.exec(input))) {
                sign = match[1] === '-' ? -1 : 1;
                duration = {
                    y: parseIso(match[2], sign),
                    M: parseIso(match[3], sign),
                    w: parseIso(match[4], sign),
                    d: parseIso(match[5], sign),
                    h: parseIso(match[6], sign),
                    m: parseIso(match[7], sign),
                    s: parseIso(match[8], sign),
                };
            } else if (duration == null) {
                // checks for null or undefined
                duration = {};
            } else if (
                typeof duration === 'object' &&
                ('from' in duration || 'to' in duration)
            ) {
                diffRes = momentsDifference(
                    createLocal(duration.from),
                    createLocal(duration.to)
                );

                duration = {};
                duration.ms = diffRes.milliseconds;
                duration.M = diffRes.months;
            }

            ret = new Duration(duration);

            if (isDuration(input) && hasOwnProp(input, '_locale')) {
                ret._locale = input._locale;
            }

            if (isDuration(input) && hasOwnProp(input, '_isValid')) {
                ret._isValid = input._isValid;
            }

            return ret;
        }

        createDuration.fn = Duration.prototype;
        createDuration.invalid = createInvalid$1;

        function parseIso(inp, sign) {
            // We'd normally use ~~inp for this, but unfortunately it also
            // converts floats to ints.
            // inp may be undefined, so careful calling replace on it.
            var res = inp && parseFloat(inp.replace(',', '.'));
            // apply sign while we're at it
            return (isNaN(res) ? 0 : res) * sign;
        }

        function positiveMomentsDifference(base, other) {
            var res = {};

            res.months =
                other.month() - base.month() + (other.year() - base.year()) * 12;
            if (base.clone().add(res.months, 'M').isAfter(other)) {
                --res.months;
            }

            res.milliseconds = +other - +base.clone().add(res.months, 'M');

            return res;
        }

        function momentsDifference(base, other) {
            var res;
            if (!(base.isValid() && other.isValid())) {
                return { milliseconds: 0, months: 0 };
            }

            other = cloneWithOffset(other, base);
            if (base.isBefore(other)) {
                res = positiveMomentsDifference(base, other);
            } else {
                res = positiveMomentsDifference(other, base);
                res.milliseconds = -res.milliseconds;
                res.months = -res.months;
            }

            return res;
        }

        // TODO: remove 'name' arg after deprecation is removed
        function createAdder(direction, name) {
            return function (val, period) {
                var dur, tmp;
                //invert the arguments, but complain about it
                if (period !== null && !isNaN(+period)) {
                    deprecateSimple(
                        name,
                        'moment().' +
                            name +
                            '(period, number) is deprecated. Please use moment().' +
                            name +
                            '(number, period). ' +
                            'See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info.'
                    );
                    tmp = val;
                    val = period;
                    period = tmp;
                }

                dur = createDuration(val, period);
                addSubtract(this, dur, direction);
                return this;
            };
        }

        function addSubtract(mom, duration, isAdding, updateOffset) {
            var milliseconds = duration._milliseconds,
                days = absRound(duration._days),
                months = absRound(duration._months);

            if (!mom.isValid()) {
                // No op
                return;
            }

            updateOffset = updateOffset == null ? true : updateOffset;

            if (months) {
                setMonth(mom, get(mom, 'Month') + months * isAdding);
            }
            if (days) {
                set$1(mom, 'Date', get(mom, 'Date') + days * isAdding);
            }
            if (milliseconds) {
                mom._d.setTime(mom._d.valueOf() + milliseconds * isAdding);
            }
            if (updateOffset) {
                hooks.updateOffset(mom, days || months);
            }
        }

        var add = createAdder(1, 'add'),
            subtract = createAdder(-1, 'subtract');

        function isString(input) {
            return typeof input === 'string' || input instanceof String;
        }

        // type MomentInput = Moment | Date | string | number | (number | string)[] | MomentInputObject | void; // null | undefined
        function isMomentInput(input) {
            return (
                isMoment(input) ||
                isDate(input) ||
                isString(input) ||
                isNumber(input) ||
                isNumberOrStringArray(input) ||
                isMomentInputObject(input) ||
                input === null ||
                input === undefined
            );
        }

        function isMomentInputObject(input) {
            var objectTest = isObject(input) && !isObjectEmpty(input),
                propertyTest = false,
                properties = [
                    'years',
                    'year',
                    'y',
                    'months',
                    'month',
                    'M',
                    'days',
                    'day',
                    'd',
                    'dates',
                    'date',
                    'D',
                    'hours',
                    'hour',
                    'h',
                    'minutes',
                    'minute',
                    'm',
                    'seconds',
                    'second',
                    's',
                    'milliseconds',
                    'millisecond',
                    'ms',
                ],
                i,
                property;

            for (i = 0; i < properties.length; i += 1) {
                property = properties[i];
                propertyTest = propertyTest || hasOwnProp(input, property);
            }

            return objectTest && propertyTest;
        }

        function isNumberOrStringArray(input) {
            var arrayTest = isArray(input),
                dataTypeTest = false;
            if (arrayTest) {
                dataTypeTest =
                    input.filter(function (item) {
                        return !isNumber(item) && isString(input);
                    }).length === 0;
            }
            return arrayTest && dataTypeTest;
        }

        function isCalendarSpec(input) {
            var objectTest = isObject(input) && !isObjectEmpty(input),
                propertyTest = false,
                properties = [
                    'sameDay',
                    'nextDay',
                    'lastDay',
                    'nextWeek',
                    'lastWeek',
                    'sameElse',
                ],
                i,
                property;

            for (i = 0; i < properties.length; i += 1) {
                property = properties[i];
                propertyTest = propertyTest || hasOwnProp(input, property);
            }

            return objectTest && propertyTest;
        }

        function getCalendarFormat(myMoment, now) {
            var diff = myMoment.diff(now, 'days', true);
            return diff < -6
                ? 'sameElse'
                : diff < -1
                ? 'lastWeek'
                : diff < 0
                ? 'lastDay'
                : diff < 1
                ? 'sameDay'
                : diff < 2
                ? 'nextDay'
                : diff < 7
                ? 'nextWeek'
                : 'sameElse';
        }

        function calendar$1(time, formats) {
            // Support for single parameter, formats only overload to the calendar function
            if (arguments.length === 1) {
                if (!arguments[0]) {
                    time = undefined;
                    formats = undefined;
                } else if (isMomentInput(arguments[0])) {
                    time = arguments[0];
                    formats = undefined;
                } else if (isCalendarSpec(arguments[0])) {
                    formats = arguments[0];
                    time = undefined;
                }
            }
            // We want to compare the start of today, vs this.
            // Getting start-of-today depends on whether we're local/utc/offset or not.
            var now = time || createLocal(),
                sod = cloneWithOffset(now, this).startOf('day'),
                format = hooks.calendarFormat(this, sod) || 'sameElse',
                output =
                    formats &&
                    (isFunction(formats[format])
                        ? formats[format].call(this, now)
                        : formats[format]);

            return this.format(
                output || this.localeData().calendar(format, this, createLocal(now))
            );
        }

        function clone() {
            return new Moment(this);
        }

        function isAfter(input, units) {
            var localInput = isMoment(input) ? input : createLocal(input);
            if (!(this.isValid() && localInput.isValid())) {
                return false;
            }
            units = normalizeUnits(units) || 'millisecond';
            if (units === 'millisecond') {
                return this.valueOf() > localInput.valueOf();
            } else {
                return localInput.valueOf() < this.clone().startOf(units).valueOf();
            }
        }

        function isBefore(input, units) {
            var localInput = isMoment(input) ? input : createLocal(input);
            if (!(this.isValid() && localInput.isValid())) {
                return false;
            }
            units = normalizeUnits(units) || 'millisecond';
            if (units === 'millisecond') {
                return this.valueOf() < localInput.valueOf();
            } else {
                return this.clone().endOf(units).valueOf() < localInput.valueOf();
            }
        }

        function isBetween(from, to, units, inclusivity) {
            var localFrom = isMoment(from) ? from : createLocal(from),
                localTo = isMoment(to) ? to : createLocal(to);
            if (!(this.isValid() && localFrom.isValid() && localTo.isValid())) {
                return false;
            }
            inclusivity = inclusivity || '()';
            return (
                (inclusivity[0] === '('
                    ? this.isAfter(localFrom, units)
                    : !this.isBefore(localFrom, units)) &&
                (inclusivity[1] === ')'
                    ? this.isBefore(localTo, units)
                    : !this.isAfter(localTo, units))
            );
        }

        function isSame(input, units) {
            var localInput = isMoment(input) ? input : createLocal(input),
                inputMs;
            if (!(this.isValid() && localInput.isValid())) {
                return false;
            }
            units = normalizeUnits(units) || 'millisecond';
            if (units === 'millisecond') {
                return this.valueOf() === localInput.valueOf();
            } else {
                inputMs = localInput.valueOf();
                return (
                    this.clone().startOf(units).valueOf() <= inputMs &&
                    inputMs <= this.clone().endOf(units).valueOf()
                );
            }
        }

        function isSameOrAfter(input, units) {
            return this.isSame(input, units) || this.isAfter(input, units);
        }

        function isSameOrBefore(input, units) {
            return this.isSame(input, units) || this.isBefore(input, units);
        }

        function diff(input, units, asFloat) {
            var that, zoneDelta, output;

            if (!this.isValid()) {
                return NaN;
            }

            that = cloneWithOffset(input, this);

            if (!that.isValid()) {
                return NaN;
            }

            zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4;

            units = normalizeUnits(units);

            switch (units) {
                case 'year':
                    output = monthDiff(this, that) / 12;
                    break;
                case 'month':
                    output = monthDiff(this, that);
                    break;
                case 'quarter':
                    output = monthDiff(this, that) / 3;
                    break;
                case 'second':
                    output = (this - that) / 1e3;
                    break; // 1000
                case 'minute':
                    output = (this - that) / 6e4;
                    break; // 1000 * 60
                case 'hour':
                    output = (this - that) / 36e5;
                    break; // 1000 * 60 * 60
                case 'day':
                    output = (this - that - zoneDelta) / 864e5;
                    break; // 1000 * 60 * 60 * 24, negate dst
                case 'week':
                    output = (this - that - zoneDelta) / 6048e5;
                    break; // 1000 * 60 * 60 * 24 * 7, negate dst
                default:
                    output = this - that;
            }

            return asFloat ? output : absFloor(output);
        }

        function monthDiff(a, b) {
            if (a.date() < b.date()) {
                // end-of-month calculations work correct when the start month has more
                // days than the end month.
                return -monthDiff(b, a);
            }
            // difference in months
            var wholeMonthDiff = (b.year() - a.year()) * 12 + (b.month() - a.month()),
                // b is in (anchor - 1 month, anchor + 1 month)
                anchor = a.clone().add(wholeMonthDiff, 'months'),
                anchor2,
                adjust;

            if (b - anchor < 0) {
                anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
                // linear across the month
                adjust = (b - anchor) / (anchor - anchor2);
            } else {
                anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
                // linear across the month
                adjust = (b - anchor) / (anchor2 - anchor);
            }

            //check for negative zero, return zero if negative zero
            return -(wholeMonthDiff + adjust) || 0;
        }

        hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
        hooks.defaultFormatUtc = 'YYYY-MM-DDTHH:mm:ss[Z]';

        function toString() {
            return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
        }

        function toISOString(keepOffset) {
            if (!this.isValid()) {
                return null;
            }
            var utc = keepOffset !== true,
                m = utc ? this.clone().utc() : this;
            if (m.year() < 0 || m.year() > 9999) {
                return formatMoment(
                    m,
                    utc
                        ? 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]'
                        : 'YYYYYY-MM-DD[T]HH:mm:ss.SSSZ'
                );
            }
            if (isFunction(Date.prototype.toISOString)) {
                // native implementation is ~50x faster, use it when we can
                if (utc) {
                    return this.toDate().toISOString();
                } else {
                    return new Date(this.valueOf() + this.utcOffset() * 60 * 1000)
                        .toISOString()
                        .replace('Z', formatMoment(m, 'Z'));
                }
            }
            return formatMoment(
                m,
                utc ? 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYY-MM-DD[T]HH:mm:ss.SSSZ'
            );
        }

        /**
         * Return a human readable representation of a moment that can
         * also be evaluated to get a new moment which is the same
         *
         * @link https://nodejs.org/dist/latest/docs/api/util.html#util_custom_inspect_function_on_objects
         */
        function inspect() {
            if (!this.isValid()) {
                return 'moment.invalid(/* ' + this._i + ' */)';
            }
            var func = 'moment',
                zone = '',
                prefix,
                year,
                datetime,
                suffix;
            if (!this.isLocal()) {
                func = this.utcOffset() === 0 ? 'moment.utc' : 'moment.parseZone';
                zone = 'Z';
            }
            prefix = '[' + func + '("]';
            year = 0 <= this.year() && this.year() <= 9999 ? 'YYYY' : 'YYYYYY';
            datetime = '-MM-DD[T]HH:mm:ss.SSS';
            suffix = zone + '[")]';

            return this.format(prefix + year + datetime + suffix);
        }

        function format(inputString) {
            if (!inputString) {
                inputString = this.isUtc()
                    ? hooks.defaultFormatUtc
                    : hooks.defaultFormat;
            }
            var output = formatMoment(this, inputString);
            return this.localeData().postformat(output);
        }

        function from(time, withoutSuffix) {
            if (
                this.isValid() &&
                ((isMoment(time) && time.isValid()) || createLocal(time).isValid())
            ) {
                return createDuration({ to: this, from: time })
                    .locale(this.locale())
                    .humanize(!withoutSuffix);
            } else {
                return this.localeData().invalidDate();
            }
        }

        function fromNow(withoutSuffix) {
            return this.from(createLocal(), withoutSuffix);
        }

        function to(time, withoutSuffix) {
            if (
                this.isValid() &&
                ((isMoment(time) && time.isValid()) || createLocal(time).isValid())
            ) {
                return createDuration({ from: this, to: time })
                    .locale(this.locale())
                    .humanize(!withoutSuffix);
            } else {
                return this.localeData().invalidDate();
            }
        }

        function toNow(withoutSuffix) {
            return this.to(createLocal(), withoutSuffix);
        }

        // If passed a locale key, it will set the locale for this
        // instance.  Otherwise, it will return the locale configuration
        // variables for this instance.
        function locale(key) {
            var newLocaleData;

            if (key === undefined) {
                return this._locale._abbr;
            } else {
                newLocaleData = getLocale(key);
                if (newLocaleData != null) {
                    this._locale = newLocaleData;
                }
                return this;
            }
        }

        var lang = deprecate(
            'moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.',
            function (key) {
                if (key === undefined) {
                    return this.localeData();
                } else {
                    return this.locale(key);
                }
            }
        );

        function localeData() {
            return this._locale;
        }

        var MS_PER_SECOND = 1000,
            MS_PER_MINUTE = 60 * MS_PER_SECOND,
            MS_PER_HOUR = 60 * MS_PER_MINUTE,
            MS_PER_400_YEARS = (365 * 400 + 97) * 24 * MS_PER_HOUR;

        // actual modulo - handles negative numbers (for dates before 1970):
        function mod$1(dividend, divisor) {
            return ((dividend % divisor) + divisor) % divisor;
        }

        function localStartOfDate(y, m, d) {
            // the date constructor remaps years 0-99 to 1900-1999
            if (y < 100 && y >= 0) {
                // preserve leap years using a full 400 year cycle, then reset
                return new Date(y + 400, m, d) - MS_PER_400_YEARS;
            } else {
                return new Date(y, m, d).valueOf();
            }
        }

        function utcStartOfDate(y, m, d) {
            // Date.UTC remaps years 0-99 to 1900-1999
            if (y < 100 && y >= 0) {
                // preserve leap years using a full 400 year cycle, then reset
                return Date.UTC(y + 400, m, d) - MS_PER_400_YEARS;
            } else {
                return Date.UTC(y, m, d);
            }
        }

        function startOf(units) {
            var time, startOfDate;
            units = normalizeUnits(units);
            if (units === undefined || units === 'millisecond' || !this.isValid()) {
                return this;
            }

            startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;

            switch (units) {
                case 'year':
                    time = startOfDate(this.year(), 0, 1);
                    break;
                case 'quarter':
                    time = startOfDate(
                        this.year(),
                        this.month() - (this.month() % 3),
                        1
                    );
                    break;
                case 'month':
                    time = startOfDate(this.year(), this.month(), 1);
                    break;
                case 'week':
                    time = startOfDate(
                        this.year(),
                        this.month(),
                        this.date() - this.weekday()
                    );
                    break;
                case 'isoWeek':
                    time = startOfDate(
                        this.year(),
                        this.month(),
                        this.date() - (this.isoWeekday() - 1)
                    );
                    break;
                case 'day':
                case 'date':
                    time = startOfDate(this.year(), this.month(), this.date());
                    break;
                case 'hour':
                    time = this._d.valueOf();
                    time -= mod$1(
                        time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE),
                        MS_PER_HOUR
                    );
                    break;
                case 'minute':
                    time = this._d.valueOf();
                    time -= mod$1(time, MS_PER_MINUTE);
                    break;
                case 'second':
                    time = this._d.valueOf();
                    time -= mod$1(time, MS_PER_SECOND);
                    break;
            }

            this._d.setTime(time);
            hooks.updateOffset(this, true);
            return this;
        }

        function endOf(units) {
            var time, startOfDate;
            units = normalizeUnits(units);
            if (units === undefined || units === 'millisecond' || !this.isValid()) {
                return this;
            }

            startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;

            switch (units) {
                case 'year':
                    time = startOfDate(this.year() + 1, 0, 1) - 1;
                    break;
                case 'quarter':
                    time =
                        startOfDate(
                            this.year(),
                            this.month() - (this.month() % 3) + 3,
                            1
                        ) - 1;
                    break;
                case 'month':
                    time = startOfDate(this.year(), this.month() + 1, 1) - 1;
                    break;
                case 'week':
                    time =
                        startOfDate(
                            this.year(),
                            this.month(),
                            this.date() - this.weekday() + 7
                        ) - 1;
                    break;
                case 'isoWeek':
                    time =
                        startOfDate(
                            this.year(),
                            this.month(),
                            this.date() - (this.isoWeekday() - 1) + 7
                        ) - 1;
                    break;
                case 'day':
                case 'date':
                    time = startOfDate(this.year(), this.month(), this.date() + 1) - 1;
                    break;
                case 'hour':
                    time = this._d.valueOf();
                    time +=
                        MS_PER_HOUR -
                        mod$1(
                            time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE),
                            MS_PER_HOUR
                        ) -
                        1;
                    break;
                case 'minute':
                    time = this._d.valueOf();
                    time += MS_PER_MINUTE - mod$1(time, MS_PER_MINUTE) - 1;
                    break;
                case 'second':
                    time = this._d.valueOf();
                    time += MS_PER_SECOND - mod$1(time, MS_PER_SECOND) - 1;
                    break;
            }

            this._d.setTime(time);
            hooks.updateOffset(this, true);
            return this;
        }

        function valueOf() {
            return this._d.valueOf() - (this._offset || 0) * 60000;
        }

        function unix() {
            return Math.floor(this.valueOf() / 1000);
        }

        function toDate() {
            return new Date(this.valueOf());
        }

        function toArray() {
            var m = this;
            return [
                m.year(),
                m.month(),
                m.date(),
                m.hour(),
                m.minute(),
                m.second(),
                m.millisecond(),
            ];
        }

        function toObject() {
            var m = this;
            return {
                years: m.year(),
                months: m.month(),
                date: m.date(),
                hours: m.hours(),
                minutes: m.minutes(),
                seconds: m.seconds(),
                milliseconds: m.milliseconds(),
            };
        }

        function toJSON() {
            // new Date(NaN).toJSON() === null
            return this.isValid() ? this.toISOString() : null;
        }

        function isValid$2() {
            return isValid(this);
        }

        function parsingFlags() {
            return extend({}, getParsingFlags(this));
        }

        function invalidAt() {
            return getParsingFlags(this).overflow;
        }

        function creationData() {
            return {
                input: this._i,
                format: this._f,
                locale: this._locale,
                isUTC: this._isUTC,
                strict: this._strict,
            };
        }

        addFormatToken('N', 0, 0, 'eraAbbr');
        addFormatToken('NN', 0, 0, 'eraAbbr');
        addFormatToken('NNN', 0, 0, 'eraAbbr');
        addFormatToken('NNNN', 0, 0, 'eraName');
        addFormatToken('NNNNN', 0, 0, 'eraNarrow');

        addFormatToken('y', ['y', 1], 'yo', 'eraYear');
        addFormatToken('y', ['yy', 2], 0, 'eraYear');
        addFormatToken('y', ['yyy', 3], 0, 'eraYear');
        addFormatToken('y', ['yyyy', 4], 0, 'eraYear');

        addRegexToken('N', matchEraAbbr);
        addRegexToken('NN', matchEraAbbr);
        addRegexToken('NNN', matchEraAbbr);
        addRegexToken('NNNN', matchEraName);
        addRegexToken('NNNNN', matchEraNarrow);

        addParseToken(['N', 'NN', 'NNN', 'NNNN', 'NNNNN'], function (
            input,
            array,
            config,
            token
        ) {
            var era = config._locale.erasParse(input, token, config._strict);
            if (era) {
                getParsingFlags(config).era = era;
            } else {
                getParsingFlags(config).invalidEra = input;
            }
        });

        addRegexToken('y', matchUnsigned);
        addRegexToken('yy', matchUnsigned);
        addRegexToken('yyy', matchUnsigned);
        addRegexToken('yyyy', matchUnsigned);
        addRegexToken('yo', matchEraYearOrdinal);

        addParseToken(['y', 'yy', 'yyy', 'yyyy'], YEAR);
        addParseToken(['yo'], function (input, array, config, token) {
            var match;
            if (config._locale._eraYearOrdinalRegex) {
                match = input.match(config._locale._eraYearOrdinalRegex);
            }

            if (config._locale.eraYearOrdinalParse) {
                array[YEAR] = config._locale.eraYearOrdinalParse(input, match);
            } else {
                array[YEAR] = parseInt(input, 10);
            }
        });

        function localeEras(m, format) {
            var i,
                l,
                date,
                eras = this._eras || getLocale('en')._eras;
            for (i = 0, l = eras.length; i < l; ++i) {
                switch (typeof eras[i].since) {
                    case 'string':
                        // truncate time
                        date = hooks(eras[i].since).startOf('day');
                        eras[i].since = date.valueOf();
                        break;
                }

                switch (typeof eras[i].until) {
                    case 'undefined':
                        eras[i].until = +Infinity;
                        break;
                    case 'string':
                        // truncate time
                        date = hooks(eras[i].until).startOf('day').valueOf();
                        eras[i].until = date.valueOf();
                        break;
                }
            }
            return eras;
        }

        function localeErasParse(eraName, format, strict) {
            var i,
                l,
                eras = this.eras(),
                name,
                abbr,
                narrow;
            eraName = eraName.toUpperCase();

            for (i = 0, l = eras.length; i < l; ++i) {
                name = eras[i].name.toUpperCase();
                abbr = eras[i].abbr.toUpperCase();
                narrow = eras[i].narrow.toUpperCase();

                if (strict) {
                    switch (format) {
                        case 'N':
                        case 'NN':
                        case 'NNN':
                            if (abbr === eraName) {
                                return eras[i];
                            }
                            break;

                        case 'NNNN':
                            if (name === eraName) {
                                return eras[i];
                            }
                            break;

                        case 'NNNNN':
                            if (narrow === eraName) {
                                return eras[i];
                            }
                            break;
                    }
                } else if ([name, abbr, narrow].indexOf(eraName) >= 0) {
                    return eras[i];
                }
            }
        }

        function localeErasConvertYear(era, year) {
            var dir = era.since <= era.until ? +1 : -1;
            if (year === undefined) {
                return hooks(era.since).year();
            } else {
                return hooks(era.since).year() + (year - era.offset) * dir;
            }
        }

        function getEraName() {
            var i,
                l,
                val,
                eras = this.localeData().eras();
            for (i = 0, l = eras.length; i < l; ++i) {
                // truncate time
                val = this.clone().startOf('day').valueOf();

                if (eras[i].since <= val && val <= eras[i].until) {
                    return eras[i].name;
                }
                if (eras[i].until <= val && val <= eras[i].since) {
                    return eras[i].name;
                }
            }

            return '';
        }

        function getEraNarrow() {
            var i,
                l,
                val,
                eras = this.localeData().eras();
            for (i = 0, l = eras.length; i < l; ++i) {
                // truncate time
                val = this.clone().startOf('day').valueOf();

                if (eras[i].since <= val && val <= eras[i].until) {
                    return eras[i].narrow;
                }
                if (eras[i].until <= val && val <= eras[i].since) {
                    return eras[i].narrow;
                }
            }

            return '';
        }

        function getEraAbbr() {
            var i,
                l,
                val,
                eras = this.localeData().eras();
            for (i = 0, l = eras.length; i < l; ++i) {
                // truncate time
                val = this.clone().startOf('day').valueOf();

                if (eras[i].since <= val && val <= eras[i].until) {
                    return eras[i].abbr;
                }
                if (eras[i].until <= val && val <= eras[i].since) {
                    return eras[i].abbr;
                }
            }

            return '';
        }

        function getEraYear() {
            var i,
                l,
                dir,
                val,
                eras = this.localeData().eras();
            for (i = 0, l = eras.length; i < l; ++i) {
                dir = eras[i].since <= eras[i].until ? +1 : -1;

                // truncate time
                val = this.clone().startOf('day').valueOf();

                if (
                    (eras[i].since <= val && val <= eras[i].until) ||
                    (eras[i].until <= val && val <= eras[i].since)
                ) {
                    return (
                        (this.year() - hooks(eras[i].since).year()) * dir +
                        eras[i].offset
                    );
                }
            }

            return this.year();
        }

        function erasNameRegex(isStrict) {
            if (!hasOwnProp(this, '_erasNameRegex')) {
                computeErasParse.call(this);
            }
            return isStrict ? this._erasNameRegex : this._erasRegex;
        }

        function erasAbbrRegex(isStrict) {
            if (!hasOwnProp(this, '_erasAbbrRegex')) {
                computeErasParse.call(this);
            }
            return isStrict ? this._erasAbbrRegex : this._erasRegex;
        }

        function erasNarrowRegex(isStrict) {
            if (!hasOwnProp(this, '_erasNarrowRegex')) {
                computeErasParse.call(this);
            }
            return isStrict ? this._erasNarrowRegex : this._erasRegex;
        }

        function matchEraAbbr(isStrict, locale) {
            return locale.erasAbbrRegex(isStrict);
        }

        function matchEraName(isStrict, locale) {
            return locale.erasNameRegex(isStrict);
        }

        function matchEraNarrow(isStrict, locale) {
            return locale.erasNarrowRegex(isStrict);
        }

        function matchEraYearOrdinal(isStrict, locale) {
            return locale._eraYearOrdinalRegex || matchUnsigned;
        }

        function computeErasParse() {
            var abbrPieces = [],
                namePieces = [],
                narrowPieces = [],
                mixedPieces = [],
                i,
                l,
                eras = this.eras();

            for (i = 0, l = eras.length; i < l; ++i) {
                namePieces.push(regexEscape(eras[i].name));
                abbrPieces.push(regexEscape(eras[i].abbr));
                narrowPieces.push(regexEscape(eras[i].narrow));

                mixedPieces.push(regexEscape(eras[i].name));
                mixedPieces.push(regexEscape(eras[i].abbr));
                mixedPieces.push(regexEscape(eras[i].narrow));
            }

            this._erasRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
            this._erasNameRegex = new RegExp('^(' + namePieces.join('|') + ')', 'i');
            this._erasAbbrRegex = new RegExp('^(' + abbrPieces.join('|') + ')', 'i');
            this._erasNarrowRegex = new RegExp(
                '^(' + narrowPieces.join('|') + ')',
                'i'
            );
        }

        // FORMATTING

        addFormatToken(0, ['gg', 2], 0, function () {
            return this.weekYear() % 100;
        });

        addFormatToken(0, ['GG', 2], 0, function () {
            return this.isoWeekYear() % 100;
        });

        function addWeekYearFormatToken(token, getter) {
            addFormatToken(0, [token, token.length], 0, getter);
        }

        addWeekYearFormatToken('gggg', 'weekYear');
        addWeekYearFormatToken('ggggg', 'weekYear');
        addWeekYearFormatToken('GGGG', 'isoWeekYear');
        addWeekYearFormatToken('GGGGG', 'isoWeekYear');

        // ALIASES

        addUnitAlias('weekYear', 'gg');
        addUnitAlias('isoWeekYear', 'GG');

        // PRIORITY

        addUnitPriority('weekYear', 1);
        addUnitPriority('isoWeekYear', 1);

        // PARSING

        addRegexToken('G', matchSigned);
        addRegexToken('g', matchSigned);
        addRegexToken('GG', match1to2, match2);
        addRegexToken('gg', match1to2, match2);
        addRegexToken('GGGG', match1to4, match4);
        addRegexToken('gggg', match1to4, match4);
        addRegexToken('GGGGG', match1to6, match6);
        addRegexToken('ggggg', match1to6, match6);

        addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function (
            input,
            week,
            config,
            token
        ) {
            week[token.substr(0, 2)] = toInt(input);
        });

        addWeekParseToken(['gg', 'GG'], function (input, week, config, token) {
            week[token] = hooks.parseTwoDigitYear(input);
        });

        // MOMENTS

        function getSetWeekYear(input) {
            return getSetWeekYearHelper.call(
                this,
                input,
                this.week(),
                this.weekday(),
                this.localeData()._week.dow,
                this.localeData()._week.doy
            );
        }

        function getSetISOWeekYear(input) {
            return getSetWeekYearHelper.call(
                this,
                input,
                this.isoWeek(),
                this.isoWeekday(),
                1,
                4
            );
        }

        function getISOWeeksInYear() {
            return weeksInYear(this.year(), 1, 4);
        }

        function getISOWeeksInISOWeekYear() {
            return weeksInYear(this.isoWeekYear(), 1, 4);
        }

        function getWeeksInYear() {
            var weekInfo = this.localeData()._week;
            return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
        }

        function getWeeksInWeekYear() {
            var weekInfo = this.localeData()._week;
            return weeksInYear(this.weekYear(), weekInfo.dow, weekInfo.doy);
        }

        function getSetWeekYearHelper(input, week, weekday, dow, doy) {
            var weeksTarget;
            if (input == null) {
                return weekOfYear(this, dow, doy).year;
            } else {
                weeksTarget = weeksInYear(input, dow, doy);
                if (week > weeksTarget) {
                    week = weeksTarget;
                }
                return setWeekAll.call(this, input, week, weekday, dow, doy);
            }
        }

        function setWeekAll(weekYear, week, weekday, dow, doy) {
            var dayOfYearData = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy),
                date = createUTCDate(dayOfYearData.year, 0, dayOfYearData.dayOfYear);

            this.year(date.getUTCFullYear());
            this.month(date.getUTCMonth());
            this.date(date.getUTCDate());
            return this;
        }

        // FORMATTING

        addFormatToken('Q', 0, 'Qo', 'quarter');

        // ALIASES

        addUnitAlias('quarter', 'Q');

        // PRIORITY

        addUnitPriority('quarter', 7);

        // PARSING

        addRegexToken('Q', match1);
        addParseToken('Q', function (input, array) {
            array[MONTH] = (toInt(input) - 1) * 3;
        });

        // MOMENTS

        function getSetQuarter(input) {
            return input == null
                ? Math.ceil((this.month() + 1) / 3)
                : this.month((input - 1) * 3 + (this.month() % 3));
        }

        // FORMATTING

        addFormatToken('D', ['DD', 2], 'Do', 'date');

        // ALIASES

        addUnitAlias('date', 'D');

        // PRIORITY
        addUnitPriority('date', 9);

        // PARSING

        addRegexToken('D', match1to2);
        addRegexToken('DD', match1to2, match2);
        addRegexToken('Do', function (isStrict, locale) {
            // TODO: Remove "ordinalParse" fallback in next major release.
            return isStrict
                ? locale._dayOfMonthOrdinalParse || locale._ordinalParse
                : locale._dayOfMonthOrdinalParseLenient;
        });

        addParseToken(['D', 'DD'], DATE);
        addParseToken('Do', function (input, array) {
            array[DATE] = toInt(input.match(match1to2)[0]);
        });

        // MOMENTS

        var getSetDayOfMonth = makeGetSet('Date', true);

        // FORMATTING

        addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear');

        // ALIASES

        addUnitAlias('dayOfYear', 'DDD');

        // PRIORITY
        addUnitPriority('dayOfYear', 4);

        // PARSING

        addRegexToken('DDD', match1to3);
        addRegexToken('DDDD', match3);
        addParseToken(['DDD', 'DDDD'], function (input, array, config) {
            config._dayOfYear = toInt(input);
        });

        // HELPERS

        // MOMENTS

        function getSetDayOfYear(input) {
            var dayOfYear =
                Math.round(
                    (this.clone().startOf('day') - this.clone().startOf('year')) / 864e5
                ) + 1;
            return input == null ? dayOfYear : this.add(input - dayOfYear, 'd');
        }

        // FORMATTING

        addFormatToken('m', ['mm', 2], 0, 'minute');

        // ALIASES

        addUnitAlias('minute', 'm');

        // PRIORITY

        addUnitPriority('minute', 14);

        // PARSING

        addRegexToken('m', match1to2);
        addRegexToken('mm', match1to2, match2);
        addParseToken(['m', 'mm'], MINUTE);

        // MOMENTS

        var getSetMinute = makeGetSet('Minutes', false);

        // FORMATTING

        addFormatToken('s', ['ss', 2], 0, 'second');

        // ALIASES

        addUnitAlias('second', 's');

        // PRIORITY

        addUnitPriority('second', 15);

        // PARSING

        addRegexToken('s', match1to2);
        addRegexToken('ss', match1to2, match2);
        addParseToken(['s', 'ss'], SECOND);

        // MOMENTS

        var getSetSecond = makeGetSet('Seconds', false);

        // FORMATTING

        addFormatToken('S', 0, 0, function () {
            return ~~(this.millisecond() / 100);
        });

        addFormatToken(0, ['SS', 2], 0, function () {
            return ~~(this.millisecond() / 10);
        });

        addFormatToken(0, ['SSS', 3], 0, 'millisecond');
        addFormatToken(0, ['SSSS', 4], 0, function () {
            return this.millisecond() * 10;
        });
        addFormatToken(0, ['SSSSS', 5], 0, function () {
            return this.millisecond() * 100;
        });
        addFormatToken(0, ['SSSSSS', 6], 0, function () {
            return this.millisecond() * 1000;
        });
        addFormatToken(0, ['SSSSSSS', 7], 0, function () {
            return this.millisecond() * 10000;
        });
        addFormatToken(0, ['SSSSSSSS', 8], 0, function () {
            return this.millisecond() * 100000;
        });
        addFormatToken(0, ['SSSSSSSSS', 9], 0, function () {
            return this.millisecond() * 1000000;
        });

        // ALIASES

        addUnitAlias('millisecond', 'ms');

        // PRIORITY

        addUnitPriority('millisecond', 16);

        // PARSING

        addRegexToken('S', match1to3, match1);
        addRegexToken('SS', match1to3, match2);
        addRegexToken('SSS', match1to3, match3);

        var token, getSetMillisecond;
        for (token = 'SSSS'; token.length <= 9; token += 'S') {
            addRegexToken(token, matchUnsigned);
        }

        function parseMs(input, array) {
            array[MILLISECOND] = toInt(('0.' + input) * 1000);
        }

        for (token = 'S'; token.length <= 9; token += 'S') {
            addParseToken(token, parseMs);
        }

        getSetMillisecond = makeGetSet('Milliseconds', false);

        // FORMATTING

        addFormatToken('z', 0, 0, 'zoneAbbr');
        addFormatToken('zz', 0, 0, 'zoneName');

        // MOMENTS

        function getZoneAbbr() {
            return this._isUTC ? 'UTC' : '';
        }

        function getZoneName() {
            return this._isUTC ? 'Coordinated Universal Time' : '';
        }

        var proto = Moment.prototype;

        proto.add = add;
        proto.calendar = calendar$1;
        proto.clone = clone;
        proto.diff = diff;
        proto.endOf = endOf;
        proto.format = format;
        proto.from = from;
        proto.fromNow = fromNow;
        proto.to = to;
        proto.toNow = toNow;
        proto.get = stringGet;
        proto.invalidAt = invalidAt;
        proto.isAfter = isAfter;
        proto.isBefore = isBefore;
        proto.isBetween = isBetween;
        proto.isSame = isSame;
        proto.isSameOrAfter = isSameOrAfter;
        proto.isSameOrBefore = isSameOrBefore;
        proto.isValid = isValid$2;
        proto.lang = lang;
        proto.locale = locale;
        proto.localeData = localeData;
        proto.max = prototypeMax;
        proto.min = prototypeMin;
        proto.parsingFlags = parsingFlags;
        proto.set = stringSet;
        proto.startOf = startOf;
        proto.subtract = subtract;
        proto.toArray = toArray;
        proto.toObject = toObject;
        proto.toDate = toDate;
        proto.toISOString = toISOString;
        proto.inspect = inspect;
        if (typeof Symbol !== 'undefined' && Symbol.for != null) {
            proto[Symbol.for('nodejs.util.inspect.custom')] = function () {
                return 'Moment<' + this.format() + '>';
            };
        }
        proto.toJSON = toJSON;
        proto.toString = toString;
        proto.unix = unix;
        proto.valueOf = valueOf;
        proto.creationData = creationData;
        proto.eraName = getEraName;
        proto.eraNarrow = getEraNarrow;
        proto.eraAbbr = getEraAbbr;
        proto.eraYear = getEraYear;
        proto.year = getSetYear;
        proto.isLeapYear = getIsLeapYear;
        proto.weekYear = getSetWeekYear;
        proto.isoWeekYear = getSetISOWeekYear;
        proto.quarter = proto.quarters = getSetQuarter;
        proto.month = getSetMonth;
        proto.daysInMonth = getDaysInMonth;
        proto.week = proto.weeks = getSetWeek;
        proto.isoWeek = proto.isoWeeks = getSetISOWeek;
        proto.weeksInYear = getWeeksInYear;
        proto.weeksInWeekYear = getWeeksInWeekYear;
        proto.isoWeeksInYear = getISOWeeksInYear;
        proto.isoWeeksInISOWeekYear = getISOWeeksInISOWeekYear;
        proto.date = getSetDayOfMonth;
        proto.day = proto.days = getSetDayOfWeek;
        proto.weekday = getSetLocaleDayOfWeek;
        proto.isoWeekday = getSetISODayOfWeek;
        proto.dayOfYear = getSetDayOfYear;
        proto.hour = proto.hours = getSetHour;
        proto.minute = proto.minutes = getSetMinute;
        proto.second = proto.seconds = getSetSecond;
        proto.millisecond = proto.milliseconds = getSetMillisecond;
        proto.utcOffset = getSetOffset;
        proto.utc = setOffsetToUTC;
        proto.local = setOffsetToLocal;
        proto.parseZone = setOffsetToParsedOffset;
        proto.hasAlignedHourOffset = hasAlignedHourOffset;
        proto.isDST = isDaylightSavingTime;
        proto.isLocal = isLocal;
        proto.isUtcOffset = isUtcOffset;
        proto.isUtc = isUtc;
        proto.isUTC = isUtc;
        proto.zoneAbbr = getZoneAbbr;
        proto.zoneName = getZoneName;
        proto.dates = deprecate(
            'dates accessor is deprecated. Use date instead.',
            getSetDayOfMonth
        );
        proto.months = deprecate(
            'months accessor is deprecated. Use month instead',
            getSetMonth
        );
        proto.years = deprecate(
            'years accessor is deprecated. Use year instead',
            getSetYear
        );
        proto.zone = deprecate(
            'moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/',
            getSetZone
        );
        proto.isDSTShifted = deprecate(
            'isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information',
            isDaylightSavingTimeShifted
        );

        function createUnix(input) {
            return createLocal(input * 1000);
        }

        function createInZone() {
            return createLocal.apply(null, arguments).parseZone();
        }

        function preParsePostFormat(string) {
            return string;
        }

        var proto$1 = Locale.prototype;

        proto$1.calendar = calendar;
        proto$1.longDateFormat = longDateFormat;
        proto$1.invalidDate = invalidDate;
        proto$1.ordinal = ordinal;
        proto$1.preparse = preParsePostFormat;
        proto$1.postformat = preParsePostFormat;
        proto$1.relativeTime = relativeTime;
        proto$1.pastFuture = pastFuture;
        proto$1.set = set;
        proto$1.eras = localeEras;
        proto$1.erasParse = localeErasParse;
        proto$1.erasConvertYear = localeErasConvertYear;
        proto$1.erasAbbrRegex = erasAbbrRegex;
        proto$1.erasNameRegex = erasNameRegex;
        proto$1.erasNarrowRegex = erasNarrowRegex;

        proto$1.months = localeMonths;
        proto$1.monthsShort = localeMonthsShort;
        proto$1.monthsParse = localeMonthsParse;
        proto$1.monthsRegex = monthsRegex;
        proto$1.monthsShortRegex = monthsShortRegex;
        proto$1.week = localeWeek;
        proto$1.firstDayOfYear = localeFirstDayOfYear;
        proto$1.firstDayOfWeek = localeFirstDayOfWeek;

        proto$1.weekdays = localeWeekdays;
        proto$1.weekdaysMin = localeWeekdaysMin;
        proto$1.weekdaysShort = localeWeekdaysShort;
        proto$1.weekdaysParse = localeWeekdaysParse;

        proto$1.weekdaysRegex = weekdaysRegex;
        proto$1.weekdaysShortRegex = weekdaysShortRegex;
        proto$1.weekdaysMinRegex = weekdaysMinRegex;

        proto$1.isPM = localeIsPM;
        proto$1.meridiem = localeMeridiem;

        function get$1(format, index, field, setter) {
            var locale = getLocale(),
                utc = createUTC().set(setter, index);
            return locale[field](utc, format);
        }

        function listMonthsImpl(format, index, field) {
            if (isNumber(format)) {
                index = format;
                format = undefined;
            }

            format = format || '';

            if (index != null) {
                return get$1(format, index, field, 'month');
            }

            var i,
                out = [];
            for (i = 0; i < 12; i++) {
                out[i] = get$1(format, i, field, 'month');
            }
            return out;
        }

        // ()
        // (5)
        // (fmt, 5)
        // (fmt)
        // (true)
        // (true, 5)
        // (true, fmt, 5)
        // (true, fmt)
        function listWeekdaysImpl(localeSorted, format, index, field) {
            if (typeof localeSorted === 'boolean') {
                if (isNumber(format)) {
                    index = format;
                    format = undefined;
                }

                format = format || '';
            } else {
                format = localeSorted;
                index = format;
                localeSorted = false;

                if (isNumber(format)) {
                    index = format;
                    format = undefined;
                }

                format = format || '';
            }

            var locale = getLocale(),
                shift = localeSorted ? locale._week.dow : 0,
                i,
                out = [];

            if (index != null) {
                return get$1(format, (index + shift) % 7, field, 'day');
            }

            for (i = 0; i < 7; i++) {
                out[i] = get$1(format, (i + shift) % 7, field, 'day');
            }
            return out;
        }

        function listMonths(format, index) {
            return listMonthsImpl(format, index, 'months');
        }

        function listMonthsShort(format, index) {
            return listMonthsImpl(format, index, 'monthsShort');
        }

        function listWeekdays(localeSorted, format, index) {
            return listWeekdaysImpl(localeSorted, format, index, 'weekdays');
        }

        function listWeekdaysShort(localeSorted, format, index) {
            return listWeekdaysImpl(localeSorted, format, index, 'weekdaysShort');
        }

        function listWeekdaysMin(localeSorted, format, index) {
            return listWeekdaysImpl(localeSorted, format, index, 'weekdaysMin');
        }

        getSetGlobalLocale('en', {
            eras: [
                {
                    since: '0001-01-01',
                    until: +Infinity,
                    offset: 1,
                    name: 'Anno Domini',
                    narrow: 'AD',
                    abbr: 'AD',
                },
                {
                    since: '0000-12-31',
                    until: -Infinity,
                    offset: 1,
                    name: 'Before Christ',
                    narrow: 'BC',
                    abbr: 'BC',
                },
            ],
            dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
            ordinal: function (number) {
                var b = number % 10,
                    output =
                        toInt((number % 100) / 10) === 1
                            ? 'th'
                            : b === 1
                            ? 'st'
                            : b === 2
                            ? 'nd'
                            : b === 3
                            ? 'rd'
                            : 'th';
                return number + output;
            },
        });

        // Side effect imports

        hooks.lang = deprecate(
            'moment.lang is deprecated. Use moment.locale instead.',
            getSetGlobalLocale
        );
        hooks.langData = deprecate(
            'moment.langData is deprecated. Use moment.localeData instead.',
            getLocale
        );

        var mathAbs = Math.abs;

        function abs() {
            var data = this._data;

            this._milliseconds = mathAbs(this._milliseconds);
            this._days = mathAbs(this._days);
            this._months = mathAbs(this._months);

            data.milliseconds = mathAbs(data.milliseconds);
            data.seconds = mathAbs(data.seconds);
            data.minutes = mathAbs(data.minutes);
            data.hours = mathAbs(data.hours);
            data.months = mathAbs(data.months);
            data.years = mathAbs(data.years);

            return this;
        }

        function addSubtract$1(duration, input, value, direction) {
            var other = createDuration(input, value);

            duration._milliseconds += direction * other._milliseconds;
            duration._days += direction * other._days;
            duration._months += direction * other._months;

            return duration._bubble();
        }

        // supports only 2.0-style add(1, 's') or add(duration)
        function add$1(input, value) {
            return addSubtract$1(this, input, value, 1);
        }

        // supports only 2.0-style subtract(1, 's') or subtract(duration)
        function subtract$1(input, value) {
            return addSubtract$1(this, input, value, -1);
        }

        function absCeil(number) {
            if (number < 0) {
                return Math.floor(number);
            } else {
                return Math.ceil(number);
            }
        }

        function bubble() {
            var milliseconds = this._milliseconds,
                days = this._days,
                months = this._months,
                data = this._data,
                seconds,
                minutes,
                hours,
                years,
                monthsFromDays;

            // if we have a mix of positive and negative values, bubble down first
            // check: https://github.com/moment/moment/issues/2166
            if (
                !(
                    (milliseconds >= 0 && days >= 0 && months >= 0) ||
                    (milliseconds <= 0 && days <= 0 && months <= 0)
                )
            ) {
                milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
                days = 0;
                months = 0;
            }

            // The following code bubbles up values, see the tests for
            // examples of what that means.
            data.milliseconds = milliseconds % 1000;

            seconds = absFloor(milliseconds / 1000);
            data.seconds = seconds % 60;

            minutes = absFloor(seconds / 60);
            data.minutes = minutes % 60;

            hours = absFloor(minutes / 60);
            data.hours = hours % 24;

            days += absFloor(hours / 24);

            // convert days to months
            monthsFromDays = absFloor(daysToMonths(days));
            months += monthsFromDays;
            days -= absCeil(monthsToDays(monthsFromDays));

            // 12 months -> 1 year
            years = absFloor(months / 12);
            months %= 12;

            data.days = days;
            data.months = months;
            data.years = years;

            return this;
        }

        function daysToMonths(days) {
            // 400 years have 146097 days (taking into account leap year rules)
            // 400 years have 12 months === 4800
            return (days * 4800) / 146097;
        }

        function monthsToDays(months) {
            // the reverse of daysToMonths
            return (months * 146097) / 4800;
        }

        function as(units) {
            if (!this.isValid()) {
                return NaN;
            }
            var days,
                months,
                milliseconds = this._milliseconds;

            units = normalizeUnits(units);

            if (units === 'month' || units === 'quarter' || units === 'year') {
                days = this._days + milliseconds / 864e5;
                months = this._months + daysToMonths(days);
                switch (units) {
                    case 'month':
                        return months;
                    case 'quarter':
                        return months / 3;
                    case 'year':
                        return months / 12;
                }
            } else {
                // handle milliseconds separately because of floating point math errors (issue #1867)
                days = this._days + Math.round(monthsToDays(this._months));
                switch (units) {
                    case 'week':
                        return days / 7 + milliseconds / 6048e5;
                    case 'day':
                        return days + milliseconds / 864e5;
                    case 'hour':
                        return days * 24 + milliseconds / 36e5;
                    case 'minute':
                        return days * 1440 + milliseconds / 6e4;
                    case 'second':
                        return days * 86400 + milliseconds / 1000;
                    // Math.floor prevents floating point math errors here
                    case 'millisecond':
                        return Math.floor(days * 864e5) + milliseconds;
                    default:
                        throw new Error('Unknown unit ' + units);
                }
            }
        }

        // TODO: Use this.as('ms')?
        function valueOf$1() {
            if (!this.isValid()) {
                return NaN;
            }
            return (
                this._milliseconds +
                this._days * 864e5 +
                (this._months % 12) * 2592e6 +
                toInt(this._months / 12) * 31536e6
            );
        }

        function makeAs(alias) {
            return function () {
                return this.as(alias);
            };
        }

        var asMilliseconds = makeAs('ms'),
            asSeconds = makeAs('s'),
            asMinutes = makeAs('m'),
            asHours = makeAs('h'),
            asDays = makeAs('d'),
            asWeeks = makeAs('w'),
            asMonths = makeAs('M'),
            asQuarters = makeAs('Q'),
            asYears = makeAs('y');

        function clone$1() {
            return createDuration(this);
        }

        function get$2(units) {
            units = normalizeUnits(units);
            return this.isValid() ? this[units + 's']() : NaN;
        }

        function makeGetter(name) {
            return function () {
                return this.isValid() ? this._data[name] : NaN;
            };
        }

        var milliseconds = makeGetter('milliseconds'),
            seconds = makeGetter('seconds'),
            minutes = makeGetter('minutes'),
            hours = makeGetter('hours'),
            days = makeGetter('days'),
            months = makeGetter('months'),
            years = makeGetter('years');

        function weeks() {
            return absFloor(this.days() / 7);
        }

        var round = Math.round,
            thresholds = {
                ss: 44, // a few seconds to seconds
                s: 45, // seconds to minute
                m: 45, // minutes to hour
                h: 22, // hours to day
                d: 26, // days to month/week
                w: null, // weeks to month
                M: 11, // months to year
            };

        // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
        function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
            return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
        }

        function relativeTime$1(posNegDuration, withoutSuffix, thresholds, locale) {
            var duration = createDuration(posNegDuration).abs(),
                seconds = round(duration.as('s')),
                minutes = round(duration.as('m')),
                hours = round(duration.as('h')),
                days = round(duration.as('d')),
                months = round(duration.as('M')),
                weeks = round(duration.as('w')),
                years = round(duration.as('y')),
                a =
                    (seconds <= thresholds.ss && ['s', seconds]) ||
                    (seconds < thresholds.s && ['ss', seconds]) ||
                    (minutes <= 1 && ['m']) ||
                    (minutes < thresholds.m && ['mm', minutes]) ||
                    (hours <= 1 && ['h']) ||
                    (hours < thresholds.h && ['hh', hours]) ||
                    (days <= 1 && ['d']) ||
                    (days < thresholds.d && ['dd', days]);

            if (thresholds.w != null) {
                a =
                    a ||
                    (weeks <= 1 && ['w']) ||
                    (weeks < thresholds.w && ['ww', weeks]);
            }
            a = a ||
                (months <= 1 && ['M']) ||
                (months < thresholds.M && ['MM', months]) ||
                (years <= 1 && ['y']) || ['yy', years];

            a[2] = withoutSuffix;
            a[3] = +posNegDuration > 0;
            a[4] = locale;
            return substituteTimeAgo.apply(null, a);
        }

        // This function allows you to set the rounding function for relative time strings
        function getSetRelativeTimeRounding(roundingFunction) {
            if (roundingFunction === undefined) {
                return round;
            }
            if (typeof roundingFunction === 'function') {
                round = roundingFunction;
                return true;
            }
            return false;
        }

        // This function allows you to set a threshold for relative time strings
        function getSetRelativeTimeThreshold(threshold, limit) {
            if (thresholds[threshold] === undefined) {
                return false;
            }
            if (limit === undefined) {
                return thresholds[threshold];
            }
            thresholds[threshold] = limit;
            if (threshold === 's') {
                thresholds.ss = limit - 1;
            }
            return true;
        }

        function humanize(argWithSuffix, argThresholds) {
            if (!this.isValid()) {
                return this.localeData().invalidDate();
            }

            var withSuffix = false,
                th = thresholds,
                locale,
                output;

            if (typeof argWithSuffix === 'object') {
                argThresholds = argWithSuffix;
                argWithSuffix = false;
            }
            if (typeof argWithSuffix === 'boolean') {
                withSuffix = argWithSuffix;
            }
            if (typeof argThresholds === 'object') {
                th = Object.assign({}, thresholds, argThresholds);
                if (argThresholds.s != null && argThresholds.ss == null) {
                    th.ss = argThresholds.s - 1;
                }
            }

            locale = this.localeData();
            output = relativeTime$1(this, !withSuffix, th, locale);

            if (withSuffix) {
                output = locale.pastFuture(+this, output);
            }

            return locale.postformat(output);
        }

        var abs$1 = Math.abs;

        function sign(x) {
            return (x > 0) - (x < 0) || +x;
        }

        function toISOString$1() {
            // for ISO strings we do not use the normal bubbling rules:
            //  * milliseconds bubble up until they become hours
            //  * days do not bubble at all
            //  * months bubble up until they become years
            // This is because there is no context-free conversion between hours and days
            // (think of clock changes)
            // and also not between days and months (28-31 days per month)
            if (!this.isValid()) {
                return this.localeData().invalidDate();
            }

            var seconds = abs$1(this._milliseconds) / 1000,
                days = abs$1(this._days),
                months = abs$1(this._months),
                minutes,
                hours,
                years,
                s,
                total = this.asSeconds(),
                totalSign,
                ymSign,
                daysSign,
                hmsSign;

            if (!total) {
                // this is the same as C#'s (Noda) and python (isodate)...
                // but not other JS (goog.date)
                return 'P0D';
            }

            // 3600 seconds -> 60 minutes -> 1 hour
            minutes = absFloor(seconds / 60);
            hours = absFloor(minutes / 60);
            seconds %= 60;
            minutes %= 60;

            // 12 months -> 1 year
            years = absFloor(months / 12);
            months %= 12;

            // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
            s = seconds ? seconds.toFixed(3).replace(/\.?0+$/, '') : '';

            totalSign = total < 0 ? '-' : '';
            ymSign = sign(this._months) !== sign(total) ? '-' : '';
            daysSign = sign(this._days) !== sign(total) ? '-' : '';
            hmsSign = sign(this._milliseconds) !== sign(total) ? '-' : '';

            return (
                totalSign +
                'P' +
                (years ? ymSign + years + 'Y' : '') +
                (months ? ymSign + months + 'M' : '') +
                (days ? daysSign + days + 'D' : '') +
                (hours || minutes || seconds ? 'T' : '') +
                (hours ? hmsSign + hours + 'H' : '') +
                (minutes ? hmsSign + minutes + 'M' : '') +
                (seconds ? hmsSign + s + 'S' : '')
            );
        }

        var proto$2 = Duration.prototype;

        proto$2.isValid = isValid$1;
        proto$2.abs = abs;
        proto$2.add = add$1;
        proto$2.subtract = subtract$1;
        proto$2.as = as;
        proto$2.asMilliseconds = asMilliseconds;
        proto$2.asSeconds = asSeconds;
        proto$2.asMinutes = asMinutes;
        proto$2.asHours = asHours;
        proto$2.asDays = asDays;
        proto$2.asWeeks = asWeeks;
        proto$2.asMonths = asMonths;
        proto$2.asQuarters = asQuarters;
        proto$2.asYears = asYears;
        proto$2.valueOf = valueOf$1;
        proto$2._bubble = bubble;
        proto$2.clone = clone$1;
        proto$2.get = get$2;
        proto$2.milliseconds = milliseconds;
        proto$2.seconds = seconds;
        proto$2.minutes = minutes;
        proto$2.hours = hours;
        proto$2.days = days;
        proto$2.weeks = weeks;
        proto$2.months = months;
        proto$2.years = years;
        proto$2.humanize = humanize;
        proto$2.toISOString = toISOString$1;
        proto$2.toString = toISOString$1;
        proto$2.toJSON = toISOString$1;
        proto$2.locale = locale;
        proto$2.localeData = localeData;

        proto$2.toIsoString = deprecate(
            'toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)',
            toISOString$1
        );
        proto$2.lang = lang;

        // FORMATTING

        addFormatToken('X', 0, 0, 'unix');
        addFormatToken('x', 0, 0, 'valueOf');

        // PARSING

        addRegexToken('x', matchSigned);
        addRegexToken('X', matchTimestamp);
        addParseToken('X', function (input, array, config) {
            config._d = new Date(parseFloat(input) * 1000);
        });
        addParseToken('x', function (input, array, config) {
            config._d = new Date(toInt(input));
        });

        //! moment.js

        hooks.version = '2.29.1';

        setHookCallback(createLocal);

        hooks.fn = proto;
        hooks.min = min;
        hooks.max = max;
        hooks.now = now;
        hooks.utc = createUTC;
        hooks.unix = createUnix;
        hooks.months = listMonths;
        hooks.isDate = isDate;
        hooks.locale = getSetGlobalLocale;
        hooks.invalid = createInvalid;
        hooks.duration = createDuration;
        hooks.isMoment = isMoment;
        hooks.weekdays = listWeekdays;
        hooks.parseZone = createInZone;
        hooks.localeData = getLocale;
        hooks.isDuration = isDuration;
        hooks.monthsShort = listMonthsShort;
        hooks.weekdaysMin = listWeekdaysMin;
        hooks.defineLocale = defineLocale;
        hooks.updateLocale = updateLocale;
        hooks.locales = listLocales;
        hooks.weekdaysShort = listWeekdaysShort;
        hooks.normalizeUnits = normalizeUnits;
        hooks.relativeTimeRounding = getSetRelativeTimeRounding;
        hooks.relativeTimeThreshold = getSetRelativeTimeThreshold;
        hooks.calendarFormat = getCalendarFormat;
        hooks.prototype = proto;

        // currently HTML5 input type only supports 24-hour formats
        hooks.HTML5_FMT = {
            DATETIME_LOCAL: 'YYYY-MM-DDTHH:mm', // <input type="datetime-local" />
            DATETIME_LOCAL_SECONDS: 'YYYY-MM-DDTHH:mm:ss', // <input type="datetime-local" step="1" />
            DATETIME_LOCAL_MS: 'YYYY-MM-DDTHH:mm:ss.SSS', // <input type="datetime-local" step="0.001" />
            DATE: 'YYYY-MM-DD', // <input type="date" />
            TIME: 'HH:mm', // <input type="time" />
            TIME_SECONDS: 'HH:mm:ss', // <input type="time" step="1" />
            TIME_MS: 'HH:mm:ss.SSS', // <input type="time" step="0.001" />
            WEEK: 'GGGG-[W]WW', // <input type="week" />
            MONTH: 'YYYY-MM', // <input type="month" />
        };

        return hooks;

    })));
    }(moment$1));

    var moment = moment$1.exports;

    /* eslint-disable */
    //----------------------
    // <auto-generated>
    //     Generated using the NSwag toolchain v10.6.6324.28497 (NJsonSchema v8.33.6323.36213) (http://NSwag.org)
    // </auto-generated>
    //----------------------
    /** OAuthGrantType enumeration */
    exports.OAuthGrantType = void 0;
    (function (OAuthGrantType) {
        OAuthGrantType[OAuthGrantType["Undefined"] = "undefined"] = "Undefined";
        OAuthGrantType[OAuthGrantType["Authorization_code"] = "authorization_code"] = "Authorization_code";
        OAuthGrantType[OAuthGrantType["Password"] = "password"] = "Password";
        OAuthGrantType[OAuthGrantType["Client_credentials"] = "client_credentials"] = "Client_credentials";
        OAuthGrantType[OAuthGrantType["Refresh_token"] = "refresh_token"] = "Refresh_token";
        OAuthGrantType[OAuthGrantType["JwtBearer"] = "jwtBearer"] = "JwtBearer";
    })(exports.OAuthGrantType || (exports.OAuthGrantType = {}));
    /** OAuthResponseType enumeration */
    exports.OAuthResponseType = void 0;
    (function (OAuthResponseType) {
        OAuthResponseType[OAuthResponseType["Undefined"] = "undefined"] = "Undefined";
        OAuthResponseType[OAuthResponseType["Code"] = "code"] = "Code";
        OAuthResponseType[OAuthResponseType["Token"] = "token"] = "Token";
    })(exports.OAuthResponseType || (exports.OAuthResponseType = {}));
    /** OAuthTokenType enumeration */
    exports.OAuthTokenType = void 0;
    (function (OAuthTokenType) {
        OAuthTokenType[OAuthTokenType["Undefined"] = "undefined"] = "Undefined";
        OAuthTokenType[OAuthTokenType["Bearer"] = "bearer"] = "Bearer";
        OAuthTokenType[OAuthTokenType["Mac"] = "mac"] = "Mac";
    })(exports.OAuthTokenType || (exports.OAuthTokenType = {}));
    /** OAuthError enumeration */
    exports.OAuthError = void 0;
    (function (OAuthError) {
        OAuthError[OAuthError["Undefined"] = "undefined"] = "Undefined";
        OAuthError[OAuthError["Invalid_request"] = "invalid_request"] = "Invalid_request";
        OAuthError[OAuthError["Unauthorized_client"] = "unauthorized_client"] = "Unauthorized_client";
        OAuthError[OAuthError["Access_denied"] = "access_denied"] = "Access_denied";
        OAuthError[OAuthError["Unsupported_response_type"] = "unsupported_response_type"] = "Unsupported_response_type";
        OAuthError[OAuthError["Invalid_scope"] = "invalid_scope"] = "Invalid_scope";
        OAuthError[OAuthError["Server_error"] = "server_error"] = "Server_error";
        OAuthError[OAuthError["Temporarily_unavailable"] = "temporarily_unavailable"] = "Temporarily_unavailable";
    })(exports.OAuthError || (exports.OAuthError = {}));
    /** OAuthSalaxyAuthorizeMode enumeration */
    exports.OAuthSalaxyAuthorizeMode = void 0;
    (function (OAuthSalaxyAuthorizeMode) {
        OAuthSalaxyAuthorizeMode[OAuthSalaxyAuthorizeMode["Undefined"] = "undefined"] = "Undefined";
        OAuthSalaxyAuthorizeMode[OAuthSalaxyAuthorizeMode["Sign_in"] = "sign_in"] = "Sign_in";
        OAuthSalaxyAuthorizeMode[OAuthSalaxyAuthorizeMode["Sign_up"] = "sign_up"] = "Sign_up";
    })(exports.OAuthSalaxyAuthorizeMode || (exports.OAuthSalaxyAuthorizeMode = {}));

    /* tslint:disable */
    /* eslint-disable */
    //----------------------
    // <auto-generated>
    //     Generated using the NSwag toolchain v13.11.1.0 (NJsonSchema v10.4.3.0 (Newtonsoft.Json v11.0.0.0)) (http://NSwag.org)
    // </auto-generated>
    //----------------------
    // ReSharper disable InconsistentNaming
    /** AbcSection enumeration */
    exports.AbcSection = void 0;
    (function (AbcSection) {
        AbcSection["Undefined"] = "undefined";
        AbcSection["DoNotShow"] = "doNotShow";
        AbcSection["Employment"] = "employment";
        AbcSection["Insurance"] = "insurance";
        AbcSection["Salary"] = "salary";
        AbcSection["Examples"] = "examples";
        AbcSection["PalkkausGeneral"] = "palkkausGeneral";
        AbcSection["PalkkausInstructions"] = "palkkausInstructions";
        AbcSection["Blog"] = "blog";
        AbcSection["Press"] = "press";
        AbcSection["PersonEmployer"] = "personEmployer";
        AbcSection["HouseholdEmployer"] = "householdEmployer";
        AbcSection["Worker"] = "worker";
        AbcSection["Employee"] = "employee";
        AbcSection["Entrepreneur"] = "entrepreneur";
        AbcSection["Association"] = "association";
        AbcSection["BusinessOwner"] = "businessOwner";
        AbcSection["ProductLongDescription"] = "productLongDescription";
        AbcSection["DocumentTemplates"] = "documentTemplates";
        AbcSection["InstructionsAndExamples"] = "instructionsAndExamples";
    })(exports.AbcSection || (exports.AbcSection = {}));
    /** AbsenceCauseCode enumeration */
    exports.AbsenceCauseCode = void 0;
    (function (AbsenceCauseCode) {
        AbsenceCauseCode["UnpaidLeave"] = "unpaidLeave";
        AbsenceCauseCode["PersonalReason"] = "personalReason";
        AbsenceCauseCode["Illness"] = "illness";
        AbsenceCauseCode["PartTimeSickLeave"] = "partTimeSickLeave";
        AbsenceCauseCode["ParentalLeave"] = "parentalLeave";
        AbsenceCauseCode["SpecialMaternityLeave"] = "specialMaternityLeave";
        AbsenceCauseCode["Rehabilitation"] = "rehabilitation";
        AbsenceCauseCode["ChildIllness"] = "childIllness";
        AbsenceCauseCode["PartTimeChildCareLeave"] = "partTimeChildCareLeave";
        AbsenceCauseCode["Training"] = "training";
        AbsenceCauseCode["JobAlternationLeave"] = "jobAlternationLeave";
        AbsenceCauseCode["StudyLeave"] = "studyLeave";
        AbsenceCauseCode["IndustrialAction"] = "industrialAction";
        AbsenceCauseCode["InterruptionInWorkProvision"] = "interruptionInWorkProvision";
        AbsenceCauseCode["LeaveOfAbsence"] = "leaveOfAbsence";
        AbsenceCauseCode["MilitaryRefresherTraining"] = "militaryRefresherTraining";
        AbsenceCauseCode["MilitaryService"] = "militaryService";
        AbsenceCauseCode["LayOff"] = "layOff";
        AbsenceCauseCode["ChildCareLeave"] = "childCareLeave";
        AbsenceCauseCode["MidWeekHoliday"] = "midWeekHoliday";
        AbsenceCauseCode["AccruedHoliday"] = "accruedHoliday";
        AbsenceCauseCode["OccupationalAccident"] = "occupationalAccident";
        AbsenceCauseCode["AnnualLeave"] = "annualLeave";
        AbsenceCauseCode["PartTimeAbsenceDueToRehabilitation"] = "partTimeAbsenceDueToRehabilitation";
        AbsenceCauseCode["Other"] = "other";
    })(exports.AbsenceCauseCode || (exports.AbsenceCauseCode = {}));
    /** AccountantType enumeration */
    exports.AccountantType = void 0;
    (function (AccountantType) {
        AccountantType["Unknown"] = "unknown";
        AccountantType["UnlinkedPrimaryPartner"] = "unlinkedPrimaryPartner";
        AccountantType["UnlinkedAccountingOnly"] = "unlinkedAccountingOnly";
        AccountantType["PendingPrimaryPartner"] = "pendingPrimaryPartner";
        AccountantType["PrimaryPartner"] = "primaryPartner";
        AccountantType["None"] = "none";
    })(exports.AccountantType || (exports.AccountantType = {}));
    /** AccountingChannel enumeration */
    exports.AccountingChannel = void 0;
    (function (AccountingChannel) {
        AccountingChannel["Undefined"] = "undefined";
        AccountingChannel["Manual"] = "manual";
        AccountingChannel["Email"] = "email";
        AccountingChannel["Template"] = "template";
        AccountingChannel["PaymentTest"] = "paymentTest";
        AccountingChannel["PaymentAccountorGo"] = "paymentAccountorGo";
        AccountingChannel["PaymentTalenomOnline"] = "paymentTalenomOnline";
        AccountingChannel["PaymentFinagoSolo"] = "paymentFinagoSolo";
        AccountingChannel["PaymentProcountor"] = "paymentProcountor";
        AccountingChannel["PaymentKevytyrittaja"] = "paymentKevytyrittaja";
        AccountingChannel["PaymentVismaNetvisor"] = "paymentVismaNetvisor";
        AccountingChannel["PaymentCfaFinvoice"] = "paymentCfaFinvoice";
        AccountingChannel["PaymentCfaTest"] = "paymentCfaTest";
        AccountingChannel["Procountor"] = "procountor";
        AccountingChannel["VismaNetvisor"] = "vismaNetvisor";
        AccountingChannel["Fennoa"] = "fennoa";
        AccountingChannel["MeritAktiva"] = "meritAktiva";
    })(exports.AccountingChannel || (exports.AccountingChannel = {}));
    /** AccountingPeriodClosingOption enumeration */
    exports.AccountingPeriodClosingOption = void 0;
    (function (AccountingPeriodClosingOption) {
        AccountingPeriodClosingOption["Default"] = "default";
    })(exports.AccountingPeriodClosingOption || (exports.AccountingPeriodClosingOption = {}));
    /** AccountingReportRowType enumeration */
    exports.AccountingReportRowType = void 0;
    (function (AccountingReportRowType) {
        AccountingReportRowType["Debit"] = "debit";
        AccountingReportRowType["Credit"] = "credit";
        AccountingReportRowType["Total"] = "total";
        AccountingReportRowType["GroupHeader"] = "groupHeader";
        AccountingReportRowType["GroupTotal"] = "groupTotal";
        AccountingReportRowType["ChildRow"] = "childRow";
    })(exports.AccountingReportRowType || (exports.AccountingReportRowType = {}));
    /** AccountingReportTableType enumeration */
    exports.AccountingReportTableType = void 0;
    (function (AccountingReportTableType) {
        AccountingReportTableType["Classic"] = "classic";
        AccountingReportTableType["Simple"] = "simple";
        AccountingReportTableType["Mapped"] = "mapped";
    })(exports.AccountingReportTableType || (exports.AccountingReportTableType = {}));
    /** AccountingTargetStatus enumeration */
    exports.AccountingTargetStatus = void 0;
    (function (AccountingTargetStatus) {
        AccountingTargetStatus["Undefined"] = "undefined";
        AccountingTargetStatus["Enabled"] = "enabled";
        AccountingTargetStatus["Disabled"] = "disabled";
        AccountingTargetStatus["ProductTarget"] = "productTarget";
        AccountingTargetStatus["SharedTarget"] = "sharedTarget";
        AccountingTargetStatus["RulesetTemplate"] = "rulesetTemplate";
        AccountingTargetStatus["PaymentChannel"] = "paymentChannel";
    })(exports.AccountingTargetStatus || (exports.AccountingTargetStatus = {}));
    /** AgeGroupCode enumeration */
    exports.AgeGroupCode = void 0;
    (function (AgeGroupCode) {
        AgeGroupCode["A"] = "a";
        AgeGroupCode["B"] = "b";
        AgeGroupCode["C"] = "c";
        AgeGroupCode["U"] = "u";
    })(exports.AgeGroupCode || (exports.AgeGroupCode = {}));
    /** AgeRange enumeration */
    exports.AgeRange = void 0;
    (function (AgeRange) {
        AgeRange["Unknown"] = "unknown";
        AgeRange["Age_15"] = "age_15";
        AgeRange["Age16"] = "age16";
        AgeRange["Age17"] = "age17";
        AgeRange["Age18_52"] = "age18_52";
        AgeRange["Age53_62"] = "age53_62";
        AgeRange["Age63_64"] = "age63_64";
        AgeRange["Age65_67"] = "age65_67";
        AgeRange["Age68AndOVer"] = "age68AndOVer";
    })(exports.AgeRange || (exports.AgeRange = {}));
    /** AllowanceCode enumeration */
    exports.AllowanceCode = void 0;
    (function (AllowanceCode) {
        AllowanceCode["MealAllowance"] = "mealAllowance";
        AllowanceCode["PartialDailyAllowance"] = "partialDailyAllowance";
        AllowanceCode["FullDailyAllowance"] = "fullDailyAllowance";
        AllowanceCode["InternationalDailyAllowance"] = "internationalDailyAllowance";
        AllowanceCode["TaxExemptReimbursementsAbroad"] = "taxExemptReimbursementsAbroad";
    })(exports.AllowanceCode || (exports.AllowanceCode = {}));
    /** AnnualLeavePaymentKind enumeration */
    exports.AnnualLeavePaymentKind = void 0;
    (function (AnnualLeavePaymentKind) {
        AnnualLeavePaymentKind["Undefined"] = "undefined";
        AnnualLeavePaymentKind["Planned"] = "planned";
        AnnualLeavePaymentKind["ManualSalary"] = "manualSalary";
        AnnualLeavePaymentKind["ManualCompensation"] = "manualCompensation";
        AnnualLeavePaymentKind["ManualBonus"] = "manualBonus";
        AnnualLeavePaymentKind["PaidCalc"] = "paidCalc";
        AnnualLeavePaymentKind["DraftCalc"] = "draftCalc";
    })(exports.AnnualLeavePaymentKind || (exports.AnnualLeavePaymentKind = {}));
    /** ApiListItemType enumeration */
    exports.ApiListItemType = void 0;
    (function (ApiListItemType) {
        ApiListItemType["Unknown"] = "unknown";
        ApiListItemType["AccountBase"] = "accountBase";
        ApiListItemType["CompanyAccount"] = "companyAccount";
        ApiListItemType["PersonAccount"] = "personAccount";
        ApiListItemType["WorkerAccount"] = "workerAccount";
        ApiListItemType["Certificate"] = "certificate";
        ApiListItemType["SessionUserCredential"] = "sessionUserCredential";
        ApiListItemType["Onboarding"] = "onboarding";
        ApiListItemType["Profile"] = "profile";
        ApiListItemType["WorkerInvitation"] = "workerInvitation";
        ApiListItemType["Calculation"] = "calculation";
        ApiListItemType["CalculationPaid"] = "calculationPaid";
        ApiListItemType["ESalaryPayment"] = "eSalaryPayment";
        ApiListItemType["Payment"] = "payment";
        ApiListItemType["EarningsPayment"] = "earningsPayment";
        ApiListItemType["Employment"] = "employment";
        ApiListItemType["WorkerAbsences"] = "workerAbsences";
        ApiListItemType["HolidayYear"] = "holidayYear";
        ApiListItemType["Taxcard"] = "taxcard";
        ApiListItemType["PayrollDetails"] = "payrollDetails";
        ApiListItemType["Invoice"] = "invoice";
        ApiListItemType["Report"] = "report";
        ApiListItemType["BlobFile"] = "blobFile";
        ApiListItemType["Article"] = "article";
        ApiListItemType["MessageThread"] = "messageThread";
        ApiListItemType["EmailMessage"] = "emailMessage";
        ApiListItemType["AccountProducts"] = "accountProducts";
        ApiListItemType["VarmaPensionOrder"] = "varmaPensionOrder";
        ApiListItemType["IfInsuranceOrder"] = "ifInsuranceOrder";
        ApiListItemType["Historical"] = "historical";
    })(exports.ApiListItemType || (exports.ApiListItemType = {}));
    /** ApiTestErrorType enumeration */
    exports.ApiTestErrorType = void 0;
    (function (ApiTestErrorType) {
        ApiTestErrorType["Default"] = "default";
        ApiTestErrorType["UserFriendly"] = "userFriendly";
    })(exports.ApiTestErrorType || (exports.ApiTestErrorType = {}));
    /** ApiItemType enumeration */
    exports.ApiItemType = void 0;
    (function (ApiItemType) {
        ApiItemType["Undefined"] = "undefined";
        ApiItemType["Calculation"] = "calculation";
        ApiItemType["CalculationPaid"] = "calculationPaid";
        ApiItemType["PayrollDetails"] = "payrollDetails";
        ApiItemType["Invoice"] = "invoice";
        ApiItemType["EarningsPaymentReport"] = "earningsPaymentReport";
        ApiItemType["PayerSummaryReport"] = "payerSummaryReport";
        ApiItemType["Employment"] = "employment";
        ApiItemType["MessageThread"] = "messageThread";
        ApiItemType["CalendarEvent"] = "calendarEvent";
    })(exports.ApiItemType || (exports.ApiItemType = {}));
    /** ApiValidationErrorType enumeration */
    exports.ApiValidationErrorType = void 0;
    (function (ApiValidationErrorType) {
        ApiValidationErrorType["General"] = "general";
        ApiValidationErrorType["Required"] = "required";
        ApiValidationErrorType["Invalid"] = "invalid";
        ApiValidationErrorType["Warning"] = "warning";
    })(exports.ApiValidationErrorType || (exports.ApiValidationErrorType = {}));
    /** AuthenticationMethod enumeration */
    exports.AuthenticationMethod = void 0;
    (function (AuthenticationMethod) {
        AuthenticationMethod["Undefined"] = "undefined";
        AuthenticationMethod["EmailPwdLocal"] = "emailPwdLocal";
        AuthenticationMethod["Facebook"] = "facebook";
        AuthenticationMethod["Google"] = "google";
        AuthenticationMethod["Microsoft"] = "microsoft";
        AuthenticationMethod["LinkedIn"] = "linkedIn";
        AuthenticationMethod["Auth0Database"] = "auth0Database";
        AuthenticationMethod["X509"] = "x509";
        AuthenticationMethod["Salaxy"] = "salaxy";
        AuthenticationMethod["Test"] = "test";
        AuthenticationMethod["InternalLocalhost"] = "internalLocalhost";
    })(exports.AuthenticationMethod || (exports.AuthenticationMethod = {}));
    /** AuthorizationStatus enumeration */
    exports.AuthorizationStatus = void 0;
    (function (AuthorizationStatus) {
        AuthorizationStatus["Undefined"] = "undefined";
        AuthorizationStatus["OK"] = "oK";
        AuthorizationStatus["AccessDenied"] = "accessDenied";
        AuthorizationStatus["ExpiredOauthToken"] = "expiredOauthToken";
        AuthorizationStatus["InvalidOAuthToken"] = "invalidOAuthToken";
        AuthorizationStatus["NoOauthToken"] = "noOauthToken";
    })(exports.AuthorizationStatus || (exports.AuthorizationStatus = {}));
    /** AuthorizationType enumeration */
    exports.AuthorizationType = void 0;
    (function (AuthorizationType) {
        AuthorizationType["None"] = "none";
        AuthorizationType["EmployerAuthorization"] = "employerAuthorization";
        AuthorizationType["WorkerContract"] = "workerContract";
        AuthorizationType["CompanyContract"] = "companyContract";
        AuthorizationType["Manual"] = "manual";
        AuthorizationType["Temporary"] = "temporary";
    })(exports.AuthorizationType || (exports.AuthorizationType = {}));
    /** AvatarPictureType enumeration */
    exports.AvatarPictureType = void 0;
    (function (AvatarPictureType) {
        AvatarPictureType["Icon"] = "icon";
        AvatarPictureType["Uploaded"] = "uploaded";
        AvatarPictureType["Gravatar"] = "gravatar";
    })(exports.AvatarPictureType || (exports.AvatarPictureType = {}));
    /** BankPaymentType enumeration */
    exports.BankPaymentType = void 0;
    (function (BankPaymentType) {
        BankPaymentType["Unknown"] = "unknown";
        BankPaymentType["Error"] = "error";
        BankPaymentType["PaytrailBatch"] = "paytrailBatch";
        BankPaymentType["NetvisorPayment"] = "netvisorPayment";
        BankPaymentType["SalaryGrossPayment"] = "salaryGrossPayment";
        BankPaymentType["RefundPayment"] = "refundPayment";
        BankPaymentType["PartialRefundPayment"] = "partialRefundPayment";
        BankPaymentType["DoubleRefundPayment"] = "doubleRefundPayment";
        BankPaymentType["InsuranceGrossPayment"] = "insuranceGrossPayment";
        BankPaymentType["NetSalary"] = "netSalary";
        BankPaymentType["SalaryAdvance"] = "salaryAdvance";
        BankPaymentType["NetSalaryRepayment"] = "netSalaryRepayment";
        BankPaymentType["NetSalaryCorrection"] = "netSalaryCorrection";
        BankPaymentType["TaxAccount"] = "taxAccount";
        BankPaymentType["Tvr"] = "tvr";
        BankPaymentType["TvrRfnd"] = "tvrRfnd";
        BankPaymentType["PalkkausFees"] = "palkkausFees";
        BankPaymentType["BankFees"] = "bankFees";
        BankPaymentType["PaytrailFees"] = "paytrailFees";
        BankPaymentType["InsuranceLahiTapiola"] = "insuranceLahiTapiola";
        BankPaymentType["Tyel"] = "tyel";
        BankPaymentType["TyelEteraContract"] = "tyelEteraContract";
        BankPaymentType["TyelEteraTemp"] = "tyelEteraTemp";
        BankPaymentType["TyelEteraUnknown"] = "tyelEteraUnknown";
        BankPaymentType["TyelEloContract"] = "tyelEloContract";
        BankPaymentType["TyelEloTemp"] = "tyelEloTemp";
        BankPaymentType["TyelEloInitialImport"] = "tyelEloInitialImport";
        BankPaymentType["TyelVarmaContract"] = "tyelVarmaContract";
        BankPaymentType["TyelVarmaTemp"] = "tyelVarmaTemp";
        BankPaymentType["TyelIlmarinenContract"] = "tyelIlmarinenContract";
        BankPaymentType["TyelIlmarinenTemp"] = "tyelIlmarinenTemp";
        BankPaymentType["Union"] = "union";
        BankPaymentType["UnionRaksa"] = "unionRaksa";
        BankPaymentType["Foreclosure"] = "foreclosure";
        BankPaymentType["SaldoYearBegin"] = "saldoYearBegin";
        BankPaymentType["SaldoYearEnd"] = "saldoYearEnd";
    })(exports.BankPaymentType || (exports.BankPaymentType = {}));
    /** BenefitCode enumeration */
    exports.BenefitCode = void 0;
    (function (BenefitCode) {
        BenefitCode["AccommodationBenefit"] = "accommodationBenefit";
        BenefitCode["TelephoneBenefit"] = "telephoneBenefit";
        BenefitCode["MealBenefit"] = "mealBenefit";
        BenefitCode["OtherBenefits"] = "otherBenefits";
    })(exports.BenefitCode || (exports.BenefitCode = {}));
    /** BlobFileType enumeration */
    exports.BlobFileType = void 0;
    (function (BlobFileType) {
        BlobFileType["Unknown"] = "unknown";
        BlobFileType["AuthorizationPdf"] = "authorizationPdf";
        BlobFileType["Avatar"] = "avatar";
        BlobFileType["TaxCard"] = "taxCard";
        BlobFileType["ExpensesReceipt"] = "expensesReceipt";
        BlobFileType["Raksaloki"] = "raksaloki";
        BlobFileType["Template"] = "template";
        BlobFileType["EInvoice"] = "eInvoice";
        BlobFileType["Temporary"] = "temporary";
        BlobFileType["Settings"] = "settings";
        BlobFileType["Messages"] = "messages";
        BlobFileType["MonthlyReport"] = "monthlyReport";
        BlobFileType["YearlyReport"] = "yearlyReport";
        BlobFileType["CalcReport"] = "calcReport";
    })(exports.BlobFileType || (exports.BlobFileType = {}));
    /** BlobRepository enumeration */
    exports.BlobRepository = void 0;
    (function (BlobRepository) {
        BlobRepository["UserFiles"] = "userFiles";
        BlobRepository["VersionHistory"] = "versionHistory";
        BlobRepository["SystemFiles"] = "systemFiles";
        BlobRepository["Payload"] = "payload";
        BlobRepository["Reports"] = "reports";
        BlobRepository["CdnImages"] = "cdnImages";
        BlobRepository["GitContent"] = "gitContent";
        BlobRepository["FileSystemContent"] = "fileSystemContent";
    })(exports.BlobRepository || (exports.BlobRepository = {}));
    /** CalcGroup enumeration */
    exports.CalcGroup = void 0;
    (function (CalcGroup) {
        CalcGroup["Undefined"] = "undefined";
        CalcGroup["BaseSalary"] = "baseSalary";
        CalcGroup["SalaryAdditions"] = "salaryAdditions";
        CalcGroup["Benefits"] = "benefits";
        CalcGroup["Expenses"] = "expenses";
        CalcGroup["Deductions"] = "deductions";
        CalcGroup["OtherNoPayment"] = "otherNoPayment";
        CalcGroup["Totals"] = "totals";
        CalcGroup["Disabled"] = "disabled";
    })(exports.CalcGroup || (exports.CalcGroup = {}));
    /** CalculationFlag enumeration */
    exports.CalculationFlag = void 0;
    (function (CalculationFlag) {
        CalculationFlag["Exclude"] = "exclude";
        CalculationFlag["PensionInsurance"] = "pensionInsurance";
        CalculationFlag["AccidentInsurance"] = "accidentInsurance";
        CalculationFlag["UnemploymentInsurance"] = "unemploymentInsurance";
        CalculationFlag["HealthInsurance"] = "healthInsurance";
        CalculationFlag["InsurancesDeduction"] = "insurancesDeduction";
        CalculationFlag["NoTax"] = "noTax";
        CalculationFlag["Tax"] = "tax";
        CalculationFlag["TaxDeduction"] = "taxDeduction";
        CalculationFlag["CfNoPayment"] = "cfNoPayment";
        CalculationFlag["CfPayment"] = "cfPayment";
        CalculationFlag["CfDeduction"] = "cfDeduction";
        CalculationFlag["CfDeductionAtSalaxy"] = "cfDeductionAtSalaxy";
    })(exports.CalculationFlag || (exports.CalculationFlag = {}));
    /** CalculationResultRowType enumeration */
    exports.CalculationResultRowType = void 0;
    (function (CalculationResultRowType) {
        CalculationResultRowType["Undefined"] = "undefined";
        CalculationResultRowType["TotalBenefits"] = "totalBenefits";
        CalculationResultRowType["TotalExpenses"] = "totalExpenses";
        CalculationResultRowType["TotalSocialSecurityEmployer"] = "totalSocialSecurityEmployer";
        CalculationResultRowType["TotalSocialSecurityEmployerDebt"] = "totalSocialSecurityEmployerDebt";
        CalculationResultRowType["TotalSocialSecurityEmployerPayment"] = "totalSocialSecurityEmployerPayment";
        CalculationResultRowType["TotalPension"] = "totalPension";
        CalculationResultRowType["TotalPensionPayment"] = "totalPensionPayment";
        CalculationResultRowType["TotalPensionEmployer"] = "totalPensionEmployer";
        CalculationResultRowType["TotalPensionEmployerDebt"] = "totalPensionEmployerDebt";
        CalculationResultRowType["TotalPensionWorker"] = "totalPensionWorker";
        CalculationResultRowType["TotalUnemployment"] = "totalUnemployment";
        CalculationResultRowType["TotalUnemploymentPayment"] = "totalUnemploymentPayment";
        CalculationResultRowType["TotalUnemploymentEmployer"] = "totalUnemploymentEmployer";
        CalculationResultRowType["TotalUnemploymentEmployerDebt"] = "totalUnemploymentEmployerDebt";
        CalculationResultRowType["TotalUnemploymentWorker"] = "totalUnemploymentWorker";
        CalculationResultRowType["TotalPalkkaus"] = "totalPalkkaus";
        CalculationResultRowType["TotalSalary"] = "totalSalary";
        CalculationResultRowType["TotalTax"] = "totalTax";
        CalculationResultRowType["TotalTaxPayment"] = "totalTaxPayment";
        CalculationResultRowType["TotalPayment"] = "totalPayment";
        CalculationResultRowType["TotalWorkerPayment"] = "totalWorkerPayment";
    })(exports.CalculationResultRowType || (exports.CalculationResultRowType = {}));
    /** CalculationRowSource enumeration */
    exports.CalculationRowSource = void 0;
    (function (CalculationRowSource) {
        CalculationRowSource["SalaryPage"] = "salaryPage";
        CalculationRowSource["TesSelection"] = "tesSelection";
        CalculationRowSource["ManualRow"] = "manualRow";
    })(exports.CalculationRowSource || (exports.CalculationRowSource = {}));
    /** CalculationRowType enumeration */
    exports.CalculationRowType = void 0;
    (function (CalculationRowType) {
        CalculationRowType["Unknown"] = "unknown";
        CalculationRowType["Salary"] = "salary";
        CalculationRowType["HourlySalary"] = "hourlySalary";
        CalculationRowType["MonthlySalary"] = "monthlySalary";
        CalculationRowType["TotalWorkerPayment"] = "totalWorkerPayment";
        CalculationRowType["TotalEmployerPayment"] = "totalEmployerPayment";
        CalculationRowType["Compensation"] = "compensation";
        CalculationRowType["Overtime"] = "overtime";
        CalculationRowType["TesWorktimeShortening"] = "tesWorktimeShortening";
        CalculationRowType["EveningAddition"] = "eveningAddition";
        CalculationRowType["NightimeAddition"] = "nightimeAddition";
        CalculationRowType["SaturdayAddition"] = "saturdayAddition";
        CalculationRowType["SundayWork"] = "sundayWork";
        CalculationRowType["OtherAdditions"] = "otherAdditions";
        CalculationRowType["AccomodationBenefit"] = "accomodationBenefit";
        CalculationRowType["MealBenefit"] = "mealBenefit";
        CalculationRowType["PhoneBenefit"] = "phoneBenefit";
        CalculationRowType["CarBenefit"] = "carBenefit";
        CalculationRowType["OtherBenefit"] = "otherBenefit";
        CalculationRowType["HolidayCompensation"] = "holidayCompensation";
        CalculationRowType["HolidayBonus"] = "holidayBonus";
        CalculationRowType["HolidaySalary"] = "holidaySalary";
        CalculationRowType["DailyAllowance"] = "dailyAllowance";
        CalculationRowType["DailyAllowanceHalf"] = "dailyAllowanceHalf";
        CalculationRowType["MealCompensation"] = "mealCompensation";
        CalculationRowType["MilageOwnCar"] = "milageOwnCar";
        CalculationRowType["ToolCompensation"] = "toolCompensation";
        CalculationRowType["Expenses"] = "expenses";
        CalculationRowType["MilageDaily"] = "milageDaily";
        CalculationRowType["MilageOther"] = "milageOther";
        CalculationRowType["UnionPayment"] = "unionPayment";
        CalculationRowType["Foreclosure"] = "foreclosure";
        CalculationRowType["Advance"] = "advance";
        CalculationRowType["ForeclosureByPalkkaus"] = "foreclosureByPalkkaus";
        CalculationRowType["PrepaidExpenses"] = "prepaidExpenses";
        CalculationRowType["OtherDeductions"] = "otherDeductions";
        CalculationRowType["ChildCareSubsidy"] = "childCareSubsidy";
        CalculationRowType["ChainsawReduction"] = "chainsawReduction";
        CalculationRowType["NonProfitOrg"] = "nonProfitOrg";
        CalculationRowType["SubsidisedCommute"] = "subsidisedCommute";
        CalculationRowType["IrIncomeType"] = "irIncomeType";
        CalculationRowType["Board"] = "board";
        CalculationRowType["Remuneration"] = "remuneration";
        CalculationRowType["OtherCompensation"] = "otherCompensation";
        CalculationRowType["WorkingTimeCompensation"] = "workingTimeCompensation";
        CalculationRowType["EmploymentTermination"] = "employmentTermination";
        CalculationRowType["HourlySalaryWithWorkingTimeCompensation"] = "hourlySalaryWithWorkingTimeCompensation";
        CalculationRowType["Totals"] = "totals";
    })(exports.CalculationRowType || (exports.CalculationRowType = {}));
    /** CalculationRowUnit enumeration */
    exports.CalculationRowUnit = void 0;
    (function (CalculationRowUnit) {
        CalculationRowUnit["Undefined"] = "undefined";
        CalculationRowUnit["Count"] = "count";
        CalculationRowUnit["Percent"] = "percent";
        CalculationRowUnit["Days"] = "days";
        CalculationRowUnit["Kilometers"] = "kilometers";
        CalculationRowUnit["Hours"] = "hours";
    })(exports.CalculationRowUnit || (exports.CalculationRowUnit = {}));
    /** CalculationStatus enumeration */
    exports.CalculationStatus = void 0;
    (function (CalculationStatus) {
        CalculationStatus["Draft"] = "draft";
        CalculationStatus["PaymentStarted"] = "paymentStarted";
        CalculationStatus["PaymentSucceeded"] = "paymentSucceeded";
        CalculationStatus["PaymentCanceled"] = "paymentCanceled";
        CalculationStatus["PaymentError"] = "paymentError";
        CalculationStatus["PaymentWorkerCopy"] = "paymentWorkerCopy";
        CalculationStatus["WorkerRequested"] = "workerRequested";
        CalculationStatus["WorkerRequestAccepted"] = "workerRequestAccepted";
        CalculationStatus["WorkerRequestDeclined"] = "workerRequestDeclined";
        CalculationStatus["PaymentRefunded"] = "paymentRefunded";
        CalculationStatus["WaitingApproval"] = "waitingApproval";
        CalculationStatus["PayrollDraft"] = "payrollDraft";
        CalculationStatus["ProDraft"] = "proDraft";
        CalculationStatus["SharedWaiting"] = "sharedWaiting";
        CalculationStatus["SharedApproved"] = "sharedApproved";
        CalculationStatus["SharedRejected"] = "sharedRejected";
        CalculationStatus["Template"] = "template";
    })(exports.CalculationStatus || (exports.CalculationStatus = {}));
    /** CalculationStatusCategory enumeration */
    exports.CalculationStatusCategory = void 0;
    (function (CalculationStatusCategory) {
        CalculationStatusCategory["All"] = "all";
        CalculationStatusCategory["Readonly"] = "readonly";
        CalculationStatusCategory["Editable"] = "editable";
        CalculationStatusCategory["Sent"] = "sent";
        CalculationStatusCategory["Shared"] = "shared";
        CalculationStatusCategory["Received"] = "received";
    })(exports.CalculationStatusCategory || (exports.CalculationStatusCategory = {}));
    /** CalendarActionType enumeration */
    exports.CalendarActionType = void 0;
    (function (CalendarActionType) {
        CalendarActionType["Undefined"] = "undefined";
        CalendarActionType["Display"] = "display";
        CalendarActionType["Email"] = "email";
        CalendarActionType["Audio"] = "audio";
        CalendarActionType["CreateItem"] = "createItem";
        CalendarActionType["Script"] = "script";
    })(exports.CalendarActionType || (exports.CalendarActionType = {}));
    /** CalendarEventStatus enumeration */
    exports.CalendarEventStatus = void 0;
    (function (CalendarEventStatus) {
        CalendarEventStatus["Undefined"] = "undefined";
        CalendarEventStatus["Cancelled"] = "cancelled";
        CalendarEventStatus["Confirmed"] = "confirmed";
        CalendarEventStatus["Tentative"] = "tentative";
        CalendarEventStatus["NeedsAction"] = "needsAction";
        CalendarEventStatus["Completed"] = "completed";
        CalendarEventStatus["InProcess"] = "inProcess";
    })(exports.CalendarEventStatus || (exports.CalendarEventStatus = {}));
    /** CarBenefitCode enumeration */
    exports.CarBenefitCode = void 0;
    (function (CarBenefitCode) {
        CarBenefitCode["LimitedCarBenefit"] = "limitedCarBenefit";
        CarBenefitCode["FullCarBenefit"] = "fullCarBenefit";
    })(exports.CarBenefitCode || (exports.CarBenefitCode = {}));
    /** CompanyType enumeration */
    exports.CompanyType = void 0;
    (function (CompanyType) {
        CompanyType["Unknown"] = "unknown";
        CompanyType["FiOy"] = "fiOy";
        CompanyType["FiTm"] = "fiTm";
        CompanyType["FiRy"] = "fiRy";
        CompanyType["FiYy"] = "fiYy";
    })(exports.CompanyType || (exports.CompanyType = {}));
    /** ContractPartyType enumeration */
    exports.ContractPartyType = void 0;
    (function (ContractPartyType) {
        ContractPartyType["Person"] = "person";
        ContractPartyType["CustomContact"] = "customContact";
    })(exports.ContractPartyType || (exports.ContractPartyType = {}));
    /** CustomerWebApp enumeration */
    exports.CustomerWebApp = void 0;
    (function (CustomerWebApp) {
        CustomerWebApp["Palkkaus"] = "palkkaus";
        CustomerWebApp["CustomSso"] = "customSso";
        CustomerWebApp["CustomPalkkaus"] = "customPalkkaus";
        CustomerWebApp["NoCustomerUi"] = "noCustomerUi";
    })(exports.CustomerWebApp || (exports.CustomerWebApp = {}));
    /** EmployerType enumeration */
    exports.EmployerType = void 0;
    (function (EmployerType) {
        EmployerType["Unknown"] = "unknown";
        EmployerType["MiniEmployer"] = "miniEmployer";
        EmployerType["SmallEmployer"] = "smallEmployer";
        EmployerType["FullTimeEmployer"] = "fullTimeEmployer";
        EmployerType["TyelOwnContractEmployer"] = "tyelOwnContractEmployer";
    })(exports.EmployerType || (exports.EmployerType = {}));
    /** EmploymentRelationStatus enumeration */
    exports.EmploymentRelationStatus = void 0;
    (function (EmploymentRelationStatus) {
        EmploymentRelationStatus["Unknown"] = "unknown";
        EmploymentRelationStatus["Primary"] = "primary";
        EmploymentRelationStatus["SecondaryCurrent"] = "secondaryCurrent";
        EmploymentRelationStatus["Archived"] = "archived";
    })(exports.EmploymentRelationStatus || (exports.EmploymentRelationStatus = {}));
    /** EmploymentRelationType enumeration */
    exports.EmploymentRelationType = void 0;
    (function (EmploymentRelationType) {
        EmploymentRelationType["Undefined"] = "undefined";
        EmploymentRelationType["Salary"] = "salary";
        EmploymentRelationType["HourlySalary"] = "hourlySalary";
        EmploymentRelationType["MonthlySalary"] = "monthlySalary";
        EmploymentRelationType["Compensation"] = "compensation";
        EmploymentRelationType["BoardMember"] = "boardMember";
        EmploymentRelationType["Entrepreneur"] = "entrepreneur";
        EmploymentRelationType["Farmer"] = "farmer";
        EmploymentRelationType["EmployedByStateEmploymentFund"] = "employedByStateEmploymentFund";
        EmploymentRelationType["KeyEmployee"] = "keyEmployee";
        EmploymentRelationType["Athlete"] = "athlete";
        EmploymentRelationType["PerformingArtist"] = "performingArtist";
    })(exports.EmploymentRelationType || (exports.EmploymentRelationType = {}));
    /** EmploymentSalaryType enumeration */
    exports.EmploymentSalaryType = void 0;
    (function (EmploymentSalaryType) {
        EmploymentSalaryType["Unknown"] = "unknown";
        EmploymentSalaryType["HourlyPay"] = "hourlyPay";
        EmploymentSalaryType["MontlyPay"] = "montlyPay";
        EmploymentSalaryType["OtherPay"] = "otherPay";
    })(exports.EmploymentSalaryType || (exports.EmploymentSalaryType = {}));
    /** EmploymentType enumeration */
    exports.EmploymentType = void 0;
    (function (EmploymentType) {
        EmploymentType["Unknown"] = "unknown";
        EmploymentType["Job"] = "job";
        EmploymentType["PartTime"] = "partTime";
        EmploymentType["FullTime"] = "fullTime";
    })(exports.EmploymentType || (exports.EmploymentType = {}));
    /** EmploymentWorkHoursType enumeration */
    exports.EmploymentWorkHoursType = void 0;
    (function (EmploymentWorkHoursType) {
        EmploymentWorkHoursType["HoursPerDay"] = "hoursPerDay";
        EmploymentWorkHoursType["HoursPerWeek"] = "hoursPerWeek";
        EmploymentWorkHoursType["HourPerTwoWeeks"] = "hourPerTwoWeeks";
        EmploymentWorkHoursType["HourPerThreeWeeks"] = "hourPerThreeWeeks";
    })(exports.EmploymentWorkHoursType || (exports.EmploymentWorkHoursType = {}));
    /** ESalaryPaymentStatus enumeration */
    exports.ESalaryPaymentStatus = void 0;
    (function (ESalaryPaymentStatus) {
        ESalaryPaymentStatus["Created"] = "created";
        ESalaryPaymentStatus["Shared"] = "shared";
        ESalaryPaymentStatus["Opened"] = "opened";
        ESalaryPaymentStatus["EmployerResolved"] = "employerResolved";
        ESalaryPaymentStatus["EmploymentResolved"] = "employmentResolved";
        ESalaryPaymentStatus["TaxcardResolved"] = "taxcardResolved";
        ESalaryPaymentStatus["PaymentMethodResolved"] = "paymentMethodResolved";
        ESalaryPaymentStatus["CalculationCreated"] = "calculationCreated";
        ESalaryPaymentStatus["PaymentStarted"] = "paymentStarted";
        ESalaryPaymentStatus["PaymentSucceeded"] = "paymentSucceeded";
        ESalaryPaymentStatus["PaymentCanceled"] = "paymentCanceled";
    })(exports.ESalaryPaymentStatus || (exports.ESalaryPaymentStatus = {}));
    /** ExportMethod enumeration */
    exports.ExportMethod = void 0;
    (function (ExportMethod) {
        ExportMethod["Undefined"] = "undefined";
        ExportMethod["FileExcel"] = "fileExcel";
        ExportMethod["FileCsv"] = "fileCsv";
        ExportMethod["FilePdf"] = "filePdf";
        ExportMethod["CopyExcel"] = "copyExcel";
        ExportMethod["CopyCsv"] = "copyCsv";
        ExportMethod["PaymentChannel"] = "paymentChannel";
        ExportMethod["Api"] = "api";
    })(exports.ExportMethod || (exports.ExportMethod = {}));
    /** DateOfBirthAccuracy enumeration */
    exports.DateOfBirthAccuracy = void 0;
    (function (DateOfBirthAccuracy) {
        DateOfBirthAccuracy["Assumption"] = "assumption";
        DateOfBirthAccuracy["AgeBased"] = "ageBased";
        DateOfBirthAccuracy["AgeGroupBased"] = "ageGroupBased";
        DateOfBirthAccuracy["MonthCorrect"] = "monthCorrect";
        DateOfBirthAccuracy["Exact"] = "exact";
        DateOfBirthAccuracy["Verified"] = "verified";
    })(exports.DateOfBirthAccuracy || (exports.DateOfBirthAccuracy = {}));
    /** FrameworkAgreement enumeration */
    exports.FrameworkAgreement = void 0;
    (function (FrameworkAgreement) {
        FrameworkAgreement["NotDefined"] = "notDefined";
        FrameworkAgreement["Construction"] = "construction";
        FrameworkAgreement["Mll"] = "mll";
        FrameworkAgreement["ChildCare"] = "childCare";
        FrameworkAgreement["Cleaning"] = "cleaning";
        FrameworkAgreement["SantaClaus"] = "santaClaus";
        FrameworkAgreement["Entrepreneur"] = "entrepreneur";
        FrameworkAgreement["Other"] = "other";
    })(exports.FrameworkAgreement || (exports.FrameworkAgreement = {}));
    /** Gender enumeration */
    exports.Gender = void 0;
    (function (Gender) {
        Gender["Unknown"] = "unknown";
        Gender["Male"] = "male";
        Gender["Female"] = "female";
    })(exports.Gender || (exports.Gender = {}));
    /** HelttiIndustry enumeration */
    exports.HelttiIndustry = void 0;
    (function (HelttiIndustry) {
        HelttiIndustry["NotDefined"] = "notDefined";
        HelttiIndustry["I"] = "i";
        HelttiIndustry["J"] = "j";
        HelttiIndustry["K"] = "k";
        HelttiIndustry["M"] = "m";
        HelttiIndustry["O"] = "o";
        HelttiIndustry["N"] = "n";
        HelttiIndustry["P"] = "p";
        HelttiIndustry["Q"] = "q";
        HelttiIndustry["R"] = "r";
        HelttiIndustry["S"] = "s";
        HelttiIndustry["Other"] = "other";
    })(exports.HelttiIndustry || (exports.HelttiIndustry = {}));
    /** HelttiProductPackage enumeration */
    exports.HelttiProductPackage = void 0;
    (function (HelttiProductPackage) {
        HelttiProductPackage["NotDefined"] = "notDefined";
        HelttiProductPackage["Small"] = "small";
        HelttiProductPackage["Medium"] = "medium";
        HelttiProductPackage["Large"] = "large";
    })(exports.HelttiProductPackage || (exports.HelttiProductPackage = {}));
    /** IncomeEarnerType enumeration */
    exports.IncomeEarnerType = void 0;
    (function (IncomeEarnerType) {
        IncomeEarnerType["EmployedByStateEmploymentFund"] = "employedByStateEmploymentFund";
        IncomeEarnerType["JointOwnerWithPayer"] = "jointOwnerWithPayer";
        IncomeEarnerType["PartialOwner"] = "partialOwner";
        IncomeEarnerType["KeyEmployee"] = "keyEmployee";
        IncomeEarnerType["LeasedEmployeeLivingAbroad"] = "leasedEmployeeLivingAbroad";
        IncomeEarnerType["PersonWorkingInFrontierDistrict"] = "personWorkingInFrontierDistrict";
        IncomeEarnerType["PersonWorkingAbroad"] = "personWorkingAbroad";
        IncomeEarnerType["Athlete"] = "athlete";
        IncomeEarnerType["PerformingArtist"] = "performingArtist";
        IncomeEarnerType["RestrictedPeriodInFinland"] = "restrictedPeriodInFinland";
        IncomeEarnerType["NetOfTaxContract"] = "netOfTaxContract";
        IncomeEarnerType["Organization"] = "organization";
        IncomeEarnerType["PersonWorkingOnAlandFerry"] = "personWorkingOnAlandFerry";
        IncomeEarnerType["EntrepreneurOrFarmerNoPensionRequired"] = "entrepreneurOrFarmerNoPensionRequired";
    })(exports.IncomeEarnerType || (exports.IncomeEarnerType = {}));
    /** IncomeLogDiff enumeration */
    exports.IncomeLogDiff = void 0;
    (function (IncomeLogDiff) {
        IncomeLogDiff["Default"] = "default";
        IncomeLogDiff["New"] = "new";
        IncomeLogDiff["Changed"] = "changed";
        IncomeLogDiff["Removed"] = "removed";
    })(exports.IncomeLogDiff || (exports.IncomeLogDiff = {}));
    /** InvoiceStatus enumeration */
    exports.InvoiceStatus = void 0;
    (function (InvoiceStatus) {
        InvoiceStatus["Undefined"] = "undefined";
        InvoiceStatus["Forecast"] = "forecast";
        InvoiceStatus["Preview"] = "preview";
        InvoiceStatus["WaitingPalkkaus"] = "waitingPalkkaus";
        InvoiceStatus["Unread"] = "unread";
        InvoiceStatus["Read"] = "read";
        InvoiceStatus["WaitingConfirmation"] = "waitingConfirmation";
        InvoiceStatus["PaymentStarted"] = "paymentStarted";
        InvoiceStatus["Paid"] = "paid";
        InvoiceStatus["Canceled"] = "canceled";
        InvoiceStatus["Error"] = "error";
    })(exports.InvoiceStatus || (exports.InvoiceStatus = {}));
    /** InvoiceType enumeration */
    exports.InvoiceType = void 0;
    (function (InvoiceType) {
        InvoiceType["Undefined"] = "undefined";
        InvoiceType["Net"] = "net";
        InvoiceType["Tax"] = "tax";
        InvoiceType["TaxWithholding"] = "taxWithholding";
        InvoiceType["TaxSocialSecurity"] = "taxSocialSecurity";
        InvoiceType["Unemployment"] = "unemployment";
        InvoiceType["Pension"] = "pension";
        InvoiceType["Union"] = "union";
        InvoiceType["Foreclosure"] = "foreclosure";
        InvoiceType["Gross"] = "gross";
    })(exports.InvoiceType || (exports.InvoiceType = {}));
    /** Holiday enumeration */
    exports.Holiday = void 0;
    (function (Holiday) {
        Holiday["NewYearsDay"] = "newYearsDay";
        Holiday["Epiphany"] = "epiphany";
        Holiday["GoodFriday"] = "goodFriday";
        Holiday["EasterSunday"] = "easterSunday";
        Holiday["EasterSaturday"] = "easterSaturday";
        Holiday["EasterMonday"] = "easterMonday";
        Holiday["MayDay"] = "mayDay";
        Holiday["AscensionDay"] = "ascensionDay";
        Holiday["Pentecost"] = "pentecost";
        Holiday["MidsummerEve"] = "midsummerEve";
        Holiday["MidsummerDay"] = "midsummerDay";
        Holiday["AllSaintsDay"] = "allSaintsDay";
        Holiday["IndependenceDay"] = "independenceDay";
        Holiday["ChristmasEve"] = "christmasEve";
        Holiday["ChristmasDay"] = "christmasDay";
        Holiday["StStephensDay"] = "stStephensDay";
    })(exports.Holiday || (exports.Holiday = {}));
    /** HolidayAccrualSource enumeration */
    exports.HolidayAccrualSource = void 0;
    (function (HolidayAccrualSource) {
        HolidayAccrualSource["Initial"] = "initial";
        HolidayAccrualSource["Manual"] = "manual";
        HolidayAccrualSource["CalcDraft"] = "calcDraft";
        HolidayAccrualSource["CalcPaid"] = "calcPaid";
    })(exports.HolidayAccrualSource || (exports.HolidayAccrualSource = {}));
    /** HolidayBonusPaymentMethod enumeration */
    exports.HolidayBonusPaymentMethod = void 0;
    (function (HolidayBonusPaymentMethod) {
        HolidayBonusPaymentMethod["None"] = "none";
        HolidayBonusPaymentMethod["PayForHolidaySalary"] = "payForHolidaySalary";
        HolidayBonusPaymentMethod["PaySummerBonus"] = "paySummerBonus";
        HolidayBonusPaymentMethod["Pay24Days"] = "pay24Days";
        HolidayBonusPaymentMethod["PayAllBonus"] = "payAllBonus";
    })(exports.HolidayBonusPaymentMethod || (exports.HolidayBonusPaymentMethod = {}));
    /** HolidayCode enumeration */
    exports.HolidayCode = void 0;
    (function (HolidayCode) {
        HolidayCode["Undefined"] = "undefined";
        HolidayCode["Permanent14Days"] = "permanent14Days";
        HolidayCode["Permanent35Hours"] = "permanent35Hours";
        HolidayCode["TemporaryTimeOff"] = "temporaryTimeOff";
        HolidayCode["HolidayCompensation"] = "holidayCompensation";
        HolidayCode["HolidayCompensationIncluded"] = "holidayCompensationIncluded";
        HolidayCode["NoHolidays"] = "noHolidays";
    })(exports.HolidayCode || (exports.HolidayCode = {}));
    /** HolidayGroup enumeration */
    exports.HolidayGroup = void 0;
    (function (HolidayGroup) {
        HolidayGroup["Official"] = "official";
        HolidayGroup["NonBanking"] = "nonBanking";
        HolidayGroup["Holiday"] = "holiday";
    })(exports.HolidayGroup || (exports.HolidayGroup = {}));
    /** InsuranceCompany enumeration */
    exports.InsuranceCompany = void 0;
    (function (InsuranceCompany) {
        InsuranceCompany["None"] = "none";
        InsuranceCompany["L\u00E4hiTapiola"] = "l\u00E4hiTapiola";
        InsuranceCompany["Pohjola"] = "pohjola";
        InsuranceCompany["If"] = "if";
        InsuranceCompany["Fennia"] = "fennia";
        InsuranceCompany["AVakuutus"] = "aVakuutus";
        InsuranceCompany["Aktia"] = "aktia";
        InsuranceCompany["Pohjant\u00E4hti"] = "pohjant\u00E4hti";
        InsuranceCompany["Tryg"] = "tryg";
        InsuranceCompany["\u00C5lands"] = "\u00E5lands";
        InsuranceCompany["Turva"] = "turva";
        InsuranceCompany["Redarnas"] = "redarnas";
        InsuranceCompany["Folksam"] = "folksam";
        InsuranceCompany["Alandia"] = "alandia";
        InsuranceCompany["Other"] = "other";
        InsuranceCompany["Pending"] = "pending";
    })(exports.InsuranceCompany || (exports.InsuranceCompany = {}));
    /** InvoicePaymentType enumeration */
    exports.InvoicePaymentType = void 0;
    (function (InvoicePaymentType) {
        InvoicePaymentType["Default"] = "default";
        InvoicePaymentType["Salary"] = "salary";
    })(exports.InvoicePaymentType || (exports.InvoicePaymentType = {}));
    /** IrFlags enumeration */
    exports.IrFlags = void 0;
    (function (IrFlags) {
        IrFlags["NoMoney"] = "noMoney";
        IrFlags["OneOff"] = "oneOff";
        IrFlags["UnjustEnrichment"] = "unjustEnrichment";
    })(exports.IrFlags || (exports.IrFlags = {}));
    /** IrInsuranceExceptions enumeration */
    exports.IrInsuranceExceptions = void 0;
    (function (IrInsuranceExceptions) {
        IrInsuranceExceptions["IncludeAll"] = "includeAll";
        IrInsuranceExceptions["IncludePension"] = "includePension";
        IrInsuranceExceptions["IncludeHealthInsurance"] = "includeHealthInsurance";
        IrInsuranceExceptions["IncludeUnemployment"] = "includeUnemployment";
        IrInsuranceExceptions["IncludeAccidentInsurance"] = "includeAccidentInsurance";
        IrInsuranceExceptions["ExcludeAll"] = "excludeAll";
        IrInsuranceExceptions["ExcludePension"] = "excludePension";
        IrInsuranceExceptions["ExcludeHealthInsurance"] = "excludeHealthInsurance";
        IrInsuranceExceptions["ExcludeUnemployment"] = "excludeUnemployment";
        IrInsuranceExceptions["ExcludeAccidentInsurance"] = "excludeAccidentInsurance";
    })(exports.IrInsuranceExceptions || (exports.IrInsuranceExceptions = {}));
    /** IrRowSourceType enumeration */
    exports.IrRowSourceType = void 0;
    (function (IrRowSourceType) {
        IrRowSourceType["Manual"] = "manual";
        IrRowSourceType["Usecase"] = "usecase";
        IrRowSourceType["UsecaseV02"] = "usecaseV02";
    })(exports.IrRowSourceType || (exports.IrRowSourceType = {}));
    /** LegalEntityType enumeration */
    exports.LegalEntityType = void 0;
    (function (LegalEntityType) {
        LegalEntityType["Undefined"] = "undefined";
        LegalEntityType["Person"] = "person";
        LegalEntityType["Company"] = "company";
        LegalEntityType["PersonCreatedByEmployer"] = "personCreatedByEmployer";
        LegalEntityType["Partner"] = "partner";
    })(exports.LegalEntityType || (exports.LegalEntityType = {}));
    /** LocalTapiolaInsuranceOrderAuthorizationType enumeration */
    exports.LocalTapiolaInsuranceOrderAuthorizationType = void 0;
    (function (LocalTapiolaInsuranceOrderAuthorizationType) {
        LocalTapiolaInsuranceOrderAuthorizationType["Procuration"] = "procuration";
        LocalTapiolaInsuranceOrderAuthorizationType["PowerOfAttorney"] = "powerOfAttorney";
        LocalTapiolaInsuranceOrderAuthorizationType["ApparentAuthority"] = "apparentAuthority";
        LocalTapiolaInsuranceOrderAuthorizationType["Other"] = "other";
    })(exports.LocalTapiolaInsuranceOrderAuthorizationType || (exports.LocalTapiolaInsuranceOrderAuthorizationType = {}));
    /** LocalTapiolaInsuranceOrderStatus enumeration */
    exports.LocalTapiolaInsuranceOrderStatus = void 0;
    (function (LocalTapiolaInsuranceOrderStatus) {
        LocalTapiolaInsuranceOrderStatus["Draft"] = "draft";
        LocalTapiolaInsuranceOrderStatus["WaitingApproval"] = "waitingApproval";
        LocalTapiolaInsuranceOrderStatus["Checked"] = "checked";
        LocalTapiolaInsuranceOrderStatus["Sent"] = "sent";
        LocalTapiolaInsuranceOrderStatus["Handled"] = "handled";
        LocalTapiolaInsuranceOrderStatus["Canceled"] = "canceled";
    })(exports.LocalTapiolaInsuranceOrderStatus || (exports.LocalTapiolaInsuranceOrderStatus = {}));
    /** LocalTapiolaInsuranceOrderType enumeration */
    exports.LocalTapiolaInsuranceOrderType = void 0;
    (function (LocalTapiolaInsuranceOrderType) {
        LocalTapiolaInsuranceOrderType["NewWorkerInsurance"] = "newWorkerInsurance";
        LocalTapiolaInsuranceOrderType["MoveWorkerInsurance"] = "moveWorkerInsurance";
        LocalTapiolaInsuranceOrderType["NewEntrepreneurInsurance"] = "newEntrepreneurInsurance";
        LocalTapiolaInsuranceOrderType["MoveEntrepreneurInsurance"] = "moveEntrepreneurInsurance";
        LocalTapiolaInsuranceOrderType["Other"] = "other";
    })(exports.LocalTapiolaInsuranceOrderType || (exports.LocalTapiolaInsuranceOrderType = {}));
    /** MessageFrom enumeration */
    exports.MessageFrom = void 0;
    (function (MessageFrom) {
        MessageFrom["Undefined"] = "undefined";
        MessageFrom["Owner"] = "owner";
        MessageFrom["OtherParty"] = "otherParty";
        MessageFrom["System"] = "system";
    })(exports.MessageFrom || (exports.MessageFrom = {}));
    /** MessageType enumeration */
    exports.MessageType = void 0;
    (function (MessageType) {
        MessageType["Email"] = "email";
        MessageType["Sms"] = "sms";
    })(exports.MessageType || (exports.MessageType = {}));
    /** OnboardingStatus enumeration */
    exports.OnboardingStatus = void 0;
    (function (OnboardingStatus) {
        OnboardingStatus["Created"] = "created";
        OnboardingStatus["Open"] = "open";
        OnboardingStatus["Done"] = "done";
        OnboardingStatus["Expired"] = "expired";
    })(exports.OnboardingStatus || (exports.OnboardingStatus = {}));
    /** PartnerSite enumeration */
    exports.PartnerSite = void 0;
    (function (PartnerSite) {
        PartnerSite["Unknown"] = "unknown";
        PartnerSite["Palkkaus"] = "palkkaus";
        PartnerSite["PalkkausIdentified"] = "palkkausIdentified";
        PartnerSite["PalkkausEmployerFillIn"] = "palkkausEmployerFillIn";
        PartnerSite["Example"] = "example";
        PartnerSite["Dev"] = "dev";
        PartnerSite["RefugeeJobs"] = "refugeeJobs";
        PartnerSite["Talkkarit"] = "talkkarit";
        PartnerSite["Raksa"] = "raksa";
        PartnerSite["Palkkaapakolainen"] = "palkkaapakolainen";
        PartnerSite["CareDotCom"] = "careDotCom";
        PartnerSite["Duunitori"] = "duunitori";
        PartnerSite["Mol"] = "mol";
    })(exports.PartnerSite || (exports.PartnerSite = {}));
    /** PaymentChannel enumeration */
    exports.PaymentChannel = void 0;
    (function (PaymentChannel) {
        PaymentChannel["Undefined"] = "undefined";
        PaymentChannel["Test"] = "test";
        PaymentChannel["ZeroPayment"] = "zeroPayment";
        PaymentChannel["External"] = "external";
        PaymentChannel["PalkkausManual"] = "palkkausManual";
        PaymentChannel["PalkkausWS"] = "palkkausWS";
        PaymentChannel["PalkkausPersonal"] = "palkkausPersonal";
        PaymentChannel["PalkkausCfaPaytrail"] = "palkkausCfaPaytrail";
        PaymentChannel["PalkkausCfaReference"] = "palkkausCfaReference";
        PaymentChannel["PalkkausCfaFinvoice"] = "palkkausCfaFinvoice";
        PaymentChannel["PalkkausCfaTest"] = "palkkausCfaTest";
        PaymentChannel["AccountorGo"] = "accountorGo";
        PaymentChannel["TalenomOnline"] = "talenomOnline";
        PaymentChannel["TalenomCfa"] = "talenomCfa";
        PaymentChannel["HolviCfa"] = "holviCfa";
        PaymentChannel["FinagoSolo"] = "finagoSolo";
        PaymentChannel["Procountor"] = "procountor";
        PaymentChannel["Kevytyrittaja"] = "kevytyrittaja";
        PaymentChannel["VismaNetvisor"] = "vismaNetvisor";
    })(exports.PaymentChannel || (exports.PaymentChannel = {}));
    /** PaymentMethod enumeration */
    exports.PaymentMethod = void 0;
    (function (PaymentMethod) {
        PaymentMethod["Undefined"] = "undefined";
        PaymentMethod["Sepa"] = "sepa";
        PaymentMethod["Siirto"] = "siirto";
        PaymentMethod["Test"] = "test";
    })(exports.PaymentMethod || (exports.PaymentMethod = {}));
    /** PaymentStatus enumeration */
    exports.PaymentStatus = void 0;
    (function (PaymentStatus) {
        PaymentStatus["New"] = "new";
        PaymentStatus["SentToBank"] = "sentToBank";
        PaymentStatus["BankTechApproval"] = "bankTechApproval";
        PaymentStatus["BankDelivered"] = "bankDelivered";
        PaymentStatus["BankPartialError"] = "bankPartialError";
        PaymentStatus["BankError"] = "bankError";
        PaymentStatus["Paid"] = "paid";
        PaymentStatus["Unknown"] = "unknown";
        PaymentStatus["Cancelled"] = "cancelled";
    })(exports.PaymentStatus || (exports.PaymentStatus = {}));
    /** PayrollStatus enumeration */
    exports.PayrollStatus = void 0;
    (function (PayrollStatus) {
        PayrollStatus["Draft"] = "draft";
        PayrollStatus["PaymentStarted"] = "paymentStarted";
        PayrollStatus["PaymentSucceeded"] = "paymentSucceeded";
        PayrollStatus["PaymentCanceled"] = "paymentCanceled";
        PayrollStatus["PaymentError"] = "paymentError";
        PayrollStatus["WaitingApproval"] = "waitingApproval";
        PayrollStatus["Template"] = "template";
    })(exports.PayrollStatus || (exports.PayrollStatus = {}));
    /** PensionCalculation enumeration */
    exports.PensionCalculation = void 0;
    (function (PensionCalculation) {
        PensionCalculation["Undefined"] = "undefined";
        PensionCalculation["Employee"] = "employee";
        PensionCalculation["Entrepreneur"] = "entrepreneur";
        PensionCalculation["Farmer"] = "farmer";
        PensionCalculation["PartialOwner"] = "partialOwner";
        PensionCalculation["Athlete"] = "athlete";
        PensionCalculation["Compensation"] = "compensation";
        PensionCalculation["BoardRemuneration"] = "boardRemuneration";
        PensionCalculation["SmallEntrepreneur"] = "smallEntrepreneur";
        PensionCalculation["SmallFarmer"] = "smallFarmer";
    })(exports.PensionCalculation || (exports.PensionCalculation = {}));
    /** PensionCompany enumeration */
    exports.PensionCompany = void 0;
    (function (PensionCompany) {
        PensionCompany["None"] = "none";
        PensionCompany["Etera"] = "etera";
        PensionCompany["Ilmarinen"] = "ilmarinen";
        PensionCompany["Elo"] = "elo";
        PensionCompany["PensionsAlandia"] = "pensionsAlandia";
        PensionCompany["Varma"] = "varma";
        PensionCompany["Veritas"] = "veritas";
        PensionCompany["Apteekkien"] = "apteekkien";
        PensionCompany["Verso"] = "verso";
        PensionCompany["Other"] = "other";
    })(exports.PensionCompany || (exports.PensionCompany = {}));
    /** PeriodDateKind enumeration */
    exports.PeriodDateKind = void 0;
    (function (PeriodDateKind) {
        PeriodDateKind["PaidAtDate"] = "paidAtDate";
        PeriodDateKind["SalaryDate"] = "salaryDate";
    })(exports.PeriodDateKind || (exports.PeriodDateKind = {}));
    /** PeriodType enumeration */
    exports.PeriodType = void 0;
    (function (PeriodType) {
        PeriodType["Month"] = "month";
        PeriodType["Quarter"] = "quarter";
        PeriodType["Year"] = "year";
        PeriodType["Custom"] = "custom";
    })(exports.PeriodType || (exports.PeriodType = {}));
    /** PricingModel enumeration */
    exports.PricingModel = void 0;
    (function (PricingModel) {
        PricingModel["PalkkausFee"] = "palkkausFee";
        PricingModel["NoFee"] = "noFee";
        PricingModel["FixedFee"] = "fixedFee";
    })(exports.PricingModel || (exports.PricingModel = {}));
    /** ProductPackage enumeration */
    exports.ProductPackage = void 0;
    (function (ProductPackage) {
        ProductPackage["Default"] = "default";
        ProductPackage["PartnerBasic"] = "partnerBasic";
        ProductPackage["PartnerPro"] = "partnerPro";
    })(exports.ProductPackage || (exports.ProductPackage = {}));
    /** ProductListFilter enumeration */
    exports.ProductListFilter = void 0;
    (function (ProductListFilter) {
        ProductListFilter["Available"] = "available";
        ProductListFilter["All"] = "all";
    })(exports.ProductListFilter || (exports.ProductListFilter = {}));
    /** ReportCategory enumeration */
    exports.ReportCategory = void 0;
    (function (ReportCategory) {
        ReportCategory["All"] = "all";
        ReportCategory["Monthly"] = "monthly";
        ReportCategory["Yearly"] = "yearly";
    })(exports.ReportCategory || (exports.ReportCategory = {}));
    /** ReportType enumeration */
    exports.ReportType = void 0;
    (function (ReportType) {
        ReportType["Undefined"] = "undefined";
        ReportType["Example"] = "example";
        ReportType["MonthlyDetails"] = "monthlyDetails";
        ReportType["TaxMonthly4001"] = "taxMonthly4001";
        ReportType["PayerSummaryReport"] = "payerSummaryReport";
        ReportType["MonthlyPension"] = "monthlyPension";
        ReportType["MonthlyLiikekirjuri"] = "monthlyLiikekirjuri";
        ReportType["MonthlyRapko"] = "monthlyRapko";
        ReportType["MonthlyAccounting"] = "monthlyAccounting";
        ReportType["YearlyDetails"] = "yearlyDetails";
        ReportType["YearEndReport"] = "yearEndReport";
        ReportType["YearlyWorkerSummary"] = "yearlyWorkerSummary";
        ReportType["TaxYearly7801"] = "taxYearly7801";
        ReportType["Unemployment"] = "unemployment";
        ReportType["Insurance"] = "insurance";
        ReportType["HouseholdDeduction"] = "householdDeduction";
        ReportType["TaxHouseholdDeduction14B"] = "taxHouseholdDeduction14B";
        ReportType["TaxHouseholdDeduction14BSpouseA"] = "taxHouseholdDeduction14BSpouseA";
        ReportType["TaxHouseholdDeduction14BSpouseB"] = "taxHouseholdDeduction14BSpouseB";
        ReportType["SalarySlip"] = "salarySlip";
        ReportType["SalarySlipPaid"] = "salarySlipPaid";
        ReportType["SalarySlipCopy"] = "salarySlipCopy";
        ReportType["EmployerReport"] = "employerReport";
        ReportType["PaymentReport"] = "paymentReport";
        ReportType["EarningsPaymentReport"] = "earningsPaymentReport";
        ReportType["EmploymentContract"] = "employmentContract";
    })(exports.ReportType || (exports.ReportType = {}));
    /** Role enumeration */
    exports.Role = void 0;
    (function (Role) {
        Role["Undefined"] = "undefined";
        Role["Accountant"] = "accountant";
        Role["TrustedPartner"] = "trustedPartner";
        Role["AccountantCandidate"] = "accountantCandidate";
        Role["PricingPartner"] = "pricingPartner";
    })(exports.Role || (exports.Role = {}));
    /** SalaryKind enumeration */
    exports.SalaryKind = void 0;
    (function (SalaryKind) {
        SalaryKind["Undefined"] = "undefined";
        SalaryKind["FixedSalary"] = "fixedSalary";
        SalaryKind["HourlySalary"] = "hourlySalary";
        SalaryKind["MonthlySalary"] = "monthlySalary";
        SalaryKind["Compensation"] = "compensation";
        SalaryKind["TotalWorkerPayment"] = "totalWorkerPayment";
        SalaryKind["TotalEmployerPayment"] = "totalEmployerPayment";
    })(exports.SalaryKind || (exports.SalaryKind = {}));
    /** SalaryType enumeration */
    exports.SalaryType = void 0;
    (function (SalaryType) {
        SalaryType["Tyel"] = "tyel";
        SalaryType["Yel"] = "yel";
        SalaryType["Compensation"] = "compensation";
    })(exports.SalaryType || (exports.SalaryType = {}));
    /** SalaryPerMonthRange enumeration */
    exports.SalaryPerMonthRange = void 0;
    (function (SalaryPerMonthRange) {
        SalaryPerMonthRange["Unknown"] = "unknown";
        SalaryPerMonthRange["NoPensionRequired"] = "noPensionRequired";
        SalaryPerMonthRange["Normal"] = "normal";
    })(exports.SalaryPerMonthRange || (exports.SalaryPerMonthRange = {}));
    /** SalaryPerYearRange enumeration */
    exports.SalaryPerYearRange = void 0;
    (function (SalaryPerYearRange) {
        SalaryPerYearRange["ZeroSalary"] = "zeroSalary";
        SalaryPerYearRange["NoReportingNecessary"] = "noReportingNecessary";
        SalaryPerYearRange["NoMonthlyReporting"] = "noMonthlyReporting";
        SalaryPerYearRange["Normal"] = "normal";
    })(exports.SalaryPerYearRange || (exports.SalaryPerYearRange = {}));
    /** ServiceModel enumeration */
    exports.ServiceModel = void 0;
    (function (ServiceModel) {
        ServiceModel["Shared"] = "shared";
        ServiceModel["PartnerOnly"] = "partnerOnly";
    })(exports.ServiceModel || (exports.ServiceModel = {}));
    /** SettingsStatus enumeration */
    exports.SettingsStatus = void 0;
    (function (SettingsStatus) {
        SettingsStatus["Undefined"] = "undefined";
        SettingsStatus["Checked"] = "checked";
        SettingsStatus["Pending"] = "pending";
    })(exports.SettingsStatus || (exports.SettingsStatus = {}));
    /** SharingUriType enumeration */
    exports.SharingUriType = void 0;
    (function (SharingUriType) {
        SharingUriType["Undefined"] = "undefined";
        SharingUriType["Url"] = "url";
        SharingUriType["Employer"] = "employer";
    })(exports.SharingUriType || (exports.SharingUriType = {}));
    /** TaxcardApiIncomeType enumeration */
    exports.TaxcardApiIncomeType = void 0;
    (function (TaxcardApiIncomeType) {
        TaxcardApiIncomeType["Undefined"] = "undefined";
        TaxcardApiIncomeType["Salary"] = "salary";
        TaxcardApiIncomeType["DividendsForWorkAsSalary"] = "dividendsForWorkAsSalary";
        TaxcardApiIncomeType["PersonnelFundIncome"] = "personnelFundIncome";
        TaxcardApiIncomeType["SalaryOneOff"] = "salaryOneOff";
        TaxcardApiIncomeType["CompensationForUseEarnedIncome"] = "compensationForUseEarnedIncome";
        TaxcardApiIncomeType["CompensationForUseCapitalIncome"] = "compensationForUseCapitalIncome";
        TaxcardApiIncomeType["CompensationVatResponsible"] = "compensationVatResponsible";
        TaxcardApiIncomeType["Compensation"] = "compensation";
        TaxcardApiIncomeType["DividendsForWorkAsCompensation"] = "dividendsForWorkAsCompensation";
        TaxcardApiIncomeType["AthletePay"] = "athletePay";
        TaxcardApiIncomeType["OtherTaxableIncome"] = "otherTaxableIncome";
        TaxcardApiIncomeType["OtherTaxableIncomeOneOff"] = "otherTaxableIncomeOneOff";
        TaxcardApiIncomeType["EmployeeStockOption"] = "employeeStockOption";
    })(exports.TaxcardApiIncomeType || (exports.TaxcardApiIncomeType = {}));
    /** TaxcardApprovalMethod enumeration */
    exports.TaxcardApprovalMethod = void 0;
    (function (TaxcardApprovalMethod) {
        TaxcardApprovalMethod["AssureWaiting"] = "assureWaiting";
        TaxcardApprovalMethod["Approve"] = "approve";
        TaxcardApprovalMethod["Reject"] = "reject";
    })(exports.TaxcardApprovalMethod || (exports.TaxcardApprovalMethod = {}));
    /** TaxcardCalcDiffCheck enumeration */
    exports.TaxcardCalcDiffCheck = void 0;
    (function (TaxcardCalcDiffCheck) {
        TaxcardCalcDiffCheck["GetDiff"] = "getDiff";
        TaxcardCalcDiffCheck["NoCheck"] = "noCheck";
        TaxcardCalcDiffCheck["CommitDiff"] = "commitDiff";
    })(exports.TaxcardCalcDiffCheck || (exports.TaxcardCalcDiffCheck = {}));
    /** TaxCardIncomeType enumeration */
    exports.TaxCardIncomeType = void 0;
    (function (TaxCardIncomeType) {
        TaxCardIncomeType["Unknown"] = "unknown";
        TaxCardIncomeType["SalaxyCalculation"] = "salaxyCalculation";
        TaxCardIncomeType["PreviousEmployerSalaries"] = "previousEmployerSalaries";
        TaxCardIncomeType["ExternalSalaries"] = "externalSalaries";
        TaxCardIncomeType["SharedCardExtSum"] = "sharedCardExtSum";
        TaxCardIncomeType["Diff"] = "diff";
    })(exports.TaxCardIncomeType || (exports.TaxCardIncomeType = {}));
    /** TaxcardState enumeration */
    exports.TaxcardState = void 0;
    (function (TaxcardState) {
        TaxcardState["New"] = "new";
        TaxcardState["Approved"] = "approved";
        TaxcardState["EmployerAdded"] = "employerAdded";
        TaxcardState["VerifiedVero"] = "verifiedVero";
        TaxcardState["SharedWaiting"] = "sharedWaiting";
        TaxcardState["SharedApproved"] = "sharedApproved";
        TaxcardState["SharedRejected"] = "sharedRejected";
        TaxcardState["SharedRejectedWithoutOpen"] = "sharedRejectedWithoutOpen";
        TaxcardState["Shared"] = "shared";
    })(exports.TaxcardState || (exports.TaxcardState = {}));
    /** TaxcardKind enumeration */
    exports.TaxcardKind = void 0;
    (function (TaxcardKind) {
        TaxcardKind["Undefined"] = "undefined";
        TaxcardKind["NoTaxCard"] = "noTaxCard";
        TaxcardKind["Auto"] = "auto";
        TaxcardKind["DefaultYearly"] = "defaultYearly";
        TaxcardKind["Replacement"] = "replacement";
        TaxcardKind["NoWithholdingHousehold"] = "noWithholdingHousehold";
        TaxcardKind["Others"] = "others";
        TaxcardKind["Historical"] = "historical";
    })(exports.TaxcardKind || (exports.TaxcardKind = {}));
    /** TaxcardValidity enumeration */
    exports.TaxcardValidity = void 0;
    (function (TaxcardValidity) {
        TaxcardValidity["Undefined"] = "undefined";
        TaxcardValidity["Valid"] = "valid";
        TaxcardValidity["ValidJanuary"] = "validJanuary";
        TaxcardValidity["Future"] = "future";
        TaxcardValidity["Expired"] = "expired";
    })(exports.TaxcardValidity || (exports.TaxcardValidity = {}));
    /** TaxDeductionWorkCategories enumeration */
    exports.TaxDeductionWorkCategories = void 0;
    (function (TaxDeductionWorkCategories) {
        TaxDeductionWorkCategories["None"] = "none";
        TaxDeductionWorkCategories["Householdwork"] = "householdwork";
        TaxDeductionWorkCategories["Carework"] = "carework";
        TaxDeductionWorkCategories["HomeImprovement"] = "homeImprovement";
        TaxDeductionWorkCategories["OwnPropety"] = "ownPropety";
        TaxDeductionWorkCategories["RelativesProperty"] = "relativesProperty";
    })(exports.TaxDeductionWorkCategories || (exports.TaxDeductionWorkCategories = {}));
    /** TaxReportHandling enumeration */
    exports.TaxReportHandling = void 0;
    (function (TaxReportHandling) {
        TaxReportHandling["Default"] = "default";
        TaxReportHandling["NoZeroSalaryReport"] = "noZeroSalaryReport";
        TaxReportHandling["NoMonthlyReport"] = "noMonthlyReport";
        TaxReportHandling["Always"] = "always";
    })(exports.TaxReportHandling || (exports.TaxReportHandling = {}));
    /** TesSubtype enumeration */
    exports.TesSubtype = void 0;
    (function (TesSubtype) {
        TesSubtype["NotSelected"] = "notSelected";
        TesSubtype["ConstructionCarpenter"] = "constructionCarpenter";
        TesSubtype["ConstructionFloor"] = "constructionFloor";
        TesSubtype["ConstructionOther"] = "constructionOther";
        TesSubtype["ConstructionFreeContract"] = "constructionFreeContract";
    })(exports.TesSubtype || (exports.TesSubtype = {}));
    /** TestEnum enumeration */
    exports.TestEnum = void 0;
    (function (TestEnum) {
        TestEnum["ValueZero"] = "valueZero";
        TestEnum["ValueOne"] = "valueOne";
        TestEnum["ValueTwo"] = "valueTwo";
        TestEnum["ValueThree"] = "valueThree";
        TestEnum["ValueFour"] = "valueFour";
        TestEnum["ValueFive"] = "valueFive";
        TestEnum["ValueSix"] = "valueSix";
        TestEnum["ValueSeven"] = "valueSeven";
        TestEnum["ValueEight"] = "valueEight";
        TestEnum["ValueNine"] = "valueNine";
        TestEnum["ValueTen"] = "valueTen";
    })(exports.TestEnum || (exports.TestEnum = {}));
    /** ThreadedMessageType enumeration */
    exports.ThreadedMessageType = void 0;
    (function (ThreadedMessageType) {
        ThreadedMessageType["Text"] = "text";
        ThreadedMessageType["BlobFile"] = "blobFile";
        ThreadedMessageType["Calculation"] = "calculation";
        ThreadedMessageType["Payroll"] = "payroll";
        ThreadedMessageType["WorkerAccount"] = "workerAccount";
        ThreadedMessageType["Email"] = "email";
        ThreadedMessageType["Sms"] = "sms";
    })(exports.ThreadedMessageType || (exports.ThreadedMessageType = {}));
    /** UnionPaymentType enumeration */
    exports.UnionPaymentType = void 0;
    (function (UnionPaymentType) {
        UnionPaymentType["NotSelected"] = "notSelected";
        UnionPaymentType["RaksaNormal"] = "raksaNormal";
        UnionPaymentType["RaksaUnemploymentOnly"] = "raksaUnemploymentOnly";
        UnionPaymentType["Other"] = "other";
    })(exports.UnionPaymentType || (exports.UnionPaymentType = {}));
    /** Unit enumeration */
    exports.Unit = void 0;
    (function (Unit) {
        Unit["Undefined"] = "undefined";
        Unit["Hours"] = "hours";
        Unit["Days"] = "days";
        Unit["Weeks"] = "weeks";
        Unit["Period"] = "period";
        Unit["One"] = "one";
        Unit["Count"] = "count";
        Unit["Percent"] = "percent";
        Unit["Kilometers"] = "kilometers";
        Unit["Euro"] = "euro";
    })(exports.Unit || (exports.Unit = {}));
    /** VarmaPensionOrderAction enumeration */
    exports.VarmaPensionOrderAction = void 0;
    (function (VarmaPensionOrderAction) {
        VarmaPensionOrderAction["Undefined"] = "undefined";
        VarmaPensionOrderAction["NewYel"] = "newYel";
        VarmaPensionOrderAction["NewTyel"] = "newTyel";
        VarmaPensionOrderAction["Move"] = "move";
    })(exports.VarmaPensionOrderAction || (exports.VarmaPensionOrderAction = {}));
    /** VarmaPensionOrderYelPayer enumeration */
    exports.VarmaPensionOrderYelPayer = void 0;
    (function (VarmaPensionOrderYelPayer) {
        VarmaPensionOrderYelPayer["Undefined"] = "undefined";
        VarmaPensionOrderYelPayer["Entrepreneur"] = "entrepreneur";
        VarmaPensionOrderYelPayer["Company"] = "company";
    })(exports.VarmaPensionOrderYelPayer || (exports.VarmaPensionOrderYelPayer = {}));
    /** WageBasis enumeration */
    exports.WageBasis = void 0;
    (function (WageBasis) {
        WageBasis["Undefined"] = "undefined";
        WageBasis["Monthly"] = "monthly";
        WageBasis["Hourly"] = "hourly";
        WageBasis["PerformanceBased"] = "performanceBased";
        WageBasis["Other"] = "other";
    })(exports.WageBasis || (exports.WageBasis = {}));
    /** WebSiteUserRole enumeration */
    exports.WebSiteUserRole = void 0;
    (function (WebSiteUserRole) {
        WebSiteUserRole["None"] = "none";
        WebSiteUserRole["Household"] = "household";
        WebSiteUserRole["Worker"] = "worker";
        WebSiteUserRole["Company"] = "company";
    })(exports.WebSiteUserRole || (exports.WebSiteUserRole = {}));
    /** WorkflowEventFeatures enumeration */
    exports.WorkflowEventFeatures = void 0;
    (function (WorkflowEventFeatures) {
        WorkflowEventFeatures["None"] = "none";
        WorkflowEventFeatures["IsActive"] = "isActive";
        WorkflowEventFeatures["IsEditable"] = "isEditable";
        WorkflowEventFeatures["IsRemovable"] = "isRemovable";
        WorkflowEventFeatures["IsOnHold"] = "isOnHold";
    })(exports.WorkflowEventFeatures || (exports.WorkflowEventFeatures = {}));
    /** YearEndAdminCheck enumeration */
    exports.YearEndAdminCheck = void 0;
    (function (YearEndAdminCheck) {
        YearEndAdminCheck["NotChecked"] = "notChecked";
        YearEndAdminCheck["Ok"] = "ok";
        YearEndAdminCheck["OkWithWarning"] = "okWithWarning";
        YearEndAdminCheck["Errors"] = "errors";
    })(exports.YearEndAdminCheck || (exports.YearEndAdminCheck = {}));
    /** YearEndFeedbackCalculationType enumeration */
    exports.YearEndFeedbackCalculationType = void 0;
    (function (YearEndFeedbackCalculationType) {
        YearEndFeedbackCalculationType["Unknown"] = "unknown";
        YearEndFeedbackCalculationType["SalaxyPayment"] = "salaxyPayment";
        YearEndFeedbackCalculationType["External"] = "external";
        YearEndFeedbackCalculationType["Correction"] = "correction";
    })(exports.YearEndFeedbackCalculationType || (exports.YearEndFeedbackCalculationType = {}));
    /** YearEndUserFeedback enumeration */
    exports.YearEndUserFeedback = void 0;
    (function (YearEndUserFeedback) {
        YearEndUserFeedback["NotChecked"] = "notChecked";
        YearEndUserFeedback["Ok"] = "ok";
        YearEndUserFeedback["OkWithModifications"] = "okWithModifications";
        YearEndUserFeedback["WillHandleMyself"] = "willHandleMyself";
        YearEndUserFeedback["NoYearlyReports"] = "noYearlyReports";
    })(exports.YearEndUserFeedback || (exports.YearEndUserFeedback = {}));
    /** YtjCompanyType enumeration */
    exports.YtjCompanyType = void 0;
    (function (YtjCompanyType) {
        YtjCompanyType["NoCompanyType"] = "noCompanyType";
        YtjCompanyType["HousingCooperative"] = "housingCooperative";
        YtjCompanyType["Condominium"] = "condominium";
        YtjCompanyType["AsoAssociation"] = "asoAssociation";
        YtjCompanyType["OpenCompany"] = "openCompany";
        YtjCompanyType["Association"] = "association";
        YtjCompanyType["HypoAssociation"] = "hypoAssociation";
        YtjCompanyType["MutualRealEstateCompany"] = "mutualRealEstateCompany";
        YtjCompanyType["Ky"] = "ky";
        YtjCompanyType["Osuuskunta"] = "osuuskunta";
        YtjCompanyType["CooperativeBank"] = "cooperativeBank";
        YtjCompanyType["Oy"] = "oy";
        YtjCompanyType["Foundation"] = "foundation";
        YtjCompanyType["SavingsBank"] = "savingsBank";
        YtjCompanyType["FinancialAssociation"] = "financialAssociation";
        YtjCompanyType["StateEstablishment"] = "stateEstablishment";
        YtjCompanyType["InsuranceAssociation"] = "insuranceAssociation";
        YtjCompanyType["PrivateEntrepreneur"] = "privateEntrepreneur";
        YtjCompanyType["OtherAssociation"] = "otherAssociation";
        YtjCompanyType["SpecialPurposeAssociation"] = "specialPurposeAssociation";
        YtjCompanyType["ForestCareAssociation"] = "forestCareAssociation";
        YtjCompanyType["OtherFinancialAssociation"] = "otherFinancialAssociation";
        YtjCompanyType["ReligiousCommunity"] = "religiousCommunity";
        YtjCompanyType["TaxableGrouping"] = "taxableGrouping";
        YtjCompanyType["MutualInsuranceAssociation"] = "mutualInsuranceAssociation";
        YtjCompanyType["ForeignOrganisation"] = "foreignOrganisation";
        YtjCompanyType["MunicipalEstablishment"] = "municipalEstablishment";
        YtjCompanyType["FederationOfMunicipalitiesEstablishment"] = "federationOfMunicipalitiesEstablishment";
        YtjCompanyType["AlandFederation"] = "alandFederation";
        YtjCompanyType["EuropeanCooperative"] = "europeanCooperative";
        YtjCompanyType["EuropeanCooperativeBank"] = "europeanCooperativeBank";
        YtjCompanyType["ReindeerHerdingCooperative"] = "reindeerHerdingCooperative";
        YtjCompanyType["Unknown"] = "unknown";
    })(exports.YtjCompanyType || (exports.YtjCompanyType = {}));
    /** CarBenefitKind enumeration */
    exports.CarBenefitKind = void 0;
    (function (CarBenefitKind) {
        CarBenefitKind["Undefined"] = "undefined";
        CarBenefitKind["LimitedCarBenefit"] = "limitedCarBenefit";
        CarBenefitKind["FullCarBenefit"] = "fullCarBenefit";
    })(exports.CarBenefitKind || (exports.CarBenefitKind = {}));
    /** UnionPaymentKind enumeration */
    exports.UnionPaymentKind = void 0;
    (function (UnionPaymentKind) {
        UnionPaymentKind["Undefined"] = "undefined";
        UnionPaymentKind["Fixed"] = "fixed";
        UnionPaymentKind["Percentage"] = "percentage";
        UnionPaymentKind["RaksaNormal"] = "raksaNormal";
        UnionPaymentKind["RaksaUnemploymentOnly"] = "raksaUnemploymentOnly";
        UnionPaymentKind["Other"] = "other";
    })(exports.UnionPaymentKind || (exports.UnionPaymentKind = {}));
    /** NonProfitOrgKind enumeration */
    exports.NonProfitOrgKind = void 0;
    (function (NonProfitOrgKind) {
        NonProfitOrgKind["Undefined"] = "undefined";
        NonProfitOrgKind["KilometreAllowance"] = "kilometreAllowance";
        NonProfitOrgKind["DailyAllowance"] = "dailyAllowance";
        NonProfitOrgKind["AccomodationAllowance"] = "accomodationAllowance";
    })(exports.NonProfitOrgKind || (exports.NonProfitOrgKind = {}));
    /** SubsidisedCommuteKind enumeration */
    exports.SubsidisedCommuteKind = void 0;
    (function (SubsidisedCommuteKind) {
        SubsidisedCommuteKind["Undefined"] = "undefined";
        SubsidisedCommuteKind["NoDeduction"] = "noDeduction";
        SubsidisedCommuteKind["SingleDeduction"] = "singleDeduction";
        SubsidisedCommuteKind["PeriodicalDeduction"] = "periodicalDeduction";
    })(exports.SubsidisedCommuteKind || (exports.SubsidisedCommuteKind = {}));
    /** MealBenefitKind enumeration */
    exports.MealBenefitKind = void 0;
    (function (MealBenefitKind) {
        MealBenefitKind["Undefined"] = "undefined";
        MealBenefitKind["CateringContract"] = "cateringContract";
        MealBenefitKind["MealTicket"] = "mealTicket";
        MealBenefitKind["TaxableAmount"] = "taxableAmount";
        MealBenefitKind["MealAllowance"] = "mealAllowance";
        MealBenefitKind["Institute"] = "institute";
        MealBenefitKind["Teacher"] = "teacher";
        MealBenefitKind["RestaurantWorker"] = "restaurantWorker";
    })(exports.MealBenefitKind || (exports.MealBenefitKind = {}));
    /** DailyAllowanceKind enumeration */
    exports.DailyAllowanceKind = void 0;
    (function (DailyAllowanceKind) {
        DailyAllowanceKind["Undefined"] = "undefined";
        DailyAllowanceKind["FullDailyAllowance"] = "fullDailyAllowance";
        DailyAllowanceKind["PartialDailyAllowance"] = "partialDailyAllowance";
        DailyAllowanceKind["InternationalDailyAllowance"] = "internationalDailyAllowance";
        DailyAllowanceKind["MealAllowance"] = "mealAllowance";
    })(exports.DailyAllowanceKind || (exports.DailyAllowanceKind = {}));
    /** BoardKind enumeration */
    exports.BoardKind = void 0;
    (function (BoardKind) {
        BoardKind["Undefined"] = "undefined";
    })(exports.BoardKind || (exports.BoardKind = {}));
    /** RemunerationKind enumeration */
    exports.RemunerationKind = void 0;
    (function (RemunerationKind) {
        RemunerationKind["Undefined"] = "undefined";
        RemunerationKind["InitiativeFee"] = "initiativeFee";
        RemunerationKind["BonusPay"] = "bonusPay";
        RemunerationKind["Commission"] = "commission";
        RemunerationKind["PerformanceBonus"] = "performanceBonus";
        RemunerationKind["ShareIssueForEmployees"] = "shareIssueForEmployees";
        RemunerationKind["ProfitSharingBonus"] = "profitSharingBonus";
    })(exports.RemunerationKind || (exports.RemunerationKind = {}));
    /** OtherCompensationKind enumeration */
    exports.OtherCompensationKind = void 0;
    (function (OtherCompensationKind) {
        OtherCompensationKind["Undefined"] = "undefined";
        OtherCompensationKind["MeetingFee"] = "meetingFee";
        OtherCompensationKind["LectureFee"] = "lectureFee";
        OtherCompensationKind["PositionOfTrustCompensation"] = "positionOfTrustCompensation";
        OtherCompensationKind["AccruedTimeOffCompensation"] = "accruedTimeOffCompensation";
        OtherCompensationKind["MembershipOfGoverningBodyCompensation"] = "membershipOfGoverningBodyCompensation";
        OtherCompensationKind["MonetaryGiftForEmployees"] = "monetaryGiftForEmployees";
        OtherCompensationKind["UseCompensationAsEarnedIncome"] = "useCompensationAsEarnedIncome";
        OtherCompensationKind["UseCompensationAsCapitalIncome"] = "useCompensationAsCapitalIncome";
        OtherCompensationKind["OtherTaxableIncomeAsEarnedIncome"] = "otherTaxableIncomeAsEarnedIncome";
        OtherCompensationKind["StockOptionsAndGrants"] = "stockOptionsAndGrants";
        OtherCompensationKind["EmployeeInventionCompensation"] = "employeeInventionCompensation";
        OtherCompensationKind["CapitalIncomePayment"] = "capitalIncomePayment";
        OtherCompensationKind["WorkEffortBasedDividendsAsWage"] = "workEffortBasedDividendsAsWage";
        OtherCompensationKind["WorkEffortBasedDividendsAsNonWage"] = "workEffortBasedDividendsAsNonWage";
        OtherCompensationKind["EmployeeStockOption"] = "employeeStockOption";
        OtherCompensationKind["EmployeeStockOptionWithLowerPrice"] = "employeeStockOptionWithLowerPrice";
    })(exports.OtherCompensationKind || (exports.OtherCompensationKind = {}));
    /** WorkingTimeCompensationKind enumeration */
    exports.WorkingTimeCompensationKind = void 0;
    (function (WorkingTimeCompensationKind) {
        WorkingTimeCompensationKind["Undefined"] = "undefined";
        WorkingTimeCompensationKind["EmergencyWorkCompensation"] = "emergencyWorkCompensation";
        WorkingTimeCompensationKind["EveningWorkCompensation"] = "eveningWorkCompensation";
        WorkingTimeCompensationKind["EveningShiftAllowance"] = "eveningShiftAllowance";
        WorkingTimeCompensationKind["SaturdayPay"] = "saturdayPay";
        WorkingTimeCompensationKind["ExtraWorkPremium"] = "extraWorkPremium";
        WorkingTimeCompensationKind["OtherCompensation"] = "otherCompensation";
        WorkingTimeCompensationKind["WaitingTimeCompensation"] = "waitingTimeCompensation";
        WorkingTimeCompensationKind["SundayWorkCompensation"] = "sundayWorkCompensation";
        WorkingTimeCompensationKind["StandByCompensation"] = "standByCompensation";
        WorkingTimeCompensationKind["WeeklyRestCompensation"] = "weeklyRestCompensation";
        WorkingTimeCompensationKind["OvertimeCompensation"] = "overtimeCompensation";
        WorkingTimeCompensationKind["NightWorkAllowance"] = "nightWorkAllowance";
        WorkingTimeCompensationKind["NightShiftCompensation"] = "nightShiftCompensation";
    })(exports.WorkingTimeCompensationKind || (exports.WorkingTimeCompensationKind = {}));
    /** EmploymentTerminationKind enumeration */
    exports.EmploymentTerminationKind = void 0;
    (function (EmploymentTerminationKind) {
        EmploymentTerminationKind["Undefined"] = "undefined";
        EmploymentTerminationKind["NoticePeriodCompensation"] = "noticePeriodCompensation";
        EmploymentTerminationKind["MonetaryWorkingTimeBankCompensation"] = "monetaryWorkingTimeBankCompensation";
        EmploymentTerminationKind["TerminationAndLayOffDamages"] = "terminationAndLayOffDamages";
        EmploymentTerminationKind["VoluntaryTerminationCompensation"] = "voluntaryTerminationCompensation";
        EmploymentTerminationKind["PensionPaidByEmployer"] = "pensionPaidByEmployer";
    })(exports.EmploymentTerminationKind || (exports.EmploymentTerminationKind = {}));
    /** ForeclosureKind enumeration */
    exports.ForeclosureKind = void 0;
    (function (ForeclosureKind) {
        ForeclosureKind["Undefined"] = "undefined";
        ForeclosureKind["Fixed"] = "fixed";
        ForeclosureKind["Periodic"] = "periodic";
        ForeclosureKind["NonPeriodic"] = "nonPeriodic";
    })(exports.ForeclosureKind || (exports.ForeclosureKind = {}));
    /** TotalWorkerPaymentKind enumeration */
    exports.TotalWorkerPaymentKind = void 0;
    (function (TotalWorkerPaymentKind) {
        TotalWorkerPaymentKind["Undefined"] = "undefined";
        TotalWorkerPaymentKind["TotalWorkerPayment"] = "totalWorkerPayment";
        TotalWorkerPaymentKind["TotalWorkerWorkerPaymentExcludingExpenses"] = "totalWorkerWorkerPaymentExcludingExpenses";
        TotalWorkerPaymentKind["TotalWorkerPaymentAndTax"] = "totalWorkerPaymentAndTax";
        TotalWorkerPaymentKind["TotalWorkerWorkerPaymentAndTaxExcludingExpenses"] = "totalWorkerWorkerPaymentAndTaxExcludingExpenses";
    })(exports.TotalWorkerPaymentKind || (exports.TotalWorkerPaymentKind = {}));
    /** TransactionCode enumeration */
    exports.TransactionCode = void 0;
    (function (TransactionCode) {
        TransactionCode["Unknown"] = "unknown";
        TransactionCode["Ignored"] = "ignored";
        TransactionCode["TimeRatePay"] = "timeRatePay";
        TransactionCode["InitiativeFee"] = "initiativeFee";
        TransactionCode["BonusPay"] = "bonusPay";
        TransactionCode["ComplementaryWagePaidDuringBenefitPeriod"] = "complementaryWagePaidDuringBenefitPeriod";
        TransactionCode["EmergencyWorkCompensation"] = "emergencyWorkCompensation";
        TransactionCode["EveningWorkCompensation"] = "eveningWorkCompensation";
        TransactionCode["EveningShiftAllowance"] = "eveningShiftAllowance";
        TransactionCode["NoticePeriodCompensation"] = "noticePeriodCompensation";
        TransactionCode["KilometreAllowanceTaxable"] = "kilometreAllowanceTaxable";
        TransactionCode["MeetingFee"] = "meetingFee";
        TransactionCode["SaturdayPay"] = "saturdayPay";
        TransactionCode["ExtraWorkPremium"] = "extraWorkPremium";
        TransactionCode["HolidayBonus"] = "holidayBonus";
        TransactionCode["LectureFee"] = "lectureFee";
        TransactionCode["CompensationForTrustPosition"] = "compensationForTrustPosition";
        TransactionCode["OtherCompensation"] = "otherCompensation";
        TransactionCode["WaitingTimeCompensation"] = "waitingTimeCompensation";
        TransactionCode["WorkingConditionCompensation"] = "workingConditionCompensation";
        TransactionCode["PartialPayDuringSickLeave"] = "partialPayDuringSickLeave";
        TransactionCode["Commission"] = "commission";
        TransactionCode["SundayWorkCompensation"] = "sundayWorkCompensation";
        TransactionCode["BenefitArisingFromSyntheticOption"] = "benefitArisingFromSyntheticOption";
        TransactionCode["PerformanceBonus"] = "performanceBonus";
        TransactionCode["MonetaryCompensationFromWorkingTimeBank"] = "monetaryCompensationFromWorkingTimeBank";
        TransactionCode["CompensationForAccruedTimeOff"] = "compensationForAccruedTimeOff";
        TransactionCode["ShareIssueForEmployees"] = "shareIssueForEmployees";
        TransactionCode["ContractPay"] = "contractPay";
        TransactionCode["DamagesInConjunctionWithTermination"] = "damagesInConjunctionWithTermination";
        TransactionCode["StandByCompensation"] = "standByCompensation";
        TransactionCode["VoluntaryCompensationInConjunctionWithTermination"] = "voluntaryCompensationInConjunctionWithTermination";
        TransactionCode["WeeklyRestCompensation"] = "weeklyRestCompensation";
        TransactionCode["ProfitSharingBonus"] = "profitSharingBonus";
        TransactionCode["AnnualHolidayCompensation"] = "annualHolidayCompensation";
        TransactionCode["OvertimeCompensation"] = "overtimeCompensation";
        TransactionCode["NightWorkAllowance"] = "nightWorkAllowance";
        TransactionCode["NightShiftCompensation"] = "nightShiftCompensation";
        TransactionCode["OtherRegularCompensation"] = "otherRegularCompensation";
        TransactionCode["CompensationForUnusedCompensatoryLeave"] = "compensationForUnusedCompensatoryLeave";
        TransactionCode["AccommodationBenefit"] = "accommodationBenefit";
        TransactionCode["InterestBenefitForHousingLoan"] = "interestBenefitForHousingLoan";
        TransactionCode["MealAllowance"] = "mealAllowance";
        TransactionCode["CarBenefit"] = "carBenefit";
        TransactionCode["CompensationForGoverningBodyMembership"] = "compensationForGoverningBodyMembership";
        TransactionCode["ShareOfReserveDrawnFromPersonnelFund"] = "shareOfReserveDrawnFromPersonnelFund";
        TransactionCode["MonetaryGiftForEmployees"] = "monetaryGiftForEmployees";
        TransactionCode["KilometreAllowanceTaxExempt"] = "kilometreAllowanceTaxExempt";
        TransactionCode["TreatmentFeeForMunicipalVeterinarian"] = "treatmentFeeForMunicipalVeterinarian";
        TransactionCode["CompensationForUseEarnedIncome"] = "compensationForUseEarnedIncome";
        TransactionCode["CompensationForUseCapitalIncome"] = "compensationForUseCapitalIncome";
        TransactionCode["OtherTaxableBenefitForEmployees"] = "otherTaxableBenefitForEmployees";
        TransactionCode["OtherTaxableIncomeDeemedEarnedIncome"] = "otherTaxableIncomeDeemedEarnedIncome";
        TransactionCode["OtherFringeBenefit"] = "otherFringeBenefit";
        TransactionCode["KinshipCarersFee"] = "kinshipCarersFee";
        TransactionCode["StockOptions"] = "stockOptions";
        TransactionCode["WagesPaidBySubstitutePayerIncludingSocialInsurance"] = "wagesPaidBySubstitutePayerIncludingSocialInsurance";
        TransactionCode["WagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance"] = "wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance";
        TransactionCode["WagesPaidBySubstitutePayerIncludingUnemploymentInsurance"] = "wagesPaidBySubstitutePayerIncludingUnemploymentInsurance";
        TransactionCode["WagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance"] = "wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance";
        TransactionCode["WagesPaidBySubstitutePayerIncludingHealthInsurance"] = "wagesPaidBySubstitutePayerIncludingHealthInsurance";
        TransactionCode["CompensationForEmployeeInvention"] = "compensationForEmployeeInvention";
        TransactionCode["ReimbursementOfPrivateCaretakersExpenses"] = "reimbursementOfPrivateCaretakersExpenses";
        TransactionCode["PrivateCaretakersFee"] = "privateCaretakersFee";
        TransactionCode["ReimbursementOfFamilyDayCareProvidersExpenses"] = "reimbursementOfFamilyDayCareProvidersExpenses";
        TransactionCode["TelephoneBenefit"] = "telephoneBenefit";
        TransactionCode["DailyAllowance"] = "dailyAllowance";
        TransactionCode["CapitalIncomePayment"] = "capitalIncomePayment";
        TransactionCode["MealBenefit"] = "mealBenefit";
        TransactionCode["ReimbursementOfCostsPaidToConciliator"] = "reimbursementOfCostsPaidToConciliator";
        TransactionCode["NonWageCompensationForWork"] = "nonWageCompensationForWork";
        TransactionCode["SupplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund"] = "supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund";
        TransactionCode["PensionPaidByEmployer"] = "pensionPaidByEmployer";
        TransactionCode["DividendsBasedOnWages"] = "dividendsBasedOnWages";
        TransactionCode["DividendsBasedOnNonWage"] = "dividendsBasedOnNonWage";
        TransactionCode["EmployerSubsidisedCommuterTicketTaxExempt"] = "employerSubsidisedCommuterTicketTaxExempt";
        TransactionCode["EmployerSubsidisedCommuterTicketTaxable"] = "employerSubsidisedCommuterTicketTaxable";
        TransactionCode["EmployeeStockOption"] = "employeeStockOption";
        TransactionCode["WagesTransferredToAthletesSpecialFund"] = "wagesTransferredToAthletesSpecialFund";
        TransactionCode["WagesPaidFromAthletesSpecialFund"] = "wagesPaidFromAthletesSpecialFund";
        TransactionCode["WagesForInsurancePurposes"] = "wagesForInsurancePurposes";
        TransactionCode["TaxableReimbursementOfExpenses"] = "taxableReimbursementOfExpenses";
        TransactionCode["PrivateDayCareAllowanceMunicipalSupplement"] = "privateDayCareAllowanceMunicipalSupplement";
        TransactionCode["PrivateDayCareAllowanceWages"] = "privateDayCareAllowanceWages";
        TransactionCode["PrivateDayCareAllowanceNonWage"] = "privateDayCareAllowanceNonWage";
        TransactionCode["KilometreAllowancePaidByNonProfitOrganisation"] = "kilometreAllowancePaidByNonProfitOrganisation";
        TransactionCode["DailyAllowancePaidByNonProfitOrganisation"] = "dailyAllowancePaidByNonProfitOrganisation";
        TransactionCode["UnjustEnrichment"] = "unjustEnrichment";
        TransactionCode["EmployeeStockOptionLowerMarketPrice"] = "employeeStockOptionLowerMarketPrice";
        TransactionCode["RoyaltyPaidToNonResidentTaxpayer"] = "royaltyPaidToNonResidentTaxpayer";
        TransactionCode["BicycleBenefitTaxExempt"] = "bicycleBenefitTaxExempt";
        TransactionCode["BicycleBenefitTaxable"] = "bicycleBenefitTaxable";
        TransactionCode["ConditionalStockOptions"] = "conditionalStockOptions";
        TransactionCode["CompensationCollectedForCarBenefit"] = "compensationCollectedForCarBenefit";
        TransactionCode["WithholdingTax"] = "withholdingTax";
        TransactionCode["ElectedOfficialFee"] = "electedOfficialFee";
        TransactionCode["TaxAtSource"] = "taxAtSource";
        TransactionCode["TaxAtSourceDeduction"] = "taxAtSourceDeduction";
        TransactionCode["WagesPaid"] = "wagesPaid";
        TransactionCode["ReimbursementCollectedForOtherFringeBenefits"] = "reimbursementCollectedForOtherFringeBenefits";
        TransactionCode["OtherItemDeductibleFromNetWage"] = "otherItemDeductibleFromNetWage";
        TransactionCode["NetWage"] = "netWage";
        TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsurance"] = "employerPaidPremiumForCollectiveAdditionalPensionInsurance";
        TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution"] = "employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution";
        TransactionCode["EmployeesHealthInsuranceContribution"] = "employeesHealthInsuranceContribution";
        TransactionCode["EmployeesPensionInsuranceContribution"] = "employeesPensionInsuranceContribution";
        TransactionCode["EmployeesUnemploymentInsuranceContribution"] = "employeesUnemploymentInsuranceContribution";
        TransactionCode["ReimbursementForEmployerSubsidisedCommuterTicket"] = "reimbursementForEmployerSubsidisedCommuterTicket";
        TransactionCode["TaxPaidAbroad"] = "taxPaidAbroad";
        TransactionCode["Distraint"] = "distraint";
        TransactionCode["VoluntaryIndividualPensionInsurancePremium"] = "voluntaryIndividualPensionInsurancePremium";
        TransactionCode["DeductionBeforeWithholding"] = "deductionBeforeWithholding";
        TransactionCode["ReimbursementCollectedForBicycleBenefit"] = "reimbursementCollectedForBicycleBenefit";
    })(exports.TransactionCode || (exports.TransactionCode = {}));
    exports.WorkflowEventUi = void 0;
    (function (WorkflowEventUi) {
        WorkflowEventUi["Default"] = "default";
        WorkflowEventUi["Primary"] = "primary";
        WorkflowEventUi["Success"] = "success";
        WorkflowEventUi["Info"] = "info";
        WorkflowEventUi["Warning"] = "warning";
        WorkflowEventUi["Danger"] = "danger";
    })(exports.WorkflowEventUi || (exports.WorkflowEventUi = {}));
    exports.EmploymentContractListItemStatus = void 0;
    (function (EmploymentContractListItemStatus) {
        EmploymentContractListItemStatus["Draft"] = "draft";
        EmploymentContractListItemStatus["LegacyHousehold"] = "legacyHousehold";
    })(exports.EmploymentContractListItemStatus || (exports.EmploymentContractListItemStatus = {}));
    exports.UserInfoApplicationRole = void 0;
    (function (UserInfoApplicationRole) {
        UserInfoApplicationRole["Unknown"] = "unknown";
        UserInfoApplicationRole["NoAccess"] = "noAccess";
        UserInfoApplicationRole["ReadOnly"] = "readOnly";
        UserInfoApplicationRole["ReadWrite"] = "readWrite";
        UserInfoApplicationRole["Owner"] = "owner";
    })(exports.UserInfoApplicationRole || (exports.UserInfoApplicationRole = {}));

    /* tslint:disable */
    /* eslint-disable */
    //----------------------
    // <auto-generated>
    //     Generated using the NSwag toolchain v13.9.4.0 (NJsonSchema v10.3.1.0 (Newtonsoft.Json v11.0.0.0)) (http://NSwag.org)
    // </auto-generated>
    //----------------------
    // ReSharper disable InconsistentNaming
    exports.irepr = void 0;
    (function (irepr) {
        (function (EarningsPaymentReportStatus) {
            EarningsPaymentReportStatus["New"] = "new";
            EarningsPaymentReportStatus["Scheduled"] = "scheduled";
            EarningsPaymentReportStatus["Succeeded"] = "succeeded";
            EarningsPaymentReportStatus["Canceled"] = "canceled";
            EarningsPaymentReportStatus["Error"] = "error";
            EarningsPaymentReportStatus["Invalid"] = "invalid";
        })(irepr.EarningsPaymentReportStatus || (irepr.EarningsPaymentReportStatus = {}));
        (function (ReportLogEventType) {
            ReportLogEventType["Undefined"] = "undefined";
            ReportLogEventType["Sent"] = "sent";
            ReportLogEventType["Canceled"] = "canceled";
            ReportLogEventType["Error"] = "error";
        })(irepr.ReportLogEventType || (irepr.ReportLogEventType = {}));
        (function (DeliveryDataType) {
            DeliveryDataType["EarningsPaymentReports"] = "earningsPaymentReports";
            DeliveryDataType["EmployersSeparateReports"] = "employersSeparateReports";
            DeliveryDataType["BenefitsPaymentReports"] = "benefitsPaymentReports";
            DeliveryDataType["RecordSubscription"] = "recordSubscription";
            DeliveryDataType["Messages"] = "messages";
            DeliveryDataType["CancellationOfEarningsPaymentReports"] = "cancellationOfEarningsPaymentReports";
            DeliveryDataType["CancellationOfEmployersSeparateReports"] = "cancellationOfEmployersSeparateReports";
            DeliveryDataType["CancellationOfBenefitsPaymentReports"] = "cancellationOfBenefitsPaymentReports";
            DeliveryDataType["CancellationOfARecordSubscription"] = "cancellationOfARecordSubscription";
            DeliveryDataType["CancellationOfARecordContainingEarningsPaymentReports"] = "cancellationOfARecordContainingEarningsPaymentReports";
            DeliveryDataType["CancellationOfARecordContainingEmployersSeparateReports"] = "cancellationOfARecordContainingEmployersSeparateReports";
            DeliveryDataType["CancellationOfARecordContainingBenefitsPaymentReports"] = "cancellationOfARecordContainingBenefitsPaymentReports";
            DeliveryDataType["CancellationOfARecordContainingARecordSubscription"] = "cancellationOfARecordContainingARecordSubscription";
        })(irepr.DeliveryDataType || (irepr.DeliveryDataType = {}));
        (function (PayerSubOrgType) {
            PayerSubOrgType["KevasSubmitterCodes"] = "kevasSubmitterCodes";
            PayerSubOrgType["PayersOwnCodes"] = "payersOwnCodes";
            PayerSubOrgType["GovernmentAgencyIdentifier"] = "governmentAgencyIdentifier";
        })(irepr.PayerSubOrgType || (irepr.PayerSubOrgType = {}));
        (function (UnpaidAbsenceCauseCode) {
            UnpaidAbsenceCauseCode["Illness"] = "illness";
            UnpaidAbsenceCauseCode["PartTimeSickLeave"] = "partTimeSickLeave";
            UnpaidAbsenceCauseCode["ParentalLeave"] = "parentalLeave";
            UnpaidAbsenceCauseCode["SpecialMaternityLeave"] = "specialMaternityLeave";
            UnpaidAbsenceCauseCode["Rehabilitation"] = "rehabilitation";
            UnpaidAbsenceCauseCode["ChildIllness"] = "childIllness";
            UnpaidAbsenceCauseCode["PartTimeChildCareLeave"] = "partTimeChildCareLeave";
            UnpaidAbsenceCauseCode["Training"] = "training";
            UnpaidAbsenceCauseCode["JobAlternationLeave"] = "jobAlternationLeave";
            UnpaidAbsenceCauseCode["StudyLeave"] = "studyLeave";
            UnpaidAbsenceCauseCode["IndustrialAction"] = "industrialAction";
            UnpaidAbsenceCauseCode["InterruptionInWorkProvision"] = "interruptionInWorkProvision";
            UnpaidAbsenceCauseCode["LeaveOfAbsence"] = "leaveOfAbsence";
            UnpaidAbsenceCauseCode["MilitaryRefresherTraining"] = "militaryRefresherTraining";
            UnpaidAbsenceCauseCode["MilitaryService"] = "militaryService";
            UnpaidAbsenceCauseCode["LayOff"] = "layOff";
            UnpaidAbsenceCauseCode["ChildCareLeave"] = "childCareLeave";
            UnpaidAbsenceCauseCode["Other"] = "other";
        })(irepr.UnpaidAbsenceCauseCode || (irepr.UnpaidAbsenceCauseCode = {}));
        (function (PaidAbsenceCauseCode) {
            PaidAbsenceCauseCode["Illness"] = "illness";
            PaidAbsenceCauseCode["PartTimeSickLeave"] = "partTimeSickLeave";
            PaidAbsenceCauseCode["ParentalLeave"] = "parentalLeave";
            PaidAbsenceCauseCode["SpecialMaternityLeave"] = "specialMaternityLeave";
            PaidAbsenceCauseCode["Rehabilitation"] = "rehabilitation";
            PaidAbsenceCauseCode["ChildIllness"] = "childIllness";
            PaidAbsenceCauseCode["PartTimeChildCareLeave"] = "partTimeChildCareLeave";
            PaidAbsenceCauseCode["Training"] = "training";
            PaidAbsenceCauseCode["LeaveOfAbsence"] = "leaveOfAbsence";
            PaidAbsenceCauseCode["MilitaryRefresherTraining"] = "militaryRefresherTraining";
            PaidAbsenceCauseCode["MidWeekHoliday"] = "midWeekHoliday";
            PaidAbsenceCauseCode["AccruedHoliday"] = "accruedHoliday";
            PaidAbsenceCauseCode["OccupationalAccident"] = "occupationalAccident";
            PaidAbsenceCauseCode["AnnualLeave"] = "annualLeave";
            PaidAbsenceCauseCode["PartTimeAbsenceDueToRehabilitation"] = "partTimeAbsenceDueToRehabilitation";
            PaidAbsenceCauseCode["Other"] = "other";
        })(irepr.PaidAbsenceCauseCode || (irepr.PaidAbsenceCauseCode = {}));
        (function (ActionCode) {
            ActionCode["NewReport"] = "newReport";
            ActionCode["ReplacementReport"] = "replacementReport";
        })(irepr.ActionCode || (irepr.ActionCode = {}));
        (function (AddressType) {
            AddressType["AddressInHomeCountry"] = "addressInHomeCountry";
            AddressType["AddressInTheCountryOfWork"] = "addressInTheCountryOfWork";
        })(irepr.AddressType || (irepr.AddressType = {}));
        (function (AgeGroupCode) {
            AgeGroupCode["A"] = "a";
            AgeGroupCode["B"] = "b";
            AgeGroupCode["C"] = "c";
            AgeGroupCode["U"] = "u";
        })(irepr.AgeGroupCode || (irepr.AgeGroupCode = {}));
        (function (AllowanceCode) {
            AllowanceCode["MealAllowance"] = "mealAllowance";
            AllowanceCode["PartialDailyAllowance"] = "partialDailyAllowance";
            AllowanceCode["FullDailyAllowance"] = "fullDailyAllowance";
            AllowanceCode["InternationalDailyAllowance"] = "internationalDailyAllowance";
            AllowanceCode["TaxExemptReimbursementsAbroad"] = "taxExemptReimbursementsAbroad";
        })(irepr.AllowanceCode || (irepr.AllowanceCode = {}));
        (function (BenefitCode) {
            BenefitCode["AccommodationBenefit"] = "accommodationBenefit";
            BenefitCode["TelephoneBenefit"] = "telephoneBenefit";
            BenefitCode["MealBenefit"] = "mealBenefit";
            BenefitCode["OtherBenefits"] = "otherBenefits";
        })(irepr.BenefitCode || (irepr.BenefitCode = {}));
        (function (CarBenefitCode) {
            CarBenefitCode["LimitedCarBenefit"] = "limitedCarBenefit";
            CarBenefitCode["FullCarBenefit"] = "fullCarBenefit";
        })(irepr.CarBenefitCode || (irepr.CarBenefitCode = {}));
        (function (EmploymentCode) {
            EmploymentCode["FullTime"] = "fullTime";
            EmploymentCode["PartTime"] = "partTime";
            EmploymentCode["NotAvailable"] = "notAvailable";
        })(irepr.EmploymentCode || (irepr.EmploymentCode = {}));
        (function (EmploymentEndingCode) {
            EmploymentEndingCode["EmployeesResignation"] = "employeesResignation";
            EmploymentEndingCode["Other"] = "other";
            EmploymentEndingCode["TerminationGroundsRelatedToEmployeesPerson"] = "terminationGroundsRelatedToEmployeesPerson";
            EmploymentEndingCode["FinancialAndProductionRelatedGrounds"] = "financialAndProductionRelatedGrounds";
            EmploymentEndingCode["JointAgreement"] = "jointAgreement";
            EmploymentEndingCode["Retirement"] = "retirement";
            EmploymentEndingCode["TerminationOfaFixedTermEmployment"] = "terminationOfaFixedTermEmployment";
        })(irepr.EmploymentEndingCode || (irepr.EmploymentEndingCode = {}));
        (function (EmploymentEndingType) {
            EmploymentEndingType["KevaCodes"] = "kevaCodes";
            EmploymentEndingType["BankOfFinlandCodes"] = "bankOfFinlandCodes";
            EmploymentEndingType["IncomesRegisterCodes"] = "incomesRegisterCodes";
        })(irepr.EmploymentEndingType || (irepr.EmploymentEndingType = {}));
        (function (ExceptionCode) {
            ExceptionCode["NoObligationToSocialInsurance"] = "noObligationToSocialInsurance";
            ExceptionCode["NoObligationToHealthInsurance"] = "noObligationToHealthInsurance";
            ExceptionCode["NoObligationToEarningsRelatedPensionInsurance"] = "noObligationToEarningsRelatedPensionInsurance";
            ExceptionCode["NoObligationToAccidentAndOccupationalDiseaseInsurance"] = "noObligationToAccidentAndOccupationalDiseaseInsurance";
            ExceptionCode["NoObligationToUnemploymentInsurance"] = "noObligationToUnemploymentInsurance";
            ExceptionCode["NotSubjectToSocialInsurance"] = "notSubjectToSocialInsurance";
            ExceptionCode["NotSubjectToEarningsRelatedPensionInsurance"] = "notSubjectToEarningsRelatedPensionInsurance";
            ExceptionCode["NotSubjectToAccidentAndOccupationalDiseaseInsurance"] = "notSubjectToAccidentAndOccupationalDiseaseInsurance";
            ExceptionCode["NotSubjectToUnemploymentInsurance"] = "notSubjectToUnemploymentInsurance";
            ExceptionCode["NotSubjectToHealthInsurance"] = "notSubjectToHealthInsurance";
            ExceptionCode["VoluntaryEarningsRelatedPensionInsurance"] = "voluntaryEarningsRelatedPensionInsurance";
        })(irepr.ExceptionCode || (irepr.ExceptionCode = {}));
        (function (FaultyControl) {
            FaultyControl["OnlyInvalidItems"] = "onlyInvalidItems";
            FaultyControl["EntireRecordIsRejected"] = "entireRecordIsRejected";
        })(irepr.FaultyControl || (irepr.FaultyControl = {}));
        (function (Gender) {
            Gender["Male"] = "male";
            Gender["Female"] = "female";
        })(irepr.Gender || (irepr.Gender = {}));
        (function (IdType) {
            IdType["BusinessId"] = "businessId";
            IdType["PersonalIdentificationNumber"] = "personalIdentificationNumber";
            IdType["Vat"] = "vat";
            IdType["Giin"] = "giin";
            IdType["TaxIdentificationNumber"] = "taxIdentificationNumber";
            IdType["FinnishTradeRegistrationNumber"] = "finnishTradeRegistrationNumber";
            IdType["ForeignBusinessRegistrationNumber"] = "foreignBusinessRegistrationNumber";
            IdType["ForeignPersonalIdentificationNumber"] = "foreignPersonalIdentificationNumber";
            IdType["Other"] = "other";
        })(irepr.IdType || (irepr.IdType = {}));
        (function (InsuranceCode) {
            InsuranceCode["SubjectToSocialInsuranceContributions"] = "subjectToSocialInsuranceContributions";
            InsuranceCode["SubjectToEarningsRelatedPensionInsuranceContribution"] = "subjectToEarningsRelatedPensionInsuranceContribution";
            InsuranceCode["SubjectToHealthInsuranceContribution"] = "subjectToHealthInsuranceContribution";
            InsuranceCode["SubjectToUnemploymentInsuranceContribution"] = "subjectToUnemploymentInsuranceContribution";
            InsuranceCode["SubjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution"] = "subjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution";
        })(irepr.InsuranceCode || (irepr.InsuranceCode = {}));
        (function (Language) {
            Language["Fi"] = "fi";
            Language["Sv"] = "sv";
            Language["En"] = "en";
        })(irepr.Language || (irepr.Language = {}));
        (function (PayerType) {
            PayerType["PublicSector"] = "publicSector";
            PayerType["Household"] = "household";
            PayerType["TemporaryEmployer"] = "temporaryEmployer";
            PayerType["ForeignEmployer"] = "foreignEmployer";
            PayerType["State"] = "state";
            PayerType["UnincorporatedStateEnterpriseOrGovernmentalInstitution"] = "unincorporatedStateEnterpriseOrGovernmentalInstitution";
            PayerType["SpecialisedAgency"] = "specialisedAgency";
            PayerType["ForeignGroupCompany"] = "foreignGroupCompany";
        })(irepr.PayerType || (irepr.PayerType = {}));
        (function (PaymentType) {
            PaymentType["Monthly"] = "monthly";
            PaymentType["Hourly"] = "hourly";
            PaymentType["ContractPay"] = "contractPay";
        })(irepr.PaymentType || (irepr.PaymentType = {}));
        (function (PensionActCode) {
            PensionActCode["EmployeesEarningsRelatedPensionInsurance"] = "employeesEarningsRelatedPensionInsurance";
            PensionActCode["PensionInsuranceForFarmers"] = "pensionInsuranceForFarmers";
            PensionActCode["PensionInsuranceForTheSelfEmployed"] = "pensionInsuranceForTheSelfEmployed";
        })(irepr.PensionActCode || (irepr.PensionActCode = {}));
        (function (PensionProvIdCode) {
            PensionProvIdCode["None"] = "none";
            PensionProvIdCode["FinnishCentreForPensions"] = "finnishCentreForPensions";
            PensionProvIdCode["ClergyOfTheOrthodoxChurch"] = "clergyOfTheOrthodoxChurch";
            PensionProvIdCode["KevaMemberOrganisations"] = "kevaMemberOrganisations";
            PensionProvIdCode["Keva\u00C5landLandskapsregeringsPensionssystem"] = "keva\u00C5landLandskapsregeringsPensionssystem";
            PensionProvIdCode["KevaChurch"] = "kevaChurch";
            PensionProvIdCode["BankOfFinland"] = "bankOfFinland";
            PensionProvIdCode["KevaKelaEmploymentPensions"] = "kevaKelaEmploymentPensions";
            PensionProvIdCode["KevaState"] = "kevaState";
            PensionProvIdCode["Seafarers"] = "seafarers";
            PensionProvIdCode["Farmers"] = "farmers";
            PensionProvIdCode["Alandia"] = "alandia";
            PensionProvIdCode["Ilmarinen"] = "ilmarinen";
            PensionProvIdCode["Elo"] = "elo";
            PensionProvIdCode["Varma"] = "varma";
            PensionProvIdCode["Veritas"] = "veritas";
            PensionProvIdCode["Etera"] = "etera";
            PensionProvIdCode["UPM"] = "uPM";
            PensionProvIdCode["Sanoma"] = "sanoma";
            PensionProvIdCode["Sandvik"] = "sandvik";
            PensionProvIdCode["Kontino"] = "kontino";
            PensionProvIdCode["Yleisradio"] = "yleisradio";
            PensionProvIdCode["ABB"] = "aBB";
            PensionProvIdCode["LFashionGroup"] = "lFashionGroup";
            PensionProvIdCode["Honeywell"] = "honeywell";
            PensionProvIdCode["Telia"] = "telia";
            PensionProvIdCode["YaraSuomi"] = "yaraSuomi";
            PensionProvIdCode["Orion"] = "orion";
            PensionProvIdCode["Valio"] = "valio";
            PensionProvIdCode["OP"] = "oP";
            PensionProvIdCode["Verso"] = "verso";
            PensionProvIdCode["Apteekkien"] = "apteekkien";
            PensionProvIdCode["Viabek"] = "viabek";
            PensionProvIdCode["Reka"] = "reka";
        })(irepr.PensionProvIdCode || (irepr.PensionProvIdCode = {}));
        (function (ProfessionType) {
            ProfessionType["StatisticsFinland"] = "statisticsFinland";
            ProfessionType["Keva"] = "keva";
            ProfessionType["BankOfFinland"] = "bankOfFinland";
            ProfessionType["Trafi"] = "trafi";
        })(irepr.ProfessionType || (irepr.ProfessionType = {}));
        (function (RefPaymentType) {
            RefPaymentType["EarningsRelatedPensionProvider"] = "earningsRelatedPensionProvider";
        })(irepr.RefPaymentType || (irepr.RefPaymentType = {}));
        (function (RemunerationCode) {
            RemunerationCode["DailyAllowance"] = "dailyAllowance";
            RemunerationCode["AccommodationBenefit"] = "accommodationBenefit";
            RemunerationCode["CarBenefit"] = "carBenefit";
            RemunerationCode["OtherBenefit"] = "otherBenefit";
        })(irepr.RemunerationCode || (irepr.RemunerationCode = {}));
        (function (ResponsibilityCode) {
            ResponsibilityCode["ContentIssues"] = "contentIssues";
            ResponsibilityCode["TechnicalIssues"] = "technicalIssues";
        })(irepr.ResponsibilityCode || (irepr.ResponsibilityCode = {}));
        (function (TermCode) {
            TermCode["UntilFurtherNotice"] = "untilFurtherNotice";
            TermCode["FixedTerm"] = "fixedTerm";
        })(irepr.TermCode || (irepr.TermCode = {}));
        (function (TransactionCode) {
            TransactionCode["Unknown"] = "unknown";
            TransactionCode["Ignored"] = "ignored";
            TransactionCode["TimeRatePay"] = "timeRatePay";
            TransactionCode["InitiativeFee"] = "initiativeFee";
            TransactionCode["BonusPay"] = "bonusPay";
            TransactionCode["ComplementaryWagePaidDuringBenefitPeriod"] = "complementaryWagePaidDuringBenefitPeriod";
            TransactionCode["EmergencyWorkCompensation"] = "emergencyWorkCompensation";
            TransactionCode["EveningWorkCompensation"] = "eveningWorkCompensation";
            TransactionCode["EveningShiftAllowance"] = "eveningShiftAllowance";
            TransactionCode["NoticePeriodCompensation"] = "noticePeriodCompensation";
            TransactionCode["KilometreAllowanceTaxable"] = "kilometreAllowanceTaxable";
            TransactionCode["MeetingFee"] = "meetingFee";
            TransactionCode["SaturdayPay"] = "saturdayPay";
            TransactionCode["ExtraWorkPremium"] = "extraWorkPremium";
            TransactionCode["HolidayBonus"] = "holidayBonus";
            TransactionCode["LectureFee"] = "lectureFee";
            TransactionCode["CompensationForTrustPosition"] = "compensationForTrustPosition";
            TransactionCode["OtherCompensation"] = "otherCompensation";
            TransactionCode["WaitingTimeCompensation"] = "waitingTimeCompensation";
            TransactionCode["WorkingConditionCompensation"] = "workingConditionCompensation";
            TransactionCode["PartialPayDuringSickLeave"] = "partialPayDuringSickLeave";
            TransactionCode["Commission"] = "commission";
            TransactionCode["SundayWorkCompensation"] = "sundayWorkCompensation";
            TransactionCode["BenefitArisingFromSyntheticOption"] = "benefitArisingFromSyntheticOption";
            TransactionCode["PerformanceBonus"] = "performanceBonus";
            TransactionCode["MonetaryCompensationFromWorkingTimeBank"] = "monetaryCompensationFromWorkingTimeBank";
            TransactionCode["CompensationForAccruedTimeOff"] = "compensationForAccruedTimeOff";
            TransactionCode["ShareIssueForEmployees"] = "shareIssueForEmployees";
            TransactionCode["ContractPay"] = "contractPay";
            TransactionCode["DamagesInConjunctionWithTermination"] = "damagesInConjunctionWithTermination";
            TransactionCode["StandByCompensation"] = "standByCompensation";
            TransactionCode["VoluntaryCompensationInConjunctionWithTermination"] = "voluntaryCompensationInConjunctionWithTermination";
            TransactionCode["WeeklyRestCompensation"] = "weeklyRestCompensation";
            TransactionCode["ProfitSharingBonus"] = "profitSharingBonus";
            TransactionCode["AnnualHolidayCompensation"] = "annualHolidayCompensation";
            TransactionCode["OvertimeCompensation"] = "overtimeCompensation";
            TransactionCode["NightWorkAllowance"] = "nightWorkAllowance";
            TransactionCode["NightShiftCompensation"] = "nightShiftCompensation";
            TransactionCode["OtherRegularCompensation"] = "otherRegularCompensation";
            TransactionCode["CompensationForUnusedCompensatoryLeave"] = "compensationForUnusedCompensatoryLeave";
            TransactionCode["AccommodationBenefit"] = "accommodationBenefit";
            TransactionCode["InterestBenefitForHousingLoan"] = "interestBenefitForHousingLoan";
            TransactionCode["MealAllowance"] = "mealAllowance";
            TransactionCode["CarBenefit"] = "carBenefit";
            TransactionCode["CompensationForGoverningBodyMembership"] = "compensationForGoverningBodyMembership";
            TransactionCode["ShareOfReserveDrawnFromPersonnelFund"] = "shareOfReserveDrawnFromPersonnelFund";
            TransactionCode["MonetaryGiftForEmployees"] = "monetaryGiftForEmployees";
            TransactionCode["KilometreAllowanceTaxExempt"] = "kilometreAllowanceTaxExempt";
            TransactionCode["TreatmentFeeForMunicipalVeterinarian"] = "treatmentFeeForMunicipalVeterinarian";
            TransactionCode["CompensationForUseEarnedIncome"] = "compensationForUseEarnedIncome";
            TransactionCode["CompensationForUseCapitalIncome"] = "compensationForUseCapitalIncome";
            TransactionCode["OtherTaxableBenefitForEmployees"] = "otherTaxableBenefitForEmployees";
            TransactionCode["OtherTaxableIncomeDeemedEarnedIncome"] = "otherTaxableIncomeDeemedEarnedIncome";
            TransactionCode["OtherFringeBenefit"] = "otherFringeBenefit";
            TransactionCode["KinshipCarersFee"] = "kinshipCarersFee";
            TransactionCode["StockOptions"] = "stockOptions";
            TransactionCode["WagesPaidBySubstitutePayerIncludingSocialInsurance"] = "wagesPaidBySubstitutePayerIncludingSocialInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance"] = "wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingUnemploymentInsurance"] = "wagesPaidBySubstitutePayerIncludingUnemploymentInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance"] = "wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingHealthInsurance"] = "wagesPaidBySubstitutePayerIncludingHealthInsurance";
            TransactionCode["CompensationForEmployeeInvention"] = "compensationForEmployeeInvention";
            TransactionCode["ReimbursementOfPrivateCaretakersExpenses"] = "reimbursementOfPrivateCaretakersExpenses";
            TransactionCode["PrivateCaretakersFee"] = "privateCaretakersFee";
            TransactionCode["ReimbursementOfFamilyDayCareProvidersExpenses"] = "reimbursementOfFamilyDayCareProvidersExpenses";
            TransactionCode["TelephoneBenefit"] = "telephoneBenefit";
            TransactionCode["DailyAllowance"] = "dailyAllowance";
            TransactionCode["CapitalIncomePayment"] = "capitalIncomePayment";
            TransactionCode["MealBenefit"] = "mealBenefit";
            TransactionCode["ReimbursementOfCostsPaidToConciliator"] = "reimbursementOfCostsPaidToConciliator";
            TransactionCode["NonWageCompensationForWork"] = "nonWageCompensationForWork";
            TransactionCode["SupplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund"] = "supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund";
            TransactionCode["PensionPaidByEmployer"] = "pensionPaidByEmployer";
            TransactionCode["DividendsBasedOnWages"] = "dividendsBasedOnWages";
            TransactionCode["DividendsBasedOnNonWage"] = "dividendsBasedOnNonWage";
            TransactionCode["EmployerSubsidisedCommuterTicketTaxExempt"] = "employerSubsidisedCommuterTicketTaxExempt";
            TransactionCode["EmployerSubsidisedCommuterTicketTaxable"] = "employerSubsidisedCommuterTicketTaxable";
            TransactionCode["EmployeeStockOption"] = "employeeStockOption";
            TransactionCode["WagesTransferredToAthletesSpecialFund"] = "wagesTransferredToAthletesSpecialFund";
            TransactionCode["WagesPaidFromAthletesSpecialFund"] = "wagesPaidFromAthletesSpecialFund";
            TransactionCode["WagesForInsurancePurposes"] = "wagesForInsurancePurposes";
            TransactionCode["TaxableReimbursementOfExpenses"] = "taxableReimbursementOfExpenses";
            TransactionCode["PrivateDayCareAllowanceMunicipalSupplement"] = "privateDayCareAllowanceMunicipalSupplement";
            TransactionCode["PrivateDayCareAllowanceWages"] = "privateDayCareAllowanceWages";
            TransactionCode["PrivateDayCareAllowanceNonWage"] = "privateDayCareAllowanceNonWage";
            TransactionCode["KilometreAllowancePaidByNonProfitOrganisation"] = "kilometreAllowancePaidByNonProfitOrganisation";
            TransactionCode["DailyAllowancePaidByNonProfitOrganisation"] = "dailyAllowancePaidByNonProfitOrganisation";
            TransactionCode["UnjustEnrichment"] = "unjustEnrichment";
            TransactionCode["EmployeeStockOptionLowerMarketPrice"] = "employeeStockOptionLowerMarketPrice";
            TransactionCode["RoyaltyPaidToNonResidentTaxpayer"] = "royaltyPaidToNonResidentTaxpayer";
            TransactionCode["BicycleBenefitTaxExempt"] = "bicycleBenefitTaxExempt";
            TransactionCode["BicycleBenefitTaxable"] = "bicycleBenefitTaxable";
            TransactionCode["ConditionalStockOptions"] = "conditionalStockOptions";
            TransactionCode["CompensationCollectedForCarBenefit"] = "compensationCollectedForCarBenefit";
            TransactionCode["WithholdingTax"] = "withholdingTax";
            TransactionCode["ElectedOfficialFee"] = "electedOfficialFee";
            TransactionCode["TaxAtSource"] = "taxAtSource";
            TransactionCode["TaxAtSourceDeduction"] = "taxAtSourceDeduction";
            TransactionCode["WagesPaid"] = "wagesPaid";
            TransactionCode["ReimbursementCollectedForOtherFringeBenefits"] = "reimbursementCollectedForOtherFringeBenefits";
            TransactionCode["OtherItemDeductibleFromNetWage"] = "otherItemDeductibleFromNetWage";
            TransactionCode["NetWage"] = "netWage";
            TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsurance"] = "employerPaidPremiumForCollectiveAdditionalPensionInsurance";
            TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution"] = "employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution";
            TransactionCode["EmployeesHealthInsuranceContribution"] = "employeesHealthInsuranceContribution";
            TransactionCode["EmployeesPensionInsuranceContribution"] = "employeesPensionInsuranceContribution";
            TransactionCode["EmployeesUnemploymentInsuranceContribution"] = "employeesUnemploymentInsuranceContribution";
            TransactionCode["ReimbursementForEmployerSubsidisedCommuterTicket"] = "reimbursementForEmployerSubsidisedCommuterTicket";
            TransactionCode["TaxPaidAbroad"] = "taxPaidAbroad";
            TransactionCode["Distraint"] = "distraint";
            TransactionCode["VoluntaryIndividualPensionInsurancePremium"] = "voluntaryIndividualPensionInsurancePremium";
            TransactionCode["DeductionBeforeWithholding"] = "deductionBeforeWithholding";
            TransactionCode["ReimbursementCollectedForBicycleBenefit"] = "reimbursementCollectedForBicycleBenefit";
        })(irepr.TransactionCode || (irepr.TransactionCode = {}));
        (function (WageUnitCode) {
            WageUnitCode["Hour"] = "hour";
            WageUnitCode["Day"] = "day";
            WageUnitCode["Week"] = "week";
            WageUnitCode["Period"] = "period";
        })(irepr.WageUnitCode || (irepr.WageUnitCode = {}));
        (function (ReimbPaymentType) {
            ReimbPaymentType["KelaDailyAllowanceApplication"] = "kelaDailyAllowanceApplication";
            ReimbPaymentType["KelaFamilyLeaveCompensation"] = "kelaFamilyLeaveCompensation";
        })(irepr.ReimbPaymentType || (irepr.ReimbPaymentType = {}));
        (function (ApiValidationErrorType) {
            ApiValidationErrorType["General"] = "general";
            ApiValidationErrorType["Required"] = "required";
            ApiValidationErrorType["Invalid"] = "invalid";
            ApiValidationErrorType["Warning"] = "warning";
        })(irepr.ApiValidationErrorType || (irepr.ApiValidationErrorType = {}));
        (function (IrScheduleAction) {
            IrScheduleAction["Send"] = "send";
            IrScheduleAction["Cancel"] = "cancel";
        })(irepr.IrScheduleAction || (irepr.IrScheduleAction = {}));
        (function (AvatarEntityType) {
            AvatarEntityType["Undefined"] = "undefined";
            AvatarEntityType["Person"] = "person";
            AvatarEntityType["Company"] = "company";
            AvatarEntityType["PersonCreatedByEmployer"] = "personCreatedByEmployer";
            AvatarEntityType["Partner"] = "partner";
        })(irepr.AvatarEntityType || (irepr.AvatarEntityType = {}));
        (function (AvatarPictureType) {
            AvatarPictureType["Icon"] = "icon";
            AvatarPictureType["Uploaded"] = "uploaded";
            AvatarPictureType["Gravatar"] = "gravatar";
        })(irepr.AvatarPictureType || (irepr.AvatarPictureType = {}));
        (function (WorkflowEventUi) {
            WorkflowEventUi["Default"] = "default";
            WorkflowEventUi["Primary"] = "primary";
            WorkflowEventUi["Success"] = "success";
            WorkflowEventUi["Info"] = "info";
            WorkflowEventUi["Warning"] = "warning";
            WorkflowEventUi["Danger"] = "danger";
        })(irepr.WorkflowEventUi || (irepr.WorkflowEventUi = {}));
        (function (Features) {
            Features["None"] = "none";
            Features["IsActive"] = "isActive";
            Features["IsEditable"] = "isEditable";
            Features["IsRemovable"] = "isRemovable";
            Features["IsOnHold"] = "isOnHold";
        })(irepr.Features || (irepr.Features = {}));
        (function (EmploymentRegType) {
            EmploymentRegType["KevasGroundsForRegistration"] = "kevasGroundsForRegistration";
            EmploymentRegType["BankOfFinlandsGroundsForRegistration"] = "bankOfFinlandsGroundsForRegistration";
        })(irepr.EmploymentRegType || (irepr.EmploymentRegType = {}));
        (function (InternationalDataPostedCertCode) {
            InternationalDataPostedCertCode["FromFinlandA1CertificateOrAgreement"] = "fromFinlandA1CertificateOrAgreement";
            InternationalDataPostedCertCode["ToFinlandA1CertificateOrAgreement"] = "toFinlandA1CertificateOrAgreement";
            InternationalDataPostedCertCode["NonEUEEACountryOrNonSocialSecurityAgreementCountry"] = "nonEUEEACountryOrNonSocialSecurityAgreementCountry";
        })(irepr.InternationalDataPostedCertCode || (irepr.InternationalDataPostedCertCode = {}));
        (function (WorkForm) {
            WorkForm["NT1"] = "nT1";
            WorkForm["NT2"] = "nT2";
            WorkForm["EmployersReportOnPeriodsOfStayInFinland"] = "employersReportOnPeriodsOfStayInFinland";
            WorkForm["EmployeeLeasingNotice"] = "employeeLeasingNotice";
        })(irepr.WorkForm || (irepr.WorkForm = {}));
        (function (IncomeEarnerTypes) {
            IncomeEarnerTypes["EmployedByStateEmploymentFund"] = "employedByStateEmploymentFund";
            IncomeEarnerTypes["JointOwnerWithPayer"] = "jointOwnerWithPayer";
            IncomeEarnerTypes["PartialOwner"] = "partialOwner";
            IncomeEarnerTypes["KeyEmployee"] = "keyEmployee";
            IncomeEarnerTypes["LeasedEmployeeLivingAbroad"] = "leasedEmployeeLivingAbroad";
            IncomeEarnerTypes["PersonWorkingInFrontierDistrict"] = "personWorkingInFrontierDistrict";
            IncomeEarnerTypes["PersonWorkingAbroad"] = "personWorkingAbroad";
            IncomeEarnerTypes["Athlete"] = "athlete";
            IncomeEarnerTypes["PerformingArtist"] = "performingArtist";
            IncomeEarnerTypes["RestrictedPeriodInFinland"] = "restrictedPeriodInFinland";
            IncomeEarnerTypes["NetOfTaxContract"] = "netOfTaxContract";
            IncomeEarnerTypes["Organization"] = "organization";
            IncomeEarnerTypes["PersonWorkingOnAlandFerry"] = "personWorkingOnAlandFerry";
            IncomeEarnerTypes["EntrepreneurOrFarmerNoPensionRequired"] = "entrepreneurOrFarmerNoPensionRequired";
        })(irepr.IncomeEarnerTypes || (irepr.IncomeEarnerTypes = {}));
        (function (StatusResponseDeliveryDataStatus) {
            StatusResponseDeliveryDataStatus["Unknown"] = "unknown";
            StatusResponseDeliveryDataStatus["Received"] = "received";
            StatusResponseDeliveryDataStatus["InProcess"] = "inProcess";
            StatusResponseDeliveryDataStatus["Valid"] = "valid";
            StatusResponseDeliveryDataStatus["RejectedAtReception"] = "rejectedAtReception";
            StatusResponseDeliveryDataStatus["RejectedDuringProcessing"] = "rejectedDuringProcessing";
            StatusResponseDeliveryDataStatus["Cancelled"] = "cancelled";
        })(irepr.StatusResponseDeliveryDataStatus || (irepr.StatusResponseDeliveryDataStatus = {}));
    })(exports.irepr || (exports.irepr = {}));

    /* tslint:disable */
    /* eslint-disable */
    //----------------------
    // <auto-generated>
    //     Generated using the NSwag toolchain v13.9.4.0 (NJsonSchema v10.3.1.0 (Newtonsoft.Json v11.0.0.0)) (http://NSwag.org)
    // </auto-generated>
    //----------------------
    // ReSharper disable InconsistentNaming
    exports.irpsr = void 0;
    (function (irpsr) {
        (function (ReportLogEventType) {
            ReportLogEventType["Undefined"] = "undefined";
            ReportLogEventType["Sent"] = "sent";
            ReportLogEventType["Canceled"] = "canceled";
            ReportLogEventType["Error"] = "error";
        })(irpsr.ReportLogEventType || (irpsr.ReportLogEventType = {}));
        (function (DeliveryDataType) {
            DeliveryDataType["EarningsPaymentReports"] = "earningsPaymentReports";
            DeliveryDataType["EmployersSeparateReports"] = "employersSeparateReports";
            DeliveryDataType["BenefitsPaymentReports"] = "benefitsPaymentReports";
            DeliveryDataType["RecordSubscription"] = "recordSubscription";
            DeliveryDataType["Messages"] = "messages";
            DeliveryDataType["CancellationOfEarningsPaymentReports"] = "cancellationOfEarningsPaymentReports";
            DeliveryDataType["CancellationOfEmployersSeparateReports"] = "cancellationOfEmployersSeparateReports";
            DeliveryDataType["CancellationOfBenefitsPaymentReports"] = "cancellationOfBenefitsPaymentReports";
            DeliveryDataType["CancellationOfARecordSubscription"] = "cancellationOfARecordSubscription";
            DeliveryDataType["CancellationOfARecordContainingEarningsPaymentReports"] = "cancellationOfARecordContainingEarningsPaymentReports";
            DeliveryDataType["CancellationOfARecordContainingEmployersSeparateReports"] = "cancellationOfARecordContainingEmployersSeparateReports";
            DeliveryDataType["CancellationOfARecordContainingBenefitsPaymentReports"] = "cancellationOfARecordContainingBenefitsPaymentReports";
            DeliveryDataType["CancellationOfARecordContainingARecordSubscription"] = "cancellationOfARecordContainingARecordSubscription";
        })(irpsr.DeliveryDataType || (irpsr.DeliveryDataType = {}));
        (function (PayerSubOrgType) {
            PayerSubOrgType["KevasSubmitterCodes"] = "kevasSubmitterCodes";
            PayerSubOrgType["PayersOwnCodes"] = "payersOwnCodes";
            PayerSubOrgType["GovernmentAgencyIdentifier"] = "governmentAgencyIdentifier";
        })(irpsr.PayerSubOrgType || (irpsr.PayerSubOrgType = {}));
        (function (UnpaidAbsenceCauseCode) {
            UnpaidAbsenceCauseCode["Illness"] = "illness";
            UnpaidAbsenceCauseCode["PartTimeSickLeave"] = "partTimeSickLeave";
            UnpaidAbsenceCauseCode["ParentalLeave"] = "parentalLeave";
            UnpaidAbsenceCauseCode["SpecialMaternityLeave"] = "specialMaternityLeave";
            UnpaidAbsenceCauseCode["Rehabilitation"] = "rehabilitation";
            UnpaidAbsenceCauseCode["ChildIllness"] = "childIllness";
            UnpaidAbsenceCauseCode["PartTimeChildCareLeave"] = "partTimeChildCareLeave";
            UnpaidAbsenceCauseCode["Training"] = "training";
            UnpaidAbsenceCauseCode["JobAlternationLeave"] = "jobAlternationLeave";
            UnpaidAbsenceCauseCode["StudyLeave"] = "studyLeave";
            UnpaidAbsenceCauseCode["IndustrialAction"] = "industrialAction";
            UnpaidAbsenceCauseCode["InterruptionInWorkProvision"] = "interruptionInWorkProvision";
            UnpaidAbsenceCauseCode["LeaveOfAbsence"] = "leaveOfAbsence";
            UnpaidAbsenceCauseCode["MilitaryRefresherTraining"] = "militaryRefresherTraining";
            UnpaidAbsenceCauseCode["MilitaryService"] = "militaryService";
            UnpaidAbsenceCauseCode["LayOff"] = "layOff";
            UnpaidAbsenceCauseCode["ChildCareLeave"] = "childCareLeave";
            UnpaidAbsenceCauseCode["Other"] = "other";
        })(irpsr.UnpaidAbsenceCauseCode || (irpsr.UnpaidAbsenceCauseCode = {}));
        (function (PaidAbsenceCauseCode) {
            PaidAbsenceCauseCode["Illness"] = "illness";
            PaidAbsenceCauseCode["PartTimeSickLeave"] = "partTimeSickLeave";
            PaidAbsenceCauseCode["ParentalLeave"] = "parentalLeave";
            PaidAbsenceCauseCode["SpecialMaternityLeave"] = "specialMaternityLeave";
            PaidAbsenceCauseCode["Rehabilitation"] = "rehabilitation";
            PaidAbsenceCauseCode["ChildIllness"] = "childIllness";
            PaidAbsenceCauseCode["PartTimeChildCareLeave"] = "partTimeChildCareLeave";
            PaidAbsenceCauseCode["Training"] = "training";
            PaidAbsenceCauseCode["LeaveOfAbsence"] = "leaveOfAbsence";
            PaidAbsenceCauseCode["MilitaryRefresherTraining"] = "militaryRefresherTraining";
            PaidAbsenceCauseCode["MidWeekHoliday"] = "midWeekHoliday";
            PaidAbsenceCauseCode["AccruedHoliday"] = "accruedHoliday";
            PaidAbsenceCauseCode["OccupationalAccident"] = "occupationalAccident";
            PaidAbsenceCauseCode["AnnualLeave"] = "annualLeave";
            PaidAbsenceCauseCode["PartTimeAbsenceDueToRehabilitation"] = "partTimeAbsenceDueToRehabilitation";
            PaidAbsenceCauseCode["Other"] = "other";
        })(irpsr.PaidAbsenceCauseCode || (irpsr.PaidAbsenceCauseCode = {}));
        (function (ActionCode) {
            ActionCode["NewReport"] = "newReport";
            ActionCode["ReplacementReport"] = "replacementReport";
        })(irpsr.ActionCode || (irpsr.ActionCode = {}));
        (function (AddressType) {
            AddressType["AddressInHomeCountry"] = "addressInHomeCountry";
            AddressType["AddressInTheCountryOfWork"] = "addressInTheCountryOfWork";
        })(irpsr.AddressType || (irpsr.AddressType = {}));
        (function (AgeGroupCode) {
            AgeGroupCode["A"] = "a";
            AgeGroupCode["B"] = "b";
            AgeGroupCode["C"] = "c";
            AgeGroupCode["U"] = "u";
        })(irpsr.AgeGroupCode || (irpsr.AgeGroupCode = {}));
        (function (AllowanceCode) {
            AllowanceCode["MealAllowance"] = "mealAllowance";
            AllowanceCode["PartialDailyAllowance"] = "partialDailyAllowance";
            AllowanceCode["FullDailyAllowance"] = "fullDailyAllowance";
            AllowanceCode["InternationalDailyAllowance"] = "internationalDailyAllowance";
            AllowanceCode["TaxExemptReimbursementsAbroad"] = "taxExemptReimbursementsAbroad";
        })(irpsr.AllowanceCode || (irpsr.AllowanceCode = {}));
        (function (BenefitCode) {
            BenefitCode["AccommodationBenefit"] = "accommodationBenefit";
            BenefitCode["TelephoneBenefit"] = "telephoneBenefit";
            BenefitCode["MealBenefit"] = "mealBenefit";
            BenefitCode["OtherBenefits"] = "otherBenefits";
        })(irpsr.BenefitCode || (irpsr.BenefitCode = {}));
        (function (CarBenefitCode) {
            CarBenefitCode["LimitedCarBenefit"] = "limitedCarBenefit";
            CarBenefitCode["FullCarBenefit"] = "fullCarBenefit";
        })(irpsr.CarBenefitCode || (irpsr.CarBenefitCode = {}));
        (function (EmploymentCode) {
            EmploymentCode["FullTime"] = "fullTime";
            EmploymentCode["PartTime"] = "partTime";
            EmploymentCode["NotAvailable"] = "notAvailable";
        })(irpsr.EmploymentCode || (irpsr.EmploymentCode = {}));
        (function (EmploymentEndingCode) {
            EmploymentEndingCode["EmployeesResignation"] = "employeesResignation";
            EmploymentEndingCode["Other"] = "other";
            EmploymentEndingCode["TerminationGroundsRelatedToEmployeesPerson"] = "terminationGroundsRelatedToEmployeesPerson";
            EmploymentEndingCode["FinancialAndProductionRelatedGrounds"] = "financialAndProductionRelatedGrounds";
            EmploymentEndingCode["JointAgreement"] = "jointAgreement";
            EmploymentEndingCode["Retirement"] = "retirement";
            EmploymentEndingCode["TerminationOfaFixedTermEmployment"] = "terminationOfaFixedTermEmployment";
        })(irpsr.EmploymentEndingCode || (irpsr.EmploymentEndingCode = {}));
        (function (EmploymentEndingType) {
            EmploymentEndingType["KevaCodes"] = "kevaCodes";
            EmploymentEndingType["BankOfFinlandCodes"] = "bankOfFinlandCodes";
            EmploymentEndingType["IncomesRegisterCodes"] = "incomesRegisterCodes";
        })(irpsr.EmploymentEndingType || (irpsr.EmploymentEndingType = {}));
        (function (ExceptionCode) {
            ExceptionCode["NoObligationToSocialInsurance"] = "noObligationToSocialInsurance";
            ExceptionCode["NoObligationToHealthInsurance"] = "noObligationToHealthInsurance";
            ExceptionCode["NoObligationToEarningsRelatedPensionInsurance"] = "noObligationToEarningsRelatedPensionInsurance";
            ExceptionCode["NoObligationToAccidentAndOccupationalDiseaseInsurance"] = "noObligationToAccidentAndOccupationalDiseaseInsurance";
            ExceptionCode["NoObligationToUnemploymentInsurance"] = "noObligationToUnemploymentInsurance";
            ExceptionCode["NotSubjectToSocialInsurance"] = "notSubjectToSocialInsurance";
            ExceptionCode["NotSubjectToEarningsRelatedPensionInsurance"] = "notSubjectToEarningsRelatedPensionInsurance";
            ExceptionCode["NotSubjectToAccidentAndOccupationalDiseaseInsurance"] = "notSubjectToAccidentAndOccupationalDiseaseInsurance";
            ExceptionCode["NotSubjectToUnemploymentInsurance"] = "notSubjectToUnemploymentInsurance";
            ExceptionCode["NotSubjectToHealthInsurance"] = "notSubjectToHealthInsurance";
            ExceptionCode["VoluntaryEarningsRelatedPensionInsurance"] = "voluntaryEarningsRelatedPensionInsurance";
        })(irpsr.ExceptionCode || (irpsr.ExceptionCode = {}));
        (function (FaultyControl) {
            FaultyControl["OnlyInvalidItems"] = "onlyInvalidItems";
            FaultyControl["EntireRecordIsRejected"] = "entireRecordIsRejected";
        })(irpsr.FaultyControl || (irpsr.FaultyControl = {}));
        (function (Gender) {
            Gender["Male"] = "male";
            Gender["Female"] = "female";
        })(irpsr.Gender || (irpsr.Gender = {}));
        (function (IdType) {
            IdType["BusinessId"] = "businessId";
            IdType["PersonalIdentificationNumber"] = "personalIdentificationNumber";
            IdType["Vat"] = "vat";
            IdType["Giin"] = "giin";
            IdType["TaxIdentificationNumber"] = "taxIdentificationNumber";
            IdType["FinnishTradeRegistrationNumber"] = "finnishTradeRegistrationNumber";
            IdType["ForeignBusinessRegistrationNumber"] = "foreignBusinessRegistrationNumber";
            IdType["ForeignPersonalIdentificationNumber"] = "foreignPersonalIdentificationNumber";
            IdType["Other"] = "other";
        })(irpsr.IdType || (irpsr.IdType = {}));
        (function (InsuranceCode) {
            InsuranceCode["SubjectToSocialInsuranceContributions"] = "subjectToSocialInsuranceContributions";
            InsuranceCode["SubjectToEarningsRelatedPensionInsuranceContribution"] = "subjectToEarningsRelatedPensionInsuranceContribution";
            InsuranceCode["SubjectToHealthInsuranceContribution"] = "subjectToHealthInsuranceContribution";
            InsuranceCode["SubjectToUnemploymentInsuranceContribution"] = "subjectToUnemploymentInsuranceContribution";
            InsuranceCode["SubjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution"] = "subjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution";
        })(irpsr.InsuranceCode || (irpsr.InsuranceCode = {}));
        (function (Language) {
            Language["Fi"] = "fi";
            Language["Sv"] = "sv";
            Language["En"] = "en";
        })(irpsr.Language || (irpsr.Language = {}));
        (function (PayerType) {
            PayerType["PublicSector"] = "publicSector";
            PayerType["Household"] = "household";
            PayerType["TemporaryEmployer"] = "temporaryEmployer";
            PayerType["ForeignEmployer"] = "foreignEmployer";
            PayerType["State"] = "state";
            PayerType["UnincorporatedStateEnterpriseOrGovernmentalInstitution"] = "unincorporatedStateEnterpriseOrGovernmentalInstitution";
            PayerType["SpecialisedAgency"] = "specialisedAgency";
            PayerType["ForeignGroupCompany"] = "foreignGroupCompany";
        })(irpsr.PayerType || (irpsr.PayerType = {}));
        (function (PaymentType) {
            PaymentType["Monthly"] = "monthly";
            PaymentType["Hourly"] = "hourly";
            PaymentType["ContractPay"] = "contractPay";
        })(irpsr.PaymentType || (irpsr.PaymentType = {}));
        (function (PensionActCode) {
            PensionActCode["EmployeesEarningsRelatedPensionInsurance"] = "employeesEarningsRelatedPensionInsurance";
            PensionActCode["PensionInsuranceForFarmers"] = "pensionInsuranceForFarmers";
            PensionActCode["PensionInsuranceForTheSelfEmployed"] = "pensionInsuranceForTheSelfEmployed";
        })(irpsr.PensionActCode || (irpsr.PensionActCode = {}));
        (function (PensionProvIdCode) {
            PensionProvIdCode["None"] = "none";
            PensionProvIdCode["FinnishCentreForPensions"] = "finnishCentreForPensions";
            PensionProvIdCode["ClergyOfTheOrthodoxChurch"] = "clergyOfTheOrthodoxChurch";
            PensionProvIdCode["KevaMemberOrganisations"] = "kevaMemberOrganisations";
            PensionProvIdCode["Keva\u00C5landLandskapsregeringsPensionssystem"] = "keva\u00C5landLandskapsregeringsPensionssystem";
            PensionProvIdCode["KevaChurch"] = "kevaChurch";
            PensionProvIdCode["BankOfFinland"] = "bankOfFinland";
            PensionProvIdCode["KevaKelaEmploymentPensions"] = "kevaKelaEmploymentPensions";
            PensionProvIdCode["KevaState"] = "kevaState";
            PensionProvIdCode["Seafarers"] = "seafarers";
            PensionProvIdCode["Farmers"] = "farmers";
            PensionProvIdCode["Alandia"] = "alandia";
            PensionProvIdCode["Ilmarinen"] = "ilmarinen";
            PensionProvIdCode["Elo"] = "elo";
            PensionProvIdCode["Varma"] = "varma";
            PensionProvIdCode["Veritas"] = "veritas";
            PensionProvIdCode["Etera"] = "etera";
            PensionProvIdCode["UPM"] = "uPM";
            PensionProvIdCode["Sanoma"] = "sanoma";
            PensionProvIdCode["Sandvik"] = "sandvik";
            PensionProvIdCode["Kontino"] = "kontino";
            PensionProvIdCode["Yleisradio"] = "yleisradio";
            PensionProvIdCode["ABB"] = "aBB";
            PensionProvIdCode["LFashionGroup"] = "lFashionGroup";
            PensionProvIdCode["Honeywell"] = "honeywell";
            PensionProvIdCode["Telia"] = "telia";
            PensionProvIdCode["YaraSuomi"] = "yaraSuomi";
            PensionProvIdCode["Orion"] = "orion";
            PensionProvIdCode["Valio"] = "valio";
            PensionProvIdCode["OP"] = "oP";
            PensionProvIdCode["Verso"] = "verso";
            PensionProvIdCode["Apteekkien"] = "apteekkien";
            PensionProvIdCode["Viabek"] = "viabek";
            PensionProvIdCode["Reka"] = "reka";
        })(irpsr.PensionProvIdCode || (irpsr.PensionProvIdCode = {}));
        (function (ProfessionType) {
            ProfessionType["StatisticsFinland"] = "statisticsFinland";
            ProfessionType["Keva"] = "keva";
            ProfessionType["BankOfFinland"] = "bankOfFinland";
            ProfessionType["Trafi"] = "trafi";
        })(irpsr.ProfessionType || (irpsr.ProfessionType = {}));
        (function (RefPaymentType) {
            RefPaymentType["EarningsRelatedPensionProvider"] = "earningsRelatedPensionProvider";
        })(irpsr.RefPaymentType || (irpsr.RefPaymentType = {}));
        (function (RemunerationCode) {
            RemunerationCode["DailyAllowance"] = "dailyAllowance";
            RemunerationCode["AccommodationBenefit"] = "accommodationBenefit";
            RemunerationCode["CarBenefit"] = "carBenefit";
            RemunerationCode["OtherBenefit"] = "otherBenefit";
        })(irpsr.RemunerationCode || (irpsr.RemunerationCode = {}));
        (function (ResponsibilityCode) {
            ResponsibilityCode["ContentIssues"] = "contentIssues";
            ResponsibilityCode["TechnicalIssues"] = "technicalIssues";
        })(irpsr.ResponsibilityCode || (irpsr.ResponsibilityCode = {}));
        (function (TermCode) {
            TermCode["UntilFurtherNotice"] = "untilFurtherNotice";
            TermCode["FixedTerm"] = "fixedTerm";
        })(irpsr.TermCode || (irpsr.TermCode = {}));
        (function (TransactionCode) {
            TransactionCode["Unknown"] = "unknown";
            TransactionCode["Ignored"] = "ignored";
            TransactionCode["TimeRatePay"] = "timeRatePay";
            TransactionCode["InitiativeFee"] = "initiativeFee";
            TransactionCode["BonusPay"] = "bonusPay";
            TransactionCode["ComplementaryWagePaidDuringBenefitPeriod"] = "complementaryWagePaidDuringBenefitPeriod";
            TransactionCode["EmergencyWorkCompensation"] = "emergencyWorkCompensation";
            TransactionCode["EveningWorkCompensation"] = "eveningWorkCompensation";
            TransactionCode["EveningShiftAllowance"] = "eveningShiftAllowance";
            TransactionCode["NoticePeriodCompensation"] = "noticePeriodCompensation";
            TransactionCode["KilometreAllowanceTaxable"] = "kilometreAllowanceTaxable";
            TransactionCode["MeetingFee"] = "meetingFee";
            TransactionCode["SaturdayPay"] = "saturdayPay";
            TransactionCode["ExtraWorkPremium"] = "extraWorkPremium";
            TransactionCode["HolidayBonus"] = "holidayBonus";
            TransactionCode["LectureFee"] = "lectureFee";
            TransactionCode["CompensationForTrustPosition"] = "compensationForTrustPosition";
            TransactionCode["OtherCompensation"] = "otherCompensation";
            TransactionCode["WaitingTimeCompensation"] = "waitingTimeCompensation";
            TransactionCode["WorkingConditionCompensation"] = "workingConditionCompensation";
            TransactionCode["PartialPayDuringSickLeave"] = "partialPayDuringSickLeave";
            TransactionCode["Commission"] = "commission";
            TransactionCode["SundayWorkCompensation"] = "sundayWorkCompensation";
            TransactionCode["BenefitArisingFromSyntheticOption"] = "benefitArisingFromSyntheticOption";
            TransactionCode["PerformanceBonus"] = "performanceBonus";
            TransactionCode["MonetaryCompensationFromWorkingTimeBank"] = "monetaryCompensationFromWorkingTimeBank";
            TransactionCode["CompensationForAccruedTimeOff"] = "compensationForAccruedTimeOff";
            TransactionCode["ShareIssueForEmployees"] = "shareIssueForEmployees";
            TransactionCode["ContractPay"] = "contractPay";
            TransactionCode["DamagesInConjunctionWithTermination"] = "damagesInConjunctionWithTermination";
            TransactionCode["StandByCompensation"] = "standByCompensation";
            TransactionCode["VoluntaryCompensationInConjunctionWithTermination"] = "voluntaryCompensationInConjunctionWithTermination";
            TransactionCode["WeeklyRestCompensation"] = "weeklyRestCompensation";
            TransactionCode["ProfitSharingBonus"] = "profitSharingBonus";
            TransactionCode["AnnualHolidayCompensation"] = "annualHolidayCompensation";
            TransactionCode["OvertimeCompensation"] = "overtimeCompensation";
            TransactionCode["NightWorkAllowance"] = "nightWorkAllowance";
            TransactionCode["NightShiftCompensation"] = "nightShiftCompensation";
            TransactionCode["OtherRegularCompensation"] = "otherRegularCompensation";
            TransactionCode["CompensationForUnusedCompensatoryLeave"] = "compensationForUnusedCompensatoryLeave";
            TransactionCode["AccommodationBenefit"] = "accommodationBenefit";
            TransactionCode["InterestBenefitForHousingLoan"] = "interestBenefitForHousingLoan";
            TransactionCode["MealAllowance"] = "mealAllowance";
            TransactionCode["CarBenefit"] = "carBenefit";
            TransactionCode["CompensationForGoverningBodyMembership"] = "compensationForGoverningBodyMembership";
            TransactionCode["ShareOfReserveDrawnFromPersonnelFund"] = "shareOfReserveDrawnFromPersonnelFund";
            TransactionCode["MonetaryGiftForEmployees"] = "monetaryGiftForEmployees";
            TransactionCode["KilometreAllowanceTaxExempt"] = "kilometreAllowanceTaxExempt";
            TransactionCode["TreatmentFeeForMunicipalVeterinarian"] = "treatmentFeeForMunicipalVeterinarian";
            TransactionCode["CompensationForUseEarnedIncome"] = "compensationForUseEarnedIncome";
            TransactionCode["CompensationForUseCapitalIncome"] = "compensationForUseCapitalIncome";
            TransactionCode["OtherTaxableBenefitForEmployees"] = "otherTaxableBenefitForEmployees";
            TransactionCode["OtherTaxableIncomeDeemedEarnedIncome"] = "otherTaxableIncomeDeemedEarnedIncome";
            TransactionCode["OtherFringeBenefit"] = "otherFringeBenefit";
            TransactionCode["KinshipCarersFee"] = "kinshipCarersFee";
            TransactionCode["StockOptions"] = "stockOptions";
            TransactionCode["WagesPaidBySubstitutePayerIncludingSocialInsurance"] = "wagesPaidBySubstitutePayerIncludingSocialInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance"] = "wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingUnemploymentInsurance"] = "wagesPaidBySubstitutePayerIncludingUnemploymentInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance"] = "wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance";
            TransactionCode["WagesPaidBySubstitutePayerIncludingHealthInsurance"] = "wagesPaidBySubstitutePayerIncludingHealthInsurance";
            TransactionCode["CompensationForEmployeeInvention"] = "compensationForEmployeeInvention";
            TransactionCode["ReimbursementOfPrivateCaretakersExpenses"] = "reimbursementOfPrivateCaretakersExpenses";
            TransactionCode["PrivateCaretakersFee"] = "privateCaretakersFee";
            TransactionCode["ReimbursementOfFamilyDayCareProvidersExpenses"] = "reimbursementOfFamilyDayCareProvidersExpenses";
            TransactionCode["TelephoneBenefit"] = "telephoneBenefit";
            TransactionCode["DailyAllowance"] = "dailyAllowance";
            TransactionCode["CapitalIncomePayment"] = "capitalIncomePayment";
            TransactionCode["MealBenefit"] = "mealBenefit";
            TransactionCode["ReimbursementOfCostsPaidToConciliator"] = "reimbursementOfCostsPaidToConciliator";
            TransactionCode["NonWageCompensationForWork"] = "nonWageCompensationForWork";
            TransactionCode["SupplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund"] = "supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund";
            TransactionCode["PensionPaidByEmployer"] = "pensionPaidByEmployer";
            TransactionCode["DividendsBasedOnWages"] = "dividendsBasedOnWages";
            TransactionCode["DividendsBasedOnNonWage"] = "dividendsBasedOnNonWage";
            TransactionCode["EmployerSubsidisedCommuterTicketTaxExempt"] = "employerSubsidisedCommuterTicketTaxExempt";
            TransactionCode["EmployerSubsidisedCommuterTicketTaxable"] = "employerSubsidisedCommuterTicketTaxable";
            TransactionCode["EmployeeStockOption"] = "employeeStockOption";
            TransactionCode["WagesTransferredToAthletesSpecialFund"] = "wagesTransferredToAthletesSpecialFund";
            TransactionCode["WagesPaidFromAthletesSpecialFund"] = "wagesPaidFromAthletesSpecialFund";
            TransactionCode["WagesForInsurancePurposes"] = "wagesForInsurancePurposes";
            TransactionCode["TaxableReimbursementOfExpenses"] = "taxableReimbursementOfExpenses";
            TransactionCode["PrivateDayCareAllowanceMunicipalSupplement"] = "privateDayCareAllowanceMunicipalSupplement";
            TransactionCode["PrivateDayCareAllowanceWages"] = "privateDayCareAllowanceWages";
            TransactionCode["PrivateDayCareAllowanceNonWage"] = "privateDayCareAllowanceNonWage";
            TransactionCode["KilometreAllowancePaidByNonProfitOrganisation"] = "kilometreAllowancePaidByNonProfitOrganisation";
            TransactionCode["DailyAllowancePaidByNonProfitOrganisation"] = "dailyAllowancePaidByNonProfitOrganisation";
            TransactionCode["UnjustEnrichment"] = "unjustEnrichment";
            TransactionCode["EmployeeStockOptionLowerMarketPrice"] = "employeeStockOptionLowerMarketPrice";
            TransactionCode["RoyaltyPaidToNonResidentTaxpayer"] = "royaltyPaidToNonResidentTaxpayer";
            TransactionCode["BicycleBenefitTaxExempt"] = "bicycleBenefitTaxExempt";
            TransactionCode["BicycleBenefitTaxable"] = "bicycleBenefitTaxable";
            TransactionCode["ConditionalStockOptions"] = "conditionalStockOptions";
            TransactionCode["CompensationCollectedForCarBenefit"] = "compensationCollectedForCarBenefit";
            TransactionCode["WithholdingTax"] = "withholdingTax";
            TransactionCode["ElectedOfficialFee"] = "electedOfficialFee";
            TransactionCode["TaxAtSource"] = "taxAtSource";
            TransactionCode["TaxAtSourceDeduction"] = "taxAtSourceDeduction";
            TransactionCode["WagesPaid"] = "wagesPaid";
            TransactionCode["ReimbursementCollectedForOtherFringeBenefits"] = "reimbursementCollectedForOtherFringeBenefits";
            TransactionCode["OtherItemDeductibleFromNetWage"] = "otherItemDeductibleFromNetWage";
            TransactionCode["NetWage"] = "netWage";
            TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsurance"] = "employerPaidPremiumForCollectiveAdditionalPensionInsurance";
            TransactionCode["EmployerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution"] = "employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution";
            TransactionCode["EmployeesHealthInsuranceContribution"] = "employeesHealthInsuranceContribution";
            TransactionCode["EmployeesPensionInsuranceContribution"] = "employeesPensionInsuranceContribution";
            TransactionCode["EmployeesUnemploymentInsuranceContribution"] = "employeesUnemploymentInsuranceContribution";
            TransactionCode["ReimbursementForEmployerSubsidisedCommuterTicket"] = "reimbursementForEmployerSubsidisedCommuterTicket";
            TransactionCode["TaxPaidAbroad"] = "taxPaidAbroad";
            TransactionCode["Distraint"] = "distraint";
            TransactionCode["VoluntaryIndividualPensionInsurancePremium"] = "voluntaryIndividualPensionInsurancePremium";
            TransactionCode["DeductionBeforeWithholding"] = "deductionBeforeWithholding";
            TransactionCode["ReimbursementCollectedForBicycleBenefit"] = "reimbursementCollectedForBicycleBenefit";
        })(irpsr.TransactionCode || (irpsr.TransactionCode = {}));
        (function (WageUnitCode) {
            WageUnitCode["Hour"] = "hour";
            WageUnitCode["Day"] = "day";
            WageUnitCode["Week"] = "week";
            WageUnitCode["Period"] = "period";
        })(irpsr.WageUnitCode || (irpsr.WageUnitCode = {}));
        (function (ReimbPaymentType) {
            ReimbPaymentType["KelaDailyAllowanceApplication"] = "kelaDailyAllowanceApplication";
            ReimbPaymentType["KelaFamilyLeaveCompensation"] = "kelaFamilyLeaveCompensation";
        })(irpsr.ReimbPaymentType || (irpsr.ReimbPaymentType = {}));
        (function (ApiValidationErrorType) {
            ApiValidationErrorType["General"] = "general";
            ApiValidationErrorType["Required"] = "required";
            ApiValidationErrorType["Invalid"] = "invalid";
            ApiValidationErrorType["Warning"] = "warning";
        })(irpsr.ApiValidationErrorType || (irpsr.ApiValidationErrorType = {}));
        (function (IrScheduleAction) {
            IrScheduleAction["Send"] = "send";
            IrScheduleAction["Cancel"] = "cancel";
        })(irpsr.IrScheduleAction || (irpsr.IrScheduleAction = {}));
        (function (SummaryTransactionCode) {
            SummaryTransactionCode["Undefined"] = "undefined";
            SummaryTransactionCode["NoWagesPayable"] = "noWagesPayable";
            SummaryTransactionCode["EmployersHealthInsuranceContribution"] = "employersHealthInsuranceContribution";
            SummaryTransactionCode["DeductionsToBeMadeFromTheEmployersHealthInsuranceContribution"] = "deductionsToBeMadeFromTheEmployersHealthInsuranceContribution";
        })(irpsr.SummaryTransactionCode || (irpsr.SummaryTransactionCode = {}));
        (function (PayerSummaryReportStatus) {
            PayerSummaryReportStatus["New"] = "new";
            PayerSummaryReportStatus["Scheduled"] = "scheduled";
            PayerSummaryReportStatus["Succeeded"] = "succeeded";
            PayerSummaryReportStatus["Canceled"] = "canceled";
            PayerSummaryReportStatus["Error"] = "error";
            PayerSummaryReportStatus["Invalid"] = "invalid";
        })(irpsr.PayerSummaryReportStatus || (irpsr.PayerSummaryReportStatus = {}));
        (function (WorkflowEventUi) {
            WorkflowEventUi["Default"] = "default";
            WorkflowEventUi["Primary"] = "primary";
            WorkflowEventUi["Success"] = "success";
            WorkflowEventUi["Info"] = "info";
            WorkflowEventUi["Warning"] = "warning";
            WorkflowEventUi["Danger"] = "danger";
        })(irpsr.WorkflowEventUi || (irpsr.WorkflowEventUi = {}));
        (function (Features) {
            Features["None"] = "none";
            Features["IsActive"] = "isActive";
            Features["IsEditable"] = "isEditable";
            Features["IsRemovable"] = "isRemovable";
            Features["IsOnHold"] = "isOnHold";
        })(irpsr.Features || (irpsr.Features = {}));
        (function (AvatarEntityType) {
            AvatarEntityType["Undefined"] = "undefined";
            AvatarEntityType["Person"] = "person";
            AvatarEntityType["Company"] = "company";
            AvatarEntityType["PersonCreatedByEmployer"] = "personCreatedByEmployer";
            AvatarEntityType["Partner"] = "partner";
        })(irpsr.AvatarEntityType || (irpsr.AvatarEntityType = {}));
        (function (AvatarPictureType) {
            AvatarPictureType["Icon"] = "icon";
            AvatarPictureType["Uploaded"] = "uploaded";
            AvatarPictureType["Gravatar"] = "gravatar";
        })(irpsr.AvatarPictureType || (irpsr.AvatarPictureType = {}));
        (function (StatusResponseDeliveryDataStatus) {
            StatusResponseDeliveryDataStatus["Unknown"] = "unknown";
            StatusResponseDeliveryDataStatus["Received"] = "received";
            StatusResponseDeliveryDataStatus["InProcess"] = "inProcess";
            StatusResponseDeliveryDataStatus["Valid"] = "valid";
            StatusResponseDeliveryDataStatus["RejectedAtReception"] = "rejectedAtReception";
            StatusResponseDeliveryDataStatus["RejectedDuringProcessing"] = "rejectedDuringProcessing";
            StatusResponseDeliveryDataStatus["Cancelled"] = "cancelled";
        })(irpsr.StatusResponseDeliveryDataStatus || (irpsr.StatusResponseDeliveryDataStatus = {}));
    })(exports.irpsr || (exports.irpsr = {}));

    /**
     * Helpers for performing operations with objects
     */
    class Objects {
        /**
         * Returns true if an object has a property with specified name.
         * Using hasOwnProperty in a safe, easy and concise way.
         * @param obj Object to check
         * @param key String key to look for.
         */
        static has(obj, key) {
            if (!obj) {
                return false;
            }
            return Object.prototype.hasOwnProperty.call(obj, key);
        }
        /**
         * Gets a property based on multi-level property path.
         * Null / undefined at any level will return null as in AngularJS expressions.
         * @param obj Object from which to fetch a property.
         * @param propertyPath Path to the property to fetch.
         */
        static getProperty(obj, propertyPath) {
            if (!propertyPath || !obj) {
                return null;
            }
            const arr = propertyPath.split(".");
            while (arr.length) {
                const key = arr.shift();
                obj = obj[key];
                if (!obj && arr.length) {
                    return null;
                }
            }
            return obj;
        }
        /**
         * Sets a property of an object based on property path.
         * Will create the entire path even if the intermittent objects do not exist.
         * @param obj Object whose property should be set.
         * @param propertyPath Dot-delimited property path to the property that should be set.
         * @param value The value that should be set.
         * @returns The assigned value or void if the property cannot be set.
         */
        static setProperty(obj, propertyPath, value) {
            if (!propertyPath || !obj) {
                return;
            }
            const arr = propertyPath.split(".");
            while (arr.length > 1) {
                const key = arr.shift();
                if (!obj[key]) {
                    obj[key] = {};
                }
                obj = obj[key];
            }
            if (obj[arr[0]] === undefined && value === undefined) {
                return undefined; // Do not set to undefined if undefined
            }
            return obj[arr[0]] = value;
        }
        /**
         * Uses JSON Serialization / Deserialization to deep clone an object.
         * @param value Object to clone
         * @returns A cloned object.
         */
        static copy(value) {
            return JSON.parse(JSON.stringify(value, (k, v) => v === undefined ? null : v));
        }
        /**
         * Extends one object with another using Intersection types.
         * https://www.typescriptlang.org/docs/handbook/advanced-types.html
         * @param first First object.
         * @param second Second object.
         */
        static extend(first, second) {
            const result = {};
            for (const id in first) {
                if (Objects.has(first, id)) {
                    result[id] = first[id];
                }
            }
            for (const id in second) {
                if (Objects.has(second, id) && !Objects.has(result, id)) {
                    result[id] = second[id];
                }
            }
            return result;
        }
        /**
         * Compares objects a and b.
         * Returns true if the objects are equal using value-to-value comparison.
         *
         * @param a - Any object.
         * @param b - Any object to compare with.
         */
        static equal(a, b) {
            if (a === b) {
                return true;
            }
            if (a && b && typeof a === "object" && typeof b === "object") {
                const arrA = Array.isArray(a);
                const arrB = Array.isArray(b);
                let i;
                let length;
                let key;
                if (arrA && arrB) {
                    // both are arrays
                    length = a.length;
                    if (length !== b.length) {
                        // different length
                        return false;
                    }
                    for (i = length; i-- !== 0;) {
                        if (!this.equal(a[i], b[i])) {
                            // members are not equal
                            return false;
                        }
                    }
                    // all members are equal
                    return true;
                }
                if (arrA !== arrB) {
                    // either a or be is an array, not both
                    return false;
                }
                const dateA = a instanceof Date;
                const dateB = b instanceof Date;
                if (dateA !== dateB) {
                    // either a or b is a date, not both
                    return false;
                }
                if (dateA && dateB) {
                    // both are dates, test times equality
                    return a.getTime() === b.getTime();
                }
                const regexpA = a instanceof RegExp;
                const regexpB = b instanceof RegExp;
                if (regexpA !== regexpB) {
                    // either a or b is RegExp, not both
                    return false;
                }
                if (regexpA && regexpB) {
                    // both are RegExp, test string equality
                    return a.toString() === b.toString();
                }
                const keys = Object.keys(a);
                length = keys.length;
                if (length !== Object.keys(b).length) {
                    // a and b have different number of properties
                    return false;
                }
                for (i = length; i-- !== 0;) {
                    if (!Object.prototype.hasOwnProperty.call(b, keys[i])) {
                        // b does not have the same properties as a
                        return false;
                    }
                }
                for (i = length; i-- !== 0;) {
                    key = keys[i];
                    if (!this.equal(a[key], b[key])) {
                        // property has not the same value
                        return false;
                    }
                }
                // both objects are equal
                return true;
            }
            // return false, except both are NaN
            return a !== a && b !== b;
        }
    }

    /** Codegen namespace is interfaces and data that is generated in Salaxy server */
    /* eslint-disable */
    /**
     * Generated products data from the server.
     * To update, go to: https://test-api.salaxy.com/v02/api/accounts/products/anon
     * @ignore
     * TODO: Add language versioning to the server side and generation.
     * TODO: Can we package this entire namespace in a way that it is hidden to be internal only?
     */
    const accountProductDefaults = {
        "baseService": {
            "campaignCode": null,
            "id": "baseService",
            "title": "Palkkaus.fi-tili",
            "status": "Palkkaamisen peruspalvelut",
            "description": "Digiajan palkanmaksupalvelu",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/palkkaus.png",
            "articleId": "product_baseService",
            "enabled": true,
            "visible": true
        },
        "tax": {
            "paymentReferenceNumber": null,
            "isSalariesPaidThisYear": false,
            "isMonthlyReportingEnabled": true,
            "isYearlyReportingEnabled": true,
            "isNotificationsSelfHandling": false,
            "isPaymentsSelfHandling": false,
            "id": "tax",
            "title": "Yhteydet verottajaan",
            "status": "Ennakonpidätys ja ilmoitukset",
            "description": "Ennakonpidätykset ja Sotu-maksut",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/tax.png",
            "articleId": "product_tax",
            "enabled": true,
            "visible": true
        },
        "unemployment": {
            "isSalariesPaidThisYear": false,
            "isPrepayments": false,
            "isNotificationsSelfHandling": false,
            "id": "unemployment",
            "title": "Työttömyysvakuutus (TVR)",
            "status": "Maksut ja ilmoitukset TVR:ään",
            "description": "Työttömyysturva rahoitetaan TVR-maksuilla",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/tvr.png",
            "articleId": "product_unemployment",
            "enabled": true,
            "visible": true
        },
        "pension": {
            "status": "Ei sopimusta: TyEL maksetaan tilapäisenä työnantajana",
            "isPensionContractDone": false,
            "pensionCompany": "none",
            "pensionContractNumber": null,
            "isPensionSelfHandling": false,
            "isPendingContract": false,
            "enabled": true,
            "id": "pension",
            "title": "TyEL-eläkevakuutus",
            "description": "TyEL-vakuutus on työntekijän eläketurva",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/varma-tyel.png",
            "articleId": "product_pension",
            "visible": true
        },
        "insurance": {
            "status": "Ei sopimusta - Huom! 1200 € :n sääntö.",
            "isPartnerInsurance": false,
            "isInsuranceContractDone": false,
            "insuranceCompany": "none",
            "insuranceContractNumber": null,
            "enabled": false,
            "id": "insurance",
            "title": "Työtapaturmavakuutus",
            "description": "Tapaturmavakuutus vakuuttaa työntekijäsi",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/if.png",
            "articleId": "product_insurance",
            "visible": true
        },
        "accounting": {
            "status": "Ei käytössä",
            "accountantName": null,
            "accountantEmail": null,
            "accountantPhone": null,
            "enabled": false,
            "id": "accounting",
            "title": "Kirjanpitoraportit",
            "description": "Palkkalaskelmien tiedot helposti kirjanpitäjälle",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/accounting.png",
            "articleId": "product_accounting",
            "visible": true
        },
        "healthCareHeltti": {
            "package": "notDefined",
            "industry": "notDefined",
            "status": "Ei sopimusta",
            "enabled": false,
            "id": "healthCareHeltti",
            "title": "Työterveyshuolto",
            "description": "Tuottavammat ja terveemmät työntekijät",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/heltti.png",
            "articleId": "product_healthCareHeltti",
            "visible": true
        },
        "pensionEntrepreneur": {
            "status": "Ei YEL-sopimusta",
            "tempContractMarker": false,
            "enabled": false,
            "id": "pensionEntrepreneur",
            "title": "Yrittäjän eläke",
            "description": "YEL-vakuutus on yrittäjän eläketurva",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/varma-yel.png",
            "articleId": "product_pensionEntrepreneur",
            "visible": true
        },
        "taxCard": {
            "currentCard": null,
            "status": "Ei verokorttia",
            "id": "taxCard",
            "title": "Verokortti (TT)",
            "description": "Työntekijän verokortti ja sen käsittely",
            "img": "https://cdn.salaxy.com/img/elems/product-logos/palkkaus.png",
            "articleId": "product_taxCard",
            "enabled": false,
            "visible": true
        }
    };

    /**
     * This data is generated from
     * ****************************************************
     * DEVELOPERS NOTE: COPY from https://test-api.salaxy.com/docs/EnumJson / http://localhost:82/docs/EnumJson
     * DO NOT EDIT DIRECTLY
     * @ignore
     */
    const generatedEnumData = [
        {
            "name": "AbcSection",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "doNotShow"
                },
                {
                    "order": 10,
                    "name": "employment"
                },
                {
                    "order": 20,
                    "name": "insurance"
                },
                {
                    "order": 30,
                    "name": "salary"
                },
                {
                    "order": 40,
                    "name": "examples"
                },
                {
                    "order": 50,
                    "name": "palkkausGeneral"
                },
                {
                    "order": 60,
                    "name": "palkkausInstructions"
                },
                {
                    "order": 100,
                    "name": "blog"
                },
                {
                    "order": 110,
                    "name": "press"
                },
                {
                    "order": 200,
                    "name": "personEmployer"
                },
                {
                    "order": 201,
                    "name": "householdEmployer"
                },
                {
                    "order": 210,
                    "name": "worker"
                },
                {
                    "order": 211,
                    "name": "employee"
                },
                {
                    "order": 220,
                    "name": "entrepreneur"
                },
                {
                    "order": 221,
                    "name": "association"
                },
                {
                    "order": 222,
                    "name": "businessOwner"
                },
                {
                    "order": 301,
                    "name": "productLongDescription"
                },
                {
                    "order": 302,
                    "name": "documentTemplates"
                },
                {
                    "order": 303,
                    "name": "instructionsAndExamples"
                }
            ]
        },
        {
            "name": "AbsenceCauseCode",
            "description": null,
            "values": [
                {
                    "order": 10,
                    "name": "unpaidLeave"
                },
                {
                    "order": 11,
                    "name": "personalReason"
                },
                {
                    "order": 1001,
                    "name": "illness"
                },
                {
                    "order": 1002,
                    "name": "partTimeSickLeave"
                },
                {
                    "order": 1003,
                    "name": "parentalLeave"
                },
                {
                    "order": 1004,
                    "name": "specialMaternityLeave"
                },
                {
                    "order": 1005,
                    "name": "rehabilitation"
                },
                {
                    "order": 1006,
                    "name": "childIllness"
                },
                {
                    "order": 1007,
                    "name": "partTimeChildCareLeave"
                },
                {
                    "order": 1008,
                    "name": "training"
                },
                {
                    "order": 1009,
                    "name": "jobAlternationLeave"
                },
                {
                    "order": 1010,
                    "name": "studyLeave"
                },
                {
                    "order": 1011,
                    "name": "industrialAction"
                },
                {
                    "order": 1012,
                    "name": "interruptionInWorkProvision"
                },
                {
                    "order": 1013,
                    "name": "leaveOfAbsence"
                },
                {
                    "order": 1014,
                    "name": "militaryRefresherTraining"
                },
                {
                    "order": 1015,
                    "name": "militaryService"
                },
                {
                    "order": 1016,
                    "name": "layOff"
                },
                {
                    "order": 1017,
                    "name": "childCareLeave"
                },
                {
                    "order": 1018,
                    "name": "midWeekHoliday"
                },
                {
                    "order": 1019,
                    "name": "accruedHoliday"
                },
                {
                    "order": 1020,
                    "name": "occupationalAccident"
                },
                {
                    "order": 1021,
                    "name": "annualLeave"
                },
                {
                    "order": 1022,
                    "name": "partTimeAbsenceDueToRehabilitation"
                },
                {
                    "order": 1099,
                    "name": "other"
                }
            ]
        },
        {
            "name": "AccountantType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "unlinkedPrimaryPartner"
                },
                {
                    "order": 2,
                    "name": "unlinkedAccountingOnly"
                },
                {
                    "order": 3,
                    "name": "pendingPrimaryPartner"
                },
                {
                    "order": 10,
                    "name": "primaryPartner"
                }
            ]
        },
        {
            "name": "AccountingChannel",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "manual"
                },
                {
                    "order": 800,
                    "name": "email"
                },
                {
                    "order": 900,
                    "name": "template"
                },
                {
                    "order": 1001,
                    "name": "paymentTest"
                },
                {
                    "order": 1010,
                    "name": "paymentAccountorGo"
                },
                {
                    "order": 1020,
                    "name": "paymentTalenomOnline"
                },
                {
                    "order": 1040,
                    "name": "paymentFinagoSolo"
                },
                {
                    "order": 1050,
                    "name": "paymentProcountor"
                },
                {
                    "order": 1060,
                    "name": "paymentKevytyrittaja"
                },
                {
                    "order": 1070,
                    "name": "paymentVismaNetvisor"
                },
                {
                    "order": 1204,
                    "name": "paymentCfaFinvoice"
                },
                {
                    "order": 1210,
                    "name": "paymentCfaTest"
                },
                {
                    "order": 2010,
                    "name": "procountor"
                },
                {
                    "order": 2020,
                    "name": "vismaNetvisor"
                },
                {
                    "order": 2030,
                    "name": "fennoa"
                },
                {
                    "order": 2030,
                    "name": "fennoa"
                }
            ]
        },
        {
            "name": "AccountingPeriodClosingOption",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                }
            ]
        },
        {
            "name": "AccountingReportRowType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "debit"
                },
                {
                    "order": 1,
                    "name": "credit"
                },
                {
                    "order": 2,
                    "name": "total"
                },
                {
                    "order": 3,
                    "name": "groupHeader"
                },
                {
                    "order": 4,
                    "name": "groupTotal"
                },
                {
                    "order": 5,
                    "name": "childRow"
                }
            ]
        },
        {
            "name": "AccountingReportTableType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "classic"
                },
                {
                    "order": 1,
                    "name": "simple"
                },
                {
                    "order": 2,
                    "name": "mapped"
                }
            ]
        },
        {
            "name": "AccountingTargetStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "enabled"
                },
                {
                    "order": 2,
                    "name": "disabled"
                },
                {
                    "order": 10,
                    "name": "productTarget"
                },
                {
                    "order": 11,
                    "name": "sharedTarget"
                },
                {
                    "order": 12,
                    "name": "rulesetTemplate"
                },
                {
                    "order": 13,
                    "name": "paymentChannel"
                }
            ]
        },
        {
            "name": "AgeGroupCode",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "a"
                },
                {
                    "order": 2,
                    "name": "b"
                },
                {
                    "order": 3,
                    "name": "c"
                },
                {
                    "order": 4,
                    "name": "u"
                }
            ]
        },
        {
            "name": "AgeRange",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 15,
                    "name": "age_15"
                },
                {
                    "order": 16,
                    "name": "age16"
                },
                {
                    "order": 17,
                    "name": "age17"
                },
                {
                    "order": 18,
                    "name": "age18_52"
                },
                {
                    "order": 53,
                    "name": "age53_62"
                },
                {
                    "order": 63,
                    "name": "age63_64"
                },
                {
                    "order": 65,
                    "name": "age65_67"
                },
                {
                    "order": 68,
                    "name": "age68AndOVer"
                }
            ]
        },
        {
            "name": "AllowanceCode",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "mealAllowance"
                },
                {
                    "order": 2,
                    "name": "partialDailyAllowance"
                },
                {
                    "order": 3,
                    "name": "fullDailyAllowance"
                },
                {
                    "order": 4,
                    "name": "internationalDailyAllowance"
                },
                {
                    "order": 5,
                    "name": "taxExemptReimbursementsAbroad"
                }
            ]
        },
        {
            "name": "AnnualLeavePaymentKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "planned"
                },
                {
                    "order": 101,
                    "name": "manualSalary"
                },
                {
                    "order": 102,
                    "name": "manualCompensation"
                },
                {
                    "order": 103,
                    "name": "manualBonus"
                },
                {
                    "order": 201,
                    "name": "paidCalc"
                },
                {
                    "order": 202,
                    "name": "draftCalc"
                }
            ]
        },
        {
            "name": "ApiListItemType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 10,
                    "name": "accountBase"
                },
                {
                    "order": 11,
                    "name": "companyAccount"
                },
                {
                    "order": 12,
                    "name": "personAccount"
                },
                {
                    "order": 13,
                    "name": "workerAccount"
                },
                {
                    "order": 14,
                    "name": "certificate"
                },
                {
                    "order": 15,
                    "name": "sessionUserCredential"
                },
                {
                    "order": 16,
                    "name": "onboarding"
                },
                {
                    "order": 17,
                    "name": "profile"
                },
                {
                    "order": 18,
                    "name": "workerInvitation"
                },
                {
                    "order": 20,
                    "name": "calculation"
                },
                {
                    "order": 21,
                    "name": "calculationPaid"
                },
                {
                    "order": 22,
                    "name": "eSalaryPayment"
                },
                {
                    "order": 23,
                    "name": "payment"
                },
                {
                    "order": 24,
                    "name": "earningsPayment"
                },
                {
                    "order": 30,
                    "name": "employment"
                },
                {
                    "order": 31,
                    "name": "workerAbsences"
                },
                {
                    "order": 32,
                    "name": "holidayYear"
                },
                {
                    "order": 40,
                    "name": "taxcard"
                },
                {
                    "order": 41,
                    "name": "payrollDetails"
                },
                {
                    "order": 42,
                    "name": "invoice"
                },
                {
                    "order": 43,
                    "name": "report"
                },
                {
                    "order": 44,
                    "name": "blobFile"
                },
                {
                    "order": 45,
                    "name": "article"
                },
                {
                    "order": 46,
                    "name": "messageThread"
                },
                {
                    "order": 47,
                    "name": "emailMessage"
                },
                {
                    "order": 48,
                    "name": "accountProducts"
                },
                {
                    "order": 49,
                    "name": "varmaPensionOrder"
                },
                {
                    "order": 50,
                    "name": "ifInsuranceOrder"
                },
                {
                    "order": 51,
                    "name": "historical"
                }
            ]
        },
        {
            "name": "ApiTestErrorType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                },
                {
                    "order": 10,
                    "name": "userFriendly"
                }
            ]
        },
        {
            "name": "ApiItemType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "calculation"
                },
                {
                    "order": 2,
                    "name": "calculationPaid"
                },
                {
                    "order": 10,
                    "name": "payrollDetails"
                },
                {
                    "order": 20,
                    "name": "invoice"
                },
                {
                    "order": 30,
                    "name": "earningsPaymentReport"
                },
                {
                    "order": 40,
                    "name": "payerSummaryReport"
                },
                {
                    "order": 50,
                    "name": "employment"
                },
                {
                    "order": 100,
                    "name": "messageThread"
                },
                {
                    "order": 110,
                    "name": "calendarEvent"
                }
            ]
        },
        {
            "name": "ApiValidationErrorType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "general"
                },
                {
                    "order": 10,
                    "name": "required"
                },
                {
                    "order": 20,
                    "name": "invalid"
                },
                {
                    "order": 30,
                    "name": "warning"
                }
            ]
        },
        {
            "name": "AuthenticationMethod",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "emailPwdLocal"
                },
                {
                    "order": 11,
                    "name": "facebook"
                },
                {
                    "order": 12,
                    "name": "google"
                },
                {
                    "order": 13,
                    "name": "microsoft"
                },
                {
                    "order": 14,
                    "name": "linkedIn"
                },
                {
                    "order": 15,
                    "name": "auth0Database"
                },
                {
                    "order": 16,
                    "name": "x509"
                },
                {
                    "order": 17,
                    "name": "salaxy"
                },
                {
                    "order": 18,
                    "name": "test"
                },
                {
                    "order": 1000,
                    "name": "internalLocalhost"
                }
            ]
        },
        {
            "name": "AuthorizationStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "oK"
                },
                {
                    "order": -40,
                    "name": "accessDenied"
                },
                {
                    "order": -30,
                    "name": "expiredOauthToken"
                },
                {
                    "order": -20,
                    "name": "invalidOAuthToken"
                },
                {
                    "order": -10,
                    "name": "noOauthToken"
                }
            ]
        },
        {
            "name": "AuthorizationType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "employerAuthorization"
                },
                {
                    "order": 2,
                    "name": "workerContract"
                },
                {
                    "order": 3,
                    "name": "companyContract"
                },
                {
                    "order": 100,
                    "name": "manual"
                },
                {
                    "order": 999,
                    "name": "temporary"
                }
            ]
        },
        {
            "name": "AvatarPictureType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "icon"
                },
                {
                    "order": 1,
                    "name": "uploaded"
                },
                {
                    "order": 2,
                    "name": "gravatar"
                }
            ]
        },
        {
            "name": "BankPaymentType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "error"
                },
                {
                    "order": 10,
                    "name": "paytrailBatch"
                },
                {
                    "order": 11,
                    "name": "netvisorPayment"
                },
                {
                    "order": 20,
                    "name": "salaryGrossPayment"
                },
                {
                    "order": 21,
                    "name": "refundPayment"
                },
                {
                    "order": 22,
                    "name": "partialRefundPayment"
                },
                {
                    "order": 30,
                    "name": "insuranceGrossPayment"
                },
                {
                    "order": 50,
                    "name": "netSalary"
                },
                {
                    "order": 51,
                    "name": "salaryAdvance"
                },
                {
                    "order": 100,
                    "name": "taxAccount"
                },
                {
                    "order": 200,
                    "name": "tvr"
                },
                {
                    "order": 210,
                    "name": "tvrRfnd"
                },
                {
                    "order": 300,
                    "name": "palkkausFees"
                },
                {
                    "order": 310,
                    "name": "bankFees"
                },
                {
                    "order": 320,
                    "name": "paytrailFees"
                },
                {
                    "order": 501,
                    "name": "insuranceLahiTapiola"
                },
                {
                    "order": 1000,
                    "name": "tyel"
                },
                {
                    "order": 1101,
                    "name": "tyelEteraContract"
                },
                {
                    "order": 1102,
                    "name": "tyelEteraTemp"
                },
                {
                    "order": 1103,
                    "name": "tyelEteraUnknown"
                },
                {
                    "order": 1201,
                    "name": "tyelEloContract"
                },
                {
                    "order": 1202,
                    "name": "tyelEloTemp"
                },
                {
                    "order": 1203,
                    "name": "tyelEloInitialImport"
                },
                {
                    "order": 1301,
                    "name": "tyelVarmaContract"
                },
                {
                    "order": 1302,
                    "name": "tyelVarmaTemp"
                },
                {
                    "order": 1401,
                    "name": "tyelIlmarinenContract"
                },
                {
                    "order": 1402,
                    "name": "tyelIlmarinenTemp"
                },
                {
                    "order": 2000,
                    "name": "union"
                },
                {
                    "order": 2001,
                    "name": "unionRaksa"
                },
                {
                    "order": 2010,
                    "name": "foreclosure"
                },
                {
                    "order": 10001,
                    "name": "saldoYearBegin"
                },
                {
                    "order": 10002,
                    "name": "saldoYearEnd"
                }
            ]
        },
        {
            "name": "BenefitCode",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "accommodationBenefit"
                },
                {
                    "order": 2,
                    "name": "telephoneBenefit"
                },
                {
                    "order": 3,
                    "name": "mealBenefit"
                },
                {
                    "order": 4,
                    "name": "otherBenefits"
                }
            ]
        },
        {
            "name": "BlobFileType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "authorizationPdf"
                },
                {
                    "order": 2,
                    "name": "avatar"
                },
                {
                    "order": 3,
                    "name": "taxCard"
                },
                {
                    "order": 4,
                    "name": "expensesReceipt"
                },
                {
                    "order": 5,
                    "name": "raksaloki"
                },
                {
                    "order": 6,
                    "name": "template"
                },
                {
                    "order": 7,
                    "name": "eInvoice"
                },
                {
                    "order": 8,
                    "name": "temporary"
                },
                {
                    "order": 9,
                    "name": "settings"
                },
                {
                    "order": 20,
                    "name": "messages"
                },
                {
                    "order": 100,
                    "name": "monthlyReport"
                },
                {
                    "order": 101,
                    "name": "yearlyReport"
                },
                {
                    "order": 102,
                    "name": "calcReport"
                }
            ]
        },
        {
            "name": "BlobRepository",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "userFiles"
                },
                {
                    "order": 2,
                    "name": "versionHistory"
                },
                {
                    "order": 3,
                    "name": "systemFiles"
                },
                {
                    "order": 4,
                    "name": "payload"
                },
                {
                    "order": 5,
                    "name": "reports"
                },
                {
                    "order": 6,
                    "name": "cdnImages"
                },
                {
                    "order": 100,
                    "name": "gitContent"
                },
                {
                    "order": 200,
                    "name": "fileSystemContent"
                }
            ]
        },
        {
            "name": "CalcGroup",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "baseSalary"
                },
                {
                    "order": 2,
                    "name": "salaryAdditions"
                },
                {
                    "order": 3,
                    "name": "benefits"
                },
                {
                    "order": 4,
                    "name": "expenses"
                },
                {
                    "order": 5,
                    "name": "deductions"
                },
                {
                    "order": 6,
                    "name": "otherNoPayment"
                },
                {
                    "order": 100,
                    "name": "totals"
                },
                {
                    "order": 200,
                    "name": "disabled"
                }
            ]
        },
        {
            "name": "CalculationFlag",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "exclude"
                },
                {
                    "order": 1,
                    "name": "pensionInsurance"
                },
                {
                    "order": 2,
                    "name": "accidentInsurance"
                },
                {
                    "order": 3,
                    "name": "unemploymentInsurance"
                },
                {
                    "order": 4,
                    "name": "healthInsurance"
                },
                {
                    "order": 10,
                    "name": "insurancesDeduction"
                },
                {
                    "order": 100,
                    "name": "noTax"
                },
                {
                    "order": 101,
                    "name": "tax"
                },
                {
                    "order": 102,
                    "name": "taxDeduction"
                },
                {
                    "order": 200,
                    "name": "cfNoPayment"
                },
                {
                    "order": 201,
                    "name": "cfPayment"
                },
                {
                    "order": 202,
                    "name": "cfDeduction"
                },
                {
                    "order": 203,
                    "name": "cfDeductionAtSalaxy"
                }
            ]
        },
        {
            "name": "CalculationResultRowType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "totalBenefits"
                },
                {
                    "order": 2,
                    "name": "totalExpenses"
                },
                {
                    "order": 3,
                    "name": "totalSocialSecurityEmployer"
                },
                {
                    "order": 4,
                    "name": "totalSocialSecurityEmployerDebt"
                },
                {
                    "order": 5,
                    "name": "totalSocialSecurityEmployerPayment"
                },
                {
                    "order": 6,
                    "name": "totalPension"
                },
                {
                    "order": 7,
                    "name": "totalPensionPayment"
                },
                {
                    "order": 8,
                    "name": "totalPensionEmployer"
                },
                {
                    "order": 9,
                    "name": "totalPensionEmployerDebt"
                },
                {
                    "order": 10,
                    "name": "totalPensionWorker"
                },
                {
                    "order": 11,
                    "name": "totalUnemployment"
                },
                {
                    "order": 12,
                    "name": "totalUnemploymentPayment"
                },
                {
                    "order": 13,
                    "name": "totalUnemploymentEmployer"
                },
                {
                    "order": 14,
                    "name": "totalUnemploymentEmployerDebt"
                },
                {
                    "order": 15,
                    "name": "totalUnemploymentWorker"
                },
                {
                    "order": 16,
                    "name": "totalPalkkaus"
                },
                {
                    "order": 17,
                    "name": "totalSalary"
                },
                {
                    "order": 18,
                    "name": "totalTax"
                },
                {
                    "order": 19,
                    "name": "totalTaxPayment"
                },
                {
                    "order": 20,
                    "name": "totalPayment"
                },
                {
                    "order": 21,
                    "name": "totalWorkerPayment"
                }
            ]
        },
        {
            "name": "CalculationRowSource",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "salaryPage"
                },
                {
                    "order": 1,
                    "name": "tesSelection"
                },
                {
                    "order": 2,
                    "name": "manualRow"
                }
            ]
        },
        {
            "name": "CalculationRowType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "salary"
                },
                {
                    "order": 2,
                    "name": "hourlySalary"
                },
                {
                    "order": 3,
                    "name": "monthlySalary"
                },
                {
                    "order": 4,
                    "name": "totalWorkerPayment"
                },
                {
                    "order": 5,
                    "name": "totalEmployerPayment"
                },
                {
                    "order": 9,
                    "name": "compensation"
                },
                {
                    "order": 10,
                    "name": "overtime"
                },
                {
                    "order": 11,
                    "name": "tesWorktimeShortening"
                },
                {
                    "order": 12,
                    "name": "eveningAddition"
                },
                {
                    "order": 13,
                    "name": "nightimeAddition"
                },
                {
                    "order": 14,
                    "name": "saturdayAddition"
                },
                {
                    "order": 15,
                    "name": "sundayWork"
                },
                {
                    "order": 19,
                    "name": "otherAdditions"
                },
                {
                    "order": 101,
                    "name": "accomodationBenefit"
                },
                {
                    "order": 102,
                    "name": "mealBenefit"
                },
                {
                    "order": 103,
                    "name": "phoneBenefit"
                },
                {
                    "order": 104,
                    "name": "carBenefit"
                },
                {
                    "order": 199,
                    "name": "otherBenefit"
                },
                {
                    "order": 201,
                    "name": "holidayCompensation"
                },
                {
                    "order": 202,
                    "name": "holidayBonus"
                },
                {
                    "order": 203,
                    "name": "holidaySalary"
                },
                {
                    "order": 301,
                    "name": "dailyAllowance"
                },
                {
                    "order": 302,
                    "name": "dailyAllowanceHalf"
                },
                {
                    "order": 303,
                    "name": "mealCompensation"
                },
                {
                    "order": 304,
                    "name": "milageOwnCar"
                },
                {
                    "order": 305,
                    "name": "toolCompensation"
                },
                {
                    "order": 306,
                    "name": "expenses"
                },
                {
                    "order": 307,
                    "name": "milageDaily"
                },
                {
                    "order": 308,
                    "name": "milageOther"
                },
                {
                    "order": 401,
                    "name": "unionPayment"
                },
                {
                    "order": 402,
                    "name": "foreclosure"
                },
                {
                    "order": 403,
                    "name": "advance"
                },
                {
                    "order": 404,
                    "name": "foreclosureByPalkkaus"
                },
                {
                    "order": 405,
                    "name": "prepaidExpenses"
                },
                {
                    "order": 406,
                    "name": "otherDeductions"
                },
                {
                    "order": 901,
                    "name": "childCareSubsidy"
                },
                {
                    "order": 902,
                    "name": "chainsawReduction"
                },
                {
                    "order": 1010,
                    "name": "nonProfitOrg"
                },
                {
                    "order": 1020,
                    "name": "subsidisedCommute"
                },
                {
                    "order": 1030,
                    "name": "irIncomeType"
                },
                {
                    "order": 1040,
                    "name": "board"
                },
                {
                    "order": 1050,
                    "name": "remuneration"
                },
                {
                    "order": 1060,
                    "name": "otherCompensation"
                },
                {
                    "order": 1070,
                    "name": "workingTimeCompensation"
                },
                {
                    "order": 1080,
                    "name": "employmentTermination"
                },
                {
                    "order": 1090,
                    "name": "hourlySalaryWithWorkingTimeCompensation"
                },
                {
                    "order": 9999,
                    "name": "totals"
                }
            ]
        },
        {
            "name": "CalculationRowUnit",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "count"
                },
                {
                    "order": 2,
                    "name": "percent"
                },
                {
                    "order": 3,
                    "name": "days"
                },
                {
                    "order": 4,
                    "name": "kilometers"
                },
                {
                    "order": 5,
                    "name": "hours"
                }
            ]
        },
        {
            "name": "CalculationStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "draft"
                },
                {
                    "order": 1,
                    "name": "paymentStarted"
                },
                {
                    "order": 2,
                    "name": "paymentSucceeded"
                },
                {
                    "order": 3,
                    "name": "paymentCanceled"
                },
                {
                    "order": 4,
                    "name": "paymentError"
                },
                {
                    "order": 5,
                    "name": "paymentWorkerCopy"
                },
                {
                    "order": 6,
                    "name": "workerRequested"
                },
                {
                    "order": 7,
                    "name": "workerRequestAccepted"
                },
                {
                    "order": 8,
                    "name": "workerRequestDeclined"
                },
                {
                    "order": 9,
                    "name": "paymentRefunded"
                },
                {
                    "order": 100,
                    "name": "waitingApproval"
                },
                {
                    "order": 200,
                    "name": "payrollDraft"
                },
                {
                    "order": 220,
                    "name": "proDraft"
                },
                {
                    "order": 300,
                    "name": "sharedWaiting"
                },
                {
                    "order": 310,
                    "name": "sharedApproved"
                },
                {
                    "order": 320,
                    "name": "sharedRejected"
                },
                {
                    "order": 1000,
                    "name": "template"
                }
            ]
        },
        {
            "name": "CalculationStatusCategory",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "all"
                },
                {
                    "order": 1,
                    "name": "readonly"
                },
                {
                    "order": 2,
                    "name": "editable"
                },
                {
                    "order": 3,
                    "name": "sent"
                },
                {
                    "order": 4,
                    "name": "shared"
                },
                {
                    "order": 5,
                    "name": "received"
                }
            ]
        },
        {
            "name": "CalendarActionType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "display"
                },
                {
                    "order": 2,
                    "name": "email"
                },
                {
                    "order": 3,
                    "name": "audio"
                },
                {
                    "order": 10,
                    "name": "createItem"
                },
                {
                    "order": 11,
                    "name": "script"
                }
            ]
        },
        {
            "name": "CalendarEventStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 100,
                    "name": "cancelled"
                },
                {
                    "order": 101,
                    "name": "confirmed"
                },
                {
                    "order": 102,
                    "name": "tentative"
                },
                {
                    "order": 111,
                    "name": "needsAction"
                },
                {
                    "order": 112,
                    "name": "completed"
                },
                {
                    "order": 113,
                    "name": "inProcess"
                }
            ]
        },
        {
            "name": "CarBenefitCode",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "limitedCarBenefit"
                },
                {
                    "order": 2,
                    "name": "fullCarBenefit"
                }
            ]
        },
        {
            "name": "CompanyType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 100,
                    "name": "fiOy"
                },
                {
                    "order": 101,
                    "name": "fiTm"
                },
                {
                    "order": 102,
                    "name": "fiRy"
                },
                {
                    "order": 103,
                    "name": "fiYy"
                }
            ]
        },
        {
            "name": "ContractPartyType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "person"
                },
                {
                    "order": 1,
                    "name": "customContact"
                }
            ]
        },
        {
            "name": "CustomerWebApp",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "palkkaus"
                },
                {
                    "order": 1,
                    "name": "customSso"
                },
                {
                    "order": 2,
                    "name": "customPalkkaus"
                },
                {
                    "order": 999,
                    "name": "noCustomerUi"
                }
            ]
        },
        {
            "name": "EmployerType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 10,
                    "name": "miniEmployer"
                },
                {
                    "order": 20,
                    "name": "smallEmployer"
                },
                {
                    "order": 40,
                    "name": "fullTimeEmployer"
                },
                {
                    "order": 41,
                    "name": "tyelOwnContractEmployer"
                }
            ]
        },
        {
            "name": "EmploymentRelationStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "primary"
                },
                {
                    "order": 2,
                    "name": "secondaryCurrent"
                },
                {
                    "order": 100,
                    "name": "archived"
                }
            ]
        },
        {
            "name": "EmploymentRelationType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "salary"
                },
                {
                    "order": 2,
                    "name": "hourlySalary"
                },
                {
                    "order": 3,
                    "name": "monthlySalary"
                },
                {
                    "order": 100,
                    "name": "compensation"
                },
                {
                    "order": 101,
                    "name": "boardMember"
                },
                {
                    "order": 200,
                    "name": "entrepreneur"
                },
                {
                    "order": 201,
                    "name": "farmer"
                },
                {
                    "order": 301,
                    "name": "employedByStateEmploymentFund"
                },
                {
                    "order": 304,
                    "name": "keyEmployee"
                },
                {
                    "order": 308,
                    "name": "athlete"
                },
                {
                    "order": 309,
                    "name": "performingArtist"
                }
            ]
        },
        {
            "name": "EmploymentSalaryType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "hourlyPay"
                },
                {
                    "order": 2,
                    "name": "montlyPay"
                },
                {
                    "order": 3,
                    "name": "otherPay"
                }
            ]
        },
        {
            "name": "EmploymentType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "job"
                },
                {
                    "order": 2,
                    "name": "partTime"
                },
                {
                    "order": 3,
                    "name": "fullTime"
                }
            ]
        },
        {
            "name": "EmploymentWorkHoursType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "hoursPerDay"
                },
                {
                    "order": 1,
                    "name": "hoursPerWeek"
                },
                {
                    "order": 2,
                    "name": "hourPerTwoWeeks"
                },
                {
                    "order": 3,
                    "name": "hourPerThreeWeeks"
                }
            ]
        },
        {
            "name": "ESalaryPaymentStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "created"
                },
                {
                    "order": 10,
                    "name": "shared"
                },
                {
                    "order": 100,
                    "name": "opened"
                },
                {
                    "order": 200,
                    "name": "employerResolved"
                },
                {
                    "order": 210,
                    "name": "employmentResolved"
                },
                {
                    "order": 220,
                    "name": "taxcardResolved"
                },
                {
                    "order": 230,
                    "name": "paymentMethodResolved"
                },
                {
                    "order": 300,
                    "name": "calculationCreated"
                },
                {
                    "order": 400,
                    "name": "paymentStarted"
                },
                {
                    "order": 410,
                    "name": "paymentSucceeded"
                },
                {
                    "order": 450,
                    "name": "paymentCanceled"
                }
            ]
        },
        {
            "name": "ExportMethod",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "fileExcel"
                },
                {
                    "order": 2,
                    "name": "fileCsv"
                },
                {
                    "order": 3,
                    "name": "filePdf"
                },
                {
                    "order": 101,
                    "name": "copyExcel"
                },
                {
                    "order": 102,
                    "name": "copyCsv"
                },
                {
                    "order": 200,
                    "name": "paymentChannel"
                },
                {
                    "order": 201,
                    "name": "api"
                }
            ]
        },
        {
            "name": "DateOfBirthAccuracy",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "assumption"
                },
                {
                    "order": 1,
                    "name": "ageBased"
                },
                {
                    "order": 2,
                    "name": "ageGroupBased"
                },
                {
                    "order": 3,
                    "name": "monthCorrect"
                },
                {
                    "order": 10,
                    "name": "exact"
                },
                {
                    "order": 20,
                    "name": "verified"
                }
            ]
        },
        {
            "name": "FrameworkAgreement",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notDefined"
                },
                {
                    "order": 1,
                    "name": "construction"
                },
                {
                    "order": 3,
                    "name": "mll"
                },
                {
                    "order": 4,
                    "name": "childCare"
                },
                {
                    "order": 8,
                    "name": "cleaning"
                },
                {
                    "order": 10,
                    "name": "santaClaus"
                },
                {
                    "order": 1000,
                    "name": "entrepreneur"
                },
                {
                    "order": 9999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "Gender",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "male"
                },
                {
                    "order": 2,
                    "name": "female"
                }
            ]
        },
        {
            "name": "HelttiIndustry",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notDefined"
                },
                {
                    "order": 100,
                    "name": "i"
                },
                {
                    "order": 110,
                    "name": "j"
                },
                {
                    "order": 120,
                    "name": "k"
                },
                {
                    "order": 140,
                    "name": "m"
                },
                {
                    "order": 160,
                    "name": "o"
                },
                {
                    "order": 170,
                    "name": "n"
                },
                {
                    "order": 170,
                    "name": "n"
                },
                {
                    "order": 180,
                    "name": "q"
                },
                {
                    "order": 190,
                    "name": "r"
                },
                {
                    "order": 200,
                    "name": "s"
                },
                {
                    "order": 999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "HelttiProductPackage",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notDefined"
                },
                {
                    "order": 10,
                    "name": "small"
                },
                {
                    "order": 20,
                    "name": "medium"
                },
                {
                    "order": 30,
                    "name": "large"
                }
            ]
        },
        {
            "name": "IncomeEarnerType",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "employedByStateEmploymentFund"
                },
                {
                    "order": 2,
                    "name": "jointOwnerWithPayer"
                },
                {
                    "order": 3,
                    "name": "partialOwner"
                },
                {
                    "order": 4,
                    "name": "keyEmployee"
                },
                {
                    "order": 5,
                    "name": "leasedEmployeeLivingAbroad"
                },
                {
                    "order": 6,
                    "name": "personWorkingInFrontierDistrict"
                },
                {
                    "order": 7,
                    "name": "personWorkingAbroad"
                },
                {
                    "order": 8,
                    "name": "athlete"
                },
                {
                    "order": 9,
                    "name": "performingArtist"
                },
                {
                    "order": 10,
                    "name": "restrictedPeriodInFinland"
                },
                {
                    "order": 11,
                    "name": "netOfTaxContract"
                },
                {
                    "order": 12,
                    "name": "organization"
                },
                {
                    "order": 13,
                    "name": "personWorkingOnAlandFerry"
                },
                {
                    "order": 14,
                    "name": "entrepreneurOrFarmerNoPensionRequired"
                }
            ]
        },
        {
            "name": "IncomeLogDiff",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                },
                {
                    "order": 1,
                    "name": "new"
                },
                {
                    "order": 2,
                    "name": "changed"
                },
                {
                    "order": 3,
                    "name": "removed"
                }
            ]
        },
        {
            "name": "InvoiceStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 20,
                    "name": "forecast"
                },
                {
                    "order": 30,
                    "name": "preview"
                },
                {
                    "order": 50,
                    "name": "waitingPalkkaus"
                },
                {
                    "order": 100,
                    "name": "unread"
                },
                {
                    "order": 110,
                    "name": "read"
                },
                {
                    "order": 120,
                    "name": "waitingConfirmation"
                },
                {
                    "order": 130,
                    "name": "paymentStarted"
                },
                {
                    "order": 140,
                    "name": "paid"
                },
                {
                    "order": 150,
                    "name": "canceled"
                },
                {
                    "order": 160,
                    "name": "error"
                }
            ]
        },
        {
            "name": "InvoiceType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 100,
                    "name": "net"
                },
                {
                    "order": 110,
                    "name": "tax"
                },
                {
                    "order": 111,
                    "name": "taxWithholding"
                },
                {
                    "order": 112,
                    "name": "taxSocialSecurity"
                },
                {
                    "order": 120,
                    "name": "unemployment"
                },
                {
                    "order": 130,
                    "name": "pension"
                },
                {
                    "order": 140,
                    "name": "union"
                },
                {
                    "order": 150,
                    "name": "foreclosure"
                },
                {
                    "order": 200,
                    "name": "gross"
                }
            ]
        },
        {
            "name": "Holiday",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "newYearsDay"
                },
                {
                    "order": 1,
                    "name": "epiphany"
                },
                {
                    "order": 2,
                    "name": "goodFriday"
                },
                {
                    "order": 3,
                    "name": "easterSunday"
                },
                {
                    "order": 4,
                    "name": "easterSaturday"
                },
                {
                    "order": 5,
                    "name": "easterMonday"
                },
                {
                    "order": 6,
                    "name": "mayDay"
                },
                {
                    "order": 7,
                    "name": "ascensionDay"
                },
                {
                    "order": 8,
                    "name": "pentecost"
                },
                {
                    "order": 9,
                    "name": "midsummerEve"
                },
                {
                    "order": 10,
                    "name": "midsummerDay"
                },
                {
                    "order": 11,
                    "name": "allSaintsDay"
                },
                {
                    "order": 12,
                    "name": "independenceDay"
                },
                {
                    "order": 13,
                    "name": "christmasEve"
                },
                {
                    "order": 14,
                    "name": "christmasDay"
                },
                {
                    "order": 15,
                    "name": "stStephensDay"
                }
            ]
        },
        {
            "name": "HolidayAccrualSource",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "initial"
                },
                {
                    "order": 1,
                    "name": "manual"
                },
                {
                    "order": 10,
                    "name": "calcDraft"
                },
                {
                    "order": 11,
                    "name": "calcPaid"
                }
            ]
        },
        {
            "name": "HolidayBonusPaymentMethod",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "payForHolidaySalary"
                },
                {
                    "order": 2,
                    "name": "paySummerBonus"
                },
                {
                    "order": 24,
                    "name": "pay24Days"
                },
                {
                    "order": 100,
                    "name": "payAllBonus"
                }
            ]
        },
        {
            "name": "HolidayCode",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 10,
                    "name": "permanent14Days"
                },
                {
                    "order": 11,
                    "name": "permanent35Hours"
                },
                {
                    "order": 20,
                    "name": "temporaryTimeOff"
                },
                {
                    "order": 21,
                    "name": "holidayCompensation"
                },
                {
                    "order": 22,
                    "name": "holidayCompensationIncluded"
                },
                {
                    "order": 99,
                    "name": "noHolidays"
                }
            ]
        },
        {
            "name": "HolidayGroup",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "official"
                },
                {
                    "order": 2,
                    "name": "nonBanking"
                },
                {
                    "order": 4,
                    "name": "holiday"
                }
            ]
        },
        {
            "name": "InsuranceCompany",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "lähiTapiola"
                },
                {
                    "order": 2,
                    "name": "pohjola"
                },
                {
                    "order": 3,
                    "name": "if"
                },
                {
                    "order": 4,
                    "name": "fennia"
                },
                {
                    "order": 5,
                    "name": "aVakuutus"
                },
                {
                    "order": 6,
                    "name": "aktia"
                },
                {
                    "order": 7,
                    "name": "pohjantähti"
                },
                {
                    "order": 8,
                    "name": "tryg"
                },
                {
                    "order": 9,
                    "name": "ålands"
                },
                {
                    "order": 10,
                    "name": "turva"
                },
                {
                    "order": 11,
                    "name": "redarnas"
                },
                {
                    "order": 12,
                    "name": "folksam"
                },
                {
                    "order": 13,
                    "name": "alandia"
                },
                {
                    "order": 999999,
                    "name": "other"
                },
                {
                    "order": 1000000,
                    "name": "pending"
                }
            ]
        },
        {
            "name": "InvoicePaymentType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                },
                {
                    "order": 1,
                    "name": "salary"
                }
            ]
        },
        {
            "name": "IrFlags",
            "description": null,
            "values": [
                {
                    "order": 10,
                    "name": "noMoney"
                },
                {
                    "order": 20,
                    "name": "oneOff"
                },
                {
                    "order": 30,
                    "name": "unjustEnrichment"
                }
            ]
        },
        {
            "name": "IrInsuranceExceptions",
            "description": null,
            "values": [
                {
                    "order": 501,
                    "name": "includeAll"
                },
                {
                    "order": 502,
                    "name": "includePension"
                },
                {
                    "order": 503,
                    "name": "includeHealthInsurance"
                },
                {
                    "order": 505,
                    "name": "includeUnemployment"
                },
                {
                    "order": 506,
                    "name": "includeAccidentInsurance"
                },
                {
                    "order": 601,
                    "name": "excludeAll"
                },
                {
                    "order": 602,
                    "name": "excludePension"
                },
                {
                    "order": 603,
                    "name": "excludeHealthInsurance"
                },
                {
                    "order": 605,
                    "name": "excludeUnemployment"
                },
                {
                    "order": 606,
                    "name": "excludeAccidentInsurance"
                }
            ]
        },
        {
            "name": "IrRowSourceType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "manual"
                },
                {
                    "order": 1,
                    "name": "usecase"
                },
                {
                    "order": 2,
                    "name": "usecaseV02"
                }
            ]
        },
        {
            "name": "LegalEntityType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "person"
                },
                {
                    "order": 2,
                    "name": "company"
                },
                {
                    "order": 3,
                    "name": "personCreatedByEmployer"
                },
                {
                    "order": 4,
                    "name": "partner"
                }
            ]
        },
        {
            "name": "LegacyTaxcardType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "example"
                },
                {
                    "order": 1,
                    "name": "noTaxCard"
                },
                {
                    "order": 2,
                    "name": "sideIncome"
                },
                {
                    "order": 3,
                    "name": "noTaxPrepayment"
                },
                {
                    "order": 4,
                    "name": "taxCardA"
                },
                {
                    "order": 5,
                    "name": "taxCardB"
                },
                {
                    "order": 6,
                    "name": "freelancer"
                },
                {
                    "order": 7,
                    "name": "steps"
                },
                {
                    "order": 99,
                    "name": "others"
                }
            ]
        },
        {
            "name": "MessageFrom",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 10,
                    "name": "owner"
                },
                {
                    "order": 20,
                    "name": "otherParty"
                },
                {
                    "order": 90,
                    "name": "system"
                }
            ]
        },
        {
            "name": "MessageType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "email"
                },
                {
                    "order": 1,
                    "name": "sms"
                }
            ]
        },
        {
            "name": "OnboardingStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "created"
                },
                {
                    "order": 10,
                    "name": "open"
                },
                {
                    "order": 100,
                    "name": "done"
                },
                {
                    "order": 100,
                    "name": "done"
                }
            ]
        },
        {
            "name": "PartnerSite",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "palkkaus"
                },
                {
                    "order": 2,
                    "name": "palkkausIdentified"
                },
                {
                    "order": 3,
                    "name": "palkkausEmployerFillIn"
                },
                {
                    "order": 10,
                    "name": "example"
                },
                {
                    "order": 100,
                    "name": "dev"
                },
                {
                    "order": 110,
                    "name": "refugeeJobs"
                },
                {
                    "order": 120,
                    "name": "talkkarit"
                },
                {
                    "order": 130,
                    "name": "raksa"
                },
                {
                    "order": 140,
                    "name": "palkkaapakolainen"
                },
                {
                    "order": 1010,
                    "name": "careDotCom"
                },
                {
                    "order": 1020,
                    "name": "duunitori"
                },
                {
                    "order": 1030,
                    "name": "mol"
                }
            ]
        },
        {
            "name": "PaymentChannel",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "test"
                },
                {
                    "order": 10,
                    "name": "zeroPayment"
                },
                {
                    "order": 20,
                    "name": "external"
                },
                {
                    "order": 101,
                    "name": "palkkausManual"
                },
                {
                    "order": 102,
                    "name": "palkkausWS"
                },
                {
                    "order": 103,
                    "name": "palkkausPersonal"
                },
                {
                    "order": 201,
                    "name": "palkkausCfaPaytrail"
                },
                {
                    "order": 203,
                    "name": "palkkausCfaReference"
                },
                {
                    "order": 204,
                    "name": "palkkausCfaFinvoice"
                },
                {
                    "order": 210,
                    "name": "palkkausCfaTest"
                },
                {
                    "order": 1010,
                    "name": "accountorGo"
                },
                {
                    "order": 1020,
                    "name": "talenomOnline"
                },
                {
                    "order": 1021,
                    "name": "talenomCfa"
                },
                {
                    "order": 1030,
                    "name": "holviCfa"
                },
                {
                    "order": 1040,
                    "name": "finagoSolo"
                },
                {
                    "order": 1050,
                    "name": "procountor"
                },
                {
                    "order": 1060,
                    "name": "kevytyrittaja"
                },
                {
                    "order": 1070,
                    "name": "vismaNetvisor"
                }
            ]
        },
        {
            "name": "PaymentMethod",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "sepa"
                },
                {
                    "order": 2,
                    "name": "siirto"
                },
                {
                    "order": 3,
                    "name": "test"
                }
            ]
        },
        {
            "name": "PaymentStatus",
            "description": null,
            "values": [
                {
                    "order": 1,
                    "name": "new"
                },
                {
                    "order": 100,
                    "name": "sentToBank"
                },
                {
                    "order": 101,
                    "name": "bankTechApproval"
                },
                {
                    "order": 102,
                    "name": "bankDelivered"
                },
                {
                    "order": 109,
                    "name": "bankPartialError"
                },
                {
                    "order": 110,
                    "name": "bankError"
                },
                {
                    "order": 1000,
                    "name": "paid"
                },
                {
                    "order": 8999,
                    "name": "unknown"
                },
                {
                    "order": 9999,
                    "name": "cancelled"
                }
            ]
        },
        {
            "name": "PayrollStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "draft"
                },
                {
                    "order": 1,
                    "name": "paymentStarted"
                },
                {
                    "order": 2,
                    "name": "paymentSucceeded"
                },
                {
                    "order": 3,
                    "name": "paymentCanceled"
                },
                {
                    "order": 4,
                    "name": "paymentError"
                },
                {
                    "order": 100,
                    "name": "waitingApproval"
                },
                {
                    "order": 1000,
                    "name": "template"
                }
            ]
        },
        {
            "name": "PensionCalculation",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "employee"
                },
                {
                    "order": 2,
                    "name": "entrepreneur"
                },
                {
                    "order": 3,
                    "name": "farmer"
                },
                {
                    "order": 4,
                    "name": "partialOwner"
                },
                {
                    "order": 5,
                    "name": "athlete"
                },
                {
                    "order": 6,
                    "name": "compensation"
                },
                {
                    "order": 7,
                    "name": "boardRemuneration"
                },
                {
                    "order": 8,
                    "name": "smallEntrepreneur"
                },
                {
                    "order": 9,
                    "name": "smallFarmer"
                }
            ]
        },
        {
            "name": "PensionCompany",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 2,
                    "name": "etera"
                },
                {
                    "order": 3,
                    "name": "ilmarinen"
                },
                {
                    "order": 4,
                    "name": "elo"
                },
                {
                    "order": 5,
                    "name": "pensionsAlandia"
                },
                {
                    "order": 6,
                    "name": "varma"
                },
                {
                    "order": 7,
                    "name": "veritas"
                },
                {
                    "order": 8,
                    "name": "apteekkien"
                },
                {
                    "order": 9,
                    "name": "verso"
                },
                {
                    "order": 999999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "PeriodDateKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "paidAtDate"
                },
                {
                    "order": 1,
                    "name": "salaryDate"
                }
            ]
        },
        {
            "name": "PeriodType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "month"
                },
                {
                    "order": 1,
                    "name": "quarter"
                },
                {
                    "order": 2,
                    "name": "year"
                },
                {
                    "order": 99,
                    "name": "custom"
                }
            ]
        },
        {
            "name": "PricingModel",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "palkkausFee"
                },
                {
                    "order": 1,
                    "name": "noFee"
                },
                {
                    "order": 2,
                    "name": "fixedFee"
                }
            ]
        },
        {
            "name": "ProductPackage",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                },
                {
                    "order": 1,
                    "name": "partnerBasic"
                },
                {
                    "order": 2,
                    "name": "partnerPro"
                }
            ]
        },
        {
            "name": "ProductListFilter",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "available"
                },
                {
                    "order": 1,
                    "name": "all"
                }
            ]
        },
        {
            "name": "ReportCategory",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "all"
                },
                {
                    "order": 1,
                    "name": "monthly"
                },
                {
                    "order": 2,
                    "name": "yearly"
                }
            ]
        },
        {
            "name": "ReportType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "example"
                },
                {
                    "order": 100,
                    "name": "monthlyDetails"
                },
                {
                    "order": 110,
                    "name": "taxMonthly4001"
                },
                {
                    "order": 111,
                    "name": "payerSummaryReport"
                },
                {
                    "order": 120,
                    "name": "monthlyPension"
                },
                {
                    "order": 190,
                    "name": "monthlyLiikekirjuri"
                },
                {
                    "order": 200,
                    "name": "monthlyRapko"
                },
                {
                    "order": 201,
                    "name": "monthlyAccounting"
                },
                {
                    "order": 1000,
                    "name": "yearlyDetails"
                },
                {
                    "order": 1001,
                    "name": "yearEndReport"
                },
                {
                    "order": 1010,
                    "name": "yearlyWorkerSummary"
                },
                {
                    "order": 1100,
                    "name": "taxYearly7801"
                },
                {
                    "order": 1200,
                    "name": "unemployment"
                },
                {
                    "order": 1210,
                    "name": "insurance"
                },
                {
                    "order": 1800,
                    "name": "householdDeduction"
                },
                {
                    "order": 1814,
                    "name": "taxHouseholdDeduction14B"
                },
                {
                    "order": 1815,
                    "name": "taxHouseholdDeduction14BSpouseA"
                },
                {
                    "order": 1816,
                    "name": "taxHouseholdDeduction14BSpouseB"
                },
                {
                    "order": 2000,
                    "name": "salarySlip"
                },
                {
                    "order": 2001,
                    "name": "salarySlipPaid"
                },
                {
                    "order": 2002,
                    "name": "salarySlipCopy"
                },
                {
                    "order": 2010,
                    "name": "employerReport"
                },
                {
                    "order": 2020,
                    "name": "paymentReport"
                },
                {
                    "order": 2030,
                    "name": "earningsPaymentReport"
                },
                {
                    "order": 2200,
                    "name": "employmentContract"
                }
            ]
        },
        {
            "name": "Role",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "accountant"
                },
                {
                    "order": 2,
                    "name": "trustedPartner"
                },
                {
                    "order": 3,
                    "name": "accountantCandidate"
                },
                {
                    "order": 4,
                    "name": "pricingPartner"
                }
            ]
        },
        {
            "name": "SalaryKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "fixedSalary"
                },
                {
                    "order": 2,
                    "name": "hourlySalary"
                },
                {
                    "order": 3,
                    "name": "monthlySalary"
                },
                {
                    "order": 50,
                    "name": "compensation"
                },
                {
                    "order": 100,
                    "name": "totalWorkerPayment"
                },
                {
                    "order": 101,
                    "name": "totalEmployerPayment"
                }
            ]
        },
        {
            "name": "SalaryType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "tyel"
                },
                {
                    "order": 1,
                    "name": "yel"
                },
                {
                    "order": 2,
                    "name": "compensation"
                }
            ]
        },
        {
            "name": "SalaryPerMonthRange",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 50,
                    "name": "noPensionRequired"
                },
                {
                    "order": 60,
                    "name": "normal"
                }
            ]
        },
        {
            "name": "SalaryPerYearRange",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "zeroSalary"
                },
                {
                    "order": 1,
                    "name": "noReportingNecessary"
                },
                {
                    "order": 200,
                    "name": "noMonthlyReporting"
                },
                {
                    "order": 1500,
                    "name": "normal"
                }
            ]
        },
        {
            "name": "ServiceModel",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "shared"
                },
                {
                    "order": 1,
                    "name": "partnerOnly"
                }
            ]
        },
        {
            "name": "SettingsStatus",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "checked"
                },
                {
                    "order": 10,
                    "name": "pending"
                }
            ]
        },
        {
            "name": "SharingUriType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "url"
                },
                {
                    "order": 3,
                    "name": "employer"
                }
            ]
        },
        {
            "name": "TaxcardApiIncomeType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 24,
                    "name": "salary"
                },
                {
                    "order": 27,
                    "name": "dividendsForWorkAsSalary"
                },
                {
                    "order": 28,
                    "name": "personnelFundIncome"
                },
                {
                    "order": 29,
                    "name": "salaryOneOff"
                },
                {
                    "order": 33,
                    "name": "compensationForUseEarnedIncome"
                },
                {
                    "order": 34,
                    "name": "compensationForUseCapitalIncome"
                },
                {
                    "order": 35,
                    "name": "compensationVatResponsible"
                },
                {
                    "order": 36,
                    "name": "compensation"
                },
                {
                    "order": 37,
                    "name": "dividendsForWorkAsCompensation"
                },
                {
                    "order": 38,
                    "name": "athletePay"
                },
                {
                    "order": 39,
                    "name": "otherTaxableIncome"
                },
                {
                    "order": 40,
                    "name": "otherTaxableIncomeOneOff"
                },
                {
                    "order": 42,
                    "name": "employeeStockOption"
                }
            ]
        },
        {
            "name": "TaxcardApprovalMethod",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "assureWaiting"
                },
                {
                    "order": 1,
                    "name": "approve"
                },
                {
                    "order": 2,
                    "name": "reject"
                }
            ]
        },
        {
            "name": "TaxcardCalcDiffCheck",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "getDiff"
                },
                {
                    "order": 1,
                    "name": "noCheck"
                },
                {
                    "order": 2,
                    "name": "commitDiff"
                }
            ]
        },
        {
            "name": "TaxCardIncomeType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "salaxyCalculation"
                },
                {
                    "order": 2,
                    "name": "previousEmployerSalaries"
                },
                {
                    "order": 3,
                    "name": "externalSalaries"
                },
                {
                    "order": 10,
                    "name": "sharedCardExtSum"
                },
                {
                    "order": 100,
                    "name": "diff"
                }
            ]
        },
        {
            "name": "TaxcardState",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "new"
                },
                {
                    "order": 10,
                    "name": "approved"
                },
                {
                    "order": 100,
                    "name": "employerAdded"
                },
                {
                    "order": 200,
                    "name": "verifiedVero"
                },
                {
                    "order": 300,
                    "name": "sharedWaiting"
                },
                {
                    "order": 310,
                    "name": "sharedApproved"
                },
                {
                    "order": 320,
                    "name": "sharedRejected"
                },
                {
                    "order": 330,
                    "name": "sharedRejectedWithoutOpen"
                },
                {
                    "order": 500,
                    "name": "shared"
                }
            ]
        },
        {
            "name": "TaxcardKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "noTaxCard"
                },
                {
                    "order": 2,
                    "name": "auto"
                },
                {
                    "order": 10,
                    "name": "defaultYearly"
                },
                {
                    "order": 11,
                    "name": "replacement"
                },
                {
                    "order": 20,
                    "name": "noWithholdingHousehold"
                },
                {
                    "order": 99,
                    "name": "others"
                },
                {
                    "order": 900,
                    "name": "historical"
                }
            ]
        },
        {
            "name": "TaxcardValidity",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "valid"
                },
                {
                    "order": 2,
                    "name": "validJanuary"
                },
                {
                    "order": 3,
                    "name": "future"
                },
                {
                    "order": 4,
                    "name": "expired"
                }
            ]
        },
        {
            "name": "TaxDeductionWorkCategories",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "householdwork"
                },
                {
                    "order": 2,
                    "name": "carework"
                },
                {
                    "order": 4,
                    "name": "homeImprovement"
                },
                {
                    "order": 8,
                    "name": "ownPropety"
                },
                {
                    "order": 16,
                    "name": "relativesProperty"
                }
            ]
        },
        {
            "name": "TaxReportHandling",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "default"
                },
                {
                    "order": 1,
                    "name": "noZeroSalaryReport"
                },
                {
                    "order": 2,
                    "name": "noMonthlyReport"
                },
                {
                    "order": 3,
                    "name": "always"
                }
            ]
        },
        {
            "name": "TesSubtype",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notSelected"
                },
                {
                    "order": 100,
                    "name": "constructionCarpenter"
                },
                {
                    "order": 101,
                    "name": "constructionFloor"
                },
                {
                    "order": 105,
                    "name": "constructionOther"
                },
                {
                    "order": 109,
                    "name": "constructionFreeContract"
                }
            ]
        },
        {
            "name": "TestEnum",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "valueZero"
                },
                {
                    "order": 1,
                    "name": "valueOne"
                },
                {
                    "order": 2,
                    "name": "valueTwo"
                },
                {
                    "order": 3,
                    "name": "valueThree"
                },
                {
                    "order": 4,
                    "name": "valueFour"
                },
                {
                    "order": 5,
                    "name": "valueFive"
                },
                {
                    "order": 6,
                    "name": "valueSix"
                },
                {
                    "order": 7,
                    "name": "valueSeven"
                },
                {
                    "order": 8,
                    "name": "valueEight"
                },
                {
                    "order": 9,
                    "name": "valueNine"
                },
                {
                    "order": 10,
                    "name": "valueTen"
                }
            ]
        },
        {
            "name": "ThreadedMessageType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "text"
                },
                {
                    "order": 1,
                    "name": "blobFile"
                },
                {
                    "order": 10,
                    "name": "calculation"
                },
                {
                    "order": 20,
                    "name": "payroll"
                },
                {
                    "order": 30,
                    "name": "workerAccount"
                },
                {
                    "order": 500,
                    "name": "email"
                },
                {
                    "order": 510,
                    "name": "sms"
                }
            ]
        },
        {
            "name": "UnionPaymentType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notSelected"
                },
                {
                    "order": 100,
                    "name": "raksaNormal"
                },
                {
                    "order": 101,
                    "name": "raksaUnemploymentOnly"
                },
                {
                    "order": 9999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "Unit",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "hours"
                },
                {
                    "order": 2,
                    "name": "days"
                },
                {
                    "order": 3,
                    "name": "weeks"
                },
                {
                    "order": 5,
                    "name": "period"
                },
                {
                    "order": 1000,
                    "name": "one"
                },
                {
                    "order": 1001,
                    "name": "count"
                },
                {
                    "order": 1002,
                    "name": "percent"
                },
                {
                    "order": 1004,
                    "name": "kilometers"
                },
                {
                    "order": 2000,
                    "name": "euro"
                }
            ]
        },
        {
            "name": "VarmaPensionOrderAction",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "newYel"
                },
                {
                    "order": 2,
                    "name": "newTyel"
                },
                {
                    "order": 3,
                    "name": "move"
                }
            ]
        },
        {
            "name": "VarmaPensionOrderYelPayer",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "entrepreneur"
                },
                {
                    "order": 2,
                    "name": "company"
                }
            ]
        },
        {
            "name": "WageBasis",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 10,
                    "name": "monthly"
                },
                {
                    "order": 20,
                    "name": "hourly"
                },
                {
                    "order": 30,
                    "name": "performanceBased"
                },
                {
                    "order": 999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "WebSiteUserRole",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "household"
                },
                {
                    "order": 2,
                    "name": "worker"
                },
                {
                    "order": 4,
                    "name": "company"
                }
            ]
        },
        {
            "name": "WorkflowEventFeatures",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "none"
                },
                {
                    "order": 1,
                    "name": "isActive"
                },
                {
                    "order": 2,
                    "name": "isEditable"
                },
                {
                    "order": 4,
                    "name": "isRemovable"
                },
                {
                    "order": 8,
                    "name": "isOnHold"
                }
            ]
        },
        {
            "name": "YearEndAdminCheck",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notChecked"
                },
                {
                    "order": 1,
                    "name": "ok"
                },
                {
                    "order": 2,
                    "name": "okWithWarning"
                },
                {
                    "order": 3,
                    "name": "errors"
                }
            ]
        },
        {
            "name": "YearEndFeedbackCalculationType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "salaxyPayment"
                },
                {
                    "order": 10,
                    "name": "external"
                },
                {
                    "order": 100,
                    "name": "correction"
                }
            ]
        },
        {
            "name": "YearEndUserFeedback",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "notChecked"
                },
                {
                    "order": 1,
                    "name": "ok"
                },
                {
                    "order": 2,
                    "name": "okWithModifications"
                },
                {
                    "order": 3,
                    "name": "willHandleMyself"
                },
                {
                    "order": 4,
                    "name": "noYearlyReports"
                }
            ]
        },
        {
            "name": "YtjCompanyType",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "noCompanyType"
                },
                {
                    "order": 1,
                    "name": "housingCooperative"
                },
                {
                    "order": 2,
                    "name": "condominium"
                },
                {
                    "order": 4,
                    "name": "asoAssociation"
                },
                {
                    "order": 5,
                    "name": "openCompany"
                },
                {
                    "order": 6,
                    "name": "association"
                },
                {
                    "order": 9,
                    "name": "hypoAssociation"
                },
                {
                    "order": 10,
                    "name": "mutualRealEstateCompany"
                },
                {
                    "order": 13,
                    "name": "ky"
                },
                {
                    "order": 14,
                    "name": "osuuskunta"
                },
                {
                    "order": 15,
                    "name": "cooperativeBank"
                },
                {
                    "order": 16,
                    "name": "oy"
                },
                {
                    "order": 18,
                    "name": "foundation"
                },
                {
                    "order": 20,
                    "name": "savingsBank"
                },
                {
                    "order": 21,
                    "name": "financialAssociation"
                },
                {
                    "order": 22,
                    "name": "stateEstablishment"
                },
                {
                    "order": 25,
                    "name": "insuranceAssociation"
                },
                {
                    "order": 26,
                    "name": "privateEntrepreneur"
                },
                {
                    "order": 29,
                    "name": "otherAssociation"
                },
                {
                    "order": 31,
                    "name": "specialPurposeAssociation"
                },
                {
                    "order": 35,
                    "name": "forestCareAssociation"
                },
                {
                    "order": 38,
                    "name": "otherFinancialAssociation"
                },
                {
                    "order": 46,
                    "name": "religiousCommunity"
                },
                {
                    "order": 51,
                    "name": "taxableGrouping"
                },
                {
                    "order": 58,
                    "name": "mutualInsuranceAssociation"
                },
                {
                    "order": 60,
                    "name": "foreignOrganisation"
                },
                {
                    "order": 61,
                    "name": "municipalEstablishment"
                },
                {
                    "order": 62,
                    "name": "federationOfMunicipalitiesEstablishment"
                },
                {
                    "order": 64,
                    "name": "alandFederation"
                },
                {
                    "order": 83,
                    "name": "europeanCooperative"
                },
                {
                    "order": 84,
                    "name": "europeanCooperativeBank"
                },
                {
                    "order": 90,
                    "name": "reindeerHerdingCooperative"
                },
                {
                    "order": 999,
                    "name": "unknown"
                }
            ]
        },
        {
            "name": "CarBenefitKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "limitedCarBenefit"
                },
                {
                    "order": 2,
                    "name": "fullCarBenefit"
                }
            ]
        },
        {
            "name": "UnionPaymentKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "fixed"
                },
                {
                    "order": 2,
                    "name": "percentage"
                },
                {
                    "order": 100,
                    "name": "raksaNormal"
                },
                {
                    "order": 101,
                    "name": "raksaUnemploymentOnly"
                },
                {
                    "order": 9999,
                    "name": "other"
                }
            ]
        },
        {
            "name": "NonProfitOrgKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 357,
                    "name": "kilometreAllowance"
                },
                {
                    "order": 358,
                    "name": "dailyAllowance"
                },
                {
                    "order": 1358,
                    "name": "accomodationAllowance"
                }
            ]
        },
        {
            "name": "SubsidisedCommuteKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "noDeduction"
                },
                {
                    "order": 2,
                    "name": "singleDeduction"
                },
                {
                    "order": 10,
                    "name": "periodicalDeduction"
                }
            ]
        },
        {
            "name": "MealBenefitKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "cateringContract"
                },
                {
                    "order": 2,
                    "name": "mealTicket"
                },
                {
                    "order": 3,
                    "name": "taxableAmount"
                },
                {
                    "order": 4,
                    "name": "mealAllowance"
                },
                {
                    "order": 5,
                    "name": "institute"
                },
                {
                    "order": 6,
                    "name": "teacher"
                },
                {
                    "order": 7,
                    "name": "restaurantWorker"
                }
            ]
        },
        {
            "name": "DailyAllowanceKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "fullDailyAllowance"
                },
                {
                    "order": 2,
                    "name": "partialDailyAllowance"
                },
                {
                    "order": 3,
                    "name": "internationalDailyAllowance"
                },
                {
                    "order": 4,
                    "name": "mealAllowance"
                }
            ]
        },
        {
            "name": "BoardKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                }
            ]
        },
        {
            "name": "RemunerationKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 202,
                    "name": "initiativeFee"
                },
                {
                    "order": 203,
                    "name": "bonusPay"
                },
                {
                    "order": 220,
                    "name": "commission"
                },
                {
                    "order": 223,
                    "name": "performanceBonus"
                },
                {
                    "order": 226,
                    "name": "shareIssueForEmployees"
                },
                {
                    "order": 233,
                    "name": "profitSharingBonus"
                }
            ]
        },
        {
            "name": "OtherCompensationKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 210,
                    "name": "meetingFee"
                },
                {
                    "order": 214,
                    "name": "lectureFee"
                },
                {
                    "order": 215,
                    "name": "positionOfTrustCompensation"
                },
                {
                    "order": 225,
                    "name": "accruedTimeOffCompensation"
                },
                {
                    "order": 308,
                    "name": "membershipOfGoverningBodyCompensation"
                },
                {
                    "order": 310,
                    "name": "monetaryGiftForEmployees"
                },
                {
                    "order": 313,
                    "name": "useCompensationAsEarnedIncome"
                },
                {
                    "order": 314,
                    "name": "useCompensationAsCapitalIncome"
                },
                {
                    "order": 316,
                    "name": "otherTaxableIncomeAsEarnedIncome"
                },
                {
                    "order": 320,
                    "name": "stockOptionsAndGrants"
                },
                {
                    "order": 326,
                    "name": "employeeInventionCompensation"
                },
                {
                    "order": 332,
                    "name": "capitalIncomePayment"
                },
                {
                    "order": 339,
                    "name": "workEffortBasedDividendsAsWage"
                },
                {
                    "order": 340,
                    "name": "workEffortBasedDividendsAsNonWage"
                },
                {
                    "order": 343,
                    "name": "employeeStockOption"
                },
                {
                    "order": 361,
                    "name": "employeeStockOptionWithLowerPrice"
                }
            ]
        },
        {
            "name": "WorkingTimeCompensationKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 205,
                    "name": "emergencyWorkCompensation"
                },
                {
                    "order": 206,
                    "name": "eveningWorkCompensation"
                },
                {
                    "order": 207,
                    "name": "eveningShiftAllowance"
                },
                {
                    "order": 211,
                    "name": "saturdayPay"
                },
                {
                    "order": 212,
                    "name": "extraWorkPremium"
                },
                {
                    "order": 216,
                    "name": "otherCompensation"
                },
                {
                    "order": 217,
                    "name": "waitingTimeCompensation"
                },
                {
                    "order": 221,
                    "name": "sundayWorkCompensation"
                },
                {
                    "order": 230,
                    "name": "standByCompensation"
                },
                {
                    "order": 232,
                    "name": "weeklyRestCompensation"
                },
                {
                    "order": 235,
                    "name": "overtimeCompensation"
                },
                {
                    "order": 236,
                    "name": "nightWorkAllowance"
                },
                {
                    "order": 237,
                    "name": "nightShiftCompensation"
                }
            ]
        },
        {
            "name": "EmploymentTerminationKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 208,
                    "name": "noticePeriodCompensation"
                },
                {
                    "order": 224,
                    "name": "monetaryWorkingTimeBankCompensation"
                },
                {
                    "order": 229,
                    "name": "terminationAndLayOffDamages"
                },
                {
                    "order": 231,
                    "name": "voluntaryTerminationCompensation"
                },
                {
                    "order": 338,
                    "name": "pensionPaidByEmployer"
                }
            ]
        },
        {
            "name": "ForeclosureKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "fixed"
                },
                {
                    "order": 2,
                    "name": "periodic"
                },
                {
                    "order": 3,
                    "name": "nonPeriodic"
                }
            ]
        },
        {
            "name": "TotalWorkerPaymentKind",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "undefined"
                },
                {
                    "order": 1,
                    "name": "totalWorkerPayment"
                },
                {
                    "order": 2,
                    "name": "totalWorkerWorkerPaymentExcludingExpenses"
                },
                {
                    "order": 3,
                    "name": "totalWorkerPaymentAndTax"
                },
                {
                    "order": 4,
                    "name": "totalWorkerWorkerPaymentAndTaxExcludingExpenses"
                }
            ]
        },
        {
            "name": "TransactionCode",
            "description": null,
            "values": [
                {
                    "order": 0,
                    "name": "unknown"
                },
                {
                    "order": 1,
                    "name": "ignored"
                },
                {
                    "order": 201,
                    "name": "timeRatePay"
                },
                {
                    "order": 202,
                    "name": "initiativeFee"
                },
                {
                    "order": 203,
                    "name": "bonusPay"
                },
                {
                    "order": 204,
                    "name": "complementaryWagePaidDuringBenefitPeriod"
                },
                {
                    "order": 205,
                    "name": "emergencyWorkCompensation"
                },
                {
                    "order": 206,
                    "name": "eveningWorkCompensation"
                },
                {
                    "order": 207,
                    "name": "eveningShiftAllowance"
                },
                {
                    "order": 208,
                    "name": "noticePeriodCompensation"
                },
                {
                    "order": 209,
                    "name": "kilometreAllowanceTaxable"
                },
                {
                    "order": 210,
                    "name": "meetingFee"
                },
                {
                    "order": 211,
                    "name": "saturdayPay"
                },
                {
                    "order": 212,
                    "name": "extraWorkPremium"
                },
                {
                    "order": 213,
                    "name": "holidayBonus"
                },
                {
                    "order": 214,
                    "name": "lectureFee"
                },
                {
                    "order": 215,
                    "name": "compensationForTrustPosition"
                },
                {
                    "order": 216,
                    "name": "otherCompensation"
                },
                {
                    "order": 217,
                    "name": "waitingTimeCompensation"
                },
                {
                    "order": 218,
                    "name": "workingConditionCompensation"
                },
                {
                    "order": 219,
                    "name": "partialPayDuringSickLeave"
                },
                {
                    "order": 220,
                    "name": "commission"
                },
                {
                    "order": 221,
                    "name": "sundayWorkCompensation"
                },
                {
                    "order": 222,
                    "name": "benefitArisingFromSyntheticOption"
                },
                {
                    "order": 223,
                    "name": "performanceBonus"
                },
                {
                    "order": 224,
                    "name": "monetaryCompensationFromWorkingTimeBank"
                },
                {
                    "order": 225,
                    "name": "compensationForAccruedTimeOff"
                },
                {
                    "order": 226,
                    "name": "shareIssueForEmployees"
                },
                {
                    "order": 227,
                    "name": "contractPay"
                },
                {
                    "order": 229,
                    "name": "damagesInConjunctionWithTermination"
                },
                {
                    "order": 230,
                    "name": "standByCompensation"
                },
                {
                    "order": 231,
                    "name": "voluntaryCompensationInConjunctionWithTermination"
                },
                {
                    "order": 232,
                    "name": "weeklyRestCompensation"
                },
                {
                    "order": 233,
                    "name": "profitSharingBonus"
                },
                {
                    "order": 234,
                    "name": "annualHolidayCompensation"
                },
                {
                    "order": 235,
                    "name": "overtimeCompensation"
                },
                {
                    "order": 236,
                    "name": "nightWorkAllowance"
                },
                {
                    "order": 237,
                    "name": "nightShiftCompensation"
                },
                {
                    "order": 238,
                    "name": "otherRegularCompensation"
                },
                {
                    "order": 239,
                    "name": "compensationForUnusedCompensatoryLeave"
                },
                {
                    "order": 301,
                    "name": "accommodationBenefit"
                },
                {
                    "order": 302,
                    "name": "interestBenefitForHousingLoan"
                },
                {
                    "order": 303,
                    "name": "mealAllowance"
                },
                {
                    "order": 304,
                    "name": "carBenefit"
                },
                {
                    "order": 308,
                    "name": "compensationForGoverningBodyMembership"
                },
                {
                    "order": 309,
                    "name": "shareOfReserveDrawnFromPersonnelFund"
                },
                {
                    "order": 310,
                    "name": "monetaryGiftForEmployees"
                },
                {
                    "order": 311,
                    "name": "kilometreAllowanceTaxExempt"
                },
                {
                    "order": 312,
                    "name": "treatmentFeeForMunicipalVeterinarian"
                },
                {
                    "order": 313,
                    "name": "compensationForUseEarnedIncome"
                },
                {
                    "order": 314,
                    "name": "compensationForUseCapitalIncome"
                },
                {
                    "order": 315,
                    "name": "otherTaxableBenefitForEmployees"
                },
                {
                    "order": 316,
                    "name": "otherTaxableIncomeDeemedEarnedIncome"
                },
                {
                    "order": 317,
                    "name": "otherFringeBenefit"
                },
                {
                    "order": 319,
                    "name": "kinshipCarersFee"
                },
                {
                    "order": 320,
                    "name": "stockOptions"
                },
                {
                    "order": 321,
                    "name": "wagesPaidBySubstitutePayerIncludingSocialInsurance"
                },
                {
                    "order": 322,
                    "name": "wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance"
                },
                {
                    "order": 323,
                    "name": "wagesPaidBySubstitutePayerIncludingUnemploymentInsurance"
                },
                {
                    "order": 324,
                    "name": "wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance"
                },
                {
                    "order": 325,
                    "name": "wagesPaidBySubstitutePayerIncludingHealthInsurance"
                },
                {
                    "order": 326,
                    "name": "compensationForEmployeeInvention"
                },
                {
                    "order": 327,
                    "name": "reimbursementOfPrivateCaretakersExpenses"
                },
                {
                    "order": 328,
                    "name": "privateCaretakersFee"
                },
                {
                    "order": 329,
                    "name": "reimbursementOfFamilyDayCareProvidersExpenses"
                },
                {
                    "order": 330,
                    "name": "telephoneBenefit"
                },
                {
                    "order": 331,
                    "name": "dailyAllowance"
                },
                {
                    "order": 332,
                    "name": "capitalIncomePayment"
                },
                {
                    "order": 334,
                    "name": "mealBenefit"
                },
                {
                    "order": 335,
                    "name": "reimbursementOfCostsPaidToConciliator"
                },
                {
                    "order": 336,
                    "name": "nonWageCompensationForWork"
                },
                {
                    "order": 337,
                    "name": "supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund"
                },
                {
                    "order": 338,
                    "name": "pensionPaidByEmployer"
                },
                {
                    "order": 339,
                    "name": "dividendsBasedOnWages"
                },
                {
                    "order": 340,
                    "name": "dividendsBasedOnNonWage"
                },
                {
                    "order": 341,
                    "name": "employerSubsidisedCommuterTicketTaxExempt"
                },
                {
                    "order": 342,
                    "name": "employerSubsidisedCommuterTicketTaxable"
                },
                {
                    "order": 343,
                    "name": "employeeStockOption"
                },
                {
                    "order": 350,
                    "name": "wagesTransferredToAthletesSpecialFund"
                },
                {
                    "order": 351,
                    "name": "wagesPaidFromAthletesSpecialFund"
                },
                {
                    "order": 352,
                    "name": "wagesForInsurancePurposes"
                },
                {
                    "order": 353,
                    "name": "taxableReimbursementOfExpenses"
                },
                {
                    "order": 354,
                    "name": "privateDayCareAllowanceMunicipalSupplement"
                },
                {
                    "order": 355,
                    "name": "privateDayCareAllowanceWages"
                },
                {
                    "order": 356,
                    "name": "privateDayCareAllowanceNonWage"
                },
                {
                    "order": 357,
                    "name": "kilometreAllowancePaidByNonProfitOrganisation"
                },
                {
                    "order": 358,
                    "name": "dailyAllowancePaidByNonProfitOrganisation"
                },
                {
                    "order": 359,
                    "name": "unjustEnrichment"
                },
                {
                    "order": 361,
                    "name": "employeeStockOptionLowerMarketPrice"
                },
                {
                    "order": 362,
                    "name": "royaltyPaidToNonResidentTaxpayer"
                },
                {
                    "order": 363,
                    "name": "bicycleBenefitTaxExempt"
                },
                {
                    "order": 364,
                    "name": "bicycleBenefitTaxable"
                },
                {
                    "order": 365,
                    "name": "conditionalStockOptions"
                },
                {
                    "order": 401,
                    "name": "compensationCollectedForCarBenefit"
                },
                {
                    "order": 402,
                    "name": "withholdingTax"
                },
                {
                    "order": 403,
                    "name": "electedOfficialFee"
                },
                {
                    "order": 404,
                    "name": "taxAtSource"
                },
                {
                    "order": 405,
                    "name": "taxAtSourceDeduction"
                },
                {
                    "order": 406,
                    "name": "wagesPaid"
                },
                {
                    "order": 407,
                    "name": "reimbursementCollectedForOtherFringeBenefits"
                },
                {
                    "order": 408,
                    "name": "otherItemDeductibleFromNetWage"
                },
                {
                    "order": 409,
                    "name": "netWage"
                },
                {
                    "order": 410,
                    "name": "employerPaidPremiumForCollectiveAdditionalPensionInsurance"
                },
                {
                    "order": 411,
                    "name": "employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution"
                },
                {
                    "order": 412,
                    "name": "employeesHealthInsuranceContribution"
                },
                {
                    "order": 413,
                    "name": "employeesPensionInsuranceContribution"
                },
                {
                    "order": 414,
                    "name": "employeesUnemploymentInsuranceContribution"
                },
                {
                    "order": 415,
                    "name": "reimbursementForEmployerSubsidisedCommuterTicket"
                },
                {
                    "order": 416,
                    "name": "taxPaidAbroad"
                },
                {
                    "order": 417,
                    "name": "distraint"
                },
                {
                    "order": 418,
                    "name": "voluntaryIndividualPensionInsurancePremium"
                },
                {
                    "order": 419,
                    "name": "deductionBeforeWithholding"
                },
                {
                    "order": 420,
                    "name": "reimbursementCollectedForBicycleBenefit"
                }
            ]
        }
    ];

    /* eslint-disable */
    /**
     * Authentication methods for VismaSign.
     * This is the raw list of methods as returned by https://www.onnistuu.fi/api/v1/auth/methods
     * We will override most of these images as the ones below are pretty outdated.
     * @ignore
     */
    const vismaSignOriginalMethods = [{
            "identifier": "tupas-osuuspankki",
            "name": "Osuuspankki",
            "image": "https://www.onnistuu.fi/img/banks/op_white_bgr-14.gif"
        },
        {
            "identifier": "tupas-nordea",
            "name": "Nordea",
            "image": "https://www.onnistuu.fi/img/banks/nordea_button_70x70px.gif"
        },
        {
            "identifier": "tupas-danske",
            "name": "Danske Bank",
            "image": "https://www.onnistuu.fi/img/banks/danskebank.png"
        },
        {
            "identifier": "tupas-spankki",
            "name": "S-Pankki",
            "image": "https://www.onnistuu.fi/img/banks/s-pankki.png"
        },
        {
            "identifier": "tupas-handelsbanken",
            "name": "Handelsbanken",
            "image": "https://www.onnistuu.fi/img/banks/handelsbanken_painike.jpg"
        },
        {
            "identifier": "tupas-elisa",
            "name": "Mobiilivarmenne",
            "image": "https://www.onnistuu.fi/img/banks/elisa.png"
        },
        {
            "identifier": "tupas-aktia",
            "name": "Aktia",
            "image": "https://www.onnistuu.fi/img/banks/aktia.png"
        },
        {
            "identifier": "tupas-poppankki",
            "name": "POP Pankki",
            "image": "https://www.onnistuu.fi/img/banks/POPmaksunappi.png"
        },
        {
            "identifier": "tupas-sp",
            "name": "S\u00e4\u00e4st\u00f6pankki",
            "image": "https://www.onnistuu.fi/img/banks/Spmaksunappi.png"
        },
        {
            "identifier": "tupas-omasp",
            "name": "Oma S\u00e4\u00e4st\u00f6pankki",
            "image": "https://www.onnistuu.fi/img/banks/omasp.png"
        },
        {
            "identifier": "tupas-alandsbanken",
            "name": "\u00c5landsbanken",
            "image": "https://www.onnistuu.fi/img/banks/alandsbanken.gif"
        }];

    /* eslint-disable */
    /**
     * Income types code-generated from the server.
     *
     * @generator https://test-ir.salaxy.com/v01/api/codes/incometype
     * Fetch this data from https://test-ir.salaxy.com/v01/api/codes/incometype
     * There is no automatic fetching because the data updated quite seldomly
     */
    const incomeTypeCodes = [
        {
            "code": 101,
            "nameEn": "Total wages",
            "nameFi": "Palkka yhteissumma",
            "nameSv": "Lön totalt",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 102,
            "nameEn": "Total wages subject to earnings-related pension insurance contribution ",
            "nameFi": "Palkkasumma, joka on työeläkevakuutusmaksun alainen ",
            "nameSv": "Lönesumma som omfattas av arbetspensionsförsäkringsavgiften ",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 103,
            "nameEn": "Total wages subject to social insurance contributions",
            "nameFi": "Palkkasumma, joka on sosiaalivakuutusmaksujen alainen",
            "nameSv": "Lönesumma som omfattas av socialförsäkringsavgifter",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 104,
            "nameEn": "Total wages subject to health insurance contribution",
            "nameFi": "Palkkasumma, joka on sairausvakuutusmaksun alainen",
            "nameSv": "Lönesumma som omfattas av sjukförsäkringsavgiften",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 105,
            "nameEn": "Total wages subject to unemployment insurance contribution",
            "nameFi": "Palkkasumma, joka on työttömyysvakuutusmaksun alainen",
            "nameSv": "Lönesumma som omfattas av arbetslöshetsförsäkringsavgiften",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": true,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 106,
            "nameEn": "Total wages subject to accident and occupational disease insurance contribution",
            "nameFi": "Palkkasumma, joka on työtapaturma- ja ammattitautivakuutusmaksun alainen",
            "nameSv": "Lönesumma som omfattas av arbetsolycksfalls- och yrkessjukdomsförsäkringsavgiften",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": true,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": null,
            "notSupportedError": "Reporting method 1 (codes 101-106) is not supported in this version",
            "description": "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 201,
            "nameEn": "Time-rate pay",
            "nameFi": "Aikapalkka",
            "nameSv": "Tidlön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "timeRatePay",
            "notSupportedError": null,
            "description": "Palkka, jota maksetaan työhön käytetyn ajan perusteella. Aikapalkka voidaan määritellä esimerkiksi kalenterikuukaudelle.",
            "taxAndSidecostsDescr": "Aikapalkasta toimitetaan ennakonpidätys ja maksetaan työnantajan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/wages"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 202,
            "nameEn": "Initiative fee",
            "nameFi": "Aloitepalkkio",
            "nameSv": "Initiativarvode",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "initiativeFee",
            "notSupportedError": null,
            "description": "Kertapalkkio, joka maksetaan aloitteen tekijälle esimerkiksi toiminnan tai tuotteen kehittämiseen liittyvästä aloitteesta.",
            "taxAndSidecostsDescr": "Jos aloitepalkkio liittyy työsopimuksessa sovittuun työhön, palkkiota käsitellään palkkana eli ennakonpidätyksen toimittamisen lisäksi palkkiosta maksetaan sosiaalivakuutusmaksut.\r\n\r\nJos aloitepalkkion saaja ei ole palvelussuhteessa palkkion maksajaan tai aloite ei liity työntekijän työsopimuksessa sovittuun työhön, palkkiosta ei makseta sosiaalivakuutusmaksuja. Tällöin aloitepalkkiosta toimitetaan ennakonpidätys, ellei aloitepalkkion saaja kuulu ennakkoperintärekisteriin.\r\n\r\nOletuksena tulolajissa on, että se on sosiaalivakuutusmaksujen alainen.\r\n\r\nJos poikkeuksellisesti aloitepalkkio ei olisikaan edellä mainittujen maksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillisenä tietona.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 203,
            "nameEn": "Bonus pay",
            "nameFi": "Bonuspalkka",
            "nameSv": "Bonuslön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "bonusPay",
            "notSupportedError": null,
            "description": "Kertaluonteinen palkkiopalkka, jonka suuruus määräytyy kokonaan tai osittain sen mukaan, kuinka hyvin tavoitteeksi asetettu työtulos saavutetaan. Työtuloksen mittana voi olla määrä, laatu tai jokin muu tulostekijä.\r\n\r\nBonuspalkka eroaa provisiopalkasta siten, että bonuspalkka on kertaluonteinen ja provisio on jatkuvaluonteinen palkkiopalkka. Kumpikin näistä perustuu henkilön tai ryhmän ennalta asetetun tavoitteen saavuttamiseen. Bonuspalkka eroaa voittopalkkiosta siten, että voittopalkkio maksetaan ilman ennalta sovittua suunnitelmaa, yrityksen voiton perusteella.",
            "taxAndSidecostsDescr": "Bonuspalkasta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 204,
            "nameEn": "Complementary wage/salary paid during benefit period",
            "nameFi": "Etuusajalta maksettu täydennyspalkka",
            "nameSv": "Kompletteringslön för förmånstid",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "complementaryWagePaidDuringBenefitPeriod",
            "notSupportedError": null,
            "description": "Etuutta saavalle työntekijälle maksettu palkka, joka kattaa täyden palkan ja maksetun etuuden välisen erotuksen.\r\n\r\nMaksettu etuus voi olla esimerkiksi Kelan maksama sairauspäiväraha. Kun työnantaja maksaa etuusajalta täydennyspalkkaa, Kela maksaa etuuden suoraan työntekijälle, ei työnantajalle.",
            "taxAndSidecostsDescr": "Etuusajalta maksetusta täydennyspalkasta toimitetaan ennakonpidätys ja maksetaan työnantajan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/sickness"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 205,
            "nameEn": "Emergency work compensation",
            "nameFi": "Hätätyökorvaus",
            "nameSv": "Ersättning för nödarbete",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "emergencyWorkCompensation",
            "notSupportedError": null,
            "description": "Työaikakorvaus, joka maksetaan hätätyöstä. Hätätyöllä tarkoitetaan hätätilanteessa säännöllisen työajan ylittävänä aikana teetettävää työtä.",
            "taxAndSidecostsDescr": "Hätätyökorvauksesta toimitetaan ennakonpidätys ja maksetaan työnantajan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 206,
            "nameEn": "Evening work compensation",
            "nameFi": "Iltatyökorvaus",
            "nameSv": "Kvällsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "eveningWorkCompensation",
            "notSupportedError": null,
            "description": "Työaikakorvaus, joka maksetaan iltatyöstä. Iltatyökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
            "taxAndSidecostsDescr": "Iltatyökorvauksesta toimitetaan ennakonpidätys ja maksetaan työnantajan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 207,
            "nameEn": "Evening shift allowance",
            "nameFi": "Iltavuorolisä",
            "nameSv": "Kvällsskiftstillägg",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "eveningShiftAllowance",
            "notSupportedError": null,
            "description": "Vuorotyökorvaus, joka maksetaan iltavuorosta.",
            "taxAndSidecostsDescr": "Iltavuorolisästä toimitetaan ennakonpidätys ja maksetaan työnantajan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 208,
            "nameEn": "Notice period compensation",
            "nameFi": "Irtisanomisajan korvaus",
            "nameSv": "Ersättning för uppsägningstid",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "noticePeriodCompensation",
            "notSupportedError": null,
            "description": "Korvaus, jonka työnantaja maksaa työntekijälle palvelussuhteen päättyessä ilman irtisanomisaikaa eli tilanteessa, jossa jätetään irtisanomisaika noudattamatta ja työsuhde päättyy heti (TSL 6:4). Tulolajilla ilmoitetaan myös korvaus, jonka työnantaja maksaa, kun työntekijä irtisanoutuu yli 200 päivää kestäneen lomautuksen johdosta sekä, kun työnantaja irtisanoo lomautettuna olevan työntekijän.\r\n\r\nTulolajilla ilmoitetaan vain sellaiset erät, jotka maksetaan työsuhteen päättyessä ilman irtisanomisaikaa. Irtisanomisajalta maksettu palkka ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
            "taxAndSidecostsDescr": "Irtisanomisajan korvauksesta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/employmentTermination"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 209,
            "nameEn": "Kilometre allowance (taxable)",
            "nameFi": "Kilometrikorvaus (veronalainen)",
            "nameSv": "Kilometerersättning (skattepliktig)",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "kilometreAllowanceTaxable",
            "notSupportedError": null,
            "description": "Kilometrikorvaus, joka ei täytä verovapaudelle asetettuja vaatimuksia.\r\n\r\nVeronalaista kilometrikorvausta voidaan maksaa sekä palvelussuhteessa oleville että muille henkilöille. Kilometrikorvauksen veronalaisuudelle on useita perusteita. Verohallinto on määrittänyt työnantajan palvelussuhteessa maksamille verovapaille kilometrikorvauksille enimmäismäärän, jonka ylittävältä osalta kilometrikorvaus on veronalaista ansiotuloa. Henkilölle toimeksiantosuhteessa maksettava kilometrikorvaus on kokonaan veronalaista ansiotuloa.\r\n\r\n**Tulolajilla ilmoitetaan vain palvelussuhteessa oleville maksetut veronalaiset kilometrikorvaukset.** Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena ja lisätään kustannus työkorvauksen määrää (ks. Työkorvaus-tulolajin selite).",
            "taxAndSidecostsDescr": "Tulosta toimitetaan ennakonpidätys ja se on sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/expenses",
                "https://palkkaus.fi/usecases/2019/travelExpenses"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 210,
            "nameEn": "Meeting fee",
            "nameFi": "Kokouspalkkio",
            "nameSv": "Mötesarvode",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "meetingFee",
            "notSupportedError": null,
            "description": "Palkkio, joka maksetaan kokoukseen osallistumisesta.",
            "taxAndSidecostsDescr": "Kokouspalkkiosta toimitetaan ennakonpidätys.\r\n\r\nJos kokouspalkkion maksaminen perustuu palvelussuhteessa tehtyyn työhön, kokouspalkkiosta maksetaan sosiaalivakuutusmaksut. Jos kokouspalkkion saaja ei ole palvelussuhteessa, palkkion työeläkevakuutusmaksun ja työnantajan sairausvakuutusmaksun sekä työntekijän sairausvakuutuksen päivärahamaksun maksamisvelvollisuus vaihtelevat sovellettavan lain mukaan. Jos kokouspalkkion maksaminen ei perustu palvelusuhteessa tehtyyn työhön, työnantaja voi halutessaan vakuuttaa tulonsaajan vapaaehtoisesti eläkevakuutuksen osalta.\r\n\r\nOletuksena tulolajissa on, että se ei ole sosiaalivakuutusmaksujen alainen.\r\n\r\nJos kokouspalkkio on jonkun sosiaalivakuutusmaksun alainen, suorituksen maksaja ilmoittaa erillistä Vakuuttamistiedon tyyppi- tietoa käyttäen, että tulo on ilmoitetun vakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/board",
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 211,
            "nameEn": "Saturday pay",
            "nameFi": "Lauantaityökorvaus",
            "nameSv": "Ersättning för lördagsarbete",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "saturdayPay",
            "notSupportedError": null,
            "description": "Työaikakorvaus, joka maksetaan lauantaityöstä. Lauantaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
            "taxAndSidecostsDescr": "Korvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 212,
            "nameEn": "Extra work premium",
            "nameFi": "Lisätyökorvaus",
            "nameSv": "Ersättning för mertidsarbete",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "extraWorkPremium",
            "notSupportedError": null,
            "description": "Työaikakorvaus, joka maksetaan lisätyöstä.",
            "taxAndSidecostsDescr": "Korvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 213,
            "nameEn": "Holiday bonus",
            "nameFi": "Lomaraha",
            "nameSv": "Semesterpenning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "holidayBonus",
            "notSupportedError": null,
            "description": "Työsopimuksen tai työ- tai virkaehtosopimuksen perusteella vuosilomapalkan lisäksi maksettava suoritus, jonka suuruus määräytyy alalla sovellettavan työehtosopimuksen tai edellisenä lomanmääräytymisvuonna ansaittujen lomapäivien lukumäärän mukaan.\r\n\r\nVuosilomapalkan lisää koskee erittely- ja ilmoitusvelvollisuus riippumatta siitä, käytetäänkö kyseisestä erästä nimitystä lomaraha. Tällainen erittely tarvitaan esimerkiksi Kelaa ja työttömyyskassaa sekä työtapaturman korvauskäsittelyä varten.",
            "taxAndSidecostsDescr": "Lomarahasta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/holidays"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 214,
            "nameEn": "Lecture fee",
            "nameFi": "Luentopalkkio",
            "nameSv": "Föreläsningsarvode",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "lectureFee",
            "notSupportedError": null,
            "description": "Palkkio, joka maksetaan luennon pitämisestä.",
            "taxAndSidecostsDescr": "Luentopalkkiosta toimitetaan ennakonpidätys. \r\n\r\nJos luentopalkkion maksaminen perustuu palvelussuhteessa tehtyyn työhön, luentopalkkiosta maksetaan sosiaalivakuutusmaksut. \r\n\r\nJos luentopalkkion saaja ei ole palvelussuhteessa, palkkion työeläkevakuutusmaksun ja työnantajan sairausvakuutusmaksun sekä työntekijän sairausvakuutuksen päivärahamaksun maksamisvelvollisuus vaihtelevat sovellettavan lain mukaan.\r\n\r\nOletuksena tulolajissa on, että se ei ole sosiaalivakuutusmaksujen alainen.\r\n\r\nJos luentopalkkio on jonkun sosiaalivakuutusmaksun alainen, suorituksen maksaja ilmoittaa erillistä Vakuuttamistiedon tyyppi- tietoa käyttäen, että tulo on ilmoitetun vakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 215,
            "nameEn": "Compensation for acting in a position of trust",
            "nameFi": "Luottamustoimipalkkio",
            "nameSv": "Arvode för förtroendeuppdrag",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "compensationForTrustPosition",
            "notSupportedError": null,
            "description": "Palkkio, joka maksetaan muusta luottamustoimen hoitamisesta kuin kokouksiin osallistumisesta.",
            "taxAndSidecostsDescr": "Luottamustoimipalkkiosta toimitetaan ennakonpidätys.\r\n\r\nLuottamustoimipalkkiota maksettaessa palkkion saaja ei yleensä ole palvelussuhteessa palkkion maksajaan, eikä palkkiosta näissä tapauksissa makseta sosiaalivakuutusmaksuja.\r\n\r\nYksityisellä sektorilla luottamustoimipalkkiosta maksetaan kuitenkin sairausvakuutusmaksu. Lisäksi maksetaan eläkevakuutusmaksu, jos palkkion maksaja on ottanut luottamustoimen hoitajalle vapaaehtoisen työeläkevakuutuksen. Tietyistä julkisen sektorin luottamustoimipalkkioista maksetaan työeläkevakuutusmaksu.\r\n\r\nOletuksena tulolajissa on, että se on työnantajan sairausvakuutusmaksun alainen ja työntekijän sairausvakuutuksen päivärahamaksun alainen. Sitä vastoin oletuksena on, että tulo ei ole eläkevakuutusmaksun, työttömyysvakuutusmaksun eikä työtapaturma- ja ammattitautivakuutusmaksun alainen.\r\n\r\nJos luottamustoimipalkkio on esimerkiksi eläke- ja työttömyysvakuutusmaksun alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillisenä tietona näiden maksujen osalta. Samoin ilmoitetaan Vakuuttamistiedon tyyppi –tietoa käyttäen, jos luottamustoimipalkkio ei poikkeuksellisesti olisikaan sairausvakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/board",
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 216,
            "nameEn": "Other compensation",
            "nameFi": "Muu maksettu lisä",
            "nameSv": "Annat betalt tillägg",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "otherCompensation",
            "notSupportedError": null,
            "description": "Muiden maksettujen työaika- ja poikkeustilakorvausten yhteissumma. Näitä ovat mm. aattotyökorvaukset, arkipyhäkorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella maksettavat korvaukset ja lisät.",
            "taxAndSidecostsDescr": "Muista maksetuista lisistä toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation",
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 217,
            "nameEn": "Waiting time compensation",
            "nameFi": "Odotusajan korvaus",
            "nameSv": "Lön för väntetid",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "waitingTimeCompensation",
            "notSupportedError": null,
            "description": "Korvaus, johon työntekijällä on oikeus, jos palkanmaksu viivästyy työsuhteen päätyttyä. Ellei toisin sovita, palkka tulee maksaa palkanmaksukauden viimeisenä päivänä. Odotusajan korvausta voidaan maksaa enintään kuudelta päivältä.",
            "taxAndSidecostsDescr": "Odotusajan korvauksesta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 218,
            "nameEn": "Working condition compensation",
            "nameFi": "Olosuhdelisä",
            "nameSv": "Miljötillägg",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "workingConditionCompensation",
            "notSupportedError": null,
            "description": "Palkanlisä, joka maksetaan työn tekemispaikan olosuhteiden perusteella. Olosuhdelisiä ovat esimerkiksi työehtosopimuksen tai työsopimuksen perusteella maksettavat lisä, kuten vaahdotuslisä, kylmänalueenlisä ja saaristolisä.",
            "taxAndSidecostsDescr": "Olosuhdelisästä toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation",
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 219,
            "nameEn": "Partial pay during sick leave",
            "nameFi": "Sairausajan osapalkka",
            "nameSv": "Dellön för sjuktiden",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "partialPayDuringSickLeave",
            "notSupportedError": null,
            "description": "Lain tai työ- tai virkaehtosopimukseen mukaisesti alennettu sairausajan palkka. Sairausajan osapalkkana ilmoitetaan se osa tuloista, jonka henkilö on saanut sairaana ollessaan alennettuna.",
            "taxAndSidecostsDescr": "Sairausajan osapalkasta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.\r\n\r\nJos osapalkkana maksetaan sellaisia eriä, jotka eivät ole sosiaalivakuutusmaksujen alaisia (esimerkiksi vahingonkorvaus päättämistilanteessa), suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi –tietoa käyttäen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/sickness"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 220,
            "nameEn": "Commission",
            "nameFi": "Provisiopalkka",
            "nameSv": "Provisionslön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "commission",
            "notSupportedError": null,
            "description": "Pysyvän palkan osan muodostava jatkuvaluonteinen palkkiopalkka, jonka määrästä on sovittu työsopimuksessa.",
            "taxAndSidecostsDescr": "Provisiopalkasta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 221,
            "nameEn": "Sunday work compensation",
            "nameFi": "Sunnuntaityökorvaus",
            "nameSv": "Ersättning för söndagsarbete",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "sundayWorkCompensation",
            "notSupportedError": null,
            "description": "Sunnuntaityöstä maksettava työaikakorvaus. Sunnuntaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
            "taxAndSidecostsDescr": "Korvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 222,
            "nameEn": "Benefit arising from synthetic option",
            "nameFi": "Synteettinen optio",
            "nameSv": "Syntetisk option",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "benefitArisingFromSyntheticOption",
            "notSupportedError": null,
            "description": "Optio-oikeus, jonka perusteella tulonsaaja ei ole oikeutettu merkitsemään option kohde-etuutena olevia osakkeita option toteutushintaan, vaan hän saa toteutushinnan ja toteuttamishetken mukaisen osakkeen arvon erotuksen rahana (nettoarvon tilitys). Synteettinen optio ei oikeuta osakemerkintään.\r\n\r\nKäytännössä synteettisessä optiossa on kyse rahasuorituksesta, jonka suuruus määräytyy työnantajayhtiön osakekurssin kehityksen perusteella. Synteettinen optio -nimityksen sijaan järjestelyä saatetaan kutsua osakeperusteiseksi bonukseksi tai palkitsemisjärjestelmäksi.",
            "taxAndSidecostsDescr": "Optiona saatavasta edusta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/enterprise"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 223,
            "nameEn": "Performance bonus",
            "nameFi": "Tulospalkkio",
            "nameSv": "Resultatpremie",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "performanceBonus",
            "notSupportedError": null,
            "description": "Palkka, joka maksetaan organisaation ennalta sovitun tulostavoitteen täyttymisen tai ylittymisen perusteella.",
            "taxAndSidecostsDescr": "Tulospalkkiosta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 224,
            "nameEn": "Monetary compensation from a working time bank",
            "nameFi": "Työaikapankin rahakorvaus",
            "nameSv": "Penningersättning från arbetstidsbank",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "monetaryCompensationFromWorkingTimeBank",
            "notSupportedError": null,
            "description": "Palkka, joka maksetaan työaikapankkiin säästetyn ajan perusteella.\r\n\r\nTyöaikapankista voi yleensä nostaa palkkaa vain poikkeustapauksissa kuten palvelussuhteen päättyessä tai työaikapankkia purettaessa. Joissakin työaikapankeissa on mahdollista nostaa palkkaa tai\r\nvapaa-aikaa myös velaksi.",
            "taxAndSidecostsDescr": "Työaikapankin rahakorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/employmentTermination"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 225,
            "nameEn": "Compensation for accrued time off",
            "nameFi": "Työajantasauskorvaus",
            "nameSv": "Ersättning för arbetstidsutjämning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "compensationForAccruedTimeOff",
            "notSupportedError": null,
            "description": "Työaikakorvaus, joka maksetaan ansionvähentymisestä, joka aiheutuu työajan tasaamisesta keskimääräiseen viikkotyöaikaan. \r\n\r\nTyöajantasauskorvausta maksetaan säännölliseltä työajalta. Esimerkiksi niin sanottu pekkaskorvaus on työajantasauskorvaus.",
            "taxAndSidecostsDescr": "Korvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 226,
            "nameEn": "Share issue for employees",
            "nameFi": "Työsuhteeseen perustuva osakeanti",
            "nameSv": "Aktieemission som grundar sig på arbetsförhållande",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "shareIssueForEmployees",
            "notSupportedError": null,
            "description": "Yhtiön omalle henkilökunnalle suunnattu osakeanti. Etu työsuhteeseen perustuvasta oikeudesta merkitä työnantajayrityksen osakkeita on veronalaista ansiotuloa.\r\n\r\nJos etu on henkilökunnan enemmistön käytettävissä, tulolajilla ilmoitetaan ainoastaan 10 %:n alennuksen ylittävä osuus. Jos etu ei ole henkilökunnan enemmistön käytettävissä, saatava alennus on palkkaa koko määrältään. Tällöin suorituksen ilmoittamisessa käytetään palkan muita tulolajeja.\r\n\r\nMerkintäetu koskee myös yhteisön osuuksia.",
            "taxAndSidecostsDescr": "Työsuhteeseen perustuvasta osakeannista toimitetaan ennakonpidätys, mutta tulo ei ole sosiaalivakuutusmaksujen alainen.\r\n\r\nJos työsuhteeseen perustuva osakeanti on edellä mainittujen maksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi- tietoja käyttäen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 227,
            "nameEn": "Contract pay ",
            "nameFi": "Urakkapalkka ",
            "nameSv": "Ackordslön ",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "contractPay",
            "notSupportedError": null,
            "description": "Suorituspalkka, joka maksetaan ennalta sovitun työkokonaisuuden suorittamisen perusteella. Urakkapalkkaan sisältyy sekä kiinteä että muuttuva osa.",
            "taxAndSidecostsDescr": "Urakkapalkasta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/wages"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 229,
            "nameEn": "Damages in conjunction with termination and lay-off",
            "nameFi": "Vahingonkorvaus päättämis- ja lomautustilanteissa",
            "nameSv": "Skadestånd vid avslutande av ett anställningsförhållande eller permittering",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "damagesInConjunctionWithTermination",
            "notSupportedError": null,
            "description": "Korvaus, jota työnantaja on velvollinen maksamaan, kun työnantaja on päättänyt palvelussuhteen henkilön työsopimuslain, kunnallisesta viranhaltijasta annetun lain tai valtion virkamieslain vastaisesti. Kohdassa ilmoitetaan myös korvaus työsuhteen purkamisesta (TSL 8:1, 55/2001) sekä korvaus perusteettomasta lomauttamisesta.\r\n\r\nPalvelussuhteen perusteettoman päättämisen johdosta maksettava korvaus voi perustua tuomioistuimen tuomioon, tuomioistuimen vahvistamaan sovintoon, palvelussuhteen osapuolia edustavien järjestöjen väliseen neuvottelutulokseen tai palvelussuhteen osapuolten väliseen sopimukseen.\r\n\r\nVahingonkorvaus päättämis- tai lomautustilanteessa tai muu siihen verrattava korvaus ei ole veronalaista tuloa, ellei sitä ole saatu veronalaisen tulon sijaan tai korvaukseksi elatuksen vähentymisestä. Veronalaista tuloa eivät myöskään ole laissa tarkemmin määritellyt hyvitykset yhteistoimintavelvoitteiden laiminlyönneistä.\r\n\r\nTulolajilla ilmoitetaan vain veronalaiset määrät eli esimerkiksi ei yhteistoimintalain tai tasa-arvolain nojalla maksettuja verovapaita hyvityksi. Verovapaita vahingonkorvauksia ei ilmoiteta tulorekisteriin.\r\n\r\nTulolajilla ei ilmoiteta irtisanomisen yhteydessä maksettavia työsuoritukseen perustuvia korvauksia vaan ne ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
            "taxAndSidecostsDescr": "Veronalaisesta vahingonkorvauksesta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/employmentTermination"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 230,
            "nameEn": "Stand-by compensation",
            "nameFi": "Varallaolokorvaus",
            "nameSv": "Beredskapsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "standByCompensation",
            "notSupportedError": null,
            "description": "Palkka, joka maksetaan siitä, että työntekijä on tavoitettavissa ja työnantaja voi tarvittaessa kutsua hänet hoitamaan palvelussuhteeseen kuuluvia tehtäviä.",
            "taxAndSidecostsDescr": "Varallaolokorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 231,
            "nameEn": "Voluntary compensation in conjunction with termination of employment",
            "nameFi": "Vapaaehtoinen korvaus päättämistilanteessa",
            "nameSv": "Frivillig ersättning vid avslutande av ett anställningsförhållande",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "voluntaryCompensationInConjunctionWithTermination",
            "notSupportedError": null,
            "description": "Korvaus, jonka työnantaja maksaa vapaaehtoisesti irtisanomis- tai purkamistilanteessa tai silloin, kun palvelussuhde päätetään sopimalla.\r\n\r\nPäättämistilanteessa maksettavia vapaaehtoisia korvauksia ovat esimerkiksi toimitusjohtajan erokorvaus (niin sanottu kultainen kädenpuristus), erokorvaus, eroraha ja tukipaketti. Korvaus voidaan\r\nmyöntää myös muuna kuin rahana.",
            "taxAndSidecostsDescr": "Päättämiskorvauksesta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/employmentTermination"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 232,
            "nameEn": "Weekly rest compensation",
            "nameFi": "Viikkolepokorvaus",
            "nameSv": "Ersättning för veckovila",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "weeklyRestCompensation",
            "notSupportedError": null,
            "description": "Työaikakorvaus, jota maksetaan viikkolevon aikana tehdystä työstä.",
            "taxAndSidecostsDescr": "Viikkolepokorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 233,
            "nameEn": "Profit-sharing bonus",
            "nameFi": "Voittopalkkio",
            "nameSv": "Vinstarvode",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "profitSharingBonus",
            "notSupportedError": null,
            "description": "Työntekijöille ilman ennalta sovittua suunnitelmaa yhtiökokouksen päätöksellä jaettava palkkio, joka muodostuu yrityksen voiton perusteella.",
            "taxAndSidecostsDescr": "Muista kuin henkilöstörahastoon siirrettävistä voittopalkkiosta toimitetaan ennakonpidätys. Yrityksen henkilöstörahastoon maksamasta voittopalkkiosta tai henkilöstörahastosääntöjen mukaan työntekijän suoraan käteisenä nostamasta voittopalkkiosta ei makseta sosiaalivakuutusmaksuja.\r\n\r\nJos yrityksessä ei ole henkilöstörahastolain mukaista henkilöstörahastoa, voittopalkkiosta ei makseta sosiaalivakuutusmaksuja, jos voittopalkkio maksetaan työntekijälle yhtiökokouksen päätöksen perusteella voitonjakona tai käteisenä voittopalkkiona edellyttäen, että käteinen voittopalkkio maksetaan koko henkilöstölle eikä sillä pyritä korvaamaan työehtosopimuksen tai työsopimuksen edellyttämää palkkausjärjestelmää. Lisäksi edellytetään, että käteisen palkkion määräytymisperusteet ovat työntekijän eläkelain 70 §:n 3 momentin 10 kohdan ja henkilöstörahastolain 2 §:n 2 ja 3 kohdan mukaiset ja että yhtiön vapaan pääoman määrä on suurempi kuin yhtiökokouksessa päätettävän käteisen voittopalkkion ja osakkeenomistajille maksettavien osinkojen yhteismäärä. Laissa työttömyysetuuksien rahoituksesta on vastaavanlainen säännös, samoin työtapaturma- ja ammattitautilaissa.\r\n\r\nOletuksena tulolajissa on, ettei se ole sosiaalivakuutusmaksujen alainen. \r\n\r\nJos voittopalkkio on edellä mainittujen maksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi- tietoryhmää käyttäen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/renumeration"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 234,
            "nameEn": "Annual holiday compensation",
            "nameFi": "Vuosilomakorvaus",
            "nameSv": "Semesterersättning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "annualHolidayCompensation",
            "notSupportedError": null,
            "description": "Vuosiloman antamisen sijaan suoritettava rahallinen korvaus. Vuosilomakorvausta voidaan maksaa työsuhteen aikana tai työsuhteen päättyessä.\r\n\r\nTulolajilla ilmoitetaan myös **rakennusalan lomaraha**. Työehtosopimuksessa lomakorvaus, lomapalkka ja lomaraha on yhdistetty lomaraha-nimiseksi korvaukseksi, joka on 18,5 % lomanmääräytymisvuoden ansioista. Pääsääntöisesti rakennusalan lomakorvauksessa on 1/3 lomarahaa ja 2/3 lomapalkkaa tai lomakorvausta riippuen siitä, jatkuuko työsuhde vai päättyikö se.",
            "taxAndSidecostsDescr": "Vuosilomakorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/holidays"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 235,
            "nameEn": "Overtime compensation",
            "nameFi": "Ylityökorvaus",
            "nameSv": "Övertidsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "overtimeCompensation",
            "notSupportedError": null,
            "description": "Ylityöstä maksettava työaikakorvaus. Ylityönä pidetään työtä, joka tehdään työnantajan aloitteesta sovitun ylityöaikarajan ylittävänä aikana.",
            "taxAndSidecostsDescr": "Ylityökorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 236,
            "nameEn": "Night work allowance",
            "nameFi": "Yötyökorvaus",
            "nameSv": "Nattersättning",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "nightWorkAllowance",
            "notSupportedError": null,
            "description": "Työaikakorvaus, jota maksetaan yötyöstä.",
            "taxAndSidecostsDescr": "Yötyökorvauksesta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 237,
            "nameEn": "Night shift compensation",
            "nameFi": "Yövuorolisä",
            "nameSv": "Nattskiftstillägg",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "nightShiftCompensation",
            "notSupportedError": null,
            "description": "Vuorotyökorvaus, joka maksetaan yövuorosta.",
            "taxAndSidecostsDescr": "Ylivuorolisästä toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/workingTimeCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 238,
            "nameEn": "Other regular compensation",
            "nameFi": "Muu säännöllinen lisä",
            "nameSv": "Annat regelbundet tillägg ",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "otherRegularCompensation",
            "notSupportedError": null,
            "description": "Sellaisten työaika- ja poikkeustilakorvausten yhteissumma, joita maksetaan säännöllisesti ja jotka lasketaan säännöllisen työajan ansioon. Muita säännöllisiä lisiä ovat mm. luontoisedusta maksetut rahakorvaukset, aattotyökorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella säännöllisesti maksettavat korvaukset ja lisät, kuten kielilisä. Myös harvoin maksettava lisä katsotaan säännölliseksi, jos se kuuluu säännöllisen työajan ansioon.",
            "taxAndSidecostsDescr": "Säännöllisistä lisistä toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 239,
            "nameEn": "Compensation for unused compensatory leave",
            "nameFi": "Vastikevapaan korvaus",
            "nameSv": "Ersättning för kompensationsledighet ",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "compensationForUnusedCompensatoryLeave",
            "notSupportedError": null,
            "description": "Tietyillä aloilla käytössä oleva korvaus, jota maksetaan työsuhteen päättyessä pitämättömistä vapaapäivistä. Tietyillä aloilla tehty työ kerryttää tekijälleen vastikevapaata, jota työsuhteen kuluessa pidetään vapaana. Jos työntekijä ei ole pitänyt vapaapäiviä, kun työsuhde päättyy, niistä maksetaan korvausta.\r\n\r\nVuosiloman antamisen sijaan suoritettava rahallinen korvaus ilmoitetaan tulolajilla Vuosilomakorvaus (234).",
            "taxAndSidecostsDescr": "Vastikevapaan korvauksesta toimitetaan ennakon-pidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 301,
            "nameEn": "Accommodation benefit",
            "nameFi": "Asuntoetu",
            "nameSv": "Bostadsförmån",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "accommodationBenefit",
            "notSupportedError": null,
            "description": "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän käyttöön omistamansa tai hallitsemansa asunnon palvelussuhteeseen liittyvän vuokrasuhteen perusteella tai palkkaan sisältyvänä etuna.\r\n\r\nPalkkaus.fi-palvelussa asuntoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luontoisetu'-tulolajilla.",
            "taxAndSidecostsDescr": "Asuntoedusta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 302,
            "nameEn": "Interest benefit for a housing loan",
            "nameFi": "Asuntolainan korkoetu",
            "nameSv": "Ränteförmån för bostadslån",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "interestBenefitForHousingLoan",
            "notSupportedError": null,
            "description": "Veronalainen henkilökuntaetu, joka muodostuu silloin, kun työnantaja perii työntekijälle myöntämästään asuntolainasta vuotuista korkoa, joka on alempi kuin yleisesti markkinoilla käytössä oleva viitekorko.",
            "taxAndSidecostsDescr": "Asuntolainan korkoetu on ennakonpidätyksen alainen, mutta siitä ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/enterprise"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 303,
            "nameEn": "Meal allowance",
            "nameFi": "Ateriakorvaus",
            "nameSv": "Måltidsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "mealAllowance",
            "notSupportedError": null,
            "description": "Verovapaa kustannusten korvaus ateriointikuluista, jotka johtuvat työmatkasta.\r\n\r\nAteriakorvaus voidaan maksaa verovapaasti, jos työntekijällä ei ole mahdollisuutta aterioida ruokailutauon aikana tavanomaisella ruokailupaikallaan, eikä ruokailua ole järjestetty esimerkiksi kurssipakettiin kuuluvana työnantajan kustantamana ruokailuna. **Verovapaata ateriakorvausta ei voi maksaa samasta työmatkasta yhtä aikaa verovapaan päivärahan kanssa.**\r\n\r\nAteriakorvauksen perusteista ja verovapaista enimmäismääristä säädetään vuosittain annettavissa Verohallinnon päätöksissä verovapaista matkakustannusten korvauksista.\r\n\r\nKohdassa ilmoitetaan vain verovapaiden ateriakorvausten määrät. Jos maksettu korvaus ylittää Verohallinnon päätöksen enimmäismäärät tai maksetaan vastoin päätöksessä säädettyjä perusteita, suoritus ilmoitetaan sosiaalivakuutusmaksujen alaisena palkkana.\r\n\r\nAteriakorvaus voidaan ilmoittaa erikseen, mutta saatu korvaus on ilmoitettava vähintään yhteissummana Päiväraha -tulolajilla. Tällöin on lisäksi ilmoitettava tulolajiin liittyvän Korvauksen tyyppi-lisätiedon avulla, että Päiväraha-tulolajilla ilmoitettu korvaus sisältää ateriakorvauksen.\r\n\r\nJos ateriakorvaus ilmoitetaan erikseen, sitä ei lisätä enää Päiväraha -tulolajilla ilmoitettaviin määriin.",
            "taxAndSidecostsDescr": "Verovapaasta ateriakorvauksesta ei toimiteta ennakonpidätystä eikä makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/travelExpenses"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 304,
            "nameEn": "Car benefit",
            "nameFi": "Autoetu",
            "nameSv": "Bilförmån",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "carBenefit",
            "notSupportedError": null,
            "description": "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän tai tämän perheen käyttöön omistamansa tai hallitsemansa auton, jonka yksityiskäyttö on sallittu.\r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että työntekijä on maksanut luontoisedusta työnantajalle luontoisedun raha-arvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan. Peritty summa ilmoitetaan erikseen Autoedusta peritty korvaus- erittelylajilla.",
            "taxAndSidecostsDescr": "Autoedusta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/carBenefit",
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 308,
            "nameEn": "Compensation for membership of a governing body",
            "nameFi": "Hallintoelimen jäsenyydestä maksettu palkkio",
            "nameSv": "Arvode för medlemskap i förvaltningsorgan",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "compensationForGoverningBodyMembership",
            "notSupportedError": null,
            "description": "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella.\r\n\r\nJos henkilölle maksetaan palkkiota hallintoelimen jäsenyydestä, hänelle voidaan maksaa myös verovapaita matkakustannusten korvauksia työmatkasta aiheutuneiden kulujen perusteella, jos verovapaiden korvausten muut edellytykset täyttyvät.",
            "taxAndSidecostsDescr": "Jos hallintoelimen jäsenestä maksettavan palkkion saaja ei ole palvelussuhteessa, työeläkevakuutusmaksujen maksaminen palkkiosta on yksityisellä sektorilla vapaaehtoista. Julkisella alalla hallintoelimen jäsenyydestä maksettavista palkkioista suoritetaan tällöinkin sosiaalivakuutusmaksut. Jos palkkion perusteella ei henkilöön sovellettavien työeläkelakien mukaan ole tarvinnut maksaa työntekijän työeläkemaksua, suorituksesta ei makseta työnantajan sairausvakuutusmaksua eikä työntekijän päivärahamaksua, vaikka työeläkevakuutusmaksu olisikin maksettu vapaaehtoisesti. Palkkio on työtapaturma- ja ammattitautivakuutusmaksun alainen, jos se maksetaan palvelussuhteessa olevalle.\r\n\r\nHallintoelimen jäsenyydestä maksettavasta palkkiosta toimitetaan ennakonpidätys.\r\n\r\nOletuksena tulolajissa on, että hallintoelimen jäsenyydestä maksettu palkkio ei ole sosiaalivakuutusmaksujen alainen. Olennaista maksuvelvollisuuden kannalta muilla kuin julkisella alalla on se, onko palkkio maksettu työ- tai muussa palvelussuhteessa.\r\n\r\nJos palkkionsaaja on palvelusuhteessa ja palkkio on jonkun sosiaalivakuutusmaksun alainen, suorituksen maksaja ilmoittaa erillistä Vakuuttamistiedon tyyppitietoa käyttäen, että tulo on ilmoitetun vakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/board",
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 309,
            "nameEn": "Share of reserve and surplus drawn from personnel fund (taxable 80%)",
            "nameFi": "Henkilöstörahastosta nostettu rahasto-osuus ja ylijäämä (veronalainen 80%)",
            "nameSv": "Fondandel och överskott som tagits ut ur en personalfond (skattepliktig till 80 %)",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "shareOfReserveDrawnFromPersonnelFund",
            "notSupportedError": null,
            "description": "Rahasto-osuus, joka maksetaan työntekijälle henkilöstörahastosta. Henkilörahastoon osoitetut varat jaetaan rahasto-osuutena nostettaviin eriin ja käteisenä nostettaviin eriin.\r\n\r\nHenkilöstörahastosta nostetusta rahasto-osuudesta 20 % on verovapaata ja 80 %:n veronalaisesta ansiotulo-osuudesta toimitetaan ennakonpidätys. Henkilöstörahastosta nostettua ylijäämää kohdellaan verotuksessa samalla tavalla kuin henkilöstörahastosta nostettua rahasto-osuutta. Ks. Verohallinnon ohje Henkilöstörahastosta saatavien tulojen verotus (A69/200/2014).\r\n\r\nTulolajilla ilmoitetaan vain veronalainen osuus.",
            "taxAndSidecostsDescr": "Nostetusta rahasto-osuudesta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja. Sitä vastoin henkilöstörahastoon maksamisen sijaan suoraan henkilölle käteisenä nostetusta rahastoosuudesta, jota ei makseta rahastoon vaan työnantaja maksaa sen suoraan jäsenelle, maksetaan sosiaalivakuutusmaksut. Kohdassa ilmoitetaan vain rahastosta rahasto-osuutena nostetut erät. Jos suoritus on nostettu käteisenä, se ilmoitetaan tavallisena palkkana.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "benefits"
        },
        {
            "code": 310,
            "nameEn": "Monetary gift for employees",
            "nameFi": "Henkilöstölle annettu rahalahja",
            "nameSv": "Penninggåva till personalen",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "monetaryGiftForEmployees",
            "notSupportedError": null,
            "description": "Työnantajalta rahana tai siihen verrattavana suorituksena saatu lahja.",
            "taxAndSidecostsDescr": "Rahalahja katsotaan veronalaiseksi tuloksi, josta on toimitettava ennakonpidätys, mutta josta ei makseta sosiaalivakuutusmaksuja, jos rahalahja on annettu työntekijän merkkipäivän johdosta.\r\n\r\nJos rahalahja on annettu muutoin kuin työntekijän omana merkkipäivänä, maksetaan suorituksesta sosiaalivakuutusmaksut.\r\n\r\nOletuksena tulolajissa on, että tulo on sosiaalivakuutusmaksujen alainen. \r\n\r\nJos annettu rahalahja ei ole sosiaalivakuutusmaksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erikseen Vakuuttamistiedon tyyppi- tietoryhmää käyttäen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 311,
            "nameEn": "Kilometre allowance (tax-exempt)",
            "nameFi": "Kilometrikorvaus (verovapaa)",
            "nameSv": "Kilometerersättning (skattefri)",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "kilometreAllowanceTaxExempt",
            "notSupportedError": null,
            "description": "Henkilön oman tai hänen hallinnassaan olevan kulkuneuvon käytöstä suoritettava kilometrikorvaus, joka täyttää verovapaudelle asetetut vaatimukset.\r\n\r\nVerovapaata kilometrikorvausta voidaan maksaa palvelussuhteessa oleville ja henkilöille, jotka eivät ole palvelussuhteessa, mutta joille maksetaan verolainsäädännön mukaista palkkaa, esimerkiksi kokouspalkkion yhteydessä.\r\n\r\nJos kilometrikorvaus on maksettu vastoin Verohallinnon päätöksen mukaisia aika- ja kilometrirajoja, ne eivät ole miltään osin verovapaita.\r\n\r\nJos kilokorvaukset on maksettu Verohallinnon päätöksen mukaan, mutta suurempina kuin päätöksessä on määrätty verovapaaksi, osa korvauksista on veronalaista ja osa verovapaata. Veronalaiset kilometrikorvaukset ilmoitetaan erikseen omalla tulolajillaan tai palkan tulolajeilla.\r\n\r\nLiikenteenharjoittajan antaman tositteen (esimerkiksi junalipun) perusteella tulonsaajalle maksetut matkustamiskustannusten korvaukset ovat verovapaita, eikä niitä ilmoiteta.\r\n\r\n**Rakennusalalla maksettavat asunnon ja työpaikan väliset matkakustannusten korvaukset**\r\n\r\nRakennusalan työehtosopimuksen mukaan päivittäisistä asunnon ja erityisen työntekemispaikan välisistä matkoista maksetaan matkakustannusten korvausta (”tossurahaa”) erityisen taulukon mukaan eikä Verohallinnon päätöksen mukaisina kilometrikorvauksina. Työehtosopimuksen mukainen korvaus on kuitenkin pienempi kuin Verohallinnon päätöksen mukaiset kilometrikorvaukset, joten korvaus on verovapaata. \r\n\r\nRakennusalan tossurahan yhteydessä ei tarvitse ilmoittaa kilometrimäärää.",
            "taxAndSidecostsDescr": "Kohdassa ilmoitetaan vain verovapaiden kilometrikorvausten määrät. Summasta ei toimiteta ennakonpidätystä eikä makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/travelExpenses"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 312,
            "nameEn": "Treatment fee for municipal veterinarian",
            "nameFi": "Kunnaneläinlääkärin toimenpidepalkkio",
            "nameSv": "Åtgärdsarvode för kommunal veterinär",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "treatmentFeeForMunicipalVeterinarian",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 313,
            "nameEn": "Compensation for use, earned income",
            "nameFi": "Käyttökorvaus ansiotuloa",
            "nameSv": "Bruksersättning som förvärvsinkomst",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "compensationForUseEarnedIncome",
            "notSupportedError": null,
            "description": "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
            "taxAndSidecostsDescr": "Käyttökorvausta voidaan maksaa ennakkoperintärekisteriin kuuluvalle tai kuulumattomalle korvauksen saajalle. Käyttökorvaus on veronalaista ansiotuloa ja siitä toimitetaan ennakonpidätys, jos korvauksen saaja ei kuulu ennakkoperintärekisteriin.\r\n\r\nTulolajilla on ilmoitettava kaikki käyttökorvaukset, joista on toimitettu ennakonpidätys. Luonnollisille henkilöille maksetut käyttökorvaukset on ilmoitettava, vaikka niistä ei ole toimitettu ennakonpidätystä. Myös ennakkoperintärekisteriin merkitylle luonnolliselle henkilölle maksetut käyttökorvaukset on ilmoitettava.\r\n\r\nTulolajilla ilmoitetaan myös yhtymän, yhteisön tai yhteenliittymän saamat käyttökorvaukset (mutta niitä ei voi ilmoittaa Palkkaus.fi-palvelussa).\r\n\r\nTulolajilla ilmoitetaan lisäksi rajoitetusti verovelvolliselle maksetut rojalti. Jos rojalti eli käyttökorvaus on maksettu rajoitetusti verovelvolliselle luonnolliselle henkilölle, tiedot on ilmoitettava tulorekisteriin, vaikka tulonsaaja olisikin ennakkoperintärekisterissä eikä suorituksesta olisi peritty lähdeveroa.\r\n\r\nKäyttökorvauksesta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 314,
            "nameEn": "Compensation for use, capital income",
            "nameFi": "Käyttökorvaus pääomatuloa",
            "nameSv": "Bruksersättning som kapitalinkomst",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "compensationForUseCapitalIncome",
            "notSupportedError": null,
            "description": "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.\r\n\r\nJos tekijänoikeus on siirtynyt perintönä tai testamentilla tai se on hankittu vastiketta vastaan, sen käyttämisestä saatu korvaus on henkilölle Suomesta saatuna tulona ennakonpidätyksenalaista pääomatuloa ennakkoperintärekisterimerkinnästä huolimatta.",
            "taxAndSidecostsDescr": "Tulolajilla ilmoitetaan kaikki käyttökorvaukset, joista on toimitettu ennakonpidätys. Luonnollisille henkilöille maksetut käyttökorvaukset on ilmoitettava, vaikka niistä ei ole toimitettu ennakonpidätystä. Myös ennakkoperintärekisteriin merkitylle luonnolliselle henkilölle maksetut käyttökorvaukset on ilmoitettava.\r\n\r\nKäyttökorvauksesta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 315,
            "nameEn": "Other taxable benefit for employees",
            "nameFi": "Muu henkilöstölle suunnattu veronalainen etu",
            "nameSv": "Annan skattepliktig förmån till personalen",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "otherTaxableBenefitForEmployees",
            "notSupportedError": null,
            "description": "Työnantajan henkilökunnalle kollektiivisesti myöntämä muu kuin rahana annettava veronalainen etu.\r\n\r\nVeronalaisena etuna pidetään esimerkiksi vuosittain laissa säädetyn enimmäismäärän 400 euroa ylittävää omaehtoiseen liikuntaan ja kulttuuriin annettua etua.",
            "taxAndSidecostsDescr": "Jos etu on annettu kollektiivisesti koko henkilökunnalle, etu ei ole sosiaalivakuutusmaksujen alainen.\r\n\r\nOletuksena tulolajissa on, ettei etu ole sosiaalivakuutusmaksujen alainen.\r\n\r\nJos etu on annettu vain osalle henkilökuntaa, etu on kokonaisuudessaan veronalainen ja etu on sosiaalivakuutusmaksujen alainen. Tällöin suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi- tietoryhmää käyttäen.\r\n\r\nTulolajia käytetään myös tilanteessa, jossa työnantaja maksaa työntekijälle päivärahaa Verohallinnon päätöstä lievemmin perustein. Tällöin vakuuttamistiedon tyyppi-tiedon avulla voidaan ilmoittaa, jos työnantaja maksaa työntekijälle päivärahaa työehtosopimuksen perusteella Verohallinnon päätöstä lievemmin perustein, jolloin tulo ei ole työeläke-, työttömyyseikä työtapaturma- ja ammattitautivakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 316,
            "nameEn": "Other taxable income deemed earned income",
            "nameFi": "Muu veronalainen ansiotulo",
            "nameSv": "Övriga skattepliktiga förvärvsinkomster",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "otherTaxableIncomeDeemedEarnedIncome",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan muiden veronalaisten ansiotuloa olevien suoritusten yhteissumma.\r\n\r\nTulolajilla ilmoitetaan veronalaiset satunnaiset palkkiot ja palkinnot silloin, kun saaja ei ole työsuhteessa antajaan eikä suoritus ole työkorvausta. Tällaisia ovat esimerkiksi työsuhteeseen perustumaton kilpailupalkinto, asiakasyrityksen henkilökuntaan kuuluvalle tai muutoin asiakassuhteen perusteella annettu etu (esimerkiksi matka, lahjakortti tai esine), löytöpalkkio, pankin maksama niin sanottu tarkkaavaisuuspalkkio, taidekilpailun palkinto sekä kunnan asukkailleen maksama matkakorvaus.",
            "taxAndSidecostsDescr": "Muu veronalainen ansiotulo on ennakonpidätyksen alainen, mutta siitä ei makseta sosiaalivakuutusmaksuja. Merkitystä ilmoittamisen kannalta ei ole sillä, onko tulosta voitu toimittaa ennakonpidätys vai ei.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 317,
            "nameEn": "Other fringe benefit",
            "nameFi": "Muu  luontoisetu",
            "nameSv": "Övriga naturaförmåner",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "otherFringeBenefit",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan työnantajan muuna kuin rahana antamat verotettavat edut, esimeriksi autotalli-, moottoripyörä- ja vene-etu, palkaksi katsotut henki- ja eläkevakuutusmaksut, yksilöimättömästä lahjakortista syntyvä etu ja merimiespalveluksessa olevan henkilön merimiehenä saama luontoisetu.\r\n\r\nPalkkaus.fi-palvelussa asunto-, auto-, puhelin- ja ravintoetu sekä työsuhdematkalipun erät tulee aina ilmoittaa omalla tulolajillaan. Niitä ei lisätä 'Muu luontoisetu' kohtaan.\r\n\r\nJos tulonsaajalta peritään korvaus edusta, luontoisedun arvo ilmoitetaan täysimääräisenä tulolajilla ennen perityn korvauksen vähentämistä.\r\n\r\nJos verotusarvoa ei ole erikseen vahvistettu Verohallinnon antamassa päätöksessä, luontoisedun arvo on sen käypä arvo. Etu on ilmoitettava, vaikka työntekijälle ei maksettaisi lainkaan rahapalkkaa tai rahapalkka ei riittäisi ennakonpidätyksen täysimääräiseen toimittamiseen.",
            "taxAndSidecostsDescr": "Luontoisedut ovat ennakonpidätyksen alaisia ja niistä maksetaan sosiaalivakuutusmaksut. Jos edusta ei ole voitu toimittaa ennakonpidätystä rahapalkan vähyyden vuoksi, ennakonpidätystä ei ilmoiteta, jos maksaja ei ole todella perinyt veroa.\r\n\r\nJos luontoisetu ei poikkeuksellisesti olisikaan edellä mainittujen maksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi -tietoryhmää. Tällainen tilanne on esimerkiksi silloin, kun työnantaja on maksanut työntekijän puolesta tämän eläke- tai työttömyysvakuutusmaksut, jolloin tulonsaajalle muodostuu luontoisetu työantajan maksamien maksujen osalta. Tuloa ei kuitenkaan pidetä eläke- eikä työttömyysvakuutusmaksun perusteena olevana työtulona. Etu on tällöinkin sairausvakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 319,
            "nameEn": "Kinship carer's fee",
            "nameFi": "Omaishoitajan palkkio",
            "nameSv": "Arvode för närståendevårdare",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "kinshipCarersFee",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 320,
            "nameEn": "Stock options and grants",
            "nameFi": "Osakepalkkio",
            "nameSv": "Aktiepremie",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "stockOptions",
            "notSupportedError": null,
            "description": "Palkkio, joka annetaan työnantajayrityksen osakkeina tai sovitun palkkion sijaan rahasummana, joka vastaa osakkeiden arvoa.",
            "taxAndSidecostsDescr": "Osakepalkkiosta toimitetaan ennakonpidätys, mutta ei makseta sosiaalivakuutusmaksuja, jos työntekijällä on kannustinjärjestelmän perusteella mahdollisuus saada työnantajayrityksen (tai sen kanssa samaan konserniin kuuluvan yhtiön tai muuhun vastaavaan taloudelliseen yhteenliittymään kuuluvan yhtiön) arvopaperipörssissä noteerattuja osakkeita ja edun arvo riippuu kyseisten osakkeiden arvon kehityksestä palkkion lupaamisen jälkeisenä, yli vuoden mittaisena aikana. Muussa kuin edellä mainituissa tapauksissa osakepalkkio on sosiaalivakuutusmaksujen alaista tuloa.\r\n\r\nOletuksena tulolajissa on, että se on eläkevakuutusmaksun alainen, mutta ei muiden sosiaalivakuutusmaksujen alainen.\r\n\r\nJos osakepalkkio on sosiaalivakuutusmaksujen alainen tai ei ole eläkevakuutusmaksun alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi- tietoa käyttäen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 321,
            "nameEn": "Wages paid by substitute payer: the employer pays for the employer's social insurance contributions (earnings-related pension, health, unemployment, and accident and occupational disease insurance)",
            "nameFi": "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sosiaalivakuutusmaksut (työeläke-, sairaus-, työttömyys- sekä työtapaturma- ja ammattitautivakuutus)",
            "nameSv": "Lön som betalats av ställföreträdande betalare för vilken arbetsgivaren betalar arbetsgivarens socialförsäkringsavgifter (arbetspensions-, sjuk-, arbetslöshets- samt arbetsolycksfalls- och yrkessjukdomsförsäkringsavgift)",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaidBySubstitutePayerIncludingSocialInsurance",
            "notSupportedError": null,
            "description": "Varsinainen työnantaja ilmoittaa sellaiset sosiaalivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen.\r\n\r\nHUOM: Kotitalous ilmoittaa Yksityisen hoidon tuen tällä tulolajilla!\r\n\r\nVarsinaisen työnantajan antaman tiedon perusteella tuloa ei lueta tulonsaajalle verotuksessa tuloksi, mutta tieto tarvitaan tulonsaajakohtaisesti, koska henkilölle karttuu eläkettä maksetun tulon perusteella.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sosiaalivakuutusmaksut.\r\n\r\nVaikka sijaismaksaja on maksanut palkan tulonsaajille, varsinainen työnantaja on vastuussa työnantajan sosiaalivakuutusmaksujen maksamisesta. Tämän vuoksi varsinaisen työnantajan on ilmoitettava sijaismaksajan maksamat palkat tulonsaajakohtaisesti käyttäen erityisesti sijaismaksajatilanteita varten tarkoitettuja niin sanottuja infotulolajeja.\r\n\r\nSijaismaksaja ilmoittaa tiedot omalta osaltaan tulorekisteriin käyttäen muita tulolajeja sekä ohjaustietoa ”Sijaismaksaja – Kyllä”. Sijaismaksajan tiedon perusteella tulo verotetaan tulonsaajalla. Työntekijältä perittävät maksut peritään ja maksetaan sijaismaksajan maksaman palkan perusteella. Sijaismaksaja ei pääsääntöisesti kuitenkaan maksa työnantajan sosiaalivakuutusmaksuja, vaan niiden maksamisesta on vastuussa varsinainen työnantaja.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat. Ilmoittamista on kuvattu tarkemmin asiakasohjeessa\r\nTietojen ilmoittaminen tulorekisteriin: sijaismaksajan maksamat suoritukset.",
            "taxAndSidecostsDescr": "Tulolaji on työnantajan sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/substitutePayer"
            ],
            "calcGrouping": "otherNoPayment"
        },
        {
            "code": 322,
            "nameEn": "Wages paid by substitute payer: employer pays for employer's earnings-related pension insurance contribution",
            "nameFi": "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työeläkevakuutusmaksun",
            "nameSv": "Lön som ställföreträdande betalare betalat och för vilken arbetsgivaren betalar arbetsgivarens arbetspensionsförsäkringsavgift",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance",
            "notSupportedError": null,
            "description": "Varsinainen työnantaja ilmoittaa sellaiset eläkevakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa eläkevakuutusmaksusta. \r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan eläkevakuutusmaksun. Sijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
            "taxAndSidecostsDescr": "Tulolajia käytetään vain silloin, jos maksettu tulo on ollut vain eläkevakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/substitutePayer"
            ],
            "calcGrouping": "otherNoPayment"
        },
        {
            "code": 323,
            "nameEn": "Wages paid by substitute payer: employer pays for employer's unemployment insurance contribution",
            "nameFi": "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työttömyysvakuutusmaksun",
            "nameSv": "Lön som ställföreträdande betalare betalat och för vilken arbetsgivaren betalar arbetsgivarens arbetslöshetsförsäkringsavgift",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": true,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaidBySubstitutePayerIncludingUnemploymentInsurance",
            "notSupportedError": null,
            "description": "Varsinainen työnantaja ilmoittaa sellaiset työttömyysvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työttömyysvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan työttömyysvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
            "taxAndSidecostsDescr": "Tulolajia käytetään vain silloin, jos maksettu tulo on ollut vain työttömyysvakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/substitutePayer"
            ],
            "calcGrouping": "otherNoPayment"
        },
        {
            "code": 324,
            "nameEn": "Wages paid by substitute payer, employer pays for accident and occupational disease insurance contribution",
            "nameFi": "Sijaismaksajan maksama palkka, josta työnantaja maksaa työtapaturma- ja ammattitautivakuutusmaksun",
            "nameSv": "Lön som ställföreträdande betalare betalat och för vilken arbetsgivaren betalar arbetsolycksfalls- och yrkessjukdomsförsäkringsavgiften",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": true,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance",
            "notSupportedError": null,
            "description": "Varsinainen työnantaja ilmoittaa sellaiset työtapaturma- ja ammattitautivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työtapaturma- ja ammattitautivakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työtapaturma- ja ammattitautivakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
            "taxAndSidecostsDescr": "Tulolajia käytetään vain silloin, jos maksettu tulo on ollut vain työtapaturma- ja ammattitautivakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/substitutePayer"
            ],
            "calcGrouping": "otherNoPayment"
        },
        {
            "code": 325,
            "nameEn": "Wages paid by substitute payer, employer pays for employer's health insurance contribution",
            "nameFi": "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sairausvakuutusmaksun",
            "nameSv": "Lön som ställföreträdande betalare betalat och för vilken arbetsgivaren betalar arbetsgivarens sjukförsäkringsavgift",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaidBySubstitutePayerIncludingHealthInsurance",
            "notSupportedError": null,
            "description": "Varsinainen työnantaja ilmoittaa sellaiset sairausvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työnantajan sairausvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sairausvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
            "taxAndSidecostsDescr": "Tulolajia käytetään vain silloin, jos maksettu tulo on ollut vain sairausvakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/substitutePayer"
            ],
            "calcGrouping": "otherNoPayment"
        },
        {
            "code": 326,
            "nameEn": "Compensation for employee invention",
            "nameFi": "Palkkio työsuhdekeksinnöstä",
            "nameSv": "Arvode för arbetstagares uppfinning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "compensationForEmployeeInvention",
            "notSupportedError": null,
            "description": "Palkkio, joka maksetaan työsuhdekeksinnöstä.",
            "taxAndSidecostsDescr": "Jos palkkion saajan työsuhteeseen kuuluviin tehtäviin ei kuulu keksiminen, palkkiota työsuhdekeksinnöstä käsitellään käyttökorvauksena eli palkkiosta ei makseta sosiaalivakuutusmaksuja. Tällöin työsuhdekeksinnöstä maksettavasta palkkiosta toimitetaan ennakonpidätys, ellei palkkion saaja kuulu ennakkoperintärekisteriin.\r\n\r\nJos tulonsaajan työsopimuksessa sovittuihin työtehtäviin kuuluu keksiminen, palkkiota käsitellään palkkana, eli palkkiosta maksetaan ennakonpidätyksen toimittamisen lisäksi sosiaalivakuutusmaksut.\r\n\r\nOletuksena tulolajissa on, että se ei ole sosiaalivakuutusmaksujen alainen. \r\n\r\nJos palkkio työsuhdekeksinnöstä on jonkun sosiaalivakuutusmaksun alainen, suorituksen maksaja ilmoittaa erillistä Vakuuttamistiedon tyyppi- tietoa käyttäen, että tulo on ilmoitetun vakuutusmaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 327,
            "nameEn": "Reimbursement of private caretaker's expenses",
            "nameFi": "Perhehoitajan kustannusten korvaus",
            "nameSv": "Kostnadsersättning för familjevårdare",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "reimbursementOfPrivateCaretakersExpenses",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 328,
            "nameEn": "Private caretaker's fee",
            "nameFi": "Perhehoitajan palkkio",
            "nameSv": "Arvode för familjevårdare",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "privateCaretakersFee",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 329,
            "nameEn": "Reimbursement of family day care provider's expenses",
            "nameFi": "Perhepäivähoitajan kustannusten korvaus",
            "nameSv": "Kostnadsersättning för familjedagvårdare",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "reimbursementOfFamilyDayCareProvidersExpenses",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 330,
            "nameEn": "Telephone benefit",
            "nameFi": "Puhelinetu",
            "nameSv": "Telefonförmån",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "telephoneBenefit",
            "notSupportedError": null,
            "description": "Luontoisetu, joka syntyy, kun työnantaja maksaa työntekijän puhelinliittymän kulut vapaa-ajalla tapahtuvan yksityiskäytön osalta.\r\n\r\nPalkkaus.fi-palvelussa puhelinetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.",
            "taxAndSidecostsDescr": "Puhelinedusta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 331,
            "nameEn": "Daily allowance",
            "nameFi": "Päiväraha",
            "nameSv": "Dagtraktamente ",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "dailyAllowance",
            "notSupportedError": null,
            "description": "Päiväraha on verovapaa kustannusten korvaus kohtuullisesta ruokailu- ja muiden elantokustannusten lisäyksestä, joka korvauksen saajalle aiheutuu työmatkasta.\r\n\r\nKohtaan merkitään verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Päiväraha-tulolajilla ilmoitettu tulo voi olla:\r\n\r\n- kotimaan kokopäiväraha,\r\n- kotimaan osapäiväraha,\r\n- ulkomaan päiväraha tai \r\n- ulkomaan työskentelyyn liittyvät verovapaat kustannusten korvaukset.\r\n\r\nPalkkaus.fi-palvelussa Ateriakorvaukset ilmoitetaan aina erillisellä Ateriakorvaus-tulolajia.\r\n\r\nJos päivärahan tyypiksi on merkitty ulkomaan työskentelyyn liittyvät kustannusten korvaukset, kohtaan merkitään seuraavat tulonsaajan saamat verovapaat edut, jotka liittyvät ulkomaan työskentelyyn:\r\n\r\n- verovelvollisen ja hänen perheenjäsenensä muutto- ja matkakustannukset työntekovaltioon ja takaisin\r\n- työnantajan ulkomailla kustantama tavanomainen yksityinen palveluhenkilöstö\r\n- työnantajan kustantama lasten perus- ja lukiotason koulutus.\r\n\r\nTulolajilla ei ilmoiteta yömatkarahaa eikä rajanylitysrahaa. Näitä eriä ei ilmoiteta tulorekisteriin.\r\n\r\nKohtaan merkitään vain verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Verohallinto vahvistaa vuosittain verovapaat määrät antamallaan päätöksellä verovapaista matkakustannusten korvauksista.\r\n\r\nJos päivärahat tai muut kohdassa mainitut korvaukset on maksettu Verohallinnon päätöstä lievemmin perustein, korvaukset ilmoitetaan tulolajilla Palkka yhteissumma tai Muu maksettu lisä, jolloin ne ovat myös sosiaalivakuutusmaksujen alaisia.\r\n\r\nJos lievemmin perustein maksetuista korvauksista on sovittu työehtosopimuksella, korvaus ei ole eläke-, työttömyys- eikä työtapaturma- ja ammattitautivakuutusmaksun alainen. Näissä tilanteissa suorituksen\r\nmaksaja ilmoittaa tulon Palkka yhteissumma-tulolajilla tai Muu henkilöstölle suunnattu veronalainen etu, ja samalla ilmoittaa, mikä osuus suorituksesta on eläkevakuutusmaksun, työttömyysvakuutusmaksun sekä työtapaturma- ja ammattitautivakuutusmaksun alainen.\r\n\r\nTulolajin yhteydessä on annettava lisätiedot, mitä eri päivärahoja yhteissumma sisältää.",
            "taxAndSidecostsDescr": "Päivärahat eivät ole sosiaalivakuutusmaksun alaisia.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/travelExpenses"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 332,
            "nameEn": "Capital income payment",
            "nameFi": "Pääomatuloa oleva suoritus",
            "nameSv": "Prestation i form av kapitalinkomst",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "capitalIncomePayment",
            "notSupportedError": null,
            "description": "Suoritus, joka on saajalleen pääomatuloa. Tällainen on esimerkiksi työntekijälle tai osakkaalle maksettu korko, kuten palkkasaatavan korko tai pääomatuloa oleva takausprovisio.",
            "taxAndSidecostsDescr": "Suoritukset ovat ennakonpidätyksen alaisia, mutta eivät sosiaalivakuutusmaksun alaisia. \r\nViivästyskorko on vapautettu Verohallinnon päätöksellä (A136/200/2016) ennakonpidätyksen toimittamisesta.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 334,
            "nameEn": "Meal benefit",
            "nameFi": "Ravintoetu",
            "nameSv": "Kostförmån",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "mealBenefit",
            "notSupportedError": null,
            "description": "Luontoisetu, joka syntyy, kun työnantaja tarjoaa työntekijälleen aterian. Työnantaja voi tukea ruokailukustannuksia ruokailulipukkeella tai muulla kohdennetulla maksuvälineellä taikka järjestämällä työpaikkaruokailun.\r\n\r\nRavintoedun verotusarvo määräytyy Verohallinnon päätöksen mukaisena vuosittain vahvistettavana arvona.\r\n\r\nPalkkaus.fi-palvelussa ravintoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.\r\n\r\nJos työnantaja perii työntekijältä ravintoedusta sen verotusarvoa vastaavan korvauksen, tulorekisteriin ilmoitetaan tieto Ravintoedusta peritty korvaus vastaa verotusarvoa: Kyllä.",
            "taxAndSidecostsDescr": "Ravintoedusta toimitetaan ennakonpidätys ja maksetaan sosiaalivakuutusmaksut.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 335,
            "nameEn": "Reimbursement of costs, paid to conciliator",
            "nameFi": "Sovittelijan kulukorvaus",
            "nameSv": "Kostnadsersättning för medlare",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "reimbursementOfCostsPaidToConciliator",
            "notSupportedError": null,
            "description": "Veronalainen kustannusten korvaus, joka maksetaan vapaaehtoiselle sovittelijalle sovittelusta aiheutuneista kustannuksista.\r\n\r\nSovittelijan kulukorvaus on veronalaista ansiotuloa, joka otetaan huomioon suorituksen saajan verotuksessa saman suuruisena vähennyksenä ilman suorituksen saajan esittämää eri selvitystä.",
            "taxAndSidecostsDescr": "Kulukorvauksista ei toimiteta ennakonpidätystä eikä makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/expenses"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 336,
            "nameEn": "Non-wage compensation for work",
            "nameFi": "Työkorvaus",
            "nameSv": "Arbetsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": 1,
            "transactionCode": "nonWageCompensationForWork",
            "notSupportedError": null,
            "description": "Muussa kuin palvelussuhteessa tehdystä työstä maksettava tai maksettavaksi sovittu vastike, josta tulee toimittaa ennakonpidätys, jos saaja ei ole ennakkoperintärekisterissä, mutta josta ei pääsääntöisesti makseta sosiaalivakuutusmaksuja.\r\n\r\nTyökorvausta voidaan maksaa ennakkoperintärekisteriin kuuluvalle tai kuulumattomalle työn suorittajalle. Työkorvaus on veronalaista ansiotuloa ja siitä toimitetaan ennakonpidätys, jos saaja ei kuulu ennakkoperintärekisteriin. Työkorvausta on esimerkiksi tukiperheelle maksettu korvaus.\r\n\r\nTyökorvauksesta on annettava ilmoitus, jos tulonsaaja ei kuulu ennakkoperintärekisteriin tai, jos maksajana on julkisten alojen eläkelakien piiriin (JuEL) kuuluva suorituksen maksaja ja tulonsaaja on luonnollinen henkilö, joka kuuluu ennakkoperintärekisteriin, mutta hänellä ei ole yrittäjän eläkelakien mukaista vakuutusta. Koska ilmoitus tehdään aina, kun tulonsaaja ei kuulu ennakkoperintärekisteriin, saajana voi olla myös muu kuin luonnollinen henkilö, kuten yhdistys, osakeyhtiö, avoin yhtiö tai kommandiittiyhtiö. Näitä maksuja ei kuitenkaan voi tehdä Palkkaus.fi-palvelun kautta.\r\n\r\nRajoitetusti verovelvolliselle yritykselle maksetut työkorvaukset on ilmoitettava, jos maksaja on perinyt yritykseltä lähdeveron. Rajoitetusti verovelvolliselle luonnolliselle henkilölle maksettu työkorvaus on ilmoitettava aina riippumatta siitä, onko lähdeveroa peritty, esimerkiksi rajoitetusti verovelvollisen urheilijan ja taiteilijan saama palkkio on ilmoitettava.\r\n\r\n**Työkorvauksiin liittyvät kustannusten korvaukset**\r\n\r\nJos palkkio tai muu suorituksen määrä on maksettu työkorvauksena, saajalla ei voi olla verovapaisiin kuluihin merkittäviä kustannusten korvauksia.\r\n\r\nTyökorvauksiin liittyvät kustannusten korvaukset lisätään tulon määrään yhtenä summana muun palkkion kanssa. Työkorvauksen yhteydessä maksetut kustannusten korvaukset eivät ole verovapaita. Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena, eikä tulolajia kilometrikorvaus (veronalainen) saa käyttää.\r\n\r\nTyökorvauksen saaja voi vaatia matkakustannuksia vähennettäväksi esitäytetyllä veroilmoituksellaan. Elinkeinotoiminnan harjoittaja vähentää kuitenkin matkakustannukset kirjanpidossaan.\r\n\r\nTyökorvauksesta ei yleensä voi ennen ennakonpidätyksen toimittamista vähentää työstä aiheutuneita kustannusten korvauksia. Poikkeuksena ovat vain luonnollisille henkilöille työmatkasta maksetut kustannusten korvaukset, joista ei tarvitse toimittaa ennakonpidätystä, jos niiden perusteet ja määrät ovat sen mukaisia kuin Verohallinnon vuosittain antamassa päätöksessä verovapaista matkakustannusten korvauksista on määrätty. Nämäkään korvaukset eivät ole verovapaita, joten ne ilmoitetaan työkorvauksen yhteissummassa.",
            "taxAndSidecostsDescr": "Työkorvauksesta ei pääsääntöisesti makseta sosiaalivakuutusmaksuja. Julkisten alojen eläkelakien piiriin kuuluva työnantaja maksaa yksityishenkilön toimeksiantosopimuksen perusteella työeläkevakuutusmaksut, jos toimeksisaaja ei toimi yrittäjänä eli hänellä ei ole yrittäjän eläkelain mukaista vakuutusta. Lisäksi tulonsaaja maksaa työkorvauksesta työntekijän sairausvakuutuksen päivärahamaksun. Oletuksena tulolajissa on, ettei tulo ole sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/wages"
            ],
            "calcGrouping": "baseSalary"
        },
        {
            "code": 337,
            "nameEn": "Supplementary daily allowance paid by employer-specific health insurance fund",
            "nameFi": "Työnantajakohtaisen sairaskassan maksama täydennyspäiväraha",
            "nameSv": "Kompletteringsdagpenning som betalats av en arbetsgivarspecifik sjukkassa",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain sairauskassat käyttävät tätä tulolajia",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 338,
            "nameEn": "Pension paid by employer",
            "nameFi": "Työnantajan maksama eläke",
            "nameSv": "Pension som arbetsgivaren betalat ut",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "pensionPaidByEmployer",
            "notSupportedError": null,
            "description": "Eläke, jonka työnantaja maksaa entiselle työntekijälleen tai tämän edunsaajalle aiemman palvelussuhteen perusteella.",
            "taxAndSidecostsDescr": "Tulo on ennakonpidätyksen alainen. Työnantajan maksamasta eläkkeestä ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/employmentTermination"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 339,
            "nameEn": "Dividends/profit surplus based on work effort (wages)",
            "nameFi": "Työpanokseen perustuva osinko tai ylijäämä (palkka)",
            "nameSv": "Dividend eller överskott (lön) som baserar sig på arbetsinsats",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "dividendsBasedOnWages",
            "notSupportedError": null,
            "description": "Osakkaalle palkkana maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
            "taxAndSidecostsDescr": "Työpanokseen perustuvasta osingosta toimitetaan ennakonpidätys. Yhtiön osakkaalle jaettavasta osingosta ei makseta työeläkevakuutus-, työttömyysvakuutus- eikä työtapaturma- ja ammattitautivakuutuksen maksua, vaikka osinko katsottaisiin työpanokseen perustuvaa osinkoa koskevien erityissäännösten perusteella verotuksessa palkaksi. Työpanokseen perustuvasta osingosta maksetaan työnantajan sairausvakuutusmaksu ja osingon määrä on työntekijän sairausvakuutuksen päivärahamaksun perusteena.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 340,
            "nameEn": "Dividends/profit surplus (wages) based on work effort (non-wage)",
            "nameFi": "Työpanokseen perustuva osinko tai ylijäämä (työkorvaus)",
            "nameSv": "Dividend eller överskott (arbetsersättning) som baserar sig på arbetsinsats",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "dividendsBasedOnNonWage",
            "notSupportedError": null,
            "description": "Osakkaalle työkorvauksena maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.\r\n\r\nIlmoitus on annettava työkorvauksena verotettavasta työpanokseen perustuvasta osingosta tai ylijäämästä siinäkin tapauksessa, että saaja on merkitty ennakkoperintärekisteriin.",
            "taxAndSidecostsDescr": "Työpanokseen perustuvasta osingosta (työkorvausta) toimitetaan ennakonpidätys, ellei saaja ole ennakkoperintärekisterissä. Suorituksesta ei makseta sosiaalivakuutusmaksuja. Tulo on kuitenkin työntekijän sairausvakuutuksen päivärahamaksun alainen.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "salaryAdditions"
        },
        {
            "code": 341,
            "nameEn": "Employer-subsidised commuter ticket, tax-exempt share",
            "nameFi": "Työsuhdematkalipun verovapaa osuus",
            "nameSv": "Skattefri andel av personalbiljett",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employerSubsidisedCommuterTicketTaxExempt",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun verovapaa osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua tai vähennettävää määrää ei ole.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 300 euron työsuhdematkalipun. Luontoisetua ei synny.\r\n\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 300 euroa\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.",
            "taxAndSidecostsDescr": "Verovapaasta osuudesta ei toimiteta ennakonpidätystä eikä makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/subsidisedCommute",
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 342,
            "nameEn": "Employer-subsidised commuter ticket, taxable share",
            "nameFi": "Työsuhdematkalipun palkaksi katsottu osuus",
            "nameSv": "Andel av personalbiljett som betraktas som lön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "employerSubsidisedCommuterTicketTaxable",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun palkaksi katsottava osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen\r\nhenkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nTyösuhdematkalipun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.\r\n\r\nPalkaksi katsottavana osuutena ilmoitetaan se osuus edun arvosta, josta työnantaja on jo vähentänyt mahdollisesti tulonsaajalta perimänsä korvauksen.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 800 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Koska työsuhdematkalippu on veronalaista siltä osin kuin edun arvon ylittää 300 euroa ja alittaa 750 euroa, veronalaista luontoisetua syntyy 450 euroa. Työntekijä saa verovapaata etua 350 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan palkaksi katsottu määrä 450 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 350 euroa\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 5 000 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Veronalaista luontoisetua ja palkkaa on 2 050 euroa. Työntekijä saa verovapaata etua 2 950 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan luontoisedun määrä 2 050 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan verovapaan edun määrä 2 950 euroa.\r\n\r\nTyösuhdematkalipun palkaksi katsottua osuus ilmoitetaan ainoastaan omalla tulolajillaan, eikä sitä lisätä Muu luontoisetu-tulolajiin etuihin.",
            "taxAndSidecostsDescr": "Palkaksi katsottu osuus on ennakonpidätyksen ja sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/subsidisedCommute",
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 343,
            "nameEn": "Employee stock option",
            "nameFi": "Työsuhdeoptio",
            "nameSv": "Anställningsoption",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "employeeStockOption",
            "notSupportedError": null,
            "description": "Palkkio, joka annetaan työnantajayrityksen option muodossa.",
            "taxAndSidecostsDescr": "Työsuhdeoptiosta saatavasta edusta toimitetaan ennakonpidätys.\r\n\r\nTyösuhdeoptio on tuloverotuksen kohteena optiota käytettäessä. Työsuhdeoptiosta ei mene sosiaalivakuutusmaksuja sitä käytettäessä.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 350,
            "nameEn": "Wages transferred to athletes' special fund",
            "nameFi": "Urheilijarahastoon siirretty palkka",
            "nameSv": "Lön som överförs till en idrottsfond",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesTransferredToAthletesSpecialFund",
            "notSupportedError": null,
            "description": "Urheilutulon siirtäminen urheilijarahastoon tulojen jaksottamista varten. Urheilijalla on oikeus siirtää osa urheilutulosta urheilijarahastoon. Rahastoinnista sovitaan urheilijan, työnantajan ja mahdollisesti sponsorin kesken.\r\n\r\nUrheilutulo voi muodostua urheilijan palkasta tai palkkiosta. Tulolajilla ilmoitetaan vain rahastoon siirretty palkka. Siirto urheilurahastoon on tarkoitettu urheiluajan jälkeisen toimeentulon turvaamiseen.",
            "taxAndSidecostsDescr": "Urheilijan palkasta maksetaan työnantajan sairausvakuutusmaksu rahastoon siirrettävän palkan osalta. Urheilija maksaa tulosta veroa vasta silloin, kun tulo\r\nmaksetaan hänelle rahastosta. Rahastoitavasta palkasta toimitetaan ennakonpidätys ja maksetaan sairausvakuutuksen päivärahamaksu vasta silloin, kun\r\npalkka maksetaan rahastosta urheilijalle.\r\n\r\nUrheilutulon rahastointi ei vaikuta urheilijan tapaturma- ja eläketurvasta annetussa laissa eriteltyjen\r\nvakuutusmaksujen maksamiseen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/athlete"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 351,
            "nameEn": "Wages paid from athletes' special fund",
            "nameFi": "Urheilijarahastosta maksettu palkka",
            "nameSv": "Lön från en idrottsfond",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "wagesPaidFromAthletesSpecialFund",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain urheilijarahaston käyttöön: Tulolajilla ilmoitetaan vain rahastosta maksettu palkka.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 352,
            "nameEn": "Wages for insurance purposes",
            "nameFi": "Vakuutuspalkka",
            "nameSv": "Försäkringslön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": true,
            "taxDefault": 0,
            "transactionCode": "wagesForInsurancePurposes",
            "notSupportedError": "TODO: Check",
            "description": "HUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.\r\n\r\nLaskennallinen palkka, joka olisi maksettava, jos ulkomailla tehtyä työtä vastaava työ tehtäisiin Suomessa. Vakuutuspalkkaa ulkomaantyössä käytetään sosiaalivakuuttamisessa maksujen ja etuuksien määräytymisen perusteena todellisen palkan sijaan.\r\n\r\nEri toimijoiden kesken on omat sääntönsä siitä, milloin määräytymisen perusteena käytetään vakuutuspalkkaa ja milloin todellista maksettua palkkaa. Aiheeseen liittyvä tarkempi ohje löytyy Eläketurvakeskuksen soveltamisohjeista: Vakuutuspalkka eläkkeen perusteena olevana työansiona.",
            "taxAndSidecostsDescr": "Oletuksena tulolajissa on, että se on sosiaalivakuutusmaksujen alainen.\r\n\r\nJos vakuutuspalkka ei poikkeuksellisesti olisikaan jonkun tai joidenkin sosiaalivakuutusmaksujen alainen, suorituksen maksaja ilmoittaa vakuuttamistiedon erillistä Vakuuttamistiedon tyyppi- tietoa käyttäen.",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 353,
            "nameEn": "Taxable reimbursement of expenses",
            "nameFi": "Veronalainen kustannusten korvaus",
            "nameSv": "Skattepliktig kostnadsersättning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "taxableReimbursementOfExpenses",
            "notSupportedError": null,
            "description": "Veronalaisilla kustannusten korvauksilla tarkoitetaan työn suorittajalle työtehtävien hoitamisesta aiheutuneista kustannuksista maksettavaa veronalaista korvausta. Tulolajilla ilmoitetaan kaikkien sellaisten veronalaisten kustannusten korvausten yhteissumma, jota ei eritellä muilla tulolajeilla.\r\n\r\nKohdassa ilmoitetaan vain palkansaajalle maksettuja välittömästi työstä aiheutuneita kustannusten korvauksia, joita työnantaja ei ole lukenut mukaan palkan määrään toimittaessaan ennakonpidätystä (ennakkoperintälain 15 §).\r\n\r\nTyönantaja voi korvata palkansaajalle työstä välittömästi aiheutuneet kustannukset. Tällaisia ovat muun muassa kustannukset, jotka ovat aiheutuneet työvälineiden ja materiaalin hankkimisesta, tarveaineista ja suoja-asujen hankkimisesta aiheutuvat kustannukset. Esimerkiksi rakennusalalla maksetaan työehtosopimukseen perustuvaa työvälinekorvausta.\r\n\r\nTyönantaja ei toimita veronalaisista kustannusten korvauksista ennakonpidätystä eikä maksa sosiaalivakuutusmaksuja. Korvaukset ovat kuitenkin palkansaajan veronalaista tuloa. Palkansaajan tulee vaatia todelliset kustannukset vähennettäviksi esitäytetyllä veroilmoituksellaan.\r\n\r\nJos työntekijä on hankkinut työnantajan puolesta työvälineitä ja työmateriaaleja, niiden kustannuksia ei ilmoiteta, jos työnantaja on korvannut ne työntekijälle tositetta vastaan.",
            "taxAndSidecostsDescr": "Työnantaja ei toimita veronalaisista kustannusten korvauksista ennakonpidätystä eikä maksa sosiaalivakuutusmaksuja.\r\n\r\nKorvaukset ovat kuitenkin palkansaajan veronalaista tuloa. Palkansaajan tulee vaatia todelliset kustannukset vähennettäviksi esitäytetyllä veroilmoituksellaan.\r\n\r\nJos työntekijä on hankkinut työnantajan puolesta työvälineitä ja työmateriaaleja, niiden kustannuksia ei ilmoiteta, jos työnantaja on korvannut ne työntekijälle tositetta vastaan.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/expenses"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 354,
            "nameEn": "Private day care allowance municipal supplement",
            "nameFi": "Yksityisen hoidon tuen kuntalisä",
            "nameSv": "Kommuntillägg till stöd för privat vård",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "privateDayCareAllowanceMunicipalSupplement",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain kunnat käyttävät tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 355,
            "nameEn": "Private day care allowance (wages)",
            "nameFi": "Yksityisen hoidon tuki (palkka)",
            "nameSv": "Stöd för privat vård (lön)",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "privateDayCareAllowanceWages",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain Kela käyttää tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 356,
            "nameEn": "Private day care allowance (non-wage)",
            "nameFi": "Yksityisen hoidon tuki (työkorvaus)",
            "nameSv": "Stöd för privat vård (arbetsersättning)",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "privateDayCareAllowanceNonWage",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Vain Kela käyttää tätä tulolajia.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 357,
            "nameEn": "Kilometre allowance paid by non-profit organisation",
            "nameFi": "Yleishyödyllisen yhteisön maksama kilometrikorvaus",
            "nameSv": "Kilometerersättning som betalats ut av ett allmännyttigt samfund",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "kilometreAllowancePaidByNonProfitOrganisation",
            "notSupportedError": null,
            "description": "Kilometrikorvaus, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nYleishyödyllisen yhteisön hyväksi sen toimeksiannosta tehty matka rinnastetaan työmatkaan silloinkin, kun suorituksen saaja ei ole palvelussuhteessa maksajaan eikä muutoin saa palkkaa työstä, johon matka liittyy.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut korvaukset. Jos verovapaat korvaukset liittyvät yhteisön maksamaan palkkaan, ne ilmoitetaan tulolajilla Kilometrikorvaus (verovapaa) (311).\r\n\r\nTulolajilla ilmoitetaan vain tuloverolain (1535/1992) 71 § 3 momentin mukaiset verovapaat kilometrikorvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa: matkustamiskustannusten korvaus, jota voidaan suorittaa myös verovelvollisen asunnolta tehdystä matkasta. Matkustamiskustannusten korvauksesta muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 2 000 euroa kalenterivuodelta.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaa kilometrikorvauksia enemmän kuin tuloverolain 71 § 3 momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
            "taxAndSidecostsDescr": "Korvauksesta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/nonProfitOrg"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 358,
            "nameEn": "Daily allowance paid by non-profit organisation",
            "nameFi": "Yleishyödyllisen yhteisön maksama päiväraha",
            "nameSv": "Dagtraktamente som betalats ut av ett allmännyttigt samfund",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "dailyAllowancePaidByNonProfitOrganisation",
            "notSupportedError": null,
            "description": "Työmatkapäiväraha, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut päivärahat. Jos verovapaa päiväraha liittyy yhteisön maksamaan palkkaan, se ilmoitetaan tulolajilla Päiväraha (331).\r\n\r\nKohdassa ilmoitetaan vain tuloverolain 71 § 3 momentin mukaiset verovapaat korvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa:\r\n\r\n- päiväraha enintään 20 päivältä kalenterivuodessa\r\n- majoittumiskorvaus.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaan päivärahaa enemmän kuin tuloverolain 7 § momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
            "taxAndSidecostsDescr": "Päivärahasta ei makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/nonProfitOrg"
            ],
            "calcGrouping": "expenses"
        },
        {
            "code": 359,
            "nameEn": "Unjust enrichment",
            "nameFi": "Perusteeton etu",
            "nameSv": "Ogrundad förmån",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "unjustEnrichment",
            "notSupportedError": "Not supported: Does not fit Palkkaus.fi-process.",
            "description": "Perusteeton etu -tulolajia käytetään vain, jos aiheettomasti maksettu suoritus havaitaan ennen ensimmäisen ilmoituksen lähettämistä, eikä tuloa ole aiemmin ilmoitettu muuna tulona tulorekisteriin.\r\n\r\nTämä ei tyypillisesti ole mahdollista Palkkaus.fi-maksuprosessissa. Ota yhteys asiakaspalveluun, jos haluat oikeasti käyttää tätä tyyppiä.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 1,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 361,
            "nameEn": "Employee stock option with a subscription price lower than the market price at the time of issue",
            "nameFi": "Työsuhdeoptio, jonka merkintähinta on luovutusajankohtana alhaisempi kuin markkinahinta",
            "nameSv": "Anställningsoption vars teckningspris är lägre än marknadspriset vid överlåtelsetidpunkten",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employeeStockOptionLowerMarketPrice",
            "notSupportedError": null,
            "description": "Palkkio, joka annetaan työnantajayrityksen option muodossa ja jossa työntekijältä peritty option merkintähinta option luovutusajankohtana on selvästi alhaisempi kuin osakkeen markkinahinta.\r\n\r\nKäytettäessä optiota etu ilmoitetaan tulolajilla Työsuhdeoptio.",
            "taxAndSidecostsDescr": "Tulo on työeläke-, työttömyys- sekä työtapaturma- ja ammattitautivakuutusmaksun alainen.\r\n\r\nTyösuhdeoptioetu ei ole optio-oikeutta luovutettaessa tuloverotuksen alaista eikä siitä määrätä työnantajan sairausvakuutusmaksua. \r\n\r\nKäytettäessä optiota etu ilmoitetaan tulolajilla Työsuhdeoptio.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/otherCompensation"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 362,
            "nameEn": "Royalty paid to a non-resident taxpayer",
            "nameFi": "Rajoitetusti verovelvolliselle maksettu rojalti",
            "nameSv": "Royalty som betalats till en begränsat skattskyldig",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "royaltyPaidToNonResidentTaxpayer",
            "notSupportedError": null,
            "description": "Rajoitetusti verovelvolliselle henkilölle tai yhteisölle maksettava korvaus tekijänoikeuden, teollisoikeuden tai muun oikeuden käyttämisestä tai käyttöoikeudesta tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevista tiedoista.\r\n\r\nRojaltin perusteena voi olla\r\n\r\n- kirjallisen, taiteellisen tai tieteellisen teoksen tekijänoikeus\r\n- oikeus valokuvaan\r\n- patentin, tavaramerkin, mallin, muotin, piirustuksen, salaisen kaavan tai valmistustavan käyttäminen tai käyttöoikeus\r\n- teollisia, kaupallisia tai tieteellisiä kokemuksia koskevat tiedot.",
            "taxAndSidecostsDescr": "Rojaltista peritään lähdevero, jollei tuloa ole verosopimuksella vapautettu lähdeverosta. Rojalti on ilmoitettava tulorekisteriin, vaikka siitä ei olisi peritty lähdeveroa.\r\n\r\nRojalti ei ole sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "benefits"
        },
        {
            "code": 363,
            "nameEn": "Tax-exempt share of bicycle benefit",
            "nameFi": "Polkupyöräedun verovapaa osuus",
            "nameSv": "Skattefri andel av cykelförmånen",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "bicycleBenefitTaxExempt",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun verovapaa osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräetu on verovapaa, jos edun arvo on enintään 1 200 euroa vuodessa. Ylimenevä osuus on veronalaista luontoisetua. Jos työntekijä saa sekä polkupyöräedun että työsuhdematkalipun, niiden verovapaa määrä voi olla yhteensä korkeintaan 3 400 euroa.\r\n\r\nPolkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.",
            "taxAndSidecostsDescr": "Verovapaasta osuudesta ei toimiteta ennakonpidätystä eikä makseta sosiaalivakuutusmaksuja.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "benefits"
        },
        {
            "code": 364,
            "nameEn": "Bicycle benefit, taxable share",
            "nameFi": "Polkupyöräedun palkaksi katsottu osuus",
            "nameSv": "Andel av cykelförmånen som betraktas som lön",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 1,
            "transactionCode": "bicycleBenefitTaxable",
            "notSupportedError": null,
            "description": "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun palkaksi katsottava osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräedusta 1 200 euroa on verovapaata ja loput veronalaista luontoisetua.\r\n\r\nPalkaksi katsottu osuus polkupyöräedusta ilmoitetaan niiltä kuukausilta, kun etu on ollut työntekijän käytettävissä.\r\n\r\nPolkupyöräedun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.",
            "taxAndSidecostsDescr": "Palkaksi katsottu osuus on ennakonpidätyksen ja so-siaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "benefits"
        },
        {
            "code": 365,
            "nameEn": "Conditional stock options and grants",
            "nameFi": "Ehdollinen osakepalkkio",
            "nameSv": "Villkorlig aktiepremie",
            "isNegativeSupported": false,
            "pensionInsurance": true,
            "accidentInsurance": true,
            "unemploymentInsurance": true,
            "healthInsurance": true,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "conditionalStockOptions",
            "notSupportedError": null,
            "description": "Osakepalkkio, johon on liittynyt osakkeiden luovutusrajoitus ja ehdollinen palautusvelvollisuus ja josta on syntynyt sosiaalivakuutusmaksujen alaista tuloa. Tässä tilanteessa palkkiosta saatavan edun arvo ei siis alkuperäisestä suunnitelmasta poiketen riipukaan osakkeiden arvon kehityksestä osakepalkkion lupaamisen ja antamisen välisenä, vähintään vuoden pituisena aikana.\r\n\r\nTällainen tilanne voi syntyä olosuhteiden muuttuessa esimerkiksi niin, että työntekijän työsuhde päättyy vuoden määräaikana ja hän saa pitää osan osak-keista. Samalla syntyy velvollisuus maksaa sosiaali-vakuutusmaksut.\r\nTällainen osakepalkkio katsotaan sosiaalivakuutus-maksujen näkökulmasta annetuksi silloin, kun palk-kiota koskevat rasitteet päättyvät ja osakepalkkio jää henkilön omistukseen ja vallintaan. Verotuksen kan-nalta ehdollinenkin osakepalkkio katsotaan kuitenkin annetuksi jo silloin, kun henkilö saa omistusoikeuden osakkeisiin.\r\n\r\nPalkkion määrä ilmoitetaan tulolajilla Ehdollinen osakepalkkio sosiaalivakuutusmaksujen määräämistä varten. Tulolajilla ilmoitettava osakepalkkio ei ole tulonsaajalle konkreettisesti maksettavaa tuloa, vaan tietoa käytetään sosiaalivakuutusmaksujen perusteena. Tulonsaajan saama etu ja siitä tehty ennakonpidätys on ilmoitettu tulolajilla Osakepalkkio (320) jo silloin, kun tulonsaaja on saanut omistusoikeuden osakepalkkioon. Tätä tietoa käytetään esimerkiksi verotuksessa.\r\n\r\nTulolajia Ehdollinen osakepalkkio käytetään vain silloin, kun palkkioon on liittynyt sekä osakkeiden luovutusrajoitus että ehdollinen palautusvelvollisuus. Jos palkkioon on liittynyt vain toinen rasitteista, se ilmoitetaan kuten tavallinen osakepalkkio tulolajilla Osakepalkkio (320).",
            "taxAndSidecostsDescr": "Tulolaji on sosiaalivakuutusmaksujen alainen.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "benefits"
        },
        {
            "code": 401,
            "nameEn": "Compensation collected for car benefit",
            "nameFi": "Autoedusta peritty korvaus",
            "nameSv": "Ersättning för bilförmån",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": -1,
            "transactionCode": "compensationCollectedForCarBenefit",
            "notSupportedError": null,
            "description": "Luontoisedun omavastuuosuus, jonka työntekijä maksaa autoedun verotusarvosta.\r\n\r\nAutoedun omavastuuosuus alentaa verotettavaa etua tai poistaa sen kokonaan. Jos autoedun omavastuuosuus on autoedun verotusarvoa alempi, autoedun arvon ja autoedusta perityn korvauksen erotus on työntekijän veronalaista palkkaa. Jos omavastuuosuus vastaa autoedun verotusarvoa, verotettavaa etua ei synny. Tällöinkin peritty korvaus on ilmoitettava. Tällöin on lisäksi ilmoitettava edun määrä Autoetu-tulolajilla.\r\n\r\nAutoedun arvo ilmoitetaan täysimääräisenä Autoetu-tulolajilla ja Autoedusta peritty korvaustulolajilla työntekijältä perityn omavastuuosuuden määrä. \r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että suorituksen maksaja on perinyt tulonsaajalta luontoisedusta luontoisedun rahaarvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan.\r\n\r\nTyönantaja voi periä autoedun omavastuuosuuden työntekijän nettopalkasta.",
            "taxAndSidecostsDescr": "Ilmoitettu summa vähentää ennakonpidätyksen ja sosiaalimaksujen pohjana olevaa palkkaa ja vähennetään oletuksena asiakasvaratilin kautta maksetusta palkasta.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/carBenefit",
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 402,
            "nameEn": "Withholding tax",
            "nameFi": "Ennakonpidätys",
            "nameSv": "Förskottsinnehållning",
            "isNegativeSupported": true,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "withholdingTax",
            "notSupportedError": null,
            "description": "Ennakkoperintä, jossa suorituksen maksaja vähentää verovelvolliselle maksettavasta rahamäärästä ennakonpidätysprosentin mukaan lasketun veron.\r\n\r\nEnnakonpidätys voidaan ilmoittaa ilmoituksella yhteissummana, vaikka tulolajeja olisi ilmoitettu useita.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 403,
            "nameEn": "Elected official fee",
            "nameFi": "Luottamushenkilömaksu",
            "nameSv": "Avgift för förtroendevald",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "electedOfficialFee",
            "notSupportedError": "Not supported for companies / associations",
            "description": "Luottamushenkilömaksua voivat ilmoittaa vain kunta ja kuntayhtymä.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 404,
            "nameEn": "Tax at source",
            "nameFi": "Lähdevero",
            "nameSv": "Källskatt",
            "isNegativeSupported": true,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "taxAtSource",
            "notSupportedError": "International scenarios are not supported inthis version.",
            "description": "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 405,
            "nameEn": "Tax at source deduction",
            "nameFi": "Lähdeverovähennys",
            "nameSv": "Källskatteavdrag",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "taxAtSourceDeduction",
            "notSupportedError": "International scenarios are not supported inthis version.",
            "description": "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 406,
            "nameEn": "Wages paid",
            "nameFi": "Maksettava palkka",
            "nameSv": "Lön som ska betalas ut",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "wagesPaid",
            "notSupportedError": null,
            "description": "Rahapalkka, joka maksetaan ennakonpidätyksen, työntekijän sosiaalivakuutusmaksujen ja muiden vähennettävien erien vähentämisen jälkeen työntekijälle.\r\n\r\nMuihin vähennettäviin eriin voivat kuulua esimerkiksi luontoiseduista työntekijältä perittävät maksut, työnantajan pidättämät työttömyyskassan ja ammattiliiton jäsenmaksut sekä ulosottomaksut.\r\n\r\nTyönantajalle kuuluvia kulueriä, kuten esim. työmatkoilta maksettavia hotellimaksuja, matkalippukuluja tai mahdollisia työterveydenhuollon maksuja ei ilmoiteta Maksettava palkka-tulolajilla.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 407,
            "nameEn": "Reimbursement collected for other fringe benefits",
            "nameFi": "Muista luontoiseduista peritty korvaus",
            "nameSv": "Ersättning för andra naturaförmåner",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": -1,
            "transactionCode": "reimbursementCollectedForOtherFringeBenefits",
            "notSupportedError": null,
            "description": "Osuus luontoisedun arvosta, jonka työntekijä maksaa itse.\r\n\r\nKohdassa ilmoitetaan summa, joka työntekijältä on peritty muista luontoiseduista kuin autoedusta tai työsuhdematkalippuedusta.\r\n\r\nLuontoisedun arvo ilmoitetaan täysimääräisenä omalla tulolajillaan ja Muista luontoiseduista peritty korvaus-tulolajilla työntekijältä perityn omavastuuosuuden määrä.\r\n\r\nJos ravintoedusta peritty korvaus vastaa verotusarvoa, perittyä korvausta ei ilmoiteta Muusta luontoisedusta peritty korvaus-erittelylajilla. Tällöin käytetään vain tietoa Ravintoedusta peritty korvaus vastaa verotusarvoa – ”Kyllä (true)”.\r\n\r\nTyönantaja voi periä korvauksen luontoiseduista työntekijän nettopalkasta.",
            "taxAndSidecostsDescr": "Ilmoitettu summa vähentää oletuksena ennakonpidätyksen ja sosiaalimaksujen pohjana olevaa palkkaa ja vähennetään asiakasvaratilin kautta maksetusta palkasta.\r\n\r\nJos Muu luontoisetu -tulolajilla ilmoitettu luontoisetu ei ole sosiaalivakuutusmaksujen alainen ja suorituksen maksaja on ilmoittanut tämän käyttämällä Vakuuttamistiedon tyyppi - tietoryhmää, vastaava Vakuuttamistiedon tyyppi -tieto on lisättävä myös Muista luontoiseduista peritty korvaus -tulolajiin siltä osin, kun peritty korvaus kohdistuu siihen osaan etua, joka ei ole sosiaalivakuutusmaksujen alainen, ks. asiakasohje Tietojen ilmoittaminen tulorekisteriin: luontoisedut ja kustannusten korvaukset.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 408,
            "nameEn": "Other item deductible from net wage or salary",
            "nameFi": "Nettopalkasta vähennettävä muu erä",
            "nameSv": "Övriga poster som ska dras av från nettolönen",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "otherItemDeductibleFromNetWage",
            "notSupportedError": null,
            "description": "Erät, jotka palkan maksaja vähentää nettopalkasta ennen kuin maksaa suorituksen tulonsaajalle.\r\n\r\nNettopalkasta voidaan vähentää suorituksia lain tai sopimuksen perusteella. Lain perusteella tehtäviä vähennyksiä ovat esimeriksi ulosmittaukset. Työnantaja voi kuitata omia saataviaan työntekijän palkasta. Työntekijän nettopalkasta voidaan vähentää maksuja, joiden perimisestä on työnantajan ja työntekijän kesken sovittu. Nettopalkasta vähennettäviä eriä ovat esim. työttömyyskassan ja ammattiyhdistysten jäsenmaksut sekä pysäköintihallimaksut.\r\n\r\nJos ulosmittaus on jo ilmoitettu erikseen, ei sitä ilmoiteta enää tällä erittelylajilla yhteissummassa.",
            "taxAndSidecostsDescr": "Vähennetään nettopalkasta - ei vaikuta ennakonpidätykseen tai sosiaalimaksuihin.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/deductions"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 409,
            "nameEn": "Net wage or salary",
            "nameFi": "Nettopalkka",
            "nameSv": "Nettolön",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "netWage",
            "notSupportedError": null,
            "description": "Palkka, josta on toimitettu ennakonpidätys ja maksettu työntekijän sosiaalivakuutusmaksut. \r\n\r\nNettopalkka on eri asia kuin työntekijälle maksettava palkka.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 410,
            "nameEn": "Employer-paid premium for collective additional pension insurance",
            "nameFi": "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen maksu",
            "nameSv": "Kollektiv tilläggspensionsförsäkringsavgift som betalats av arbetsgivaren",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employerPaidPremiumForCollectiveAdditionalPensionInsurance",
            "notSupportedError": null,
            "description": "Työnantajan osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta laisinkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksu-kohdassa.\r\n\r\nMaksut ilmoitetaan tässä kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/pension"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 411,
            "nameEn": "Employer-paid premium for collective additional pension insurance, employee's contribution",
            "nameFi": "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen työntekijältä peritty osuus",
            "nameSv": "Andel av kollektiv tilläggspensionsförsäkringsavgift som betalats av arbetsgivaren och som debiteras arbetstagaren",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution",
            "notSupportedError": null,
            "description": "Työntekijän osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta lainkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksut-kohdassa.\r\n\r\nMaksut ilmoitetaan kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
            "taxAndSidecostsDescr": "Vähennetään oletuksena nettopalkasta. Ei vaikuta ennakonpidätykseen tai sosiaalimaksuihin.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/pension"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 412,
            "nameEn": "Employee's health insurance contribution",
            "nameFi": "Työntekijän sairausvakuutusmaksu",
            "nameSv": "Arbetstagarens sjukförsäkringsavgift",
            "isNegativeSupported": true,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employeesHealthInsuranceContribution",
            "notSupportedError": "International scenarios are not supported inthis version.",
            "description": "Yleisesti verovelvollisella tulonsaajalla sairausvakuutusmaksu sisältyy aina ennakonpidätykseen, eikä yleisesti verovelvollisten osalta tule ilmoittaa erikseen työntekijän sairausvakuutusmaksua.\r\n\r\nRajoitetusti verovelvollisen sairausvakuutusmaksu on perittävä erikseen lähdeveron lisäksi, ellei rajoitetusti verovelvollinen ole hakeutunut verotusmenettelystä annetun lain mukaiseen verotukseen, jolloin sairausvakuutusmaksu sisältyy ennakonpidätykseen tai ennakkoveroon. \r\n\r\nHUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 413,
            "nameEn": "Employee's pension insurance contribution",
            "nameFi": "Työntekijän työeläkevakuutusmaksu",
            "nameSv": "Arbetspensionsförsäkringsavgift för arbetstagare",
            "isNegativeSupported": true,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employeesPensionInsuranceContribution",
            "notSupportedError": null,
            "description": "Bruttopalkan perusteella määräytyvä työeläkevakuutusmaksu, jolla työntekijä osallistuu työeläkevakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työeläkevakuutusmaksu vähennetään työntekijän nettopalkasta.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/deductions"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 414,
            "nameEn": "Employee's unemployment insurance contribution",
            "nameFi": "Työntekijän työttömyysvakuutusmaksu",
            "nameSv": "Arbetstagarens arbetslöshetsförsäkringsavgift",
            "isNegativeSupported": true,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "employeesUnemploymentInsuranceContribution",
            "notSupportedError": null,
            "description": "Bruttopalkan perusteella määräytyvä työttömyysvakuutusmaksu, jolla työntekijä osallistuu työttömyysvakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työttömyysvakuutusmaksu vähennetään työntekijän nettopalkasta.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/deductions"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 415,
            "nameEn": "Reimbursement for employer-subsidised commuter ticket",
            "nameFi": "Työsuhdematkalipusta peritty korvaus",
            "nameSv": "Ersättning för personalbiljett",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "reimbursementForEmployerSubsidisedCommuterTicket",
            "notSupportedError": null,
            "description": "Luontoisedun omavastuuosuus, jonka työntekijä maksaa työsuhdematkalipun euromääräisestä arvosta.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nTyönantaja voi periä työsuhdematkalipun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 200 euroa. Veronalaista luontoisetua ja palkkaa on 250 euroa. Työntekijä saa verovapaata etua 550 euroa.\r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus -tulolajilla ilmoitetaan palkaksi katsottu määrä 250 euroa.\r\n- Työsuhdematkalipusta peritty korvaus-erittelylajilla ilmoitetaan työntekijältä peritty määrä 200 euroa.\r\n- Työsuhdematkalipun verovapaa osuus- tulolajilla ilmoitetaan verovapaan edun määrä 550 euroa.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 500 euroa. Veronalaisen luontoisedun määrä on 450 euroa. Koska peritty korvaus on suurempi kuin edun verotusarvo, verotettavaa luontoisetua ei synny. Työntekijä saa verovapaata etua 500 euroa.\r\n\r\n- Työsuhdematkalipusta peritty korvaus -erittelylajilla ilmoitetaan työntekijältä peritty määrä 500 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 500 euroa.",
            "taxAndSidecostsDescr": "Vähennetään oletuksena nettopalkasta. Ei vaikuta ennakonpidätykseen tai sosiaalimaksuihin, sillä summa pitää olla jo vähennetty kohdasta 342 'Työsuhdematkalipun palkaksi katsottu osuus'.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/subsidisedCommute",
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 416,
            "nameEn": "Tax paid abroad",
            "nameFi": "Ulkomaille maksettu vero",
            "nameSv": "Skatt som betalats till utlandet",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "taxPaidAbroad",
            "notSupportedError": "International scenarios are not supported inthis version.",
            "description": "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
            "taxAndSidecostsDescr": "",
            "isSupported": false,
            "paymentDefault": 0,
            "usecases": [],
            "calcGrouping": "disabled"
        },
        {
            "code": 417,
            "nameEn": "Distraint",
            "nameFi": "Ulosmittaus",
            "nameSv": "Utmätning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "distraint",
            "notSupportedError": null,
            "description": "Perittäväksi tulleen velan maksamiseksi tehtävä pidätys. Ulosmitattava määrä lasketaan nettotulosta.\r\n\r\nUlosmitattaessa velallisen ja hänen perheensä toimeentuloa varten tulee kuitenkin jättää suojaosuus.",
            "taxAndSidecostsDescr": "Vähennetään oletuksena nettopalkasta. Ei vaikuta ennakonpidätykseen tai sosiaalimaksuihin.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/deductions"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 418,
            "nameEn": "Voluntary individual pension insurance premium",
            "nameFi": "Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu",
            "nameSv": "Försäkringsavgift för en individuell, frivillig pensionsförsäkring",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "voluntaryIndividualPensionInsurancePremium",
            "notSupportedError": null,
            "description": "Lakisääteistä eläketurvaa täydentävän tietylle henkilölle kohdistetun eläkevakuutuksen maksu. Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu voi olla jatkuva tai kertamaksullinen.\r\n\r\nKohtaan merkitään työntekijälle otetun vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Samoin ilmoitetaan myös sellaisen yksilöllisen eläkevakuutuksen maksut, joka on otettu avoimen yhtiön yhtiömiehelle, kommandiittiyhtiön vastuunalaiselle yhtiömiehelle tai sellaisen osakeyhtiön osakkaalle, joka ei ole työeläkevakuutettu.\r\n\r\nJos työnantajan ottaman vapaaehtoisen yksilöllisen eläkevakuutuksen maksut ylittävät 8 500 euroa, ne ovat veronalaista tuloa, joista maksetaan myös työnantajan sosiaalivakuutusmaksut. Jos maksut ylittävät 8 500 euroa vuodessa, suorituksen maksaja ilmoittaa ylimenevät maksut muuna luontoisetuina. \r\n\r\nEsimerkki: Työnantaja maksaa työntekijälle ottamansa vapaaehtoisen yksilöllisen eläkevakuutuksen maksuja vuodessa 12000 euroa. Työnantaja ilmoittaa tiedot seuraavasti:\r\n\r\n- Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu 8 500 euroa.\r\n- Muu verotettava luontoisetu 3 500 euroa.\r\n\r\nKohtaan merkitään vain vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Jos työnantaja on maksanut työntekijän puolesta työntekijän pakollisia eläkevakuutusmaksuja, työttömyysvakuutusmaksuja ja työntekijän itsensä ottaman vapaaehtoisen eläkevakuutuksen maksuja, nämä ilmoitetaan Muu luontoisetu-tulolajilla.",
            "taxAndSidecostsDescr": "",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/pension"
            ],
            "calcGrouping": "benefits"
        },
        {
            "code": 419,
            "nameEn": "Deduction before withholding",
            "nameFi": "Vähennys ennen ennakonpidätystä",
            "nameSv": "Avdrag före förskottsinnehållning",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": true,
            "taxDefault": -1,
            "transactionCode": "deductionBeforeWithholding",
            "notSupportedError": null,
            "description": "Työntekijän itsensä maksama työn tekemisestä aiheutunut välitön kustannus, jonka työnantaja vähentää bruttopalkasta ennen ennakonpidätyksen osuuden määrittämistä. Tällaisia kustannuksia ovat muun muassa palkansaajan maksamat kustannukset moottorisahan käytöstä ja muut työvälineistä aiheutuneet kustannukset. Samoin tällaisia kustannuksia ovat myös menot työmatkoista, jos työnantaja ei maksa niistä verovapaita korvauksia palkan lisäksi.\r\n\r\nKohtaan merkitään vähennys, joka on tehty ennen ennakonpidätyksen toimittamista. Vähennysten osuuden tulee sisältyä myös tulolajilla ilmoitettuun palkkaan.\r\n\r\n**Esimerkki**: Työntekijä on maksanut itse moottorisahan käytöstä aiheutuneet 4 500 euron kustannukset. Työntekijälle maksettu koko vuoden palkka on 20 000 euroa, josta sahatyön osuus on 14 000 euroa. Työntekijä on vaatinut ennakkoperinnässä vähennystä käyttökustannuksista 40 %:n mukaisesti sahatyöstä maksetun palkan määrästä eli 5 600 euroa. Ennakonpidätys on toimitettu ennakkoperintälain (1118/1996) 15 § mukaisesti 14 400 eurosta (20 000 - 5 600).\r\n\r\nPalkan tulolajilla ilmoitetaan 20 000 euroa ja Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 5 600 euroa. Työntekijän tulee vaatia esitäytetyllä veroilmoituksellaan moottorisahavähennystä ja selvittää sahatyöstä maksetun palkan määrä. Lopullisessa verotuksessa vähennys lasketaan sahatyöstä maksetun palkan (14 000 euroa) osuudesta.\r\n\r\nJos työnantaja on maksanut moottorisahan käytöstä aiheutuneet kustannukset, katso ilmoittaminen kohdasta: Veronalaiset kustannusten korvaukset. \r\n\r\n**Esimerkki**: Pelimannille luvataan soittokeikasta 200 euroa. Matkustaminen ja matkustamisen kustannukset eivät vaikuta maksettavaan bruttomäärään. Matkakustannuksia ei siis korvata.\r\n\r\nTyönantaja voi laskea määrän, josta toimittaa ennakonpidätyksen siten, että vähentää työntekijän antaman selvityksen perusteella saman määrän, jonka voisi korvata verovapaasti. Esimerkiksi matkustamiskustannuksia 51 km x 41 snt/km eli 20,91 snt ja osapäivärahan 19 euroa vastaavat määrät, jolloin pidätyksenalaiseksi määräksi jää 160,09 euroa. Samasta määrästä maksetaan myös työnantajan sairausvakuutusmaksu. \r\n\r\nPalkan tulolajilla ilmoitetaan 200 euroa, Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 39,91 euroa. Verovapaiden matkakustannusten korvausten tulolajeilla ei tehdä merkintöjä. \r\n\r\nTyöntekijän on vaadittava vähennyksiä veroilmoituksessaan ja vähennys on yleensä alempi kuin ennakonpidätyksen yhteydessä.",
            "taxAndSidecostsDescr": "Ilmoitettu summa vähentää ennakonpidätyksen ja sosiaalimaksujen pohjana olevaa palkkaa ja vähennetään oletuksena asiakasvaratilin kautta maksetusta palkasta.\r\n\r\nJos samalla maksukerralla maksetaan sosiaalivakuutusmaksun alaista palkkaa ja sellaista EPL 13 §:n muista palkkaa, joka ei ole sosiaalivakuutusmaksujen perusteena ja jompaankumpaan tuloon kohdistuu kuluja, jotka vähennetään ennen ennakonpidätyksen toimittamista, maksajan on annettava tiedot palkoista kahdella eri ilmoituksella ja eriteltävä, kumpaan tuloon vähennys kohdistuu.",
            "isSupported": true,
            "paymentDefault": 0,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/deductions"
            ],
            "calcGrouping": "deductions"
        },
        {
            "code": 420,
            "nameEn": "Reimbursement collected for bicycle benefit",
            "nameFi": "Polkupyöräedusta peritty korvaus",
            "nameSv": "Ersättning för cykelförmån",
            "isNegativeSupported": false,
            "pensionInsurance": false,
            "accidentInsurance": false,
            "unemploymentInsurance": false,
            "healthInsurance": false,
            "insuranceInformationAllowed": false,
            "taxDefault": 0,
            "transactionCode": "reimbursementCollectedForBicycleBenefit",
            "notSupportedError": null,
            "description": "Luontoisedun omavastuuosuus, jonka työntekijä maksaa polkupyöräedun euromääräisestä arvosta.\r\n\r\nOsa polkupyöräedun arvosta on verovapaata ja osa veronalaista. Polkupyöräedusta 1 200 euroa on verovapaata, ja ylittävä osa on veronalaista luontoisetua. Polkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole. Edusta peritty korvaus alentaa verotettavan edun määrää.\r\n\r\nTyönantaja voi periä polkupyöräedun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nPolkupyöräedun ilmoittaminen poikkeaa muista luontoiseduista samalla tavalla kuin työsuhdematkalipun ilmoittaminen, koska työnantajan on vähennettävä polkupyöräedun palkaksi katsottavasta osuudesta työntekijältä perimänsä korvaus. Palkaksi katsottavana osuutena ilmoitetaan jäljelle jäävä määrä.",
            "taxAndSidecostsDescr": "Vähennetään oletuksena nettopalkasta. Ei vaikuta ennakonpidätykseen tai sosiaalimaksuihin, sillä summa pitää olla jo vähennetty kohdasta 364 'Polkupyöräedun palkaksi katsottu osuus'.",
            "isSupported": true,
            "paymentDefault": -1,
            "usecases": [
                "https://palkkaus.fi/usecases/2019/benefits"
            ],
            "calcGrouping": "deductions"
        }
    ];

    /* eslint-disable */
    /**
     * Yearly changing numbers. Contains side cost percentages.
     *
     */
    const yearlyChangingNumbersYears = [
        {
            "year": 2018,
            "sideCosts": {
                "unemploymentTotalPercent": 0.0255,
                "unemploymentEmployerPercent": 0.0065,
                "unemploymentWorkerPercent": 0.019,
                "tyelIncomeLimit": 58.27,
                "tyelBasePercent": 0.253,
                "tyelWorkerPercent": 0.0635,
                "tyel53Percent": 0.0785,
                "tyelContractEmployerLimit": 8346,
                "socSecPercent": 0.0086,
                "illnessInsurancePercent": 0.0153,
                "taxFreeDailyAllowance": 42,
                "taxFreeDailyHalfAllowance": 19,
                "taxFreeMealAllowance": 10.5,
                "taxFreeKmAllowance": 0.42,
                "unionPaymentRaksaA": 0.0155,
                "unionPaymentRaksaAoTa": 0.0055,
                "stepsTaxCardLimit1": 10500,
                "stepsTaxCardLimit2": 17950,
                "householdDeduction": 0.20,
            }
        },
        {
            "year": 2019,
            "sideCosts": {
                "unemploymentTotalPercent": 0.02,
                "unemploymentEmployerPercent": 0.005,
                "unemploymentWorkerPercent": 0.015,
                "tyelIncomeLimit": 59.36,
                "tyelBasePercent": 0.252,
                "tyelWorkerPercent": 0.0675,
                "tyel53Percent": 0.0825,
                "tyelContractEmployerLimit": 8502,
                "socSecPercent": 0.0077,
                "illnessInsurancePercent": 0.0154,
                "taxFreeDailyAllowance": 42,
                "taxFreeDailyHalfAllowance": 19,
                "taxFreeMealAllowance": 10.5,
                "taxFreeKmAllowance": 0.43,
                "unionPaymentRaksaA": 0.0149,
                "unionPaymentRaksaAoTa": 0.0052,
                "stepsTaxCardLimit1": 10500,
                "stepsTaxCardLimit2": 17950,
                "householdDeduction": 0.20,
            }
        },
        {
            "year": 2020,
            "sideCosts": {
                "unemploymentEmployerPercent": 0.0045,
                "unemploymentWorkerPercent": 0.0125,
                "unemploymentPartialOwnerWorkerPercent": 0.0065,
                "tyelIncomeLimit": 60.57,
                "tyelBasePercent": 0.253,
                "tyelWorkerPercent": 0.0715,
                "tyel53Percent": 0.0865,
                "tyelContractEmployerLimit": 8676.0,
                "socSecPercent": 0.0134,
                "illnessInsurancePercent": 0.0186,
                "taxFreeDailyAllowance": 43.0,
                "taxFreeDailyHalfAllowance": 20.0,
                "taxFreeMealAllowance": 10.75,
                "taxFreeKmAllowance": 0.43,
                "unionPaymentRaksaA": 0.0149,
                "unionPaymentRaksaAoTa": 0.0052,
                "palkkausVatPercent": 0.24,
                "householdDeduction": 0.15,
                "mealBenefitCateringContract": 6.8,
                "mealBenefitInstitute": 5.1,
                "mealBenefitRestaurantWorker": 5.78,
                "mealBenefitTeacher": 4.08,
                "mealTicketLowerLimit": 6.8,
                "mealTicketUpperLimit": 10.7,
                "mealTicketTaxable": 0.75,
                "protectedPortionBase": 22.63,
                "protectedPortionDependant": 8.12
            },
        },
        {
            "year": 2021,
            "sideCosts": {
                "unemploymentEmployerPercent": 0.005,
                "unemploymentWorkerPercent": 0.014,
                "unemploymentPartialOwnerWorkerPercent": 0.0065,
                "tyelIncomeLimit": 61.37,
                "tyelBasePercent": 0.248,
                "tyelWorkerPercent": 0.0715,
                "tyel53Percent": 0.0865,
                "tyelContractEmployerLimit": 8790.0,
                "socSecPercent": 0.0153,
                "illnessInsurancePercent": 0.0136,
                "taxFreeDailyAllowance": 44.0,
                "taxFreeDailyHalfAllowance": 20.0,
                "taxFreeMealAllowance": 11.0,
                "taxFreeKmAllowance": 0.44,
                "unionPaymentRaksaA": 0.0149,
                "unionPaymentRaksaAoTa": 0.0052,
                "palkkausVatPercent": 0.24,
                "householdDeduction": 0.15,
                "mealBenefitCateringContract": 6.9,
                "mealBenefitInstitute": 5.18,
                "mealBenefitRestaurantWorker": 5.87,
                "mealBenefitTeacher": 4.14,
                "mealTicketLowerLimit": 6.9,
                "mealTicketUpperLimit": 10.9,
                "mealTicketTaxable": 0.75,
                "protectedPortionBase": 22.71,
                "protectedPortionDependant": 8.15
            },
        },
    ];

    /* eslint-disable */
    /**
     * Yearly holidays.
     */
    const yearlyHolidaysYears = [
        {
            "year": 2018,
            "holidays": {
                "newYearsDay": {
                    "date": "2018-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2018-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2018-03-30",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2018-03-31",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2018-04-01",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2018-04-02",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2018-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2018-05-10",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2018-05-20",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2018-06-22",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2018-06-23",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2018-11-24",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2018-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2018-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2018-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2018-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
        {
            "year": 2019,
            "holidays": {
                "newYearsDay": {
                    "date": "2019-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2019-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2019-04-19",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2019-04-20",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2019-04-21",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2019-04-22",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2019-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2019-05-30",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2019-06-09",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2019-06-21",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2019-06-22",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2019-11-23",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2019-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2019-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2019-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2019-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
        {
            "year": 2020,
            "holidays": {
                "newYearsDay": {
                    "date": "2020-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2020-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2020-04-10",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2020-04-11",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2020-04-12",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2020-04-13",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2020-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2020-05-21",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2020-05-31",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2020-06-19",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2020-06-20",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2020-11-21",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2020-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2020-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2020-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2020-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
        {
            "year": 2021,
            "holidays": {
                "newYearsDay": {
                    "date": "2021-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2021-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2021-04-02",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2021-04-03",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2021-04-04",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2021-04-05",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2021-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2021-05-13",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2021-05-23",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2021-06-25",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2021-06-26",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2021-11-20",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2021-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2021-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2021-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2021-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
        {
            "year": 2022,
            "holidays": {
                "newYearsDay": {
                    "date": "2022-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2022-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2022-04-15",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2022-04-16",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2022-04-17",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2022-04-18",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2022-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2022-05-26",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2022-06-05",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2022-06-24",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2022-06-25",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2022-11-19",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2022-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2022-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2022-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2022-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
        {
            "year": 2023,
            "holidays": {
                "newYearsDay": {
                    "date": "2023-01-01",
                    "holiday": "newYearsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "uudenvuodenpäivä",
                    "name": "New Year's Day"
                },
                "epiphany": {
                    "date": "2023-01-06",
                    "holiday": "epiphany",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "loppiainen",
                    "name": "Epiphany"
                },
                "goodFriday": {
                    "date": "2023-04-07",
                    "holiday": "goodFriday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pitkäperjantai",
                    "name": "Good Friday"
                },
                "easterSaturday": {
                    "date": "2023-04-08",
                    "holiday": "easterSaturday",
                    "holidayGroups": [
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäislauantai",
                    "name": "Easter Saturday"
                },
                "easterSunday": {
                    "date": "2023-04-09",
                    "holiday": "easterSunday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pääsiäispäivä",
                    "name": "Easter Sunday"
                },
                "easterMonday": {
                    "date": "2023-04-10",
                    "holiday": "easterMonday",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. pääsiäispäivä",
                    "name": "Easter Monday"
                },
                "mayDay": {
                    "date": "2023-05-01",
                    "holiday": "mayDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "vappu",
                    "name": "May Day"
                },
                "ascensionDay": {
                    "date": "2023-05-18",
                    "holiday": "ascensionDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "helatorstai",
                    "name": "Ascension Day"
                },
                "pentecost": {
                    "date": "2023-05-28",
                    "holiday": "pentecost",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "helluntaipäivä",
                    "name": "Pentecost"
                },
                "midsummerEve": {
                    "date": "2023-06-23",
                    "holiday": "midsummerEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "juhannusaatto",
                    "name": "Midsummer Eve"
                },
                "midsummerDay": {
                    "date": "2023-06-24",
                    "holiday": "midsummerDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "juhannuspäivä",
                    "name": "Midsummer Day"
                },
                "allSaintsDay": {
                    "date": "2023-11-25",
                    "holiday": "allSaintsDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "pyhäinpäivä",
                    "name": "All Saints' Day"
                },
                "independenceDay": {
                    "date": "2023-12-06",
                    "holiday": "independenceDay",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "itsenäisyyspäivä",
                    "name": "Independence Day"
                },
                "christmasEve": {
                    "date": "2023-12-24",
                    "holiday": "christmasEve",
                    "holidayGroups": [
                        "official",
                        "holiday"
                    ],
                    "localName": "jouluaatto",
                    "name": "Christmas Eve"
                },
                "christmasDay": {
                    "date": "2023-12-25",
                    "holiday": "christmasDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "joulupäivä",
                    "name": "Christmas Day"
                },
                "stStephensDay": {
                    "date": "2023-12-26",
                    "holiday": "stStephensDay",
                    "holidayGroups": [
                        "official",
                        "nonBanking",
                        "holiday"
                    ],
                    "localName": "2. joulupäivä",
                    "name": "St. Stephen's Day"
                }
            }
        },
    ];

    /**
     * Date parsing, formatting and other operations within the Salaxy framework.
     * We use ISO 8601 as our Date format even inside JavaScript objects.
     * This is because of the following reasons:
     *
     * 1. Generally, we do not want to use JavaScript Date because it presents so many problems.
     * 2. However, MomentJS is scheduled to be factored out to anothe JavaScript library
     * 3. We store dates in JSON as ISO 8601 date strings - this is universally the best practice
     * 4. ISO 8601 provides a way to present a date without time and most importantly timezone information
     * 5. ISO 8601 date also works best for most UI components (because of similar reasons as above).
     *
     * This library is only for Dates. Time / DateTime functionality is in DateTimes util library.
     *
     * Implementation uses MomentJS, but we should avoid using MomentJS outside this library:
     * It will be written out in a near future!
     */
    class Dates {
        /**
         * Gets an object and returns it as the date format that is stored in the data structure with ISO 8601 format.
         *
         * @param datelyObject - Object that should be converted to Salaxy date: A string, JS Date object, Moment etc.
         *
         * @returns The date in simplest possible ISO 8601 compliant format "YYYY-MM-DD", null if empty, not valid date etc.
         */
        static asDate(datelyObject) {
            const asMoment = this.asMoment(datelyObject);
            return asMoment ? asMoment.format("YYYY-MM-DD") : null;
        }
        /**
         * Gets a date based on year, month and day (1-based).
         * NOTE: Unlike Date() constructor or Moment() constructor, all numbers are 1-based:
         * (2017, 2, 2) results to "2017-02-02" and not "2017-01-01T22:00:00.000Z" like in those constructors.
         *
         * @param year Year component of the date or "today" for today's year.
         * @param month Month component of the date (1-12) or "today" for today's month.
         * @param day Day of month component of the date (1-31) or "today" for today's day of month.
         */
        static getDate(year, month, day) {
            year = (year === "today") ? Dates.getTodayMoment().year() : year;
            month = (month === "today") ? Dates.getTodayMoment().month() : (month - 1);
            day = (day === "today") ? Dates.getTodayMoment().date() : day;
            if (!day || month == null || month < 0 || month > 11 || !year) {
                return null;
            }
            return this.asDate([year, month, day]);
        }
        /**
         * Converts the date to a JavaScript Date. Please note, that also time part is returned.
         * @param  datelyObject - Object that should be converted: A string, JS Date object, Moment etc.
         *
         * @returns JavaScript date corresponding to input or null.
         */
        static asJSDate(datelyObject) {
            const asMoment = this.asMoment(datelyObject);
            return asMoment ? asMoment.toDate() : null;
        }
        /**
         * Checks whether a string is in ISO format.
         *
         * @param dateCancidate Value that is evaluated.
         * @param required If set to true, the value is required. If required is null the false will be returned.
         * @param format Date or datetime format. Default is ISO_8601 (ISO datetime).
         *
         * @returns True if the given date is in the correct format.
         */
        static isValidDateTime(dateCancidate, required = false, format = null) {
            if (!dateCancidate) {
                return !required;
            }
            return moment(dateCancidate, format || moment.ISO_8601, true).isValid();
        }
        /**
         * Parses a date. In ISO format by default.
         *
         * @param dateCancidate Value that is parsed.
         * @param format Date or datetime format. Default is ISO_8601 (ISO datetime).
         * Note that result is just a date (not datetime).
         *
         * @returns True if the given date is in the correct format.
         */
        static parseDate(dateCancidate, format = null) {
            if (!dateCancidate) {
                return null;
            }
            else {
                return Dates.asDate(moment(dateCancidate, format || moment.ISO_8601, true));
            }
        }
        /** Get the date of today as ISO 8601 string format (YYYY-MM-DD)  */
        static getToday() {
            return this.asDate(this.getTodayMoment());
        }
        /**
         * FOR INTERNAL USE ONLY - THIS SUPPORT WILL BE DROPPED!
         * =====================================================
         * Get the date of today as moment object.
         */
        static getTodayMoment() {
            return moment(0, "HH");
        }
        /**
         * FOR INTERNAL USE ONLY - THIS SUPPORT WILL BE DROPPED!
         * =====================================================
         * Gets the dately object as a Moment. If empty, not valid etc., returns null.
         * If the object is already a returns the given objectg (no clone).
         *
         * @param datelyObject Object that should be converted to Moment: A string, JS Date object, Moment, array of numbers etc.
         * @param allowInvalid If true, returns also invalid moment object. By default returns null, if the dately object is parsed as invalid moment.
         *
         * @returns A Moment object or null.
         */
        static asMoment(datelyObject, allowInvalid = false) {
            if (!datelyObject) {
                return null;
            }
            if (moment.isMoment(datelyObject)) {
                if (datelyObject.isValid() || allowInvalid) {
                    return datelyObject;
                }
                return null;
            }
            return this.getMoment(datelyObject, allowInvalid);
        }
        /**
         * FOR INTERNAL USE ONLY - THIS SUPPORT WILL BE DROPPED!
         * =====================================================
         * Gets the dately object as a Moment. If empty, not valid etc., returns null.
         * If the object is already a Moment, clones it.
         *
         * @param datelyObject Object that should be converted to Moment: A string, JS Date object, Moment, array of numbers etc.
         * @param allowInvalid If true, returns also invalid moment object. By default returns null, if the dately object is parsed as invalid moment.
         *
         * @returns A Moment object or null.
         */
        static getMoment(datelyObject, allowInvalid = false) {
            if (!datelyObject) {
                return null;
            }
            let asMoment;
            if (moment.isMoment(datelyObject)) {
                asMoment = datelyObject.clone();
            }
            else {
                if (moment.isDate(datelyObject) || Number.isInteger(datelyObject) || Array.isArray(datelyObject)) {
                    asMoment = moment(datelyObject);
                }
                else if (datelyObject === "today") {
                    asMoment = this.getTodayMoment();
                }
                else if (datelyObject === "yesterday") {
                    asMoment = this.getTodayMoment().add(-1, "day");
                }
                else if (datelyObject === "tomorrow") {
                    asMoment = this.getTodayMoment().add(1, "day");
                }
                else {
                    asMoment = moment(datelyObject, moment.ISO_8601);
                }
            }
            if (asMoment.isValid() || allowInvalid) {
                return asMoment;
            }
            return null;
        }
        /**
         * Returns the duration  object. If parameter is empty or null, defaults to Today (not Now).
         *
         * @param startDatelyObject Duration start that is parsed using getMoment().
         * Null before conversion defaults to today. Invalid objects result to invalid duration.
         * @param endDatelyObject Duration start that is parsed using getMoment().
         * Null before conversion defaults to today. Invalid objects result to invalid duration.
         *
         * @returns A Duration object, which may also be invalid
         */
        static getDuration(startDatelyObject, endDatelyObject) {
            startDatelyObject = this.asMoment(startDatelyObject, true) || this.getTodayMoment();
            endDatelyObject = this.asMoment(endDatelyObject, true) || this.getTodayMoment();
            return moment.duration(endDatelyObject.diff(startDatelyObject));
        }
        /**
         * Gets a date range based on start and end dates.
         * Days count is calculated using getWorkdays() (Add parameter if you need other day count methods).
         *
         * @param start DateRange start that should be converted to Salaxy date: A string, JS Date object, Moment etc.
         * @param end DateRange end that should be converted to Salaxy date: A string, JS Date object, Moment etc.
         *
         * @returns DateRange based on start and end dates.
         */
        static getDateRange(start, end) {
            const result = {
                start: this.asDate(start),
                end: this.asDate(end),
            };
            result.daysCount = this.getWorkdays(result.start, result.end).length;
            return result;
        }
        /**
         * Gets the workdays between two dates including both days.
         * Returns empty array if there is no start or if end is before start.
         * Returns 1 day if there is no end: we assume that the user selected just one day.
         *
         * @param startDate A dately value as the start date (a string, JS Date object, Moment etc.).
         * @param endDate Corresponding end date.
         * @param holidayGroup The holiday group for defining which days not to count as working day. The default group is Holiday.
         *
         * @returns The work days between the two dates (inclusive).
         */
        static getWorkdays(startDate, endDate, holidayGroup = exports.HolidayGroup.Holiday) {
            startDate = this.asDate(startDate);
            endDate = this.asDate(endDate) || startDate;
            if (!startDate) {
                return [];
            }
            const result = [];
            while (startDate <= endDate) {
                if (this.isWorkday(startDate, holidayGroup)) {
                    result.push(startDate);
                }
                startDate = this.asDate(this.getMoment(startDate).add(1, "day"));
            }
            return result;
        }
        /**
         * Gets the vacation days (6 day week Mon-Sat) between two dates including both days.
         * Returns empty array if there is no start or if end is before start.
         * Returns 1 day if there is no end: we assume that the user selected just one day.
         *
         * @param startDate A dately value as the start date (a string, JS Date object, Moment etc.).
         * @param endDate Corresponding end date.
         * @param holidayGroup The holiday group for defining which days not to count as working day. The default group is Holiday.
         *
         * @returns The work days between the two dates (inclusive).
         */
        static getVacationDays(startDate, endDate, holidayGroup = exports.HolidayGroup.Holiday) {
            startDate = this.asDate(startDate);
            endDate = this.asDate(endDate) || startDate;
            if (!startDate) {
                return [];
            }
            const result = [];
            while (startDate <= endDate) {
                if (this.isVacationday(startDate, holidayGroup)) {
                    result.push(startDate);
                }
                startDate = this.asDate(this.getMoment(startDate).add(1, "day"));
            }
            return result;
        }
        /**
         * Gets a formatted range of two dates.
         *
         * @param startDate A dately value as the start date (a string, JS Date object, Moment etc.).
         * @param endDate Corresponding end date.
         *
         * @returns A date range string with possible formats:
         * "12. - 25.11.2019", "12.11. - 02.12.2019", "12.11.2019 - 12.01.2020", "12.01.2020 -" or "- 12.01.2020".
         */
        static getFormattedRange(startDate, endDate) {
            const formatFull = "DD.MM.YYYY";
            const start = this.asMoment(startDate);
            const end = this.asMoment(endDate);
            if (!start && !end) {
                return "-";
            }
            if (!start && end) {
                return " - " + end.format(formatFull);
            }
            if (start && !end) {
                return start.format(formatFull) + " - ";
            }
            if (start.isSame(end)) {
                return start.format(formatFull);
            }
            const formatMonthDay = "DD.MM.";
            const formatDay = "DD.";
            const sameYear = start.year() === end.year();
            const sameMonth = start.month() === end.month();
            return start.format(sameYear ? (sameMonth ? formatDay : formatMonthDay) : formatFull) + " - " + end.format(formatFull);
        }
        /**
         * Output a formatted date with default date format (may later be localized).
         * @param date A dately value (a string, JS Date object, Moment etc.) that is formatted.
         * @returns A Simple date string e.g. "12.11.2019", if the value is empty or null returns "-".
         */
        static getFormattedDate(date) {
            return this.format(date, "DD.MM.YYYY", "-");
        }
        /**
         * Formats a dately object using MomentJs formatting:
         * Default is "DD.MM.YYYY"
         * For all format string options, see https://momentjs.com/docs/#/displaying/format/
         */
        static format(date, format = "DD.MM.YYYY", nullValue = null) {
            const asMoment = this.asMoment(date);
            return asMoment ? asMoment.format(format) : nullValue;
        }
        /**
         * Gets the month as 1-based number: 1-12.
         *
         * @param datelyObject - Object that represents month: A string, JS Date object, Moment etc.
         * Special string "today" can be used for fetching today's date.
         *
         * @returns Month between 1-12 or null if the object is not a dately object.
         */
        static getMonth(datelyObject) {
            const asMoment = this.asMoment(datelyObject);
            return asMoment ? asMoment.month() + 1 : null;
        }
        /**
         * Gets the year of a date or today.
         *
         * @param datelyObject - Object that represents month: A string, JS Date object, Moment etc.
         * Special string "today" can be used for fetching today's date.
         *
         * @returns Year, e.g. 2019 or null if the object is not a dately object.
         */
        static getYear(datelyObject) {
            const asMoment = this.asMoment(datelyObject);
            return asMoment ? asMoment.year() : null;
        }
        /**
         * Returns true if the given dately object is a holiday.
         * Please note that the weekend days are not by default holidays
         *
         * @param datelyObject - Object that should be converted to date: A string, JS Date object, Moment etc.
         * @param holidayGroup - The holiday group for defining which days to count as holiday day. The default group is Holiday.
         *
         * @returns True, if the given dately object is a holiday.
         */
        static isHoliday(datelyObject, holidayGroup = exports.HolidayGroup.Holiday) {
            const moment = this.asMoment(datelyObject);
            if (!moment) {
                return false;
            }
            if (!holidayGroup) {
                return false;
            }
            const holidays = this.getYearlyHolidays(moment);
            for (const holiday in holidays) {
                if (Objects.has(holidays, holiday)) {
                    const holidayDate = holidays[holiday];
                    if (this.asDate(holidayDate.date) === this.asDate(moment) &&
                        holidayDate.holidayGroups.find((x) => x === holidayGroup)) {
                        return true;
                    }
                }
            }
            return false;
        }
        /**
         * Returns true if the given dately object is a working day (5 day week).
         * The day must not be Saturday or Sunday or not any day in the given holiday categoria.
         *
         * @param datelyObject - Object that should be converted to date: A string, JS Date object, Moment etc.
         * @param holidayGroup - The holiday group for defining which days to count as holiday day. The default group is Holiday.
         *
         * @returns True, if the given dately object is a working day.
         */
        static isWorkday(datelyObject, holidayGroup = exports.HolidayGroup.Holiday) {
            const moment = this.asMoment(datelyObject);
            if (!moment) {
                return false;
            }
            if (moment.isoWeekday() === 6 || // Saturday
                moment.isoWeekday() === 7) { // Sunday
                return false;
            }
            if (!holidayGroup) {
                return true;
            }
            return !this.isHoliday(moment, holidayGroup);
        }
        /**
         * Returns true if the given dately object is a vacation day (6 day week).
         * The day must not be Sunday or not any day in the given holiday categoria.
         *
         * @param datelyObject - Object that should be converted to date: A string, JS Date object, Moment etc.
         * @param holidayGroup - The holiday group for defining which days to count as holiday day. The default group is Holiday.
         *
         * @returns True, if the given dately object is a working day.
         */
        static isVacationday(datelyObject, holidayGroup = exports.HolidayGroup.Holiday) {
            const moment = this.asMoment(datelyObject);
            if (!moment) {
                return false;
            }
            if (moment.isoWeekday() === 7) { // Sunday
                return false;
            }
            if (!holidayGroup) {
                return true;
            }
            return !this.isHoliday(moment, holidayGroup);
        }
        /**
         * Adds given amount of working days to given day.
         *
         * @param datelyObject - Object that should be converted to date: A string, JS Date object, Moment etc.
         * @param days - Working days to add (positive or negative number).
         * If zero or not set, returns datelyObject or next workday if datelyObject is not a workday.
         * @param holidayGroup - The holiday group for defining which days to count as holiday day. The default group is Holiday.
         *
         * @returns New working day with amount of given working days ahead.
         */
        static addWorkdays(datelyObject, days, holidayGroup = exports.HolidayGroup.Holiday) {
            const newMoment = this.getMoment(datelyObject);
            if (!newMoment) {
                return null;
            }
            if (!days) {
                // eslint-disable-next-line no-constant-condition
                while (true) {
                    if (this.isWorkday(newMoment, holidayGroup)) {
                        return this.asDate(newMoment);
                    }
                    newMoment.add(1, "days");
                }
            }
            let count = 0;
            const step = days > 0 ? 1 : -1;
            // eslint-disable-next-line no-constant-condition
            while (true) {
                newMoment.add(step, "days");
                if (this.isWorkday(newMoment, holidayGroup)) {
                    count += step;
                    if (count === days) {
                        return this.asDate(newMoment);
                    }
                }
            }
        }
        /**
         * Parses the given date range string to a DateRange. Parsing is implemented for optimized date
         * range inputs. Optimized date range means that dates can be given in a shorter/truncated form,
         * for improved user efficiency, by omitting some components from the date strings:
         *
         * - For the end date the day is the only mandatory component, if month and/or year are omitted
         * they default to current month and year respectively
         * - For the start date all components are optional, any omitted components default to the
         * respective components from the end date.
         *
         * For example the result of parsing a user inputted optimized date range string "1 - 20" would be
         * { start: 2021-04-01, end: 2021-04-20, }.
         *
         * @param dateRange The optimized date range string to be parsed. The dates in the date range
         * should be in finnish format (dd.mm.yyyy or without leading zeros d.m.yyyy). Start and end dates
         * should be separated by a hyphen (-). Examples of valid date range strings could be e.g.
         * "1.4 - 1.5.2021" or "1" which would be parsed as { start: 2021-04-01, end: 2021-05-01 } and
         * { start: 2021-04-01, end: 2021-04-01 } respectively (at the time of writing this documentation).
         * @returns The parsed DateRange.
         */
        static parseOptimizedDateRange(dateRange) {
            this.validateDaterangeInput(dateRange);
            return this.validateDaterange({
                start: this.parseStartDateFrom(dateRange),
                end: this.parseEndDateFrom(dateRange),
            });
        }
        /**
         * Parses a start date from an optimized date range string. Uses the end date as the default if
         * start date or some of its components are not given.
         */
        static parseStartDateFrom(dateRange) {
            if (dateRange.indexOf('-') < 0) {
                return this.parseEndDateFrom(dateRange);
            }
            const fromDate = dateRange.substring(0, dateRange.indexOf('-')).trim();
            return this.parseOptimizedDate(fromDate, this.parseEndDateFrom(dateRange));
        }
        /**
         * Parses an end date from an optimized date range string. Uses the current date as the default if
         * month and/or year are not given.
         */
        static parseEndDateFrom(dateRange) {
            const toDate = dateRange.substring(dateRange.indexOf('-') + 1).trim();
            return this.parseOptimizedDate(toDate);
        }
        /**
         * Parses an optimized date string. The date string should be in finnish format (dd.mm.yyyy with
         * or without leading zeros). Month and/or year component can be omitted from the date string
         * (optimized date string...) in which case they are taken from the default date. If default date
         * is not given the current date is used
         */
        static parseOptimizedDate(date, defaultDate = this.getToday()) {
            const dateComponents = date.split('.');
            dateComponents[0] = this.parseDay(dateComponents[0]);
            dateComponents[1] = this.parseMonth(dateComponents[1], defaultDate);
            dateComponents[2] = this.parseYear(dateComponents[2], defaultDate);
            return this.validateDate(dateComponents.reverse().join('-'));
        }
        static parseDay(day) {
            if (!day || day.length > 2) {
                return '';
            }
            return day.length === 1 ? '0' + day : day;
        }
        /**
         * Parses an optimized month string. If the month is not given it is taken from the default date.
         */
        static parseMonth(month, defaultDate) {
            if ((month === null || month === void 0 ? void 0 : month.length) > 2) {
                return '';
            }
            if (!month) {
                month = this.getMonth(defaultDate).toString();
            }
            return month.length === 1 ? '0' + month : month;
        }
        /**
         * Parses an optimized year string. Defaults to the 3rd millenium, if the given year doesn't
         * contain the millenia e.g. "21" --> "2021". If the year is not given at all, it is taken from
         * the default date.
         */
        static parseYear(year, defaultDate) {
            if ((year === null || year === void 0 ? void 0 : year.length) > 4) {
                return '';
            }
            if (!year) {
                return this.getYear(defaultDate).toString();
            }
            return (parseInt(year) % 2000 + 2000).toString();
        }
        /**
         * Validates that the given date range string only contains allowed character (numbers, periods,
         * dashes, spaces).
         */
        static validateDaterangeInput(dateRange) {
            if (dateRange.match(/^[0-9 -.]*$/) === null) {
                throw new Error('Invalid date range! The given date range contains invalid characters.');
            }
            return dateRange;
        }
        /**
         * Validates that a date is real valid date (e.g. not 31.2.2021) by trying to convert it to ISO
         * string. The conversion will throw an exception if the given date is out or range.
         */
        static validateDate(date) {
            this.asMoment(date).toISOString();
            return date;
        }
        /**
         * Validates a date range by checking that the start date is not after the end date.
         */
        static validateDaterange(dateRange) {
            if (this.asMoment(dateRange.start).isAfter(dateRange.end)) {
                throw new Error('Invalid date range! Start date needs to be before the end date.');
            }
            return dateRange;
        }
        /**
         * Gets holidays for the given year.
         *
         * @param forYear - Dately object (JavaScript date, ISO string formatted date, or moment) for which the holidays are fetched.
         *
         * @returns The holidays for the given year.
         */
        static getYearlyHolidays(forYear) {
            const year = Dates.getYear(forYear);
            const yearlyHolidays = yearlyHolidaysYears.find((x) => x.year === year);
            if (!yearlyHolidays) {
                throw new Error(`Year ${year} not supported.`);
            }
            return yearlyHolidays.holidays;
        }
    }

    /**
     * Helpers for generating barcode values.
     * Using format Code 128 (Uniform Symbology Specification)
     * with Modulo 103 algorithm for generating checksum.
     *
     * @see http://www.finanssiala.fi/maksujenvalitys/dokumentit/Bank_bar_code_guide.pdf
     * @see http://www.finanssiala.fi/maksujenvalitys/dokumentit/Pankkiviivakoodi-opas.pdf
     */
    class Barcodes {
        /**
         * Creates a value for barcode labels. Does not calculate the checksum by default.
         * Most libraries, which generate barcode labels, add the checksum and start and stop sequences automatically,
         * and those are not needed to be calculated externally.
         *
         * @param iban - IBAN bank account number. Must be a Finnish IBAN. Can have whitespace.
         * @param ref - Reference number as a string. Either fully numeric (max len 20), or starting with RF (max len 2+23).
         * Can have whitespace.
         * @param dueDate - Date as ISO date (yyyy-MM-dd). False fills values with zeros.
         * @param euroAmount - Euro amount. If string, e.g. 3030,33€, is converted to cents amount 303033.
         * Alternative can be passed cents amount as number.
         * @param includeChecksum - Calculates and appends the checksum to the end of the barcode value.
         *
         * @returns The barcode value (without start and end characters). The checksum is only calculated and appended to the value if the includeChecksum-parameter is true.
         */
        static getValue(iban, ref, dueDate, euroAmount, includeChecksum = false) {
            // --- Sanitize params
            // Remove white space from IBAN and ref
            iban = (iban + "").replace(/\s/g, "");
            ref = (ref + "").replace(/\s/g, "");
            // Format date to YYMMDD
            if (dueDate) {
                dueDate = Dates.format(dueDate, "YYMMDD");
            }
            // Format amount
            const amountInCents = (typeof euroAmount === "string") ? euroAmount.replace(/[, ]+/g, "") : euroAmount.toString();
            // --- Build value string
            let bcVal = "";
            // #1 segment: 1 char for version number - if refNo starts with "RF" => 5, else => 4
            const versionNo = ref.substring(0, 2).toUpperCase() === "RF" ? "5" : "4";
            bcVal += versionNo;
            // #2 segment: 16 chars for IBAN numeric part
            bcVal += iban.length === 18 ? iban.substr(2) : "0".repeat(18);
            // #3 segment: 6 chars euro amount without cents
            let euros = amountInCents.slice(0, -2);
            while (euros.length < 6) {
                euros = "0" + euros;
            }
            bcVal += euros;
            // #4 segment: 2 chars cents amount
            const cents = amountInCents.slice(-2);
            bcVal += (cents.length === 1) ? "0" + cents : cents;
            // #5 segment, when version 4: 3 zeros followed by 20 char national ref no
            if (versionNo === "4") {
                bcVal += "000";
                let refNo = ref.substr(0); // Copy value
                while (refNo.length < 20) {
                    refNo = "0" + refNo;
                }
                bcVal += refNo;
                // #5 segment, when version 5: 23 chars RF ref no's numeric part (RF04 9991 -> 04000000000000000009991)
            }
            else {
                bcVal += ref.substr(2, 2);
                let refNo = ref.substr(4);
                while (refNo.length < 21) {
                    refNo = "0" + refNo;
                }
                bcVal += refNo;
            }
            // #6 segment: 6 chars due date in YYMMDD format. If false, fill with zeros.
            bcVal += (dueDate) ? dueDate : "000000";
            if (includeChecksum) {
                // #7 segment: 2 chars Modulo 103 checksum
                let modulo = this.modulo103(bcVal).toString();
                modulo = modulo.length === 1 ? "0" + modulo : modulo;
                bcVal += modulo;
            }
            return bcVal;
        }
        /**
         * Takes in 54 characters string value and calculates modulo 103 value.
         *
         * @param val - 54 characters long string of numbers
         *
         * @returns A Number within range of 0-99. Reminder of dividing the sum total of weighted value pairs with 103.
         */
        static modulo103(val) {
            const pairs = val.match(/(..?)/g); // https://stackoverflow.com/questions/17286535/split-a-string-to-array-containing-pairs-of-two
            let total = 105;
            for (let i = 1; i <= 27; i++) {
                const num = Number(pairs[i - 1]);
                total += num * i;
            }
            return total % 103;
        }
    }

    /**
     * Utility for accessing brand specific style definitions.
     */
    class Brand {
        /**
         * Returns the brand color defined as a global variable.
         *
         * @param color - Named brand color to get.
         * @param format - Color format: hex like '#289548', rgb like 'rgb(40, 149, 72)' or rgba like 'rgba(40, 149, 72, 1.0'. Default value is hex.
         */
        static getBrandColor(color, format = "hex") {
            let value = Brand.getStylePropertyValue("--salaxy-brand-" + color);
            if (!value) {
                switch (color) {
                    case "primary":
                        value = "#009fd8";
                        break;
                    case "success":
                        value = "#289548";
                        break;
                    case "info":
                        value = "#91d214";
                        break;
                    case "warning":
                        value = "#ff911e";
                        break;
                    case "danger":
                        value = "#d55552";
                        break;
                }
            }
            switch (format) {
                case "rgb":
                    return Brand.hex2rgb(value);
                case "rgba":
                    return Brand.hex2rgba(value);
                case "hex":
                default:
                    return value;
            }
        }
        /**
         * Returns the css global style variable.
         * E.g. --salaxy-brand-primary
         * @param variableName - Variable name.
         */
        static getStylePropertyValue(variableName) {
            const style = getComputedStyle(document.body);
            if (!style) {
                return null;
            }
            let value = style.getPropertyValue(variableName);
            if (!value) {
                return null;
            }
            value = value.trim();
            return value;
        }
        /**
         * Returns the hex color value as rgb css function.
         * For example:  value '#289548' as 'rgba(40, 149, 72)'
         *
         * @param hexValue - Hex value.
         */
        static hex2rgb(hexValue) {
            const values = this.hex2rgbaValues(hexValue);
            if (!values) {
                return null;
            }
            return `rgb(${values[0]}, ${values[1]}, ${values[2]})`;
        }
        /**
         * Returns the hex color value as rgba css function.
         * For example:  value '#289548ff' as 'rgba(40, 149, 72, 1)'
         *
         * @param hexValue - Hex value.
         */
        static hex2rgba(hexValue) {
            const values = this.hex2rgbaValues(hexValue);
            if (!values) {
                return null;
            }
            return `rgba(${values[0]}, ${values[1]}, ${values[2]}, ${values[3].toFixed(2)})`;
        }
        /**
         * Returns the rgb value as hex string.
         * For example: 40, 149, 72 as #289548.
         *
         * @param r - red parameter.
         * @param g - green parameter.
         * @param b - blue parameter.
         */
        static rgb2hex(r, g, b) {
            /* eslint-disable-next-line */
            return "#" + ((b | g << 8 | r << 16) | 1 << 24).toString(16).substring(1);
        }
        /**
         * Returns the rgba value as hex string.
         * For example: 40, 149, 72, 1 as #289548ff.
         *
         * @param r - red parameter.
         * @param g - green parameter.
         * @param b - blue parameter.
         * @param a - alpha parameter.
         */
        static rgba2hex(r, g, b, a) {
            /* eslint-disable-next-line */
            const hexA = (Math.round(255 * a) | 1 << 8).toString(16).substring(1);
            return Brand.rgb2hex(r, g, b) + hexA;
        }
        static hex2rgbaValues(hexValue) {
            if (!hexValue) {
                return null;
            }
            hexValue = hexValue.trim().toLowerCase();
            if (hexValue.length === 0) {
                return;
            }
            if (hexValue.charAt(0) === "#") {
                // remove #
                hexValue = hexValue.substring(1);
            }
            if (hexValue.length === 3) {
                // add alpha
                hexValue += "f";
            }
            if (hexValue.length === 4) {
                // convert to 8 digits
                hexValue =
                    hexValue.charAt(0) +
                        hexValue.charAt(0) +
                        hexValue.charAt(1) +
                        hexValue.charAt(1) +
                        hexValue.charAt(2) +
                        hexValue.charAt(2) +
                        hexValue.charAt(3) +
                        hexValue.charAt(3);
            }
            if (hexValue.length === 6) {
                // add alpha
                hexValue += "ff";
            }
            if (hexValue.length !== 8) {
                // invalid length
                return null;
            }
            const r = parseInt(hexValue.substring(0, 2), 16);
            const g = parseInt(hexValue.substring(2, 4), 16);
            const b = parseInt(hexValue.substring(4, 6), 16);
            const a = ((0.0 + parseInt(hexValue.substring(6, 8), 16)) / 255.0);
            return [r, g, b, a];
        }
    }

    var SALAXY$n = {
    	ENUM: {
    		EarningsPaymentReportStatus: {
    			"new": {
    				label: "Uusi"
    			},
    			succeeded: {
    				label: "Onnistuneesti lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			},
    			invalid: {
    				label: "Tarkastettava"
    			},
    			scheduled: {
    				label: "Ajastettu"
    			}
    		},
    		PayerSummaryReportStatus: {
    			"new": {
    				label: "Uusi"
    			},
    			succeeded: {
    				label: "Onnistuneesti lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			},
    			invalid: {
    				label: "Tarkastettava"
    			},
    			scheduled: {
    				label: "Ajastettu"
    			}
    		},
    		SummaryTransactionCode: {
    			noWagesPayable: {
    				label: "Ei palkanmaksua"
    			},
    			employersHealthInsuranceContribution: {
    				label: "Sairausvakuutusmaksu"
    			},
    			deductionsToBeMadeFromTheEmployersHealthInsuranceContribution: {
    				label: "Vähennykset SAVA-maksusta"
    			}
    		},
    		ReportLogEventType: {
    			"undefined": {
    				label: "Määrittämätön"
    			},
    			sent: {
    				label: "Lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			}
    		},
    		DeliveryDataType: {
    			earningsPaymentReports: {
    				label: "Palkkatietoilmoitukset"
    			},
    			employersSeparateReports: {
    				label: "Työnantajan erillisilmoitukset"
    			},
    			benefitsPaymentReports: {
    				label: "Etuustietoilmoitukset"
    			},
    			recordSubscription: {
    				label: "Aineistotilaus"
    			},
    			cancellationOfEarningsPaymentReports: {
    				label: "Palkkatietoilmoitusten mitätöiminen"
    			},
    			cancellationOfEmployersSeparateReports: {
    				label: "Työnantajan erillisilmoitusten mitätöiminen"
    			},
    			cancellationOfBenefitsPaymentReports: {
    				label: "Etuustietoilmoitusten mitätöiminen"
    			},
    			cancellationOfARecordSubscription: {
    				label: "Aineistotilauksen mitätöiminen"
    			},
    			cancellationOfARecordContainingEarningsPaymentReports: {
    				label: "Palkkatietoilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingEmployersSeparateReports: {
    				label: "Työnantajan erillisilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingBenefitsPaymentReports: {
    				label: "Etuustietoilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingARecordSubscription: {
    				label: "Aineistotilauksen sisältävän aineiston mitätöiminen"
    			}
    		},
    		IdType: {
    			businessId: {
    				label: "Y-tunnus"
    			},
    			personalIdentificationNumber: {
    				label: "Henkilötunnus (suomalainen)"
    			},
    			vat: {
    				label: "Alv-tunniste (VAT)"
    			},
    			giin: {
    				label: "GIIN-tunniste"
    			},
    			taxIdentificationNumber: {
    				label: "Verotunniste(TIN)"
    			},
    			finnishTradeRegistrationNumber: {
    				label: "Kaupparekisteritunnus"
    			},
    			foreignBusinessRegistrationNumber: {
    				label: "Ulkomainen yritystunnus"
    			},
    			foreignPersonalIdentificationNumber: {
    				label: "Ulkomainen henkilötunnus"
    			},
    			other: {
    				label: "Muu tunnus"
    			}
    		},
    		ResponsibilityCode: {
    			contentIssues: {
    				label: "Sisällölliset asiat"
    			},
    			technicalIssues: {
    				label: "Tekniset asiat"
    			}
    		},
    		PayerType: {
    			publicSector: {
    				label: "Julkisyhteisö"
    			},
    			household: {
    				label: "Kotitalous"
    			},
    			temporaryEmployer: {
    				label: "Tilapäinen työnantaja"
    			},
    			foreignEmployer: {
    				label: "Ulkomainen työnantaja"
    			},
    			state: {
    				label: "Valtio"
    			},
    			unincorporatedStateEnterpriseOrGovernmentalInstitution: {
    				label: "Valtion liikelaitos tai erillishallinnollinen valtion laitos"
    			},
    			specialisedAgency: {
    				label: "Kansainvälinen erityisjärjestö"
    			},
    			foreignGroupCompany: {
    				label: "Ulkomainen konserniyhtiö"
    			}
    		},
    		PayerSubOrgType: {
    			kevasSubmitterCodes: {
    				label: "Kevan ilmoittajakoodisto"
    			},
    			payersOwnCodes: {
    				label: "Maksajan oma koodisto"
    			},
    			governmentAgencyIdentifier: {
    				label: "Virastotunniste"
    			}
    		},
    		AddressType: {
    			addressInHomeCountry: {
    				label: "Osoite kotivaltiossa"
    			},
    			addressInTheCountryOfWork: {
    				label: "Osoite työskentelyvaltiossa"
    			}
    		},
    		ProfessionType: {
    			statisticsFinland: {
    				label: "Tilastokeskuksen ammattiluokitus (TK10)"
    			},
    			keva: {
    				label: "Kevan nimikkeistö"
    			},
    			bankOfFinland: {
    				label: "Suomen Pankin nimikkeistö"
    			},
    			trafi: {
    				label: "Trafin nimikkeistö"
    			}
    		},
    		EmploymentCode: {
    			fullTime: {
    				label: "Kokoaikainen"
    			},
    			partTime: {
    				label: "Osa-aikainen"
    			},
    			notAvailable: {
    				label: "Tietoa ei ole saatavilla"
    			}
    		},
    		TermCode: {
    			untilFurtherNotice: {
    				label: "Toistaiseksi voimassa oleva"
    			},
    			fixedTerm: {
    				label: "Määräaikainen"
    			}
    		},
    		PaymentType: {
    			monthly: {
    				label: "Kuukausipalkka"
    			},
    			hourly: {
    				label: "Tuntipalkka"
    			},
    			contractPay: {
    				label: "Urakkapalkka"
    			}
    		},
    		EmploymentEndingType: {
    			kevaCodes: {
    				label: "Kevan koodisto"
    			},
    			bankOfFinlandCodes: {
    				label: "Suomen Pankin koodisto"
    			},
    			incomesRegisterCodes: {
    				label: "Tulorekisterin koodisto"
    			}
    		},
    		PensionActCode: {
    			employeesEarningsRelatedPensionInsurance: {
    				label: "Työntekijän työeläkevakuutus"
    			},
    			pensionInsuranceForFarmers: {
    				label: "Maatalousyrittäjän eläkevakuutus (MYEL)"
    			},
    			pensionInsuranceForTheSelfEmployed: {
    				label: "Yrittäjän eläkevakuutus (YEL)"
    			}
    		},
    		PensionProvIdCode: {
    			none: {
    				label: "Ei valintaa"
    			},
    			finnishCentreForPensions: {
    				label: "Eläketurvakeskus"
    			},
    			clergyOfTheOrthodoxChurch: {
    				label: "Ortodoksisen kirkon papiston eläkekassa"
    			},
    			kevaMemberOrganisations: {
    				label: "Kevan jäsenyhteisöt"
    			},
    			"kevaÅlandLandskapsregeringsPensionssystem": {
    				label: "Keva - Åland landskapsregerings pensionssystem"
    			},
    			kevaChurch: {
    				label: "Keva - kirkko"
    			},
    			bankOfFinland: {
    				label: "Suomen Pankki"
    			},
    			kevaKelaEmploymentPensions: {
    				label: "Keva - Kelan toimisuhde-eläkkeet"
    			},
    			KevaState: {
    				label: "Keva - valtio"
    			},
    			seafarers: {
    				label: "Merimieseläkekassa"
    			},
    			farmers: {
    				label: "Maatalousyrittäjien eläkelaitos"
    			},
    			alandia: {
    				label: "Ålandia"
    			},
    			ilmarinen: {
    				label: "Keskinäinen Eläkevakuutusyhtiö Ilmarinen"
    			},
    			elo: {
    				label: "Keskinäinen Työeläkevakuutusyhtiö Elo"
    			},
    			varma: {
    				label: "Keskinäinen työeläkevakuutusyhtiö Varma"
    			},
    			veritas: {
    				label: "Eläkevakuutusosakeyhtiö Veritas"
    			},
    			uPM: {
    				label: "UPM Sellutehtaiden eläkesäätiö"
    			},
    			sanoma: {
    				label: "Sanoman Eläkesäätiö"
    			},
    			sandvik: {
    				label: "Sandvik Eläkesäätiö"
    			},
    			kontino: {
    				label: "Kontinon yhteiseläkesäätiö"
    			},
    			yleisradio: {
    				label: "Yleisradion eläkesäätiö"
    			},
    			aBB: {
    				label: "ABB Eläkesäätiö"
    			},
    			lFashionGroup: {
    				label: "L-Fashion Group Oy:n eläkesäätiö"
    			},
    			honeywell: {
    				label: "Honeywell Oy:n Henkilökunnan Eläkesäätiö"
    			},
    			telia: {
    				label: "Telian Eläkesäätiö"
    			},
    			yaraSuomi: {
    				label: "Yara Suomen Eläkesäätiö"
    			},
    			orion: {
    				label: "Orionin Eläkesäätiö"
    			},
    			valio: {
    				label: "Valion Eläkekassa"
    			},
    			oP: {
    				label: "OP-Eläkekassa"
    			},
    			verso: {
    				label: "Eläkekassa Verso"
    			},
    			apteekkien: {
    				label: "Apteekkien Eläkekassa"
    			},
    			viabek: {
    				label: "Liikennepalvelualojen Eläkekassa Viabek"
    			},
    			reka: {
    				label: "Reka Eläkekassa"
    			},
    			etera: {
    				label: "Etera"
    			}
    		},
    		PaidAbsenceCauseCode: {
    			illness: {
    				label: "Sairaus"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			midWeekHoliday: {
    				label: "Arkipyhä"
    			},
    			accruedHoliday: {
    				label: "Pekkasvapaa"
    			},
    			occupationalAccident: {
    				label: "Työtapaturma"
    			},
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Osittainen poissaolo kuntoutuksen vuoksi"
    			},
    			other: {
    				label: "Muu syy"
    			}
    		},
    		UnpaidAbsenceCauseCode: {
    			illness: {
    				label: "Sairaus"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			jobAlternationLeave: {
    				label: "Vuorotteluvapaa"
    			},
    			studyLeave: {
    				label: "Opintovapaa"
    			},
    			industrialAction: {
    				label: "Lakko tai työsulku"
    			},
    			interruptionInWorkProvision: {
    				label: "Työtarjonnan keskeytyminen"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			militaryService: {
    				label: "Varusmiespalvelus tai siviilipalvelus"
    			},
    			layOff: {
    				label: "Lomautus"
    			},
    			childCareLeave: {
    				label: "Hoitovapaa"
    			},
    			other: {
    				label: "Muu syy"
    			}
    		},
    		AllowanceCode: {
    			mealAllowance: {
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				label: "Osapäiväraha"
    			},
    			fullDailyAllowance: {
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				label: "Ulkomaan päiväraha"
    			},
    			taxExemptReimbursementsAbroad: {
    				label: "Ulkomaan työskentelyyn liittyvät verovapaat korvaukset"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				label: "Asuntoetu"
    			},
    			telephoneBenefit: {
    				label: "Puhelinetu"
    			},
    			mealBenefit: {
    				label: "Ravintoetu"
    			},
    			otherBenefits: {
    				label: "Muut edut"
    			}
    		},
    		InsuranceCode: {
    			subjectToSocialInsuranceContributions: {
    				label: "Sosiaalivakuutusmaksujen alainen (koodit 2-6)"
    			},
    			subjectToEarningsRelatedPensionInsuranceContribution: {
    				label: "Työeläkevakuutusmaksun alainen"
    			},
    			subjectToHealthInsuranceContribution: {
    				label: "Sairausvakuutusmaksun alainen"
    			},
    			subjectToUnemploymentInsuranceContribution: {
    				label: "Työttömyysvakuutusmaksun alainen"
    			},
    			subjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution: {
    				label: "Työtapaturma- ja ammattitautivakuutusmaksun alainen"
    			}
    		},
    		WageUnitCode: {
    			hour: {
    				label: "Tunti"
    			},
    			day: {
    				label: "Päivä"
    			},
    			week: {
    				label: "Viikko"
    			},
    			period: {
    				label: "Jakso"
    			}
    		},
    		ReimbPaymentType: {
    			kelaDailyAllowanceApplication: {
    				label: "Kela, päivärahahakemus"
    			},
    			kelaFamilyLeaveCompensation: {
    				label: "Kela, perhevapaakorvaus"
    			}
    		},
    		ScheduleAction: {
    			send: {
    				label: "Lähetä"
    			},
    			cancel: {
    				label: "Mitätöi"
    			}
    		},
    		ActionCode: {
    			newReport: {
    				label: "Uusi ilmoitus"
    			},
    			replacementReport: {
    				label: "Korvaava ilmoitus"
    			}
    		},
    		Template: {
    			"": {
    				label: ""
    			}
    		}
    	}
    };

    var SALAXY$m = {
    	ENUM: {
    		ApiItemType: {
    			"undefined": {
    				description: "",
    				label: "Valitse..."
    			},
    			calculation: {
    				description: "",
    				label: "Laskelma"
    			},
    			calculationPaid: {
    				description: "",
    				label: "Maksettu laskelma"
    			},
    			payrollDetails: {
    				description: "",
    				label: "Palkkalista"
    			},
    			invoice: {
    				description: "",
    				label: "Lasku / palkanmaksu"
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "Palkkatietoilmoitus tulorekisteriin"
    			},
    			payerSummaryReport: {
    				description: "",
    				label: "Kuukausi-ilmoitus tulorekisteriin"
    			},
    			employment: {
    				description: "",
    				label: "Työsuhde"
    			},
    			messageThread: {
    				description: "",
    				label: "Viestiketju"
    			},
    			calendarEvent: {
    				description: "",
    				label: "Kalenteritapahtuma"
    			}
    		},
    		CalendarActionType: {
    			"undefined": {
    				description: "",
    				label: "Valitse..."
    			},
    			display: {
    				description: "",
    				label: "Notifikaatio"
    			},
    			email: {
    				description: "",
    				label: "Sähköposti"
    			},
    			audio: {
    				description: "",
    				label: "EI käytössä: Ääniviesti"
    			},
    			createItem: {
    				description: "",
    				label: "Tietueen luonti"
    			},
    			script: {
    				description: "",
    				label: "Automaattinen skripti"
    			}
    		},
    		Weekday: {
    			day1: {
    				label: "Maanantai",
    				short: "ma"
    			},
    			day2: {
    				label: "Tiistai",
    				short: "ti"
    			},
    			day3: {
    				label: "Keskiviikko",
    				short: "ke"
    			},
    			day4: {
    				label: "Torstai",
    				short: "to"
    			},
    			day5: {
    				label: "Perjantai",
    				short: "pe"
    			},
    			day6: {
    				label: "Lauantai",
    				short: "la"
    			},
    			day7: {
    				label: "Sunnuntai",
    				short: "su"
    			}
    		},
    		AbcSection: {
    			association: {
    				description: "",
    				label: "Yhdistyksille"
    			},
    			blog: {
    				description: "",
    				label: "Blogi"
    			},
    			businessOwner: {
    				description: "",
    				label: "Yrittäjä"
    			},
    			documentTemplates: {
    				description: "",
    				label: "Dokumenttipohjat ja lomakkeet"
    			},
    			doNotShow: {
    				description: "",
    				label: "Ei näytetä käyttäjälle (vain Admin)"
    			},
    			employee: {
    				description: "",
    				label: "Työntekijä"
    			},
    			employment: {
    				description: "",
    				label: "Työsuhde ja työsopimus"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Yritys tai yhteisö"
    			},
    			examples: {
    				description: "",
    				label: "Esimerkkejä palkkaustilanteista"
    			},
    			householdEmployer: {
    				description: "",
    				label: "Kotitalous"
    			},
    			instructionsAndExamples: {
    				description: "",
    				label: "Käyttöohjeet"
    			},
    			insurance: {
    				description: "",
    				label: "Vakuutukset ja eläke"
    			},
    			palkkausGeneral: {
    				description: "",
    				label: "Tietoa Palkkaus.fi -palvelusta"
    			},
    			palkkausInstructions: {
    				description: "",
    				label: "Käyttöohjeita"
    			},
    			personEmployer: {
    				description: "",
    				label: "Kotitalous työnantajana"
    			},
    			press: {
    				description: "",
    				label: "Tiedotteet ja materiaali"
    			},
    			productLongDescription: {
    				description: "",
    				label: "Tuotekuvaus"
    			},
    			salary: {
    				description: "",
    				label: "Palkanlaskenta ja palkanmaksu"
    			},
    			"undefined": {
    				description: "",
    				label: "Uudet artikkelit / ei määritetty"
    			},
    			worker: {
    				description: "",
    				label: "Työntekijälle"
    			}
    		},
    		AbsenceCauseCode: {
    			accruedHoliday: {
    				label: "Pekkasvapaa"
    			},
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			childCareLeave: {
    				label: "Hoitovapaa"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			illness: {
    				label: "Sairaus"
    			},
    			industrialAction: {
    				label: "Lakko tai työsulku"
    			},
    			interruptionInWorkProvision: {
    				label: "Työtarjonnan keskeytyminen"
    			},
    			jobAlternationLeave: {
    				label: "Vuorotteluvapaa"
    			},
    			layOff: {
    				label: "Lomautus"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			midWeekHoliday: {
    				label: "Arkipyhä"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			militaryService: {
    				label: "Varusmiespalvelus tai siviilipalvelus"
    			},
    			occupationalAccident: {
    				label: "Työtapaturma"
    			},
    			other: {
    				label: "Muu syy"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Osittainen poissaolo kuntoutuksen vuoksi"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			personalReason: {
    				label: "Henkilökohtainen muu syy"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			studyLeave: {
    				label: "Opintovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			unpaidLeave: {
    				label: "Palkaton loma / vapaa"
    			}
    		},
    		AgeGroupCode: {
    			a: {
    				label: "Ikäryhmä A"
    			},
    			b: {
    				label: "Ikäryhmä B"
    			},
    			c: {
    				label: "Ikäryhmä C"
    			},
    			u: {
    				label: "Ikäryhmä U"
    			}
    		},
    		AgeRange: {
    			age_15: {
    				description: "",
    				label: "-15v"
    			},
    			age16: {
    				description: "",
    				label: "16v"
    			},
    			age17: {
    				description: "",
    				label: "17v"
    			},
    			age18_52: {
    				description: "",
    				label: "18-52v"
    			},
    			age53_62: {
    				description: "",
    				label: "53-62v"
    			},
    			age63_64: {
    				description: "",
    				label: "63-64v"
    			},
    			age65_67: {
    				description: "",
    				label: "65-67v"
    			},
    			age68AndOVer: {
    				description: "",
    				label: "68v-"
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		AllowanceCode: {
    			expensesWorkingAbroad: {
    				label: "Ulkomaan työskentelyyn liittyvät verovapaat korvaukset"
    			},
    			fullDailyAllowance: {
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				label: "Ulkomaan päiväraha"
    			},
    			mealAllowance: {
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				label: "Osapäiväraha"
    			},
    			taxExemptReimbursementsAbroad: {
    				description: "",
    				label: "TaxExemptReimbursementsAbroad"
    			}
    		},
    		AnnualLeavePaymentKind: {
    			draftCalc: {
    				description: "Palkkalaskelman luonnos, jossa lomaan liittyviä maksuja",
    				label: "Palkkalaskelman luonnos"
    			},
    			endSaldo: {
    				description: "Seuraavalle lomavuodelle tai maksettavaa",
    				label: "Maksettavaa tai siirrettävää"
    			},
    			info: {
    				description: "Lomavuodella ei ole maksuja",
    				label: "Ei rivejä"
    			},
    			manualBonus: {
    				description: "Lomaraha maksettuna erikseen maksettuna erillisenä korjauksena",
    				label: "Lomaraha"
    			},
    			manualCompensation: {
    				description: "Korvattavat lomapäivät ja niiden korvaus sekä lomaraha manuaalisena korjauksena",
    				label: "Lomakorvaus"
    			},
    			manualSalary: {
    				description: "Pidetyt lomapäivät ja niiden palkka sekä lomaraha manuaalisena korjauksena",
    				label: "Loma-ajan palkka"
    			},
    			paidCalc: {
    				description: "Palkkaus.fi-palvelussa maksettu palkka, jossa lomaan liittyviä maksuja",
    				label: "Maksettu palkkalaskelma"
    			},
    			startSaldo: {
    				description: "Edellisenä vuonna kertyneet lomat",
    				label: "Käytettävissä olevat"
    			},
    			total: {
    				description: "Käytetyt/maksetut lomapäivät, -rahat ja korvaukset",
    				label: "Yhteensä"
    			},
    			"undefined": {
    				description: "Rivin tyyppiä ei ole valittu",
    				label: "Valitse..."
    			},
    			planned: {
    				description: "",
    				label: "Planned"
    			}
    		},
    		ApiTestErrorType: {
    			"default": {
    				description: "",
    				label: "Oletus"
    			},
    			userFriendly: {
    				description: "",
    				label: "User Friendly"
    			}
    		},
    		ApiValidationErrorType: {
    			general: {
    				label: "Yleisvirhe"
    			},
    			invalid: {
    				label: "Virhe"
    			},
    			required: {
    				label: "Arvo puuttuu"
    			},
    			warning: {
    				label: "Varoitus"
    			}
    		},
    		AuthenticationMethod: {
    			auth0Database: {
    				description: "",
    				label: "Sähköpostitunnistus"
    			},
    			emailPwdLocal: {
    				description: "",
    				label: "Tietokanta"
    			},
    			facebook: {
    				description: "",
    				label: "Facebook -tili"
    			},
    			google: {
    				description: "",
    				label: "Google / Gmail authentication"
    			},
    			internalLocalhost: {
    				description: "",
    				label: ""
    			},
    			linkedIn: {
    				description: "",
    				label: "LinkedIn authentication"
    			},
    			microsoft: {
    				description: "",
    				label: "Microsoft -tili"
    			},
    			salaxy: {
    				description: "",
    				label: ""
    			},
    			test: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Tuntematon"
    			},
    			x509: {
    				description: "",
    				label: ""
    			}
    		},
    		AuthorizationStatus: {
    			accessDenied: {
    				description: "",
    				label: "Pääsy kielletty"
    			},
    			expiredOauthTokdescription: "",
    			expiredOauthToklabel: "Istunto vanheutunut",
    			invalidOAuthTokdescription: "",
    			invalidOAuthToklabel: "Virhe tunnistuksessa",
    			noOauthTokdescription: "",
    			noOauthToklabel: "Virhe tunnistuksessa (ei tokenia)",
    			oK: {
    				description: "",
    				label: "OK"
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritetty"
    			},
    			expiredOauthToken: {
    				description: "",
    				label: "ExpiredOauthToken"
    			},
    			invalidOAuthToken: {
    				description: "",
    				label: "InvalidOAuthToken"
    			},
    			noOauthToken: {
    				description: "",
    				label: "NoOauthToken"
    			}
    		},
    		AuthorizationType: {
    			companyContract: {
    				description: "",
    				label: "Työnantajan sopimus"
    			},
    			employerAuthorization: {
    				description: "",
    				label: "Valtakirja"
    			},
    			manual: {
    				description: "",
    				label: "Valtakirja toimitettu erikseen"
    			},
    			none: {
    				description: "",
    				label: "Ei tunnistettu"
    			},
    			temporary: {
    				description: "",
    				label: "Temp tunnistus"
    			},
    			workerContract: {
    				description: "",
    				label: "Työntekijän sopimus"
    			}
    		},
    		AvatarPictureType: {
    			gravatar: {
    				description: "",
    				label: "Gravatar"
    			},
    			icon: {
    				description: "",
    				label: "Kuvake"
    			},
    			uploaded: {
    				description: "",
    				label: "Kuva omalta koneelta"
    			}
    		},
    		BlobFileType: {
    			authorizationPdf: {
    				description: "",
    				label: "Valtakirja"
    			},
    			avatar: {
    				description: "",
    				label: "Ikoni"
    			},
    			calcReport: {
    				description: "",
    				label: ""
    			},
    			expensesReceipt: {
    				description: "",
    				label: "Kulukuitti"
    			},
    			monthlyReport: {
    				description: "",
    				label: "Kuukausiraportti"
    			},
    			raksaloki: {
    				description: "",
    				label: "Kulukuitti"
    			},
    			taxCard: {
    				description: "",
    				label: "Verokortti"
    			},
    			template: {
    				description: "",
    				label: "Kaavain"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			},
    			yearlyReport: {
    				description: "",
    				label: "Vuosiraportti"
    			},
    			eInvoice: {
    				description: "",
    				label: "EInvoice"
    			},
    			messages: {
    				description: "",
    				label: "Messages"
    			},
    			settings: {
    				description: "",
    				label: "Settings"
    			},
    			temporary: {
    				description: "",
    				label: "Temporary"
    			}
    		},
    		BlobRepository: {
    			cdnImages: {
    				description: "",
    				label: "Content Delivery Network (CDN)"
    			},
    			fileSystemContent: {
    				description: "",
    				label: "Content in local file system (localhost/dev/test only)"
    			},
    			gitContent: {
    				description: "",
    				label: "Content in GIT"
    			},
    			payload: {
    				description: "",
    				label: "Payload of large objects"
    			},
    			reports: {
    				description: "",
    				label: "Backoffice reports"
    			},
    			systemFiles: {
    				description: "",
    				label: "System Files"
    			},
    			userFiles: {
    				description: "",
    				label: "User Files"
    			},
    			versionHistory: {
    				description: "",
    				label: "Version History"
    			}
    		},
    		BoardKind: {
    			"undefined": {
    				description: "Oletusarvo",
    				label: "Oletusarvo"
    			}
    		},
    		CalculationFlag: {
    			accidentInsurance: {
    				label: "Työtapaturma- ja ammattitautivakuutusmaksun alainen"
    			},
    			cfDeduction: {
    				label: "Vähennetään maksusta asiakasvaratililla"
    			},
    			cfDeductionAtSalaxy: {
    				label: "Vähennetään nettomaksusta asiakasvaratililtä työntekijälle"
    			},
    			cfNoPayment: {
    				label: "Ei makseta asiakasvaratilin kautta"
    			},
    			cfPayment: {
    				label: "Maksetaan asiakasvaratilin kautta"
    			},
    			exclude: {
    				label: "Ei käytetä laskelmalla"
    			},
    			healthInsurance: {
    				label: "Sairausvakuutusmaksun alainen"
    			},
    			noTax: {
    				label: "Ei vaikuta ennakonpidätykseen"
    			},
    			pensionInsurance: {
    				label: "Työeläkevakuutusmaksun alainen"
    			},
    			tax: {
    				label: "Lasketaan ennakonpidätykseen"
    			},
    			taxDeduction: {
    				label: "Vähentää ennakonpidätrystä"
    			},
    			unemploymentInsurance: {
    				label: "Työttömyysvakuutusmaksun alainen"
    			},
    			insurancesDeduction: {
    				description: "",
    				label: "InsurancesDeduction"
    			}
    		},
    		CalculationRowSource: {
    			manualRow: {
    				description: "",
    				label: "Lisärivi"
    			},
    			salaryPage: {
    				description: "",
    				label: "Pääpalkka"
    			},
    			tesSelection: {
    				description: "",
    				label: "Käyttötapaus"
    			}
    		},
    		CalculationRowType: {
    			accomodationBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Työnantajan maksaman asunnon verotusarvo.",
    				iconText: "as",
    				label: "Asuntoetu",
    				moreInfo: "",
    				priceLabel: "Asuntoetu / kk"
    			},
    			advance: {
    				amountLabel: "Määrä",
    				description: "Palkasta on saatettu jo maksaa ennakkoa. Tällöin tämä osuus vähennetään maksettavasta nettomaksusta.",
    				iconText: "adv",
    				label: "Palkkaennakko",
    				moreInfo: "",
    				priceLabel: "Maksettu ennakko"
    			},
    			board: {
    				amountLabel: "Määrä",
    				description: "Hallintoelimen jäsenyydestä maksettu palkkio",
    				iconText: "hal",
    				label: "Hallituspalkkio",
    				moreInfo: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella. ",
    				priceLabel: "Hinta"
    			},
    			carBenefit: {
    				description: "Autoetu lisätietoineen ja vähennyksineen",
    				iconText: "auto",
    				label: "Autoetu",
    				moreInfo: "Autoedulla tarkoitetaan verotettavaa etua, jonka työntekijä saa, jos saa käyttää työnantajan omistamaa tai hallinnoimaa (esim. leasing-auto) autoa työajojen lisäksi yksityisajoihin.\r\n\r\nAutoetu on palkanlisänä tiukasti säännelty. Sen arvo lasketaan verottajan laskentakaavalla ja sekä arvo että laskennan perusteet tulee ilmoittaa tulorekisteriin. Ilmoitus pitää tehdä myös silloin, jos työntekijä maksaa verotusarvosta korvauksen, jolloin verotettavaa tuloa ei synny. Ilmoitus tehdään lähtökohtaisesti palkkakausittain tai vähintään kuukausittain, jos muuta palkkaa ei makseta. Kilometrikohtaisessa laskennassa ilmoituksen saa vaihtoehtoisesti tehdä vain vuoden lopussa.\r\n\r\nLaskentakaava perustuu kahteen luokkaan: vapaa autoetu ja käyttöetu:\r\n\r\n- Vapaassa autoedussa työnantaja huolehtii kaikista auton käyttöön liittyvistä kustannuksista, kuten polttonesteistä, vakuutuksista, veroista, huolloista ja korjauksista.\r\n- Käyttöedussa auton käyttäjä maksaa yksityisajojensa polttoainekustannukset, sopimuksen mukaan mahd. myös muita kustannuksia.\r\n\r\nLisäksi laskentakaavaan tarvitaan auton hinta, lisävarusteiden hinta sekä ikäryhmä A, B tai C. Tyypillisesti arvo lasketaan kiinteällä 18 000 kilometrin vuosittaisella yksityisajolla. Jos yksityisajot ovat ilmiselvästi suuremmat, tulisi käyttää kilometreihin perustuvaa menetelmää. \r\n\r\nAutoedun verotusarvon laskuri löytyy osoitteesta: https://www.vero.fi/autoetulaskuri\r\n\r\nVaihtoehtoisesti verotusarvon voi laskea kilometripohjaisesti. Tämä on edullisempaa, jos yksityisajon kilometrit ovat vähäiset, mutta erittäin työlästä, koska silloin yksityisajoista tulee pitää ajopäiväkirjaa ja nämä kilometrit pitää raportoida verottajalle. Jos käytetään kilometrikohtaista laskentaa, autoedun voi ilmoittaa vain kerran vuodessa: tyypillisesti vuoden viimeisellä palkkalaskelmalla.",
    				priceLabel: "Autoedun verotusarvo"
    			},
    			chainsawReduction: {
    				amountLabel: "",
    				description: "Moottorisahavähennys erikoistapauksena (Ennakkoperintälaki 15 §, Ennakkoperintäasetus 18 §)",
    				iconText: "",
    				label: "Valitse...",
    				moreInfo: "",
    				priceLabel: "Määrä"
    			},
    			childCareSubsidy: {
    				amountLabel: "Määrä",
    				description: "KELA:n ja kuntien työntekijälle suoraan maksama tuki lastenhoidosta.",
    				iconText: "KELA",
    				label: "Yksityisen hoidon tuki",
    				moreInfo: "KELA:n maksama yksityisen hoidon tuki vaikuttaa sivukuihin, sillä työnantaja maksaa esim. työeläkkeen myös siltä osin. Tämä summa tulee riveinä työnantajan maksaman palkan päälle.",
    				priceLabel: "KELA maksaa"
    			},
    			compensation: {
    				amountLabel: "Määrä",
    				description: "Työstä, tehtävästä tai palveluksesta maksettu korvaus, joka ei ole palkkaa.",
    				iconText: "T",
    				label: "Työkorvaus",
    				moreInfo: "",
    				priceLabel: "Työkorvaus"
    			},
    			dailyAllowance: {
    				amountLabel: "Päiviä",
    				description: "Päiväraha: kokopäiväraha (oletus) sekä muut päivärahat",
    				iconText: "pv",
    				label: "Päiväraha",
    				moreInfo: "Päiväraha on korvaus kohtuullisesta ruokailu- ja muiden elinkustannusten lisäyksestä, jotka palkansaajalle aiheutuu työmatkasta. Kuitenkaan näihin kustannuksiin ei kuulu matkustamisesta (liput, kilometrit) eikä majoittumisesta aiheutuvia kustannuksia: Ne maksetaan tyypillisesti erikseen kuittien perusteella tai kilometrikorvauksina.\r\n\r\nVerohallinto määrittää päivärahan enimmäismäärät vuosittain. Usein käytetään näitä enimmäismääriä, mutta tämä ei ole mikään yleispätevä sääntö. Verottoman enimmäismäärän yrittävät summat tulee maksaa verotettavana palkkana tai työkorvauksena.\r\n\r\nPäivärahan saaminen edellyttää työ- tai virkamatkaa, joka suuntautuu yli 15 kilometriä työpaikalta tai kotoa riippuen siitä, kummasta alkaen matka tehdään. Osapäivärahaa maksetaan kotimaassa yli kuusi tuntia ja kokopäivärahaa yli 10 tuntia kestävältä työmatkalta. Päivärahan sijaan on myös mahdollista maksaa ateriakorvausta. Ateriakorvauksen saamisen edellytyksenä on, ettei matka oikeuta päivärahaan tai sitä ei muutoin makseta.",
    				priceLabel: "Euroa / päivä"
    			},
    			dailyAllowanceHalf: {
    				amountLabel: "Päiviä",
    				description: "Päiväraha, kun työmatka on yli 6-10 tuntia.",
    				iconText: 19,
    				label: "Osapäiväraha",
    				moreInfo: "Päivärahaa maksetaan 19 euroa, jos työmatka on yli 6 tuntia. Matkakohde pitää olla 15 km päässä joko asunnosta tai varsinaisesta työpaikasta.",
    				priceLabel: "Euroa / päivä"
    			},
    			employmentTermination: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva työsuhteen päättämiseen liittyvä korvaus",
    				iconText: "off",
    				label: "Työsuhteen päättäminen",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			eveningAddition: {
    				amountLabel: "Tuntia",
    				description: "Korvaus iltatyöstä työsopimuksen tai työehtosopimuksen mukaan.",
    				iconText: "ILTA",
    				label: "Iltavuorolisä",
    				moreInfo: "",
    				priceLabel: "Lisä (per tunti)"
    			},
    			expenses: {
    				amountLabel: "Määrä",
    				description: "Kulukorvaukset (esim. taksi, tarvikkeet) maksetaan yleensä kuittien perusteella.",
    				iconText: "kulu",
    				label: "Kulukorvaus",
    				moreInfo: "",
    				priceLabel: "Kulukorvaus"
    			},
    			foreclosure: {
    				amountLabel: "Määrä",
    				description: "Työnantajan suoraan ulosottomiehelle maksama ulosottopidätys. Tehdään palkasta maksukiellon perusteella.",
    				iconText: "ulos",
    				label: "Ulosmittaus (työnantaja)",
    				moreInfo: "Velallisen suojaosuuteen, 22,63 euroa päivässä, lisätään 8,12 euroa päivässä jokaisen hänen elatuksensa varassa olevan henkilön osalta. Velallisen elatuksen varassa olevien henkilöiden lukumäärä on mainittu maksukiellossa, jonka ulosottoviranomainen on lähettänyt palkan tai muun tulon maksajalle.\r\nSuojaosuus lasketaan kertomalla päiväkohtainen suojaosuus palkanmaksukauden päivien lukumäärällä. Jos palkka tai muu tulo maksetaan kalenterikuukaudelta, päivien lukumäärä on aina 30.\r\nUlosmitattava määrä:\r\n1. Jos palkka on pienempi kuin suojaosuus, siitä ei ulosmitata mitään.\r\n2. Jos palkka ylittää suojaosuuden, mutta on enintään kaksi kertaa suojaosuus, ulosmitataan suojaosuuden ylittävästä palkasta kaksi kolmasosaa (tulorajaulosmittaus 2/3 x (palkka – suojaosuus)).\r\n3. Jos palkka on suurempi kuin kaksi kertaa suojaosuus, mutta enintään neljä kertaa suojaosuus, ulosmitataan nettopalkasta yksi kolmasosa (1/3).\r\n4. Jos palkka on suurempi kuin neljä kertaa velallisen suojaosuuden määrä ulosmitataan yksi kolmasosa neljä kertaa velallisen suojaosuuden määrää vastaavasta palkanosasta ja lisäksi neljä viidesosaa sen ylittävästä palkanosasta. Palkasta ulosmitataan kuitenkin enintään puolet.\r\n",
    				priceLabel: "Summa"
    			},
    			foreclosureByPalkkaus: {
    				amountLabel: "",
    				description: "Palkkaus.fi-palvelun maksama ulosottopidätys. Tehdään palkasta ulosottomiehen lähettämän maksukiellon perusteella.",
    				iconText: "",
    				label: "Ulosmittaus (Palkkaus.fi)",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			holidayBonus: {
    				amountLabel: "Prosentti",
    				description: "Työehtosopimuksissa (TES) määritetty lomaraha (ent. lomaltapaluuraha).",
    				iconText: "TES",
    				label: "Lomaraha",
    				moreInfo: "",
    				priceLabel: "Loma-ajan palkka"
    			},
    			holidayCompensation: {
    				amountLabel: "Prosentti",
    				description: "Korvaus loma-ajan palkasta, jos lomaa ei pidetä.",
    				iconText: "11,5%",
    				label: "Lomakorvaus",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			holidaySalary: {
    				amountLabel: "",
    				description: "Palkka loma-ajalta.",
    				iconText: "loma",
    				label: "Loma-ajan palkka",
    				moreInfo: "",
    				priceLabel: "Loma-ajan palkka"
    			},
    			hourlySalary: {
    				amountLabel: "Tunteja",
    				description: "Palkanmaksu tehtyjen tuntien perusteella.",
    				iconText: "h",
    				label: "Tuntipalkka",
    				moreInfo: "",
    				priceLabel: "Tuntipalkka"
    			},
    			hourlySalaryWithWorkingTimeCompensation: {
    				amountLabel: "Tunteja",
    				description: "Tuntipalkka ja lisä tehtyjen tuntien perusteella.",
    				iconText: "lisä",
    				label: "Tuntipalkka",
    				moreInfo: "",
    				priceLabel: "Tuntipalkka"
    			},
    			irIncomeType: {
    				amountLabel: "Määrä",
    				description: "Erikseen syötetty tulorekisterin tulolaji, mikä ei sisälly muihin laskelman riveihin.",
    				iconText: "katre",
    				label: "Muu tulorekisterin tulolaji",
    				moreInfo: "Laskelma luo automaattisesti tulorekisterin tulolajit kullekin riville rivin tyypin perusteella. Tällä rivityyppillä voi erikseen syöttää tulorekisterin tulolajin, mikä ei sisälly muihin laskelman riveihin.",
    				priceLabel: "Hinta"
    			},
    			mealBenefit: {
    				amountLabel: "Aterioita",
    				description: "Työnantaja työntekijälleen tarjoama ateria. Myös lounassetelit.",
    				iconText: "rav",
    				label: "Ravintoetu",
    				moreInfo: "Ravintoetu on luontoisetu, joka syntyy, kun työnantaja tarjoaa työntekijälleen aterian. Työnantaja voi tukea ruokailukustannuksia ruokailulipukkeella (Lounasseteli) tai muulla kohdennetulla maksuvälineellä taikka järjestämällä työpaikkaruokailun (sopimusruokailu).\r\n\r\nRavintoedun verotusarvo määräytyy Verohallinnon päätöksen mukaisena vuosittain vahvistettavana arvona.\r\n\r\nVaihtoehtona ravintoedulle on ateriakorvaus, joka maksetaan työmatkalla. Sen maksaminen edellyttää, että työmatkasta ei suoriteta päivärahaa ja että palkansaajalla ei työn vuoksi ole mahdollisuutta ruokailutauon aikana aterioida tavanomaisella ruokailupaikallaan. Ateriakorvaus on veroton etu (maksetaan työntekijälle), kun erilaiset ravintoedut ovat luontoisetuja (yritys maksaa kulun ja siitä osa on työntekijältä verotettavaa tuloa).\r\n\r\nJos työntekijän palkanmaksupäivä on esimerkiksi kuukauden puolivälissä, tieto ravintoedun määrästä annetaan tulorekisteriin ennen kuin on varmuutta ravintoedun kokonaismäärästä kyseisenä kuukautena (esimerkiksi mahdolliset poissaolot eivät ole työnantajan tiedossa). Tämän vuoksi on katsottu riittäväksi, että ravintoetu täsmäytetään oikean suuruiseksi viimeistään seuraavalla palkkatietoilmoituksella. \r\n\r\nJoulukuun ravintoedun määrä on korjattava antamalla joulukuulle korvaava ilmoitus, jotta ravintoedun määrä korjaantuu oikealle verovuodelle. Tästä täsmäyksestä Palkkaus.fi antaa erillinen ohjeistuksen ennen vuodenvaihdetta.\r\n\r\nJos ravintoetu toteutetaan saldoon perustuvaa maksuvälinettä käyttäen (esimerkiksi maksukortti), täsmäytys on tehtävä vähintään vuoden viimeisellä palkkatietoilmoituksella.",
    				priceLabel: "Nimellisarvo / kustannus"
    			},
    			mealCompensation: {
    				amountLabel: "Päiviä",
    				description: "Ateriakorvauksen voi maksaa työmatkoilta, joilta ei makseta päivärahaa.",
    				iconText: "ater",
    				label: "Ateriakorvaus",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageDaily: {
    				amountLabel: "Päiviä",
    				description: "Kilometrikorvaus määritetään monissa TES:ssa päiväkohtaisina summina.",
    				iconText: "km",
    				label: "Kilometrikorvaus, päiväkohtainen",
    				moreInfo: "Esim. Rakennusalan työehtosopimuksissa on päiväkohtaiset taulukot, joiden perusteella maksetaan kilometrikorvauksia.",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageOther: {
    				amountLabel: "Päiviä",
    				description: "Kilometrikorvaus jollain muulla perusteella.",
    				iconText: "km",
    				label: "Kilometrikorvaus, muu peruste",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageOwnCar: {
    				amountLabel: "Kilometriä",
    				description: "Kilometrikorvaus oman auton käytöstä.",
    				iconText: "0,43€",
    				label: "Kilometrikorvaus, oma auto",
    				moreInfo: "",
    				priceLabel: "Korvaus / km"
    			},
    			monthlySalary: {
    				amountLabel: "Kuukausia",
    				description: "Kiinteä kuukausipalkka.",
    				iconText: "kk",
    				label: "Kuukausipalkka",
    				moreInfo: "",
    				priceLabel: "Kuukausipalkka"
    			},
    			nightimeAddition: {
    				amountLabel: "Tuntia",
    				description: "Korvaus yötyöstä työsopimuksen tai työehtosopimuksen mukaan.",
    				iconText: "YÖ",
    				label: "Yövuorolisä",
    				moreInfo: "",
    				priceLabel: "Lisä (per tunti)"
    			},
    			nonProfitOrg: {
    				amountLabel: "Määrä",
    				description: "Yleishyödylliset yhteisön vapaaehtoistyöhön osallistuvalle maksama päiväraha tai kilometrikorvaus.",
    				iconText: "NPO",
    				label: "Yleishyödyllisen yhteisön päiväraha ja km-korvaus",
    				moreInfo: "Yleishyödylliset yhteisöt voivat maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle päivärahaa tai kilometrikorvausta.\r\n\r\nHUOM: Tällä käyttöliittymällä ilmoitetaan vain vapaaehtoistyöhön liittyvät korvaukset. Jos korvaukset liittyvät työsuhteeseen / palkkaan, korvaukset tulee ilmoittaa erikseen kohdissa Kilometrikorvaus ja Päiväraha (kokopäiväraha tai osapäiväraha). Jos yhdistys ei ole verottajan määrittelemällä tavalla yleishyödyllinen, työsuhteen ulkopuoliset korvaukset, myös vapaaehtoistyöstä, tulee tilittää ennakonpidätyksen alaisena työkorvauksena. Vastaanottaja voi sitten vähentää ne omassa verotuksessaan.\r\n\r\nIlmoitettava tulo on määritetty tuloverolain 71 § 3 momentissa: \r\n\r\nPäivärahan voi maksaa enintään 20 päivältä per kalenterivuosi ja lisäksi majoittumiskorvaus (tositteen perusteella).\r\n\r\nKilometrikorvausten osalta voidaan maksaa matkustamiskustannusten korvaus. Toisin kuin tavallisessa kilometrikorvauksessa, tämä voidaan maksaa myös verovelvollisen asunnolta tehdystä matkasta. Muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 3 000 euroa kalenterivuodelta.\r\n\r\nTässä kohdassa ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaan päivärahaa enemmän kuin yllä, ylimenevä osa ilmoitetaan työkorvauksena. Palkkaus.fi ei tarkkaile näitä rajoja, vaan työnantajan tulee huolehtia tästä itse.",
    				priceLabel: "Hinta"
    			},
    			otherAdditions: {
    				amountLabel: "Määrä",
    				description: "Muut sovitut aikaperusteiset lisät.",
    				iconText: "lis",
    				label: "Muut lisät",
    				moreInfo: "",
    				priceLabel: "Euroa"
    			},
    			otherBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Muu työnantajan maksama verotettava etu.",
    				iconText: "etu",
    				label: "Muu etu",
    				moreInfo: "",
    				priceLabel: "Muu etu € / kk"
    			},
    			otherCompensation: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva muu korvaus tai palkkio",
    				iconText: "muu",
    				label: "Muu palkkio tai korvaus",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			otherDeductions: {
    				amountLabel: "Määrä",
    				description: "Muu vähennys, joka pienentää työntekijälle maksettavan nettopalkan määrää.",
    				iconText: "muu",
    				label: "Muu vähennys",
    				moreInfo: "",
    				priceLabel: "Vähennys"
    			},
    			overtime: {
    				amountLabel: "Prosentti",
    				description: "Ylityökorvaukset",
    				iconText: "yli",
    				label: "Ylityöt",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			phoneBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Puhelimen verotusarvo, jonka verottaja määrittää vuosittain.",
    				iconText: 20,
    				label: "Puhelinetu",
    				moreInfo: "",
    				priceLabel: "Puhelinetu / kk"
    			},
    			prepaidExpenses: {
    				amountLabel: "Määrä",
    				description: "Kulukorvaus, mikä on jo maksettu ennakkoon työntekijälle. Vähentää työntekijälle maksettavien kulukorvausten määrää.",
    				iconText: "ek",
    				label: "Ennakkoon maksettu kulukorvaus",
    				moreInfo: "",
    				priceLabel: "Maksettu ennakko"
    			},
    			remuneration: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva kertaluonteinen palkkio",
    				iconText: "ker",
    				label: "Kertaluonteinen palkkio",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			salary: {
    				amountLabel: "",
    				description: "Kertakorvaus työstä.",
    				iconText: "P",
    				label: "Palkka tai palkkio",
    				moreInfo: "",
    				priceLabel: "Palkka tai palkkio"
    			},
    			saturdayAddition: {
    				amountLabel: "Prosentti",
    				description: "Lisä lauantaina tehdystä työstä työ- tai työehtosopimuksen mukaan.",
    				iconText: "LA",
    				label: "Lauantailisä",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			subsidisedCommute: {
    				description: "Työnantajan työntekijälle kustantaman ja hankkima julkisen liikenteen henkilökohtainen joukkoliikennelippu, josta osa on verovapaata.",
    				iconText: "bus",
    				label: "Työsuhdematkalippu",
    				moreInfo: "Työsuhdematkalippu on työnantajan työntekijälle kustantaman ja hankkima julkisen liikenteen henkilökohtainen joukkoliikennelippu. Osa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista: Verovapaata on 0 - 300€ ja 750 – 3 400€ vuonna 2019. Palkkaus.fi-palvelu laskee verovapaan ja veronalaisen palkan lipun hinnan perusteella. Lisäksi otetaan huomioon työntekijän mahdollisesti työnantajalle maksama korvaus. Kaikki nämä tiedot täytyy ilmoittaa erikseen verottajalle / tulorekisteriin. \r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nMatkalippu on yleensä voimassa vuoden ja siksi työsuhdematkalipussa veronalaisen ja verovapaan edun määriä arvioidaan vuositasolla (toisin kuin muissa luontoiseduissa kuukausitasolla). \r\n\r\nJos kuitenkin korvausta peritään työntekijän palkasta säännöllisesti koko vuosi (esim. kerran kuussa) voi maksaja jo etukäteen ennakoida tilanteen ja jakaa verovapaan osuuden koko vuodelle ja ilmoittaa sen kuukausittain tulorekisteriin samassa yhteydessä perimänsä korvauksen kanssa. Tämä siitä huolimatta, että työsuhdematkalippu annettaisiin kerran vuodessa.\r\n\r\nJos haluat toteuttaa tämän jaon, sinun tulee asettaa kohta ”Jaetaanko korvaus palkkajaksoihin?” arvoksi Kyllä ja määrittää jaksojen määrä (oletus 12 kuukautta). Palkkaus.fi-palvelu ei tässä versiossa tarkkaile, että vuodessa kaikki jaksot tulee ilmoitettua oikein, joten sinun tulee vuoden lopussa tarkistaa, että kaikki työsuhdematkalipun osat on ilmoitettu. Erityisen huolellisesti pitää tarkistaa, jos vaihdat jaksojen määrää kesken vuotta (ei suositeltavaa). Ota tarvittaessa yhteyttä Palkanlaskijaasi tai Palkkaus.fi-asiakaspalveluun.",
    				priceLabel: "Lipun hinta"
    			},
    			sundayWork: {
    				amountLabel: "Määrä",
    				description: "Lain määrittämä 100% korotus sunnuntailta ja pyhäpäiviltä.",
    				iconText: "SU",
    				label: "Sunnuntai- / pyhätyö",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			tesWorktimeShortening: {
    				amountLabel: "Prosentti",
    				description: "Erillinen palkanosa on määritetty rakennusalan työehtosopimuksissa.",
    				iconText: "TES",
    				label: "Erillinen palkan osa (Raksa)",
    				moreInfo: "Ne ovat pääosin yleissitovia ja koskevat myös yksityishenkilöä. Entinen työajan lyhennysraha / \"pekkaset\".",
    				priceLabel: "Palkkasumma"
    			},
    			toolCompensation: {
    				amountLabel: "Päiviä",
    				description: "Työvälinekorvaus on määritetty joissakin työehtosopimuksissa (esim. Raksa TES).",
    				iconText: "tool",
    				label: "Työvälinekorvaus",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			totalEmployerPayment: {
    				amountLabel: "",
    				description: "Laskee palkan kokonaiskustannuksena: Mitä palkanmaksu maksaa sivukuluineen työnantajalle. Lopputulos on kertakorvaus.",
    				iconText: "sum",
    				label: "Kokonaiskustannus",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totals: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Yhteensä",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totalWorkerPayment: {
    				amountLabel: "",
    				description: "Palkan laskenta annetun summan perusteella",
    				iconText: "net",
    				label: "Nettopalkka",
    				moreInfo: "Palkan laskenta annetun summan perusteella",
    				priceLabel: ""
    			},
    			unionPayment: {
    				amountLabel: "",
    				description: "Työnantajan tulee maksaa AY-jäsenmaksu suoraan liitolle, jos työntekijä näin toivoo.",
    				iconText: "AY",
    				label: "Ay-jäsenmaksu",
    				moreInfo: "Työnantaja voi vähentää AY-jäsenmaksun suoraan työntekijän palkasta ja tilittää sen ammattiliitolle. Vähentäminen tapahtuu työntekijän pyynnöstä / suostumuksella. AY jäsenmaksu vähennetään työntekijän nettopalkasta.\n\nMenettelystä on saatettu sopia työehtosopimuksessa, jolloin AY-jäsenmaksun tilittäminen saattaa olla työnantajan velvollisuus. Siitä, koskeeko velvollisuus vain järjestäytyneitä työnantajia, ei tätä kirjoitettaessa ole täyttä varmuutta: Asiasta on eri mielipiteitä ja siitä ei ole oikeustapausta.\n\nTyypillisesti tilanne on Palkkaus.fi-asiakkailla (pienillä yrityksillä) harvinainen ja siksi AY-jäsenmaksuja ei toistaiseksi hoideta Palkkaus.fi-palvelun kautta: Työnantajan tulee itse hoitaa jäsenmaksun tilitys suoraan liittoon. Näin vähennettävä summa vähennetään Palkkaus.fi-palvelun asiakasvaratilille maksettavasta summasta. Lisätietoja ilmoittamisesta ja maksamisesta saat kyseisestä liitosta.",
    				priceLabel: ""
    			},
    			unknown: {
    				amountLabel: "Kuukausia",
    				description: "Rivin tyyppiä ei ole vielä määritetty.",
    				iconText: "?",
    				label: "Valitse...",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			workingTimeCompensation: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva aikaperusteinen lisä ",
    				iconText: "lisä",
    				label: "Aikaperusteinen lisä",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			}
    		},
    		CalculationRowUnit: {
    			count: {
    				description: "",
    				label: "kpl"
    			},
    			days: {
    				description: "",
    				label: "päivää"
    			},
    			kilometers: {
    				description: "",
    				label: "km"
    			},
    			percent: {
    				description: "",
    				label: "%"
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			hours: {
    				description: "",
    				label: "h"
    			}
    		},
    		CalculationStatus: {
    			draft: {
    				description: "",
    				label: "Luonnos"
    			},
    			externalPaymentCanceled: {
    				description: "",
    				label: "Maksu peruttu salaxy-palvelun ulkopuolella"
    			},
    			externalPaymentError: {
    				description: "",
    				label: "Virheitä (salaxy-palvelun ulkopuolella)"
    			},
    			externalPaymentStarted: {
    				description: "",
    				label: "Maksu aloitettu salaxy-palvelun ulkopuolella"
    			},
    			externalPaymentSucceeded: {
    				description: "",
    				label: "Maksettu salaxy-palvelun ulkopuolella"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "Maksu peruttu"
    			},
    			paymentError: {
    				description: "",
    				label: "Virheitä"
    			},
    			paymentRefunded: {
    				description: "",
    				label: "Maksu peruttu ja rahat palautettu"
    			},
    			paymentStarted: {
    				description: "",
    				label: "Maksu aloitettu"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "Maksettu"
    			},
    			paymentWorkerCopy: {
    				description: "",
    				label: "Työntekijän kopio"
    			},
    			payrollDraft: {
    				description: "",
    				label: "Palkkalistan laskelma"
    			},
    			proDraft: {
    				label: "Oma luonnos (pro)"
    			},
    			sharedApproved: {
    				description: "",
    				label: "Hyväksytty jaettu laskelma"
    			},
    			sharedRejected: {
    				description: "",
    				label: "Hylätty jaettu laskelma"
    			},
    			sharedWaiting: {
    				description: "",
    				label: "Vastaanotettu jaettu laskelma"
    			},
    			template: {
    				description: "",
    				label: "Toistuva laskelma"
    			},
    			workerRequestAccepted: {
    				description: "",
    				label: "Työntekijän tekemä, hyväksytty"
    			},
    			workerRequestDeclined: {
    				description: "",
    				label: "Työntekijän tekemä, hylätty"
    			},
    			workerRequested: {
    				description: "",
    				label: "Työntekijän tekemä"
    			},
    			waitingApproval: {
    				description: "",
    				label: "Yrityksen tarkistuksessa"
    			}
    		},
    		CarBenefitCode: {
    			fullCarBenefit: {
    				label: "Vapaa autoetu"
    			},
    			limitedCarBenefit: {
    				label: "Käyttöetu"
    			}
    		},
    		CarBenefitKind: {
    			fullCarBenefit: {
    				description: "Työnantaja huolehtii kaikista auton käyttöön liittyvistä kustannuksista",
    				label: "Vapaa autoetu"
    			},
    			limitedCarBenefit: {
    				description: "Auton käyttäjä maksaa yksityisajojensa polttoainekustannukset",
    				label: "Käyttöetu"
    			},
    			"undefined": {
    				description: "Autoedun tyypin valinta on pakollinen",
    				label: "Ei valintaa"
    			}
    		},
    		CompanyType: {
    			fiOy: {
    				description: "",
    				label: "Osakeyhtiö, myös asunto-osakeyhtiö ja Oyj"
    			},
    			fiRy: {
    				description: "",
    				label: "Yhdistys"
    			},
    			fiTm: {
    				description: "",
    				label: "Yksityinen elinkeinonharjoittaja tai Maatalousyrittäjä"
    			},
    			fiYy: {
    				description: "",
    				label: "Muu, esim. Kommandiittiyhtiö, Avoin yhtiö,"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		ContractPartyType: {
    			customContact: {
    				description: "",
    				label: "Custom Contact"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			}
    		},
    		DailyAllowanceKind: {
    			fullDailyAllowance: {
    				description: "Päiväraha, kun työmatka on yli 10 tuntia.",
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				description: "Päiväraha ulkomaanmatkoilla",
    				label: "Ulkomaan päiväraha"
    			},
    			mealAllowance: {
    				description: "Ateriakorvauksen voi maksaa työmatkoilta, joilta ei makseta päivärahaa. Ateriakorvauksen enimmäismäärä on 10,75€ per ateria, 2 ateriaa per päivä (21,50 € per päivä).",
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				description: "Päiväraha, kun työmatka on 6-10 tuntia.",
    				label: "Osapäiväraha"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus kokopäiväraha",
    				label: "Ei valintaa"
    			}
    		},
    		DateOfBirthAccuracy: {
    			ageBased: {
    				description: "",
    				label: "Iän perusteella"
    			},
    			ageGroupBased: {
    				description: "",
    				label: "Ikäluokan perusteella"
    			},
    			assumption: {
    				description: "",
    				label: "Oletus"
    			},
    			exact: {
    				description: "",
    				label: "Täsmällinen päivämäärä"
    			},
    			monthCorrect: {
    				description: "",
    				label: "Kuukausi oikea"
    			},
    			verified: {
    				description: "",
    				label: "Vahvistettu"
    			}
    		},
    		EmployerType: {
    			fullTimeEmployer: {
    				description: "",
    				label: "Jatkuva sopimus tai yli 8178€ / 6kk"
    			},
    			miniEmployer: {
    				description: "",
    				label: "Alle 1300€ / vuosi"
    			},
    			smallEmployer: {
    				description: "",
    				label: "Väh. 1300€ / vuosi"
    			},
    			tyelOwnContractEmployer: {
    				description: "",
    				label: "TyEL-vuosisopimus tai hoitaa itse."
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		EmploymentContract: {
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			benefitsException: {
    				label: "Benefits"
    			},
    			noticeException: {
    				label: "Irtisanomisaika"
    			},
    			otherTerms: {
    				label: "Muut sopimusehdot"
    			}
    		},
    		EmploymentRelationType: {
    			athlete: {
    				description: "",
    				label: "Urheilija"
    			},
    			boardMember: {
    				description: "",
    				label: "Hallituksen jäsen"
    			},
    			compensation: {
    				description: "",
    				label: "Suoritus työkorvauksena"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "Valtion työllisyysmäärärahoilla työllistetty (palkkatuki)"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Yrittäjä tai yrittäjän perheenjäsen"
    			},
    			entrepreneurNew: {
    				description: "",
    				label: "Yrittäjä, osaomistaja tai yrittäjän perheenjäsen"
    			},
    			farmer: {
    				description: "",
    				label: "Maatalousyrittäjä tai perheenjäsen"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Tuntipalkkainen työntekijä"
    			},
    			keyEmployee: {
    				description: "",
    				label: "Avainhenkilö. Ulkomaalainen asiantuntija, jolle maksetaan avainhenkilön lähdeveron alainen palkka."
    			},
    			monthlySalary: {
    				description: "",
    				label: "Kuukausipalkkainen työntekijä"
    			},
    			performingArtist: {
    				description: "",
    				label: "Esiintyvä taiteilija"
    			},
    			salary: {
    				description: "",
    				label: "Keikkatyöntekijä (kertaluonteinen työ)"
    			},
    			"undefined": {
    				description: "",
    				label: "Muu"
    			}
    		},
    		EmploymentSalaryType: {
    			hourlyPay: {
    				description: "",
    				label: "tuntipalkka",
    				metricAbbrev: "/h"
    			},
    			montlyPay: {
    				description: "",
    				label: "kuukausipalkka",
    				metricAbbrev: "/kk"
    			},
    			otherPay: {
    				description: "",
    				label: "muu"
    			},
    			unknown: {
    				description: "",
    				label: "Valitse..."
    			}
    		},
    		EmploymentTerminationKind: {
    			monetaryWorkingTimeBankCompensation: {
    				description: "Palkka, joka maksetaan työaikapankkiin säästetyn ajan perusteella.",
    				label: "Työaikapankin rahakorvaus"
    			},
    			noticePeriodCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa työntekijälle palvelussuhteen päättyessä ilman irtisanomisaikaa eli tilanteessa, jossa jätetään irtisanomisaika noudattamatta ja työsuhde päättyy heti (TSL 6:4). Tulolajilla ilmoitetaan myös korvaus, jonka työnantaja maksaa, kun työntekijä irtisanoutuu yli 200 päivää kestäneen lomautuksen johdosta sekä, kun työnantaja irtisanoo lomautettuna olevan työntekijän.",
    				label: "Irtisanomisajan korvaus"
    			},
    			pensionPaidByEmployer: {
    				description: "Eläke, jonka työnantaja maksaa entiselle työntekijälleen tai tämän edunsaajalle aiemman palvelussuhteen perusteella.",
    				label: "Työnantajan maksama eläke"
    			},
    			terminationAndLayOffDamages: {
    				description: "Korvaus, jota työnantaja on velvollinen maksamaan, kun työnantaja on päättänyt palvelussuhteen henkilön työsopimuslain, kunnallisesta viranhaltijasta annetun lain tai valtion virkamieslain vastaisesti. Kohdassa ilmoitetaan myös korvaus työsuhteen purkamisesta (TSL 8:1, 55/2001) sekä korvaus perusteettomasta lomauttamisesta.",
    				label: "Vahingonkorvaus päättämis- ja lomautustilanteissa"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			voluntaryTerminationCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa vapaaehtoisesti irtisanomis- tai purkamistilanteessa tai silloin, kun palvelussuhde päätetään sopimalla.",
    				label: "Vapaaehtoinen korvaus päättämistilanteessa"
    			}
    		},
    		EmploymentType: {
    			fullTime: {
    				description: "",
    				label: "Toistaiseksi voimassaoleva"
    			},
    			job: {
    				description: "",
    				label: "Kertaluonteinen työ"
    			},
    			partTime: {
    				description: "",
    				label: "Määräaikainen"
    			},
    			unknown: {
    				description: "",
    				label: "Valitse..."
    			}
    		},
    		EmploymentWorkHoursType: {
    			hourPerThreeWeeks: {
    				description: "",
    				label: "3 viikon jaksossa"
    			},
    			hourPerTwoWeeks: {
    				description: "",
    				label: "2 viikon jaksossa"
    			},
    			hoursPerDay: {
    				description: "",
    				label: "tuntia/vuorokaudessa"
    			},
    			hoursPerWeek: {
    				description: "",
    				label: "tuntia/viikkossa"
    			}
    		},
    		ForeclosureKind: {
    			fixed: {
    				description: "Ulosottoon tilitetään kiinteä summa",
    				label: "Ulosmittaus, kiinteä summa"
    			},
    			periodic: {
    				description: "Ulosottoon tilitettävä määrä lasketaan määräajoin maksettavan palkan perusteella",
    				label: "Ulosmittaus, määräajoin maks. palkka"
    			},
    			nonPeriodic: {
    				description: "Ulosottoon tilitettävä määrä lasketaan muuten kuin määräajoin maksettavan palkan perusteella",
    				label: "Ulosmittaus, ei määräajoin maks. palkka"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus kiinteä summa",
    				label: "Ei valintaa"
    			}
    		},
    		FrameworkAgreement: {
    			childCare: {
    				description: "Jatkuva lastenhoitaja ja siihen liittyvät tuet",
    				label: "Muu lastenhoito"
    			},
    			cleaning: {
    				description: "Siivous- ja kiinteistöalan työ",
    				label: "Siivous- ja kiinteistöala"
    			},
    			construction: {
    				description: "Työtä rakennusalan työehtosopimuksen mukaan",
    				label: "Rakennus- ja remonttityö"
    			},
    			entrepreneur: {
    				description: "Yrittäjän oma palkka",
    				label: "Yrittäjä"
    			},
    			mll: {
    				description: "MLL:n lastenhoitajan palkka",
    				label: "MLL suositukset"
    			},
    			notDefined: {
    				description: "Työehtosopimusta tai suositusta ei ole määritetty",
    				label: "Ei määritetty"
    			},
    			other: {
    				description: "Muu työ - ei työehtosopimusta",
    				label: "Muu työ"
    			},
    			santaClaus: {
    				description: "Joulupukin kotikäynti jouluaattona",
    				label: "Joulupukki kotiin"
    			}
    		},
    		Gender: {
    			female: {
    				description: "",
    				label: "Nainen"
    			},
    			male: {
    				description: "",
    				label: "Mies"
    			},
    			unknown: {
    				description: "",
    				label: "-"
    			}
    		},
    		HelttiIndustry: {
    			i: {
    				description: "",
    				label: "Majoitus- ja ravitsemistoiminta"
    			},
    			j: {
    				description: "",
    				label: "Informaatio ja viestintä"
    			},
    			k: {
    				description: "",
    				label: "Rahoitus- ja vakuutustoiminta"
    			},
    			m: {
    				description: "",
    				label: "Ammatillinen, tieteellinen ja tekninen toiminta"
    			},
    			n: {
    				description: "",
    				label: "Hallinto- ja tukipalvelutoiminta"
    			},
    			notDefined: {
    				description: "",
    				label: "Ei valintaa"
    			},
    			o: {
    				description: "",
    				label: "Julkinen hallinto ja maanpuolustus"
    			},
    			other: {
    				description: "",
    				label: "Muu toimiala"
    			},
    			q: {
    				description: "",
    				label: "Terveys- ja sosiaalipalvelut"
    			},
    			r: {
    				description: "",
    				label: "Taiteet, viihde ja virkistys"
    			},
    			s: {
    				description: "",
    				label: "Muu palvelutoiminta"
    			},
    			p: {
    				description: "",
    				label: "P"
    			}
    		},
    		HelttiProductPackage: {
    			large: {
    				description: "",
    				label: "Tuloskunto"
    			},
    			medium: {
    				description: "",
    				label: "Kasvuyritys"
    			},
    			notDefined: {
    				description: "",
    				label: "Ei sopimusta"
    			},
    			small: {
    				description: "",
    				label: "Start-up"
    			}
    		},
    		HolidayAccrualKind: {
    			calculation: {
    				label: "Palkkalaskelma"
    			},
    			holiday: {
    				label: "Pidetty loma"
    			},
    			holidayCompensated: {
    				label: "Lomakorvaus"
    			},
    			manual: {
    				label: "Käsin syötetty kuukausikertymä"
    			},
    			saldo: {
    				label: "Lomavuoden alku- tai loppusaldo"
    			}
    		},
    		HolidayBonusPaymentMethod: {
    			none: {
    				label: "Ei lomarahaa"
    			},
    			pay24Days: {
    				label: "Kesäloma 24 päivää (ei huomioi lomasuunnitelmia)"
    			},
    			payAllBonus: {
    				label: "Kaikki kerralla (kesäloman yhteydessä)"
    			},
    			payForHolidaySalary: {
    				label: "Maksetun loma-ajan palkan mukaan (oletus)"
    			},
    			paySummerBonus: {
    				label: "Suunnitellut kesälomat (max 24pv)"
    			}
    		},
    		HolidayCode: {
    			holidayCompensation: {
    				description: "Lomia ei kerrytetä, vaan lomakorvaus (+mahd. lomaraha) maksetaan heti palkan yhteydessä. Tyypillistä rakennusalalla ja muissa keikkatöissä.",
    				label: "Lomakorvaus palkanmaksun yhteydessä"
    			},
    			holidayCompensationIncluded: {
    				description: "Lomia ei kerrytetä, vaan lomakorvaus on sovittu osaksi kokonaispalkkaa. Tämä on mahdollista vain poikkeustilanteissa, lähinnä kotitalouksilla.",
    				label: "Lomakorvaus osa kokonaispalkkaa"
    			},
    			noHolidays: {
    				description: "Maksun saajalla ei ole lomaoikeutta. Esim. hallituksen jäsen, työkorvauksella työskentelevät, joskus toimitusjohtaja.",
    				label: "Ei lomaoikeutta"
    			},
    			permanent14Days: {
    				description: "Työntekijä työskentelee työsopimuksen mukaan kaikkina kuukausina vähintään 14 päivää / kk",
    				label: "14 päivän sääntö"
    			},
    			permanent35Hours: {
    				description: "Työntekijä työskentelee vähintään yhtenä kuukautena vuodessa 35 tuntia / kk, mutta ei kuulu 14 päivän säännön piiriin.",
    				label: "35 tunnin sääntö"
    			},
    			temporaryTimeOff: {
    				description: "Työntekijä ei kuulu 14 päivän tai 35 tunnin säännön piiriin.",
    				label: "Vapaan ansaintasääntö"
    			},
    			"undefined": {
    				description: "Loma-ajan kertymistä ei ole määritetty",
    				label: "Lisää lomakirjanpito"
    			}
    		},
    		HolidayYearInitType: {
    			createEmpty: {
    				label: "Luo uusi tyhjä lomavuosi"
    			},
    			createFromCalcs: {
    				label: "Luo lomavuosi ja lisää lomapäivät maksettujen laskelmien perusteella"
    			},
    			createMonths: {
    				label: "Luo lomavuosi ja lisää kaikki kuukaudet"
    			},
    			ignore: {
    				label: "Älä lisää tai poista lomavuotta"
    			}
    		},
    		HouseHoldUseCaseTree: {
    			childCare: {
    				description: "Tilapäinen tai jatkuva",
    				kela: {
    					description: "Lastenhoitaja kotona.",
    					label: "Yksityisen hoidon tuki"
    				},
    				label: "Lastenhoito",
    				mll: {
    					description: "Palkka MLL:n suosituksen mukaan.",
    					label: "Tilapäinen lastenhoito"
    				},
    				other: {
    					description: "Muu jatkuva tai tilapäinen lastenhoito",
    					label: "Muu lastenhoito"
    				}
    			},
    			cleaning: {
    				"51530": {
    					description: "Kiinteistöhuollon työntekijät",
    					label: "Kiinteistöhuolto"
    				},
    				"53221": {
    					description: "Kotityöpalvelutyöntekijän työ",
    					label: "Kotityöpalvelu"
    				},
    				"61132": {
    					description: "Puutarha- ja kasvihuonetyöntekijät",
    					label: "Puutarhanhoito"
    				},
    				"91110": {
    					description: "Siivoustyö tai kotiapulainen",
    					label: "Kotisiivous tai kotiapulainen"
    				},
    				description: "Kodin siivous ja kunnossapito",
    				label: "Kotisiivous",
    				other: {
    					description: "Kodin muut työt",
    					label: "Muut"
    				}
    			},
    			construction: {
    				label: "Rakennus- ja remonttityö",
    				description: "Rakennusalan työehtosopimus.",
    				carpenter: {
    					description: "Kirvesmiehet, muurarit, laattatyöt, kattotyöt",
    					label: "Talonrakennus"
    				},
    				floor: {
    					description: "Lattianpäällystys",
    					label: "Lattiatyöt"
    				},
    				"new": {
    					description: "Uudisrakennukset, ei kotital.vähennystä",
    					label: "Uudisrakentaminen"
    				},
    				painter: {
    					description: "Rakennusmaalarit ym.",
    					label: "Maalaaminen"
    				},
    				renovation: {
    					description: "Kotital.vähennyksen alainen remontointi",
    					label: "Remontointi"
    				},
    				other: {
    					description: "Muu kuin yllä kuvattu työ",
    					label: "Muu työ"
    				}
    			},
    			other: {
    				"51530": {
    					description: "",
    					label: "Kiinteistöhuollon työntekijät"
    				},
    				"53112": {
    					description: "",
    					label: "Perhepäivähoitajat (sis. kotitalousten lastenhoitajat)"
    				},
    				"53212": {
    					description: "",
    					label: "Kehitysvammaisten hoitajat"
    				},
    				"53213": {
    					description: "",
    					label: "Sosiaalialan hoitajat"
    				},
    				"53221": {
    					description: "",
    					label: "Kotityöpalvelutyöntekijät"
    				},
    				"53222": {
    					description: "",
    					label: "Henkilökohtaiset avustajat, omaishoitajat ym."
    				},
    				"61112": {
    					description: "",
    					label: "Peltoviljelytyönjohtajat ja -työntekijät"
    				},
    				"61132": {
    					description: "",
    					label: "Puutarha- ja kasvihuonetyönjohtajat ja - työntekijät"
    				},
    				"62100": {
    					description: "",
    					label: "Metsurit ja metsätyöntekijät"
    				},
    				"71110": {
    					description: "",
    					label: "Talonrakentajat (sis. remonttimiehet)"
    				},
    				"71150": {
    					description: "",
    					label: "Kirvesmiehet ja rakennuspuusepät"
    				},
    				"71210": {
    					description: "",
    					label: "Kattoasentajat ja -korjaajat"
    				},
    				"71310": {
    					description: "",
    					label: "Rakennusmaalarit ym."
    				},
    				"91110": {
    					description: "",
    					label: "Kotiapulaiset ja -siivoojat"
    				},
    				description: "Yleinen palkkalaskuri",
    				label: "Muu työ",
    				"undefined": {
    					description: "",
    					label: "Muu työ"
    				}
    			}
    		},
    		InsuranceCompany: {
    			aktia: {
    				description: "",
    				label: "Aktia"
    			},
    			alandia: {
    				description: "",
    				label: "Alandia"
    			},
    			aVakuutus: {
    				description: "",
    				label: "A-Vakuutus"
    			},
    			fennia: {
    				description: "",
    				label: "Fennia"
    			},
    			folksam: {
    				description: "",
    				label: "Folksam"
    			},
    			"if": {
    				description: "",
    				label: "If"
    			},
    			"lähiTapiola": {
    				description: "",
    				label: "LähiTapiola"
    			},
    			none: {
    				description: "",
    				label: "Ei yhtiötä"
    			},
    			other: {
    				description: "",
    				label: "Muu vakuutusyhtiö"
    			},
    			"pohjantähti": {
    				description: "",
    				label: "Pohjantähti"
    			},
    			pohjola: {
    				description: "",
    				label: "OP Vakuutus"
    			},
    			redarnas: {
    				description: "",
    				label: "Redarnas"
    			},
    			tryg: {
    				description: "",
    				label: "Tryg"
    			},
    			turva: {
    				description: "",
    				label: "Turva"
    			},
    			"ålands": {
    				description: "",
    				label: "Ålands"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			}
    		},
    		InvoiceStatus: {
    			canceled: {
    				description: "Maksu on peruttu",
    				label: "Maksu peruttu"
    			},
    			error: {
    				description: "Maksujärjestelmä on merkinnyt maksun virheelliseksi",
    				label: "Virhetila"
    			},
    			forecast: {
    				description: "Tämä lasku on Palkkaus.fi arvio: Varsinainen lasku tulee muusta kanavasta.",
    				label: "Arvio"
    			},
    			paid: {
    				description: "Maksukanava on antanut maksukuittauksen: Maksu maksetaan 99% todennäköisyydellä. 100% varmuuden saat maksukanavasta.",
    				label: "Maksettu"
    			},
    			paymentStarted: {
    				description: "Maksuprosessi on aloitettu, esim. maksu on lähetetty pankkiin.",
    				label: "Maksu aloitettu"
    			},
    			preview: {
    				description: "Esikatselulasku on tämänhetkinen tilanne. Laskuun saattaa olla tulossa vielä uusia eriä.",
    				label: "Esikatselu"
    			},
    			read: {
    				description: "Lasku on lähetetty maksujärjestelmään",
    				label: "Käsittelyssä"
    			},
    			"undefined": {
    				description: "Luonnos, ei maksujonossa",
    				label: "Valitse..."
    			},
    			unread: {
    				description: "Laskua ei vielä ole lähetetty maksujärjestelmään",
    				label: "Ei käsitelty"
    			},
    			waitingConfirmation: {
    				description: "Odottaa maksun manuaalista vahvistusta yrittäjältä / tilitoimistolta. Tulorekisteri-ilmoitus lähetetään vasta tämän jälkeen.",
    				label: "Odottaa vahvistusta"
    			},
    			waitingPalkkaus: {
    				description: "Palkkalaskelma on otettu tarkistukseen, koska siinä saattaa olla virheitä. Maksu lähtee tarkistuksen jälkeen, yleensä seuraavana arkipäivänä.",
    				label: "Tarkistuksessa"
    			}
    		},
    		IrFlags: {
    			noMoney: {
    				label: "Maksettu muuna kuin rahana (NoMoney)"
    			},
    			oneOff: {
    				label: "Kertakorvaus (OneOff)"
    			},
    			unjustEnrichment: {
    				label: "Perusteeton etu (UnjustEnrichment)"
    			}
    		},
    		IrInsuranceExceptions: {
    			excludeAccidentInsurance: {
    				label: "Poista työtapaturma- ja ammattitautivakuutusmaksun alaisuus"
    			},
    			excludeAll: {
    				label: "Poista kaikkien sosiaalivakuutusmaksujen alaisuus"
    			},
    			excludeHealthInsurance: {
    				label: "Poista sairausvakuutusmaksun alaisuus"
    			},
    			excludePension: {
    				label: "Poista työeläkevakuutusmaksun alaisuus"
    			},
    			excludeUnemployment: {
    				label: "Poista työttömyysvakuutusmaksun alaisuus"
    			},
    			includeAccidentInsurance: {
    				label: "Lisää työtapaturma- ja ammattitautivakuutusmaksun alaisuus"
    			},
    			includeAll: {
    				label: "Lisää kaikkien sosiaalivakuutusmaksujen alaisuus"
    			},
    			includeHealthInsurance: {
    				label: "Lisää sairausvakuutusmaksun alaisuus"
    			},
    			includePension: {
    				label: "Lisää työeläkevakuutusmaksun alaisuus"
    			},
    			includeUnemployment: {
    				label: "Lisää työttömyysvakuutusmaksun alaisuus"
    			}
    		},
    		LegalEntityType: {
    			company: {
    				description: "",
    				label: "Y-tunnus"
    			},
    			partner: {
    				description: "",
    				label: "Kumppani"
    			},
    			person: {
    				description: "",
    				label: "Henkilö"
    			},
    			personCreatedByEmployer: {
    				description: "",
    				label: "Työntekijä (TA:n)"
    			},
    			"undefined": {
    				description: "",
    				label: "Tuntematon"
    			}
    		},
    		MealBenefitKind: {
    			cateringContract: {
    				description: "Työnantajan järjestämä sopimusruokailu, jonka kustannus työnantajalle on verottajan vuosittain määrittämissä rajoissa (vuonna 2020/2021 6,80€ - 10,70€/6,90€ - 10.90€). Arvostetaan verotuksessa 6,80€/6,90€ per ruokailu.",
    				label: "Sopimusruokailu"
    			},
    			institute: {
    				description: "Sairaalan, koulun, päiväkodin tai muun vastaavan laitoksen henkilökuntaan kuuluvan laitosruokailun yhteydessä saaman ravintoedun raha-arvona pidetään 2020/2021 5,10€/5,18€ ateriaa kohden.",
    				label: "Sairaalan, koulun, päiväkodin tms. ruokailu"
    			},
    			mealAllowance: {
    				description: "Ateriakorvauksen on veroton korvaus (muut tämän sivun valinnat ovat ravintoetuja / luontoisetuja). Sen maksaminen edellyttää, että työmatkasta ei suoriteta päivärahaa ja että palkansaajalla ei työn vuoksi ole mahdollisuutta ruokailutauon aikana aterioida tavanomaisella ruokailupaikallaan. \nLuonnollisesti näiltä päiviltä ei voi käyttää muuta ravintoetua. Ateriakorvauksen enimmäismäärä on 2020/2021 10,75€/11,00€ per ateria, 2 ateriaa per päivä (21,50€/22,00€ per päivä).",
    				label: "Ateriakorvaus"
    			},
    			mealTicket: {
    				description: "Ruokailulipuke (Lounasseteli tai vast.) tai maksukortille ladattava tapahtuma, jonka kustannus työnantajalle on verottajan vuosittain määrittämissä rajoissa (vuonna 2020/2021 6,80€ - 10,70€/6,90€ - 10,90€). Arvostetaan verotuksessa 75 % nimellisarvosta.",
    				label: "Ruokailulipuke"
    			},
    			restaurantWorker: {
    				description: "Hotelli- ja ravintola-alan henkilökunnan ja lentohenkilöstön lennon aikana saaman ravintoedun raha-arvona pidetään 2020/2021 5,78€/5,87€ ateriaa kohden.",
    				label: "Hotelli- ja ravintola-alan ravintoetu"
    			},
    			taxableAmount: {
    				description: "Jos ravintoedun hankkimisesta työnantajalle aiheutuneiden kustannusten määrä on yli tai alle verottajan vuosittain määrittämien rajojen (2020/2021 6,80€ - 10,70€/6,90€ - 10,90€) tai jos etu muuten ylittää rajata (esim. enemmän ruokailuja kuin työpäiviä), edun arvona pidetään suoria kustannuksia / nimellisarvoa.\nKäytä tätä valintaa myös, jos et halua käyttää automaattista verotusarvon laskentaa muissa kohdissa.",
    				label: "Ravintoetu, verotusarvo"
    			},
    			teacher: {
    				description: "Koulun, päiväkodin tai vastaavan laitoksen henkilökunnan oppilaiden tai hoidettavien ruokailun valvonnan yhteydessä saaman ravintoedun arvo on 2020/2021 4,08€/4,14€ ateriaa kohden.",
    				label: "Ruokailu valvonnan yhteydessä"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus \"Verotusarvo\".",
    				label: "Ei valintaa"
    			}
    		},
    		MessageType: {
    			email: {
    				description: "",
    				label: "Sähköposti"
    			},
    			sms: {
    				description: "",
    				label: "Tekstiviesti"
    			}
    		},
    		NonProfitOrgKind: {
    			accomodationAllowance: {
    				description: "Majoittumiskorvaus (tositteen perusteella). ",
    				label: "Majoittumiskorvaus"
    			},
    			dailyAllowance: {
    				description: "Päivärahan voi maksaa enintään 20 päivältä per kalenterivuosi ja lisäksi majoittumiskorvaus (kts alla).",
    				label: "Päiväraha"
    			},
    			kilometreAllowance: {
    				description: "Matkakustannusten korvaus. Muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 3 000 euroa kalenterivuodelta.",
    				label: "Kilometrikorvaus"
    			},
    			"undefined": {
    				description: "Tyyppiä ei määritetty, oletus kilometrikorvaus",
    				label: "Tyyppiä ei määritetty"
    			}
    		},
    		OnboardingStatus: {
    			created: {
    				description: "",
    				label: "Luotu"
    			},
    			done: {
    				description: "",
    				label: "Valmis"
    			},
    			opdescription: "",
    			oplabel: "Käynnissä",
    			expired: {
    				description: "",
    				label: "Expired"
    			},
    			open: {
    				description: "",
    				label: "Open"
    			}
    		},
    		OtherCompensationKind: {
    			accruedTimeOffCompensation: {
    				description: "Työaikakorvaus, joka maksetaan ansionvähentymisestä, joka aiheutuu työajan tasaamisesta keskimääräiseen viikkotyöaikaan.",
    				label: "Työajantasauskorvaus"
    			},
    			capitalIncomePayment: {
    				description: "Suoritus, joka on saajalleen pääomatuloa. Tällainen on esimerkiksi työntekijälle tai osakkaalle maksettu korko, kuten palkkasaatavan korko tai pääomatuloa oleva takausprovisio.",
    				label: "Pääomatuloa oleva suoritus"
    			},
    			employeeInventionCompensation: {
    				description: "Palkkio, joka maksetaan työsuhdekeksinnöstä.",
    				label: "Palkkio työsuhdekeksinnöstä"
    			},
    			employeeStockOption: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa.",
    				label: "Työsuhdeoptio"
    			},
    			employeeStockOptionWithLowerPrice: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa ja jossa työntekijältä peritty option merkintähinta option luovutusajankohtana on selvästi alhaisempi kuin osakkeen markkinahinta.",
    				label: "Työsuhdeoptio, jonka merkintähinta on luovutusajankohtana alhaisempi kuin markkinahinta"
    			},
    			lectureFee: {
    				description: "Palkkio, joka maksetaan luennon pitämisestä.",
    				label: "Luentopalkkio"
    			},
    			meetingFee: {
    				description: "Palkkio, joka maksetaan kokoukseen osallistumisesta.",
    				label: "Kokouspalkkio"
    			},
    			membershipOfGoverningBodyCompensation: {
    				description: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella.",
    				label: "Hallintoelimen jäsenyydestä maksettu palkkio"
    			},
    			monetaryGiftForEmployees: {
    				description: "Työnantajalta rahana tai siihen verrattavana suorituksena saatu lahja.",
    				label: "Henkilöstölle annettu rahalahja"
    			},
    			otherTaxableIncomeAsEarnedIncome: {
    				description: "Tulolajilla ilmoitetaan muiden veronalaisten ansiotuloa olevien suoritusten yhteissumma.",
    				label: "Muu veronalainen ansiotulo"
    			},
    			positionOfTrustCompensation: {
    				description: "Palkkio, joka maksetaan muusta luottamustoimen hoitamisesta kuin kokouksiin osallistumisesta.",
    				label: "Luottamustoimipalkkio"
    			},
    			stockOptionsAndGrants: {
    				description: "Palkkio, joka annetaan työnantajayrityksen osakkeina tai sovitun palkkion sijaan rahasummana, joka vastaa osakkeiden arvoa.",
    				label: "Osakepalkkio"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			useCompensationAsCapitalIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus pääomatuloa"
    			},
    			useCompensationAsEarnedIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus ansiotuloa"
    			},
    			workEffortBasedDividendsAsNonWage: {
    				description: "Osakkaalle työkorvauksena maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (työkorvaus)"
    			},
    			workEffortBasedDividendsAsWage: {
    				description: "Osakkaalle palkkana maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (palkka)"
    			}
    		},
    		PalkkausCampaignPricing: {
    			campaignAfterFirstUse: {
    				description: "",
    				label: "Campaign After First Use"
    			},
    			campaignFirstUse: {
    				description: "",
    				label: "Campaign First Use"
    			},
    			companyFreeTrial: {
    				description: "",
    				label: "Company Free Trial"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		PartnerSite: {
    			careDotCom: {
    				description: "",
    				label: "Care Dot Com"
    			},
    			dev: {
    				description: "",
    				label: "Dev"
    			},
    			duunitori: {
    				description: "",
    				label: "Duunitori"
    			},
    			example: {
    				description: "",
    				label: "Example"
    			},
    			mol: {
    				description: "",
    				label: "Mol"
    			},
    			palkkaapakolaindescription: "",
    			palkkaapakolainlabel: "Palkkaapakolainen",
    			palkkaus: {
    				description: "",
    				label: "Palkkaus"
    			},
    			palkkausEmployerFillIn: {
    				description: "",
    				label: "Palkkaus Employer Fill In"
    			},
    			palkkausIdentified: {
    				description: "",
    				label: "Palkkaus Identified"
    			},
    			raksa: {
    				description: "",
    				label: "Raksa"
    			},
    			refugeeJobs: {
    				description: "",
    				label: "Refugee Jobs"
    			},
    			talkkarit: {
    				description: "",
    				label: "Talkkarit"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			palkkaapakolainen: {
    				description: "",
    				label: "Palkkaapakolainen"
    			}
    		},
    		PaymentChannel: {
    			"undefined": {
    				description: "Maksutapaa ei ole määritetty. Tyypillisesti tämä on virhetilanne.",
    				label: "Valitse maksutapa..."
    			},
    			test: {
    				description: "Maksumenetelmä testaamiseen. Erilliset laskut (nettopalkka, ennakonpidätys jne.) Toimii vain testiympäristössä.",
    				label: "Testimaksu"
    			},
    			zeroPayment: {
    				description: "Maksun summa on nolla. Tätä maksua ei lähetetä maksukanavaan, vaan se merkitään suoraan maksetuksi.",
    				label: "Nollamaksu (ei maksettavaa)"
    			},
    			external: {
    				description: "Maksu on jo maksettu Palkkaus.fi palvelun ulkopuolella (tai tullaan maksamaan). Maksusta tehdään kuitenkin ilmoitus tulorekisteriin.",
    				label: "Jo maksettu / ulkopuolinen"
    			},
    			palkkausManual: {
    				description: "Maksuaineisto luodaan erillisinä laskuina Maksut-listaan. Sieltä ne voidaan tulostaa PDF-tiedostona, kopoida verkkopankkiin tai ladata SEPA-maksuaineistona.",
    				label: "Maksuaineiston luonti"
    			},
    			palkkausWS: {
    				description: "Palkasta, ennakonpidätyksestä jne. luodaan erilliset laskut ja ne lähetetään pankkien WS-kanavaan. Vaatii sopimuksen pankin kanssa.",
    				label: "Maksut suoraan pankkiin"
    			},
    			palkkausPersonal: {
    				description: "Maksut luodaan erillisinä laskuina Maksut-listaan. Sieltä ne voidaan tulostaa PDF-tiedostona tai kopoida verkkopankkiin.",
    				label: "Maksujen luonti"
    			},
    			palkkausCfaPaytrail: {
    				description: "Palkka, ennakonpidätys ja työeläke yhdellä helpolla verkkomaksulla Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Paytrail verkkomaksu (yksi maksu)"
    			},
    			palkkausCfaReference: {
    				description: "PDF-paperilasku, jonka voit maksaa missä tahansa verkkopankissa tai ohjelmassa. Palkka, ennakonpidätys ja työeläke yhdessä laskussa Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Yksi tilisiirto viitenumerolla"
    			},
    			palkkausCfaFinvoice: {
    				description: "Lähetä itsellesi verkkolasku, jossa palkka, ennakonpidätys ja työeläke on yhdessä helpossa laskussa. Maksa esim. kirjanpito-ohjelmassasi. Maksu tapahtuu Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Verkkolasku (yksi maksu)"
    			},
    			palkkausCfaTest: {
    				description: "Testimaksu asiakasvaratilin kautta (yksi maksu). Toimii vain testiympäristössä.",
    				label: "Asiakasvaratilin testimaksu"
    			},
    			accountorGo: {
    				description: "Laskujen lähetys suoraan Accountor Go -palveluun. Käytössä vain Accountor Go -palvelun kautta.",
    				label: "Accountor Go"
    			},
    			talenomCfa: {
    				description: "Laskujen lähetys Talenom Online -palveluun verkkolaskuna. Käytössä vain Talenom Online -palvelun kautta.",
    				label: "Talenom verkkolasku"
    			},
    			talenomOnline: {
    				description: "Laskujen lähetys suoraan Talenom Online -palveluun. Käytössä vain Talenom Online -palvelun kautta.",
    				label: "Talenom Online"
    			},
    			holviCfa: {
    				description: "Palkka, ennakonpidätys ja työeläke yhdellä helpolla Holvi-maksulla Palkkaus.fi asiakasvaratilin kautta. Käytössä vain Holvi-palvelun kautta.",
    				label: "Holvi"
    			},
    			finagoSolo: {
    				description: "Laskujen lähetys Finago Soloon. Käytössä vain Finago Solo -palvelun kautta.",
    				label: "Finago Solo"
    			},
    			procountor: {
    				description: "Laskujen lähetys Procountoriin",
    				label: "Procountor"
    			},
    			kevytyrittaja: {
    				description: "Laskujen lähetys Talenom Kevytyrittäjä -palveluun. Käytössä vain Talenom Kevytyrittäjä -sovelluksen kautta.",
    				label: "Kevytyrittäjä"
    			},
    			vismaNetvisor: {
    				description: "Laskujen lähetys Visma Netvisor -ohjelmistoon",
    				label: "Visma Netvisor"
    			}
    		},
    		PaymentStatus: {
    			bankDelivered: {
    				description: "",
    				label: "Pankki hyväksynyt"
    			},
    			bankError: {
    				description: "",
    				label: "Virhe pankissa"
    			},
    			bankPartialError: {
    				description: "",
    				label: "Virhe pankissa"
    			},
    			bankTechApproval: {
    				description: "",
    				label: "Pankki vastaanottanut"
    			},
    			cancelled: {
    				description: "",
    				label: "Peruttu"
    			},
    			"new": {
    				description: "",
    				label: "Uusi"
    			},
    			paid: {
    				description: "",
    				label: "Maksettu"
    			},
    			sentToBank: {
    				description: "",
    				label: "Lähetetty pankkiin"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon"
    			}
    		},
    		PayrollStatus: {
    			draft: {
    				label: "Luonnos"
    			},
    			paymentCanceled: {
    				label: "Maksu peruttu"
    			},
    			paymentError: {
    				label: "Virheitä"
    			},
    			paymentStarted: {
    				label: "Maksu aloitettu"
    			},
    			paymentSucceeded: {
    				label: "Maksettu"
    			},
    			template: {
    				label: "Toistuva palkkalista"
    			},
    			waitingApproval: {
    				label: "Yrityksen tarkistuksessa"
    			}
    		},
    		PensionCalculation: {
    			athlete: {
    				description: "",
    				label: "Urheilijan eläke"
    			},
    			boardRemuneration: {
    				description: "",
    				label: "Hallituspalkkio"
    			},
    			compensation: {
    				description: "",
    				label: "Työkorvaus"
    			},
    			employee: {
    				description: "",
    				label: "TyEL"
    			},
    			entrepreneur: {
    				description: "",
    				label: "YEL"
    			},
    			farmer: {
    				description: "",
    				label: "MYEL"
    			},
    			partialOwner: {
    				description: "Osaomistajan palkka\r\nYrityksen osaomistajat maksavat pienempää palkansaajan työttömyysvakuutusmaksua kuin työntekijät.\r\n•\tOsaomistajan työnantajan työttömyysvakuutusmaksu on 0,5 % palkkasummasta.\r\n•\tPalkansaajan, joka on myös osaomistaja, työttömyysvakuutusmaksuprosentti on 0,78 %. Työnantaja pidättää sen palkasta palkanmaksun yhteydessä.\r\nJos et ole ole yrityksessä johtavassa asemassa, mutta työskentelet yrityksessä tai yhteisössä, olet osaomistaja, jos jokin seuraavista ehdoista täyttyy: \r\n•\tsinulla itselläsi on vähintään 50 % kyseisen yrityksen tai yhteisön osakepääomasta, äänivallasta tai muusta määräämisvallasta \r\n•\tperheenjäsenelläsi on vähintään 50 % kyseisen yrityksen tai yhteisön osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\n•\tsinulla on yhdessä perheenjäsentesi kanssa vähintään 50 % osakepääomasta, äänivallasta tai muusta määräämisvallasta. \r\nJos olet johtavassa asemassa mutta et ole yrittäjä, voit olla työttömyysturvalain mukaan osaomistaja, jos työskentelet yrityksessä ja jompikumpi seuraavista ehdoista täyttyy:\r\n•\tsinulla on vähintään 15 % osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\n•\tperheenjäsenilläsi tai teillä yhdessä on vähintään 30 % osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\nTyöllisyysrahaston sivulla voi testata onko osaomistaja vai palkansaaja\r\nhttps://www.tyollisyysrahasto.fi/tyottomyysvakuusmaksut/tyottomyysvakuutusmaksuvelvollisuus/#osaomistaja\r\nOsaomistajan käsite on määritelty työttömyysturvalaissa (1290/2002). \r\n",
    				label: "TyEL, osaomistaja"
    			},
    			smallEntrepreneur: {
    				description: "Tulonsaaja täyttää yrittäjän eläkelain 3 §:ssä säädetyt edellytykset, mutta jolla ei ole vakuuttamisvelvollisuutta yrittäjän eläkelain 4 §:n nojalla.\nVakuuttamisvelvollisuus ei koske:\n1) yrittäjän toimintaa ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua, eikä työskentelyä, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt yrittäjä täyttää 68 vuotta, vuosina 1958–1961 syntynyt yrittäjä täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt yrittäjä täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/72)\n2) yrittäjän toimintaa, joka ei ole jatkunut yhdenjaksoisesti vähintään neljää kuukautta;\n3) yrittäjän toimintaa, joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) yrittäjää, jonka työtulo tässä laissa tarkoitetusta yrittäjätoiminnasta on arvioitava pienemmäksi kuin 5 504,14 euroa vuodessa;\n5) ansiotyötä, jonka perusteella yrittäjällä on oikeus eläkkeeseen muun työeläkelain mukaan; eikä\n6) yrittäjää, johon ei sovelleta Suomen lainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/355)",
    				label: "YEL, ei vakuuttamisvelvollisuutta"
    			},
    			smallFarmer: {
    				description: "Tulonsaaja täyttää maatalousyrittäjän eläkelain 3 §:ssä säädetyt edellytykset, mutta jolla ei ole mainitun lain perusteella vakuuttamisvelvollisuutta.\n\nVakuuttamisvelvollisuus ei koske:\n1) maatalousyrittäjän toimintaa ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua eikä toimintaa, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt maatalousyrittäjä täyttää 68 vuotta, vuosina 1958–1961 syntynyt maatalousyrittäjä täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt maatalousyrittäjä täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/75)\n2) maatalousyrittäjän toimintaa, joka ei ole jatkunut 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alun jälkeen yhdenjaksoisesti vähintään neljää kuukautta;\n3) maatalousyrittäjän toimintaa, joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) maatalousyrittäjää, jonka työtulo tässä laissa tarkoitetusta maatalousyrittäjätoiminnasta on arvioitava pienemmäksi kuin 2 752,07 euroa vuodessa;\n5) maatilatalouden, kalastuksen tai poronhoidon yhteydessä harjoitettua toimintaa, jota on pidettävä eri yrityksenä;\n6) ansiotyötä, jonka perusteella maatalousyrittäjällä on oikeus eläkkeeseen muun työeläkelain mukaan; eikä\n7) maatalousyrittäjää, johon ei sovelleta Suomen lainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/357)\n\nVakuuttamisvelvollisuus ei koske apurahansaajan työtä:\n1) ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua, eikä työtä, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt apurahansaaja täyttää 68 vuotta, vuosina 1958–1961 syntynyt apurahansaaja täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt apurahansaaja täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/75)\n2) joka ei ole jatkunut 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alun jälkeen yhdenjaksoisesti vähintään neljä kuukautta;\n3) joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) ajalta, jona apurahansaajalle arvioitava vuotuinen työtulo on pienempi kuin 2 752,07 euroa;\n5) jos apurahansaajaan ei sovelleta Suomen sosiaaliturvalainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/357)",
    				label: "MYEL, ei vakuuttamisvelvollisuutta"
    			},
    			"undefined": {
    				description: "",
    				label: "(Ei valittu vielä)"
    			}
    		},
    		PensionCompany: {
    			apteekkien: {
    				description: "",
    				label: "Apteekkien Eläkekassa"
    			},
    			elo: {
    				description: "",
    				label: "Elo"
    			},
    			etera: {
    				description: "",
    				label: "Etera"
    			},
    			ilmarindescription: "",
    			ilmarinen: {
    				label: "Ilmarinen"
    			},
    			none: {
    				description: "",
    				label: "Ei yhtiötä"
    			},
    			other: {
    				description: "",
    				label: "Muu yhtiö"
    			},
    			pensionsAlandia: {
    				description: "",
    				label: "Alandia"
    			},
    			varma: {
    				description: "",
    				label: "Varma"
    			},
    			veritas: {
    				description: "",
    				label: "Veritas"
    			},
    			verso: {
    				description: "",
    				label: "Verso"
    			}
    		},
    		PeriodType: {
    			custom: {
    				label: "Muu periodi"
    			},
    			month: {
    				label: "Kuukausi"
    			},
    			quarter: {
    				label: "Neljännesvuosi"
    			},
    			year: {
    				label: "Kalenterivuosi"
    			}
    		},
    		PricingModel: {
    			fixedFee: {
    				label: "Kiinteä palkkio"
    			},
    			noFee: {
    				label: "Ei palkkiota"
    			},
    			palkkausFee: {
    				label: "Palkkaus.fi-palkkio (1%)"
    			}
    		},
    		Product: {
    			accounting: {
    				description: "Palkkalaskelmien tiedot helposti kirjanpitäjälle",
    				label: "Kirjanpitoraportit",
    				status: "Ei käytössä"
    			},
    			baseService: {
    				description: "Digiajan palkanmaksupalvelu",
    				label: "Palkkaus.fi-tili",
    				status: "Palkkaamisen peruspalvelut"
    			},
    			healthCareHeltti: {
    				description: "Tuottavammat ja terveemmät työntekijät",
    				label: "Työterveyshuolto",
    				status: "Ei sopimusta"
    			},
    			insurance: {
    				description: "Tapaturmavakuutus vakuuttaa työntekijäsi",
    				label: "Työtapaturmavakuutus",
    				status: "Ei sopimusta. HUOM: työnantajan 1300 € :n vuosisääntö!"
    			},
    			pension: {
    				description: "TyEL-vakuutus on työntekijän eläketurva",
    				label: "TyEL-eläkevakuutus",
    				status: "Ei sopimusta: TyEL maksetaan tilapäisenä työnantajana"
    			},
    			pensionEntrepreneur: {
    				description: "YEL-vakuutus on yrittäjän eläketurva",
    				label: "Yrittäjän eläke",
    				status: "Ei YEL-sopimusta"
    			},
    			tax: {
    				description: "Ennakonpidätykset ja sava-maksu",
    				label: "Yhteydet verottajaan",
    				status: "Ennakonpidätys ja ilmoitukset"
    			},
    			taxCard: {
    				description: "Työntekijän verokortti ja sen käsittely",
    				label: "Verokortti (TT)",
    				status: "Ei verokorttia"
    			},
    			unemployment: {
    				description: "Työttömyysturva rahoitetaan TVR-maksuilla",
    				label: "Työttömyysvakuutus (TVR)",
    				status: "Ilmoitukset TVR:lle",
    				statusTVR: "Ilmoitukset TVR:lle"
    			}
    		},
    		ProductListFilter: {
    			all: {
    				description: "",
    				label: "All products, even the currently non-available."
    			},
    			available: {
    				description: "",
    				label: "Available products for this user"
    			}
    		},
    		Profession: {
    			"0": {
    				label: "Sotilaat"
    			},
    			"1": {
    				label: "Johtajat"
    			},
    			"2": {
    				label: "Erityisasiantuntijat"
    			},
    			"3": {
    				label: "Asiantuntijat"
    			},
    			"4": {
    				label: "Toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"5": {
    				label: "Palvelu- ja myyntityöntekijät"
    			},
    			"6": {
    				label: "Maanviljelijät, metsätyöntekijät ym."
    			},
    			"7": {
    				label: "Rakennus-, korjaus- ja valmistustyöntekijät"
    			},
    			"8": {
    				label: "Prosessi- ja kuljetustyöntekijät"
    			},
    			"9": {
    				label: "Muut työntekijät"
    			},
    			"11": {
    				label: "Johtajat, ylimmät virkamiehet ja järjestöjen johtajat"
    			},
    			"12": {
    				label: "Hallintojohtajat ja kaupalliset johtajat"
    			},
    			"13": {
    				label: "Tuotantotoiminnan ja yhteiskunnan peruspalvelujen johtajat"
    			},
    			"14": {
    				label: "Hotelli- ja ravintola-alan, vähittäiskaupan ja muiden palvelualojen johtajat"
    			},
    			"21": {
    				label: "Luonnontieteiden ja tekniikan erityisasiantuntijat"
    			},
    			"22": {
    				label: "Terveydenhuollon erityisasiantuntijat"
    			},
    			"23": {
    				label: "Opettajat ja muut opetusalan erityisasiantuntijat"
    			},
    			"24": {
    				label: "Liike-elämän ja hallinnon erityisasiantuntijat"
    			},
    			"25": {
    				label: "Tieto- ja viestintäteknologian erityisasiantuntijat"
    			},
    			"26": {
    				label: "Lainopilliset, sosiaalialan ja kulttuurialan erityisasiantuntijat"
    			},
    			"31": {
    				label: "Luonnontieteiden ja tekniikan asiantuntijat"
    			},
    			"32": {
    				label: "Terveydenhuollon asiantuntijat"
    			},
    			"33": {
    				label: "Liike-elämän ja hallinnon asiantuntijat"
    			},
    			"34": {
    				label: "Lainopilliset avustajat sekä sosiaali- ja kulttuurialan asiantuntijat"
    			},
    			"35": {
    				label: "Informaatio- ja tietoliikenneteknologian asiantuntijat"
    			},
    			"41": {
    				label: "Toimistotyöntekijät"
    			},
    			"42": {
    				label: "Asiakaspalvelutyöntekijät"
    			},
    			"43": {
    				label: "Laskennan ja varastoinnin toimistotyöntekijät"
    			},
    			"44": {
    				label: "Muut toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"51": {
    				label: "Palvelutyöntekijät"
    			},
    			"52": {
    				label: "Myyjät, kauppiaat ym."
    			},
    			"53": {
    				label: "Hoivapalvelun ja terveydenhuollon työntekijät"
    			},
    			"54": {
    				label: "Suojelu- ja vartiointityöntekijät"
    			},
    			"61": {
    				label: "Maanviljelijät ja eläintenkasvattajat ym."
    			},
    			"62": {
    				label: "Metsä- ja kalatalouden työntekijät"
    			},
    			"63": {
    				label: "Kotitarveviljelijät, -kalastajat ja -metsästäjät"
    			},
    			"71": {
    				label: "Rakennustyöntekijät ym. (pl. sähköasentajat)"
    			},
    			"72": {
    				label: "Konepaja- ja valimotyöntekijät sekä asentajat ja korjaajat"
    			},
    			"73": {
    				label: "Käsityötuotteiden valmistajat, hienomekaanikot sekä painoalan työntekijät"
    			},
    			"74": {
    				label: "Sähkö- ja elektroniikka-alan työntekijät"
    			},
    			"75": {
    				label: "Elintarvike-, puutyö- ja vaatetus- ja jalkinealan valmistustyöntekijät ym."
    			},
    			"81": {
    				label: "Prosessityöntekijät"
    			},
    			"82": {
    				label: "Teollisuustuotteiden kokoonpanijat"
    			},
    			"83": {
    				label: "Kuljetustyöntekijät"
    			},
    			"91": {
    				label: "Siivoojat, kotiapulaiset ja muut puhdistustyöntekijät"
    			},
    			"92": {
    				label: "Maa-, metsä- ja kalatalouden avustavat työntekijät"
    			},
    			"93": {
    				label: "Teollisuuden ja rakentamisen avustavat työntekijät"
    			},
    			"94": {
    				label: "Avustavat keittiö- ja ruokatyöntekijät"
    			},
    			"95": {
    				label: "Katumyyjät, kengänkiillottajat ym."
    			},
    			"96": {
    				label: "Katujen puhtaanapidon ja jätehuollon työntekijät ym."
    			},
    			"111": {
    				label: "Ylimmät virkamiehet ja järjestöjen johtajat"
    			},
    			"112": {
    				label: "Toimitusjohtajat ja pääjohtajat"
    			},
    			"121": {
    				label: "Liiketoiminta- ja hallintojohtajat"
    			},
    			"122": {
    				label: "Myynti-, markkinointi- ja kehitysjohtajat"
    			},
    			"131": {
    				label: "Maa-, metsä- ja kalatalouden johtajat"
    			},
    			"132": {
    				label: "Teollisuuden tuotantojohtajat sekä kaivos-, rakennus- ja jakelujohtajat"
    			},
    			"133": {
    				label: "Tieto- ja viestintäteknologiajohtajat"
    			},
    			"134": {
    				label: "Yhteiskunnan peruspalvelujen sekä rahoitus- ja vakuutuspalvelujen johtajat"
    			},
    			"141": {
    				label: "Hotellin- ja ravintolanjohtajat"
    			},
    			"142": {
    				label: "Vähittäis- ja tukkukaupan johtajat"
    			},
    			"143": {
    				label: "Muiden palvelualojen johtajat"
    			},
    			"211": {
    				label: "Luonnon- ja geotieteen erityisasiantuntijat"
    			},
    			"212": {
    				label: "Matematiikan ja tilastotieteen erityisasiantuntijat"
    			},
    			"213": {
    				label: "Biotieteiden erityisasiantuntijat"
    			},
    			"214": {
    				label: "Tekniikan erityisasiantuntijat (pl. sähköteknologia)"
    			},
    			"215": {
    				label: "Sähköteknologian erityisasiantuntijat"
    			},
    			"216": {
    				label: "Arkkitehdit, suunnittelijat ja maanmittaajat"
    			},
    			"221": {
    				label: "Lääkärit"
    			},
    			"222": {
    				label: "Hoitotyön erityisasiantuntijat"
    			},
    			"223": {
    				label: "Perinteisen ja vaihtoehtohoidon erityisasiantuntijat"
    			},
    			"224": {
    				label: "Muiden terveyspalvelujen tuottajat"
    			},
    			"225": {
    				label: "Eläinlääkärit"
    			},
    			"226": {
    				label: "Muut terveydenhuollon erityisasiantuntijat"
    			},
    			"231": {
    				label: "Yliopisto- ja korkeakouluopettajat"
    			},
    			"232": {
    				label: "Ammatillisen koulutuksen opettajat"
    			},
    			"233": {
    				label: "Lukion ja peruskoulun yläluokkien opettajat"
    			},
    			"234": {
    				label: "Peruskoulun alaluokkien opettajat ja lastentarhanopettajat"
    			},
    			"235": {
    				label: "Muut opetusalan erityisasiantuntijat"
    			},
    			"241": {
    				label: "Rahoitusalan erityisasiantuntijat"
    			},
    			"242": {
    				label: "Hallinnon erityisasiantuntijat"
    			},
    			"243": {
    				label: "Myynnin, markkinoinnin ja tiedotuksen erityisasiantuntijat"
    			},
    			"251": {
    				label: "Systeemityön erityisasiantuntijat"
    			},
    			"252": {
    				label: "Tietokantojen,- verkkojen ja järjestelmäohjelmistojen erityisasiantuntijat"
    			},
    			"261": {
    				label: "Lainopilliset erityisasiantuntijat"
    			},
    			"262": {
    				label: "Kirjaston- ja arkistonhoitajat sekä museoalan erityisasiantuntijat"
    			},
    			"263": {
    				label: "Yhteiskunta- ja sosiaalialan sekä uskonnollisen elämän erityisasiantuntijat"
    			},
    			"264": {
    				label: "Toimittajat, kirjailijat ja kielitieteilijät"
    			},
    			"265": {
    				label: "Taiteilijat"
    			},
    			"311": {
    				label: "Fysiikan, kemian ja teknisten alojen asiantuntijat"
    			},
    			"312": {
    				label: "Työnjohtajat kaivos-, teollisuus- ja rakennustoiminnassa"
    			},
    			"313": {
    				label: "Prosessinvalvonnan asiantuntijat"
    			},
    			"314": {
    				label: "Biotieteiden asiantuntijat"
    			},
    			"315": {
    				label: "Laiva-, lento- ja satamaliikenteen päälliköt ja ohjaajat"
    			},
    			"321": {
    				label: "Terveydenhuollon tekniset asiantuntijat"
    			},
    			"322": {
    				label: "Sairaanhoitajat, kätilöt ym."
    			},
    			"323": {
    				label: "Luontais- ja vaihtoehtohoitajat"
    			},
    			"324": {
    				label: "Seminologit ym."
    			},
    			"325": {
    				label: "Muut terveydenhuollon asiantuntijat"
    			},
    			"331": {
    				label: "Rahoitus-, vakuutus- ja laskentatoimen asiantuntijat"
    			},
    			"332": {
    				label: "Myynti- ja ostoagentit"
    			},
    			"333": {
    				label: "Yrityspalveluiden välittäjät"
    			},
    			"334": {
    				label: "Hallinnolliset ja erikoistuneet sihteerit"
    			},
    			"335": {
    				label: "Julkishallinnon valmistelu- ja valvontavirkamiehet"
    			},
    			"341": {
    				label: "Lainopilliset asiantuntijat sekä sosiaalialan ja seurakunnan työntekijät"
    			},
    			"342": {
    				label: "Urheilijat, urheiluvalmentajat, liikunnanohjaajat ym."
    			},
    			"343": {
    				label: "Taide- ja kulttuurialan asiantuntijat sekä keittiöpäälliköt"
    			},
    			"351": {
    				label: "Informaatio- ja tietoliikenneteknologian teknikot sekä käyttäjätukihenkilöt"
    			},
    			"352": {
    				label: "Teleliikenne- sekä radio- ja tv-teknikot"
    			},
    			"411": {
    				label: "Toimistoavustajat"
    			},
    			"412": {
    				label: "Yleissihteerit"
    			},
    			"413": {
    				label: "Tekstinkäsittelijät ja tallentajat"
    			},
    			"421": {
    				label: "Rahaliikenteen asiakaspalvelutyöntekijät"
    			},
    			"422": {
    				label: "Muut asiakaspalvelutyöntekijät"
    			},
    			"431": {
    				label: "Palkanlaskijat, vakuutuskäsittelijät ym."
    			},
    			"432": {
    				label: "Kuljetuksen ja varastoinnin toimistotyöntekijät"
    			},
    			"441": {
    				label: "Muut toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"511": {
    				label: "Matkustuspalvelutyöntekijät, konduktöörit ja oppaat"
    			},
    			"512": {
    				label: "Ravintola- ja suurtaloustyöntekijät"
    			},
    			"513": {
    				label: "Tarjoilutyöntekijät"
    			},
    			"514": {
    				label: "Kampaajat, parturit, kosmetologit ym."
    			},
    			"515": {
    				label: "Kiinteistöhuollon ja siivoustyön esimiehet"
    			},
    			"516": {
    				label: "Muut henkilökohtaisen palvelun työntekijät"
    			},
    			"521": {
    				label: "Katu- ja torikauppiaat"
    			},
    			"522": {
    				label: "Myyjät ja kauppiaat"
    			},
    			"523": {
    				label: "Kassanhoitajat ja lipunmyyjät"
    			},
    			"524": {
    				label: "Muut myyntityöntekijät"
    			},
    			"531": {
    				label: "Lastenhoitajat ja koulunkäyntiavustajat"
    			},
    			"532": {
    				label: "Lähihoitajat, muut terveydenhuollon työntekijät ja kodinhoitajat"
    			},
    			"541": {
    				label: "Suojelu- ja vartiointityöntekijät"
    			},
    			"611": {
    				label: "Pelto- ja puutarhaviljelijät"
    			},
    			"612": {
    				label: "Eläintenkasvattajat"
    			},
    			"613": {
    				label: "Yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat"
    			},
    			"621": {
    				label: "Metsurit ja metsätyöntekijät"
    			},
    			"622": {
    				label: "Kalanviljelijät, kalastajat ja metsästäjät"
    			},
    			"631": {
    				label: "Kotitarveviljelijät"
    			},
    			"632": {
    				label: "Kotitarvekarjankasvattajat"
    			},
    			"633": {
    				label: "Kotitarveviljelijät ja -karjankasvattajat"
    			},
    			"634": {
    				label: "Kotitarvekalastajat ja -metsästäjät"
    			},
    			"711": {
    				label: "Rakennustyöntekijät ym."
    			},
    			"712": {
    				label: "Rakennusten viimeistelytyöntekijät"
    			},
    			"713": {
    				label: "Maalarit ja rakennuspuhdistajat"
    			},
    			"721": {
    				label: "Valimotyöntekijät, hitsaajat, levysepät ym."
    			},
    			"722": {
    				label: "Sepät, työkaluntekijät ja koneenasettajat"
    			},
    			"723": {
    				label: "Koneasentajat ja -korjaajat"
    			},
    			"731": {
    				label: "Käsityötuotteiden valmistajat ja hienomekaanikot"
    			},
    			"732": {
    				label: "Painoalan työntekijät"
    			},
    			"741": {
    				label: "Sähkölaitteiden asentajat ja korjaajat"
    			},
    			"742": {
    				label: "Elektroniikka- ja tietoliikenneasentajat ja -korjaajat"
    			},
    			"751": {
    				label: "Lihanleikkaajat, leipurit, meijeristit ym."
    			},
    			"752": {
    				label: "Puutavaran käsittelijät, puusepät ym."
    			},
    			"753": {
    				label: "Vaatetusalan työntekijät ym."
    			},
    			"754": {
    				label: "Muut pääluokkaan 7 luokiteltavat työntekijät"
    			},
    			"811": {
    				label: "Kaivos- ja louhintatyön koneenkäyttäjät"
    			},
    			"812": {
    				label: "Metalliteollisuuden prosessityöntekijät ja viimeistelijät"
    			},
    			"813": {
    				label: "Kemianteollisuuden ja valokuvatuotteiden valmistuksen prosessityöntekijät"
    			},
    			"814": {
    				label: "Kumi-, muovi- ja paperituotteiden valmistuksen prosessityöntekijät"
    			},
    			"815": {
    				label: "Tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät"
    			},
    			"816": {
    				label: "Elintarviketeollisuuden prosessityöntekijät"
    			},
    			"817": {
    				label: "Sahatavaran sekä paperin ja kartongin valmistuksen prosessityöntekijät"
    			},
    			"818": {
    				label: "Muut prosessityöntekijät"
    			},
    			"821": {
    				label: "Teollisuustuotteiden kokoonpanijat"
    			},
    			"831": {
    				label: "Raideliikenteen kuljettajat ja työntekijät"
    			},
    			"832": {
    				label: "Henkilö- ja pakettiauton- sekä moottoripyöränkuljettajat"
    			},
    			"833": {
    				label: "Raskaiden moottoriajoneuvojen kuljettajat"
    			},
    			"834": {
    				label: "Työkoneiden kuljettajat"
    			},
    			"835": {
    				label: "Kansimiehistö ym. vesiliikenteen työntekijät"
    			},
    			"911": {
    				label: "Koti-, hotelli- ja toimistosiivoojat ym."
    			},
    			"912": {
    				label: "Ajoneuvojen puhdistajat, ikkunanpesijät ym."
    			},
    			"921": {
    				label: "Maa-, metsä- ja kalatalouden avustavat työntekijät"
    			},
    			"931": {
    				label: "Avustavat kaivos- ja rakennustyöntekijät"
    			},
    			"932": {
    				label: "Valmistusalan avustavat työntekijät"
    			},
    			"933": {
    				label: "Rahdinkäsittelijät ja varastotyöntekijät ym."
    			},
    			"941": {
    				label: "Avustavat keittiö- ja ruokatyöntekijät"
    			},
    			"951": {
    				label: "Mainosten jakajat, kengänkiillottajat ym."
    			},
    			"952": {
    				label: "Katumyyjät (pl. elintarvikkeet)"
    			},
    			"961": {
    				label: "Jätehuoltotyöntekijät"
    			},
    			"962": {
    				label: "Sanomalehtien jakajat, lähetit ym."
    			},
    			"1111": {
    				label: "Lainsäätäjät"
    			},
    			"1112": {
    				label: "Julkishallinnon ylimmät virkamiehet"
    			},
    			"1113": {
    				label: "Heimo- ja kyläpäälliköt"
    			},
    			"1114": {
    				label: "Järjestöjen johtajat"
    			},
    			"1120": {
    				label: "Toimitusjohtajat ja pääjohtajat"
    			},
    			"1211": {
    				label: "Talousjohtajat"
    			},
    			"1212": {
    				label: "Henkilöstöjohtajat"
    			},
    			"1213": {
    				label: "Politiikka- ja suunnittelujohtajat"
    			},
    			"1219": {
    				label: "Muut hallintojohtajat ja kaupalliset johtajat"
    			},
    			"1221": {
    				label: "Myynti- ja markkinointijohtajat"
    			},
    			"1222": {
    				label: "Mainos- ja tiedotusjohtajat"
    			},
    			"1223": {
    				label: "Tutkimus- ja kehitysjohtajat"
    			},
    			"1311": {
    				label: "Maa- ja metsätalouden johtajat"
    			},
    			"1312": {
    				label: "Vesiviljely- ja kalatalouden johtajat"
    			},
    			"1321": {
    				label: "Teollisuuden tuotantojohtajat"
    			},
    			"1322": {
    				label: "Kaivostoiminnan tuotantojohtajat"
    			},
    			"1323": {
    				label: "Rakennustoiminnan tuotantojohtajat"
    			},
    			"1324": {
    				label: "Hankinta- ja jakelujohtajat"
    			},
    			"1330": {
    				label: "Tieto- ja viestintäteknologiajohtajat"
    			},
    			"1341": {
    				label: "Lastenhoidon johtajat"
    			},
    			"1342": {
    				label: "Terveydenhuollon johtajat"
    			},
    			"1343": {
    				label: "Vanhustenhuollon johtajat"
    			},
    			"1344": {
    				label: "Sosiaalihuollon johtajat"
    			},
    			"1345": {
    				label: "Opetusalan johtajat"
    			},
    			"1346": {
    				label: "Rahoitus- ja vakuutuspalvelujen johtajat"
    			},
    			"1349": {
    				label: "Muut yhteiskunnan palvelujen johtajat"
    			},
    			"1411": {
    				label: "Hotellinjohtajat"
    			},
    			"1412": {
    				label: "Ravintolanjohtajat"
    			},
    			"1420": {
    				label: "Vähittäis- ja tukkukaupan johtajat"
    			},
    			"1431": {
    				label: "Urheilu-, vapaa-aika- ja kulttuurikeskusten johtajat"
    			},
    			"1439": {
    				label: "Muut palvelualojen johtajat"
    			},
    			"2111": {
    				label: "Fyysikot ja astronomit"
    			},
    			"2112": {
    				label: "Meteorologit"
    			},
    			"2113": {
    				label: "Kemistit"
    			},
    			"2114": {
    				label: "Geologit ja geofyysikot"
    			},
    			"2120": {
    				label: "Matemaatikot, aktuaarit ja tilastotieteilijät"
    			},
    			"2131": {
    				label: "Biologit, kasvi- ja eläintieteilijät ym. erityisasiantuntijat"
    			},
    			"2132": {
    				label: "Maa-, metsä- ja kalatalouden erityisasiantuntijat"
    			},
    			"2133": {
    				label: "Ympäristön- ja luonnonsuojelun erityisasiantuntijat"
    			},
    			"2141": {
    				label: "Teollisen valmistuksen ja tuotantotekniikan erityisasiantuntijat"
    			},
    			"2142": {
    				label: "Maa- ja vesirakentamisen erityisasiantuntijat"
    			},
    			"2143": {
    				label: "Ympäristötekniikan erityisasiantuntijat"
    			},
    			"2144": {
    				label: "Konetekniikan erityisasiantuntijat"
    			},
    			"2145": {
    				label: "Puunjalostuksen ja kemian prosessitekniikan erityisasiantuntijat"
    			},
    			"2146": {
    				label: "Kaivosteollisuuden, metallurgian ym. erityisasiantuntijat"
    			},
    			"2149": {
    				label: "Muut tekniikan erityisasiantuntijat"
    			},
    			"2151": {
    				label: "Sähkötekniikan erityisasiantuntijat"
    			},
    			"2152": {
    				label: "Elektroniikan erityisasiantuntijat"
    			},
    			"2153": {
    				label: "ICT-alan erityisasiantuntijat"
    			},
    			"2161": {
    				label: "Talonrakennuksen arkkitehdit"
    			},
    			"2162": {
    				label: "Maisema-arkkitehdit"
    			},
    			"2163": {
    				label: "Tuote- ja vaatesuunnittelijat"
    			},
    			"2164": {
    				label: "Yhdyskunta- ja liikennesuunnittelijat"
    			},
    			"2165": {
    				label: "Kartoituksen ja maanmittauksen erityisasiantuntijat"
    			},
    			"2166": {
    				label: "Graafiset ja multimediasuunnittelijat"
    			},
    			"2211": {
    				label: "Yleislääkärit"
    			},
    			"2212": {
    				label: "Ylilääkärit ja erikoislääkärit"
    			},
    			"2221": {
    				label: "Ylihoitajat ja osastonhoitajat"
    			},
    			"2222": {
    				label: "Johtavat kätilöt"
    			},
    			"2230": {
    				label: "Perinteisen ja vaihtoehtohoidon erityisasiantuntijat"
    			},
    			"2240": {
    				label: "Muiden terveyspalvelujen tuottajat"
    			},
    			"2250": {
    				label: "Eläinlääkärit"
    			},
    			"2261": {
    				label: "Hammaslääkärit"
    			},
    			"2262": {
    				label: "Proviisorit"
    			},
    			"2263": {
    				label: "Ympäristöterveyden ja työsuojelun erityisasiantuntijat"
    			},
    			"2264": {
    				label: "Fysioterapian erityisasiantuntijat"
    			},
    			"2265": {
    				label: "Ravitsemusalan erityisasiantuntijat"
    			},
    			"2266": {
    				label: "Kuulontutkijat ja puheterapeutit"
    			},
    			"2267": {
    				label: "Optometrian erityisasiantuntijat"
    			},
    			"2269": {
    				label: "Muut muualla luokittelemattomat terveydenhuollon erityisasiantuntijat"
    			},
    			"2310": {
    				label: "Yliopistojen ja ammattikorkeakoulujen opettajat"
    			},
    			"2320": {
    				label: "Ammatillisen koulutuksen opettajat"
    			},
    			"2330": {
    				label: "Lukion ja peruskoulun yläluokkien opettajat"
    			},
    			"2341": {
    				label: "Peruskoulun alaluokkien opettajat"
    			},
    			"2342": {
    				label: "Lastentarhanopettajat"
    			},
    			"2351": {
    				label: "Opetusmenetelmien erityisasiantuntijat"
    			},
    			"2352": {
    				label: "Erityisopettajat"
    			},
    			"2353": {
    				label: "Muut kieltenopettajat"
    			},
    			"2354": {
    				label: "Muut musiikin opettajat"
    			},
    			"2355": {
    				label: "Muut taideaineiden opettajat"
    			},
    			"2356": {
    				label: "Muut tietotekniikan opettajat ja kouluttajat"
    			},
    			"2359": {
    				label: "Opinto-ohjaajat ja muut opetuksen erityisasiantuntijat"
    			},
    			"2411": {
    				label: "Laskentatoimen erityisasiantuntijat ja tilintarkastajat"
    			},
    			"2412": {
    				label: "Rahoitus- ja sijoitusneuvojat"
    			},
    			"2413": {
    				label: "Rahoitusanalyytikot"
    			},
    			"2421": {
    				label: "Johtamisen ja organisaatioiden erityisasiantuntijat"
    			},
    			"2422": {
    				label: "Hallinnon ja elinkeinojen kehittämisen erityisasiantuntijat"
    			},
    			"2423": {
    				label: "Henkilöstöhallinnon erityisasiantuntijat ja urasuunnittelijat"
    			},
    			"2424": {
    				label: "Henkilöstön kehittämisen erityisasiantuntijat ja kouluttajat"
    			},
    			"2431": {
    				label: "Mainonnan ja markkinoinnin erityisasiantuntijat"
    			},
    			"2432": {
    				label: "Tiedottajat"
    			},
    			"2433": {
    				label: "Myynti-insinöörit ja lääke-esittelijät (pl. tieto- ja viestintätekniikka)"
    			},
    			"2434": {
    				label: "Tieto- ja viestintätekniikan myynnin erityisasiantuntijat"
    			},
    			"2511": {
    				label: "Sovellusarkkitehdit"
    			},
    			"2512": {
    				label: "Sovellussuunnittelijat"
    			},
    			"2513": {
    				label: "Web- ja multimediakehittäjät"
    			},
    			"2514": {
    				label: "Sovellusohjelmoijat"
    			},
    			"2519": {
    				label: "Muut ohjelmisto- ja sovelluskehittäjät"
    			},
    			"2521": {
    				label: "Tietokantasuunnittelijat ja -vastaavat"
    			},
    			"2522": {
    				label: "Tietojärjestelmien ylläpitäjät"
    			},
    			"2523": {
    				label: "Tietoverkkojen erityisasiantuntijat"
    			},
    			"2529": {
    				label: "Muut tietokanta- ja tietoverkkojen erityisasiantuntijat"
    			},
    			"2611": {
    				label: "Asianajajat"
    			},
    			"2612": {
    				label: "Tuomioistuinlakimiehet"
    			},
    			"2619": {
    				label: "Muut lainopilliset erityisasiantuntijat"
    			},
    			"2621": {
    				label: "Arkistonhoitajat ja museoalan erityisasiantuntijat"
    			},
    			"2622": {
    				label: "Kirjastonhoitajat, informaatikot ym."
    			},
    			"2631": {
    				label: "Ekonomistit"
    			},
    			"2632": {
    				label: "Yhteiskunta- ja kulttuuritutkijat"
    			},
    			"2633": {
    				label: "Historioitsijat, politiikan tutkijat ja filosofit"
    			},
    			"2634": {
    				label: "Psykologit"
    			},
    			"2635": {
    				label: "Sosiaalityön erityisasiantuntijat"
    			},
    			"2636": {
    				label: "Papit ym. uskonnollisen elämän erityisasiantuntijat"
    			},
    			"2641": {
    				label: "Kirjailijat ym."
    			},
    			"2642": {
    				label: "Toimittajat"
    			},
    			"2643": {
    				label: "Kääntäjät, tulkit ja muut kielitieteilijät"
    			},
    			"2651": {
    				label: "Kuvataiteilijat"
    			},
    			"2652": {
    				label: "Muusikot, laulajat ja säveltäjät"
    			},
    			"2653": {
    				label: "Tanssitaiteilijat ja koreografit"
    			},
    			"2654": {
    				label: "Ohjaajat ja tuottajat"
    			},
    			"2655": {
    				label: "Näyttelijät"
    			},
    			"2656": {
    				label: "Juontajat, kuuluttajat ym."
    			},
    			"2659": {
    				label: "Muut taiteilijat"
    			},
    			"3111": {
    				label: "Luonnontieteen tekniset asiantuntijat"
    			},
    			"3112": {
    				label: "Rakentamisen asiantuntijat"
    			},
    			"3113": {
    				label: "Sähkötekniikan asiantuntijat"
    			},
    			"3114": {
    				label: "Elektroniikan asiantuntijat"
    			},
    			"3115": {
    				label: "Konetekniikan asiantuntijat"
    			},
    			"3116": {
    				label: "Kemian prosessitekniikan asiantuntijat"
    			},
    			"3117": {
    				label: "Kaivosteollisuuden ja metallurgian asiantuntijat"
    			},
    			"3118": {
    				label: "Tekniset piirtäjät"
    			},
    			"3119": {
    				label: "Muut fysiikan, kemian ja teknisten alojen asiantuntijat"
    			},
    			"3121": {
    				label: "Kaivostyönjohtajat"
    			},
    			"3122": {
    				label: "Teollisuuden työnjohtajat"
    			},
    			"3123": {
    				label: "Rakennusalan työnjohtajat"
    			},
    			"3131": {
    				label: "Voimalaitosten prosessinhoitajat"
    			},
    			"3132": {
    				label: "Jätteenpoltto- ja vedenpuhdistuslaitosten prosessinhoitajat"
    			},
    			"3133": {
    				label: "Kemianteollisuuden prosessinhoitajat"
    			},
    			"3134": {
    				label: "Öljy- ja maakaasujalostamon prosessinhoitajat"
    			},
    			"3135": {
    				label: "Metallien jalostuksen prosessinhoitajat"
    			},
    			"3139": {
    				label: "Muut prosessinvalvonnan asiantuntijat"
    			},
    			"3141": {
    				label: "Laborantit ym."
    			},
    			"3142": {
    				label: "Maa- ja kalatalousteknikot"
    			},
    			"3143": {
    				label: "Metsätalousteknikot"
    			},
    			"3151": {
    				label: "Laivojen konepäälliköt ja -mestarit"
    			},
    			"3152": {
    				label: "Vesiliikenteen perämiehet ja päälliköt"
    			},
    			"3153": {
    				label: "Lentokapteenit ja -perämiehet"
    			},
    			"3154": {
    				label: "Lennonjohtajat"
    			},
    			"3155": {
    				label: "Lennonvalvonnan tekniset asiantuntijat"
    			},
    			"3211": {
    				label: "Lääketieteellisen kuvantamis- ja laitetekniikan asiantuntijat"
    			},
    			"3212": {
    				label: "Bioanalyytikot (terveydenhuolto)"
    			},
    			"3213": {
    				label: "Farmaseutit"
    			},
    			"3214": {
    				label: "Hammas- ja apuvälineteknikot"
    			},
    			"3221": {
    				label: "Sairaanhoitajat ym."
    			},
    			"3222": {
    				label: "Kätilöt"
    			},
    			"3230": {
    				label: "Luontais- ja vaihtoehtohoitajat"
    			},
    			"3240": {
    				label: "Seminologit ym."
    			},
    			"3251": {
    				label: "Suuhygienistit"
    			},
    			"3252": {
    				label: "Potilas- ja terveystietojen käsittelijät"
    			},
    			"3253": {
    				label: "Terveysneuvojat"
    			},
    			"3254": {
    				label: "Optikot"
    			},
    			"3255": {
    				label: "Fysioterapeutit ym."
    			},
    			"3256": {
    				label: "Avustavat hoitotyöntekijät"
    			},
    			"3257": {
    				label: "Terveys- ja työsuojelutarkastajat"
    			},
    			"3258": {
    				label: "Sairaankuljetuksen ensihoitajat"
    			},
    			"3259": {
    				label: "Muut muualla luokittelemattomat terveydenhuollon asiantuntijat"
    			},
    			"3311": {
    				label: "Arvopaperi- ja valuuttakauppiaat"
    			},
    			"3312": {
    				label: "Luotto- ja laina-asiantuntijat"
    			},
    			"3313": {
    				label: "Kirjanpidon ja laskentatoimen asiantuntijat"
    			},
    			"3314": {
    				label: "Tilastointi- ja matematiikka-asiantuntijat"
    			},
    			"3315": {
    				label: "Arvioitsijat ja vahinkotarkastajat"
    			},
    			"3321": {
    				label: "Vakuutusalan palvelumyyjät"
    			},
    			"3322": {
    				label: "Myyntiedustajat"
    			},
    			"3323": {
    				label: "Sisäänostajat"
    			},
    			"3324": {
    				label: "Kaupanvälittäjät"
    			},
    			"3331": {
    				label: "Huolitsijat, tulli- ja laivanselvittäjät"
    			},
    			"3332": {
    				label: "Konferenssi- ja tapahtumajärjestäjät"
    			},
    			"3333": {
    				label: "Työnvälittäjät"
    			},
    			"3334": {
    				label: "Kiinteistönvälittäjät ja isännöitsijät"
    			},
    			"3339": {
    				label: "Muut liike-elämän asiantuntijat"
    			},
    			"3341": {
    				label: "Toimistotyön esimiehet"
    			},
    			"3342": {
    				label: "Asianajosihteerit"
    			},
    			"3343": {
    				label: "Johdon sihteerit ja osastosihteerit"
    			},
    			"3344": {
    				label: "Toimistosihteerit (terveydenhuolto)"
    			},
    			"3351": {
    				label: "Tulli- ja rajavirkamiehet"
    			},
    			"3352": {
    				label: "Verovalmistelijat ja -tarkastajat"
    			},
    			"3353": {
    				label: "Sosiaaliturvaetuuksien käsittelijät"
    			},
    			"3354": {
    				label: "Lupavirkamiehet"
    			},
    			"3355": {
    				label: "Komisariot ja ylikonstaapelit"
    			},
    			"3359": {
    				label: "Muut julkishallinnon valmistelu- ja valvontavirkamiehet"
    			},
    			"3411": {
    				label: "Lainopilliset avustajat ja järjestöalan asiantuntijat"
    			},
    			"3412": {
    				label: "Sosiaalialan ohjaajat ja neuvojat ym."
    			},
    			"3413": {
    				label: "Seurakuntatyöntekijät"
    			},
    			"3421": {
    				label: "Urheilijat"
    			},
    			"3422": {
    				label: "Urheiluvalmentajat ja toimitsijat"
    			},
    			"3423": {
    				label: "Liikunnan ja vapaa-ajan ohjaajat"
    			},
    			"3431": {
    				label: "Valokuvaajat"
    			},
    			"3432": {
    				label: "Sisustussuunnittelijat ym."
    			},
    			"3433": {
    				label: "Gallerioiden, museoiden ja kirjastojen tekniset työntekijät"
    			},
    			"3434": {
    				label: "Keittiöpäälliköt"
    			},
    			"3435": {
    				label: "Muut taide- ja kulttuurialan asiantuntijat"
    			},
    			"3511": {
    				label: "Käytön operaattorit"
    			},
    			"3512": {
    				label: "Käytön tukihenkilöt"
    			},
    			"3513": {
    				label: "Tietoverkkoteknikot"
    			},
    			"3514": {
    				label: "Webmasterit ja -teknikot"
    			},
    			"3521": {
    				label: "Lähetys- ja audiovisuaaliteknikot"
    			},
    			"3522": {
    				label: "Televiestinnän tekniset asiantuntijat"
    			},
    			"4110": {
    				label: "Toimistoavustajat"
    			},
    			"4120": {
    				label: "Yleissihteerit"
    			},
    			"4131": {
    				label: "Tekstinkäsittelijät"
    			},
    			"4132": {
    				label: "Tallentajat"
    			},
    			"4211": {
    				label: "Pankki- ym. toimihenkilöt"
    			},
    			"4212": {
    				label: "Vedonvälittäjät, bingo- ja kasinopelin hoitajat ym."
    			},
    			"4213": {
    				label: "Panttilainaajat"
    			},
    			"4214": {
    				label: "Maksujenperijät"
    			},
    			"4221": {
    				label: "Matkatoimistovirkailijat"
    			},
    			"4222": {
    				label: "Puhelinpalveluneuvojat"
    			},
    			"4223": {
    				label: "Puhelinvaihteenhoitajat"
    			},
    			"4224": {
    				label: "Hotellin vastaanottovirkailijat"
    			},
    			"4225": {
    				label: "Informaatiopisteen asiakasneuvojat"
    			},
    			"4226": {
    				label: "Vastaanoton ja neuvonnan hoitajat"
    			},
    			"4227": {
    				label: "Tutkimus- ja markkinatutkimushaastattelijat"
    			},
    			"4229": {
    				label: "Muualla luokittelemattomat asiakaspalvelutyöntekijät"
    			},
    			"4311": {
    				label: "Taloushallinnon toimistotyöntekijät"
    			},
    			"4312": {
    				label: "Tilasto-, rahoitus- ja vakuutusalan toimistotyöntekijät"
    			},
    			"4313": {
    				label: "Palkanlaskijat"
    			},
    			"4321": {
    				label: "Varastonhoitajat ym."
    			},
    			"4322": {
    				label: "Tuotannon valmistelijat"
    			},
    			"4323": {
    				label: "Kuljetuksen toimistotyöntekijät"
    			},
    			"4411": {
    				label: "Kirjastotyöntekijät"
    			},
    			"4412": {
    				label: "Postinkantajat ja -lajittelijat"
    			},
    			"4413": {
    				label: "Koodaajat, oikolukijat ym."
    			},
    			"4414": {
    				label: "Kirjurit ym."
    			},
    			"4415": {
    				label: "Arkistotyöntekijät"
    			},
    			"4416": {
    				label: "Henkilöstöhallinnon avustavat toimistotyöntekijät"
    			},
    			"4419": {
    				label: "Muut muualla luokittelemattomat toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"5111": {
    				label: "Lentoemännät, purserit ym."
    			},
    			"5112": {
    				label: "Konduktöörit, lipuntarkastajat ym."
    			},
    			"5113": {
    				label: "Matkaoppaat"
    			},
    			"5120": {
    				label: "Ravintola- ja suurtaloustyöntekijät"
    			},
    			"5131": {
    				label: "Tarjoilijat"
    			},
    			"5132": {
    				label: "Baarimestarit"
    			},
    			"5141": {
    				label: "Kampaajat ja parturit"
    			},
    			"5142": {
    				label: "Kosmetologit ym."
    			},
    			"5151": {
    				label: "Siivoustyön esimiehet toimistoissa, hotelleissa ja muissa laitoksissa"
    			},
    			"5152": {
    				label: "Yksityiskotien taloudenhoitajat"
    			},
    			"5153": {
    				label: "Kiinteistöhuollon työntekijät"
    			},
    			"5161": {
    				label: "Astrologit, ennustajat ym."
    			},
    			"5162": {
    				label: "Henkilökohtaiset palvelijat"
    			},
    			"5163": {
    				label: "Hautauspalvelutyöntekijät"
    			},
    			"5164": {
    				label: "Eläintenhoitajat ja lemmikkieläinten trimmaajat"
    			},
    			"5165": {
    				label: "Ajo-opettajat"
    			},
    			"5169": {
    				label: "Muualla luokittelemattomat henkilökohtaisen palvelun työntekijät"
    			},
    			"5211": {
    				label: "Kioski- ja torimyyjät"
    			},
    			"5212": {
    				label: "Katumyyjät (elintarvikkeet)"
    			},
    			"5221": {
    				label: "Kauppiaat (pienyrittäjät)"
    			},
    			"5222": {
    				label: "Myymäläesimiehet"
    			},
    			"5223": {
    				label: "Myyjät"
    			},
    			"5230": {
    				label: "Kassanhoitajat ja lipunmyyjät"
    			},
    			"5241": {
    				label: "Mallit"
    			},
    			"5242": {
    				label: "Tuote-esittelijät"
    			},
    			"5243": {
    				label: "Suoramyyjät"
    			},
    			"5244": {
    				label: "Puhelin- ja asiakaspalvelukeskusten myyjät"
    			},
    			"5245": {
    				label: "Huoltamotyöntekijät"
    			},
    			"5246": {
    				label: "Kahvila- ja baarimyyjät"
    			},
    			"5249": {
    				label: "Muut muualla luokittelemattomat myyntityöntekijät"
    			},
    			"5311": {
    				label: "Lastenhoitotyöntekijät"
    			},
    			"5312": {
    				label: "Koulunkäyntiavustajat"
    			},
    			"5321": {
    				label: "Lähihoitajat"
    			},
    			"5322": {
    				label: "Kodinhoitajat (kotipalvelutoiminta)"
    			},
    			"5329": {
    				label: "Muut terveydenhuoltoalan työntekijät"
    			},
    			"5411": {
    				label: "Palomiehet"
    			},
    			"5412": {
    				label: "Poliisit"
    			},
    			"5413": {
    				label: "Vanginvartijat"
    			},
    			"5414": {
    				label: "Vartijat"
    			},
    			"5419": {
    				label: "Muut suojelu- ja vartiointityöntekijät"
    			},
    			"6111": {
    				label: "Pelto- ja avomaaviljelijät"
    			},
    			"6112": {
    				label: "Hedelmäpuiden ja pensaiden yms. kasvattajat"
    			},
    			"6113": {
    				label: "Puutarhurit, kasvihuoneviljelijät ja -työntekijät"
    			},
    			"6114": {
    				label: "Yhdistetyn maan- ja vihannesviljelyn tai puutarhanhoidon ym. harjoittajat"
    			},
    			"6121": {
    				label: "Liha- ja lypsykarjan kasvattajat sekä muiden kotieläinten kasvattajat"
    			},
    			"6122": {
    				label: "Siipikarjankasvattajat"
    			},
    			"6123": {
    				label: "Mehiläistenhoitajat ym."
    			},
    			"6129": {
    				label: "Muut eläinten kasvattajat ja hoitajat"
    			},
    			"6130": {
    				label: "Yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat"
    			},
    			"6210": {
    				label: "Metsurit ja metsätyöntekijät"
    			},
    			"6221": {
    				label: "Kalanviljelijät ja -viljelytyöntekijät"
    			},
    			"6222": {
    				label: "Kalastajat"
    			},
    			"6223": {
    				label: "Syvänmerenkalastajat"
    			},
    			"6224": {
    				label: "Riistanhoitajat ja metsästäjät"
    			},
    			"6310": {
    				label: "Kotitarveviljelijät"
    			},
    			"6320": {
    				label: "Kotitarvekarjankasvattajat"
    			},
    			"6330": {
    				label: "Kotitarveviljelijät ja -karjankasvattajat"
    			},
    			"6340": {
    				label: "Kotitarvekalastajat ja -metsästäjät"
    			},
    			"7111": {
    				label: "Talonrakentajat"
    			},
    			"7112": {
    				label: "Muurarit ym."
    			},
    			"7113": {
    				label: "Kivenhakkaajat ja -leikkaajat ym."
    			},
    			"7114": {
    				label: "Betonirakentajat ja raudoittajat"
    			},
    			"7115": {
    				label: "Kirvesmiehet ja rakennuspuusepät"
    			},
    			"7119": {
    				label: "Muut rakennustyöntekijät"
    			},
    			"7121": {
    				label: "Kattoasentajat ja -korjaajat"
    			},
    			"7122": {
    				label: "Lattianpäällystystyöntekijät"
    			},
    			"7123": {
    				label: "Rappaajat"
    			},
    			"7124": {
    				label: "Eristäjät"
    			},
    			"7125": {
    				label: "Lasinasentajat"
    			},
    			"7126": {
    				label: "Putkiasentajat"
    			},
    			"7127": {
    				label: "Ilmastointi- ja jäähdytyslaiteasentajat"
    			},
    			"7131": {
    				label: "Rakennusmaalarit ym."
    			},
    			"7132": {
    				label: "Ruiskumaalaajat ja -lakkaajat"
    			},
    			"7133": {
    				label: "Rakennuspuhdistajat ja nuohoojat"
    			},
    			"7211": {
    				label: "Muotin- ja keernantekijät"
    			},
    			"7212": {
    				label: "Hitsaajat ja kaasuleikkaajat"
    			},
    			"7213": {
    				label: "Ohutlevysepät"
    			},
    			"7214": {
    				label: "Paksulevysepät ja rautarakennetyöntekijät"
    			},
    			"7215": {
    				label: "Kaapelin- ja köysienasentajat"
    			},
    			"7221": {
    				label: "Sepät"
    			},
    			"7222": {
    				label: "Työkaluntekijät ja lukkosepät"
    			},
    			"7223": {
    				label: "Koneenasettajat ja koneistajat"
    			},
    			"7224": {
    				label: "Konehiojat, kiillottajat ja teroittajat"
    			},
    			"7231": {
    				label: "Moottoriajoneuvojen asentajat ja korjaajat"
    			},
    			"7232": {
    				label: "Lentokoneasentajat ja -korjaajat"
    			},
    			"7233": {
    				label: "Maatalous- ja teollisuuskoneasentajat ja -korjaajat"
    			},
    			"7234": {
    				label: "Polkupyöränkorjaajat ym."
    			},
    			"7311": {
    				label: "Kellosepät ja muut hienomekaanisten instrumenttien tekijät ja korjaajat"
    			},
    			"7312": {
    				label: "Soittimien tekijät ja virittäjät"
    			},
    			"7313": {
    				label: "Koru-, kulta- ja hopeasepät"
    			},
    			"7314": {
    				label: "Saven- ja tiilenvalajat ja dreijaajat"
    			},
    			"7315": {
    				label: "Lasinpuhaltajat, -leikkaajat, -hiojat ja -viimeistelijät"
    			},
    			"7316": {
    				label: "Kaivertajat, etsaajat ja koristemaalarit"
    			},
    			"7317": {
    				label: "Puu-, kori- yms. käsityötuotteiden tekijät"
    			},
    			"7318": {
    				label: "Tekstiili-, nahka- yms. käsityötuotteiden tekijät"
    			},
    			"7319": {
    				label: "Muut käsityöntekijät"
    			},
    			"7321": {
    				label: "Painopinnanvalmistajat"
    			},
    			"7322": {
    				label: "Painajat"
    			},
    			"7323": {
    				label: "Jälkikäsittelijät ja sitomotyöntekijät"
    			},
    			"7411": {
    				label: "Rakennussähköasentajat"
    			},
    			"7412": {
    				label: "Muut sähköasentajat"
    			},
    			"7413": {
    				label: "Linja-asentajat ja -korjaajat"
    			},
    			"7421": {
    				label: "Elektroniikka- ja automaatiolaitteiden asentajat ja korjaajat"
    			},
    			"7422": {
    				label: "Tieto- ja viestintäteknologian asentajat ja korjaajat"
    			},
    			"7511": {
    				label: "Lihanleikkaajat, kalankäsittelijät ym."
    			},
    			"7512": {
    				label: "Leipurit ja kondiittorit"
    			},
    			"7513": {
    				label: "Meijeristit, juustomestarit ym."
    			},
    			"7514": {
    				label: "Hedelmä- ja vihannestuotteiden valmistajat"
    			},
    			"7515": {
    				label: "Ruokien ja juomien laaduntarkkailijat"
    			},
    			"7516": {
    				label: "Tupakkatuotteiden valmistajat"
    			},
    			"7521": {
    				label: "Raakapuun käsittelijät"
    			},
    			"7522": {
    				label: "Huonekalupuusepät ym."
    			},
    			"7523": {
    				label: "Konepuusepät"
    			},
    			"7531": {
    				label: "Vaatturit, pukuompelijat, turkkurit ja hatuntekijät"
    			},
    			"7532": {
    				label: "Leikkaajat ja mallimestarit"
    			},
    			"7533": {
    				label: "Koru- ja muut tekstiiliompelijat"
    			},
    			"7534": {
    				label: "Verhoilijat"
    			},
    			"7535": {
    				label: "Turkisten muokkaajat ja nahkurit"
    			},
    			"7536": {
    				label: "Suutarit ym."
    			},
    			"7541": {
    				label: "Vedenalaistyöntekijät"
    			},
    			"7542": {
    				label: "Panostajat ja räjäyttäjät"
    			},
    			"7543": {
    				label: "Luokittelijat ja laaduntarkkailijat (pl. ruoat ja juomat)"
    			},
    			"7544": {
    				label: "Savupuhdistajat, tuholais- ja rikkakasvintorjujat"
    			},
    			"7549": {
    				label: "Muut pääluokkaan 7 muualla luokittelemattomat työntekijät"
    			},
    			"8111": {
    				label: "Kaivos- ja louhostyöntekijät"
    			},
    			"8112": {
    				label: "Rikastustyöntekijät"
    			},
    			"8113": {
    				label: "Iskuporaajat ja syväkairaajat"
    			},
    			"8114": {
    				label: "Betonituote- ym. teollisuuden prosessityöntekijät"
    			},
    			"8121": {
    				label: "Metalliteollisuuden prosessityöntekijät"
    			},
    			"8122": {
    				label: "Metallien teolliset päällystäjät ja viimeistelijät"
    			},
    			"8131": {
    				label: "Kemianteollisuuden prosessityöntekijät ym."
    			},
    			"8132": {
    				label: "Valokuvatuotteiden valmistuksen prosessityöntekijät"
    			},
    			"8141": {
    				label: "Kumituoteteollisuuden prosessityöntekijät"
    			},
    			"8142": {
    				label: "Muovituoteteollisuuden prosessityöntekijät"
    			},
    			"8143": {
    				label: "Paperituoteteollisuuden prosessityöntekijät"
    			},
    			"8151": {
    				label: "Kuitujenkäsittely-, kehruu- ja puolauskoneiden hoitajat"
    			},
    			"8152": {
    				label: "Kutoma- ja neulekoneiden hoitajat"
    			},
    			"8153": {
    				label: "Teollisuusompelijat"
    			},
    			"8154": {
    				label: "Valkaisu-, värjäys- ja puhdistuskoneiden hoitajat"
    			},
    			"8155": {
    				label: "Turkisten ja nahkojen teolliset käsittelijät, värjääjät ym."
    			},
    			"8156": {
    				label: "Jalkine- ja laukkuteollisuuden prosessityöntekijät"
    			},
    			"8157": {
    				label: "Pesulatyöntekijät"
    			},
    			"8159": {
    				label: "Muut tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät"
    			},
    			"8160": {
    				label: "Elintarviketeollisuuden prosessityöntekijät"
    			},
    			"8171": {
    				label: "Paperimassan sekä paperin ja kartongin valmistuksen prosessityöntekijät"
    			},
    			"8172": {
    				label: "Puu- ja sahatavaran prosessityöntekijät"
    			},
    			"8181": {
    				label: "Lasi- ja keramiikkateollisuuden uunienhoitajat"
    			},
    			"8182": {
    				label: "Höyrykoneiden ja lämmityskattiloiden hoitajat, lämmittäjät ym."
    			},
    			"8183": {
    				label: "Pakkaus-, pullotus- ja etiköintikoneiden hoitajat"
    			},
    			"8189": {
    				label: "Muut muualla luokittelemattomat prosessityöntekijät"
    			},
    			"8211": {
    				label: "Konepaja- ja metallituotteiden kokoonpanijat"
    			},
    			"8212": {
    				label: "Sähkö- ja elektroniikkalaitteiden kokoonpanijat"
    			},
    			"8219": {
    				label: "Muut teollisuustuotteiden kokoonpanijat"
    			},
    			"8311": {
    				label: "Veturinkuljettajat"
    			},
    			"8312": {
    				label: "Jarru-, turvalaite- ja vaihdetyöntekijät"
    			},
    			"8321": {
    				label: "Moottoripyörälähetit yms."
    			},
    			"8322": {
    				label: "Henkilö-, taksi- ja pakettiautonkuljettajat"
    			},
    			"8331": {
    				label: "Linja-auton- ja raitiovaununkuljettajat"
    			},
    			"8332": {
    				label: "Kuorma-auton ja erikoisajoneuvojen kuljettajat"
    			},
    			"8341": {
    				label: "Maa- ja metsätaloustyökoneiden kuljettajat"
    			},
    			"8342": {
    				label: "Maansiirtokoneiden ym. kuljettajat"
    			},
    			"8343": {
    				label: "Nosturinkuljettajat"
    			},
    			"8344": {
    				label: "Ahtaajat ja trukinkuljettajat ym."
    			},
    			"8350": {
    				label: "Kansimiehistö ym. vesiliikenteen työntekijät"
    			},
    			"9111": {
    				label: "Kotiapulaiset ja -siivoojat"
    			},
    			"9112": {
    				label: "Toimisto- ja laitossiivoojat ym."
    			},
    			"9121": {
    				label: "Puhdistajat ja prässääjät"
    			},
    			"9122": {
    				label: "Ajoneuvojen pesijät"
    			},
    			"9123": {
    				label: "Ikkunanpesijät"
    			},
    			"9129": {
    				label: "Muut puhdistustyöntekijät"
    			},
    			"9211": {
    				label: "Maanviljelyn avustavat työntekijät"
    			},
    			"9212": {
    				label: "Karjankasvatuksen avustavat työntekijät"
    			},
    			"9213": {
    				label: "Yhdistetyn maanviljelyn ja karjankasvatuksen avustavat työntekijät"
    			},
    			"9214": {
    				label: "Avustavat puutarhatyöntekijät"
    			},
    			"9215": {
    				label: "Metsätalouden avustavat työntekijät"
    			},
    			"9216": {
    				label: "Kalatalouden ja vesiviljelyn avustavat työntekijät"
    			},
    			"9311": {
    				label: "Kaivosten avustavat työntekijät"
    			},
    			"9312": {
    				label: "Maa- ja vesirakentamisen avustavat työntekijät"
    			},
    			"9313": {
    				label: "Rakennusalan avustavat työntekijät"
    			},
    			"9321": {
    				label: "Käsinpakkaajat"
    			},
    			"9329": {
    				label: "Muut valmistusalan avustavat työntekijät"
    			},
    			"9331": {
    				label: "Käsivetoisten- ja poljinkulkuneuvojen kuljettajat"
    			},
    			"9332": {
    				label: "Eläinvetoisten kulkuneuvojen kuljettajat"
    			},
    			"9333": {
    				label: "Rahdinkäsittelijät, varastotyöntekijät ym."
    			},
    			"9334": {
    				label: "Hyllyjen täyttäjät"
    			},
    			"9411": {
    				label: "Pikaruokatyöntekijät"
    			},
    			"9412": {
    				label: "Avustavat keittiötyöntekijät"
    			},
    			"9510": {
    				label: "Mainosten jakajat, kengänkiillottajat ym."
    			},
    			"9520": {
    				label: "Katumyyjät (pl. elintarvikkeet)"
    			},
    			"9611": {
    				label: "Jätteiden kerääjät"
    			},
    			"9612": {
    				label: "Jätteiden lajittelijat"
    			},
    			"9613": {
    				label: "Kadunlakaisijat ym."
    			},
    			"9621": {
    				label: "Sanomalehtien jakajat, lähetit ja kantajat"
    			},
    			"9622": {
    				label: "Satunnaistöiden tekijät"
    			},
    			"9623": {
    				label: "Mittareiden lukijat ym."
    			},
    			"9624": {
    				label: "Vedenhakijat ja polttopuun kerääjät"
    			},
    			"9629": {
    				label: "Muut muualla luokittelemattomat työntekijät"
    			},
    			"11121": {
    				label: "Valtion keskushallinnon johtajat"
    			},
    			"11122": {
    				label: "Alue- ja paikallishallinnon johtajat ja ylimmät virkamiehet"
    			},
    			"11141": {
    				label: "Työmarkkina- ja elinkeinoelämän järjestöjen johtajat"
    			},
    			"11142": {
    				label: "Muiden järjestöjen johtajat"
    			},
    			"21321": {
    				label: "Maa- ja kalatalouden erityisasiantuntijat"
    			},
    			"21322": {
    				label: "Metsätalouden erityisasiantuntijat"
    			},
    			"21531": {
    				label: "Tietoliikenneteknologian tutkijat"
    			},
    			"21532": {
    				label: "Tietotekniikka-alan tutkijat"
    			},
    			"22121": {
    				label: "Ylilääkärit"
    			},
    			"22122": {
    				label: "Erikoislääkärit"
    			},
    			"22211": {
    				label: "Ylihoitajat"
    			},
    			"22212": {
    				label: "Osastonhoitajat"
    			},
    			"23101": {
    				label: "Professorit"
    			},
    			"23102": {
    				label: "Lehtorit ja yliassistentit (yliopisto)"
    			},
    			"23103": {
    				label: "Assistentit ja tuntiopettajat (yliopisto)"
    			},
    			"23104": {
    				label: "Yliopettajat (AMK)"
    			},
    			"23105": {
    				label: "Lehtorit (AMK)"
    			},
    			"23106": {
    				label: "Tuntiopettajat ym. (AMK)"
    			},
    			"23301": {
    				label: "Matemaattisten aineiden opettajat"
    			},
    			"23302": {
    				label: "Äidinkielen ja kirjallisuuden opettajat"
    			},
    			"23303": {
    				label: "Taito- ja taideaineiden opettajat"
    			},
    			"23304": {
    				label: "Muut peruskoulun yläluokkien ja lukion opettajat"
    			},
    			"23411": {
    				label: "Luokanopettajat"
    			},
    			"23412": {
    				label: "Aineenopettajat (peruskoulun alaluokat)"
    			},
    			"23591": {
    				label: "Opinto-ohjaajat"
    			},
    			"23592": {
    				label: "Muut opetuksen erityisasiantuntijat"
    			},
    			"26211": {
    				label: "Arkistonhoitajat"
    			},
    			"26212": {
    				label: "Museoalan erityisasiantuntijat"
    			},
    			"26351": {
    				label: "Sosiaalityöntekijät ym."
    			},
    			"26352": {
    				label: "Sosiaalialan suunnittelijat ym."
    			},
    			"26421": {
    				label: "Toimituspäälliköt ja -sihteerit, ohjelmapäälliköt"
    			},
    			"26422": {
    				label: "Lehtien yms. toimittajat"
    			},
    			"26423": {
    				label: "Radio- ja tv-toimittajat"
    			},
    			"31121": {
    				label: "Talonrakentamisen asiantuntijat"
    			},
    			"31122": {
    				label: "Maa- ja vesirakentamisen asiantuntijat"
    			},
    			"31123": {
    				label: "Kartoituksen ja maanmittauksen asiantuntijat"
    			},
    			"31521": {
    				label: "Isojen alusten päälliköt ja perämiehet"
    			},
    			"31522": {
    				label: "Pienten alusten päälliköt"
    			},
    			"31523": {
    				label: "Satamaliikenteen ohjaajat ja satamakapteenit"
    			},
    			"32141": {
    				label: "Hammasteknikot"
    			},
    			"32142": {
    				label: "Apuvälineteknikot"
    			},
    			"32211": {
    				label: "Sairaanhoitajat"
    			},
    			"32212": {
    				label: "Terveydenhoitajat"
    			},
    			"32591": {
    				label: "Toimintaterapeutit"
    			},
    			"32592": {
    				label: "Muut terapeutit"
    			},
    			"33341": {
    				label: "Kiinteistönvälittäjät"
    			},
    			"33342": {
    				label: "Isännöitsijät"
    			},
    			"34111": {
    				label: "Oikeudenkäyntiasiamiehet ja ulosottomiehet"
    			},
    			"34112": {
    				label: "Asiamiehet, toimitsijat ym. järjestöalan asiantuntijat"
    			},
    			"34113": {
    				label: "Kuluttajaneuvojat ym."
    			},
    			"34121": {
    				label: "Sosiaalialan ohjaajat"
    			},
    			"34122": {
    				label: "Nuorisotyön ohjaajat (ei srk.)"
    			},
    			"34123": {
    				label: "Työn- ja askarteluohjaajat"
    			},
    			"34131": {
    				label: "Diakonit ja diakonissat"
    			},
    			"34139": {
    				label: "Muut seurakuntatyöntekijät"
    			},
    			"34351": {
    				label: "Kuvaussihteerit ja muut näyttämötekniset asiantuntijat"
    			},
    			"34359": {
    				label: "Muut taide- ja kulttuurialan tekniset asiantuntijat"
    			},
    			"42291": {
    				label: "Hälytyspäivystäjät"
    			},
    			"42299": {
    				label: "Muut muualla luokittelemattomat asiakaspalvelutyöntekijät"
    			},
    			"43231": {
    				label: "Raideliikenteen ohjaajat, ajojärjestelijät ym."
    			},
    			"43239": {
    				label: "Muut kuljetuksen ja huolinnan toimistotyöntekijät"
    			},
    			"44121": {
    				label: "Postinkantajat"
    			},
    			"44122": {
    				label: "Toimistovahtimestarit"
    			},
    			"51201": {
    				label: "Kokit, keittäjät ja kylmäköt"
    			},
    			"51202": {
    				label: "Ravintolaesimiehet ja vuoropäälliköt"
    			},
    			"51631": {
    				label: "Hautaustoimistonhoitajat ym."
    			},
    			"51632": {
    				label: "Muut hautaustyöntekijät"
    			},
    			"53111": {
    				label: "Päiväkotien ja muiden laitosten lastenhoitajat ym."
    			},
    			"53112": {
    				label: "Perhepäivähoitajat"
    			},
    			"53113": {
    				label: "Lastenkerhojen ohjaajat ym."
    			},
    			"53211": {
    				label: "Mielenterveyshoitajat"
    			},
    			"53212": {
    				label: "Kehitysvammaisten hoitajat"
    			},
    			"53213": {
    				label: "Sosiaalialan hoitajat"
    			},
    			"53219": {
    				label: "Muut lähihoitajat"
    			},
    			"53221": {
    				label: "Kotityöpalvelutyöntekijät"
    			},
    			"53222": {
    				label: "Henkilökohtaiset avustajat, omaishoitajat ym."
    			},
    			"53291": {
    				label: "Hammashoitajat"
    			},
    			"53292": {
    				label: "Välinehuoltajat"
    			},
    			"53293": {
    				label: "Apteekkien lääketyöntekijät"
    			},
    			"53294": {
    				label: "Hierojat ja kuntohoitajat"
    			},
    			"61111": {
    				label: "Peltoviljelijät"
    			},
    			"61112": {
    				label: "Peltoviljelytyönjohtajat ja -työntekijät"
    			},
    			"61131": {
    				label: "Puutarhurit ja kasvihuoneviljelijät"
    			},
    			"61132": {
    				label: "Puutarha- ja kasvihuonetyönjohtajat ja - työntekijät"
    			},
    			"61211": {
    				label: "Karjankasvattajat ym."
    			},
    			"61212": {
    				label: "Karjanhoitajat ym."
    			},
    			"61213": {
    				label: "Lemmikkieläinten kasvattajat"
    			},
    			"61214": {
    				label: "Maatalouslomittajat"
    			},
    			"61291": {
    				label: "Turkiseläinten ja porojen kasvattajat"
    			},
    			"61299": {
    				label: "Muualla luokittelemattomat eläintenhoitajat"
    			},
    			"62211": {
    				label: "Kalanviljely-yrittäjät"
    			},
    			"62212": {
    				label: "Kalanviljelytyönjohtajat ja -työntekijät"
    			},
    			"74211": {
    				label: "Elektroniikka-asentajat ja -korjaajat"
    			},
    			"74212": {
    				label: "Automaatioasentajat ja -korjaajat"
    			},
    			"81821": {
    				label: "Voima- ja jätteenkäsittelylaitosten laitosmiehet"
    			},
    			"81822": {
    				label: "Muut laitosmiehet ym."
    			},
    			"83441": {
    				label: "Ahtaajat"
    			},
    			"83442": {
    				label: "Trukinkuljettajat ym."
    			},
    			"91121": {
    				label: "Toimistosiivoojat ym."
    			},
    			"91122": {
    				label: "Hotellisiivoojat"
    			},
    			"91123": {
    				label: "Sairaala- ja laitosapulaiset"
    			},
    			"91124": {
    				label: "Päiväkotiapulaiset"
    			},
    			"91129": {
    				label: "Muut muualla luokittelemattomat siivoojat"
    			},
    			"01": {
    				label: "Upseerit"
    			},
    			"011": {
    				label: "Upseerit"
    			},
    			"0110": {
    				label: "Upseerit"
    			},
    			"02": {
    				label: "Aliupseerit"
    			},
    			"021": {
    				label: "Aliupseerit"
    			},
    			"0210": {
    				label: "Aliupseerit"
    			},
    			"03": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"031": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"0310": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"#X": {
    				label: "Tuntematon"
    			},
    			"#XX": {
    				label: "Tuntematon"
    			},
    			"#XXX": {
    				label: "Tuntematon"
    			},
    			"#XXXX": {
    				label: "Tuntematon"
    			},
    			"#XXXXX": {
    				label: "Tuntematon"
    			}
    		},
    		RemunerationKind: {
    			bonusPay: {
    				description: "Kertaluonteinen palkkiopalkka, jonka suuruus määräytyy kokonaan tai osittain sen mukaan, kuinka hyvin tavoitteeksi asetettu työtulos saavutetaan. Työtuloksen mittana voi olla määrä, laatu tai jokin muu tulostekijä.",
    				label: "Bonuspalkka"
    			},
    			commission: {
    				description: "Pysyvän palkan osan muodostava jatkuvaluonteinen palkkiopalkka, jonka määrästä on sovittu työsopimuksessa.",
    				label: "Provisiopalkka"
    			},
    			initiativeFee: {
    				description: "Kertapalkkio, joka maksetaan aloitteen tekijälle esimerkiksi toiminnan tai tuotteen kehittämiseen liittyvästä aloitteesta.",
    				label: "Aloitepalkkio"
    			},
    			performanceBonus: {
    				description: "Palkka, joka maksetaan organisaation ennalta sovitun tulostavoitteen täyttymisen tai ylittymisen perusteella.",
    				label: "Tulospalkkio"
    			},
    			profitSharingBonus: {
    				description: "Työntekijöille ilman ennalta sovittua suunnitelmaa yhtiökokouksen päätöksellä jaettava palkkio, joka muodostuu yrityksen voiton perusteella.",
    				label: "Voittopalkkio"
    			},
    			shareIssueForEmployees: {
    				description: "Yhtiön omalle henkilökunnalle suunnattu osakeanti. Etu työsuhteeseen perustuvasta oikeudesta merkitä työnantajayrityksen osakkeita on veronalaista ansiotuloa.",
    				label: "Työsuhteeseen perustuva osakeanti"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			}
    		},
    		ReportCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			monthly: {
    				description: "",
    				label: "Monthly"
    			},
    			yearly: {
    				description: "",
    				label: "Yearly"
    			}
    		},
    		ReportType: {
    			employerReport: {
    				description: "",
    				label: "Työnantajan laskelma"
    			},
    			employmentContract: {
    				description: "",
    				label: "Työsopimus"
    			},
    			example: {
    				description: "",
    				label: "Esimerkki"
    			},
    			householdDeduction: {
    				description: "",
    				label: "Kotitalousvähennys"
    			},
    			insurance: {
    				description: "",
    				label: "Vakuutus"
    			},
    			monthlyDetails: {
    				description: "",
    				label: "Kuukausittaisia tietoja"
    			},
    			monthlyLiikekirjuri: {
    				description: "",
    				label: ""
    			},
    			monthlyPension: {
    				description: "",
    				label: "Kuukausittainen eläke"
    			},
    			monthlyRapko: {
    				description: "",
    				label: ""
    			},
    			paymentReport: {
    				description: "",
    				label: "Maksuraportti"
    			},
    			salarySlip: {
    				description: "",
    				label: "Palkkakuitti"
    			},
    			salarySlipPaid: {
    				description: "",
    				label: "Palkkakuitti maksettu"
    			},
    			taxHouseholdDeduction14B: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseA: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseB: {
    				description: "",
    				label: ""
    			},
    			taxMonthly4001: {
    				description: "",
    				label: "Kuukausittainen vero"
    			},
    			taxYearly7801: {
    				description: "",
    				label: "Vuosittainen vero"
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritelty"
    			},
    			unemployment: {
    				description: "",
    				label: "Työttömyys"
    			},
    			yearEndReport: {
    				description: "",
    				label: "Loppuvuoden raportti"
    			},
    			yearlyDetails: {
    				description: "",
    				label: "Vuosittaisia tietoja"
    			},
    			yearlyWorkerSummary: {
    				description: "",
    				label: ""
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "EarningsPaymentReport"
    			},
    			employersSeparateReport: {
    				description: "",
    				label: "EmployersSeparateReport"
    			},
    			monthlyAccounting: {
    				description: "",
    				label: "MonthlyAccounting"
    			},
    			salarySlipCopy: {
    				description: "",
    				label: "SalarySlipCopy"
    			}
    		},
    		SalaryKind: {
    			compensation: {
    				description: "",
    				label: "Korvaus",
    				labels: {
    					household: "Työkorvaus"
    				}
    			},
    			fixedSalary: {
    				description: "",
    				label: "Kiinteä palkka"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Tuntipalkka"
    			},
    			monthlySalary: {
    				description: "",
    				label: "Kuukausipalkka"
    			},
    			salary: {
    				description: "",
    				label: "Kertakorvaus"
    			},
    			totalEmployerPayment: {
    				description: "",
    				label: ""
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritelty"
    			}
    		},
    		SalaryPeriod: {
    			fixed: {
    				description: "",
    				label: "Fixed"
    			},
    			hourly: {
    				description: "",
    				label: "Hourly"
    			},
    			monthly: {
    				description: "",
    				label: "Monthly"
    			},
    			notDefined: {
    				description: "",
    				label: "Not Defined"
    			}
    		},
    		SalaryPerMonthRange: {
    			noPensionRequired: {
    				description: "",
    				label: "Alle 57,10 €"
    			},
    			normal: {
    				description: "",
    				label: "57,10 € tai yli"
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		SalaryPerYearRange: {
    			noMonthlyReporting: {
    				description: "",
    				label: "201 - 1500 €"
    			},
    			noReportingNecessary: {
    				description: "",
    				label: "1 - 200 €"
    			},
    			normal: {
    				description: "",
    				label: "Yli 1500 €"
    			},
    			zeroSalary: {
    				description: "",
    				label: 0
    			}
    		},
    		ServiceModel: {
    			partnerOnly: {
    				label: "Tilitoimistokäyttö"
    			},
    			shared: {
    				label: "Yhteiskäyttö"
    			}
    		},
    		SubsidisedCommuteKind: {
    			noDeduction: {
    				description: "Ei vähennystä työntekijän palkasta",
    				label: "Ei vähennystä"
    			},
    			periodicalDeduction: {
    				description: "Vähennys työntekijän palkasta jaetaan palkkakausiin (oletus 12 kuukautta).",
    				label: "Vähennys jaetaan palkkakausille"
    			},
    			singleDeduction: {
    				description: "Vähennys työntekijän palkasta otetaan kerralla (tästä palkkalaskelmasta)",
    				label: "Vähennys palkasta kerralla"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus \"Ei vähennystä\"",
    				label: "Ei valintaa"
    			}
    		},
    		TaxCardIncomeType: {
    			externalSalaries: {
    				description: "",
    				label: "Ulkopuoliset palkat"
    			},
    			previousEmployerSalaries: {
    				description: "Merkitse tähän työntekijän edellisten työnantajien kuluvana verovuonna maksamat palkat yhteensä euroina.",
    				label: "Edellisten työnantajien maksamat palkat"
    			},
    			salaxyCalculation: {
    				description: "",
    				label: "Salaxy laskelma"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon"
    			},
    			diff: {
    				description: "",
    				label: "Diff"
    			},
    			sharedCardExtSum: {
    				description: "",
    				label: "SharedCardExtSum"
    			}
    		},
    		TaxcardKind: {
    			defaultYearly: {
    				label: "Alkuperäinen verokortti"
    			},
    			historical: {
    				label: "Vanha verokortti"
    			},
    			noTaxCard: {
    				label: "Ei verokorttia (60%)"
    			},
    			noWithholdingHousehold: {
    				label: "Ei ennakonpidätystä (alle 1500€)"
    			},
    			others: {
    				label: "Muu verokortti"
    			},
    			replacement: {
    				label: "Muutosverokortti"
    			},
    			"undefined": {
    				label: "Verokorttia ei ole määritetty"
    			}
    		},
    		TaxcardState: {
    			employerAdded: {
    				label: "Työnantajan lisäämä"
    			},
    			expired: {
    				label: "Vanhentunut"
    			},
    			"new": {
    				label: "Uusi verokortti"
    			},
    			replaced: {
    				label: "Korvattu (ei käytössä)"
    			},
    			shared: {
    				label: "Työntekijän jakama"
    			},
    			sharedApproved: {
    				label: "Työnantajan hyväksymä"
    			},
    			sharedRejected: {
    				label: "Työnantaja hylännyt"
    			},
    			sharedRejectedWithoutOpen: {
    				label: "Työnantaja hylännyt avaamatta"
    			},
    			sharedWaiting: {
    				label: "Odottaa hyväksyntää"
    			},
    			verifiedVero: {
    				label: "Haettu Verolta"
    			},
    			approved: {
    				description: "",
    				label: "Approved"
    			}
    		},
    		TaxcardValidity: {
    			expired: {
    				label: "Vanhentunut"
    			},
    			future: {
    				label: "Tuleva"
    			},
    			"undefined": {
    				label: "Virhe"
    			},
    			valid: {
    				label: "Voimassa"
    			},
    			validJanuary: {
    				label: "Tammikuun voimassa"
    			}
    		},
    		TaxDeductionWorkCategories: {
    			carework: {
    				description: "",
    				label: "Hoiva- tai hoitotyö"
    			},
    			homeImprovement: {
    				description: "",
    				label: "Asunnon kunnossapito- tai perusparannustyö"
    			},
    			householdwork: {
    				description: "",
    				label: "Kotitaloustyö"
    			},
    			none: {
    				description: "",
    				label: "Ei valintaa"
    			},
    			ownPropety: {
    				description: "",
    				label: "Omassa käytössä oleva asunto tai vapaa-ajan asunto"
    			},
    			relativesProperty: {
    				description: "",
    				label: "Vanhempien, isovanhempien yms. käytössä oleva asunto tai vapaa-ajan asunto."
    			}
    		},
    		TesSubtype: {
    			constructionCarpenter: {
    				description: "",
    				label: "Construction Carpenter"
    			},
    			constructionFloor: {
    				description: "",
    				label: "Construction Floor"
    			},
    			constructionFreeContract: {
    				description: "",
    				label: "Construction Free Contract"
    			},
    			constructionOther: {
    				description: "",
    				label: "Construction Other"
    			},
    			notSelected: {
    				description: "",
    				label: "Not Selected"
    			}
    		},
    		TestEnum: {
    			valueEight: {
    				description: "",
    				label: "Value Eight"
    			},
    			valueFive: {
    				description: "",
    				label: "Value Five"
    			},
    			valueFour: {
    				description: "",
    				label: "Value Four"
    			},
    			valueNine: {
    				description: "",
    				label: "Value Nine"
    			},
    			valueOne: {
    				description: "",
    				label: "Value One"
    			},
    			valueSevdescription: "",
    			valueSevlabel: "Value Seven",
    			valueSix: {
    				description: "",
    				label: "Value Six"
    			},
    			valueTdescription: "",
    			valueThree: {
    				description: "",
    				label: "Value Three"
    			},
    			valueTlabel: "Value Ten",
    			valueTwo: {
    				description: "",
    				label: "Value Two"
    			},
    			valueZero: {
    				description: "",
    				label: "Value Zero"
    			},
    			valueSeven: {
    				description: "",
    				label: "ValueSeven"
    			},
    			valueTen: {
    				description: "",
    				label: "ValueTen"
    			}
    		},
    		UnionPaymentKind: {
    			fixed: {
    				description: "Yleinen kiinteä AY-jäsenmaksu.",
    				label: "AY-jäsenmaksu, kiinteä summa"
    			},
    			percentage: {
    				description: "Yleinen jäsenmaksu, lasketaan %-osuudella palkasta.",
    				label: "AY-jäsenmaksu, %-osuus palkasta"
    			},
    			other: {
    				description: "Muu liitto",
    				label: "AY-jäsenmaksu, muu liitto"
    			},
    			raksaNormal: {
    				description: "Rakennusliiton normaalijäsenyys",
    				label: "AY-jäsenmaksu, Rakennusliitto A-jäsen"
    			},
    			raksaUnemploymentOnly: {
    				description: "Rakennusliiton jäsenyys vain työttömyyskassassa",
    				label: "AY-jäsenmaksu, Rakennusliitto OA- tai TA-jäsen"
    			},
    			"undefined": {
    				description: "AY-jäsenmaksun tyyppiä ei valittu",
    				label: "Ei jäsenyyttä"
    			}
    		},
    		UnionPaymentType: {
    			notSelected: {
    				description: "",
    				label: "Ei jäsenyyttä"
    			},
    			other: {
    				description: "",
    				label: "Muu liitto..."
    			},
    			raksaNormal: {
    				description: "",
    				label: "Rakennusliitto, A-jäsen"
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: "Rakennusliitto, OA- tai TA-jäsen"
    			}
    		},
    		WageBasis: {
    			hourly: {
    				label: "Tuntipalkka"
    			},
    			monthly: {
    				label: "Kuukausipalkka"
    			},
    			other: {
    				label: "Palkanmaksu ilman lomalaskentaa"
    			},
    			performanceBased: {
    				label: "Suorituspalkka"
    			},
    			"undefined": {
    				label: "Valitse..."
    			}
    		},
    		WebSiteUserRole: {
    			company: {
    				description: "",
    				label: "Yrittäjä"
    			},
    			household: {
    				description: "",
    				label: "Työnantaja (kotitalous)"
    			},
    			none: {
    				description: "",
    				label: "Valitse rooli..."
    			},
    			worker: {
    				description: "",
    				label: "Työntekijä"
    			}
    		},
    		TotalWorkerPaymentKind: {
    			totalWorkerPayment: {
    				description: "Rahapalkka ja kulukorvaukset",
    				label: "Rahapalkka ja kulukorvaukset"
    			},
    			totalWorkerWorkerPaymentExcludingExpenses: {
    				description: "Rahapalkka, ei sisällä kulukorvauksia",
    				label: "Rahapalkka, ei sisällä kulukorvauksia"
    			},
    			totalWorkerPaymentAndTax: {
    				description: "Rahapalkka ja ennakonpidätys",
    				label: "Rahapalkka ja ennakonpidätys"
    			},
    			totalWorkerWorkerPaymentAndTaxExcludingExpenses: {
    				description: "Rahapalkka ja ennakonpidätys, ei sisällä kulukorvauksia",
    				label: "Rahapalkka ja ennakonpidätys, ei sisällä kulukorvauksia"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			}
    		},
    		WorkingTimeCompensationKind: {
    			emergencyWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan hätätyöstä. Hätätyöllä tarkoitetaan hätätilanteessa säännöllisen työajan ylittävänä aikana teetettävää työtä.",
    				label: "Hätätyökorvaus"
    			},
    			eveningShiftAllowance: {
    				description: "Vuorotyökorvaus, joka maksetaan iltavuorosta.",
    				label: "Iltavuorolisä"
    			},
    			eveningWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan iltatyöstä. Iltatyökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Iltatyökorvaus"
    			},
    			extraWorkPremium: {
    				description: "Työaikakorvaus, joka maksetaan lisätyöstä.",
    				label: "Lisätyökorvaus"
    			},
    			nightShiftCompensation: {
    				description: "Vuorotyökorvaus, joka maksetaan yövuorosta.",
    				label: "Yövuorolisä"
    			},
    			nightWorkAllowance: {
    				description: "Työaikakorvaus, jota maksetaan yötyöstä.",
    				label: "Yötyökorvaus"
    			},
    			otherCompensation: {
    				description: "Muiden maksettujen työaika- ja poikkeustilakorvausten yhteissumma. Näitä ovat mm. aattotyökorvaukset, arkipyhäkorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella\n maksettavat korvaukset ja lisät.",
    				label: "Muu maksettu lisä"
    			},
    			overtimeCompensation: {
    				description: "Ylityöstä maksettava työaikakorvaus. Ylityönä pidetään työtä, joka tehdään työnantajan aloitteesta sovitun ylityöaikarajan ylittävänä aikana.",
    				label: "Ylityökorvaus"
    			},
    			saturdayPay: {
    				description: "Työaikakorvaus, joka maksetaan lauantaityöstä.\n Lauantaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Lauantaityökorvaus"
    			},
    			standByCompensation: {
    				description: "Palkka, joka maksetaan siitä, että työntekijä on tavoitettavissa ja työnantaja voi tarvittaessa kutsua hänet hoitamaan palvelussuhteeseen kuuluvia tehtäviä.",
    				label: "Varallaolokorvaus"
    			},
    			sundayWorkCompensation: {
    				description: "Sunnuntaityöstä maksettava työaikakorvaus. Sunnuntaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Sunnuntaityökorvaus"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			waitingTimeCompensation: {
    				description: "Korvaus, johon työntekijällä on oikeus, jos palkanmaksu viivästyy työsuhteen päätyttyä. Ellei toisin sovita, palkka tulee maksaa palkanmaksukauden\n viimeisenä päivänä. Odotusajan korvausta voidaan\n maksaa enintään kuudelta päivältä.",
    				label: "Odotusajan korvaus"
    			},
    			weeklyRestCompensation: {
    				description: "Työaikakorvaus, jota maksetaan viikkolevon aikana tehdystä työstä.",
    				label: "Viikkolepokorvaus"
    			}
    		},
    		YtjCompanyType: {
    			alandFederation: {
    				description: "",
    				label: "Ahvenanmaan liikelaitos"
    			},
    			asoAssociation: {
    				description: "",
    				label: "Asumisoikeusyhdistys"
    			},
    			association: {
    				description: "",
    				label: "Aatteellinen yhdistys"
    			},
    			condominium: {
    				description: "",
    				label: "Asunto-osakeyhtiö"
    			},
    			cooperativeBank: {
    				description: "",
    				label: "Osuuspankki"
    			},
    			europeanCooperative: {
    				description: "",
    				label: "Eurooppaosuuskunta"
    			},
    			europeanCooperativeBank: {
    				description: "",
    				label: "Eurooppaosuuspankki"
    			},
    			federationOfMunicipalitiesEstablishment: {
    				description: "",
    				label: "Kuntainliiton liikelaitos"
    			},
    			financialAssociation: {
    				description: "",
    				label: "Taloudellinen yhdistys"
    			},
    			forestCareAssociation: {
    				description: "",
    				label: "Metsänhoitoyhdistys"
    			},
    			foundation: {
    				description: "",
    				label: "Säätiö"
    			},
    			housingCooperative: {
    				description: "",
    				label: "Asunto-osuuskunta"
    			},
    			hypoAssociation: {
    				description: "",
    				label: "Hypoteekkiyhdistys"
    			},
    			insuranceAssociation: {
    				description: "",
    				label: "Vakuutusyhdistys"
    			},
    			ky: {
    				description: "",
    				label: "Kommandiittiyhtiö"
    			},
    			municipalEstablishment: {
    				description: "",
    				label: "Kunnallinen liikelaitos"
    			},
    			mutualInsuranceAssociation: {
    				description: "",
    				label: "Keskinäinen vahinkovak.yhdistys"
    			},
    			noCompanyType: {
    				description: "",
    				label: "Ei yritysmuotoa"
    			},
    			openCompany: {
    				description: "",
    				label: "Avoin yhtiö"
    			},
    			osuuskunta: {
    				description: "",
    				label: "Osuuskunta"
    			},
    			otherAssociation: {
    				description: "",
    				label: "Muu yhdistys"
    			},
    			otherFinancialAssociation: {
    				description: "",
    				label: "Muu taloudellinen yhdistys"
    			},
    			oy: {
    				description: "",
    				label: "Osakeyhtiö"
    			},
    			privateEntrepreneur: {
    				description: "",
    				label: "Yksityinen elinkeinonharjoittaja l. toiminimi"
    			},
    			reindeerHerdingCooperative: {
    				description: "",
    				label: "Paliskunta"
    			},
    			savingsBank: {
    				description: "",
    				label: "Säästöpankki"
    			},
    			specialPurposeAssociation: {
    				description: "",
    				label: "Erityislainsäädäntöön perustuva yhdistys"
    			},
    			stateEstablishment: {
    				description: "",
    				label: "Valtion liikelaitos"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon yritysmuoto"
    			},
    			mutualRealEstateCompany: {
    				description: "",
    				label: "MutualRealEstateCompany"
    			},
    			taxableGrouping: {
    				description: "",
    				label: "TaxableGrouping"
    			}
    		},
    		AccountantType: {
    			primaryPartner: {
    				description: "",
    				label: "PrimaryPartner"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			unlinkedAccountingOnly: {
    				description: "",
    				label: "UnlinkedAccountingOnly"
    			},
    			unlinkedPrimaryPartner: {
    				description: "",
    				label: "UnlinkedPrimaryPartner"
    			}
    		},
    		AccountingPeriodClosingOption: {
    			"default": {
    				description: "",
    				label: "Default"
    			}
    		},
    		AccountingReportRowType: {
    			childRow: {
    				description: "",
    				label: "ChildRow"
    			},
    			credit: {
    				description: "",
    				label: "Credit"
    			},
    			debit: {
    				description: "",
    				label: "Debit"
    			},
    			groupHeader: {
    				description: "",
    				label: "GroupHeader"
    			},
    			groupTotal: {
    				description: "",
    				label: "GroupTotal"
    			},
    			total: {
    				description: "",
    				label: "Total"
    			}
    		},
    		AccountingReportTableType: {
    			classic: {
    				description: "",
    				label: "Classic"
    			},
    			mapped: {
    				description: "",
    				label: "Mapped"
    			},
    			simple: {
    				description: "",
    				label: "Simple"
    			}
    		},
    		ApiListItemType: {
    			accountBase: {
    				description: "",
    				label: "AccountBase"
    			},
    			accountProducts: {
    				description: "",
    				label: "AccountProducts"
    			},
    			article: {
    				description: "",
    				label: "Article"
    			},
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			calculationPaid: {
    				description: "",
    				label: "CalculationPaid"
    			},
    			certificate: {
    				description: "",
    				label: "Certificate"
    			},
    			companyAccount: {
    				description: "",
    				label: "CompanyAccount"
    			},
    			eSalaryPayment: {
    				description: "",
    				label: "ESalaryPayment"
    			},
    			earningsPayment: {
    				description: "",
    				label: "EarningsPayment"
    			},
    			emailMessage: {
    				description: "",
    				label: "EmailMessage"
    			},
    			employment: {
    				description: "",
    				label: "Employment"
    			},
    			historical: {
    				description: "",
    				label: "Historical"
    			},
    			holidayYear: {
    				description: "",
    				label: "HolidayYear"
    			},
    			ifInsuranceOrder: {
    				description: "",
    				label: "IfInsuranceOrder"
    			},
    			invoice: {
    				description: "",
    				label: "Invoice"
    			},
    			messageThread: {
    				description: "",
    				label: "MessageThread"
    			},
    			onboarding: {
    				description: "",
    				label: "Onboarding"
    			},
    			payment: {
    				description: "",
    				label: "Payment"
    			},
    			payrollDetails: {
    				description: "",
    				label: "PayrollDetails"
    			},
    			personAccount: {
    				description: "",
    				label: "PersonAccount"
    			},
    			profile: {
    				description: "",
    				label: "Profile"
    			},
    			report: {
    				description: "",
    				label: "Report"
    			},
    			sessionUserCredential: {
    				description: "",
    				label: "SessionUserCredential"
    			},
    			taxcard: {
    				description: "",
    				label: "Taxcard"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			varmaPensionOrder: {
    				description: "",
    				label: "VarmaPensionOrder"
    			},
    			workerAbsences: {
    				description: "",
    				label: "WorkerAbsences"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			},
    			workerInvitation: {
    				description: "",
    				label: "WorkerInvitation"
    			}
    		},
    		BankPaymentType: {
    			bankFees: {
    				description: "",
    				label: "BankFees"
    			},
    			error: {
    				description: "",
    				label: "Error"
    			},
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			insuranceGrossPayment: {
    				description: "",
    				label: "InsuranceGrossPayment"
    			},
    			insuranceLahiTapiola: {
    				description: "",
    				label: "InsuranceLahiTapiola"
    			},
    			netSalary: {
    				description: "",
    				label: "NetSalary"
    			},
    			netvisorPayment: {
    				description: "",
    				label: "NetvisorPayment"
    			},
    			palkkausFees: {
    				description: "",
    				label: "PalkkausFees"
    			},
    			partialRefundPayment: {
    				description: "",
    				label: "PartialRefundPayment"
    			},
    			paytrailBatch: {
    				description: "",
    				label: "PaytrailBatch"
    			},
    			paytrailFees: {
    				description: "",
    				label: "PaytrailFees"
    			},
    			refundPayment: {
    				description: "",
    				label: "RefundPayment"
    			},
    			salaryAdvance: {
    				description: "",
    				label: "SalaryAdvance"
    			},
    			salaryGrossPayment: {
    				description: "",
    				label: "SalaryGrossPayment"
    			},
    			saldoYearBegin: {
    				description: "",
    				label: "SaldoYearBegin"
    			},
    			saldoYearEnd: {
    				description: "",
    				label: "SaldoYearEnd"
    			},
    			taxAccount: {
    				description: "",
    				label: "TaxAccount"
    			},
    			tvr: {
    				description: "",
    				label: "Tvr"
    			},
    			tvrRfnd: {
    				description: "",
    				label: "TvrRfnd"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			tyelEloContract: {
    				description: "",
    				label: "TyelEloContract"
    			},
    			tyelEloInitialImport: {
    				description: "",
    				label: "TyelEloInitialImport"
    			},
    			tyelEloTemp: {
    				description: "",
    				label: "TyelEloTemp"
    			},
    			tyelEteraContract: {
    				description: "",
    				label: "TyelEteraContract"
    			},
    			tyelEteraTemp: {
    				description: "",
    				label: "TyelEteraTemp"
    			},
    			tyelEteraUnknown: {
    				description: "",
    				label: "TyelEteraUnknown"
    			},
    			tyelIlmarinenContract: {
    				description: "",
    				label: "TyelIlmarinenContract"
    			},
    			tyelIlmarinenTemp: {
    				description: "",
    				label: "TyelIlmarinenTemp"
    			},
    			tyelVarmaContract: {
    				description: "",
    				label: "TyelVarmaContract"
    			},
    			tyelVarmaTemp: {
    				description: "",
    				label: "TyelVarmaTemp"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			},
    			unionRaksa: {
    				description: "",
    				label: "UnionRaksa"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				description: "",
    				label: "AccommodationBenefit"
    			},
    			mealBenefit: {
    				description: "",
    				label: "MealBenefit"
    			},
    			otherBenefits: {
    				description: "",
    				label: "OtherBenefits"
    			},
    			telephoneBenefit: {
    				description: "",
    				label: "TelephoneBenefit"
    			}
    		},
    		CalcGroup: {
    			baseSalary: {
    				description: "",
    				label: "BaseSalary"
    			},
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			disabled: {
    				description: "",
    				label: "Disabled"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			otherNoPayment: {
    				description: "",
    				label: "OtherNoPayment"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			totals: {
    				description: "",
    				label: "Totals"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationResultRowType: {
    			totalBenefits: {
    				description: "",
    				label: "TotalBenefits"
    			},
    			totalExpenses: {
    				description: "",
    				label: "TotalExpenses"
    			},
    			totalPalkkaus: {
    				description: "",
    				label: "TotalPalkkaus"
    			},
    			totalPayment: {
    				description: "",
    				label: "TotalPayment"
    			},
    			totalPension: {
    				description: "",
    				label: "TotalPension"
    			},
    			totalPensionEmployer: {
    				description: "",
    				label: "TotalPensionEmployer"
    			},
    			totalPensionEmployerDebt: {
    				description: "",
    				label: "TotalPensionEmployerDebt"
    			},
    			totalPensionPayment: {
    				description: "",
    				label: "TotalPensionPayment"
    			},
    			totalPensionWorker: {
    				description: "",
    				label: "TotalPensionWorker"
    			},
    			totalSalary: {
    				description: "",
    				label: "TotalSalary"
    			},
    			totalSocialSecurityEmployer: {
    				description: "",
    				label: "TotalSocialSecurityEmployer"
    			},
    			totalSocialSecurityEmployerDebt: {
    				description: "",
    				label: "TotalSocialSecurityEmployerDebt"
    			},
    			totalSocialSecurityEmployerPayment: {
    				description: "",
    				label: "TotalSocialSecurityEmployerPayment"
    			},
    			totalTax: {
    				description: "",
    				label: "TotalTax"
    			},
    			totalTaxPayment: {
    				description: "",
    				label: "TotalTaxPayment"
    			},
    			totalUnemployment: {
    				description: "",
    				label: "TotalUnemployment"
    			},
    			totalUnemploymentEmployer: {
    				description: "",
    				label: "TotalUnemploymentEmployer"
    			},
    			totalUnemploymentEmployerDebt: {
    				description: "",
    				label: "TotalUnemploymentEmployerDebt"
    			},
    			totalUnemploymentPayment: {
    				description: "",
    				label: "TotalUnemploymentPayment"
    			},
    			totalUnemploymentWorker: {
    				description: "",
    				label: "TotalUnemploymentWorker"
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: "TotalWorkerPayment"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationRowCategory: {
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			holidays: {
    				description: "",
    				label: "Holidays"
    			},
    			other: {
    				description: "",
    				label: "Other"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			salaryCalc: {
    				description: "",
    				label: "SalaryCalc"
    			}
    		},
    		CalculationStatusCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			editable: {
    				description: "",
    				label: "Editable"
    			},
    			readonly: {
    				description: "",
    				label: "Readonly"
    			},
    			received: {
    				description: "",
    				label: "Received"
    			},
    			sent: {
    				description: "",
    				label: "Sent"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			}
    		},
    		ESalaryPaymentStatus: {
    			calculationCreated: {
    				description: "",
    				label: "CalculationCreated"
    			},
    			created: {
    				description: "",
    				label: "Created"
    			},
    			employerResolved: {
    				description: "",
    				label: "EmployerResolved"
    			},
    			employmentResolved: {
    				description: "",
    				label: "EmploymentResolved"
    			},
    			opened: {
    				description: "",
    				label: "Opened"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "PaymentCanceled"
    			},
    			paymentMethodResolved: {
    				description: "",
    				label: "PaymentMethodResolved"
    			},
    			paymentStarted: {
    				description: "",
    				label: "PaymentStarted"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "PaymentSucceeded"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			},
    			taxcardResolved: {
    				description: "",
    				label: "TaxcardResolved"
    			}
    		},
    		EmploymentRelationStatus: {
    			archived: {
    				description: "",
    				label: "Archived"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			secondaryCurrent: {
    				description: "",
    				label: "SecondaryCurrent"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		Holiday: {
    			allSaintsDay: {
    				description: "",
    				label: "AllSaintsDay"
    			},
    			ascensionDay: {
    				description: "",
    				label: "AscensionDay"
    			},
    			christmasDay: {
    				description: "",
    				label: "ChristmasDay"
    			},
    			christmasEve: {
    				description: "",
    				label: "ChristmasEve"
    			},
    			easterMonday: {
    				description: "",
    				label: "EasterMonday"
    			},
    			easterSaturday: {
    				description: "",
    				label: "EasterSaturday"
    			},
    			easterSunday: {
    				description: "",
    				label: "EasterSunday"
    			},
    			epiphany: {
    				description: "",
    				label: "Epiphany"
    			},
    			goodFriday: {
    				description: "",
    				label: "GoodFriday"
    			},
    			independenceDay: {
    				description: "",
    				label: "IndependenceDay"
    			},
    			mayDay: {
    				description: "",
    				label: "MayDay"
    			},
    			midsummerDay: {
    				description: "",
    				label: "MidsummerDay"
    			},
    			midsummerEve: {
    				description: "",
    				label: "MidsummerEve"
    			},
    			newYearsDay: {
    				description: "",
    				label: "NewYearsDay"
    			},
    			pentecost: {
    				description: "",
    				label: "Pentecost"
    			},
    			stStephensDay: {
    				description: "",
    				label: "StStephensDay"
    			}
    		},
    		HolidayAccrualSource: {
    			calcDraft: {
    				description: "",
    				label: "CalcDraft"
    			},
    			calcPaid: {
    				description: "",
    				label: "CalcPaid"
    			},
    			initial: {
    				description: "",
    				label: "Initial"
    			},
    			manual: {
    				description: "",
    				label: "Manual"
    			}
    		},
    		HolidayGroup: {
    			holiday: {
    				description: "",
    				label: "Holiday"
    			},
    			nonBanking: {
    				description: "",
    				label: "NonBanking"
    			},
    			official: {
    				description: "",
    				label: "Official"
    			}
    		},
    		IncomeEarnerType: {
    			athlete: {
    				description: "",
    				label: "Urheilija"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "Valtion työllisyysmäärärahoilla työllistetty"
    			},
    			entrepreneurOrFarmerNoPensionRequired: {
    				description: "",
    				label: "Yrittäjä, ei YEL- tai MYEL-vakuuttamisvelvollisuutta"
    			},
    			jointOwnerWithPayer: {
    				description: "",
    				label: "Osakasasemassa maksajaan"
    			},
    			keyEmployee: {
    				description: "",
    				label: "Avainhenkilö"
    			},
    			leasedEmployeeLivingAbroad: {
    				description: "",
    				label: "Ulkomailla asuva vuokratyöntekijä"
    			},
    			netOfTaxContract: {
    				description: "",
    				label: "Työnantaja maksaa työntekijän puolesta verot (nettopalkkasopimus)"
    			},
    			organization: {
    				description: "",
    				label: "Yhteisö"
    			},
    			partialOwner: {
    				description: "",
    				label: "Osaomistaja"
    			},
    			performingArtist: {
    				description: "",
    				label: "Esiintyvä taiteilija"
    			},
    			personWorkingAbroad: {
    				description: "",
    				label: "Ulkomailla työskentelevä henkilö"
    			},
    			personWorkingInFrontierDistrict: {
    				description: "",
    				label: "Rajakunnassa työskentelevä henkilö"
    			},
    			personWorkingOnAlandFerry: {
    				description: "",
    				label: "Ahvenanmaan maantielautalla työskentelevä henkilö"
    			},
    			restrictedPeriodInFinland: {
    				description: "",
    				label: "Tulonsaaja on oleskellut Suomessa enintään 183 päivää verosopimuksessa tarkoitettuna ajanjaksona"
    			}
    		},
    		IncomeLogDiff: {
    			changed: {
    				description: "",
    				label: "Changed"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			"new": {
    				description: "",
    				label: "New"
    			},
    			removed: {
    				description: "",
    				label: "Removed"
    			}
    		},
    		InvoicePaymentType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			}
    		},
    		InvoiceType: {
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			gross: {
    				description: "",
    				label: "Gross"
    			},
    			net: {
    				description: "",
    				label: "Net"
    			},
    			pension: {
    				description: "",
    				label: "Pension"
    			},
    			tax: {
    				description: "",
    				label: "Tax"
    			},
    			taxSocialSecurity: {
    				description: "",
    				label: "TaxSocialSecurity"
    			},
    			taxWithholding: {
    				description: "",
    				label: "TaxWithholding"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			unemployment: {
    				description: "",
    				label: "Unemployment"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			}
    		},
    		IrRowSourceType: {
    			manual: {
    				description: "",
    				label: "Manual"
    			},
    			usecase: {
    				description: "",
    				label: "Usecase"
    			},
    			usecaseV02: {
    				description: "",
    				label: "UsecaseV02"
    			}
    		},
    		LegacyTaxcardType: {
    			example: {
    				description: "",
    				label: "Example"
    			},
    			freelancer: {
    				description: "",
    				label: "Freelancer"
    			},
    			noTaxCard: {
    				description: "",
    				label: "NoTaxCard"
    			},
    			noTaxPrepayment: {
    				description: "",
    				label: "NoTaxPrepayment"
    			},
    			others: {
    				description: "",
    				label: "Others"
    			},
    			sideIncome: {
    				description: "",
    				label: "SideIncome"
    			},
    			steps: {
    				description: "",
    				label: "Steps"
    			},
    			taxCardA: {
    				description: "",
    				label: "TaxCardA"
    			},
    			taxCardB: {
    				description: "",
    				label: "TaxCardB"
    			}
    		},
    		MessageFrom: {
    			otherParty: {
    				description: "",
    				label: "OtherParty"
    			},
    			owner: {
    				description: "",
    				label: "Owner"
    			},
    			system: {
    				description: "",
    				label: "System"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthError: {
    			"<any>access_denied": {
    				description: "",
    				label: "Access_denied"
    			},
    			"<any>invalid_request": {
    				description: "",
    				label: "Invalid_request"
    			},
    			"<any>invalid_scope": {
    				description: "",
    				label: "Invalid_scope"
    			},
    			"<any>server_error": {
    				description: "",
    				label: "Server_error"
    			},
    			"<any>temporarily_unavailable": {
    				description: "",
    				label: "Temporarily_unavailable"
    			},
    			"<any>unauthorized_client": {
    				description: "",
    				label: "Unauthorized_client"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			"<any>unsupported_response_type": {
    				description: "",
    				label: "Unsupported_response_type"
    			}
    		},
    		OAuthGrantType: {
    			"<any>authorization_code": {
    				description: "",
    				label: "Authorization_code"
    			},
    			"<any>client_credentials": {
    				description: "",
    				label: "Client_credentials"
    			},
    			"<any>jwtBearer": {
    				description: "",
    				label: "JwtBearer"
    			},
    			"<any>password": {
    				description: "",
    				label: "Password"
    			},
    			"<any>refresh_token": {
    				description: "",
    				label: "Refresh_token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthResponseType: {
    			"<any>code": {
    				description: "",
    				label: "Code"
    			},
    			"<any>token": {
    				description: "",
    				label: "Token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthSalaxyAuthorizeMode: {
    			"<any>sign_in": {
    				description: "",
    				label: "Sign_in"
    			},
    			"<any>sign_up": {
    				description: "",
    				label: "Sign_up"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthTokenType: {
    			"<any>bearer": {
    				description: "",
    				label: "Bearer"
    			},
    			"<any>mac": {
    				description: "",
    				label: "Mac"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentMethod: {
    			sepa: {
    				description: "",
    				label: "Sepa"
    			},
    			siirto: {
    				description: "",
    				label: "Siirto"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PeriodDateKind: {
    			paidAtDate: {
    				description: "",
    				label: "PaidAtDate"
    			},
    			salaryDate: {
    				description: "",
    				label: "SalaryDate"
    			}
    		},
    		Role: {
    			accountant: {
    				description: "",
    				label: "Accountant"
    			},
    			accountantCandidate: {
    				description: "",
    				label: "AccountantCandidate"
    			},
    			pricingPartner: {
    				description: "",
    				label: "PricingPartner"
    			},
    			trustedPartner: {
    				description: "",
    				label: "TrustedPartner"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SalaryType: {
    			compensation: {
    				description: "",
    				label: "Compensation"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			yel: {
    				description: "",
    				label: "Yel"
    			}
    		},
    		SettingsStatus: {
    			checked: {
    				description: "",
    				label: "Checked"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SharingUriType: {
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			url: {
    				description: "",
    				label: "Url"
    			}
    		},
    		SystemRole: {
    			anon: {
    				description: "",
    				label: "Anon"
    			},
    			any: {
    				description: "",
    				label: "Any"
    			},
    			auth: {
    				description: "",
    				label: "Auth"
    			},
    			company: {
    				description: "",
    				label: "Company"
    			},
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			fiOy: {
    				description: "",
    				label: "FiOy"
    			},
    			fiRy: {
    				description: "",
    				label: "FiRy"
    			},
    			fiTm: {
    				description: "",
    				label: "FiTm"
    			},
    			hidden: {
    				description: "",
    				label: "Hidden"
    			},
    			household: {
    				description: "",
    				label: "Household"
    			},
    			init: {
    				description: "",
    				label: "Init"
    			},
    			nonVerified: {
    				description: "",
    				label: "NonVerified"
    			},
    			partnerMessaging: {
    				description: "",
    				label: "PartnerMessaging"
    			},
    			partnerService: {
    				description: "",
    				label: "PartnerService"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			prod: {
    				description: "",
    				label: "Prod"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			worker: {
    				description: "",
    				label: "Worker"
    			}
    		},
    		TableFormat: {
    			csv: {
    				description: "",
    				label: "Csv"
    			},
    			txt: {
    				description: "",
    				label: "Txt"
    			},
    			xlsx: {
    				description: "",
    				label: "Xlsx"
    			}
    		},
    		TaxReportHandling: {
    			always: {
    				description: "",
    				label: "Palkkaus.fi lähettää työnantajan erillisilmoituksen siinäkin tapauksessa, jossa työnantaja ei ole maksanut palkkoja viimeisen puolen vuoden aikana"
    			},
    			"default": {
    				description: "",
    				label: "Palkkaus.fi lähettää kaikki ilmoitukset"
    			},
    			noMonthlyReport: {
    				description: "",
    				label: "Palkkaus.fi ei lähetä työnantajan erillisilmoitusta"
    			},
    			noZeroSalaryReport: {
    				description: "",
    				label: "Palkkaus.fi ei lähetä Nollailmoitusta"
    			}
    		},
    		TaxcardApprovalMethod: {
    			approve: {
    				description: "",
    				label: "Approve"
    			},
    			assureWaiting: {
    				description: "",
    				label: "AssureWaiting"
    			},
    			reject: {
    				description: "",
    				label: "Reject"
    			}
    		},
    		TaxcardCalcDiffCheck: {
    			commitDiff: {
    				description: "",
    				label: "CommitDiff"
    			},
    			getDiff: {
    				description: "",
    				label: "GetDiff"
    			},
    			noCheck: {
    				description: "",
    				label: "NoCheck"
    			}
    		},
    		ThreadedMessageType: {
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			email: {
    				description: "",
    				label: "Email"
    			},
    			payroll: {
    				description: "",
    				label: "Payroll"
    			},
    			sms: {
    				description: "",
    				label: "Sms"
    			},
    			text: {
    				description: "",
    				label: "Text"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			}
    		},
    		TransactionCode: {
    			"null": {
    				description: "Ei käytössä: Palkkaus.fi-palvelussa ei käytetä ilmoitustapaa 1.",
    				label: "Palkkasumma, joka on työtapaturma- ja ammattitautivakuutusmaksun alainen"
    			},
    			timeRatePay: {
    				description: "Palkka, jota maksetaan työhön käytetyn ajan perusteella. Aikapalkka voidaan määritellä esimerkiksi kalenterikuukaudelle.",
    				label: "Aikapalkka"
    			},
    			initiativeFee: {
    				description: "Kertapalkkio, joka maksetaan aloitteen tekijälle esimerkiksi toiminnan tai tuotteen kehittämiseen liittyvästä aloitteesta.",
    				label: "Aloitepalkkio"
    			},
    			bonusPay: {
    				description: "Kertaluonteinen palkkiopalkka, jonka suuruus määräytyy kokonaan tai osittain sen mukaan, kuinka hyvin tavoitteeksi asetettu työtulos saavutetaan. Työtuloksen mittana voi olla määrä, laatu tai jokin muu tulostekijä.\r\n\r\nBonuspalkka eroaa provisiopalkasta siten, että bonuspalkka on kertaluonteinen ja provisio on jatkuvaluonteinen palkkiopalkka. Kumpikin näistä perustuu henkilön tai ryhmän ennalta asetetun tavoitteen saavuttamiseen. Bonuspalkka eroaa voittopalkkiosta siten, että voittopalkkio maksetaan ilman ennalta sovittua suunnitelmaa, yrityksen voiton perusteella.",
    				label: "Bonuspalkka"
    			},
    			complementaryWagePaidDuringBenefitPeriod: {
    				description: "Etuutta saavalle työntekijälle maksettu palkka, joka kattaa täyden palkan ja maksetun etuuden välisen erotuksen.\r\n\r\nMaksettu etuus voi olla esimerkiksi Kelan maksama sairauspäiväraha. Kun työnantaja maksaa etuusajalta täydennyspalkkaa, Kela maksaa etuuden suoraan työntekijälle, ei työnantajalle.",
    				label: "Etuusajalta maksettu täydennyspalkka"
    			},
    			emergencyWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan hätätyöstä. Hätätyöllä tarkoitetaan hätätilanteessa säännöllisen työajan ylittävänä aikana teetettävää työtä.",
    				label: "Hätätyökorvaus"
    			},
    			eveningWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan iltatyöstä. Iltatyökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Iltatyökorvaus"
    			},
    			eveningShiftAllowance: {
    				description: "Vuorotyökorvaus, joka maksetaan iltavuorosta.",
    				label: "Iltavuorolisä"
    			},
    			noticePeriodCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa työntekijälle palvelussuhteen päättyessä ilman irtisanomisaikaa eli tilanteessa, jossa jätetään irtisanomisaika noudattamatta ja työsuhde päättyy heti (TSL 6:4). Tulolajilla ilmoitetaan myös korvaus, jonka työnantaja maksaa, kun työntekijä irtisanoutuu yli 200 päivää kestäneen lomautuksen johdosta sekä, kun työnantaja irtisanoo lomautettuna olevan työntekijän.\r\n\r\nTulolajilla ilmoitetaan vain sellaiset erät, jotka maksetaan työsuhteen päättyessä ilman irtisanomisaikaa. Irtisanomisajalta maksettu palkka ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
    				label: "Irtisanomisajan korvaus"
    			},
    			kilometreAllowanceTaxable: {
    				description: "Kilometrikorvaus, joka ei täytä verovapaudelle asetettuja vaatimuksia.\r\n\r\nVeronalaista kilometrikorvausta voidaan maksaa sekä palvelussuhteessa oleville että muille henkilöille. Kilometrikorvauksen veronalaisuudelle on useita perusteita. Verohallinto on määrittänyt työnantajan palvelussuhteessa maksamille verovapaille kilometrikorvauksille enimmäismäärän, jonka ylittävältä osalta kilometrikorvaus on veronalaista ansiotuloa. Henkilölle toimeksiantosuhteessa maksettava kilometrikorvaus on kokonaan veronalaista ansiotuloa.\r\n\r\n**Tulolajilla ilmoitetaan vain palvelussuhteessa oleville maksetut veronalaiset kilometrikorvaukset.** Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena ja lisätään kustannus työkorvauksen määrää (ks. Työkorvaus-tulolajin selite).",
    				label: "Kilometrikorvaus (veronalainen)"
    			},
    			meetingFee: {
    				description: "Palkkio, joka maksetaan kokoukseen osallistumisesta.",
    				label: "Kokouspalkkio"
    			},
    			saturdayPay: {
    				description: "Työaikakorvaus, joka maksetaan lauantaityöstä. Lauantaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Lauantaityökorvaus"
    			},
    			extraWorkPremium: {
    				description: "Työaikakorvaus, joka maksetaan lisätyöstä.",
    				label: "Lisätyökorvaus"
    			},
    			holidayBonus: {
    				description: "Työsopimuksen tai työ- tai virkaehtosopimuksen perusteella vuosilomapalkan lisäksi maksettava suoritus, jonka suuruus määräytyy alalla sovellettavan työehtosopimuksen tai edellisenä lomanmääräytymisvuonna ansaittujen lomapäivien lukumäärän mukaan.\r\n\r\nVuosilomapalkan lisää koskee erittely- ja ilmoitusvelvollisuus riippumatta siitä, käytetäänkö kyseisestä erästä nimitystä lomaraha. Tällainen erittely tarvitaan esimerkiksi Kelaa ja työttömyyskassaa sekä työtapaturman korvauskäsittelyä varten.",
    				label: "Lomaraha"
    			},
    			lectureFee: {
    				description: "Palkkio, joka maksetaan luennon pitämisestä.",
    				label: "Luentopalkkio"
    			},
    			compensationForTrustPosition: {
    				description: "Palkkio, joka maksetaan muusta luottamustoimen hoitamisesta kuin kokouksiin osallistumisesta.",
    				label: "Luottamustoimipalkkio"
    			},
    			otherCompensation: {
    				description: "Muiden maksettujen työaika- ja poikkeustilakorvausten yhteissumma. Näitä ovat mm. aattotyökorvaukset, arkipyhäkorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella maksettavat korvaukset ja lisät.",
    				label: "Muu maksettu lisä"
    			},
    			waitingTimeCompensation: {
    				description: "Korvaus, johon työntekijällä on oikeus, jos palkanmaksu viivästyy työsuhteen päätyttyä. Ellei toisin sovita, palkka tulee maksaa palkanmaksukauden viimeisenä päivänä. Odotusajan korvausta voidaan maksaa enintään kuudelta päivältä.",
    				label: "Odotusajan korvaus"
    			},
    			workingConditionCompensation: {
    				description: "Palkanlisä, joka maksetaan työn tekemispaikan olosuhteiden perusteella. Olosuhdelisiä ovat esimerkiksi työehtosopimuksen tai työsopimuksen perusteella maksettavat lisä, kuten vaahdotuslisä, kylmänalueenlisä ja saaristolisä.",
    				label: "Olosuhdelisä"
    			},
    			partialPayDuringSickLeave: {
    				description: "Lain tai työ- tai virkaehtosopimukseen mukaisesti alennettu sairausajan palkka. Sairausajan osapalkkana ilmoitetaan se osa tuloista, jonka henkilö on saanut sairaana ollessaan alennettuna.",
    				label: "Sairausajan osapalkka"
    			},
    			commission: {
    				description: "Pysyvän palkan osan muodostava jatkuvaluonteinen palkkiopalkka, jonka määrästä on sovittu työsopimuksessa.",
    				label: "Provisiopalkka"
    			},
    			sundayWorkCompensation: {
    				description: "Sunnuntaityöstä maksettava työaikakorvaus. Sunnuntaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Sunnuntaityökorvaus"
    			},
    			benefitArisingFromSyntheticOption: {
    				description: "Optio-oikeus, jonka perusteella tulonsaaja ei ole oikeutettu merkitsemään option kohde-etuutena olevia osakkeita option toteutushintaan, vaan hän saa toteutushinnan ja toteuttamishetken mukaisen osakkeen arvon erotuksen rahana (nettoarvon tilitys). Synteettinen optio ei oikeuta osakemerkintään.\r\n\r\nKäytännössä synteettisessä optiossa on kyse rahasuorituksesta, jonka suuruus määräytyy työnantajayhtiön osakekurssin kehityksen perusteella. Synteettinen optio -nimityksen sijaan järjestelyä saatetaan kutsua osakeperusteiseksi bonukseksi tai palkitsemisjärjestelmäksi.",
    				label: "Synteettinen optio"
    			},
    			performanceBonus: {
    				description: "Palkka, joka maksetaan organisaation ennalta sovitun tulostavoitteen täyttymisen tai ylittymisen perusteella.",
    				label: "Tulospalkkio"
    			},
    			monetaryCompensationFromWorkingTimeBank: {
    				description: "Palkka, joka maksetaan työaikapankkiin säästetyn ajan perusteella.\r\n\r\nTyöaikapankista voi yleensä nostaa palkkaa vain poikkeustapauksissa kuten palvelussuhteen päättyessä tai työaikapankkia purettaessa. Joissakin työaikapankeissa on mahdollista nostaa palkkaa tai\r\nvapaa-aikaa myös velaksi.",
    				label: "Työaikapankin rahakorvaus"
    			},
    			compensationForAccruedTimeOff: {
    				description: "Työaikakorvaus, joka maksetaan ansionvähentymisestä, joka aiheutuu työajan tasaamisesta keskimääräiseen viikkotyöaikaan. \r\n\r\nTyöajantasauskorvausta maksetaan säännölliseltä työajalta. Esimerkiksi niin sanottu pekkaskorvaus on työajantasauskorvaus.",
    				label: "Työajantasauskorvaus"
    			},
    			shareIssueForEmployees: {
    				description: "Yhtiön omalle henkilökunnalle suunnattu osakeanti. Etu työsuhteeseen perustuvasta oikeudesta merkitä työnantajayrityksen osakkeita on veronalaista ansiotuloa.\r\n\r\nJos etu on henkilökunnan enemmistön käytettävissä, tulolajilla ilmoitetaan ainoastaan 10 %:n alennuksen ylittävä osuus. Jos etu ei ole henkilökunnan enemmistön käytettävissä, saatava alennus on palkkaa koko määrältään. Tällöin suorituksen ilmoittamisessa käytetään palkan muita tulolajeja.\r\n\r\nMerkintäetu koskee myös yhteisön osuuksia.",
    				label: "Työsuhteeseen perustuva osakeanti"
    			},
    			contractPay: {
    				description: "Suorituspalkka, joka maksetaan ennalta sovitun työkokonaisuuden suorittamisen perusteella. Urakkapalkkaan sisältyy sekä kiinteä että muuttuva osa.",
    				label: "Urakkapalkka "
    			},
    			damagesInConjunctionWithTermination: {
    				description: "Korvaus, jota työnantaja on velvollinen maksamaan, kun työnantaja on päättänyt palvelussuhteen henkilön työsopimuslain, kunnallisesta viranhaltijasta annetun lain tai valtion virkamieslain vastaisesti. Kohdassa ilmoitetaan myös korvaus työsuhteen purkamisesta (TSL 8:1, 55/2001) sekä korvaus perusteettomasta lomauttamisesta.\r\n\r\nPalvelussuhteen perusteettoman päättämisen johdosta maksettava korvaus voi perustua tuomioistuimen tuomioon, tuomioistuimen vahvistamaan sovintoon, palvelussuhteen osapuolia edustavien järjestöjen väliseen neuvottelutulokseen tai palvelussuhteen osapuolten väliseen sopimukseen.\r\n\r\nVahingonkorvaus päättämis- tai lomautustilanteessa tai muu siihen verrattava korvaus ei ole veronalaista tuloa, ellei sitä ole saatu veronalaisen tulon sijaan tai korvaukseksi elatuksen vähentymisestä. Veronalaista tuloa eivät myöskään ole laissa tarkemmin määritellyt hyvitykset yhteistoimintavelvoitteiden laiminlyönneistä.\r\n\r\nTulolajilla ilmoitetaan vain veronalaiset määrät eli esimerkiksi ei yhteistoimintalain tai tasa-arvolain nojalla maksettuja verovapaita hyvityksi. Verovapaita vahingonkorvauksia ei ilmoiteta tulorekisteriin.\r\n\r\nTulolajilla ei ilmoiteta irtisanomisen yhteydessä maksettavia työsuoritukseen perustuvia korvauksia vaan ne ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
    				label: "Vahingonkorvaus päättämis- ja lomautustilanteissa"
    			},
    			standByCompensation: {
    				description: "Palkka, joka maksetaan siitä, että työntekijä on tavoitettavissa ja työnantaja voi tarvittaessa kutsua hänet hoitamaan palvelussuhteeseen kuuluvia tehtäviä.",
    				label: "Varallaolokorvaus"
    			},
    			voluntaryCompensationInConjunctionWithTermination: {
    				description: "Korvaus, jonka työnantaja maksaa vapaaehtoisesti irtisanomis- tai purkamistilanteessa tai silloin, kun palvelussuhde päätetään sopimalla.\r\n\r\nPäättämistilanteessa maksettavia vapaaehtoisia korvauksia ovat esimerkiksi toimitusjohtajan erokorvaus (niin sanottu kultainen kädenpuristus), erokorvaus, eroraha ja tukipaketti. Korvaus voidaan\r\nmyöntää myös muuna kuin rahana.",
    				label: "Vapaaehtoinen korvaus päättämistilanteessa"
    			},
    			weeklyRestCompensation: {
    				description: "Työaikakorvaus, jota maksetaan viikkolevon aikana tehdystä työstä.",
    				label: "Viikkolepokorvaus"
    			},
    			profitSharingBonus: {
    				description: "Työntekijöille ilman ennalta sovittua suunnitelmaa yhtiökokouksen päätöksellä jaettava palkkio, joka muodostuu yrityksen voiton perusteella.",
    				label: "Voittopalkkio"
    			},
    			annualHolidayCompensation: {
    				description: "Vuosiloman antamisen sijaan suoritettava rahallinen korvaus. Vuosilomakorvausta voidaan maksaa työsuhteen aikana tai työsuhteen päättyessä.\r\n\r\nTulolajilla ilmoitetaan myös **rakennusalan lomaraha**. Työehtosopimuksessa lomakorvaus, lomapalkka ja lomaraha on yhdistetty lomaraha-nimiseksi korvaukseksi, joka on 18,5 % lomanmääräytymisvuoden ansioista. Pääsääntöisesti rakennusalan lomakorvauksessa on 1/3 lomarahaa ja 2/3 lomapalkkaa tai lomakorvausta riippuen siitä, jatkuuko työsuhde vai päättyikö se.",
    				label: "Vuosilomakorvaus"
    			},
    			overtimeCompensation: {
    				description: "Ylityöstä maksettava työaikakorvaus. Ylityönä pidetään työtä, joka tehdään työnantajan aloitteesta sovitun ylityöaikarajan ylittävänä aikana.",
    				label: "Ylityökorvaus"
    			},
    			nightWorkAllowance: {
    				description: "Työaikakorvaus, jota maksetaan yötyöstä.",
    				label: "Yötyökorvaus"
    			},
    			nightShiftCompensation: {
    				description: "Vuorotyökorvaus, joka maksetaan yövuorosta.",
    				label: "Yövuorolisä"
    			},
    			otherRegularCompensation: {
    				description: "Sellaisten työaika- ja poikkeustilakorvausten yhteissumma, joita maksetaan säännöllisesti ja jotka lasketaan säännöllisen työajan ansioon. Muita säännöllisiä lisiä ovat mm. luontoisedusta maksetut rahakorvaukset, aattotyökorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella säännöllisesti maksettavat korvaukset ja lisät, kuten kielilisä. Myös harvoin maksettava lisä katsotaan säännölliseksi, jos se kuuluu säännöllisen työajan ansioon.",
    				label: "Muu säännöllinen lisä"
    			},
    			compensationForUnusedCompensatoryLeave: {
    				description: "Tietyillä aloilla käytössä oleva korvaus, jota maksetaan työsuhteen päättyessä pitämättömistä vapaapäivistä. Tietyillä aloilla tehty työ kerryttää tekijälleen vastikevapaata, jota työsuhteen kuluessa pidetään vapaana. Jos työntekijä ei ole pitänyt vapaapäiviä, kun työsuhde päättyy, niistä maksetaan korvausta.\r\n\r\nVuosiloman antamisen sijaan suoritettava rahallinen korvaus ilmoitetaan tulolajilla Vuosilomakorvaus (234).",
    				label: "Vastikevapaan korvaus"
    			},
    			accommodationBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän käyttöön omistamansa tai hallitsemansa asunnon palvelussuhteeseen liittyvän vuokrasuhteen perusteella tai palkkaan sisältyvänä etuna.\r\n\r\nPalkkaus.fi-palvelussa asuntoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luontoisetu'-tulolajilla.",
    				label: "Asuntoetu"
    			},
    			interestBenefitForHousingLoan: {
    				description: "Veronalainen henkilökuntaetu, joka muodostuu silloin, kun työnantaja perii työntekijälle myöntämästään asuntolainasta vuotuista korkoa, joka on alempi kuin yleisesti markkinoilla käytössä oleva viitekorko.",
    				label: "Asuntolainan korkoetu"
    			},
    			mealAllowance: {
    				description: "Verovapaa kustannusten korvaus ateriointikuluista, jotka johtuvat työmatkasta.\r\n\r\nAteriakorvaus voidaan maksaa verovapaasti, jos työntekijällä ei ole mahdollisuutta aterioida ruokailutauon aikana tavanomaisella ruokailupaikallaan, eikä ruokailua ole järjestetty esimerkiksi kurssipakettiin kuuluvana työnantajan kustantamana ruokailuna. **Verovapaata ateriakorvausta ei voi maksaa samasta työmatkasta yhtä aikaa verovapaan päivärahan kanssa.**\r\n\r\nAteriakorvauksen perusteista ja verovapaista enimmäismääristä säädetään vuosittain annettavissa Verohallinnon päätöksissä verovapaista matkakustannusten korvauksista.\r\n\r\nKohdassa ilmoitetaan vain verovapaiden ateriakorvausten määrät. Jos maksettu korvaus ylittää Verohallinnon päätöksen enimmäismäärät tai maksetaan vastoin päätöksessä säädettyjä perusteita, suoritus ilmoitetaan sosiaalivakuutusmaksujen alaisena palkkana.\r\n\r\nAteriakorvaus voidaan ilmoittaa erikseen, mutta saatu korvaus on ilmoitettava vähintään yhteissummana Päiväraha -tulolajilla. Tällöin on lisäksi ilmoitettava tulolajiin liittyvän Korvauksen tyyppi-lisätiedon avulla, että Päiväraha-tulolajilla ilmoitettu korvaus sisältää ateriakorvauksen.\r\n\r\nJos ateriakorvaus ilmoitetaan erikseen, sitä ei lisätä enää Päiväraha -tulolajilla ilmoitettaviin määriin.",
    				label: "Ateriakorvaus"
    			},
    			carBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän tai tämän perheen käyttöön omistamansa tai hallitsemansa auton, jonka yksityiskäyttö on sallittu.\r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että työntekijä on maksanut luontoisedusta työnantajalle luontoisedun raha-arvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan. Peritty summa ilmoitetaan erikseen Autoedusta peritty korvaus- erittelylajilla.",
    				label: "Autoetu"
    			},
    			compensationForGoverningBodyMembership: {
    				description: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella.\r\n\r\nJos henkilölle maksetaan palkkiota hallintoelimen jäsenyydestä, hänelle voidaan maksaa myös verovapaita matkakustannusten korvauksia työmatkasta aiheutuneiden kulujen perusteella, jos verovapaiden korvausten muut edellytykset täyttyvät.",
    				label: "Hallintoelimen jäsenyydestä maksettu palkkio"
    			},
    			shareOfReserveDrawnFromPersonnelFund: {
    				description: "Rahasto-osuus, joka maksetaan työntekijälle henkilöstörahastosta. Henkilörahastoon osoitetut varat jaetaan rahasto-osuutena nostettaviin eriin ja käteisenä nostettaviin eriin.\r\n\r\nHenkilöstörahastosta nostetusta rahasto-osuudesta 20 % on verovapaata ja 80 %:n veronalaisesta ansiotulo-osuudesta toimitetaan ennakonpidätys. Henkilöstörahastosta nostettua ylijäämää kohdellaan verotuksessa samalla tavalla kuin henkilöstörahastosta nostettua rahasto-osuutta. Ks. Verohallinnon ohje Henkilöstörahastosta saatavien tulojen verotus (A69/200/2014).\r\n\r\nTulolajilla ilmoitetaan vain veronalainen osuus.",
    				label: "Henkilöstörahastosta nostettu rahasto-osuus ja ylijäämä (veronalainen 80%)"
    			},
    			monetaryGiftForEmployees: {
    				description: "Työnantajalta rahana tai siihen verrattavana suorituksena saatu lahja.",
    				label: "Henkilöstölle annettu rahalahja"
    			},
    			kilometreAllowanceTaxExempt: {
    				description: "Henkilön oman tai hänen hallinnassaan olevan kulkuneuvon käytöstä suoritettava kilometrikorvaus, joka täyttää verovapaudelle asetetut vaatimukset.\r\n\r\nVerovapaata kilometrikorvausta voidaan maksaa palvelussuhteessa oleville ja henkilöille, jotka eivät ole palvelussuhteessa, mutta joille maksetaan verolainsäädännön mukaista palkkaa, esimerkiksi kokouspalkkion yhteydessä.\r\n\r\nJos kilometrikorvaus on maksettu vastoin Verohallinnon päätöksen mukaisia aika- ja kilometrirajoja, ne eivät ole miltään osin verovapaita.\r\n\r\nJos kilokorvaukset on maksettu Verohallinnon päätöksen mukaan, mutta suurempina kuin päätöksessä on määrätty verovapaaksi, osa korvauksista on veronalaista ja osa verovapaata. Veronalaiset kilometrikorvaukset ilmoitetaan erikseen omalla tulolajillaan tai palkan tulolajeilla.\r\n\r\nLiikenteenharjoittajan antaman tositteen (esimerkiksi junalipun) perusteella tulonsaajalle maksetut matkustamiskustannusten korvaukset ovat verovapaita, eikä niitä ilmoiteta.\r\n\r\n**Rakennusalalla maksettavat asunnon ja työpaikan väliset matkakustannusten korvaukset**\r\n\r\nRakennusalan työehtosopimuksen mukaan päivittäisistä asunnon ja erityisen työntekemispaikan välisistä matkoista maksetaan matkakustannusten korvausta (”tossurahaa”) erityisen taulukon mukaan eikä Verohallinnon päätöksen mukaisina kilometrikorvauksina. Työehtosopimuksen mukainen korvaus on kuitenkin pienempi kuin Verohallinnon päätöksen mukaiset kilometrikorvaukset, joten korvaus on verovapaata. \r\n\r\nRakennusalan tossurahan yhteydessä ei tarvitse ilmoittaa kilometrimäärää.",
    				label: "Kilometrikorvaus (verovapaa)"
    			},
    			treatmentFeeForMunicipalVeterinarian: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Kunnaneläinlääkärin toimenpidepalkkio"
    			},
    			compensationForUseEarnedIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus ansiotuloa"
    			},
    			compensationForUseCapitalIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.\r\n\r\nJos tekijänoikeus on siirtynyt perintönä tai testamentilla tai se on hankittu vastiketta vastaan, sen käyttämisestä saatu korvaus on henkilölle Suomesta saatuna tulona ennakonpidätyksenalaista pääomatuloa ennakkoperintärekisterimerkinnästä huolimatta.",
    				label: "Käyttökorvaus pääomatuloa"
    			},
    			otherTaxableBenefitForEmployees: {
    				description: "Työnantajan henkilökunnalle kollektiivisesti myöntämä muu kuin rahana annettava veronalainen etu.\r\n\r\nVeronalaisena etuna pidetään esimerkiksi vuosittain laissa säädetyn enimmäismäärän 400 euroa ylittävää omaehtoiseen liikuntaan ja kulttuuriin annettua etua.",
    				label: "Muu henkilöstölle suunnattu veronalainen etu"
    			},
    			otherTaxableIncomeDeemedEarnedIncome: {
    				description: "Tulolajilla ilmoitetaan muiden veronalaisten ansiotuloa olevien suoritusten yhteissumma.\r\n\r\nTulolajilla ilmoitetaan veronalaiset satunnaiset palkkiot ja palkinnot silloin, kun saaja ei ole työsuhteessa antajaan eikä suoritus ole työkorvausta. Tällaisia ovat esimerkiksi työsuhteeseen perustumaton kilpailupalkinto, asiakasyrityksen henkilökuntaan kuuluvalle tai muutoin asiakassuhteen perusteella annettu etu (esimerkiksi matka, lahjakortti tai esine), löytöpalkkio, pankin maksama niin sanottu tarkkaavaisuuspalkkio, taidekilpailun palkinto sekä kunnan asukkailleen maksama matkakorvaus.",
    				label: "Muu veronalainen ansiotulo"
    			},
    			otherFringeBenefit: {
    				description: "Tulolajilla ilmoitetaan työnantajan muuna kuin rahana antamat verotettavat edut, esimeriksi autotalli-, moottoripyörä- ja vene-etu, palkaksi katsotut henki- ja eläkevakuutusmaksut, yksilöimättömästä lahjakortista syntyvä etu ja merimiespalveluksessa olevan henkilön merimiehenä saama luontoisetu.\r\n\r\nPalkkaus.fi-palvelussa asunto-, auto-, puhelin- ja ravintoetu sekä työsuhdematkalipun erät tulee aina ilmoittaa omalla tulolajillaan. Niitä ei lisätä 'Muu luontoisetu' kohtaan.\r\n\r\nJos tulonsaajalta peritään korvaus edusta, luontoisedun arvo ilmoitetaan täysimääräisenä tulolajilla ennen perityn korvauksen vähentämistä.\r\n\r\nJos verotusarvoa ei ole erikseen vahvistettu Verohallinnon antamassa päätöksessä, luontoisedun arvo on sen käypä arvo. Etu on ilmoitettava, vaikka työntekijälle ei maksettaisi lainkaan rahapalkkaa tai rahapalkka ei riittäisi ennakonpidätyksen täysimääräiseen toimittamiseen.",
    				label: "Muu  luontoisetu"
    			},
    			kinshipCarersFee: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Omaishoitajan palkkio"
    			},
    			stockOptions: {
    				description: "Palkkio, joka annetaan työnantajayrityksen osakkeina tai sovitun palkkion sijaan rahasummana, joka vastaa osakkeiden arvoa.",
    				label: "Osakepalkkio"
    			},
    			wagesPaidBySubstitutePayerIncludingSocialInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset sosiaalivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen.\r\n\r\nHUOM: Kotitalous ilmoittaa Yksityisen hoidon tuen tällä tulolajilla!\r\n\r\nVarsinaisen työnantajan antaman tiedon perusteella tuloa ei lueta tulonsaajalle verotuksessa tuloksi, mutta tieto tarvitaan tulonsaajakohtaisesti, koska henkilölle karttuu eläkettä maksetun tulon perusteella.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sosiaalivakuutusmaksut.\r\n\r\nVaikka sijaismaksaja on maksanut palkan tulonsaajille, varsinainen työnantaja on vastuussa työnantajan sosiaalivakuutusmaksujen maksamisesta. Tämän vuoksi varsinaisen työnantajan on ilmoitettava sijaismaksajan maksamat palkat tulonsaajakohtaisesti käyttäen erityisesti sijaismaksajatilanteita varten tarkoitettuja niin sanottuja infotulolajeja.\r\n\r\nSijaismaksaja ilmoittaa tiedot omalta osaltaan tulorekisteriin käyttäen muita tulolajeja sekä ohjaustietoa ”Sijaismaksaja – Kyllä”. Sijaismaksajan tiedon perusteella tulo verotetaan tulonsaajalla. Työntekijältä perittävät maksut peritään ja maksetaan sijaismaksajan maksaman palkan perusteella. Sijaismaksaja ei pääsääntöisesti kuitenkaan maksa työnantajan sosiaalivakuutusmaksuja, vaan niiden maksamisesta on vastuussa varsinainen työnantaja.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat. Ilmoittamista on kuvattu tarkemmin asiakasohjeessa\r\nTietojen ilmoittaminen tulorekisteriin: sijaismaksajan maksamat suoritukset.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sosiaalivakuutusmaksut (työeläke-, sairaus-, työttömyys- sekä työtapaturma- ja ammattitautivakuutus)"
    			},
    			wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset eläkevakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa eläkevakuutusmaksusta. \r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan eläkevakuutusmaksun. Sijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työeläkevakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingUnemploymentInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset työttömyysvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työttömyysvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan työttömyysvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työttömyysvakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset työtapaturma- ja ammattitautivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työtapaturma- ja ammattitautivakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työtapaturma- ja ammattitautivakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työtapaturma- ja ammattitautivakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingHealthInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset sairausvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työnantajan sairausvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sairausvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sairausvakuutusmaksun"
    			},
    			compensationForEmployeeInvention: {
    				description: "Palkkio, joka maksetaan työsuhdekeksinnöstä.",
    				label: "Palkkio työsuhdekeksinnöstä"
    			},
    			reimbursementOfPrivateCaretakersExpenses: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhehoitajan kustannusten korvaus"
    			},
    			privateCaretakersFee: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhehoitajan palkkio"
    			},
    			reimbursementOfFamilyDayCareProvidersExpenses: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhepäivähoitajan kustannusten korvaus"
    			},
    			telephoneBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja maksaa työntekijän puhelinliittymän kulut vapaa-ajalla tapahtuvan yksityiskäytön osalta.\r\n\r\nPalkkaus.fi-palvelussa puhelinetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.",
    				label: "Puhelinetu"
    			},
    			dailyAllowance: {
    				description: "Päiväraha on verovapaa kustannusten korvaus kohtuullisesta ruokailu- ja muiden elantokustannusten lisäyksestä, joka korvauksen saajalle aiheutuu työmatkasta.\r\n\r\nKohtaan merkitään verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Päiväraha-tulolajilla ilmoitettu tulo voi olla:\r\n\r\n- kotimaan kokopäiväraha,\r\n- kotimaan osapäiväraha,\r\n- ulkomaan päiväraha tai \r\n- ulkomaan työskentelyyn liittyvät verovapaat kustannusten korvaukset.\r\n\r\nPalkkaus.fi-palvelussa Ateriakorvaukset ilmoitetaan aina erillisellä Ateriakorvaus-tulolajia.\r\n\r\nJos päivärahan tyypiksi on merkitty ulkomaan työskentelyyn liittyvät kustannusten korvaukset, kohtaan merkitään seuraavat tulonsaajan saamat verovapaat edut, jotka liittyvät ulkomaan työskentelyyn:\r\n\r\n- verovelvollisen ja hänen perheenjäsenensä muutto- ja matkakustannukset työntekovaltioon ja takaisin\r\n- työnantajan ulkomailla kustantama tavanomainen yksityinen palveluhenkilöstö\r\n- työnantajan kustantama lasten perus- ja lukiotason koulutus.\r\n\r\nTulolajilla ei ilmoiteta yömatkarahaa eikä rajanylitysrahaa. Näitä eriä ei ilmoiteta tulorekisteriin.\r\n\r\nKohtaan merkitään vain verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Verohallinto vahvistaa vuosittain verovapaat määrät antamallaan päätöksellä verovapaista matkakustannusten korvauksista.\r\n\r\nJos päivärahat tai muut kohdassa mainitut korvaukset on maksettu Verohallinnon päätöstä lievemmin perustein, korvaukset ilmoitetaan tulolajilla Palkka yhteissumma tai Muu maksettu lisä, jolloin ne ovat myös sosiaalivakuutusmaksujen alaisia.\r\n\r\nJos lievemmin perustein maksetuista korvauksista on sovittu työehtosopimuksella, korvaus ei ole eläke-, työttömyys- eikä työtapaturma- ja ammattitautivakuutusmaksun alainen. Näissä tilanteissa suorituksen\r\nmaksaja ilmoittaa tulon Palkka yhteissumma-tulolajilla tai Muu henkilöstölle suunnattu veronalainen etu, ja samalla ilmoittaa, mikä osuus suorituksesta on eläkevakuutusmaksun, työttömyysvakuutusmaksun sekä työtapaturma- ja ammattitautivakuutusmaksun alainen.\r\n\r\nTulolajin yhteydessä on annettava lisätiedot, mitä eri päivärahoja yhteissumma sisältää.",
    				label: "Päiväraha"
    			},
    			capitalIncomePayment: {
    				description: "Suoritus, joka on saajalleen pääomatuloa. Tällainen on esimerkiksi työntekijälle tai osakkaalle maksettu korko, kuten palkkasaatavan korko tai pääomatuloa oleva takausprovisio.",
    				label: "Pääomatuloa oleva suoritus"
    			},
    			mealBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja tarjoaa työntekijälleen aterian. Työnantaja voi tukea ruokailukustannuksia ruokailulipukkeella tai muulla kohdennetulla maksuvälineellä taikka järjestämällä työpaikkaruokailun.\r\n\r\nRavintoedun verotusarvo määräytyy Verohallinnon päätöksen mukaisena vuosittain vahvistettavana arvona.\r\n\r\nPalkkaus.fi-palvelussa ravintoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.\r\n\r\nJos työnantaja perii työntekijältä ravintoedusta sen verotusarvoa vastaavan korvauksen, tulorekisteriin ilmoitetaan tieto Ravintoedusta peritty korvaus vastaa verotusarvoa: Kyllä.",
    				label: "Ravintoetu"
    			},
    			reimbursementOfCostsPaidToConciliator: {
    				description: "Veronalainen kustannusten korvaus, joka maksetaan vapaaehtoiselle sovittelijalle sovittelusta aiheutuneista kustannuksista.\r\n\r\nSovittelijan kulukorvaus on veronalaista ansiotuloa, joka otetaan huomioon suorituksen saajan verotuksessa saman suuruisena vähennyksenä ilman suorituksen saajan esittämää eri selvitystä.",
    				label: "Sovittelijan kulukorvaus"
    			},
    			nonWageCompensationForWork: {
    				description: "Muussa kuin palvelussuhteessa tehdystä työstä maksettava tai maksettavaksi sovittu vastike, josta tulee toimittaa ennakonpidätys, jos saaja ei ole ennakkoperintärekisterissä, mutta josta ei pääsääntöisesti makseta sosiaalivakuutusmaksuja.\r\n\r\nTyökorvausta voidaan maksaa ennakkoperintärekisteriin kuuluvalle tai kuulumattomalle työn suorittajalle. Työkorvaus on veronalaista ansiotuloa ja siitä toimitetaan ennakonpidätys, jos saaja ei kuulu ennakkoperintärekisteriin. Työkorvausta on esimerkiksi tukiperheelle maksettu korvaus.\r\n\r\nTyökorvauksesta on annettava ilmoitus, jos tulonsaaja ei kuulu ennakkoperintärekisteriin tai, jos maksajana on julkisten alojen eläkelakien piiriin (JuEL) kuuluva suorituksen maksaja ja tulonsaaja on luonnollinen henkilö, joka kuuluu ennakkoperintärekisteriin, mutta hänellä ei ole yrittäjän eläkelakien mukaista vakuutusta. Koska ilmoitus tehdään aina, kun tulonsaaja ei kuulu ennakkoperintärekisteriin, saajana voi olla myös muu kuin luonnollinen henkilö, kuten yhdistys, osakeyhtiö, avoin yhtiö tai kommandiittiyhtiö. Näitä maksuja ei kuitenkaan voi tehdä Palkkaus.fi-palvelun kautta.\r\n\r\nRajoitetusti verovelvolliselle yritykselle maksetut työkorvaukset on ilmoitettava, jos maksaja on perinyt yritykseltä lähdeveron. Rajoitetusti verovelvolliselle luonnolliselle henkilölle maksettu työkorvaus on ilmoitettava aina riippumatta siitä, onko lähdeveroa peritty, esimerkiksi rajoitetusti verovelvollisen urheilijan ja taiteilijan saama palkkio on ilmoitettava.\r\n\r\n**Työkorvauksiin liittyvät kustannusten korvaukset**\r\n\r\nJos palkkio tai muu suorituksen määrä on maksettu työkorvauksena, saajalla ei voi olla verovapaisiin kuluihin merkittäviä kustannusten korvauksia.\r\n\r\nTyökorvauksiin liittyvät kustannusten korvaukset lisätään tulon määrään yhtenä summana muun palkkion kanssa. Työkorvauksen yhteydessä maksetut kustannusten korvaukset eivät ole verovapaita. Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena, eikä tulolajia kilometrikorvaus (veronalainen) saa käyttää.\r\n\r\nTyökorvauksen saaja voi vaatia matkakustannuksia vähennettäväksi esitäytetyllä veroilmoituksellaan. Elinkeinotoiminnan harjoittaja vähentää kuitenkin matkakustannukset kirjanpidossaan.\r\n\r\nTyökorvauksesta ei yleensä voi ennen ennakonpidätyksen toimittamista vähentää työstä aiheutuneita kustannusten korvauksia. Poikkeuksena ovat vain luonnollisille henkilöille työmatkasta maksetut kustannusten korvaukset, joista ei tarvitse toimittaa ennakonpidätystä, jos niiden perusteet ja määrät ovat sen mukaisia kuin Verohallinnon vuosittain antamassa päätöksessä verovapaista matkakustannusten korvauksista on määrätty. Nämäkään korvaukset eivät ole verovapaita, joten ne ilmoitetaan työkorvauksen yhteissummassa.",
    				label: "Työkorvaus"
    			},
    			supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund: {
    				description: "Vain sairauskassat käyttävät tätä tulolajia",
    				label: "Työnantajakohtaisen sairaskassan maksama täydennyspäiväraha"
    			},
    			pensionPaidByEmployer: {
    				description: "Eläke, jonka työnantaja maksaa entiselle työntekijälleen tai tämän edunsaajalle aiemman palvelussuhteen perusteella.",
    				label: "Työnantajan maksama eläke"
    			},
    			dividendsBasedOnWages: {
    				description: "Osakkaalle palkkana maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (palkka)"
    			},
    			dividendsBasedOnNonWage: {
    				description: "Osakkaalle työkorvauksena maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.\r\n\r\nIlmoitus on annettava työkorvauksena verotettavasta työpanokseen perustuvasta osingosta tai ylijäämästä siinäkin tapauksessa, että saaja on merkitty ennakkoperintärekisteriin.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (työkorvaus)"
    			},
    			employerSubsidisedCommuterTicketTaxExempt: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun verovapaa osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua tai vähennettävää määrää ei ole.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 300 euron työsuhdematkalipun. Luontoisetua ei synny.\r\n\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 300 euroa\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.",
    				label: "Työsuhdematkalipun verovapaa osuus"
    			},
    			employerSubsidisedCommuterTicketTaxable: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun palkaksi katsottava osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen\r\nhenkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nTyösuhdematkalipun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.\r\n\r\nPalkaksi katsottavana osuutena ilmoitetaan se osuus edun arvosta, josta työnantaja on jo vähentänyt mahdollisesti tulonsaajalta perimänsä korvauksen.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 800 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Koska työsuhdematkalippu on veronalaista siltä osin kuin edun arvon ylittää 300 euroa ja alittaa 750 euroa, veronalaista luontoisetua syntyy 450 euroa. Työntekijä saa verovapaata etua 350 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan palkaksi katsottu määrä 450 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 350 euroa\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 5 000 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Veronalaista luontoisetua ja palkkaa on 2 050 euroa. Työntekijä saa verovapaata etua 2 950 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan luontoisedun määrä 2 050 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan verovapaan edun määrä 2 950 euroa.\r\n\r\nTyösuhdematkalipun palkaksi katsottua osuus ilmoitetaan ainoastaan omalla tulolajillaan, eikä sitä lisätä Muu luontoisetu-tulolajiin etuihin.",
    				label: "Työsuhdematkalipun palkaksi katsottu osuus"
    			},
    			employeeStockOption: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa.",
    				label: "Työsuhdeoptio"
    			},
    			wagesTransferredToAthletesSpecialFund: {
    				description: "Urheilutulon siirtäminen urheilijarahastoon tulojen jaksottamista varten. Urheilijalla on oikeus siirtää osa urheilutulosta urheilijarahastoon. Rahastoinnista sovitaan urheilijan, työnantajan ja mahdollisesti sponsorin kesken.\r\n\r\nUrheilutulo voi muodostua urheilijan palkasta tai palkkiosta. Tulolajilla ilmoitetaan vain rahastoon siirretty palkka. Siirto urheilurahastoon on tarkoitettu urheiluajan jälkeisen toimeentulon turvaamiseen.",
    				label: "Urheilijarahastoon siirretty palkka"
    			},
    			wagesPaidFromAthletesSpecialFund: {
    				description: "Vain urheilijarahaston käyttöön: Tulolajilla ilmoitetaan vain rahastosta maksettu palkka.",
    				label: "Urheilijarahastosta maksettu palkka"
    			},
    			wagesForInsurancePurposes: {
    				description: "HUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.\r\n\r\nLaskennallinen palkka, joka olisi maksettava, jos ulkomailla tehtyä työtä vastaava työ tehtäisiin Suomessa. Vakuutuspalkkaa ulkomaantyössä käytetään sosiaalivakuuttamisessa maksujen ja etuuksien määräytymisen perusteena todellisen palkan sijaan.\r\n\r\nEri toimijoiden kesken on omat sääntönsä siitä, milloin määräytymisen perusteena käytetään vakuutuspalkkaa ja milloin todellista maksettua palkkaa. Aiheeseen liittyvä tarkempi ohje löytyy Eläketurvakeskuksen soveltamisohjeista: Vakuutuspalkka eläkkeen perusteena olevana työansiona.",
    				label: "Vakuutuspalkka"
    			},
    			taxableReimbursementOfExpenses: {
    				description: "Veronalaisilla kustannusten korvauksilla tarkoitetaan työn suorittajalle työtehtävien hoitamisesta aiheutuneista kustannuksista maksettavaa veronalaista korvausta. Tulolajilla ilmoitetaan kaikkien sellaisten veronalaisten kustannusten korvausten yhteissumma, jota ei eritellä muilla tulolajeilla.\r\n\r\nKohdassa ilmoitetaan vain palkansaajalle maksettuja välittömästi työstä aiheutuneita kustannusten korvauksia, joita työnantaja ei ole lukenut mukaan palkan määrään toimittaessaan ennakonpidätystä (ennakkoperintälain 15 §).\r\n\r\nTyönantaja voi korvata palkansaajalle työstä välittömästi aiheutuneet kustannukset. Tällaisia ovat muun muassa kustannukset, jotka ovat aiheutuneet työvälineiden ja materiaalin hankkimisesta, tarveaineista ja suoja-asujen hankkimisesta aiheutuvat kustannukset. Esimerkiksi rakennusalalla maksetaan työehtosopimukseen perustuvaa työvälinekorvausta.\r\n\r\nTyönantaja ei toimita veronalaisista kustannusten korvauksista ennakonpidätystä eikä maksa sosiaalivakuutusmaksuja. Korvaukset ovat kuitenkin palkansaajan veronalaista tuloa. Palkansaajan tulee vaatia todelliset kustannukset vähennettäviksi esitäytetyllä veroilmoituksellaan.\r\n\r\nJos työntekijä on hankkinut työnantajan puolesta työvälineitä ja työmateriaaleja, niiden kustannuksia ei ilmoiteta, jos työnantaja on korvannut ne työntekijälle tositetta vastaan.",
    				label: "Veronalainen kustannusten korvaus"
    			},
    			privateDayCareAllowanceMunicipalSupplement: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Yksityisen hoidon tuen kuntalisä"
    			},
    			privateDayCareAllowanceWages: {
    				description: "Vain Kela käyttää tätä tulolajia.",
    				label: "Yksityisen hoidon tuki (palkka)"
    			},
    			privateDayCareAllowanceNonWage: {
    				description: "Vain Kela käyttää tätä tulolajia.",
    				label: "Yksityisen hoidon tuki (työkorvaus)"
    			},
    			kilometreAllowancePaidByNonProfitOrganisation: {
    				description: "Kilometrikorvaus, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nYleishyödyllisen yhteisön hyväksi sen toimeksiannosta tehty matka rinnastetaan työmatkaan silloinkin, kun suorituksen saaja ei ole palvelussuhteessa maksajaan eikä muutoin saa palkkaa työstä, johon matka liittyy.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut korvaukset. Jos verovapaat korvaukset liittyvät yhteisön maksamaan palkkaan, ne ilmoitetaan tulolajilla Kilometrikorvaus (verovapaa) (311).\r\n\r\nTulolajilla ilmoitetaan vain tuloverolain (1535/1992) 71 § 3 momentin mukaiset verovapaat kilometrikorvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa: matkustamiskustannusten korvaus, jota voidaan suorittaa myös verovelvollisen asunnolta tehdystä matkasta. Matkustamiskustannusten korvauksesta muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 2 000 euroa kalenterivuodelta.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaa kilometrikorvauksia enemmän kuin tuloverolain 71 § 3 momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
    				label: "Yleishyödyllisen yhteisön maksama kilometrikorvaus"
    			},
    			dailyAllowancePaidByNonProfitOrganisation: {
    				description: "Työmatkapäiväraha, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut päivärahat. Jos verovapaa päiväraha liittyy yhteisön maksamaan palkkaan, se ilmoitetaan tulolajilla Päiväraha (331).\r\n\r\nKohdassa ilmoitetaan vain tuloverolain 71 § 3 momentin mukaiset verovapaat korvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa:\r\n\r\n- päiväraha enintään 20 päivältä kalenterivuodessa\r\n- majoittumiskorvaus.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaan päivärahaa enemmän kuin tuloverolain 7 § momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
    				label: "Yleishyödyllisen yhteisön maksama päiväraha"
    			},
    			unjustEnrichment: {
    				description: "Perusteeton etu -tulolajia käytetään vain, jos aiheettomasti maksettu suoritus havaitaan ennen ensimmäisen ilmoituksen lähettämistä, eikä tuloa ole aiemmin ilmoitettu muuna tulona tulorekisteriin.\r\n\r\nTämä ei tyypillisesti ole mahdollista Palkkaus.fi-maksuprosessissa. Ota yhteys asiakaspalveluun, jos haluat oikeasti käyttää tätä tyyppiä.",
    				label: "Perusteeton etu"
    			},
    			employeeStockOptionLowerMarketPrice: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa ja jossa työntekijältä peritty option merkintähinta option luovutusajankohtana on selvästi alhaisempi kuin osakkeen markkinahinta.\r\n\r\nKäytettäessä optiota etu ilmoitetaan tulolajilla Työsuhdeoptio.",
    				label: "Työsuhdeoptio, jonka merkintähinta on luovutusajankohtana alhaisempi kuin markkinahinta"
    			},
    			royaltyPaidToNonResidentTaxpayer: {
    				description: "Rajoitetusti verovelvolliselle henkilölle tai yhteisölle maksettava korvaus tekijänoikeuden, teollisoikeuden tai muun oikeuden käyttämisestä tai käyttöoikeudesta tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevista tiedoista.\r\n\r\nRojaltin perusteena voi olla\r\n\r\n- kirjallisen, taiteellisen tai tieteellisen teoksen tekijänoikeus\r\n- oikeus valokuvaan\r\n- patentin, tavaramerkin, mallin, muotin, piirustuksen, salaisen kaavan tai valmistustavan käyttäminen tai käyttöoikeus\r\n- teollisia, kaupallisia tai tieteellisiä kokemuksia koskevat tiedot.",
    				label: "Rajoitetusti verovelvolliselle maksettu rojalti"
    			},
    			bicycleBenefitTaxExempt: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun verovapaa osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräetu on verovapaa, jos edun arvo on enintään 1 200 euroa vuodessa. Ylimenevä osuus on veronalaista luontoisetua. Jos työntekijä saa sekä polkupyöräedun että työsuhdematkalipun, niiden verovapaa määrä voi olla yhteensä korkeintaan 3 400 euroa.\r\n\r\nPolkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.",
    				label: "Polkupyöräedun verovapaa osuus"
    			},
    			bicycleBenefitTaxable: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun palkaksi katsottava osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräedusta 1 200 euroa on verovapaata ja loput veronalaista luontoisetua.\r\n\r\nPalkaksi katsottu osuus polkupyöräedusta ilmoitetaan niiltä kuukausilta, kun etu on ollut työntekijän käytettävissä.\r\n\r\nPolkupyöräedun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.",
    				label: "Polkupyöräedun palkaksi katsottu osuus"
    			},
    			conditionalStockOptions: {
    				description: "Osakepalkkio, johon on liittynyt osakkeiden luovutusrajoitus ja ehdollinen palautusvelvollisuus ja josta on syntynyt sosiaalivakuutusmaksujen alaista tuloa. Tässä tilanteessa palkkiosta saatavan edun arvo ei siis alkuperäisestä suunnitelmasta poiketen riipukaan osakkeiden arvon kehityksestä osakepalkkion lupaamisen ja antamisen välisenä, vähintään vuoden pituisena aikana.\r\n\r\nTällainen tilanne voi syntyä olosuhteiden muuttuessa esimerkiksi niin, että työntekijän työsuhde päättyy vuoden määräaikana ja hän saa pitää osan osak-keista. Samalla syntyy velvollisuus maksaa sosiaali-vakuutusmaksut.\r\nTällainen osakepalkkio katsotaan sosiaalivakuutus-maksujen näkökulmasta annetuksi silloin, kun palk-kiota koskevat rasitteet päättyvät ja osakepalkkio jää henkilön omistukseen ja vallintaan. Verotuksen kan-nalta ehdollinenkin osakepalkkio katsotaan kuitenkin annetuksi jo silloin, kun henkilö saa omistusoikeuden osakkeisiin.\r\n\r\nPalkkion määrä ilmoitetaan tulolajilla Ehdollinen osakepalkkio sosiaalivakuutusmaksujen määräämistä varten. Tulolajilla ilmoitettava osakepalkkio ei ole tulonsaajalle konkreettisesti maksettavaa tuloa, vaan tietoa käytetään sosiaalivakuutusmaksujen perusteena. Tulonsaajan saama etu ja siitä tehty ennakonpidätys on ilmoitettu tulolajilla Osakepalkkio (320) jo silloin, kun tulonsaaja on saanut omistusoikeuden osakepalkkioon. Tätä tietoa käytetään esimerkiksi verotuksessa.\r\n\r\nTulolajia Ehdollinen osakepalkkio käytetään vain silloin, kun palkkioon on liittynyt sekä osakkeiden luovutusrajoitus että ehdollinen palautusvelvollisuus. Jos palkkioon on liittynyt vain toinen rasitteista, se ilmoitetaan kuten tavallinen osakepalkkio tulolajilla Osakepalkkio (320).",
    				label: "Ehdollinen osakepalkkio"
    			},
    			compensationCollectedForCarBenefit: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa autoedun verotusarvosta.\r\n\r\nAutoedun omavastuuosuus alentaa verotettavaa etua tai poistaa sen kokonaan. Jos autoedun omavastuuosuus on autoedun verotusarvoa alempi, autoedun arvon ja autoedusta perityn korvauksen erotus on työntekijän veronalaista palkkaa. Jos omavastuuosuus vastaa autoedun verotusarvoa, verotettavaa etua ei synny. Tällöinkin peritty korvaus on ilmoitettava. Tällöin on lisäksi ilmoitettava edun määrä Autoetu-tulolajilla.\r\n\r\nAutoedun arvo ilmoitetaan täysimääräisenä Autoetu-tulolajilla ja Autoedusta peritty korvaustulolajilla työntekijältä perityn omavastuuosuuden määrä. \r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että suorituksen maksaja on perinyt tulonsaajalta luontoisedusta luontoisedun rahaarvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan.\r\n\r\nTyönantaja voi periä autoedun omavastuuosuuden työntekijän nettopalkasta.",
    				label: "Autoedusta peritty korvaus"
    			},
    			withholdingTax: {
    				description: "Ennakkoperintä, jossa suorituksen maksaja vähentää verovelvolliselle maksettavasta rahamäärästä ennakonpidätysprosentin mukaan lasketun veron.\r\n\r\nEnnakonpidätys voidaan ilmoittaa ilmoituksella yhteissummana, vaikka tulolajeja olisi ilmoitettu useita.",
    				label: "Ennakonpidätys"
    			},
    			electedOfficialFee: {
    				description: "Luottamushenkilömaksua voivat ilmoittaa vain kunta ja kuntayhtymä.",
    				label: "Luottamushenkilömaksu"
    			},
    			taxAtSource: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Lähdevero"
    			},
    			taxAtSourceDeduction: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Lähdeverovähennys"
    			},
    			wagesPaid: {
    				description: "Rahapalkka, joka maksetaan ennakonpidätyksen, työntekijän sosiaalivakuutusmaksujen ja muiden vähennettävien erien vähentämisen jälkeen työntekijälle.\r\n\r\nMuihin vähennettäviin eriin voivat kuulua esimerkiksi luontoiseduista työntekijältä perittävät maksut, työnantajan pidättämät työttömyyskassan ja ammattiliiton jäsenmaksut sekä ulosottomaksut.\r\n\r\nTyönantajalle kuuluvia kulueriä, kuten esim. työmatkoilta maksettavia hotellimaksuja, matkalippukuluja tai mahdollisia työterveydenhuollon maksuja ei ilmoiteta Maksettava palkka-tulolajilla.",
    				label: "Maksettava palkka"
    			},
    			reimbursementCollectedForOtherFringeBenefits: {
    				description: "Osuus luontoisedun arvosta, jonka työntekijä maksaa itse.\r\n\r\nKohdassa ilmoitetaan summa, joka työntekijältä on peritty muista luontoiseduista kuin autoedusta tai työsuhdematkalippuedusta.\r\n\r\nLuontoisedun arvo ilmoitetaan täysimääräisenä omalla tulolajillaan ja Muista luontoiseduista peritty korvaus-tulolajilla työntekijältä perityn omavastuuosuuden määrä.\r\n\r\nJos ravintoedusta peritty korvaus vastaa verotusarvoa, perittyä korvausta ei ilmoiteta Muusta luontoisedusta peritty korvaus-erittelylajilla. Tällöin käytetään vain tietoa Ravintoedusta peritty korvaus vastaa verotusarvoa – ”Kyllä (true)”.\r\n\r\nTyönantaja voi periä korvauksen luontoiseduista työntekijän nettopalkasta.",
    				label: "Muista luontoiseduista peritty korvaus"
    			},
    			otherItemDeductibleFromNetWage: {
    				description: "Erät, jotka palkan maksaja vähentää nettopalkasta ennen kuin maksaa suorituksen tulonsaajalle.\r\n\r\nNettopalkasta voidaan vähentää suorituksia lain tai sopimuksen perusteella. Lain perusteella tehtäviä vähennyksiä ovat esimeriksi ulosmittaukset. Työnantaja voi kuitata omia saataviaan työntekijän palkasta. Työntekijän nettopalkasta voidaan vähentää maksuja, joiden perimisestä on työnantajan ja työntekijän kesken sovittu. Nettopalkasta vähennettäviä eriä ovat esim. työttömyyskassan ja ammattiyhdistysten jäsenmaksut sekä pysäköintihallimaksut.\r\n\r\nJos ulosmittaus on jo ilmoitettu erikseen, ei sitä ilmoiteta enää tällä erittelylajilla yhteissummassa.",
    				label: "Nettopalkasta vähennettävä muu erä"
    			},
    			netWage: {
    				description: "Palkka, josta on toimitettu ennakonpidätys ja maksettu työntekijän sosiaalivakuutusmaksut. \r\n\r\nNettopalkka on eri asia kuin työntekijälle maksettava palkka.",
    				label: "Nettopalkka"
    			},
    			employerPaidPremiumForCollectiveAdditionalPensionInsurance: {
    				description: "Työnantajan osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta laisinkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksu-kohdassa.\r\n\r\nMaksut ilmoitetaan tässä kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
    				label: "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen maksu"
    			},
    			employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution: {
    				description: "Työntekijän osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta lainkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksut-kohdassa.\r\n\r\nMaksut ilmoitetaan kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
    				label: "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen työntekijältä peritty osuus"
    			},
    			employeesHealthInsuranceContribution: {
    				description: "Yleisesti verovelvollisella tulonsaajalla sairausvakuutusmaksu sisältyy aina ennakonpidätykseen, eikä yleisesti verovelvollisten osalta tule ilmoittaa erikseen työntekijän sairausvakuutusmaksua.\r\n\r\nRajoitetusti verovelvollisen sairausvakuutusmaksu on perittävä erikseen lähdeveron lisäksi, ellei rajoitetusti verovelvollinen ole hakeutunut verotusmenettelystä annetun lain mukaiseen verotukseen, jolloin sairausvakuutusmaksu sisältyy ennakonpidätykseen tai ennakkoveroon. \r\n\r\nHUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Työntekijän sairausvakuutusmaksu"
    			},
    			employeesPensionInsuranceContribution: {
    				description: "Bruttopalkan perusteella määräytyvä työeläkevakuutusmaksu, jolla työntekijä osallistuu työeläkevakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työeläkevakuutusmaksu vähennetään työntekijän nettopalkasta.",
    				label: "Työntekijän työeläkevakuutusmaksu"
    			},
    			employeesUnemploymentInsuranceContribution: {
    				description: "Bruttopalkan perusteella määräytyvä työttömyysvakuutusmaksu, jolla työntekijä osallistuu työttömyysvakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työttömyysvakuutusmaksu vähennetään työntekijän nettopalkasta.",
    				label: "Työntekijän työttömyysvakuutusmaksu"
    			},
    			reimbursementForEmployerSubsidisedCommuterTicket: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa työsuhdematkalipun euromääräisestä arvosta.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nTyönantaja voi periä työsuhdematkalipun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 200 euroa. Veronalaista luontoisetua ja palkkaa on 250 euroa. Työntekijä saa verovapaata etua 550 euroa.\r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus -tulolajilla ilmoitetaan palkaksi katsottu määrä 250 euroa.\r\n- Työsuhdematkalipusta peritty korvaus-erittelylajilla ilmoitetaan työntekijältä peritty määrä 200 euroa.\r\n- Työsuhdematkalipun verovapaa osuus- tulolajilla ilmoitetaan verovapaan edun määrä 550 euroa.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 500 euroa. Veronalaisen luontoisedun määrä on 450 euroa. Koska peritty korvaus on suurempi kuin edun verotusarvo, verotettavaa luontoisetua ei synny. Työntekijä saa verovapaata etua 500 euroa.\r\n\r\n- Työsuhdematkalipusta peritty korvaus -erittelylajilla ilmoitetaan työntekijältä peritty määrä 500 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 500 euroa.",
    				label: "Työsuhdematkalipusta peritty korvaus"
    			},
    			taxPaidAbroad: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Ulkomaille maksettu vero"
    			},
    			distraint: {
    				description: "Perittäväksi tulleen velan maksamiseksi tehtävä pidätys. Ulosmitattava määrä lasketaan nettotulosta.\r\n\r\nUlosmitattaessa velallisen ja hänen perheensä toimeentuloa varten tulee kuitenkin jättää suojaosuus.",
    				label: "Ulosmittaus"
    			},
    			voluntaryIndividualPensionInsurancePremium: {
    				description: "Lakisääteistä eläketurvaa täydentävän tietylle henkilölle kohdistetun eläkevakuutuksen maksu. Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu voi olla jatkuva tai kertamaksullinen.\r\n\r\nKohtaan merkitään työntekijälle otetun vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Samoin ilmoitetaan myös sellaisen yksilöllisen eläkevakuutuksen maksut, joka on otettu avoimen yhtiön yhtiömiehelle, kommandiittiyhtiön vastuunalaiselle yhtiömiehelle tai sellaisen osakeyhtiön osakkaalle, joka ei ole työeläkevakuutettu.\r\n\r\nJos työnantajan ottaman vapaaehtoisen yksilöllisen eläkevakuutuksen maksut ylittävät 8 500 euroa, ne ovat veronalaista tuloa, joista maksetaan myös työnantajan sosiaalivakuutusmaksut. Jos maksut ylittävät 8 500 euroa vuodessa, suorituksen maksaja ilmoittaa ylimenevät maksut muuna luontoisetuina. \r\n\r\nEsimerkki: Työnantaja maksaa työntekijälle ottamansa vapaaehtoisen yksilöllisen eläkevakuutuksen maksuja vuodessa 12000 euroa. Työnantaja ilmoittaa tiedot seuraavasti:\r\n\r\n- Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu 8 500 euroa.\r\n- Muu verotettava luontoisetu 3 500 euroa.\r\n\r\nKohtaan merkitään vain vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Jos työnantaja on maksanut työntekijän puolesta työntekijän pakollisia eläkevakuutusmaksuja, työttömyysvakuutusmaksuja ja työntekijän itsensä ottaman vapaaehtoisen eläkevakuutuksen maksuja, nämä ilmoitetaan Muu luontoisetu-tulolajilla.",
    				label: "Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu"
    			},
    			deductionBeforeWithholding: {
    				description: "Työntekijän itsensä maksama työn tekemisestä aiheutunut välitön kustannus, jonka työnantaja vähentää bruttopalkasta ennen ennakonpidätyksen osuuden määrittämistä. Tällaisia kustannuksia ovat muun muassa palkansaajan maksamat kustannukset moottorisahan käytöstä ja muut työvälineistä aiheutuneet kustannukset. Samoin tällaisia kustannuksia ovat myös menot työmatkoista, jos työnantaja ei maksa niistä verovapaita korvauksia palkan lisäksi.\r\n\r\nKohtaan merkitään vähennys, joka on tehty ennen ennakonpidätyksen toimittamista. Vähennysten osuuden tulee sisältyä myös tulolajilla ilmoitettuun palkkaan.\r\n\r\n**Esimerkki**: Työntekijä on maksanut itse moottorisahan käytöstä aiheutuneet 4 500 euron kustannukset. Työntekijälle maksettu koko vuoden palkka on 20 000 euroa, josta sahatyön osuus on 14 000 euroa. Työntekijä on vaatinut ennakkoperinnässä vähennystä käyttökustannuksista 40 %:n mukaisesti sahatyöstä maksetun palkan määrästä eli 5 600 euroa. Ennakonpidätys on toimitettu ennakkoperintälain (1118/1996) 15 § mukaisesti 14 400 eurosta (20 000 - 5 600).\r\n\r\nPalkan tulolajilla ilmoitetaan 20 000 euroa ja Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 5 600 euroa. Työntekijän tulee vaatia esitäytetyllä veroilmoituksellaan moottorisahavähennystä ja selvittää sahatyöstä maksetun palkan määrä. Lopullisessa verotuksessa vähennys lasketaan sahatyöstä maksetun palkan (14 000 euroa) osuudesta.\r\n\r\nJos työnantaja on maksanut moottorisahan käytöstä aiheutuneet kustannukset, katso ilmoittaminen kohdasta: Veronalaiset kustannusten korvaukset. \r\n\r\n**Esimerkki**: Pelimannille luvataan soittokeikasta 200 euroa. Matkustaminen ja matkustamisen kustannukset eivät vaikuta maksettavaan bruttomäärään. Matkakustannuksia ei siis korvata.\r\n\r\nTyönantaja voi laskea määrän, josta toimittaa ennakonpidätyksen siten, että vähentää työntekijän antaman selvityksen perusteella saman määrän, jonka voisi korvata verovapaasti. Esimerkiksi matkustamiskustannuksia 51 km x 41 snt/km eli 20,91 snt ja osapäivärahan 19 euroa vastaavat määrät, jolloin pidätyksenalaiseksi määräksi jää 160,09 euroa. Samasta määrästä maksetaan myös työnantajan sairausvakuutusmaksu. \r\n\r\nPalkan tulolajilla ilmoitetaan 200 euroa, Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 39,91 euroa. Verovapaiden matkakustannusten korvausten tulolajeilla ei tehdä merkintöjä. \r\n\r\nTyöntekijän on vaadittava vähennyksiä veroilmoituksessaan ja vähennys on yleensä alempi kuin ennakonpidätyksen yhteydessä.",
    				label: "Vähennys ennen ennakonpidätystä"
    			},
    			reimbursementCollectedForBicycleBenefit: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa polkupyöräedun euromääräisestä arvosta.\r\n\r\nOsa polkupyöräedun arvosta on verovapaata ja osa veronalaista. Polkupyöräedusta 1 200 euroa on verovapaata, ja ylittävä osa on veronalaista luontoisetua. Polkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole. Edusta peritty korvaus alentaa verotettavan edun määrää.\r\n\r\nTyönantaja voi periä polkupyöräedun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nPolkupyöräedun ilmoittaminen poikkeaa muista luontoiseduista samalla tavalla kuin työsuhdematkalipun ilmoittaminen, koska työnantajan on vähennettävä polkupyöräedun palkaksi katsottavasta osuudesta työntekijältä perimänsä korvaus. Palkaksi katsottavana osuutena ilmoitetaan jäljelle jäävä määrä.",
    				label: "Polkupyöräedusta peritty korvaus"
    			}
    		},
    		Unit: {
    			count: {
    				description: "",
    				label: "Count"
    			},
    			days: {
    				description: "",
    				label: "Days"
    			},
    			euro: {
    				description: "",
    				label: "Euro"
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			},
    			kilometers: {
    				description: "",
    				label: "Kilometers"
    			},
    			one: {
    				description: "",
    				label: "One"
    			},
    			percent: {
    				description: "",
    				label: "Percent"
    			},
    			period: {
    				description: "",
    				label: "Period"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			weeks: {
    				description: "",
    				label: "Weeks"
    			}
    		},
    		VarmaPensionOrderAction: {
    			move: {
    				description: "",
    				label: "Move"
    			},
    			newTyel: {
    				description: "",
    				label: "NewTyel"
    			},
    			newYel: {
    				description: "",
    				label: "NewYel"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		VarmaPensionOrderYelPayer: {
    			company: {
    				description: "",
    				label: "Company"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		WorkflowEventFeatures: {
    			isActive: {
    				description: "",
    				label: "IsActive"
    			},
    			isEditable: {
    				description: "",
    				label: "IsEditable"
    			},
    			isOnHold: {
    				description: "",
    				label: "IsOnHold"
    			},
    			isRemovable: {
    				description: "",
    				label: "IsRemovable"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		WorkflowEventUi: {
    			danger: {
    				description: "",
    				label: "Danger"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			info: {
    				description: "",
    				label: "Info"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			success: {
    				description: "",
    				label: "Success"
    			},
    			warning: {
    				description: "",
    				label: "Warning"
    			}
    		},
    		YearEndAdminCheck: {
    			errors: {
    				description: "",
    				label: "Errors"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithWarning: {
    				description: "",
    				label: "OkWithWarning"
    			}
    		},
    		YearEndFeedbackCalculationType: {
    			correction: {
    				description: "",
    				label: "Correction"
    			},
    			external: {
    				description: "",
    				label: "External"
    			},
    			salaxyPayment: {
    				description: "",
    				label: "SalaxyPayment"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		YearEndUserFeedback: {
    			noYearlyReports: {
    				description: "",
    				label: "NoYearlyReports"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithModifications: {
    				description: "",
    				label: "OkWithModifications"
    			},
    			willHandleMyself: {
    				description: "",
    				label: "WillHandleMyself"
    			}
    		}
    	}
    };

    var SALAXY$l = {
    	MODEL: {
    		DeliveryData: {
    			title: "Aineiston tiedot",
    			deliveryId: {
    				title: "Aineistoviite"
    			},
    			deliveryDataType: {
    				title: "Tyyppi"
    			},
    			timestamp: {
    				title: "Muodostamishetki"
    			},
    			contactPersons: {
    				title: "Yhteyshenkilöt"
    			},
    			reportDate: {
    				title: "Ilmoituspäivä"
    			}
    		},
    		EarningsPayment: {
    			id: {
    				title: "Tunniste"
    			},
    			createdAt: {
    				title: "Muodostettu"
    			},
    			updatedAt: {
    				title: "Päivitetty"
    			},
    			owner: {
    				title: "Omistaja"
    			},
    			partner: {
    				title: "Partneri"
    			},
    			status: {
    				title: "Lähetyksen tila"
    			},
    			irStatus: {
    				title: "Tulorekisteritila"
    			},
    			reportLog: {
    				title: "Tulorekisteritapahtumat"
    			},
    			scheduledFor: {
    				title: "Ajastus"
    			},
    			isModified: {
    				title: "Muokattu tulorekisterilähetyksen jälkeen"
    			}
    		},
    		ContactPerson: {
    			title: "Yhteyshenkilö",
    			email: {
    				title: "Sähköposti"
    			},
    			name: {
    				title: "Nimi"
    			},
    			telephone: {
    				title: "Puhelinnumero"
    			},
    			responsibilityCode: {
    				title: "Vastuualue"
    			}
    		},
    		Id: {
    			code: {
    				title: "Tunniste"
    			},
    			type: {
    				title: "Tyyppi"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		PaymentPeriod: {
    			title: "Palkanmaksukausi ja maksu- tai ilmoituspäivä",
    			paymentDate: {
    				title: "Suorituksen maksupäivä tai muu ilmoituspäivä"
    			},
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		Payer: {
    			title: "Maksaja",
    			subOrgs: {
    				title: "Maksajan aliorganisaatiot"
    			},
    			payerIds: {
    				title: "Maksajan tunnisteet"
    			},
    			pensionInsurances: {
    				title: "Työeläkevakuutukset"
    			},
    			accidentInsurances: {
    				title: "Työtapaturmavakuutukset"
    			}
    		},
    		PayerBasic: {
    			companyName: {
    				title: "Yrityksen nimi"
    			},
    			missingId: {
    				title: "Maksajalla ei ole asiakastunnistetta"
    			}
    		},
    		PayerOther: {
    			payerTypes: {
    				title: "Maksajan tyypit"
    			}
    		},
    		Address: {
    			co: {
    				title: "c/o-tarkenne"
    			},
    			street: {
    				title: "Katuosoite"
    			},
    			poBox: {
    				title: "PL"
    			},
    			postalCode: {
    				title: "Postinumero"
    			},
    			postOffice: {
    				title: "Postitoimipaikka"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		TypedAddress: {
    			title: "Tulonsaajan osoite",
    			co: {
    				title: "c/o-tarkenne"
    			},
    			street: {
    				title: "Katuosoite"
    			},
    			poBox: {
    				title: "PL"
    			},
    			postalCode: {
    				title: "Postinumero"
    			},
    			postOffice: {
    				title: "Postitoimipaikka"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			},
    			addressType: {
    				title: "Osoitteen tyyppi"
    			}
    		},
    		SubstitutePayer: {
    			title: "Sijaismaksaja",
    			acts: {
    				title: "Maksaja toimii sijaismaksajana"
    			},
    			employerName: {
    				title: "Varsinaisen työnantajan nimi"
    			},
    			wageSec: {
    				title: "Maksettu palkkaturvana"
    			}
    		},
    		IncomeEarner: {
    			title: "Tulonsaaja",
    			incomeEarnerIds: {
    				title: "Tulonsaajan tunnisteet"
    			},
    			addresses: {
    				title: "Tulonsaajan osoitteet "
    			},
    			professions: {
    				title: "Ammattiluokat"
    			}
    		},
    		IncomeEarnerOther: {
    			incomeEarnerTypes: {
    				title: "Tulonsaajan lisätiedot"
    			},
    			payments: {
    				title: "Maksujen tiedot"
    			}
    		},
    		Payment: {
    			paymentType: {
    				title: "Maksutiedon tyyppi"
    			},
    			paymentRef: {
    				title: "Maksun viitenumero"
    			},
    			paymentSpecifier: {
    				title: "Maksun tarkenne"
    			}
    		},
    		IncomeEarnerBasic: {
    			missingId: {
    				title: "Tulonsaajalla ei ole asiakastunnistetta"
    			},
    			companyName: {
    				title: "Yrityksen nimi"
    			},
    			firstName: {
    				title: "Etunimi"
    			},
    			lastName: {
    				title: "Sukunimi"
    			},
    			birthDate: {
    				title: "Syntymäaika"
    			},
    			gender: {
    				title: "Sukupuoli"
    			}
    		},
    		Profession: {
    			code: {
    				title: "Ammattiluokan tai nimikkeen tunniste"
    			},
    			type: {
    				title: "Ammattiluokan tai nimikkeistön tyyppi"
    			},
    			title: {
    				index: "Ammattiluokka tai -nimike",
    				title: "Ammattinimike (vapaamuotoinen)"
    			}
    		},
    		Employment: {
    			title: "Palvelussuhdetiedot",
    			employed: {
    				title: "Palvelussuhteessa"
    			},
    			employmentCode: {
    				title: "Palvelussuhteen tyyppi"
    			},
    			termCode: {
    				title: "Palvelussuhteen kesto"
    			},
    			partTime: {
    				title: "Osa-aikaisuuden prosentti"
    			},
    			hoursPerWeek: {
    				title: "Säännöllinen sovittu viikkotyöaika"
    			},
    			paymentTypes: {
    				title: "Palkkauksen muodot"
    			},
    			employmentPeriods: {
    				title: "Palvelussuhteen voimassaolo"
    			},
    			employmentEndings: {
    				title: "Palvelussuhteen päättyminen"
    			}
    		},
    		Period: {
    			title: "Palvelussuhteen voimassaoloaika",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		EmploymentEnding: {
    			title: "Palvelussuhteen päättyminen",
    			code: {
    				title: "Palvelussuhteen päättymisen syykoodi"
    			},
    			type: {
    				title: "Palvelussuhteen päättymisen syykoodisto"
    			}
    		},
    		PensionInsurance: {
    			title: "Työeläkevakuutus",
    			pensionActCode: {
    				title: "Tieto työeläkevakuutuksesta"
    			},
    			pensionProvIdCode: {
    				title: "Työeläkelaitoksen yhtiötunnus"
    			},
    			pensionPolicyNo: {
    				title: "Eläkejärjestelynumero"
    			}
    		},
    		AccidentInsurance: {
    			title: "Työtapaturmavakuutus",
    			accInsPolicyNo: {
    				title: "Työtapaturmavakuutuksen vakuutusnumero"
    			}
    		},
    		Absence: {
    			absenceRepStartDate: {
    				title: "Poissaolojen ilmoitusjakson alkupäivä"
    			},
    			absenceRepEndDate: {
    				title: "Poissaolojen ilmoitusjakson loppupäivä"
    			}
    		},
    		UnpaidAbsence: {
    			unpaidAbsencePeriods: {
    				title: "Palkaton poissaolo"
    			}
    		},
    		UnpaidAbsencePeriod: {
    			title: "Poissaolojakso",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			},
    			absenceDays: {
    				title: "Poissaolopäivien lukumäärä"
    			},
    			causeCode: {
    				title: "Poissaolon syy"
    			}
    		},
    		PaidAbsence: {
    			paidAbsencePeriods: {
    				title: "Palkallinen poissaolo"
    			}
    		},
    		PaidAbsencePeriod: {
    			title: "Poissaolojakso",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			},
    			absenceDays: {
    				title: "Poissaolopäivien lukumäärä"
    			},
    			absenceUntil: {
    				title: "Poissaolo jatkuu, mihin asti"
    			},
    			causeCode: {
    				title: "Poissaolon syy"
    			},
    			amount: {
    				title: "Palkan määrä palkalliselta poissaoloajalta"
    			},
    			paidAbsencePeriodInfo: {
    				title: "Korvaushakemuksen lisätiedot"
    			},
    			reimbApp: {
    				title: "Korvaushakemustiedot"
    			}
    		},
    		ReimbPayment: {
    			title: "Korvaushakemustiedot",
    			paymentType: {
    				title: "Maksutiedon tyyppi"
    			},
    			paymentRef: {
    				title: "Maksun viitenumero"
    			},
    			paymentSpecifier: {
    				title: "Maksun tarkenne"
    			}
    		},
    		Report: {
    			transactions: {
    				title: "Tulolajit"
    			}
    		},
    		Transaction: {
    			title: "Tulolaji",
    			dailyAllowance: {
    				title: "Päiväraha"
    			},
    			earningPeriods: {
    				title: "Ansaintakaudet"
    			},
    			insuranceData: {
    				title: "Vakuuttamistiedot"
    			},
    			otherBenefit: {
    				title: "Muu luontoisetu"
    			},
    			unitWages: {
    				title: "Palkan yksikkö"
    			},
    			carBenefit: {
    				title: "Autoetu"
    			},
    			kmAllowance: {
    				title: "Kilometrikorvaus (verovapaa) "
    			},
    			mealBenefit: {
    				title: "Ravintoetu"
    			},
    			recoveryData: {
    				title: "Takaisinperinnän lisätiedot"
    			},
    			sailorIncome: {
    				title: "Merityötulo"
    			},
    			sixMonthRule: {
    				title: "6 kk sääntö"
    			},
    			transactionBasic: {
    				title: "Tulolajin yleistiedot"
    			}
    		},
    		SubOrg: {
    			title: "Aliorganisaatio",
    			type: {
    				title: "Aliorganisaation tunnisteen tyyppi"
    			},
    			code: {
    				title: "Aliorganisaation tunniste"
    			}
    		},
    		EarningPeriod: {
    			title: "Ansaintakausi",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		TransactionInclusion: {
    			title: "Vakuuttamistieto",
    			insuranceCode: {
    				title: "Vakuuttamistiedon tyyppi"
    			},
    			included: {
    				title: "Vakuutusmaksun perusteena"
    			}
    		},
    		UnitWage: {
    			title: "Yksikköpalkka",
    			unitPrice: {
    				title: "Yksikköhinta"
    			},
    			unitAmount: {
    				title: "Yksiköiden määrä "
    			},
    			unitCode: {
    				title: "Yksikkö"
    			}
    		},
    		CarBenefit: {
    			title: "",
    			carBenefitCode: {
    				title: "Autoedun tyyppi"
    			},
    			ageGroupCode: {
    				title: "Auton ikäryhmä"
    			},
    			kilometers: {
    				title: "Auton kilometrimäärä"
    			}
    		},
    		KmAllowance: {
    			title: "Kilometrikorvaus (verovapaa)",
    			kilometers: {
    				title: "Kilometrien määrä"
    			}
    		},
    		MealBenefit: {
    			title: "Ravintoetu",
    			taxValue: {
    				title: "Ravintoedusta peritty korvaus vastaa verotusarvoa"
    			}
    		},
    		RecoveryData: {
    			title: "Takaisinperinnän lisätiedot",
    			recoveryDate: {
    				title: "Takaisinperintäpäivä"
    			},
    			taxAtSource: {
    				title: "Takaisinperittyyn määrään kohdistuva lähdevero"
    			},
    			withhold: {
    				title: "Takaisinperittyyn määrään kohdistuva ennakonpidätys"
    			},
    			origPaymentPeriods: {
    				title: "Alkuperäiset palkanmaksukaudet"
    			}
    		},
    		OrigPaymentPeriod: {
    			title: "Alkuperäinen palkanmaksukausi",
    			paymentDate: {
    				title: "Maksupäivä"
    			},
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		SailorIncome: {
    			title: "Merityötulo",
    			crossTradeTime: {
    				title: "Cross-trade-aika"
    			},
    			sailorIncomeProp: {
    				title: "Suoritus maksettu merityötulona "
    			},
    			withdrawalPeriod: {
    				title: "Nostettavissaolokausi"
    			}
    		},
    		WithdrawalPeriod: {
    			title: "Nostettavissaolokausi",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		SixMonthRule: {
    			title: "6 KK sääntö",
    			applicable: {
    				title: "Kuuden kuukauden sääntö soveltuu "
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		TransactionBasic: {
    			title: "Tulolajin yleistiedot",
    			transactionCode: {
    				title: "Tulolajin koodi"
    			},
    			amount: {
    				title: "Määrä"
    			},
    			noMoney: {
    				title: "Maksettu muuna kuin rahana"
    			},
    			oneOff: {
    				title: "Kertakorvaus"
    			},
    			recovery: {
    				title: "Takaisinperintä"
    			},
    			unjustEnrichment: {
    				title: "Perusteeton etu"
    			}
    		},
    		ApiValidation: {
    			title: "Validointi",
    			isValid: {
    				title: "Validi"
    			},
    			hasAllRequiredFields: {
    				title: "Kaikki pakolliset kentät"
    			},
    			errors: {
    				title: "Virheet"
    			}
    		},
    		ApiValidationError: {
    			title: "Validointivirhe",
    			code: {
    				title: "Koodi"
    			},
    			key: {
    				title: "Avain"
    			},
    			msg: {
    				title: "Viesti"
    			},
    			type: {
    				title: "Tyyppi"
    			}
    		},
    		PayerSummary: {
    			title: "Erillisilmoitus",
    			id: {
    				title: "Tunniste"
    			},
    			createdAt: {
    				title: "Muodostettu"
    			},
    			updatedAt: {
    				title: "Päivitetty"
    			},
    			owner: {
    				title: "Omistaja"
    			},
    			partner: {
    				title: "Partneri"
    			},
    			status: {
    				title: "Lähetyksen tila"
    			},
    			irStatus: {
    				title: "Tulorekisteritila"
    			},
    			reportLog: {
    				title: "Tulorekisteritapahtumat"
    			},
    			scheduledFor: {
    				title: "Ajastus"
    			},
    			isModified: {
    				title: "Muokattu tulorekisterilähetyksen jälkeen"
    			}
    		},
    		ReportLogEntry: {
    			title: "Tulorekisteritapahtuma",
    			irTimestamp: {
    				title: "Tulorekisterin aikaleima"
    			},
    			eventType: {
    				title: "Tapahtumatyyppi"
    			},
    			reportId: {
    				title: "Raporttitunnus"
    			},
    			irReportId: {
    				title: "Tulorekisterin raporttitunnus"
    			},
    			irDeliveryId: {
    				title: "Toimitustunnus"
    			},
    			irResponseId: {
    				title: "Vastaustunnus"
    			},
    			irVersion: {
    				title: "Tulorekisteriversio"
    			},
    			user: {
    				title: "Käyttäjä"
    			},
    			versionId: {
    				title: "Versiotunnus"
    			},
    			message: {
    				title: "Viesti"
    			}
    		},
    		ApiScheduleObject: {
    			title: "Jono-objekti",
    			id: {
    				title: "Raporttitunnus"
    			},
    			startAt: {
    				title: "Ajastus"
    			},
    			action: {
    				title: "Operaatio"
    			}
    		},
    		PaymentMonth: {
    			title: "Kohdekausi",
    			year: {
    				title: "Vuosi"
    			},
    			month: {
    				title: "Kuukausi"
    			}
    		},
    		ReportData: {
    			title: "Palvelun tiedot",
    			actionCode: {
    				title: "Toimenpiteen tyyppi"
    			},
    			irReportId: {
    				title: "Tulorekisterin ilmoitusviite"
    			},
    			reportId: {
    				title: "Maksajan ilmoitusviite"
    			},
    			reportVersion: {
    				title: "Ilmoituksen versionumero"
    			}
    		}
    	}
    };

    var SALAXY$k = {
    	MODEL: {
    		AbsencePeriod: {
    			title: "absence period",
    			descr: "Period of absence",
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			}
    		},
    		AccountBase: {
    			title: "account base",
    			descr: "Account - either Person or Company - is a juridical entity Paying or receiving salaries, making contracts etc. It may be related to zero, one or multiple UserAccounts/Credentials.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AccountInIndex: {
    			title: "account in index",
    			descr: "Object that represents a Salaxy account.",
    			avatar: {
    				title: "Avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "Sähköpostiosoite",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "Tilinumero (IBAN)",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "Y-tunnus / Hetu",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "Puhelinnumero",
    				descr: "Telephone number."
    			}
    		},
    		AccountProducts: {
    			title: "account products",
    			descr: "Provides an overview and the possibility to edit products that are enabled to this account",
    			accounting: {
    				title: "accounting",
    				descr: "Bookkeeping reports for salarypayments"
    			},
    			baseService: {
    				title: "base service",
    				descr: "The basic services for Salary payment."
    			},
    			healthCareHeltti: {
    				title: "health care heltti",
    				descr: "Employer health care from the company Heltti"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Reports and payments for mandatory employment insurances."
    			},
    			pension: {
    				title: "pension",
    				descr: "Reports ant payments to Pension companies."
    			},
    			pensionEntrepreneur: {
    				title: "pension entrepreneur",
    				descr: "Pension payments for an entrepreneur"
    			},
    			pricing: {
    				title: "pricing",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			tax: {
    				title: "tax",
    				descr: "Reports and payments to tax authorities."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "Reports and payments to TVR (the unemployment insurance fund of Finland)"
    			}
    		},
    		AccountValidation: {
    			title: "account validation",
    			descr: "Validation errors for the onboarding process",
    			canStartSign: {
    				title: "can start sign",
    				descr: "TODO: NOt implemented yet - should be implemented\nIf true, the onboarding object has sufficient data to start the Digital Signature process (Visma Sign).\nDepending on the use case this may or may not be required for committing the data as account data."
    			},
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			existingAccount: {
    				title: "existing account",
    				descr: "Existing account based on official id (Personal ID / Business ID)."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, none of the accounts in ExistingAccounts cannot be accessed by the current user."
    			},
    			isCompleted: {
    				title: "is completed",
    				descr: "If true, the validation process has been completed. The IsValid may be true or false."
    			},
    			isOfficialIdValid: {
    				title: "is official id valid",
    				descr: "If true the official ID is technically valid.\nOfficial ID is either Finnish Personal ID (HETU, Henkilötunnus) or Business ID (Y-tunnus)."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		Accountant: {
    			title: "accountant",
    			descr: "The person or account reponsible for accounting",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			type: {
    				title: "type",
    				descr: "If true, the accountant is a registered Palkkaus.fi account who typically has access to this account."
    			}
    		},
    		AccountingData: {
    			title: "accounting data",
    			descr: "Accounting data for reports.",
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			ledgerAccounts: {
    				title: "ledger accounts",
    				descr: "All ledgers."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			}
    		},
    		AccountingDataSummary: {
    			title: "accounting data summary",
    			descr: "Summary of the accounting data. Contains main figures: totals and workers and employer total figures.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary of the calculations included in the report."
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment of the calculations included in the report."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "The total number of distinct workers included in the report."
    			}
    		},
    		AccountingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			accountantEmail: {
    				title: "accountant email"
    			},
    			accountantName: {
    				title: "accountant name"
    			},
    			accountantPhone: {
    				title: "accountant phone"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by AccountantName, AccountantEmail and AccountantPhone."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		AccountingReportRow: {
    			title: "accounting report row",
    			descr: "Accounting report table row.",
    			accountNumber: {
    				title: "account number",
    				descr: "Proposed account number for the row."
    			},
    			accountText: {
    				title: "account text",
    				descr: "Proposed account text for the row."
    			},
    			additionalInformation: {
    				title: "additional information",
    				descr: "Additional information for the row."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount for the row: May be positive or negative(depending on the depet/credit and account type)."
    			},
    			count: {
    				title: "count",
    				descr: "Count of units. Default is 1."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the unit."
    			},
    			productCode: {
    				title: "product code",
    				descr: "Returns product code."
    			},
    			productName: {
    				title: "product name",
    				descr: "Returns product name. (Code or ResultCode in this order)."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Row type. Default is the Booking."
    			},
    			styles: {
    				title: "styles",
    				descr: "Space separated set of class names for CSS styling purposes."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the booking entry / header."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit identifier;"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percentage of the value added tax that is included in the amount."
    			}
    		},
    		AccountingReportTable: {
    			title: "accounting report table",
    			descr: "Contains accounting rows for one or many calculations.",
    			businessId: {
    				title: "business id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			rows: {
    				title: "rows",
    				descr: "All rows in the report table."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			},
    			tableType: {
    				title: "table type",
    				descr: "Accounting report table type."
    			}
    		},
    		AccountingReportTableSummary: {
    			title: "accounting report table summary",
    			descr: "Summary of the accounting report. Contains main figures: totals and workers and employer total figures.",
    			count: {
    				title: "count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		AccountingRuleSet: {
    			title: "accounting rule set",
    			descr: "Ruleset containing required tabular logic and accounts for creating accounting entries.",
    			accounts: {
    				title: "accounts",
    				descr: "Accounts which are used in the tabular logic for creating accounting entries."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the ruleset."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the rule set is read only."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the ruleset."
    			},
    			rows: {
    				title: "rows",
    				descr: "Tabular logic for creating accounting entries."
    			},
    			templateId: {
    				title: "template id",
    				descr: "Template to which this is based on."
    			}
    		},
    		AccountingRuleSetAccount: {
    			title: "accounting rule set account",
    			descr: "Single account with number, grouping and default text.",
    			account: {
    				title: "account",
    				descr: "Account number."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the account."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the account is non-removable template."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the account."
    			}
    		},
    		AccountingRuleSetRow: {
    			title: "accounting rule set row",
    			descr: "Contains a single row of the tabular logic for creating accounting entries.",
    			creditGroupingId: {
    				title: "credit grouping id",
    				descr: "Additional flags for directing the logic for creating accounting entry based on given entries."
    			},
    			creditGroupingText: {
    				title: "credit grouping text"
    			},
    			creditId: {
    				title: "credit id"
    			},
    			debitGroupingId: {
    				title: "debit grouping id",
    				descr: "Id of the credit account."
    			},
    			debitGroupingText: {
    				title: "debit grouping text"
    			},
    			debitId: {
    				title: "debit id",
    				descr: "Id of the debit account."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if the row is in use."
    			},
    			entryCode: {
    				title: "entry code",
    				descr: "A single code in the code group: single income type code or total row type."
    			},
    			entryCodeGroup: {
    				title: "entry code group",
    				descr: "Entry group: income type or total type."
    			},
    			flags: {
    				title: "flags"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the row."
    			},
    			isIncluded: {
    				title: "is included",
    				descr: "Boolean indicating whether the system should make an accounting entry based on this row."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the row is non-removable template."
    			}
    		},
    		AccountingSettings: {
    			title: "accounting settings",
    			descr: "Settings related to accounting reports.",
    			accountant: {
    				title: "accountant",
    				descr: "The person or company responsible for accounting."
    			},
    			defaultTargetId: {
    				title: "Kohteen tunniste",
    				descr: "Default accounting target."
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			ruleSets: {
    				title: "rule sets",
    				descr: "Rulesets containing required tabular logic and accounts for creating accounting entries."
    			},
    			software: {
    				title: "software",
    				descr: "Integrated accounting software."
    			},
    			targets: {
    				title: "Kohteet",
    				descr: "Accounting target systems and the rulesets assigned to them."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		AccountingSoftwareSettings: {
    			title: "accounting software settings",
    			descr: "Setting related to accounting software"
    		},
    		AccountingTarget: {
    			title: "accounting target",
    			descr: "Accounting target systems and the rulesets assigned to them.",
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if this target is in use"
    			},
    			id: {
    				title: "id",
    				descr: "Id of the target definition."
    			},
    			image: {
    				title: "image",
    				descr: "Illustrative image for the target system."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the target system."
    			},
    			ruleSetId: {
    				title: "rule set id",
    				descr: "Assigned ruleset for this target system."
    			}
    		},
    		AccountingTargetSettings: {
    			id: {
    				title: "Tunniste"
    			},
    			method: {
    				title: "Tyyppi"
    			},
    			availableMethods: {
    				title: "Tyyppivalinnat"
    			},
    			avatar: {
    				title: "Avatar"
    			},
    			data: {
    				title: "Data"
    			}
    		},
    		Ajax: {
    			title: "ajax",
    			descr: "Abstracts the interface for Http / Ajax functionality that is required by Salaxy core classes.\nThis interface is implemented in this project for jQuery and Node.\nAdditional implementations exists for Angular.js (salaxy-lib-ng1) and Angular.io / ngx (salaxy-lib-ng).\nYou may also create your own implementation for the framework or http component that you want to support.",
    			serverAddress: {
    				title: "server address",
    				descr: "The server address - root of the server. This is settable field.\nWill probably be changed to a configuration object in the final version."
    			},
    			useCredentials: {
    				title: "use credentials",
    				descr: "By default credentials are not used in http-calls.\nEnable credentials with this flag (force disabled when the token is set)."
    			},
    			getApiAddress: {
    				title: "get api address",
    				descr: "Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api'"
    			},
    			getCurrentToken: {
    				title: "get current token",
    				descr: "Gets the current Salaxy Bearer token"
    			},
    			getHTML: {
    				title: "get h t m l",
    				descr: "Gets a HTML-message from server using the API\n\n**Returns**: A Promise with result html. Standard Promise rejection to be used for error handling.\n"
    			},
    			getJSON: {
    				title: "get j s o n",
    				descr: "Gets a JSON-message from server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			getServerAddress: {
    				title: "get server address",
    				descr: "Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com'"
    			},
    			postHTML: {
    				title: "post h t m l",
    				descr: "POSTS data to server and receives back HTML.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			postJSON: {
    				title: "post j s o n",
    				descr: "POSTS data to server and receives back a JSON-message.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			remove: {
    				title: "remove",
    				descr: "Sends a DELETE-message to server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			setCurrentToken: {
    				title: "set current token",
    				descr: "Sets Salaxy Bearer token for this http context."
    			}
    		},
    		AnnualLeave: {
    			title: "annual leave",
    			descr: "A planned annual Leave - not sure whether it will kept or is it just planned",
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeavePayment: {
    			title: "annual leave payment",
    			descr: "Payment related to annual leave: holiday salary, bonus and compensation",
    			calcId: {
    				title: "calc id",
    				descr: "If the payment is defined by a calculation, stores the calculation ID."
    			},
    			date: {
    				title: "date",
    				descr: "The reference date: Typically the Paid at date.\nThe items are typically sorted based on this."
    			},
    			holidayBonus: {
    				title: "holiday bonus",
    				descr: "Bonus for either Holiday Salary or Compensation."
    			},
    			holidayCompensation: {
    				title: "holiday compensation",
    				descr: "Compensation that is paid for holidays not kept."
    			},
    			holidayDays: {
    				title: "holiday days",
    				descr: "Days of holidays that are paid or compensated"
    			},
    			holidaySalary: {
    				title: "holiday salary",
    				descr: "Salary that is paid for holidays"
    			},
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the payment entry"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeaves: {
    			title: "annual leaves",
    			descr: "Data about the annual leaves planned and used during the holiday year.",
    			paid: {
    				title: "paid",
    				descr: "Paid or otherwise handled annual leaves.\nPRELIMINARY: This property is under revision - it may be moved or renamed without being a breaking change."
    			},
    			planned: {
    				title: "planned",
    				descr: "Planned annual leaves."
    			}
    		},
    		ApiCrudObject: {
    			title: "api crud object",
    			descr: "Base interface for CRUD objects in the Salaxy API.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Partner service that has be used in modifying the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated. Typically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		ApiListItem: {
    			title: "api list item",
    			descr: "The line item class for search and listing results. Use this class for building user interfaces.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			listType: {
    				title: "list type",
    				descr: "List item type of the item if the item supports a custom OData interface."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			type: {
    				title: "type",
    				descr: "The payload type of the item."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ApiListItemValidation: {
    			title: "api list item validation",
    			descr: "ApiValidation object that is specialized for list items. Provides if and index properties for connecting a validation to a specific item in the list",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the item this validation is for.\nIdentifiers are not always available (if the item has not been saved), so index is a more generic approach."
    			},
    			index: {
    				title: "index",
    				descr: "Zero based index of the item this validation is for."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiProfileSearchResult: {
    			title: "api profile search result",
    			descr: "Profile search result for the API",
    			cities: {
    				title: "cities"
    			},
    			facets: {
    				title: "facets",
    				descr: "Dynamic for search facets if faceting is being used.\nNOTE: Facet names are lower camel case - using the JavaScript naming conventions as in the original JSON."
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			},
    			results: {
    				title: "results",
    				descr: "Actual search result - list of items"
    			},
    			searchText: {
    				title: "search text",
    				descr: "Search text that was input by the user"
    			},
    			searchTextTechnical: {
    				title: "search text technical",
    				descr: "Search text as it is sent to the search provider:\nContains provider specific wildcards etc."
    			},
    			sort: {
    				title: "sort",
    				descr: "Sort string"
    			},
    			totalCount: {
    				title: "total count",
    				descr: "Total count returned by the search service.\nThis may be more than the count of Result list"
    			}
    		},
    		ApiProfileSearchResultProfile: {
    			title: "api profile search result profile",
    			avatar: {
    				title: "avatar"
    			},
    			cities: {
    				title: "cities"
    			},
    			id: {
    				title: "id"
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			}
    		},
    		ApiValidation: {
    			title: "api validation",
    			descr: "Common base class / interface for data validation.",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiValidationError: {
    			title: "api validation error",
    			descr: "Validation errors used in API output.",
    			code: {
    				title: "code",
    				descr: "Some error providers may have a unique technical code for error or validator"
    			},
    			key: {
    				title: "key",
    				descr: "Full path to Name of the property / field (Member name in JSON).\nThis may be null/empty for type General."
    			},
    			msg: {
    				title: "msg",
    				descr: "Validation error message in the requested language."
    			},
    			type: {
    				title: "type",
    				descr: "High level type: Specifies the relation to object and its Member."
    			}
    		},
    		AppStatus: {
    			title: "app status",
    			descr: "Supports the UserSession object by providing additional information about the status\nof fetching the session, data, expiration etc.",
    			clientRoles: {
    				title: "client roles",
    				descr: "Application specific roles that are set on the client (browser code)\nas opposed to normal server-side defined roles."
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "Indicates that we are running in test environment with test data.\nThis information is typically available already before the session is available and may be used in login / register.\nNull indicates that the information is not available. This should typically be considered false for security."
    			},
    			sessionCheckInProgress: {
    				title: "session check in progress",
    				descr: "If true, the session check from server is in progress"
    			},
    			sessionExpired: {
    				title: "session expired",
    				descr: "PLACEHOLDER: Indicates that the session has expired (as opposed to not checked yet).\nThis property has not yet been implemented."
    			}
    		},
    		Article: {
    			title: {
    				title: "title",
    				descr: "Title: Shown in te beginning of the text, in the browser window header, in the lists etc."
    			},
    			descr: "Read-only Article object for the API",
    			author: {
    				title: "author",
    				descr: "Author of the article."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			date: {
    				title: "date",
    				descr: "Publish date"
    			},
    			file: {
    				title: "file",
    				descr: "File path as in CDN"
    			},
    			html: {
    				title: "html",
    				descr: "Article text and/or markup.\nThis is not serialized to XML, but read/written separately to blob storage - typically using MarkDown."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			longSummary: {
    				title: "long summary",
    				descr: "A slightly longer summary, shown in Facebook, Google+\nand LinkedIn likes/shares."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			picture: {
    				title: "picture",
    				descr: "URL to a picture that should be used in social media sharing"
    			},
    			section: {
    				title: "section",
    				descr: "The section where the article should appear"
    			},
    			shortSummary: {
    				title: "short summary",
    				descr: "Shorter summary (max 140 chars), shown as default\nin Tweets."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AssureCompanyAccountRequest: {
    			title: "assure company account request",
    			descr: "Request to assure that a company account exists: Either an account is returned or an account stub (non-verified) will be created.",
    			contactFirstName: {
    				title: "contact first name",
    				descr: "First name of the contact person"
    			},
    			contactLastName: {
    				title: "contact last name",
    				descr: "Last name of the contact person"
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Company number in the the trade register (Y-tunnus).\nFormat is \"1234567-8\" - 7 numbers, one control character, with or without the dash.\nAlso, the international VAT format is allowed: FI12345678"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The account ID at the partner site.\nIf present, this ID is used to restrict the authorization to one account at partner site only."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number for the contact person"
    			}
    		},
    		AttributeDoc: {
    			title: "attribute doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attr: {
    				title: "attr",
    				descr: "HTML attribute name e.g. my-prop for myProp."
    			},
    			binding: {
    				title: "binding",
    				descr: "Angular JS binding type"
    			},
    			controller: {
    				title: "controller",
    				descr: "Controller that implements the property"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the memeber."
    			}
    		},
    		Author: {
    			title: "author",
    			descr: "Author of an article in the CMS",
    			companyDescription: {
    				title: "company description",
    				descr: "Short description text / label for the organization."
    			},
    			companyLogo: {
    				title: "company logo",
    				descr: "Company or organization logo - expect that this may be null."
    			},
    			companyUrl: {
    				title: "company url",
    				descr: "Link to a web site - typically of the organization that the author represents"
    			},
    			description: {
    				title: "description",
    				descr: "Short description text"
    			},
    			guid: {
    				title: "guid",
    				descr: "Unique key for the object"
    			},
    			image: {
    				title: "image",
    				descr: "Public image for the author"
    			},
    			name: {
    				title: "name",
    				descr: "Person name"
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Exact time of last technical save: Not editable by user and will change in imports/exports etc."
    			},
    			type: {
    				title: "type",
    				descr: "Supports the Cosmos DB migration by serializing the type of the current object as a short type string for querying."
    			}
    		},
    		Authority: {
    			title: "authority",
    			descr: "A person or other party that authorized the current person.",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account ID. This is optional and should not be set if the ID is the same as the target account / Person."
    			},
    			authorizedAt: {
    				title: "authorized at",
    				descr: "Date for the authorization."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail of the person. This is currently not used, but is expected to be used in the future.\nSo fill it in as it was required unless you have very compelling reasons."
    			},
    			name: {
    				title: "name",
    				descr: "Name of the authority."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Person ID is the key to signature. This is currently always required."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number of the person. This is currently optional.\nIdea is to send SMS invitations in some scenarios."
    			}
    		},
    		AuthorizationContract: {
    			title: "authorization contract",
    			descr: "Contract / authorization document that is the bases for Salaxy acting in behalf of this account",
    			authorizationType: {
    				title: "authorization type",
    				descr: "Type of the authorization"
    			},
    			authorizedBy: {
    				title: "authorized by",
    				descr: "A person or other party that authorized the current person."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "If true, there is a valid authorization - the account has been validated."
    			},
    			signedAt: {
    				title: "signed at",
    				descr: "Time when the contract was signed"
    			}
    		},
    		Avatar: {
    			title: "avatar",
    			descr: "Defines an avatar for an account, profile etc. - mainly the image that should be shown, names and short description",
    			color: {
    				title: "color",
    				descr: "Color - currently only used by type Icon"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			description: {
    				title: "description",
    				descr: "Short description of the user.\nThis may be overriden by a context specific value by the business logic."
    			},
    			displayName: {
    				title: "display name",
    				descr: "Display name. For a person this is 'FirstName LastName' (auto-created)."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type: person/company"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			initials: {
    				title: "initials",
    				descr: "Initials - currently only used by type Icon"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pictureType: {
    				title: "picture type",
    				descr: "Type of the Avatar picture."
    			},
    			sortableName: {
    				title: "sortable name",
    				descr: "Sortable name for ordered lists etc. For a person this is 'LastName, FirstName' (auto-created).\nDepending on the data storage, this may be all-lowercase for easier sorting and search."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			url: {
    				title: "url",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			}
    		},
    		BaseSalaryProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		BlobFile: {
    			title: "blob file",
    			descr: "The BlobFile stores metadata about miscellaneous BLOB-files that are stored to the system. For example Receipts and Authorizations.",
    			blobUri: {
    				title: "blob uri",
    				descr: "Full unique URI in the storage"
    			},
    			container: {
    				title: "container",
    				descr: "The logical container - first level folder for the file."
    			},
    			filePath: {
    				title: "file path",
    				descr: "Technical file name in storage.  In practical terms, this is the relative path\nexcluding the container part \"/[container]/\".\nI.e. this property is without the starting slash \"/\".\nIt must be unique and is used for Uri.\nFor user files this is constructed \"[Username]/[Type]/[UniqueFileName].[xxx]\""
    			},
    			fileType: {
    				title: "file type",
    				descr: "File type (mime type etc.) based on the ending of the file name.\nNull for folders."
    			},
    			id: {
    				title: "id"
    			},
    			isCollection: {
    				title: "is collection",
    				descr: "If true, this is a collection (folder in file system)"
    			},
    			length: {
    				title: "length",
    				descr: "The length of the file in bytes."
    			},
    			metadata: {
    				title: "metadata"
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "If created, contains uri to a PNG image for preview purposes."
    			},
    			type: {
    				title: "type",
    				descr: "Strongly typed logical type of the file. For user files, this is a sub folder."
    			},
    			updatedAt: {
    				title: "updated at"
    			},
    			userFriendlyName: {
    				title: "user friendly name",
    				descr: "A user friendly name that can be used in listings.\nIn the upload scenarios this is usually the original file name when uploaded.\nIt is not necessarily unique. No path information"
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		BlobFileMetadata: {
    			title: "blob file metadata",
    			descr: "Metadata related to blob files as a strongly typed object.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object related to the report.\nFor example for the salary payments this is the other party (employer or worker)"
    			},
    			data: {
    				title: "data",
    				descr: "Arbitrary metadata."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Identifier of the original calculation etc. business object."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "Whether the document is digitally signed (or manually checked as being signed).\nAt the time of writing, this is used only for AuthorizationPdf documents."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "Logical date for the file. E.g. first of the month for a monthly report."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype is internal typing within a BlobFileType. This may or may not be a sub-folder.\nThere is no enumeration on File level - there may be one under services specific to one BlobFileType."
    			}
    		},
    		BoardUsecase: {
    			title: "board usecase",
    			descr: "Handles the Calculation of CalculationRowType.Board to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		BusinessObject: {
    			title: "business object",
    			descr: "Business object in the Salaxy repository",
    			id: {
    				title: "id",
    				descr: "Id of the business object"
    			},
    			name: {
    				title: "name",
    				descr: "Optional name for the business object."
    			},
    			object: {
    				title: "object",
    				descr: "Optional property for object instance."
    			},
    			owner: {
    				title: "owner",
    				descr: "Partition key of the business object"
    			},
    			type: {
    				title: "type",
    				descr: "Business object type"
    			}
    		},
    		CalcEmployer: {
    			title: "calc employer",
    			descr: "Employer of a calculation",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account identifier for the Employer."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image information and type of employer.\nNOTE: Recalculate-method retains the values for display purposes if AccountId or IsSelf is not set.\nThis is mainly for demo and testing in anonymous mode.\nHowever, in storage methods, the value is reset based on values of AccountId or IsSelf\neven if they are not set: Anything stored in Avatar will be ignored."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			}
    		},
    		CalcHolidayBonus: {
    			title: "calc holiday bonus",
    			descr: "Holiday Bonus data that is stored into a Salary calculation.",
    			alreadyPaid: {
    				title: "already paid",
    				descr: "The sum of holiday bonus that has already been paid this year.\nThe payment is never so big that this would become more than BonusTotalHolidaySalary\n(by this calculation - the limit can be exceeded manually)."
    			},
    			totalHolidaySalary: {
    				title: "total holiday salary",
    				descr: "Total yearly holiday salary for the purposes of holiday bonus."
    			},
    			yearlyLeaveDays: {
    				title: "yearly leave days",
    				descr: "Number of yearly leave days. This is for the purposes of calculating the Bonus,\nso this is yearly salary days, not the complete saldo (including StartSaldo)."
    			}
    		},
    		CalcHourlyData: {
    			title: "calc hourly data",
    			descr: "Holiday calculation based on hourly salary OR performance based salaries. May also contain other data related to working hours.",
    			isHourly: {
    				title: "is hourly",
    				descr: "If true, the hourly / performance based calculation is applied."
    			}
    		},
    		CalcInfo: {
    			title: "calc info",
    			descr: "Informational object that stores data about the salary calculation / payment that is not directly related to calculating the salary.",
    			backofficeNotes: {
    				title: "backoffice notes",
    				descr: "Notes related to the payment and other backoffice operations:\nFor example the justification why the sum is in calculation vs. Paytrail payment."
    			},
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			paymentChannel: {
    				title: "payment channel",
    				descr: "Channel that is used for payment."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "Payment reference number in payment service"
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "The Payroll Id, if this calculation is part of a Payroll list."
    			},
    			pensionPaymentDate: {
    				title: "pension payment date",
    				descr: "Calculated pension insurance payment date."
    			},
    			pensionPaymentRef: {
    				title: "pension payment ref",
    				descr: "Pension insurance reference number."
    			},
    			pensionPaymentSpecifier: {
    				title: "pension payment specifier",
    				descr: "Pension insurance payment specifier."
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "End date of the work"
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Start date of the work"
    			},
    			workerMessage: {
    				title: "worker message",
    				descr: "Message from worker to employer - when worker sends the salary calculation to the employer."
    			}
    		},
    		CalcMonthlyData: {
    			title: "calc monthly data",
    			descr: "Period recalculation parameters that are related to Monthly salary. Could theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested.",
    			absencesDaySalary: {
    				title: "absences day salary",
    				descr: "Price for a single absence day. Typically based on MonthlySalary, but may be modified."
    			},
    			isMonthly: {
    				title: "is monthly",
    				descr: "If true the Monthly (weekly) calculation is being applied to absences and holidays."
    			},
    			monthlySalary: {
    				title: "monthly salary",
    				descr: "Monthly salary in holiday and absences calculation.\nTypically, the same as the actual monthly calculation, but may be modified."
    			}
    		},
    		CalcRowConfig: {
    			title: "calc row config",
    			descr: {
    				title: "descr",
    				descr: "Description for the row type"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount input"
    			},
    			category: {
    				title: "category",
    				descr: "Categorization of the row types."
    			},
    			color: {
    				title: "color",
    				descr: "Color is based on the category"
    			},
    			favorite: {
    				title: "favorite",
    				descr: "Favorite row types and their sort order.\nThis is currently fixed, but may later be based on user profile (e.g. most used types)."
    			},
    			iconText: {
    				title: "icon text",
    				descr: "Icon short text is 1-5 characters suitable for icon.\nHas 5 characters only if very narrow, e.g. \"1\" and comma - typically 2-3 chars."
    			},
    			kind: {
    				title: "kind",
    				descr: "If the row has a kind selection (the main subtype), definition of the subtypes."
    			},
    			label: {
    				title: "label",
    				descr: "Label for the row type"
    			},
    			moreInfo: {
    				title: "more info",
    				descr: "More info HTML"
    			},
    			name: {
    				title: "name",
    				descr: "Name / unique identifier of the row type."
    			},
    			order: {
    				title: "order",
    				descr: "Order number in the lists"
    			},
    			price: {
    				title: "price",
    				descr: "Price input"
    			},
    			total: {
    				title: "total",
    				descr: "The total input or read-only field."
    			}
    		},
    		CalcTax: {
    			title: "calc tax",
    			descr: "Tax card information stored as part of the Calculation object",
    			estimatedTax: {
    				title: "estimated tax",
    				descr: "Calculated widthholding tax. Note that this is an estimation until the calculation is actually paid.\nAlso, if the salary is not paid as cash (e.g. fringe benefits), there may not be enough money to make the full deduction."
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Possibility to set a fixed widthholding tax amount for this calculation regardless of\nthe percentage etc. in the tax card."
    			},
    			hasNewSharedCard: {
    				title: "has new shared card",
    				descr: "A flag that during last recalculation, there was a new shared taxcard that was waiting approval."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "The API returns true if there is a valid taxcard for the PaidAt date for this calculation\nor if FixedTaxAmount is set for th calculation."
    			},
    			snapshot: {
    				title: "snapshot",
    				descr: "Snapshot of the tax card for salary slip etc. purposes.\nUpdated in Recalculate / Save based on TaxCardId"
    			},
    			taxPercentForVisualization: {
    				title: "tax percent for visualization",
    				descr: "Tax percent that is used in visualization. This is mainly used in Unit testing and anonymous calcultors.\nNot used in real paid calculations. This is percent. I.e. for 50% set 50, not 0.5.\nIf PaymentData or AccountId lead to a valid tax card type and percent, or if there is FixedTaxAmount, this value is not used."
    			},
    			taxcardId: {
    				title: "taxcard id",
    				descr: "Identifier for a stored tax card if one is applied to this calculation.\nSystem updates the TaxCardId automatically in Recalculate / Save if it finds a better candidate\nand also updates the Snapshot."
    			}
    		},
    		CalcWorker: {
    			title: "calc worker",
    			descr: "Worker object for the current calculation",
    			accountId: {
    				title: "account id",
    				descr: "Palkkaus.fi Account Id for the worker.\nAlternatively you can set the Payment data with the required infromation"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image and properties of the Worker.\nBased on either the AccountId or PaymentData"
    			},
    			dateOfBirth: {
    				title: "date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment Id for the worker. This is currently resolved using the workers AccountId."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the currently Account\nIf set to true, will set current Account as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			},
    			paymentData: {
    				title: "payment data",
    				descr: "Data that is needed for salary payment.\nThis object may be null if AccountId has been set.\nAlso, if AccountId has been set, these properties will be ignored:\nThe Worker will decide on payment properties herself."
    			},
    			tax: {
    				title: "tax",
    				descr: "New Widthholding tax logic that is based on separately stored tax cards."
    			}
    		},
    		CalcWorkerPaymentData: {
    			title: "calc worker payment data",
    			descr: "The required information for paying the Salary to Worker.",
    			email: {
    				title: "email",
    				descr: "Contact e-mail address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name of the person."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "IBAN number for the Bank account"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name / Surname of the person."
    			},
    			socialSecurityNumber: {
    				title: "social security number",
    				descr: "Gets the Social security number the way the user set it in input.\nHOWEVER: The getter will change the string to upper invariant/trim and if the last character is \"*\", it will be replaced by calculated checksum."
    			},
    			socialSecurityNumberValid: {
    				title: "social security number valid",
    				descr: "Social security number if valid or null"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		CalcWorkflow: {
    			title: "calc workflow",
    			descr: "Defines the properties of salary calculation workflow - typically the workflow from draft to Payment.",
    			paidAt: {
    				title: "paid at",
    				descr: "Time when this calculation was paid (if it was paid) to Palkkaus service."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			salaryPaidAt: {
    				title: "salary paid at",
    				descr: "Time when the salary was paid by Palkkaus service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from Draft to PaymentSucceeded"
    			}
    		},
    		CalcWorktime: {
    			title: "calc worktime",
    			descr: "Worktime data that is stored as part of Salary calculation: work days, absences, holidays and holiday accrual. In the future may also contain relevant part of timesheet data (work hours).",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of days of non-paid absences.\nThese are deducted from the Monthly salary, not from Hourly salary at the moment."
    			},
    			accruals: {
    				title: "accruals",
    				descr: "Accruals for each calendar month in this calculation."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Parameters related to Holiday Bonus."
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period that is updated from employment at each recalculation (as opposed to holiday specification)."
    			},
    			holidaySpec: {
    				title: "holiday spec",
    				descr: "Specification for the holiday calculation"
    			},
    			hourly: {
    				title: "hourly",
    				descr: "Parameters related to Hourly or Performance based salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			leavesDailySalary: {
    				title: "leaves daily salary",
    				descr: "Price for a single holiday day"
    			},
    			leavesDays: {
    				title: "leaves days",
    				descr: "Number of paid Leave days that is paid in this salary calculation / for this period."
    			},
    			monthly: {
    				title: "monthly",
    				descr: "Parameters related to Monthly salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			workDays: {
    				title: "work days",
    				descr: "Describes the working days in the period.\nCurrently, uses the DaysCount, but later the full days array will be taken to use.\nThis includes the absence days (divider in AbsencesDaySalary)."
    			}
    		},
    		Calculation: {
    			title: "calculation",
    			descr: "Calculation is the most important model in the Palkkaus.fi API. We suggest you open the separately provided documentation and especially the class diagram to understand the whole model in detail.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer",
    				descr: "The employer object for this calculation.\nTypically, you do not need to set this before the actual payment process."
    			},
    			framework: {
    				title: "framework",
    				descr: "DEPRECIATED: Usecase will be replacing this functionality in a next major release.\nThe Framework Agreement (TES) parameters that affect the framework based side costs etc. of this calculation"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "Information about the work that is not directly related to calculating salary and expenses."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "The results of the calculation. In the API, this is a read-only object."
    			},
    			rows: {
    				title: "rows",
    				descr: "Set of rows that make up the salary calculation:\nExpenses, Benefits, deductions etc. Note that side costs are not rows."
    			},
    			salary: {
    				title: "salary",
    				descr: "The main salary element.\nAlternatively the salary can be set in the Rows object - this will be the only way to set multiple salary rows (not yet supported)."
    			},
    			sharing: {
    				title: "sharing",
    				descr: "Sharing of the calculation. This object cannot be modified directly,\nuse specialized methods in Calculations service."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing salary calculations.\nIn practice, a usecase is either a user interface or a microservice that provides a salary calculation\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of calculation.\nNOTE: This functionality will replace Framework property in a next major release."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker (employee) that has performed the work."
    			},
    			workflow: {
    				title: "workflow",
    				descr: "Information about the workflow and state of the calculation."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "Readonly workflow data, which is exposed to API."
    			},
    			worktime: {
    				title: "worktime",
    				descr: "Worktime data that is stored as part of Salary calculation:\nwork days, absences, holidays and holiday accrual.\nIn the future may also contain relevant part of timesheet data (work hours)."
    			}
    		},
    		CalculationData: {
    			title: "calculation data",
    			descr: "Calculation specific business data for index.",
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "Payroll id."
    			},
    			pensionPayment: {
    				title: "pension payment",
    				descr: "The total pension payment for this calculation."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "A flag indicating if the requested date for the SalaryDate has been set.\nIf the date has been set, the value of the flag is 'SALA'."
    			},
    			socialSecurityPayment: {
    				title: "social security payment",
    				descr: "The social security payment- paid as part of taxation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from the end-user point-of-view."
    			},
    			taxPayment: {
    				title: "tax payment",
    				descr: "The taxes that were deducted from the worker salary a paid as part of taxation."
    			},
    			usecaseUri: {
    				title: "usecase uri",
    				descr: "Use case uri."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "Work end date."
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Work start date."
    			},
    			workerPayment: {
    				title: "worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			}
    		},
    		CalculationListItem: {
    			title: "calculation list item",
    			descr: "Represents a single item in a list of calculations in the user interface. It contains all essential information of the calculation and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		CalculationResult: {
    			title: "calculation result",
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			irRows: {
    				title: "ir rows",
    				descr: "Rows that are compatible with Incomes Register (tulorekisteri) process.\nThese are available only for calculations paid after 7.2.2019 (TODO: estimation as of writing)."
    			},
    			responsibilities: {
    				title: "responsibilities",
    				descr: "Responsibilities of Employer and parameters that affect those responsibilities."
    			},
    			rows: {
    				title: "rows",
    				descr: "The result rows. Note that this collection also contains the main Salary entry as one of the rows."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			validation: {
    				title: "validation",
    				descr: "The validation result for the current calculation.\nPlease note that this is not serialized into the data storage."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		CalculationSettings: {
    			title: "calculation settings",
    			descr: "Asetukset, jotka vaikuttavat palkanmaksuun",
    			household: {
    				title: "household",
    				descr: "Settings that are specific to Households logic.\nThese do not apply to Companies and may be null for them"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Settings related to mandatory accident insurance."
    			},
    			marketing: {
    				title: "marketing",
    				descr: "Marketing settings, E.g. campaigns, customer groups etc."
    			},
    			pension: {
    				title: "pension",
    				descr: "Pension insurance related settings."
    			},
    			pensionContracts: {
    				title: "Työeläkevakuutukset"
    			},
    			incomesRegister: {
    				title: "Tulorekisteri",
    				descr: "Ilmoittaminen tulorekisteriin"
    			},
    			insuranceContracts: {
    				title: "Tapaturmavakuutukset"
    			},
    			tax: {
    				title: "tax",
    				descr: "Settings related to taxation. Mainly widthholding tax, but potentially other taxes as well."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		CalculationSharing: {
    			title: "calculation sharing",
    			descr: "Defines the properties related to sharing and sending of the calculation.",
    			type: {
    				title: "type",
    				descr: "Type of sharing for this calculation."
    			},
    			uri: {
    				title: "uri",
    				descr: "Sharing uri."
    			}
    		},
    		CalculationSummary: {
    			title: "calculation summary",
    			descr: "Summarises from a large calculation object the relevant data for yearend feedback.",
    			benefitsInKindWithoutPhone: {
    				title: "benefits in kind without phone",
    				descr: "Benefits excluding phone benefit: Accomodation, Meals and other"
    			},
    			calculationId: {
    				title: "calculation id",
    				descr: "Identifier of the calculation or correction."
    			},
    			compensations: {
    				title: "compensations",
    				descr: "Compensations is daily allowance (including half days) plus meals compensation."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Total gross salary"
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes."
    			},
    			message: {
    				title: "message",
    				descr: "Message related to this row. Mainly for backoffice purposes."
    			},
    			milageCompensation: {
    				title: "milage compensation",
    				descr: "Milage compensation includes MilageOwnCar, Daily milage as well as other milage"
    			},
    			originalIsDeductible: {
    				title: "original is deductible",
    				descr: "If true the original calculation was marked as deductible (household deduction)."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "Date when the calculation was paid or where the correction should be set."
    			},
    			phoneBenefit: {
    				title: "phone benefit",
    				descr: "Phone benefit in euros"
    			},
    			salary: {
    				title: "salary",
    				descr: "Taxable salary of the calculation"
    			},
    			salaryType: {
    				title: "salary type",
    				descr: "Type of salary from Pension and Taxation point-of-view"
    			},
    			selectedIsDeductible: {
    				title: "selected is deductible",
    				descr: "If true the the calculation is currently marked as deductible (household deduction).\nDefault value is OriginalIsDeductible."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "The mandatory side costs for taxation purposes"
    			},
    			socialInsuranceDeductionsFromEmployee: {
    				title: "social insurance deductions from employee",
    				descr: "Pension and unemployment insurance deductions from the Worker"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "Categories for household deduction.\nUI implementors should force this enumeration being a single value although historically flags were supported."
    			},
    			type: {
    				title: "type",
    				descr: "Type of Calculation summary"
    			},
    			withholdingTax: {
    				title: "withholding tax",
    				descr: "Widthholding tax deducted from the Worker"
    			}
    		},
    		CalculationsDataRow: {
    			title: "calculations data row",
    			descr: "Data row for calculations data. This is typically used for imports and exports.",
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Tax if calculated"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the calculation"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End date of the work"
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start date of the work"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID for the worker."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			status: {
    				title: "status",
    				descr: "Read only calculation status"
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			}
    		},
    		CarBenefit: {
    			title: "car benefit",
    			ageGroupCode: {
    				title: "age group code"
    			},
    			carBenefitCode: {
    				title: "car benefit code"
    			},
    			kilometers: {
    				title: "kilometers"
    			}
    		},
    		CarBenefitUsecase: {
    			title: "car benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.CarBenefitUsecase to IrRows",
    			ageGroup: {
    				title: "age group",
    				descr: "Age group as in A,B,C or I for international."
    			},
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isKilometersBased: {
    				title: "is kilometers based",
    				descr: "If true, the calculation is kilometers based."
    			},
    			kilometers: {
    				title: "kilometers",
    				descr: "Number of kilometers is filled in only if that type of benefit calculation is used."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Certificate: {
    			title: {
    				title: "title",
    				descr: "End user description for the certificate."
    			},
    			descr: "User certificate.",
    			bytes: {
    				title: "bytes",
    				descr: "Certificate bytes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			password: {
    				title: "password",
    				descr: "Password for issuing a new certificate. Not persisted."
    			},
    			thumbprint: {
    				title: "thumbprint",
    				descr: "Certificate thumbprint."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		City: {
    			title: "city",
    			count: {
    				title: "count"
    			},
    			grouping: {
    				title: "grouping"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			nameLang2: {
    				title: "name lang2"
    			}
    		},
    		ClassDoc: {
    			title: "class doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			extendedBy: {
    				title: "extended by",
    				descr: "Types that extend this type (extendedBy)"
    			},
    			"extends": {
    				title: "extends",
    				descr: "Collection of reference types that this class extends (extendedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			implementedBy: {
    				title: "implemented by",
    				descr: "Types that iplement this interface (implementedBy)"
    			},
    			"implements": {
    				title: "implements",
    				descr: "Collection of interfaces that this class implements (implementedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			members: {
    				title: "members",
    				descr: "Public properties, fields and functions."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			usages: {
    				title: "usages",
    				descr: "Salaxy specific logic may add usages here.\nCurrently, components will add themselves to controllers."
    			}
    		},
    		ClassMemberDoc: {
    			title: "class member doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Display grouping for the member: E.g. accessors are considered properties here."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			parameters: {
    				title: "parameters",
    				descr: "For grouping type \"Method\", lists parameters\nTODO: Consider making a separate type for properties."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the member."
    			}
    		},
    		CompanyAccount: {
    			title: "company account",
    			descr: "A company account - note that this can also be an association etc. - something that has a Company ID.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			companyType: {
    				title: "company type",
    				descr: "High level company type"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Majority shareholders (over 25% stake) are stored in this field.\nOwnership is a list of Shareholder objects."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		CompanyAccountSettings: {
    			calc: {
    				title: "Palkanmaksun asetukset",
    				descr: "Asetukset, jotka vaikuttavat palkanmaksuun"
    			},
    			partner: {
    				title: "Kumppaniasetukset"
    			},
    			accounting: {
    				title: "Kirjanpidon asetukset"
    			},
    			payments: {
    				title: "Maksamisen asetukset"
    			},
    			reporting: {
    				title: "Raportoinnin asetukset"
    			}
    		},
    		CompanyInput: {
    			title: "company input",
    			descr: "Represents the input for creating a company account.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the company"
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a company account based on company data."
    			},
    			doYtjUpdate: {
    				title: "do ytj update",
    				descr: "If true, will fetch the data from the Finnish Business Infromation System (YTJ/BIS).\nFetching is done firstly based on Business ID, if that is not valid then based on name."
    			},
    			logo: {
    				title: "logo",
    				descr: "URL to a Logo picture (preferably PNG). This logo should be a square - icon type of logo for avatar."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Company ownership for KYC"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "Identifier at the partner site - typicelly an organization ID."
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given BusinessID exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			},
    			ytjDetails: {
    				title: "ytj details",
    				descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)"
    			},
    			ytjSearch: {
    				title: "ytj search",
    				descr: "A search to Finnish Business Infromation System (YTJ/BIS). This search is triggered if you set the RawInput field."
    			}
    		},
    		ComponentDoc: {
    			title: "component doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attributes: {
    				title: "attributes",
    				descr: "HTML attributes available to specify options to the component."
    			},
    			controller: {
    				title: "controller",
    				descr: "Name of the controller that implements the functionality of the component"
    			},
    			defaultTemplate: {
    				title: "default template",
    				descr: "The default template (view) used in the rendering of the component."
    			},
    			elem: {
    				title: "elem",
    				descr: "The component tag / element in HTML"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			require: {
    				title: "require",
    				descr: "Required components in the outside scope"
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			transclude: {
    				title: "transclude",
    				descr: "Transclude slots"
    			}
    		},
    		Config: {
    			title: "config",
    			descr: "Environment specific configuration for Salaxy API's and JavaScript in general",
    			apiServer: {
    				title: "api server",
    				descr: "The base address of the Salaxy http API server, e.g. https://secure.salaxy.com"
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "A flag indicating if the current configuration is for test environment only. If true, the site is for testing only."
    			},
    			useCookie: {
    				title: "use cookie",
    				descr: "A flag indicating if the site sets a Salaxy cookie which persists over web sessions. If true, the cookie will be created."
    			},
    			wwwServer: {
    				title: "www server",
    				descr: "The base address of the Palkkaus web site, e.g. https://www.palkkaus.fi"
    			}
    		},
    		Contact: {
    			title: "contact",
    			descr: "Common contact information object for Person, Company, Location etc.",
    			city: {
    				title: "city",
    				descr: "City or community (postitoimipaikka)"
    			},
    			countryCode: {
    				title: "country code",
    				descr: "This is the ISO code for country.\nCurrently, it should always be \"fi\"."
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail address. Always lower invariant, trimmed and empty string set to null."
    			},
    			postalCode: {
    				title: "postal code",
    				descr: "Postal code"
    			},
    			street: {
    				title: "street",
    				descr: "Street address"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		ContractParty: {
    			title: "contract party",
    			descr: "A signing party in a contract.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the Party.\nNote that this is not automatically fetched by all methods: Especially when IsSelf is true."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information - note that for most signatures, only E-mail and phone number are required."
    			},
    			contractPartyType: {
    				title: "contract party type",
    				descr: "Party type (Person or Prospect)"
    			},
    			iban: {
    				title: "iban",
    				descr: "Bank account IBAN number"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the party: Currently always an account, but may be an external ID later."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true, contact is an employer"
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nIf set to false AND was previously true, will set the Profile properties null (unless explicitly set to something else)"
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "True if the person has signed the contract."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official ID if needed in signature"
    			},
    			role: {
    				title: "role",
    				descr: "Role of the person - justification for authorization"
    			},
    			signatureUrl: {
    				title: "signature url",
    				descr: "Reserved for cases where there are several signees and each has a separate signature document."
    			}
    		},
    		ContractSalary: {
    			title: "contract salary",
    			descr: "Part of the Employment contract that is relevant to paying the salary",
    			collectiveContract: {
    				title: "collective contract"
    			},
    			endDate: {
    				title: "end date",
    				descr: "End date for the employment.\nOnly set if the contract is Fixed term."
    			},
    			payDay: {
    				title: "pay day"
    			},
    			salaryAmount: {
    				title: "salary amount"
    			},
    			salaryCompension: {
    				title: "salary compension"
    			},
    			salaryType: {
    				title: "salary type"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Starting date for the employment.\nShould always be set for a valid contract."
    			},
    			workHours: {
    				title: "work hours"
    			},
    			workType: {
    				title: "work type"
    			},
    			worksHoursType: {
    				title: "works hours type"
    			}
    		},
    		CoreTypeDoc: {
    			title: "core type doc",
    			descr: "H",
    			classes: {
    				title: "classes",
    				descr: "H"
    			},
    			enums: {
    				title: "enums",
    				descr: "H"
    			}
    		},
    		CustomerFundsSettings: {
    			title: "customer funds settings",
    			descr: "Settings related to payment methods that use the customer funds account (gross payment).",
    			isPensionSelfHandling: {
    				title: "Työnantaja hoitaa itse TYEL-maksut",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "Työnantaja hoitaa itse ennakonpidätyksen maksun",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			isWorkerSelfHandling: {
    				title: "Työnantaja hoitaa itse nettopalkan maksun",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			}
    		},
    		DailyAllowanceUsecase: {
    			title: "daily allowance usecase",
    			descr: "Handles the Calculation of CalculationRowType.DailyAllowance to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		DataRow: {
    			title: "data row",
    			descr: "Interface for data row for exports and imports",
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		DateRange: {
    			title: "date range",
    			descr: "Describes the range of dates from start to end and the number of days in between.",
    			days: {
    				title: "days",
    				descr: "The collection of days contained in the DateRange if specified by the user.\nAlternative to DaysCount. If set, should also set DaysCount for backward compatibility."
    			},
    			daysCount: {
    				title: "days count",
    				descr: "Number of days in the range.\nDepending of the context, this may be mathematical (end - start) or\nexplicitly specified (typically working days).\nTypically an integer, and calculations and UI's may choose to round it."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			}
    		},
    		DeclarationReflection: {
    			title: "declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			}
    		},
    		EInvoiceAddress: {
    			title: "e invoice address",
    			descr: "Result item of the EInvoice address query.",
    			intermediator: {
    				title: "intermediator",
    				descr: "Intermediator id for this receiver address."
    			},
    			intermediatorName: {
    				title: "intermediator name",
    				descr: "Intermediator name for this receiver address."
    			},
    			receiver: {
    				title: "receiver",
    				descr: "Receiver EInvoice address of company."
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Name for the company."
    			},
    			receiverOfficialId: {
    				title: "receiver official id",
    				descr: "Official id of the company."
    			}
    		},
    		ESalaryPayment: {
    			title: "e salary payment",
    			descr: "E-salary / E-palkka is a way for a partner to direct an Employer to Salary payment without having access to Employer account. In first instance, E-salary is targeting Household Employers only.",
    			calc: {
    				title: "calc",
    				descr: "Calculation that should be paid by the Employer."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			household: {
    				title: "household",
    				descr: "Known data for the employer household that is paying the salary.\nIt is understood that the caller may not have all the information:\nPalkkaus.fi will ask the rest if necessary."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			request: {
    				title: "request",
    				descr: "Payment request and how to return to the calling service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the Payment.\nNOTE: You can only set states Created, Shared and Opened.\nOther states are set by the Salaxy service and cannot be modified throgh API."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides the validation of the ESalaryPayment object after the object has been saved."
    			}
    		},
    		EmailMessage: {
    			title: "email message",
    			descr: "API EmailMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (email address)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			subject: {
    				title: "subject",
    				descr: "Message subject"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (email address)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		EmployerCalculationDTO: {
    			title: "employer calculation d t o",
    			descr: "Models the business logic of Employer side of the calculation. For documentation, see the IEmployerCalculation interface.",
    			allSideCosts: {
    				title: "all side costs",
    				descr: "All side cost: The mandatory side costs + Palkkaus and partner fees"
    			},
    			deductionForeclosure: {
    				title: "deduction foreclosure",
    				descr: "Foreclosure that has already been paid by the employer."
    			},
    			deductionOtherDeductions: {
    				title: "deduction other deductions",
    				descr: "Other deductions that are deducted from the net salary."
    			},
    			deductionPensionSelfPayment: {
    				title: "deduction pension self payment",
    				descr: "Pension payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionSalaryAdvance: {
    				title: "deduction salary advance",
    				descr: "Salary advance - this part has already been paid."
    			},
    			deductionTaxAndSocialSecuritySelfPayment: {
    				title: "deduction tax and social security self payment",
    				descr: "Tax and social security payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnemploymentSelfPayment: {
    				title: "deduction unemployment self payment",
    				descr: "Unemployment insurance (TVR) payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnionPayment: {
    				title: "deduction union payment",
    				descr: "Union payment if it is deducted from the final figure"
    			},
    			deductionWorkerSelfPayment: {
    				title: "deduction worker self payment",
    				descr: "Worker salary and expense payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			finalCost: {
    				title: "final cost",
    				descr: "The final cost after household decuction from the tax authorities:\nAdd to the payment the deductions (paid by employer) and deduct House hold deduction (later potentially other such subsidies)"
    			},
    			foreclosureByPalkkaus: {
    				title: "foreclosure by palkkaus",
    				descr: "Foreclosure that Palkkaus.fi pays in behalf of employer."
    			},
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction that the tax man will reimburse in taxation.\nThis is an estimation: There is minimum and maximum limits etc."
    			},
    			mandatorySideCosts: {
    				title: "mandatory side costs",
    				descr: "The mandatory side costs are Pension (TyEL), Social secuirty and Unemployment insurance."
    			},
    			palkkaus: {
    				title: "palkkaus",
    				descr: "The Palkkaus.fi fee including potential partner fees."
    			},
    			pension: {
    				title: "pension",
    				descr: "The part of the Pension that Employer pays"
    			},
    			socialSecurity: {
    				title: "social security",
    				descr: "The social security payment- paid as part of taxation (ennakonpidätys)."
    			},
    			totalDeductions: {
    				title: "total deductions",
    				descr: "Total deductions that are made from the payment that is paid to Palkkaus and then to Worker"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment that is Paid to Palkkaus.fi"
    			},
    			totalPaymentLegacy: {
    				title: "total payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			totalSalaryCost: {
    				title: "total salary cost",
    				descr: "Total salary cost is the paid salary, deductions and side costs, but not expenses."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The part of the unemployment insurance that the employer pays"
    			}
    		},
    		EmployerGroups: {
    			title: "employer groups",
    			descr: "Employer groups that are set on Example calculations. These will be replaced by properties from Worker / Employer",
    			campaignPricing: {
    				title: "campaign pricing",
    				descr: "Campaign pricing may affect Palkkaus fees.\nThis field is set by the system when coming to the payment based on Employer properties (usually campaign code)."
    			},
    			employerType: {
    				title: "employer type",
    				descr: "Very small employers (households) do not required to report to the tax authorities."
    			},
    			isPrivatePerson: {
    				title: "is private person",
    				descr: "If true, the Employer is a private person.\nOtherwise Company or Unknown."
    			},
    			salaryPerMonth: {
    				title: "salary per month",
    				descr: "Very small salaries do not require Pension payments."
    			},
    			salaryPerYear: {
    				title: "salary per year",
    				descr: "Ranges for salary / year will affect the side costs."
    			}
    		},
    		EmployerInfo: {
    			title: "employer info",
    			descr: "Employer insurance information.",
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "The id of the insurance company."
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Insurance contract number."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "Indicates if the employer has an insurance."
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "Indicates if the insurance is provided by Palkkaus partnership insurance company."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "Indicates if the employer has a pension contract."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the mployer handles the pension himself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The id of the pension company."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Pension contract number."
    			}
    		},
    		EmploymentContract: {
    			title: "employment contract",
    			annualLeave: {
    				title: "annual leave"
    			},
    			benefitsException: {
    				title: "benefits exception"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isWorkAtHome: {
    				title: "is work at home"
    			},
    			mainWorkTasks: {
    				title: "main work tasks"
    			},
    			noticeExeption: {
    				title: "notice exeption"
    			},
    			otherPayDescription: {
    				title: "other pay description"
    			},
    			otherTerms: {
    				title: "other terms"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partTimeReason: {
    				title: "part time reason"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			salary: {
    				title: "salary"
    			},
    			testPeriod: {
    				title: "test period"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workPlace: {
    				title: "work place"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		EmploymentData: {
    			title: "employment data",
    			descr: "Employment specific business data for index.",
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			}
    		},
    		EmploymentListItem: {
    			title: "employment list item",
    			descr: "Represents a single item in a list of employments in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		EmploymentRelation: {
    			title: "employment relation",
    			descr: "Represents the Employment Relation: It is basically the properties of a Worker from the point-of-view of employer and as data owned by Employer whereas WorkerAccount is preliminarily owned by Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General information about the Employment relation."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		EmploymentRelationInfo: {
    			title: "employment relation info",
    			descr: "General information about the Employment relation.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of this employment relationship"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			workerAccountId: {
    				title: "worker account id",
    				descr: "The Account ID of the Worker if Worker"
    			}
    		},
    		EmploymentTerminationUsecase: {
    			title: "employment termination usecase",
    			descr: "Handles the Calculation of CalculationRowType.EmploymentTermination to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		EnumDoc: {
    			title: "enum doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			names: {
    				title: "names",
    				descr: "Member names"
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			values: {
    				title: "values",
    				descr: "Member values"
    			}
    		},
    		EnumMetadata: {
    			title: "enum metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Type name of the enumeration (UpperCamelCase)"
    			},
    			values: {
    				title: "values",
    				descr: "Values of the enumeration."
    			}
    		},
    		EnumValueMetadata: {
    			title: "enum value metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied.\nNOTE: Not all enumerations have description in the globalization files:\nThey are added only if necessary to the UI."
    			},
    			label: {
    				title: "label",
    				descr: "Label of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Text value of the enumeration (lowerCamelCase)"
    			},
    			order: {
    				title: "order",
    				descr: "Sorting order number of the value within the enum."
    			}
    		},
    		FileData: {
    			title: "file data",
    			descr: "Report specific business data for index.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "Number of calculations in the report."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id for the report."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "Number of workers in the report."
    			}
    		},
    		FileListItem: {
    			title: "file list item",
    			descr: "Represents a single item in a list of reports in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		FileParameter: {
    			title: "file parameter",
    			data: {
    				title: "data"
    			},
    			fileName: {
    				title: "file name"
    			}
    		},
    		FileType: {
    			title: "file type",
    			extension: {
    				title: "extension"
    			},
    			isImage: {
    				title: "is image"
    			},
    			isText: {
    				title: "is text"
    			},
    			mimeType: {
    				title: "mime type"
    			}
    		},
    		HealthCareHelttiProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer health care from the company Heltti",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by HelttiProductPackage."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			industry: {
    				title: "industry",
    				descr: "Industry of work"
    			},
    			"package": {
    				title: "package",
    				descr: "Defines the product package"
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		HolidayAccrual: {
    			title: "holiday accrual",
    			descr: "Yearly log for Holiday accrual.",
    			defaultAccrual: {
    				title: "default accrual",
    				descr: "Default accrual for months where the Accrual occurs.\nWhen Holiday year is recalculated, this is determined either from Employment Period or AccrualFixed if set."
    			},
    			endSaldo: {
    				title: "end saldo",
    				descr: "End saldo from the previous year. This is Total + Start Saldo."
    			},
    			months: {
    				title: "months",
    				descr: "Holiday accrual entries during the period."
    			},
    			period: {
    				title: "period",
    				descr: "The period for holiday accrual.\nThis is also the period of the holiday year that the Worker is Employed,\ni.e. it may be shorter than HolidayYear Period if the employment relation\nbegins or ends within the holiday year."
    			},
    			startSaldo: {
    				title: "start saldo",
    				descr: "Start saldo from the previous year."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the current year. This does not include Start saldo."
    			}
    		},
    		HolidayAccrualEntry: {
    			title: "holiday accrual entry",
    			descr: "Entry in the Yearly Holiday Accrual list.",
    			calcId: {
    				title: "calc id",
    				descr: "Identier of the calculation if Kind"
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Days added to holiday Accrual: Typically days per month."
    			},
    			index: {
    				title: "index",
    				descr: "Index of the month from 0-11 for easier iteration.\nThis is also the real identifier of the month when editing the entries through API.\nMonths 4-12 are index 0-8 and month 1-3 are index 9-11"
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			source: {
    				title: "source",
    				descr: "Source of the holiday accrual."
    			}
    		},
    		HolidayDate: {
    			title: "holiday date",
    			date: {
    				title: "date"
    			},
    			holiday: {
    				title: "holiday"
    			},
    			holidayGroups: {
    				title: "holiday groups"
    			},
    			localName: {
    				title: "local name"
    			},
    			name: {
    				title: "name"
    			}
    		},
    		HolidaySpecification: {
    			title: "holiday specification",
    			descr: "Specifies how the holidays are handled for this Worker.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			}
    		},
    		HolidaySpecificationForYear: {
    			title: "holiday specification for year",
    			descr: "Extends the Holiday specification with the properties of employment relation that are required for a specific year.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period for the purposes of holiday calculation (annual leave accrual).\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Wage basis for a Holiday Year.\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year.\nNOTE: This is not in HolidaySpecification because, the same value is stored in Employment relation."
    			}
    		},
    		HolidayYear: {
    			title: "holiday year",
    			descr: "Holiday year is in Finland from April to end of march. This is the time from which the holidays for the following summer and year are determined.",
    			accrual: {
    				title: "accrual",
    				descr: "Accrual of holidays (lomakirjanpito)."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation for the worker whose holidays are managed."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			leaves: {
    				title: "leaves",
    				descr: "Payments related to Annual Leaves (holidays)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			period: {
    				title: "period",
    				descr: "The holiday period for the year: Should always be 2.5.[Year]-30.4.[Year+1]"
    			},
    			spec: {
    				title: "spec",
    				descr: "Specifies how the holidays are handled.\nIf the Spec.Code is Undefined in Save, the contents is fetched from the current employment relation."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Salaxy ID of the worker. This value is required when inserting and cannot be edited later.\nWorkerSnapshot is automatically updated upon save."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			},
    			year: {
    				title: "year",
    				descr: "Year as integer. This value is required when inserting and cannot be edited later.\nPeriod will be set accordingly, or adjusted in case it is set below."
    			}
    		},
    		Holidays: {
    			title: "holidays",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		Holidays2: {
    			title: "holidays2",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		HouseholdSettings: {
    			title: "household settings",
    			descr: "Settings that are specific to Households. These do not apply to Companies.",
    			deductionInsuranceAmount: {
    				title: "deduction insurance amount",
    				descr: "Accident insurance amount reported for Household deduction purposes"
    			},
    			deductionSpouseName: {
    				title: "deduction spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			deductionSpousePersonalId: {
    				title: "deduction spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		HouseholdYearlyFeedback: {
    			title: "household yearly feedback",
    			descr: "Yearly feedback data that is specific to household customers. Main part is the information realted to household deduction and how it can be divided between spouses.",
    			insuranceAmount: {
    				title: "insurance amount",
    				descr: "Amount of accident insurance paid during this year as reported by the user\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			insuranceAmountFromAccount: {
    				title: "insurance amount from account",
    				descr: "Amount of accident insurance paid during this year from parner insurance (in Palkkaus.fi records).\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			spouseName: {
    				title: "spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			spousePersonalId: {
    				title: "spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		IEmployerResponsibilities: {
    			title: "i employer responsibilities",
    			descr: "Parameters that affect the calculation. EmployerResponsibilities is historical name - currentlty to object holds also other properties that are not well described as responsibilities. These based on the EmployerGroups selection or properties of the Employer and Worker snapshots as well as this Calculation",
    			calculationPaymentDate: {
    				title: "calculation payment date",
    				descr: "The assumed salary payment date from the worker perspective.\nThe system calculates the nearest possible salary date if no any salary date has been given by the\nemployer. If given, the system confirms the given date.\nThis date affects the yearly percents used in the salary calculation."
    			},
    			employerGroups: {
    				title: "employer groups",
    				descr: "Employer groups that are set on Example calculations.\nThese will be replaced by properties from Worker / Employer when these persons are set."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isPensionMonthlyContract: {
    				title: "is pension monthly contract",
    				descr: "If true, the employer has a Pension contract and makes the payments\nin a monthly schedule which means that Pension interest is calculated for the 20th of the next month.\nCurrently, this is only enabled to Etera - other companies will follow."
    			},
    			noAccidentInsurance: {
    				title: "no accident insurance",
    				descr: "If true, this employer is not required to get an accident insurance."
    			},
    			noPensionPayment: {
    				title: "no pension payment",
    				descr: "Pension is not paid if the salary to this Worker is less than a certain amount in a month (about 60€)\nor worker is less than 17 or more than 64 years old."
    			},
    			noSocialSecurity: {
    				title: "no social security",
    				descr: "If worker is less than 16 or more than 67, social secuiry payment is not paid"
    			},
    			noTaxResponsibility: {
    				title: "no tax responsibility",
    				descr: "If Employer pays to this Worker max 200 euro, he does not need to even make the reports to tax office.\nHe may do so though"
    			},
    			noTvr: {
    				title: "no tvr",
    				descr: "If Worker is less than 17 or more than 64, TVR is not paid.\nAlso if the Employer has no accident insurance."
    			},
    			noWithholdingTax: {
    				title: "no withholding tax",
    				descr: "If Employer pays to this Worker max 1500 euro, he does not need to deduct the witholding tax.\nHe may do so though"
    			},
    			pensionAgeRange: {
    				title: "pension age range",
    				descr: "The age range that is used in the business logic that determines the employer responsibilities.\nThis property replaces the EmployerGroups.AgeRange that was previous get/set.\nThis is based on WorkerDateOfBirth and CalculationPaymentDate."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Pension calculation type."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The name of the Pension company because this affects the Pension percentage used in the calculation (only in contract situation).\nIt also needs to be shown in the reports."
    			},
    			pensionOldAgePayment: {
    				title: "pension old age payment",
    				descr: "If the Worker is from 53 to 62 years old, more of the Pension payment\nis deducted from the Worker side of the salary (less for the Employer).\nNOTE: People from 63 to 67 years pay the same amount as 17-52 years old."
    			},
    			pensionRegularEmployer: {
    				title: "pension regular employer",
    				descr: "The employer should have a Pension contract (not a temporary employer).\nIt does not necessarily mean that the user has a contract yet.\nNOTE: At the moment, this is not used for calculation - only in the Side costs visualizer tool."
    			},
    			taxAndSocialSecuritySelfPayment: {
    				title: "tax and social security self payment",
    				descr: "Tax and social security payments are paid directly by the Employer - not by Palkkaus.fi"
    			},
    			unemploymentSelfPayment: {
    				title: "unemployment self payment",
    				descr: "If true, the employer pays the Unemployment insurance directly to TVR.\nAs of writing, this is true for all companies and with calculations starting 1.8.2018,\nfalse for households or calculations before the date."
    			},
    			workerDateOfBirth: {
    				title: "worker date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerDateOfBirthAccuracy: {
    				title: "worker date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerSelfPayment: {
    				title: "worker self payment",
    				descr: "Worker salary and expense payments are paid directly by the Employer - not by Palkkaus.fi"
    			}
    		},
    		IncomesRegisterSettings: {
    			irReporting: {
    				title: "Kuukausittainen ilmoittaminen",
    				editLabel: "Kuukausittainen ilmoittaminen"
    			},
    			taxReferenceNumber: {
    				title: "Verorekisterinumero"
    			}
    		},
    		InsuranceContract: {
    			company: {
    				title: "Yhtiö",
    				editLabel: "Valitse yhtiö"
    			},
    			id: {
    				title: "Tunniste"
    			},
    			status: {
    				title: "Tila"
    			},
    			contractNumber: {
    				title: "Sopimusnumero"
    			},
    			startDate: {
    				title: "Alkamispäivä"
    			},
    			endDate: {
    				title: "Päättymispäivä"
    			},
    			isPartnerInsurance: {
    				title: "Yhteistyökumppani"
    			}
    		},
    		IUserObjectIndex: {
    			title: "i user object index",
    			descr: "Generic index for all User Object Containers.",
    			additionalInfo: {
    				title: "additional info",
    				descr: "A short detail describing the transaction.\nShould contain the relevant information for administration purposes."
    			},
    			businessAttributes: {
    				title: "business attributes",
    				descr: "Business attributes to include further information of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			containerGuid: {
    				title: "container guid",
    				descr: "Guid of the container.\nThis is not always unique within the index.\nHowver, within the container this is unique."
    			},
    			containerType: {
    				title: "container type",
    				descr: "The container type that is the source of this index item."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "Created at time"
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id for the object in 3rd party system. E.g. Filing code in MoneyTransfer."
    			},
    			flags: {
    				title: "flags",
    				descr: "Classifications for the object. First implementation supports BusinessEventType enumeration for values."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			handledAt: {
    				title: "handled at",
    				descr: "This is the recorded handling date for the object. E.g. when the calculation was paid to worker."
    			},
    			icon: {
    				title: "icon",
    				descr: "Icon for the purposes of listings."
    			},
    			id: {
    				title: "id",
    				descr: "Unique key for the object. Unique within an Index.\nIn the case of UserObject, this is [Guid]_[ContainerType]_[Owner].\nFor non-user objects, it should typically be just a Guid."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Messages (Error, Warning, OnHold, Note) that are active in the system.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the object is also the Partition key.\n Mandatory in most storage scenarios."
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Latest partner information."
    			},
    			payloadType: {
    				title: "payload type",
    				descr: "The business object type that is the source of this event."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			reference: {
    				title: "reference",
    				descr: "Reference information. E.g. Payment or MoneyTransfer reference number."
    			},
    			revenue: {
    				title: "revenue",
    				descr: "Estimated revenue that the transaction generated to Palkkaus.fi. Only add here the revenue, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Updated at time: Note that in typical implementations, this is the UserUpdatedAt time."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		IVerifiedAccount: {
    			title: "i verified account",
    			descr: "An account that is verified. Typically this means a digitally signed contract. This means that the account also has an Identity object. However, as the Identity object contains confidential information, it is not necessarily present in all method calls.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Pricing partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated"
    			}
    		},
    		Identity: {
    			title: "identity",
    			descr: "Identity is the official identity of an actor. This part is not freely editable as changing it requires signature or some other means. Identity is also often fetched from external sources such as govenment registries.",
    			contract: {
    				title: "contract",
    				descr: "The latest authorization contract that is the bases for this Identity"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name OR company name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			isPep: {
    				title: "is pep",
    				descr: "Indicates whether the principal user of the customer is a politically exposed person or not."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name\nAvailable only to real persons (as opposed to companies)."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the actor:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)"
    			},
    			primaryContact: {
    				title: "primary contact",
    				descr: "Mainly for company accounts, but in special cases for Persons,\nthis is the main Contact party and usually the signer for the account."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			}
    		},
    		ImportBatchInfo: {
    			title: "import batch info",
    			descr: "Contains the basic details and the current status of the batch phase.\nTypically this data is created and updated during the import session.",
    			createdAt: {
    				title: "created at",
    				descr: "Date and time when the batch was created."
    			},
    			id: {
    				title: "id",
    				descr: "Unique id for the batch."
    			},
    			name: {
    				title: "name",
    				descr: "User friendly name for the batch."
    			},
    			phase: {
    				title: "phase",
    				descr: "Current phase of the batch:\n - ```new``` A new batch. Contains no data.\n - ```read``` The data file (e.g. csv) has been read from file or clipboard.\n - ```mapped``` The data has been mapped to a supported tabular format.\n - ```validated``` The data has been validated.\n - ```imported``` The data has been imported to backend.\n - ```rejected``` The batch has been rejected manually."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of the current phase:\n - ```info``` Typically the default status for the phase which has not yet been done.\n - ```success``` The current phase has been successfully finished.\n - ```warning``` The current phase has been finished with warnings.\n - ```error``` The current phase contains errors."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Date and time when the batch was last updated."
    			}
    		},
    		ImportResult: {
    			title: "import result",
    			descr: "Result for importing data (CSV, Excel copy-paste etc.)",
    			data: {
    				title: "data",
    				descr: "Result data is an array of objects with properties specified as headers"
    			},
    			errors: {
    				title: "errors",
    				descr: "Errors as validation errors."
    			},
    			headers: {
    				title: "headers",
    				descr: "Headers for the import: These are also the properties for the JSON"
    			},
    			ignoredLines: {
    				title: "ignored lines",
    				descr: "Number of ignored lines (empty line or rem/annotated) in the import."
    			}
    		},
    		ImportableCrudApi: {
    			title: "importable crud api",
    			descr: "Interface for APIs which implement import/export functionality.",
    			constructor: {
    				title: "constructor",
    				descr: "Constructor creates a new CRUD api with given Ajax-implementation"
    			},
    			ajax: {
    				title: "ajax"
    			},
    			baseUrl: {
    				title: "base url",
    				descr: "Deriving classes should define the baseURl for the method. E.g. \"/calculations\"\nAjax implementations will add the beginning (e.g. \"https://test-api.salaxy.com/api/v02\")"
    			},
    			"delete": {
    				title: "delete",
    				descr: "Deletes an single item from the sotrage\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			deleteWorkflowEvent: {
    				title: "delete workflow event",
    				descr: "Deletes the worklflow item or items from the item.\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			getAll: {
    				title: "get all",
    				descr: "Gets all the items of a given type.\n\n**Returns**: A Promise with result data array.\n"
    			},
    			getBaseUrl: {
    				title: "get base url",
    				descr: "Returns the base url for this api."
    			},
    			getBlank: {
    				title: "get blank",
    				descr: "Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.\nThe basic idea is that all the child object properties should be non-null to avoid null reference exceptions.\nIn special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.\nStrings, dates and booleans may be null (false if bool), but typically just undefined."
    			},
    			getListItemMappers: {
    				title: "get list item mappers",
    				descr: "Available list item mappers for import/export"
    			},
    			getOData: {
    				title: "get o data",
    				descr: "Makes an OData query to the service's main OData method."
    			},
    			getODataBase: {
    				title: "get o data base",
    				descr: "Makes an OData query to the the given service URL."
    			},
    			getODataUrl: {
    				title: "get o data url",
    				descr: "Gets the OData URL for v03 compatible services"
    			},
    			getObjectMappers: {
    				title: "get object mappers",
    				descr: "Available object mappers for import/export"
    			},
    			getSingle: {
    				title: "get single",
    				descr: "Gets a single item based on identier\n\n**Returns**: A Promise with result data.\n"
    			},
    			getValidationResult: {
    				title: "get validation result",
    				descr: "Returns the validation object for the validated object."
    			},
    			save: {
    				title: "save",
    				descr: "Saves an item to the storage.\nIf id is null, this is add/insert. If id exists, this is update.\n\n**Returns**: A Promise with result data as saved to the storage (contains id, createAt, owner etc.).\n"
    			},
    			saveWorkflowEvent: {
    				title: "save workflow event",
    				descr: "Adds/updates the workflow event for the item.\n\n**Returns**: Workflow data for the item.\n"
    			},
    			validate: {
    				title: "validate",
    				descr: "Validates the item using a server side call.\n\n**Returns**: A Promise with result data containing validation property.\n"
    			}
    		},
    		IncomeTypeMetadata: {
    			title: "income type metadata",
    			descr: "Describes the properties and behavior of the Income types in Incomes register.",
    			accidentInsurance: {
    				title: "accident insurance",
    				descr: "If true Accident insurance payments are made for this income type by default."
    			},
    			calcGrouping: {
    				title: "calc grouping",
    				descr: "Grouping that is used in reporting etc."
    			},
    			code: {
    				title: "code",
    				descr: "Code in incomes register. Language versioned."
    			},
    			description: {
    				title: "description",
    				descr: "Description of the row.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			healthInsurance: {
    				title: "health insurance",
    				descr: "If true Health insurance payments are made for this income type by default."
    			},
    			insuranceInformationAllowed: {
    				title: "insurance information allowed",
    				descr: "If true the default values for insurances may be overriden for a single row."
    			},
    			isNegativeSupported: {
    				title: "is negative supported",
    				descr: "If true, teh row may have a negative value."
    			},
    			isSupported: {
    				title: "is supported",
    				descr: "If false, the row type is not supported in this version"
    			},
    			label: {
    				title: "label",
    				descr: "Language versioned label for the income type."
    			},
    			notSupportedError: {
    				title: "not supported error",
    				descr: "When isSupported is false, there is a describtive text that tells why the type is not supported. Text is always in Finnish."
    			},
    			paymentDefault: {
    				title: "payment default",
    				descr: "Default behavior for payment through Salaxy Customer Funds account"
    			},
    			pensionInsurance: {
    				title: "pension insurance",
    				descr: "If true Pension payments are made for this income type by default."
    			},
    			taxAndSidecostsDescr: {
    				title: "tax and sidecosts descr",
    				descr: "Description of how the row is taken into widthholding tax\nand how insurance payments (pension, unemployment, accident and health care) are made.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			taxDefault: {
    				title: "tax default",
    				descr: "Describes the behavior of this row in calculating widthholding tax."
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "If true Unemployment insurance payments are made for this income type by default."
    			}
    		},
    		InsuranceProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer insurance. Currently, this is the mandatory employer insurance only.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPartnerInsurance."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "Company for the current insurance"
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		InsuranceSettings: {
    			title: "insurance settings",
    			company: {
    				title: "company",
    				descr: "Company for the current insurance"
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the insurance pension"
    			}
    		},
    		Invoice: {
    			title: "invoice",
    			descr: "Invoice for the pending payment in the business system. For example, a salary gross payment created for the salary calculation and delivered as an e-invoice to the employer. For example, a salary net payment, created for the employer which is self paying the net payment to the worker.",
    			content: {
    				title: "content",
    				descr: "Invoice data in XML or json format."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			header: {
    				title: "header",
    				descr: "Invoice header details."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Invoice payer details."
    			},
    			pdf: {
    				title: "pdf",
    				descr: "Invoice in pdf format."
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Invoice recipient details."
    			},
    			rows: {
    				title: "rows",
    				descr: "List of invoice rows."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		InvoiceData: {
    			title: "invoice data",
    			descr: "Invoice specific business data for index.",
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			}
    		},
    		InvoiceHeader: {
    			title: "invoice header",
    			descr: "Header for the invoice",
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business object ids (typically calculations) on which the payment is based on."
    			},
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			externalId: {
    				title: "external id",
    				descr: "The external ID is used for matching the executed payment from the 3rd party system\nto the pending payment in the Palkkaus system."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this invoice is done.\nFor salaries, this is salary date, for monthly / quarterly payments, this is the first day of period."
    			},
    			message: {
    				title: "message",
    				descr: "The free text that is displayed / sent with the invoice.\nMay have multiple lines."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Payment reference (e.g. RF0710010831).\nPlease note that in salary payment scenarios, this is not necessarily unique."
    			},
    			status: {
    				title: "status",
    				descr: "The current notified status of the invoice in external system."
    			},
    			total: {
    				title: "total",
    				descr: "Total payable amount for the invoice including VAT."
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Total amount for the invoice excluding VAT."
    			},
    			type: {
    				title: "type",
    				descr: "Invoice type."
    			}
    		},
    		InvoiceListItem: {
    			title: "invoice list item",
    			descr: "Represents a single item in a list of invoices in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		InvoiceParty: {
    			title: "invoice party",
    			descr: "Party in an Varma Invoice: Either Payer or Recipient.",
    			companyId: {
    				title: "company id",
    				descr: "Finnish Company ID (Y-tunnus)."
    			},
    			companyName: {
    				title: "company name",
    				descr: "Organisation name"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information"
    			},
    			contactPersonName: {
    				title: "contact person name",
    				descr: "Contact person Full name"
    			}
    		},
    		InvoicePayer: {
    			title: "invoice payer",
    			descr: "Payment payer party of the invoice.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the party."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the party."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official identifier of the party:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)."
    			},
    			partnerKey: {
    				title: "partner key",
    				descr: "Optional ID for identifying the party in the third party system.\nThis is the same partner key that is given as part of Single Sign-On process."
    			}
    		},
    		InvoiceRecipient: {
    			title: "invoice recipient",
    			descr: "Payment receiver party of the invoice.",
    			beneficiary: {
    				title: "beneficiary",
    				descr: "The ultimate beneficiary of the money:\nThis may be a Palkkaus.fi Worker Account (Avatar id is not null)\nor some other entity: Tax authorities, pension company etc. (Avatar id is null).\nTypically beneficiary is the same as Iban, OfficialId, FullName and Contact in this object,\nbut this is not true in edge cases (e.g. customer funds account)."
    			},
    			contact: {
    				title: "contact",
    				descr: "Optional contact info. Not available for all recipients."
    			},
    			fullName: {
    				title: "full name",
    				descr: "The full name of the payment receiver for payment purposes."
    			},
    			iban: {
    				title: "iban",
    				descr: "The International Bank Account Number of the receiver."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the party: Finnish Personal Id (HETU) for a Person or Company Id (Y-tunnus) for a company or association.\nWARNING: This is currently needed for Finvoice only (https://www.finanssiala.fi/finvoice/Sivut/default.aspx). As such we consider removing it from persons.\nPlease inform if you use this property!"
    			}
    		},
    		InvoiceRow: {
    			title: "invoice row",
    			descr: "Row for the invoice",
    			accountingAccountNumber: {
    				title: "accounting account number",
    				descr: "Suggested account number (RowNormalProposedAccountIdentifier)."
    			},
    			accountingCode: {
    				title: "accounting code",
    				descr: "Textual code for accounting mapping (RowProposedAccountText, Raportointikoodi).\nIn Finvoice: \"Accounting information agreed between seller and buyer.\""
    			},
    			articleId: {
    				title: "article id",
    				descr: "Article Identifier is given by seller, EAN etc.\nIn the case of Salaxy salaries, this is CalculationRowType."
    			},
    			articleName: {
    				title: "article name",
    				descr: "Name of the product or service.\nIn the case of Salaxy salaries, this is language versioned text of CalculationRowType."
    			},
    			businessObject: {
    				title: "business object",
    				descr: "Business object id (typically calculation) on which the row is based on."
    			},
    			count: {
    				title: "count",
    				descr: "Count (DeliveredQuantity in Finvoice) is the quantity of product or service."
    			},
    			message: {
    				title: "message",
    				descr: "Message (RowFreeText in Finvoice) is the message is mainly designed for accounting systems."
    			},
    			price: {
    				title: "price",
    				descr: "Price including VAT (UnitPriceVatIncludedAmount, Yksikköhinta verollinen)"
    			},
    			priceExVat: {
    				title: "price ex vat",
    				descr: "Price of one product or service without Vat (UnitPriceAmount, Yksikköhinta veroton).\nYksikköhinta voidaan ilmoittaa myös verollisena UnitPriceVatIncludedAmount- elementissä laskutuskäytännön ja verottajan ALV-ohjeistuksen mukaan."
    			},
    			total: {
    				title: "total",
    				descr: "Row total with VAT (RowAmount, Yhteensä verollinen)"
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Row total excluding VAT (RowVatExcludedAmount, Yhteensä veroton)"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Vat percent related to the product or service (RowVatRatePercent, Alv%)"
    			}
    		},
    		InvoiceSettings: {
    			title: "invoice settings",
    			descr: "Settings related to invoices that are sent to customers.",
    			eInvoiceIntermediator: {
    				title: "Välittäjä",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "Verkkolaskuosoite",
    				descr: "Receiver's address for the e-invoice."
    			},
    			sepaBankPartyId: {
    				title: "SEPA-maksatustunnus"
    			},
    			ibanNumber: {
    				title: "Tilinumero"
    			}
    		},
    		InvoiceStatusNotification: {
    			title: "invoice status notification",
    			descr: "Contents for the invoice status notification message.",
    			amount: {
    				title: "amount",
    				descr: "Required realized amount of the transaction."
    			},
    			date: {
    				title: "date",
    				descr: "Required Date for the transaction (booking date)."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id can be set in the notification for the invoice for later messaging.\nIf the notification message contains an External Id AND regular Id, the External Id is updated in the Salaxy database.\nIf the notification contains External Id, but NO regular Id, an invoice is fetched based on External Id.\nIn the latter case, a non-existent Id or multiple invoices with the same External Id will throw an exception and the entire update process is canceled."
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID for the invoice in Salaxy system.\nIf omitted, the resolving to an invoice will be made using external id."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			receiverFullName: {
    				title: "receiver full name",
    				descr: "Optional full name of the payment receiver. Should be sent only if the user or process has the possibility of changing the name."
    			},
    			receiverIban: {
    				title: "receiver iban",
    				descr: "Optional bank account IBAN of the receiver. Should be sent only if the user or process has the possibility of changing the IBAN."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Optional payment reference number. Should be sent only if the user or process has the possibility of changing or removing the reference number.\nChanging the reference number will typically lead to an error in the back office process that needs to be manually handled."
    			},
    			status: {
    				title: "status",
    				descr: "New status for the invoice. Required main payload of the call."
    			}
    		},
    		IrDetails: {
    			title: "ir details",
    			descr: "Reporting details to Incomes registry (Tulorekisteri).",
    			carBenefit: {
    				title: "car benefit",
    				descr: "Provides the additional data needed for car benefit."
    			},
    			code: {
    				title: "code",
    				descr: "The number code in Incomes Register."
    			},
    			dailyAllowance: {
    				title: "daily allowance",
    				descr: "Daily allowance code."
    			},
    			flags: {
    				title: "flags",
    				descr: "Boolean flags that can be set on transaction."
    			},
    			insuranceExceptions: {
    				title: "insurance exceptions",
    				descr: "Defines the exceptions to Insurance / Sidecost defaults that are set by the row type."
    			},
    			mealBenefitIsTaxValue: {
    				title: "meal benefit is tax value",
    				descr: "Set this to true if meak benefit is of tax value - false if it si not.\nThe value should be null if the row is not mealBenefit."
    			}
    		},
    		IrIncomeTypeUsecase: {
    			title: "ir income type usecase",
    			descr: "Handles the Calculation of CalculationRowType.IrIncomeType to IrRows",
    			irData: {
    				title: "ir data",
    				descr: "Reporting details to Incomes registry (Tulorekisteri)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		IrRow: {
    			title: "ir row",
    			descr: "Business level modeling corresponding to WageReportsToIRTypes Transaction. Also contains additional data for accounting and integration purposes.",
    			accounting: {
    				title: "accounting",
    				descr: "Accounting related data for the row."
    			},
    			calcData: {
    				title: "calc data",
    				descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR)."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row. Default is one."
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			earningPeriod: {
    				title: "earning period",
    				descr: "Earnings period if differnet than the report period.\nNote that this property may be (and by default is) null."
    			},
    			irData: {
    				title: "ir data",
    				descr: "Details for the National Incomes registry"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row."
    			},
    			total: {
    				title: "total",
    				descr: "Simple multiplication: Count * Price - no other logic. Not used as Input."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the row - basically describes the source for this row:\nEither Manual (default) or Usecase logic, which may be legacy / API v02 or new / API v03."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. If specified as Undefined, set by the server-logic based on row type etc."
    			}
    		},
    		IrRowCalculationProperties: {
    			title: "ir row calculation properties",
    			descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR).",
    			behavior: {
    				title: "behavior",
    				descr: "Defines the behavior of the row in Salaxy calculation process."
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Grouping for reports etc."
    			}
    		},
    		JobProfile: {
    			title: "job profile",
    			backgroundImage: {
    				title: "background image",
    				descr: "Background image for the profile page"
    			},
    			cities: {
    				title: "cities",
    				descr: "A collection of known city IDs.\nQuery the interface for city IDs"
    			},
    			description: {
    				title: "description",
    				descr: "Long description: This field accepts MarkDown, potentially also HTML in the future (at the moment HTML may create an error)."
    			},
    			jobDetail: {
    				title: "job detail",
    				descr: "Detailed description of the Job"
    			},
    			jobTitle: {
    				title: "job title",
    				descr: "Title for the Job"
    			},
    			jobs: {
    				title: "jobs",
    				descr: "A collection of job ID's that this work or worker belongs"
    			},
    			links: {
    				title: "links",
    				descr: "External links to web sites, social media pages etc."
    			},
    			location: {
    				title: "location",
    				descr: "Free text for location of work or worker"
    			},
    			salaryMax: {
    				title: "salary max",
    				descr: "Max salary for the given salary period"
    			},
    			salaryMin: {
    				title: "salary min",
    				descr: "Minimum salary for the given salary period"
    			},
    			salaryPeriod: {
    				title: "salary period",
    				descr: "Period for which the Salary period is set."
    			}
    		},
    		LedgerAccount: {
    			title: "ledger account",
    			descr: "Represents a single accounting ledger.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			entries: {
    				title: "entries",
    				descr: "Legder entries"
    			},
    			saldo: {
    				title: "saldo",
    				descr: "Current saldo for the ledger."
    			}
    		},
    		LedgerEntry: {
    			title: "ledger entry",
    			descr: "Represents a single ledger entry.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			amount: {
    				title: "amount",
    				descr: "Posted amount."
    			},
    			description: {
    				title: "description",
    				descr: "Entry description."
    			},
    			dimension: {
    				title: "dimension",
    				descr: "Dimension data."
    			},
    			entryDate: {
    				title: "entry date",
    				descr: "Entry date."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the entry."
    			},
    			isDebit: {
    				title: "is debit",
    				descr: "Debit indicator."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "VAT percent."
    			}
    		},
    		"Map": {
    			title: "map",
    			descr: "Map between a hierarchical Salaxy object and a row in a table/dataset."
    		},
    		MapSelector: {
    			title: {
    				title: "title",
    				descr: "Title of the property. If null, path is used with reflection."
    			},
    			descr: {
    				title: "descr",
    				descr: "Description of the property. If null, path is used with reflection."
    			},
    			fromObject: {
    				title: "from object",
    				descr: "Function that defines how the row value(s) is derived from the object, typically in export.\nIf null, this is defined by path."
    			},
    			fromRow: {
    				title: "from row",
    				descr: "Function that sets the value from row back to object, typically in import.\nIf null, this is defined by path."
    			},
    			path: {
    				title: "path",
    				descr: "Defines source property as a simple object path if possible.\nIf a more complex logic is needed, you can define get and set functions below.\nThe property is also used in generating documentation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the property. If null, path is used with reflection."
    			}
    		},
    		MarketingSettings: {
    			title: "marketing settings",
    			descr: "Marketing related settings which may affect the pricing. E.g. campaigns, customer groups etc.",
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing."
    			}
    		},
    		MealBenefitUsecase: {
    			title: "meal benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.MealBenefit to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isTaxValue: {
    				title: "is tax value",
    				descr: "If true, the deduction from Worker salary corresponds to taxable value."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			taxablePrice: {
    				title: "taxable price",
    				descr: "The taxable price calculated by the server."
    			}
    		},
    		Measurement: {
    			title: "measurement",
    			descr: "Measurement value.",
    			tag: {
    				title: "tag",
    				descr: "Tag for the measurement."
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Time when the measurement was recorded."
    			},
    			value: {
    				title: "value",
    				descr: "Actual measurement value."
    			}
    		},
    		Message: {
    			title: "message",
    			descr: "A message within a messaging thread.",
    			content: {
    				title: "content",
    				descr: "Textual content: If message, this is the content text.\nIf other type, short title / description typically shown in the title bar."
    			},
    			contentData: {
    				title: "content data",
    				descr: "Other misc. data that is used in displaying the content in the immediate message view.\nDo not include the entire object here. Just the immediately available data."
    			},
    			contentId: {
    				title: "content id",
    				descr: "Identifier of the external content.\nBlobFile Id for attachments, Calculation ID for calculations etc."
    			},
    			fromParty: {
    				title: "from party",
    				descr: "The party / type of the message sender."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the message."
    			},
    			preview: {
    				title: "preview",
    				descr: "Content type specific text for preview.\nThis may be URI, ID or boolean.\nFor BlobFile, it is currently the BlobFile Guid (the same as ContentId) or null,\nbut in the future, the field may also contain an URL without that being a breaking change!"
    			},
    			readByOtherParty: {
    				title: "read by other party",
    				descr: "If set, specifies the time when the message has been read by the other party."
    			},
    			readByOwner: {
    				title: "read by owner",
    				descr: "If set, specifies the time when the message has been read by the thread owner."
    			},
    			time: {
    				title: "time",
    				descr: "Time for the message"
    			},
    			type: {
    				title: "type",
    				descr: "Content type of the message"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validates the object that is the payload of the message."
    			}
    		},
    		MessageData: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Message thread specific business data for index.",
    			otherPartyLastMessage: {
    				title: "other party last message",
    				descr: "Last message that was created by the other party"
    			},
    			otherPartyUnreadCount: {
    				title: "other party unread count",
    				descr: "Number of messages that have been read by the Other party and have not been read by the owner."
    			},
    			ownerLastMessage: {
    				title: "owner last message",
    				descr: "Last message that was was created by the owner"
    			},
    			ownerUnreadCount: {
    				title: "owner unread count",
    				descr: "Number of messages by Owner that have not been read by the other party"
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			}
    		},
    		MessageListItem: {
    			title: "message list item",
    			descr: "Represents a single item in a list of message threads in the user interface. It contains header data of the message thread and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		MessageThread: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Messages thread groups togehter a set of related messages and/or attachments.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			messages: {
    				title: "messages",
    				descr: "List of messages and attachments in this thread"
    			},
    			otherParty: {
    				title: "other party",
    				descr: "Palkkaus.fi id (IBAN) of the other party.\nThe other party is the second entity(besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party is the second entity (besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyWorkflow: {
    				title: "other party workflow",
    				descr: "The workflow from the other party point-of-view."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "The owner of the thread: The entity that has the right to delete or move the thread.\nThis may or may not be the originator the message thread.\nThe owner is typically Company or Household, in some cases Worker person.\nIt should NOT be partner or person from a company."
    			},
    			ownerWorkflow: {
    				title: "owner workflow",
    				descr: "The workflow from the Owner point-of-view."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		MessageThreadWorkflow: {
    			title: "message thread workflow",
    			descr: "Defines workflow actions for a message thread.",
    			lastMessageText: {
    				title: "last message text",
    				descr: "Text for the last message (written by this party). This is for preview purposes: May be shortened."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The salary date for the discussion.\nOwner's and other party's are typically the same, but they may differ:\ne.g. one may be the requested / estimated date and the other the first possible date if the requested is unrealistic."
    			},
    			unreadCount: {
    				title: "unread count",
    				descr: "Number of messages that this party has posted that have not been read by the other party."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Time of the last message posted by this party."
    			}
    		},
    		MonthlyHolidayAccrual: {
    			title: "monthly holiday accrual",
    			descr: "Holiday Accrual for a calendar month in releation to a calculation.",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of absence days used in the calculation."
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Holiday accrual for this month.\nNote that -1 is indication that accrual should not be updated for this particylar month.\nThere is no possiblity for negative accrual."
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			workDays: {
    				title: "work days",
    				descr: "Work days in determining the accrual for the month in 14 days rule."
    			},
    			workHours: {
    				title: "work hours",
    				descr: "Work hours in determining the accrual fot the month in 35 hour rule."
    			}
    		},
    		NonProfitOrgUsecase: {
    			title: "non profit org usecase",
    			descr: "Handles the Calculation of CalculationRowType.NonProfitOrg to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		OAuthMessage: {
    			title: "o auth message",
    			descr: "Based on standard https://tools.ietf.org/html/rfc6749 .",
    			access_token: {
    				title: "access_token",
    				descr: "The access token issued by the authorization server."
    			},
    			assertion: {
    				title: "assertion",
    				descr: "The value of the \"assertion\" parameter MUST contain a single JWT."
    			},
    			client_id: {
    				title: "client_id",
    				descr: "The client identifier issued to the client during the registration process."
    			},
    			client_secret: {
    				title: "client_secret",
    				descr: "The client secret. The client MAY omit the parameter if the client secret is an empty string."
    			},
    			code: {
    				title: "code",
    				descr: "The authorization code received from the authorization server."
    			},
    			connection: {
    				title: "connection",
    				descr: "Auth0 connection"
    			},
    			error: {
    				title: "error",
    				descr: "A single ASCII [USASCII] error code."
    			},
    			error_description: {
    				title: "error_description",
    				descr: "Human-readable ASCII [USASCII] text providing additional information, used to assist the client developer in understanding the error that occurred."
    			},
    			error_uri: {
    				title: "error_uri",
    				descr: "A URI identifying a human-readable web page with information about the error, used to provide the client developer with additional information about the error."
    			},
    			expires_in: {
    				title: "expires_in",
    				descr: "The lifetime in seconds of the access token.  For example, the value \"3600\" denotes that the access token will expire in one hour from the time the response was generated. If omitted, the authorization server SHOULD provide the expiration time via other means or document the default value."
    			},
    			grant_type: {
    				title: "grant_type",
    				descr: "Requested grant type."
    			},
    			id_token: {
    				title: "id_token",
    				descr: "The id token issued by the authorization server."
    			},
    			password: {
    				title: "password",
    				descr: "The resource owner password."
    			},
    			redirect_uri: {
    				title: "redirect_uri",
    				descr: "After completing its interaction with the resource owner, the authorization server directs the resource owner's user-agent back to the client."
    			},
    			refresh_token: {
    				title: "refresh_token",
    				descr: "The refresh token, which can be used to obtain new access tokens."
    			},
    			response_type: {
    				title: "response_type",
    				descr: "The value MUST be one of \"code\" for requesting an authorization code, \"token\" for  requesting an access token(implicit grant), or a registered extension value."
    			},
    			salaxy_language: {
    				title: "salaxy_language",
    				descr: "User interface language."
    			},
    			salaxy_onboarding_uri: {
    				title: "salaxy_onboarding_uri",
    				descr: "Salaxy specific onboarding uri."
    			},
    			salaxy_role: {
    				title: "salaxy_role",
    				descr: "Users role."
    			},
    			scope: {
    				title: "scope",
    				descr: "The value of the scope parameter is expressed as a list of space-delimited, case-sensitive strings.The strings are defined by the authorization server."
    			},
    			state: {
    				title: "state",
    				descr: "An opaque value used by the client to maintain state between the request and callback.The authorization server includes this value when redirecting the user-agent back to the client. The parameter SHOULD be used for preventing cross-site request forgery."
    			},
    			token_type: {
    				title: "token_type",
    				descr: "The type of the token issued."
    			},
    			username: {
    				title: "username",
    				descr: "The resource owner username."
    			}
    		},
    		ODataQueryOptions: {
    			title: "o data query options",
    			descr: "Defines the query options of OData",
    			$count: {
    				title: "$count",
    				descr: "Tells the server to include the total count of matching entities in the response. (Useful for server-side paging.)"
    			},
    			$expand: {
    				title: "$expand",
    				descr: "Expands related entities inline. At the moment, this typically not implemented."
    			},
    			$filter: {
    				title: "$filter",
    				descr: "Filters the results, based on a Boolean condition."
    			},
    			$orderby: {
    				title: "$orderby",
    				descr: "Sorts the results."
    			},
    			$search: {
    				title: "$search",
    				descr: "Performs free-text search using the in-built logic in the API."
    			},
    			$select: {
    				title: "$select",
    				descr: "Selects which properties to include in the response."
    			},
    			$skip: {
    				title: "$skip",
    				descr: "Skips the first n results."
    			},
    			$top: {
    				title: "$top",
    				descr: "Returns only the first n the results."
    			},
    			anon: {
    				title: "anon",
    				descr: "If true, perfoms the query also in anonymous mode\n(no token, token expired or invalid token).\nDefault is that the query is done only if user has a valid token."
    			}
    		},
    		ODataResult: {
    			title: "o data result",
    			descr: "Result from OData query",
    			count: {
    				title: "count",
    				descr: "Total count of the items (server-side/storage) if updated in the previous query.\nNote that this may be null if the count is not fetched."
    			},
    			items: {
    				title: "items",
    				descr: "Actual results array"
    			},
    			nextPageLink: {
    				title: "next page link",
    				descr: "Next page link"
    			}
    		},
    		Occupation: {
    			title: "occupation",
    			descr: "Occupation according to Statistics Finland.\nUsed for insurance pricing and reporting.",
    			code: {
    				title: "code",
    				descr: "Statistics Finland code. There may be duplicates with different id."
    			},
    			freeText: {
    				title: "free text",
    				descr: "Free text\nTODO: Do we need to expose this? Why?"
    			},
    			id: {
    				title: "id",
    				descr: "Salaxy identifier for the Occupation. Begins with code, but may have a suffix."
    			},
    			keyword: {
    				title: "keyword",
    				descr: "Keywords for searching etc."
    			},
    			label: {
    				title: "label",
    				descr: "Label"
    			},
    			lang: {
    				title: "lang",
    				descr: "Language for the entry"
    			},
    			search: {
    				title: "search",
    				descr: "Lower case text that contains the id, label and potentially other text that should match in searches."
    			}
    		},
    		Onboarding: {
    			title: "onboarding",
    			descr: "Provides the data for onboarding process for a company. Onboarding is the process where Salaxy gathers sufficient information to at least create an acccount, often even more so that the user can pay a salaary with the account.",
    			accountType: {
    				title: "account type",
    				descr: "Type of the account to be created."
    			},
    			appState: {
    				title: "app state",
    				descr: "Custom state object that is guaranteed to survive the round-trip in the onboarding process.\nCalling application to the onboarding may pass here any data that it needs after the onboarding process is ready and user has been authenticated."
    			},
    			company: {
    				title: "company",
    				descr: "Input that is needed for creating a company account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			credentials: {
    				title: "credentials"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			person: {
    				title: "person",
    				descr: "Input that is needed for creating a personal account.\nThis may be\n1) the principal account if the account is created for 1a) Worker or 1b) Household or\n2) creator owner if the account is created for a Company."
    			},
    			products: {
    				title: "products",
    				descr: "Possibility to define products.\nThese can only be defined to one account as part of the onboarding process:\nIf company account is created, the products are always for Company, not for the Person =&gt;\nPerson will have default products in that case and they need to be edited in a separate process once the accounts have been created."
    			},
    			signature: {
    				title: "signature",
    				descr: "Input and post data for digital signature."
    			},
    			status: {
    				title: "status",
    				descr: "The server-side status of the onboarding process.\nNOTE that there is a more relaxed user interface-level state in ui.state and ui.stateFullText\nthat can be used in onboarding user interfaces."
    			},
    			ui: {
    				title: "ui",
    				descr: "User interface related data: Mainly the state of the UI."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation status and errors."
    			}
    		},
    		OnboardingContactVerification: {
    			title: "onboarding contact verification",
    			descr: "Contact verification object for single message type.",
    			pin: {
    				title: "pin",
    				descr: "Given pin code."
    			}
    		},
    		OnboardingContactVerifications: {
    			title: "onboarding contact verifications",
    			descr: "Contact verification object for onboarding.",
    			email: {
    				title: "email",
    				descr: "Verification for email contact."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Verification for telephone contact."
    			}
    		},
    		OnboardingCredentials: {
    			title: "onboarding credentials",
    			descr: "Stores information about the credentials that were used in the Onboarding process.",
    			credentialId: {
    				title: "credential id",
    				descr: "Identifier of the credentials (UserAccount Guid in the database)."
    			},
    			partnerId: {
    				title: "partner id",
    				descr: "The partner that initiated the Onboarding process."
    			}
    		},
    		OnboardingUIState: {
    			title: "onboarding u i state",
    			descr: "Stores data for the onboarding user interface.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "URL where the user should be redirected back to if the process is canceled or there is an urecoverable error."
    			},
    			invitationKey: {
    				title: "invitation key",
    				descr: "Salaxy worker invitation key."
    			},
    			oAuthResponseType: {
    				title: "o auth response type",
    				descr: "Response type to OAuth -request"
    			},
    			state: {
    				title: "state",
    				descr: "State of the user interface. Typically an enumeration that stores where the UI was. E.g. Wizard step."
    			},
    			stateFullText: {
    				title: "state full text",
    				descr: "Full description of the user interface state for admin and analyses purposes."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "URL where the user should be redirected back to when the onboarding process is complete."
    			}
    		},
    		OtherCompensationUsecase: {
    			title: "other compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.OtherCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Ownership: {
    			title: "ownership",
    			isRequiredTracking: {
    				title: "is required tracking",
    				descr: "If true, authorities require tracking the shareholders.\nIn Finland, with current legistlation, this is if a shareholder has over 25% of the shares."
    			},
    			shareHolders: {
    				title: "share holders",
    				descr: "List of shareholders for the company"
    			}
    		},
    		PageResultOfApiListItem: {
    			title: "page result of api list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfArticle: {
    			title: "page result of article",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfCalculationListItem: {
    			title: "page result of calculation list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfEmploymentListItem: {
    			title: "page result of employment list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfFileListItem: {
    			title: "page result of file list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfInvoiceListItem: {
    			title: "page result of invoice list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfMessageListItem: {
    			title: "page result of message list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfPayrollListItem: {
    			title: "page result of payroll list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfSampleListItem: {
    			title: "page result of sample list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfTaxcardListItem: {
    			title: "page result of taxcard list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfWorkerListItem: {
    			title: "page result of worker list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PartnerAccounting: {
    			title: "partner accounting",
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, partner settings override company settings."
    			},
    			targetIds: {
    				title: "target ids",
    				descr: "Available accounting targets."
    			}
    		},
    		PartnerBranding: {
    			title: "partner branding",
    			descr: "Partner branding settings: logo, css etc.",
    			css: {
    				title: "css",
    				descr: "Css path for the primary partner."
    			},
    			logo: {
    				title: "logo",
    				descr: "Url to partner logo."
    			},
    			text: {
    				title: "text",
    				descr: "Long marketing text."
    			},
    			url: {
    				title: "url",
    				descr: "Marketing url."
    			}
    		},
    		PartnerCompanyAccountInfo: {
    			title: "partner company account info",
    			descr: "Information about a Customer account for a Partner",
    			accountId: {
    				title: "account id",
    				descr: "Account Id"
    			},
    			accountIdFormatted: {
    				title: "account id formatted",
    				descr: "Account Id fomratted for dispaly to end user"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for display information"
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If true, the current user (the partner) is authorized to access the account"
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has a verified owner."
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The partner account ID to which the access is currently restricted."
    			}
    		},
    		PartnerMessaging: {
    			title: "partner messaging",
    			descr: "Messaging settings for the partner.",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the messaging is switched on for this partner."
    			}
    		},
    		PartnerService: {
    			title: "partner service",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the calculation data delivery by forms is switched on for this partner."
    			}
    		},
    		PartnerServiceModel: {
    			title: "Palvelumalli",
    			"package": {
    				title: "Paketti"
    			},
    			features: {
    				title: "Aktivoidut palvelut"
    			},
    			pricing: {
    				title: "Hinnoittelu"
    			},
    			branding: {
    				title: "Brändi"
    			},
    			site: {
    				title: "Sivuston ominaisuudet"
    			}
    		},
    		PartnerWebSite: {
    			title: "partner web site",
    			descr: "Stores information about the web site that is used for customer (employer) to access the customer web site.",
    			url: {
    				title: "url",
    				descr: "The main URL that is displayed.\nDefault is https://www.palkkaus.fi/company"
    			}
    		},
    		Payer: {
    			title: "payer",
    			descr: "The paying party",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		Payment: {
    			title: "payment",
    			descr: "Payment object requested to be paid for completing the underlying business objects.",
    			amount: {
    				title: "amount",
    				descr: "Amount requested to be paid."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business objects (calculations etc.) on which the payment is based on."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payment."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			message: {
    				title: "message",
    				descr: "Message for the payment."
    			},
    			method: {
    				title: "method",
    				descr: "Payment method."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer as it would be on the invoice"
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Recipient as it would be on the invoice"
    			},
    			reference: {
    				title: "reference",
    				descr: "Bank reference number/string. Will be generated by the system automatically."
    			},
    			request: {
    				title: "request",
    				descr: "Request specific payment parameters."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the payment."
    			},
    			statusText: {
    				title: "status text",
    				descr: "Status text giving more information about the status."
    			},
    			type: {
    				title: "type",
    				descr: "Payment type."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PaymentChannelSettings: {
    			channel: {
    				title: "Maksukanava"
    			},
    			isEnabled: {
    				title: "Valittu"
    			},
    			isAvailable: {
    				title: "Valittavissa"
    			},
    			data: {
    				title: "Data"
    			},
    			accountingTargetId: {
    				title: "Kohteen tunniste"
    			}
    		},
    		PaymentRequestParameters: {
    			title: "payment request parameters",
    			descr: "Contains request specific parameters for payment.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "Return url for failed / cancelled payment."
    			},
    			paymentStartedUrl: {
    				title: "payment started url",
    				descr: "If this URL is set, the payment allows aynchronous payment methods\nsuch as payment with invoice (viitelasku).\nThis means that the payment UI returns in status InProgress and\npayment is finalized later (may take hours, days or even be canceled)."
    			},
    			sendEmails: {
    				title: "send emails",
    				descr: "Indicates whether to send emails according to workflow after payment.\nTODO: This should be retrieved from the partner profile etc."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "Return url for successful payment."
    			}
    		},
    		PaymentSettings: {
    			title: "payment settings",
    			descr: "Payment related settings",
    			channels: {
    				title: "Maksukanavat"
    			},
    			customerFunds: {
    				title: "Asiakasvara-asetukset",
    				descr: "Settings related to payment methods that use the customer funds account (gross payment)."
    			},
    			defaultChannel: {
    				title: "Oletuskanava"
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			invoice: {
    				title: "Verkkolaskuasetukset",
    				descr: "Settings related to invoices: Either salary payment or other invoicing."
    			},
    			pricing: {
    				title: "pricing",
    				descr: "The pricing partner (accounant or software) that defines the pricing of the service.\nThis data is typically read-only in normal settings view."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		Payroll03Info: {
    			title: "payroll03 info",
    			descr: "Provides Read-only information about the Payroll In the case of non-paid Payrolls, this is the latest save/validation. In the case of paid Payrolls, this is the status of the Payroll when it was paid.",
    			calcCount: {
    				title: "calc count",
    				descr: "Count of calculations in this Payroll"
    			},
    			date: {
    				title: "date",
    				descr: "Date that the Info was last updated."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payroll. Date when the payment should be made to achieve SalaryDate."
    			},
    			fees: {
    				title: "fees",
    				descr: "Total Palkkaus.fi fees for the Payroll"
    			},
    			isReadyForPayment: {
    				title: "is ready for payment",
    				descr: "If true, all calculations have been validated and they are valid for payment.\nIf false, validation has not been done or some calculations are not valid for payment."
    			},
    			paymentDate: {
    				title: "payment date",
    				descr: "Paid at date for the payroll. (Date of the employer's payment)."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "The reference number of the payment after payment is made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			status: {
    				title: "status",
    				descr: "Status for the payroll and its calculations in the payment Workflow."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment for the Payroll."
    			}
    		},
    		PayrollDetails: {
    			title: "payroll details",
    			descr: "New version 03 of Payroll. Payroll (Palkkalista) is a list of employees who receive salary or wages from a particular organization. Typical usecase is that a a company has e.g. a monthly salary list that is paid at the end of month. For next month, a copy is then made from the latest list and the copy is potentially modified with the changes of that particular month.",
    			calcs: {
    				title: "calcs",
    				descr: "Snapshots of the calculations based on the ID's specified in Info"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing Payroll lists.\nIn practice, a usecase is either a user interface or a microservice that provides a Payroll processing\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of the Payroll."
    			},
    			validations: {
    				title: "validations",
    				descr: "When saved or validated, provides a validation object for each calculation:\nWhether the calculations are ready for payment."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		PayrollInput: {
    			title: {
    				title: "title",
    				descr: "Title that can be specified by the end user."
    			},
    			descr: "Input that user defines about the Payroll",
    			calculations: {
    				title: "calculations",
    				descr: "List of calculations that are attached to this calculation."
    			},
    			noUpdateFromEmployment: {
    				title: "no update from employment",
    				descr: "If true, the calculation rows are not updated from employment when the Payroll is copied.\nDefault is that the rows and other data is updated in copy / create new etc."
    			},
    			period: {
    				title: "period",
    				descr: "The Default Salary period for the Payroll.\nThis may be overriden for an individual calculation."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "What types of wages are paid with this payroll.\nMay affect the rendering of the user interface (e.g. for     holidays, hours and absences).\nAlso affects which Workers are show in the add / link dialog by default."
    			}
    		},
    		PayrollListItem: {
    			title: "payroll list item",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PensionContract: {
    			title: "Työeläkevakuutus",
    			descr: "",
    			company: {
    				title: "Yhtiö",
    				editLabel: "Valitse yhtiö"
    			},
    			id: {
    				title: "Tunniste"
    			},
    			status: {
    				title: "Tila"
    			},
    			contractNumber: {
    				title: "Sopimusnumero"
    			},
    			startDate: {
    				title: "Alkamispäivä"
    			},
    			endDate: {
    				title: "Päättymispäivä"
    			}
    		},
    		PensionEntrepreneurProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by TempContractMarker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			tempContractMarker: {
    				title: "temp contract marker",
    				descr: "If true, user wants to order a YEL-contract."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Pension for Workers: The payments and monthly reporting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPensionSelfHandling."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isPendingContract: {
    				title: "is pending contract",
    				descr: "If true, there is a new pending contract."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The pension company where the user has a contract."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Number for the contract."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionSettings: {
    			title: "pension settings",
    			company: {
    				title: "company",
    				descr: "The pension company where the user has a contract."
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Number for the contract."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the pension insurance."
    			}
    		},
    		PeriodRecalculationData: {
    			title: "period recalculation data",
    			descr: "Data that is needed for determening PeriodRecalculationParams and thus at the end recalculating holidays and absences to a calculation.",
    			absences: {
    				title: "absences",
    				descr: "Absences that should be deducted from monthly salary.\nIf not provided, absences are not calculated."
    			},
    			calc: {
    				title: "calc",
    				descr: "The salary calculation that should be recalculated."
    			},
    			holidayYear: {
    				title: "holiday year",
    				descr: "Holiday year with holidays that should be added as salary rows.\nIf not provided, holidays are not calculated."
    			}
    		},
    		PersonAccount: {
    			title: "person account",
    			descr: "An account for a Person - can be both Employer and Worker",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			contactVerificationPin: {
    				title: "contact verification pin",
    				descr: "Contact verification pin for securing contact address changes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerInfo: {
    				title: "employer info",
    				descr: "The employer information in case the person is an employer"
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			lastLoginAs: {
    				title: "last login as",
    				descr: "Type of the last login - the user role"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "The worker information in case the person is a Worker"
    			}
    		},
    		PersonInput: {
    			title: "person input",
    			bankAccountIban: {
    				title: "bank account iban",
    				descr: "Bank account number for salary payment"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the user / personal account"
    			},
    			contactVerification: {
    				title: "contact verification",
    				descr: "Verification objects for contact verification."
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a personal account based on this data.\nOtherwise this is just user data for Company."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Personal ID (Henkilötunnus /HETU)"
    			},
    			picture: {
    				title: "picture",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given PersonalId exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			}
    		},
    		PricingPartner: {
    			title: "pricing partner",
    			descr: "Defines the partner, service and pricing model for the customer.",
    			accountId: {
    				title: "account id",
    				descr: "Partner account id."
    			},
    			fixedFee: {
    				title: "fixed fee",
    				descr: "Fee for one calculation, which is visible and paid by customer.\nThis is not relevant if the PricingModel is PalkkausFee"
    			},
    			pricingModel: {
    				title: "pricing model",
    				descr: "Pricing model by the partner."
    			},
    			serviceModel: {
    				title: "service model",
    				descr: "Service model by the partner."
    			}
    		},
    		PricingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Defines the pricing partner, service model and pricing model.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically if partner set and partner specific pricing model."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			pricingPartner: {
    				title: "pricing partner",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of pricing."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PrimaryPartner: {
    			title: "primary partner",
    			descr: "Primary partner settings.",
    			accounting: {
    				title: "accounting"
    			},
    			branding: {
    				title: "branding"
    			},
    			description: {
    				title: "description",
    				descr: "Description of the primary account."
    			},
    			id: {
    				title: "id",
    				descr: "Primary account id."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "Boolean indicating if this partner has been authorized to act on behalf the current account."
    			},
    			isPrimary: {
    				title: "is primary",
    				descr: "Boolean indicating if this partner is the primary partner of the current account."
    			},
    			key: {
    				title: "key",
    				descr: "Human-readable, human decided, recognizable short unique key for the partner:\nAll lower-case, a-z, 0-9, \"-\" and \"_\". Used in URL's etc."
    			},
    			messaging: {
    				title: "messaging"
    			},
    			name: {
    				title: "name",
    				descr: "Display name for the account."
    			},
    			service: {
    				title: "service"
    			},
    			site: {
    				title: "site"
    			}
    		},
    		PrimaryPartnerSettings: {
    			accountId: {
    				title: "Palkkaus-tilin tunniste"
    			},
    			info: {
    				title: "Tiedot"
    			},
    			type: {
    				title: "Tyyppi"
    			},
    			serviceModel: {
    				title: "Palvelumalli"
    			},
    			serviceModelId: {
    				title: "Palvelumallin id"
    			}
    		},
    		Product: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Base inteface for Products.\nThe common properties for all Products: BaseSalaryProduct, TaxProduct, UnemploymentProduct etc.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user. Restrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product.\nE.g. Raksaloki is available for households only and this is a Company account =&gt; Should not show the product."
    			}
    		},
    		Profile: {
    			title: "profile",
    			descr: "Profile object for either Worker or Employer",
    			accountId: {
    				title: "account id",
    				descr: "Account if known"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "The avatar object: Picture name and other such basic info."
    			},
    			contact: {
    				title: "contact",
    				descr: "Other contact information for this profile."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "The legal entity type of the profile: Person or Company"
    			},
    			expiresAt: {
    				title: "expires at",
    				descr: "Expiration date for the profile will be unpublished."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id (e.g. member number)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isPublic: {
    				title: "is public",
    				descr: "If true, the profile is set to public."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			job: {
    				title: "job"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			sortableName: {
    				title: "sortable name"
    			},
    			source: {
    				title: "source",
    				descr: "The source of this profile: Palkkaus section or partner site.\nThis is a strongly typed object that may be undefined - see SourceId in that case."
    			},
    			sourceId: {
    				title: "source id",
    				descr: "The identifier string of the source. This will always return a distinct value."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			userUpdatedAt: {
    				title: "user updated at",
    				descr: "Last time user updated the profile"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		ProfileJob: {
    			title: "profile job",
    			descr: "A job for a Public Profile (Osaajaprofiili) or later also for Job ads.",
    			category: {
    				title: "category"
    			},
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Job"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			picture: {
    				title: "picture"
    			},
    			synonyms: {
    				title: "synonyms"
    			}
    		},
    		ProfileJobCategory: {
    			title: "profile job category",
    			descr: "Category of Profile Jobs",
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Category"
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID of the Category"
    			},
    			name: {
    				title: "name",
    				descr: "Name / title of the Category"
    			},
    			picture: {
    				title: "picture",
    				descr: "Backround picture for the Category"
    			}
    		},
    		Recipient: {
    			title: "recipient",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		RemunerationUsecase: {
    			title: "remuneration usecase",
    			descr: "Handles the Calculation of CalculationRowType.Remuneration to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Report: {
    			title: {
    				title: "title",
    				descr: "Short title for the report that can be shown to the end user."
    			},
    			descr: "Describes a logical generated report. This is a report instance e.g. a monthly report for January 2017. It may contain several files of different types: PDF, preview PNG, HTML etc.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			htmlUrl: {
    				title: "html url",
    				descr: "HTML for the report. Use salaxy-rpt.css for formatting.\nFor preview and demo, this is the format to show to the user - PDF is not generated."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isDemoData: {
    				title: "is demo data",
    				descr: "If true, the report is created using demo data - should be used only in the beginning when the user has not paid any salaries.\nThis may be e.g. calculations in draft stage or completely invented demo data such as example workers.\nDemo data should not be confused to test database (test-api.palkkaus.fi / test-api.salaxy.com):\nPaid calculations in test database are production data."
    			},
    			isPreview: {
    				title: "is preview",
    				descr: "If true, the report is in preview state, typically in the middle of the month.\nThis means that the data itself is production, its just not final."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			jsonUrl: {
    				title: "json url",
    				descr: "Some reports are also available in JSON format, from which you can create lists, CSV, Excel files etc."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pdfUrl: {
    				title: "pdf url",
    				descr: "PDF report for viewing and printing. This is typically the main report.\nFor preview and demo, there is typically only HTML version - no PDF."
    			},
    			reportDate: {
    				title: "report date",
    				descr: "Report date: For monthly and yearly reports this is the first day of the year/month.\nFor ad-hoc reports, this is the start date of the period."
    			},
    			thumbnail: {
    				title: "thumbnail",
    				descr: "Thumbnail image for the report.\nFor preview and demo data this is typically a fixed image - the thumbnail is only generated when PDF is generated."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the report"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			urlsExpireAt: {
    				title: "urls expire at",
    				descr: "Time when the PDF, HTML or JSON URL's expire IF they are generated in a way that\nthey are callable without authentication for a specified time period.\nThis property is null if the URL requires authentication."
    			}
    		},
    		ReportingArea: {
    			title: "reporting area",
    			bottom: {
    				title: "bottom",
    				descr: "Bottom margin."
    			},
    			left: {
    				title: "left",
    				descr: "Left margin."
    			},
    			right: {
    				title: "right",
    				descr: "Right margin."
    			},
    			top: {
    				title: "top",
    				descr: "Top margin."
    			}
    		},
    		ReportingSettings: {
    			title: "reporting settings",
    			descr: "Settings related to reporting",
    			css: {
    				title: "css",
    				descr: "Additional CSS that is used to customize the layout of the report."
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			footerTemplate: {
    				title: "footer template",
    				descr: "HTML template for the print footer.\nUse the following handlebars tags to inject values into the footer:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			headerTemplate: {
    				title: "header template",
    				descr: "HTML template for the print header.\nUse the following handlebars tags to inject values into the header:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			img2: {
    				title: "img2",
    				descr: "Secondary image that can be used in header / footer as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			layoutTemplateId: {
    				title: "Ulkoasun tunniste"
    			},
    			layoutTemplateVariables: {
    				title: "Ulkoasun muuttujat"
    			},
    			logo: {
    				title: "Logo",
    				descr: "Main logo image as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			margins: {
    				title: "margins",
    				descr: "Defines the margins of the content area"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		ResultRow: {
    			title: "result row",
    			descr: "Defines a Calculation row in the calculation results",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowSource: {
    				title: "row source",
    				descr: "Source of the calculation row - affects how the row is handled in editing / recalculation scenarios."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is kept as-is in the Salaxy system."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the row - always Price * Count"
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "Base salary (Peruspalkka) is the monthly salary or total salary without addiotions (overtime, holidays etc.)"
    			},
    			totalDeduction: {
    				title: "total deduction",
    				descr: "Deductions from the salary: Union payment, Advance, Foreclosure"
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Taxfree expenses compensation"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary (Bruttopalkka) is \"real\" salary including the additions (overtime, holidays etc.) but not benefits."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "Social Security Base (Sosiaaliturvan alainen ansio) is typically the same as TotalTaxable,\nbut e.g. child care subsidy (yksityisen hoidon tuki) is handled differently."
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable income (Ennakonpidätyksen alainen ansio) is the salary from taxation point-of-view. It is basically the Gross Salary plus benefits."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row"
    			},
    			userRowIndex: {
    				title: "user row index",
    				descr: "Row index that matches this row to a user defined row (which is zero-based).\nIf the row is generated by calculation business logic (e.g. based on framework agreement), the index is -1."
    			}
    		},
    		RootDeclarationReflection: {
    			title: "root declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			}
    		},
    		RowAccountingData: {
    			title: "row accounting data",
    			descr: "Provides row specific data for accounting.",
    			accountNumberFlags: {
    				title: "account number flags",
    				descr: "Special flags for account number mapping."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percent for VAT calculation expressed as decimal: 24% is 0.24.\nThe price in the root level is assumed being with tax."
    			}
    		},
    		Salary: {
    			title: "salary",
    			descr: "Defines the main salary that is the bases for the calculation",
    			amount: {
    				title: "amount",
    				descr: "Amount of e.g. hours, months for the salary. Default is 1."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If set to true, will calculate the household deduction. Applies only to natural persons - as oppsed to companies."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the salary being calculated"
    			},
    			message: {
    				title: "message",
    				descr: "Optional message for the salary row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price of one unit - e.g. hour, month."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			unit: {
    				title: "unit",
    				descr: "E.g. hour, month. Typically this parameter can be left empty/null and the unit is set according to type."
    			}
    		},
    		SampleApiObject: {
    			title: "sample api object",
    			descr: "Sample object for creating v03 controller",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SampleData: {
    			title: "sample data",
    			descr: "Sample custom data in search index",
    			test: {
    				title: "test",
    				descr: "A test string"
    			}
    		},
    		SampleListItem: {
    			title: "sample list item",
    			descr: "Available Search index data of the Sample object",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ServiceModelAccounting: {
    			title: "Kirjanpito",
    			defaultTargetId: {
    				title: "Oletuskohde"
    			},
    			targets: {
    				title: "Kohteet"
    			}
    		},
    		ServiceModelBranding: {
    			title: "Brändi",
    			logo: {
    				title: "Logo"
    			},
    			skin: {
    				title: "Tyylitiedosto"
    			}
    		},
    		ServiceModelDto: {
    			title: "Palvelumalli",
    			id: {
    				title: "Id"
    			},
    			name: {
    				title: "Nimi"
    			}
    		},
    		ServiceModelFeatures: {
    			title: "Aktivoitavat palvelut",
    			messaging: {
    				title: "Viestit"
    			},
    			forms: {
    				title: "Aineiston lähetys"
    			},
    			accounting: {
    				title: "Kirjanpito"
    			},
    			payments: {
    				title: "Maksaminen"
    			},
    			reporting: {
    				title: "Tulosteet"
    			}
    		},
    		ServiceModelPayments: {
    			title: "Maksaminen",
    			defaultChannel: {
    				title: "Oletusmaksutapa"
    			},
    			channels: {
    				title: "Maksutavat"
    			}
    		},
    		ServiceModelPricing: {
    			title: "Hinnoittelu",
    			pricingModel: {
    				title: "Hinnoittelumalli"
    			},
    			fixedFee: {
    				title: "Kiinteä hinta"
    			}
    		},
    		ServiceModelSite: {
    			title: "Sivuston ominaisuudet",
    			webApp: {
    				title: "Sovellus / sivusto"
    			},
    			webAppName: {
    				title: "Sovelluksen / sivuston nimi"
    			},
    			webAppUrl: {
    				title: "Sivuston osoite"
    			},
    			texts: {
    				title: "Tekstit"
    			},
    			sitemap: {
    				title: "Sivukartta"
    			}
    		},
    		SessionUserCredential: {
    			title: "session user credential",
    			descr: "Represents a user credential in session: This credential is typically based on Claims and is availble without going to database.",
    			authenticationMethod: {
    				title: "authentication method",
    				descr: "The ultimate authentication provider that authenticated the user, e.g. Facebook, Google."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object that contains the visual reresentation of the User account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail address if provided by the authentication provider."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identitier for the user ID coming from the Auth0 database:\n'[MethodText]|[ProviderId]', e.g. 'facebook|12345678901234567'\n(Number may contain more characters - not sure for the exact cahracters count)"
    			}
    		},
    		SharedTaxcardExistsResult: {
    			title: "shared taxcard exists result",
    			descr: "Provides a a result to query of whether a taxcard exists for a personal ID (no personal information).",
    			isError: {
    				title: "is error",
    				descr: "If true there was an error fetching the information.\nTypically, this would be that there were too many attempts to fetch Personal ID's and request was classified as phishing.\nIn this case, you should currently create the employment relation and fetch the taxcard infor after that.\nWe may later introduce a \"Are you robot\" type of interface to go around this (if necessary)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard. Undefined if none is found."
    			},
    			ownedExists: {
    				title: "owned exists",
    				descr: "Returns true if there is already an owned taxcard for the employer."
    			},
    			ownedTaxcard: {
    				title: "owned taxcard",
    				descr: "If Owned taxcard exists, it is returned here."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID that was used in fetching.\nIt may be validated and formatted, or just the original string if there were issues."
    			},
    			sharedExists: {
    				title: "shared exists",
    				descr: "Returns true if a shared taxcard exists."
    			},
    			uri: {
    				title: "uri",
    				descr: "Uri for the Shared taxcard"
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity of the taxcard. Note that this property is null when Exists is false."
    			}
    		},
    		Shareholder: {
    			title: "shareholder",
    			descr: "Defines a shareholder. At the moment, this is only a name and it is used only if the person has more than 25% of the shares. Expect this object to be developed with, Ownership percent or number of shares, Id or other such link to the actual person etc.",
    			name: {
    				title: "name",
    				descr: "Name of the shareholder."
    			}
    		},
    		SignatureData: {
    			title: "signature data",
    			digitalSignature: {
    				title: "digital signature"
    			},
    			email: {
    				title: "email"
    			},
    			isPep: {
    				title: "is pep"
    			},
    			isProcura: {
    				title: "is procura",
    				descr: "The account is authorized to sign on behalf of the actual account."
    			},
    			personName: {
    				title: "person name",
    				descr: "Name of the signing person.\nBy default, this is Onboarding Person, but it may be a completely different person."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID of the person, who has the right to sign."
    			},
    			telephone: {
    				title: "telephone"
    			}
    		},
    		SmsMessage: {
    			title: "sms message",
    			descr: "API SmsMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (characters or numbers)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (phone)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SubsidisedCommuteUsecase: {
    			title: "subsidised commute usecase",
    			descr: "Handles the Calculation of CalculationRowType.SubsidisedCommute to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			periodDivider: {
    				title: "period divider",
    				descr: "Number of periods if Kind is PeriodicalDeduction.\nOtherwise this value is null."
    			}
    		},
    		TaxCardIncome: {
    			title: "tax card income",
    			descr: "Defines income that is recorded in a Taxcard: A paid salary calulation, income from previous employers or other salaries that affect the income calculation.",
    			description: {
    				title: "description",
    				descr: "A short description of the paid calculation.\nThis is a mandatory field if the Id is null."
    			},
    			diff: {
    				title: "diff",
    				descr: "When loading calculations from the database (in taxcard Save and FindBy with diff),\nthis property identifies the differences in the entry."
    			},
    			endDate: {
    				title: "end date",
    				descr: "End of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkEndDate, but may be different in edge cases."
    			},
    			id: {
    				title: "id",
    				descr: "Calculation id in the Salaxy database.\nIf the calculation is not in the Salaxy Database, leave this null."
    			},
    			income: {
    				title: "income",
    				descr: "Taxable income from this calculation"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the row, typically the employer that paid the salary.\nNull meaning that it is the owner of the original taxcard."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "The paidAt date which tells mainly to which year the payment should be attributed.\nThis is reliable only if the salary is paid by Salaxy (TaxCardIncomeType.SalaxyCalculation).\nIn other cases it is just the reporting date."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Beginning of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkStartDate, but may be different in edge cases."
    			},
    			tax: {
    				title: "tax",
    				descr: "Widthholding tax in euro"
    			},
    			type: {
    				title: "type",
    				descr: "Type of income"
    			}
    		},
    		TaxProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Service related to integration, reporting and payments to tax authorities.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		TaxSettings: {
    			title: "tax settings",
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			}
    		},
    		Taxcard: {
    			title: "taxcard",
    			descr: "Full tax card that is stored in database and that is used in tax card percentage calculations. Also contains the salary history in the income log.",
    			card: {
    				title: "card",
    				descr: "The input that is coming from the Worker / Tax authorities"
    			},
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"[Server]/[PayloadType]/[Owner]/[Id]\", e.g. \"https://test-secure.salaxy.com/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment for which the tax card is created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			incomeLog: {
    				title: "income log",
    				descr: "Calculations that have been paid to this tax card."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "Read-only calculation results and other logic values on the Taxcard."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker, whose salaries are paid based on this tax card."
    			}
    		},
    		TaxcardData: {
    			title: "taxcard data",
    			descr: "Tax card specific business data for index.",
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"/[PayloadType]/[Owner]/[Id]\", e.g. \"/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the taxcard validity. This may be null on NoTaxcard types."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "The type of the tax card."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the taxcard validity"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income paid to the taxcard during the year (forYear)."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity resolved as enumeration in relation to payment date given in the query.\nThis is not available in direct database OData queries as it is relevant to current / given date."
    			}
    		},
    		TaxcardInput: {
    			title: "taxcard input",
    			descr: "Input from user for a tax card: What is written in the card plus the state infomration that is directly editable by the user.",
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			state: {
    				title: "state",
    				descr: "Tax card approval state"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardListItem: {
    			title: "taxcard list item",
    			descr: "List item of taxcard searches.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		TaxcardResult: {
    			title: "taxcard result",
    			descr: "Provides the results of the tax card calculation and business logic.",
    			tax: {
    				title: "tax",
    				descr: "Provides a decimal number for a recommended widthholding tax IF it is possible to calculate that based on given information.\nNull, if the information is not sufficient to give a recommendation."
    			},
    			yearlyPreviousIncome: {
    				title: "yearly previous income",
    				descr: "Sum of income for this year based on the items marked on the tax card."
    			}
    		},
    		TaxcardResults: {
    			title: "taxcard results",
    			descr: "Read-only calculations and other logic methods on a Taxcard.",
    			payment: {
    				title: "payment",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for the actual year, which is used for income limit.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			paymentJanuary: {
    				title: "payment january",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income for the actual year, which is used for income limit.\nNote that this does not include January income that may be paid for previous year taxcard."
    			},
    			totalIncomeJanuary: {
    				title: "total income january",
    				descr: "Total income for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January."
    			},
    			validityToday: {
    				title: "validity today",
    				descr: "Gets the validity of the taxcard today. This value reflects purely the date value - it does not take into account approval or replacement."
    			}
    		},
    		TaxcardSnapshot: {
    			title: "taxcard snapshot",
    			descr: "Basic information about the taxcard: This object has the infromation printed in SalarySlip etc. but not the full salaries paid to the taxcard etc.",
    			cumulativeIncome: {
    				title: "cumulative income",
    				descr: "Cumulative income on the taxcard - including this calculation.\nNote that with shared taxcards, this may include income from other employers."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit.\nThis is percent. I.e. for 50% set 50, not 0.5."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardUiInfo: {
    			title: "taxcard ui info",
    			descr: "Display information and language versioned texts that make sense to\nend-users about the status of taxcard.",
    			kind: {
    				title: "kind",
    				descr: "Kind that is quaranteed to be non-null (undefined). This is about the existing taxcard,\nthere may still be a new card \"waiting\" and that may have another kind specified."
    			},
    			percent: {
    				title: "percent",
    				descr: "The tax percent if it is relevant for the taxcard.\nNumber is 0-60, not decimal e.g. 0.60. Null if not relevant."
    			},
    			shortText: {
    				title: "short text",
    				descr: "Shorter version of the descriptive text to the end user."
    			},
    			status: {
    				title: "status",
    				descr: "Status as in\n\"ok\": Valid taxcard,\n\"missing\": No taxcard, employer should add a new one.,\n\"waiting\": A shared taxcard is waiting for approval. Existing may be valid or not.\nor null: No calculation or employment relation specified."
    			},
    			text: {
    				title: "text",
    				descr: "Descriptive text to the end user containing the percent if relevant."
    			}
    		},
    		TaxcardWaitingApproval: {
    			title: "taxcard waiting approval",
    			descr: "Informs the employer that there is a Worker taxcard waiting for approval potentially without revealing personal information.",
    			createdAt: {
    				title: "created at",
    				descr: "Time when the taxcard was added."
    			},
    			fullCard: {
    				title: "full card",
    				descr: "If there is an owner instance of taxcard for the employer, the copy is added here.\nInstance exists only if there is a confirmed employment relation or if employer explicitly requests the taxcard."
    			},
    			isRelationUnconfirmed: {
    				title: "is relation unconfirmed",
    				descr: "If true, the relation has not yet been confirmed:\nI.e. the employer has just input the Finnish Personal Id (HETU) of the Worker.\nThe employment relation is confirmed when salaries are paid or salary request is received from Worker."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner account that added the taxcard."
    			},
    			state: {
    				title: "state",
    				descr: "Status of the taxcard"
    			},
    			uri: {
    				title: "uri",
    				descr: "URI for the shared card: This is the taxcard owned by the Worker and to view it\nyou need to request your own copy (would be in FullCard)."
    			}
    		},
    		TaxcardsDataRow: {
    			title: "taxcards data row",
    			descr: "Data row for taxcards data. This is typically used for imports and exports.",
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the taxcard"
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used. Any income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tax authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		TesParameters: {
    			title: "tes parameters",
    			descr: "Captures the collection of parameters that different Framework agreements (TES) have.",
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses amount"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Travel expences per day"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "The amount of kilometers travelled by day - used to define the amount of travel expenses in aomse framework agreements."
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "Marker that the payments defined by framework agreement are included in the salary (will not be added by the calculation)."
    			},
    			isYksityisenHoidonTuki: {
    				title: "is yksityisen hoidon tuki",
    				descr: "If true will calculate the Finnish child care subsidy"
    			},
    			numberOfDays: {
    				title: "number of days",
    				descr: "Number of days may affect calculation of different payments calculated based on a Framework agreement"
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype when one framework has several options"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the work framework"
    			},
    			yksityisenHoidonTukiAmount: {
    				title: "yksityisen hoidon tuki amount",
    				descr: "Amount of Finnish child care subsidy"
    			}
    		},
    		TestAccountInfo: {
    			title: "test account info",
    			descr: "Object containing essential details of the test account",
    			token: {
    				title: "token",
    				descr: "Bearer token for authentication."
    			},
    			userCredential: {
    				title: "user credential",
    				descr: "The current user credentials."
    			}
    		},
    		TestValues: {
    			title: "test values",
    			descr: "Test object with value properties.",
    			arrayValue: {
    				title: "array value",
    				descr: "Test array"
    			},
    			boolValue: {
    				title: "bool value",
    				descr: "Test bool."
    			},
    			dateTimeValue: {
    				title: "date time value",
    				descr: "Test DateTime."
    			},
    			dateValue: {
    				title: "date value",
    				descr: "Test Date."
    			},
    			decimalValue: {
    				title: "decimal value",
    				descr: "Test decimal."
    			},
    			dictValue: {
    				title: "dict value",
    				descr: "Free values dictionary."
    			},
    			doubleValue: {
    				title: "double value",
    				descr: "Test double."
    			},
    			enumValue: {
    				title: "enum value",
    				descr: "Test enum"
    			},
    			intValue: {
    				title: "int value",
    				descr: "Test integer."
    			},
    			objectValue: {
    				title: "object value",
    				descr: "Test object."
    			},
    			stringValue: {
    				title: "string value",
    				descr: "Test string."
    			}
    		},
    		TotalCalculationDTO: {
    			title: "total calculation d t o",
    			descr: "Calculates the total numbers - the ones that are common to Employer and worker.",
    			pension: {
    				title: "pension",
    				descr: "The total pension payment for this calculation"
    			},
    			total: {
    				title: "total",
    				descr: "The total of all calculation rows"
    			},
    			totalAccidentInsuranceBase: {
    				title: "total accident insurance base",
    				descr: "Base income for calculating the Accident insurance contributions."
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "The base salary without overtime and other such additions that are part of gross salary."
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Tax free expenses."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary is the salary including additions like overtime, but not benefits or expenses"
    			},
    			totalHealthInsuranceBase: {
    				title: "total health insurance base",
    				descr: "Base income for calculating the Health insurance contributions."
    			},
    			totalPayable: {
    				title: "total payable",
    				descr: "Total sum of rows that are paid through Salaxy Customer Funds account.\nThis is the Gross sum before side costs, deductions etc., but it is not Gross salary as it includes expenses etc."
    			},
    			totalPensionInsuranceBase: {
    				title: "total pension insurance base",
    				descr: "Base income for calculating the Pension insurance contributions."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "DEPRICATED: This is now the same as TotalPensionInsuranceBase. For other social insurances, you may want to use\nTotalUnemploymentInsuranceBase, TotalHealthInsuranceBase or TotalAccidentInsuranceBase,"
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable salary is the salary from the taxation point of view - includes some benefits."
    			},
    			totalUnemploymentInsuranceBase: {
    				title: "total unemployment insurance base",
    				descr: "Base income for calculating the Unemployment insurance contributions."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The total unemployment payment sum."
    			}
    		},
    		TypeDocType: {
    			title: "type doc type",
    			descr: "Simplified read-only interface for typedoc type.",
    			elementType: {
    				title: "element type",
    				descr: "Element type in array"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier in reference"
    			},
    			name: {
    				title: "name",
    				descr: "Name of type in intrinsic and typeParameter"
    			},
    			type: {
    				title: "type",
    				descr: "Type of type"
    			},
    			typeArguments: {
    				title: "type arguments",
    				descr: "Type arguments in reference"
    			},
    			types: {
    				title: "types",
    				descr: "Collection of types in union"
    			},
    			value: {
    				title: "value",
    				descr: "Value if literal"
    			}
    		},
    		UnemploymentProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Product that helps employer to handle the mandatory unemployment insurance reports and payments.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isNotificationsSelfHandling: {
    				title: "is notifications self handling",
    				descr: "If true, the account himself handles the unemployment insurance notifications."
    			},
    			isPrepayments: {
    				title: "is prepayments",
    				descr: "If true, the account has made prepayments or is registered to make prepayments.\nPalkkaus needs to contact the account."
    			},
    			isSalariesPaidThisYear: {
    				title: "is salaries paid this year",
    				descr: "If true, the account has paid salaries this year outside Palkkaus.fi.\nThis affects the yearly reporting - Palkkaus needs to contact the account.\nNote that this is a shared option with the taxation product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		UnionPaymentUsecase: {
    			title: "union payment usecase",
    			descr: "Handles the Calculation of CalculationRowType.UnionPayment to IrRows",
    			isDeductedAtBackoffice: {
    				title: "is deducted at backoffice",
    				descr: "If true, deduction is done in Salaxy backoffice, not directly by employer.\nAs of writing, this is only done for Rakennusliitto / Construction."
    			},
    			isFixedSum: {
    				title: "is fixed sum",
    				descr: "If true, the value is a fixed sum: the price should not be updated based on salary and\nthe count should not be updated based on percentage."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Usecase: {
    			title: "usecase",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UsecaseChildCare: {
    			title: "usecase child care",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isChildcareSubsidy: {
    				title: "is childcare subsidy",
    				descr: "If true, the salary calculation includes child care subsidy paid by the government / community."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isSunday: {
    				title: "is sunday",
    				descr: "If true, the work is done on Sunday or public holiday.\nThis means that the salary is increased by 100% according to Finnish law"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			subsidyAmount: {
    				title: "subsidy amount",
    				descr: "Amount for the child care subsidy paid by the government / community."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			useCustomPrice: {
    				title: "use custom price",
    				descr: "If true, a custom price is used instead of the recommended price.\nCurrently, this only affects the MLL case as the normal childCare price is always set by the user."
    			}
    		},
    		UsecaseCleaning: {
    			title: "usecase cleaning",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isContractLessThanYear: {
    				title: "is contract less than year",
    				descr: "If salaries are compensated (when holidays are not kept, isFullTime=false),\nthe percentage is different depending on whether the person has been working for a full year or not."
    			},
    			isFullTime: {
    				title: "is full time",
    				descr: "If true, this is a full time contract in the respect of holidays:\nHolidays are kept during summer and salary is paid for these holidays."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseConstruction: {
    			title: "usecase construction",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses (työkalukorvaus) if isExpensesCustom=true"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Daily travel expenses if isExpensesCustom=true"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "Daily distance (km) from worker home to construction site."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isExpensesCustom: {
    				title: "is expenses custom",
    				descr: "If true, the expenses are agreed in a bispoken way.\nDefault is that expenses are based on Framework agreement (TES) tables."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "If true, the TES additions are included in the agreed salary."
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseData: {
    			title: "usecase data",
    			descr: "A usecase for creating and editing salary calculations and other . In practice, a usecase is either a user interface or a microservice that provides the bases for the object. Provides the possibility to store useacase specific data as part of object. NOTE: For Calculations, this functionality will replace TesParameters in the 3.0 version.",
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			description: {
    				title: "description",
    				descr: "Optional description of the use case for the end user.\nShown in detail views when explaining the logic of the usecase."
    			},
    			label: {
    				title: "label",
    				descr: "A short label of that can be shown to the end user."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this may resolve to a user interface, microservice or a to manifest registering one.\nFor now, just use a unique string starting with a domain you own."
    			}
    		},
    		UsecaseGroup: {
    			title: "usecase group",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case group (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG)"
    			},
    			children: {
    				title: "children",
    				descr: "Collection of use cases - in the future, may also support sub groups."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG )"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the group"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UserDefinedRow: {
    			title: "user defined row",
    			descr: "This is a user defined calculation row as opposed to CalculationRow that may be either based on a user defined row or generated by the system based on e.g. TES parameters.",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowIndex: {
    				title: "row index",
    				descr: "Zero based row index that can be used to match the row to result rows or when deleting.\nNOTE: The Row ID is given values on the fly so setting this will not have any effect at the moment.\nIt may reorder the entries in later implementations."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is passed to the result calculations."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. Guessed based on the RowType and count, but you are better of setting it to be sure."
    			}
    		},
    		UserSession: {
    			title: "user session",
    			descr: "Passes the data about the authorization that is the bases for callin the services that require user authorization.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar that should be shown for the current session.\nTypically CurrentAccount.Avatar, but could theoretically be something different."
    			},
    			currentAccount: {
    				title: "current account",
    				descr: "The current Palkkaus.fi-account that the user acting as"
    			},
    			currentCredential: {
    				title: "current credential",
    				descr: "The current user credentials."
    			},
    			currentPartner: {
    				title: "current partner",
    				descr: "The primary partner of the current account."
    			},
    			hasCredential: {
    				title: "has credential",
    				descr: "If false, the current user does not have a credential in the system =&gt; need to redirect to login"
    			},
    			hasCurrentAccount: {
    				title: "has current account",
    				descr: "If false, the current user does not have a current account.\nIf HasCredential is true, this means that the user should be redirected to Account creation wizard."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, the current user needs to be authenticated or authenticated user does not have acces to the system."
    			},
    			requestingPartnerSite: {
    				title: "requesting partner site",
    				descr: "The site from which the API call is being made."
    			},
    			status: {
    				title: "status",
    				descr: "Authorization status"
    			},
    			systemAlertHtml: {
    				title: "system alert html",
    				descr: "Contains html for system alert."
    			}
    		},
    		VarmaPensionOrder: {
    			title: "varma pension order",
    			descr: "Contains order data for Varma pensions (TyEL, YEL) and required operations.",
    			action: {
    				title: "action",
    				descr: "Action for the order."
    			},
    			orderer: {
    				title: "orderer",
    				descr: "Orderer information."
    			},
    			tyel: {
    				title: "tyel",
    				descr: "TyEL data for the order"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation information."
    			},
    			yel: {
    				title: "yel",
    				descr: "YEL data for the order"
    			}
    		},
    		VarmaPensionOrderTyel: {
    			title: "varma pension order tyel",
    			descr: "Varma pension order TyEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startMonth: {
    				title: "start month",
    				descr: "Start month for the new order."
    			},
    			startYear: {
    				title: "start year",
    				descr: "Start year for the new order."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yerly income."
    			}
    		},
    		VarmaPensionOrderYel: {
    			title: "varma pension order yel",
    			descr: "Varma pension order YEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			isYearlyIncomeChange: {
    				title: "is yearly income change",
    				descr: "Indicates if the existing yearly income estimate will be changed."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer definition."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date for the new pension."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yearly income."
    			}
    		},
    		VismaSignPostData: {
    			title: "visma sign post data",
    			descr: "Data for building the Visma Sign API request form. This is the legacy API v0.2: https://allekirjoitus.visma.fi/download/visma_sign-api.pdf",
    			auth_service: {
    				title: "auth_service",
    				descr: "Specifies the Tupas authentication service (e.g. 'osuuspankki', 'nordea').\nIf not specified, Visma Sign will show its own selection screen."
    			},
    			customer: {
    				title: "customer",
    				descr: "Palkkaus.fi Customer code to Visma Sign-system"
    			},
    			data: {
    				title: "data",
    				descr: "Rest of the data as encrypted JSON."
    			},
    			documentUrl: {
    				title: "document url",
    				descr: "Document URL for preview purposes.\nTo Visma Sign, this is sent within the encrypted data field."
    			},
    			iv: {
    				title: "iv",
    				descr: "Encryption Initialization vector"
    			},
    			return_failure: {
    				title: "return_failure",
    				descr: "Failure address - needs to be non-encrypted, because the error may be in the encryption"
    			}
    		},
    		WelcomeDataWorker: {
    			title: "welcome data worker",
    			descr: "Data that is used for displaying the welcome page for the Worker account.",
    			activeTaxcard: {
    				title: "active taxcard",
    				descr: "Active taxcard added by the Worker or null, if none is added (or none is valid for today)."
    			},
    			invitation: {
    				title: "invitation",
    				descr: "Invitation for the worker to store a taxcard for salary payment."
    			},
    			timeline: {
    				title: "timeline",
    				descr: "The welcome page timeline cards."
    			}
    		},
    		WorkDescription: {
    			title: "work description",
    			descr: "Describes the work that is beign performed - mainly for Employment relation purposes.",
    			description: {
    				title: "description",
    				descr: "Free text description of the work that is being performed."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			salaryDefaults: {
    				title: "salary defaults",
    				descr: "Contains default calculation rows for the worker.\nTypically used for setting for example monthly salary or hourly salary price for workers."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Default bases for wage."
    			}
    		},
    		WorkerAbsencePeriod: {
    			title: "worker absence period",
    			descr: "Absence Period with Worker information.",
    			absencesId: {
    				title: "absences id",
    				descr: "WorkerAbsences storage object that contains the period"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			},
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker that owns the Absence period"
    			}
    		},
    		WorkerAbsences: {
    			title: "worker absences",
    			descr: "Absences for a single Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation id for the worker whose absences are recorded."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			periods: {
    				title: "periods",
    				descr: "Collection of absence periods."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Account ID for the worker whose absences are recorded."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			}
    		},
    		WorkerAccount: {
    			title: "worker account",
    			descr: "Account data that is relevant for a Worker.",
    			ageRange: {
    				title: "age range",
    				descr: "Age range that is relevant to workers salary calculation and/or employment.\nSet this especially if OfficialPersonId is not set.\nIf OfficialPersonId is valid, this property is always set based on it."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "If created (and still owned) by an employer, this is the AccountOwnerId of the employer who created this account."
    			},
    			employment: {
    				title: "employment",
    				descr: "Current Employment Relation: The data that is owned by the Employer about the Worker\neven if the Worker has registered and created an account.\nIf employer has several Employment Relations, this is the one that is marked as Primary."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The unique id for the employment. This does not change within one employment."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			officialPersonId: {
    				title: "official person id",
    				descr: "The Person ID of the worker if known."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides validation errors currently only in Save operations,\nbut potentially in the future also in detail/list views."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "Information needed when salary is paid to this account and other such stuff about the account as a worker."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		WorkerAccountEmployment: {
    			title: "worker account employment",
    			descr: "Employment Relation as it is shown as part of WorkerAccount.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isAccountOwner: {
    				title: "is account owner",
    				descr: "If true, the worker is the same person as the actual signed owner of the account."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		WorkerCalculationDTO: {
    			title: "worker calculation d t o",
    			descr: "The Worker side of the calculation. SalarySlip is largely written based on this calculation.",
    			benefits: {
    				title: "benefits",
    				descr: "The benefits as valued by the tax authority."
    			},
    			deductions: {
    				title: "deductions",
    				descr: "All the items that are deducted from the salary (this is also the order if there is not enough payable money):\nSalaryAdvance + Tax + Pension + UnemploymentInsurance + UnionPayment + OtherDeductions"
    			},
    			foreclosure: {
    				title: "foreclosure",
    				descr: "The foreclosure that been deducted from the salary by the employer."
    			},
    			fullOtherDeductions: {
    				title: "full other deductions",
    				descr: "Other deductions that are deducted from the net salary.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullPension: {
    				title: "full pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullPrepaidExpenses: {
    				title: "full prepaid expenses",
    				descr: "Prepaid expenses that are deducted from the expenses to be paid to the worker.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullSalaryAdvance: {
    				title: "full salary advance",
    				descr: "The salary advance that was paid to the Worker\nThe original calulated version - there might not be enough money to finally pay this."
    			},
    			fullTax: {
    				title: "full tax",
    				descr: "Widthholding tax: The original calulated version - there might not be enough money to finally pay this."
    			},
    			fullUnemploymentInsurance: {
    				title: "full unemployment insurance",
    				descr: "Workers part of the unemployment insurance.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullUnionPayment: {
    				title: "full union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			otherDeductions: {
    				title: "other deductions",
    				descr: "Part of the other deductions that are deducted from the net salary."
    			},
    			pension: {
    				title: "pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary."
    			},
    			prepaidExpenses: {
    				title: "prepaid expenses",
    				descr: "Part of the prepaid expenses that are deducted from the expenses."
    			},
    			salaryAdvance: {
    				title: "salary advance",
    				descr: "The salary advance that was paid to the Worker"
    			},
    			salaryAfterTax: {
    				title: "salary after tax",
    				descr: "Net salary after tax - before worker foreclosure or deductions of the side costs"
    			},
    			salaryAfterTaxAndForeclosure: {
    				title: "salary after tax and foreclosure",
    				descr: "Net salary after tax and foreclosure."
    			},
    			salaryPayment: {
    				title: "salary payment",
    				descr: "The payment that is made to the Worker based on salary after all deductions.\nDoes not include expenses."
    			},
    			tax: {
    				title: "tax",
    				descr: "Ennakonpidätys: The taxes that were deducted from the worker salary."
    			},
    			totalWorkerPayment: {
    				title: "total worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			},
    			totalWorkerPaymentLegacy: {
    				title: "total worker payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "Workers part of the unemployment insurance."
    			},
    			unionPayment: {
    				title: "union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			workerSideCosts: {
    				title: "worker side costs",
    				descr: "The side costs that are deducted from the Worker salary according to law:\nPension, Unemployment and Union payment. Tax is not included in this number.\nThis is not an official number, really used in the charts etc."
    			}
    		},
    		WorkerCurrentTaxcards: {
    			title: "worker current taxcards",
    			descr: "Contains the status and necessary data with Workers current taxcards.",
    			active: {
    				title: "active",
    				descr: "The currently active taxcard if one exists. This property is null by default."
    			},
    			employers: {
    				title: "employers",
    				descr: "Employer taxcards for the Worker account: One for each employment relationship."
    			},
    			employment: {
    				title: "employment",
    				descr: "Employment relation if the search is done based on employment relation ID and not Current worker account."
    			},
    			list: {
    				title: "list",
    				descr: "All the relevant taxcards as list items."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID that is used for taxcard binding. This always exists as opposed to Employment."
    			},
    			waitingApproval: {
    				title: "waiting approval",
    				descr: "A shared taxcard waiting for approval if one exists. This property is null by default."
    			}
    		},
    		WorkerData: {
    			title: "worker data",
    			descr: "Worker specific business data for index.",
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation."
    			}
    		},
    		WorkerInfo: {
    			title: "worker info",
    			descr: "Information directly related to Employment (as Worker)",
    			defaultTaxCard: {
    				title: "default tax card",
    				descr: "The Tax card that is used by default."
    			}
    		},
    		WorkerListItem: {
    			title: "worker list item",
    			descr: "Represents a single item in a list of workers in the user interface. It contains all essential information of the worker and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		WorkerProfileProperties: {
    			title: "worker profile properties",
    			dateOfBirth: {
    				title: "date of birth"
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy"
    			},
    			gender: {
    				title: "gender"
    			},
    			hasDriversLisence: {
    				title: "has drivers lisence"
    			},
    			hasTaxCard: {
    				title: "has tax card"
    			}
    		},
    		WorkerSummary: {
    			title: "worker summary",
    			descr: "Year end summary for one Worker account.",
    			calculations: {
    				title: "calculations",
    				descr: "Calculations that have been paid through Salaxy system."
    			},
    			corrections: {
    				title: "corrections",
    				descr: "Corrections that are made to existing calculations as well as\nsalary calculations that are paid outside Salaxy."
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes when set on worker level, not for each calculation separately."
    			},
    			summary: {
    				title: "summary",
    				descr: "Total of Calculations and Corrections as calculatied by the server side logic."
    			},
    			worker: {
    				title: "worker",
    				descr: "Worker account to which the salaries have been paid."
    			}
    		},
    		WorkersDataRow: {
    			title: "workers data row",
    			descr: "Data row for employments data. This is typically used for imports and exports.",
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the worker"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		WorkflowData: {
    			title: "workflow data",
    			descr: "Defines the data that IWorkflowItem must provide.",
    			events: {
    				title: "events",
    				descr: "Workflow events for the item."
    			}
    		},
    		WorkflowEvent: {
    			title: "workflow event",
    			descr: "Event in the workflow",
    			assignedTo: {
    				title: "assigned to",
    				descr: "Credential of the user which is responsible of the event."
    			},
    			features: {
    				title: "features",
    				descr: "Features for this event."
    			},
    			id: {
    				title: "id",
    				descr: "Event-provider specific identifier for the event.\nFor deletable events, this should identify , but do NOT trust this to be 100% unique."
    			},
    			message: {
    				title: "message",
    				descr: "Message for backoffice purposes from the User or system that created the event."
    			},
    			time: {
    				title: "time",
    				descr: "UTC time for when the event occured."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the event. This typically maps to parent-object specific enumeration."
    			},
    			ui: {
    				title: "ui",
    				descr: "User Interface styling of the event: Indicates the severity for the end-user."
    			},
    			user: {
    				title: "user",
    				descr: "User or system that created the event."
    			}
    		},
    		WorkingTimeCompensationUsecase: {
    			title: "working time compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.WorkingTimeCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		YearlyChangingNumbers: {
    			title: "yearly changing numbers",
    			descr: "Parameters that are changing yearly. This class is made for the change of the year and it is typically updated in December. It is not for recording longer history of numbers.",
    			holidays: {
    				title: "holidays",
    				descr: "Returns holidays for the year."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "Returns yearly sidecosts."
    			},
    			year: {
    				title: "year",
    				descr: "Year for numbers."
    			}
    		},
    		YearlyFeedback: {
    			title: "yearly feedback",
    			descr: "The feedback from the account holder at the end-of-year. Possibility to feed in changes, corrections and salaries that are paid outside the Salaxy system.",
    			adminNotes: {
    				title: "admin notes",
    				descr: "Full description of yearly process status for backoffice pusposes. Shown in backoffice detail views."
    			},
    			adminShortNote: {
    				title: "admin short note",
    				descr: "Short notice for backoffice purposes. This is the text visible in the lists."
    			},
    			adminStatus: {
    				title: "admin status",
    				descr: "Status for administrator check."
    			},
    			bankIban: {
    				title: "bank iban",
    				descr: "Bank IBAN number for yearly refunding of extra customer assets.\nREADONLY: Note that once set to a valid IBAN, this number is read-only and must be changed through customer service or onboarding (requires authentication)."
    			},
    			employerAvatar: {
    				title: "employer avatar",
    				descr: "Avatar for the employer of display purposes in reports etc."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "Account ID for the employer."
    			},
    			externalWorkerInsuranceWork: {
    				title: "external worker insurance work",
    				descr: "Temporary user interface helper for external workers (insurance) =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			externalWorkerSalaries: {
    				title: "external worker salaries",
    				descr: "Temporary user interface helper for external workers =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			household: {
    				title: "household",
    				descr: "Yearly feedback data that is specific to household customers.\nMain part is the information realted to household deduction and how it can be divided between spouses."
    			},
    			sendInsuranceReport: {
    				title: "send insurance report",
    				descr: "If true, Salaxy should report the salaries to Insurance company.\nCurrently, this is available only for Companies and for If insurance company only."
    			},
    			userFeedbackText: {
    				title: "user feedback text",
    				descr: "Free text from the account holder to Salaxy backoffice."
    			},
    			userFeedbackTime: {
    				title: "user feedback time",
    				descr: "Time when the feedback was given."
    			},
    			userStatus: {
    				title: "user status",
    				descr: "The satus that the account holder gives to yearly material."
    			},
    			workerSummaries: {
    				title: "worker summaries",
    				descr: "List of Workers for which salaries have been paid during the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the feedback"
    			}
    		},
    		YearlyHolidays: {
    			title: "yearly holidays",
    			descr: "Holidays for the entire year.",
    			holidays: {
    				title: "holidays",
    				descr: "Holidays for the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the holidays."
    			}
    		},
    		YearlySideCosts: {
    			title: "yearly side costs",
    			descr: "Encapsulates yearly side costs into one class.",
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction"
    			},
    			illnessInsurancePercent: {
    				title: "illness insurance percent",
    				descr: "Illness Insurance care percent that is withhold from the Worker. (vakuutetun sairausvakuutusmaksu: päivärahamaksu + sairaanhoitomaksu"
    			},
    			palkkausVatPercent: {
    				title: "palkkaus vat percent",
    				descr: "ALV-% for the fee."
    			},
    			socSecPercent: {
    				title: "soc sec percent",
    				descr: "Social Security payment for employer (SAVA-maksu)"
    			},
    			taxFreeDailyAllowance: {
    				title: "tax free daily allowance",
    				descr: "Tax free daily allowance for full day approved by tax authorities."
    			},
    			taxFreeDailyHalfAllowance: {
    				title: "tax free daily half allowance",
    				descr: "Tax free daily allowance for half day approved by tax authorities."
    			},
    			taxFreeKmAllowance: {
    				title: "tax free km allowance",
    				descr: "Tax free milage allowance approved by tax authorities."
    			},
    			taxFreeMealAllowance: {
    				title: "tax free meal allowance",
    				descr: "Tax free meal allowance approved by tax authorities."
    			},
    			tyel53Percent: {
    				title: "tyel53 percent",
    				descr: "Pension payment base percentage - deducted from 53-62 year old Worker"
    			},
    			tyelBasePercent: {
    				title: "tyel base percent",
    				descr: "Pension payment base percentage"
    			},
    			tyelContractEmployerLimit: {
    				title: "tyel contract employer limit",
    				descr: "Pension contract employer six month limit"
    			},
    			tyelIncomeLimit: {
    				title: "tyel income limit",
    				descr: "Pension payment income limit per month"
    			},
    			tyelWorkerPercent: {
    				title: "tyel worker percent",
    				descr: "Pension payment base percentage - deducted from Worker salary"
    			},
    			unemploymentEmployerPercent: {
    				title: "unemployment employer percent",
    				descr: "Unemployment insurance payment percent - Employer"
    			},
    			unemploymentPartialOwnerWorkerPercent: {
    				title: "unemployment partial owner worker percent",
    				descr: "Unemployment insurance payment percent for partial owner worker - deducted from Worker"
    			},
    			unemploymentWorkerPercent: {
    				title: "unemployment worker percent",
    				descr: "Unemployment insurance payment percent - deducted from Worker"
    			},
    			unionPaymentRaksaA: {
    				title: "union payment raksa a",
    				descr: "Union payment: Rakennusliitto, A (typical)"
    			},
    			unionPaymentRaksaAoTa: {
    				title: "union payment raksa ao ta",
    				descr: "Union payment: Rakennusliitto, AO and TA"
    			}
    		},
    		YtjCompany: {
    			title: "ytj company",
    			descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			companyTypeText: {
    				title: "company type text",
    				descr: "YTJ's legal type of the company as a string. This is only to be used for display purposes.\nSource: YritysHaku.YritysMuoto"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the"
    			},
    			contactPerson: {
    				title: "contact person",
    				descr: "Names are not necessarily avaiable. If they are, they may not be reliable."
    			},
    			contactPersonPersonalId: {
    				title: "contact person personal id",
    				descr: "Social security number of the contact person in the company, if available."
    			},
    			contactPhone: {
    				title: "contact phone",
    				descr: "Mobile for or other company phone.\nDifference between Contact.Phone and ContactPhone is that\nthis (ContactPhone) is first mobile number, then company general phone if the former was null.\nContact.Phone is fisrt general phone number and mobile number only if not available."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company.\nSource: Tiedostonimi.Tiedostonimi"
    			},
    			sectorOfBusinessCode: {
    				title: "sector of business code",
    				descr: "The sector where the company is active. (Toimialatieto.Koodi)"
    			},
    			sectorOfBusinessText: {
    				title: "sector of business text",
    				descr: "Text of the sector where company operates."
    			},
    			www: {
    				title: "www",
    				descr: "Web site address"
    			},
    			ytjCompanyType: {
    				title: "ytj company type",
    				descr: "The internal YTJ code number for the company's legal type."
    			}
    		},
    		YtjSearchResult: {
    			title: "ytj search result",
    			descr: "Search result for free text search from the Finnish Business Infromation System (YTJ/BIS). Returns a list of companies without details.",
    			counter: {
    				title: "counter",
    				descr: "API quota counter"
    			},
    			isSuccess: {
    				title: "is success",
    				descr: "If true, search is a success. This may mean that no results were found though."
    			},
    			isTooGeneric: {
    				title: "is too generic",
    				descr: "If true, the searh is too generic.\nTypically either there was more than 200 results or less than 3 characters in the search string.\nBoth of these return an error in the API."
    			},
    			rawInput: {
    				title: "raw input",
    				descr: "The original search string - without trimming etc. that are done in the actual search."
    			},
    			result: {
    				title: "result",
    				descr: "Result of the search."
    			},
    			status: {
    				title: "status",
    				descr: "Technical status - for developers / admin."
    			},
    			userMessage: {
    				title: "user message",
    				descr: "A message that can be shown to the end user."
    			}
    		},
    		YtjSearchResultCompany: {
    			title: "ytj search result company",
    			descr: "Search result list level of detail of companies coming from YTJ.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			id: {
    				title: "id",
    				descr: "Numeric ID (\"Yritysavain\")"
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the company. This is a free text type. Not necessary parseable to company type enumeration in details."
    			}
    		},
    		CoreDictionaryLang: {
    			title: "core dictionary lang",
    			descr: "Defines the current structure for the core dictionary for each language.\nLanugages are fi, en, sv and sx.",
    			SALAXY: {
    				title: "s a l a x y",
    				descr: "SALAXY is the dictionary root object for each language."
    			}
    		}
    	}
    };

    var SALAXY$j = {
    	UI_TERMS: {
    		add: "Lisää",
    		addNew: "Lisää uusi",
    		addRow: "Lisää rivi",
    		areYouSure: "Oletko varma?",
    		back: "Takaisin",
    		calculate: "Laske",
    		calculations: "Laskelmat",
    		cancel: "Peruuta",
    		cancelEditing: "Peru muokkaaminen",
    		choose: "Valitse...",
    		chooseWorkDateRange: "Määritä työn alku- ja loppupäivät",
    		chooseWorkDuration: "Määritä työn kesto...",
    		clear: "Tyhjennä",
    		close: "Sulje",
    		confirm: "Vahvista",
    		copy: "Kopio",
    		copyAsNew: "Kopioi uudeksi",
    		copyAsRequest: "Kopio pyynnöksi",
    		copyToClipboard: "Kopioi",
    		dateFormats: {
    			dm: "pv.kk."
    		},
    		dateNotChosen: "päivämäärä ei valittuna",
    		"delete": "Poista",
    		demo: "KOKEILUVERSIO",
    		dismiss: "Hylkää",
    		done: "Valmis",
    		edit: "Muokkaa",
    		gotIt: "Ymmärrän",
    		isDeleting: "Poistetaan...",
    		isSaving: "Tallennetaan...",
    		learnMore: "Lue lisää",
    		loading: "Ladataan",
    		login: "Kirjaudu",
    		logout: "Kirjaudu ulos",
    		logoutAlt: "Päätä nykyinen sessio",
    		makeOrder: "Tee tilaus",
    		minCharacters: "Kirjoita vähintään 3 merkkiä...",
    		monthHalf: "½ kuukautta",
    		monthShort0: "joulu",
    		monthShort1: "tammi",
    		monthShort10: "loka",
    		monthShort11: "marras",
    		monthShort12: "joulu",
    		monthShort2: "helmi",
    		monthShort3: "maalis",
    		monthShort4: "huhti",
    		monthShort5: "touko",
    		monthShort6: "kesä",
    		monthShort7: "heinä",
    		monthShort8: "elo",
    		monthShort9: "syys",
    		moreInfo: "Lisätietoja",
    		navigation: "Navigaatio",
    		newCalculation: "Uusi laskelma",
    		next: "Seuraava",
    		no: "Ei",
    		noteRequiredField: "* Pakollinen kenttä",
    		noThanks: "Ei kiitos",
    		ok: "OK",
    		oldVersion: "vanha versio",
    		orChoose: "...tai valitse...",
    		otherPeriod: "Muu periodi",
    		pay: "Maksa",
    		pleaseWait: "Ole hyvä ja odota...",
    		postpone: "Ei nyt",
    		previous: "Edellinen",
    		printable: "Tuloste",
    		recalculate: "Päivitä laskelma",
    		register: "Rekisteröidy",
    		save: "Tallenna",
    		saveEdited: "Tallenna muutokset",
    		search: "Hae...",
    		searchByName: "Hae nimeltä...",
    		select: "Valitse",
    		selectAlt: "Valitse...",
    		sendEmail: "Lähetä sähköposti",
    		show: "Näytä",
    		skip: "Ohita",
    		sureToDeleteRecord: "Haluatko varmasti poistaa tämän tietueen?",
    		today: "Tämä päivä",
    		updateCalculation: "Päivitä laskelma",
    		updatingCalculation: "Päivitetään laskelmaa...",
    		use: "Ota käyttöön",
    		week2: "2 viikkoa",
    		yes: "Kyllä",
    		timeToday: "Tänään",
    		timeYesterday: "Eilen"
    	}
    };

    var SALAXY$i = {
    	VALIDATION: {
    		_notes: "Please note that all calc or payroll specific messages shoud be placed under Calc or Payroll -nodes. Messages in the Validation-root are obsolete.",
    		Files: {
    			ContentTypeError: "Tiedostomuotoa ei hyväksytä",
    			Empty: "Ladattava tiedosto on tyhjä",
    			MaxFileSize: "Ladattava tiedosto on liian suuri",
    			NoFiles: "Kutsun mukana ei tullut yhtään tiedostoa.",
    			SaveError: "Tuntematon virhe tiedoston tallennuksessa."
    		},
    		IllegalWorkerForCompanyType: {
    			description: "Yksityinen elinkeinonharjoittaja eli toiminimi ei voi maksaa itselleen palkkaa. Palkanmaksu hoidetaan yksityisnostoina. Kirjanpitäjäsi voi neuvoa sinua miten toimia.",
    			label: "Ei-sallittu palkka"
    		},
    		IncompleteAdvancePayment: {
    			description: "Palkkaennakko laskelmalla on liian suuri. Korjaa palkkaennakko pienemmäksi tai bruttopalkka suuremmaksi. Mahdolliset ennakkoon maksetut kulukorvaukset tulee ilmoittaa omalla palkkalajilla.",
    			label: "Palkka ei riitä ilmoitettuun palkkaennakkoon"
    		},
    		IncompleteForeclosure: {
    			description: "Rahapalkka ei riitä ulosmittauksen tekemiseen. Tarkista ulosmittauksen määrä maksukiellosta ja muista huomioida maksukiellossa ilmoitettu suojaosuus.",
    			label: "Palkka ei riitä ulosmittauksen tekemiseen"
    		},
    		IncompleteOtherDeductions: {
    			description: "Palkka ei riitä ilmoitettuihin muihin vähennyksiin",
    			label: "Palkka ei riitä"
    		},
    		IncompletePensionPayment: {
    			description: "Palkka ei riitä työntekijän osuuteen TyEL-maksusta",
    			label: "Palkka ei riitä"
    		},
    		IncompletePrepaidExpenses: {
    			description: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri. Ennakkoon maksettu kulukorvaus ylittää laskelmalle lisätyt kulukorvaukset.",
    			label: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri"
    		},
    		IncompleteUnemploymentInsurancePayment: {
    			description: "Palkka ei riitä työntekijän osuuteen työttömyysvakuutusmaksusta",
    			label: "Palkka ei riitä"
    		},
    		IncompleteUnionPayment: {
    			description: "Palkka ei riitä AY-jäsenmaksuun",
    			label: "Palkka ei riitä"
    		},
    		MissingOccupationCode: {
    			description: "Työn tyyppi puuttuu. Lisää se kohdassa 1 hakemalla Tilastokeskuksen luokituksesta. Ammattiluokitus on pakollinen tieto Tulorekisterin palkkatietoilmoituksessa.",
    			label: "Ammattiluokitus puuttuu"
    		},
    		MissingWorkerIbanNumber: {
    			label: "Tilinumero puuttuu",
    			description: "Työntekijän tiedoista puuttuu tilinumero (IBAN-muodossa)."
    		},
    		NegativeRowTotalNotSupported: {
    			description: "Rivin yhteenlaskettu summa on negatiivinen. Rivillä ei sallita negatiivista summaa.",
    			label: "Rivillä ei sallita negatiivista summaa"
    		},
    		Calc: {
    			IllegalWorkerForCompanyType: {
    				description: "Yksityinen elinkeinonharjoittaja eli toiminimi ei voi maksaa itselleen palkkaa. Palkanmaksu hoidetaan yksityisnostoina. Kirjanpitäjäsi voi neuvoa sinua miten toimia.",
    				label: "Ei-sallittu palkka"
    			},
    			IncompleteAdvancePayment: {
    				description: "Palkkaennakko laskelmalla on liian suuri. Korjaa palkkaennakko pienemmäksi tai bruttopalkka suuremmaksi. Mahdolliset ennakkoon maksetut kulukorvaukset tulee ilmoittaa omalla palkkalajilla.",
    				label: "Palkka ei riitä ilmoitettuun palkkaennakkoon"
    			},
    			IncompleteForeclosure: {
    				description: "Rahapalkka ei riitä ulosmittauksen tekemiseen. Tarkista ulosmittauksen määrä maksukiellosta ja muista huomioida maksukiellossa ilmoitettu suojaosuus.",
    				label: "Palkka ei riitä ulosmittauksen tekemiseen"
    			},
    			IncompleteOtherDeductions: {
    				description: "Palkka ei riitä ilmoitettuihin muihin vähennyksiin",
    				label: "Palkka ei riitä"
    			},
    			IncompletePensionPayment: {
    				description: "Palkka ei riitä työntekijän osuuteen TyEL-maksusta",
    				label: "Palkka ei riitä"
    			},
    			IncompletePrepaidExpenses: {
    				description: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri. Ennakkoon maksettu kulukorvaus ylittää laskelmalle lisätyt kulukorvaukset.",
    				label: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri"
    			},
    			IncompleteUnemploymentInsurancePayment: {
    				description: "Palkka ei riitä työntekijän osuuteen työttömyysvakuutusmaksusta",
    				label: "Palkka ei riitä"
    			},
    			IncompleteUnionPayment: {
    				description: "Palkka ei riitä AY-jäsenmaksuun",
    				label: "Palkka ei riitä"
    			},
    			MissingOccupationCode: {
    				description: "Työn tyyppi puuttuu. Lisää se kohdassa 1 hakemalla Tilastokeskuksen luokituksesta. Ammattiluokitus on pakollinen tieto Tulorekisterin palkkatietoilmoituksessa.",
    				label: "Ammattiluokitus puuttuu"
    			},
    			MissingWorkerIbanNumber: {
    				label: "Työntekijän tilinumero puuttuu",
    				description: "Työntekijän tiedoista puuttuu tilinumero (IBAN-muodossa)."
    			},
    			InvalidWorkerIbanNumber: {
    				label: "Työntekijän tilinumero on virheellinen",
    				description: "Työntekijän tiedoissa on virheellinen tilinumero (IBAN-muodossa)."
    			},
    			NegativeRowTotalNotSupported: {
    				description: "Rivin yhteenlaskettu summa on negatiivinen. Rivillä ei sallita negatiivista summaa.",
    				label: "Rivillä ei sallita negatiivista summaa"
    			},
    			MissingWorkerContactStreet: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen katuosoite puuttuu"
    			},
    			MissingWorkerContactCity: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen postitoimipaikka puuttuu"
    			},
    			MissingWorkerContactPostalCode: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen postinumero puuttuu"
    			},
    			MissingWorker: {
    				label: "Laskelmalta puuttuu työntekijä"
    			},
    			MissingWorkerFirstName: {
    				label: "Työntekijän etunimi puuttuu"
    			},
    			MissingWorkerLastName: {
    				label: "Työntekijän sukunimi puuttuu"
    			},
    			MissingWorkerSocialSecurityNumber: {
    				label: "Työntekijän henkilötunnus puuttuu"
    			},
    			MissingWorkerSocialSecurityNumber_InTest: {
    				label: "Työntekijän henkilötunnus puuttuu. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    			},
    			InvalidWorkerSocialSecurityNumber: {
    				label: "Työntekijän henkilötunnus on virheellinen"
    			},
    			InvalidWorkerSocialSecurityNumber_InTest: {
    				label: "Työntekijän henkilötunnus on virheellinen. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    			},
    			InvalidCarBenefitRow: {
    				label: "Autoedusta puuttuu auton ikäryhmä ja/tai autoedun tyyppi"
    			},
    			InvalidRemunerationRow: {
    				label: "Kertaluontoisen palkkion tulolaji puuttuu"
    			}
    		},
    		Payroll: {
    			Calc: {
    				EmailAddress: {
    					label: "Sähköposti on virheellinen"
    				},
    				FirstNameNotEmpty: {
    					label: "Työntekijän etunimi puuttuu"
    				},
    				IbanInvalid: {
    					label: "Tilinumero on virheellinen (IBAN-muoto)"
    				},
    				IbanNotEmpty: {
    					label: "Tilinumero puuttuu"
    				},
    				InvalidStatusForPayroll: {
    					label: "Palkkalistan laskelmalla väärä status."
    				},
    				LastNameNotEmpty: {
    					label: "Työntekijän sukunimi puuttuu"
    				},
    				PayrollIdConflict: {
    					label: "Laskelman palkkalista-tunnus on virheellinen."
    				},
    				SocialSecurityNumber_InTest: {
    					label: "Henkilötunnus on virheellinen. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    				},
    				SocialSecurityNumber: {
    					label: "Henkilötunnus on virheellinen."
    				},
    				TaxCardType: {
    					label: "Verokortti puuttuu tai tyyppiä ei ole määritetty."
    				},
    				TaxPercent: {
    					label: "Veroprosenttia ei ole määritetty"
    				},
    				TelephoneOrEmailNotEmpty: {
    					label: "Anna jompikumpi tai molemmat yhteystiedot (puhelinnumero ja sähköposti)."
    				},
    				TotalGreaterThan0: {
    					label: "Palkkalaskelman summa on nolla"
    				}
    			},
    			NoCalculations: {
    				label: "Ei yhtään laskelmaa."
    			}
    		},
    		SharedTaxcard: {
    			description: "Työntekijäsi on jakanut uuden verokortin ja toivoo, että sitä käytetään palkanmaksuun. Tarkista ja hyväksy verokortti palkkalaskurin ruudun 1. Työntekijä kautta ennen palkan maksamista, jotta ennakonpidätys päivittyy laskelmalle oikein.",
    			label: "Uusi jaettu verokortti!"
    		},
    		ValidationErrors: {
    			unknown: "Validointivirhe: {{error}}",
    			maxlength: "Pituus enintään {{maxlength}} merkkiä",
    			minlength: "Pituus vähintään {{minlength}} merkkiä",
    			required: "Kenttä on pakollinen",
    			sxyCompanyIdFi: "Y-tunnuksen muoto on väärä",
    			sxyCurrency: "Rahamäärän muoto on väärä",
    			sxyEmail: "Sähköpostin muoto on väärä",
    			sxyIban: "TIlinumeron (IBAN) muoto on väärä",
    			sxyInteger: "Kentän pitää olla kokonaisluku",
    			sxyMobilePhone: "Puhelinnumeron muoto on väärä",
    			sxyNumber: "Kentän pitää olla numero",
    			sxyPensionContractNumber: "Tarkista sopimusnumero",
    			sxyPersonalIdFi: "Henkilötunnuksen muoto on väärä",
    			sxyOfficialIdFi: "Y-tunnuksen tai henkilötunnuksen muoto on väärä",
    			sxyPostalCodeFi: "Postinumeron muoto on väärä",
    			sxySmsVerificationCode: "Tarkista varmennuskoodi",
    			sxyTaxPercent: "Veroprosentin muoto on väärä",
    			sxyTemporaryPensionContractNumber: "Syötit tilapäisen sopimuksen numeron."
    		}
    	}
    };

    var SALAXY$h = {
    	ENUM: {
    		EarningsPaymentReportStatus: {
    			"new": {
    				label: "Uusi"
    			},
    			succeeded: {
    				label: "Onnistuneesti lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			},
    			invalid: {
    				label: "Tarkastettava"
    			},
    			scheduled: {
    				label: "Ajastettu"
    			}
    		},
    		PayerSummaryReportStatus: {
    			"new": {
    				label: "Uusi"
    			},
    			succeeded: {
    				label: "Onnistuneesti lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			},
    			invalid: {
    				label: "Tarkastettava"
    			},
    			scheduled: {
    				label: "Ajastettu"
    			}
    		},
    		SummaryTransactionCode: {
    			noWagesPayable: {
    				label: "Ei palkanmaksua",
    				description: "Tieto annetaan, jos maksajalla ei ole kyseisen kuukauden aikana ollut laisinkaan palkanmaksua. Jos maksaja on maksanut palkkaa ko. kuukauden aikana, tietoa ei anneta. Ei palkanmaksua -tieto voidaan ilmoittaa kuudelta kalenterikuukaudelta etukäteen, jos tiedetään, ettei seuraavan puolen vuoden aikana ole tarkoitus palkata ketään. Tieto on annettava vain, jos maksaja on rekisteröitynyt säännöllisesti palkkoja maksavaksi työnantajaksi Verohallinnon työnantajarekisteriin. Ei palkanmaksua-tietoa ei voida antaa aliorganisaatioittain. Tieto annetaan vain, jos suorituksen maksaja ei ole maksanut miltään osin palkkaa ko. kuukauden aikana."
    			},
    			employersHealthInsuranceContribution: {
    				label: "Sairausvakuutusmaksu",
    				description: "Maksaja ilmoittaa kyseisen kuukauden aikana maksamisensa palkkojen perusteella laskemansa työnantajan sairausvakuutusmaksun määrän. Kohdassa ei ilmoiteta palkkojen yhteismäärää, vaan palkoista lasketun työnantajan sairausvakuutusmaksun yhteismäärä ilmoituskuukauden ajalta. Summasta ei vähennetä Työnantajan sairausvakuutusmaksusta tehtävät vähennykset -tulolajilla ilmoitettuja vähennyksiä."
    			},
    			deductionsToBeMadeFromTheEmployersHealthInsuranceContribution: {
    				label: "Vähennykset SAVA-maksusta",
    				description: "Tiedossa ilmoitetaan maksajan oma-aloitteisesti sairausvakuutusmaksuista tehdyt vähennykset. Työnantaja voi tehdä vähennyksen esimerkiksi silloin, jos se on maksanut työntekijälle palkkaa ja samaan aikaan työnantajalle on maksettu lakiin perustuvaa eläkettä, päivärahaa tai muuta ansionmenetystä korvaavaa etuutta, joka on myönnetty työntekijälle. Työnantajalla on oikeus saada maksamansa työnantajan sairausvakuutusmaksu Verohallinnolta takaisin siltä osin, kuin se vastaa työnantajalle maksettua suoritusta."
    			}
    		},
    		ReportLogEventType: {
    			"undefined": {
    				label: "Määrittämätön"
    			},
    			sent: {
    				label: "Lähetetty"
    			},
    			canceled: {
    				label: "Mitätöity"
    			},
    			error: {
    				label: "Virheellinen"
    			}
    		},
    		DeliveryDataType: {
    			earningsPaymentReports: {
    				label: "Palkkatietoilmoitukset"
    			},
    			employersSeparateReports: {
    				label: "Työnantajan erillisilmoitukset"
    			},
    			benefitsPaymentReports: {
    				label: "Etuustietoilmoitukset"
    			},
    			recordSubscription: {
    				label: "Aineistotilaus"
    			},
    			cancellationOfEarningsPaymentReports: {
    				label: "Palkkatietoilmoitusten mitätöiminen"
    			},
    			cancellationOfEmployersSeparateReports: {
    				label: "Työnantajan erillisilmoitusten mitätöiminen"
    			},
    			cancellationOfBenefitsPaymentReports: {
    				label: "Etuustietoilmoitusten mitätöiminen"
    			},
    			cancellationOfARecordSubscription: {
    				label: "Aineistotilauksen mitätöiminen"
    			},
    			cancellationOfARecordContainingEarningsPaymentReports: {
    				label: "Palkkatietoilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingEmployersSeparateReports: {
    				label: "Työnantajan erillisilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingBenefitsPaymentReports: {
    				label: "Etuustietoilmoituksia sisältävän aineiston mitätöiminen"
    			},
    			cancellationOfARecordContainingARecordSubscription: {
    				label: "Aineistotilauksen sisältävän aineiston mitätöiminen"
    			}
    		},
    		IdType: {
    			businessId: {
    				label: "Y-tunnus"
    			},
    			personalIdentificationNumber: {
    				label: "Henkilötunnus (suomalainen)"
    			},
    			vat: {
    				label: "Alv-tunniste (VAT)"
    			},
    			giin: {
    				label: "GIIN-tunniste"
    			},
    			taxIdentificationNumber: {
    				label: "Verotunniste(TIN)"
    			},
    			finnishTradeRegistrationNumber: {
    				label: "Kaupparekisteritunnus"
    			},
    			foreignBusinessRegistrationNumber: {
    				label: "Ulkomainen yritystunnus"
    			},
    			foreignPersonalIdentificationNumber: {
    				label: "Ulkomainen henkilötunnus"
    			},
    			other: {
    				label: "Muu tunnus"
    			}
    		},
    		ResponsibilityCode: {
    			contentIssues: {
    				label: "Sisällölliset asiat"
    			},
    			technicalIssues: {
    				label: "Tekniset asiat"
    			}
    		},
    		PayerType: {
    			publicSector: {
    				label: "Julkisyhteisö"
    			},
    			household: {
    				label: "Kotitalous"
    			},
    			temporaryEmployer: {
    				label: "Tilapäinen työnantaja"
    			},
    			foreignEmployer: {
    				label: "Ulkomainen työnantaja"
    			},
    			state: {
    				label: "Valtio"
    			},
    			unincorporatedStateEnterpriseOrGovernmentalInstitution: {
    				label: "Valtion liikelaitos tai erillishallinnollinen valtion laitos"
    			},
    			specialisedAgency: {
    				label: "Kansainvälinen erityisjärjestö"
    			},
    			foreignGroupCompany: {
    				label: "Ulkomainen konserniyhtiö"
    			}
    		},
    		PayerSubOrgType: {
    			kevasSubmitterCodes: {
    				label: "Kevan ilmoittajakoodisto"
    			},
    			payersOwnCodes: {
    				label: "Maksajan oma koodisto"
    			},
    			governmentAgencyIdentifier: {
    				label: "Virastotunniste"
    			}
    		},
    		AddressType: {
    			addressInHomeCountry: {
    				label: "Osoite kotivaltiossa"
    			},
    			addressInTheCountryOfWork: {
    				label: "Osoite työskentelyvaltiossa"
    			}
    		},
    		ProfessionType: {
    			statisticsFinland: {
    				label: "Tilastokeskuksen ammattiluokitus (TK10)"
    			},
    			keva: {
    				label: "Kevan nimikkeistö"
    			},
    			bankOfFinland: {
    				label: "Suomen Pankin nimikkeistö"
    			},
    			trafi: {
    				label: "Trafin nimikkeistö"
    			}
    		},
    		EmploymentCode: {
    			fullTime: {
    				label: "Kokoaikainen"
    			},
    			partTime: {
    				label: "Osa-aikainen"
    			},
    			notAvailable: {
    				label: "Tietoa ei ole saatavilla"
    			}
    		},
    		TermCode: {
    			untilFurtherNotice: {
    				label: "Toistaiseksi voimassa oleva"
    			},
    			fixedTerm: {
    				label: "Määräaikainen"
    			}
    		},
    		PaymentType: {
    			monthly: {
    				label: "Kuukausipalkka"
    			},
    			hourly: {
    				label: "Tuntipalkka"
    			},
    			contractPay: {
    				label: "Urakkapalkka"
    			}
    		},
    		EmploymentEndingType: {
    			kevaCodes: {
    				label: "Kevan koodisto"
    			},
    			bankOfFinlandCodes: {
    				label: "Suomen Pankin koodisto"
    			},
    			incomesRegisterCodes: {
    				label: "Tulorekisterin koodisto"
    			}
    		},
    		PensionActCode: {
    			employeesEarningsRelatedPensionInsurance: {
    				label: "Työntekijän työeläkevakuutus"
    			},
    			pensionInsuranceForFarmers: {
    				label: "Maatalousyrittäjän eläkevakuutus (MYEL)"
    			},
    			pensionInsuranceForTheSelfEmployed: {
    				label: "Yrittäjän eläkevakuutus (YEL)"
    			}
    		},
    		PensionProvIdCode: {
    			none: {
    				label: "Ei valintaa"
    			},
    			finnishCentreForPensions: {
    				label: "Eläketurvakeskus"
    			},
    			clergyOfTheOrthodoxChurch: {
    				label: "Ortodoksisen kirkon papiston eläkekassa"
    			},
    			kevaMemberOrganisations: {
    				label: "Kevan jäsenyhteisöt"
    			},
    			"kevaÅlandLandskapsregeringsPensionssystem": {
    				label: "Keva - Åland landskapsregerings pensionssystem"
    			},
    			kevaChurch: {
    				label: "Keva - kirkko"
    			},
    			bankOfFinland: {
    				label: "Suomen Pankki"
    			},
    			kevaKelaEmploymentPensions: {
    				label: "Keva - Kelan toimisuhde-eläkkeet"
    			},
    			KevaState: {
    				label: "Keva - valtio"
    			},
    			seafarers: {
    				label: "Merimieseläkekassa"
    			},
    			farmers: {
    				label: "Maatalousyrittäjien eläkelaitos"
    			},
    			alandia: {
    				label: "Ålandia"
    			},
    			ilmarinen: {
    				label: "Keskinäinen Eläkevakuutusyhtiö Ilmarinen"
    			},
    			elo: {
    				label: "Keskinäinen Työeläkevakuutusyhtiö Elo"
    			},
    			varma: {
    				label: "Keskinäinen työeläkevakuutusyhtiö Varma"
    			},
    			veritas: {
    				label: "Eläkevakuutusosakeyhtiö Veritas"
    			},
    			uPM: {
    				label: "UPM Sellutehtaiden eläkesäätiö"
    			},
    			sanoma: {
    				label: "Sanoman Eläkesäätiö"
    			},
    			sandvik: {
    				label: "Sandvik Eläkesäätiö"
    			},
    			kontino: {
    				label: "Kontinon yhteiseläkesäätiö"
    			},
    			yleisradio: {
    				label: "Yleisradion eläkesäätiö"
    			},
    			aBB: {
    				label: "ABB Eläkesäätiö"
    			},
    			lFashionGroup: {
    				label: "L-Fashion Group Oy:n eläkesäätiö"
    			},
    			honeywell: {
    				label: "Honeywell Oy:n Henkilökunnan Eläkesäätiö"
    			},
    			telia: {
    				label: "Telian Eläkesäätiö"
    			},
    			yaraSuomi: {
    				label: "Yara Suomen Eläkesäätiö"
    			},
    			orion: {
    				label: "Orionin Eläkesäätiö"
    			},
    			valio: {
    				label: "Valion Eläkekassa"
    			},
    			oP: {
    				label: "OP-Eläkekassa"
    			},
    			verso: {
    				label: "Eläkekassa Verso"
    			},
    			apteekkien: {
    				label: "Apteekkien Eläkekassa"
    			},
    			viabek: {
    				label: "Liikennepalvelualojen Eläkekassa Viabek"
    			},
    			reka: {
    				label: "Reka Eläkekassa"
    			},
    			etera: {
    				label: "Etera"
    			}
    		},
    		PaidAbsenceCauseCode: {
    			illness: {
    				label: "Sairaus"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			midWeekHoliday: {
    				label: "Arkipyhä"
    			},
    			accruedHoliday: {
    				label: "Pekkasvapaa"
    			},
    			occupationalAccident: {
    				label: "Työtapaturma"
    			},
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Osittainen poissaolo kuntoutuksen vuoksi"
    			},
    			other: {
    				label: "Muu syy"
    			}
    		},
    		UnpaidAbsenceCauseCode: {
    			illness: {
    				label: "Sairaus"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			jobAlternationLeave: {
    				label: "Vuorotteluvapaa"
    			},
    			studyLeave: {
    				label: "Opintovapaa"
    			},
    			industrialAction: {
    				label: "Lakko tai työsulku"
    			},
    			interruptionInWorkProvision: {
    				label: "Työtarjonnan keskeytyminen"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			militaryService: {
    				label: "Varusmiespalvelus tai siviilipalvelus"
    			},
    			layOff: {
    				label: "Lomautus"
    			},
    			childCareLeave: {
    				label: "Hoitovapaa"
    			},
    			other: {
    				label: "Muu syy"
    			}
    		},
    		AllowanceCode: {
    			mealAllowance: {
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				label: "Osapäiväraha"
    			},
    			fullDailyAllowance: {
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				label: "Ulkomaan päiväraha"
    			},
    			taxExemptReimbursementsAbroad: {
    				label: "Ulkomaan työskentelyyn liittyvät verovapaat korvaukset"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				label: "Asuntoetu"
    			},
    			telephoneBenefit: {
    				label: "Puhelinetu"
    			},
    			mealBenefit: {
    				label: "Ravintoetu"
    			},
    			otherBenefits: {
    				label: "Muut edut"
    			}
    		},
    		InsuranceCode: {
    			subjectToSocialInsuranceContributions: {
    				label: "Sosiaalivakuutusmaksujen alainen (koodit 2-6)"
    			},
    			subjectToEarningsRelatedPensionInsuranceContribution: {
    				label: "Työeläkevakuutusmaksun alainen"
    			},
    			subjectToHealthInsuranceContribution: {
    				label: "Sairausvakuutusmaksun alainen"
    			},
    			subjectToUnemploymentInsuranceContribution: {
    				label: "Työttömyysvakuutusmaksun alainen"
    			},
    			subjectToAccidentInsuranceAndOccupationalDiseaseInsuranceContribution: {
    				label: "Työtapaturma- ja ammattitautivakuutusmaksun alainen"
    			}
    		},
    		WageUnitCode: {
    			hour: {
    				label: "Tunti"
    			},
    			day: {
    				label: "Päivä"
    			},
    			week: {
    				label: "Viikko"
    			},
    			period: {
    				label: "Jakso"
    			}
    		},
    		ReimbPaymentType: {
    			kelaDailyAllowanceApplication: {
    				label: "Kela, päivärahahakemus"
    			},
    			kelaFamilyLeaveCompensation: {
    				label: "Kela, perhevapaakorvaus"
    			}
    		},
    		ScheduleAction: {
    			send: {
    				label: "Lähetä"
    			},
    			cancel: {
    				label: "Mitätöi"
    			}
    		},
    		ActionCode: {
    			newReport: {
    				label: "Uusi ilmoitus"
    			},
    			replacementReport: {
    				label: "Korvaava ilmoitus"
    			}
    		},
    		Template: {
    			"": {
    				label: ""
    			}
    		}
    	}
    };

    var SALAXY$g = {
    	ENUM: {
    		ApiItemType: {
    			"undefined": {
    				description: "",
    				label: "Valitse..."
    			},
    			calculation: {
    				description: "",
    				label: "Laskelma"
    			},
    			calculationPaid: {
    				description: "",
    				label: "Maksettu laskelma"
    			},
    			payrollDetails: {
    				description: "",
    				label: "Palkkalista"
    			},
    			invoice: {
    				description: "",
    				label: "Lasku / palkanmaksu"
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "Palkkatietoilmoitus tulorekisteriin"
    			},
    			payerSummaryReport: {
    				description: "",
    				label: "Kuukausi-ilmoitus tulorekisteriin"
    			},
    			employment: {
    				description: "",
    				label: "Työsuhde"
    			},
    			messageThread: {
    				description: "",
    				label: "Viestiketju"
    			},
    			calendarEvent: {
    				description: "",
    				label: "Kalenteritapahtuma"
    			}
    		},
    		CalendarActionType: {
    			"undefined": {
    				description: "",
    				label: "Valitse..."
    			},
    			display: {
    				description: "",
    				label: "Notifikaatio"
    			},
    			email: {
    				description: "",
    				label: "Sähköposti"
    			},
    			audio: {
    				description: "",
    				label: "EI käytössä: Ääniviesti"
    			},
    			createItem: {
    				description: "",
    				label: "Tietueen luonti"
    			},
    			script: {
    				description: "",
    				label: "Automaattinen skripti"
    			}
    		},
    		Weekday: {
    			day1: {
    				label: "Maanantai",
    				short: "ma"
    			},
    			day2: {
    				label: "Tiistai",
    				short: "ti"
    			},
    			day3: {
    				label: "Keskiviikko",
    				short: "ke"
    			},
    			day4: {
    				label: "Torstai",
    				short: "to"
    			},
    			day5: {
    				label: "Perjantai",
    				short: "pe"
    			},
    			day6: {
    				label: "Lauantai",
    				short: "la"
    			},
    			day7: {
    				label: "Sunnuntai",
    				short: "su"
    			}
    		},
    		AbcSection: {
    			association: {
    				description: "",
    				label: "Yhdistyksille"
    			},
    			blog: {
    				description: "",
    				label: "Blogi"
    			},
    			businessOwner: {
    				description: "",
    				label: "Yrittäjä"
    			},
    			documentTemplates: {
    				description: "",
    				label: "Dokumenttipohjat ja lomakkeet"
    			},
    			doNotShow: {
    				description: "",
    				label: "Ei näytetä käyttäjälle (vain Admin)"
    			},
    			employee: {
    				description: "",
    				label: "Työntekijä"
    			},
    			employment: {
    				description: "",
    				label: "Työsuhde ja työsopimus"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Yritys tai yhteisö"
    			},
    			examples: {
    				description: "",
    				label: "Esimerkkejä palkkaustilanteista"
    			},
    			householdEmployer: {
    				description: "",
    				label: "Kotitalous"
    			},
    			instructionsAndExamples: {
    				description: "",
    				label: "Käyttöohjeet"
    			},
    			insurance: {
    				description: "",
    				label: "Vakuutukset ja eläke"
    			},
    			palkkausGeneral: {
    				description: "",
    				label: "Tietoa Palkkaus.fi -palvelusta"
    			},
    			palkkausInstructions: {
    				description: "",
    				label: "Käyttöohjeita"
    			},
    			personEmployer: {
    				description: "",
    				label: "Kotitalous työnantajana"
    			},
    			press: {
    				description: "",
    				label: "Tiedotteet ja materiaali"
    			},
    			productLongDescription: {
    				description: "",
    				label: "Tuotekuvaus"
    			},
    			salary: {
    				description: "",
    				label: "Palkanlaskenta ja palkanmaksu"
    			},
    			"undefined": {
    				description: "",
    				label: "Uudet artikkelit / ei määritetty"
    			},
    			worker: {
    				description: "",
    				label: "Työntekijälle"
    			}
    		},
    		AbsenceCauseCode: {
    			accruedHoliday: {
    				label: "Pekkasvapaa"
    			},
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			childCareLeave: {
    				label: "Hoitovapaa"
    			},
    			childIllness: {
    				label: "Lapsen sairaus tai pakottava perhesyy"
    			},
    			illness: {
    				label: "Sairaus"
    			},
    			industrialAction: {
    				label: "Lakko tai työsulku"
    			},
    			interruptionInWorkProvision: {
    				label: "Työtarjonnan keskeytyminen"
    			},
    			jobAlternationLeave: {
    				label: "Vuorotteluvapaa"
    			},
    			layOff: {
    				label: "Lomautus"
    			},
    			leaveOfAbsence: {
    				label: "Virkavapaa"
    			},
    			midWeekHoliday: {
    				label: "Arkipyhä"
    			},
    			militaryRefresherTraining: {
    				label: "Kertausharjoitus"
    			},
    			militaryService: {
    				label: "Varusmiespalvelus tai siviilipalvelus"
    			},
    			occupationalAccident: {
    				label: "Työtapaturma"
    			},
    			other: {
    				label: "Muu syy"
    			},
    			parentalLeave: {
    				label: "Äitiys-, isyys- ja vanhempainvapaa"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Osittainen poissaolo kuntoutuksen vuoksi"
    			},
    			partTimeChildCareLeave: {
    				label: "Osittainen hoitovapaa"
    			},
    			partTimeSickLeave: {
    				label: "Osa-aikainen sairauspoissaolo"
    			},
    			personalReason: {
    				label: "Henkilökohtainen muu syy"
    			},
    			rehabilitation: {
    				label: "Kuntoutus"
    			},
    			specialMaternityLeave: {
    				label: "Erityisäitiysvapaa"
    			},
    			studyLeave: {
    				label: "Opintovapaa"
    			},
    			training: {
    				label: "Koulutus"
    			},
    			unpaidLeave: {
    				label: "Palkaton loma / vapaa"
    			}
    		},
    		AgeGroupCode: {
    			a: {
    				label: "Ikäryhmä A"
    			},
    			b: {
    				label: "Ikäryhmä B"
    			},
    			c: {
    				label: "Ikäryhmä C"
    			},
    			u: {
    				label: "Ikäryhmä U"
    			}
    		},
    		AgeRange: {
    			age_15: {
    				description: "",
    				label: "-15v"
    			},
    			age16: {
    				description: "",
    				label: "16v"
    			},
    			age17: {
    				description: "",
    				label: "17v"
    			},
    			age18_52: {
    				description: "",
    				label: "18-52v"
    			},
    			age53_62: {
    				description: "",
    				label: "53-62v"
    			},
    			age63_64: {
    				description: "",
    				label: "63-64v"
    			},
    			age65_67: {
    				description: "",
    				label: "65-67v"
    			},
    			age68AndOVer: {
    				description: "",
    				label: "68v-"
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		AllowanceCode: {
    			expensesWorkingAbroad: {
    				label: "Ulkomaan työskentelyyn liittyvät verovapaat korvaukset"
    			},
    			fullDailyAllowance: {
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				label: "Ulkomaan päiväraha"
    			},
    			mealAllowance: {
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				label: "Osapäiväraha"
    			},
    			taxExemptReimbursementsAbroad: {
    				description: "",
    				label: "TaxExemptReimbursementsAbroad"
    			}
    		},
    		AnnualLeavePaymentKind: {
    			draftCalc: {
    				description: "Palkkalaskelman luonnos, jossa lomaan liittyviä maksuja",
    				label: "Palkkalaskelman luonnos"
    			},
    			endSaldo: {
    				description: "Seuraavalle lomavuodelle tai maksettavaa",
    				label: "Maksettavaa tai siirrettävää"
    			},
    			info: {
    				description: "Lomavuodella ei ole maksuja",
    				label: "Ei rivejä"
    			},
    			manualBonus: {
    				description: "Lomaraha maksettuna erikseen maksettuna erillisenä korjauksena",
    				label: "Lomaraha"
    			},
    			manualCompensation: {
    				description: "Korvattavat lomapäivät ja niiden korvaus sekä lomaraha manuaalisena korjauksena",
    				label: "Lomakorvaus"
    			},
    			manualSalary: {
    				description: "Pidetyt lomapäivät ja niiden palkka sekä lomaraha manuaalisena korjauksena",
    				label: "Loma-ajan palkka"
    			},
    			paidCalc: {
    				description: "Palkkaus.fi-palvelussa maksettu palkka, jossa lomaan liittyviä maksuja",
    				label: "Maksettu palkkalaskelma"
    			},
    			startSaldo: {
    				description: "Edellisenä vuonna kertyneet lomat",
    				label: "Käytettävissä olevat"
    			},
    			total: {
    				description: "Käytetyt/maksetut lomapäivät, -rahat ja korvaukset",
    				label: "Yhteensä"
    			},
    			"undefined": {
    				description: "Rivin tyyppiä ei ole valittu",
    				label: "Valitse..."
    			},
    			planned: {
    				description: "",
    				label: "Planned"
    			}
    		},
    		ApiTestErrorType: {
    			"default": {
    				description: "",
    				label: "Oletus"
    			},
    			userFriendly: {
    				description: "",
    				label: "User Friendly"
    			}
    		},
    		ApiValidationErrorType: {
    			general: {
    				label: "Yleisvirhe"
    			},
    			invalid: {
    				label: "Virhe"
    			},
    			required: {
    				label: "Arvo puuttuu"
    			},
    			warning: {
    				label: "Varoitus"
    			}
    		},
    		AuthenticationMethod: {
    			auth0Database: {
    				description: "",
    				label: "Sähköpostitunnistus"
    			},
    			emailPwdLocal: {
    				description: "",
    				label: "Tietokanta"
    			},
    			facebook: {
    				description: "",
    				label: "Facebook -tili"
    			},
    			google: {
    				description: "",
    				label: "Google / Gmail authentication"
    			},
    			internalLocalhost: {
    				description: "",
    				label: ""
    			},
    			linkedIn: {
    				description: "",
    				label: "LinkedIn authentication"
    			},
    			microsoft: {
    				description: "",
    				label: "Microsoft -tili"
    			},
    			salaxy: {
    				description: "",
    				label: ""
    			},
    			test: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Tuntematon"
    			},
    			x509: {
    				description: "",
    				label: ""
    			}
    		},
    		AuthorizationStatus: {
    			accessDenied: {
    				description: "",
    				label: "Pääsy kielletty"
    			},
    			expiredOauthTokdescription: "",
    			expiredOauthToklabel: "Istunto vanheutunut",
    			invalidOAuthTokdescription: "",
    			invalidOAuthToklabel: "Virhe tunnistuksessa",
    			noOauthTokdescription: "",
    			noOauthToklabel: "Virhe tunnistuksessa (ei tokenia)",
    			oK: {
    				description: "",
    				label: "OK"
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritetty"
    			},
    			expiredOauthToken: {
    				description: "",
    				label: "ExpiredOauthToken"
    			},
    			invalidOAuthToken: {
    				description: "",
    				label: "InvalidOAuthToken"
    			},
    			noOauthToken: {
    				description: "",
    				label: "NoOauthToken"
    			}
    		},
    		AuthorizationType: {
    			companyContract: {
    				description: "",
    				label: "Työnantajan sopimus"
    			},
    			employerAuthorization: {
    				description: "",
    				label: "Valtakirja"
    			},
    			manual: {
    				description: "",
    				label: "Valtakirja toimitettu erikseen"
    			},
    			none: {
    				description: "",
    				label: "Ei tunnistettu"
    			},
    			temporary: {
    				description: "",
    				label: "Temp tunnistus"
    			},
    			workerContract: {
    				description: "",
    				label: "Työntekijän sopimus"
    			}
    		},
    		AvatarPictureType: {
    			gravatar: {
    				description: "",
    				label: "Gravatar"
    			},
    			icon: {
    				description: "",
    				label: "Kuvake"
    			},
    			uploaded: {
    				description: "",
    				label: "Kuva omalta koneelta"
    			}
    		},
    		BlobFileType: {
    			authorizationPdf: {
    				description: "",
    				label: "Valtakirja"
    			},
    			avatar: {
    				description: "",
    				label: "Ikoni"
    			},
    			calcReport: {
    				description: "",
    				label: ""
    			},
    			expensesReceipt: {
    				description: "",
    				label: "Kulukuitti"
    			},
    			monthlyReport: {
    				description: "",
    				label: "Kuukausiraportti"
    			},
    			raksaloki: {
    				description: "",
    				label: "Kulukuitti"
    			},
    			taxCard: {
    				description: "",
    				label: "Verokortti"
    			},
    			template: {
    				description: "",
    				label: "Kaavain"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			},
    			yearlyReport: {
    				description: "",
    				label: "Vuosiraportti"
    			},
    			eInvoice: {
    				description: "",
    				label: "EInvoice"
    			},
    			messages: {
    				description: "",
    				label: "Messages"
    			},
    			settings: {
    				description: "",
    				label: "Settings"
    			},
    			temporary: {
    				description: "",
    				label: "Temporary"
    			}
    		},
    		BlobRepository: {
    			cdnImages: {
    				description: "",
    				label: "Content Delivery Network (CDN)"
    			},
    			fileSystemContent: {
    				description: "",
    				label: "Content in local file system (localhost/dev/test only)"
    			},
    			gitContent: {
    				description: "",
    				label: "Content in GIT"
    			},
    			payload: {
    				description: "",
    				label: "Payload of large objects"
    			},
    			reports: {
    				description: "",
    				label: "Backoffice reports"
    			},
    			systemFiles: {
    				description: "",
    				label: "System Files"
    			},
    			userFiles: {
    				description: "",
    				label: "User Files"
    			},
    			versionHistory: {
    				description: "",
    				label: "Version History"
    			}
    		},
    		BoardKind: {
    			"undefined": {
    				description: "Oletusarvo",
    				label: "Oletusarvo"
    			}
    		},
    		CalculationFlag: {
    			accidentInsurance: {
    				label: "Työtapaturma- ja ammattitautivakuutusmaksun alainen"
    			},
    			cfDeduction: {
    				label: "Vähennetään maksusta asiakasvaratililla"
    			},
    			cfDeductionAtSalaxy: {
    				label: "Vähennetään nettomaksusta asiakasvaratililtä työntekijälle"
    			},
    			cfNoPayment: {
    				label: "Ei makseta asiakasvaratilin kautta"
    			},
    			cfPayment: {
    				label: "Maksetaan asiakasvaratilin kautta"
    			},
    			exclude: {
    				label: "Ei käytetä laskelmalla"
    			},
    			healthInsurance: {
    				label: "Sairausvakuutusmaksun alainen"
    			},
    			noTax: {
    				label: "Ei vaikuta ennakonpidätykseen"
    			},
    			pensionInsurance: {
    				label: "Työeläkevakuutusmaksun alainen"
    			},
    			tax: {
    				label: "Lasketaan ennakonpidätykseen"
    			},
    			taxDeduction: {
    				label: "Vähentää ennakonpidätrystä"
    			},
    			unemploymentInsurance: {
    				label: "Työttömyysvakuutusmaksun alainen"
    			},
    			insurancesDeduction: {
    				description: "",
    				label: "InsurancesDeduction"
    			}
    		},
    		CalculationRowSource: {
    			manualRow: {
    				description: "",
    				label: "Lisärivi"
    			},
    			salaryPage: {
    				description: "",
    				label: "Pääpalkka"
    			},
    			tesSelection: {
    				description: "",
    				label: "Käyttötapaus"
    			}
    		},
    		CalculationRowType: {
    			accomodationBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Työnantajan maksaman asunnon verotusarvo.",
    				iconText: "as",
    				label: "Asuntoetu",
    				moreInfo: "",
    				priceLabel: "Asuntoetu / kk"
    			},
    			advance: {
    				amountLabel: "Määrä",
    				description: "Palkasta on saatettu jo maksaa ennakkoa. Tällöin tämä osuus vähennetään maksettavasta nettomaksusta.",
    				iconText: "adv",
    				label: "Palkkaennakko",
    				moreInfo: "",
    				priceLabel: "Maksettu ennakko"
    			},
    			board: {
    				amountLabel: "Määrä",
    				description: "Hallintoelimen jäsenyydestä maksettu palkkio",
    				iconText: "hal",
    				label: "Hallituspalkkio",
    				moreInfo: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella. ",
    				priceLabel: "Hinta"
    			},
    			carBenefit: {
    				description: "Autoetu lisätietoineen ja vähennyksineen",
    				iconText: "auto",
    				label: "Autoetu",
    				moreInfo: "Autoedulla tarkoitetaan verotettavaa etua, jonka työntekijä saa, jos saa käyttää työnantajan omistamaa tai hallinnoimaa (esim. leasing-auto) autoa työajojen lisäksi yksityisajoihin.\r\n\r\nAutoetu on palkanlisänä tiukasti säännelty. Sen arvo lasketaan verottajan laskentakaavalla ja sekä arvo että laskennan perusteet tulee ilmoittaa tulorekisteriin. Ilmoitus pitää tehdä myös silloin, jos työntekijä maksaa verotusarvosta korvauksen, jolloin verotettavaa tuloa ei synny. Ilmoitus tehdään lähtökohtaisesti palkkakausittain tai vähintään kuukausittain, jos muuta palkkaa ei makseta. Kilometrikohtaisessa laskennassa ilmoituksen saa vaihtoehtoisesti tehdä vain vuoden lopussa.\r\n\r\nLaskentakaava perustuu kahteen luokkaan: vapaa autoetu ja käyttöetu:\r\n\r\n- Vapaassa autoedussa työnantaja huolehtii kaikista auton käyttöön liittyvistä kustannuksista, kuten polttonesteistä, vakuutuksista, veroista, huolloista ja korjauksista.\r\n- Käyttöedussa auton käyttäjä maksaa yksityisajojensa polttoainekustannukset, sopimuksen mukaan mahd. myös muita kustannuksia.\r\n\r\nLisäksi laskentakaavaan tarvitaan auton hinta, lisävarusteiden hinta sekä ikäryhmä A, B tai C. Tyypillisesti arvo lasketaan kiinteällä 18 000 kilometrin vuosittaisella yksityisajolla. Jos yksityisajot ovat ilmiselvästi suuremmat, tulisi käyttää kilometreihin perustuvaa menetelmää. \r\n\r\nAutoedun verotusarvon laskuri löytyy osoitteesta: https://www.vero.fi/autoetulaskuri\r\n\r\nVaihtoehtoisesti verotusarvon voi laskea kilometripohjaisesti. Tämä on edullisempaa, jos yksityisajon kilometrit ovat vähäiset, mutta erittäin työlästä, koska silloin yksityisajoista tulee pitää ajopäiväkirjaa ja nämä kilometrit pitää raportoida verottajalle. Jos käytetään kilometrikohtaista laskentaa, autoedun voi ilmoittaa vain kerran vuodessa: tyypillisesti vuoden viimeisellä palkkalaskelmalla.",
    				priceLabel: "Autoedun verotusarvo"
    			},
    			chainsawReduction: {
    				amountLabel: "",
    				description: "Moottorisahavähennys erikoistapauksena (Ennakkoperintälaki 15 §, Ennakkoperintäasetus 18 §)",
    				iconText: "",
    				label: "Valitse...",
    				moreInfo: "",
    				priceLabel: "Määrä"
    			},
    			childCareSubsidy: {
    				amountLabel: "Määrä",
    				description: "KELA:n ja kuntien työntekijälle suoraan maksama tuki lastenhoidosta.",
    				iconText: "KELA",
    				label: "Yksityisen hoidon tuki",
    				moreInfo: "KELA:n maksama yksityisen hoidon tuki vaikuttaa sivukuihin, sillä työnantaja maksaa esim. työeläkkeen myös siltä osin. Tämä summa tulee riveinä työnantajan maksaman palkan päälle.",
    				priceLabel: "KELA maksaa"
    			},
    			compensation: {
    				amountLabel: "Määrä",
    				description: "Työstä, tehtävästä tai palveluksesta maksettu korvaus, joka ei ole palkkaa.",
    				iconText: "T",
    				label: "Työkorvaus",
    				moreInfo: "",
    				priceLabel: "Työkorvaus"
    			},
    			dailyAllowance: {
    				amountLabel: "Päiviä",
    				description: "Päiväraha: kokopäiväraha (oletus) sekä muut päivärahat",
    				iconText: "pv",
    				label: "Päiväraha",
    				moreInfo: "Päiväraha on korvaus kohtuullisesta ruokailu- ja muiden elinkustannusten lisäyksestä, jotka palkansaajalle aiheutuu työmatkasta. Kuitenkaan näihin kustannuksiin ei kuulu matkustamisesta (liput, kilometrit) eikä majoittumisesta aiheutuvia kustannuksia: Ne maksetaan tyypillisesti erikseen kuittien perusteella tai kilometrikorvauksina.\r\n\r\nVerohallinto määrittää päivärahan enimmäismäärät vuosittain. Usein käytetään näitä enimmäismääriä, mutta tämä ei ole mikään yleispätevä sääntö. Verottoman enimmäismäärän yrittävät summat tulee maksaa verotettavana palkkana tai työkorvauksena.\r\n\r\nPäivärahan saaminen edellyttää työ- tai virkamatkaa, joka suuntautuu yli 15 kilometriä työpaikalta tai kotoa riippuen siitä, kummasta alkaen matka tehdään. Osapäivärahaa maksetaan kotimaassa yli kuusi tuntia ja kokopäivärahaa yli 10 tuntia kestävältä työmatkalta. Päivärahan sijaan on myös mahdollista maksaa ateriakorvausta. Ateriakorvauksen saamisen edellytyksenä on, ettei matka oikeuta päivärahaan tai sitä ei muutoin makseta.",
    				priceLabel: "Euroa / päivä"
    			},
    			dailyAllowanceHalf: {
    				amountLabel: "Päiviä",
    				description: "Päiväraha, kun työmatka on yli 6-10 tuntia.",
    				iconText: 19,
    				label: "Osapäiväraha",
    				moreInfo: "Päivärahaa maksetaan 19 euroa, jos työmatka on yli 6 tuntia. Matkakohde pitää olla 15 km päässä joko asunnosta tai varsinaisesta työpaikasta.",
    				priceLabel: "Euroa / päivä"
    			},
    			employmentTermination: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva työsuhteen päättämiseen liittyvä korvaus",
    				iconText: "off",
    				label: "Työsuhteen päättäminen",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			eveningAddition: {
    				amountLabel: "Tuntia",
    				description: "Korvaus iltatyöstä työsopimuksen tai työehtosopimuksen mukaan.",
    				iconText: "ILTA",
    				label: "Iltavuorolisä",
    				moreInfo: "",
    				priceLabel: "Lisä (per tunti)"
    			},
    			expenses: {
    				amountLabel: "Määrä",
    				description: "Kulukorvaukset (esim. taksi, tarvikkeet) maksetaan yleensä kuittien perusteella.",
    				iconText: "kulu",
    				label: "Kulukorvaus",
    				moreInfo: "",
    				priceLabel: "Kulukorvaus"
    			},
    			foreclosure: {
    				amountLabel: "Määrä",
    				description: "Työnantajan suoraan ulosottomiehelle maksama ulosottopidätys. Tehdään palkasta maksukiellon perusteella.",
    				iconText: "ulos",
    				label: "Ulosmittaus (työnantaja)",
    				moreInfo: "Velallisen suojaosuuteen, 22,63 euroa päivässä, lisätään 8,12 euroa päivässä jokaisen hänen elatuksensa varassa olevan henkilön osalta. Velallisen elatuksen varassa olevien henkilöiden lukumäärä on mainittu maksukiellossa, jonka ulosottoviranomainen on lähettänyt palkan tai muun tulon maksajalle.\r\nSuojaosuus lasketaan kertomalla päiväkohtainen suojaosuus palkanmaksukauden päivien lukumäärällä. Jos palkka tai muu tulo maksetaan kalenterikuukaudelta, päivien lukumäärä on aina 30.\r\nUlosmitattava määrä:\r\n1. Jos palkka on pienempi kuin suojaosuus, siitä ei ulosmitata mitään.\r\n2. Jos palkka ylittää suojaosuuden, mutta on enintään kaksi kertaa suojaosuus, ulosmitataan suojaosuuden ylittävästä palkasta kaksi kolmasosaa (tulorajaulosmittaus 2/3 x (palkka – suojaosuus)).\r\n3. Jos palkka on suurempi kuin kaksi kertaa suojaosuus, mutta enintään neljä kertaa suojaosuus, ulosmitataan nettopalkasta yksi kolmasosa (1/3).\r\n4. Jos palkka on suurempi kuin neljä kertaa velallisen suojaosuuden määrä ulosmitataan yksi kolmasosa neljä kertaa velallisen suojaosuuden määrää vastaavasta palkanosasta ja lisäksi neljä viidesosaa sen ylittävästä palkanosasta. Palkasta ulosmitataan kuitenkin enintään puolet.\r\n",
    				priceLabel: "Summa"
    			},
    			foreclosureByPalkkaus: {
    				amountLabel: "",
    				description: "Palkkaus.fi-palvelun maksama ulosottopidätys. Tehdään palkasta ulosottomiehen lähettämän maksukiellon perusteella.",
    				iconText: "",
    				label: "Ulosmittaus (Palkkaus.fi)",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			holidayBonus: {
    				amountLabel: "Prosentti",
    				description: "Työehtosopimuksissa (TES) määritetty lomaraha (ent. lomaltapaluuraha).",
    				iconText: "TES",
    				label: "Lomaraha",
    				moreInfo: "",
    				priceLabel: "Loma-ajan palkka"
    			},
    			holidayCompensation: {
    				amountLabel: "Prosentti",
    				description: "Korvaus loma-ajan palkasta, jos lomaa ei pidetä.",
    				iconText: "11,5%",
    				label: "Lomakorvaus",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			holidaySalary: {
    				amountLabel: "",
    				description: "Palkka loma-ajalta.",
    				iconText: "loma",
    				label: "Loma-ajan palkka",
    				moreInfo: "",
    				priceLabel: "Loma-ajan palkka"
    			},
    			hourlySalary: {
    				amountLabel: "Tunteja",
    				description: "Palkanmaksu tehtyjen tuntien perusteella.",
    				iconText: "h",
    				label: "Tuntipalkka",
    				moreInfo: "",
    				priceLabel: "Tuntipalkka"
    			},
    			hourlySalaryWithWorkingTimeCompensation: {
    				amountLabel: "Tunteja",
    				description: "Tuntipalkka ja lisä tehtyjen tuntien perusteella.",
    				iconText: "lisä",
    				label: "Tuntipalkka ja lisä",
    				moreInfo: "",
    				priceLabel: "Tuntipalkka"
    			},
    			irIncomeType: {
    				amountLabel: "Määrä",
    				description: "Erikseen syötetty tulorekisterin tulolaji, mikä ei sisälly muihin laskelman riveihin.",
    				iconText: "katre",
    				label: "Muu tulorekisterin tulolaji",
    				moreInfo: "Laskelma luo automaattisesti tulorekisterin tulolajit kullekin riville rivin tyypin perusteella. Tällä rivityyppillä voi erikseen syöttää tulorekisterin tulolajin, mikä ei sisälly muihin laskelman riveihin.",
    				priceLabel: "Hinta"
    			},
    			mealBenefit: {
    				amountLabel: "Aterioita",
    				description: "Työnantaja työntekijälleen tarjoama ateria. Myös lounassetelit.",
    				iconText: "rav",
    				label: "Ravintoetu",
    				moreInfo: "Ravintoetu on luontoisetu, joka syntyy, kun työnantaja tarjoaa työntekijälleen aterian. Työnantaja voi tukea ruokailukustannuksia ruokailulipukkeella (Lounasseteli) tai muulla kohdennetulla maksuvälineellä taikka järjestämällä työpaikkaruokailun (sopimusruokailu).\r\n\r\nRavintoedun verotusarvo määräytyy Verohallinnon päätöksen mukaisena vuosittain vahvistettavana arvona.\r\n\r\nVaihtoehtona ravintoedulle on ateriakorvaus, joka maksetaan työmatkalla. Sen maksaminen edellyttää, että työmatkasta ei suoriteta päivärahaa ja että palkansaajalla ei työn vuoksi ole mahdollisuutta ruokailutauon aikana aterioida tavanomaisella ruokailupaikallaan. Ateriakorvaus on veroton etu (maksetaan työntekijälle), kun erilaiset ravintoedut ovat luontoisetuja (yritys maksaa kulun ja siitä osa on työntekijältä verotettavaa tuloa).\r\n\r\nJos työntekijän palkanmaksupäivä on esimerkiksi kuukauden puolivälissä, tieto ravintoedun määrästä annetaan tulorekisteriin ennen kuin on varmuutta ravintoedun kokonaismäärästä kyseisenä kuukautena (esimerkiksi mahdolliset poissaolot eivät ole työnantajan tiedossa). Tämän vuoksi on katsottu riittäväksi, että ravintoetu täsmäytetään oikean suuruiseksi viimeistään seuraavalla palkkatietoilmoituksella. \r\n\r\nJoulukuun ravintoedun määrä on korjattava antamalla joulukuulle korvaava ilmoitus, jotta ravintoedun määrä korjaantuu oikealle verovuodelle. Tästä täsmäyksestä Palkkaus.fi antaa erillinen ohjeistuksen ennen vuodenvaihdetta.\r\n\r\nJos ravintoetu toteutetaan saldoon perustuvaa maksuvälinettä käyttäen (esimerkiksi maksukortti), täsmäytys on tehtävä vähintään vuoden viimeisellä palkkatietoilmoituksella.",
    				priceLabel: "Nimellisarvo / kustannus"
    			},
    			mealCompensation: {
    				amountLabel: "Päiviä",
    				description: "Ateriakorvauksen voi maksaa työmatkoilta, joilta ei makseta päivärahaa.",
    				iconText: "ater",
    				label: "Ateriakorvaus",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageDaily: {
    				amountLabel: "Päiviä",
    				description: "Kilometrikorvaus määritetään monissa TES:ssa päiväkohtaisina summina.",
    				iconText: "km",
    				label: "Kilometrikorvaus, päiväkohtainen",
    				moreInfo: "Esim. Rakennusalan työehtosopimuksissa on päiväkohtaiset taulukot, joiden perusteella maksetaan kilometrikorvauksia.",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageOther: {
    				amountLabel: "Päiviä",
    				description: "Kilometrikorvaus jollain muulla perusteella.",
    				iconText: "km",
    				label: "Kilometrikorvaus, muu peruste",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			milageOwnCar: {
    				amountLabel: "Kilometriä",
    				description: "Kilometrikorvaus oman auton käytöstä.",
    				iconText: "km",
    				label: "Kilometrikorvaus, oma auto",
    				moreInfo: "",
    				priceLabel: "Korvaus / km"
    			},
    			monthlySalary: {
    				amountLabel: "Kuukausia",
    				description: "Kiinteä kuukausipalkka.",
    				iconText: "kk",
    				label: "Kuukausipalkka",
    				moreInfo: "",
    				priceLabel: "Kuukausipalkka"
    			},
    			nightimeAddition: {
    				amountLabel: "Tuntia",
    				description: "Korvaus yötyöstä työsopimuksen tai työehtosopimuksen mukaan.",
    				iconText: "YÖ",
    				label: "Yövuorolisä",
    				moreInfo: "",
    				priceLabel: "Lisä (per tunti)"
    			},
    			nonProfitOrg: {
    				amountLabel: "Määrä",
    				description: "Yleishyödylliset yhteisön vapaaehtoistyöhön osallistuvalle maksama päiväraha tai kilometrikorvaus.",
    				iconText: "NPO",
    				label: "Yleishyödyllisen yhteisön päiväraha ja km-korvaus",
    				moreInfo: "Yleishyödylliset yhteisöt voivat maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle päivärahaa tai kilometrikorvausta.\r\n\r\nHUOM: Tällä käyttöliittymällä ilmoitetaan vain vapaaehtoistyöhön liittyvät korvaukset. Jos korvaukset liittyvät työsuhteeseen / palkkaan, korvaukset tulee ilmoittaa erikseen kohdissa Kilometrikorvaus ja Päiväraha (kokopäiväraha tai osapäiväraha). Jos yhdistys ei ole verottajan määrittelemällä tavalla yleishyödyllinen, työsuhteen ulkopuoliset korvaukset, myös vapaaehtoistyöstä, tulee tilittää ennakonpidätyksen alaisena työkorvauksena. Vastaanottaja voi sitten vähentää ne omassa verotuksessaan.\r\n\r\nIlmoitettava tulo on määritetty tuloverolain 71 § 3 momentissa: \r\n\r\nPäivärahan voi maksaa enintään 20 päivältä per kalenterivuosi ja lisäksi majoittumiskorvaus (tositteen perusteella).\r\n\r\nKilometrikorvausten osalta voidaan maksaa matkustamiskustannusten korvaus. Toisin kuin tavallisessa kilometrikorvauksessa, tämä voidaan maksaa myös verovelvollisen asunnolta tehdystä matkasta. Muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 3 000 euroa kalenterivuodelta.\r\n\r\nTässä kohdassa ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaan päivärahaa enemmän kuin yllä, ylimenevä osa ilmoitetaan työkorvauksena. Palkkaus.fi ei tarkkaile näitä rajoja, vaan työnantajan tulee huolehtia tästä itse.",
    				priceLabel: "Hinta"
    			},
    			otherAdditions: {
    				amountLabel: "Määrä",
    				description: "Muut sovitut aikaperusteiset lisät.",
    				iconText: "lis",
    				label: "Muut lisät",
    				moreInfo: "",
    				priceLabel: "Euroa"
    			},
    			otherBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Muu työnantajan maksama verotettava etu.",
    				iconText: "etu",
    				label: "Muu etu",
    				moreInfo: "",
    				priceLabel: "Muu etu € / kk"
    			},
    			otherCompensation: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva muu korvaus tai palkkio",
    				iconText: "muu",
    				label: "Muu palkkio tai korvaus",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			otherDeductions: {
    				amountLabel: "Määrä",
    				description: "Muu vähennys, joka pienentää työntekijälle maksettavan nettopalkan määrää.",
    				iconText: "muu",
    				label: "Muu vähennys",
    				moreInfo: "",
    				priceLabel: "Vähennys"
    			},
    			overtime: {
    				amountLabel: "Prosentti",
    				description: "Ylityökorvaukset",
    				iconText: "yli",
    				label: "Ylityöt",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			phoneBenefit: {
    				amountLabel: "Kuukausia",
    				description: "Puhelimen verotusarvo, jonka verottaja määrittää vuosittain.",
    				iconText: 20,
    				label: "Puhelinetu",
    				moreInfo: "",
    				priceLabel: "Puhelinetu / kk"
    			},
    			prepaidExpenses: {
    				amountLabel: "Määrä",
    				description: "Kulukorvaus, mikä on jo maksettu ennakkoon työntekijälle. Vähentää työntekijälle maksettavien kulukorvausten määrää.",
    				iconText: "ek",
    				label: "Ennakkoon maksettu kulukorvaus",
    				moreInfo: "",
    				priceLabel: "Maksettu ennakko"
    			},
    			remuneration: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva kertaluonteinen palkkio",
    				iconText: "ker",
    				label: "Kertaluonteinen palkkio",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			},
    			salary: {
    				amountLabel: "",
    				description: "Kertakorvaus työstä.",
    				iconText: "P",
    				label: "Palkka tai palkkio",
    				moreInfo: "",
    				priceLabel: "Palkka tai palkkio"
    			},
    			saturdayAddition: {
    				amountLabel: "Prosentti",
    				description: "Lisä lauantaina tehdystä työstä työ- tai työehtosopimuksen mukaan.",
    				iconText: "LA",
    				label: "Lauantailisä",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			subsidisedCommute: {
    				description: "Työnantajan työntekijälle kustantaman ja hankkima julkisen liikenteen henkilökohtainen joukkoliikennelippu, josta osa on verovapaata.",
    				iconText: "bus",
    				label: "Työsuhdematkalippu",
    				moreInfo: "Työsuhdematkalippu on työnantajan työntekijälle kustantaman ja hankkima julkisen liikenteen henkilökohtainen joukkoliikennelippu. Osa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista: Verovapaata on 0 - 300€ ja 750 – 3 400€ vuonna 2019. Palkkaus.fi-palvelu laskee verovapaan ja veronalaisen palkan lipun hinnan perusteella. Lisäksi otetaan huomioon työntekijän mahdollisesti työnantajalle maksama korvaus. Kaikki nämä tiedot täytyy ilmoittaa erikseen verottajalle / tulorekisteriin. \r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nMatkalippu on yleensä voimassa vuoden ja siksi työsuhdematkalipussa veronalaisen ja verovapaan edun määriä arvioidaan vuositasolla (toisin kuin muissa luontoiseduissa kuukausitasolla). \r\n\r\nJos kuitenkin korvausta peritään työntekijän palkasta säännöllisesti koko vuosi (esim. kerran kuussa) voi maksaja jo etukäteen ennakoida tilanteen ja jakaa verovapaan osuuden koko vuodelle ja ilmoittaa sen kuukausittain tulorekisteriin samassa yhteydessä perimänsä korvauksen kanssa. Tämä siitä huolimatta, että työsuhdematkalippu annettaisiin kerran vuodessa.\r\n\r\nJos haluat toteuttaa tämän jaon, sinun tulee asettaa kohta ”Jaetaanko korvaus palkkajaksoihin?” arvoksi Kyllä ja määrittää jaksojen määrä (oletus 12 kuukautta). Palkkaus.fi-palvelu ei tässä versiossa tarkkaile, että vuodessa kaikki jaksot tulee ilmoitettua oikein, joten sinun tulee vuoden lopussa tarkistaa, että kaikki työsuhdematkalipun osat on ilmoitettu. Erityisen huolellisesti pitää tarkistaa, jos vaihdat jaksojen määrää kesken vuotta (ei suositeltavaa). Ota tarvittaessa yhteyttä Palkanlaskijaasi tai Palkkaus.fi-asiakaspalveluun.",
    				priceLabel: "Lipun hinta"
    			},
    			sundayWork: {
    				amountLabel: "Määrä",
    				description: "Lain määrittämä 100% korotus sunnuntailta ja pyhäpäiviltä.",
    				iconText: "SU",
    				label: "Sunnuntai- / pyhätyö",
    				moreInfo: "",
    				priceLabel: "Palkkasumma"
    			},
    			tesWorktimeShortening: {
    				amountLabel: "Prosentti",
    				description: "Erillinen palkanosa on määritetty rakennusalan työehtosopimuksissa.",
    				iconText: "TES",
    				label: "Erillinen palkan osa (Raksa)",
    				moreInfo: "Ne ovat pääosin yleissitovia ja koskevat myös yksityishenkilöä. Entinen työajan lyhennysraha / \"pekkaset\".",
    				priceLabel: "Palkkasumma"
    			},
    			toolCompensation: {
    				amountLabel: "Päiviä",
    				description: "Työvälinekorvaus on määritetty joissakin työehtosopimuksissa (esim. Raksa TES).",
    				iconText: "tool",
    				label: "Työvälinekorvaus",
    				moreInfo: "",
    				priceLabel: "Korvaus / päivä"
    			},
    			totalEmployerPayment: {
    				amountLabel: "",
    				description: "Laskee palkan kokonaiskustannuksena: Mitä palkanmaksu maksaa sivukuluineen työnantajalle. Lopputulos on kertakorvaus.",
    				iconText: "sum",
    				label: "Kokonaiskustannus",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totals: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Yhteensä",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totalWorkerPayment: {
    				amountLabel: "",
    				description: "Palkan laskenta annetun summan perusteella",
    				iconText: "net",
    				label: "Nettopalkka",
    				moreInfo: "Palkan laskenta annetun summan perusteella",
    				priceLabel: ""
    			},
    			unionPayment: {
    				amountLabel: "",
    				description: "Työnantajan tulee maksaa AY-jäsenmaksu suoraan liitolle, jos työntekijä näin toivoo.",
    				iconText: "AY",
    				label: "Ay-jäsenmaksu",
    				moreInfo: "Työnantaja voi vähentää AY-jäsenmaksun suoraan työntekijän palkasta ja tilittää sen ammattiliitolle. Vähentäminen tapahtuu työntekijän pyynnöstä / suostumuksella. AY jäsenmaksu vähennetään työntekijän nettopalkasta.\n\nMenettelystä on saatettu sopia työehtosopimuksessa, jolloin AY-jäsenmaksun tilittäminen saattaa olla työnantajan velvollisuus. Siitä, koskeeko velvollisuus vain järjestäytyneitä työnantajia, ei tätä kirjoitettaessa ole täyttä varmuutta: Asiasta on eri mielipiteitä ja siitä ei ole oikeustapausta.\n\nTyypillisesti tilanne on Palkkaus.fi-asiakkailla (pienillä yrityksillä) harvinainen ja siksi AY-jäsenmaksuja ei toistaiseksi hoideta Palkkaus.fi-palvelun kautta: Työnantajan tulee itse hoitaa jäsenmaksun tilitys suoraan liittoon. Näin vähennettävä summa vähennetään Palkkaus.fi-palvelun asiakasvaratilille maksettavasta summasta. Lisätietoja ilmoittamisesta ja maksamisesta saat kyseisestä liitosta.",
    				priceLabel: ""
    			},
    			unknown: {
    				amountLabel: "Määrä",
    				description: "Rivin tyyppiä ei ole vielä määritetty.",
    				iconText: "?",
    				label: "Valitse...",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			workingTimeCompensation: {
    				amountLabel: "Määrä",
    				description: "Tulorekisterin tulolajeissa oleva aikaperusteinen lisä ",
    				iconText: "lisä",
    				label: "Aikaperusteinen lisä",
    				moreInfo: "",
    				priceLabel: "Hinta"
    			}
    		},
    		CalculationRowUnit: {
    			count: {
    				description: "",
    				label: "kpl"
    			},
    			days: {
    				description: "",
    				label: "päivää"
    			},
    			kilometers: {
    				description: "",
    				label: "km"
    			},
    			percent: {
    				description: "",
    				label: "%"
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			hours: {
    				description: "",
    				label: "h"
    			}
    		},
    		CalculationStatus: {
    			draft: {
    				description: "",
    				label: "Luonnos"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "Maksu peruttu"
    			},
    			paymentError: {
    				description: "",
    				label: "Virheitä"
    			},
    			paymentRefunded: {
    				description: "",
    				label: "Maksu palautettu"
    			},
    			paymentStarted: {
    				description: "",
    				label: "Maksu aloitettu"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "Maksettu"
    			},
    			paymentWorkerCopy: {
    				description: "",
    				label: "Työntekijän kopio"
    			},
    			payrollDraft: {
    				description: "",
    				label: "Palkkalistan laskelma"
    			},
    			proDraft: {
    				label: "Oma luonnos (pro)"
    			},
    			sharedApproved: {
    				description: "",
    				label: "Hyväksytty jaettu laskelma"
    			},
    			sharedRejected: {
    				description: "",
    				label: "Hylätty jaettu laskelma"
    			},
    			sharedWaiting: {
    				description: "",
    				label: "Vastaanotettu jaettu laskelma"
    			},
    			template: {
    				description: "",
    				label: "Toistuva laskelma"
    			},
    			workerRequestAccepted: {
    				description: "",
    				label: "Työntekijän tekemä, hyväksytty"
    			},
    			workerRequestDeclined: {
    				description: "",
    				label: "Työntekijän tekemä, hylätty"
    			},
    			workerRequested: {
    				description: "",
    				label: "Työntekijän tekemä"
    			},
    			waitingApproval: {
    				description: "",
    				label: "Yrityksen tarkistuksessa"
    			}
    		},
    		CarBenefitCode: {
    			fullCarBenefit: {
    				label: "Vapaa autoetu"
    			},
    			limitedCarBenefit: {
    				label: "Käyttöetu"
    			}
    		},
    		CarBenefitKind: {
    			fullCarBenefit: {
    				description: "Työnantaja huolehtii kaikista auton käyttöön liittyvistä kustannuksista",
    				label: "Vapaa autoetu"
    			},
    			limitedCarBenefit: {
    				description: "Auton käyttäjä maksaa yksityisajojensa polttoainekustannukset",
    				label: "Käyttöetu"
    			},
    			"undefined": {
    				description: "Autoedun tyypin valinta on pakollinen",
    				label: "Ei valintaa"
    			}
    		},
    		CompanyType: {
    			fiOy: {
    				description: "",
    				label: "Osakeyhtiö, myös asunto-osakeyhtiö ja Oyj"
    			},
    			fiRy: {
    				description: "",
    				label: "Yhdistys"
    			},
    			fiTm: {
    				description: "",
    				label: "Yksityinen elinkeinonharjoittaja tai Maatalousyrittäjä"
    			},
    			fiYy: {
    				description: "",
    				label: "Muu, esim. Kommandiittiyhtiö, Avoin yhtiö,"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		ContractPartyType: {
    			customContact: {
    				description: "",
    				label: "Custom Contact"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			}
    		},
    		DailyAllowanceKind: {
    			fullDailyAllowance: {
    				description: "Päiväraha, kun työmatka on yli 10 tuntia.",
    				label: "Kokopäiväraha"
    			},
    			internationalDailyAllowance: {
    				description: "Päiväraha ulkomaanmatkoilla",
    				label: "Ulkomaan päiväraha"
    			},
    			mealAllowance: {
    				description: "Ateriakorvauksen voi maksaa työmatkoilta, joilta ei makseta päivärahaa. Ateriakorvauksen enimmäismäärä on 10,75€ per ateria, 2 ateriaa per päivä (21,50 € per päivä).",
    				label: "Ateriakorvaus"
    			},
    			partialDailyAllowance: {
    				description: "Päiväraha, kun työmatka on 6-10 tuntia.",
    				label: "Osapäiväraha"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus kokopäiväraha",
    				label: "Ei valintaa"
    			}
    		},
    		DateOfBirthAccuracy: {
    			ageBased: {
    				description: "",
    				label: "Iän perusteella"
    			},
    			ageGroupBased: {
    				description: "",
    				label: "Ikäluokan perusteella"
    			},
    			assumption: {
    				description: "",
    				label: "Oletus"
    			},
    			exact: {
    				description: "",
    				label: "Täsmällinen päivämäärä"
    			},
    			monthCorrect: {
    				description: "",
    				label: "Kuukausi oikea"
    			},
    			verified: {
    				description: "",
    				label: "Vahvistettu"
    			}
    		},
    		EmployerType: {
    			fullTimeEmployer: {
    				description: "",
    				label: "Jatkuva sopimus tai yli 8178€ / 6kk"
    			},
    			miniEmployer: {
    				description: "",
    				label: "Alle 1300€ / vuosi"
    			},
    			smallEmployer: {
    				description: "",
    				label: "Väh. 1300€ / vuosi"
    			},
    			tyelOwnContractEmployer: {
    				description: "",
    				label: "TyEL-vuosisopimus tai hoitaa itse."
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		EmploymentContract: {
    			annualLeave: {
    				label: "Vuosiloma"
    			},
    			benefitsException: {
    				label: "Benefits"
    			},
    			noticeException: {
    				label: "Irtisanomisaika"
    			},
    			otherTerms: {
    				label: "Muut sopimusehdot"
    			}
    		},
    		EmploymentRelationType: {
    			athlete: {
    				description: "",
    				label: "Urheilija"
    			},
    			boardMember: {
    				description: "",
    				label: "Hallituksen jäsen"
    			},
    			compensation: {
    				description: "",
    				label: "Suoritus työkorvauksena"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "Valtion työllisyysmäärärahoilla työllistetty (palkkatuki)"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Yrittäjä tai yrittäjän perheenjäsen"
    			},
    			entrepreneurNew: {
    				description: "",
    				label: "Yrittäjä, osaomistaja tai yrittäjän perheenjäsen"
    			},
    			farmer: {
    				description: "",
    				label: "Maatalousyrittäjä tai perheenjäsen"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Tuntipalkkainen työntekijä"
    			},
    			keyEmployee: {
    				description: "",
    				label: "Avainhenkilö. Ulkomaalainen asiantuntija, jolle maksetaan avainhenkilön lähdeveron alainen palkka."
    			},
    			monthlySalary: {
    				description: "",
    				label: "Kuukausipalkkainen työntekijä"
    			},
    			performingArtist: {
    				description: "",
    				label: "Esiintyvä taiteilija"
    			},
    			salary: {
    				description: "",
    				label: "Keikkatyöntekijä (kertaluonteinen työ)"
    			},
    			"undefined": {
    				description: "",
    				label: "Muu"
    			}
    		},
    		EmploymentSalaryType: {
    			hourlyPay: {
    				description: "",
    				label: "tuntipalkka",
    				metricAbbrev: "/h"
    			},
    			montlyPay: {
    				description: "",
    				label: "kuukausipalkka",
    				metricAbbrev: "/kk"
    			},
    			otherPay: {
    				description: "",
    				label: "muu"
    			},
    			unknown: {
    				description: "",
    				label: "Valitse..."
    			}
    		},
    		EmploymentTerminationKind: {
    			monetaryWorkingTimeBankCompensation: {
    				description: "Palkka, joka maksetaan työaikapankkiin säästetyn ajan perusteella.",
    				label: "Työaikapankin rahakorvaus"
    			},
    			noticePeriodCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa työntekijälle palvelussuhteen päättyessä ilman irtisanomisaikaa eli tilanteessa, jossa jätetään irtisanomisaika noudattamatta ja työsuhde päättyy heti (TSL 6:4). Tulolajilla ilmoitetaan myös korvaus, jonka työnantaja maksaa, kun työntekijä irtisanoutuu yli 200 päivää kestäneen lomautuksen johdosta sekä, kun työnantaja irtisanoo lomautettuna olevan työntekijän.",
    				label: "Irtisanomisajan korvaus"
    			},
    			pensionPaidByEmployer: {
    				description: "Eläke, jonka työnantaja maksaa entiselle työntekijälleen tai tämän edunsaajalle aiemman palvelussuhteen perusteella.",
    				label: "Työnantajan maksama eläke"
    			},
    			terminationAndLayOffDamages: {
    				description: "Korvaus, jota työnantaja on velvollinen maksamaan, kun työnantaja on päättänyt palvelussuhteen henkilön työsopimuslain, kunnallisesta viranhaltijasta annetun lain tai valtion virkamieslain vastaisesti. Kohdassa ilmoitetaan myös korvaus työsuhteen purkamisesta (TSL 8:1, 55/2001) sekä korvaus perusteettomasta lomauttamisesta.",
    				label: "Vahingonkorvaus päättämis- ja lomautustilanteissa"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			voluntaryTerminationCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa vapaaehtoisesti irtisanomis- tai purkamistilanteessa tai silloin, kun palvelussuhde päätetään sopimalla.",
    				label: "Vapaaehtoinen korvaus päättämistilanteessa"
    			}
    		},
    		EmploymentType: {
    			fullTime: {
    				description: "",
    				label: "Toistaiseksi voimassaoleva"
    			},
    			job: {
    				description: "",
    				label: "Kertaluonteinen työ"
    			},
    			partTime: {
    				description: "",
    				label: "Määräaikainen"
    			},
    			unknown: {
    				description: "",
    				label: "Valitse..."
    			}
    		},
    		EmploymentWorkHoursType: {
    			hourPerThreeWeeks: {
    				description: "",
    				label: "3 viikon jaksossa"
    			},
    			hourPerTwoWeeks: {
    				description: "",
    				label: "2 viikon jaksossa"
    			},
    			hoursPerDay: {
    				description: "",
    				label: "tuntia/vuorokaudessa"
    			},
    			hoursPerWeek: {
    				description: "",
    				label: "tuntia/viikkossa"
    			}
    		},
    		ForeclosureKind: {
    			fixed: {
    				description: "Ulosottoon tilitetään kiinteä summa",
    				label: "Ulosmittaus, kiinteä summa"
    			},
    			periodic: {
    				description: "Ulosottoon tilitettävä määrä lasketaan määräajoin maksettavan palkan perusteella",
    				label: "Ulosmittaus, määräajoin maks. palkka"
    			},
    			nonPeriodic: {
    				description: "Ulosottoon tilitettävä määrä lasketaan muuten kuin määräajoin maksettavan palkan perusteella",
    				label: "Ulosmittaus, ei määräajoin maks. palkka"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus kiinteä summa",
    				label: "Ei valintaa"
    			}
    		},
    		FrameworkAgreement: {
    			childCare: {
    				description: "Jatkuva lastenhoitaja ja siihen liittyvät tuet",
    				label: "Muu lastenhoito"
    			},
    			cleaning: {
    				description: "Siivous- ja kiinteistöalan työ",
    				label: "Siivous- ja kiinteistöala"
    			},
    			construction: {
    				description: "Työtä rakennusalan työehtosopimuksen mukaan",
    				label: "Rakennus- ja remonttityö"
    			},
    			entrepreneur: {
    				description: "Yrittäjän oma palkka",
    				label: "Yrittäjä"
    			},
    			mll: {
    				description: "MLL:n lastenhoitajan palkka",
    				label: "MLL suositukset"
    			},
    			notDefined: {
    				description: "Työehtosopimusta tai suositusta ei ole määritetty",
    				label: "Ei määritetty"
    			},
    			other: {
    				description: "Muu työ - ei työehtosopimusta",
    				label: "Muu työ"
    			},
    			santaClaus: {
    				description: "Joulupukin kotikäynti jouluaattona",
    				label: "Joulupukki kotiin"
    			}
    		},
    		Gender: {
    			female: {
    				description: "",
    				label: "Nainen"
    			},
    			male: {
    				description: "",
    				label: "Mies"
    			},
    			unknown: {
    				description: "",
    				label: "-"
    			}
    		},
    		HelttiIndustry: {
    			i: {
    				description: "",
    				label: "Majoitus- ja ravitsemistoiminta"
    			},
    			j: {
    				description: "",
    				label: "Informaatio ja viestintä"
    			},
    			k: {
    				description: "",
    				label: "Rahoitus- ja vakuutustoiminta"
    			},
    			m: {
    				description: "",
    				label: "Ammatillinen, tieteellinen ja tekninen toiminta"
    			},
    			n: {
    				description: "",
    				label: "Hallinto- ja tukipalvelutoiminta"
    			},
    			notDefined: {
    				description: "",
    				label: "Ei valintaa"
    			},
    			o: {
    				description: "",
    				label: "Julkinen hallinto ja maanpuolustus"
    			},
    			other: {
    				description: "",
    				label: "Muu toimiala"
    			},
    			q: {
    				description: "",
    				label: "Terveys- ja sosiaalipalvelut"
    			},
    			r: {
    				description: "",
    				label: "Taiteet, viihde ja virkistys"
    			},
    			s: {
    				description: "",
    				label: "Muu palvelutoiminta"
    			},
    			p: {
    				description: "",
    				label: "P"
    			}
    		},
    		HelttiProductPackage: {
    			large: {
    				description: "",
    				label: "Tuloskunto"
    			},
    			medium: {
    				description: "",
    				label: "Kasvuyritys"
    			},
    			notDefined: {
    				description: "",
    				label: "Ei sopimusta"
    			},
    			small: {
    				description: "",
    				label: "Start-up"
    			}
    		},
    		HolidayAccrualKind: {
    			calculation: {
    				label: "Palkkalaskelma"
    			},
    			holiday: {
    				label: "Pidetty loma"
    			},
    			holidayCompensated: {
    				label: "Lomakorvaus"
    			},
    			manual: {
    				label: "Käsin syötetty kuukausikertymä"
    			},
    			saldo: {
    				label: "Lomavuoden alku- tai loppusaldo"
    			}
    		},
    		HolidayBonusPaymentMethod: {
    			none: {
    				label: "Ei lomarahaa"
    			},
    			pay24Days: {
    				label: "Kesäloma 24 päivää (ei huomioi lomasuunnitelmia)"
    			},
    			payAllBonus: {
    				label: "Kaikki kerralla (kesäloman yhteydessä)"
    			},
    			payForHolidaySalary: {
    				label: "Maksetun loma-ajan palkan mukaan (oletus)"
    			},
    			paySummerBonus: {
    				label: "Suunnitellut kesälomat (max 24pv)"
    			}
    		},
    		HolidayCode: {
    			holidayCompensation: {
    				description: "Lomia ei kerrytetä, vaan lomakorvaus (+mahd. lomaraha) maksetaan heti palkan yhteydessä. Tyypillistä rakennusalalla ja muissa keikkatöissä.",
    				label: "Lomakorvaus palkanmaksun yhteydessä"
    			},
    			holidayCompensationIncluded: {
    				description: "Lomia ei kerrytetä, vaan lomakorvaus on sovittu osaksi kokonaispalkkaa. Tämä on mahdollista vain poikkeustilanteissa, lähinnä kotitalouksilla.",
    				label: "Lomakorvaus osa kokonaispalkkaa"
    			},
    			noHolidays: {
    				description: "Maksun saajalla ei ole lomaoikeutta. Esim. hallituksen jäsen, työkorvauksella työskentelevät, joskus toimitusjohtaja.",
    				label: "Ei lomaoikeutta"
    			},
    			permanent14Days: {
    				description: "Työntekijä työskentelee työsopimuksen mukaan kaikkina kuukausina vähintään 14 päivää / kk",
    				label: "14 päivän sääntö"
    			},
    			permanent35Hours: {
    				description: "Työntekijä työskentelee vähintään yhtenä kuukautena vuodessa 35 tuntia / kk, mutta ei kuulu 14 päivän säännön piiriin.",
    				label: "35 tunnin sääntö"
    			},
    			temporaryTimeOff: {
    				description: "Työntekijä ei kuulu 14 päivän tai 35 tunnin säännön piiriin.",
    				label: "Vapaan ansaintasääntö"
    			},
    			"undefined": {
    				description: "Loma-ajan kertymistä ei ole määritetty",
    				label: "Lisää lomakirjanpito"
    			}
    		},
    		HolidayYearInitType: {
    			createEmpty: {
    				label: "Luo uusi tyhjä lomavuosi"
    			},
    			createFromCalcs: {
    				label: "Luo lomavuosi ja lisää lomapäivät maksettujen laskelmien perusteella"
    			},
    			createMonths: {
    				label: "Luo lomavuosi ja lisää kaikki kuukaudet"
    			},
    			ignore: {
    				label: "Älä lisää tai poista lomavuotta"
    			}
    		},
    		HouseHoldUseCaseTree: {
    			childCare: {
    				description: "Tilapäinen tai jatkuva",
    				kela: {
    					description: "Lastenhoitaja kotona.",
    					label: "Yksityisen hoidon tuki"
    				},
    				label: "Lastenhoito",
    				mll: {
    					description: "Palkka MLL:n suosituksen mukaan.",
    					label: "Tilapäinen lastenhoito"
    				},
    				other: {
    					description: "Muu jatkuva tai tilapäinen lastenhoito",
    					label: "Muu lastenhoito"
    				}
    			},
    			cleaning: {
    				"51530": {
    					description: "Kiinteistöhuollon työntekijät",
    					label: "Kiinteistöhuolto"
    				},
    				"53221": {
    					description: "Kotityöpalvelutyöntekijän työ",
    					label: "Kotityöpalvelu"
    				},
    				"61132": {
    					description: "Puutarha- ja kasvihuonetyöntekijät",
    					label: "Puutarhanhoito"
    				},
    				"91110": {
    					description: "Siivoustyö tai kotiapulainen",
    					label: "Kotisiivous tai kotiapulainen"
    				},
    				description: "Kodin siivous ja kunnossapito",
    				label: "Kotisiivous",
    				other: {
    					description: "Kodin muut työt",
    					label: "Muut"
    				}
    			},
    			construction: {
    				label: "Rakennus- ja remonttityö",
    				description: "Rakennusalan työehtosopimus.",
    				carpenter: {
    					description: "Kirvesmiehet, muurarit, laattatyöt, kattotyöt",
    					label: "Talonrakennus"
    				},
    				floor: {
    					description: "Lattianpäällystys",
    					label: "Lattiatyöt"
    				},
    				"new": {
    					description: "Uudisrakennukset, ei kotital.vähennystä",
    					label: "Uudisrakentaminen"
    				},
    				painter: {
    					description: "Rakennusmaalarit ym.",
    					label: "Maalaaminen"
    				},
    				renovation: {
    					description: "Kotital.vähennyksen alainen remontointi",
    					label: "Remontointi"
    				},
    				other: {
    					description: "Muu kuin yllä kuvattu työ",
    					label: "Muu työ"
    				}
    			},
    			other: {
    				"51530": {
    					description: "",
    					label: "Kiinteistöhuollon työntekijät"
    				},
    				"53112": {
    					description: "",
    					label: "Perhepäivähoitajat (sis. kotitalousten lastenhoitajat)"
    				},
    				"53212": {
    					description: "",
    					label: "Kehitysvammaisten hoitajat"
    				},
    				"53213": {
    					description: "",
    					label: "Sosiaalialan hoitajat"
    				},
    				"53221": {
    					description: "",
    					label: "Kotityöpalvelutyöntekijät"
    				},
    				"53222": {
    					description: "",
    					label: "Henkilökohtaiset avustajat, omaishoitajat ym."
    				},
    				"61112": {
    					description: "",
    					label: "Peltoviljelytyönjohtajat ja -työntekijät"
    				},
    				"61132": {
    					description: "",
    					label: "Puutarha- ja kasvihuonetyönjohtajat ja - työntekijät"
    				},
    				"62100": {
    					description: "",
    					label: "Metsurit ja metsätyöntekijät"
    				},
    				"71110": {
    					description: "",
    					label: "Talonrakentajat (sis. remonttimiehet)"
    				},
    				"71150": {
    					description: "",
    					label: "Kirvesmiehet ja rakennuspuusepät"
    				},
    				"71210": {
    					description: "",
    					label: "Kattoasentajat ja -korjaajat"
    				},
    				"71310": {
    					description: "",
    					label: "Rakennusmaalarit ym."
    				},
    				"91110": {
    					description: "",
    					label: "Kotiapulaiset ja -siivoojat"
    				},
    				description: "Yleinen palkkalaskuri",
    				label: "Muu työ",
    				"undefined": {
    					description: "",
    					label: "Muu työ"
    				}
    			}
    		},
    		InsuranceCompany: {
    			aktia: {
    				description: "",
    				label: "Aktia"
    			},
    			alandia: {
    				description: "",
    				label: "Alandia"
    			},
    			aVakuutus: {
    				description: "",
    				label: "A-Vakuutus"
    			},
    			fennia: {
    				description: "",
    				label: "Fennia"
    			},
    			folksam: {
    				description: "",
    				label: "Folksam"
    			},
    			"if": {
    				description: "",
    				label: "If"
    			},
    			"lähiTapiola": {
    				description: "",
    				label: "LähiTapiola"
    			},
    			none: {
    				description: "",
    				label: "Ei yhtiötä"
    			},
    			other: {
    				description: "",
    				label: "Muu vakuutusyhtiö"
    			},
    			"pohjantähti": {
    				description: "",
    				label: "Pohjantähti"
    			},
    			pohjola: {
    				description: "",
    				label: "OP Vakuutus"
    			},
    			redarnas: {
    				description: "",
    				label: "Redarnas"
    			},
    			tryg: {
    				description: "",
    				label: "Tryg"
    			},
    			turva: {
    				description: "",
    				label: "Turva"
    			},
    			"ålands": {
    				description: "",
    				label: "Ålands"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			}
    		},
    		InvoiceStatus: {
    			canceled: {
    				description: "Maksu on peruttu",
    				label: "Maksu peruttu"
    			},
    			error: {
    				description: "Maksujärjestelmä on merkinnyt maksun virheelliseksi",
    				label: "Virhetila"
    			},
    			forecast: {
    				description: "Tämä lasku on Palkkaus.fi arvio: Varsinainen lasku tulee muusta kanavasta.",
    				label: "Arvio"
    			},
    			paid: {
    				description: "Maksukanava on antanut maksukuittauksen: Maksu maksetaan 99% todennäköisyydellä. 100% varmuuden saat maksukanavasta.",
    				label: "Maksettu"
    			},
    			paymentStarted: {
    				description: "Maksuprosessi on aloitettu, esim. maksu on lähetetty pankkiin.",
    				label: "Maksu aloitettu"
    			},
    			preview: {
    				description: "Esikatselulasku on tämänhetkinen tilanne. Laskuun saattaa olla tulossa vielä uusia eriä.",
    				label: "Esikatselu"
    			},
    			read: {
    				description: "Lasku on lähetetty maksujärjestelmään",
    				label: "Käsittelyssä"
    			},
    			"undefined": {
    				description: "Luonnos, ei maksujonossa",
    				label: "Valitse..."
    			},
    			unread: {
    				description: "Laskua ei vielä ole lähetetty maksujärjestelmään",
    				label: "Ei käsitelty"
    			},
    			waitingConfirmation: {
    				description: "Odottaa maksun manuaalista vahvistusta yrittäjältä / tilitoimistolta. Tulorekisteri-ilmoitus lähetetään vasta tämän jälkeen.",
    				label: "Odottaa vahvistusta"
    			},
    			waitingPalkkaus: {
    				description: "Palkkalaskelma on otettu tarkistukseen, koska siinä saattaa olla virheitä. Maksu lähtee tarkistuksen jälkeen, yleensä seuraavana arkipäivänä.",
    				label: "Tarkistuksessa"
    			}
    		},
    		IrFlags: {
    			noMoney: {
    				label: "Maksettu muuna kuin rahana (NoMoney)"
    			},
    			oneOff: {
    				label: "Kertakorvaus (OneOff)"
    			},
    			unjustEnrichment: {
    				label: "Perusteeton etu (UnjustEnrichment)"
    			}
    		},
    		IrInsuranceExceptions: {
    			excludeAccidentInsurance: {
    				label: "Poista työtapaturma- ja ammattitautivakuutusmaksun alaisuus"
    			},
    			excludeAll: {
    				label: "Poista kaikkien sosiaalivakuutusmaksujen alaisuus"
    			},
    			excludeHealthInsurance: {
    				label: "Poista sairausvakuutusmaksun alaisuus"
    			},
    			excludePension: {
    				label: "Poista työeläkevakuutusmaksun alaisuus"
    			},
    			excludeUnemployment: {
    				label: "Poista työttömyysvakuutusmaksun alaisuus"
    			},
    			includeAccidentInsurance: {
    				label: "Lisää työtapaturma- ja ammattitautivakuutusmaksun alaisuus"
    			},
    			includeAll: {
    				label: "Lisää kaikkien sosiaalivakuutusmaksujen alaisuus"
    			},
    			includeHealthInsurance: {
    				label: "Lisää sairausvakuutusmaksun alaisuus"
    			},
    			includePension: {
    				label: "Lisää työeläkevakuutusmaksun alaisuus"
    			},
    			includeUnemployment: {
    				label: "Lisää työttömyysvakuutusmaksun alaisuus"
    			}
    		},
    		LegalEntityType: {
    			company: {
    				description: "",
    				label: "Y-tunnus"
    			},
    			partner: {
    				description: "",
    				label: "Kumppani"
    			},
    			person: {
    				description: "",
    				label: "Henkilö"
    			},
    			personCreatedByEmployer: {
    				description: "",
    				label: "Työntekijä (TA:n)"
    			},
    			"undefined": {
    				description: "",
    				label: "Tuntematon"
    			}
    		},
    		MealBenefitKind: {
    			cateringContract: {
    				description: "Työnantajan järjestämä sopimusruokailu, jonka kustannus työnantajalle on verottajan vuosittain määrittämissä rajoissa (vuonna 2020/2021 6,80€ - 10,70€/6,90€ - 10.90€). Arvostetaan verotuksessa 6,80€/6,90€ per ruokailu.",
    				label: "Sopimusruokailu"
    			},
    			institute: {
    				description: "Sairaalan, koulun, päiväkodin tai muun vastaavan laitoksen henkilökuntaan kuuluvan laitosruokailun yhteydessä saaman ravintoedun raha-arvona pidetään 2020/2021 5,10€/5,18€ ateriaa kohden.",
    				label: "Sairaalan, koulun, päiväkodin tms. ruokailu"
    			},
    			mealAllowance: {
    				description: "Ateriakorvauksen on veroton korvaus (muut tämän sivun valinnat ovat ravintoetuja / luontoisetuja). Sen maksaminen edellyttää, että työmatkasta ei suoriteta päivärahaa ja että palkansaajalla ei työn vuoksi ole mahdollisuutta ruokailutauon aikana aterioida tavanomaisella ruokailupaikallaan. \nLuonnollisesti näiltä päiviltä ei voi käyttää muuta ravintoetua. Ateriakorvauksen enimmäismäärä on 2020/2021 10,75€/11,00€ per ateria, 2 ateriaa per päivä (21,50€/22,00€ per päivä).",
    				label: "Ateriakorvaus"
    			},
    			mealTicket: {
    				description: "Ruokailulipuke (Lounasseteli tai vast.) tai maksukortille ladattava tapahtuma, jonka kustannus työnantajalle on verottajan vuosittain määrittämissä rajoissa (vuonna 2020/2021 6,80€ - 10,70€/6,90€ - 10,90€). Arvostetaan verotuksessa 75 % nimellisarvosta.",
    				label: "Ruokailulipuke"
    			},
    			restaurantWorker: {
    				description: "Hotelli- ja ravintola-alan henkilökunnan ja lentohenkilöstön lennon aikana saaman ravintoedun raha-arvona pidetään 2020/2021 5,78€/5,87€ ateriaa kohden.",
    				label: "Hotelli- ja ravintola-alan ravintoetu"
    			},
    			taxableAmount: {
    				description: "Jos ravintoedun hankkimisesta työnantajalle aiheutuneiden kustannusten määrä on yli tai alle verottajan vuosittain määrittämien rajojen (2020/2021 6,80€ - 10,70€/6,90€ - 10,90€) tai jos etu muuten ylittää rajata (esim. enemmän ruokailuja kuin työpäiviä), edun arvona pidetään suoria kustannuksia / nimellisarvoa.\nKäytä tätä valintaa myös, jos et halua käyttää automaattista verotusarvon laskentaa muissa kohdissa.",
    				label: "Ravintoetu, verotusarvo"
    			},
    			teacher: {
    				description: "Koulun, päiväkodin tai vastaavan laitoksen henkilökunnan oppilaiden tai hoidettavien ruokailun valvonnan yhteydessä saaman ravintoedun arvo on 2020/2021 4,08€/4,14€ ateriaa kohden.",
    				label: "Ruokailu valvonnan yhteydessä"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus \"Verotusarvo\".",
    				label: "Ei valintaa"
    			}
    		},
    		MessageType: {
    			email: {
    				description: "",
    				label: "Sähköposti"
    			},
    			sms: {
    				description: "",
    				label: "Tekstiviesti"
    			}
    		},
    		NonProfitOrgKind: {
    			accomodationAllowance: {
    				description: "Majoittumiskorvaus (tositteen perusteella). ",
    				label: "Majoittumiskorvaus"
    			},
    			dailyAllowance: {
    				description: "Päivärahan voi maksaa enintään 20 päivältä per kalenterivuosi ja lisäksi majoittumiskorvaus (kts alla).",
    				label: "Päiväraha"
    			},
    			kilometreAllowance: {
    				description: "Matkakustannusten korvaus. Muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 3 000 euroa kalenterivuodelta.",
    				label: "Kilometrikorvaus"
    			},
    			"undefined": {
    				description: "Tyyppiä ei määritetty, oletus kilometrikorvaus",
    				label: "Tyyppiä ei määritetty"
    			}
    		},
    		OnboardingStatus: {
    			created: {
    				description: "",
    				label: "Luotu"
    			},
    			done: {
    				description: "",
    				label: "Valmis"
    			},
    			opdescription: "",
    			oplabel: "Käynnissä",
    			expired: {
    				description: "",
    				label: "Expired"
    			},
    			open: {
    				description: "",
    				label: "Open"
    			}
    		},
    		OtherCompensationKind: {
    			accruedTimeOffCompensation: {
    				description: "Työaikakorvaus, joka maksetaan ansionvähentymisestä, joka aiheutuu työajan tasaamisesta keskimääräiseen viikkotyöaikaan.",
    				label: "Työajantasauskorvaus"
    			},
    			capitalIncomePayment: {
    				description: "Suoritus, joka on saajalleen pääomatuloa. Tällainen on esimerkiksi työntekijälle tai osakkaalle maksettu korko, kuten palkkasaatavan korko tai pääomatuloa oleva takausprovisio.",
    				label: "Pääomatuloa oleva suoritus"
    			},
    			employeeInventionCompensation: {
    				description: "Palkkio, joka maksetaan työsuhdekeksinnöstä.",
    				label: "Palkkio työsuhdekeksinnöstä"
    			},
    			employeeStockOption: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa.",
    				label: "Työsuhdeoptio"
    			},
    			employeeStockOptionWithLowerPrice: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa ja jossa työntekijältä peritty option merkintähinta option luovutusajankohtana on selvästi alhaisempi kuin osakkeen markkinahinta.",
    				label: "Työsuhdeoptio, jonka merkintähinta on luovutusajankohtana alhaisempi kuin markkinahinta"
    			},
    			lectureFee: {
    				description: "Palkkio, joka maksetaan luennon pitämisestä.",
    				label: "Luentopalkkio"
    			},
    			meetingFee: {
    				description: "Palkkio, joka maksetaan kokoukseen osallistumisesta.",
    				label: "Kokouspalkkio"
    			},
    			membershipOfGoverningBodyCompensation: {
    				description: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella.",
    				label: "Hallintoelimen jäsenyydestä maksettu palkkio"
    			},
    			monetaryGiftForEmployees: {
    				description: "Työnantajalta rahana tai siihen verrattavana suorituksena saatu lahja.",
    				label: "Henkilöstölle annettu rahalahja"
    			},
    			otherTaxableIncomeAsEarnedIncome: {
    				description: "Tulolajilla ilmoitetaan muiden veronalaisten ansiotuloa olevien suoritusten yhteissumma.",
    				label: "Muu veronalainen ansiotulo"
    			},
    			positionOfTrustCompensation: {
    				description: "Palkkio, joka maksetaan muusta luottamustoimen hoitamisesta kuin kokouksiin osallistumisesta.",
    				label: "Luottamustoimipalkkio"
    			},
    			stockOptionsAndGrants: {
    				description: "Palkkio, joka annetaan työnantajayrityksen osakkeina tai sovitun palkkion sijaan rahasummana, joka vastaa osakkeiden arvoa.",
    				label: "Osakepalkkio"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			useCompensationAsCapitalIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus pääomatuloa"
    			},
    			useCompensationAsEarnedIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus ansiotuloa"
    			},
    			workEffortBasedDividendsAsNonWage: {
    				description: "Osakkaalle työkorvauksena maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (työkorvaus)"
    			},
    			workEffortBasedDividendsAsWage: {
    				description: "Osakkaalle palkkana maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (palkka)"
    			}
    		},
    		PalkkausCampaignPricing: {
    			campaignAfterFirstUse: {
    				description: "",
    				label: "Campaign After First Use"
    			},
    			campaignFirstUse: {
    				description: "",
    				label: "Campaign First Use"
    			},
    			companyFreeTrial: {
    				description: "",
    				label: "Company Free Trial"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		PartnerSite: {
    			careDotCom: {
    				description: "",
    				label: "Care Dot Com"
    			},
    			dev: {
    				description: "",
    				label: "Dev"
    			},
    			duunitori: {
    				description: "",
    				label: "Duunitori"
    			},
    			example: {
    				description: "",
    				label: "Example"
    			},
    			mol: {
    				description: "",
    				label: "Mol"
    			},
    			palkkaapakolaindescription: "",
    			palkkaapakolainlabel: "Palkkaapakolainen",
    			palkkaus: {
    				description: "",
    				label: "Palkkaus"
    			},
    			palkkausEmployerFillIn: {
    				description: "",
    				label: "Palkkaus Employer Fill In"
    			},
    			palkkausIdentified: {
    				description: "",
    				label: "Palkkaus Identified"
    			},
    			raksa: {
    				description: "",
    				label: "Raksa"
    			},
    			refugeeJobs: {
    				description: "",
    				label: "Refugee Jobs"
    			},
    			talkkarit: {
    				description: "",
    				label: "Talkkarit"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			palkkaapakolainen: {
    				description: "",
    				label: "Palkkaapakolainen"
    			}
    		},
    		PaymentChannel: {
    			"undefined": {
    				description: "Maksutapaa ei ole määritetty. Tyypillisesti tämä on virhetilanne.",
    				label: "Valitse maksutapa..."
    			},
    			test: {
    				description: "Maksumenetelmä testaamiseen. Erilliset laskut (nettopalkka, ennakonpidätys jne.) Toimii vain testiympäristössä.",
    				label: "Testimaksu"
    			},
    			zeroPayment: {
    				description: "Maksun summa on nolla. Tätä maksua ei lähetetä maksukanavaan, vaan se merkitään suoraan maksetuksi.",
    				label: "Nollamaksu (ei maksettavaa)"
    			},
    			external: {
    				description: "Maksu on jo maksettu Palkkaus.fi palvelun ulkopuolella (tai tullaan maksamaan). Maksusta tehdään kuitenkin ilmoitus tulorekisteriin.",
    				label: "Jo maksettu / ulkopuolinen"
    			},
    			palkkausManual: {
    				description: "Maksuaineisto luodaan erillisinä laskuina Maksut-listaan. Sieltä ne voidaan tulostaa PDF-tiedostona, kopoida verkkopankkiin tai ladata SEPA-maksuaineistona.",
    				label: "Maksuaineiston luonti"
    			},
    			palkkausWS: {
    				description: "Palkasta, ennakonpidätyksestä jne. luodaan erilliset laskut ja ne lähetetään pankkien WS-kanavaan. Vaatii sopimuksen pankin kanssa.",
    				label: "Maksut suoraan pankkiin"
    			},
    			palkkausPersonal: {
    				description: "Maksut luodaan erillisinä laskuina Maksut-listaan. Sieltä ne voidaan tulostaa PDF-tiedostona tai kopoida verkkopankkiin.",
    				label: "Maksujen luonti"
    			},
    			palkkausCfaPaytrail: {
    				description: "Palkka, ennakonpidätys ja työeläke yhdellä helpolla verkkomaksulla Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Paytrail verkkomaksu (yksi maksu)"
    			},
    			palkkausCfaReference: {
    				description: "PDF-paperilasku, jonka voit maksaa missä tahansa verkkopankissa tai ohjelmassa. Palkka, ennakonpidätys ja työeläke yhdessä laskussa Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Yksi tilisiirto viitenumerolla"
    			},
    			palkkausCfaFinvoice: {
    				description: "Lähetä itsellesi verkkolasku, jossa palkka, ennakonpidätys ja työeläke on yhdessä helpossa laskussa. Maksa esim. kirjanpito-ohjelmassasi. Maksu tapahtuu Palkkaus.fi asiakasvaratilin kautta.",
    				label: "Verkkolasku (yksi maksu)"
    			},
    			palkkausCfaTest: {
    				description: "Testimaksu asiakasvaratilin kautta (yksi maksu). Toimii vain testiympäristössä.",
    				label: "Asiakasvaratilin testimaksu"
    			},
    			accountorGo: {
    				description: "Laskujen lähetys suoraan Accountor Go -palveluun. Käytössä vain Accountor Go -palvelun kautta.",
    				label: "Accountor Go"
    			},
    			talenomCfa: {
    				description: "Laskujen lähetys Talenom Online -palveluun verkkolaskuna. Käytössä vain Talenom Online -palvelun kautta.",
    				label: "Talenom verkkolasku"
    			},
    			talenomOnline: {
    				description: "Laskujen lähetys suoraan Talenom Online -palveluun. Käytössä vain Talenom Online -palvelun kautta.",
    				label: "Talenom Online"
    			},
    			holviCfa: {
    				description: "Palkka, ennakonpidätys ja työeläke yhdellä helpolla Holvi-maksulla Palkkaus.fi asiakasvaratilin kautta. Käytössä vain Holvi-palvelun kautta.",
    				label: "Holvi"
    			},
    			finagoSolo: {
    				description: "Laskujen lähetys Finago Soloon. Käytössä vain Finago Solo -palvelun kautta.",
    				label: "Finago Solo"
    			},
    			procountor: {
    				description: "Laskujen lähetys Procountoriin",
    				label: "Procountor"
    			},
    			kevytyrittaja: {
    				description: "Laskujen lähetys Talenom Kevytyrittäjä -palveluun. Käytössä vain Talenom Kevytyrittäjä -sovelluksen kautta.",
    				label: "Kevytyrittäjä"
    			},
    			vismaNetvisor: {
    				description: "Laskujen lähetys Visma Netvisor -ohjelmistoon",
    				label: "Visma Netvisor"
    			}
    		},
    		PaymentStatus: {
    			bankDelivered: {
    				description: "",
    				label: "Pankki hyväksynyt"
    			},
    			bankError: {
    				description: "",
    				label: "Virhe pankissa"
    			},
    			bankPartialError: {
    				description: "",
    				label: "Virhe pankissa"
    			},
    			bankTechApproval: {
    				description: "",
    				label: "Pankki vastaanottanut"
    			},
    			cancelled: {
    				description: "",
    				label: "Peruttu"
    			},
    			"new": {
    				description: "",
    				label: "Uusi"
    			},
    			paid: {
    				description: "",
    				label: "Maksettu"
    			},
    			sentToBank: {
    				description: "",
    				label: "Lähetetty pankkiin"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon"
    			}
    		},
    		PayrollStatus: {
    			draft: {
    				label: "Luonnos"
    			},
    			paymentCanceled: {
    				label: "Maksu peruttu"
    			},
    			paymentError: {
    				label: "Virheitä"
    			},
    			paymentStarted: {
    				label: "Maksu aloitettu"
    			},
    			paymentSucceeded: {
    				label: "Maksettu"
    			},
    			template: {
    				label: "Toistuva palkkalista"
    			},
    			waitingApproval: {
    				label: "Yrityksen tarkistuksessa"
    			}
    		},
    		PensionCalculation: {
    			athlete: {
    				description: "",
    				label: "Urheilijan eläke"
    			},
    			boardRemuneration: {
    				description: "",
    				label: "Hallituspalkkio"
    			},
    			compensation: {
    				description: "",
    				label: "Työkorvaus"
    			},
    			employee: {
    				description: "",
    				label: "TyEL"
    			},
    			entrepreneur: {
    				description: "",
    				label: "YEL"
    			},
    			farmer: {
    				description: "",
    				label: "MYEL"
    			},
    			partialOwner: {
    				description: "Osaomistajan palkka\r\nYrityksen osaomistajat maksavat pienempää palkansaajan työttömyysvakuutusmaksua kuin työntekijät.\r\n•\tOsaomistajan työnantajan työttömyysvakuutusmaksu on 0,5 % palkkasummasta.\r\n•\tPalkansaajan, joka on myös osaomistaja, työttömyysvakuutusmaksuprosentti on 0,78 %. Työnantaja pidättää sen palkasta palkanmaksun yhteydessä.\r\nJos et ole ole yrityksessä johtavassa asemassa, mutta työskentelet yrityksessä tai yhteisössä, olet osaomistaja, jos jokin seuraavista ehdoista täyttyy: \r\n•\tsinulla itselläsi on vähintään 50 % kyseisen yrityksen tai yhteisön osakepääomasta, äänivallasta tai muusta määräämisvallasta \r\n•\tperheenjäsenelläsi on vähintään 50 % kyseisen yrityksen tai yhteisön osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\n•\tsinulla on yhdessä perheenjäsentesi kanssa vähintään 50 % osakepääomasta, äänivallasta tai muusta määräämisvallasta. \r\nJos olet johtavassa asemassa mutta et ole yrittäjä, voit olla työttömyysturvalain mukaan osaomistaja, jos työskentelet yrityksessä ja jompikumpi seuraavista ehdoista täyttyy:\r\n•\tsinulla on vähintään 15 % osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\n•\tperheenjäsenilläsi tai teillä yhdessä on vähintään 30 % osakepääomasta, äänivallasta tai muusta määräämisvallasta\r\nTyöllisyysrahaston sivulla voi testata onko osaomistaja vai palkansaaja\r\nhttps://www.tyollisyysrahasto.fi/tyottomyysvakuusmaksut/tyottomyysvakuutusmaksuvelvollisuus/#osaomistaja\r\nOsaomistajan käsite on määritelty työttömyysturvalaissa (1290/2002). \r\n",
    				label: "TyEL, osaomistaja"
    			},
    			smallEntrepreneur: {
    				description: "Tulonsaaja täyttää yrittäjän eläkelain 3 §:ssä säädetyt edellytykset, mutta jolla ei ole vakuuttamisvelvollisuutta yrittäjän eläkelain 4 §:n nojalla.\nVakuuttamisvelvollisuus ei koske:\n1) yrittäjän toimintaa ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua, eikä työskentelyä, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt yrittäjä täyttää 68 vuotta, vuosina 1958–1961 syntynyt yrittäjä täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt yrittäjä täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/72)\n2) yrittäjän toimintaa, joka ei ole jatkunut yhdenjaksoisesti vähintään neljää kuukautta;\n3) yrittäjän toimintaa, joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) yrittäjää, jonka työtulo tässä laissa tarkoitetusta yrittäjätoiminnasta on arvioitava pienemmäksi kuin 5 504,14 euroa vuodessa;\n5) ansiotyötä, jonka perusteella yrittäjällä on oikeus eläkkeeseen muun työeläkelain mukaan; eikä\n6) yrittäjää, johon ei sovelleta Suomen lainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/355)",
    				label: "YEL, ei vakuuttamisvelvollisuutta"
    			},
    			smallFarmer: {
    				description: "Tulonsaaja täyttää maatalousyrittäjän eläkelain 3 §:ssä säädetyt edellytykset, mutta jolla ei ole mainitun lain perusteella vakuuttamisvelvollisuutta.\n\nVakuuttamisvelvollisuus ei koske:\n1) maatalousyrittäjän toimintaa ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua eikä toimintaa, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt maatalousyrittäjä täyttää 68 vuotta, vuosina 1958–1961 syntynyt maatalousyrittäjä täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt maatalousyrittäjä täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/75)\n2) maatalousyrittäjän toimintaa, joka ei ole jatkunut 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alun jälkeen yhdenjaksoisesti vähintään neljää kuukautta;\n3) maatalousyrittäjän toimintaa, joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) maatalousyrittäjää, jonka työtulo tässä laissa tarkoitetusta maatalousyrittäjätoiminnasta on arvioitava pienemmäksi kuin 2 752,07 euroa vuodessa;\n5) maatilatalouden, kalastuksen tai poronhoidon yhteydessä harjoitettua toimintaa, jota on pidettävä eri yrityksenä;\n6) ansiotyötä, jonka perusteella maatalousyrittäjällä on oikeus eläkkeeseen muun työeläkelain mukaan; eikä\n7) maatalousyrittäjää, johon ei sovelleta Suomen lainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/357)\n\nVakuuttamisvelvollisuus ei koske apurahansaajan työtä:\n1) ennen 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alkua, eikä työtä, joka jatkuu sen kalenterikuukauden jälkeen, jolloin vuonna 1957 tai sitä ennen syntynyt apurahansaaja täyttää 68 vuotta, vuosina 1958–1961 syntynyt apurahansaaja täyttää 69 vuotta ja vuonna 1962 ja sen jälkeen syntynyt apurahansaaja täyttää 70 vuotta (vakuuttamisvelvollisuuden yläikäraja); (29.1.2016/75)\n2) joka ei ole jatkunut 18 vuoden iän täyttämistä seuraavan kalenterikuukauden alun jälkeen yhdenjaksoisesti vähintään neljä kuukautta;\n3) joka on alkanut tai jatkunut sen jälkeen, kun hän on siirtynyt työeläkelakien mukaiselle vanhuuseläkkeelle;\n4) ajalta, jona apurahansaajalle arvioitava vuotuinen työtulo on pienempi kuin 2 752,07 euroa;\n5) jos apurahansaajaan ei sovelleta Suomen sosiaaliturvalainsäädäntöä EU:n sosiaaliturvan perusasetuksen tai sosiaaliturvasopimuksen sovellettavaa lainsäädäntöä koskevien määräysten perusteella. (14.5.2010/357)",
    				label: "MYEL, ei vakuuttamisvelvollisuutta"
    			},
    			"undefined": {
    				description: "",
    				label: "(Ei valittu vielä)"
    			}
    		},
    		PensionCompany: {
    			apteekkien: {
    				description: "",
    				label: "Apteekkien Eläkekassa"
    			},
    			elo: {
    				description: "",
    				label: "Elo"
    			},
    			etera: {
    				description: "",
    				label: "Etera"
    			},
    			ilmarindescription: "",
    			ilmarinen: {
    				label: "Ilmarinen"
    			},
    			none: {
    				description: "",
    				label: "Ei yhtiötä"
    			},
    			other: {
    				description: "",
    				label: "Muu yhtiö"
    			},
    			pensionsAlandia: {
    				description: "",
    				label: "Alandia"
    			},
    			varma: {
    				description: "",
    				label: "Varma"
    			},
    			veritas: {
    				description: "",
    				label: "Veritas"
    			},
    			verso: {
    				description: "",
    				label: "Verso"
    			}
    		},
    		PeriodType: {
    			custom: {
    				label: "Muu periodi"
    			},
    			month: {
    				label: "Kuukausi"
    			},
    			quarter: {
    				label: "Neljännesvuosi"
    			},
    			year: {
    				label: "Kalenterivuosi"
    			}
    		},
    		PricingModel: {
    			fixedFee: {
    				label: "Kiinteä palkkio"
    			},
    			noFee: {
    				label: "Ei palkkiota"
    			},
    			palkkausFee: {
    				label: "Palkkaus.fi-palkkio (1%)"
    			}
    		},
    		Product: {
    			accounting: {
    				description: "Palkkalaskelmien tiedot helposti kirjanpitäjälle",
    				label: "Kirjanpitoraportit",
    				status: "Ei käytössä"
    			},
    			baseService: {
    				description: "Digiajan palkanmaksupalvelu",
    				label: "Palkkaus.fi-tili",
    				status: "Palkkaamisen peruspalvelut"
    			},
    			healthCareHeltti: {
    				description: "Tuottavammat ja terveemmät työntekijät",
    				label: "Työterveyshuolto",
    				status: "Ei sopimusta"
    			},
    			insurance: {
    				description: "Tapaturmavakuutus vakuuttaa työntekijäsi",
    				label: "Työtapaturmavakuutus",
    				status: "Ei sopimusta. HUOM: työnantajan 1300 € :n vuosisääntö!"
    			},
    			pension: {
    				description: "TyEL-vakuutus on työntekijän eläketurva",
    				label: "TyEL-eläkevakuutus",
    				status: "Ei sopimusta: TyEL maksetaan tilapäisenä työnantajana"
    			},
    			pensionEntrepreneur: {
    				description: "YEL-vakuutus on yrittäjän eläketurva",
    				label: "Yrittäjän eläke",
    				status: "Ei YEL-sopimusta"
    			},
    			tax: {
    				description: "Ennakonpidätykset ja sava-maksu",
    				label: "Yhteydet verottajaan",
    				status: "Ennakonpidätys ja ilmoitukset"
    			},
    			taxCard: {
    				description: "Työntekijän verokortti ja sen käsittely",
    				label: "Verokortti (TT)",
    				status: "Ei verokorttia"
    			},
    			unemployment: {
    				description: "Työttömyysturva rahoitetaan TVR-maksuilla",
    				label: "Työttömyysvakuutus (TVR)",
    				status: "Ilmoitukset TVR:lle",
    				statusTVR: "Ilmoitukset TVR:lle"
    			}
    		},
    		ProductListFilter: {
    			all: {
    				description: "",
    				label: "All products, even the currently non-available."
    			},
    			available: {
    				description: "",
    				label: "Available products for this user"
    			}
    		},
    		Profession: {
    			"0": {
    				label: "Sotilaat"
    			},
    			"1": {
    				label: "Johtajat"
    			},
    			"2": {
    				label: "Erityisasiantuntijat"
    			},
    			"3": {
    				label: "Asiantuntijat"
    			},
    			"4": {
    				label: "Toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"5": {
    				label: "Palvelu- ja myyntityöntekijät"
    			},
    			"6": {
    				label: "Maanviljelijät, metsätyöntekijät ym."
    			},
    			"7": {
    				label: "Rakennus-, korjaus- ja valmistustyöntekijät"
    			},
    			"8": {
    				label: "Prosessi- ja kuljetustyöntekijät"
    			},
    			"9": {
    				label: "Muut työntekijät"
    			},
    			"11": {
    				label: "Johtajat, ylimmät virkamiehet ja järjestöjen johtajat"
    			},
    			"12": {
    				label: "Hallintojohtajat ja kaupalliset johtajat"
    			},
    			"13": {
    				label: "Tuotantotoiminnan ja yhteiskunnan peruspalvelujen johtajat"
    			},
    			"14": {
    				label: "Hotelli- ja ravintola-alan, vähittäiskaupan ja muiden palvelualojen johtajat"
    			},
    			"21": {
    				label: "Luonnontieteiden ja tekniikan erityisasiantuntijat"
    			},
    			"22": {
    				label: "Terveydenhuollon erityisasiantuntijat"
    			},
    			"23": {
    				label: "Opettajat ja muut opetusalan erityisasiantuntijat"
    			},
    			"24": {
    				label: "Liike-elämän ja hallinnon erityisasiantuntijat"
    			},
    			"25": {
    				label: "Tieto- ja viestintäteknologian erityisasiantuntijat"
    			},
    			"26": {
    				label: "Lainopilliset, sosiaalialan ja kulttuurialan erityisasiantuntijat"
    			},
    			"31": {
    				label: "Luonnontieteiden ja tekniikan asiantuntijat"
    			},
    			"32": {
    				label: "Terveydenhuollon asiantuntijat"
    			},
    			"33": {
    				label: "Liike-elämän ja hallinnon asiantuntijat"
    			},
    			"34": {
    				label: "Lainopilliset avustajat sekä sosiaali- ja kulttuurialan asiantuntijat"
    			},
    			"35": {
    				label: "Informaatio- ja tietoliikenneteknologian asiantuntijat"
    			},
    			"41": {
    				label: "Toimistotyöntekijät"
    			},
    			"42": {
    				label: "Asiakaspalvelutyöntekijät"
    			},
    			"43": {
    				label: "Laskennan ja varastoinnin toimistotyöntekijät"
    			},
    			"44": {
    				label: "Muut toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"51": {
    				label: "Palvelutyöntekijät"
    			},
    			"52": {
    				label: "Myyjät, kauppiaat ym."
    			},
    			"53": {
    				label: "Hoivapalvelun ja terveydenhuollon työntekijät"
    			},
    			"54": {
    				label: "Suojelu- ja vartiointityöntekijät"
    			},
    			"61": {
    				label: "Maanviljelijät ja eläintenkasvattajat ym."
    			},
    			"62": {
    				label: "Metsä- ja kalatalouden työntekijät"
    			},
    			"63": {
    				label: "Kotitarveviljelijät, -kalastajat ja -metsästäjät"
    			},
    			"71": {
    				label: "Rakennustyöntekijät ym. (pl. sähköasentajat)"
    			},
    			"72": {
    				label: "Konepaja- ja valimotyöntekijät sekä asentajat ja korjaajat"
    			},
    			"73": {
    				label: "Käsityötuotteiden valmistajat, hienomekaanikot sekä painoalan työntekijät"
    			},
    			"74": {
    				label: "Sähkö- ja elektroniikka-alan työntekijät"
    			},
    			"75": {
    				label: "Elintarvike-, puutyö- ja vaatetus- ja jalkinealan valmistustyöntekijät ym."
    			},
    			"81": {
    				label: "Prosessityöntekijät"
    			},
    			"82": {
    				label: "Teollisuustuotteiden kokoonpanijat"
    			},
    			"83": {
    				label: "Kuljetustyöntekijät"
    			},
    			"91": {
    				label: "Siivoojat, kotiapulaiset ja muut puhdistustyöntekijät"
    			},
    			"92": {
    				label: "Maa-, metsä- ja kalatalouden avustavat työntekijät"
    			},
    			"93": {
    				label: "Teollisuuden ja rakentamisen avustavat työntekijät"
    			},
    			"94": {
    				label: "Avustavat keittiö- ja ruokatyöntekijät"
    			},
    			"95": {
    				label: "Katumyyjät, kengänkiillottajat ym."
    			},
    			"96": {
    				label: "Katujen puhtaanapidon ja jätehuollon työntekijät ym."
    			},
    			"111": {
    				label: "Ylimmät virkamiehet ja järjestöjen johtajat"
    			},
    			"112": {
    				label: "Toimitusjohtajat ja pääjohtajat"
    			},
    			"121": {
    				label: "Liiketoiminta- ja hallintojohtajat"
    			},
    			"122": {
    				label: "Myynti-, markkinointi- ja kehitysjohtajat"
    			},
    			"131": {
    				label: "Maa-, metsä- ja kalatalouden johtajat"
    			},
    			"132": {
    				label: "Teollisuuden tuotantojohtajat sekä kaivos-, rakennus- ja jakelujohtajat"
    			},
    			"133": {
    				label: "Tieto- ja viestintäteknologiajohtajat"
    			},
    			"134": {
    				label: "Yhteiskunnan peruspalvelujen sekä rahoitus- ja vakuutuspalvelujen johtajat"
    			},
    			"141": {
    				label: "Hotellin- ja ravintolanjohtajat"
    			},
    			"142": {
    				label: "Vähittäis- ja tukkukaupan johtajat"
    			},
    			"143": {
    				label: "Muiden palvelualojen johtajat"
    			},
    			"211": {
    				label: "Luonnon- ja geotieteen erityisasiantuntijat"
    			},
    			"212": {
    				label: "Matematiikan ja tilastotieteen erityisasiantuntijat"
    			},
    			"213": {
    				label: "Biotieteiden erityisasiantuntijat"
    			},
    			"214": {
    				label: "Tekniikan erityisasiantuntijat (pl. sähköteknologia)"
    			},
    			"215": {
    				label: "Sähköteknologian erityisasiantuntijat"
    			},
    			"216": {
    				label: "Arkkitehdit, suunnittelijat ja maanmittaajat"
    			},
    			"221": {
    				label: "Lääkärit"
    			},
    			"222": {
    				label: "Hoitotyön erityisasiantuntijat"
    			},
    			"223": {
    				label: "Perinteisen ja vaihtoehtohoidon erityisasiantuntijat"
    			},
    			"224": {
    				label: "Muiden terveyspalvelujen tuottajat"
    			},
    			"225": {
    				label: "Eläinlääkärit"
    			},
    			"226": {
    				label: "Muut terveydenhuollon erityisasiantuntijat"
    			},
    			"231": {
    				label: "Yliopisto- ja korkeakouluopettajat"
    			},
    			"232": {
    				label: "Ammatillisen koulutuksen opettajat"
    			},
    			"233": {
    				label: "Lukion ja peruskoulun yläluokkien opettajat"
    			},
    			"234": {
    				label: "Peruskoulun alaluokkien opettajat ja lastentarhanopettajat"
    			},
    			"235": {
    				label: "Muut opetusalan erityisasiantuntijat"
    			},
    			"241": {
    				label: "Rahoitusalan erityisasiantuntijat"
    			},
    			"242": {
    				label: "Hallinnon erityisasiantuntijat"
    			},
    			"243": {
    				label: "Myynnin, markkinoinnin ja tiedotuksen erityisasiantuntijat"
    			},
    			"251": {
    				label: "Systeemityön erityisasiantuntijat"
    			},
    			"252": {
    				label: "Tietokantojen,- verkkojen ja järjestelmäohjelmistojen erityisasiantuntijat"
    			},
    			"261": {
    				label: "Lainopilliset erityisasiantuntijat"
    			},
    			"262": {
    				label: "Kirjaston- ja arkistonhoitajat sekä museoalan erityisasiantuntijat"
    			},
    			"263": {
    				label: "Yhteiskunta- ja sosiaalialan sekä uskonnollisen elämän erityisasiantuntijat"
    			},
    			"264": {
    				label: "Toimittajat, kirjailijat ja kielitieteilijät"
    			},
    			"265": {
    				label: "Taiteilijat"
    			},
    			"311": {
    				label: "Fysiikan, kemian ja teknisten alojen asiantuntijat"
    			},
    			"312": {
    				label: "Työnjohtajat kaivos-, teollisuus- ja rakennustoiminnassa"
    			},
    			"313": {
    				label: "Prosessinvalvonnan asiantuntijat"
    			},
    			"314": {
    				label: "Biotieteiden asiantuntijat"
    			},
    			"315": {
    				label: "Laiva-, lento- ja satamaliikenteen päälliköt ja ohjaajat"
    			},
    			"321": {
    				label: "Terveydenhuollon tekniset asiantuntijat"
    			},
    			"322": {
    				label: "Sairaanhoitajat, kätilöt ym."
    			},
    			"323": {
    				label: "Luontais- ja vaihtoehtohoitajat"
    			},
    			"324": {
    				label: "Seminologit ym."
    			},
    			"325": {
    				label: "Muut terveydenhuollon asiantuntijat"
    			},
    			"331": {
    				label: "Rahoitus-, vakuutus- ja laskentatoimen asiantuntijat"
    			},
    			"332": {
    				label: "Myynti- ja ostoagentit"
    			},
    			"333": {
    				label: "Yrityspalveluiden välittäjät"
    			},
    			"334": {
    				label: "Hallinnolliset ja erikoistuneet sihteerit"
    			},
    			"335": {
    				label: "Julkishallinnon valmistelu- ja valvontavirkamiehet"
    			},
    			"341": {
    				label: "Lainopilliset asiantuntijat sekä sosiaalialan ja seurakunnan työntekijät"
    			},
    			"342": {
    				label: "Urheilijat, urheiluvalmentajat, liikunnanohjaajat ym."
    			},
    			"343": {
    				label: "Taide- ja kulttuurialan asiantuntijat sekä keittiöpäälliköt"
    			},
    			"351": {
    				label: "Informaatio- ja tietoliikenneteknologian teknikot sekä käyttäjätukihenkilöt"
    			},
    			"352": {
    				label: "Teleliikenne- sekä radio- ja tv-teknikot"
    			},
    			"411": {
    				label: "Toimistoavustajat"
    			},
    			"412": {
    				label: "Yleissihteerit"
    			},
    			"413": {
    				label: "Tekstinkäsittelijät ja tallentajat"
    			},
    			"421": {
    				label: "Rahaliikenteen asiakaspalvelutyöntekijät"
    			},
    			"422": {
    				label: "Muut asiakaspalvelutyöntekijät"
    			},
    			"431": {
    				label: "Palkanlaskijat, vakuutuskäsittelijät ym."
    			},
    			"432": {
    				label: "Kuljetuksen ja varastoinnin toimistotyöntekijät"
    			},
    			"441": {
    				label: "Muut toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"511": {
    				label: "Matkustuspalvelutyöntekijät, konduktöörit ja oppaat"
    			},
    			"512": {
    				label: "Ravintola- ja suurtaloustyöntekijät"
    			},
    			"513": {
    				label: "Tarjoilutyöntekijät"
    			},
    			"514": {
    				label: "Kampaajat, parturit, kosmetologit ym."
    			},
    			"515": {
    				label: "Kiinteistöhuollon ja siivoustyön esimiehet"
    			},
    			"516": {
    				label: "Muut henkilökohtaisen palvelun työntekijät"
    			},
    			"521": {
    				label: "Katu- ja torikauppiaat"
    			},
    			"522": {
    				label: "Myyjät ja kauppiaat"
    			},
    			"523": {
    				label: "Kassanhoitajat ja lipunmyyjät"
    			},
    			"524": {
    				label: "Muut myyntityöntekijät"
    			},
    			"531": {
    				label: "Lastenhoitajat ja koulunkäyntiavustajat"
    			},
    			"532": {
    				label: "Lähihoitajat, muut terveydenhuollon työntekijät ja kodinhoitajat"
    			},
    			"541": {
    				label: "Suojelu- ja vartiointityöntekijät"
    			},
    			"611": {
    				label: "Pelto- ja puutarhaviljelijät"
    			},
    			"612": {
    				label: "Eläintenkasvattajat"
    			},
    			"613": {
    				label: "Yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat"
    			},
    			"621": {
    				label: "Metsurit ja metsätyöntekijät"
    			},
    			"622": {
    				label: "Kalanviljelijät, kalastajat ja metsästäjät"
    			},
    			"631": {
    				label: "Kotitarveviljelijät"
    			},
    			"632": {
    				label: "Kotitarvekarjankasvattajat"
    			},
    			"633": {
    				label: "Kotitarveviljelijät ja -karjankasvattajat"
    			},
    			"634": {
    				label: "Kotitarvekalastajat ja -metsästäjät"
    			},
    			"711": {
    				label: "Rakennustyöntekijät ym."
    			},
    			"712": {
    				label: "Rakennusten viimeistelytyöntekijät"
    			},
    			"713": {
    				label: "Maalarit ja rakennuspuhdistajat"
    			},
    			"721": {
    				label: "Valimotyöntekijät, hitsaajat, levysepät ym."
    			},
    			"722": {
    				label: "Sepät, työkaluntekijät ja koneenasettajat"
    			},
    			"723": {
    				label: "Koneasentajat ja -korjaajat"
    			},
    			"731": {
    				label: "Käsityötuotteiden valmistajat ja hienomekaanikot"
    			},
    			"732": {
    				label: "Painoalan työntekijät"
    			},
    			"741": {
    				label: "Sähkölaitteiden asentajat ja korjaajat"
    			},
    			"742": {
    				label: "Elektroniikka- ja tietoliikenneasentajat ja -korjaajat"
    			},
    			"751": {
    				label: "Lihanleikkaajat, leipurit, meijeristit ym."
    			},
    			"752": {
    				label: "Puutavaran käsittelijät, puusepät ym."
    			},
    			"753": {
    				label: "Vaatetusalan työntekijät ym."
    			},
    			"754": {
    				label: "Muut pääluokkaan 7 luokiteltavat työntekijät"
    			},
    			"811": {
    				label: "Kaivos- ja louhintatyön koneenkäyttäjät"
    			},
    			"812": {
    				label: "Metalliteollisuuden prosessityöntekijät ja viimeistelijät"
    			},
    			"813": {
    				label: "Kemianteollisuuden ja valokuvatuotteiden valmistuksen prosessityöntekijät"
    			},
    			"814": {
    				label: "Kumi-, muovi- ja paperituotteiden valmistuksen prosessityöntekijät"
    			},
    			"815": {
    				label: "Tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät"
    			},
    			"816": {
    				label: "Elintarviketeollisuuden prosessityöntekijät"
    			},
    			"817": {
    				label: "Sahatavaran sekä paperin ja kartongin valmistuksen prosessityöntekijät"
    			},
    			"818": {
    				label: "Muut prosessityöntekijät"
    			},
    			"821": {
    				label: "Teollisuustuotteiden kokoonpanijat"
    			},
    			"831": {
    				label: "Raideliikenteen kuljettajat ja työntekijät"
    			},
    			"832": {
    				label: "Henkilö- ja pakettiauton- sekä moottoripyöränkuljettajat"
    			},
    			"833": {
    				label: "Raskaiden moottoriajoneuvojen kuljettajat"
    			},
    			"834": {
    				label: "Työkoneiden kuljettajat"
    			},
    			"835": {
    				label: "Kansimiehistö ym. vesiliikenteen työntekijät"
    			},
    			"911": {
    				label: "Koti-, hotelli- ja toimistosiivoojat ym."
    			},
    			"912": {
    				label: "Ajoneuvojen puhdistajat, ikkunanpesijät ym."
    			},
    			"921": {
    				label: "Maa-, metsä- ja kalatalouden avustavat työntekijät"
    			},
    			"931": {
    				label: "Avustavat kaivos- ja rakennustyöntekijät"
    			},
    			"932": {
    				label: "Valmistusalan avustavat työntekijät"
    			},
    			"933": {
    				label: "Rahdinkäsittelijät ja varastotyöntekijät ym."
    			},
    			"941": {
    				label: "Avustavat keittiö- ja ruokatyöntekijät"
    			},
    			"951": {
    				label: "Mainosten jakajat, kengänkiillottajat ym."
    			},
    			"952": {
    				label: "Katumyyjät (pl. elintarvikkeet)"
    			},
    			"961": {
    				label: "Jätehuoltotyöntekijät"
    			},
    			"962": {
    				label: "Sanomalehtien jakajat, lähetit ym."
    			},
    			"1111": {
    				label: "Lainsäätäjät"
    			},
    			"1112": {
    				label: "Julkishallinnon ylimmät virkamiehet"
    			},
    			"1113": {
    				label: "Heimo- ja kyläpäälliköt"
    			},
    			"1114": {
    				label: "Järjestöjen johtajat"
    			},
    			"1120": {
    				label: "Toimitusjohtajat ja pääjohtajat"
    			},
    			"1211": {
    				label: "Talousjohtajat"
    			},
    			"1212": {
    				label: "Henkilöstöjohtajat"
    			},
    			"1213": {
    				label: "Politiikka- ja suunnittelujohtajat"
    			},
    			"1219": {
    				label: "Muut hallintojohtajat ja kaupalliset johtajat"
    			},
    			"1221": {
    				label: "Myynti- ja markkinointijohtajat"
    			},
    			"1222": {
    				label: "Mainos- ja tiedotusjohtajat"
    			},
    			"1223": {
    				label: "Tutkimus- ja kehitysjohtajat"
    			},
    			"1311": {
    				label: "Maa- ja metsätalouden johtajat"
    			},
    			"1312": {
    				label: "Vesiviljely- ja kalatalouden johtajat"
    			},
    			"1321": {
    				label: "Teollisuuden tuotantojohtajat"
    			},
    			"1322": {
    				label: "Kaivostoiminnan tuotantojohtajat"
    			},
    			"1323": {
    				label: "Rakennustoiminnan tuotantojohtajat"
    			},
    			"1324": {
    				label: "Hankinta- ja jakelujohtajat"
    			},
    			"1330": {
    				label: "Tieto- ja viestintäteknologiajohtajat"
    			},
    			"1341": {
    				label: "Lastenhoidon johtajat"
    			},
    			"1342": {
    				label: "Terveydenhuollon johtajat"
    			},
    			"1343": {
    				label: "Vanhustenhuollon johtajat"
    			},
    			"1344": {
    				label: "Sosiaalihuollon johtajat"
    			},
    			"1345": {
    				label: "Opetusalan johtajat"
    			},
    			"1346": {
    				label: "Rahoitus- ja vakuutuspalvelujen johtajat"
    			},
    			"1349": {
    				label: "Muut yhteiskunnan palvelujen johtajat"
    			},
    			"1411": {
    				label: "Hotellinjohtajat"
    			},
    			"1412": {
    				label: "Ravintolanjohtajat"
    			},
    			"1420": {
    				label: "Vähittäis- ja tukkukaupan johtajat"
    			},
    			"1431": {
    				label: "Urheilu-, vapaa-aika- ja kulttuurikeskusten johtajat"
    			},
    			"1439": {
    				label: "Muut palvelualojen johtajat"
    			},
    			"2111": {
    				label: "Fyysikot ja astronomit"
    			},
    			"2112": {
    				label: "Meteorologit"
    			},
    			"2113": {
    				label: "Kemistit"
    			},
    			"2114": {
    				label: "Geologit ja geofyysikot"
    			},
    			"2120": {
    				label: "Matemaatikot, aktuaarit ja tilastotieteilijät"
    			},
    			"2131": {
    				label: "Biologit, kasvi- ja eläintieteilijät ym. erityisasiantuntijat"
    			},
    			"2132": {
    				label: "Maa-, metsä- ja kalatalouden erityisasiantuntijat"
    			},
    			"2133": {
    				label: "Ympäristön- ja luonnonsuojelun erityisasiantuntijat"
    			},
    			"2141": {
    				label: "Teollisen valmistuksen ja tuotantotekniikan erityisasiantuntijat"
    			},
    			"2142": {
    				label: "Maa- ja vesirakentamisen erityisasiantuntijat"
    			},
    			"2143": {
    				label: "Ympäristötekniikan erityisasiantuntijat"
    			},
    			"2144": {
    				label: "Konetekniikan erityisasiantuntijat"
    			},
    			"2145": {
    				label: "Puunjalostuksen ja kemian prosessitekniikan erityisasiantuntijat"
    			},
    			"2146": {
    				label: "Kaivosteollisuuden, metallurgian ym. erityisasiantuntijat"
    			},
    			"2149": {
    				label: "Muut tekniikan erityisasiantuntijat"
    			},
    			"2151": {
    				label: "Sähkötekniikan erityisasiantuntijat"
    			},
    			"2152": {
    				label: "Elektroniikan erityisasiantuntijat"
    			},
    			"2153": {
    				label: "ICT-alan erityisasiantuntijat"
    			},
    			"2161": {
    				label: "Talonrakennuksen arkkitehdit"
    			},
    			"2162": {
    				label: "Maisema-arkkitehdit"
    			},
    			"2163": {
    				label: "Tuote- ja vaatesuunnittelijat"
    			},
    			"2164": {
    				label: "Yhdyskunta- ja liikennesuunnittelijat"
    			},
    			"2165": {
    				label: "Kartoituksen ja maanmittauksen erityisasiantuntijat"
    			},
    			"2166": {
    				label: "Graafiset ja multimediasuunnittelijat"
    			},
    			"2211": {
    				label: "Yleislääkärit"
    			},
    			"2212": {
    				label: "Ylilääkärit ja erikoislääkärit"
    			},
    			"2221": {
    				label: "Ylihoitajat ja osastonhoitajat"
    			},
    			"2222": {
    				label: "Johtavat kätilöt"
    			},
    			"2230": {
    				label: "Perinteisen ja vaihtoehtohoidon erityisasiantuntijat"
    			},
    			"2240": {
    				label: "Muiden terveyspalvelujen tuottajat"
    			},
    			"2250": {
    				label: "Eläinlääkärit"
    			},
    			"2261": {
    				label: "Hammaslääkärit"
    			},
    			"2262": {
    				label: "Proviisorit"
    			},
    			"2263": {
    				label: "Ympäristöterveyden ja työsuojelun erityisasiantuntijat"
    			},
    			"2264": {
    				label: "Fysioterapian erityisasiantuntijat"
    			},
    			"2265": {
    				label: "Ravitsemusalan erityisasiantuntijat"
    			},
    			"2266": {
    				label: "Kuulontutkijat ja puheterapeutit"
    			},
    			"2267": {
    				label: "Optometrian erityisasiantuntijat"
    			},
    			"2269": {
    				label: "Muut muualla luokittelemattomat terveydenhuollon erityisasiantuntijat"
    			},
    			"2310": {
    				label: "Yliopistojen ja ammattikorkeakoulujen opettajat"
    			},
    			"2320": {
    				label: "Ammatillisen koulutuksen opettajat"
    			},
    			"2330": {
    				label: "Lukion ja peruskoulun yläluokkien opettajat"
    			},
    			"2341": {
    				label: "Peruskoulun alaluokkien opettajat"
    			},
    			"2342": {
    				label: "Lastentarhanopettajat"
    			},
    			"2351": {
    				label: "Opetusmenetelmien erityisasiantuntijat"
    			},
    			"2352": {
    				label: "Erityisopettajat"
    			},
    			"2353": {
    				label: "Muut kieltenopettajat"
    			},
    			"2354": {
    				label: "Muut musiikin opettajat"
    			},
    			"2355": {
    				label: "Muut taideaineiden opettajat"
    			},
    			"2356": {
    				label: "Muut tietotekniikan opettajat ja kouluttajat"
    			},
    			"2359": {
    				label: "Opinto-ohjaajat ja muut opetuksen erityisasiantuntijat"
    			},
    			"2411": {
    				label: "Laskentatoimen erityisasiantuntijat ja tilintarkastajat"
    			},
    			"2412": {
    				label: "Rahoitus- ja sijoitusneuvojat"
    			},
    			"2413": {
    				label: "Rahoitusanalyytikot"
    			},
    			"2421": {
    				label: "Johtamisen ja organisaatioiden erityisasiantuntijat"
    			},
    			"2422": {
    				label: "Hallinnon ja elinkeinojen kehittämisen erityisasiantuntijat"
    			},
    			"2423": {
    				label: "Henkilöstöhallinnon erityisasiantuntijat ja urasuunnittelijat"
    			},
    			"2424": {
    				label: "Henkilöstön kehittämisen erityisasiantuntijat ja kouluttajat"
    			},
    			"2431": {
    				label: "Mainonnan ja markkinoinnin erityisasiantuntijat"
    			},
    			"2432": {
    				label: "Tiedottajat"
    			},
    			"2433": {
    				label: "Myynti-insinöörit ja lääke-esittelijät (pl. tieto- ja viestintätekniikka)"
    			},
    			"2434": {
    				label: "Tieto- ja viestintätekniikan myynnin erityisasiantuntijat"
    			},
    			"2511": {
    				label: "Sovellusarkkitehdit"
    			},
    			"2512": {
    				label: "Sovellussuunnittelijat"
    			},
    			"2513": {
    				label: "Web- ja multimediakehittäjät"
    			},
    			"2514": {
    				label: "Sovellusohjelmoijat"
    			},
    			"2519": {
    				label: "Muut ohjelmisto- ja sovelluskehittäjät"
    			},
    			"2521": {
    				label: "Tietokantasuunnittelijat ja -vastaavat"
    			},
    			"2522": {
    				label: "Tietojärjestelmien ylläpitäjät"
    			},
    			"2523": {
    				label: "Tietoverkkojen erityisasiantuntijat"
    			},
    			"2529": {
    				label: "Muut tietokanta- ja tietoverkkojen erityisasiantuntijat"
    			},
    			"2611": {
    				label: "Asianajajat"
    			},
    			"2612": {
    				label: "Tuomioistuinlakimiehet"
    			},
    			"2619": {
    				label: "Muut lainopilliset erityisasiantuntijat"
    			},
    			"2621": {
    				label: "Arkistonhoitajat ja museoalan erityisasiantuntijat"
    			},
    			"2622": {
    				label: "Kirjastonhoitajat, informaatikot ym."
    			},
    			"2631": {
    				label: "Ekonomistit"
    			},
    			"2632": {
    				label: "Yhteiskunta- ja kulttuuritutkijat"
    			},
    			"2633": {
    				label: "Historioitsijat, politiikan tutkijat ja filosofit"
    			},
    			"2634": {
    				label: "Psykologit"
    			},
    			"2635": {
    				label: "Sosiaalityön erityisasiantuntijat"
    			},
    			"2636": {
    				label: "Papit ym. uskonnollisen elämän erityisasiantuntijat"
    			},
    			"2641": {
    				label: "Kirjailijat ym."
    			},
    			"2642": {
    				label: "Toimittajat"
    			},
    			"2643": {
    				label: "Kääntäjät, tulkit ja muut kielitieteilijät"
    			},
    			"2651": {
    				label: "Kuvataiteilijat"
    			},
    			"2652": {
    				label: "Muusikot, laulajat ja säveltäjät"
    			},
    			"2653": {
    				label: "Tanssitaiteilijat ja koreografit"
    			},
    			"2654": {
    				label: "Ohjaajat ja tuottajat"
    			},
    			"2655": {
    				label: "Näyttelijät"
    			},
    			"2656": {
    				label: "Juontajat, kuuluttajat ym."
    			},
    			"2659": {
    				label: "Muut taiteilijat"
    			},
    			"3111": {
    				label: "Luonnontieteen tekniset asiantuntijat"
    			},
    			"3112": {
    				label: "Rakentamisen asiantuntijat"
    			},
    			"3113": {
    				label: "Sähkötekniikan asiantuntijat"
    			},
    			"3114": {
    				label: "Elektroniikan asiantuntijat"
    			},
    			"3115": {
    				label: "Konetekniikan asiantuntijat"
    			},
    			"3116": {
    				label: "Kemian prosessitekniikan asiantuntijat"
    			},
    			"3117": {
    				label: "Kaivosteollisuuden ja metallurgian asiantuntijat"
    			},
    			"3118": {
    				label: "Tekniset piirtäjät"
    			},
    			"3119": {
    				label: "Muut fysiikan, kemian ja teknisten alojen asiantuntijat"
    			},
    			"3121": {
    				label: "Kaivostyönjohtajat"
    			},
    			"3122": {
    				label: "Teollisuuden työnjohtajat"
    			},
    			"3123": {
    				label: "Rakennusalan työnjohtajat"
    			},
    			"3131": {
    				label: "Voimalaitosten prosessinhoitajat"
    			},
    			"3132": {
    				label: "Jätteenpoltto- ja vedenpuhdistuslaitosten prosessinhoitajat"
    			},
    			"3133": {
    				label: "Kemianteollisuuden prosessinhoitajat"
    			},
    			"3134": {
    				label: "Öljy- ja maakaasujalostamon prosessinhoitajat"
    			},
    			"3135": {
    				label: "Metallien jalostuksen prosessinhoitajat"
    			},
    			"3139": {
    				label: "Muut prosessinvalvonnan asiantuntijat"
    			},
    			"3141": {
    				label: "Laborantit ym."
    			},
    			"3142": {
    				label: "Maa- ja kalatalousteknikot"
    			},
    			"3143": {
    				label: "Metsätalousteknikot"
    			},
    			"3151": {
    				label: "Laivojen konepäälliköt ja -mestarit"
    			},
    			"3152": {
    				label: "Vesiliikenteen perämiehet ja päälliköt"
    			},
    			"3153": {
    				label: "Lentokapteenit ja -perämiehet"
    			},
    			"3154": {
    				label: "Lennonjohtajat"
    			},
    			"3155": {
    				label: "Lennonvalvonnan tekniset asiantuntijat"
    			},
    			"3211": {
    				label: "Lääketieteellisen kuvantamis- ja laitetekniikan asiantuntijat"
    			},
    			"3212": {
    				label: "Bioanalyytikot (terveydenhuolto)"
    			},
    			"3213": {
    				label: "Farmaseutit"
    			},
    			"3214": {
    				label: "Hammas- ja apuvälineteknikot"
    			},
    			"3221": {
    				label: "Sairaanhoitajat ym."
    			},
    			"3222": {
    				label: "Kätilöt"
    			},
    			"3230": {
    				label: "Luontais- ja vaihtoehtohoitajat"
    			},
    			"3240": {
    				label: "Seminologit ym."
    			},
    			"3251": {
    				label: "Suuhygienistit"
    			},
    			"3252": {
    				label: "Potilas- ja terveystietojen käsittelijät"
    			},
    			"3253": {
    				label: "Terveysneuvojat"
    			},
    			"3254": {
    				label: "Optikot"
    			},
    			"3255": {
    				label: "Fysioterapeutit ym."
    			},
    			"3256": {
    				label: "Avustavat hoitotyöntekijät"
    			},
    			"3257": {
    				label: "Terveys- ja työsuojelutarkastajat"
    			},
    			"3258": {
    				label: "Sairaankuljetuksen ensihoitajat"
    			},
    			"3259": {
    				label: "Muut muualla luokittelemattomat terveydenhuollon asiantuntijat"
    			},
    			"3311": {
    				label: "Arvopaperi- ja valuuttakauppiaat"
    			},
    			"3312": {
    				label: "Luotto- ja laina-asiantuntijat"
    			},
    			"3313": {
    				label: "Kirjanpidon ja laskentatoimen asiantuntijat"
    			},
    			"3314": {
    				label: "Tilastointi- ja matematiikka-asiantuntijat"
    			},
    			"3315": {
    				label: "Arvioitsijat ja vahinkotarkastajat"
    			},
    			"3321": {
    				label: "Vakuutusalan palvelumyyjät"
    			},
    			"3322": {
    				label: "Myyntiedustajat"
    			},
    			"3323": {
    				label: "Sisäänostajat"
    			},
    			"3324": {
    				label: "Kaupanvälittäjät"
    			},
    			"3331": {
    				label: "Huolitsijat, tulli- ja laivanselvittäjät"
    			},
    			"3332": {
    				label: "Konferenssi- ja tapahtumajärjestäjät"
    			},
    			"3333": {
    				label: "Työnvälittäjät"
    			},
    			"3334": {
    				label: "Kiinteistönvälittäjät ja isännöitsijät"
    			},
    			"3339": {
    				label: "Muut liike-elämän asiantuntijat"
    			},
    			"3341": {
    				label: "Toimistotyön esimiehet"
    			},
    			"3342": {
    				label: "Asianajosihteerit"
    			},
    			"3343": {
    				label: "Johdon sihteerit ja osastosihteerit"
    			},
    			"3344": {
    				label: "Toimistosihteerit (terveydenhuolto)"
    			},
    			"3351": {
    				label: "Tulli- ja rajavirkamiehet"
    			},
    			"3352": {
    				label: "Verovalmistelijat ja -tarkastajat"
    			},
    			"3353": {
    				label: "Sosiaaliturvaetuuksien käsittelijät"
    			},
    			"3354": {
    				label: "Lupavirkamiehet"
    			},
    			"3355": {
    				label: "Komisariot ja ylikonstaapelit"
    			},
    			"3359": {
    				label: "Muut julkishallinnon valmistelu- ja valvontavirkamiehet"
    			},
    			"3411": {
    				label: "Lainopilliset avustajat ja järjestöalan asiantuntijat"
    			},
    			"3412": {
    				label: "Sosiaalialan ohjaajat ja neuvojat ym."
    			},
    			"3413": {
    				label: "Seurakuntatyöntekijät"
    			},
    			"3421": {
    				label: "Urheilijat"
    			},
    			"3422": {
    				label: "Urheiluvalmentajat ja toimitsijat"
    			},
    			"3423": {
    				label: "Liikunnan ja vapaa-ajan ohjaajat"
    			},
    			"3431": {
    				label: "Valokuvaajat"
    			},
    			"3432": {
    				label: "Sisustussuunnittelijat ym."
    			},
    			"3433": {
    				label: "Gallerioiden, museoiden ja kirjastojen tekniset työntekijät"
    			},
    			"3434": {
    				label: "Keittiöpäälliköt"
    			},
    			"3435": {
    				label: "Muut taide- ja kulttuurialan asiantuntijat"
    			},
    			"3511": {
    				label: "Käytön operaattorit"
    			},
    			"3512": {
    				label: "Käytön tukihenkilöt"
    			},
    			"3513": {
    				label: "Tietoverkkoteknikot"
    			},
    			"3514": {
    				label: "Webmasterit ja -teknikot"
    			},
    			"3521": {
    				label: "Lähetys- ja audiovisuaaliteknikot"
    			},
    			"3522": {
    				label: "Televiestinnän tekniset asiantuntijat"
    			},
    			"4110": {
    				label: "Toimistoavustajat"
    			},
    			"4120": {
    				label: "Yleissihteerit"
    			},
    			"4131": {
    				label: "Tekstinkäsittelijät"
    			},
    			"4132": {
    				label: "Tallentajat"
    			},
    			"4211": {
    				label: "Pankki- ym. toimihenkilöt"
    			},
    			"4212": {
    				label: "Vedonvälittäjät, bingo- ja kasinopelin hoitajat ym."
    			},
    			"4213": {
    				label: "Panttilainaajat"
    			},
    			"4214": {
    				label: "Maksujenperijät"
    			},
    			"4221": {
    				label: "Matkatoimistovirkailijat"
    			},
    			"4222": {
    				label: "Puhelinpalveluneuvojat"
    			},
    			"4223": {
    				label: "Puhelinvaihteenhoitajat"
    			},
    			"4224": {
    				label: "Hotellin vastaanottovirkailijat"
    			},
    			"4225": {
    				label: "Informaatiopisteen asiakasneuvojat"
    			},
    			"4226": {
    				label: "Vastaanoton ja neuvonnan hoitajat"
    			},
    			"4227": {
    				label: "Tutkimus- ja markkinatutkimushaastattelijat"
    			},
    			"4229": {
    				label: "Muualla luokittelemattomat asiakaspalvelutyöntekijät"
    			},
    			"4311": {
    				label: "Taloushallinnon toimistotyöntekijät"
    			},
    			"4312": {
    				label: "Tilasto-, rahoitus- ja vakuutusalan toimistotyöntekijät"
    			},
    			"4313": {
    				label: "Palkanlaskijat"
    			},
    			"4321": {
    				label: "Varastonhoitajat ym."
    			},
    			"4322": {
    				label: "Tuotannon valmistelijat"
    			},
    			"4323": {
    				label: "Kuljetuksen toimistotyöntekijät"
    			},
    			"4411": {
    				label: "Kirjastotyöntekijät"
    			},
    			"4412": {
    				label: "Postinkantajat ja -lajittelijat"
    			},
    			"4413": {
    				label: "Koodaajat, oikolukijat ym."
    			},
    			"4414": {
    				label: "Kirjurit ym."
    			},
    			"4415": {
    				label: "Arkistotyöntekijät"
    			},
    			"4416": {
    				label: "Henkilöstöhallinnon avustavat toimistotyöntekijät"
    			},
    			"4419": {
    				label: "Muut muualla luokittelemattomat toimisto- ja asiakaspalvelutyöntekijät"
    			},
    			"5111": {
    				label: "Lentoemännät, purserit ym."
    			},
    			"5112": {
    				label: "Konduktöörit, lipuntarkastajat ym."
    			},
    			"5113": {
    				label: "Matkaoppaat"
    			},
    			"5120": {
    				label: "Ravintola- ja suurtaloustyöntekijät"
    			},
    			"5131": {
    				label: "Tarjoilijat"
    			},
    			"5132": {
    				label: "Baarimestarit"
    			},
    			"5141": {
    				label: "Kampaajat ja parturit"
    			},
    			"5142": {
    				label: "Kosmetologit ym."
    			},
    			"5151": {
    				label: "Siivoustyön esimiehet toimistoissa, hotelleissa ja muissa laitoksissa"
    			},
    			"5152": {
    				label: "Yksityiskotien taloudenhoitajat"
    			},
    			"5153": {
    				label: "Kiinteistöhuollon työntekijät"
    			},
    			"5161": {
    				label: "Astrologit, ennustajat ym."
    			},
    			"5162": {
    				label: "Henkilökohtaiset palvelijat"
    			},
    			"5163": {
    				label: "Hautauspalvelutyöntekijät"
    			},
    			"5164": {
    				label: "Eläintenhoitajat ja lemmikkieläinten trimmaajat"
    			},
    			"5165": {
    				label: "Ajo-opettajat"
    			},
    			"5169": {
    				label: "Muualla luokittelemattomat henkilökohtaisen palvelun työntekijät"
    			},
    			"5211": {
    				label: "Kioski- ja torimyyjät"
    			},
    			"5212": {
    				label: "Katumyyjät (elintarvikkeet)"
    			},
    			"5221": {
    				label: "Kauppiaat (pienyrittäjät)"
    			},
    			"5222": {
    				label: "Myymäläesimiehet"
    			},
    			"5223": {
    				label: "Myyjät"
    			},
    			"5230": {
    				label: "Kassanhoitajat ja lipunmyyjät"
    			},
    			"5241": {
    				label: "Mallit"
    			},
    			"5242": {
    				label: "Tuote-esittelijät"
    			},
    			"5243": {
    				label: "Suoramyyjät"
    			},
    			"5244": {
    				label: "Puhelin- ja asiakaspalvelukeskusten myyjät"
    			},
    			"5245": {
    				label: "Huoltamotyöntekijät"
    			},
    			"5246": {
    				label: "Kahvila- ja baarimyyjät"
    			},
    			"5249": {
    				label: "Muut muualla luokittelemattomat myyntityöntekijät"
    			},
    			"5311": {
    				label: "Lastenhoitotyöntekijät"
    			},
    			"5312": {
    				label: "Koulunkäyntiavustajat"
    			},
    			"5321": {
    				label: "Lähihoitajat"
    			},
    			"5322": {
    				label: "Kodinhoitajat (kotipalvelutoiminta)"
    			},
    			"5329": {
    				label: "Muut terveydenhuoltoalan työntekijät"
    			},
    			"5411": {
    				label: "Palomiehet"
    			},
    			"5412": {
    				label: "Poliisit"
    			},
    			"5413": {
    				label: "Vanginvartijat"
    			},
    			"5414": {
    				label: "Vartijat"
    			},
    			"5419": {
    				label: "Muut suojelu- ja vartiointityöntekijät"
    			},
    			"6111": {
    				label: "Pelto- ja avomaaviljelijät"
    			},
    			"6112": {
    				label: "Hedelmäpuiden ja pensaiden yms. kasvattajat"
    			},
    			"6113": {
    				label: "Puutarhurit, kasvihuoneviljelijät ja -työntekijät"
    			},
    			"6114": {
    				label: "Yhdistetyn maan- ja vihannesviljelyn tai puutarhanhoidon ym. harjoittajat"
    			},
    			"6121": {
    				label: "Liha- ja lypsykarjan kasvattajat sekä muiden kotieläinten kasvattajat"
    			},
    			"6122": {
    				label: "Siipikarjankasvattajat"
    			},
    			"6123": {
    				label: "Mehiläistenhoitajat ym."
    			},
    			"6129": {
    				label: "Muut eläinten kasvattajat ja hoitajat"
    			},
    			"6130": {
    				label: "Yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat"
    			},
    			"6210": {
    				label: "Metsurit ja metsätyöntekijät"
    			},
    			"6221": {
    				label: "Kalanviljelijät ja -viljelytyöntekijät"
    			},
    			"6222": {
    				label: "Kalastajat"
    			},
    			"6223": {
    				label: "Syvänmerenkalastajat"
    			},
    			"6224": {
    				label: "Riistanhoitajat ja metsästäjät"
    			},
    			"6310": {
    				label: "Kotitarveviljelijät"
    			},
    			"6320": {
    				label: "Kotitarvekarjankasvattajat"
    			},
    			"6330": {
    				label: "Kotitarveviljelijät ja -karjankasvattajat"
    			},
    			"6340": {
    				label: "Kotitarvekalastajat ja -metsästäjät"
    			},
    			"7111": {
    				label: "Talonrakentajat"
    			},
    			"7112": {
    				label: "Muurarit ym."
    			},
    			"7113": {
    				label: "Kivenhakkaajat ja -leikkaajat ym."
    			},
    			"7114": {
    				label: "Betonirakentajat ja raudoittajat"
    			},
    			"7115": {
    				label: "Kirvesmiehet ja rakennuspuusepät"
    			},
    			"7119": {
    				label: "Muut rakennustyöntekijät"
    			},
    			"7121": {
    				label: "Kattoasentajat ja -korjaajat"
    			},
    			"7122": {
    				label: "Lattianpäällystystyöntekijät"
    			},
    			"7123": {
    				label: "Rappaajat"
    			},
    			"7124": {
    				label: "Eristäjät"
    			},
    			"7125": {
    				label: "Lasinasentajat"
    			},
    			"7126": {
    				label: "Putkiasentajat"
    			},
    			"7127": {
    				label: "Ilmastointi- ja jäähdytyslaiteasentajat"
    			},
    			"7131": {
    				label: "Rakennusmaalarit ym."
    			},
    			"7132": {
    				label: "Ruiskumaalaajat ja -lakkaajat"
    			},
    			"7133": {
    				label: "Rakennuspuhdistajat ja nuohoojat"
    			},
    			"7211": {
    				label: "Muotin- ja keernantekijät"
    			},
    			"7212": {
    				label: "Hitsaajat ja kaasuleikkaajat"
    			},
    			"7213": {
    				label: "Ohutlevysepät"
    			},
    			"7214": {
    				label: "Paksulevysepät ja rautarakennetyöntekijät"
    			},
    			"7215": {
    				label: "Kaapelin- ja köysienasentajat"
    			},
    			"7221": {
    				label: "Sepät"
    			},
    			"7222": {
    				label: "Työkaluntekijät ja lukkosepät"
    			},
    			"7223": {
    				label: "Koneenasettajat ja koneistajat"
    			},
    			"7224": {
    				label: "Konehiojat, kiillottajat ja teroittajat"
    			},
    			"7231": {
    				label: "Moottoriajoneuvojen asentajat ja korjaajat"
    			},
    			"7232": {
    				label: "Lentokoneasentajat ja -korjaajat"
    			},
    			"7233": {
    				label: "Maatalous- ja teollisuuskoneasentajat ja -korjaajat"
    			},
    			"7234": {
    				label: "Polkupyöränkorjaajat ym."
    			},
    			"7311": {
    				label: "Kellosepät ja muut hienomekaanisten instrumenttien tekijät ja korjaajat"
    			},
    			"7312": {
    				label: "Soittimien tekijät ja virittäjät"
    			},
    			"7313": {
    				label: "Koru-, kulta- ja hopeasepät"
    			},
    			"7314": {
    				label: "Saven- ja tiilenvalajat ja dreijaajat"
    			},
    			"7315": {
    				label: "Lasinpuhaltajat, -leikkaajat, -hiojat ja -viimeistelijät"
    			},
    			"7316": {
    				label: "Kaivertajat, etsaajat ja koristemaalarit"
    			},
    			"7317": {
    				label: "Puu-, kori- yms. käsityötuotteiden tekijät"
    			},
    			"7318": {
    				label: "Tekstiili-, nahka- yms. käsityötuotteiden tekijät"
    			},
    			"7319": {
    				label: "Muut käsityöntekijät"
    			},
    			"7321": {
    				label: "Painopinnanvalmistajat"
    			},
    			"7322": {
    				label: "Painajat"
    			},
    			"7323": {
    				label: "Jälkikäsittelijät ja sitomotyöntekijät"
    			},
    			"7411": {
    				label: "Rakennussähköasentajat"
    			},
    			"7412": {
    				label: "Muut sähköasentajat"
    			},
    			"7413": {
    				label: "Linja-asentajat ja -korjaajat"
    			},
    			"7421": {
    				label: "Elektroniikka- ja automaatiolaitteiden asentajat ja korjaajat"
    			},
    			"7422": {
    				label: "Tieto- ja viestintäteknologian asentajat ja korjaajat"
    			},
    			"7511": {
    				label: "Lihanleikkaajat, kalankäsittelijät ym."
    			},
    			"7512": {
    				label: "Leipurit ja kondiittorit"
    			},
    			"7513": {
    				label: "Meijeristit, juustomestarit ym."
    			},
    			"7514": {
    				label: "Hedelmä- ja vihannestuotteiden valmistajat"
    			},
    			"7515": {
    				label: "Ruokien ja juomien laaduntarkkailijat"
    			},
    			"7516": {
    				label: "Tupakkatuotteiden valmistajat"
    			},
    			"7521": {
    				label: "Raakapuun käsittelijät"
    			},
    			"7522": {
    				label: "Huonekalupuusepät ym."
    			},
    			"7523": {
    				label: "Konepuusepät"
    			},
    			"7531": {
    				label: "Vaatturit, pukuompelijat, turkkurit ja hatuntekijät"
    			},
    			"7532": {
    				label: "Leikkaajat ja mallimestarit"
    			},
    			"7533": {
    				label: "Koru- ja muut tekstiiliompelijat"
    			},
    			"7534": {
    				label: "Verhoilijat"
    			},
    			"7535": {
    				label: "Turkisten muokkaajat ja nahkurit"
    			},
    			"7536": {
    				label: "Suutarit ym."
    			},
    			"7541": {
    				label: "Vedenalaistyöntekijät"
    			},
    			"7542": {
    				label: "Panostajat ja räjäyttäjät"
    			},
    			"7543": {
    				label: "Luokittelijat ja laaduntarkkailijat (pl. ruoat ja juomat)"
    			},
    			"7544": {
    				label: "Savupuhdistajat, tuholais- ja rikkakasvintorjujat"
    			},
    			"7549": {
    				label: "Muut pääluokkaan 7 muualla luokittelemattomat työntekijät"
    			},
    			"8111": {
    				label: "Kaivos- ja louhostyöntekijät"
    			},
    			"8112": {
    				label: "Rikastustyöntekijät"
    			},
    			"8113": {
    				label: "Iskuporaajat ja syväkairaajat"
    			},
    			"8114": {
    				label: "Betonituote- ym. teollisuuden prosessityöntekijät"
    			},
    			"8121": {
    				label: "Metalliteollisuuden prosessityöntekijät"
    			},
    			"8122": {
    				label: "Metallien teolliset päällystäjät ja viimeistelijät"
    			},
    			"8131": {
    				label: "Kemianteollisuuden prosessityöntekijät ym."
    			},
    			"8132": {
    				label: "Valokuvatuotteiden valmistuksen prosessityöntekijät"
    			},
    			"8141": {
    				label: "Kumituoteteollisuuden prosessityöntekijät"
    			},
    			"8142": {
    				label: "Muovituoteteollisuuden prosessityöntekijät"
    			},
    			"8143": {
    				label: "Paperituoteteollisuuden prosessityöntekijät"
    			},
    			"8151": {
    				label: "Kuitujenkäsittely-, kehruu- ja puolauskoneiden hoitajat"
    			},
    			"8152": {
    				label: "Kutoma- ja neulekoneiden hoitajat"
    			},
    			"8153": {
    				label: "Teollisuusompelijat"
    			},
    			"8154": {
    				label: "Valkaisu-, värjäys- ja puhdistuskoneiden hoitajat"
    			},
    			"8155": {
    				label: "Turkisten ja nahkojen teolliset käsittelijät, värjääjät ym."
    			},
    			"8156": {
    				label: "Jalkine- ja laukkuteollisuuden prosessityöntekijät"
    			},
    			"8157": {
    				label: "Pesulatyöntekijät"
    			},
    			"8159": {
    				label: "Muut tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät"
    			},
    			"8160": {
    				label: "Elintarviketeollisuuden prosessityöntekijät"
    			},
    			"8171": {
    				label: "Paperimassan sekä paperin ja kartongin valmistuksen prosessityöntekijät"
    			},
    			"8172": {
    				label: "Puu- ja sahatavaran prosessityöntekijät"
    			},
    			"8181": {
    				label: "Lasi- ja keramiikkateollisuuden uunienhoitajat"
    			},
    			"8182": {
    				label: "Höyrykoneiden ja lämmityskattiloiden hoitajat, lämmittäjät ym."
    			},
    			"8183": {
    				label: "Pakkaus-, pullotus- ja etiköintikoneiden hoitajat"
    			},
    			"8189": {
    				label: "Muut muualla luokittelemattomat prosessityöntekijät"
    			},
    			"8211": {
    				label: "Konepaja- ja metallituotteiden kokoonpanijat"
    			},
    			"8212": {
    				label: "Sähkö- ja elektroniikkalaitteiden kokoonpanijat"
    			},
    			"8219": {
    				label: "Muut teollisuustuotteiden kokoonpanijat"
    			},
    			"8311": {
    				label: "Veturinkuljettajat"
    			},
    			"8312": {
    				label: "Jarru-, turvalaite- ja vaihdetyöntekijät"
    			},
    			"8321": {
    				label: "Moottoripyörälähetit yms."
    			},
    			"8322": {
    				label: "Henkilö-, taksi- ja pakettiautonkuljettajat"
    			},
    			"8331": {
    				label: "Linja-auton- ja raitiovaununkuljettajat"
    			},
    			"8332": {
    				label: "Kuorma-auton ja erikoisajoneuvojen kuljettajat"
    			},
    			"8341": {
    				label: "Maa- ja metsätaloustyökoneiden kuljettajat"
    			},
    			"8342": {
    				label: "Maansiirtokoneiden ym. kuljettajat"
    			},
    			"8343": {
    				label: "Nosturinkuljettajat"
    			},
    			"8344": {
    				label: "Ahtaajat ja trukinkuljettajat ym."
    			},
    			"8350": {
    				label: "Kansimiehistö ym. vesiliikenteen työntekijät"
    			},
    			"9111": {
    				label: "Kotiapulaiset ja -siivoojat"
    			},
    			"9112": {
    				label: "Toimisto- ja laitossiivoojat ym."
    			},
    			"9121": {
    				label: "Puhdistajat ja prässääjät"
    			},
    			"9122": {
    				label: "Ajoneuvojen pesijät"
    			},
    			"9123": {
    				label: "Ikkunanpesijät"
    			},
    			"9129": {
    				label: "Muut puhdistustyöntekijät"
    			},
    			"9211": {
    				label: "Maanviljelyn avustavat työntekijät"
    			},
    			"9212": {
    				label: "Karjankasvatuksen avustavat työntekijät"
    			},
    			"9213": {
    				label: "Yhdistetyn maanviljelyn ja karjankasvatuksen avustavat työntekijät"
    			},
    			"9214": {
    				label: "Avustavat puutarhatyöntekijät"
    			},
    			"9215": {
    				label: "Metsätalouden avustavat työntekijät"
    			},
    			"9216": {
    				label: "Kalatalouden ja vesiviljelyn avustavat työntekijät"
    			},
    			"9311": {
    				label: "Kaivosten avustavat työntekijät"
    			},
    			"9312": {
    				label: "Maa- ja vesirakentamisen avustavat työntekijät"
    			},
    			"9313": {
    				label: "Rakennusalan avustavat työntekijät"
    			},
    			"9321": {
    				label: "Käsinpakkaajat"
    			},
    			"9329": {
    				label: "Muut valmistusalan avustavat työntekijät"
    			},
    			"9331": {
    				label: "Käsivetoisten- ja poljinkulkuneuvojen kuljettajat"
    			},
    			"9332": {
    				label: "Eläinvetoisten kulkuneuvojen kuljettajat"
    			},
    			"9333": {
    				label: "Rahdinkäsittelijät, varastotyöntekijät ym."
    			},
    			"9334": {
    				label: "Hyllyjen täyttäjät"
    			},
    			"9411": {
    				label: "Pikaruokatyöntekijät"
    			},
    			"9412": {
    				label: "Avustavat keittiötyöntekijät"
    			},
    			"9510": {
    				label: "Mainosten jakajat, kengänkiillottajat ym."
    			},
    			"9520": {
    				label: "Katumyyjät (pl. elintarvikkeet)"
    			},
    			"9611": {
    				label: "Jätteiden kerääjät"
    			},
    			"9612": {
    				label: "Jätteiden lajittelijat"
    			},
    			"9613": {
    				label: "Kadunlakaisijat ym."
    			},
    			"9621": {
    				label: "Sanomalehtien jakajat, lähetit ja kantajat"
    			},
    			"9622": {
    				label: "Satunnaistöiden tekijät"
    			},
    			"9623": {
    				label: "Mittareiden lukijat ym."
    			},
    			"9624": {
    				label: "Vedenhakijat ja polttopuun kerääjät"
    			},
    			"9629": {
    				label: "Muut muualla luokittelemattomat työntekijät"
    			},
    			"11121": {
    				label: "Valtion keskushallinnon johtajat"
    			},
    			"11122": {
    				label: "Alue- ja paikallishallinnon johtajat ja ylimmät virkamiehet"
    			},
    			"11141": {
    				label: "Työmarkkina- ja elinkeinoelämän järjestöjen johtajat"
    			},
    			"11142": {
    				label: "Muiden järjestöjen johtajat"
    			},
    			"21321": {
    				label: "Maa- ja kalatalouden erityisasiantuntijat"
    			},
    			"21322": {
    				label: "Metsätalouden erityisasiantuntijat"
    			},
    			"21531": {
    				label: "Tietoliikenneteknologian tutkijat"
    			},
    			"21532": {
    				label: "Tietotekniikka-alan tutkijat"
    			},
    			"22121": {
    				label: "Ylilääkärit"
    			},
    			"22122": {
    				label: "Erikoislääkärit"
    			},
    			"22211": {
    				label: "Ylihoitajat"
    			},
    			"22212": {
    				label: "Osastonhoitajat"
    			},
    			"23101": {
    				label: "Professorit"
    			},
    			"23102": {
    				label: "Lehtorit ja yliassistentit (yliopisto)"
    			},
    			"23103": {
    				label: "Assistentit ja tuntiopettajat (yliopisto)"
    			},
    			"23104": {
    				label: "Yliopettajat (AMK)"
    			},
    			"23105": {
    				label: "Lehtorit (AMK)"
    			},
    			"23106": {
    				label: "Tuntiopettajat ym. (AMK)"
    			},
    			"23301": {
    				label: "Matemaattisten aineiden opettajat"
    			},
    			"23302": {
    				label: "Äidinkielen ja kirjallisuuden opettajat"
    			},
    			"23303": {
    				label: "Taito- ja taideaineiden opettajat"
    			},
    			"23304": {
    				label: "Muut peruskoulun yläluokkien ja lukion opettajat"
    			},
    			"23411": {
    				label: "Luokanopettajat"
    			},
    			"23412": {
    				label: "Aineenopettajat (peruskoulun alaluokat)"
    			},
    			"23591": {
    				label: "Opinto-ohjaajat"
    			},
    			"23592": {
    				label: "Muut opetuksen erityisasiantuntijat"
    			},
    			"26211": {
    				label: "Arkistonhoitajat"
    			},
    			"26212": {
    				label: "Museoalan erityisasiantuntijat"
    			},
    			"26351": {
    				label: "Sosiaalityöntekijät ym."
    			},
    			"26352": {
    				label: "Sosiaalialan suunnittelijat ym."
    			},
    			"26421": {
    				label: "Toimituspäälliköt ja -sihteerit, ohjelmapäälliköt"
    			},
    			"26422": {
    				label: "Lehtien yms. toimittajat"
    			},
    			"26423": {
    				label: "Radio- ja tv-toimittajat"
    			},
    			"31121": {
    				label: "Talonrakentamisen asiantuntijat"
    			},
    			"31122": {
    				label: "Maa- ja vesirakentamisen asiantuntijat"
    			},
    			"31123": {
    				label: "Kartoituksen ja maanmittauksen asiantuntijat"
    			},
    			"31521": {
    				label: "Isojen alusten päälliköt ja perämiehet"
    			},
    			"31522": {
    				label: "Pienten alusten päälliköt"
    			},
    			"31523": {
    				label: "Satamaliikenteen ohjaajat ja satamakapteenit"
    			},
    			"32141": {
    				label: "Hammasteknikot"
    			},
    			"32142": {
    				label: "Apuvälineteknikot"
    			},
    			"32211": {
    				label: "Sairaanhoitajat"
    			},
    			"32212": {
    				label: "Terveydenhoitajat"
    			},
    			"32591": {
    				label: "Toimintaterapeutit"
    			},
    			"32592": {
    				label: "Muut terapeutit"
    			},
    			"33341": {
    				label: "Kiinteistönvälittäjät"
    			},
    			"33342": {
    				label: "Isännöitsijät"
    			},
    			"34111": {
    				label: "Oikeudenkäyntiasiamiehet ja ulosottomiehet"
    			},
    			"34112": {
    				label: "Asiamiehet, toimitsijat ym. järjestöalan asiantuntijat"
    			},
    			"34113": {
    				label: "Kuluttajaneuvojat ym."
    			},
    			"34121": {
    				label: "Sosiaalialan ohjaajat"
    			},
    			"34122": {
    				label: "Nuorisotyön ohjaajat (ei srk.)"
    			},
    			"34123": {
    				label: "Työn- ja askarteluohjaajat"
    			},
    			"34131": {
    				label: "Diakonit ja diakonissat"
    			},
    			"34139": {
    				label: "Muut seurakuntatyöntekijät"
    			},
    			"34351": {
    				label: "Kuvaussihteerit ja muut näyttämötekniset asiantuntijat"
    			},
    			"34359": {
    				label: "Muut taide- ja kulttuurialan tekniset asiantuntijat"
    			},
    			"42291": {
    				label: "Hälytyspäivystäjät"
    			},
    			"42299": {
    				label: "Muut muualla luokittelemattomat asiakaspalvelutyöntekijät"
    			},
    			"43231": {
    				label: "Raideliikenteen ohjaajat, ajojärjestelijät ym."
    			},
    			"43239": {
    				label: "Muut kuljetuksen ja huolinnan toimistotyöntekijät"
    			},
    			"44121": {
    				label: "Postinkantajat"
    			},
    			"44122": {
    				label: "Toimistovahtimestarit"
    			},
    			"51201": {
    				label: "Kokit, keittäjät ja kylmäköt"
    			},
    			"51202": {
    				label: "Ravintolaesimiehet ja vuoropäälliköt"
    			},
    			"51631": {
    				label: "Hautaustoimistonhoitajat ym."
    			},
    			"51632": {
    				label: "Muut hautaustyöntekijät"
    			},
    			"53111": {
    				label: "Päiväkotien ja muiden laitosten lastenhoitajat ym."
    			},
    			"53112": {
    				label: "Perhepäivähoitajat"
    			},
    			"53113": {
    				label: "Lastenkerhojen ohjaajat ym."
    			},
    			"53211": {
    				label: "Mielenterveyshoitajat"
    			},
    			"53212": {
    				label: "Kehitysvammaisten hoitajat"
    			},
    			"53213": {
    				label: "Sosiaalialan hoitajat"
    			},
    			"53219": {
    				label: "Muut lähihoitajat"
    			},
    			"53221": {
    				label: "Kotityöpalvelutyöntekijät"
    			},
    			"53222": {
    				label: "Henkilökohtaiset avustajat, omaishoitajat ym."
    			},
    			"53291": {
    				label: "Hammashoitajat"
    			},
    			"53292": {
    				label: "Välinehuoltajat"
    			},
    			"53293": {
    				label: "Apteekkien lääketyöntekijät"
    			},
    			"53294": {
    				label: "Hierojat ja kuntohoitajat"
    			},
    			"61111": {
    				label: "Peltoviljelijät"
    			},
    			"61112": {
    				label: "Peltoviljelytyönjohtajat ja -työntekijät"
    			},
    			"61131": {
    				label: "Puutarhurit ja kasvihuoneviljelijät"
    			},
    			"61132": {
    				label: "Puutarha- ja kasvihuonetyönjohtajat ja - työntekijät"
    			},
    			"61211": {
    				label: "Karjankasvattajat ym."
    			},
    			"61212": {
    				label: "Karjanhoitajat ym."
    			},
    			"61213": {
    				label: "Lemmikkieläinten kasvattajat"
    			},
    			"61214": {
    				label: "Maatalouslomittajat"
    			},
    			"61291": {
    				label: "Turkiseläinten ja porojen kasvattajat"
    			},
    			"61299": {
    				label: "Muualla luokittelemattomat eläintenhoitajat"
    			},
    			"62211": {
    				label: "Kalanviljely-yrittäjät"
    			},
    			"62212": {
    				label: "Kalanviljelytyönjohtajat ja -työntekijät"
    			},
    			"74211": {
    				label: "Elektroniikka-asentajat ja -korjaajat"
    			},
    			"74212": {
    				label: "Automaatioasentajat ja -korjaajat"
    			},
    			"81821": {
    				label: "Voima- ja jätteenkäsittelylaitosten laitosmiehet"
    			},
    			"81822": {
    				label: "Muut laitosmiehet ym."
    			},
    			"83441": {
    				label: "Ahtaajat"
    			},
    			"83442": {
    				label: "Trukinkuljettajat ym."
    			},
    			"91121": {
    				label: "Toimistosiivoojat ym."
    			},
    			"91122": {
    				label: "Hotellisiivoojat"
    			},
    			"91123": {
    				label: "Sairaala- ja laitosapulaiset"
    			},
    			"91124": {
    				label: "Päiväkotiapulaiset"
    			},
    			"91129": {
    				label: "Muut muualla luokittelemattomat siivoojat"
    			},
    			"01": {
    				label: "Upseerit"
    			},
    			"011": {
    				label: "Upseerit"
    			},
    			"0110": {
    				label: "Upseerit"
    			},
    			"02": {
    				label: "Aliupseerit"
    			},
    			"021": {
    				label: "Aliupseerit"
    			},
    			"0210": {
    				label: "Aliupseerit"
    			},
    			"03": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"031": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"0310": {
    				label: "Sotilasammattihenkilöstö"
    			},
    			"#X": {
    				label: "Tuntematon"
    			},
    			"#XX": {
    				label: "Tuntematon"
    			},
    			"#XXX": {
    				label: "Tuntematon"
    			},
    			"#XXXX": {
    				label: "Tuntematon"
    			},
    			"#XXXXX": {
    				label: "Tuntematon"
    			}
    		},
    		RemunerationKind: {
    			bonusPay: {
    				description: "Kertaluonteinen palkkiopalkka, jonka suuruus määräytyy kokonaan tai osittain sen mukaan, kuinka hyvin tavoitteeksi asetettu työtulos saavutetaan. Työtuloksen mittana voi olla määrä, laatu tai jokin muu tulostekijä.",
    				label: "Bonuspalkka"
    			},
    			commission: {
    				description: "Pysyvän palkan osan muodostava jatkuvaluonteinen palkkiopalkka, jonka määrästä on sovittu työsopimuksessa.",
    				label: "Provisiopalkka"
    			},
    			initiativeFee: {
    				description: "Kertapalkkio, joka maksetaan aloitteen tekijälle esimerkiksi toiminnan tai tuotteen kehittämiseen liittyvästä aloitteesta.",
    				label: "Aloitepalkkio"
    			},
    			performanceBonus: {
    				description: "Palkka, joka maksetaan organisaation ennalta sovitun tulostavoitteen täyttymisen tai ylittymisen perusteella.",
    				label: "Tulospalkkio"
    			},
    			profitSharingBonus: {
    				description: "Työntekijöille ilman ennalta sovittua suunnitelmaa yhtiökokouksen päätöksellä jaettava palkkio, joka muodostuu yrityksen voiton perusteella.",
    				label: "Voittopalkkio"
    			},
    			shareIssueForEmployees: {
    				description: "Yhtiön omalle henkilökunnalle suunnattu osakeanti. Etu työsuhteeseen perustuvasta oikeudesta merkitä työnantajayrityksen osakkeita on veronalaista ansiotuloa.",
    				label: "Työsuhteeseen perustuva osakeanti"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			}
    		},
    		ReportCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			monthly: {
    				description: "",
    				label: "Monthly"
    			},
    			yearly: {
    				description: "",
    				label: "Yearly"
    			}
    		},
    		ReportType: {
    			employerReport: {
    				description: "",
    				label: "Työnantajan laskelma"
    			},
    			employmentContract: {
    				description: "",
    				label: "Työsopimus"
    			},
    			example: {
    				description: "",
    				label: "Esimerkki"
    			},
    			householdDeduction: {
    				description: "",
    				label: "Kotitalousvähennys"
    			},
    			insurance: {
    				description: "",
    				label: "Vakuutus"
    			},
    			monthlyDetails: {
    				description: "",
    				label: "Kuukausittaisia tietoja"
    			},
    			monthlyLiikekirjuri: {
    				description: "",
    				label: ""
    			},
    			monthlyPension: {
    				description: "",
    				label: "Kuukausittainen eläke"
    			},
    			monthlyRapko: {
    				description: "",
    				label: ""
    			},
    			paymentReport: {
    				description: "",
    				label: "Maksuraportti"
    			},
    			salarySlip: {
    				description: "",
    				label: "Työntekijän palkkalaskelma"
    			},
    			salarySlipPaid: {
    				description: "",
    				label: "Työntekijän palkkalaskelma, maksettu"
    			},
    			taxHouseholdDeduction14B: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseA: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseB: {
    				description: "",
    				label: ""
    			},
    			taxMonthly4001: {
    				description: "",
    				label: "Kuukausittainen vero"
    			},
    			taxYearly7801: {
    				description: "",
    				label: "Vuosittainen vero"
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritelty"
    			},
    			unemployment: {
    				description: "",
    				label: "Työttömyys"
    			},
    			yearEndReport: {
    				description: "",
    				label: "Loppuvuoden raportti"
    			},
    			yearlyDetails: {
    				description: "",
    				label: "Vuosittaisia tietoja"
    			},
    			yearlyWorkerSummary: {
    				description: "",
    				label: ""
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "EarningsPaymentReport"
    			},
    			employersSeparateReport: {
    				description: "",
    				label: "EmployersSeparateReport"
    			},
    			monthlyAccounting: {
    				description: "",
    				label: "MonthlyAccounting"
    			},
    			salarySlipCopy: {
    				description: "",
    				label: "SalarySlipCopy"
    			}
    		},
    		SalaryKind: {
    			compensation: {
    				description: "",
    				label: "Korvaus",
    				labels: {
    					household: "Työkorvaus"
    				}
    			},
    			fixedSalary: {
    				description: "",
    				label: "Kiinteä palkka"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Tuntipalkka"
    			},
    			monthlySalary: {
    				description: "",
    				label: "Kuukausipalkka"
    			},
    			salary: {
    				description: "",
    				label: "Kertakorvaus"
    			},
    			totalEmployerPayment: {
    				description: "",
    				label: ""
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Ei määritelty"
    			}
    		},
    		SalaryPeriod: {
    			fixed: {
    				description: "",
    				label: "Fixed"
    			},
    			hourly: {
    				description: "",
    				label: "Hourly"
    			},
    			monthly: {
    				description: "",
    				label: "Monthly"
    			},
    			notDefined: {
    				description: "",
    				label: "Not Defined"
    			}
    		},
    		SalaryPerMonthRange: {
    			noPensionRequired: {
    				description: "",
    				label: "Alle 57,10 €"
    			},
    			normal: {
    				description: "",
    				label: "57,10 € tai yli"
    			},
    			unknown: {
    				description: "",
    				label: "Ei tiedossa"
    			}
    		},
    		SalaryPerYearRange: {
    			noMonthlyReporting: {
    				description: "",
    				label: "201 - 1500 €"
    			},
    			noReportingNecessary: {
    				description: "",
    				label: "1 - 200 €"
    			},
    			normal: {
    				description: "",
    				label: "Yli 1500 €"
    			},
    			zeroSalary: {
    				description: "",
    				label: 0
    			}
    		},
    		ServiceModel: {
    			partnerOnly: {
    				label: "Tilitoimistokäyttö"
    			},
    			shared: {
    				label: "Yhteiskäyttö"
    			}
    		},
    		SubsidisedCommuteKind: {
    			noDeduction: {
    				description: "Ei vähennystä työntekijän palkasta",
    				label: "Ei vähennystä"
    			},
    			periodicalDeduction: {
    				description: "Vähennys työntekijän palkasta jaetaan palkkakausiin (oletus 12 kuukautta).",
    				label: "Vähennys jaetaan palkkakausille"
    			},
    			singleDeduction: {
    				description: "Vähennys työntekijän palkasta otetaan kerralla (tästä palkkalaskelmasta)",
    				label: "Vähennys palkasta kerralla"
    			},
    			"undefined": {
    				description: "Ei valintaa, oletus \"Ei vähennystä\"",
    				label: "Ei valintaa"
    			}
    		},
    		TaxCardIncomeType: {
    			externalSalaries: {
    				description: "",
    				label: "Ulkopuoliset palkat"
    			},
    			previousEmployerSalaries: {
    				description: "Merkitse tähän työntekijän edellisten työnantajien kuluvana verovuonna maksamat palkat yhteensä euroina.",
    				label: "Edellisten työnantajien maksamat palkat"
    			},
    			salaxyCalculation: {
    				description: "",
    				label: "Salaxy laskelma"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon"
    			},
    			diff: {
    				description: "",
    				label: "Diff"
    			},
    			sharedCardExtSum: {
    				description: "",
    				label: "SharedCardExtSum"
    			}
    		},
    		TaxcardKind: {
    			defaultYearly: {
    				label: "Alkuperäinen verokortti"
    			},
    			auto: {
    				label: "Automaattisesti päivittyvä verokortti"
    			},
    			historical: {
    				label: "Vanha verokortti"
    			},
    			noTaxCard: {
    				label: "Ei verokorttia (60%)"
    			},
    			noWithholdingHousehold: {
    				label: "Ei ennakonpidätystä (alle 1500€)"
    			},
    			others: {
    				label: "Muu verokortti"
    			},
    			replacement: {
    				label: "Muutosverokortti"
    			},
    			"undefined": {
    				label: "Verokorttia ei ole määritetty"
    			}
    		},
    		TaxcardState: {
    			employerAdded: {
    				label: "Työnantajan lisäämä"
    			},
    			expired: {
    				label: "Vanhentunut"
    			},
    			"new": {
    				label: "Uusi verokortti"
    			},
    			replaced: {
    				label: "Korvattu (ei käytössä)"
    			},
    			shared: {
    				label: "Työntekijän jakama"
    			},
    			sharedApproved: {
    				label: "Työnantajan hyväksymä"
    			},
    			sharedRejected: {
    				label: "Työnantaja hylännyt"
    			},
    			sharedRejectedWithoutOpen: {
    				label: "Työnantaja hylännyt avaamatta"
    			},
    			sharedWaiting: {
    				label: "Odottaa hyväksyntää"
    			},
    			verifiedVero: {
    				label: "Haettu Verolta"
    			},
    			approved: {
    				description: "",
    				label: "Approved"
    			}
    		},
    		TaxcardValidity: {
    			expired: {
    				label: "Vanhentunut"
    			},
    			future: {
    				label: "Tuleva"
    			},
    			"undefined": {
    				label: "Virhe"
    			},
    			valid: {
    				label: "Voimassa"
    			},
    			validJanuary: {
    				label: "Tammikuun voimassa"
    			}
    		},
    		TaxDeductionWorkCategories: {
    			carework: {
    				description: "",
    				label: "Hoiva- tai hoitotyö"
    			},
    			homeImprovement: {
    				description: "",
    				label: "Asunnon kunnossapito- tai perusparannustyö"
    			},
    			householdwork: {
    				description: "",
    				label: "Kotitaloustyö"
    			},
    			none: {
    				description: "",
    				label: "Ei valintaa"
    			},
    			ownPropety: {
    				description: "",
    				label: "Omassa käytössä oleva asunto tai vapaa-ajan asunto"
    			},
    			relativesProperty: {
    				description: "",
    				label: "Vanhempien, isovanhempien yms. käytössä oleva asunto tai vapaa-ajan asunto."
    			}
    		},
    		TesSubtype: {
    			constructionCarpenter: {
    				description: "",
    				label: "Construction Carpenter"
    			},
    			constructionFloor: {
    				description: "",
    				label: "Construction Floor"
    			},
    			constructionFreeContract: {
    				description: "",
    				label: "Construction Free Contract"
    			},
    			constructionOther: {
    				description: "",
    				label: "Construction Other"
    			},
    			notSelected: {
    				description: "",
    				label: "Not Selected"
    			}
    		},
    		TestEnum: {
    			valueEight: {
    				description: "",
    				label: "Value Eight"
    			},
    			valueFive: {
    				description: "",
    				label: "Value Five"
    			},
    			valueFour: {
    				description: "",
    				label: "Value Four"
    			},
    			valueNine: {
    				description: "",
    				label: "Value Nine"
    			},
    			valueOne: {
    				description: "",
    				label: "Value One"
    			},
    			valueSevdescription: "",
    			valueSevlabel: "Value Seven",
    			valueSix: {
    				description: "",
    				label: "Value Six"
    			},
    			valueTdescription: "",
    			valueThree: {
    				description: "",
    				label: "Value Three"
    			},
    			valueTlabel: "Value Ten",
    			valueTwo: {
    				description: "",
    				label: "Value Two"
    			},
    			valueZero: {
    				description: "",
    				label: "Value Zero"
    			},
    			valueSeven: {
    				description: "",
    				label: "ValueSeven"
    			},
    			valueTen: {
    				description: "",
    				label: "ValueTen"
    			}
    		},
    		UnionPaymentKind: {
    			fixed: {
    				description: "Yleinen kiinteä AY-jäsenmaksu.",
    				label: "AY-jäsenmaksu, kiinteä summa"
    			},
    			percentage: {
    				description: "Yleinen jäsenmaksu, lasketaan %-osuudella palkasta.",
    				label: "AY-jäsenmaksu, %-osuus palkasta"
    			},
    			other: {
    				description: "Muu liitto",
    				label: "AY-jäsenmaksu, muu liitto"
    			},
    			raksaNormal: {
    				description: "Rakennusliiton normaalijäsenyys",
    				label: "AY-jäsenmaksu, Rakennusliitto A-jäsen"
    			},
    			raksaUnemploymentOnly: {
    				description: "Rakennusliiton jäsenyys vain työttömyyskassassa",
    				label: "AY-jäsenmaksu, Rakennusliitto OA- tai TA-jäsen"
    			},
    			"undefined": {
    				description: "AY-jäsenmaksun tyyppiä ei valittu",
    				label: "Ei jäsenyyttä"
    			}
    		},
    		UnionPaymentType: {
    			notSelected: {
    				description: "",
    				label: "Ei jäsenyyttä"
    			},
    			other: {
    				description: "",
    				label: "Muu liitto..."
    			},
    			raksaNormal: {
    				description: "",
    				label: "Rakennusliitto, A-jäsen"
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: "Rakennusliitto, OA- tai TA-jäsen"
    			}
    		},
    		WageBasis: {
    			hourly: {
    				label: "Tuntipalkka"
    			},
    			monthly: {
    				label: "Kuukausipalkka"
    			},
    			other: {
    				label: "Palkanmaksu ilman lomalaskentaa"
    			},
    			performanceBased: {
    				label: "Suorituspalkka"
    			},
    			"undefined": {
    				label: "Valitse..."
    			}
    		},
    		WebSiteUserRole: {
    			company: {
    				description: "",
    				label: "Yrittäjä"
    			},
    			household: {
    				description: "",
    				label: "Työnantaja (kotitalous)"
    			},
    			none: {
    				description: "",
    				label: "Valitse rooli..."
    			},
    			worker: {
    				description: "",
    				label: "Työntekijä"
    			}
    		},
    		TotalWorkerPaymentKind: {
    			totalWorkerPayment: {
    				description: "Rahapalkka ja kulukorvaukset",
    				label: "Rahapalkka ja kulukorvaukset"
    			},
    			totalWorkerWorkerPaymentExcludingExpenses: {
    				description: "Rahapalkka, ei sisällä kulukorvauksia",
    				label: "Rahapalkka, ei sisällä kulukorvauksia"
    			},
    			totalWorkerPaymentAndTax: {
    				description: "Rahapalkka ja ennakonpidätys",
    				label: "Rahapalkka ja ennakonpidätys"
    			},
    			totalWorkerWorkerPaymentAndTaxExcludingExpenses: {
    				description: "Rahapalkka ja ennakonpidätys, ei sisällä kulukorvauksia",
    				label: "Rahapalkka ja ennakonpidätys, ei sisällä kulukorvauksia"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			}
    		},
    		WorkingTimeCompensationKind: {
    			emergencyWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan hätätyöstä. Hätätyöllä tarkoitetaan hätätilanteessa säännöllisen työajan ylittävänä aikana teetettävää työtä.",
    				label: "Hätätyökorvaus"
    			},
    			eveningShiftAllowance: {
    				description: "Vuorotyökorvaus, joka maksetaan iltavuorosta.",
    				label: "Iltavuorolisä"
    			},
    			eveningWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan iltatyöstä. Iltatyökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Iltatyökorvaus"
    			},
    			extraWorkPremium: {
    				description: "Työaikakorvaus, joka maksetaan lisätyöstä.",
    				label: "Lisätyökorvaus"
    			},
    			nightShiftCompensation: {
    				description: "Vuorotyökorvaus, joka maksetaan yövuorosta.",
    				label: "Yövuorolisä"
    			},
    			nightWorkAllowance: {
    				description: "Työaikakorvaus, jota maksetaan yötyöstä.",
    				label: "Yötyökorvaus"
    			},
    			otherCompensation: {
    				description: "Muiden maksettujen työaika- ja poikkeustilakorvausten yhteissumma. Näitä ovat mm. aattotyökorvaukset, arkipyhäkorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella\n maksettavat korvaukset ja lisät.",
    				label: "Muu maksettu lisä"
    			},
    			overtimeCompensation: {
    				description: "Ylityöstä maksettava työaikakorvaus. Ylityönä pidetään työtä, joka tehdään työnantajan aloitteesta sovitun ylityöaikarajan ylittävänä aikana.",
    				label: "Ylityökorvaus"
    			},
    			saturdayPay: {
    				description: "Työaikakorvaus, joka maksetaan lauantaityöstä.\n Lauantaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Lauantaityökorvaus"
    			},
    			standByCompensation: {
    				description: "Palkka, joka maksetaan siitä, että työntekijä on tavoitettavissa ja työnantaja voi tarvittaessa kutsua hänet hoitamaan palvelussuhteeseen kuuluvia tehtäviä.",
    				label: "Varallaolokorvaus"
    			},
    			sundayWorkCompensation: {
    				description: "Sunnuntaityöstä maksettava työaikakorvaus. Sunnuntaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Sunnuntaityökorvaus"
    			},
    			"undefined": {
    				description: "Ei valintaa",
    				label: "Ei valintaa"
    			},
    			waitingTimeCompensation: {
    				description: "Korvaus, johon työntekijällä on oikeus, jos palkanmaksu viivästyy työsuhteen päätyttyä. Ellei toisin sovita, palkka tulee maksaa palkanmaksukauden\n viimeisenä päivänä. Odotusajan korvausta voidaan\n maksaa enintään kuudelta päivältä.",
    				label: "Odotusajan korvaus"
    			},
    			weeklyRestCompensation: {
    				description: "Työaikakorvaus, jota maksetaan viikkolevon aikana tehdystä työstä.",
    				label: "Viikkolepokorvaus"
    			}
    		},
    		YtjCompanyType: {
    			alandFederation: {
    				description: "",
    				label: "Ahvenanmaan liikelaitos"
    			},
    			asoAssociation: {
    				description: "",
    				label: "Asumisoikeusyhdistys"
    			},
    			association: {
    				description: "",
    				label: "Aatteellinen yhdistys"
    			},
    			condominium: {
    				description: "",
    				label: "Asunto-osakeyhtiö"
    			},
    			cooperativeBank: {
    				description: "",
    				label: "Osuuspankki"
    			},
    			europeanCooperative: {
    				description: "",
    				label: "Eurooppaosuuskunta"
    			},
    			europeanCooperativeBank: {
    				description: "",
    				label: "Eurooppaosuuspankki"
    			},
    			federationOfMunicipalitiesEstablishment: {
    				description: "",
    				label: "Kuntainliiton liikelaitos"
    			},
    			financialAssociation: {
    				description: "",
    				label: "Taloudellinen yhdistys"
    			},
    			forestCareAssociation: {
    				description: "",
    				label: "Metsänhoitoyhdistys"
    			},
    			foundation: {
    				description: "",
    				label: "Säätiö"
    			},
    			housingCooperative: {
    				description: "",
    				label: "Asunto-osuuskunta"
    			},
    			hypoAssociation: {
    				description: "",
    				label: "Hypoteekkiyhdistys"
    			},
    			insuranceAssociation: {
    				description: "",
    				label: "Vakuutusyhdistys"
    			},
    			ky: {
    				description: "",
    				label: "Kommandiittiyhtiö"
    			},
    			municipalEstablishment: {
    				description: "",
    				label: "Kunnallinen liikelaitos"
    			},
    			mutualInsuranceAssociation: {
    				description: "",
    				label: "Keskinäinen vahinkovak.yhdistys"
    			},
    			noCompanyType: {
    				description: "",
    				label: "Ei yritysmuotoa"
    			},
    			openCompany: {
    				description: "",
    				label: "Avoin yhtiö"
    			},
    			osuuskunta: {
    				description: "",
    				label: "Osuuskunta"
    			},
    			otherAssociation: {
    				description: "",
    				label: "Muu yhdistys"
    			},
    			otherFinancialAssociation: {
    				description: "",
    				label: "Muu taloudellinen yhdistys"
    			},
    			oy: {
    				description: "",
    				label: "Osakeyhtiö"
    			},
    			privateEntrepreneur: {
    				description: "",
    				label: "Yksityinen elinkeinonharjoittaja l. toiminimi"
    			},
    			reindeerHerdingCooperative: {
    				description: "",
    				label: "Paliskunta"
    			},
    			savingsBank: {
    				description: "",
    				label: "Säästöpankki"
    			},
    			specialPurposeAssociation: {
    				description: "",
    				label: "Erityislainsäädäntöön perustuva yhdistys"
    			},
    			stateEstablishment: {
    				description: "",
    				label: "Valtion liikelaitos"
    			},
    			unknown: {
    				description: "",
    				label: "Tuntematon yritysmuoto"
    			},
    			mutualRealEstateCompany: {
    				description: "",
    				label: "MutualRealEstateCompany"
    			},
    			taxableGrouping: {
    				description: "",
    				label: "TaxableGrouping"
    			}
    		},
    		AccountantType: {
    			primaryPartner: {
    				description: "",
    				label: "PrimaryPartner"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			unlinkedAccountingOnly: {
    				description: "",
    				label: "UnlinkedAccountingOnly"
    			},
    			unlinkedPrimaryPartner: {
    				description: "",
    				label: "UnlinkedPrimaryPartner"
    			}
    		},
    		AccountingPeriodClosingOption: {
    			"default": {
    				description: "",
    				label: "Default"
    			}
    		},
    		AccountingReportRowType: {
    			childRow: {
    				description: "",
    				label: "ChildRow"
    			},
    			credit: {
    				description: "",
    				label: "Credit"
    			},
    			debit: {
    				description: "",
    				label: "Debit"
    			},
    			groupHeader: {
    				description: "",
    				label: "GroupHeader"
    			},
    			groupTotal: {
    				description: "",
    				label: "GroupTotal"
    			},
    			total: {
    				description: "",
    				label: "Total"
    			}
    		},
    		AccountingReportTableType: {
    			classic: {
    				description: "",
    				label: "Classic"
    			},
    			mapped: {
    				description: "",
    				label: "Mapped"
    			},
    			simple: {
    				description: "",
    				label: "Simple"
    			}
    		},
    		ApiListItemType: {
    			accountBase: {
    				description: "",
    				label: "AccountBase"
    			},
    			accountProducts: {
    				description: "",
    				label: "AccountProducts"
    			},
    			article: {
    				description: "",
    				label: "Article"
    			},
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			calculationPaid: {
    				description: "",
    				label: "CalculationPaid"
    			},
    			certificate: {
    				description: "",
    				label: "Certificate"
    			},
    			companyAccount: {
    				description: "",
    				label: "CompanyAccount"
    			},
    			eSalaryPayment: {
    				description: "",
    				label: "ESalaryPayment"
    			},
    			earningsPayment: {
    				description: "",
    				label: "EarningsPayment"
    			},
    			emailMessage: {
    				description: "",
    				label: "EmailMessage"
    			},
    			employment: {
    				description: "",
    				label: "Employment"
    			},
    			historical: {
    				description: "",
    				label: "Historical"
    			},
    			holidayYear: {
    				description: "",
    				label: "HolidayYear"
    			},
    			ifInsuranceOrder: {
    				description: "",
    				label: "IfInsuranceOrder"
    			},
    			invoice: {
    				description: "",
    				label: "Invoice"
    			},
    			messageThread: {
    				description: "",
    				label: "MessageThread"
    			},
    			onboarding: {
    				description: "",
    				label: "Onboarding"
    			},
    			payment: {
    				description: "",
    				label: "Payment"
    			},
    			payrollDetails: {
    				description: "",
    				label: "PayrollDetails"
    			},
    			personAccount: {
    				description: "",
    				label: "PersonAccount"
    			},
    			profile: {
    				description: "",
    				label: "Profile"
    			},
    			report: {
    				description: "",
    				label: "Report"
    			},
    			sessionUserCredential: {
    				description: "",
    				label: "SessionUserCredential"
    			},
    			taxcard: {
    				description: "",
    				label: "Taxcard"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			varmaPensionOrder: {
    				description: "",
    				label: "VarmaPensionOrder"
    			},
    			workerAbsences: {
    				description: "",
    				label: "WorkerAbsences"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			},
    			workerInvitation: {
    				description: "",
    				label: "WorkerInvitation"
    			}
    		},
    		BankPaymentType: {
    			bankFees: {
    				description: "",
    				label: "BankFees"
    			},
    			error: {
    				description: "",
    				label: "Error"
    			},
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			insuranceGrossPayment: {
    				description: "",
    				label: "InsuranceGrossPayment"
    			},
    			insuranceLahiTapiola: {
    				description: "",
    				label: "InsuranceLahiTapiola"
    			},
    			netSalary: {
    				description: "",
    				label: "NetSalary"
    			},
    			netvisorPayment: {
    				description: "",
    				label: "NetvisorPayment"
    			},
    			palkkausFees: {
    				description: "",
    				label: "PalkkausFees"
    			},
    			partialRefundPayment: {
    				description: "",
    				label: "PartialRefundPayment"
    			},
    			paytrailBatch: {
    				description: "",
    				label: "PaytrailBatch"
    			},
    			paytrailFees: {
    				description: "",
    				label: "PaytrailFees"
    			},
    			refundPayment: {
    				description: "",
    				label: "RefundPayment"
    			},
    			salaryAdvance: {
    				description: "",
    				label: "SalaryAdvance"
    			},
    			salaryGrossPayment: {
    				description: "",
    				label: "SalaryGrossPayment"
    			},
    			saldoYearBegin: {
    				description: "",
    				label: "SaldoYearBegin"
    			},
    			saldoYearEnd: {
    				description: "",
    				label: "SaldoYearEnd"
    			},
    			taxAccount: {
    				description: "",
    				label: "TaxAccount"
    			},
    			tvr: {
    				description: "",
    				label: "Tvr"
    			},
    			tvrRfnd: {
    				description: "",
    				label: "TvrRfnd"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			tyelEloContract: {
    				description: "",
    				label: "TyelEloContract"
    			},
    			tyelEloInitialImport: {
    				description: "",
    				label: "TyelEloInitialImport"
    			},
    			tyelEloTemp: {
    				description: "",
    				label: "TyelEloTemp"
    			},
    			tyelEteraContract: {
    				description: "",
    				label: "TyelEteraContract"
    			},
    			tyelEteraTemp: {
    				description: "",
    				label: "TyelEteraTemp"
    			},
    			tyelEteraUnknown: {
    				description: "",
    				label: "TyelEteraUnknown"
    			},
    			tyelIlmarinenContract: {
    				description: "",
    				label: "TyelIlmarinenContract"
    			},
    			tyelIlmarinenTemp: {
    				description: "",
    				label: "TyelIlmarinenTemp"
    			},
    			tyelVarmaContract: {
    				description: "",
    				label: "TyelVarmaContract"
    			},
    			tyelVarmaTemp: {
    				description: "",
    				label: "TyelVarmaTemp"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			},
    			unionRaksa: {
    				description: "",
    				label: "UnionRaksa"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				description: "",
    				label: "AccommodationBenefit"
    			},
    			mealBenefit: {
    				description: "",
    				label: "MealBenefit"
    			},
    			otherBenefits: {
    				description: "",
    				label: "OtherBenefits"
    			},
    			telephoneBenefit: {
    				description: "",
    				label: "TelephoneBenefit"
    			}
    		},
    		CalcGroup: {
    			baseSalary: {
    				description: "",
    				label: "BaseSalary"
    			},
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			disabled: {
    				description: "",
    				label: "Disabled"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			otherNoPayment: {
    				description: "",
    				label: "OtherNoPayment"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			totals: {
    				description: "",
    				label: "Totals"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationResultRowType: {
    			totalBenefits: {
    				description: "",
    				label: "TotalBenefits"
    			},
    			totalExpenses: {
    				description: "",
    				label: "TotalExpenses"
    			},
    			totalPalkkaus: {
    				description: "",
    				label: "TotalPalkkaus"
    			},
    			totalPayment: {
    				description: "",
    				label: "TotalPayment"
    			},
    			totalPension: {
    				description: "",
    				label: "TotalPension"
    			},
    			totalPensionEmployer: {
    				description: "",
    				label: "TotalPensionEmployer"
    			},
    			totalPensionEmployerDebt: {
    				description: "",
    				label: "TotalPensionEmployerDebt"
    			},
    			totalPensionPayment: {
    				description: "",
    				label: "TotalPensionPayment"
    			},
    			totalPensionWorker: {
    				description: "",
    				label: "TotalPensionWorker"
    			},
    			totalSalary: {
    				description: "",
    				label: "TotalSalary"
    			},
    			totalSocialSecurityEmployer: {
    				description: "",
    				label: "TotalSocialSecurityEmployer"
    			},
    			totalSocialSecurityEmployerDebt: {
    				description: "",
    				label: "TotalSocialSecurityEmployerDebt"
    			},
    			totalSocialSecurityEmployerPayment: {
    				description: "",
    				label: "TotalSocialSecurityEmployerPayment"
    			},
    			totalTax: {
    				description: "",
    				label: "TotalTax"
    			},
    			totalTaxPayment: {
    				description: "",
    				label: "TotalTaxPayment"
    			},
    			totalUnemployment: {
    				description: "",
    				label: "TotalUnemployment"
    			},
    			totalUnemploymentEmployer: {
    				description: "",
    				label: "TotalUnemploymentEmployer"
    			},
    			totalUnemploymentEmployerDebt: {
    				description: "",
    				label: "TotalUnemploymentEmployerDebt"
    			},
    			totalUnemploymentPayment: {
    				description: "",
    				label: "TotalUnemploymentPayment"
    			},
    			totalUnemploymentWorker: {
    				description: "",
    				label: "TotalUnemploymentWorker"
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: "TotalWorkerPayment"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationRowCategory: {
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			holidays: {
    				description: "",
    				label: "Holidays"
    			},
    			other: {
    				description: "",
    				label: "Other"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			salaryCalc: {
    				description: "",
    				label: "SalaryCalc"
    			}
    		},
    		CalculationStatusCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			editable: {
    				description: "",
    				label: "Editable"
    			},
    			readonly: {
    				description: "",
    				label: "Readonly"
    			},
    			received: {
    				description: "",
    				label: "Received"
    			},
    			sent: {
    				description: "",
    				label: "Sent"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			}
    		},
    		ESalaryPaymentStatus: {
    			calculationCreated: {
    				description: "",
    				label: "CalculationCreated"
    			},
    			created: {
    				description: "",
    				label: "Created"
    			},
    			employerResolved: {
    				description: "",
    				label: "EmployerResolved"
    			},
    			employmentResolved: {
    				description: "",
    				label: "EmploymentResolved"
    			},
    			opened: {
    				description: "",
    				label: "Opened"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "PaymentCanceled"
    			},
    			paymentMethodResolved: {
    				description: "",
    				label: "PaymentMethodResolved"
    			},
    			paymentStarted: {
    				description: "",
    				label: "PaymentStarted"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "PaymentSucceeded"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			},
    			taxcardResolved: {
    				description: "",
    				label: "TaxcardResolved"
    			}
    		},
    		EmploymentRelationStatus: {
    			archived: {
    				description: "",
    				label: "Archived"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			secondaryCurrent: {
    				description: "",
    				label: "SecondaryCurrent"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		Holiday: {
    			allSaintsDay: {
    				description: "",
    				label: "AllSaintsDay"
    			},
    			ascensionDay: {
    				description: "",
    				label: "AscensionDay"
    			},
    			christmasDay: {
    				description: "",
    				label: "ChristmasDay"
    			},
    			christmasEve: {
    				description: "",
    				label: "ChristmasEve"
    			},
    			easterMonday: {
    				description: "",
    				label: "EasterMonday"
    			},
    			easterSaturday: {
    				description: "",
    				label: "EasterSaturday"
    			},
    			easterSunday: {
    				description: "",
    				label: "EasterSunday"
    			},
    			epiphany: {
    				description: "",
    				label: "Epiphany"
    			},
    			goodFriday: {
    				description: "",
    				label: "GoodFriday"
    			},
    			independenceDay: {
    				description: "",
    				label: "IndependenceDay"
    			},
    			mayDay: {
    				description: "",
    				label: "MayDay"
    			},
    			midsummerDay: {
    				description: "",
    				label: "MidsummerDay"
    			},
    			midsummerEve: {
    				description: "",
    				label: "MidsummerEve"
    			},
    			newYearsDay: {
    				description: "",
    				label: "NewYearsDay"
    			},
    			pentecost: {
    				description: "",
    				label: "Pentecost"
    			},
    			stStephensDay: {
    				description: "",
    				label: "StStephensDay"
    			}
    		},
    		HolidayAccrualSource: {
    			calcDraft: {
    				description: "",
    				label: "CalcDraft"
    			},
    			calcPaid: {
    				description: "",
    				label: "CalcPaid"
    			},
    			initial: {
    				description: "",
    				label: "Initial"
    			},
    			manual: {
    				description: "",
    				label: "Manual"
    			}
    		},
    		HolidayGroup: {
    			holiday: {
    				description: "",
    				label: "Holiday"
    			},
    			nonBanking: {
    				description: "",
    				label: "NonBanking"
    			},
    			official: {
    				description: "",
    				label: "Official"
    			}
    		},
    		IncomeEarnerType: {
    			athlete: {
    				description: "",
    				label: "Urheilija"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "Valtion työllisyysmäärärahoilla työllistetty"
    			},
    			entrepreneurOrFarmerNoPensionRequired: {
    				description: "",
    				label: "Yrittäjä, ei YEL- tai MYEL-vakuuttamisvelvollisuutta"
    			},
    			jointOwnerWithPayer: {
    				description: "",
    				label: "Osakasasemassa maksajaan"
    			},
    			keyEmployee: {
    				description: "",
    				label: "Avainhenkilö"
    			},
    			leasedEmployeeLivingAbroad: {
    				description: "",
    				label: "Ulkomailla asuva vuokratyöntekijä"
    			},
    			netOfTaxContract: {
    				description: "",
    				label: "Työnantaja maksaa työntekijän puolesta verot (nettopalkkasopimus)"
    			},
    			organization: {
    				description: "",
    				label: "Yhteisö"
    			},
    			partialOwner: {
    				description: "",
    				label: "Osaomistaja"
    			},
    			performingArtist: {
    				description: "",
    				label: "Esiintyvä taiteilija"
    			},
    			personWorkingAbroad: {
    				description: "",
    				label: "Ulkomailla työskentelevä henkilö"
    			},
    			personWorkingInFrontierDistrict: {
    				description: "",
    				label: "Rajakunnassa työskentelevä henkilö"
    			},
    			personWorkingOnAlandFerry: {
    				description: "",
    				label: "Ahvenanmaan maantielautalla työskentelevä henkilö"
    			},
    			restrictedPeriodInFinland: {
    				description: "",
    				label: "Tulonsaaja on oleskellut Suomessa enintään 183 päivää verosopimuksessa tarkoitettuna ajanjaksona"
    			}
    		},
    		IncomeLogDiff: {
    			changed: {
    				description: "",
    				label: "Changed"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			"new": {
    				description: "",
    				label: "New"
    			},
    			removed: {
    				description: "",
    				label: "Removed"
    			}
    		},
    		InvoicePaymentType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			}
    		},
    		InvoiceType: {
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			gross: {
    				description: "",
    				label: "Gross"
    			},
    			net: {
    				description: "",
    				label: "Net"
    			},
    			pension: {
    				description: "",
    				label: "Pension"
    			},
    			tax: {
    				description: "",
    				label: "Tax"
    			},
    			taxSocialSecurity: {
    				description: "",
    				label: "TaxSocialSecurity"
    			},
    			taxWithholding: {
    				description: "",
    				label: "TaxWithholding"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			unemployment: {
    				description: "",
    				label: "Unemployment"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			}
    		},
    		IrRowSourceType: {
    			manual: {
    				description: "",
    				label: "Manual"
    			},
    			usecase: {
    				description: "",
    				label: "Usecase"
    			},
    			usecaseV02: {
    				description: "",
    				label: "UsecaseV02"
    			}
    		},
    		LegacyTaxcardType: {
    			example: {
    				description: "",
    				label: "Example"
    			},
    			freelancer: {
    				description: "",
    				label: "Freelancer"
    			},
    			noTaxCard: {
    				description: "",
    				label: "NoTaxCard"
    			},
    			noTaxPrepayment: {
    				description: "",
    				label: "NoTaxPrepayment"
    			},
    			others: {
    				description: "",
    				label: "Others"
    			},
    			sideIncome: {
    				description: "",
    				label: "SideIncome"
    			},
    			steps: {
    				description: "",
    				label: "Steps"
    			},
    			taxCardA: {
    				description: "",
    				label: "TaxCardA"
    			},
    			taxCardB: {
    				description: "",
    				label: "TaxCardB"
    			}
    		},
    		MessageFrom: {
    			otherParty: {
    				description: "",
    				label: "OtherParty"
    			},
    			owner: {
    				description: "",
    				label: "Owner"
    			},
    			system: {
    				description: "",
    				label: "System"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthError: {
    			"<any>access_denied": {
    				description: "",
    				label: "Access_denied"
    			},
    			"<any>invalid_request": {
    				description: "",
    				label: "Invalid_request"
    			},
    			"<any>invalid_scope": {
    				description: "",
    				label: "Invalid_scope"
    			},
    			"<any>server_error": {
    				description: "",
    				label: "Server_error"
    			},
    			"<any>temporarily_unavailable": {
    				description: "",
    				label: "Temporarily_unavailable"
    			},
    			"<any>unauthorized_client": {
    				description: "",
    				label: "Unauthorized_client"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			"<any>unsupported_response_type": {
    				description: "",
    				label: "Unsupported_response_type"
    			}
    		},
    		OAuthGrantType: {
    			"<any>authorization_code": {
    				description: "",
    				label: "Authorization_code"
    			},
    			"<any>client_credentials": {
    				description: "",
    				label: "Client_credentials"
    			},
    			"<any>jwtBearer": {
    				description: "",
    				label: "JwtBearer"
    			},
    			"<any>password": {
    				description: "",
    				label: "Password"
    			},
    			"<any>refresh_token": {
    				description: "",
    				label: "Refresh_token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthResponseType: {
    			"<any>code": {
    				description: "",
    				label: "Code"
    			},
    			"<any>token": {
    				description: "",
    				label: "Token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthSalaxyAuthorizeMode: {
    			"<any>sign_in": {
    				description: "",
    				label: "Sign_in"
    			},
    			"<any>sign_up": {
    				description: "",
    				label: "Sign_up"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthTokenType: {
    			"<any>bearer": {
    				description: "",
    				label: "Bearer"
    			},
    			"<any>mac": {
    				description: "",
    				label: "Mac"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentMethod: {
    			sepa: {
    				description: "",
    				label: "Sepa"
    			},
    			siirto: {
    				description: "",
    				label: "Siirto"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PeriodDateKind: {
    			paidAtDate: {
    				description: "",
    				label: "PaidAtDate"
    			},
    			salaryDate: {
    				description: "",
    				label: "SalaryDate"
    			}
    		},
    		Role: {
    			accountant: {
    				description: "",
    				label: "Accountant"
    			},
    			accountantCandidate: {
    				description: "",
    				label: "AccountantCandidate"
    			},
    			pricingPartner: {
    				description: "",
    				label: "PricingPartner"
    			},
    			trustedPartner: {
    				description: "",
    				label: "TrustedPartner"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SalaryType: {
    			compensation: {
    				description: "",
    				label: "Compensation"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			yel: {
    				description: "",
    				label: "Yel"
    			}
    		},
    		SettingsStatus: {
    			checked: {
    				description: "",
    				label: "Ok"
    			},
    			pending: {
    				description: "",
    				label: "Käsittelyssä"
    			},
    			"undefined": {
    				description: "",
    				label: "-"
    			}
    		},
    		SharingUriType: {
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			url: {
    				description: "",
    				label: "Url"
    			}
    		},
    		SystemRole: {
    			anon: {
    				description: "",
    				label: "Anon"
    			},
    			any: {
    				description: "",
    				label: "Any"
    			},
    			auth: {
    				description: "",
    				label: "Auth"
    			},
    			company: {
    				description: "",
    				label: "Company"
    			},
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			fiOy: {
    				description: "",
    				label: "FiOy"
    			},
    			fiRy: {
    				description: "",
    				label: "FiRy"
    			},
    			fiTm: {
    				description: "",
    				label: "FiTm"
    			},
    			hidden: {
    				description: "",
    				label: "Hidden"
    			},
    			household: {
    				description: "",
    				label: "Household"
    			},
    			init: {
    				description: "",
    				label: "Init"
    			},
    			nonVerified: {
    				description: "",
    				label: "NonVerified"
    			},
    			partnerMessaging: {
    				description: "",
    				label: "PartnerMessaging"
    			},
    			partnerService: {
    				description: "",
    				label: "PartnerService"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			prod: {
    				description: "",
    				label: "Prod"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			worker: {
    				description: "",
    				label: "Worker"
    			}
    		},
    		TableFormat: {
    			csv: {
    				description: "",
    				label: "Csv"
    			},
    			txt: {
    				description: "",
    				label: "Txt"
    			},
    			xlsx: {
    				description: "",
    				label: "Xlsx"
    			}
    		},
    		TaxReportHandling: {
    			always: {
    				description: "",
    				label: "Palkkaus.fi lähettää työnantajan erillisilmoituksen siinäkin tapauksessa, jossa työnantaja ei ole maksanut palkkoja viimeisen puolen vuoden aikana"
    			},
    			"default": {
    				description: "",
    				label: "Palkkaus.fi lähettää kaikki ilmoitukset"
    			},
    			noMonthlyReport: {
    				description: "",
    				label: "Palkkaus.fi ei lähetä työnantajan erillisilmoitusta"
    			},
    			noZeroSalaryReport: {
    				description: "",
    				label: "Palkkaus.fi ei lähetä Nollailmoitusta"
    			}
    		},
    		TaxcardApprovalMethod: {
    			approve: {
    				description: "",
    				label: "Approve"
    			},
    			assureWaiting: {
    				description: "",
    				label: "AssureWaiting"
    			},
    			reject: {
    				description: "",
    				label: "Reject"
    			}
    		},
    		TaxcardCalcDiffCheck: {
    			commitDiff: {
    				description: "",
    				label: "CommitDiff"
    			},
    			getDiff: {
    				description: "",
    				label: "GetDiff"
    			},
    			noCheck: {
    				description: "",
    				label: "NoCheck"
    			}
    		},
    		ThreadedMessageType: {
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			email: {
    				description: "",
    				label: "Email"
    			},
    			payroll: {
    				description: "",
    				label: "Payroll"
    			},
    			sms: {
    				description: "",
    				label: "Sms"
    			},
    			text: {
    				description: "",
    				label: "Text"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			}
    		},
    		TransactionCode: {
    			timeRatePay: {
    				description: "Palkka, jota maksetaan työhön käytetyn ajan perusteella. Aikapalkka voidaan määritellä esimerkiksi kalenterikuukaudelle.",
    				label: "Aikapalkka"
    			},
    			initiativeFee: {
    				description: "Kertapalkkio, joka maksetaan aloitteen tekijälle esimerkiksi toiminnan tai tuotteen kehittämiseen liittyvästä aloitteesta.",
    				label: "Aloitepalkkio"
    			},
    			bonusPay: {
    				description: "Kertaluonteinen palkkiopalkka, jonka suuruus määräytyy kokonaan tai osittain sen mukaan, kuinka hyvin tavoitteeksi asetettu työtulos saavutetaan. Työtuloksen mittana voi olla määrä, laatu tai jokin muu tulostekijä.\r\n\r\nBonuspalkka eroaa provisiopalkasta siten, että bonuspalkka on kertaluonteinen ja provisio on jatkuvaluonteinen palkkiopalkka. Kumpikin näistä perustuu henkilön tai ryhmän ennalta asetetun tavoitteen saavuttamiseen. Bonuspalkka eroaa voittopalkkiosta siten, että voittopalkkio maksetaan ilman ennalta sovittua suunnitelmaa, yrityksen voiton perusteella.",
    				label: "Bonuspalkka"
    			},
    			complementaryWagePaidDuringBenefitPeriod: {
    				description: "Etuutta saavalle työntekijälle maksettu palkka, joka kattaa täyden palkan ja maksetun etuuden välisen erotuksen.\r\n\r\nMaksettu etuus voi olla esimerkiksi Kelan maksama sairauspäiväraha. Kun työnantaja maksaa etuusajalta täydennyspalkkaa, Kela maksaa etuuden suoraan työntekijälle, ei työnantajalle.",
    				label: "Etuusajalta maksettu täydennyspalkka"
    			},
    			emergencyWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan hätätyöstä. Hätätyöllä tarkoitetaan hätätilanteessa säännöllisen työajan ylittävänä aikana teetettävää työtä.",
    				label: "Hätätyökorvaus"
    			},
    			eveningWorkCompensation: {
    				description: "Työaikakorvaus, joka maksetaan iltatyöstä. Iltatyökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Iltatyökorvaus"
    			},
    			eveningShiftAllowance: {
    				description: "Vuorotyökorvaus, joka maksetaan iltavuorosta.",
    				label: "Iltavuorolisä"
    			},
    			noticePeriodCompensation: {
    				description: "Korvaus, jonka työnantaja maksaa työntekijälle palvelussuhteen päättyessä ilman irtisanomisaikaa eli tilanteessa, jossa jätetään irtisanomisaika noudattamatta ja työsuhde päättyy heti (TSL 6:4). Tulolajilla ilmoitetaan myös korvaus, jonka työnantaja maksaa, kun työntekijä irtisanoutuu yli 200 päivää kestäneen lomautuksen johdosta sekä, kun työnantaja irtisanoo lomautettuna olevan työntekijän.\r\n\r\nTulolajilla ilmoitetaan vain sellaiset erät, jotka maksetaan työsuhteen päättyessä ilman irtisanomisaikaa. Irtisanomisajalta maksettu palkka ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
    				label: "Irtisanomisajan korvaus"
    			},
    			kilometreAllowanceTaxable: {
    				description: "Kilometrikorvaus, joka ei täytä verovapaudelle asetettuja vaatimuksia.\r\n\r\nVeronalaista kilometrikorvausta voidaan maksaa sekä palvelussuhteessa oleville että muille henkilöille. Kilometrikorvauksen veronalaisuudelle on useita perusteita. Verohallinto on määrittänyt työnantajan palvelussuhteessa maksamille verovapaille kilometrikorvauksille enimmäismäärän, jonka ylittävältä osalta kilometrikorvaus on veronalaista ansiotuloa. Henkilölle toimeksiantosuhteessa maksettava kilometrikorvaus on kokonaan veronalaista ansiotuloa.\r\n\r\n**Tulolajilla ilmoitetaan vain palvelussuhteessa oleville maksetut veronalaiset kilometrikorvaukset.** Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena ja lisätään kustannus työkorvauksen määrää (ks. Työkorvaus-tulolajin selite).",
    				label: "Kilometrikorvaus (veronalainen)"
    			},
    			meetingFee: {
    				description: "Palkkio, joka maksetaan kokoukseen osallistumisesta.",
    				label: "Kokouspalkkio"
    			},
    			saturdayPay: {
    				description: "Työaikakorvaus, joka maksetaan lauantaityöstä. Lauantaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Lauantaityökorvaus"
    			},
    			extraWorkPremium: {
    				description: "Työaikakorvaus, joka maksetaan lisätyöstä.",
    				label: "Lisätyökorvaus"
    			},
    			holidayBonus: {
    				description: "Työsopimuksen tai työ- tai virkaehtosopimuksen perusteella vuosilomapalkan lisäksi maksettava suoritus, jonka suuruus määräytyy alalla sovellettavan työehtosopimuksen tai edellisenä lomanmääräytymisvuonna ansaittujen lomapäivien lukumäärän mukaan.\r\n\r\nVuosilomapalkan lisää koskee erittely- ja ilmoitusvelvollisuus riippumatta siitä, käytetäänkö kyseisestä erästä nimitystä lomaraha. Tällainen erittely tarvitaan esimerkiksi Kelaa ja työttömyyskassaa sekä työtapaturman korvauskäsittelyä varten.",
    				label: "Lomaraha"
    			},
    			lectureFee: {
    				description: "Palkkio, joka maksetaan luennon pitämisestä.",
    				label: "Luentopalkkio"
    			},
    			compensationForTrustPosition: {
    				description: "Palkkio, joka maksetaan muusta luottamustoimen hoitamisesta kuin kokouksiin osallistumisesta.",
    				label: "Luottamustoimipalkkio"
    			},
    			otherCompensation: {
    				description: "Muiden maksettujen työaika- ja poikkeustilakorvausten yhteissumma. Näitä ovat mm. aattotyökorvaukset, arkipyhäkorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella maksettavat korvaukset ja lisät.",
    				label: "Muu maksettu lisä"
    			},
    			waitingTimeCompensation: {
    				description: "Korvaus, johon työntekijällä on oikeus, jos palkanmaksu viivästyy työsuhteen päätyttyä. Ellei toisin sovita, palkka tulee maksaa palkanmaksukauden viimeisenä päivänä. Odotusajan korvausta voidaan maksaa enintään kuudelta päivältä.",
    				label: "Odotusajan korvaus"
    			},
    			workingConditionCompensation: {
    				description: "Palkanlisä, joka maksetaan työn tekemispaikan olosuhteiden perusteella. Olosuhdelisiä ovat esimerkiksi työehtosopimuksen tai työsopimuksen perusteella maksettavat lisä, kuten vaahdotuslisä, kylmänalueenlisä ja saaristolisä.",
    				label: "Olosuhdelisä"
    			},
    			partialPayDuringSickLeave: {
    				description: "Lain tai työ- tai virkaehtosopimukseen mukaisesti alennettu sairausajan palkka. Sairausajan osapalkkana ilmoitetaan se osa tuloista, jonka henkilö on saanut sairaana ollessaan alennettuna.",
    				label: "Sairausajan osapalkka"
    			},
    			commission: {
    				description: "Pysyvän palkan osan muodostava jatkuvaluonteinen palkkiopalkka, jonka määrästä on sovittu työsopimuksessa.",
    				label: "Provisiopalkka"
    			},
    			sundayWorkCompensation: {
    				description: "Sunnuntaityöstä maksettava työaikakorvaus. Sunnuntaityökorvausta voidaan maksaa säännölliseltä työajalta tai säännöllisen työajan ylittävältä ajalta.",
    				label: "Sunnuntaityökorvaus"
    			},
    			benefitArisingFromSyntheticOption: {
    				description: "Optio-oikeus, jonka perusteella tulonsaaja ei ole oikeutettu merkitsemään option kohde-etuutena olevia osakkeita option toteutushintaan, vaan hän saa toteutushinnan ja toteuttamishetken mukaisen osakkeen arvon erotuksen rahana (nettoarvon tilitys). Synteettinen optio ei oikeuta osakemerkintään.\r\n\r\nKäytännössä synteettisessä optiossa on kyse rahasuorituksesta, jonka suuruus määräytyy työnantajayhtiön osakekurssin kehityksen perusteella. Synteettinen optio -nimityksen sijaan järjestelyä saatetaan kutsua osakeperusteiseksi bonukseksi tai palkitsemisjärjestelmäksi.",
    				label: "Synteettinen optio"
    			},
    			performanceBonus: {
    				description: "Palkka, joka maksetaan organisaation ennalta sovitun tulostavoitteen täyttymisen tai ylittymisen perusteella.",
    				label: "Tulospalkkio"
    			},
    			monetaryCompensationFromWorkingTimeBank: {
    				description: "Palkka, joka maksetaan työaikapankkiin säästetyn ajan perusteella.\r\n\r\nTyöaikapankista voi yleensä nostaa palkkaa vain poikkeustapauksissa kuten palvelussuhteen päättyessä tai työaikapankkia purettaessa. Joissakin työaikapankeissa on mahdollista nostaa palkkaa tai\r\nvapaa-aikaa myös velaksi.",
    				label: "Työaikapankin rahakorvaus"
    			},
    			compensationForAccruedTimeOff: {
    				description: "Työaikakorvaus, joka maksetaan ansionvähentymisestä, joka aiheutuu työajan tasaamisesta keskimääräiseen viikkotyöaikaan. \r\n\r\nTyöajantasauskorvausta maksetaan säännölliseltä työajalta. Esimerkiksi niin sanottu pekkaskorvaus on työajantasauskorvaus.",
    				label: "Työajantasauskorvaus"
    			},
    			shareIssueForEmployees: {
    				description: "Yhtiön omalle henkilökunnalle suunnattu osakeanti. Etu työsuhteeseen perustuvasta oikeudesta merkitä työnantajayrityksen osakkeita on veronalaista ansiotuloa.\r\n\r\nJos etu on henkilökunnan enemmistön käytettävissä, tulolajilla ilmoitetaan ainoastaan 10 %:n alennuksen ylittävä osuus. Jos etu ei ole henkilökunnan enemmistön käytettävissä, saatava alennus on palkkaa koko määrältään. Tällöin suorituksen ilmoittamisessa käytetään palkan muita tulolajeja.\r\n\r\nMerkintäetu koskee myös yhteisön osuuksia.",
    				label: "Työsuhteeseen perustuva osakeanti"
    			},
    			contractPay: {
    				description: "Suorituspalkka, joka maksetaan ennalta sovitun työkokonaisuuden suorittamisen perusteella. Urakkapalkkaan sisältyy sekä kiinteä että muuttuva osa.",
    				label: "Urakkapalkka "
    			},
    			damagesInConjunctionWithTermination: {
    				description: "Korvaus, jota työnantaja on velvollinen maksamaan, kun työnantaja on päättänyt palvelussuhteen henkilön työsopimuslain, kunnallisesta viranhaltijasta annetun lain tai valtion virkamieslain vastaisesti. Kohdassa ilmoitetaan myös korvaus työsuhteen purkamisesta (TSL 8:1, 55/2001) sekä korvaus perusteettomasta lomauttamisesta.\r\n\r\nPalvelussuhteen perusteettoman päättämisen johdosta maksettava korvaus voi perustua tuomioistuimen tuomioon, tuomioistuimen vahvistamaan sovintoon, palvelussuhteen osapuolia edustavien järjestöjen väliseen neuvottelutulokseen tai palvelussuhteen osapuolten väliseen sopimukseen.\r\n\r\nVahingonkorvaus päättämis- tai lomautustilanteessa tai muu siihen verrattava korvaus ei ole veronalaista tuloa, ellei sitä ole saatu veronalaisen tulon sijaan tai korvaukseksi elatuksen vähentymisestä. Veronalaista tuloa eivät myöskään ole laissa tarkemmin määritellyt hyvitykset yhteistoimintavelvoitteiden laiminlyönneistä.\r\n\r\nTulolajilla ilmoitetaan vain veronalaiset määrät eli esimerkiksi ei yhteistoimintalain tai tasa-arvolain nojalla maksettuja verovapaita hyvityksi. Verovapaita vahingonkorvauksia ei ilmoiteta tulorekisteriin.\r\n\r\nTulolajilla ei ilmoiteta irtisanomisen yhteydessä maksettavia työsuoritukseen perustuvia korvauksia vaan ne ilmoitetaan muilla palkan suorituslajeilla, esim. Palkka yhteissumma- tai Aikapalkka-tulolajeilla.",
    				label: "Vahingonkorvaus päättämis- ja lomautustilanteissa"
    			},
    			standByCompensation: {
    				description: "Palkka, joka maksetaan siitä, että työntekijä on tavoitettavissa ja työnantaja voi tarvittaessa kutsua hänet hoitamaan palvelussuhteeseen kuuluvia tehtäviä.",
    				label: "Varallaolokorvaus"
    			},
    			voluntaryCompensationInConjunctionWithTermination: {
    				description: "Korvaus, jonka työnantaja maksaa vapaaehtoisesti irtisanomis- tai purkamistilanteessa tai silloin, kun palvelussuhde päätetään sopimalla.\r\n\r\nPäättämistilanteessa maksettavia vapaaehtoisia korvauksia ovat esimerkiksi toimitusjohtajan erokorvaus (niin sanottu kultainen kädenpuristus), erokorvaus, eroraha ja tukipaketti. Korvaus voidaan\r\nmyöntää myös muuna kuin rahana.",
    				label: "Vapaaehtoinen korvaus päättämistilanteessa"
    			},
    			weeklyRestCompensation: {
    				description: "Työaikakorvaus, jota maksetaan viikkolevon aikana tehdystä työstä.",
    				label: "Viikkolepokorvaus"
    			},
    			profitSharingBonus: {
    				description: "Työntekijöille ilman ennalta sovittua suunnitelmaa yhtiökokouksen päätöksellä jaettava palkkio, joka muodostuu yrityksen voiton perusteella.",
    				label: "Voittopalkkio"
    			},
    			annualHolidayCompensation: {
    				description: "Vuosiloman antamisen sijaan suoritettava rahallinen korvaus. Vuosilomakorvausta voidaan maksaa työsuhteen aikana tai työsuhteen päättyessä.\r\n\r\nTulolajilla ilmoitetaan myös **rakennusalan lomaraha**. Työehtosopimuksessa lomakorvaus, lomapalkka ja lomaraha on yhdistetty lomaraha-nimiseksi korvaukseksi, joka on 18,5 % lomanmääräytymisvuoden ansioista. Pääsääntöisesti rakennusalan lomakorvauksessa on 1/3 lomarahaa ja 2/3 lomapalkkaa tai lomakorvausta riippuen siitä, jatkuuko työsuhde vai päättyikö se.",
    				label: "Vuosilomakorvaus"
    			},
    			overtimeCompensation: {
    				description: "Ylityöstä maksettava työaikakorvaus. Ylityönä pidetään työtä, joka tehdään työnantajan aloitteesta sovitun ylityöaikarajan ylittävänä aikana.",
    				label: "Ylityökorvaus"
    			},
    			nightWorkAllowance: {
    				description: "Työaikakorvaus, jota maksetaan yötyöstä.",
    				label: "Yötyökorvaus"
    			},
    			nightShiftCompensation: {
    				description: "Vuorotyökorvaus, joka maksetaan yövuorosta.",
    				label: "Yövuorolisä"
    			},
    			otherRegularCompensation: {
    				description: "Sellaisten työaika- ja poikkeustilakorvausten yhteissumma, joita maksetaan säännöllisesti ja jotka lasketaan säännöllisen työajan ansioon. Muita säännöllisiä lisiä ovat mm. luontoisedusta maksetut rahakorvaukset, aattotyökorvaukset ja muut mahdolliset työehtosopimusten tai työsopimusten perusteella säännöllisesti maksettavat korvaukset ja lisät, kuten kielilisä. Myös harvoin maksettava lisä katsotaan säännölliseksi, jos se kuuluu säännöllisen työajan ansioon.",
    				label: "Muu säännöllinen lisä"
    			},
    			compensationForUnusedCompensatoryLeave: {
    				description: "Tietyillä aloilla käytössä oleva korvaus, jota maksetaan työsuhteen päättyessä pitämättömistä vapaapäivistä. Tietyillä aloilla tehty työ kerryttää tekijälleen vastikevapaata, jota työsuhteen kuluessa pidetään vapaana. Jos työntekijä ei ole pitänyt vapaapäiviä, kun työsuhde päättyy, niistä maksetaan korvausta.\r\n\r\nVuosiloman antamisen sijaan suoritettava rahallinen korvaus ilmoitetaan tulolajilla Vuosilomakorvaus (234).",
    				label: "Vastikevapaan korvaus"
    			},
    			accommodationBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän käyttöön omistamansa tai hallitsemansa asunnon palvelussuhteeseen liittyvän vuokrasuhteen perusteella tai palkkaan sisältyvänä etuna.\r\n\r\nPalkkaus.fi-palvelussa asuntoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luontoisetu'-tulolajilla.",
    				label: "Asuntoetu"
    			},
    			interestBenefitForHousingLoan: {
    				description: "Veronalainen henkilökuntaetu, joka muodostuu silloin, kun työnantaja perii työntekijälle myöntämästään asuntolainasta vuotuista korkoa, joka on alempi kuin yleisesti markkinoilla käytössä oleva viitekorko.",
    				label: "Asuntolainan korkoetu"
    			},
    			mealAllowance: {
    				description: "Verovapaa kustannusten korvaus ateriointikuluista, jotka johtuvat työmatkasta.\r\n\r\nAteriakorvaus voidaan maksaa verovapaasti, jos työntekijällä ei ole mahdollisuutta aterioida ruokailutauon aikana tavanomaisella ruokailupaikallaan, eikä ruokailua ole järjestetty esimerkiksi kurssipakettiin kuuluvana työnantajan kustantamana ruokailuna. **Verovapaata ateriakorvausta ei voi maksaa samasta työmatkasta yhtä aikaa verovapaan päivärahan kanssa.**\r\n\r\nAteriakorvauksen perusteista ja verovapaista enimmäismääristä säädetään vuosittain annettavissa Verohallinnon päätöksissä verovapaista matkakustannusten korvauksista.\r\n\r\nKohdassa ilmoitetaan vain verovapaiden ateriakorvausten määrät. Jos maksettu korvaus ylittää Verohallinnon päätöksen enimmäismäärät tai maksetaan vastoin päätöksessä säädettyjä perusteita, suoritus ilmoitetaan sosiaalivakuutusmaksujen alaisena palkkana.\r\n\r\nAteriakorvaus voidaan ilmoittaa erikseen, mutta saatu korvaus on ilmoitettava vähintään yhteissummana Päiväraha -tulolajilla. Tällöin on lisäksi ilmoitettava tulolajiin liittyvän Korvauksen tyyppi-lisätiedon avulla, että Päiväraha-tulolajilla ilmoitettu korvaus sisältää ateriakorvauksen.\r\n\r\nJos ateriakorvaus ilmoitetaan erikseen, sitä ei lisätä enää Päiväraha -tulolajilla ilmoitettaviin määriin.",
    				label: "Ateriakorvaus"
    			},
    			carBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja antaa työntekijän tai tämän perheen käyttöön omistamansa tai hallitsemansa auton, jonka yksityiskäyttö on sallittu.\r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että työntekijä on maksanut luontoisedusta työnantajalle luontoisedun raha-arvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan. Peritty summa ilmoitetaan erikseen Autoedusta peritty korvaus- erittelylajilla.",
    				label: "Autoetu"
    			},
    			compensationForGoverningBodyMembership: {
    				description: "Luottamustoimipalkkio, joka maksetaan yhteisön hallituksen tai vastaavan hallintoelimen jäsenyyden perusteella.\r\n\r\nJos henkilölle maksetaan palkkiota hallintoelimen jäsenyydestä, hänelle voidaan maksaa myös verovapaita matkakustannusten korvauksia työmatkasta aiheutuneiden kulujen perusteella, jos verovapaiden korvausten muut edellytykset täyttyvät.",
    				label: "Hallintoelimen jäsenyydestä maksettu palkkio"
    			},
    			shareOfReserveDrawnFromPersonnelFund: {
    				description: "Rahasto-osuus, joka maksetaan työntekijälle henkilöstörahastosta. Henkilörahastoon osoitetut varat jaetaan rahasto-osuutena nostettaviin eriin ja käteisenä nostettaviin eriin.\r\n\r\nHenkilöstörahastosta nostetusta rahasto-osuudesta 20 % on verovapaata ja 80 %:n veronalaisesta ansiotulo-osuudesta toimitetaan ennakonpidätys. Henkilöstörahastosta nostettua ylijäämää kohdellaan verotuksessa samalla tavalla kuin henkilöstörahastosta nostettua rahasto-osuutta. Ks. Verohallinnon ohje Henkilöstörahastosta saatavien tulojen verotus (A69/200/2014).\r\n\r\nTulolajilla ilmoitetaan vain veronalainen osuus.",
    				label: "Henkilöstörahastosta nostettu rahasto-osuus ja ylijäämä (veronalainen 80%)"
    			},
    			monetaryGiftForEmployees: {
    				description: "Työnantajalta rahana tai siihen verrattavana suorituksena saatu lahja.",
    				label: "Henkilöstölle annettu rahalahja"
    			},
    			kilometreAllowanceTaxExempt: {
    				description: "Henkilön oman tai hänen hallinnassaan olevan kulkuneuvon käytöstä suoritettava kilometrikorvaus, joka täyttää verovapaudelle asetetut vaatimukset.\r\n\r\nVerovapaata kilometrikorvausta voidaan maksaa palvelussuhteessa oleville ja henkilöille, jotka eivät ole palvelussuhteessa, mutta joille maksetaan verolainsäädännön mukaista palkkaa, esimerkiksi kokouspalkkion yhteydessä.\r\n\r\nJos kilometrikorvaus on maksettu vastoin Verohallinnon päätöksen mukaisia aika- ja kilometrirajoja, ne eivät ole miltään osin verovapaita.\r\n\r\nJos kilokorvaukset on maksettu Verohallinnon päätöksen mukaan, mutta suurempina kuin päätöksessä on määrätty verovapaaksi, osa korvauksista on veronalaista ja osa verovapaata. Veronalaiset kilometrikorvaukset ilmoitetaan erikseen omalla tulolajillaan tai palkan tulolajeilla.\r\n\r\nLiikenteenharjoittajan antaman tositteen (esimerkiksi junalipun) perusteella tulonsaajalle maksetut matkustamiskustannusten korvaukset ovat verovapaita, eikä niitä ilmoiteta.\r\n\r\n**Rakennusalalla maksettavat asunnon ja työpaikan väliset matkakustannusten korvaukset**\r\n\r\nRakennusalan työehtosopimuksen mukaan päivittäisistä asunnon ja erityisen työntekemispaikan välisistä matkoista maksetaan matkakustannusten korvausta (”tossurahaa”) erityisen taulukon mukaan eikä Verohallinnon päätöksen mukaisina kilometrikorvauksina. Työehtosopimuksen mukainen korvaus on kuitenkin pienempi kuin Verohallinnon päätöksen mukaiset kilometrikorvaukset, joten korvaus on verovapaata. \r\n\r\nRakennusalan tossurahan yhteydessä ei tarvitse ilmoittaa kilometrimäärää.",
    				label: "Kilometrikorvaus (verovapaa)"
    			},
    			treatmentFeeForMunicipalVeterinarian: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Kunnaneläinlääkärin toimenpidepalkkio"
    			},
    			compensationForUseEarnedIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.",
    				label: "Käyttökorvaus ansiotuloa"
    			},
    			compensationForUseCapitalIncome: {
    				description: "Korvaus, joka suoritetaan tekijänoikeuden, teollisoikeuden tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevien tietojen luovuttamisesta tai käyttöoikeudesta.\r\n\r\nJos tekijänoikeus on siirtynyt perintönä tai testamentilla tai se on hankittu vastiketta vastaan, sen käyttämisestä saatu korvaus on henkilölle Suomesta saatuna tulona ennakonpidätyksenalaista pääomatuloa ennakkoperintärekisterimerkinnästä huolimatta.",
    				label: "Käyttökorvaus pääomatuloa"
    			},
    			otherTaxableBenefitForEmployees: {
    				description: "Työnantajan henkilökunnalle kollektiivisesti myöntämä muu kuin rahana annettava veronalainen etu.\r\n\r\nVeronalaisena etuna pidetään esimerkiksi vuosittain laissa säädetyn enimmäismäärän 400 euroa ylittävää omaehtoiseen liikuntaan ja kulttuuriin annettua etua.",
    				label: "Muu henkilöstölle suunnattu veronalainen etu"
    			},
    			otherTaxableIncomeDeemedEarnedIncome: {
    				description: "Tulolajilla ilmoitetaan muiden veronalaisten ansiotuloa olevien suoritusten yhteissumma.\r\n\r\nTulolajilla ilmoitetaan veronalaiset satunnaiset palkkiot ja palkinnot silloin, kun saaja ei ole työsuhteessa antajaan eikä suoritus ole työkorvausta. Tällaisia ovat esimerkiksi työsuhteeseen perustumaton kilpailupalkinto, asiakasyrityksen henkilökuntaan kuuluvalle tai muutoin asiakassuhteen perusteella annettu etu (esimerkiksi matka, lahjakortti tai esine), löytöpalkkio, pankin maksama niin sanottu tarkkaavaisuuspalkkio, taidekilpailun palkinto sekä kunnan asukkailleen maksama matkakorvaus.",
    				label: "Muu veronalainen ansiotulo"
    			},
    			otherFringeBenefit: {
    				description: "Tulolajilla ilmoitetaan työnantajan muuna kuin rahana antamat verotettavat edut, esimeriksi autotalli-, moottoripyörä- ja vene-etu, palkaksi katsotut henki- ja eläkevakuutusmaksut, yksilöimättömästä lahjakortista syntyvä etu ja merimiespalveluksessa olevan henkilön merimiehenä saama luontoisetu.\r\n\r\nPalkkaus.fi-palvelussa asunto-, auto-, puhelin- ja ravintoetu sekä työsuhdematkalipun erät tulee aina ilmoittaa omalla tulolajillaan. Niitä ei lisätä 'Muu luontoisetu' kohtaan.\r\n\r\nJos tulonsaajalta peritään korvaus edusta, luontoisedun arvo ilmoitetaan täysimääräisenä tulolajilla ennen perityn korvauksen vähentämistä.\r\n\r\nJos verotusarvoa ei ole erikseen vahvistettu Verohallinnon antamassa päätöksessä, luontoisedun arvo on sen käypä arvo. Etu on ilmoitettava, vaikka työntekijälle ei maksettaisi lainkaan rahapalkkaa tai rahapalkka ei riittäisi ennakonpidätyksen täysimääräiseen toimittamiseen.",
    				label: "Muu  luontoisetu"
    			},
    			kinshipCarersFee: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Omaishoitajan palkkio"
    			},
    			stockOptions: {
    				description: "Palkkio, joka annetaan työnantajayrityksen osakkeina tai sovitun palkkion sijaan rahasummana, joka vastaa osakkeiden arvoa.",
    				label: "Osakepalkkio"
    			},
    			wagesPaidBySubstitutePayerIncludingSocialInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset sosiaalivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen.\r\n\r\nHUOM: Kotitalous ilmoittaa Yksityisen hoidon tuen tällä tulolajilla!\r\n\r\nVarsinaisen työnantajan antaman tiedon perusteella tuloa ei lueta tulonsaajalle verotuksessa tuloksi, mutta tieto tarvitaan tulonsaajakohtaisesti, koska henkilölle karttuu eläkettä maksetun tulon perusteella.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sosiaalivakuutusmaksut.\r\n\r\nVaikka sijaismaksaja on maksanut palkan tulonsaajille, varsinainen työnantaja on vastuussa työnantajan sosiaalivakuutusmaksujen maksamisesta. Tämän vuoksi varsinaisen työnantajan on ilmoitettava sijaismaksajan maksamat palkat tulonsaajakohtaisesti käyttäen erityisesti sijaismaksajatilanteita varten tarkoitettuja niin sanottuja infotulolajeja.\r\n\r\nSijaismaksaja ilmoittaa tiedot omalta osaltaan tulorekisteriin käyttäen muita tulolajeja sekä ohjaustietoa ”Sijaismaksaja – Kyllä”. Sijaismaksajan tiedon perusteella tulo verotetaan tulonsaajalla. Työntekijältä perittävät maksut peritään ja maksetaan sijaismaksajan maksaman palkan perusteella. Sijaismaksaja ei pääsääntöisesti kuitenkaan maksa työnantajan sosiaalivakuutusmaksuja, vaan niiden maksamisesta on vastuussa varsinainen työnantaja.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat. Ilmoittamista on kuvattu tarkemmin asiakasohjeessa\r\nTietojen ilmoittaminen tulorekisteriin: sijaismaksajan maksamat suoritukset.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sosiaalivakuutusmaksut (työeläke-, sairaus-, työttömyys- sekä työtapaturma- ja ammattitautivakuutus)"
    			},
    			wagesPaidBySubstitutePayerIncludingEarningsRelatedPensionInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset eläkevakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa eläkevakuutusmaksusta. \r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan eläkevakuutusmaksun. Sijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työeläkevakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingUnemploymentInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset työttömyysvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työttömyysvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan työttömyysvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan työttömyysvakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingAccidentAndOccupationalDiseaseInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset työtapaturma- ja ammattitautivakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työtapaturma- ja ammattitautivakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työtapaturma- ja ammattitautivakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työtapaturma- ja ammattitautivakuutusmaksun"
    			},
    			wagesPaidBySubstitutePayerIncludingHealthInsurance: {
    				description: "Varsinainen työnantaja ilmoittaa sellaiset sairausvakuutusmaksujen perusteena olevat palkat yhteensä, jotka sijaismaksaja on maksanut ja joista sijaismaksaja on toimittanut ennakonpidätyksen, mutta joista työnantaja vastaa työnantajan sairausvakuutusmaksusta.\r\n\r\nTulolajia käytetään myös silloin, kun varsinainen työnantaja maksaa palkkaturvana maksettujen palkkojen perusteella työnantajan sairausvakuutusmaksun.\r\n\r\nSijaismaksajia voivat olla esimerkiksi ulosottovirastot, Kela, kunnat, Elinkeino-. liikenne- ja ympäristökeskukset, konkurssipesät ja takaajat.",
    				label: "Sijaismaksajan maksama palkka, josta työnantaja maksaa työnantajan sairausvakuutusmaksun"
    			},
    			compensationForEmployeeInvention: {
    				description: "Palkkio, joka maksetaan työsuhdekeksinnöstä.",
    				label: "Palkkio työsuhdekeksinnöstä"
    			},
    			reimbursementOfPrivateCaretakersExpenses: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhehoitajan kustannusten korvaus"
    			},
    			privateCaretakersFee: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhehoitajan palkkio"
    			},
    			reimbursementOfFamilyDayCareProvidersExpenses: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Perhepäivähoitajan kustannusten korvaus"
    			},
    			telephoneBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja maksaa työntekijän puhelinliittymän kulut vapaa-ajalla tapahtuvan yksityiskäytön osalta.\r\n\r\nPalkkaus.fi-palvelussa puhelinetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.",
    				label: "Puhelinetu"
    			},
    			dailyAllowance: {
    				description: "Päiväraha on verovapaa kustannusten korvaus kohtuullisesta ruokailu- ja muiden elantokustannusten lisäyksestä, joka korvauksen saajalle aiheutuu työmatkasta.\r\n\r\nKohtaan merkitään verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Päiväraha-tulolajilla ilmoitettu tulo voi olla:\r\n\r\n- kotimaan kokopäiväraha,\r\n- kotimaan osapäiväraha,\r\n- ulkomaan päiväraha tai \r\n- ulkomaan työskentelyyn liittyvät verovapaat kustannusten korvaukset.\r\n\r\nPalkkaus.fi-palvelussa Ateriakorvaukset ilmoitetaan aina erillisellä Ateriakorvaus-tulolajia.\r\n\r\nJos päivärahan tyypiksi on merkitty ulkomaan työskentelyyn liittyvät kustannusten korvaukset, kohtaan merkitään seuraavat tulonsaajan saamat verovapaat edut, jotka liittyvät ulkomaan työskentelyyn:\r\n\r\n- verovelvollisen ja hänen perheenjäsenensä muutto- ja matkakustannukset työntekovaltioon ja takaisin\r\n- työnantajan ulkomailla kustantama tavanomainen yksityinen palveluhenkilöstö\r\n- työnantajan kustantama lasten perus- ja lukiotason koulutus.\r\n\r\nTulolajilla ei ilmoiteta yömatkarahaa eikä rajanylitysrahaa. Näitä eriä ei ilmoiteta tulorekisteriin.\r\n\r\nKohtaan merkitään vain verovapaiden päivärahojen ja ateriakorvausten yhteismäärä. Verohallinto vahvistaa vuosittain verovapaat määrät antamallaan päätöksellä verovapaista matkakustannusten korvauksista.\r\n\r\nJos päivärahat tai muut kohdassa mainitut korvaukset on maksettu Verohallinnon päätöstä lievemmin perustein, korvaukset ilmoitetaan tulolajilla Palkka yhteissumma tai Muu maksettu lisä, jolloin ne ovat myös sosiaalivakuutusmaksujen alaisia.\r\n\r\nJos lievemmin perustein maksetuista korvauksista on sovittu työehtosopimuksella, korvaus ei ole eläke-, työttömyys- eikä työtapaturma- ja ammattitautivakuutusmaksun alainen. Näissä tilanteissa suorituksen\r\nmaksaja ilmoittaa tulon Palkka yhteissumma-tulolajilla tai Muu henkilöstölle suunnattu veronalainen etu, ja samalla ilmoittaa, mikä osuus suorituksesta on eläkevakuutusmaksun, työttömyysvakuutusmaksun sekä työtapaturma- ja ammattitautivakuutusmaksun alainen.\r\n\r\nTulolajin yhteydessä on annettava lisätiedot, mitä eri päivärahoja yhteissumma sisältää.",
    				label: "Päiväraha"
    			},
    			capitalIncomePayment: {
    				description: "Suoritus, joka on saajalleen pääomatuloa. Tällainen on esimerkiksi työntekijälle tai osakkaalle maksettu korko, kuten palkkasaatavan korko tai pääomatuloa oleva takausprovisio.",
    				label: "Pääomatuloa oleva suoritus"
    			},
    			mealBenefit: {
    				description: "Luontoisetu, joka syntyy, kun työnantaja tarjoaa työntekijälleen aterian. Työnantaja voi tukea ruokailukustannuksia ruokailulipukkeella tai muulla kohdennetulla maksuvälineellä taikka järjestämällä työpaikkaruokailun.\r\n\r\nRavintoedun verotusarvo määräytyy Verohallinnon päätöksen mukaisena vuosittain vahvistettavana arvona.\r\n\r\nPalkkaus.fi-palvelussa ravintoetu ilmoitetaan aina tällä tulolajilla, ei koskaan 'Muu luotoisetu'-tulolajilla.\r\n\r\nJos työnantaja perii työntekijältä ravintoedusta sen verotusarvoa vastaavan korvauksen, tulorekisteriin ilmoitetaan tieto Ravintoedusta peritty korvaus vastaa verotusarvoa: Kyllä.",
    				label: "Ravintoetu"
    			},
    			reimbursementOfCostsPaidToConciliator: {
    				description: "Veronalainen kustannusten korvaus, joka maksetaan vapaaehtoiselle sovittelijalle sovittelusta aiheutuneista kustannuksista.\r\n\r\nSovittelijan kulukorvaus on veronalaista ansiotuloa, joka otetaan huomioon suorituksen saajan verotuksessa saman suuruisena vähennyksenä ilman suorituksen saajan esittämää eri selvitystä.",
    				label: "Sovittelijan kulukorvaus"
    			},
    			nonWageCompensationForWork: {
    				description: "Muussa kuin palvelussuhteessa tehdystä työstä maksettava tai maksettavaksi sovittu vastike, josta tulee toimittaa ennakonpidätys, jos saaja ei ole ennakkoperintärekisterissä, mutta josta ei pääsääntöisesti makseta sosiaalivakuutusmaksuja.\r\n\r\nTyökorvausta voidaan maksaa ennakkoperintärekisteriin kuuluvalle tai kuulumattomalle työn suorittajalle. Työkorvaus on veronalaista ansiotuloa ja siitä toimitetaan ennakonpidätys, jos saaja ei kuulu ennakkoperintärekisteriin. Työkorvausta on esimerkiksi tukiperheelle maksettu korvaus.\r\n\r\nTyökorvauksesta on annettava ilmoitus, jos tulonsaaja ei kuulu ennakkoperintärekisteriin tai, jos maksajana on julkisten alojen eläkelakien piiriin (JuEL) kuuluva suorituksen maksaja ja tulonsaaja on luonnollinen henkilö, joka kuuluu ennakkoperintärekisteriin, mutta hänellä ei ole yrittäjän eläkelakien mukaista vakuutusta. Koska ilmoitus tehdään aina, kun tulonsaaja ei kuulu ennakkoperintärekisteriin, saajana voi olla myös muu kuin luonnollinen henkilö, kuten yhdistys, osakeyhtiö, avoin yhtiö tai kommandiittiyhtiö. Näitä maksuja ei kuitenkaan voi tehdä Palkkaus.fi-palvelun kautta.\r\n\r\nRajoitetusti verovelvolliselle yritykselle maksetut työkorvaukset on ilmoitettava, jos maksaja on perinyt yritykseltä lähdeveron. Rajoitetusti verovelvolliselle luonnolliselle henkilölle maksettu työkorvaus on ilmoitettava aina riippumatta siitä, onko lähdeveroa peritty, esimerkiksi rajoitetusti verovelvollisen urheilijan ja taiteilijan saama palkkio on ilmoitettava.\r\n\r\n**Työkorvauksiin liittyvät kustannusten korvaukset**\r\n\r\nJos palkkio tai muu suorituksen määrä on maksettu työkorvauksena, saajalla ei voi olla verovapaisiin kuluihin merkittäviä kustannusten korvauksia.\r\n\r\nTyökorvauksiin liittyvät kustannusten korvaukset lisätään tulon määrään yhtenä summana muun palkkion kanssa. Työkorvauksen yhteydessä maksetut kustannusten korvaukset eivät ole verovapaita. Jos veronalainen kilometrikorvaus maksetaan toimeksiantosuhteessa olevalle henkilölle tai yhteisölle, korvaus ilmoitetaan työkorvauksena, eikä tulolajia kilometrikorvaus (veronalainen) saa käyttää.\r\n\r\nTyökorvauksen saaja voi vaatia matkakustannuksia vähennettäväksi esitäytetyllä veroilmoituksellaan. Elinkeinotoiminnan harjoittaja vähentää kuitenkin matkakustannukset kirjanpidossaan.\r\n\r\nTyökorvauksesta ei yleensä voi ennen ennakonpidätyksen toimittamista vähentää työstä aiheutuneita kustannusten korvauksia. Poikkeuksena ovat vain luonnollisille henkilöille työmatkasta maksetut kustannusten korvaukset, joista ei tarvitse toimittaa ennakonpidätystä, jos niiden perusteet ja määrät ovat sen mukaisia kuin Verohallinnon vuosittain antamassa päätöksessä verovapaista matkakustannusten korvauksista on määrätty. Nämäkään korvaukset eivät ole verovapaita, joten ne ilmoitetaan työkorvauksen yhteissummassa.",
    				label: "Työkorvaus"
    			},
    			supplementaryDailyAllowancePaidByEmployerSpecificHealthInsuranceFund: {
    				description: "Vain sairauskassat käyttävät tätä tulolajia",
    				label: "Työnantajakohtaisen sairaskassan maksama täydennyspäiväraha"
    			},
    			pensionPaidByEmployer: {
    				description: "Eläke, jonka työnantaja maksaa entiselle työntekijälleen tai tämän edunsaajalle aiemman palvelussuhteen perusteella.",
    				label: "Työnantajan maksama eläke"
    			},
    			dividendsBasedOnWages: {
    				description: "Osakkaalle palkkana maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (palkka)"
    			},
    			dividendsBasedOnNonWage: {
    				description: "Osakkaalle työkorvauksena maksettava osinko, jonka jakoperusteena on osingonsaajan tai tämän intressipiiriin kuuluvan henkilön työpanos.\r\n\r\nIlmoitus on annettava työkorvauksena verotettavasta työpanokseen perustuvasta osingosta tai ylijäämästä siinäkin tapauksessa, että saaja on merkitty ennakkoperintärekisteriin.",
    				label: "Työpanokseen perustuva osinko tai ylijäämä (työkorvaus)"
    			},
    			employerSubsidisedCommuterTicketTaxExempt: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun verovapaa osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua tai vähennettävää määrää ei ole.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 300 euron työsuhdematkalipun. Luontoisetua ei synny.\r\n\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 300 euroa\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.",
    				label: "Työsuhdematkalipun verovapaa osuus"
    			},
    			employerSubsidisedCommuterTicketTaxable: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle kustantaman julkisen liikenteen henkilökohtainen joukkoliikennelipun palkaksi katsottava osuus.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen\r\nhenkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nTyösuhdematkalipun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.\r\n\r\nPalkaksi katsottavana osuutena ilmoitetaan se osuus edun arvosta, josta työnantaja on jo vähentänyt mahdollisesti tulonsaajalta perimänsä korvauksen.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 800 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Koska työsuhdematkalippu on veronalaista siltä osin kuin edun arvon ylittää 300 euroa ja alittaa 750 euroa, veronalaista luontoisetua syntyy 450 euroa. Työntekijä saa verovapaata etua 350 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan palkaksi katsottu määrä 450 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 350 euroa\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 5 000 euron arvoisen työsuhdematkalipun. Työnantaja ei peri työntekijältä korvausta lipun hinnasta. Veronalaista luontoisetua ja palkkaa on 2 050 euroa. Työntekijä saa verovapaata etua 2 950 euroa. \r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus-tulolajilla ilmoitetaan luontoisedun määrä 2 050 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan verovapaan edun määrä 2 950 euroa.\r\n\r\nTyösuhdematkalipun palkaksi katsottua osuus ilmoitetaan ainoastaan omalla tulolajillaan, eikä sitä lisätä Muu luontoisetu-tulolajiin etuihin.",
    				label: "Työsuhdematkalipun palkaksi katsottu osuus"
    			},
    			employeeStockOption: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa.",
    				label: "Työsuhdeoptio"
    			},
    			wagesTransferredToAthletesSpecialFund: {
    				description: "Urheilutulon siirtäminen urheilijarahastoon tulojen jaksottamista varten. Urheilijalla on oikeus siirtää osa urheilutulosta urheilijarahastoon. Rahastoinnista sovitaan urheilijan, työnantajan ja mahdollisesti sponsorin kesken.\r\n\r\nUrheilutulo voi muodostua urheilijan palkasta tai palkkiosta. Tulolajilla ilmoitetaan vain rahastoon siirretty palkka. Siirto urheilurahastoon on tarkoitettu urheiluajan jälkeisen toimeentulon turvaamiseen.",
    				label: "Urheilijarahastoon siirretty palkka"
    			},
    			wagesPaidFromAthletesSpecialFund: {
    				description: "Vain urheilijarahaston käyttöön: Tulolajilla ilmoitetaan vain rahastosta maksettu palkka.",
    				label: "Urheilijarahastosta maksettu palkka"
    			},
    			wagesForInsurancePurposes: {
    				description: "HUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.\r\n\r\nLaskennallinen palkka, joka olisi maksettava, jos ulkomailla tehtyä työtä vastaava työ tehtäisiin Suomessa. Vakuutuspalkkaa ulkomaantyössä käytetään sosiaalivakuuttamisessa maksujen ja etuuksien määräytymisen perusteena todellisen palkan sijaan.\r\n\r\nEri toimijoiden kesken on omat sääntönsä siitä, milloin määräytymisen perusteena käytetään vakuutuspalkkaa ja milloin todellista maksettua palkkaa. Aiheeseen liittyvä tarkempi ohje löytyy Eläketurvakeskuksen soveltamisohjeista: Vakuutuspalkka eläkkeen perusteena olevana työansiona.",
    				label: "Vakuutuspalkka"
    			},
    			taxableReimbursementOfExpenses: {
    				description: "Veronalaisilla kustannusten korvauksilla tarkoitetaan työn suorittajalle työtehtävien hoitamisesta aiheutuneista kustannuksista maksettavaa veronalaista korvausta. Tulolajilla ilmoitetaan kaikkien sellaisten veronalaisten kustannusten korvausten yhteissumma, jota ei eritellä muilla tulolajeilla.\r\n\r\nKohdassa ilmoitetaan vain palkansaajalle maksettuja välittömästi työstä aiheutuneita kustannusten korvauksia, joita työnantaja ei ole lukenut mukaan palkan määrään toimittaessaan ennakonpidätystä (ennakkoperintälain 15 §).\r\n\r\nTyönantaja voi korvata palkansaajalle työstä välittömästi aiheutuneet kustannukset. Tällaisia ovat muun muassa kustannukset, jotka ovat aiheutuneet työvälineiden ja materiaalin hankkimisesta, tarveaineista ja suoja-asujen hankkimisesta aiheutuvat kustannukset. Esimerkiksi rakennusalalla maksetaan työehtosopimukseen perustuvaa työvälinekorvausta.\r\n\r\nTyönantaja ei toimita veronalaisista kustannusten korvauksista ennakonpidätystä eikä maksa sosiaalivakuutusmaksuja. Korvaukset ovat kuitenkin palkansaajan veronalaista tuloa. Palkansaajan tulee vaatia todelliset kustannukset vähennettäviksi esitäytetyllä veroilmoituksellaan.\r\n\r\nJos työntekijä on hankkinut työnantajan puolesta työvälineitä ja työmateriaaleja, niiden kustannuksia ei ilmoiteta, jos työnantaja on korvannut ne työntekijälle tositetta vastaan.",
    				label: "Veronalainen kustannusten korvaus"
    			},
    			privateDayCareAllowanceMunicipalSupplement: {
    				description: "Vain kunnat käyttävät tätä tulolajia.",
    				label: "Yksityisen hoidon tuen kuntalisä"
    			},
    			privateDayCareAllowanceWages: {
    				description: "Vain Kela käyttää tätä tulolajia.",
    				label: "Yksityisen hoidon tuki (palkka)"
    			},
    			privateDayCareAllowanceNonWage: {
    				description: "Vain Kela käyttää tätä tulolajia.",
    				label: "Yksityisen hoidon tuki (työkorvaus)"
    			},
    			kilometreAllowancePaidByNonProfitOrganisation: {
    				description: "Kilometrikorvaus, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nYleishyödyllisen yhteisön hyväksi sen toimeksiannosta tehty matka rinnastetaan työmatkaan silloinkin, kun suorituksen saaja ei ole palvelussuhteessa maksajaan eikä muutoin saa palkkaa työstä, johon matka liittyy.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut korvaukset. Jos verovapaat korvaukset liittyvät yhteisön maksamaan palkkaan, ne ilmoitetaan tulolajilla Kilometrikorvaus (verovapaa) (311).\r\n\r\nTulolajilla ilmoitetaan vain tuloverolain (1535/1992) 71 § 3 momentin mukaiset verovapaat kilometrikorvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa: matkustamiskustannusten korvaus, jota voidaan suorittaa myös verovelvollisen asunnolta tehdystä matkasta. Matkustamiskustannusten korvauksesta muulla kuin julkisella kulkuneuvolla tehdystä matkasta on verovapaata enintään 2 000 euroa kalenterivuodelta.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaa kilometrikorvauksia enemmän kuin tuloverolain 71 § 3 momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
    				label: "Yleishyödyllisen yhteisön maksama kilometrikorvaus"
    			},
    			dailyAllowancePaidByNonProfitOrganisation: {
    				description: "Työmatkapäiväraha, jonka yleishyödyllinen yhteisö maksaa korvauksettomaan vapaaehtoistyöhön osallistuvalle.\r\n\r\nTulolajilla ilmoitetaan vain vapaaehtoistyöhön osallistuville maksetut päivärahat. Jos verovapaa päiväraha liittyy yhteisön maksamaan palkkaan, se ilmoitetaan tulolajilla Päiväraha (331).\r\n\r\nKohdassa ilmoitetaan vain tuloverolain 71 § 3 momentin mukaiset verovapaat korvaukset. Lainkohdan mukaan matkakustannusten korvauksesta, jonka yleishyödyllinen yhteisö maksaa, on verovapaata tuloa:\r\n\r\n- päiväraha enintään 20 päivältä kalenterivuodessa\r\n- majoittumiskorvaus.\r\n\r\nTulolajilla ilmoitetaan vain verovapaat määrät. Jos yleishyödyllinen yhteisö maksaan päivärahaa enemmän kuin tuloverolain 7 § momentissa säädetään, ylimenevä osa ilmoitetaan työkorvauksena.",
    				label: "Yleishyödyllisen yhteisön maksama päiväraha"
    			},
    			unjustEnrichment: {
    				description: "Perusteeton etu -tulolajia käytetään vain, jos aiheettomasti maksettu suoritus havaitaan ennen ensimmäisen ilmoituksen lähettämistä, eikä tuloa ole aiemmin ilmoitettu muuna tulona tulorekisteriin.\r\n\r\nTämä ei tyypillisesti ole mahdollista Palkkaus.fi-maksuprosessissa. Ota yhteys asiakaspalveluun, jos haluat oikeasti käyttää tätä tyyppiä.",
    				label: "Perusteeton etu"
    			},
    			employeeStockOptionLowerMarketPrice: {
    				description: "Palkkio, joka annetaan työnantajayrityksen option muodossa ja jossa työntekijältä peritty option merkintähinta option luovutusajankohtana on selvästi alhaisempi kuin osakkeen markkinahinta.\r\n\r\nKäytettäessä optiota etu ilmoitetaan tulolajilla Työsuhdeoptio.",
    				label: "Työsuhdeoptio, jonka merkintähinta on luovutusajankohtana alhaisempi kuin markkinahinta"
    			},
    			royaltyPaidToNonResidentTaxpayer: {
    				description: "Rajoitetusti verovelvolliselle henkilölle tai yhteisölle maksettava korvaus tekijänoikeuden, teollisoikeuden tai muun oikeuden käyttämisestä tai käyttöoikeudesta tai teollisia, kaupallisia tai tieteellisiä kokemuksia koskevista tiedoista.\r\n\r\nRojaltin perusteena voi olla\r\n\r\n- kirjallisen, taiteellisen tai tieteellisen teoksen tekijänoikeus\r\n- oikeus valokuvaan\r\n- patentin, tavaramerkin, mallin, muotin, piirustuksen, salaisen kaavan tai valmistustavan käyttäminen tai käyttöoikeus\r\n- teollisia, kaupallisia tai tieteellisiä kokemuksia koskevat tiedot.",
    				label: "Rajoitetusti verovelvolliselle maksettu rojalti"
    			},
    			bicycleBenefitTaxExempt: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun verovapaa osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräetu on verovapaa, jos edun arvo on enintään 1 200 euroa vuodessa. Ylimenevä osuus on veronalaista luontoisetua. Jos työntekijä saa sekä polkupyöräedun että työsuhdematkalipun, niiden verovapaa määrä voi olla yhteensä korkeintaan 3 400 euroa.\r\n\r\nPolkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.",
    				label: "Polkupyöräedun verovapaa osuus"
    			},
    			bicycleBenefitTaxable: {
    				description: "Tulolajilla ilmoitetaan työnantajan työntekijälle antaman polkupyöräedun palkaksi katsottava osuus. Polkupyöräetu syntyy, kun työnantaja antaa omistamansa tai hallitsemansa polkupyörän työntekijän yksityiskäyttöön tai muulla vastaavalla tavalla järjestää työntekijälle mahdollisuuden käyttää polkupyörää vapaaajallaan.\r\n\r\nPolkupyöräedusta 1 200 euroa on verovapaata ja loput veronalaista luontoisetua.\r\n\r\nPalkaksi katsottu osuus polkupyöräedusta ilmoitetaan niiltä kuukausilta, kun etu on ollut työntekijän käytettävissä.\r\n\r\nPolkupyöräedun palkaksi katsottava osuus ja verovapaa osuus on eriteltävä, koska ne vaikuttavat työntekijän kodin ja työpaikan välisten matkakulujen vähennyskelpoisuuteen.",
    				label: "Polkupyöräedun palkaksi katsottu osuus"
    			},
    			conditionalStockOptions: {
    				description: "Osakepalkkio, johon on liittynyt osakkeiden luovutusrajoitus ja ehdollinen palautusvelvollisuus ja josta on syntynyt sosiaalivakuutusmaksujen alaista tuloa. Tässä tilanteessa palkkiosta saatavan edun arvo ei siis alkuperäisestä suunnitelmasta poiketen riipukaan osakkeiden arvon kehityksestä osakepalkkion lupaamisen ja antamisen välisenä, vähintään vuoden pituisena aikana.\r\n\r\nTällainen tilanne voi syntyä olosuhteiden muuttuessa esimerkiksi niin, että työntekijän työsuhde päättyy vuoden määräaikana ja hän saa pitää osan osak-keista. Samalla syntyy velvollisuus maksaa sosiaali-vakuutusmaksut.\r\nTällainen osakepalkkio katsotaan sosiaalivakuutus-maksujen näkökulmasta annetuksi silloin, kun palk-kiota koskevat rasitteet päättyvät ja osakepalkkio jää henkilön omistukseen ja vallintaan. Verotuksen kan-nalta ehdollinenkin osakepalkkio katsotaan kuitenkin annetuksi jo silloin, kun henkilö saa omistusoikeuden osakkeisiin.\r\n\r\nPalkkion määrä ilmoitetaan tulolajilla Ehdollinen osakepalkkio sosiaalivakuutusmaksujen määräämistä varten. Tulolajilla ilmoitettava osakepalkkio ei ole tulonsaajalle konkreettisesti maksettavaa tuloa, vaan tietoa käytetään sosiaalivakuutusmaksujen perusteena. Tulonsaajan saama etu ja siitä tehty ennakonpidätys on ilmoitettu tulolajilla Osakepalkkio (320) jo silloin, kun tulonsaaja on saanut omistusoikeuden osakepalkkioon. Tätä tietoa käytetään esimerkiksi verotuksessa.\r\n\r\nTulolajia Ehdollinen osakepalkkio käytetään vain silloin, kun palkkioon on liittynyt sekä osakkeiden luovutusrajoitus että ehdollinen palautusvelvollisuus. Jos palkkioon on liittynyt vain toinen rasitteista, se ilmoitetaan kuten tavallinen osakepalkkio tulolajilla Osakepalkkio (320).",
    				label: "Ehdollinen osakepalkkio"
    			},
    			compensationCollectedForCarBenefit: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa autoedun verotusarvosta.\r\n\r\nAutoedun omavastuuosuus alentaa verotettavaa etua tai poistaa sen kokonaan. Jos autoedun omavastuuosuus on autoedun verotusarvoa alempi, autoedun arvon ja autoedusta perityn korvauksen erotus on työntekijän veronalaista palkkaa. Jos omavastuuosuus vastaa autoedun verotusarvoa, verotettavaa etua ei synny. Tällöinkin peritty korvaus on ilmoitettava. Tällöin on lisäksi ilmoitettava edun määrä Autoetu-tulolajilla.\r\n\r\nAutoedun arvo ilmoitetaan täysimääräisenä Autoetu-tulolajilla ja Autoedusta peritty korvaustulolajilla työntekijältä perityn omavastuuosuuden määrä. \r\n\r\nAutoetu on ilmoitettava siinäkin tapauksessa, että suorituksen maksaja on perinyt tulonsaajalta luontoisedusta luontoisedun rahaarvoa vastaavan tai sitä suuremman korvauksen, eikä palkkaan lisättävää määrää ole jäänyt lainkaan.\r\n\r\nTyönantaja voi periä autoedun omavastuuosuuden työntekijän nettopalkasta.",
    				label: "Autoedusta peritty korvaus"
    			},
    			withholdingTax: {
    				description: "Ennakkoperintä, jossa suorituksen maksaja vähentää verovelvolliselle maksettavasta rahamäärästä ennakonpidätysprosentin mukaan lasketun veron.\r\n\r\nEnnakonpidätys voidaan ilmoittaa ilmoituksella yhteissummana, vaikka tulolajeja olisi ilmoitettu useita.",
    				label: "Ennakonpidätys"
    			},
    			electedOfficialFee: {
    				description: "Luottamushenkilömaksua voivat ilmoittaa vain kunta ja kuntayhtymä.",
    				label: "Luottamushenkilömaksu"
    			},
    			taxAtSource: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Lähdevero"
    			},
    			taxAtSourceDeduction: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Lähdeverovähennys"
    			},
    			wagesPaid: {
    				description: "Rahapalkka, joka maksetaan ennakonpidätyksen, työntekijän sosiaalivakuutusmaksujen ja muiden vähennettävien erien vähentämisen jälkeen työntekijälle.\r\n\r\nMuihin vähennettäviin eriin voivat kuulua esimerkiksi luontoiseduista työntekijältä perittävät maksut, työnantajan pidättämät työttömyyskassan ja ammattiliiton jäsenmaksut sekä ulosottomaksut.\r\n\r\nTyönantajalle kuuluvia kulueriä, kuten esim. työmatkoilta maksettavia hotellimaksuja, matkalippukuluja tai mahdollisia työterveydenhuollon maksuja ei ilmoiteta Maksettava palkka-tulolajilla.",
    				label: "Maksettava palkka"
    			},
    			reimbursementCollectedForOtherFringeBenefits: {
    				description: "Osuus luontoisedun arvosta, jonka työntekijä maksaa itse.\r\n\r\nKohdassa ilmoitetaan summa, joka työntekijältä on peritty muista luontoiseduista kuin autoedusta tai työsuhdematkalippuedusta.\r\n\r\nLuontoisedun arvo ilmoitetaan täysimääräisenä omalla tulolajillaan ja Muista luontoiseduista peritty korvaus-tulolajilla työntekijältä perityn omavastuuosuuden määrä.\r\n\r\nJos ravintoedusta peritty korvaus vastaa verotusarvoa, perittyä korvausta ei ilmoiteta Muusta luontoisedusta peritty korvaus-erittelylajilla. Tällöin käytetään vain tietoa Ravintoedusta peritty korvaus vastaa verotusarvoa – ”Kyllä (true)”.\r\n\r\nTyönantaja voi periä korvauksen luontoiseduista työntekijän nettopalkasta.",
    				label: "Muista luontoiseduista peritty korvaus"
    			},
    			otherItemDeductibleFromNetWage: {
    				description: "Erät, jotka palkan maksaja vähentää nettopalkasta ennen kuin maksaa suorituksen tulonsaajalle.\r\n\r\nNettopalkasta voidaan vähentää suorituksia lain tai sopimuksen perusteella. Lain perusteella tehtäviä vähennyksiä ovat esimeriksi ulosmittaukset. Työnantaja voi kuitata omia saataviaan työntekijän palkasta. Työntekijän nettopalkasta voidaan vähentää maksuja, joiden perimisestä on työnantajan ja työntekijän kesken sovittu. Nettopalkasta vähennettäviä eriä ovat esim. työttömyyskassan ja ammattiyhdistysten jäsenmaksut sekä pysäköintihallimaksut.\r\n\r\nJos ulosmittaus on jo ilmoitettu erikseen, ei sitä ilmoiteta enää tällä erittelylajilla yhteissummassa.",
    				label: "Nettopalkasta vähennettävä muu erä"
    			},
    			netWage: {
    				description: "Palkka, josta on toimitettu ennakonpidätys ja maksettu työntekijän sosiaalivakuutusmaksut. \r\n\r\nNettopalkka on eri asia kuin työntekijälle maksettava palkka.",
    				label: "Nettopalkka"
    			},
    			employerPaidPremiumForCollectiveAdditionalPensionInsurance: {
    				description: "Työnantajan osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta laisinkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksu-kohdassa.\r\n\r\nMaksut ilmoitetaan tässä kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
    				label: "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen maksu"
    			},
    			employerPaidPremiumForCollectiveAdditionalPensionInsuranceEmployeesContribution: {
    				description: "Työntekijän osuus kollektiivisen lisäeläkkeen kattamiseksi maksettavasta lisäeläkevakuutuksen maksusta.\r\n\r\nMaksu ilmoitetaan vain, jos työntekijä maksaa itse osan maksuista. Jos työnantaja maksaa kollektiivisen lisäeläketurvan maksut kokonaan, tietoja ei ilmoiteta lainkaan palkkatietoilmoituksella. \r\n\r\nKohdassa ilmoitetaan vain 6.5.2004 tai sen jälkeen otetun kollektiivisen lisäeläketurvan maksut. Jos työntekijä on ollut vakuutettuna ennen 6.5.2004, häneltä perityt maksut ilmoitetaan vain Työntekijän työeläkevakuutusmaksut-kohdassa.\r\n\r\nMaksut ilmoitetaan kohdassa myös silloin, kun uusi työntekijä on tullut 6.5.2004 jälkeen mukaan työnantajan ennen tuota päivää ottamaan kollektiiviseen vakuutusjärjestelmään.",
    				label: "Työnantajan maksaman kollektiivisen lisäeläkevakuutuksen työntekijältä peritty osuus"
    			},
    			employeesHealthInsuranceContribution: {
    				description: "Yleisesti verovelvollisella tulonsaajalla sairausvakuutusmaksu sisältyy aina ennakonpidätykseen, eikä yleisesti verovelvollisten osalta tule ilmoittaa erikseen työntekijän sairausvakuutusmaksua.\r\n\r\nRajoitetusti verovelvollisen sairausvakuutusmaksu on perittävä erikseen lähdeveron lisäksi, ellei rajoitetusti verovelvollinen ole hakeutunut verotusmenettelystä annetun lain mukaiseen verotukseen, jolloin sairausvakuutusmaksu sisältyy ennakonpidätykseen tai ennakkoveroon. \r\n\r\nHUOM: Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Työntekijän sairausvakuutusmaksu"
    			},
    			employeesPensionInsuranceContribution: {
    				description: "Bruttopalkan perusteella määräytyvä työeläkevakuutusmaksu, jolla työntekijä osallistuu työeläkevakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työeläkevakuutusmaksu vähennetään työntekijän nettopalkasta.",
    				label: "Työntekijän työeläkevakuutusmaksu"
    			},
    			employeesUnemploymentInsuranceContribution: {
    				description: "Bruttopalkan perusteella määräytyvä työttömyysvakuutusmaksu, jolla työntekijä osallistuu työttömyysvakuutuksen rahoittamiseen.\r\n\r\nTyöntekijän työttömyysvakuutusmaksu vähennetään työntekijän nettopalkasta.",
    				label: "Työntekijän työttömyysvakuutusmaksu"
    			},
    			reimbursementForEmployerSubsidisedCommuterTicket: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa työsuhdematkalipun euromääräisestä arvosta.\r\n\r\nOsa työsuhdematkalipun arvosta on verovapaata ja osa veronalaista. Veronalainen osuus työsuhdematkalipusta on luontoisetua. Työnantajan työntekijälle antama enintään 300 euron arvoinen joukkoliikenteen henkilökohtainen matkalippu asunnon ja työpaikan välistä matkaa varten on verovapaata tuloa. Lisäksi matkalippu on verovapaata tuloa 750 euron ylittävältä osalta 3 400 euroon asti. Työsuhdematkalipusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole.\r\n\r\nJos palkansaaja ostaa matkalipun itse ja työnantaja maksaa työntekijälle matkalipun hinnan tai osan siitä, työnantajan maksama korvaus katsotaan kokonaisuudessaan työntekijän palkaksi. Tällöin työntekijälle maksettu lipun hinta ilmoitetaan vain palkkana.\r\n\r\nTyönantaja voi periä työsuhdematkalipun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 200 euroa. Veronalaista luontoisetua ja palkkaa on 250 euroa. Työntekijä saa verovapaata etua 550 euroa.\r\n\r\n- Työsuhdematkalipun palkaksi katsottu osuus -tulolajilla ilmoitetaan palkaksi katsottu määrä 250 euroa.\r\n- Työsuhdematkalipusta peritty korvaus-erittelylajilla ilmoitetaan työntekijältä peritty määrä 200 euroa.\r\n- Työsuhdematkalipun verovapaa osuus- tulolajilla ilmoitetaan verovapaan edun määrä 550 euroa.\r\n\r\nEsimerkki: Työnantaja on antanut työntekijälle 1 000 euron arvoisen työsuhdematkalipun. Työnantaja perii työntekijän nettopalkasta 500 euroa. Veronalaisen luontoisedun määrä on 450 euroa. Koska peritty korvaus on suurempi kuin edun verotusarvo, verotettavaa luontoisetua ei synny. Työntekijä saa verovapaata etua 500 euroa.\r\n\r\n- Työsuhdematkalipusta peritty korvaus -erittelylajilla ilmoitetaan työntekijältä peritty määrä 500 euroa.\r\n- Työsuhdematkalipun verovapaa osuus-tulolajilla ilmoitetaan 500 euroa.",
    				label: "Työsuhdematkalipusta peritty korvaus"
    			},
    			taxPaidAbroad: {
    				description: "Palkkaus.fi-perusversio ei tue kaikkia kansainvälisiä tilanteita oikein! Jos maksat palkkoja ulkomailla työskentelevälle henkilölle, ota ensin yhteyttä asiakaspalveluun.",
    				label: "Ulkomaille maksettu vero"
    			},
    			distraint: {
    				description: "Perittäväksi tulleen velan maksamiseksi tehtävä pidätys. Ulosmitattava määrä lasketaan nettotulosta.\r\n\r\nUlosmitattaessa velallisen ja hänen perheensä toimeentuloa varten tulee kuitenkin jättää suojaosuus.",
    				label: "Ulosmittaus"
    			},
    			voluntaryIndividualPensionInsurancePremium: {
    				description: "Lakisääteistä eläketurvaa täydentävän tietylle henkilölle kohdistetun eläkevakuutuksen maksu. Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu voi olla jatkuva tai kertamaksullinen.\r\n\r\nKohtaan merkitään työntekijälle otetun vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Samoin ilmoitetaan myös sellaisen yksilöllisen eläkevakuutuksen maksut, joka on otettu avoimen yhtiön yhtiömiehelle, kommandiittiyhtiön vastuunalaiselle yhtiömiehelle tai sellaisen osakeyhtiön osakkaalle, joka ei ole työeläkevakuutettu.\r\n\r\nJos työnantajan ottaman vapaaehtoisen yksilöllisen eläkevakuutuksen maksut ylittävät 8 500 euroa, ne ovat veronalaista tuloa, joista maksetaan myös työnantajan sosiaalivakuutusmaksut. Jos maksut ylittävät 8 500 euroa vuodessa, suorituksen maksaja ilmoittaa ylimenevät maksut muuna luontoisetuina. \r\n\r\nEsimerkki: Työnantaja maksaa työntekijälle ottamansa vapaaehtoisen yksilöllisen eläkevakuutuksen maksuja vuodessa 12000 euroa. Työnantaja ilmoittaa tiedot seuraavasti:\r\n\r\n- Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu 8 500 euroa.\r\n- Muu verotettava luontoisetu 3 500 euroa.\r\n\r\nKohtaan merkitään vain vapaaehtoisen yksilöllisen eläkevakuutuksen maksut. Jos työnantaja on maksanut työntekijän puolesta työntekijän pakollisia eläkevakuutusmaksuja, työttömyysvakuutusmaksuja ja työntekijän itsensä ottaman vapaaehtoisen eläkevakuutuksen maksuja, nämä ilmoitetaan Muu luontoisetu-tulolajilla.",
    				label: "Vapaaehtoisen yksilöllisen eläkevakuutuksen maksu"
    			},
    			deductionBeforeWithholding: {
    				description: "Työntekijän itsensä maksama työn tekemisestä aiheutunut välitön kustannus, jonka työnantaja vähentää bruttopalkasta ennen ennakonpidätyksen osuuden määrittämistä. Tällaisia kustannuksia ovat muun muassa palkansaajan maksamat kustannukset moottorisahan käytöstä ja muut työvälineistä aiheutuneet kustannukset. Samoin tällaisia kustannuksia ovat myös menot työmatkoista, jos työnantaja ei maksa niistä verovapaita korvauksia palkan lisäksi.\r\n\r\nKohtaan merkitään vähennys, joka on tehty ennen ennakonpidätyksen toimittamista. Vähennysten osuuden tulee sisältyä myös tulolajilla ilmoitettuun palkkaan.\r\n\r\n**Esimerkki**: Työntekijä on maksanut itse moottorisahan käytöstä aiheutuneet 4 500 euron kustannukset. Työntekijälle maksettu koko vuoden palkka on 20 000 euroa, josta sahatyön osuus on 14 000 euroa. Työntekijä on vaatinut ennakkoperinnässä vähennystä käyttökustannuksista 40 %:n mukaisesti sahatyöstä maksetun palkan määrästä eli 5 600 euroa. Ennakonpidätys on toimitettu ennakkoperintälain (1118/1996) 15 § mukaisesti 14 400 eurosta (20 000 - 5 600).\r\n\r\nPalkan tulolajilla ilmoitetaan 20 000 euroa ja Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 5 600 euroa. Työntekijän tulee vaatia esitäytetyllä veroilmoituksellaan moottorisahavähennystä ja selvittää sahatyöstä maksetun palkan määrä. Lopullisessa verotuksessa vähennys lasketaan sahatyöstä maksetun palkan (14 000 euroa) osuudesta.\r\n\r\nJos työnantaja on maksanut moottorisahan käytöstä aiheutuneet kustannukset, katso ilmoittaminen kohdasta: Veronalaiset kustannusten korvaukset. \r\n\r\n**Esimerkki**: Pelimannille luvataan soittokeikasta 200 euroa. Matkustaminen ja matkustamisen kustannukset eivät vaikuta maksettavaan bruttomäärään. Matkakustannuksia ei siis korvata.\r\n\r\nTyönantaja voi laskea määrän, josta toimittaa ennakonpidätyksen siten, että vähentää työntekijän antaman selvityksen perusteella saman määrän, jonka voisi korvata verovapaasti. Esimerkiksi matkustamiskustannuksia 51 km x 41 snt/km eli 20,91 snt ja osapäivärahan 19 euroa vastaavat määrät, jolloin pidätyksenalaiseksi määräksi jää 160,09 euroa. Samasta määrästä maksetaan myös työnantajan sairausvakuutusmaksu. \r\n\r\nPalkan tulolajilla ilmoitetaan 200 euroa, Vähennys ennen ennakonpidätystä -erittelylajilla ilmoitetaan 39,91 euroa. Verovapaiden matkakustannusten korvausten tulolajeilla ei tehdä merkintöjä. \r\n\r\nTyöntekijän on vaadittava vähennyksiä veroilmoituksessaan ja vähennys on yleensä alempi kuin ennakonpidätyksen yhteydessä.",
    				label: "Vähennys ennen ennakonpidätystä"
    			},
    			reimbursementCollectedForBicycleBenefit: {
    				description: "Luontoisedun omavastuuosuus, jonka työntekijä maksaa polkupyöräedun euromääräisestä arvosta.\r\n\r\nOsa polkupyöräedun arvosta on verovapaata ja osa veronalaista. Polkupyöräedusta 1 200 euroa on verovapaata, ja ylittävä osa on veronalaista luontoisetua. Polkupyöräedusta on aina annettava ilmoitus, myös silloin kun verotettavaa etua ei ole. Edusta peritty korvaus alentaa verotettavan edun määrää.\r\n\r\nTyönantaja voi periä polkupyöräedun omavastuuosuuden työntekijän nettopalkasta.\r\n\r\nPolkupyöräedun ilmoittaminen poikkeaa muista luontoiseduista samalla tavalla kuin työsuhdematkalipun ilmoittaminen, koska työnantajan on vähennettävä polkupyöräedun palkaksi katsottavasta osuudesta työntekijältä perimänsä korvaus. Palkaksi katsottavana osuutena ilmoitetaan jäljelle jäävä määrä.",
    				label: "Polkupyöräedusta peritty korvaus"
    			}
    		},
    		Unit: {
    			count: {
    				description: "",
    				label: "Count"
    			},
    			days: {
    				description: "",
    				label: "Days"
    			},
    			euro: {
    				description: "",
    				label: "Euro"
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			},
    			kilometers: {
    				description: "",
    				label: "Kilometers"
    			},
    			one: {
    				description: "",
    				label: "One"
    			},
    			percent: {
    				description: "",
    				label: "Percent"
    			},
    			period: {
    				description: "",
    				label: "Period"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			weeks: {
    				description: "",
    				label: "Weeks"
    			}
    		},
    		VarmaPensionOrderAction: {
    			move: {
    				description: "",
    				label: "Move"
    			},
    			newTyel: {
    				description: "",
    				label: "NewTyel"
    			},
    			newYel: {
    				description: "",
    				label: "NewYel"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		VarmaPensionOrderYelPayer: {
    			company: {
    				description: "",
    				label: "Company"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		WorkflowEventFeatures: {
    			isActive: {
    				description: "",
    				label: "IsActive"
    			},
    			isEditable: {
    				description: "",
    				label: "IsEditable"
    			},
    			isOnHold: {
    				description: "",
    				label: "IsOnHold"
    			},
    			isRemovable: {
    				description: "",
    				label: "IsRemovable"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		WorkflowEventUi: {
    			danger: {
    				description: "",
    				label: "Danger"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			info: {
    				description: "",
    				label: "Info"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			success: {
    				description: "",
    				label: "Success"
    			},
    			warning: {
    				description: "",
    				label: "Warning"
    			}
    		},
    		YearEndAdminCheck: {
    			errors: {
    				description: "",
    				label: "Errors"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithWarning: {
    				description: "",
    				label: "OkWithWarning"
    			}
    		},
    		YearEndFeedbackCalculationType: {
    			correction: {
    				description: "",
    				label: "Correction"
    			},
    			external: {
    				description: "",
    				label: "External"
    			},
    			salaxyPayment: {
    				description: "",
    				label: "SalaxyPayment"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		YearEndUserFeedback: {
    			noYearlyReports: {
    				description: "",
    				label: "NoYearlyReports"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithModifications: {
    				description: "",
    				label: "OkWithModifications"
    			},
    			willHandleMyself: {
    				description: "",
    				label: "WillHandleMyself"
    			}
    		}
    	}
    };

    var SALAXY$f = {
    	MODEL: {
    		DeliveryData: {
    			title: "Aineiston tiedot",
    			deliveryId: {
    				title: "Aineistoviite"
    			},
    			deliveryDataType: {
    				title: "Tyyppi"
    			},
    			timestamp: {
    				title: "Muodostamishetki"
    			},
    			contactPersons: {
    				title: "Yhteyshenkilöt"
    			},
    			reportDate: {
    				title: "Ilmoituspäivä"
    			}
    		},
    		EarningsPayment: {
    			id: {
    				title: "Tunniste"
    			},
    			createdAt: {
    				title: "Muodostettu"
    			},
    			updatedAt: {
    				title: "Päivitetty"
    			},
    			owner: {
    				title: "Omistaja"
    			},
    			partner: {
    				title: "Partneri"
    			},
    			status: {
    				title: "Lähetyksen tila"
    			},
    			irStatus: {
    				title: "Tulorekisteritila"
    			},
    			reportLog: {
    				title: "Tulorekisteritapahtumat"
    			},
    			scheduledFor: {
    				title: "Ajastus"
    			},
    			isModified: {
    				title: "Muokattu tulorekisterilähetyksen jälkeen"
    			}
    		},
    		ContactPerson: {
    			title: "Yhteyshenkilö",
    			email: {
    				title: "Sähköposti"
    			},
    			name: {
    				title: "Nimi"
    			},
    			telephone: {
    				title: "Puhelinnumero"
    			},
    			responsibilityCode: {
    				title: "Vastuualue"
    			}
    		},
    		Id: {
    			code: {
    				title: "Tunniste"
    			},
    			type: {
    				title: "Tyyppi"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		PaymentPeriod: {
    			title: "Palkanmaksukausi ja maksu- tai ilmoituspäivä",
    			paymentDate: {
    				title: "Suorituksen maksupäivä tai muu ilmoituspäivä"
    			},
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		Payer: {
    			title: "Maksaja",
    			subOrgs: {
    				title: "Maksajan aliorganisaatiot"
    			},
    			payerIds: {
    				title: "Maksajan tunnisteet"
    			},
    			pensionInsurances: {
    				title: "Työeläkevakuutukset"
    			},
    			accidentInsurances: {
    				title: "Työtapaturmavakuutukset"
    			}
    		},
    		PayerBasic: {
    			companyName: {
    				title: "Yrityksen nimi"
    			},
    			missingId: {
    				title: "Maksajalla ei ole asiakastunnistetta"
    			}
    		},
    		PayerOther: {
    			payerTypes: {
    				title: "Maksajan tyypit"
    			}
    		},
    		Address: {
    			co: {
    				title: "c/o-tarkenne"
    			},
    			street: {
    				title: "Katuosoite"
    			},
    			poBox: {
    				title: "PL"
    			},
    			postalCode: {
    				title: "Postinumero"
    			},
    			postOffice: {
    				title: "Postitoimipaikka"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		TypedAddress: {
    			title: "Tulonsaajan osoite",
    			co: {
    				title: "c/o-tarkenne"
    			},
    			street: {
    				title: "Katuosoite"
    			},
    			poBox: {
    				title: "PL"
    			},
    			postalCode: {
    				title: "Postinumero"
    			},
    			postOffice: {
    				title: "Postitoimipaikka"
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			},
    			addressType: {
    				title: "Osoitteen tyyppi"
    			}
    		},
    		SubstitutePayer: {
    			title: "Sijaismaksaja",
    			acts: {
    				title: "Maksaja toimii sijaismaksajana"
    			},
    			employerName: {
    				title: "Varsinaisen työnantajan nimi"
    			},
    			wageSec: {
    				title: "Maksettu palkkaturvana"
    			}
    		},
    		IncomeEarner: {
    			title: "Tulonsaaja",
    			incomeEarnerIds: {
    				title: "Tulonsaajan tunnisteet"
    			},
    			addresses: {
    				title: "Tulonsaajan osoitteet "
    			},
    			professions: {
    				title: "Ammattiluokat"
    			}
    		},
    		IncomeEarnerOther: {
    			incomeEarnerTypes: {
    				title: "Tulonsaajan lisätiedot"
    			},
    			payments: {
    				title: "Maksujen tiedot"
    			}
    		},
    		Payment: {
    			paymentType: {
    				title: "Maksutiedon tyyppi"
    			},
    			paymentRef: {
    				title: "Maksun viitenumero"
    			},
    			paymentSpecifier: {
    				title: "Maksun tarkenne"
    			}
    		},
    		IncomeEarnerBasic: {
    			missingId: {
    				title: "Tulonsaajalla ei ole asiakastunnistetta"
    			},
    			companyName: {
    				title: "Yrityksen nimi"
    			},
    			firstName: {
    				title: "Etunimi"
    			},
    			lastName: {
    				title: "Sukunimi"
    			},
    			birthDate: {
    				title: "Syntymäaika"
    			},
    			gender: {
    				title: "Sukupuoli"
    			}
    		},
    		Profession: {
    			code: {
    				title: "Ammattiluokan tai nimikkeen tunniste"
    			},
    			type: {
    				title: "Ammattiluokan tai nimikkeistön tyyppi"
    			},
    			title: {
    				index: "Ammattiluokka tai -nimike",
    				title: "Ammattinimike (vapaamuotoinen)"
    			}
    		},
    		Employment: {
    			title: "Palvelussuhdetiedot",
    			employed: {
    				title: "Palvelussuhteessa"
    			},
    			employmentCode: {
    				title: "Palvelussuhteen tyyppi"
    			},
    			termCode: {
    				title: "Palvelussuhteen kesto"
    			},
    			partTime: {
    				title: "Osa-aikaisuuden prosentti"
    			},
    			hoursPerWeek: {
    				title: "Säännöllinen sovittu viikkotyöaika"
    			},
    			paymentTypes: {
    				title: "Palkkauksen muodot"
    			},
    			employmentPeriods: {
    				title: "Palvelussuhteen voimassaolo"
    			},
    			employmentEndings: {
    				title: "Palvelussuhteen päättyminen"
    			}
    		},
    		Period: {
    			title: "Palvelussuhteen voimassaoloaika",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		EmploymentEnding: {
    			title: "Palvelussuhteen päättyminen",
    			code: {
    				title: "Palvelussuhteen päättymisen syykoodi"
    			},
    			type: {
    				title: "Palvelussuhteen päättymisen syykoodisto"
    			}
    		},
    		PensionInsurance: {
    			title: "Työeläkevakuutus",
    			pensionActCode: {
    				title: "Tieto työeläkevakuutuksesta"
    			},
    			pensionProvIdCode: {
    				title: "Työeläkelaitoksen yhtiötunnus"
    			},
    			pensionPolicyNo: {
    				title: "Eläkejärjestelynumero"
    			}
    		},
    		AccidentInsurance: {
    			title: "Työtapaturmavakuutus",
    			accInsPolicyNo: {
    				title: "Työtapaturmavakuutuksen vakuutusnumero"
    			}
    		},
    		Absence: {
    			absenceRepStartDate: {
    				title: "Poissaolojen ilmoitusjakson alkupäivä"
    			},
    			absenceRepEndDate: {
    				title: "Poissaolojen ilmoitusjakson loppupäivä"
    			}
    		},
    		UnpaidAbsence: {
    			unpaidAbsencePeriods: {
    				title: "Palkaton poissaolo"
    			}
    		},
    		UnpaidAbsencePeriod: {
    			title: "Poissaolojakso",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			},
    			absenceDays: {
    				title: "Poissaolopäivien lukumäärä"
    			},
    			causeCode: {
    				title: "Poissaolon syy"
    			}
    		},
    		PaidAbsence: {
    			paidAbsencePeriods: {
    				title: "Palkallinen poissaolo"
    			}
    		},
    		PaidAbsencePeriod: {
    			title: "Poissaolojakso",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			},
    			absenceDays: {
    				title: "Poissaolopäivien lukumäärä"
    			},
    			absenceUntil: {
    				title: "Poissaolo jatkuu, mihin asti"
    			},
    			causeCode: {
    				title: "Poissaolon syy"
    			},
    			amount: {
    				title: "Palkan määrä palkalliselta poissaoloajalta"
    			},
    			paidAbsencePeriodInfo: {
    				title: "Korvaushakemuksen lisätiedot"
    			},
    			reimbApp: {
    				title: "Korvaushakemustiedot"
    			}
    		},
    		ReimbPayment: {
    			title: "Korvaushakemustiedot",
    			paymentType: {
    				title: "Maksutiedon tyyppi"
    			},
    			paymentRef: {
    				title: "Maksun viitenumero"
    			},
    			paymentSpecifier: {
    				title: "Maksun tarkenne"
    			}
    		},
    		Report: {
    			transactions: {
    				title: "Tulolajit"
    			}
    		},
    		Transaction: {
    			title: "Tulolaji",
    			dailyAllowance: {
    				title: "Päiväraha"
    			},
    			earningPeriods: {
    				title: "Ansaintakaudet"
    			},
    			insuranceData: {
    				title: "Vakuuttamistiedot"
    			},
    			otherBenefit: {
    				title: "Muu luontoisetu"
    			},
    			unitWages: {
    				title: "Palkan yksikkö"
    			},
    			carBenefit: {
    				title: "Autoetu"
    			},
    			kmAllowance: {
    				title: "Kilometrikorvaus (verovapaa) "
    			},
    			mealBenefit: {
    				title: "Ravintoetu"
    			},
    			recoveryData: {
    				title: "Takaisinperinnän lisätiedot"
    			},
    			sailorIncome: {
    				title: "Merityötulo"
    			},
    			sixMonthRule: {
    				title: "6 kk sääntö"
    			},
    			transactionBasic: {
    				title: "Tulolajin yleistiedot"
    			}
    		},
    		SubOrg: {
    			title: "Aliorganisaatio",
    			type: {
    				title: "Aliorganisaation tunnisteen tyyppi"
    			},
    			code: {
    				title: "Aliorganisaation tunniste"
    			}
    		},
    		EarningPeriod: {
    			title: "Ansaintakausi",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		TransactionInclusion: {
    			title: "Vakuuttamistieto",
    			insuranceCode: {
    				title: "Vakuuttamistiedon tyyppi"
    			},
    			included: {
    				title: "Vakuutusmaksun perusteena"
    			}
    		},
    		UnitWage: {
    			title: "Yksikköpalkka",
    			unitPrice: {
    				title: "Yksikköhinta"
    			},
    			unitAmount: {
    				title: "Yksiköiden määrä "
    			},
    			unitCode: {
    				title: "Yksikkö"
    			}
    		},
    		CarBenefit: {
    			title: "",
    			carBenefitCode: {
    				title: "Autoedun tyyppi"
    			},
    			ageGroupCode: {
    				title: "Auton ikäryhmä"
    			},
    			kilometers: {
    				title: "Auton kilometrimäärä"
    			}
    		},
    		KmAllowance: {
    			title: "Kilometrikorvaus (verovapaa)",
    			kilometers: {
    				title: "Kilometrien määrä"
    			}
    		},
    		MealBenefit: {
    			title: "Ravintoetu",
    			taxValue: {
    				title: "Ravintoedusta peritty korvaus vastaa verotusarvoa"
    			}
    		},
    		RecoveryData: {
    			title: "Takaisinperinnän lisätiedot",
    			recoveryDate: {
    				title: "Takaisinperintäpäivä"
    			},
    			taxAtSource: {
    				title: "Takaisinperittyyn määrään kohdistuva lähdevero"
    			},
    			withhold: {
    				title: "Takaisinperittyyn määrään kohdistuva ennakonpidätys"
    			},
    			origPaymentPeriods: {
    				title: "Alkuperäiset palkanmaksukaudet"
    			}
    		},
    		OrigPaymentPeriod: {
    			title: "Alkuperäinen palkanmaksukausi",
    			paymentDate: {
    				title: "Maksupäivä"
    			},
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		SailorIncome: {
    			title: "Merityötulo",
    			crossTradeTime: {
    				title: "Cross-trade-aika"
    			},
    			sailorIncomeProp: {
    				title: "Suoritus maksettu merityötulona "
    			},
    			withdrawalPeriod: {
    				title: "Nostettavissaolokausi"
    			}
    		},
    		WithdrawalPeriod: {
    			title: "Nostettavissaolokausi",
    			startDate: {
    				title: "Alkupäivä"
    			},
    			endDate: {
    				title: "Loppupäivä"
    			}
    		},
    		SixMonthRule: {
    			title: "6 KK sääntö",
    			applicable: {
    				title: "Kuuden kuukauden sääntö soveltuu "
    			},
    			countryCode: {
    				title: "Maakoodi"
    			},
    			countryName: {
    				title: "Maan nimi"
    			}
    		},
    		TransactionBasic: {
    			title: "Tulolajin yleistiedot",
    			transactionCode: {
    				title: "Tulolajin koodi"
    			},
    			amount: {
    				title: "Määrä"
    			},
    			noMoney: {
    				title: "Maksettu muuna kuin rahana"
    			},
    			oneOff: {
    				title: "Kertakorvaus"
    			},
    			recovery: {
    				title: "Takaisinperintä"
    			},
    			unjustEnrichment: {
    				title: "Perusteeton etu"
    			}
    		},
    		ApiValidation: {
    			title: "Validointi",
    			isValid: {
    				title: "Validi"
    			},
    			hasAllRequiredFields: {
    				title: "Kaikki pakolliset kentät"
    			},
    			errors: {
    				title: "Virheet"
    			}
    		},
    		ApiValidationError: {
    			title: "Validointivirhe",
    			code: {
    				title: "Koodi"
    			},
    			key: {
    				title: "Avain"
    			},
    			msg: {
    				title: "Viesti"
    			},
    			type: {
    				title: "Tyyppi"
    			}
    		},
    		PayerSummary: {
    			title: "Erillisilmoitus",
    			id: {
    				title: "Tunniste"
    			},
    			createdAt: {
    				title: "Muodostettu"
    			},
    			updatedAt: {
    				title: "Päivitetty"
    			},
    			owner: {
    				title: "Omistaja"
    			},
    			partner: {
    				title: "Partneri"
    			},
    			status: {
    				title: "Lähetyksen tila"
    			},
    			irStatus: {
    				title: "Tulorekisteritila"
    			},
    			reportLog: {
    				title: "Tulorekisteritapahtumat"
    			},
    			scheduledFor: {
    				title: "Ajastus"
    			},
    			isModified: {
    				title: "Muokattu tulorekisterilähetyksen jälkeen"
    			}
    		},
    		ReportLogEntry: {
    			title: "Tulorekisteritapahtuma",
    			irTimestamp: {
    				title: "Tulorekisterin aikaleima"
    			},
    			eventType: {
    				title: "Tapahtumatyyppi"
    			},
    			reportId: {
    				title: "Raporttitunnus"
    			},
    			irReportId: {
    				title: "Tulorekisterin raporttitunnus"
    			},
    			irDeliveryId: {
    				title: "Toimitustunnus"
    			},
    			irResponseId: {
    				title: "Vastaustunnus"
    			},
    			irVersion: {
    				title: "Tulorekisteriversio"
    			},
    			user: {
    				title: "Käyttäjä"
    			},
    			versionId: {
    				title: "Versiotunnus"
    			},
    			message: {
    				title: "Viesti"
    			}
    		},
    		ApiScheduleObject: {
    			title: "Jono-objekti",
    			id: {
    				title: "Raporttitunnus"
    			},
    			startAt: {
    				title: "Ajastus"
    			},
    			action: {
    				title: "Operaatio"
    			}
    		},
    		PaymentMonth: {
    			title: "Kohdekausi",
    			year: {
    				title: "Vuosi"
    			},
    			month: {
    				title: "Kuukausi"
    			}
    		},
    		ReportData: {
    			title: "Palvelun tiedot",
    			actionCode: {
    				title: "Toimenpiteen tyyppi"
    			},
    			irReportId: {
    				title: "Tulorekisterin ilmoitusviite"
    			},
    			reportId: {
    				title: "Maksajan ilmoitusviite"
    			},
    			reportVersion: {
    				title: "Ilmoituksen versionumero"
    			}
    		}
    	}
    };

    var SALAXY$e = {
    	MODEL: {
    		AbsencePeriod: {
    			title: "absence period",
    			descr: "Period of absence",
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			}
    		},
    		AccountBase: {
    			title: "account base",
    			descr: "Account - either Person or Company - is a juridical entity Paying or receiving salaries, making contracts etc. It may be related to zero, one or multiple UserAccounts/Credentials.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AccountInIndex: {
    			title: "account in index",
    			descr: "Object that represents a Salaxy account.",
    			avatar: {
    				title: "Avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "Sähköpostiosoite",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "Tilinumero (IBAN)",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "Y-tunnus / Hetu",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "Puhelinnumero",
    				descr: "Telephone number."
    			}
    		},
    		AccountProducts: {
    			title: "account products",
    			descr: "Provides an overview and the possibility to edit products that are enabled to this account",
    			accounting: {
    				title: "accounting",
    				descr: "Bookkeeping reports for salarypayments"
    			},
    			baseService: {
    				title: "base service",
    				descr: "The basic services for Salary payment."
    			},
    			healthCareHeltti: {
    				title: "health care heltti",
    				descr: "Employer health care from the company Heltti"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Reports and payments for mandatory employment insurances."
    			},
    			pension: {
    				title: "pension",
    				descr: "Reports ant payments to Pension companies."
    			},
    			pensionEntrepreneur: {
    				title: "pension entrepreneur",
    				descr: "Pension payments for an entrepreneur"
    			},
    			pricing: {
    				title: "pricing",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			tax: {
    				title: "tax",
    				descr: "Reports and payments to tax authorities."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "Reports and payments to TVR (the unemployment insurance fund of Finland)"
    			}
    		},
    		AccountValidation: {
    			title: "account validation",
    			descr: "Validation errors for the onboarding process",
    			canStartSign: {
    				title: "can start sign",
    				descr: "TODO: NOt implemented yet - should be implemented\nIf true, the onboarding object has sufficient data to start the Digital Signature process (Visma Sign).\nDepending on the use case this may or may not be required for committing the data as account data."
    			},
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			existingAccount: {
    				title: "existing account",
    				descr: "Existing account based on official id (Personal ID / Business ID)."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, none of the accounts in ExistingAccounts cannot be accessed by the current user."
    			},
    			isCompleted: {
    				title: "is completed",
    				descr: "If true, the validation process has been completed. The IsValid may be true or false."
    			},
    			isOfficialIdValid: {
    				title: "is official id valid",
    				descr: "If true the official ID is technically valid.\nOfficial ID is either Finnish Personal ID (HETU, Henkilötunnus) or Business ID (Y-tunnus)."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		Accountant: {
    			title: "accountant",
    			descr: "The person or account reponsible for accounting",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			type: {
    				title: "type",
    				descr: "If true, the accountant is a registered Palkkaus.fi account who typically has access to this account."
    			}
    		},
    		AccountingData: {
    			title: "accounting data",
    			descr: "Accounting data for reports.",
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			ledgerAccounts: {
    				title: "ledger accounts",
    				descr: "All ledgers."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			}
    		},
    		AccountingDataSummary: {
    			title: "accounting data summary",
    			descr: "Summary of the accounting data. Contains main figures: totals and workers and employer total figures.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary of the calculations included in the report."
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment of the calculations included in the report."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "The total number of distinct workers included in the report."
    			}
    		},
    		AccountingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			accountantEmail: {
    				title: "accountant email"
    			},
    			accountantName: {
    				title: "accountant name"
    			},
    			accountantPhone: {
    				title: "accountant phone"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by AccountantName, AccountantEmail and AccountantPhone."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		AccountingReportRow: {
    			title: "accounting report row",
    			descr: "Accounting report table row.",
    			accountNumber: {
    				title: "account number",
    				descr: "Proposed account number for the row."
    			},
    			accountText: {
    				title: "account text",
    				descr: "Proposed account text for the row."
    			},
    			additionalInformation: {
    				title: "additional information",
    				descr: "Additional information for the row."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount for the row: May be positive or negative(depending on the depet/credit and account type)."
    			},
    			count: {
    				title: "count",
    				descr: "Count of units. Default is 1."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the unit."
    			},
    			productCode: {
    				title: "product code",
    				descr: "Returns product code."
    			},
    			productName: {
    				title: "product name",
    				descr: "Returns product name. (Code or ResultCode in this order)."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Row type. Default is the Booking."
    			},
    			styles: {
    				title: "styles",
    				descr: "Space separated set of class names for CSS styling purposes."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the booking entry / header."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit identifier;"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percentage of the value added tax that is included in the amount."
    			}
    		},
    		AccountingReportTable: {
    			title: "accounting report table",
    			descr: "Contains accounting rows for one or many calculations.",
    			businessId: {
    				title: "business id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			rows: {
    				title: "rows",
    				descr: "All rows in the report table."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			},
    			tableType: {
    				title: "table type",
    				descr: "Accounting report table type."
    			}
    		},
    		AccountingReportTableSummary: {
    			title: "accounting report table summary",
    			descr: "Summary of the accounting report. Contains main figures: totals and workers and employer total figures.",
    			count: {
    				title: "count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		AccountingRuleSet: {
    			title: "accounting rule set",
    			descr: "Ruleset containing required tabular logic and accounts for creating accounting entries.",
    			accounts: {
    				title: "accounts",
    				descr: "Accounts which are used in the tabular logic for creating accounting entries."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the ruleset."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the rule set is read only."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the ruleset."
    			},
    			rows: {
    				title: "rows",
    				descr: "Tabular logic for creating accounting entries."
    			},
    			templateId: {
    				title: "template id",
    				descr: "Template to which this is based on."
    			}
    		},
    		AccountingRuleSetAccount: {
    			title: "accounting rule set account",
    			descr: "Single account with number, grouping and default text.",
    			account: {
    				title: "account",
    				descr: "Account number."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the account."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the account is non-removable template."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the account."
    			}
    		},
    		AccountingRuleSetRow: {
    			title: "accounting rule set row",
    			descr: "Contains a single row of the tabular logic for creating accounting entries.",
    			creditGroupingId: {
    				title: "credit grouping id",
    				descr: "Additional flags for directing the logic for creating accounting entry based on given entries."
    			},
    			creditGroupingText: {
    				title: "credit grouping text"
    			},
    			creditId: {
    				title: "credit id"
    			},
    			debitGroupingId: {
    				title: "debit grouping id",
    				descr: "Id of the credit account."
    			},
    			debitGroupingText: {
    				title: "debit grouping text"
    			},
    			debitId: {
    				title: "debit id",
    				descr: "Id of the debit account."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if the row is in use."
    			},
    			entryCode: {
    				title: "entry code",
    				descr: "A single code in the code group: single income type code or total row type."
    			},
    			entryCodeGroup: {
    				title: "entry code group",
    				descr: "Entry group: income type or total type."
    			},
    			flags: {
    				title: "flags"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the row."
    			},
    			isIncluded: {
    				title: "is included",
    				descr: "Boolean indicating whether the system should make an accounting entry based on this row."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the row is non-removable template."
    			}
    		},
    		AccountingSettings: {
    			title: "accounting settings",
    			descr: "Settings related to accounting reports.",
    			accountant: {
    				title: "accountant",
    				descr: "The person or company responsible for accounting."
    			},
    			defaultTargetId: {
    				title: "Kohteen tunniste",
    				descr: "Default accounting target."
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			ruleSets: {
    				title: "rule sets",
    				descr: "Rulesets containing required tabular logic and accounts for creating accounting entries."
    			},
    			software: {
    				title: "software",
    				descr: "Integrated accounting software."
    			},
    			targets: {
    				title: "Kohteet",
    				descr: "Accounting target systems and the rulesets assigned to them."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		AccountingSoftwareSettings: {
    			title: "accounting software settings",
    			descr: "Setting related to accounting software"
    		},
    		AccountingTarget: {
    			title: "accounting target",
    			descr: "Accounting target systems and the rulesets assigned to them.",
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if this target is in use"
    			},
    			id: {
    				title: "id",
    				descr: "Id of the target definition."
    			},
    			image: {
    				title: "image",
    				descr: "Illustrative image for the target system."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the target system."
    			},
    			ruleSetId: {
    				title: "rule set id",
    				descr: "Assigned ruleset for this target system."
    			}
    		},
    		AccountingTargetSettings: {
    			id: {
    				title: "Tunniste"
    			},
    			method: {
    				title: "Tyyppi"
    			},
    			availableMethods: {
    				title: "Tyyppivalinnat"
    			},
    			avatar: {
    				title: "Avatar"
    			},
    			data: {
    				title: "Data"
    			}
    		},
    		Ajax: {
    			title: "ajax",
    			descr: "Abstracts the interface for Http / Ajax functionality that is required by Salaxy core classes.\nThis interface is implemented in this project for jQuery and Node.\nAdditional implementations exists for Angular.js (salaxy-lib-ng1) and Angular.io / ngx (salaxy-lib-ng).\nYou may also create your own implementation for the framework or http component that you want to support.",
    			serverAddress: {
    				title: "server address",
    				descr: "The server address - root of the server. This is settable field.\nWill probably be changed to a configuration object in the final version."
    			},
    			useCredentials: {
    				title: "use credentials",
    				descr: "By default credentials are not used in http-calls.\nEnable credentials with this flag (force disabled when the token is set)."
    			},
    			getApiAddress: {
    				title: "get api address",
    				descr: "Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api'"
    			},
    			getCurrentToken: {
    				title: "get current token",
    				descr: "Gets the current Salaxy Bearer token"
    			},
    			getHTML: {
    				title: "get h t m l",
    				descr: "Gets a HTML-message from server using the API\n\n**Returns**: A Promise with result html. Standard Promise rejection to be used for error handling.\n"
    			},
    			getJSON: {
    				title: "get j s o n",
    				descr: "Gets a JSON-message from server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			getServerAddress: {
    				title: "get server address",
    				descr: "Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com'"
    			},
    			postHTML: {
    				title: "post h t m l",
    				descr: "POSTS data to server and receives back HTML.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			postJSON: {
    				title: "post j s o n",
    				descr: "POSTS data to server and receives back a JSON-message.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			remove: {
    				title: "remove",
    				descr: "Sends a DELETE-message to server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			setCurrentToken: {
    				title: "set current token",
    				descr: "Sets Salaxy Bearer token for this http context."
    			}
    		},
    		AnnualLeave: {
    			title: "annual leave",
    			descr: "A planned annual Leave - not sure whether it will kept or is it just planned",
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeavePayment: {
    			title: "annual leave payment",
    			descr: "Payment related to annual leave: holiday salary, bonus and compensation",
    			calcId: {
    				title: "calc id",
    				descr: "If the payment is defined by a calculation, stores the calculation ID."
    			},
    			date: {
    				title: "date",
    				descr: "The reference date: Typically the Paid at date.\nThe items are typically sorted based on this."
    			},
    			holidayBonus: {
    				title: "holiday bonus",
    				descr: "Bonus for either Holiday Salary or Compensation."
    			},
    			holidayCompensation: {
    				title: "holiday compensation",
    				descr: "Compensation that is paid for holidays not kept."
    			},
    			holidayDays: {
    				title: "holiday days",
    				descr: "Days of holidays that are paid or compensated"
    			},
    			holidaySalary: {
    				title: "holiday salary",
    				descr: "Salary that is paid for holidays"
    			},
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the payment entry"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeaves: {
    			title: "annual leaves",
    			descr: "Data about the annual leaves planned and used during the holiday year.",
    			paid: {
    				title: "paid",
    				descr: "Paid or otherwise handled annual leaves.\nPRELIMINARY: This property is under revision - it may be moved or renamed without being a breaking change."
    			},
    			planned: {
    				title: "planned",
    				descr: "Planned annual leaves."
    			}
    		},
    		ApiCrudObject: {
    			title: "api crud object",
    			descr: "Base interface for CRUD objects in the Salaxy API.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Partner service that has be used in modifying the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated. Typically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		ApiListItem: {
    			title: "api list item",
    			descr: "The line item class for search and listing results. Use this class for building user interfaces.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			listType: {
    				title: "list type",
    				descr: "List item type of the item if the item supports a custom OData interface."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			type: {
    				title: "type",
    				descr: "The payload type of the item."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ApiListItemValidation: {
    			title: "api list item validation",
    			descr: "ApiValidation object that is specialized for list items. Provides if and index properties for connecting a validation to a specific item in the list",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the item this validation is for.\nIdentifiers are not always available (if the item has not been saved), so index is a more generic approach."
    			},
    			index: {
    				title: "index",
    				descr: "Zero based index of the item this validation is for."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiProfileSearchResult: {
    			title: "api profile search result",
    			descr: "Profile search result for the API",
    			cities: {
    				title: "cities"
    			},
    			facets: {
    				title: "facets",
    				descr: "Dynamic for search facets if faceting is being used.\nNOTE: Facet names are lower camel case - using the JavaScript naming conventions as in the original JSON."
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			},
    			results: {
    				title: "results",
    				descr: "Actual search result - list of items"
    			},
    			searchText: {
    				title: "search text",
    				descr: "Search text that was input by the user"
    			},
    			searchTextTechnical: {
    				title: "search text technical",
    				descr: "Search text as it is sent to the search provider:\nContains provider specific wildcards etc."
    			},
    			sort: {
    				title: "sort",
    				descr: "Sort string"
    			},
    			totalCount: {
    				title: "total count",
    				descr: "Total count returned by the search service.\nThis may be more than the count of Result list"
    			}
    		},
    		ApiProfileSearchResultProfile: {
    			title: "api profile search result profile",
    			avatar: {
    				title: "avatar"
    			},
    			cities: {
    				title: "cities"
    			},
    			id: {
    				title: "id"
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			}
    		},
    		ApiValidation: {
    			title: "api validation",
    			descr: "Common base class / interface for data validation.",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiValidationError: {
    			title: "api validation error",
    			descr: "Validation errors used in API output.",
    			code: {
    				title: "code",
    				descr: "Some error providers may have a unique technical code for error or validator"
    			},
    			key: {
    				title: "key",
    				descr: "Full path to Name of the property / field (Member name in JSON).\nThis may be null/empty for type General."
    			},
    			msg: {
    				title: "msg",
    				descr: "Validation error message in the requested language."
    			},
    			type: {
    				title: "type",
    				descr: "High level type: Specifies the relation to object and its Member."
    			}
    		},
    		AppStatus: {
    			title: "app status",
    			descr: "Supports the UserSession object by providing additional information about the status\nof fetching the session, data, expiration etc.",
    			clientRoles: {
    				title: "client roles",
    				descr: "Application specific roles that are set on the client (browser code)\nas opposed to normal server-side defined roles."
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "Indicates that we are running in test environment with test data.\nThis information is typically available already before the session is available and may be used in login / register.\nNull indicates that the information is not available. This should typically be considered false for security."
    			},
    			sessionCheckInProgress: {
    				title: "session check in progress",
    				descr: "If true, the session check from server is in progress"
    			},
    			sessionExpired: {
    				title: "session expired",
    				descr: "PLACEHOLDER: Indicates that the session has expired (as opposed to not checked yet).\nThis property has not yet been implemented."
    			}
    		},
    		Article: {
    			title: {
    				title: "title",
    				descr: "Title: Shown in te beginning of the text, in the browser window header, in the lists etc."
    			},
    			descr: "Read-only Article object for the API",
    			author: {
    				title: "author",
    				descr: "Author of the article."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			date: {
    				title: "date",
    				descr: "Publish date"
    			},
    			file: {
    				title: "file",
    				descr: "File path as in CDN"
    			},
    			html: {
    				title: "html",
    				descr: "Article text and/or markup.\nThis is not serialized to XML, but read/written separately to blob storage - typically using MarkDown."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			longSummary: {
    				title: "long summary",
    				descr: "A slightly longer summary, shown in Facebook, Google+\nand LinkedIn likes/shares."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			picture: {
    				title: "picture",
    				descr: "URL to a picture that should be used in social media sharing"
    			},
    			section: {
    				title: "section",
    				descr: "The section where the article should appear"
    			},
    			shortSummary: {
    				title: "short summary",
    				descr: "Shorter summary (max 140 chars), shown as default\nin Tweets."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AssureCompanyAccountRequest: {
    			title: "assure company account request",
    			descr: "Request to assure that a company account exists: Either an account is returned or an account stub (non-verified) will be created.",
    			contactFirstName: {
    				title: "contact first name",
    				descr: "First name of the contact person"
    			},
    			contactLastName: {
    				title: "contact last name",
    				descr: "Last name of the contact person"
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Company number in the the trade register (Y-tunnus).\nFormat is \"1234567-8\" - 7 numbers, one control character, with or without the dash.\nAlso, the international VAT format is allowed: FI12345678"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The account ID at the partner site.\nIf present, this ID is used to restrict the authorization to one account at partner site only."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number for the contact person"
    			}
    		},
    		AttributeDoc: {
    			title: "attribute doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attr: {
    				title: "attr",
    				descr: "HTML attribute name e.g. my-prop for myProp."
    			},
    			binding: {
    				title: "binding",
    				descr: "Angular JS binding type"
    			},
    			controller: {
    				title: "controller",
    				descr: "Controller that implements the property"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the memeber."
    			}
    		},
    		Author: {
    			title: "author",
    			descr: "Author of an article in the CMS",
    			companyDescription: {
    				title: "company description",
    				descr: "Short description text / label for the organization."
    			},
    			companyLogo: {
    				title: "company logo",
    				descr: "Company or organization logo - expect that this may be null."
    			},
    			companyUrl: {
    				title: "company url",
    				descr: "Link to a web site - typically of the organization that the author represents"
    			},
    			description: {
    				title: "description",
    				descr: "Short description text"
    			},
    			guid: {
    				title: "guid",
    				descr: "Unique key for the object"
    			},
    			image: {
    				title: "image",
    				descr: "Public image for the author"
    			},
    			name: {
    				title: "name",
    				descr: "Person name"
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Exact time of last technical save: Not editable by user and will change in imports/exports etc."
    			},
    			type: {
    				title: "type",
    				descr: "Supports the Cosmos DB migration by serializing the type of the current object as a short type string for querying."
    			}
    		},
    		Authority: {
    			title: "authority",
    			descr: "A person or other party that authorized the current person.",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account ID. This is optional and should not be set if the ID is the same as the target account / Person."
    			},
    			authorizedAt: {
    				title: "authorized at",
    				descr: "Date for the authorization."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail of the person. This is currently not used, but is expected to be used in the future.\nSo fill it in as it was required unless you have very compelling reasons."
    			},
    			name: {
    				title: "name",
    				descr: "Name of the authority."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Person ID is the key to signature. This is currently always required."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number of the person. This is currently optional.\nIdea is to send SMS invitations in some scenarios."
    			}
    		},
    		AuthorizationContract: {
    			title: "authorization contract",
    			descr: "Contract / authorization document that is the bases for Salaxy acting in behalf of this account",
    			authorizationType: {
    				title: "authorization type",
    				descr: "Type of the authorization"
    			},
    			authorizedBy: {
    				title: "authorized by",
    				descr: "A person or other party that authorized the current person."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "If true, there is a valid authorization - the account has been validated."
    			},
    			signedAt: {
    				title: "signed at",
    				descr: "Time when the contract was signed"
    			}
    		},
    		Avatar: {
    			title: "avatar",
    			descr: "Defines an avatar for an account, profile etc. - mainly the image that should be shown, names and short description",
    			color: {
    				title: "color",
    				descr: "Color - currently only used by type Icon"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			description: {
    				title: "description",
    				descr: "Short description of the user.\nThis may be overriden by a context specific value by the business logic."
    			},
    			displayName: {
    				title: "display name",
    				descr: "Display name. For a person this is 'FirstName LastName' (auto-created)."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type: person/company"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			initials: {
    				title: "initials",
    				descr: "Initials - currently only used by type Icon"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pictureType: {
    				title: "picture type",
    				descr: "Type of the Avatar picture."
    			},
    			sortableName: {
    				title: "sortable name",
    				descr: "Sortable name for ordered lists etc. For a person this is 'LastName, FirstName' (auto-created).\nDepending on the data storage, this may be all-lowercase for easier sorting and search."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			url: {
    				title: "url",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			}
    		},
    		BaseSalaryProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		BlobFile: {
    			title: "blob file",
    			descr: "The BlobFile stores metadata about miscellaneous BLOB-files that are stored to the system. For example Receipts and Authorizations.",
    			blobUri: {
    				title: "blob uri",
    				descr: "Full unique URI in the storage"
    			},
    			container: {
    				title: "container",
    				descr: "The logical container - first level folder for the file."
    			},
    			filePath: {
    				title: "file path",
    				descr: "Technical file name in storage.  In practical terms, this is the relative path\nexcluding the container part \"/[container]/\".\nI.e. this property is without the starting slash \"/\".\nIt must be unique and is used for Uri.\nFor user files this is constructed \"[Username]/[Type]/[UniqueFileName].[xxx]\""
    			},
    			fileType: {
    				title: "file type",
    				descr: "File type (mime type etc.) based on the ending of the file name.\nNull for folders."
    			},
    			id: {
    				title: "id"
    			},
    			isCollection: {
    				title: "is collection",
    				descr: "If true, this is a collection (folder in file system)"
    			},
    			length: {
    				title: "length",
    				descr: "The length of the file in bytes."
    			},
    			metadata: {
    				title: "metadata"
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "If created, contains uri to a PNG image for preview purposes."
    			},
    			type: {
    				title: "type",
    				descr: "Strongly typed logical type of the file. For user files, this is a sub folder."
    			},
    			updatedAt: {
    				title: "updated at"
    			},
    			userFriendlyName: {
    				title: "user friendly name",
    				descr: "A user friendly name that can be used in listings.\nIn the upload scenarios this is usually the original file name when uploaded.\nIt is not necessarily unique. No path information"
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		BlobFileMetadata: {
    			title: "blob file metadata",
    			descr: "Metadata related to blob files as a strongly typed object.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object related to the report.\nFor example for the salary payments this is the other party (employer or worker)"
    			},
    			data: {
    				title: "data",
    				descr: "Arbitrary metadata."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Identifier of the original calculation etc. business object."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "Whether the document is digitally signed (or manually checked as being signed).\nAt the time of writing, this is used only for AuthorizationPdf documents."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "Logical date for the file. E.g. first of the month for a monthly report."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype is internal typing within a BlobFileType. This may or may not be a sub-folder.\nThere is no enumeration on File level - there may be one under services specific to one BlobFileType."
    			}
    		},
    		BoardUsecase: {
    			title: "board usecase",
    			descr: "Handles the Calculation of CalculationRowType.Board to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		BusinessObject: {
    			title: "business object",
    			descr: "Business object in the Salaxy repository",
    			id: {
    				title: "id",
    				descr: "Id of the business object"
    			},
    			name: {
    				title: "name",
    				descr: "Optional name for the business object."
    			},
    			object: {
    				title: "object",
    				descr: "Optional property for object instance."
    			},
    			owner: {
    				title: "owner",
    				descr: "Partition key of the business object"
    			},
    			type: {
    				title: "type",
    				descr: "Business object type"
    			}
    		},
    		CalcEmployer: {
    			title: "calc employer",
    			descr: "Employer of a calculation",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account identifier for the Employer."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image information and type of employer.\nNOTE: Recalculate-method retains the values for display purposes if AccountId or IsSelf is not set.\nThis is mainly for demo and testing in anonymous mode.\nHowever, in storage methods, the value is reset based on values of AccountId or IsSelf\neven if they are not set: Anything stored in Avatar will be ignored."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			}
    		},
    		CalcHolidayBonus: {
    			title: "calc holiday bonus",
    			descr: "Holiday Bonus data that is stored into a Salary calculation.",
    			alreadyPaid: {
    				title: "already paid",
    				descr: "The sum of holiday bonus that has already been paid this year.\nThe payment is never so big that this would become more than BonusTotalHolidaySalary\n(by this calculation - the limit can be exceeded manually)."
    			},
    			totalHolidaySalary: {
    				title: "total holiday salary",
    				descr: "Total yearly holiday salary for the purposes of holiday bonus."
    			},
    			yearlyLeaveDays: {
    				title: "yearly leave days",
    				descr: "Number of yearly leave days. This is for the purposes of calculating the Bonus,\nso this is yearly salary days, not the complete saldo (including StartSaldo)."
    			}
    		},
    		CalcHourlyData: {
    			title: "calc hourly data",
    			descr: "Holiday calculation based on hourly salary OR performance based salaries. May also contain other data related to working hours.",
    			isHourly: {
    				title: "is hourly",
    				descr: "If true, the hourly / performance based calculation is applied."
    			}
    		},
    		CalcInfo: {
    			title: "calc info",
    			descr: "Informational object that stores data about the salary calculation / payment that is not directly related to calculating the salary.",
    			backofficeNotes: {
    				title: "backoffice notes",
    				descr: "Notes related to the payment and other backoffice operations:\nFor example the justification why the sum is in calculation vs. Paytrail payment."
    			},
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			paymentChannel: {
    				title: "payment channel",
    				descr: "Channel that is used for payment."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "Payment reference number in payment service"
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "The Payroll Id, if this calculation is part of a Payroll list."
    			},
    			pensionPaymentDate: {
    				title: "pension payment date",
    				descr: "Calculated pension insurance payment date."
    			},
    			pensionPaymentRef: {
    				title: "pension payment ref",
    				descr: "Pension insurance reference number."
    			},
    			pensionPaymentSpecifier: {
    				title: "pension payment specifier",
    				descr: "Pension insurance payment specifier."
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "End date of the work"
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Start date of the work"
    			},
    			workerMessage: {
    				title: "worker message",
    				descr: "Message from worker to employer - when worker sends the salary calculation to the employer."
    			}
    		},
    		CalcMonthlyData: {
    			title: "calc monthly data",
    			descr: "Period recalculation parameters that are related to Monthly salary. Could theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested.",
    			absencesDaySalary: {
    				title: "absences day salary",
    				descr: "Price for a single absence day. Typically based on MonthlySalary, but may be modified."
    			},
    			isMonthly: {
    				title: "is monthly",
    				descr: "If true the Monthly (weekly) calculation is being applied to absences and holidays."
    			},
    			monthlySalary: {
    				title: "monthly salary",
    				descr: "Monthly salary in holiday and absences calculation.\nTypically, the same as the actual monthly calculation, but may be modified."
    			}
    		},
    		CalcRowConfig: {
    			title: "calc row config",
    			descr: {
    				title: "descr",
    				descr: "Description for the row type"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount input"
    			},
    			category: {
    				title: "category",
    				descr: "Categorization of the row types."
    			},
    			color: {
    				title: "color",
    				descr: "Color is based on the category"
    			},
    			favorite: {
    				title: "favorite",
    				descr: "Favorite row types and their sort order.\nThis is currently fixed, but may later be based on user profile (e.g. most used types)."
    			},
    			iconText: {
    				title: "icon text",
    				descr: "Icon short text is 1-5 characters suitable for icon.\nHas 5 characters only if very narrow, e.g. \"1\" and comma - typically 2-3 chars."
    			},
    			kind: {
    				title: "kind",
    				descr: "If the row has a kind selection (the main subtype), definition of the subtypes."
    			},
    			label: {
    				title: "label",
    				descr: "Label for the row type"
    			},
    			moreInfo: {
    				title: "more info",
    				descr: "More info HTML"
    			},
    			name: {
    				title: "name",
    				descr: "Name / unique identifier of the row type."
    			},
    			order: {
    				title: "order",
    				descr: "Order number in the lists"
    			},
    			price: {
    				title: "price",
    				descr: "Price input"
    			},
    			total: {
    				title: "total",
    				descr: "The total input or read-only field."
    			}
    		},
    		CalcTax: {
    			title: "calc tax",
    			descr: "Tax card information stored as part of the Calculation object",
    			estimatedTax: {
    				title: "estimated tax",
    				descr: "Calculated widthholding tax. Note that this is an estimation until the calculation is actually paid.\nAlso, if the salary is not paid as cash (e.g. fringe benefits), there may not be enough money to make the full deduction."
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Possibility to set a fixed widthholding tax amount for this calculation regardless of\nthe percentage etc. in the tax card."
    			},
    			hasNewSharedCard: {
    				title: "has new shared card",
    				descr: "A flag that during last recalculation, there was a new shared taxcard that was waiting approval."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "The API returns true if there is a valid taxcard for the PaidAt date for this calculation\nor if FixedTaxAmount is set for th calculation."
    			},
    			snapshot: {
    				title: "snapshot",
    				descr: "Snapshot of the tax card for salary slip etc. purposes.\nUpdated in Recalculate / Save based on TaxCardId"
    			},
    			taxPercentForVisualization: {
    				title: "tax percent for visualization",
    				descr: "Tax percent that is used in visualization. This is mainly used in Unit testing and anonymous calcultors.\nNot used in real paid calculations. This is percent. I.e. for 50% set 50, not 0.5.\nIf PaymentData or AccountId lead to a valid tax card type and percent, or if there is FixedTaxAmount, this value is not used."
    			},
    			taxcardId: {
    				title: "taxcard id",
    				descr: "Identifier for a stored tax card if one is applied to this calculation.\nSystem updates the TaxCardId automatically in Recalculate / Save if it finds a better candidate\nand also updates the Snapshot."
    			}
    		},
    		CalcWorker: {
    			title: "calc worker",
    			descr: "Worker object for the current calculation",
    			accountId: {
    				title: "account id",
    				descr: "Palkkaus.fi Account Id for the worker.\nAlternatively you can set the Payment data with the required infromation"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image and properties of the Worker.\nBased on either the AccountId or PaymentData"
    			},
    			dateOfBirth: {
    				title: "date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment Id for the worker. This is currently resolved using the workers AccountId."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the currently Account\nIf set to true, will set current Account as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			},
    			paymentData: {
    				title: "payment data",
    				descr: "Data that is needed for salary payment.\nThis object may be null if AccountId has been set.\nAlso, if AccountId has been set, these properties will be ignored:\nThe Worker will decide on payment properties herself."
    			},
    			tax: {
    				title: "tax",
    				descr: "New Widthholding tax logic that is based on separately stored tax cards."
    			}
    		},
    		CalcWorkerPaymentData: {
    			title: "calc worker payment data",
    			descr: "The required information for paying the Salary to Worker.",
    			email: {
    				title: "email",
    				descr: "Contact e-mail address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name of the person."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "IBAN number for the Bank account"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name / Surname of the person."
    			},
    			socialSecurityNumber: {
    				title: "social security number",
    				descr: "Gets the Social security number the way the user set it in input.\nHOWEVER: The getter will change the string to upper invariant/trim and if the last character is \"*\", it will be replaced by calculated checksum."
    			},
    			socialSecurityNumberValid: {
    				title: "social security number valid",
    				descr: "Social security number if valid or null"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		CalcWorkflow: {
    			title: "calc workflow",
    			descr: "Defines the properties of salary calculation workflow - typically the workflow from draft to Payment.",
    			paidAt: {
    				title: "paid at",
    				descr: "Time when this calculation was paid (if it was paid) to Palkkaus service."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			salaryPaidAt: {
    				title: "salary paid at",
    				descr: "Time when the salary was paid by Palkkaus service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from Draft to PaymentSucceeded"
    			}
    		},
    		CalcWorktime: {
    			title: "calc worktime",
    			descr: "Worktime data that is stored as part of Salary calculation: work days, absences, holidays and holiday accrual. In the future may also contain relevant part of timesheet data (work hours).",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of days of non-paid absences.\nThese are deducted from the Monthly salary, not from Hourly salary at the moment."
    			},
    			accruals: {
    				title: "accruals",
    				descr: "Accruals for each calendar month in this calculation."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Parameters related to Holiday Bonus."
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period that is updated from employment at each recalculation (as opposed to holiday specification)."
    			},
    			holidaySpec: {
    				title: "holiday spec",
    				descr: "Specification for the holiday calculation"
    			},
    			hourly: {
    				title: "hourly",
    				descr: "Parameters related to Hourly or Performance based salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			leavesDailySalary: {
    				title: "leaves daily salary",
    				descr: "Price for a single holiday day"
    			},
    			leavesDays: {
    				title: "leaves days",
    				descr: "Number of paid Leave days that is paid in this salary calculation / for this period."
    			},
    			monthly: {
    				title: "monthly",
    				descr: "Parameters related to Monthly salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			workDays: {
    				title: "work days",
    				descr: "Describes the working days in the period.\nCurrently, uses the DaysCount, but later the full days array will be taken to use.\nThis includes the absence days (divider in AbsencesDaySalary)."
    			}
    		},
    		Calculation: {
    			title: "calculation",
    			descr: "Calculation is the most important model in the Palkkaus.fi API. We suggest you open the separately provided documentation and especially the class diagram to understand the whole model in detail.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer",
    				descr: "The employer object for this calculation.\nTypically, you do not need to set this before the actual payment process."
    			},
    			framework: {
    				title: "framework",
    				descr: "DEPRECIATED: Usecase will be replacing this functionality in a next major release.\nThe Framework Agreement (TES) parameters that affect the framework based side costs etc. of this calculation"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "Information about the work that is not directly related to calculating salary and expenses."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "The results of the calculation. In the API, this is a read-only object."
    			},
    			rows: {
    				title: "rows",
    				descr: "Set of rows that make up the salary calculation:\nExpenses, Benefits, deductions etc. Note that side costs are not rows."
    			},
    			salary: {
    				title: "salary",
    				descr: "The main salary element.\nAlternatively the salary can be set in the Rows object - this will be the only way to set multiple salary rows (not yet supported)."
    			},
    			sharing: {
    				title: "sharing",
    				descr: "Sharing of the calculation. This object cannot be modified directly,\nuse specialized methods in Calculations service."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing salary calculations.\nIn practice, a usecase is either a user interface or a microservice that provides a salary calculation\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of calculation.\nNOTE: This functionality will replace Framework property in a next major release."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker (employee) that has performed the work."
    			},
    			workflow: {
    				title: "workflow",
    				descr: "Information about the workflow and state of the calculation."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "Readonly workflow data, which is exposed to API."
    			},
    			worktime: {
    				title: "worktime",
    				descr: "Worktime data that is stored as part of Salary calculation:\nwork days, absences, holidays and holiday accrual.\nIn the future may also contain relevant part of timesheet data (work hours)."
    			}
    		},
    		CalculationData: {
    			title: "calculation data",
    			descr: "Calculation specific business data for index.",
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "Payroll id."
    			},
    			pensionPayment: {
    				title: "pension payment",
    				descr: "The total pension payment for this calculation."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "A flag indicating if the requested date for the SalaryDate has been set.\nIf the date has been set, the value of the flag is 'SALA'."
    			},
    			socialSecurityPayment: {
    				title: "social security payment",
    				descr: "The social security payment- paid as part of taxation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from the end-user point-of-view."
    			},
    			taxPayment: {
    				title: "tax payment",
    				descr: "The taxes that were deducted from the worker salary a paid as part of taxation."
    			},
    			usecaseUri: {
    				title: "usecase uri",
    				descr: "Use case uri."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "Work end date."
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Work start date."
    			},
    			workerPayment: {
    				title: "worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			}
    		},
    		CalculationListItem: {
    			title: "calculation list item",
    			descr: "Represents a single item in a list of calculations in the user interface. It contains all essential information of the calculation and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		CalculationResult: {
    			title: "calculation result",
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			irRows: {
    				title: "ir rows",
    				descr: "Rows that are compatible with Incomes Register (tulorekisteri) process.\nThese are available only for calculations paid after 7.2.2019 (TODO: estimation as of writing)."
    			},
    			responsibilities: {
    				title: "responsibilities",
    				descr: "Responsibilities of Employer and parameters that affect those responsibilities."
    			},
    			rows: {
    				title: "rows",
    				descr: "The result rows. Note that this collection also contains the main Salary entry as one of the rows."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			validation: {
    				title: "validation",
    				descr: "The validation result for the current calculation.\nPlease note that this is not serialized into the data storage."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		CalculationSettings: {
    			title: "calculation settings",
    			descr: "Asetukset, jotka vaikuttavat palkanmaksuun",
    			household: {
    				title: "household",
    				descr: "Settings that are specific to Households logic.\nThese do not apply to Companies and may be null for them"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Settings related to mandatory accident insurance."
    			},
    			marketing: {
    				title: "marketing",
    				descr: "Marketing settings, E.g. campaigns, customer groups etc."
    			},
    			pension: {
    				title: "pension",
    				descr: "Pension insurance related settings."
    			},
    			pensionContracts: {
    				title: "Työeläkevakuutukset"
    			},
    			incomesRegister: {
    				title: "Tulorekisteri",
    				descr: "Ilmoittaminen tulorekisteriin"
    			},
    			insuranceContracts: {
    				title: "Tapaturmavakuutukset"
    			},
    			tax: {
    				title: "tax",
    				descr: "Settings related to taxation. Mainly widthholding tax, but potentially other taxes as well."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		CalculationSharing: {
    			title: "calculation sharing",
    			descr: "Defines the properties related to sharing and sending of the calculation.",
    			type: {
    				title: "type",
    				descr: "Type of sharing for this calculation."
    			},
    			uri: {
    				title: "uri",
    				descr: "Sharing uri."
    			}
    		},
    		CalculationSummary: {
    			title: "calculation summary",
    			descr: "Summarises from a large calculation object the relevant data for yearend feedback.",
    			benefitsInKindWithoutPhone: {
    				title: "benefits in kind without phone",
    				descr: "Benefits excluding phone benefit: Accomodation, Meals and other"
    			},
    			calculationId: {
    				title: "calculation id",
    				descr: "Identifier of the calculation or correction."
    			},
    			compensations: {
    				title: "compensations",
    				descr: "Compensations is daily allowance (including half days) plus meals compensation."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Total gross salary"
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes."
    			},
    			message: {
    				title: "message",
    				descr: "Message related to this row. Mainly for backoffice purposes."
    			},
    			milageCompensation: {
    				title: "milage compensation",
    				descr: "Milage compensation includes MilageOwnCar, Daily milage as well as other milage"
    			},
    			originalIsDeductible: {
    				title: "original is deductible",
    				descr: "If true the original calculation was marked as deductible (household deduction)."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "Date when the calculation was paid or where the correction should be set."
    			},
    			phoneBenefit: {
    				title: "phone benefit",
    				descr: "Phone benefit in euros"
    			},
    			salary: {
    				title: "salary",
    				descr: "Taxable salary of the calculation"
    			},
    			salaryType: {
    				title: "salary type",
    				descr: "Type of salary from Pension and Taxation point-of-view"
    			},
    			selectedIsDeductible: {
    				title: "selected is deductible",
    				descr: "If true the the calculation is currently marked as deductible (household deduction).\nDefault value is OriginalIsDeductible."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "The mandatory side costs for taxation purposes"
    			},
    			socialInsuranceDeductionsFromEmployee: {
    				title: "social insurance deductions from employee",
    				descr: "Pension and unemployment insurance deductions from the Worker"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "Categories for household deduction.\nUI implementors should force this enumeration being a single value although historically flags were supported."
    			},
    			type: {
    				title: "type",
    				descr: "Type of Calculation summary"
    			},
    			withholdingTax: {
    				title: "withholding tax",
    				descr: "Widthholding tax deducted from the Worker"
    			}
    		},
    		CalculationsDataRow: {
    			title: "calculations data row",
    			descr: "Data row for calculations data. This is typically used for imports and exports.",
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Tax if calculated"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the calculation"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End date of the work"
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start date of the work"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID for the worker."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			status: {
    				title: "status",
    				descr: "Read only calculation status"
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			}
    		},
    		CarBenefit: {
    			title: "car benefit",
    			ageGroupCode: {
    				title: "age group code"
    			},
    			carBenefitCode: {
    				title: "car benefit code"
    			},
    			kilometers: {
    				title: "kilometers"
    			}
    		},
    		CarBenefitUsecase: {
    			title: "car benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.CarBenefitUsecase to IrRows",
    			ageGroup: {
    				title: "age group",
    				descr: "Age group as in A,B,C or I for international."
    			},
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isKilometersBased: {
    				title: "is kilometers based",
    				descr: "If true, the calculation is kilometers based."
    			},
    			kilometers: {
    				title: "kilometers",
    				descr: "Number of kilometers is filled in only if that type of benefit calculation is used."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Certificate: {
    			title: {
    				title: "title",
    				descr: "End user description for the certificate."
    			},
    			descr: "User certificate.",
    			bytes: {
    				title: "bytes",
    				descr: "Certificate bytes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			password: {
    				title: "password",
    				descr: "Password for issuing a new certificate. Not persisted."
    			},
    			thumbprint: {
    				title: "thumbprint",
    				descr: "Certificate thumbprint."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		City: {
    			title: "city",
    			count: {
    				title: "count"
    			},
    			grouping: {
    				title: "grouping"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			nameLang2: {
    				title: "name lang2"
    			}
    		},
    		ClassDoc: {
    			title: "class doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			extendedBy: {
    				title: "extended by",
    				descr: "Types that extend this type (extendedBy)"
    			},
    			"extends": {
    				title: "extends",
    				descr: "Collection of reference types that this class extends (extendedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			implementedBy: {
    				title: "implemented by",
    				descr: "Types that iplement this interface (implementedBy)"
    			},
    			"implements": {
    				title: "implements",
    				descr: "Collection of interfaces that this class implements (implementedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			members: {
    				title: "members",
    				descr: "Public properties, fields and functions."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			usages: {
    				title: "usages",
    				descr: "Salaxy specific logic may add usages here.\nCurrently, components will add themselves to controllers."
    			}
    		},
    		ClassMemberDoc: {
    			title: "class member doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Display grouping for the member: E.g. accessors are considered properties here."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			parameters: {
    				title: "parameters",
    				descr: "For grouping type \"Method\", lists parameters\nTODO: Consider making a separate type for properties."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the member."
    			}
    		},
    		CompanyAccount: {
    			title: "company account",
    			descr: "A company account - note that this can also be an association etc. - something that has a Company ID.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			companyType: {
    				title: "company type",
    				descr: "High level company type"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Majority shareholders (over 25% stake) are stored in this field.\nOwnership is a list of Shareholder objects."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		CompanyAccountSettings: {
    			calc: {
    				title: "Palkanmaksun asetukset",
    				descr: "Asetukset, jotka vaikuttavat palkanmaksuun"
    			},
    			partner: {
    				title: "Kumppaniasetukset"
    			},
    			accounting: {
    				title: "Kirjanpidon asetukset"
    			},
    			payments: {
    				title: "Maksamisen asetukset"
    			},
    			reporting: {
    				title: "Raportoinnin asetukset"
    			}
    		},
    		CompanyInput: {
    			title: "company input",
    			descr: "Represents the input for creating a company account.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the company"
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a company account based on company data."
    			},
    			doYtjUpdate: {
    				title: "do ytj update",
    				descr: "If true, will fetch the data from the Finnish Business Infromation System (YTJ/BIS).\nFetching is done firstly based on Business ID, if that is not valid then based on name."
    			},
    			logo: {
    				title: "logo",
    				descr: "URL to a Logo picture (preferably PNG). This logo should be a square - icon type of logo for avatar."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Company ownership for KYC"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "Identifier at the partner site - typicelly an organization ID."
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given BusinessID exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			},
    			ytjDetails: {
    				title: "ytj details",
    				descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)"
    			},
    			ytjSearch: {
    				title: "ytj search",
    				descr: "A search to Finnish Business Infromation System (YTJ/BIS). This search is triggered if you set the RawInput field."
    			}
    		},
    		ComponentDoc: {
    			title: "component doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attributes: {
    				title: "attributes",
    				descr: "HTML attributes available to specify options to the component."
    			},
    			controller: {
    				title: "controller",
    				descr: "Name of the controller that implements the functionality of the component"
    			},
    			defaultTemplate: {
    				title: "default template",
    				descr: "The default template (view) used in the rendering of the component."
    			},
    			elem: {
    				title: "elem",
    				descr: "The component tag / element in HTML"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			require: {
    				title: "require",
    				descr: "Required components in the outside scope"
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			transclude: {
    				title: "transclude",
    				descr: "Transclude slots"
    			}
    		},
    		Config: {
    			title: "config",
    			descr: "Environment specific configuration for Salaxy API's and JavaScript in general",
    			apiServer: {
    				title: "api server",
    				descr: "The base address of the Salaxy http API server, e.g. https://secure.salaxy.com"
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "A flag indicating if the current configuration is for test environment only. If true, the site is for testing only."
    			},
    			useCookie: {
    				title: "use cookie",
    				descr: "A flag indicating if the site sets a Salaxy cookie which persists over web sessions. If true, the cookie will be created."
    			},
    			wwwServer: {
    				title: "www server",
    				descr: "The base address of the Palkkaus web site, e.g. https://www.palkkaus.fi"
    			}
    		},
    		Contact: {
    			title: "contact",
    			descr: "Common contact information object for Person, Company, Location etc.",
    			city: {
    				title: "city",
    				descr: "City or community (postitoimipaikka)"
    			},
    			countryCode: {
    				title: "country code",
    				descr: "This is the ISO code for country.\nCurrently, it should always be \"fi\"."
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail address. Always lower invariant, trimmed and empty string set to null."
    			},
    			postalCode: {
    				title: "postal code",
    				descr: "Postal code"
    			},
    			street: {
    				title: "street",
    				descr: "Street address"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		ContractParty: {
    			title: "contract party",
    			descr: "A signing party in a contract.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the Party.\nNote that this is not automatically fetched by all methods: Especially when IsSelf is true."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information - note that for most signatures, only E-mail and phone number are required."
    			},
    			contractPartyType: {
    				title: "contract party type",
    				descr: "Party type (Person or Prospect)"
    			},
    			iban: {
    				title: "iban",
    				descr: "Bank account IBAN number"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the party: Currently always an account, but may be an external ID later."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true, contact is an employer"
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nIf set to false AND was previously true, will set the Profile properties null (unless explicitly set to something else)"
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "True if the person has signed the contract."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official ID if needed in signature"
    			},
    			role: {
    				title: "role",
    				descr: "Role of the person - justification for authorization"
    			},
    			signatureUrl: {
    				title: "signature url",
    				descr: "Reserved for cases where there are several signees and each has a separate signature document."
    			}
    		},
    		ContractSalary: {
    			title: "contract salary",
    			descr: "Part of the Employment contract that is relevant to paying the salary",
    			collectiveContract: {
    				title: "collective contract"
    			},
    			endDate: {
    				title: "end date",
    				descr: "End date for the employment.\nOnly set if the contract is Fixed term."
    			},
    			payDay: {
    				title: "pay day"
    			},
    			salaryAmount: {
    				title: "salary amount"
    			},
    			salaryCompension: {
    				title: "salary compension"
    			},
    			salaryType: {
    				title: "salary type"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Starting date for the employment.\nShould always be set for a valid contract."
    			},
    			workHours: {
    				title: "work hours"
    			},
    			workType: {
    				title: "work type"
    			},
    			worksHoursType: {
    				title: "works hours type"
    			}
    		},
    		CoreTypeDoc: {
    			title: "core type doc",
    			descr: "H",
    			classes: {
    				title: "classes",
    				descr: "H"
    			},
    			enums: {
    				title: "enums",
    				descr: "H"
    			}
    		},
    		CustomerFundsSettings: {
    			title: "customer funds settings",
    			descr: "Settings related to payment methods that use the customer funds account (gross payment).",
    			isPensionSelfHandling: {
    				title: "Työnantaja hoitaa itse TYEL-maksut",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "Työnantaja hoitaa itse ennakonpidätyksen maksun",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			isWorkerSelfHandling: {
    				title: "Työnantaja hoitaa itse nettopalkan maksun",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			}
    		},
    		DailyAllowanceUsecase: {
    			title: "daily allowance usecase",
    			descr: "Handles the Calculation of CalculationRowType.DailyAllowance to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		DataRow: {
    			title: "data row",
    			descr: "Interface for data row for exports and imports",
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		DateRange: {
    			title: "date range",
    			descr: "Describes the range of dates from start to end and the number of days in between.",
    			days: {
    				title: "days",
    				descr: "The collection of days contained in the DateRange if specified by the user.\nAlternative to DaysCount. If set, should also set DaysCount for backward compatibility."
    			},
    			daysCount: {
    				title: "days count",
    				descr: "Number of days in the range.\nDepending of the context, this may be mathematical (end - start) or\nexplicitly specified (typically working days).\nTypically an integer, and calculations and UI's may choose to round it."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			}
    		},
    		DeclarationReflection: {
    			title: "declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			}
    		},
    		EInvoiceAddress: {
    			title: "e invoice address",
    			descr: "Result item of the EInvoice address query.",
    			intermediator: {
    				title: "intermediator",
    				descr: "Intermediator id for this receiver address."
    			},
    			intermediatorName: {
    				title: "intermediator name",
    				descr: "Intermediator name for this receiver address."
    			},
    			receiver: {
    				title: "receiver",
    				descr: "Receiver EInvoice address of company."
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Name for the company."
    			},
    			receiverOfficialId: {
    				title: "receiver official id",
    				descr: "Official id of the company."
    			}
    		},
    		ESalaryPayment: {
    			title: "e salary payment",
    			descr: "E-salary / E-palkka is a way for a partner to direct an Employer to Salary payment without having access to Employer account. In first instance, E-salary is targeting Household Employers only.",
    			calc: {
    				title: "calc",
    				descr: "Calculation that should be paid by the Employer."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			household: {
    				title: "household",
    				descr: "Known data for the employer household that is paying the salary.\nIt is understood that the caller may not have all the information:\nPalkkaus.fi will ask the rest if necessary."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			request: {
    				title: "request",
    				descr: "Payment request and how to return to the calling service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the Payment.\nNOTE: You can only set states Created, Shared and Opened.\nOther states are set by the Salaxy service and cannot be modified throgh API."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides the validation of the ESalaryPayment object after the object has been saved."
    			}
    		},
    		EmailMessage: {
    			title: "email message",
    			descr: "API EmailMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (email address)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			subject: {
    				title: "subject",
    				descr: "Message subject"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (email address)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		EmployerCalculationDTO: {
    			title: "employer calculation d t o",
    			descr: "Models the business logic of Employer side of the calculation. For documentation, see the IEmployerCalculation interface.",
    			allSideCosts: {
    				title: "all side costs",
    				descr: "All side cost: The mandatory side costs + Palkkaus and partner fees"
    			},
    			deductionForeclosure: {
    				title: "deduction foreclosure",
    				descr: "Foreclosure that has already been paid by the employer."
    			},
    			deductionOtherDeductions: {
    				title: "deduction other deductions",
    				descr: "Other deductions that are deducted from the net salary."
    			},
    			deductionPensionSelfPayment: {
    				title: "deduction pension self payment",
    				descr: "Pension payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionSalaryAdvance: {
    				title: "deduction salary advance",
    				descr: "Salary advance - this part has already been paid."
    			},
    			deductionTaxAndSocialSecuritySelfPayment: {
    				title: "deduction tax and social security self payment",
    				descr: "Tax and social security payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnemploymentSelfPayment: {
    				title: "deduction unemployment self payment",
    				descr: "Unemployment insurance (TVR) payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnionPayment: {
    				title: "deduction union payment",
    				descr: "Union payment if it is deducted from the final figure"
    			},
    			deductionWorkerSelfPayment: {
    				title: "deduction worker self payment",
    				descr: "Worker salary and expense payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			finalCost: {
    				title: "final cost",
    				descr: "The final cost after household decuction from the tax authorities:\nAdd to the payment the deductions (paid by employer) and deduct House hold deduction (later potentially other such subsidies)"
    			},
    			foreclosureByPalkkaus: {
    				title: "foreclosure by palkkaus",
    				descr: "Foreclosure that Palkkaus.fi pays in behalf of employer."
    			},
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction that the tax man will reimburse in taxation.\nThis is an estimation: There is minimum and maximum limits etc."
    			},
    			mandatorySideCosts: {
    				title: "mandatory side costs",
    				descr: "The mandatory side costs are Pension (TyEL), Social secuirty and Unemployment insurance."
    			},
    			palkkaus: {
    				title: "palkkaus",
    				descr: "The Palkkaus.fi fee including potential partner fees."
    			},
    			pension: {
    				title: "pension",
    				descr: "The part of the Pension that Employer pays"
    			},
    			socialSecurity: {
    				title: "social security",
    				descr: "The social security payment- paid as part of taxation (ennakonpidätys)."
    			},
    			totalDeductions: {
    				title: "total deductions",
    				descr: "Total deductions that are made from the payment that is paid to Palkkaus and then to Worker"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment that is Paid to Palkkaus.fi"
    			},
    			totalPaymentLegacy: {
    				title: "total payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			totalSalaryCost: {
    				title: "total salary cost",
    				descr: "Total salary cost is the paid salary, deductions and side costs, but not expenses."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The part of the unemployment insurance that the employer pays"
    			}
    		},
    		EmployerGroups: {
    			title: "employer groups",
    			descr: "Employer groups that are set on Example calculations. These will be replaced by properties from Worker / Employer",
    			campaignPricing: {
    				title: "campaign pricing",
    				descr: "Campaign pricing may affect Palkkaus fees.\nThis field is set by the system when coming to the payment based on Employer properties (usually campaign code)."
    			},
    			employerType: {
    				title: "employer type",
    				descr: "Very small employers (households) do not required to report to the tax authorities."
    			},
    			isPrivatePerson: {
    				title: "is private person",
    				descr: "If true, the Employer is a private person.\nOtherwise Company or Unknown."
    			},
    			salaryPerMonth: {
    				title: "salary per month",
    				descr: "Very small salaries do not require Pension payments."
    			},
    			salaryPerYear: {
    				title: "salary per year",
    				descr: "Ranges for salary / year will affect the side costs."
    			}
    		},
    		EmployerInfo: {
    			title: "employer info",
    			descr: "Employer insurance information.",
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "The id of the insurance company."
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Insurance contract number."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "Indicates if the employer has an insurance."
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "Indicates if the insurance is provided by Palkkaus partnership insurance company."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "Indicates if the employer has a pension contract."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the mployer handles the pension himself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The id of the pension company."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Pension contract number."
    			}
    		},
    		EmploymentContract: {
    			title: "employment contract",
    			annualLeave: {
    				title: "annual leave"
    			},
    			benefitsException: {
    				title: "benefits exception"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isWorkAtHome: {
    				title: "is work at home"
    			},
    			mainWorkTasks: {
    				title: "main work tasks"
    			},
    			noticeExeption: {
    				title: "notice exeption"
    			},
    			otherPayDescription: {
    				title: "other pay description"
    			},
    			otherTerms: {
    				title: "other terms"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partTimeReason: {
    				title: "part time reason"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			salary: {
    				title: "salary"
    			},
    			testPeriod: {
    				title: "test period"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workPlace: {
    				title: "work place"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		EmploymentData: {
    			title: "employment data",
    			descr: "Employment specific business data for index.",
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			}
    		},
    		EmploymentListItem: {
    			title: "employment list item",
    			descr: "Represents a single item in a list of employments in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		EmploymentRelation: {
    			title: "employment relation",
    			descr: "Represents the Employment Relation: It is basically the properties of a Worker from the point-of-view of employer and as data owned by Employer whereas WorkerAccount is preliminarily owned by Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General information about the Employment relation."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		EmploymentRelationInfo: {
    			title: "employment relation info",
    			descr: "General information about the Employment relation.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of this employment relationship"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			workerAccountId: {
    				title: "worker account id",
    				descr: "The Account ID of the Worker if Worker"
    			}
    		},
    		EmploymentTerminationUsecase: {
    			title: "employment termination usecase",
    			descr: "Handles the Calculation of CalculationRowType.EmploymentTermination to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		EnumDoc: {
    			title: "enum doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			names: {
    				title: "names",
    				descr: "Member names"
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			values: {
    				title: "values",
    				descr: "Member values"
    			}
    		},
    		EnumMetadata: {
    			title: "enum metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Type name of the enumeration (UpperCamelCase)"
    			},
    			values: {
    				title: "values",
    				descr: "Values of the enumeration."
    			}
    		},
    		EnumValueMetadata: {
    			title: "enum value metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied.\nNOTE: Not all enumerations have description in the globalization files:\nThey are added only if necessary to the UI."
    			},
    			label: {
    				title: "label",
    				descr: "Label of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Text value of the enumeration (lowerCamelCase)"
    			},
    			order: {
    				title: "order",
    				descr: "Sorting order number of the value within the enum."
    			}
    		},
    		FileData: {
    			title: "file data",
    			descr: "Report specific business data for index.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "Number of calculations in the report."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id for the report."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "Number of workers in the report."
    			}
    		},
    		FileListItem: {
    			title: "file list item",
    			descr: "Represents a single item in a list of reports in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		FileParameter: {
    			title: "file parameter",
    			data: {
    				title: "data"
    			},
    			fileName: {
    				title: "file name"
    			}
    		},
    		FileType: {
    			title: "file type",
    			extension: {
    				title: "extension"
    			},
    			isImage: {
    				title: "is image"
    			},
    			isText: {
    				title: "is text"
    			},
    			mimeType: {
    				title: "mime type"
    			}
    		},
    		HealthCareHelttiProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer health care from the company Heltti",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by HelttiProductPackage."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			industry: {
    				title: "industry",
    				descr: "Industry of work"
    			},
    			"package": {
    				title: "package",
    				descr: "Defines the product package"
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		HolidayAccrual: {
    			title: "holiday accrual",
    			descr: "Yearly log for Holiday accrual.",
    			defaultAccrual: {
    				title: "default accrual",
    				descr: "Default accrual for months where the Accrual occurs.\nWhen Holiday year is recalculated, this is determined either from Employment Period or AccrualFixed if set."
    			},
    			endSaldo: {
    				title: "end saldo",
    				descr: "End saldo from the previous year. This is Total + Start Saldo."
    			},
    			months: {
    				title: "months",
    				descr: "Holiday accrual entries during the period."
    			},
    			period: {
    				title: "period",
    				descr: "The period for holiday accrual.\nThis is also the period of the holiday year that the Worker is Employed,\ni.e. it may be shorter than HolidayYear Period if the employment relation\nbegins or ends within the holiday year."
    			},
    			startSaldo: {
    				title: "start saldo",
    				descr: "Start saldo from the previous year."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the current year. This does not include Start saldo."
    			}
    		},
    		HolidayAccrualEntry: {
    			title: "holiday accrual entry",
    			descr: "Entry in the Yearly Holiday Accrual list.",
    			calcId: {
    				title: "calc id",
    				descr: "Identier of the calculation if Kind"
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Days added to holiday Accrual: Typically days per month."
    			},
    			index: {
    				title: "index",
    				descr: "Index of the month from 0-11 for easier iteration.\nThis is also the real identifier of the month when editing the entries through API.\nMonths 4-12 are index 0-8 and month 1-3 are index 9-11"
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			source: {
    				title: "source",
    				descr: "Source of the holiday accrual."
    			}
    		},
    		HolidayDate: {
    			title: "holiday date",
    			date: {
    				title: "date"
    			},
    			holiday: {
    				title: "holiday"
    			},
    			holidayGroups: {
    				title: "holiday groups"
    			},
    			localName: {
    				title: "local name"
    			},
    			name: {
    				title: "name"
    			}
    		},
    		HolidaySpecification: {
    			title: "holiday specification",
    			descr: "Specifies how the holidays are handled for this Worker.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			}
    		},
    		HolidaySpecificationForYear: {
    			title: "holiday specification for year",
    			descr: "Extends the Holiday specification with the properties of employment relation that are required for a specific year.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period for the purposes of holiday calculation (annual leave accrual).\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Wage basis for a Holiday Year.\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year.\nNOTE: This is not in HolidaySpecification because, the same value is stored in Employment relation."
    			}
    		},
    		HolidayYear: {
    			title: "holiday year",
    			descr: "Holiday year is in Finland from April to end of march. This is the time from which the holidays for the following summer and year are determined.",
    			accrual: {
    				title: "accrual",
    				descr: "Accrual of holidays (lomakirjanpito)."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation for the worker whose holidays are managed."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			leaves: {
    				title: "leaves",
    				descr: "Payments related to Annual Leaves (holidays)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			period: {
    				title: "period",
    				descr: "The holiday period for the year: Should always be 2.5.[Year]-30.4.[Year+1]"
    			},
    			spec: {
    				title: "spec",
    				descr: "Specifies how the holidays are handled.\nIf the Spec.Code is Undefined in Save, the contents is fetched from the current employment relation."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Salaxy ID of the worker. This value is required when inserting and cannot be edited later.\nWorkerSnapshot is automatically updated upon save."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			},
    			year: {
    				title: "year",
    				descr: "Year as integer. This value is required when inserting and cannot be edited later.\nPeriod will be set accordingly, or adjusted in case it is set below."
    			}
    		},
    		Holidays: {
    			title: "holidays",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		Holidays2: {
    			title: "holidays2",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		HouseholdSettings: {
    			title: "household settings",
    			descr: "Settings that are specific to Households. These do not apply to Companies.",
    			deductionInsuranceAmount: {
    				title: "deduction insurance amount",
    				descr: "Accident insurance amount reported for Household deduction purposes"
    			},
    			deductionSpouseName: {
    				title: "deduction spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			deductionSpousePersonalId: {
    				title: "deduction spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		HouseholdYearlyFeedback: {
    			title: "household yearly feedback",
    			descr: "Yearly feedback data that is specific to household customers. Main part is the information realted to household deduction and how it can be divided between spouses.",
    			insuranceAmount: {
    				title: "insurance amount",
    				descr: "Amount of accident insurance paid during this year as reported by the user\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			insuranceAmountFromAccount: {
    				title: "insurance amount from account",
    				descr: "Amount of accident insurance paid during this year from parner insurance (in Palkkaus.fi records).\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			spouseName: {
    				title: "spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			spousePersonalId: {
    				title: "spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		IEmployerResponsibilities: {
    			title: "i employer responsibilities",
    			descr: "Parameters that affect the calculation. EmployerResponsibilities is historical name - currentlty to object holds also other properties that are not well described as responsibilities. These based on the EmployerGroups selection or properties of the Employer and Worker snapshots as well as this Calculation",
    			calculationPaymentDate: {
    				title: "calculation payment date",
    				descr: "The assumed salary payment date from the worker perspective.\nThe system calculates the nearest possible salary date if no any salary date has been given by the\nemployer. If given, the system confirms the given date.\nThis date affects the yearly percents used in the salary calculation."
    			},
    			employerGroups: {
    				title: "employer groups",
    				descr: "Employer groups that are set on Example calculations.\nThese will be replaced by properties from Worker / Employer when these persons are set."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isPensionMonthlyContract: {
    				title: "is pension monthly contract",
    				descr: "If true, the employer has a Pension contract and makes the payments\nin a monthly schedule which means that Pension interest is calculated for the 20th of the next month.\nCurrently, this is only enabled to Etera - other companies will follow."
    			},
    			noAccidentInsurance: {
    				title: "no accident insurance",
    				descr: "If true, this employer is not required to get an accident insurance."
    			},
    			noPensionPayment: {
    				title: "no pension payment",
    				descr: "Pension is not paid if the salary to this Worker is less than a certain amount in a month (about 60€)\nor worker is less than 17 or more than 64 years old."
    			},
    			noSocialSecurity: {
    				title: "no social security",
    				descr: "If worker is less than 16 or more than 67, social secuiry payment is not paid"
    			},
    			noTaxResponsibility: {
    				title: "no tax responsibility",
    				descr: "If Employer pays to this Worker max 200 euro, he does not need to even make the reports to tax office.\nHe may do so though"
    			},
    			noTvr: {
    				title: "no tvr",
    				descr: "If Worker is less than 17 or more than 64, TVR is not paid.\nAlso if the Employer has no accident insurance."
    			},
    			noWithholdingTax: {
    				title: "no withholding tax",
    				descr: "If Employer pays to this Worker max 1500 euro, he does not need to deduct the witholding tax.\nHe may do so though"
    			},
    			pensionAgeRange: {
    				title: "pension age range",
    				descr: "The age range that is used in the business logic that determines the employer responsibilities.\nThis property replaces the EmployerGroups.AgeRange that was previous get/set.\nThis is based on WorkerDateOfBirth and CalculationPaymentDate."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Pension calculation type."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The name of the Pension company because this affects the Pension percentage used in the calculation (only in contract situation).\nIt also needs to be shown in the reports."
    			},
    			pensionOldAgePayment: {
    				title: "pension old age payment",
    				descr: "If the Worker is from 53 to 62 years old, more of the Pension payment\nis deducted from the Worker side of the salary (less for the Employer).\nNOTE: People from 63 to 67 years pay the same amount as 17-52 years old."
    			},
    			pensionRegularEmployer: {
    				title: "pension regular employer",
    				descr: "The employer should have a Pension contract (not a temporary employer).\nIt does not necessarily mean that the user has a contract yet.\nNOTE: At the moment, this is not used for calculation - only in the Side costs visualizer tool."
    			},
    			taxAndSocialSecuritySelfPayment: {
    				title: "tax and social security self payment",
    				descr: "Tax and social security payments are paid directly by the Employer - not by Palkkaus.fi"
    			},
    			unemploymentSelfPayment: {
    				title: "unemployment self payment",
    				descr: "If true, the employer pays the Unemployment insurance directly to TVR.\nAs of writing, this is true for all companies and with calculations starting 1.8.2018,\nfalse for households or calculations before the date."
    			},
    			workerDateOfBirth: {
    				title: "worker date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerDateOfBirthAccuracy: {
    				title: "worker date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerSelfPayment: {
    				title: "worker self payment",
    				descr: "Worker salary and expense payments are paid directly by the Employer - not by Palkkaus.fi"
    			}
    		},
    		IncomesRegisterSettings: {
    			irReporting: {
    				title: "Kuukausittainen ilmoittaminen",
    				editLabel: "Kuukausittainen ilmoittaminen"
    			},
    			taxReferenceNumber: {
    				title: "Verorekisterinumero"
    			}
    		},
    		InsuranceContract: {
    			company: {
    				title: "Yhtiö",
    				editLabel: "Valitse yhtiö"
    			},
    			id: {
    				title: "Tunniste"
    			},
    			status: {
    				title: "Tila"
    			},
    			contractNumber: {
    				title: "Sopimusnumero"
    			},
    			startDate: {
    				title: "Alkamispäivä"
    			},
    			endDate: {
    				title: "Päättymispäivä"
    			},
    			isPartnerInsurance: {
    				title: "Yhteistyökumppani"
    			}
    		},
    		IUserObjectIndex: {
    			title: "i user object index",
    			descr: "Generic index for all User Object Containers.",
    			additionalInfo: {
    				title: "additional info",
    				descr: "A short detail describing the transaction.\nShould contain the relevant information for administration purposes."
    			},
    			businessAttributes: {
    				title: "business attributes",
    				descr: "Business attributes to include further information of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			containerGuid: {
    				title: "container guid",
    				descr: "Guid of the container.\nThis is not always unique within the index.\nHowver, within the container this is unique."
    			},
    			containerType: {
    				title: "container type",
    				descr: "The container type that is the source of this index item."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "Created at time"
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id for the object in 3rd party system. E.g. Filing code in MoneyTransfer."
    			},
    			flags: {
    				title: "flags",
    				descr: "Classifications for the object. First implementation supports BusinessEventType enumeration for values."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			handledAt: {
    				title: "handled at",
    				descr: "This is the recorded handling date for the object. E.g. when the calculation was paid to worker."
    			},
    			icon: {
    				title: "icon",
    				descr: "Icon for the purposes of listings."
    			},
    			id: {
    				title: "id",
    				descr: "Unique key for the object. Unique within an Index.\nIn the case of UserObject, this is [Guid]_[ContainerType]_[Owner].\nFor non-user objects, it should typically be just a Guid."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Messages (Error, Warning, OnHold, Note) that are active in the system.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the object is also the Partition key.\n Mandatory in most storage scenarios."
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Latest partner information."
    			},
    			payloadType: {
    				title: "payload type",
    				descr: "The business object type that is the source of this event."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			reference: {
    				title: "reference",
    				descr: "Reference information. E.g. Payment or MoneyTransfer reference number."
    			},
    			revenue: {
    				title: "revenue",
    				descr: "Estimated revenue that the transaction generated to Palkkaus.fi. Only add here the revenue, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Updated at time: Note that in typical implementations, this is the UserUpdatedAt time."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		IVerifiedAccount: {
    			title: "i verified account",
    			descr: "An account that is verified. Typically this means a digitally signed contract. This means that the account also has an Identity object. However, as the Identity object contains confidential information, it is not necessarily present in all method calls.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Pricing partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated"
    			}
    		},
    		Identity: {
    			title: "identity",
    			descr: "Identity is the official identity of an actor. This part is not freely editable as changing it requires signature or some other means. Identity is also often fetched from external sources such as govenment registries.",
    			contract: {
    				title: "contract",
    				descr: "The latest authorization contract that is the bases for this Identity"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name OR company name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			isPep: {
    				title: "is pep",
    				descr: "Indicates whether the principal user of the customer is a politically exposed person or not."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name\nAvailable only to real persons (as opposed to companies)."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the actor:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)"
    			},
    			primaryContact: {
    				title: "primary contact",
    				descr: "Mainly for company accounts, but in special cases for Persons,\nthis is the main Contact party and usually the signer for the account."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			}
    		},
    		ImportBatchInfo: {
    			title: "import batch info",
    			descr: "Contains the basic details and the current status of the batch phase.\nTypically this data is created and updated during the import session.",
    			createdAt: {
    				title: "created at",
    				descr: "Date and time when the batch was created."
    			},
    			id: {
    				title: "id",
    				descr: "Unique id for the batch."
    			},
    			name: {
    				title: "name",
    				descr: "User friendly name for the batch."
    			},
    			phase: {
    				title: "phase",
    				descr: "Current phase of the batch:\n - ```new``` A new batch. Contains no data.\n - ```read``` The data file (e.g. csv) has been read from file or clipboard.\n - ```mapped``` The data has been mapped to a supported tabular format.\n - ```validated``` The data has been validated.\n - ```imported``` The data has been imported to backend.\n - ```rejected``` The batch has been rejected manually."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of the current phase:\n - ```info``` Typically the default status for the phase which has not yet been done.\n - ```success``` The current phase has been successfully finished.\n - ```warning``` The current phase has been finished with warnings.\n - ```error``` The current phase contains errors."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Date and time when the batch was last updated."
    			}
    		},
    		ImportResult: {
    			title: "import result",
    			descr: "Result for importing data (CSV, Excel copy-paste etc.)",
    			data: {
    				title: "data",
    				descr: "Result data is an array of objects with properties specified as headers"
    			},
    			errors: {
    				title: "errors",
    				descr: "Errors as validation errors."
    			},
    			headers: {
    				title: "headers",
    				descr: "Headers for the import: These are also the properties for the JSON"
    			},
    			ignoredLines: {
    				title: "ignored lines",
    				descr: "Number of ignored lines (empty line or rem/annotated) in the import."
    			}
    		},
    		ImportableCrudApi: {
    			title: "importable crud api",
    			descr: "Interface for APIs which implement import/export functionality.",
    			constructor: {
    				title: "constructor",
    				descr: "Constructor creates a new CRUD api with given Ajax-implementation"
    			},
    			ajax: {
    				title: "ajax"
    			},
    			baseUrl: {
    				title: "base url",
    				descr: "Deriving classes should define the baseURl for the method. E.g. \"/calculations\"\nAjax implementations will add the beginning (e.g. \"https://test-api.salaxy.com/api/v02\")"
    			},
    			"delete": {
    				title: "delete",
    				descr: "Deletes an single item from the sotrage\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			deleteWorkflowEvent: {
    				title: "delete workflow event",
    				descr: "Deletes the worklflow item or items from the item.\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			getAll: {
    				title: "get all",
    				descr: "Gets all the items of a given type.\n\n**Returns**: A Promise with result data array.\n"
    			},
    			getBaseUrl: {
    				title: "get base url",
    				descr: "Returns the base url for this api."
    			},
    			getBlank: {
    				title: "get blank",
    				descr: "Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.\nThe basic idea is that all the child object properties should be non-null to avoid null reference exceptions.\nIn special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.\nStrings, dates and booleans may be null (false if bool), but typically just undefined."
    			},
    			getListItemMappers: {
    				title: "get list item mappers",
    				descr: "Available list item mappers for import/export"
    			},
    			getOData: {
    				title: "get o data",
    				descr: "Makes an OData query to the service's main OData method."
    			},
    			getODataBase: {
    				title: "get o data base",
    				descr: "Makes an OData query to the the given service URL."
    			},
    			getODataUrl: {
    				title: "get o data url",
    				descr: "Gets the OData URL for v03 compatible services"
    			},
    			getObjectMappers: {
    				title: "get object mappers",
    				descr: "Available object mappers for import/export"
    			},
    			getSingle: {
    				title: "get single",
    				descr: "Gets a single item based on identier\n\n**Returns**: A Promise with result data.\n"
    			},
    			getValidationResult: {
    				title: "get validation result",
    				descr: "Returns the validation object for the validated object."
    			},
    			save: {
    				title: "save",
    				descr: "Saves an item to the storage.\nIf id is null, this is add/insert. If id exists, this is update.\n\n**Returns**: A Promise with result data as saved to the storage (contains id, createAt, owner etc.).\n"
    			},
    			saveWorkflowEvent: {
    				title: "save workflow event",
    				descr: "Adds/updates the workflow event for the item.\n\n**Returns**: Workflow data for the item.\n"
    			},
    			validate: {
    				title: "validate",
    				descr: "Validates the item using a server side call.\n\n**Returns**: A Promise with result data containing validation property.\n"
    			}
    		},
    		IncomeTypeMetadata: {
    			title: "income type metadata",
    			descr: "Describes the properties and behavior of the Income types in Incomes register.",
    			accidentInsurance: {
    				title: "accident insurance",
    				descr: "If true Accident insurance payments are made for this income type by default."
    			},
    			calcGrouping: {
    				title: "calc grouping",
    				descr: "Grouping that is used in reporting etc."
    			},
    			code: {
    				title: "code",
    				descr: "Code in incomes register. Language versioned."
    			},
    			description: {
    				title: "description",
    				descr: "Description of the row.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			healthInsurance: {
    				title: "health insurance",
    				descr: "If true Health insurance payments are made for this income type by default."
    			},
    			insuranceInformationAllowed: {
    				title: "insurance information allowed",
    				descr: "If true the default values for insurances may be overriden for a single row."
    			},
    			isNegativeSupported: {
    				title: "is negative supported",
    				descr: "If true, teh row may have a negative value."
    			},
    			isSupported: {
    				title: "is supported",
    				descr: "If false, the row type is not supported in this version"
    			},
    			label: {
    				title: "label",
    				descr: "Language versioned label for the income type."
    			},
    			notSupportedError: {
    				title: "not supported error",
    				descr: "When isSupported is false, there is a describtive text that tells why the type is not supported. Text is always in Finnish."
    			},
    			paymentDefault: {
    				title: "payment default",
    				descr: "Default behavior for payment through Salaxy Customer Funds account"
    			},
    			pensionInsurance: {
    				title: "pension insurance",
    				descr: "If true Pension payments are made for this income type by default."
    			},
    			taxAndSidecostsDescr: {
    				title: "tax and sidecosts descr",
    				descr: "Description of how the row is taken into widthholding tax\nand how insurance payments (pension, unemployment, accident and health care) are made.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			taxDefault: {
    				title: "tax default",
    				descr: "Describes the behavior of this row in calculating widthholding tax."
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "If true Unemployment insurance payments are made for this income type by default."
    			}
    		},
    		InsuranceProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer insurance. Currently, this is the mandatory employer insurance only.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPartnerInsurance."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "Company for the current insurance"
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		InsuranceSettings: {
    			title: "insurance settings",
    			company: {
    				title: "company",
    				descr: "Company for the current insurance"
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the insurance pension"
    			}
    		},
    		Invoice: {
    			title: "invoice",
    			descr: "Invoice for the pending payment in the business system. For example, a salary gross payment created for the salary calculation and delivered as an e-invoice to the employer. For example, a salary net payment, created for the employer which is self paying the net payment to the worker.",
    			content: {
    				title: "content",
    				descr: "Invoice data in XML or json format."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			header: {
    				title: "header",
    				descr: "Invoice header details."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Invoice payer details."
    			},
    			pdf: {
    				title: "pdf",
    				descr: "Invoice in pdf format."
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Invoice recipient details."
    			},
    			rows: {
    				title: "rows",
    				descr: "List of invoice rows."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		InvoiceData: {
    			title: "invoice data",
    			descr: "Invoice specific business data for index.",
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			}
    		},
    		InvoiceHeader: {
    			title: "invoice header",
    			descr: "Header for the invoice",
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business object ids (typically calculations) on which the payment is based on."
    			},
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			externalId: {
    				title: "external id",
    				descr: "The external ID is used for matching the executed payment from the 3rd party system\nto the pending payment in the Palkkaus system."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this invoice is done.\nFor salaries, this is salary date, for monthly / quarterly payments, this is the first day of period."
    			},
    			message: {
    				title: "message",
    				descr: "The free text that is displayed / sent with the invoice.\nMay have multiple lines."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Payment reference (e.g. RF0710010831).\nPlease note that in salary payment scenarios, this is not necessarily unique."
    			},
    			status: {
    				title: "status",
    				descr: "The current notified status of the invoice in external system."
    			},
    			total: {
    				title: "total",
    				descr: "Total payable amount for the invoice including VAT."
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Total amount for the invoice excluding VAT."
    			},
    			type: {
    				title: "type",
    				descr: "Invoice type."
    			}
    		},
    		InvoiceListItem: {
    			title: "invoice list item",
    			descr: "Represents a single item in a list of invoices in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		InvoiceParty: {
    			title: "invoice party",
    			descr: "Party in an Varma Invoice: Either Payer or Recipient.",
    			companyId: {
    				title: "company id",
    				descr: "Finnish Company ID (Y-tunnus)."
    			},
    			companyName: {
    				title: "company name",
    				descr: "Organisation name"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information"
    			},
    			contactPersonName: {
    				title: "contact person name",
    				descr: "Contact person Full name"
    			}
    		},
    		InvoicePayer: {
    			title: "invoice payer",
    			descr: "Payment payer party of the invoice.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the party."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the party."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official identifier of the party:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)."
    			},
    			partnerKey: {
    				title: "partner key",
    				descr: "Optional ID for identifying the party in the third party system.\nThis is the same partner key that is given as part of Single Sign-On process."
    			}
    		},
    		InvoiceRecipient: {
    			title: "invoice recipient",
    			descr: "Payment receiver party of the invoice.",
    			beneficiary: {
    				title: "beneficiary",
    				descr: "The ultimate beneficiary of the money:\nThis may be a Palkkaus.fi Worker Account (Avatar id is not null)\nor some other entity: Tax authorities, pension company etc. (Avatar id is null).\nTypically beneficiary is the same as Iban, OfficialId, FullName and Contact in this object,\nbut this is not true in edge cases (e.g. customer funds account)."
    			},
    			contact: {
    				title: "contact",
    				descr: "Optional contact info. Not available for all recipients."
    			},
    			fullName: {
    				title: "full name",
    				descr: "The full name of the payment receiver for payment purposes."
    			},
    			iban: {
    				title: "iban",
    				descr: "The International Bank Account Number of the receiver."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the party: Finnish Personal Id (HETU) for a Person or Company Id (Y-tunnus) for a company or association.\nWARNING: This is currently needed for Finvoice only (https://www.finanssiala.fi/finvoice/Sivut/default.aspx). As such we consider removing it from persons.\nPlease inform if you use this property!"
    			}
    		},
    		InvoiceRow: {
    			title: "invoice row",
    			descr: "Row for the invoice",
    			accountingAccountNumber: {
    				title: "accounting account number",
    				descr: "Suggested account number (RowNormalProposedAccountIdentifier)."
    			},
    			accountingCode: {
    				title: "accounting code",
    				descr: "Textual code for accounting mapping (RowProposedAccountText, Raportointikoodi).\nIn Finvoice: \"Accounting information agreed between seller and buyer.\""
    			},
    			articleId: {
    				title: "article id",
    				descr: "Article Identifier is given by seller, EAN etc.\nIn the case of Salaxy salaries, this is CalculationRowType."
    			},
    			articleName: {
    				title: "article name",
    				descr: "Name of the product or service.\nIn the case of Salaxy salaries, this is language versioned text of CalculationRowType."
    			},
    			businessObject: {
    				title: "business object",
    				descr: "Business object id (typically calculation) on which the row is based on."
    			},
    			count: {
    				title: "count",
    				descr: "Count (DeliveredQuantity in Finvoice) is the quantity of product or service."
    			},
    			message: {
    				title: "message",
    				descr: "Message (RowFreeText in Finvoice) is the message is mainly designed for accounting systems."
    			},
    			price: {
    				title: "price",
    				descr: "Price including VAT (UnitPriceVatIncludedAmount, Yksikköhinta verollinen)"
    			},
    			priceExVat: {
    				title: "price ex vat",
    				descr: "Price of one product or service without Vat (UnitPriceAmount, Yksikköhinta veroton).\nYksikköhinta voidaan ilmoittaa myös verollisena UnitPriceVatIncludedAmount- elementissä laskutuskäytännön ja verottajan ALV-ohjeistuksen mukaan."
    			},
    			total: {
    				title: "total",
    				descr: "Row total with VAT (RowAmount, Yhteensä verollinen)"
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Row total excluding VAT (RowVatExcludedAmount, Yhteensä veroton)"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Vat percent related to the product or service (RowVatRatePercent, Alv%)"
    			}
    		},
    		InvoiceSettings: {
    			title: "invoice settings",
    			descr: "Settings related to invoices that are sent to customers.",
    			eInvoiceIntermediator: {
    				title: "Välittäjä",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "Verkkolaskuosoite",
    				descr: "Receiver's address for the e-invoice."
    			},
    			sepaBankPartyId: {
    				title: "SEPA-maksatustunnus"
    			},
    			ibanNumber: {
    				title: "Tilinumero"
    			}
    		},
    		InvoiceStatusNotification: {
    			title: "invoice status notification",
    			descr: "Contents for the invoice status notification message.",
    			amount: {
    				title: "amount",
    				descr: "Required realized amount of the transaction."
    			},
    			date: {
    				title: "date",
    				descr: "Required Date for the transaction (booking date)."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id can be set in the notification for the invoice for later messaging.\nIf the notification message contains an External Id AND regular Id, the External Id is updated in the Salaxy database.\nIf the notification contains External Id, but NO regular Id, an invoice is fetched based on External Id.\nIn the latter case, a non-existent Id or multiple invoices with the same External Id will throw an exception and the entire update process is canceled."
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID for the invoice in Salaxy system.\nIf omitted, the resolving to an invoice will be made using external id."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			receiverFullName: {
    				title: "receiver full name",
    				descr: "Optional full name of the payment receiver. Should be sent only if the user or process has the possibility of changing the name."
    			},
    			receiverIban: {
    				title: "receiver iban",
    				descr: "Optional bank account IBAN of the receiver. Should be sent only if the user or process has the possibility of changing the IBAN."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Optional payment reference number. Should be sent only if the user or process has the possibility of changing or removing the reference number.\nChanging the reference number will typically lead to an error in the back office process that needs to be manually handled."
    			},
    			status: {
    				title: "status",
    				descr: "New status for the invoice. Required main payload of the call."
    			}
    		},
    		IrDetails: {
    			title: "ir details",
    			descr: "Reporting details to Incomes registry (Tulorekisteri).",
    			carBenefit: {
    				title: "car benefit",
    				descr: "Provides the additional data needed for car benefit."
    			},
    			code: {
    				title: "code",
    				descr: "The number code in Incomes Register."
    			},
    			dailyAllowance: {
    				title: "daily allowance",
    				descr: "Daily allowance code."
    			},
    			flags: {
    				title: "flags",
    				descr: "Boolean flags that can be set on transaction."
    			},
    			insuranceExceptions: {
    				title: "insurance exceptions",
    				descr: "Defines the exceptions to Insurance / Sidecost defaults that are set by the row type."
    			},
    			mealBenefitIsTaxValue: {
    				title: "meal benefit is tax value",
    				descr: "Set this to true if meak benefit is of tax value - false if it si not.\nThe value should be null if the row is not mealBenefit."
    			}
    		},
    		IrIncomeTypeUsecase: {
    			title: "ir income type usecase",
    			descr: "Handles the Calculation of CalculationRowType.IrIncomeType to IrRows",
    			irData: {
    				title: "ir data",
    				descr: "Reporting details to Incomes registry (Tulorekisteri)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		IrRow: {
    			title: "ir row",
    			descr: "Business level modeling corresponding to WageReportsToIRTypes Transaction. Also contains additional data for accounting and integration purposes.",
    			accounting: {
    				title: "accounting",
    				descr: "Accounting related data for the row."
    			},
    			calcData: {
    				title: "calc data",
    				descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR)."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row. Default is one."
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			earningPeriod: {
    				title: "earning period",
    				descr: "Earnings period if differnet than the report period.\nNote that this property may be (and by default is) null."
    			},
    			irData: {
    				title: "ir data",
    				descr: "Details for the National Incomes registry"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row."
    			},
    			total: {
    				title: "total",
    				descr: "Simple multiplication: Count * Price - no other logic. Not used as Input."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the row - basically describes the source for this row:\nEither Manual (default) or Usecase logic, which may be legacy / API v02 or new / API v03."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. If specified as Undefined, set by the server-logic based on row type etc."
    			}
    		},
    		IrRowCalculationProperties: {
    			title: "ir row calculation properties",
    			descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR).",
    			behavior: {
    				title: "behavior",
    				descr: "Defines the behavior of the row in Salaxy calculation process."
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Grouping for reports etc."
    			}
    		},
    		JobProfile: {
    			title: "job profile",
    			backgroundImage: {
    				title: "background image",
    				descr: "Background image for the profile page"
    			},
    			cities: {
    				title: "cities",
    				descr: "A collection of known city IDs.\nQuery the interface for city IDs"
    			},
    			description: {
    				title: "description",
    				descr: "Long description: This field accepts MarkDown, potentially also HTML in the future (at the moment HTML may create an error)."
    			},
    			jobDetail: {
    				title: "job detail",
    				descr: "Detailed description of the Job"
    			},
    			jobTitle: {
    				title: "job title",
    				descr: "Title for the Job"
    			},
    			jobs: {
    				title: "jobs",
    				descr: "A collection of job ID's that this work or worker belongs"
    			},
    			links: {
    				title: "links",
    				descr: "External links to web sites, social media pages etc."
    			},
    			location: {
    				title: "location",
    				descr: "Free text for location of work or worker"
    			},
    			salaryMax: {
    				title: "salary max",
    				descr: "Max salary for the given salary period"
    			},
    			salaryMin: {
    				title: "salary min",
    				descr: "Minimum salary for the given salary period"
    			},
    			salaryPeriod: {
    				title: "salary period",
    				descr: "Period for which the Salary period is set."
    			}
    		},
    		LedgerAccount: {
    			title: "ledger account",
    			descr: "Represents a single accounting ledger.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			entries: {
    				title: "entries",
    				descr: "Legder entries"
    			},
    			saldo: {
    				title: "saldo",
    				descr: "Current saldo for the ledger."
    			}
    		},
    		LedgerEntry: {
    			title: "ledger entry",
    			descr: "Represents a single ledger entry.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			amount: {
    				title: "amount",
    				descr: "Posted amount."
    			},
    			description: {
    				title: "description",
    				descr: "Entry description."
    			},
    			dimension: {
    				title: "dimension",
    				descr: "Dimension data."
    			},
    			entryDate: {
    				title: "entry date",
    				descr: "Entry date."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the entry."
    			},
    			isDebit: {
    				title: "is debit",
    				descr: "Debit indicator."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "VAT percent."
    			}
    		},
    		"Map": {
    			title: "map",
    			descr: "Map between a hierarchical Salaxy object and a row in a table/dataset."
    		},
    		MapSelector: {
    			title: {
    				title: "title",
    				descr: "Title of the property. If null, path is used with reflection."
    			},
    			descr: {
    				title: "descr",
    				descr: "Description of the property. If null, path is used with reflection."
    			},
    			fromObject: {
    				title: "from object",
    				descr: "Function that defines how the row value(s) is derived from the object, typically in export.\nIf null, this is defined by path."
    			},
    			fromRow: {
    				title: "from row",
    				descr: "Function that sets the value from row back to object, typically in import.\nIf null, this is defined by path."
    			},
    			path: {
    				title: "path",
    				descr: "Defines source property as a simple object path if possible.\nIf a more complex logic is needed, you can define get and set functions below.\nThe property is also used in generating documentation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the property. If null, path is used with reflection."
    			}
    		},
    		MarketingSettings: {
    			title: "marketing settings",
    			descr: "Marketing related settings which may affect the pricing. E.g. campaigns, customer groups etc.",
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing."
    			}
    		},
    		MealBenefitUsecase: {
    			title: "meal benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.MealBenefit to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isTaxValue: {
    				title: "is tax value",
    				descr: "If true, the deduction from Worker salary corresponds to taxable value."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			taxablePrice: {
    				title: "taxable price",
    				descr: "The taxable price calculated by the server."
    			}
    		},
    		Measurement: {
    			title: "measurement",
    			descr: "Measurement value.",
    			tag: {
    				title: "tag",
    				descr: "Tag for the measurement."
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Time when the measurement was recorded."
    			},
    			value: {
    				title: "value",
    				descr: "Actual measurement value."
    			}
    		},
    		Message: {
    			title: "message",
    			descr: "A message within a messaging thread.",
    			content: {
    				title: "content",
    				descr: "Textual content: If message, this is the content text.\nIf other type, short title / description typically shown in the title bar."
    			},
    			contentData: {
    				title: "content data",
    				descr: "Other misc. data that is used in displaying the content in the immediate message view.\nDo not include the entire object here. Just the immediately available data."
    			},
    			contentId: {
    				title: "content id",
    				descr: "Identifier of the external content.\nBlobFile Id for attachments, Calculation ID for calculations etc."
    			},
    			fromParty: {
    				title: "from party",
    				descr: "The party / type of the message sender."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the message."
    			},
    			preview: {
    				title: "preview",
    				descr: "Content type specific text for preview.\nThis may be URI, ID or boolean.\nFor BlobFile, it is currently the BlobFile Guid (the same as ContentId) or null,\nbut in the future, the field may also contain an URL without that being a breaking change!"
    			},
    			readByOtherParty: {
    				title: "read by other party",
    				descr: "If set, specifies the time when the message has been read by the other party."
    			},
    			readByOwner: {
    				title: "read by owner",
    				descr: "If set, specifies the time when the message has been read by the thread owner."
    			},
    			time: {
    				title: "time",
    				descr: "Time for the message"
    			},
    			type: {
    				title: "type",
    				descr: "Content type of the message"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validates the object that is the payload of the message."
    			}
    		},
    		MessageData: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Message thread specific business data for index.",
    			otherPartyLastMessage: {
    				title: "other party last message",
    				descr: "Last message that was created by the other party"
    			},
    			otherPartyUnreadCount: {
    				title: "other party unread count",
    				descr: "Number of messages that have been read by the Other party and have not been read by the owner."
    			},
    			ownerLastMessage: {
    				title: "owner last message",
    				descr: "Last message that was was created by the owner"
    			},
    			ownerUnreadCount: {
    				title: "owner unread count",
    				descr: "Number of messages by Owner that have not been read by the other party"
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			}
    		},
    		MessageListItem: {
    			title: "message list item",
    			descr: "Represents a single item in a list of message threads in the user interface. It contains header data of the message thread and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		MessageThread: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Messages thread groups togehter a set of related messages and/or attachments.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			messages: {
    				title: "messages",
    				descr: "List of messages and attachments in this thread"
    			},
    			otherParty: {
    				title: "other party",
    				descr: "Palkkaus.fi id (IBAN) of the other party.\nThe other party is the second entity(besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party is the second entity (besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyWorkflow: {
    				title: "other party workflow",
    				descr: "The workflow from the other party point-of-view."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "The owner of the thread: The entity that has the right to delete or move the thread.\nThis may or may not be the originator the message thread.\nThe owner is typically Company or Household, in some cases Worker person.\nIt should NOT be partner or person from a company."
    			},
    			ownerWorkflow: {
    				title: "owner workflow",
    				descr: "The workflow from the Owner point-of-view."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		MessageThreadWorkflow: {
    			title: "message thread workflow",
    			descr: "Defines workflow actions for a message thread.",
    			lastMessageText: {
    				title: "last message text",
    				descr: "Text for the last message (written by this party). This is for preview purposes: May be shortened."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The salary date for the discussion.\nOwner's and other party's are typically the same, but they may differ:\ne.g. one may be the requested / estimated date and the other the first possible date if the requested is unrealistic."
    			},
    			unreadCount: {
    				title: "unread count",
    				descr: "Number of messages that this party has posted that have not been read by the other party."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Time of the last message posted by this party."
    			}
    		},
    		MonthlyHolidayAccrual: {
    			title: "monthly holiday accrual",
    			descr: "Holiday Accrual for a calendar month in releation to a calculation.",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of absence days used in the calculation."
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Holiday accrual for this month.\nNote that -1 is indication that accrual should not be updated for this particylar month.\nThere is no possiblity for negative accrual."
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			workDays: {
    				title: "work days",
    				descr: "Work days in determining the accrual for the month in 14 days rule."
    			},
    			workHours: {
    				title: "work hours",
    				descr: "Work hours in determining the accrual fot the month in 35 hour rule."
    			}
    		},
    		NonProfitOrgUsecase: {
    			title: "non profit org usecase",
    			descr: "Handles the Calculation of CalculationRowType.NonProfitOrg to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		OAuthMessage: {
    			title: "o auth message",
    			descr: "Based on standard https://tools.ietf.org/html/rfc6749 .",
    			access_token: {
    				title: "access_token",
    				descr: "The access token issued by the authorization server."
    			},
    			assertion: {
    				title: "assertion",
    				descr: "The value of the \"assertion\" parameter MUST contain a single JWT."
    			},
    			client_id: {
    				title: "client_id",
    				descr: "The client identifier issued to the client during the registration process."
    			},
    			client_secret: {
    				title: "client_secret",
    				descr: "The client secret. The client MAY omit the parameter if the client secret is an empty string."
    			},
    			code: {
    				title: "code",
    				descr: "The authorization code received from the authorization server."
    			},
    			connection: {
    				title: "connection",
    				descr: "Auth0 connection"
    			},
    			error: {
    				title: "error",
    				descr: "A single ASCII [USASCII] error code."
    			},
    			error_description: {
    				title: "error_description",
    				descr: "Human-readable ASCII [USASCII] text providing additional information, used to assist the client developer in understanding the error that occurred."
    			},
    			error_uri: {
    				title: "error_uri",
    				descr: "A URI identifying a human-readable web page with information about the error, used to provide the client developer with additional information about the error."
    			},
    			expires_in: {
    				title: "expires_in",
    				descr: "The lifetime in seconds of the access token.  For example, the value \"3600\" denotes that the access token will expire in one hour from the time the response was generated. If omitted, the authorization server SHOULD provide the expiration time via other means or document the default value."
    			},
    			grant_type: {
    				title: "grant_type",
    				descr: "Requested grant type."
    			},
    			id_token: {
    				title: "id_token",
    				descr: "The id token issued by the authorization server."
    			},
    			password: {
    				title: "password",
    				descr: "The resource owner password."
    			},
    			redirect_uri: {
    				title: "redirect_uri",
    				descr: "After completing its interaction with the resource owner, the authorization server directs the resource owner's user-agent back to the client."
    			},
    			refresh_token: {
    				title: "refresh_token",
    				descr: "The refresh token, which can be used to obtain new access tokens."
    			},
    			response_type: {
    				title: "response_type",
    				descr: "The value MUST be one of \"code\" for requesting an authorization code, \"token\" for  requesting an access token(implicit grant), or a registered extension value."
    			},
    			salaxy_language: {
    				title: "salaxy_language",
    				descr: "User interface language."
    			},
    			salaxy_onboarding_uri: {
    				title: "salaxy_onboarding_uri",
    				descr: "Salaxy specific onboarding uri."
    			},
    			salaxy_role: {
    				title: "salaxy_role",
    				descr: "Users role."
    			},
    			scope: {
    				title: "scope",
    				descr: "The value of the scope parameter is expressed as a list of space-delimited, case-sensitive strings.The strings are defined by the authorization server."
    			},
    			state: {
    				title: "state",
    				descr: "An opaque value used by the client to maintain state between the request and callback.The authorization server includes this value when redirecting the user-agent back to the client. The parameter SHOULD be used for preventing cross-site request forgery."
    			},
    			token_type: {
    				title: "token_type",
    				descr: "The type of the token issued."
    			},
    			username: {
    				title: "username",
    				descr: "The resource owner username."
    			}
    		},
    		ODataQueryOptions: {
    			title: "o data query options",
    			descr: "Defines the query options of OData",
    			$count: {
    				title: "$count",
    				descr: "Tells the server to include the total count of matching entities in the response. (Useful for server-side paging.)"
    			},
    			$expand: {
    				title: "$expand",
    				descr: "Expands related entities inline. At the moment, this typically not implemented."
    			},
    			$filter: {
    				title: "$filter",
    				descr: "Filters the results, based on a Boolean condition."
    			},
    			$orderby: {
    				title: "$orderby",
    				descr: "Sorts the results."
    			},
    			$search: {
    				title: "$search",
    				descr: "Performs free-text search using the in-built logic in the API."
    			},
    			$select: {
    				title: "$select",
    				descr: "Selects which properties to include in the response."
    			},
    			$skip: {
    				title: "$skip",
    				descr: "Skips the first n results."
    			},
    			$top: {
    				title: "$top",
    				descr: "Returns only the first n the results."
    			},
    			anon: {
    				title: "anon",
    				descr: "If true, perfoms the query also in anonymous mode\n(no token, token expired or invalid token).\nDefault is that the query is done only if user has a valid token."
    			}
    		},
    		ODataResult: {
    			title: "o data result",
    			descr: "Result from OData query",
    			count: {
    				title: "count",
    				descr: "Total count of the items (server-side/storage) if updated in the previous query.\nNote that this may be null if the count is not fetched."
    			},
    			items: {
    				title: "items",
    				descr: "Actual results array"
    			},
    			nextPageLink: {
    				title: "next page link",
    				descr: "Next page link"
    			}
    		},
    		Occupation: {
    			title: "occupation",
    			descr: "Occupation according to Statistics Finland.\nUsed for insurance pricing and reporting.",
    			code: {
    				title: "code",
    				descr: "Statistics Finland code. There may be duplicates with different id."
    			},
    			freeText: {
    				title: "free text",
    				descr: "Free text\nTODO: Do we need to expose this? Why?"
    			},
    			id: {
    				title: "id",
    				descr: "Salaxy identifier for the Occupation. Begins with code, but may have a suffix."
    			},
    			keyword: {
    				title: "keyword",
    				descr: "Keywords for searching etc."
    			},
    			label: {
    				title: "label",
    				descr: "Label"
    			},
    			lang: {
    				title: "lang",
    				descr: "Language for the entry"
    			},
    			search: {
    				title: "search",
    				descr: "Lower case text that contains the id, label and potentially other text that should match in searches."
    			}
    		},
    		Onboarding: {
    			title: "onboarding",
    			descr: "Provides the data for onboarding process for a company. Onboarding is the process where Salaxy gathers sufficient information to at least create an acccount, often even more so that the user can pay a salaary with the account.",
    			accountType: {
    				title: "account type",
    				descr: "Type of the account to be created."
    			},
    			appState: {
    				title: "app state",
    				descr: "Custom state object that is guaranteed to survive the round-trip in the onboarding process.\nCalling application to the onboarding may pass here any data that it needs after the onboarding process is ready and user has been authenticated."
    			},
    			company: {
    				title: "company",
    				descr: "Input that is needed for creating a company account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			credentials: {
    				title: "credentials"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			person: {
    				title: "person",
    				descr: "Input that is needed for creating a personal account.\nThis may be\n1) the principal account if the account is created for 1a) Worker or 1b) Household or\n2) creator owner if the account is created for a Company."
    			},
    			products: {
    				title: "products",
    				descr: "Possibility to define products.\nThese can only be defined to one account as part of the onboarding process:\nIf company account is created, the products are always for Company, not for the Person =&gt;\nPerson will have default products in that case and they need to be edited in a separate process once the accounts have been created."
    			},
    			signature: {
    				title: "signature",
    				descr: "Input and post data for digital signature."
    			},
    			status: {
    				title: "status",
    				descr: "The server-side status of the onboarding process.\nNOTE that there is a more relaxed user interface-level state in ui.state and ui.stateFullText\nthat can be used in onboarding user interfaces."
    			},
    			ui: {
    				title: "ui",
    				descr: "User interface related data: Mainly the state of the UI."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation status and errors."
    			}
    		},
    		OnboardingContactVerification: {
    			title: "onboarding contact verification",
    			descr: "Contact verification object for single message type.",
    			pin: {
    				title: "pin",
    				descr: "Given pin code."
    			}
    		},
    		OnboardingContactVerifications: {
    			title: "onboarding contact verifications",
    			descr: "Contact verification object for onboarding.",
    			email: {
    				title: "email",
    				descr: "Verification for email contact."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Verification for telephone contact."
    			}
    		},
    		OnboardingCredentials: {
    			title: "onboarding credentials",
    			descr: "Stores information about the credentials that were used in the Onboarding process.",
    			credentialId: {
    				title: "credential id",
    				descr: "Identifier of the credentials (UserAccount Guid in the database)."
    			},
    			partnerId: {
    				title: "partner id",
    				descr: "The partner that initiated the Onboarding process."
    			}
    		},
    		OnboardingUIState: {
    			title: "onboarding u i state",
    			descr: "Stores data for the onboarding user interface.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "URL where the user should be redirected back to if the process is canceled or there is an urecoverable error."
    			},
    			invitationKey: {
    				title: "invitation key",
    				descr: "Salaxy worker invitation key."
    			},
    			oAuthResponseType: {
    				title: "o auth response type",
    				descr: "Response type to OAuth -request"
    			},
    			state: {
    				title: "state",
    				descr: "State of the user interface. Typically an enumeration that stores where the UI was. E.g. Wizard step."
    			},
    			stateFullText: {
    				title: "state full text",
    				descr: "Full description of the user interface state for admin and analyses purposes."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "URL where the user should be redirected back to when the onboarding process is complete."
    			}
    		},
    		OtherCompensationUsecase: {
    			title: "other compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.OtherCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Ownership: {
    			title: "ownership",
    			isRequiredTracking: {
    				title: "is required tracking",
    				descr: "If true, authorities require tracking the shareholders.\nIn Finland, with current legistlation, this is if a shareholder has over 25% of the shares."
    			},
    			shareHolders: {
    				title: "share holders",
    				descr: "List of shareholders for the company"
    			}
    		},
    		PageResultOfApiListItem: {
    			title: "page result of api list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfArticle: {
    			title: "page result of article",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfCalculationListItem: {
    			title: "page result of calculation list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfEmploymentListItem: {
    			title: "page result of employment list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfFileListItem: {
    			title: "page result of file list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfInvoiceListItem: {
    			title: "page result of invoice list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfMessageListItem: {
    			title: "page result of message list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfPayrollListItem: {
    			title: "page result of payroll list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfSampleListItem: {
    			title: "page result of sample list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfTaxcardListItem: {
    			title: "page result of taxcard list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfWorkerListItem: {
    			title: "page result of worker list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PartnerAccounting: {
    			title: "partner accounting",
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, partner settings override company settings."
    			},
    			targetIds: {
    				title: "target ids",
    				descr: "Available accounting targets."
    			}
    		},
    		PartnerBranding: {
    			title: "partner branding",
    			descr: "Partner branding settings: logo, css etc.",
    			css: {
    				title: "css",
    				descr: "Css path for the primary partner."
    			},
    			logo: {
    				title: "logo",
    				descr: "Url to partner logo."
    			},
    			text: {
    				title: "text",
    				descr: "Long marketing text."
    			},
    			url: {
    				title: "url",
    				descr: "Marketing url."
    			}
    		},
    		PartnerCompanyAccountInfo: {
    			title: "partner company account info",
    			descr: "Information about a Customer account for a Partner",
    			accountId: {
    				title: "account id",
    				descr: "Account Id"
    			},
    			accountIdFormatted: {
    				title: "account id formatted",
    				descr: "Account Id fomratted for dispaly to end user"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for display information"
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If true, the current user (the partner) is authorized to access the account"
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has a verified owner."
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The partner account ID to which the access is currently restricted."
    			}
    		},
    		PartnerMessaging: {
    			title: "partner messaging",
    			descr: "Messaging settings for the partner.",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the messaging is switched on for this partner."
    			}
    		},
    		PartnerService: {
    			title: "partner service",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the calculation data delivery by forms is switched on for this partner."
    			}
    		},
    		PartnerServiceModel: {
    			title: "Palvelumalli",
    			"package": {
    				title: "Paketti"
    			},
    			features: {
    				title: "Aktivoidut palvelut"
    			},
    			pricing: {
    				title: "Hinnoittelu"
    			},
    			branding: {
    				title: "Brändi"
    			},
    			site: {
    				title: "Sivuston ominaisuudet"
    			}
    		},
    		PartnerWebSite: {
    			title: "partner web site",
    			descr: "Stores information about the web site that is used for customer (employer) to access the customer web site.",
    			url: {
    				title: "url",
    				descr: "The main URL that is displayed.\nDefault is https://www.palkkaus.fi/company"
    			}
    		},
    		Payer: {
    			title: "payer",
    			descr: "The paying party",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		Payment: {
    			title: "payment",
    			descr: "Payment object requested to be paid for completing the underlying business objects.",
    			amount: {
    				title: "amount",
    				descr: "Amount requested to be paid."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business objects (calculations etc.) on which the payment is based on."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payment."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			message: {
    				title: "message",
    				descr: "Message for the payment."
    			},
    			method: {
    				title: "method",
    				descr: "Payment method."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer as it would be on the invoice"
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Recipient as it would be on the invoice"
    			},
    			reference: {
    				title: "reference",
    				descr: "Bank reference number/string. Will be generated by the system automatically."
    			},
    			request: {
    				title: "request",
    				descr: "Request specific payment parameters."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the payment."
    			},
    			statusText: {
    				title: "status text",
    				descr: "Status text giving more information about the status."
    			},
    			type: {
    				title: "type",
    				descr: "Payment type."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PaymentChannelSettings: {
    			channel: {
    				title: "Maksukanava"
    			},
    			isEnabled: {
    				title: "Valittu"
    			},
    			isAvailable: {
    				title: "Valittavissa"
    			},
    			data: {
    				title: "Data"
    			},
    			accountingTargetId: {
    				title: "Kohteen tunniste"
    			}
    		},
    		PaymentRequestParameters: {
    			title: "payment request parameters",
    			descr: "Contains request specific parameters for payment.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "Return url for failed / cancelled payment."
    			},
    			paymentStartedUrl: {
    				title: "payment started url",
    				descr: "If this URL is set, the payment allows aynchronous payment methods\nsuch as payment with invoice (viitelasku).\nThis means that the payment UI returns in status InProgress and\npayment is finalized later (may take hours, days or even be canceled)."
    			},
    			sendEmails: {
    				title: "send emails",
    				descr: "Indicates whether to send emails according to workflow after payment.\nTODO: This should be retrieved from the partner profile etc."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "Return url for successful payment."
    			}
    		},
    		PaymentSettings: {
    			title: "payment settings",
    			descr: "Payment related settings",
    			channels: {
    				title: "Maksukanavat"
    			},
    			customerFunds: {
    				title: "Asiakasvara-asetukset",
    				descr: "Settings related to payment methods that use the customer funds account (gross payment)."
    			},
    			defaultChannel: {
    				title: "Oletuskanava"
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			invoice: {
    				title: "Verkkolaskuasetukset",
    				descr: "Settings related to invoices: Either salary payment or other invoicing."
    			},
    			pricing: {
    				title: "pricing",
    				descr: "The pricing partner (accounant or software) that defines the pricing of the service.\nThis data is typically read-only in normal settings view."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		Payroll03Info: {
    			title: "payroll03 info",
    			descr: "Provides Read-only information about the Payroll In the case of non-paid Payrolls, this is the latest save/validation. In the case of paid Payrolls, this is the status of the Payroll when it was paid.",
    			calcCount: {
    				title: "calc count",
    				descr: "Count of calculations in this Payroll"
    			},
    			date: {
    				title: "date",
    				descr: "Date that the Info was last updated."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payroll. Date when the payment should be made to achieve SalaryDate."
    			},
    			fees: {
    				title: "fees",
    				descr: "Total Palkkaus.fi fees for the Payroll"
    			},
    			isReadyForPayment: {
    				title: "is ready for payment",
    				descr: "If true, all calculations have been validated and they are valid for payment.\nIf false, validation has not been done or some calculations are not valid for payment."
    			},
    			paymentDate: {
    				title: "payment date",
    				descr: "Paid at date for the payroll. (Date of the employer's payment)."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "The reference number of the payment after payment is made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			status: {
    				title: "status",
    				descr: "Status for the payroll and its calculations in the payment Workflow."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment for the Payroll."
    			}
    		},
    		PayrollDetails: {
    			title: "payroll details",
    			descr: "New version 03 of Payroll. Payroll (Palkkalista) is a list of employees who receive salary or wages from a particular organization. Typical usecase is that a a company has e.g. a monthly salary list that is paid at the end of month. For next month, a copy is then made from the latest list and the copy is potentially modified with the changes of that particular month.",
    			calcs: {
    				title: "calcs",
    				descr: "Snapshots of the calculations based on the ID's specified in Info"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing Payroll lists.\nIn practice, a usecase is either a user interface or a microservice that provides a Payroll processing\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of the Payroll."
    			},
    			validations: {
    				title: "validations",
    				descr: "When saved or validated, provides a validation object for each calculation:\nWhether the calculations are ready for payment."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		PayrollInput: {
    			title: {
    				title: "title",
    				descr: "Title that can be specified by the end user."
    			},
    			descr: "Input that user defines about the Payroll",
    			calculations: {
    				title: "calculations",
    				descr: "List of calculations that are attached to this calculation."
    			},
    			noUpdateFromEmployment: {
    				title: "no update from employment",
    				descr: "If true, the calculation rows are not updated from employment when the Payroll is copied.\nDefault is that the rows and other data is updated in copy / create new etc."
    			},
    			period: {
    				title: "period",
    				descr: "The Default Salary period for the Payroll.\nThis may be overriden for an individual calculation."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "What types of wages are paid with this payroll.\nMay affect the rendering of the user interface (e.g. for     holidays, hours and absences).\nAlso affects which Workers are show in the add / link dialog by default."
    			}
    		},
    		PayrollListItem: {
    			title: "payroll list item",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PensionContract: {
    			title: "Työeläkevakuutus",
    			descr: "",
    			company: {
    				title: "Yhtiö",
    				editLabel: "Valitse yhtiö"
    			},
    			id: {
    				title: "Tunniste"
    			},
    			status: {
    				title: "Tila"
    			},
    			contractNumber: {
    				title: "Sopimusnumero"
    			},
    			startDate: {
    				title: "Alkamispäivä"
    			},
    			endDate: {
    				title: "Päättymispäivä"
    			}
    		},
    		PensionEntrepreneurProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by TempContractMarker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			tempContractMarker: {
    				title: "temp contract marker",
    				descr: "If true, user wants to order a YEL-contract."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Pension for Workers: The payments and monthly reporting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPensionSelfHandling."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isPendingContract: {
    				title: "is pending contract",
    				descr: "If true, there is a new pending contract."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The pension company where the user has a contract."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Number for the contract."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionSettings: {
    			title: "pension settings",
    			company: {
    				title: "company",
    				descr: "The pension company where the user has a contract."
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Number for the contract."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the pension insurance."
    			}
    		},
    		PeriodRecalculationData: {
    			title: "period recalculation data",
    			descr: "Data that is needed for determening PeriodRecalculationParams and thus at the end recalculating holidays and absences to a calculation.",
    			absences: {
    				title: "absences",
    				descr: "Absences that should be deducted from monthly salary.\nIf not provided, absences are not calculated."
    			},
    			calc: {
    				title: "calc",
    				descr: "The salary calculation that should be recalculated."
    			},
    			holidayYear: {
    				title: "holiday year",
    				descr: "Holiday year with holidays that should be added as salary rows.\nIf not provided, holidays are not calculated."
    			}
    		},
    		PersonAccount: {
    			title: "person account",
    			descr: "An account for a Person - can be both Employer and Worker",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			contactVerificationPin: {
    				title: "contact verification pin",
    				descr: "Contact verification pin for securing contact address changes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerInfo: {
    				title: "employer info",
    				descr: "The employer information in case the person is an employer"
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			lastLoginAs: {
    				title: "last login as",
    				descr: "Type of the last login - the user role"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "The worker information in case the person is a Worker"
    			}
    		},
    		PersonInput: {
    			title: "person input",
    			bankAccountIban: {
    				title: "bank account iban",
    				descr: "Bank account number for salary payment"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the user / personal account"
    			},
    			contactVerification: {
    				title: "contact verification",
    				descr: "Verification objects for contact verification."
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a personal account based on this data.\nOtherwise this is just user data for Company."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Personal ID (Henkilötunnus /HETU)"
    			},
    			picture: {
    				title: "picture",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given PersonalId exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			}
    		},
    		PricingPartner: {
    			title: "pricing partner",
    			descr: "Defines the partner, service and pricing model for the customer.",
    			accountId: {
    				title: "account id",
    				descr: "Partner account id."
    			},
    			fixedFee: {
    				title: "fixed fee",
    				descr: "Fee for one calculation, which is visible and paid by customer.\nThis is not relevant if the PricingModel is PalkkausFee"
    			},
    			pricingModel: {
    				title: "pricing model",
    				descr: "Pricing model by the partner."
    			},
    			serviceModel: {
    				title: "service model",
    				descr: "Service model by the partner."
    			}
    		},
    		PricingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Defines the pricing partner, service model and pricing model.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically if partner set and partner specific pricing model."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			pricingPartner: {
    				title: "pricing partner",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of pricing."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PrimaryPartner: {
    			title: "primary partner",
    			descr: "Primary partner settings.",
    			accounting: {
    				title: "accounting"
    			},
    			branding: {
    				title: "branding"
    			},
    			description: {
    				title: "description",
    				descr: "Description of the primary account."
    			},
    			id: {
    				title: "id",
    				descr: "Primary account id."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "Boolean indicating if this partner has been authorized to act on behalf the current account."
    			},
    			isPrimary: {
    				title: "is primary",
    				descr: "Boolean indicating if this partner is the primary partner of the current account."
    			},
    			key: {
    				title: "key",
    				descr: "Human-readable, human decided, recognizable short unique key for the partner:\nAll lower-case, a-z, 0-9, \"-\" and \"_\". Used in URL's etc."
    			},
    			messaging: {
    				title: "messaging"
    			},
    			name: {
    				title: "name",
    				descr: "Display name for the account."
    			},
    			service: {
    				title: "service"
    			},
    			site: {
    				title: "site"
    			}
    		},
    		PrimaryPartnerSettings: {
    			accountId: {
    				title: "Palkkaus-tilin tunniste"
    			},
    			info: {
    				title: "Tiedot"
    			},
    			type: {
    				title: "Tyyppi"
    			},
    			serviceModel: {
    				title: "Palvelumalli"
    			},
    			serviceModelId: {
    				title: "Palvelumallin id"
    			}
    		},
    		Product: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Base inteface for Products.\nThe common properties for all Products: BaseSalaryProduct, TaxProduct, UnemploymentProduct etc.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user. Restrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product.\nE.g. Raksaloki is available for households only and this is a Company account =&gt; Should not show the product."
    			}
    		},
    		Profile: {
    			title: "profile",
    			descr: "Profile object for either Worker or Employer",
    			accountId: {
    				title: "account id",
    				descr: "Account if known"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "The avatar object: Picture name and other such basic info."
    			},
    			contact: {
    				title: "contact",
    				descr: "Other contact information for this profile."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "The legal entity type of the profile: Person or Company"
    			},
    			expiresAt: {
    				title: "expires at",
    				descr: "Expiration date for the profile will be unpublished."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id (e.g. member number)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isPublic: {
    				title: "is public",
    				descr: "If true, the profile is set to public."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			job: {
    				title: "job"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			sortableName: {
    				title: "sortable name"
    			},
    			source: {
    				title: "source",
    				descr: "The source of this profile: Palkkaus section or partner site.\nThis is a strongly typed object that may be undefined - see SourceId in that case."
    			},
    			sourceId: {
    				title: "source id",
    				descr: "The identifier string of the source. This will always return a distinct value."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			userUpdatedAt: {
    				title: "user updated at",
    				descr: "Last time user updated the profile"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		ProfileJob: {
    			title: "profile job",
    			descr: "A job for a Public Profile (Osaajaprofiili) or later also for Job ads.",
    			category: {
    				title: "category"
    			},
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Job"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			picture: {
    				title: "picture"
    			},
    			synonyms: {
    				title: "synonyms"
    			}
    		},
    		ProfileJobCategory: {
    			title: "profile job category",
    			descr: "Category of Profile Jobs",
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Category"
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID of the Category"
    			},
    			name: {
    				title: "name",
    				descr: "Name / title of the Category"
    			},
    			picture: {
    				title: "picture",
    				descr: "Backround picture for the Category"
    			}
    		},
    		Recipient: {
    			title: "recipient",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		RemunerationUsecase: {
    			title: "remuneration usecase",
    			descr: "Handles the Calculation of CalculationRowType.Remuneration to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Report: {
    			title: {
    				title: "title",
    				descr: "Short title for the report that can be shown to the end user."
    			},
    			descr: "Describes a logical generated report. This is a report instance e.g. a monthly report for January 2017. It may contain several files of different types: PDF, preview PNG, HTML etc.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			htmlUrl: {
    				title: "html url",
    				descr: "HTML for the report. Use salaxy-rpt.css for formatting.\nFor preview and demo, this is the format to show to the user - PDF is not generated."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isDemoData: {
    				title: "is demo data",
    				descr: "If true, the report is created using demo data - should be used only in the beginning when the user has not paid any salaries.\nThis may be e.g. calculations in draft stage or completely invented demo data such as example workers.\nDemo data should not be confused to test database (test-api.palkkaus.fi / test-api.salaxy.com):\nPaid calculations in test database are production data."
    			},
    			isPreview: {
    				title: "is preview",
    				descr: "If true, the report is in preview state, typically in the middle of the month.\nThis means that the data itself is production, its just not final."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			jsonUrl: {
    				title: "json url",
    				descr: "Some reports are also available in JSON format, from which you can create lists, CSV, Excel files etc."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pdfUrl: {
    				title: "pdf url",
    				descr: "PDF report for viewing and printing. This is typically the main report.\nFor preview and demo, there is typically only HTML version - no PDF."
    			},
    			reportDate: {
    				title: "report date",
    				descr: "Report date: For monthly and yearly reports this is the first day of the year/month.\nFor ad-hoc reports, this is the start date of the period."
    			},
    			thumbnail: {
    				title: "thumbnail",
    				descr: "Thumbnail image for the report.\nFor preview and demo data this is typically a fixed image - the thumbnail is only generated when PDF is generated."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the report"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			urlsExpireAt: {
    				title: "urls expire at",
    				descr: "Time when the PDF, HTML or JSON URL's expire IF they are generated in a way that\nthey are callable without authentication for a specified time period.\nThis property is null if the URL requires authentication."
    			}
    		},
    		ReportingArea: {
    			title: "reporting area",
    			bottom: {
    				title: "bottom",
    				descr: "Bottom margin."
    			},
    			left: {
    				title: "left",
    				descr: "Left margin."
    			},
    			right: {
    				title: "right",
    				descr: "Right margin."
    			},
    			top: {
    				title: "top",
    				descr: "Top margin."
    			}
    		},
    		ReportingSettings: {
    			title: "reporting settings",
    			descr: "Settings related to reporting",
    			css: {
    				title: "css",
    				descr: "Additional CSS that is used to customize the layout of the report."
    			},
    			denyServiceModel: {
    				title: "Ohita palvelumalli"
    			},
    			footerTemplate: {
    				title: "footer template",
    				descr: "HTML template for the print footer.\nUse the following handlebars tags to inject values into the footer:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			headerTemplate: {
    				title: "header template",
    				descr: "HTML template for the print header.\nUse the following handlebars tags to inject values into the header:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			img2: {
    				title: "img2",
    				descr: "Secondary image that can be used in header / footer as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			layoutTemplateId: {
    				title: "Ulkoasun tunniste"
    			},
    			layoutTemplateVariables: {
    				title: "Ulkoasun muuttujat"
    			},
    			logo: {
    				title: "Logo",
    				descr: "Main logo image as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			margins: {
    				title: "margins",
    				descr: "Defines the margins of the content area"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		ResultRow: {
    			title: "result row",
    			descr: "Defines a Calculation row in the calculation results",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowSource: {
    				title: "row source",
    				descr: "Source of the calculation row - affects how the row is handled in editing / recalculation scenarios."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is kept as-is in the Salaxy system."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the row - always Price * Count"
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "Base salary (Peruspalkka) is the monthly salary or total salary without addiotions (overtime, holidays etc.)"
    			},
    			totalDeduction: {
    				title: "total deduction",
    				descr: "Deductions from the salary: Union payment, Advance, Foreclosure"
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Taxfree expenses compensation"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary (Bruttopalkka) is \"real\" salary including the additions (overtime, holidays etc.) but not benefits."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "Social Security Base (Sosiaaliturvan alainen ansio) is typically the same as TotalTaxable,\nbut e.g. child care subsidy (yksityisen hoidon tuki) is handled differently."
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable income (Ennakonpidätyksen alainen ansio) is the salary from taxation point-of-view. It is basically the Gross Salary plus benefits."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row"
    			},
    			userRowIndex: {
    				title: "user row index",
    				descr: "Row index that matches this row to a user defined row (which is zero-based).\nIf the row is generated by calculation business logic (e.g. based on framework agreement), the index is -1."
    			}
    		},
    		RootDeclarationReflection: {
    			title: "root declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			}
    		},
    		RowAccountingData: {
    			title: "row accounting data",
    			descr: "Provides row specific data for accounting.",
    			accountNumberFlags: {
    				title: "account number flags",
    				descr: "Special flags for account number mapping."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percent for VAT calculation expressed as decimal: 24% is 0.24.\nThe price in the root level is assumed being with tax."
    			}
    		},
    		Salary: {
    			title: "salary",
    			descr: "Defines the main salary that is the bases for the calculation",
    			amount: {
    				title: "amount",
    				descr: "Amount of e.g. hours, months for the salary. Default is 1."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If set to true, will calculate the household deduction. Applies only to natural persons - as oppsed to companies."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the salary being calculated"
    			},
    			message: {
    				title: "message",
    				descr: "Optional message for the salary row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price of one unit - e.g. hour, month."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			unit: {
    				title: "unit",
    				descr: "E.g. hour, month. Typically this parameter can be left empty/null and the unit is set according to type."
    			}
    		},
    		SampleApiObject: {
    			title: "sample api object",
    			descr: "Sample object for creating v03 controller",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SampleData: {
    			title: "sample data",
    			descr: "Sample custom data in search index",
    			test: {
    				title: "test",
    				descr: "A test string"
    			}
    		},
    		SampleListItem: {
    			title: "sample list item",
    			descr: "Available Search index data of the Sample object",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ServiceModelAccounting: {
    			title: "Kirjanpito",
    			defaultTargetId: {
    				title: "Oletuskohde"
    			},
    			targets: {
    				title: "Kohteet"
    			}
    		},
    		ServiceModelBranding: {
    			title: "Brändi",
    			logo: {
    				title: "Logo"
    			},
    			skin: {
    				title: "Tyylitiedosto"
    			}
    		},
    		ServiceModelDto: {
    			title: "Palvelumalli",
    			id: {
    				title: "Id"
    			},
    			name: {
    				title: "Nimi"
    			}
    		},
    		ServiceModelFeatures: {
    			title: "Aktivoitavat palvelut",
    			messaging: {
    				title: "Viestit"
    			},
    			forms: {
    				title: "Aineiston lähetys"
    			},
    			accounting: {
    				title: "Kirjanpito"
    			},
    			payments: {
    				title: "Maksaminen"
    			},
    			reporting: {
    				title: "Tulosteet"
    			}
    		},
    		ServiceModelPayments: {
    			title: "Maksaminen",
    			defaultChannel: {
    				title: "Oletusmaksutapa"
    			},
    			channels: {
    				title: "Maksutavat"
    			}
    		},
    		ServiceModelPricing: {
    			title: "Hinnoittelu",
    			pricingModel: {
    				title: "Hinnoittelumalli"
    			},
    			fixedFee: {
    				title: "Kiinteä hinta"
    			}
    		},
    		ServiceModelSite: {
    			title: "Sivuston ominaisuudet",
    			webApp: {
    				title: "Sovellus / sivusto"
    			},
    			webAppName: {
    				title: "Sovelluksen / sivuston nimi"
    			},
    			webAppUrl: {
    				title: "Sivuston osoite"
    			},
    			texts: {
    				title: "Tekstit"
    			},
    			sitemap: {
    				title: "Sivukartta"
    			}
    		},
    		SessionUserCredential: {
    			title: "session user credential",
    			descr: "Represents a user credential in session: This credential is typically based on Claims and is availble without going to database.",
    			authenticationMethod: {
    				title: "authentication method",
    				descr: "The ultimate authentication provider that authenticated the user, e.g. Facebook, Google."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object that contains the visual reresentation of the User account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail address if provided by the authentication provider."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identitier for the user ID coming from the Auth0 database:\n'[MethodText]|[ProviderId]', e.g. 'facebook|12345678901234567'\n(Number may contain more characters - not sure for the exact cahracters count)"
    			}
    		},
    		SharedTaxcardExistsResult: {
    			title: "shared taxcard exists result",
    			descr: "Provides a a result to query of whether a taxcard exists for a personal ID (no personal information).",
    			isError: {
    				title: "is error",
    				descr: "If true there was an error fetching the information.\nTypically, this would be that there were too many attempts to fetch Personal ID's and request was classified as phishing.\nIn this case, you should currently create the employment relation and fetch the taxcard infor after that.\nWe may later introduce a \"Are you robot\" type of interface to go around this (if necessary)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard. Undefined if none is found."
    			},
    			ownedExists: {
    				title: "owned exists",
    				descr: "Returns true if there is already an owned taxcard for the employer."
    			},
    			ownedTaxcard: {
    				title: "owned taxcard",
    				descr: "If Owned taxcard exists, it is returned here."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID that was used in fetching.\nIt may be validated and formatted, or just the original string if there were issues."
    			},
    			sharedExists: {
    				title: "shared exists",
    				descr: "Returns true if a shared taxcard exists."
    			},
    			uri: {
    				title: "uri",
    				descr: "Uri for the Shared taxcard"
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity of the taxcard. Note that this property is null when Exists is false."
    			}
    		},
    		Shareholder: {
    			title: "shareholder",
    			descr: "Defines a shareholder. At the moment, this is only a name and it is used only if the person has more than 25% of the shares. Expect this object to be developed with, Ownership percent or number of shares, Id or other such link to the actual person etc.",
    			name: {
    				title: "name",
    				descr: "Name of the shareholder."
    			}
    		},
    		SignatureData: {
    			title: "signature data",
    			digitalSignature: {
    				title: "digital signature"
    			},
    			email: {
    				title: "email"
    			},
    			isPep: {
    				title: "is pep"
    			},
    			isProcura: {
    				title: "is procura",
    				descr: "The account is authorized to sign on behalf of the actual account."
    			},
    			personName: {
    				title: "person name",
    				descr: "Name of the signing person.\nBy default, this is Onboarding Person, but it may be a completely different person."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID of the person, who has the right to sign."
    			},
    			telephone: {
    				title: "telephone"
    			}
    		},
    		SmsMessage: {
    			title: "sms message",
    			descr: "API SmsMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (characters or numbers)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (phone)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SubsidisedCommuteUsecase: {
    			title: "subsidised commute usecase",
    			descr: "Handles the Calculation of CalculationRowType.SubsidisedCommute to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			periodDivider: {
    				title: "period divider",
    				descr: "Number of periods if Kind is PeriodicalDeduction.\nOtherwise this value is null."
    			}
    		},
    		TaxCardIncome: {
    			title: "tax card income",
    			descr: "Defines income that is recorded in a Taxcard: A paid salary calulation, income from previous employers or other salaries that affect the income calculation.",
    			description: {
    				title: "description",
    				descr: "A short description of the paid calculation.\nThis is a mandatory field if the Id is null."
    			},
    			diff: {
    				title: "diff",
    				descr: "When loading calculations from the database (in taxcard Save and FindBy with diff),\nthis property identifies the differences in the entry."
    			},
    			endDate: {
    				title: "end date",
    				descr: "End of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkEndDate, but may be different in edge cases."
    			},
    			id: {
    				title: "id",
    				descr: "Calculation id in the Salaxy database.\nIf the calculation is not in the Salaxy Database, leave this null."
    			},
    			income: {
    				title: "income",
    				descr: "Taxable income from this calculation"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the row, typically the employer that paid the salary.\nNull meaning that it is the owner of the original taxcard."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "The paidAt date which tells mainly to which year the payment should be attributed.\nThis is reliable only if the salary is paid by Salaxy (TaxCardIncomeType.SalaxyCalculation).\nIn other cases it is just the reporting date."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Beginning of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkStartDate, but may be different in edge cases."
    			},
    			tax: {
    				title: "tax",
    				descr: "Widthholding tax in euro"
    			},
    			type: {
    				title: "type",
    				descr: "Type of income"
    			}
    		},
    		TaxProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Service related to integration, reporting and payments to tax authorities.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		TaxSettings: {
    			title: "tax settings",
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			}
    		},
    		Taxcard: {
    			title: "taxcard",
    			descr: "Full tax card that is stored in database and that is used in tax card percentage calculations. Also contains the salary history in the income log.",
    			card: {
    				title: "card",
    				descr: "The input that is coming from the Worker / Tax authorities"
    			},
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"[Server]/[PayloadType]/[Owner]/[Id]\", e.g. \"https://test-secure.salaxy.com/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment for which the tax card is created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			incomeLog: {
    				title: "income log",
    				descr: "Calculations that have been paid to this tax card."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "Read-only calculation results and other logic values on the Taxcard."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker, whose salaries are paid based on this tax card."
    			}
    		},
    		TaxcardData: {
    			title: "taxcard data",
    			descr: "Tax card specific business data for index.",
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"/[PayloadType]/[Owner]/[Id]\", e.g. \"/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the taxcard validity. This may be null on NoTaxcard types."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "The type of the tax card."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the taxcard validity"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income paid to the taxcard during the year (forYear)."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity resolved as enumeration in relation to payment date given in the query.\nThis is not available in direct database OData queries as it is relevant to current / given date."
    			}
    		},
    		TaxcardInput: {
    			title: "taxcard input",
    			descr: "Input from user for a tax card: What is written in the card plus the state infomration that is directly editable by the user.",
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			state: {
    				title: "state",
    				descr: "Tax card approval state"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardListItem: {
    			title: "taxcard list item",
    			descr: "List item of taxcard searches.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		TaxcardResult: {
    			title: "taxcard result",
    			descr: "Provides the results of the tax card calculation and business logic.",
    			tax: {
    				title: "tax",
    				descr: "Provides a decimal number for a recommended widthholding tax IF it is possible to calculate that based on given information.\nNull, if the information is not sufficient to give a recommendation."
    			},
    			yearlyPreviousIncome: {
    				title: "yearly previous income",
    				descr: "Sum of income for this year based on the items marked on the tax card."
    			}
    		},
    		TaxcardResults: {
    			title: "taxcard results",
    			descr: "Read-only calculations and other logic methods on a Taxcard.",
    			payment: {
    				title: "payment",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for the actual year, which is used for income limit.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			paymentJanuary: {
    				title: "payment january",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income for the actual year, which is used for income limit.\nNote that this does not include January income that may be paid for previous year taxcard."
    			},
    			totalIncomeJanuary: {
    				title: "total income january",
    				descr: "Total income for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January."
    			},
    			validityToday: {
    				title: "validity today",
    				descr: "Gets the validity of the taxcard today. This value reflects purely the date value - it does not take into account approval or replacement."
    			}
    		},
    		TaxcardSnapshot: {
    			title: "taxcard snapshot",
    			descr: "Basic information about the taxcard: This object has the infromation printed in SalarySlip etc. but not the full salaries paid to the taxcard etc.",
    			cumulativeIncome: {
    				title: "cumulative income",
    				descr: "Cumulative income on the taxcard - including this calculation.\nNote that with shared taxcards, this may include income from other employers."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit.\nThis is percent. I.e. for 50% set 50, not 0.5."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardUiInfo: {
    			title: "taxcard ui info",
    			descr: "Display information and language versioned texts that make sense to\nend-users about the status of taxcard.",
    			kind: {
    				title: "kind",
    				descr: "Kind that is quaranteed to be non-null (undefined). This is about the existing taxcard,\nthere may still be a new card \"waiting\" and that may have another kind specified."
    			},
    			percent: {
    				title: "percent",
    				descr: "The tax percent if it is relevant for the taxcard.\nNumber is 0-60, not decimal e.g. 0.60. Null if not relevant."
    			},
    			shortText: {
    				title: "short text",
    				descr: "Shorter version of the descriptive text to the end user."
    			},
    			status: {
    				title: "status",
    				descr: "Status as in\n\"ok\": Valid taxcard,\n\"missing\": No taxcard, employer should add a new one.,\n\"waiting\": A shared taxcard is waiting for approval. Existing may be valid or not.\nor null: No calculation or employment relation specified."
    			},
    			text: {
    				title: "text",
    				descr: "Descriptive text to the end user containing the percent if relevant."
    			}
    		},
    		TaxcardWaitingApproval: {
    			title: "taxcard waiting approval",
    			descr: "Informs the employer that there is a Worker taxcard waiting for approval potentially without revealing personal information.",
    			createdAt: {
    				title: "created at",
    				descr: "Time when the taxcard was added."
    			},
    			fullCard: {
    				title: "full card",
    				descr: "If there is an owner instance of taxcard for the employer, the copy is added here.\nInstance exists only if there is a confirmed employment relation or if employer explicitly requests the taxcard."
    			},
    			isRelationUnconfirmed: {
    				title: "is relation unconfirmed",
    				descr: "If true, the relation has not yet been confirmed:\nI.e. the employer has just input the Finnish Personal Id (HETU) of the Worker.\nThe employment relation is confirmed when salaries are paid or salary request is received from Worker."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner account that added the taxcard."
    			},
    			state: {
    				title: "state",
    				descr: "Status of the taxcard"
    			},
    			uri: {
    				title: "uri",
    				descr: "URI for the shared card: This is the taxcard owned by the Worker and to view it\nyou need to request your own copy (would be in FullCard)."
    			}
    		},
    		TaxcardsDataRow: {
    			title: "taxcards data row",
    			descr: "Data row for taxcards data. This is typically used for imports and exports.",
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the taxcard"
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used. Any income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tax authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		TesParameters: {
    			title: "tes parameters",
    			descr: "Captures the collection of parameters that different Framework agreements (TES) have.",
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses amount"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Travel expences per day"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "The amount of kilometers travelled by day - used to define the amount of travel expenses in aomse framework agreements."
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "Marker that the payments defined by framework agreement are included in the salary (will not be added by the calculation)."
    			},
    			isYksityisenHoidonTuki: {
    				title: "is yksityisen hoidon tuki",
    				descr: "If true will calculate the Finnish child care subsidy"
    			},
    			numberOfDays: {
    				title: "number of days",
    				descr: "Number of days may affect calculation of different payments calculated based on a Framework agreement"
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype when one framework has several options"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the work framework"
    			},
    			yksityisenHoidonTukiAmount: {
    				title: "yksityisen hoidon tuki amount",
    				descr: "Amount of Finnish child care subsidy"
    			}
    		},
    		TestAccountInfo: {
    			title: "test account info",
    			descr: "Object containing essential details of the test account",
    			token: {
    				title: "token",
    				descr: "Bearer token for authentication."
    			},
    			userCredential: {
    				title: "user credential",
    				descr: "The current user credentials."
    			}
    		},
    		TestValues: {
    			title: "test values",
    			descr: "Test object with value properties.",
    			arrayValue: {
    				title: "array value",
    				descr: "Test array"
    			},
    			boolValue: {
    				title: "bool value",
    				descr: "Test bool."
    			},
    			dateTimeValue: {
    				title: "date time value",
    				descr: "Test DateTime."
    			},
    			dateValue: {
    				title: "date value",
    				descr: "Test Date."
    			},
    			decimalValue: {
    				title: "decimal value",
    				descr: "Test decimal."
    			},
    			dictValue: {
    				title: "dict value",
    				descr: "Free values dictionary."
    			},
    			doubleValue: {
    				title: "double value",
    				descr: "Test double."
    			},
    			enumValue: {
    				title: "enum value",
    				descr: "Test enum"
    			},
    			intValue: {
    				title: "int value",
    				descr: "Test integer."
    			},
    			objectValue: {
    				title: "object value",
    				descr: "Test object."
    			},
    			stringValue: {
    				title: "string value",
    				descr: "Test string."
    			}
    		},
    		TotalCalculationDTO: {
    			title: "total calculation d t o",
    			descr: "Calculates the total numbers - the ones that are common to Employer and worker.",
    			pension: {
    				title: "pension",
    				descr: "The total pension payment for this calculation"
    			},
    			total: {
    				title: "total",
    				descr: "The total of all calculation rows"
    			},
    			totalAccidentInsuranceBase: {
    				title: "total accident insurance base",
    				descr: "Base income for calculating the Accident insurance contributions."
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "The base salary without overtime and other such additions that are part of gross salary."
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Tax free expenses."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary is the salary including additions like overtime, but not benefits or expenses"
    			},
    			totalHealthInsuranceBase: {
    				title: "total health insurance base",
    				descr: "Base income for calculating the Health insurance contributions."
    			},
    			totalPayable: {
    				title: "total payable",
    				descr: "Total sum of rows that are paid through Salaxy Customer Funds account.\nThis is the Gross sum before side costs, deductions etc., but it is not Gross salary as it includes expenses etc."
    			},
    			totalPensionInsuranceBase: {
    				title: "total pension insurance base",
    				descr: "Base income for calculating the Pension insurance contributions."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "DEPRICATED: This is now the same as TotalPensionInsuranceBase. For other social insurances, you may want to use\nTotalUnemploymentInsuranceBase, TotalHealthInsuranceBase or TotalAccidentInsuranceBase,"
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable salary is the salary from the taxation point of view - includes some benefits."
    			},
    			totalUnemploymentInsuranceBase: {
    				title: "total unemployment insurance base",
    				descr: "Base income for calculating the Unemployment insurance contributions."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The total unemployment payment sum."
    			}
    		},
    		TypeDocType: {
    			title: "type doc type",
    			descr: "Simplified read-only interface for typedoc type.",
    			elementType: {
    				title: "element type",
    				descr: "Element type in array"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier in reference"
    			},
    			name: {
    				title: "name",
    				descr: "Name of type in intrinsic and typeParameter"
    			},
    			type: {
    				title: "type",
    				descr: "Type of type"
    			},
    			typeArguments: {
    				title: "type arguments",
    				descr: "Type arguments in reference"
    			},
    			types: {
    				title: "types",
    				descr: "Collection of types in union"
    			},
    			value: {
    				title: "value",
    				descr: "Value if literal"
    			}
    		},
    		UnemploymentProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Product that helps employer to handle the mandatory unemployment insurance reports and payments.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isNotificationsSelfHandling: {
    				title: "is notifications self handling",
    				descr: "If true, the account himself handles the unemployment insurance notifications."
    			},
    			isPrepayments: {
    				title: "is prepayments",
    				descr: "If true, the account has made prepayments or is registered to make prepayments.\nPalkkaus needs to contact the account."
    			},
    			isSalariesPaidThisYear: {
    				title: "is salaries paid this year",
    				descr: "If true, the account has paid salaries this year outside Palkkaus.fi.\nThis affects the yearly reporting - Palkkaus needs to contact the account.\nNote that this is a shared option with the taxation product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		UnionPaymentUsecase: {
    			title: "union payment usecase",
    			descr: "Handles the Calculation of CalculationRowType.UnionPayment to IrRows",
    			isDeductedAtBackoffice: {
    				title: "is deducted at backoffice",
    				descr: "If true, deduction is done in Salaxy backoffice, not directly by employer.\nAs of writing, this is only done for Rakennusliitto / Construction."
    			},
    			isFixedSum: {
    				title: "is fixed sum",
    				descr: "If true, the value is a fixed sum: the price should not be updated based on salary and\nthe count should not be updated based on percentage."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Usecase: {
    			title: "usecase",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UsecaseChildCare: {
    			title: "usecase child care",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isChildcareSubsidy: {
    				title: "is childcare subsidy",
    				descr: "If true, the salary calculation includes child care subsidy paid by the government / community."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isSunday: {
    				title: "is sunday",
    				descr: "If true, the work is done on Sunday or public holiday.\nThis means that the salary is increased by 100% according to Finnish law"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			subsidyAmount: {
    				title: "subsidy amount",
    				descr: "Amount for the child care subsidy paid by the government / community."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			useCustomPrice: {
    				title: "use custom price",
    				descr: "If true, a custom price is used instead of the recommended price.\nCurrently, this only affects the MLL case as the normal childCare price is always set by the user."
    			}
    		},
    		UsecaseCleaning: {
    			title: "usecase cleaning",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isContractLessThanYear: {
    				title: "is contract less than year",
    				descr: "If salaries are compensated (when holidays are not kept, isFullTime=false),\nthe percentage is different depending on whether the person has been working for a full year or not."
    			},
    			isFullTime: {
    				title: "is full time",
    				descr: "If true, this is a full time contract in the respect of holidays:\nHolidays are kept during summer and salary is paid for these holidays."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseConstruction: {
    			title: "usecase construction",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses (työkalukorvaus) if isExpensesCustom=true"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Daily travel expenses if isExpensesCustom=true"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "Daily distance (km) from worker home to construction site."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isExpensesCustom: {
    				title: "is expenses custom",
    				descr: "If true, the expenses are agreed in a bispoken way.\nDefault is that expenses are based on Framework agreement (TES) tables."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "If true, the TES additions are included in the agreed salary."
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseData: {
    			title: "usecase data",
    			descr: "A usecase for creating and editing salary calculations and other . In practice, a usecase is either a user interface or a microservice that provides the bases for the object. Provides the possibility to store useacase specific data as part of object. NOTE: For Calculations, this functionality will replace TesParameters in the 3.0 version.",
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			description: {
    				title: "description",
    				descr: "Optional description of the use case for the end user.\nShown in detail views when explaining the logic of the usecase."
    			},
    			label: {
    				title: "label",
    				descr: "A short label of that can be shown to the end user."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this may resolve to a user interface, microservice or a to manifest registering one.\nFor now, just use a unique string starting with a domain you own."
    			}
    		},
    		UsecaseGroup: {
    			title: "usecase group",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case group (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG)"
    			},
    			children: {
    				title: "children",
    				descr: "Collection of use cases - in the future, may also support sub groups."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG )"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the group"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UserDefinedRow: {
    			title: "user defined row",
    			descr: "This is a user defined calculation row as opposed to CalculationRow that may be either based on a user defined row or generated by the system based on e.g. TES parameters.",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowIndex: {
    				title: "row index",
    				descr: "Zero based row index that can be used to match the row to result rows or when deleting.\nNOTE: The Row ID is given values on the fly so setting this will not have any effect at the moment.\nIt may reorder the entries in later implementations."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is passed to the result calculations."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. Guessed based on the RowType and count, but you are better of setting it to be sure."
    			}
    		},
    		UserSession: {
    			title: "user session",
    			descr: "Passes the data about the authorization that is the bases for callin the services that require user authorization.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar that should be shown for the current session.\nTypically CurrentAccount.Avatar, but could theoretically be something different."
    			},
    			currentAccount: {
    				title: "current account",
    				descr: "The current Palkkaus.fi-account that the user acting as"
    			},
    			currentCredential: {
    				title: "current credential",
    				descr: "The current user credentials."
    			},
    			currentPartner: {
    				title: "current partner",
    				descr: "The primary partner of the current account."
    			},
    			hasCredential: {
    				title: "has credential",
    				descr: "If false, the current user does not have a credential in the system =&gt; need to redirect to login"
    			},
    			hasCurrentAccount: {
    				title: "has current account",
    				descr: "If false, the current user does not have a current account.\nIf HasCredential is true, this means that the user should be redirected to Account creation wizard."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, the current user needs to be authenticated or authenticated user does not have acces to the system."
    			},
    			requestingPartnerSite: {
    				title: "requesting partner site",
    				descr: "The site from which the API call is being made."
    			},
    			status: {
    				title: "status",
    				descr: "Authorization status"
    			},
    			systemAlertHtml: {
    				title: "system alert html",
    				descr: "Contains html for system alert."
    			}
    		},
    		VarmaPensionOrder: {
    			title: "varma pension order",
    			descr: "Contains order data for Varma pensions (TyEL, YEL) and required operations.",
    			action: {
    				title: "action",
    				descr: "Action for the order."
    			},
    			orderer: {
    				title: "orderer",
    				descr: "Orderer information."
    			},
    			tyel: {
    				title: "tyel",
    				descr: "TyEL data for the order"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation information."
    			},
    			yel: {
    				title: "yel",
    				descr: "YEL data for the order"
    			}
    		},
    		VarmaPensionOrderTyel: {
    			title: "varma pension order tyel",
    			descr: "Varma pension order TyEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startMonth: {
    				title: "start month",
    				descr: "Start month for the new order."
    			},
    			startYear: {
    				title: "start year",
    				descr: "Start year for the new order."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yerly income."
    			}
    		},
    		VarmaPensionOrderYel: {
    			title: "varma pension order yel",
    			descr: "Varma pension order YEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			isYearlyIncomeChange: {
    				title: "is yearly income change",
    				descr: "Indicates if the existing yearly income estimate will be changed."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer definition."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date for the new pension."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yearly income."
    			}
    		},
    		VismaSignPostData: {
    			title: "visma sign post data",
    			descr: "Data for building the Visma Sign API request form. This is the legacy API v0.2: https://allekirjoitus.visma.fi/download/visma_sign-api.pdf",
    			auth_service: {
    				title: "auth_service",
    				descr: "Specifies the Tupas authentication service (e.g. 'osuuspankki', 'nordea').\nIf not specified, Visma Sign will show its own selection screen."
    			},
    			customer: {
    				title: "customer",
    				descr: "Palkkaus.fi Customer code to Visma Sign-system"
    			},
    			data: {
    				title: "data",
    				descr: "Rest of the data as encrypted JSON."
    			},
    			documentUrl: {
    				title: "document url",
    				descr: "Document URL for preview purposes.\nTo Visma Sign, this is sent within the encrypted data field."
    			},
    			iv: {
    				title: "iv",
    				descr: "Encryption Initialization vector"
    			},
    			return_failure: {
    				title: "return_failure",
    				descr: "Failure address - needs to be non-encrypted, because the error may be in the encryption"
    			}
    		},
    		WelcomeDataWorker: {
    			title: "welcome data worker",
    			descr: "Data that is used for displaying the welcome page for the Worker account.",
    			activeTaxcard: {
    				title: "active taxcard",
    				descr: "Active taxcard added by the Worker or null, if none is added (or none is valid for today)."
    			},
    			invitation: {
    				title: "invitation",
    				descr: "Invitation for the worker to store a taxcard for salary payment."
    			},
    			timeline: {
    				title: "timeline",
    				descr: "The welcome page timeline cards."
    			}
    		},
    		WorkDescription: {
    			title: "work description",
    			descr: "Describes the work that is beign performed - mainly for Employment relation purposes.",
    			description: {
    				title: "description",
    				descr: "Free text description of the work that is being performed."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			salaryDefaults: {
    				title: "salary defaults",
    				descr: "Contains default calculation rows for the worker.\nTypically used for setting for example monthly salary or hourly salary price for workers."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Default bases for wage."
    			}
    		},
    		WorkerAbsencePeriod: {
    			title: "worker absence period",
    			descr: "Absence Period with Worker information.",
    			absencesId: {
    				title: "absences id",
    				descr: "WorkerAbsences storage object that contains the period"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			},
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker that owns the Absence period"
    			}
    		},
    		WorkerAbsences: {
    			title: "worker absences",
    			descr: "Absences for a single Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation id for the worker whose absences are recorded."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			periods: {
    				title: "periods",
    				descr: "Collection of absence periods."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Account ID for the worker whose absences are recorded."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			}
    		},
    		WorkerAccount: {
    			title: "worker account",
    			descr: "Account data that is relevant for a Worker.",
    			ageRange: {
    				title: "age range",
    				descr: "Age range that is relevant to workers salary calculation and/or employment.\nSet this especially if OfficialPersonId is not set.\nIf OfficialPersonId is valid, this property is always set based on it."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "If created (and still owned) by an employer, this is the AccountOwnerId of the employer who created this account."
    			},
    			employment: {
    				title: "employment",
    				descr: "Current Employment Relation: The data that is owned by the Employer about the Worker\neven if the Worker has registered and created an account.\nIf employer has several Employment Relations, this is the one that is marked as Primary."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The unique id for the employment. This does not change within one employment."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			officialPersonId: {
    				title: "official person id",
    				descr: "The Person ID of the worker if known."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides validation errors currently only in Save operations,\nbut potentially in the future also in detail/list views."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "Information needed when salary is paid to this account and other such stuff about the account as a worker."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		WorkerAccountEmployment: {
    			title: "worker account employment",
    			descr: "Employment Relation as it is shown as part of WorkerAccount.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isAccountOwner: {
    				title: "is account owner",
    				descr: "If true, the worker is the same person as the actual signed owner of the account."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		WorkerCalculationDTO: {
    			title: "worker calculation d t o",
    			descr: "The Worker side of the calculation. SalarySlip is largely written based on this calculation.",
    			benefits: {
    				title: "benefits",
    				descr: "The benefits as valued by the tax authority."
    			},
    			deductions: {
    				title: "deductions",
    				descr: "All the items that are deducted from the salary (this is also the order if there is not enough payable money):\nSalaryAdvance + Tax + Pension + UnemploymentInsurance + UnionPayment + OtherDeductions"
    			},
    			foreclosure: {
    				title: "foreclosure",
    				descr: "The foreclosure that been deducted from the salary by the employer."
    			},
    			fullOtherDeductions: {
    				title: "full other deductions",
    				descr: "Other deductions that are deducted from the net salary.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullPension: {
    				title: "full pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullPrepaidExpenses: {
    				title: "full prepaid expenses",
    				descr: "Prepaid expenses that are deducted from the expenses to be paid to the worker.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullSalaryAdvance: {
    				title: "full salary advance",
    				descr: "The salary advance that was paid to the Worker\nThe original calulated version - there might not be enough money to finally pay this."
    			},
    			fullTax: {
    				title: "full tax",
    				descr: "Widthholding tax: The original calulated version - there might not be enough money to finally pay this."
    			},
    			fullUnemploymentInsurance: {
    				title: "full unemployment insurance",
    				descr: "Workers part of the unemployment insurance.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullUnionPayment: {
    				title: "full union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			otherDeductions: {
    				title: "other deductions",
    				descr: "Part of the other deductions that are deducted from the net salary."
    			},
    			pension: {
    				title: "pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary."
    			},
    			prepaidExpenses: {
    				title: "prepaid expenses",
    				descr: "Part of the prepaid expenses that are deducted from the expenses."
    			},
    			salaryAdvance: {
    				title: "salary advance",
    				descr: "The salary advance that was paid to the Worker"
    			},
    			salaryAfterTax: {
    				title: "salary after tax",
    				descr: "Net salary after tax - before worker foreclosure or deductions of the side costs"
    			},
    			salaryAfterTaxAndForeclosure: {
    				title: "salary after tax and foreclosure",
    				descr: "Net salary after tax and foreclosure."
    			},
    			salaryPayment: {
    				title: "salary payment",
    				descr: "The payment that is made to the Worker based on salary after all deductions.\nDoes not include expenses."
    			},
    			tax: {
    				title: "tax",
    				descr: "Ennakonpidätys: The taxes that were deducted from the worker salary."
    			},
    			totalWorkerPayment: {
    				title: "total worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			},
    			totalWorkerPaymentLegacy: {
    				title: "total worker payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "Workers part of the unemployment insurance."
    			},
    			unionPayment: {
    				title: "union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			workerSideCosts: {
    				title: "worker side costs",
    				descr: "The side costs that are deducted from the Worker salary according to law:\nPension, Unemployment and Union payment. Tax is not included in this number.\nThis is not an official number, really used in the charts etc."
    			}
    		},
    		WorkerCurrentTaxcards: {
    			title: "worker current taxcards",
    			descr: "Contains the status and necessary data with Workers current taxcards.",
    			active: {
    				title: "active",
    				descr: "The currently active taxcard if one exists. This property is null by default."
    			},
    			employers: {
    				title: "employers",
    				descr: "Employer taxcards for the Worker account: One for each employment relationship."
    			},
    			employment: {
    				title: "employment",
    				descr: "Employment relation if the search is done based on employment relation ID and not Current worker account."
    			},
    			list: {
    				title: "list",
    				descr: "All the relevant taxcards as list items."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID that is used for taxcard binding. This always exists as opposed to Employment."
    			},
    			waitingApproval: {
    				title: "waiting approval",
    				descr: "A shared taxcard waiting for approval if one exists. This property is null by default."
    			}
    		},
    		WorkerData: {
    			title: "worker data",
    			descr: "Worker specific business data for index.",
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation."
    			}
    		},
    		WorkerInfo: {
    			title: "worker info",
    			descr: "Information directly related to Employment (as Worker)",
    			defaultTaxCard: {
    				title: "default tax card",
    				descr: "The Tax card that is used by default."
    			}
    		},
    		WorkerListItem: {
    			title: "worker list item",
    			descr: "Represents a single item in a list of workers in the user interface. It contains all essential information of the worker and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		WorkerProfileProperties: {
    			title: "worker profile properties",
    			dateOfBirth: {
    				title: "date of birth"
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy"
    			},
    			gender: {
    				title: "gender"
    			},
    			hasDriversLisence: {
    				title: "has drivers lisence"
    			},
    			hasTaxCard: {
    				title: "has tax card"
    			}
    		},
    		WorkerSummary: {
    			title: "worker summary",
    			descr: "Year end summary for one Worker account.",
    			calculations: {
    				title: "calculations",
    				descr: "Calculations that have been paid through Salaxy system."
    			},
    			corrections: {
    				title: "corrections",
    				descr: "Corrections that are made to existing calculations as well as\nsalary calculations that are paid outside Salaxy."
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes when set on worker level, not for each calculation separately."
    			},
    			summary: {
    				title: "summary",
    				descr: "Total of Calculations and Corrections as calculatied by the server side logic."
    			},
    			worker: {
    				title: "worker",
    				descr: "Worker account to which the salaries have been paid."
    			}
    		},
    		WorkersDataRow: {
    			title: "workers data row",
    			descr: "Data row for employments data. This is typically used for imports and exports.",
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the worker"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		WorkflowData: {
    			title: "workflow data",
    			descr: "Defines the data that IWorkflowItem must provide.",
    			events: {
    				title: "events",
    				descr: "Workflow events for the item."
    			}
    		},
    		WorkflowEvent: {
    			title: "workflow event",
    			descr: "Event in the workflow",
    			assignedTo: {
    				title: "assigned to",
    				descr: "Credential of the user which is responsible of the event."
    			},
    			features: {
    				title: "features",
    				descr: "Features for this event."
    			},
    			id: {
    				title: "id",
    				descr: "Event-provider specific identifier for the event.\nFor deletable events, this should identify , but do NOT trust this to be 100% unique."
    			},
    			message: {
    				title: "message",
    				descr: "Message for backoffice purposes from the User or system that created the event."
    			},
    			time: {
    				title: "time",
    				descr: "UTC time for when the event occured."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the event. This typically maps to parent-object specific enumeration."
    			},
    			ui: {
    				title: "ui",
    				descr: "User Interface styling of the event: Indicates the severity for the end-user."
    			},
    			user: {
    				title: "user",
    				descr: "User or system that created the event."
    			}
    		},
    		WorkingTimeCompensationUsecase: {
    			title: "working time compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.WorkingTimeCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		YearlyChangingNumbers: {
    			title: "yearly changing numbers",
    			descr: "Parameters that are changing yearly. This class is made for the change of the year and it is typically updated in December. It is not for recording longer history of numbers.",
    			holidays: {
    				title: "holidays",
    				descr: "Returns holidays for the year."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "Returns yearly sidecosts."
    			},
    			year: {
    				title: "year",
    				descr: "Year for numbers."
    			}
    		},
    		YearlyFeedback: {
    			title: "yearly feedback",
    			descr: "The feedback from the account holder at the end-of-year. Possibility to feed in changes, corrections and salaries that are paid outside the Salaxy system.",
    			adminNotes: {
    				title: "admin notes",
    				descr: "Full description of yearly process status for backoffice pusposes. Shown in backoffice detail views."
    			},
    			adminShortNote: {
    				title: "admin short note",
    				descr: "Short notice for backoffice purposes. This is the text visible in the lists."
    			},
    			adminStatus: {
    				title: "admin status",
    				descr: "Status for administrator check."
    			},
    			bankIban: {
    				title: "bank iban",
    				descr: "Bank IBAN number for yearly refunding of extra customer assets.\nREADONLY: Note that once set to a valid IBAN, this number is read-only and must be changed through customer service or onboarding (requires authentication)."
    			},
    			employerAvatar: {
    				title: "employer avatar",
    				descr: "Avatar for the employer of display purposes in reports etc."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "Account ID for the employer."
    			},
    			externalWorkerInsuranceWork: {
    				title: "external worker insurance work",
    				descr: "Temporary user interface helper for external workers (insurance) =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			externalWorkerSalaries: {
    				title: "external worker salaries",
    				descr: "Temporary user interface helper for external workers =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			household: {
    				title: "household",
    				descr: "Yearly feedback data that is specific to household customers.\nMain part is the information realted to household deduction and how it can be divided between spouses."
    			},
    			sendInsuranceReport: {
    				title: "send insurance report",
    				descr: "If true, Salaxy should report the salaries to Insurance company.\nCurrently, this is available only for Companies and for If insurance company only."
    			},
    			userFeedbackText: {
    				title: "user feedback text",
    				descr: "Free text from the account holder to Salaxy backoffice."
    			},
    			userFeedbackTime: {
    				title: "user feedback time",
    				descr: "Time when the feedback was given."
    			},
    			userStatus: {
    				title: "user status",
    				descr: "The satus that the account holder gives to yearly material."
    			},
    			workerSummaries: {
    				title: "worker summaries",
    				descr: "List of Workers for which salaries have been paid during the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the feedback"
    			}
    		},
    		YearlyHolidays: {
    			title: "yearly holidays",
    			descr: "Holidays for the entire year.",
    			holidays: {
    				title: "holidays",
    				descr: "Holidays for the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the holidays."
    			}
    		},
    		YearlySideCosts: {
    			title: "yearly side costs",
    			descr: "Encapsulates yearly side costs into one class.",
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction"
    			},
    			illnessInsurancePercent: {
    				title: "illness insurance percent",
    				descr: "Illness Insurance care percent that is withhold from the Worker. (vakuutetun sairausvakuutusmaksu: päivärahamaksu + sairaanhoitomaksu"
    			},
    			palkkausVatPercent: {
    				title: "palkkaus vat percent",
    				descr: "ALV-% for the fee."
    			},
    			socSecPercent: {
    				title: "soc sec percent",
    				descr: "Social Security payment for employer (SAVA-maksu)"
    			},
    			taxFreeDailyAllowance: {
    				title: "tax free daily allowance",
    				descr: "Tax free daily allowance for full day approved by tax authorities."
    			},
    			taxFreeDailyHalfAllowance: {
    				title: "tax free daily half allowance",
    				descr: "Tax free daily allowance for half day approved by tax authorities."
    			},
    			taxFreeKmAllowance: {
    				title: "tax free km allowance",
    				descr: "Tax free milage allowance approved by tax authorities."
    			},
    			taxFreeMealAllowance: {
    				title: "tax free meal allowance",
    				descr: "Tax free meal allowance approved by tax authorities."
    			},
    			tyel53Percent: {
    				title: "tyel53 percent",
    				descr: "Pension payment base percentage - deducted from 53-62 year old Worker"
    			},
    			tyelBasePercent: {
    				title: "tyel base percent",
    				descr: "Pension payment base percentage"
    			},
    			tyelContractEmployerLimit: {
    				title: "tyel contract employer limit",
    				descr: "Pension contract employer six month limit"
    			},
    			tyelIncomeLimit: {
    				title: "tyel income limit",
    				descr: "Pension payment income limit per month"
    			},
    			tyelWorkerPercent: {
    				title: "tyel worker percent",
    				descr: "Pension payment base percentage - deducted from Worker salary"
    			},
    			unemploymentEmployerPercent: {
    				title: "unemployment employer percent",
    				descr: "Unemployment insurance payment percent - Employer"
    			},
    			unemploymentPartialOwnerWorkerPercent: {
    				title: "unemployment partial owner worker percent",
    				descr: "Unemployment insurance payment percent for partial owner worker - deducted from Worker"
    			},
    			unemploymentWorkerPercent: {
    				title: "unemployment worker percent",
    				descr: "Unemployment insurance payment percent - deducted from Worker"
    			},
    			unionPaymentRaksaA: {
    				title: "union payment raksa a",
    				descr: "Union payment: Rakennusliitto, A (typical)"
    			},
    			unionPaymentRaksaAoTa: {
    				title: "union payment raksa ao ta",
    				descr: "Union payment: Rakennusliitto, AO and TA"
    			}
    		},
    		YtjCompany: {
    			title: "ytj company",
    			descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			companyTypeText: {
    				title: "company type text",
    				descr: "YTJ's legal type of the company as a string. This is only to be used for display purposes.\nSource: YritysHaku.YritysMuoto"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the"
    			},
    			contactPerson: {
    				title: "contact person",
    				descr: "Names are not necessarily avaiable. If they are, they may not be reliable."
    			},
    			contactPersonPersonalId: {
    				title: "contact person personal id",
    				descr: "Social security number of the contact person in the company, if available."
    			},
    			contactPhone: {
    				title: "contact phone",
    				descr: "Mobile for or other company phone.\nDifference between Contact.Phone and ContactPhone is that\nthis (ContactPhone) is first mobile number, then company general phone if the former was null.\nContact.Phone is fisrt general phone number and mobile number only if not available."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company.\nSource: Tiedostonimi.Tiedostonimi"
    			},
    			sectorOfBusinessCode: {
    				title: "sector of business code",
    				descr: "The sector where the company is active. (Toimialatieto.Koodi)"
    			},
    			sectorOfBusinessText: {
    				title: "sector of business text",
    				descr: "Text of the sector where company operates."
    			},
    			www: {
    				title: "www",
    				descr: "Web site address"
    			},
    			ytjCompanyType: {
    				title: "ytj company type",
    				descr: "The internal YTJ code number for the company's legal type."
    			}
    		},
    		YtjSearchResult: {
    			title: "ytj search result",
    			descr: "Search result for free text search from the Finnish Business Infromation System (YTJ/BIS). Returns a list of companies without details.",
    			counter: {
    				title: "counter",
    				descr: "API quota counter"
    			},
    			isSuccess: {
    				title: "is success",
    				descr: "If true, search is a success. This may mean that no results were found though."
    			},
    			isTooGeneric: {
    				title: "is too generic",
    				descr: "If true, the searh is too generic.\nTypically either there was more than 200 results or less than 3 characters in the search string.\nBoth of these return an error in the API."
    			},
    			rawInput: {
    				title: "raw input",
    				descr: "The original search string - without trimming etc. that are done in the actual search."
    			},
    			result: {
    				title: "result",
    				descr: "Result of the search."
    			},
    			status: {
    				title: "status",
    				descr: "Technical status - for developers / admin."
    			},
    			userMessage: {
    				title: "user message",
    				descr: "A message that can be shown to the end user."
    			}
    		},
    		YtjSearchResultCompany: {
    			title: "ytj search result company",
    			descr: "Search result list level of detail of companies coming from YTJ.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			id: {
    				title: "id",
    				descr: "Numeric ID (\"Yritysavain\")"
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the company. This is a free text type. Not necessary parseable to company type enumeration in details."
    			}
    		},
    		CoreDictionaryLang: {
    			title: "core dictionary lang",
    			descr: "Defines the current structure for the core dictionary for each language.\nLanugages are fi, en, sv and sx.",
    			SALAXY: {
    				title: "s a l a x y",
    				descr: "SALAXY is the dictionary root object for each language."
    			}
    		}
    	}
    };

    var SALAXY$d = {
    	UI_TERMS: {
    		add: "Lisää",
    		addNew: "Lisää uusi",
    		addRow: "Lisää rivi",
    		areYouSure: "Oletko varma?",
    		back: "Takaisin",
    		calculate: "Laske",
    		calculations: "Laskelmat",
    		cancel: "Peruuta",
    		cancelEditing: "Peru muokkaaminen",
    		choose: "Valitse...",
    		chooseWorkDateRange: "Määritä työn alku- ja loppupäivät",
    		chooseWorkDuration: "Määritä työn kesto...",
    		clear: "Tyhjennä",
    		close: "Sulje",
    		confirm: "Vahvista",
    		copy: "Kopio",
    		copyAsNew: "Kopioi uudeksi",
    		copyAsRequest: "Kopio pyynnöksi",
    		copyToClipboard: "Kopioi",
    		dateFormats: {
    			dm: "pv.kk."
    		},
    		dateNotChosen: "päivämäärä ei valittuna",
    		"delete": "Poista",
    		demo: "KOKEILUVERSIO",
    		dismiss: "Hylkää",
    		done: "Valmis",
    		edit: "Muokkaa",
    		gotIt: "Ymmärrän",
    		isDeleting: "Poistetaan...",
    		isSaving: "Tallennetaan...",
    		learnMore: "Lue lisää",
    		loading: "Ladataan",
    		login: "Kirjaudu",
    		logout: "Kirjaudu ulos",
    		logoutAlt: "Päätä nykyinen sessio",
    		makeOrder: "Tee tilaus",
    		minCharacters: "Kirjoita vähintään 3 merkkiä...",
    		monthHalf: "½ kuukautta",
    		monthShort0: "joulu",
    		monthShort1: "tammi",
    		monthShort10: "loka",
    		monthShort11: "marras",
    		monthShort12: "joulu",
    		monthShort2: "helmi",
    		monthShort3: "maalis",
    		monthShort4: "huhti",
    		monthShort5: "touko",
    		monthShort6: "kesä",
    		monthShort7: "heinä",
    		monthShort8: "elo",
    		monthShort9: "syys",
    		moreInfo: "Lisätietoja",
    		navigation: "Navigaatio",
    		newCalculation: "Uusi laskelma",
    		next: "Seuraava",
    		no: "Ei",
    		noteRequiredField: "* Pakollinen kenttä",
    		noThanks: "Ei kiitos",
    		ok: "OK",
    		oldVersion: "vanha versio",
    		orChoose: "...tai valitse...",
    		otherPeriod: "Muu periodi",
    		pay: "Maksa",
    		pleaseWait: "Ole hyvä ja odota...",
    		postpone: "Ei nyt",
    		previous: "Edellinen",
    		printable: "Tuloste",
    		recalculate: "Päivitä laskelma",
    		register: "Rekisteröidy",
    		save: "Tallenna",
    		saveEdited: "Tallenna muutokset",
    		search: "Hae...",
    		searchByName: "Hae nimeltä...",
    		select: "Valitse",
    		selectAlt: "Valitse...",
    		sendEmail: "Lähetä sähköposti",
    		show: "Näytä",
    		skip: "Ohita",
    		sureToDeleteRecord: "Haluatko varmasti poistaa tämän tietueen?",
    		today: "Tämä päivä",
    		updateCalculation: "Päivitä laskelma",
    		updatingCalculation: "Päivitetään laskelmaa...",
    		use: "Ota käyttöön",
    		week2: "2 viikkoa",
    		yes: "Kyllä",
    		timeToday: "Tänään",
    		timeYesterday: "Eilen"
    	}
    };

    var SALAXY$c = {
    	VALIDATION: {
    		_notes: "Please note that all calc or payroll specific messages shoud be placed under Calc or Payroll -nodes. Messages in the Validation-root are obsolete.",
    		Files: {
    			ContentTypeError: "Tiedostomuotoa ei hyväksytä",
    			Empty: "Ladattava tiedosto on tyhjä",
    			MaxFileSize: "Ladattava tiedosto on liian suuri",
    			NoFiles: "Kutsun mukana ei tullut yhtään tiedostoa.",
    			SaveError: "Tuntematon virhe tiedoston tallennuksessa."
    		},
    		IllegalWorkerForCompanyType: {
    			description: "Yksityinen elinkeinonharjoittaja eli toiminimi ei voi maksaa itselleen palkkaa. Palkanmaksu hoidetaan yksityisnostoina. Kirjanpitäjäsi voi neuvoa sinua miten toimia.",
    			label: "Ei-sallittu palkka"
    		},
    		IncompleteAdvancePayment: {
    			description: "Palkkaennakko laskelmalla on liian suuri. Korjaa palkkaennakko pienemmäksi tai bruttopalkka suuremmaksi. Mahdolliset ennakkoon maksetut kulukorvaukset tulee ilmoittaa omalla palkkalajilla.",
    			label: "Palkka ei riitä ilmoitettuun palkkaennakkoon"
    		},
    		IncompleteForeclosure: {
    			description: "Rahapalkka ei riitä ulosmittauksen tekemiseen. Tarkista ulosmittauksen määrä maksukiellosta ja muista huomioida maksukiellossa ilmoitettu suojaosuus.",
    			label: "Palkka ei riitä ulosmittauksen tekemiseen"
    		},
    		IncompleteOtherDeductions: {
    			description: "Palkka ei riitä ilmoitettuihin muihin vähennyksiin",
    			label: "Palkka ei riitä"
    		},
    		IncompletePensionPayment: {
    			description: "Palkka ei riitä työntekijän osuuteen TyEL-maksusta",
    			label: "Palkka ei riitä"
    		},
    		IncompletePrepaidExpenses: {
    			description: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri. Ennakkoon maksettu kulukorvaus ylittää laskelmalle lisätyt kulukorvaukset.",
    			label: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri"
    		},
    		IncompleteUnemploymentInsurancePayment: {
    			description: "Palkka ei riitä työntekijän osuuteen työttömyysvakuutusmaksusta",
    			label: "Palkka ei riitä"
    		},
    		IncompleteUnionPayment: {
    			description: "Palkka ei riitä AY-jäsenmaksuun",
    			label: "Palkka ei riitä"
    		},
    		MissingOccupationCode: {
    			description: "Työn tyyppi puuttuu. Lisää se kohdassa 1 hakemalla Tilastokeskuksen luokituksesta. Ammattiluokitus on pakollinen tieto Tulorekisterin palkkatietoilmoituksessa.",
    			label: "Ammattiluokitus puuttuu"
    		},
    		MissingWorkerIbanNumber: {
    			label: "Tilinumero puuttuu",
    			description: "Työntekijän tiedoista puuttuu tilinumero (IBAN-muodossa)."
    		},
    		NegativeRowTotalNotSupported: {
    			description: "Rivin yhteenlaskettu summa on negatiivinen. Rivillä ei sallita negatiivista summaa.",
    			label: "Rivillä ei sallita negatiivista summaa"
    		},
    		Calc: {
    			IllegalWorkerForCompanyType: {
    				description: "Yksityinen elinkeinonharjoittaja eli toiminimi ei voi maksaa itselleen palkkaa. Palkanmaksu hoidetaan yksityisnostoina. Kirjanpitäjäsi voi neuvoa sinua miten toimia.",
    				label: "Ei-sallittu palkka"
    			},
    			IncompleteAdvancePayment: {
    				description: "Palkkaennakko laskelmalla on liian suuri. Korjaa palkkaennakko pienemmäksi tai bruttopalkka suuremmaksi. Mahdolliset ennakkoon maksetut kulukorvaukset tulee ilmoittaa omalla palkkalajilla.",
    				label: "Palkka ei riitä ilmoitettuun palkkaennakkoon"
    			},
    			IncompleteForeclosure: {
    				description: "Rahapalkka ei riitä ulosmittauksen tekemiseen. Tarkista ulosmittauksen määrä maksukiellosta ja muista huomioida maksukiellossa ilmoitettu suojaosuus.",
    				label: "Palkka ei riitä ulosmittauksen tekemiseen"
    			},
    			IncompleteOtherDeductions: {
    				description: "Palkka ei riitä ilmoitettuihin muihin vähennyksiin",
    				label: "Palkka ei riitä"
    			},
    			IncompletePensionPayment: {
    				description: "Palkka ei riitä työntekijän osuuteen TyEL-maksusta",
    				label: "Palkka ei riitä"
    			},
    			IncompletePrepaidExpenses: {
    				description: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri. Ennakkoon maksettu kulukorvaus ylittää laskelmalle lisätyt kulukorvaukset.",
    				label: "Ennakkoon maksettu kulukorvaus laskelmalla on liian suuri"
    			},
    			IncompleteUnemploymentInsurancePayment: {
    				description: "Palkka ei riitä työntekijän osuuteen työttömyysvakuutusmaksusta",
    				label: "Palkka ei riitä"
    			},
    			IncompleteUnionPayment: {
    				description: "Palkka ei riitä AY-jäsenmaksuun",
    				label: "Palkka ei riitä"
    			},
    			MissingOccupationCode: {
    				description: "Työn tyyppi puuttuu. Lisää se kohdassa 1 hakemalla Tilastokeskuksen luokituksesta. Ammattiluokitus on pakollinen tieto Tulorekisterin palkkatietoilmoituksessa.",
    				label: "Ammattiluokitus puuttuu"
    			},
    			MissingWorkerIbanNumber: {
    				label: "Työntekijän tilinumero puuttuu",
    				description: "Työntekijän tiedoista puuttuu tilinumero (IBAN-muodossa)."
    			},
    			InvalidWorkerIbanNumber: {
    				label: "Työntekijän tilinumero on virheellinen",
    				description: "Työntekijän tiedoissa on virheellinen tilinumero (IBAN-muodossa)."
    			},
    			NegativeRowTotalNotSupported: {
    				description: "Rivin yhteenlaskettu summa on negatiivinen. Rivillä ei sallita negatiivista summaa.",
    				label: "Rivillä ei sallita negatiivista summaa"
    			},
    			MissingWorkerContactStreet: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen katuosoite puuttuu"
    			},
    			MissingWorkerContactCity: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen postitoimipaikka puuttuu"
    			},
    			MissingWorkerContactPostalCode: {
    				description: "Työntekijän osoitetiedot ovat pakollisia, jos työntekijällä on keinotekoinen henkilötunnus. Keinotekoisen henkilötunnuksen loppuosa alkaa numerolla 9.",
    				label: "Työntekijän osoitteen postinumero puuttuu"
    			},
    			MissingWorker: {
    				label: "Laskelmalta puuttuu työntekijä"
    			},
    			MissingWorkerFirstName: {
    				label: "Työntekijän etunimi puuttuu"
    			},
    			MissingWorkerLastName: {
    				label: "Työntekijän sukunimi puuttuu"
    			},
    			MissingWorkerSocialSecurityNumber: {
    				label: "Työntekijän henkilötunnus puuttuu"
    			},
    			MissingWorkerSocialSecurityNumber_InTest: {
    				label: "Työntekijän henkilötunnus puuttuu. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    			},
    			InvalidWorkerSocialSecurityNumber: {
    				label: "Työntekijän henkilötunnus on virheellinen"
    			},
    			InvalidWorkerSocialSecurityNumber_InTest: {
    				label: "Työntekijän henkilötunnus on virheellinen. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    			},
    			InvalidCarBenefitRow: {
    				label: "Autoedusta puuttuu auton ikäryhmä ja/tai autoedun tyyppi"
    			},
    			InvalidRemunerationRow: {
    				label: "Kertaluontoisen palkkion tulolaji puuttuu"
    			}
    		},
    		Payroll: {
    			Calc: {
    				EmailAddress: {
    					label: "Sähköposti on virheellinen"
    				},
    				FirstNameNotEmpty: {
    					label: "Työntekijän etunimi puuttuu"
    				},
    				IbanInvalid: {
    					label: "Tilinumero on virheellinen (IBAN-muoto)"
    				},
    				IbanNotEmpty: {
    					label: "Tilinumero puuttuu"
    				},
    				InvalidStatusForPayroll: {
    					label: "Palkkalistan laskelmalla väärä status."
    				},
    				LastNameNotEmpty: {
    					label: "Työntekijän sukunimi puuttuu"
    				},
    				PayrollIdConflict: {
    					label: "Laskelman palkkalista-tunnus on virheellinen."
    				},
    				SocialSecurityNumber_InTest: {
    					label: "Henkilötunnus on virheellinen. Testissä voit käyttää tarkistusnumerona tähteä ('*')"
    				},
    				SocialSecurityNumber: {
    					label: "Henkilötunnus on virheellinen."
    				},
    				TaxCardType: {
    					label: "Verokortti puuttuu tai tyyppiä ei ole määritetty."
    				},
    				TaxPercent: {
    					label: "Veroprosenttia ei ole määritetty"
    				},
    				TelephoneOrEmailNotEmpty: {
    					label: "Anna jompikumpi tai molemmat yhteystiedot (puhelinnumero ja sähköposti)."
    				},
    				TotalGreaterThan0: {
    					label: "Palkkalaskelman summa on nolla"
    				}
    			},
    			NoCalculations: {
    				label: "Ei yhtään laskelmaa."
    			}
    		},
    		SharedTaxcard: {
    			description: "Työntekijäsi on jakanut uuden verokortin ja toivoo, että sitä käytetään palkanmaksuun. Tarkista ja hyväksy verokortti palkkalaskurin ruudun 1. Työntekijä kautta ennen palkan maksamista, jotta ennakonpidätys päivittyy laskelmalle oikein.",
    			label: "Uusi jaettu verokortti!"
    		},
    		ValidationErrors: {
    			unknown: "Validointivirhe: {{error}}",
    			maxlength: "Pituus enintään {{maxlength}} merkkiä",
    			minlength: "Pituus vähintään {{minlength}} merkkiä",
    			required: "Kenttä on pakollinen",
    			sxyCompanyIdFi: "Y-tunnuksen muoto on väärä",
    			sxyCurrency: "Rahamäärän muoto on väärä",
    			sxyEmail: "Sähköpostin muoto on väärä",
    			sxyIban: "TIlinumeron (IBAN) muoto on väärä",
    			sxyInteger: "Kentän pitää olla kokonaisluku",
    			sxyMobilePhone: "Puhelinnumeron muoto on väärä",
    			sxyNumber: "Kentän pitää olla numero",
    			sxyPensionContractNumber: "Tarkista sopimusnumero",
    			sxyPersonalIdFi: "Henkilötunnuksen muoto on väärä",
    			sxyOfficialIdFi: "Y-tunnuksen tai henkilötunnuksen muoto on väärä",
    			sxyPostalCodeFi: "Postinumeron muoto on väärä",
    			sxySmsVerificationCode: "Tarkista varmennuskoodi",
    			sxyTaxPercent: "Veroprosentin muoto on väärä",
    			sxyTemporaryPensionContractNumber: "Syötit tilapäisen sopimuksen numeron."
    		}
    	}
    };

    var SALAXY$b = {
    	ENUM: {
    	}
    };

    var SALAXY$a = {
    	ENUM: {
    		AbcSection: {
    			association: {
    				description: "",
    				label: "Associations"
    			},
    			blog: {
    				description: "",
    				label: "Blog"
    			},
    			businessOwner: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			documentTemplates: {
    				description: "",
    				label: "Documents and forms"
    			},
    			doNotShow: {
    				description: "",
    				label: "Not shown to user (Admin only)"
    			},
    			employee: {
    				description: "",
    				label: "Employee"
    			},
    			employment: {
    				description: "",
    				label: "Employment relationship and contract"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Company or community"
    			},
    			examples: {
    				description: "",
    				label: "Examples of hiring situations"
    			},
    			householdEmployer: {
    				description: "",
    				label: "Household"
    			},
    			instructionsAndExamples: {
    				description: "",
    				label: "User manual"
    			},
    			insurance: {
    				description: "",
    				label: "Insurances and pensions"
    			},
    			palkkausGeneral: {
    				description: "",
    				label: "Information about Palkkaus.fi service"
    			},
    			palkkausInstructions: {
    				description: "",
    				label: "User manuals"
    			},
    			personEmployer: {
    				description: "",
    				label: "Household employee"
    			},
    			press: {
    				description: "",
    				label: "Releases and material"
    			},
    			productLongDescription: {
    				description: "",
    				label: "Product description"
    			},
    			salary: {
    				description: "",
    				label: "Payroll calculation and salary payment"
    			},
    			"undefined": {
    				description: "",
    				label: "New article / undefined"
    			},
    			worker: {
    				description: "",
    				label: "Employer"
    			}
    		},
    		AbsenceCauseCode: {
    			accruedHoliday: {
    				label: "Accrued holiday"
    			},
    			annualLeave: {
    				label: "Annual leave"
    			},
    			childCareLeave: {
    				label: "Child care leave"
    			},
    			childIllness: {
    				label: "Child's illness or a compelling family reason"
    			},
    			illness: {
    				label: "Illness"
    			},
    			industrialAction: {
    				label: "Industrial action or lock-out"
    			},
    			interruptionInWorkProvision: {
    				label: "Interruption in work provision"
    			},
    			jobAlternationLeave: {
    				label: "Job alternation leave"
    			},
    			layOff: {
    				label: "Lay-off"
    			},
    			leaveOfAbsence: {
    				label: "Leave of absence"
    			},
    			midWeekHoliday: {
    				label: "Mid-week holiday"
    			},
    			militaryRefresherTraining: {
    				label: "Military refresher training"
    			},
    			militaryService: {
    				label: "Military or non-military service"
    			},
    			occupationalAccident: {
    				label: "Occupational accident"
    			},
    			other: {
    				label: "Other"
    			},
    			parentalLeave: {
    				label: "Maternity, paternity and parental leave"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Part-time absence due to rehabilitation"
    			},
    			partTimeChildCareLeave: {
    				label: "Part-time child care leave"
    			},
    			partTimeSickLeave: {
    				label: "Part-time sick leave"
    			},
    			personalReason: {
    				label: ""
    			},
    			rehabilitation: {
    				label: "Rehabilitation"
    			},
    			specialMaternityLeave: {
    				label: "Special maternity leave"
    			},
    			studyLeave: {
    				label: "Study leave"
    			},
    			training: {
    				label: "Training, education"
    			},
    			unpaidLeave: {
    				label: ""
    			}
    		},
    		AgeGroupCode: {
    			a: {
    				label: ""
    			},
    			b: {
    				label: ""
    			},
    			c: {
    				label: ""
    			},
    			u: {
    				label: ""
    			}
    		},
    		AgeRange: {
    			age_15: {
    				description: "",
    				label: "15 yrs"
    			},
    			age16: {
    				description: "",
    				label: "16 yrs"
    			},
    			age17: {
    				description: "",
    				label: "17 yrs"
    			},
    			age18_52: {
    				description: "",
    				label: "18-52 yrs"
    			},
    			age53_62: {
    				description: "",
    				label: "53-62 yrs"
    			},
    			age63_64: {
    				description: "",
    				label: "63-64 yrs"
    			},
    			age65_67: {
    				description: "",
    				label: "65-67 yrs"
    			},
    			age68AndOVer: {
    				description: "",
    				label: "68 yrs and over"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		AllowanceCode: {
    			expensesWorkingAbroad: {
    				label: ""
    			},
    			fullDailyAllowance: {
    				label: ""
    			},
    			internationalDailyAllowance: {
    				label: ""
    			},
    			mealAllowance: {
    				label: ""
    			},
    			partialDailyAllowance: {
    				label: ""
    			},
    			taxExemptReimbursementsAbroad: {
    				description: "",
    				label: "TaxExemptReimbursementsAbroad"
    			}
    		},
    		AnnualLeavePaymentKind: {
    			draftCalc: {
    				description: "",
    				label: ""
    			},
    			endSaldo: {
    				description: "",
    				label: ""
    			},
    			info: {
    				description: "",
    				label: ""
    			},
    			manualBonus: {
    				description: "",
    				label: ""
    			},
    			manualCompensation: {
    				description: "",
    				label: ""
    			},
    			manualSalary: {
    				description: "",
    				label: ""
    			},
    			paidCalc: {
    				description: "",
    				label: ""
    			},
    			startSaldo: {
    				description: "",
    				label: ""
    			},
    			total: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			planned: {
    				description: "",
    				label: "Planned"
    			}
    		},
    		ApiTestErrorType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			userFriendly: {
    				description: "",
    				label: "User friendly"
    			}
    		},
    		ApiValidationErrorType: {
    			general: {
    				label: "General"
    			},
    			invalid: {
    				label: "Invalid"
    			},
    			required: {
    				label: "Required"
    			},
    			warning: {
    				label: ""
    			}
    		},
    		AuthenticationMethod: {
    			auth0Database: {
    				description: "",
    				label: "Email-Password authentication"
    			},
    			emailPwdLocal: {
    				description: "",
    				label: "Database"
    			},
    			facebook: {
    				description: "",
    				label: "Facebook account"
    			},
    			google: {
    				description: "",
    				label: "Google or Gmail account"
    			},
    			internalLocalhost: {
    				description: "",
    				label: "Internal"
    			},
    			linkedIn: {
    				description: "",
    				label: "LinkedIn account"
    			},
    			microsoft: {
    				description: "",
    				label: "Microsoft account"
    			},
    			salaxy: {
    				description: "",
    				label: "Proxy to another account"
    			},
    			test: {
    				description: "",
    				label: "Temporary test credential"
    			},
    			"undefined": {
    				description: "",
    				label: "Unknown"
    			},
    			x509: {
    				description: "",
    				label: "X.509 certicate"
    			}
    		},
    		AuthorizationStatus: {
    			accessDenied: {
    				description: "",
    				label: "Access Denied"
    			},
    			expiredOauthTokdescription: "",
    			expiredOauthToklabel: "Session expired",
    			invalidOAuthTokdescription: "",
    			invalidOAuthToklabel: "Error in authentication",
    			noOauthTokdescription: "",
    			noOauthToklabel: "No Oauth Token",
    			oK: {
    				description: "",
    				label: "OK"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			expiredOauthToken: {
    				description: "",
    				label: "ExpiredOauthToken"
    			},
    			invalidOAuthToken: {
    				description: "",
    				label: "InvalidOAuthToken"
    			},
    			noOauthToken: {
    				description: "",
    				label: "NoOauthToken"
    			}
    		},
    		AuthorizationType: {
    			companyContract: {
    				description: "",
    				label: "Employer contract"
    			},
    			employerAuthorization: {
    				description: "",
    				label: "Power of attorney"
    			},
    			manual: {
    				description: "",
    				label: "Power of attorney submitted separately"
    			},
    			none: {
    				description: "",
    				label: "Authentication failed"
    			},
    			temporary: {
    				description: "",
    				label: "Temporary identification"
    			},
    			workerContract: {
    				description: "",
    				label: "Employee contract"
    			}
    		},
    		AvatarPictureType: {
    			gravatar: {
    				description: "",
    				label: "Gravatar"
    			},
    			icon: {
    				description: "",
    				label: "Icon"
    			},
    			uploaded: {
    				description: "",
    				label: "Picture from own computer"
    			}
    		},
    		BlobFileType: {
    			authorizationPdf: {
    				description: "",
    				label: "Power of attorney"
    			},
    			avatar: {
    				description: "",
    				label: "Icon"
    			},
    			calcReport: {
    				description: "",
    				label: ""
    			},
    			expensesReceipt: {
    				description: "",
    				label: "Expense receipt"
    			},
    			monthlyReport: {
    				description: "",
    				label: "Monthly Report"
    			},
    			raksaloki: {
    				description: "",
    				label: "Expense receipt"
    			},
    			taxCard: {
    				description: "",
    				label: "Tax Card"
    			},
    			template: {
    				description: "",
    				label: "Template"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			yearlyReport: {
    				description: "",
    				label: "Yearly Report"
    			},
    			eInvoice: {
    				description: "",
    				label: "EInvoice"
    			},
    			messages: {
    				description: "",
    				label: "Messages"
    			},
    			settings: {
    				description: "",
    				label: "Settings"
    			},
    			temporary: {
    				description: "",
    				label: "Temporary"
    			}
    		},
    		BlobRepository: {
    			cdnImages: {
    				description: "",
    				label: ""
    			},
    			fileSystemContent: {
    				description: "",
    				label: ""
    			},
    			gitContent: {
    				description: "",
    				label: ""
    			},
    			payload: {
    				description: "",
    				label: ""
    			},
    			reports: {
    				description: "",
    				label: ""
    			},
    			systemFiles: {
    				description: "",
    				label: ""
    			},
    			userFiles: {
    				description: "",
    				label: ""
    			},
    			versionHistory: {
    				description: "",
    				label: ""
    			}
    		},
    		BoardKind: {
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		CalculationFlag: {
    			accidentInsurance: {
    				label: ""
    			},
    			cfDeduction: {
    				label: ""
    			},
    			cfDeductionAtSalaxy: {
    				label: ""
    			},
    			cfNoPayment: {
    				label: ""
    			},
    			cfPayment: {
    				label: ""
    			},
    			exclude: {
    				label: ""
    			},
    			healthInsurance: {
    				label: ""
    			},
    			noTax: {
    				label: ""
    			},
    			pensionInsurance: {
    				label: ""
    			},
    			tax: {
    				label: ""
    			},
    			taxDeduction: {
    				label: ""
    			},
    			unemploymentInsurance: {
    				label: ""
    			},
    			insurancesDeduction: {
    				description: "",
    				label: "InsurancesDeduction"
    			}
    		},
    		CalculationRowSource: {
    			manualRow: {
    				description: "",
    				label: "Additional row"
    			},
    			salaryPage: {
    				description: "",
    				label: "Salary Page"
    			},
    			tesSelection: {
    				description: "",
    				label: "Use case"
    			}
    		},
    		CalculationRowType: {
    			accomodationBenefit: {
    				amountLabel: "",
    				description: "Taxable value of employee accomodation benefit.",
    				iconText: "",
    				label: "Accomodation benefit",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			advance: {
    				amountLabel: "",
    				description: "Salary advance deducation. Salary paid in advance and deducted from net salary.",
    				iconText: "",
    				label: "Salary advance",
    				moreInfo: "",
    				priceLabel: "Paid in advance"
    			},
    			board: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			carBenefit: {
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			chainsawReduction: {
    				amountLabel: "",
    				description: "Deduction for use of chainsaw, special cases (pre payment law 15 §, pre payment regulation 15 §).",
    				iconText: "",
    				label: "Choose…",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			childCareSubsidy: {
    				amountLabel: "",
    				description: "KELA support to communal workers for childcare. Affects incidental expenses.",
    				iconText: "KELA",
    				label: "Childcare subsidy",
    				moreInfo: "",
    				priceLabel: "Paid for by KELA"
    			},
    			compensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Work compensation",
    				moreInfo: "",
    				priceLabel: "Work compensation"
    			},
    			dailyAllowance: {
    				amountLabel: "Days",
    				description: "Daily allowance, 40 euros per day, if commute is more than 10 hours or over 15 km.",
    				iconText: "40",
    				label: "Daily allowance",
    				moreInfo: "",
    				priceLabel: "Euro / day"
    			},
    			dailyAllowanceHalf: {
    				amountLabel: "Days",
    				description: "Half day allowance, 19 euros per day, if commute is more than 6 hours or over 15 km.",
    				iconText: 19,
    				label: "Half day allowance",
    				moreInfo: "",
    				priceLabel: "Euro / day"
    			},
    			employmentTermination: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			eveningAddition: {
    				amountLabel: "Hours",
    				description: "Addition for evening work according to collective bargaining agreement.",
    				iconText: "EVNG",
    				label: "Evening shift addition",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			expenses: {
    				amountLabel: "",
    				description: "Employee expense claims (use of taxi, office supplies etc.).",
    				iconText: "expense",
    				label: "Expense claim",
    				moreInfo: "",
    				priceLabel: "Expense claim"
    			},
    			foreclosure: {
    				amountLabel: "",
    				description: "Employee salary foreclosure, paid directly to bailiff.",
    				iconText: "",
    				label: "Salary foreclosure",
    				moreInfo: "",
    				priceLabel: "Sum"
    			},
    			foreclosureByPalkkaus: {
    				amountLabel: "",
    				description: "Employee salary foreclosure made by Palkkaus.fi, paid directly to bailiff.",
    				iconText: "",
    				label: "Salary foreclosure made by Palkkaus.fi",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			holidayBonus: {
    				amountLabel: "Percent",
    				description: "Holiday bonus according to collective bargaining agreement.",
    				iconText: "CBA",
    				label: "Holiday bonus",
    				moreInfo: "",
    				priceLabel: "Salary for holiday period"
    			},
    			holidayCompensation: {
    				amountLabel: "Percent",
    				description: "Taxable value of employee holiday compensation if holiday is not held.",
    				iconText: "11,5%",
    				label: "Holiday compensation",
    				moreInfo: "",
    				priceLabel: "Salary amount"
    			},
    			holidaySalary: {
    				amountLabel: "",
    				description: "Salary for annual holiday.",
    				iconText: "vacation",
    				label: "Holiday salary",
    				moreInfo: "",
    				priceLabel: "Salary for holiday period"
    			},
    			hourlySalary: {
    				amountLabel: "",
    				description: "Salary payment based on hours put in.",
    				iconText: "h",
    				label: "Hourly salary",
    				moreInfo: "",
    				priceLabel: "Hourly salary"
    			},
    			irIncomeType: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			mealBenefit: {
    				amountLabel: "",
    				description: "Taxable value of employee meal benefit.",
    				iconText: "food",
    				label: "Meal benefit",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			mealCompensation: {
    				amountLabel: "Days",
    				description: "Meal compensation when daily allowence is not paid.",
    				iconText: "meal",
    				label: "Meal compensation",
    				moreInfo: "",
    				priceLabel: "Compensation / day"
    			},
    			milageDaily: {
    				amountLabel: "Days",
    				description: "Mileage compensation accoring to collective bargaining agreement.",
    				iconText: "km",
    				label: "Daily mileage compensation",
    				moreInfo: "",
    				priceLabel: "Compensation / day"
    			},
    			milageOther: {
    				amountLabel: "Days",
    				description: "Mileage compensation due to other justifications.",
    				iconText: "km",
    				label: "Other mileage compensation",
    				moreInfo: "",
    				priceLabel: "Compensation / day"
    			},
    			milageOwnCar: {
    				amountLabel: "",
    				description: "Mileage compensation due to use of own car.",
    				iconText: "0,43€",
    				label: "Mileage compensation, own car",
    				moreInfo: "",
    				priceLabel: "Compensation / km"
    			},
    			monthlySalary: {
    				amountLabel: "",
    				description: "Fixed monthly income.",
    				iconText: "month",
    				label: "Monthly salary",
    				moreInfo: "",
    				priceLabel: "Monthly salary"
    			},
    			nightimeAddition: {
    				amountLabel: "Hours",
    				description: "Addition for nighttime work according to collective bargaining agreement.",
    				iconText: "NIGHT",
    				label: "Night shift addition",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			nonProfitOrg: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherAdditions: {
    				amountLabel: "",
    				description: "Other additions according to collective bargaining agreement.",
    				iconText: "addition",
    				label: "Other additions",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherBenefit: {
    				amountLabel: "",
    				description: "Taxable value of other employee benefits.",
    				iconText: "Benefit",
    				label: "Other benefits",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherCompensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherDeductions: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			overtime: {
    				amountLabel: "Percent",
    				description: "Overtime payments.",
    				iconText: "over",
    				label: "Overtime work compensation",
    				moreInfo: "",
    				priceLabel: "Salary amount"
    			},
    			phoneBenefit: {
    				amountLabel: "Months",
    				description: "Taxable value of employee mobile phone benefit.",
    				iconText: 20,
    				label: "Mobile phone benefit",
    				moreInfo: "",
    				priceLabel: "Mobile phone benefit / mon"
    			},
    			prepaidExpenses: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			remuneration: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			salary: {
    				amountLabel: "",
    				description: "Lump sum for work effort.",
    				iconText: "D",
    				label: "Salary or commission",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			saturdayAddition: {
    				amountLabel: "Percent",
    				description: "Addition for Saturday work according to collective bargaining agreement.",
    				iconText: "SAT",
    				label: "Saturday addition",
    				moreInfo: "",
    				priceLabel: "Salary amount"
    			},
    			subsidisedCommute: {
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			sundayWork: {
    				amountLabel: "",
    				description: "Addition for Sunday work according to collective bargaining agreement.",
    				iconText: "SUN",
    				label: "Sunday or public holiday addition",
    				moreInfo: "",
    				priceLabel: "Salary amount"
    			},
    			tesWorktimeShortening: {
    				amountLabel: "Percent",
    				description: "Separate salary portion that is defined by the sector/industry collective bargaining agreement. Aka. \"pekkaspäivät\" in Finnish.",
    				iconText: "CBA",
    				label: "Compensation for accrued time off",
    				moreInfo: "",
    				priceLabel: "Salary amount"
    			},
    			toolCompensation: {
    				amountLabel: "Days",
    				description: "Compensation for use of own tools for work. Compensation defined according to collective bargaining agreement.",
    				iconText: "",
    				label: "Work tool compensation",
    				moreInfo: "",
    				priceLabel: "Compensation / day"
    			},
    			totalEmployerPayment: {
    				amountLabel: "",
    				description: "Total cost calculation: what a salary payment will cost for the employer.",
    				iconText: "sum",
    				label: "Total cost",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totals: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Total",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totalWorkerPayment: {
    				amountLabel: "",
    				description: "Net salary calculation: what is left when tax deductions etc. are made from gross salary.",
    				iconText: "net",
    				label: "Net salary",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			unionPayment: {
    				amountLabel: "",
    				description: "Employee union payment made directly to trade union, if employer chooses to do so.",
    				iconText: "Trade union",
    				label: "Union payment",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			unknown: {
    				amountLabel: "",
    				description: "Row type undefined.",
    				iconText: "?",
    				label: "Choose…",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			workingTimeCompensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			}
    		},
    		CalculationRowUnit: {
    			count: {
    				description: "",
    				label: "amount"
    			},
    			days: {
    				description: "",
    				label: "days"
    			},
    			kilometers: {
    				description: "",
    				label: "km"
    			},
    			percent: {
    				description: "",
    				label: "%"
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			}
    		},
    		CalculationStatus: {
    			draft: {
    				description: "",
    				label: "Draft"
    			},
    			externalPaymentCanceled: {
    				description: "",
    				label: "Payment canceled outside of Salaxy-service"
    			},
    			externalPaymentError: {
    				description: "",
    				label: "Errors (outside of Salaxy-service)"
    			},
    			externalPaymentStarted: {
    				description: "",
    				label: "Payment initiated outside of Salaxy-service"
    			},
    			externalPaymentSucceeded: {
    				description: "",
    				label: "Payment completed outside of Salaxy-service"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "Payment canceled"
    			},
    			paymentError: {
    				description: "",
    				label: "Error in payment"
    			},
    			paymentRefunded: {
    				description: "",
    				label: "Payment canceled and refunded"
    			},
    			paymentStarted: {
    				description: "",
    				label: "Payment started"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "Payment succeeded"
    			},
    			paymentWorkerCopy: {
    				description: "",
    				label: "Copy for employee"
    			},
    			payrollDraft: {
    				description: "",
    				label: ""
    			},
    			proDraft: {
    				label: ""
    			},
    			sharedApproved: {
    				description: "",
    				label: ""
    			},
    			sharedRejected: {
    				description: "",
    				label: ""
    			},
    			sharedWaiting: {
    				description: "",
    				label: ""
    			},
    			template: {
    				description: "",
    				label: "Template"
    			},
    			workerRequestAccepted: {
    				description: "",
    				label: "Payslip made by employee accepted"
    			},
    			workerRequestDeclined: {
    				description: "",
    				label: "Payslip made by employee rejected"
    			},
    			workerRequested: {
    				description: "",
    				label: "Payslip made by employee"
    			}
    		},
    		CarBenefitCode: {
    			fullCarBenefit: {
    				label: ""
    			},
    			limitedCarBenefit: {
    				label: ""
    			}
    		},
    		CarBenefitKind: {
    			fullCarBenefit: {
    				description: "",
    				label: ""
    			},
    			limitedCarBenefit: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		CompanyType: {
    			fiOy: {
    				description: "",
    				label: "Limited companies, also and public limited companies"
    			},
    			fiRy: {
    				description: "",
    				label: "Associations"
    			},
    			fiTm: {
    				description: "",
    				label: "Private trader or farmer"
    			},
    			fiYy: {
    				description: "",
    				label: "Other, e.g. limited partnerships, open companies etc."
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		ContractPartyType: {
    			customContact: {
    				description: "",
    				label: ""
    			},
    			person: {
    				description: "",
    				label: ""
    			}
    		},
    		DailyAllowanceKind: {
    			fullDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			internationalDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			mealAllowance: {
    				description: "",
    				label: ""
    			},
    			partialDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		DateOfBirthAccuracy: {
    			ageBased: {
    				description: "",
    				label: "Age Based"
    			},
    			ageGroupBased: {
    				description: "",
    				label: "Age Group Based"
    			},
    			assumption: {
    				description: "",
    				label: "Assumption"
    			},
    			exact: {
    				description: "",
    				label: "Exact date"
    			},
    			monthCorrect: {
    				description: "",
    				label: "Month Correct"
    			},
    			verified: {
    				description: "",
    				label: "Verified"
    			}
    		},
    		EmployerType: {
    			fullTimeEmployer: {
    				description: "",
    				label: "Ongoing contract or over 8178 € / 6 months"
    			},
    			miniEmployer: {
    				description: "",
    				label: "Under 1300 € per year"
    			},
    			smallEmployer: {
    				description: "",
    				label: "1300 € or more per year"
    			},
    			tyelOwnContractEmployer: {
    				description: "",
    				label: "TyEL-annual contract or taken care of by customer"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		EmploymentContract: {
    			annualLeave: {
    				label: ""
    			},
    			benefitsException: {
    				label: ""
    			},
    			noticeException: {
    				label: ""
    			},
    			otherTerms: {
    				label: ""
    			}
    		},
    		EmploymentRelationType: {
    			athlete: {
    				description: "",
    				label: ""
    			},
    			boardMember: {
    				description: "",
    				label: ""
    			},
    			compensation: {
    				description: "",
    				label: ""
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: ""
    			},
    			entrepreneur: {
    				description: "",
    				label: ""
    			},
    			entrepreneurNew: {
    				description: "",
    				label: ""
    			},
    			farmer: {
    				description: "",
    				label: ""
    			},
    			hourlySalary: {
    				description: "",
    				label: ""
    			},
    			keyEmployee: {
    				description: "",
    				label: ""
    			},
    			monthlySalary: {
    				description: "",
    				label: ""
    			},
    			performingArtist: {
    				description: "",
    				label: ""
    			},
    			salary: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		EmploymentSalaryType: {
    			hourlyPay: {
    				description: "",
    				label: "Hourly salary",
    				metricAbbrev: "/h"
    			},
    			montlyPay: {
    				description: "",
    				label: "Monthly salary",
    				metricAbbrev: "/mo"
    			},
    			otherPay: {
    				description: "",
    				label: "Other"
    			},
    			unknown: {
    				description: "",
    				label: "Choose…"
    			}
    		},
    		EmploymentTerminationKind: {
    			monetaryWorkingTimeBankCompensation: {
    				description: "",
    				label: ""
    			},
    			noticePeriodCompensation: {
    				description: "",
    				label: ""
    			},
    			pensionPaidByEmployer: {
    				description: "",
    				label: ""
    			},
    			terminationAndLayOffDamages: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			voluntaryTerminationCompensation: {
    				description: "",
    				label: ""
    			}
    		},
    		EmploymentType: {
    			fullTime: {
    				description: "",
    				label: "Permanent employment"
    			},
    			job: {
    				description: "",
    				label: "One-off work"
    			},
    			partTime: {
    				description: "",
    				label: "Part-time employment"
    			},
    			unknown: {
    				description: "",
    				label: "Choose…"
    			}
    		},
    		EmploymentWorkHoursType: {
    			hourPerThreeWeeks: {
    				description: "",
    				label: "Hours/three weeks"
    			},
    			hourPerTwoWeeks: {
    				description: "",
    				label: "Hours/two weeks"
    			},
    			hoursPerDay: {
    				description: "",
    				label: "Hours/day"
    			},
    			hoursPerWeek: {
    				description: "",
    				label: "Hours/week"
    			}
    		},
    		ForeclosureKind: {
    			fixed: {
    				description: "",
    				label: "Fixed amount"
    			},
    			periodic: {
    				description: "",
    				label: "Calculated from periodic wage"
    			},
    			nonPeriodic: {
    				description: "",
    				label: "Calculated from non-periodic wage"
    			},
    			"undefined": {
    				description: "",
    				label: "No selection"
    			}
    		},
    		FrameworkAgreement: {
    			childCare: {
    				description: "Permanent nanny and subsidies.",
    				label: "Other childcare"
    			},
    			cleaning: {
    				description: "Cleaning and real estate work.",
    				label: "Cleaning- and real estate sector"
    			},
    			construction: {
    				description: "Work according to construction sector colloctive bargaining agreement.",
    				label: "Construction and renovation work"
    			},
    			entrepreneur: {
    				description: "Entrepreneurs salary.",
    				label: "Entrepreneur"
    			},
    			mll: {
    				description: "Salary for MLL nanny.",
    				label: "MLL recommondations"
    			},
    			notDefined: {
    				description: "Collective bargaining agreement or recommendation undefined.",
    				label: "Undefined"
    			},
    			other: {
    				description: "Other work och no collective bargaining agreement.",
    				label: "Other work"
    			},
    			santaClaus: {
    				description: "Santa Claus home visit.",
    				label: "Santa Claus"
    			}
    		},
    		Gender: {
    			female: {
    				description: "",
    				label: "Female"
    			},
    			male: {
    				description: "",
    				label: "Male"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		HelttiIndustry: {
    			i: {
    				description: "",
    				label: "Housing and hopsitality operations"
    			},
    			j: {
    				description: "",
    				label: "Information systems and communication"
    			},
    			k: {
    				description: "",
    				label: "Finance and insurance operations"
    			},
    			m: {
    				description: "",
    				label: "Professionell, vetenskaplig och teknisk verksamhet"
    			},
    			n: {
    				description: "",
    				label: "Administrative and support operations"
    			},
    			notDefined: {
    				description: "",
    				label: "Choose…"
    			},
    			o: {
    				description: "",
    				label: "Public administration and national defense"
    			},
    			other: {
    				description: "",
    				label: "Other industry"
    			},
    			q: {
    				description: "",
    				label: "Health and social service operations"
    			},
    			r: {
    				description: "",
    				label: "Arts, entertainment and recreational operations"
    			},
    			s: {
    				description: "",
    				label: "Other service operations"
    			},
    			p: {
    				description: "",
    				label: "P"
    			}
    		},
    		HelttiProductPackage: {
    			large: {
    				description: "",
    				label: "Hifi"
    			},
    			medium: {
    				description: "",
    				label: "Growth"
    			},
    			notDefined: {
    				description: "",
    				label: "No contract"
    			},
    			small: {
    				description: "",
    				label: "Start"
    			}
    		},
    		HolidayAccrualKind: {
    			calculation: {
    				label: ""
    			},
    			holiday: {
    				label: ""
    			},
    			holidayCompensated: {
    				label: ""
    			},
    			manual: {
    				label: ""
    			},
    			saldo: {
    				label: ""
    			}
    		},
    		HolidayBonusPaymentMethod: {
    			none: {
    				label: ""
    			},
    			pay24Days: {
    				label: ""
    			},
    			payAllBonus: {
    				label: ""
    			},
    			payForHolidaySalary: {
    				label: ""
    			},
    			paySummerBonus: {
    				label: ""
    			}
    		},
    		HolidayCode: {
    			holidayCompensation: {
    				description: "",
    				label: ""
    			},
    			holidayCompensationIncluded: {
    				description: "",
    				label: ""
    			},
    			noHolidays: {
    				description: "",
    				label: ""
    			},
    			permanent14Days: {
    				description: "",
    				label: ""
    			},
    			permanent35Hours: {
    				description: "",
    				label: ""
    			},
    			temporaryTimeOff: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		HolidayYearInitType: {
    			createEmpty: {
    				label: ""
    			},
    			createFromCalcs: {
    				label: ""
    			},
    			createMonths: {
    				label: ""
    			},
    			ignore: {
    				label: ""
    			}
    		},
    		HouseHoldUseCaseTree: {
    			childCare: {
    				description: "",
    				kela: {
    					description: "",
    					label: ""
    				},
    				label: "",
    				mll: {
    					description: "",
    					label: ""
    				},
    				other: {
    					description: "",
    					label: ""
    				}
    			},
    			cleaning: {
    				"51530": {
    					description: "",
    					label: ""
    				},
    				"53221": {
    					description: "",
    					label: ""
    				},
    				"61132": {
    					description: "",
    					label: ""
    				},
    				"91110": {
    					description: "",
    					label: ""
    				},
    				description: "",
    				label: "",
    				other: {
    					description: "",
    					label: ""
    				}
    			},
    			construction: {
    				carpenter: {
    					description: "",
    					label: ""
    				},
    				description: "",
    				floor: {
    					description: "",
    					label: ""
    				},
    				label: "",
    				"new": {
    					description: "",
    					label: ""
    				},
    				painter: {
    					description: "",
    					label: ""
    				},
    				renovation: {
    					description: "",
    					label: ""
    				},
    				roof: {
    					description: "",
    					label: ""
    				}
    			},
    			other: {
    				"51530": {
    					description: "",
    					label: "Building caretakers"
    				},
    				"53112": {
    					description: "",
    					label: "Private childminders"
    				},
    				"53212": {
    					description: "",
    					label: "Mental handicap nurses"
    				},
    				"53213": {
    					description: "",
    					label: "Social work assistants"
    				},
    				"53221": {
    					description: "",
    					label: "Household service workers"
    				},
    				"53222": {
    					description: "",
    					label: "Personal care workers, family carers, etc."
    				},
    				"61112": {
    					description: "",
    					label: "Field crop supervisors and workers"
    				},
    				"61132": {
    					description: "",
    					label: "Gardeners, horticultural and nursery growers and workers"
    				},
    				"62100": {
    					description: "",
    					label: "Forestry and related workers"
    				},
    				"71110": {
    					description: "",
    					label: "House builders"
    				},
    				"71150": {
    					description: "",
    					label: "Carpenters and joiners"
    				},
    				"71210": {
    					description: "",
    					label: "Roofers"
    				},
    				"71310": {
    					description: "",
    					label: "Painters and related workers"
    				},
    				"91110": {
    					description: "",
    					label: "Domestic cleaners and helpers"
    				},
    				description: "",
    				label: "Other",
    				"undefined": {
    					description: "",
    					label: "Other"
    				}
    			}
    		},
    		InsuranceCompany: {
    			aktia: {
    				description: "",
    				label: "Aktia"
    			},
    			alandia: {
    				description: "",
    				label: "Alandia"
    			},
    			aVakuutus: {
    				description: "",
    				label: "A-Vakuutus"
    			},
    			fennia: {
    				description: "",
    				label: "Fennia"
    			},
    			folksam: {
    				description: "",
    				label: "Folksam"
    			},
    			"if": {
    				description: "",
    				label: "If"
    			},
    			"lähiTapiola": {
    				description: "",
    				label: "LähiTapiola"
    			},
    			none: {
    				description: "",
    				label: "(Choose insurance company)"
    			},
    			other: {
    				description: "",
    				label: "Other company"
    			},
    			"pohjantähti": {
    				description: "",
    				label: "Pohjantähti"
    			},
    			pohjola: {
    				description: "",
    				label: "OP Vakuutus"
    			},
    			redarnas: {
    				description: "",
    				label: "Redarnas"
    			},
    			tryg: {
    				description: "",
    				label: "Tryg"
    			},
    			turva: {
    				description: "",
    				label: "Turva"
    			},
    			"ålands": {
    				description: "",
    				label: "Ålands"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			}
    		},
    		InvoiceStatus: {
    			canceled: {
    				description: "",
    				label: ""
    			},
    			error: {
    				description: "",
    				label: ""
    			},
    			forecast: {
    				description: "",
    				label: ""
    			},
    			paid: {
    				description: "",
    				label: ""
    			},
    			paymentStarted: {
    				description: "",
    				label: ""
    			},
    			preview: {
    				description: "",
    				label: ""
    			},
    			read: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			unread: {
    				description: "",
    				label: ""
    			},
    			waitingConfirmation: {
    				description: "",
    				label: ""
    			},
    			waitingPalkkaus: {
    				description: "",
    				label: ""
    			}
    		},
    		IrFlags: {
    			noMoney: {
    				label: ""
    			},
    			oneOff: {
    				label: ""
    			},
    			unjustEnrichment: {
    				label: ""
    			}
    		},
    		IrInsuranceExceptions: {
    			excludeAccidentInsurance: {
    				label: ""
    			},
    			excludeAll: {
    				label: ""
    			},
    			excludeHealthInsurance: {
    				label: ""
    			},
    			excludePension: {
    				label: ""
    			},
    			excludeUnemployment: {
    				label: ""
    			},
    			includeAccidentInsurance: {
    				label: ""
    			},
    			includeAll: {
    				label: ""
    			},
    			includeHealthInsurance: {
    				label: ""
    			},
    			includePension: {
    				label: ""
    			},
    			includeUnemployment: {
    				label: ""
    			}
    		},
    		LegalEntityType: {
    			company: {
    				description: "",
    				label: "Business ID"
    			},
    			partner: {
    				description: "",
    				label: "Partner"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			personCreatedByEmployer: {
    				description: "",
    				label: "Employee"
    			},
    			"undefined": {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		MealBenefitKind: {
    			cateringContract: {
    				description: "",
    				label: ""
    			},
    			institute: {
    				description: "",
    				label: ""
    			},
    			mealAllowance: {
    				description: "",
    				label: ""
    			},
    			mealTicket: {
    				description: "",
    				label: ""
    			},
    			restaurantWorker: {
    				description: "",
    				label: ""
    			},
    			taxableAmount: {
    				description: "",
    				label: ""
    			},
    			teacher: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		MessageType: {
    			email: {
    				description: "",
    				label: "E-mail"
    			},
    			sms: {
    				description: "",
    				label: "SMS"
    			}
    		},
    		NonProfitOrgKind: {
    			accomodationAllowance: {
    				description: "",
    				label: ""
    			},
    			dailyAllowance: {
    				description: "",
    				label: ""
    			},
    			kilometreAllowance: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		OnboardingStatus: {
    			created: {
    				description: "",
    				label: "Created"
    			},
    			done: {
    				description: "",
    				label: "Done"
    			},
    			opdescription: "",
    			oplabel: "In progress",
    			expired: {
    				description: "",
    				label: "Expired"
    			},
    			open: {
    				description: "",
    				label: "Open"
    			}
    		},
    		OtherCompensationKind: {
    			accruedTimeOffCompensation: {
    				description: "",
    				label: ""
    			},
    			capitalIncomePayment: {
    				description: "",
    				label: ""
    			},
    			employeeInventionCompensation: {
    				description: "",
    				label: ""
    			},
    			employeeStockOption: {
    				description: "",
    				label: ""
    			},
    			employeeStockOptionWithLowerPrice: {
    				description: "",
    				label: ""
    			},
    			lectureFee: {
    				description: "",
    				label: ""
    			},
    			meetingFee: {
    				description: "",
    				label: ""
    			},
    			membershipOfGoverningBodyCompensation: {
    				description: "",
    				label: ""
    			},
    			monetaryGiftForEmployees: {
    				description: "",
    				label: ""
    			},
    			otherTaxableIncomeAsEarnedIncome: {
    				description: "",
    				label: ""
    			},
    			positionOfTrustCompensation: {
    				description: "",
    				label: ""
    			},
    			stockOptionsAndGrants: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			useCompensationAsCapitalIncome: {
    				description: "",
    				label: ""
    			},
    			useCompensationAsEarnedIncome: {
    				description: "",
    				label: ""
    			},
    			workEffortBasedDividendsAsNonWage: {
    				description: "",
    				label: ""
    			},
    			workEffortBasedDividendsAsWage: {
    				description: "",
    				label: ""
    			}
    		},
    		PalkkausCampaignPricing: {
    			campaignAfterFirstUse: {
    				description: "",
    				label: ""
    			},
    			campaignFirstUse: {
    				description: "",
    				label: ""
    			},
    			companyFreeTrial: {
    				description: "",
    				label: ""
    			},
    			none: {
    				description: "",
    				label: ""
    			}
    		},
    		PartnerSite: {
    			careDotCom: {
    				description: "",
    				label: ""
    			},
    			dev: {
    				description: "",
    				label: ""
    			},
    			duunitori: {
    				description: "",
    				label: ""
    			},
    			example: {
    				description: "",
    				label: ""
    			},
    			mol: {
    				description: "",
    				label: ""
    			},
    			palkkaapakolaindescription: "",
    			palkkaapakolainlabel: "",
    			palkkaus: {
    				description: "",
    				label: ""
    			},
    			palkkausEmployerFillIn: {
    				description: "",
    				label: ""
    			},
    			palkkausIdentified: {
    				description: "",
    				label: ""
    			},
    			raksa: {
    				description: "",
    				label: ""
    			},
    			refugeeJobs: {
    				description: "",
    				label: ""
    			},
    			talkkarit: {
    				description: "",
    				label: ""
    			},
    			unknown: {
    				description: "",
    				label: ""
    			},
    			palkkaapakolainen: {
    				description: "",
    				label: "Palkkaapakolainen"
    			}
    		},
    		PaymentChannel: {
    			accountorGo: {
    				description: "",
    				label: ""
    			},
    			finagoSolo: {
    				description: "",
    				label: ""
    			},
    			holviCfa: {
    				description: "",
    				label: ""
    			},
    			kevytyrittaja: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaFinvoice: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaPaytrail: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaReference: {
    				description: "",
    				label: ""
    			},
    			palkkausManual: {
    				description: "",
    				label: ""
    			},
    			palkkausWS: {
    				description: "",
    				label: ""
    			},
    			palkkausPersonal: {
    				description: "",
    				label: ""
    			},
    			procountor: {
    				description: "",
    				label: ""
    			},
    			talenomCfa: {
    				description: "",
    				label: ""
    			},
    			talenomOnline: {
    				description: "",
    				label: ""
    			},
    			test: {
    				description: "",
    				label: ""
    			},
    			zeroPayment: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentStatus: {
    			bankDelivered: {
    				description: "",
    				label: "Approved by bank"
    			},
    			bankError: {
    				description: "",
    				label: "Error in bank"
    			},
    			bankPartialError: {
    				description: "",
    				label: "Error in bank"
    			},
    			bankTechApproval: {
    				description: "",
    				label: "Received by bank"
    			},
    			cancelled: {
    				description: "",
    				label: "Payment canceled"
    			},
    			"new": {
    				description: "",
    				label: "New"
    			},
    			paid: {
    				description: "",
    				label: "Payment made"
    			},
    			sentToBank: {
    				description: "",
    				label: "Sent to bank"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		PayrollStatus: {
    			draft: {
    				label: "Draft"
    			},
    			paymentCanceled: {
    				label: "Payment canceled"
    			},
    			paymentError: {
    				label: "Error in payment"
    			},
    			paymentStarted: {
    				label: "Payment started"
    			},
    			paymentSucceeded: {
    				label: "Payment succeeded"
    			},
    			template: {
    				label: "Template"
    			}
    		},
    		PensionCalculation: {
    			athlete: {
    				description: "",
    				label: ""
    			},
    			boardRemuneration: {
    				description: "",
    				label: ""
    			},
    			compensation: {
    				description: "",
    				label: ""
    			},
    			employee: {
    				description: "",
    				label: ""
    			},
    			entrepreneur: {
    				description: "",
    				label: ""
    			},
    			farmer: {
    				description: "",
    				label: ""
    			},
    			partialOwner: {
    				description: "",
    				label: ""
    			},
    			smallEntrepreneur: {
    				description: "",
    				label: ""
    			},
    			smallFarmer: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		PensionCompany: {
    			apteekkien: {
    				description: "",
    				label: "Apteekkien Eläkekassa"
    			},
    			elo: {
    				description: "",
    				label: "Elo"
    			},
    			etera: {
    				description: "",
    				label: "Etera"
    			},
    			ilmarindescription: "",
    			ilmarinen: {
    				label: "Ilmarinen"
    			},
    			none: {
    				description: "",
    				label: "Choose…"
    			},
    			other: {
    				description: "",
    				label: "Other company"
    			},
    			pensionsAlandia: {
    				description: "",
    				label: "Alandia"
    			},
    			varma: {
    				description: "",
    				label: "Varma"
    			},
    			veritas: {
    				description: "",
    				label: "Veritas"
    			},
    			verso: {
    				description: "",
    				label: "Verso"
    			}
    		},
    		PeriodType: {
    			custom: {
    				label: ""
    			},
    			month: {
    				label: ""
    			},
    			quarter: {
    				label: ""
    			},
    			year: {
    				label: ""
    			}
    		},
    		PricingModel: {
    			fixedFee: {
    				label: ""
    			},
    			noFee: {
    				label: ""
    			},
    			palkkausFee: {
    				label: ""
    			}
    		},
    		Product: {
    			accounting: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			baseService: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			healthCareHeltti: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			insurance: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			pension: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			pensionEntrepreneur: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			tax: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			taxCard: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			unemployment: {
    				description: "",
    				label: "",
    				status: "",
    				statusTVR: ""
    			}
    		},
    		ProductListFilter: {
    			all: {
    				description: "",
    				label: ""
    			},
    			available: {
    				description: "",
    				label: ""
    			}
    		},
    		Profession: {
    			"0": {
    				label: "Armed forces"
    			},
    			"1": {
    				label: "Managers"
    			},
    			"2": {
    				label: "Professionals"
    			},
    			"3": {
    				label: "Technicians and associate professionals"
    			},
    			"4": {
    				label: "Clerical support workers"
    			},
    			"5": {
    				label: "Service and sales workers"
    			},
    			"6": {
    				label: "Skilled agricultural, forestry and fishery workers"
    			},
    			"7": {
    				label: "Craft and related trades workers"
    			},
    			"8": {
    				label: "Plant and machine operators, and assemblers"
    			},
    			"9": {
    				label: "Elementary occupations"
    			},
    			"11": {
    				label: "Chief executives, senior officials and legislators"
    			},
    			"12": {
    				label: "Administrative and commercial managers"
    			},
    			"13": {
    				label: "Production and specialised services managers"
    			},
    			"14": {
    				label: "Hospitality, retail and other services managers"
    			},
    			"21": {
    				label: "Science and engineering professionals"
    			},
    			"22": {
    				label: "Health professionals"
    			},
    			"23": {
    				label: "Teaching professionals"
    			},
    			"24": {
    				label: "Business and administration professionals"
    			},
    			"25": {
    				label: "Information and communications technology professionals"
    			},
    			"26": {
    				label: "Legal, social and cultural professionals"
    			},
    			"31": {
    				label: "Science and engineering associate professionals"
    			},
    			"32": {
    				label: "Health associate professionals"
    			},
    			"33": {
    				label: "Business and administration associate professionals"
    			},
    			"34": {
    				label: "Legal, social, cultural and related associate professionals"
    			},
    			"35": {
    				label: "Information and communications technicians"
    			},
    			"41": {
    				label: "General and keyboard clerks"
    			},
    			"42": {
    				label: "Customer services clerks"
    			},
    			"43": {
    				label: "Numerical and material recording clerks"
    			},
    			"44": {
    				label: "Other clerical support workers"
    			},
    			"51": {
    				label: "Personal service workers"
    			},
    			"52": {
    				label: "Sales workers"
    			},
    			"53": {
    				label: "Personal care workers"
    			},
    			"54": {
    				label: "Protective services workers"
    			},
    			"61": {
    				label: "Market-oriented skilled agricultural workers"
    			},
    			"62": {
    				label: "Market-oriented skilled forestry, fishery and hunting workers"
    			},
    			"63": {
    				label: "Subsistence farmers, fishers, hunters and gatherers"
    			},
    			"71": {
    				label: "Building and related trades workers, excluding electricians"
    			},
    			"72": {
    				label: "Metal, machinery and related trades workers"
    			},
    			"73": {
    				label: "Handicraft and printing workers"
    			},
    			"74": {
    				label: "Electrical and electronic trades workers"
    			},
    			"75": {
    				label: "Food processing, wood working, garment and other craft and related trades workers"
    			},
    			"81": {
    				label: "Stationary plant and machine operators"
    			},
    			"82": {
    				label: "Assemblers"
    			},
    			"83": {
    				label: "Drivers and mobile plant operators"
    			},
    			"91": {
    				label: "Cleaners and helpers"
    			},
    			"92": {
    				label: "Agricultural, forestry and fishery labourers"
    			},
    			"93": {
    				label: "Labourers in mining, construction, manufacturing and transport"
    			},
    			"94": {
    				label: "Food preparation assistants"
    			},
    			"95": {
    				label: "Street and related sales and service workers"
    			},
    			"96": {
    				label: "Refuse workers and other elementary workers"
    			},
    			"111": {
    				label: "Legislators and senior officials"
    			},
    			"112": {
    				label: "Managing directors and chief executives"
    			},
    			"121": {
    				label: "Business services and administration managers"
    			},
    			"122": {
    				label: "Sales, marketing and development managers"
    			},
    			"131": {
    				label: "Production managers in agriculture, forestry and fisheries"
    			},
    			"132": {
    				label: "Manufacturing, mining, construction, and distribution managers"
    			},
    			"133": {
    				label: "Information and communications technology service managers"
    			},
    			"134": {
    				label: "Professional services managers"
    			},
    			"141": {
    				label: "Hotel and restaurant managers"
    			},
    			"142": {
    				label: "Retail and wholesale trade managers"
    			},
    			"143": {
    				label: "Other services managers"
    			},
    			"211": {
    				label: "Physical and earth science professionals"
    			},
    			"212": {
    				label: "Mathematicians, actuaries and statisticians"
    			},
    			"213": {
    				label: "Life science professionals"
    			},
    			"214": {
    				label: "Engineering professionals (excluding electrotechnology)"
    			},
    			"215": {
    				label: "Electrotechnology engineers"
    			},
    			"216": {
    				label: "Architects, planners, surveyors and designers"
    			},
    			"221": {
    				label: "Medical doctors"
    			},
    			"222": {
    				label: "Nursing and midwifery professionals"
    			},
    			"223": {
    				label: "Traditional and complementary medicine professionals"
    			},
    			"224": {
    				label: "Paramedical practitioners"
    			},
    			"225": {
    				label: "Veterinarians"
    			},
    			"226": {
    				label: "Other health professionals"
    			},
    			"231": {
    				label: "University and higher education teachers"
    			},
    			"232": {
    				label: "Vocational education teachers"
    			},
    			"233": {
    				label: "Secondary education teachers"
    			},
    			"234": {
    				label: "Primary school and early childhood teachers"
    			},
    			"235": {
    				label: "Other teaching professionals"
    			},
    			"241": {
    				label: "Finance professionals"
    			},
    			"242": {
    				label: "Administration professionals"
    			},
    			"243": {
    				label: "Sales, marketing and public relations professionals"
    			},
    			"251": {
    				label: "Software and applications developers and analysts"
    			},
    			"252": {
    				label: "Database and network professionals"
    			},
    			"261": {
    				label: "Legal professionals"
    			},
    			"262": {
    				label: "Librarians, archivists and curators"
    			},
    			"263": {
    				label: "Social and religious professionals"
    			},
    			"264": {
    				label: "Authors, journalists and linguists"
    			},
    			"265": {
    				label: "Creative and performing artists"
    			},
    			"311": {
    				label: "Physical and engineering science technicians"
    			},
    			"312": {
    				label: "Mining, manufacturing and construction supervisors"
    			},
    			"313": {
    				label: "Process control technicians"
    			},
    			"314": {
    				label: "Life science technicians and related associate professionals"
    			},
    			"315": {
    				label: "Ship and aircraft controllers and technicians"
    			},
    			"321": {
    				label: "Medical and pharmaceutical technicians"
    			},
    			"322": {
    				label: "Nursing and midwifery associate professionals"
    			},
    			"323": {
    				label: "Traditional and complementary medicine associate professionals"
    			},
    			"324": {
    				label: "Veterinary technicians and assistants"
    			},
    			"325": {
    				label: "Other health associate professionals"
    			},
    			"331": {
    				label: "Financial and mathematical associate professionals"
    			},
    			"332": {
    				label: "Sales and purchasing agents and brokers"
    			},
    			"333": {
    				label: "Business services agents"
    			},
    			"334": {
    				label: "Administrative and specialised secretaries"
    			},
    			"335": {
    				label: "Regulatory government associate professionals"
    			},
    			"341": {
    				label: "Legal, social and religious associate professionals"
    			},
    			"342": {
    				label: "Sports and fitness workers"
    			},
    			"343": {
    				label: "Artistic, cultural and culinary associate professionals"
    			},
    			"351": {
    				label: "Information and communications technology operations and user support technicians"
    			},
    			"352": {
    				label: "Telecommunications and broadcasting technicians"
    			},
    			"411": {
    				label: "General office clerks"
    			},
    			"412": {
    				label: "Secretaries (general)"
    			},
    			"413": {
    				label: "Keyboard operators"
    			},
    			"421": {
    				label: "Tellers, money collectors and related clerks"
    			},
    			"422": {
    				label: "Client information workers"
    			},
    			"431": {
    				label: "Numerical clerks"
    			},
    			"432": {
    				label: "Material-recording and transport clerks"
    			},
    			"441": {
    				label: "Other clerical support workers"
    			},
    			"511": {
    				label: "Travel attendants, conductors and guides"
    			},
    			"512": {
    				label: "Cooks"
    			},
    			"513": {
    				label: "Waiters and bartenders"
    			},
    			"514": {
    				label: "Hairdressers, beauticians and related workers"
    			},
    			"515": {
    				label: "Building and housekeeping supervisors"
    			},
    			"516": {
    				label: "Other personal services workers"
    			},
    			"521": {
    				label: "Street and market salespersons"
    			},
    			"522": {
    				label: "Shop salespersons"
    			},
    			"523": {
    				label: "Cashiers and ticket clerks"
    			},
    			"524": {
    				label: "Other sales workers"
    			},
    			"531": {
    				label: "Child care workers and teachers' aides"
    			},
    			"532": {
    				label: "Personal care workers in health services"
    			},
    			"541": {
    				label: "Protective services workers"
    			},
    			"611": {
    				label: "Market gardeners and crop growers"
    			},
    			"612": {
    				label: "Animal producers"
    			},
    			"613": {
    				label: "Mixed crop and animal producers"
    			},
    			"621": {
    				label: "Forestry and related workers"
    			},
    			"622": {
    				label: "Fishery workers, hunters and trappers"
    			},
    			"631": {
    				label: "Subsistence crop farmers"
    			},
    			"632": {
    				label: "Subsistence livestock farmers"
    			},
    			"633": {
    				label: "Subsistence mixed crop and livestock farmers"
    			},
    			"634": {
    				label: "Subsistence fishers, hunters, trappers and gatherers"
    			},
    			"711": {
    				label: "Building frame and related trades workers"
    			},
    			"712": {
    				label: "Building finishers and related trades workers"
    			},
    			"713": {
    				label: "Painters, building structure cleaners and related trades workers"
    			},
    			"721": {
    				label: "Sheet and structural metal workers, moulders and welders, and related workers"
    			},
    			"722": {
    				label: "Blacksmiths, toolmakers and related trades workers"
    			},
    			"723": {
    				label: "Machinery mechanics and repairers"
    			},
    			"731": {
    				label: "Handicraft workers"
    			},
    			"732": {
    				label: "Printing trades workers"
    			},
    			"741": {
    				label: "Electrical equipment installers and repairers"
    			},
    			"742": {
    				label: "Electronics and telecommunications installers and repairers"
    			},
    			"751": {
    				label: "Food processing and related trades workers"
    			},
    			"752": {
    				label: "Wood treaters, cabinet-makers and related trades workers"
    			},
    			"753": {
    				label: "Garment and related trades workers"
    			},
    			"754": {
    				label: "Other craft and related workers"
    			},
    			"811": {
    				label: "Mining and mineral processing plant operators"
    			},
    			"812": {
    				label: "Metal processing and finishing plant operators"
    			},
    			"813": {
    				label: "Chemical and photographic products plant and machine operators"
    			},
    			"814": {
    				label: "Rubber, plastic and paper products machine operators"
    			},
    			"815": {
    				label: "Textile, fur and leather products machine operators"
    			},
    			"816": {
    				label: "Food and related products machine operators"
    			},
    			"817": {
    				label: "Wood processing and papermaking plant operators"
    			},
    			"818": {
    				label: "Other stationary plant and machine operators"
    			},
    			"821": {
    				label: "Assemblers"
    			},
    			"831": {
    				label: "Locomotive engine drivers and related workers"
    			},
    			"832": {
    				label: "Car, van and motorcycle drivers"
    			},
    			"833": {
    				label: "Heavy truck and bus drivers"
    			},
    			"834": {
    				label: "Mobile plant operators"
    			},
    			"835": {
    				label: "Ships' deck crews and related workers"
    			},
    			"911": {
    				label: "Domestic, hotel and office cleaners and helpers"
    			},
    			"912": {
    				label: "Vehicle, window, laundry and other hand cleaning workers"
    			},
    			"921": {
    				label: "Agricultural, forestry and fishery labourers"
    			},
    			"931": {
    				label: "Mining and construction labourers"
    			},
    			"932": {
    				label: "Manufacturing labourers"
    			},
    			"933": {
    				label: "Transport and storage labourers"
    			},
    			"941": {
    				label: "Food preparation assistants"
    			},
    			"951": {
    				label: "Street and related service workers"
    			},
    			"952": {
    				label: "Street vendors (excluding food)"
    			},
    			"961": {
    				label: "Refuse workers"
    			},
    			"962": {
    				label: "Other elementary workers"
    			},
    			"1111": {
    				label: "Legislators"
    			},
    			"1112": {
    				label: "Senior government officials"
    			},
    			"1113": {
    				label: "Traditional chiefs and heads of village"
    			},
    			"1114": {
    				label: "Senior officials of special-interest organizations"
    			},
    			"1120": {
    				label: "Managing directors and chief executives"
    			},
    			"1211": {
    				label: "Finance managers"
    			},
    			"1212": {
    				label: "Human resource managers"
    			},
    			"1213": {
    				label: "Policy and planning managers"
    			},
    			"1219": {
    				label: "Business services and administration managers not elsewhere classified"
    			},
    			"1221": {
    				label: "Sales and marketing managers"
    			},
    			"1222": {
    				label: "Advertising and public relations managers"
    			},
    			"1223": {
    				label: "Research and development managers"
    			},
    			"1311": {
    				label: "Agricultural and forestry production managers"
    			},
    			"1312": {
    				label: "Aquaculture and fisheries production managers"
    			},
    			"1321": {
    				label: "Manufacturing managers"
    			},
    			"1322": {
    				label: "Mining managers"
    			},
    			"1323": {
    				label: "Construction managers"
    			},
    			"1324": {
    				label: "Supply, distribution and related managers"
    			},
    			"1330": {
    				label: "Information and communications technology service managers"
    			},
    			"1341": {
    				label: "Child care services managers"
    			},
    			"1342": {
    				label: "Health services managers"
    			},
    			"1343": {
    				label: "Aged care services managers"
    			},
    			"1344": {
    				label: "Social welfare managers"
    			},
    			"1345": {
    				label: "Education managers"
    			},
    			"1346": {
    				label: "Financial and insurance services branch managers"
    			},
    			"1349": {
    				label: "Professional services managers not elsewhere classified"
    			},
    			"1411": {
    				label: "Hotel managers"
    			},
    			"1412": {
    				label: "Restaurant managers"
    			},
    			"1420": {
    				label: "Retail and wholesale trade managers"
    			},
    			"1431": {
    				label: "Sports, recreation and cultural centre managers"
    			},
    			"1439": {
    				label: "Services managers not elsewhere classified"
    			},
    			"2111": {
    				label: "Physicists and astronomers"
    			},
    			"2112": {
    				label: "Meteorologists"
    			},
    			"2113": {
    				label: "Chemists"
    			},
    			"2114": {
    				label: "Geologists and geophysicists"
    			},
    			"2120": {
    				label: "Mathematicians, actuaries and statisticians"
    			},
    			"2131": {
    				label: "Biologists, botanists, zoologists and related professionals"
    			},
    			"2132": {
    				label: "Farming, forestry and fisheries advisers"
    			},
    			"2133": {
    				label: "Environmental protection professionals"
    			},
    			"2141": {
    				label: "Industrial and production engineers"
    			},
    			"2142": {
    				label: "Civil engineers"
    			},
    			"2143": {
    				label: "Environmental engineers"
    			},
    			"2144": {
    				label: "Mechanical engineers"
    			},
    			"2145": {
    				label: "Chemical engineers"
    			},
    			"2146": {
    				label: "Mining engineers, metallurgists and related professionals"
    			},
    			"2149": {
    				label: "Engineering professionals not elsewhere classified"
    			},
    			"2151": {
    				label: "Electrical engineers"
    			},
    			"2152": {
    				label: "Electronics engineers"
    			},
    			"2153": {
    				label: "Telecommunications engineers"
    			},
    			"2161": {
    				label: "Building architects"
    			},
    			"2162": {
    				label: "Landscape architects"
    			},
    			"2163": {
    				label: "Product and garment designers"
    			},
    			"2164": {
    				label: "Town and traffic planners"
    			},
    			"2165": {
    				label: "Cartographers and surveyors"
    			},
    			"2166": {
    				label: "Graphic and multimedia designers"
    			},
    			"2211": {
    				label: "Generalist medical practitioners"
    			},
    			"2212": {
    				label: "Specialist medical practitioners"
    			},
    			"2221": {
    				label: "Nursing professionals"
    			},
    			"2222": {
    				label: "Midwifery professionals"
    			},
    			"2230": {
    				label: "Traditional and complementary medicine professionals"
    			},
    			"2240": {
    				label: "Paramedical practitioners"
    			},
    			"2250": {
    				label: "Veterinarians"
    			},
    			"2261": {
    				label: "Dentists"
    			},
    			"2262": {
    				label: "Pharmacists"
    			},
    			"2263": {
    				label: "Environmental and occupational health and hygiene professionals"
    			},
    			"2264": {
    				label: "Physiotherapists"
    			},
    			"2265": {
    				label: "Dieticians and nutritionists"
    			},
    			"2266": {
    				label: "Audiologists and speech therapists"
    			},
    			"2267": {
    				label: "Optometrists and ophthalmic opticians"
    			},
    			"2269": {
    				label: "Health professionals not elsewhere classified"
    			},
    			"2310": {
    				label: "University and higher education teachers"
    			},
    			"2320": {
    				label: "Vocational education teachers"
    			},
    			"2330": {
    				label: "Secondary education teachers"
    			},
    			"2341": {
    				label: "Primary school teachers"
    			},
    			"2342": {
    				label: "Early childhood educators"
    			},
    			"2351": {
    				label: "Education methods specialists"
    			},
    			"2352": {
    				label: "Special needs teachers"
    			},
    			"2353": {
    				label: "Other language teachers"
    			},
    			"2354": {
    				label: "Other music teachers"
    			},
    			"2355": {
    				label: "Other arts teachers"
    			},
    			"2356": {
    				label: "Information technology trainers"
    			},
    			"2359": {
    				label: "Teaching professionals not elsewhere classified"
    			},
    			"2411": {
    				label: "Accountants"
    			},
    			"2412": {
    				label: "Financial and investment advisers"
    			},
    			"2413": {
    				label: "Financial analysts"
    			},
    			"2421": {
    				label: "Management and organization analysts"
    			},
    			"2422": {
    				label: "Policy administration professionals"
    			},
    			"2423": {
    				label: "Personnel and careers professionals"
    			},
    			"2424": {
    				label: "Training and staff development professionals"
    			},
    			"2431": {
    				label: "Advertising and marketing professionals"
    			},
    			"2432": {
    				label: "Public relations professionals"
    			},
    			"2433": {
    				label: "Technical and medical sales professionals (excluding ICT)"
    			},
    			"2434": {
    				label: "Information and communications technology sales professionals"
    			},
    			"2511": {
    				label: "Systems analysts"
    			},
    			"2512": {
    				label: "Software developers"
    			},
    			"2513": {
    				label: "Web and multimedia developers"
    			},
    			"2514": {
    				label: "Applications programmers"
    			},
    			"2519": {
    				label: "Software and applications developers and analysts not elsewhere classified"
    			},
    			"2521": {
    				label: "Database designers and administrators"
    			},
    			"2522": {
    				label: "Systems administrators"
    			},
    			"2523": {
    				label: "Computer network professionals"
    			},
    			"2529": {
    				label: "Database and network professionals not elsewhere classified"
    			},
    			"2611": {
    				label: "Lawyers"
    			},
    			"2612": {
    				label: "Judges"
    			},
    			"2619": {
    				label: "Legal professionals not elsewhere classified"
    			},
    			"2621": {
    				label: "Archivists and curators"
    			},
    			"2622": {
    				label: "Librarians and related information professionals"
    			},
    			"2631": {
    				label: "Economists"
    			},
    			"2632": {
    				label: "Sociologists, anthropologists and related professionals"
    			},
    			"2633": {
    				label: "Philosophers, historians and political scientists"
    			},
    			"2634": {
    				label: "Psychologists"
    			},
    			"2635": {
    				label: "Social work and counselling professionals"
    			},
    			"2636": {
    				label: "Religious professionals"
    			},
    			"2641": {
    				label: "Authors and related writers"
    			},
    			"2642": {
    				label: "Journalists"
    			},
    			"2643": {
    				label: "Translators, interpreters and other linguists"
    			},
    			"2651": {
    				label: "Visual artists"
    			},
    			"2652": {
    				label: "Musicians, singers and composers"
    			},
    			"2653": {
    				label: "Dancers and choreographers"
    			},
    			"2654": {
    				label: "Film, stage and related directors and producers"
    			},
    			"2655": {
    				label: "Actors"
    			},
    			"2656": {
    				label: "Announcers on radio, television and other media"
    			},
    			"2659": {
    				label: "Creative and performing artists not elsewhere classified"
    			},
    			"3111": {
    				label: "Chemical and physical science technicians"
    			},
    			"3112": {
    				label: "Civil engineering technicians"
    			},
    			"3113": {
    				label: "Electrical engineering technicians"
    			},
    			"3114": {
    				label: "Electronics engineering technicians"
    			},
    			"3115": {
    				label: "Mechanical engineering technicians"
    			},
    			"3116": {
    				label: "Chemical engineering technicians"
    			},
    			"3117": {
    				label: "Mining and metallurgical technicians"
    			},
    			"3118": {
    				label: "Draughtspersons"
    			},
    			"3119": {
    				label: "Physical and engineering science technicians not elsewhere classified"
    			},
    			"3121": {
    				label: "Mining supervisors"
    			},
    			"3122": {
    				label: "Manufacturing supervisors"
    			},
    			"3123": {
    				label: "Construction supervisors"
    			},
    			"3131": {
    				label: "Power production plant operators"
    			},
    			"3132": {
    				label: "Incinerator and water treatment plant operators"
    			},
    			"3133": {
    				label: "Chemical processing plant controllers"
    			},
    			"3134": {
    				label: "Petroleum and natural gas refining plant operators"
    			},
    			"3135": {
    				label: "Metal production process controllers"
    			},
    			"3139": {
    				label: "Process control technicians not elsewhere classified"
    			},
    			"3141": {
    				label: "Life science technicians (excluding medical)"
    			},
    			"3142": {
    				label: "Agricultural technicians"
    			},
    			"3143": {
    				label: "Forestry technicians"
    			},
    			"3151": {
    				label: "Ships' engineers"
    			},
    			"3152": {
    				label: "Ships' deck officers and pilots"
    			},
    			"3153": {
    				label: "Aircraft pilots and related associate professionals"
    			},
    			"3154": {
    				label: "Air traffic controllers"
    			},
    			"3155": {
    				label: "Air traffic safety electronics technicians"
    			},
    			"3211": {
    				label: "Medical imaging and therapeutic equipment technicians"
    			},
    			"3212": {
    				label: "Medical and pathology laboratory technicians"
    			},
    			"3213": {
    				label: "Pharmaceutical technicians and assistants"
    			},
    			"3214": {
    				label: "Medical and dental prosthetic technicians"
    			},
    			"3221": {
    				label: "Nursing associate professionals"
    			},
    			"3222": {
    				label: "Midwifery associate professionals"
    			},
    			"3230": {
    				label: "Traditional and complementary medicine associate professionals"
    			},
    			"3240": {
    				label: "Veterinary technicians and assistants"
    			},
    			"3251": {
    				label: "Dental assistants and therapists"
    			},
    			"3252": {
    				label: "Medical records and health information technicians"
    			},
    			"3253": {
    				label: "Community health workers"
    			},
    			"3254": {
    				label: "Dispensing opticians"
    			},
    			"3255": {
    				label: "Physiotherapy technicians and assistants"
    			},
    			"3256": {
    				label: "Medical assistants"
    			},
    			"3257": {
    				label: "Environmental and occupational health inspectors and associates"
    			},
    			"3258": {
    				label: "Ambulance workers"
    			},
    			"3259": {
    				label: "Health associate professionals not elsewhere classified"
    			},
    			"3311": {
    				label: "Securities and finance dealers and brokers"
    			},
    			"3312": {
    				label: "Credit and loans officers"
    			},
    			"3313": {
    				label: "Accounting associate professionals"
    			},
    			"3314": {
    				label: "Statistical, mathematical and related associate professionals"
    			},
    			"3315": {
    				label: "Valuers and loss assessors"
    			},
    			"3321": {
    				label: "Insurance representatives"
    			},
    			"3322": {
    				label: "Commercial sales representatives"
    			},
    			"3323": {
    				label: "Buyers"
    			},
    			"3324": {
    				label: "Trade brokers"
    			},
    			"3331": {
    				label: "Clearing and forwarding agents"
    			},
    			"3332": {
    				label: "Conference and event planners"
    			},
    			"3333": {
    				label: "Employment agents and contractors"
    			},
    			"3334": {
    				label: "Real estate agents and property managers"
    			},
    			"3339": {
    				label: "Business services agents not elsewhere classified"
    			},
    			"3341": {
    				label: "Office supervisors"
    			},
    			"3342": {
    				label: "Legal secretaries"
    			},
    			"3343": {
    				label: "Administrative and executive secretaries"
    			},
    			"3344": {
    				label: "Medical secretaries"
    			},
    			"3351": {
    				label: "Customs and border inspectors"
    			},
    			"3352": {
    				label: "Government tax and excise officials"
    			},
    			"3353": {
    				label: "Government social benefits officials"
    			},
    			"3354": {
    				label: "Government licensing officials"
    			},
    			"3355": {
    				label: "Police inspectors and detectives"
    			},
    			"3359": {
    				label: "Regulatory government associate professionals not elsewhere classified"
    			},
    			"3411": {
    				label: "Legal and Related Associate Professionals"
    			},
    			"3412": {
    				label: "Social work associate professionals"
    			},
    			"3413": {
    				label: "Religious associate professionals"
    			},
    			"3421": {
    				label: "Athletes and sports players"
    			},
    			"3422": {
    				label: "Sports coaches, instructors and officials"
    			},
    			"3423": {
    				label: "Fitness and recreation instructors and program leaders"
    			},
    			"3431": {
    				label: "Photographers"
    			},
    			"3432": {
    				label: "Interior designers and decorators"
    			},
    			"3433": {
    				label: "Gallery, museum and library technicians"
    			},
    			"3434": {
    				label: "Chefs"
    			},
    			"3435": {
    				label: "Other artistic and cultural associate professionals"
    			},
    			"3511": {
    				label: "Information and communications technology operations technicians"
    			},
    			"3512": {
    				label: "Information and communications technology user support technicians"
    			},
    			"3513": {
    				label: "Computer network and systems technicians"
    			},
    			"3514": {
    				label: "Web technicians"
    			},
    			"3521": {
    				label: "Broadcasting and audio-visual technicians"
    			},
    			"3522": {
    				label: "Telecommunications engineering technicians"
    			},
    			"4110": {
    				label: "General office clerks"
    			},
    			"4120": {
    				label: "Secretaries (general)"
    			},
    			"4131": {
    				label: "Typists and word processing operators"
    			},
    			"4132": {
    				label: "Data entry clerks"
    			},
    			"4211": {
    				label: "Bank tellers and related clerks"
    			},
    			"4212": {
    				label: "Bookmakers, croupiers and related gaming workers"
    			},
    			"4213": {
    				label: "Pawnbrokers and money-lenders"
    			},
    			"4214": {
    				label: "Debt-collectors and related workers"
    			},
    			"4221": {
    				label: "Travel consultants and clerks"
    			},
    			"4222": {
    				label: "Contact centre information clerks"
    			},
    			"4223": {
    				label: "Telephone switchboard operators"
    			},
    			"4224": {
    				label: "Hotel receptionists"
    			},
    			"4225": {
    				label: "Enquiry clerks"
    			},
    			"4226": {
    				label: "Receptionists (general)"
    			},
    			"4227": {
    				label: "Survey and market research interviewers"
    			},
    			"4229": {
    				label: "Client information workers not elsewhere classified"
    			},
    			"4311": {
    				label: "Accounting and bookkeeping clerks"
    			},
    			"4312": {
    				label: "Statistical, finance and insurance clerks"
    			},
    			"4313": {
    				label: "Payroll clerks"
    			},
    			"4321": {
    				label: "Stock clerks"
    			},
    			"4322": {
    				label: "Production clerks"
    			},
    			"4323": {
    				label: "Transport clerks"
    			},
    			"4411": {
    				label: "Library clerks"
    			},
    			"4412": {
    				label: "Mail carriers and sorting clerks"
    			},
    			"4413": {
    				label: "Coding, proof-reading and related clerks"
    			},
    			"4414": {
    				label: "Scribes and related workers"
    			},
    			"4415": {
    				label: "Filing and copying clerks"
    			},
    			"4416": {
    				label: "Personnel clerks"
    			},
    			"4419": {
    				label: "Clerical support workers not elsewhere classified"
    			},
    			"5111": {
    				label: "Travel attendants and travel stewards"
    			},
    			"5112": {
    				label: "Transport conductors"
    			},
    			"5113": {
    				label: "Travel guides"
    			},
    			"5120": {
    				label: "Cooks"
    			},
    			"5131": {
    				label: "Waiters"
    			},
    			"5132": {
    				label: "Bartenders"
    			},
    			"5141": {
    				label: "Hairdressers"
    			},
    			"5142": {
    				label: "Beauticians and related workers"
    			},
    			"5151": {
    				label: "Cleaning and housekeeping supervisors in offices, hotels and other establishments"
    			},
    			"5152": {
    				label: "Domestic housekeepers"
    			},
    			"5153": {
    				label: "Building caretakers"
    			},
    			"5161": {
    				label: "Astrologers, fortune-tellers and related workers"
    			},
    			"5162": {
    				label: "Companions and valets"
    			},
    			"5163": {
    				label: "Undertakers and embalmers"
    			},
    			"5164": {
    				label: "Pet groomers and animal care workers"
    			},
    			"5165": {
    				label: "Driving instructors"
    			},
    			"5169": {
    				label: "Personal services workers not elsewhere classified"
    			},
    			"5211": {
    				label: "Stall and market salespersons"
    			},
    			"5212": {
    				label: "Street food salespersons"
    			},
    			"5221": {
    				label: "Shop keepers"
    			},
    			"5222": {
    				label: "Shop supervisors"
    			},
    			"5223": {
    				label: "Shop sales assistants"
    			},
    			"5230": {
    				label: "Cashiers and ticket clerks"
    			},
    			"5241": {
    				label: "Fashion and other models"
    			},
    			"5242": {
    				label: "Sales demonstrators"
    			},
    			"5243": {
    				label: "Door to door salespersons"
    			},
    			"5244": {
    				label: "Contact centre salespersons"
    			},
    			"5245": {
    				label: "Service station attendants"
    			},
    			"5246": {
    				label: "Food service counter attendants"
    			},
    			"5249": {
    				label: "Sales workers not elsewhere classified"
    			},
    			"5311": {
    				label: "Child care workers"
    			},
    			"5312": {
    				label: "Teachers' aides"
    			},
    			"5321": {
    				label: "Health care assistants"
    			},
    			"5322": {
    				label: "Home-based personal care workers"
    			},
    			"5329": {
    				label: "Personal care workers in health services not elsewhere classified"
    			},
    			"5411": {
    				label: "Fire-fighters"
    			},
    			"5412": {
    				label: "Police officers"
    			},
    			"5413": {
    				label: "Prison guards"
    			},
    			"5414": {
    				label: "Security guards"
    			},
    			"5419": {
    				label: "Protective services workers not elsewhere classified"
    			},
    			"6111": {
    				label: "Field crop and vegetable growers"
    			},
    			"6112": {
    				label: "Tree and shrub crop growers"
    			},
    			"6113": {
    				label: "Gardeners, horticultural and nursery growers"
    			},
    			"6114": {
    				label: "Mixed crop growers"
    			},
    			"6121": {
    				label: "Livestock and dairy producers"
    			},
    			"6122": {
    				label: "Poultry producers"
    			},
    			"6123": {
    				label: "Apiarists and sericulturists"
    			},
    			"6129": {
    				label: "Animal producers not elsewhere classified"
    			},
    			"6130": {
    				label: "Mixed crop and animal producers"
    			},
    			"6210": {
    				label: "Forestry and related workers"
    			},
    			"6221": {
    				label: "Aquaculture workers"
    			},
    			"6222": {
    				label: "Inland and coastal waters fishery workers"
    			},
    			"6223": {
    				label: "Deep-sea fishery workers"
    			},
    			"6224": {
    				label: "Hunters and trappers"
    			},
    			"6310": {
    				label: "Subsistence crop farmers"
    			},
    			"6320": {
    				label: "Subsistence livestock farmers"
    			},
    			"6330": {
    				label: "Subsistence mixed crop and livestock farmers"
    			},
    			"6340": {
    				label: "Subsistence fishers, hunters, trappers and gatherers"
    			},
    			"7111": {
    				label: "House builders"
    			},
    			"7112": {
    				label: "Bricklayers and related workers"
    			},
    			"7113": {
    				label: "Stonemasons, stone cutters, splitters and carvers"
    			},
    			"7114": {
    				label: "Concrete placers, concrete finishers and related workers"
    			},
    			"7115": {
    				label: "Carpenters and joiners"
    			},
    			"7119": {
    				label: "Building frame and related trades workers not elsewhere classified"
    			},
    			"7121": {
    				label: "Roofers"
    			},
    			"7122": {
    				label: "Floor layers and tile setters"
    			},
    			"7123": {
    				label: "Plasterers"
    			},
    			"7124": {
    				label: "Insulation workers"
    			},
    			"7125": {
    				label: "Glaziers"
    			},
    			"7126": {
    				label: "Plumbers and pipe fitters"
    			},
    			"7127": {
    				label: "Air conditioning and refrigeration mechanics"
    			},
    			"7131": {
    				label: "Painters and related workers"
    			},
    			"7132": {
    				label: "Spray painters and varnishers"
    			},
    			"7133": {
    				label: "Building structure cleaners"
    			},
    			"7211": {
    				label: "Metal moulders and coremakers"
    			},
    			"7212": {
    				label: "Welders and flamecutters"
    			},
    			"7213": {
    				label: "Sheet-metal workers"
    			},
    			"7214": {
    				label: "Structural-metal preparers and erectors"
    			},
    			"7215": {
    				label: "Riggers and cable splicers"
    			},
    			"7221": {
    				label: "Blacksmiths, hammersmiths and forging press workers"
    			},
    			"7222": {
    				label: "Toolmakers and related workers"
    			},
    			"7223": {
    				label: "Metal working machine tool setters and operators"
    			},
    			"7224": {
    				label: "Metal polishers, wheel grinders and tool sharpeners"
    			},
    			"7231": {
    				label: "Motor vehicle mechanics and repairers"
    			},
    			"7232": {
    				label: "Aircraft engine mechanics and repairers"
    			},
    			"7233": {
    				label: "Agricultural and industrial machinery mechanics and repairers"
    			},
    			"7234": {
    				label: "Bicycle and related repairers"
    			},
    			"7311": {
    				label: "Precision-instrument makers and repairers"
    			},
    			"7312": {
    				label: "Musical instrument makers and tuners"
    			},
    			"7313": {
    				label: "Jewellery and precious-metal workers"
    			},
    			"7314": {
    				label: "Potters and related workers"
    			},
    			"7315": {
    				label: "Glass makers, cutters, grinders and finishers"
    			},
    			"7316": {
    				label: "Sign writers, decorative painters, engravers and etchers"
    			},
    			"7317": {
    				label: "Handicraft workers in wood, basketry and related materials"
    			},
    			"7318": {
    				label: "Handicraft workers in textile, leather and related materials"
    			},
    			"7319": {
    				label: "Handicraft workers not elsewhere classified"
    			},
    			"7321": {
    				label: "Pre-press technicians"
    			},
    			"7322": {
    				label: "Printers"
    			},
    			"7323": {
    				label: "Print finishing and binding workers"
    			},
    			"7411": {
    				label: "Building and related electricians"
    			},
    			"7412": {
    				label: "Electrical mechanics and fitters"
    			},
    			"7413": {
    				label: "Electrical line installers and repairers"
    			},
    			"7421": {
    				label: "Electronics mechanics and servicers"
    			},
    			"7422": {
    				label: "Information and communications technology installers and servicers"
    			},
    			"7511": {
    				label: "Butchers, fishmongers and related food preparers"
    			},
    			"7512": {
    				label: "Bakers, pastry-cooks and confectionery makers"
    			},
    			"7513": {
    				label: "Dairy-products makers"
    			},
    			"7514": {
    				label: "Fruit, vegetable and related preservers"
    			},
    			"7515": {
    				label: "Food and beverage tasters and graders"
    			},
    			"7516": {
    				label: "Tobacco preparers and tobacco products makers"
    			},
    			"7521": {
    				label: "Wood treaters"
    			},
    			"7522": {
    				label: "Cabinet-makers and related workers"
    			},
    			"7523": {
    				label: "Woodworking-machine tool setters and operators"
    			},
    			"7531": {
    				label: "Tailors, dressmakers, furriers and hatters"
    			},
    			"7532": {
    				label: "Garment and related pattern-makers and cutters"
    			},
    			"7533": {
    				label: "Sewing, embroidery and related workers"
    			},
    			"7534": {
    				label: "Upholsterers and related workers"
    			},
    			"7535": {
    				label: "Pelt dressers, tanners and fellmongers"
    			},
    			"7536": {
    				label: "Shoemakers and related workers"
    			},
    			"7541": {
    				label: "Underwater divers"
    			},
    			"7542": {
    				label: "Shotfirers and blasters"
    			},
    			"7543": {
    				label: "Product graders and testers (excluding foods and beverages)"
    			},
    			"7544": {
    				label: "Fumigators and other pest and weed controllers"
    			},
    			"7549": {
    				label: "Craft and related workers not elsewhere classified"
    			},
    			"8111": {
    				label: "Miners and quarriers"
    			},
    			"8112": {
    				label: "Mineral and stone processing plant operators"
    			},
    			"8113": {
    				label: "Well drillers and borers and related workers"
    			},
    			"8114": {
    				label: "Cement, stone and other mineral products machine operators"
    			},
    			"8121": {
    				label: "Metal processing plant operators"
    			},
    			"8122": {
    				label: "Metal finishing, plating and coating machine operators"
    			},
    			"8131": {
    				label: "Chemical products plant and machine operators"
    			},
    			"8132": {
    				label: "Photographic products machine operators"
    			},
    			"8141": {
    				label: "Rubber products machine operators"
    			},
    			"8142": {
    				label: "Plastic products machine operators"
    			},
    			"8143": {
    				label: "Paper products machine operators"
    			},
    			"8151": {
    				label: "Fibre preparing, spinning and winding machine operators"
    			},
    			"8152": {
    				label: "Weaving and knitting machine operators"
    			},
    			"8153": {
    				label: "Sewing machine operators"
    			},
    			"8154": {
    				label: "Bleaching, dyeing and fabric cleaning machine operators"
    			},
    			"8155": {
    				label: "Fur and leather preparing machine operators"
    			},
    			"8156": {
    				label: "Shoemaking and related machine operators"
    			},
    			"8157": {
    				label: "Laundry machine operators"
    			},
    			"8159": {
    				label: "Textile, fur and leather products machine operators not elsewhere classified"
    			},
    			"8160": {
    				label: "Food and related products machine operators"
    			},
    			"8171": {
    				label: "Pulp and papermaking plant operators"
    			},
    			"8172": {
    				label: "Wood processing plant operators"
    			},
    			"8181": {
    				label: "Glass and ceramics plant operators"
    			},
    			"8182": {
    				label: "Steam engine and boiler operators"
    			},
    			"8183": {
    				label: "Packing, bottling and labelling machine operators"
    			},
    			"8189": {
    				label: "Stationary plant and machine operators not elsewhere classified"
    			},
    			"8211": {
    				label: "Mechanical machinery assemblers"
    			},
    			"8212": {
    				label: "Electrical and electronic equipment assemblers"
    			},
    			"8219": {
    				label: "Assemblers not elsewhere classified"
    			},
    			"8311": {
    				label: "Locomotive engine drivers"
    			},
    			"8312": {
    				label: "Railway brake, signal and switch operators"
    			},
    			"8321": {
    				label: "Motorcycle drivers"
    			},
    			"8322": {
    				label: "Car, taxi and van drivers"
    			},
    			"8331": {
    				label: "Bus and tram drivers"
    			},
    			"8332": {
    				label: "Heavy truck and lorry drivers"
    			},
    			"8341": {
    				label: "Mobile farm and forestry plant operators"
    			},
    			"8342": {
    				label: "Earthmoving and related plant operators"
    			},
    			"8343": {
    				label: "Crane, hoist and related plant operators"
    			},
    			"8344": {
    				label: "Lifting truck operators"
    			},
    			"8350": {
    				label: "Ships' deck crews and related workers"
    			},
    			"9111": {
    				label: "Domestic cleaners and helpers"
    			},
    			"9112": {
    				label: "Cleaners and helpers in offices, hotels and other establishments"
    			},
    			"9121": {
    				label: "Hand launderers and pressers"
    			},
    			"9122": {
    				label: "Vehicle cleaners"
    			},
    			"9123": {
    				label: "Window cleaners"
    			},
    			"9129": {
    				label: "Other cleaning workers"
    			},
    			"9211": {
    				label: "Crop farm labourers"
    			},
    			"9212": {
    				label: "Livestock farm labourers"
    			},
    			"9213": {
    				label: "Mixed crop and livestock farm labourers"
    			},
    			"9214": {
    				label: "Garden and horticultural labourers"
    			},
    			"9215": {
    				label: "Forestry labourers"
    			},
    			"9216": {
    				label: "Fishery and aquaculture labourers"
    			},
    			"9311": {
    				label: "Mining and quarrying labourers"
    			},
    			"9312": {
    				label: "Civil engineering labourers"
    			},
    			"9313": {
    				label: "Building construction labourers"
    			},
    			"9321": {
    				label: "Hand packers"
    			},
    			"9329": {
    				label: "Manufacturing labourers not elsewhere classified"
    			},
    			"9331": {
    				label: "Hand and pedal vehicle drivers"
    			},
    			"9332": {
    				label: "Drivers of animal-drawn vehicles and machinery"
    			},
    			"9333": {
    				label: "Freight handlers"
    			},
    			"9334": {
    				label: "Shelf fillers"
    			},
    			"9411": {
    				label: "Fast food preparers"
    			},
    			"9412": {
    				label: "Kitchen helpers"
    			},
    			"9510": {
    				label: "Street and related service workers"
    			},
    			"9520": {
    				label: "Street vendors (excluding food)"
    			},
    			"9611": {
    				label: "Garbage and recycling collectors"
    			},
    			"9612": {
    				label: "Refuse sorters"
    			},
    			"9613": {
    				label: "Sweepers and related labourers"
    			},
    			"9621": {
    				label: "Messengers, package deliverers and luggage porters"
    			},
    			"9622": {
    				label: "Odd job persons"
    			},
    			"9623": {
    				label: "Meter readers and vending-machine collectors"
    			},
    			"9624": {
    				label: "Water and firewood collectors"
    			},
    			"9629": {
    				label: "Elementary workers not elsewhere classified"
    			},
    			"11121": {
    				label: "Senior central government officials"
    			},
    			"11122": {
    				label: "Senior district and local government officials and senior officials"
    			},
    			"11141": {
    				label: "Senior officials of employers', workers' and other economic-interest organisations"
    			},
    			"11142": {
    				label: "Senior officials of humanitarian and other special-interest organisations"
    			},
    			"21321": {
    				label: "Agronomists and fishery professionals"
    			},
    			"21322": {
    				label: "Forestry professionals"
    			},
    			"21531": {
    				label: "Telecommunications researchers"
    			},
    			"21532": {
    				label: "Information technology researchers"
    			},
    			"22121": {
    				label: "Senior physicians"
    			},
    			"22122": {
    				label: "Specialists"
    			},
    			"22211": {
    				label: "Matrons"
    			},
    			"22212": {
    				label: "Ward sisters"
    			},
    			"23101": {
    				label: "Professors"
    			},
    			"23102": {
    				label: "Lecturers and senior assistants (university)"
    			},
    			"23103": {
    				label: "Assistants and part-time lecturers (university)"
    			},
    			"23104": {
    				label: "Head teachers (polytechnics)"
    			},
    			"23105": {
    				label: "Lecturers (polytechnics)"
    			},
    			"23106": {
    				label: "Part-time lecturers, etc. (polytechnics)"
    			},
    			"23301": {
    				label: "Teachers in mathematical subjects"
    			},
    			"23302": {
    				label: "Native language and literature teachers"
    			},
    			"23303": {
    				label: "Crafts and art teachers"
    			},
    			"23304": {
    				label: "Other secondary education teaching professionals"
    			},
    			"23411": {
    				label: "Class teachers in primary education"
    			},
    			"23412": {
    				label: "Subject teachers in primary education"
    			},
    			"23591": {
    				label: "Career counsellors"
    			},
    			"23592": {
    				label: "Other teaching professionals"
    			},
    			"26211": {
    				label: "Archivists"
    			},
    			"26212": {
    				label: "Curators"
    			},
    			"26351": {
    				label: "Social workers, etc."
    			},
    			"26352": {
    				label: "Social planners, etc."
    			},
    			"26421": {
    				label: "Managing editors and subeditors"
    			},
    			"26422": {
    				label: "Journalists"
    			},
    			"26423": {
    				label: "Radio and television journalists"
    			},
    			"31121": {
    				label: "Building construction technicians"
    			},
    			"31122": {
    				label: "Civil engineering technicians"
    			},
    			"31123": {
    				label: "Cartographic and surveying technicians"
    			},
    			"31521": {
    				label: "Large ships' officers and mates"
    			},
    			"31522": {
    				label: "Small ships' officers"
    			},
    			"31523": {
    				label: "Harbour traffic controllers and harbour masters"
    			},
    			"32141": {
    				label: "Dental technicians"
    			},
    			"32142": {
    				label: "Prosthetist-orthotists"
    			},
    			"32211": {
    				label: "Nurses"
    			},
    			"32212": {
    				label: "Public health nurses"
    			},
    			"32591": {
    				label: "Occupational therapists"
    			},
    			"32592": {
    				label: "Other therapists"
    			},
    			"33341": {
    				label: "Estate agents"
    			},
    			"33342": {
    				label: "Property managers"
    			},
    			"34111": {
    				label: "Legal and related business associate professionals"
    			},
    			"34112": {
    				label: "Agents, officials and other organisation associate professionals"
    			},
    			"34113": {
    				label: "Consumer advisers, etc."
    			},
    			"34121": {
    				label: "Social instructors"
    			},
    			"34122": {
    				label: "Youth leaders (not parishes)"
    			},
    			"34123": {
    				label: "Work and craft leaders"
    			},
    			"34131": {
    				label: "Deacons and deaconesses"
    			},
    			"34139": {
    				label: "Other religious associate professionals"
    			},
    			"34351": {
    				label: "Producer's assistants and other stagecraft associate professionals"
    			},
    			"34359": {
    				label: "Other artistic and cultural associate professionals"
    			},
    			"42291": {
    				label: "Emergency officers"
    			},
    			"42299": {
    				label: "Other client information clerks not elsewhere classified"
    			},
    			"43231": {
    				label: "Rail traffic controllers, traffic co-ordinators, etc."
    			},
    			"43239": {
    				label: "Other transport clerks"
    			},
    			"44121": {
    				label: "Mail carriers"
    			},
    			"44122": {
    				label: "Porters"
    			},
    			"51201": {
    				label: "Cooks"
    			},
    			"51202": {
    				label: "Restaurant services supervisors and shift managers"
    			},
    			"51631": {
    				label: "Undertakers and related funeral workers"
    			},
    			"51632": {
    				label: "Other funeral workers"
    			},
    			"53111": {
    				label: "Childminders in kindergartens and other institutions"
    			},
    			"53112": {
    				label: "Private childminders"
    			},
    			"53113": {
    				label: "Children's club leaders"
    			},
    			"53211": {
    				label: "Practical mental nurses"
    			},
    			"53212": {
    				label: "Mental handicap nurses"
    			},
    			"53213": {
    				label: "Social work assistants"
    			},
    			"53219": {
    				label: "Other practical nurses"
    			},
    			"53221": {
    				label: "Household service workers"
    			},
    			"53222": {
    				label: "Personal care workers, family carers, etc."
    			},
    			"53291": {
    				label: "Dental assistants"
    			},
    			"53292": {
    				label: "Equipment maintenance assistants"
    			},
    			"53293": {
    				label: "Pharmaceutical assistants"
    			},
    			"53294": {
    				label: "Massage therapists and practical rehabilitation nurses"
    			},
    			"61111": {
    				label: "Crop growers"
    			},
    			"61112": {
    				label: "Field crop supervisors and workers"
    			},
    			"61131": {
    				label: "Gardeners, horticultural and nursery growers"
    			},
    			"61132": {
    				label: "Gardeners, horticultural and nursery growers and workers"
    			},
    			"61211": {
    				label: "Dairy and livestock producers"
    			},
    			"61212": {
    				label: "Dairy and livestock workers"
    			},
    			"61213": {
    				label: "Pet producers"
    			},
    			"61214": {
    				label: "Farm relief workers"
    			},
    			"61291": {
    				label: "Fur and reindeer producers"
    			},
    			"61299": {
    				label: "Animal workers not elsewhere classified"
    			},
    			"62211": {
    				label: "Fish farmers"
    			},
    			"62212": {
    				label: "Fishery supervisors and workers"
    			},
    			"74211": {
    				label: "Electronics installers and repairers"
    			},
    			"74212": {
    				label: "Automation installers and repairers"
    			},
    			"81821": {
    				label: "Power-production and waste treatment plant workers"
    			},
    			"81822": {
    				label: "Other plant workers, etc."
    			},
    			"83441": {
    				label: "Stevedores"
    			},
    			"83442": {
    				label: "Lifting-truck operators, etc."
    			},
    			"91121": {
    				label: "Office cleaners, etc."
    			},
    			"91122": {
    				label: "Hotel cleaners"
    			},
    			"91123": {
    				label: "Hospital and institutional helpers"
    			},
    			"91124": {
    				label: "Kindergarten assistants"
    			},
    			"91129": {
    				label: "Other cleaners not elsewhere classified"
    			},
    			"01": {
    				label: "Commissioned armed forces officers"
    			},
    			"011": {
    				label: "Commissioned armed forces officers"
    			},
    			"0110": {
    				label: "Commissioned armed forces officers"
    			},
    			"02": {
    				label: "Non-commissioned armed forces officers"
    			},
    			"021": {
    				label: "Non-commissioned armed forces officers"
    			},
    			"0210": {
    				label: "Non-commissioned armed forces officers"
    			},
    			"03": {
    				label: "Armed forces occupations, other ranks"
    			},
    			"031": {
    				label: "Armded forces occupations, other ranks"
    			},
    			"0310": {
    				label: "Armed forces occupations, other ranks"
    			},
    			"#X": {
    				label: "Unknown"
    			},
    			"#XX": {
    				label: "Unknown"
    			},
    			"#XXX": {
    				label: "Unknown"
    			},
    			"#XXXX": {
    				label: "Unknown"
    			},
    			"#XXXXX": {
    				label: "Unknown"
    			}
    		},
    		RemunerationKind: {
    			bonusPay: {
    				description: "",
    				label: ""
    			},
    			commission: {
    				description: "",
    				label: ""
    			},
    			initiativeFee: {
    				description: "",
    				label: ""
    			},
    			performanceBonus: {
    				description: "",
    				label: ""
    			},
    			profitSharingBonus: {
    				description: "",
    				label: ""
    			},
    			shareIssueForEmployees: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		ReportCategory: {
    			all: {
    				description: "",
    				label: ""
    			},
    			monthly: {
    				description: "",
    				label: ""
    			},
    			yearly: {
    				description: "",
    				label: ""
    			}
    		},
    		ReportType: {
    			employerReport: {
    				description: "",
    				label: "Employer Report"
    			},
    			employmentContract: {
    				description: "",
    				label: "Employment Contract"
    			},
    			example: {
    				description: "",
    				label: "Example"
    			},
    			householdDeduction: {
    				description: "",
    				label: "Household Deduction"
    			},
    			insurance: {
    				description: "",
    				label: "Insurance"
    			},
    			monthlyDetails: {
    				description: "",
    				label: "Monthly Details"
    			},
    			monthlyLiikekirjuri: {
    				description: "",
    				label: "Monthly Liikekirjuri"
    			},
    			monthlyPension: {
    				description: "",
    				label: "Monthly Pension"
    			},
    			monthlyRapko: {
    				description: "",
    				label: "Monthly Rapko"
    			},
    			paymentReport: {
    				description: "",
    				label: "Payment Report"
    			},
    			salarySlip: {
    				description: "",
    				label: "Salary Slip"
    			},
    			salarySlipPaid: {
    				description: "",
    				label: "Salary Slip Paid"
    			},
    			taxHouseholdDeduction14B: {
    				description: "",
    				label: "Tax Household Deduction14B"
    			},
    			taxHouseholdDeduction14BSpouseA: {
    				description: "",
    				label: "Tax Household Deduction14BSpouse A"
    			},
    			taxHouseholdDeduction14BSpouseB: {
    				description: "",
    				label: "Tax Household Deduction14BSpouse B"
    			},
    			taxMonthly4001: {
    				description: "",
    				label: "Tax Monthly4001"
    			},
    			taxYearly7801: {
    				description: "",
    				label: "Tax Yearly7801"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			unemployment: {
    				description: "",
    				label: "Unemployment"
    			},
    			yearEndReport: {
    				description: "",
    				label: "Year End Report"
    			},
    			yearlyDetails: {
    				description: "",
    				label: "Yearly Details"
    			},
    			yearlyWorkerSummary: {
    				description: "",
    				label: "Yearly Worker Summary"
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "EarningsPaymentReport"
    			},
    			employersSeparateReport: {
    				description: "",
    				label: "EmployersSeparateReport"
    			},
    			monthlyAccounting: {
    				description: "",
    				label: "MonthlyAccounting"
    			},
    			salarySlipCopy: {
    				description: "",
    				label: "SalarySlipCopy"
    			}
    		},
    		SalaryKind: {
    			compensation: {
    				description: "",
    				label: "Compensation",
    				labels: {
    					household: ""
    				}
    			},
    			fixedSalary: {
    				description: "",
    				label: "Fixed Salary"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Hourly Salary"
    			},
    			monthlySalary: {
    				description: "",
    				label: "Monthly Salary"
    			},
    			salary: {
    				description: "",
    				label: ""
    			},
    			totalEmployerPayment: {
    				description: "",
    				label: "Total Employer Payment"
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: "Total Worker Payment"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SalaryPeriod: {
    			fixed: {
    				description: "",
    				label: ""
    			},
    			hourly: {
    				description: "",
    				label: ""
    			},
    			monthly: {
    				description: "",
    				label: ""
    			},
    			notDefined: {
    				description: "",
    				label: ""
    			}
    		},
    		SalaryPerMonthRange: {
    			noPensionRequired: {
    				description: "",
    				label: "Under 57,10 €"
    			},
    			normal: {
    				description: "",
    				label: "57,10 € or over"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		SalaryPerYearRange: {
    			noMonthlyReporting: {
    				description: "",
    				label: "202 - 1500 €"
    			},
    			noReportingNecessary: {
    				description: "",
    				label: "2 - 200 €"
    			},
    			normal: {
    				description: "",
    				label: "Over 1500 €"
    			},
    			zeroSalary: {
    				description: "",
    				label: 1
    			}
    		},
    		ServiceModel: {
    			partnerOnly: {
    				label: ""
    			},
    			shared: {
    				label: ""
    			}
    		},
    		SubsidisedCommuteKind: {
    			noDeduction: {
    				description: "",
    				label: ""
    			},
    			periodicalDeduction: {
    				description: "",
    				label: ""
    			},
    			singleDeduction: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		TaxCardIncomeType: {
    			externalSalaries: {
    				description: "",
    				label: "External Salaries"
    			},
    			previousEmployerSalaries: {
    				description: "Salaries earned earlier under this year. Mark them here as one total sum in euros.",
    				label: "Salary paid by former employers"
    			},
    			salaxyCalculation: {
    				description: "",
    				label: "Salaxy Calculation"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			diff: {
    				description: "",
    				label: "Diff"
    			},
    			sharedCardExtSum: {
    				description: "",
    				label: "SharedCardExtSum"
    			}
    		},
    		TaxcardKind: {
    			defaultYearly: {
    				label: ""
    			},
    			historical: {
    				label: ""
    			},
    			noTaxCard: {
    				label: ""
    			},
    			noWithholdingHousehold: {
    				label: ""
    			},
    			others: {
    				label: ""
    			},
    			replacement: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			}
    		},
    		TaxcardState: {
    			employerAdded: {
    				label: "Added by employer"
    			},
    			expired: {
    				label: "Expired"
    			},
    			"new": {
    				label: "New"
    			},
    			replaced: {
    				label: "Replaced by new"
    			},
    			shared: {
    				label: "Shared by Worker"
    			},
    			sharedApproved: {
    				label: "Approved by employer"
    			},
    			sharedRejected: {
    				label: "Rejected"
    			},
    			sharedRejectedWithoutOpen: {
    				label: "Rejected without open"
    			},
    			sharedWaiting: {
    				label: "Pending"
    			},
    			verifiedVero: {
    				label: "Fetched from Tax authorities"
    			},
    			approved: {
    				description: "",
    				label: "Approved"
    			}
    		},
    		TaxcardValidity: {
    			expired: {
    				label: ""
    			},
    			future: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			},
    			valid: {
    				label: ""
    			},
    			validJanuary: {
    				label: ""
    			}
    		},
    		TaxDeductionWorkCategories: {
    			carework: {
    				description: "",
    				label: "Nursing and care work"
    			},
    			homeImprovement: {
    				description: "",
    				label: "Home improvement work"
    			},
    			householdwork: {
    				description: "",
    				label: "Household work"
    			},
    			none: {
    				description: "",
    				label: "Choose…"
    			},
    			ownPropety: {
    				description: "",
    				label: "Leisure home in own use"
    			},
    			relativesProperty: {
    				description: "",
    				label: "Leisure home of parents or relatives"
    			}
    		},
    		TesSubtype: {
    			constructionCarpenter: {
    				description: "",
    				label: ""
    			},
    			constructionFloor: {
    				description: "",
    				label: ""
    			},
    			constructionFreeContract: {
    				description: "",
    				label: ""
    			},
    			constructionOther: {
    				description: "",
    				label: ""
    			},
    			notSelected: {
    				description: "",
    				label: ""
    			}
    		},
    		TestEnum: {
    			valueEight: {
    				description: "",
    				label: ""
    			},
    			valueFive: {
    				description: "",
    				label: ""
    			},
    			valueFour: {
    				description: "",
    				label: ""
    			},
    			valueNine: {
    				description: "",
    				label: ""
    			},
    			valueOne: {
    				description: "",
    				label: ""
    			},
    			valueSevdescription: "",
    			valueSevlabel: "",
    			valueSix: {
    				description: "",
    				label: ""
    			},
    			valueTdescription: "",
    			valueThree: {
    				description: "",
    				label: ""
    			},
    			valueTlabel: "",
    			valueTwo: {
    				description: "",
    				label: ""
    			},
    			valueZero: {
    				description: "",
    				label: ""
    			},
    			valueSeven: {
    				description: "",
    				label: "ValueSeven"
    			},
    			valueTen: {
    				description: "",
    				label: "ValueTen"
    			}
    		},
    		UnionPaymentKind: {
    			fixed: {
    				description: "",
    				label: ""
    			},
    			percentage: {
    				description: "",
    				label: ""
    			},
    			other: {
    				description: "",
    				label: ""
    			},
    			raksaNormal: {
    				description: "",
    				label: ""
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		UnionPaymentType: {
    			notSelected: {
    				description: "",
    				label: "No membership"
    			},
    			other: {
    				description: "",
    				label: "Other union"
    			},
    			raksaNormal: {
    				description: "",
    				label: ""
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: ""
    			}
    		},
    		WageBasis: {
    			hourly: {
    				label: ""
    			},
    			monthly: {
    				label: ""
    			},
    			other: {
    				label: ""
    			},
    			performanceBased: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			}
    		},
    		WebSiteUserRole: {
    			company: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			household: {
    				description: "",
    				label: "Employer"
    			},
    			none: {
    				description: "",
    				label: "Choose…"
    			},
    			worker: {
    				description: "",
    				label: "Employee"
    			}
    		},
    		WorkingTimeCompensationKind: {
    			emergencyWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			eveningShiftAllowance: {
    				description: "",
    				label: ""
    			},
    			eveningWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			extraWorkPremium: {
    				description: "",
    				label: ""
    			},
    			nightShiftCompensation: {
    				description: "",
    				label: ""
    			},
    			nightWorkAllowance: {
    				description: "",
    				label: ""
    			},
    			otherCompensation: {
    				description: "",
    				label: ""
    			},
    			overtimeCompensation: {
    				description: "",
    				label: ""
    			},
    			saturdayPay: {
    				description: "",
    				label: ""
    			},
    			standByCompensation: {
    				description: "",
    				label: ""
    			},
    			sundayWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			waitingTimeCompensation: {
    				description: "",
    				label: ""
    			},
    			weeklyRestCompensation: {
    				description: "",
    				label: ""
    			}
    		},
    		YtjCompanyType: {
    			alandFederation: {
    				description: "",
    				label: "Ålandian enterprise"
    			},
    			asoAssociation: {
    				description: "",
    				label: "Housing right association"
    			},
    			association: {
    				description: "",
    				label: "Non-profit association"
    			},
    			condominium: {
    				description: "",
    				label: "Housing company"
    			},
    			cooperativeBank: {
    				description: "",
    				label: "Bank cooperative"
    			},
    			europeanCooperative: {
    				description: "",
    				label: "European cooperative"
    			},
    			europeanCooperativeBank: {
    				description: "",
    				label: "European bank cooperative"
    			},
    			federationOfMunicipalitiesEstablishment: {
    				description: "",
    				label: "Federal enterprise"
    			},
    			financialAssociation: {
    				description: "",
    				label: "Financial association"
    			},
    			forestCareAssociation: {
    				description: "",
    				label: "Forestry association"
    			},
    			foundation: {
    				description: "",
    				label: "Foundation"
    			},
    			housingCooperative: {
    				description: "",
    				label: "Housing cooperative"
    			},
    			hypoAssociation: {
    				description: "",
    				label: "Mortgage society"
    			},
    			insuranceAssociation: {
    				description: "",
    				label: "Insurance association"
    			},
    			ky: {
    				description: "",
    				label: "Limited partnership"
    			},
    			municipalEstablishment: {
    				description: "",
    				label: "Municipial enterprise"
    			},
    			mutualInsuranceAssociation: {
    				description: "",
    				label: "Mutual insurance association"
    			},
    			noCompanyType: {
    				description: "",
    				label: "No company form"
    			},
    			openCompany: {
    				description: "",
    				label: "Open company"
    			},
    			osuuskunta: {
    				description: "",
    				label: "Cooperative"
    			},
    			otherAssociation: {
    				description: "",
    				label: "Other association"
    			},
    			otherFinancialAssociation: {
    				description: "",
    				label: "Other financial association"
    			},
    			oy: {
    				description: "",
    				label: "Limited company"
    			},
    			privateEntrepreneur: {
    				description: "",
    				label: "Private trader, trade name"
    			},
    			reindeerHerdingCooperative: {
    				description: "",
    				label: "Reindeer grazing association"
    			},
    			savingsBank: {
    				description: "",
    				label: "Savings bank"
    			},
    			specialPurposeAssociation: {
    				description: "",
    				label: "Association based on special legislation"
    			},
    			stateEstablishment: {
    				description: "",
    				label: "State-owned enterprise"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown company form"
    			},
    			mutualRealEstateCompany: {
    				description: "",
    				label: "MutualRealEstateCompany"
    			},
    			taxableGrouping: {
    				description: "",
    				label: "TaxableGrouping"
    			}
    		},
    		AccountantType: {
    			primaryPartner: {
    				description: "",
    				label: "PrimaryPartner"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			unlinkedAccountingOnly: {
    				description: "",
    				label: "UnlinkedAccountingOnly"
    			},
    			unlinkedPrimaryPartner: {
    				description: "",
    				label: "UnlinkedPrimaryPartner"
    			}
    		},
    		AccountingPeriodClosingOption: {
    			"default": {
    				description: "",
    				label: "Default"
    			}
    		},
    		AccountingReportRowType: {
    			childRow: {
    				description: "",
    				label: "ChildRow"
    			},
    			credit: {
    				description: "",
    				label: "Credit"
    			},
    			debit: {
    				description: "",
    				label: "Debit"
    			},
    			groupHeader: {
    				description: "",
    				label: "GroupHeader"
    			},
    			groupTotal: {
    				description: "",
    				label: "GroupTotal"
    			},
    			total: {
    				description: "",
    				label: "Total"
    			}
    		},
    		AccountingReportTableType: {
    			classic: {
    				description: "",
    				label: "Classic"
    			},
    			mapped: {
    				description: "",
    				label: "Mapped"
    			},
    			simple: {
    				description: "",
    				label: "Simple"
    			}
    		},
    		ApiListItemType: {
    			accountBase: {
    				description: "",
    				label: "AccountBase"
    			},
    			accountProducts: {
    				description: "",
    				label: "AccountProducts"
    			},
    			article: {
    				description: "",
    				label: "Article"
    			},
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			calculationPaid: {
    				description: "",
    				label: "CalculationPaid"
    			},
    			certificate: {
    				description: "",
    				label: "Certificate"
    			},
    			companyAccount: {
    				description: "",
    				label: "CompanyAccount"
    			},
    			eSalaryPayment: {
    				description: "",
    				label: "ESalaryPayment"
    			},
    			earningsPayment: {
    				description: "",
    				label: "EarningsPayment"
    			},
    			emailMessage: {
    				description: "",
    				label: "EmailMessage"
    			},
    			employment: {
    				description: "",
    				label: "Employment"
    			},
    			historical: {
    				description: "",
    				label: "Historical"
    			},
    			holidayYear: {
    				description: "",
    				label: "HolidayYear"
    			},
    			ifInsuranceOrder: {
    				description: "",
    				label: "IfInsuranceOrder"
    			},
    			invoice: {
    				description: "",
    				label: "Invoice"
    			},
    			messageThread: {
    				description: "",
    				label: "MessageThread"
    			},
    			onboarding: {
    				description: "",
    				label: "Onboarding"
    			},
    			payment: {
    				description: "",
    				label: "Payment"
    			},
    			payrollDetails: {
    				description: "",
    				label: "PayrollDetails"
    			},
    			personAccount: {
    				description: "",
    				label: "PersonAccount"
    			},
    			profile: {
    				description: "",
    				label: "Profile"
    			},
    			report: {
    				description: "",
    				label: "Report"
    			},
    			sessionUserCredential: {
    				description: "",
    				label: "SessionUserCredential"
    			},
    			taxcard: {
    				description: "",
    				label: "Taxcard"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			varmaPensionOrder: {
    				description: "",
    				label: "VarmaPensionOrder"
    			},
    			workerAbsences: {
    				description: "",
    				label: "WorkerAbsences"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			},
    			workerInvitation: {
    				description: "",
    				label: "WorkerInvitation"
    			}
    		},
    		BankPaymentType: {
    			bankFees: {
    				description: "",
    				label: "BankFees"
    			},
    			error: {
    				description: "",
    				label: "Error"
    			},
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			insuranceGrossPayment: {
    				description: "",
    				label: "InsuranceGrossPayment"
    			},
    			insuranceLahiTapiola: {
    				description: "",
    				label: "InsuranceLahiTapiola"
    			},
    			netSalary: {
    				description: "",
    				label: "NetSalary"
    			},
    			netvisorPayment: {
    				description: "",
    				label: "NetvisorPayment"
    			},
    			palkkausFees: {
    				description: "",
    				label: "PalkkausFees"
    			},
    			partialRefundPayment: {
    				description: "",
    				label: "PartialRefundPayment"
    			},
    			paytrailBatch: {
    				description: "",
    				label: "PaytrailBatch"
    			},
    			paytrailFees: {
    				description: "",
    				label: "PaytrailFees"
    			},
    			refundPayment: {
    				description: "",
    				label: "RefundPayment"
    			},
    			salaryAdvance: {
    				description: "",
    				label: "SalaryAdvance"
    			},
    			salaryGrossPayment: {
    				description: "",
    				label: "SalaryGrossPayment"
    			},
    			saldoYearBegin: {
    				description: "",
    				label: "SaldoYearBegin"
    			},
    			saldoYearEnd: {
    				description: "",
    				label: "SaldoYearEnd"
    			},
    			taxAccount: {
    				description: "",
    				label: "TaxAccount"
    			},
    			tvr: {
    				description: "",
    				label: "Tvr"
    			},
    			tvrRfnd: {
    				description: "",
    				label: "TvrRfnd"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			tyelEloContract: {
    				description: "",
    				label: "TyelEloContract"
    			},
    			tyelEloInitialImport: {
    				description: "",
    				label: "TyelEloInitialImport"
    			},
    			tyelEloTemp: {
    				description: "",
    				label: "TyelEloTemp"
    			},
    			tyelEteraContract: {
    				description: "",
    				label: "TyelEteraContract"
    			},
    			tyelEteraTemp: {
    				description: "",
    				label: "TyelEteraTemp"
    			},
    			tyelEteraUnknown: {
    				description: "",
    				label: "TyelEteraUnknown"
    			},
    			tyelIlmarinenContract: {
    				description: "",
    				label: "TyelIlmarinenContract"
    			},
    			tyelIlmarinenTemp: {
    				description: "",
    				label: "TyelIlmarinenTemp"
    			},
    			tyelVarmaContract: {
    				description: "",
    				label: "TyelVarmaContract"
    			},
    			tyelVarmaTemp: {
    				description: "",
    				label: "TyelVarmaTemp"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			},
    			unionRaksa: {
    				description: "",
    				label: "UnionRaksa"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				description: "",
    				label: "AccommodationBenefit"
    			},
    			mealBenefit: {
    				description: "",
    				label: "MealBenefit"
    			},
    			otherBenefits: {
    				description: "",
    				label: "OtherBenefits"
    			},
    			telephoneBenefit: {
    				description: "",
    				label: "TelephoneBenefit"
    			}
    		},
    		CalcGroup: {
    			baseSalary: {
    				description: "",
    				label: "BaseSalary"
    			},
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			disabled: {
    				description: "",
    				label: "Disabled"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			otherNoPayment: {
    				description: "",
    				label: "OtherNoPayment"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			totals: {
    				description: "",
    				label: "Totals"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationResultRowType: {
    			totalBenefits: {
    				description: "",
    				label: "TotalBenefits"
    			},
    			totalExpenses: {
    				description: "",
    				label: "TotalExpenses"
    			},
    			totalPalkkaus: {
    				description: "",
    				label: "TotalPalkkaus"
    			},
    			totalPayment: {
    				description: "",
    				label: "TotalPayment"
    			},
    			totalPension: {
    				description: "",
    				label: "TotalPension"
    			},
    			totalPensionEmployer: {
    				description: "",
    				label: "TotalPensionEmployer"
    			},
    			totalPensionEmployerDebt: {
    				description: "",
    				label: "TotalPensionEmployerDebt"
    			},
    			totalPensionPayment: {
    				description: "",
    				label: "TotalPensionPayment"
    			},
    			totalPensionWorker: {
    				description: "",
    				label: "TotalPensionWorker"
    			},
    			totalSalary: {
    				description: "",
    				label: "TotalSalary"
    			},
    			totalSocialSecurityEmployer: {
    				description: "",
    				label: "TotalSocialSecurityEmployer"
    			},
    			totalSocialSecurityEmployerDebt: {
    				description: "",
    				label: "TotalSocialSecurityEmployerDebt"
    			},
    			totalSocialSecurityEmployerPayment: {
    				description: "",
    				label: "TotalSocialSecurityEmployerPayment"
    			},
    			totalTax: {
    				description: "",
    				label: "TotalTax"
    			},
    			totalTaxPayment: {
    				description: "",
    				label: "TotalTaxPayment"
    			},
    			totalUnemployment: {
    				description: "",
    				label: "TotalUnemployment"
    			},
    			totalUnemploymentEmployer: {
    				description: "",
    				label: "TotalUnemploymentEmployer"
    			},
    			totalUnemploymentEmployerDebt: {
    				description: "",
    				label: "TotalUnemploymentEmployerDebt"
    			},
    			totalUnemploymentPayment: {
    				description: "",
    				label: "TotalUnemploymentPayment"
    			},
    			totalUnemploymentWorker: {
    				description: "",
    				label: "TotalUnemploymentWorker"
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: "TotalWorkerPayment"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationRowCategory: {
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			holidays: {
    				description: "",
    				label: "Holidays"
    			},
    			other: {
    				description: "",
    				label: "Other"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			salaryCalc: {
    				description: "",
    				label: "SalaryCalc"
    			}
    		},
    		CalculationStatusCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			editable: {
    				description: "",
    				label: "Editable"
    			},
    			readonly: {
    				description: "",
    				label: "Readonly"
    			},
    			received: {
    				description: "",
    				label: "Received"
    			},
    			sent: {
    				description: "",
    				label: "Sent"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			}
    		},
    		ESalaryPaymentStatus: {
    			calculationCreated: {
    				description: "",
    				label: "CalculationCreated"
    			},
    			created: {
    				description: "",
    				label: "Created"
    			},
    			employerResolved: {
    				description: "",
    				label: "EmployerResolved"
    			},
    			employmentResolved: {
    				description: "",
    				label: "EmploymentResolved"
    			},
    			opened: {
    				description: "",
    				label: "Opened"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "PaymentCanceled"
    			},
    			paymentMethodResolved: {
    				description: "",
    				label: "PaymentMethodResolved"
    			},
    			paymentStarted: {
    				description: "",
    				label: "PaymentStarted"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "PaymentSucceeded"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			},
    			taxcardResolved: {
    				description: "",
    				label: "TaxcardResolved"
    			}
    		},
    		EmploymentRelationStatus: {
    			archived: {
    				description: "",
    				label: "Archived"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			secondaryCurrent: {
    				description: "",
    				label: "SecondaryCurrent"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		Holiday: {
    			allSaintsDay: {
    				description: "",
    				label: "AllSaintsDay"
    			},
    			ascensionDay: {
    				description: "",
    				label: "AscensionDay"
    			},
    			christmasDay: {
    				description: "",
    				label: "ChristmasDay"
    			},
    			christmasEve: {
    				description: "",
    				label: "ChristmasEve"
    			},
    			easterMonday: {
    				description: "",
    				label: "EasterMonday"
    			},
    			easterSaturday: {
    				description: "",
    				label: "EasterSaturday"
    			},
    			easterSunday: {
    				description: "",
    				label: "EasterSunday"
    			},
    			epiphany: {
    				description: "",
    				label: "Epiphany"
    			},
    			goodFriday: {
    				description: "",
    				label: "GoodFriday"
    			},
    			independenceDay: {
    				description: "",
    				label: "IndependenceDay"
    			},
    			mayDay: {
    				description: "",
    				label: "MayDay"
    			},
    			midsummerDay: {
    				description: "",
    				label: "MidsummerDay"
    			},
    			midsummerEve: {
    				description: "",
    				label: "MidsummerEve"
    			},
    			newYearsDay: {
    				description: "",
    				label: "NewYearsDay"
    			},
    			pentecost: {
    				description: "",
    				label: "Pentecost"
    			},
    			stStephensDay: {
    				description: "",
    				label: "StStephensDay"
    			}
    		},
    		HolidayAccrualSource: {
    			calcDraft: {
    				description: "",
    				label: "CalcDraft"
    			},
    			calcPaid: {
    				description: "",
    				label: "CalcPaid"
    			},
    			initial: {
    				description: "",
    				label: "Initial"
    			},
    			manual: {
    				description: "",
    				label: "Manual"
    			}
    		},
    		HolidayGroup: {
    			holiday: {
    				description: "",
    				label: "Holiday"
    			},
    			nonBanking: {
    				description: "",
    				label: "NonBanking"
    			},
    			official: {
    				description: "",
    				label: "Official"
    			}
    		},
    		IncomeEarnerType: {
    			athlete: {
    				description: "",
    				label: "Athlete"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "EmployedByStateEmploymentFund"
    			},
    			entrepreneurOrFarmerNoPensionRequired: {
    				description: "",
    				label: "EntrepreneurOrFarmerNoPensionRequired"
    			},
    			jointOwnerWithPayer: {
    				description: "",
    				label: "JointOwnerWithPayer"
    			},
    			keyEmployee: {
    				description: "",
    				label: "KeyEmployee"
    			},
    			leasedEmployeeLivingAbroad: {
    				description: "",
    				label: "LeasedEmployeeLivingAbroad"
    			},
    			netOfTaxContract: {
    				description: "",
    				label: "NetOfTaxContract"
    			},
    			organization: {
    				description: "",
    				label: "Organization"
    			},
    			partialOwner: {
    				description: "",
    				label: "PartialOwner"
    			},
    			performingArtist: {
    				description: "",
    				label: "PerformingArtist"
    			},
    			personWorkingAbroad: {
    				description: "",
    				label: "PersonWorkingAbroad"
    			},
    			personWorkingInFrontierDistrict: {
    				description: "",
    				label: "PersonWorkingInFrontierDistrict"
    			},
    			personWorkingOnAlandFerry: {
    				description: "",
    				label: "PersonWorkingOnAlandFerry"
    			},
    			restrictedPeriodInFinland: {
    				description: "",
    				label: "RestrictedPeriodInFinland"
    			}
    		},
    		IncomeLogDiff: {
    			changed: {
    				description: "",
    				label: "Changed"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			"new": {
    				description: "",
    				label: "New"
    			},
    			removed: {
    				description: "",
    				label: "Removed"
    			}
    		},
    		InvoicePaymentType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			}
    		},
    		InvoiceType: {
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			gross: {
    				description: "",
    				label: "Gross"
    			},
    			net: {
    				description: "",
    				label: "Net"
    			},
    			pension: {
    				description: "",
    				label: "Pension"
    			},
    			tax: {
    				description: "",
    				label: "Tax"
    			},
    			taxSocialSecurity: {
    				description: "",
    				label: "TaxSocialSecurity"
    			},
    			taxWithholding: {
    				description: "",
    				label: "TaxWithholding"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			unemployment: {
    				description: "",
    				label: "Unemployment"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			}
    		},
    		IrRowSourceType: {
    			manual: {
    				description: "",
    				label: "Manual"
    			},
    			usecase: {
    				description: "",
    				label: "Usecase"
    			},
    			usecaseV02: {
    				description: "",
    				label: "UsecaseV02"
    			}
    		},
    		LegacyTaxcardType: {
    			example: {
    				description: "",
    				label: "Example"
    			},
    			freelancer: {
    				description: "",
    				label: "Freelancer"
    			},
    			noTaxCard: {
    				description: "",
    				label: "NoTaxCard"
    			},
    			noTaxPrepayment: {
    				description: "",
    				label: "NoTaxPrepayment"
    			},
    			others: {
    				description: "",
    				label: "Others"
    			},
    			sideIncome: {
    				description: "",
    				label: "SideIncome"
    			},
    			steps: {
    				description: "",
    				label: "Steps"
    			},
    			taxCardA: {
    				description: "",
    				label: "TaxCardA"
    			},
    			taxCardB: {
    				description: "",
    				label: "TaxCardB"
    			}
    		},
    		MessageFrom: {
    			otherParty: {
    				description: "",
    				label: "OtherParty"
    			},
    			owner: {
    				description: "",
    				label: "Owner"
    			},
    			system: {
    				description: "",
    				label: "System"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthError: {
    			"<any>access_denied": {
    				description: "",
    				label: "Access_denied"
    			},
    			"<any>invalid_request": {
    				description: "",
    				label: "Invalid_request"
    			},
    			"<any>invalid_scope": {
    				description: "",
    				label: "Invalid_scope"
    			},
    			"<any>server_error": {
    				description: "",
    				label: "Server_error"
    			},
    			"<any>temporarily_unavailable": {
    				description: "",
    				label: "Temporarily_unavailable"
    			},
    			"<any>unauthorized_client": {
    				description: "",
    				label: "Unauthorized_client"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			"<any>unsupported_response_type": {
    				description: "",
    				label: "Unsupported_response_type"
    			}
    		},
    		OAuthGrantType: {
    			"<any>authorization_code": {
    				description: "",
    				label: "Authorization_code"
    			},
    			"<any>client_credentials": {
    				description: "",
    				label: "Client_credentials"
    			},
    			"<any>jwtBearer": {
    				description: "",
    				label: "JwtBearer"
    			},
    			"<any>password": {
    				description: "",
    				label: "Password"
    			},
    			"<any>refresh_token": {
    				description: "",
    				label: "Refresh_token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthResponseType: {
    			"<any>code": {
    				description: "",
    				label: "Code"
    			},
    			"<any>token": {
    				description: "",
    				label: "Token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthSalaxyAuthorizeMode: {
    			"<any>sign_in": {
    				description: "",
    				label: "Sign_in"
    			},
    			"<any>sign_up": {
    				description: "",
    				label: "Sign_up"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthTokenType: {
    			"<any>bearer": {
    				description: "",
    				label: "Bearer"
    			},
    			"<any>mac": {
    				description: "",
    				label: "Mac"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentMethod: {
    			sepa: {
    				description: "",
    				label: "Sepa"
    			},
    			siirto: {
    				description: "",
    				label: "Siirto"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PeriodDateKind: {
    			paidAtDate: {
    				description: "",
    				label: "PaidAtDate"
    			},
    			salaryDate: {
    				description: "",
    				label: "SalaryDate"
    			}
    		},
    		Role: {
    			accountant: {
    				description: "",
    				label: "Accountant"
    			},
    			accountantCandidate: {
    				description: "",
    				label: "AccountantCandidate"
    			},
    			pricingPartner: {
    				description: "",
    				label: "PricingPartner"
    			},
    			trustedPartner: {
    				description: "",
    				label: "TrustedPartner"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SalaryType: {
    			compensation: {
    				description: "",
    				label: "Compensation"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			yel: {
    				description: "",
    				label: "Yel"
    			}
    		},
    		SettingsStatus: {
    			checked: {
    				description: "",
    				label: "Checked"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SharingUriType: {
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			url: {
    				description: "",
    				label: "Url"
    			}
    		},
    		SystemRole: {
    			anon: {
    				description: "",
    				label: "Anon"
    			},
    			any: {
    				description: "",
    				label: "Any"
    			},
    			auth: {
    				description: "",
    				label: "Auth"
    			},
    			company: {
    				description: "",
    				label: "Company"
    			},
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			fiOy: {
    				description: "",
    				label: "FiOy"
    			},
    			fiRy: {
    				description: "",
    				label: "FiRy"
    			},
    			fiTm: {
    				description: "",
    				label: "FiTm"
    			},
    			hidden: {
    				description: "",
    				label: "Hidden"
    			},
    			household: {
    				description: "",
    				label: "Household"
    			},
    			init: {
    				description: "",
    				label: "Init"
    			},
    			nonVerified: {
    				description: "",
    				label: "NonVerified"
    			},
    			partnerMessaging: {
    				description: "",
    				label: "PartnerMessaging"
    			},
    			partnerService: {
    				description: "",
    				label: "PartnerService"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			prod: {
    				description: "",
    				label: "Prod"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			worker: {
    				description: "",
    				label: "Worker"
    			}
    		},
    		TableFormat: {
    			csv: {
    				description: "",
    				label: "Csv"
    			},
    			txt: {
    				description: "",
    				label: "Txt"
    			},
    			xlsx: {
    				description: "",
    				label: "Xlsx"
    			}
    		},
    		TaxReportHandling: {
    			always: {
    				description: "",
    				label: "Always"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			noMonthlyReport: {
    				description: "",
    				label: "NoMonthlyReport"
    			},
    			noZeroSalaryReport: {
    				description: "",
    				label: "NoZeroSalaryReport"
    			}
    		},
    		TaxcardApprovalMethod: {
    			approve: {
    				description: "",
    				label: "Approve"
    			},
    			assureWaiting: {
    				description: "",
    				label: "AssureWaiting"
    			},
    			reject: {
    				description: "",
    				label: "Reject"
    			}
    		},
    		TaxcardCalcDiffCheck: {
    			commitDiff: {
    				description: "",
    				label: "CommitDiff"
    			},
    			getDiff: {
    				description: "",
    				label: "GetDiff"
    			},
    			noCheck: {
    				description: "",
    				label: "NoCheck"
    			}
    		},
    		ThreadedMessageType: {
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			email: {
    				description: "",
    				label: "Email"
    			},
    			payroll: {
    				description: "",
    				label: "Payroll"
    			},
    			sms: {
    				description: "",
    				label: "Sms"
    			},
    			text: {
    				description: "",
    				label: "Text"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			}
    		},
    		Unit: {
    			count: {
    				description: "",
    				label: "Count"
    			},
    			days: {
    				description: "",
    				label: "Days"
    			},
    			euro: {
    				description: "",
    				label: "Euro"
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			},
    			kilometers: {
    				description: "",
    				label: "Kilometers"
    			},
    			one: {
    				description: "",
    				label: "One"
    			},
    			percent: {
    				description: "",
    				label: "Percent"
    			},
    			period: {
    				description: "",
    				label: "Period"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			weeks: {
    				description: "",
    				label: "Weeks"
    			}
    		},
    		VarmaPensionOrderAction: {
    			move: {
    				description: "",
    				label: "Move"
    			},
    			newTyel: {
    				description: "",
    				label: "NewTyel"
    			},
    			newYel: {
    				description: "",
    				label: "NewYel"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		VarmaPensionOrderYelPayer: {
    			company: {
    				description: "",
    				label: "Company"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		WorkflowEventFeatures: {
    			isActive: {
    				description: "",
    				label: "IsActive"
    			},
    			isEditable: {
    				description: "",
    				label: "IsEditable"
    			},
    			isOnHold: {
    				description: "",
    				label: "IsOnHold"
    			},
    			isRemovable: {
    				description: "",
    				label: "IsRemovable"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		WorkflowEventUi: {
    			danger: {
    				description: "",
    				label: "Danger"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			info: {
    				description: "",
    				label: "Info"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			success: {
    				description: "",
    				label: "Success"
    			},
    			warning: {
    				description: "",
    				label: "Warning"
    			}
    		},
    		YearEndAdminCheck: {
    			errors: {
    				description: "",
    				label: "Errors"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithWarning: {
    				description: "",
    				label: "OkWithWarning"
    			}
    		},
    		YearEndFeedbackCalculationType: {
    			correction: {
    				description: "",
    				label: "Correction"
    			},
    			external: {
    				description: "",
    				label: "External"
    			},
    			salaxyPayment: {
    				description: "",
    				label: "SalaxyPayment"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		YearEndUserFeedback: {
    			noYearlyReports: {
    				description: "",
    				label: "NoYearlyReports"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithModifications: {
    				description: "",
    				label: "OkWithModifications"
    			},
    			willHandleMyself: {
    				description: "",
    				label: "WillHandleMyself"
    			}
    		}
    	}
    };

    var SALAXY$9 = {
    	MODEL: {
    	}
    };

    var SALAXY$8 = {
    	MODEL: {
    		AbsencePeriod: {
    			title: "absence period",
    			descr: "Period of absence",
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			}
    		},
    		AccountBase: {
    			title: "account base",
    			descr: "Account - either Person or Company - is a juridical entity Paying or receiving salaries, making contracts etc. It may be related to zero, one or multiple UserAccounts/Credentials.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AccountInIndex: {
    			title: "account in index",
    			descr: "Object that represents a Salaxy account.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			}
    		},
    		AccountProducts: {
    			title: "account products",
    			descr: "Provides an overview and the possibility to edit products that are enabled to this account",
    			accounting: {
    				title: "accounting",
    				descr: "Bookkeeping reports for salarypayments"
    			},
    			baseService: {
    				title: "base service",
    				descr: "The basic services for Salary payment."
    			},
    			healthCareHeltti: {
    				title: "health care heltti",
    				descr: "Employer health care from the company Heltti"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Reports and payments for mandatory employment insurances."
    			},
    			pension: {
    				title: "pension",
    				descr: "Reports ant payments to Pension companies."
    			},
    			pensionEntrepreneur: {
    				title: "pension entrepreneur",
    				descr: "Pension payments for an entrepreneur"
    			},
    			pricing: {
    				title: "pricing",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			tax: {
    				title: "tax",
    				descr: "Reports and payments to tax authorities."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "Reports and payments to TVR (the unemployment insurance fund of Finland)"
    			}
    		},
    		AccountValidation: {
    			title: "account validation",
    			descr: "Validation errors for the onboarding process",
    			canStartSign: {
    				title: "can start sign",
    				descr: "TODO: NOt implemented yet - should be implemented\nIf true, the onboarding object has sufficient data to start the Digital Signature process (Visma Sign).\nDepending on the use case this may or may not be required for committing the data as account data."
    			},
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			existingAccount: {
    				title: "existing account",
    				descr: "Existing account based on official id (Personal ID / Business ID)."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, none of the accounts in ExistingAccounts cannot be accessed by the current user."
    			},
    			isCompleted: {
    				title: "is completed",
    				descr: "If true, the validation process has been completed. The IsValid may be true or false."
    			},
    			isOfficialIdValid: {
    				title: "is official id valid",
    				descr: "If true the official ID is technically valid.\nOfficial ID is either Finnish Personal ID (HETU, Henkilötunnus) or Business ID (Y-tunnus)."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		Accountant: {
    			title: "accountant",
    			descr: "The person or account reponsible for accounting",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			type: {
    				title: "type",
    				descr: "If true, the accountant is a registered Palkkaus.fi account who typically has access to this account."
    			}
    		},
    		AccountingData: {
    			title: "accounting data",
    			descr: "Accounting data for reports.",
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			ledgerAccounts: {
    				title: "ledger accounts",
    				descr: "All ledgers."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			}
    		},
    		AccountingDataSummary: {
    			title: "accounting data summary",
    			descr: "Summary of the accounting data. Contains main figures: totals and workers and employer total figures.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary of the calculations included in the report."
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment of the calculations included in the report."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "The total number of distinct workers included in the report."
    			}
    		},
    		AccountingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			accountantEmail: {
    				title: "accountant email"
    			},
    			accountantName: {
    				title: "accountant name"
    			},
    			accountantPhone: {
    				title: "accountant phone"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by AccountantName, AccountantEmail and AccountantPhone."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		AccountingReportRow: {
    			title: "accounting report row",
    			descr: "Accounting report table row.",
    			accountNumber: {
    				title: "account number",
    				descr: "Proposed account number for the row."
    			},
    			accountText: {
    				title: "account text",
    				descr: "Proposed account text for the row."
    			},
    			additionalInformation: {
    				title: "additional information",
    				descr: "Additional information for the row."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount for the row: May be positive or negative(depending on the depet/credit and account type)."
    			},
    			count: {
    				title: "count",
    				descr: "Count of units. Default is 1."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the unit."
    			},
    			productCode: {
    				title: "product code",
    				descr: "Returns product code."
    			},
    			productName: {
    				title: "product name",
    				descr: "Returns product name. (Code or ResultCode in this order)."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Row type. Default is the Booking."
    			},
    			styles: {
    				title: "styles",
    				descr: "Space separated set of class names for CSS styling purposes."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the booking entry / header."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit identifier;"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percentage of the value added tax that is included in the amount."
    			}
    		},
    		AccountingReportTable: {
    			title: "accounting report table",
    			descr: "Contains accounting rows for one or many calculations.",
    			businessId: {
    				title: "business id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			rows: {
    				title: "rows",
    				descr: "All rows in the report table."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			},
    			tableType: {
    				title: "table type",
    				descr: "Accounting report table type."
    			}
    		},
    		AccountingReportTableSummary: {
    			title: "accounting report table summary",
    			descr: "Summary of the accounting report. Contains main figures: totals and workers and employer total figures.",
    			count: {
    				title: "count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		AccountingRuleSet: {
    			title: "accounting rule set",
    			descr: "Ruleset containing required tabular logic and accounts for creating accounting entries.",
    			accounts: {
    				title: "accounts",
    				descr: "Accounts which are used in the tabular logic for creating accounting entries."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the ruleset."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the rule set is read only."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the ruleset."
    			},
    			rows: {
    				title: "rows",
    				descr: "Tabular logic for creating accounting entries."
    			},
    			templateId: {
    				title: "template id",
    				descr: "Template to which this is based on."
    			}
    		},
    		AccountingRuleSetAccount: {
    			title: "accounting rule set account",
    			descr: "Single account with number, grouping and default text.",
    			account: {
    				title: "account",
    				descr: "Account number."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the account."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the account is non-removable template."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the account."
    			}
    		},
    		AccountingRuleSetRow: {
    			title: "accounting rule set row",
    			descr: "Contains a single row of the tabular logic for creating accounting entries.",
    			creditGroupingId: {
    				title: "credit grouping id",
    				descr: "Additional flags for directing the logic for creating accounting entry based on given entries."
    			},
    			creditGroupingText: {
    				title: "credit grouping text"
    			},
    			creditId: {
    				title: "credit id"
    			},
    			debitGroupingId: {
    				title: "debit grouping id",
    				descr: "Id of the credit account."
    			},
    			debitGroupingText: {
    				title: "debit grouping text"
    			},
    			debitId: {
    				title: "debit id",
    				descr: "Id of the debit account."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if the row is in use."
    			},
    			entryCode: {
    				title: "entry code",
    				descr: "A single code in the code group: single income type code or total row type."
    			},
    			entryCodeGroup: {
    				title: "entry code group",
    				descr: "Entry group: income type or total type."
    			},
    			flags: {
    				title: "flags"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the row."
    			},
    			isIncluded: {
    				title: "is included",
    				descr: "Boolean indicating whether the system should make an accounting entry based on this row."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the row is non-removable template."
    			}
    		},
    		AccountingSettings: {
    			title: "accounting settings",
    			descr: "Settings related to accounting reports.",
    			accountant: {
    				title: "accountant",
    				descr: "The person or company responsible for accounting."
    			},
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			ruleSets: {
    				title: "rule sets",
    				descr: "Rulesets containing required tabular logic and accounts for creating accounting entries."
    			},
    			software: {
    				title: "software",
    				descr: "Integrated accounting software."
    			},
    			targets: {
    				title: "targets",
    				descr: "Accounting target systems and the rulesets assigned to them."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		AccountingSoftwareSettings: {
    			title: "accounting software settings",
    			descr: "Setting related to accounting software"
    		},
    		AccountingTarget: {
    			title: "accounting target",
    			descr: "Accounting target systems and the rulesets assigned to them.",
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if this target is in use"
    			},
    			id: {
    				title: "id",
    				descr: "Id of the target definition."
    			},
    			image: {
    				title: "image",
    				descr: "Illustrative image for the target system."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the target system."
    			},
    			ruleSetId: {
    				title: "rule set id",
    				descr: "Assigned ruleset for this target system."
    			}
    		},
    		Ajax: {
    			title: "ajax",
    			descr: "Abstracts the interface for Http / Ajax functionality that is required by Salaxy core classes.\nThis interface is implemented in this project for jQuery and Node.\nAdditional implementations exists for Angular.js (salaxy-lib-ng1) and Angular.io / ngx (salaxy-lib-ng).\nYou may also create your own implementation for the framework or http component that you want to support.",
    			serverAddress: {
    				title: "server address",
    				descr: "The server address - root of the server. This is settable field.\nWill probably be changed to a configuration object in the final version."
    			},
    			useCredentials: {
    				title: "use credentials",
    				descr: "By default credentials are not used in http-calls.\nEnable credentials with this flag (force disabled when the token is set)."
    			},
    			getApiAddress: {
    				title: "get api address",
    				descr: "Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api'"
    			},
    			getCurrentToken: {
    				title: "get current token",
    				descr: "Gets the current Salaxy Bearer token"
    			},
    			getHTML: {
    				title: "get h t m l",
    				descr: "Gets a HTML-message from server using the API\n\n**Returns**: A Promise with result html. Standard Promise rejection to be used for error handling.\n"
    			},
    			getJSON: {
    				title: "get j s o n",
    				descr: "Gets a JSON-message from server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			getServerAddress: {
    				title: "get server address",
    				descr: "Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com'"
    			},
    			postHTML: {
    				title: "post h t m l",
    				descr: "POSTS data to server and receives back HTML.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			postJSON: {
    				title: "post j s o n",
    				descr: "POSTS data to server and receives back a JSON-message.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			remove: {
    				title: "remove",
    				descr: "Sends a DELETE-message to server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			setCurrentToken: {
    				title: "set current token",
    				descr: "Sets Salaxy Bearer token for this http context."
    			}
    		},
    		AnnualLeave: {
    			title: "annual leave",
    			descr: "A planned annual Leave - not sure whether it will kept or is it just planned",
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeavePayment: {
    			title: "annual leave payment",
    			descr: "Payment related to annual leave: holiday salary, bonus and compensation",
    			calcId: {
    				title: "calc id",
    				descr: "If the payment is defined by a calculation, stores the calculation ID."
    			},
    			date: {
    				title: "date",
    				descr: "The reference date: Typically the Paid at date.\nThe items are typically sorted based on this."
    			},
    			holidayBonus: {
    				title: "holiday bonus",
    				descr: "Bonus for either Holiday Salary or Compensation."
    			},
    			holidayCompensation: {
    				title: "holiday compensation",
    				descr: "Compensation that is paid for holidays not kept."
    			},
    			holidayDays: {
    				title: "holiday days",
    				descr: "Days of holidays that are paid or compensated"
    			},
    			holidaySalary: {
    				title: "holiday salary",
    				descr: "Salary that is paid for holidays"
    			},
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the payment entry"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeaves: {
    			title: "annual leaves",
    			descr: "Data about the annual leaves planned and used during the holiday year.",
    			paid: {
    				title: "paid",
    				descr: "Paid or otherwise handled annual leaves.\nPRELIMINARY: This property is under revision - it may be moved or renamed without being a breaking change."
    			},
    			planned: {
    				title: "planned",
    				descr: "Planned annual leaves."
    			}
    		},
    		ApiCrudObject: {
    			title: "api crud object",
    			descr: "Base interface for CRUD objects in the Salaxy API.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Partner service that has be used in modifying the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated. Typically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		ApiListItem: {
    			title: "api list item",
    			descr: "The line item class for search and listing results. Use this class for building user interfaces.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			listType: {
    				title: "list type",
    				descr: "List item type of the item if the item supports a custom OData interface."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			type: {
    				title: "type",
    				descr: "The payload type of the item."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ApiListItemValidation: {
    			title: "api list item validation",
    			descr: "ApiValidation object that is specialized for list items. Provides if and index properties for connecting a validation to a specific item in the list",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the item this validation is for.\nIdentifiers are not always available (if the item has not been saved), so index is a more generic approach."
    			},
    			index: {
    				title: "index",
    				descr: "Zero based index of the item this validation is for."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiProfileSearchResult: {
    			title: "api profile search result",
    			descr: "Profile search result for the API",
    			cities: {
    				title: "cities"
    			},
    			facets: {
    				title: "facets",
    				descr: "Dynamic for search facets if faceting is being used.\nNOTE: Facet names are lower camel case - using the JavaScript naming conventions as in the original JSON."
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			},
    			results: {
    				title: "results",
    				descr: "Actual search result - list of items"
    			},
    			searchText: {
    				title: "search text",
    				descr: "Search text that was input by the user"
    			},
    			searchTextTechnical: {
    				title: "search text technical",
    				descr: "Search text as it is sent to the search provider:\nContains provider specific wildcards etc."
    			},
    			sort: {
    				title: "sort",
    				descr: "Sort string"
    			},
    			totalCount: {
    				title: "total count",
    				descr: "Total count returned by the search service.\nThis may be more than the count of Result list"
    			}
    		},
    		ApiProfileSearchResultProfile: {
    			title: "api profile search result profile",
    			avatar: {
    				title: "avatar"
    			},
    			cities: {
    				title: "cities"
    			},
    			id: {
    				title: "id"
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			}
    		},
    		ApiValidation: {
    			title: "api validation",
    			descr: "Common base class / interface for data validation.",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiValidationError: {
    			title: "api validation error",
    			descr: "Validation errors used in API output.",
    			code: {
    				title: "code",
    				descr: "Some error providers may have a unique technical code for error or validator"
    			},
    			key: {
    				title: "key",
    				descr: "Full path to Name of the property / field (Member name in JSON).\nThis may be null/empty for type General."
    			},
    			msg: {
    				title: "msg",
    				descr: "Validation error message in the requested language."
    			},
    			type: {
    				title: "type",
    				descr: "High level type: Specifies the relation to object and its Member."
    			}
    		},
    		AppStatus: {
    			title: "app status",
    			descr: "Supports the UserSession object by providing additional information about the status\nof fetching the session, data, expiration etc.",
    			clientRoles: {
    				title: "client roles",
    				descr: "Application specific roles that are set on the client (browser code)\nas opposed to normal server-side defined roles."
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "Indicates that we are running in test environment with test data.\nThis information is typically available already before the session is available and may be used in login / register.\nNull indicates that the information is not available. This should typically be considered false for security."
    			},
    			sessionCheckInProgress: {
    				title: "session check in progress",
    				descr: "If true, the session check from server is in progress"
    			},
    			sessionExpired: {
    				title: "session expired",
    				descr: "PLACEHOLDER: Indicates that the session has expired (as opposed to not checked yet).\nThis property has not yet been implemented."
    			}
    		},
    		Article: {
    			title: {
    				title: "title",
    				descr: "Title: Shown in te beginning of the text, in the browser window header, in the lists etc."
    			},
    			descr: "Read-only Article object for the API",
    			author: {
    				title: "author",
    				descr: "Author of the article."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			date: {
    				title: "date",
    				descr: "Publish date"
    			},
    			file: {
    				title: "file",
    				descr: "File path as in CDN"
    			},
    			html: {
    				title: "html",
    				descr: "Article text and/or markup.\nThis is not serialized to XML, but read/written separately to blob storage - typically using MarkDown."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			longSummary: {
    				title: "long summary",
    				descr: "A slightly longer summary, shown in Facebook, Google+\nand LinkedIn likes/shares."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			picture: {
    				title: "picture",
    				descr: "URL to a picture that should be used in social media sharing"
    			},
    			section: {
    				title: "section",
    				descr: "The section where the article should appear"
    			},
    			shortSummary: {
    				title: "short summary",
    				descr: "Shorter summary (max 140 chars), shown as default\nin Tweets."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AssureCompanyAccountRequest: {
    			title: "assure company account request",
    			descr: "Request to assure that a company account exists: Either an account is returned or an account stub (non-verified) will be created.",
    			contactFirstName: {
    				title: "contact first name",
    				descr: "First name of the contact person"
    			},
    			contactLastName: {
    				title: "contact last name",
    				descr: "Last name of the contact person"
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Company number in the the trade register (Y-tunnus).\nFormat is \"1234567-8\" - 7 numbers, one control character, with or without the dash.\nAlso, the international VAT format is allowed: FI12345678"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The account ID at the partner site.\nIf present, this ID is used to restrict the authorization to one account at partner site only."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number for the contact person"
    			}
    		},
    		AttributeDoc: {
    			title: "attribute doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attr: {
    				title: "attr",
    				descr: "HTML attribute name e.g. my-prop for myProp."
    			},
    			binding: {
    				title: "binding",
    				descr: "Angular JS binding type"
    			},
    			controller: {
    				title: "controller",
    				descr: "Controller that implements the property"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the memeber."
    			}
    		},
    		Author: {
    			title: "author",
    			descr: "Author of an article in the CMS",
    			companyDescription: {
    				title: "company description",
    				descr: "Short description text / label for the organization."
    			},
    			companyLogo: {
    				title: "company logo",
    				descr: "Company or organization logo - expect that this may be null."
    			},
    			companyUrl: {
    				title: "company url",
    				descr: "Link to a web site - typically of the organization that the author represents"
    			},
    			description: {
    				title: "description",
    				descr: "Short description text"
    			},
    			guid: {
    				title: "guid",
    				descr: "Unique key for the object"
    			},
    			image: {
    				title: "image",
    				descr: "Public image for the author"
    			},
    			name: {
    				title: "name",
    				descr: "Person name"
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Exact time of last technical save: Not editable by user and will change in imports/exports etc."
    			},
    			type: {
    				title: "type",
    				descr: "Supports the Cosmos DB migration by serializing the type of the current object as a short type string for querying."
    			}
    		},
    		Authority: {
    			title: "authority",
    			descr: "A person or other party that authorized the current person.",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account ID. This is optional and should not be set if the ID is the same as the target account / Person."
    			},
    			authorizedAt: {
    				title: "authorized at",
    				descr: "Date for the authorization."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail of the person. This is currently not used, but is expected to be used in the future.\nSo fill it in as it was required unless you have very compelling reasons."
    			},
    			name: {
    				title: "name",
    				descr: "Name of the authority."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Person ID is the key to signature. This is currently always required."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number of the person. This is currently optional.\nIdea is to send SMS invitations in some scenarios."
    			}
    		},
    		AuthorizationContract: {
    			title: "authorization contract",
    			descr: "Contract / authorization document that is the bases for Salaxy acting in behalf of this account",
    			authorizationType: {
    				title: "authorization type",
    				descr: "Type of the authorization"
    			},
    			authorizedBy: {
    				title: "authorized by",
    				descr: "A person or other party that authorized the current person."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "If true, there is a valid authorization - the account has been validated."
    			},
    			signedAt: {
    				title: "signed at",
    				descr: "Time when the contract was signed"
    			}
    		},
    		Avatar: {
    			title: "avatar",
    			descr: "Defines an avatar for an account, profile etc. - mainly the image that should be shown, names and short description",
    			color: {
    				title: "color",
    				descr: "Color - currently only used by type Icon"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			description: {
    				title: "description",
    				descr: "Short description of the user.\nThis may be overriden by a context specific value by the business logic."
    			},
    			displayName: {
    				title: "display name",
    				descr: "Display name. For a person this is 'FirstName LastName' (auto-created)."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type: person/company"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			initials: {
    				title: "initials",
    				descr: "Initials - currently only used by type Icon"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pictureType: {
    				title: "picture type",
    				descr: "Type of the Avatar picture."
    			},
    			sortableName: {
    				title: "sortable name",
    				descr: "Sortable name for ordered lists etc. For a person this is 'LastName, FirstName' (auto-created).\nDepending on the data storage, this may be all-lowercase for easier sorting and search."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			url: {
    				title: "url",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			}
    		},
    		BaseSalaryProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		BlobFile: {
    			title: "blob file",
    			descr: "The BlobFile stores metadata about miscellaneous BLOB-files that are stored to the system. For example Receipts and Authorizations.",
    			blobUri: {
    				title: "blob uri",
    				descr: "Full unique URI in the storage"
    			},
    			container: {
    				title: "container",
    				descr: "The logical container - first level folder for the file."
    			},
    			filePath: {
    				title: "file path",
    				descr: "Technical file name in storage.  In practical terms, this is the relative path\nexcluding the container part \"/[container]/\".\nI.e. this property is without the starting slash \"/\".\nIt must be unique and is used for Uri.\nFor user files this is constructed \"[Username]/[Type]/[UniqueFileName].[xxx]\""
    			},
    			fileType: {
    				title: "file type",
    				descr: "File type (mime type etc.) based on the ending of the file name.\nNull for folders."
    			},
    			id: {
    				title: "id"
    			},
    			isCollection: {
    				title: "is collection",
    				descr: "If true, this is a collection (folder in file system)"
    			},
    			length: {
    				title: "length",
    				descr: "The length of the file in bytes."
    			},
    			metadata: {
    				title: "metadata"
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "If created, contains uri to a PNG image for preview purposes."
    			},
    			type: {
    				title: "type",
    				descr: "Strongly typed logical type of the file. For user files, this is a sub folder."
    			},
    			updatedAt: {
    				title: "updated at"
    			},
    			userFriendlyName: {
    				title: "user friendly name",
    				descr: "A user friendly name that can be used in listings.\nIn the upload scenarios this is usually the original file name when uploaded.\nIt is not necessarily unique. No path information"
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		BlobFileMetadata: {
    			title: "blob file metadata",
    			descr: "Metadata related to blob files as a strongly typed object.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object related to the report.\nFor example for the salary payments this is the other party (employer or worker)"
    			},
    			data: {
    				title: "data",
    				descr: "Arbitrary metadata."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Identifier of the original calculation etc. business object."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "Whether the document is digitally signed (or manually checked as being signed).\nAt the time of writing, this is used only for AuthorizationPdf documents."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "Logical date for the file. E.g. first of the month for a monthly report."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype is internal typing within a BlobFileType. This may or may not be a sub-folder.\nThere is no enumeration on File level - there may be one under services specific to one BlobFileType."
    			}
    		},
    		BoardUsecase: {
    			title: "board usecase",
    			descr: "Handles the Calculation of CalculationRowType.Board to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		BusinessObject: {
    			title: "business object",
    			descr: "Business object in the Salaxy repository",
    			id: {
    				title: "id",
    				descr: "Id of the business object"
    			},
    			name: {
    				title: "name",
    				descr: "Optional name for the business object."
    			},
    			object: {
    				title: "object",
    				descr: "Optional property for object instance."
    			},
    			owner: {
    				title: "owner",
    				descr: "Partition key of the business object"
    			},
    			type: {
    				title: "type",
    				descr: "Business object type"
    			}
    		},
    		CalcEmployer: {
    			title: "calc employer",
    			descr: "Employer of a calculation",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account identifier for the Employer."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image information and type of employer.\nNOTE: Recalculate-method retains the values for display purposes if AccountId or IsSelf is not set.\nThis is mainly for demo and testing in anonymous mode.\nHowever, in storage methods, the value is reset based on values of AccountId or IsSelf\neven if they are not set: Anything stored in Avatar will be ignored."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			}
    		},
    		CalcHolidayBonus: {
    			title: "calc holiday bonus",
    			descr: "Holiday Bonus data that is stored into a Salary calculation.",
    			alreadyPaid: {
    				title: "already paid",
    				descr: "The sum of holiday bonus that has already been paid this year.\nThe payment is never so big that this would become more than BonusTotalHolidaySalary\n(by this calculation - the limit can be exceeded manually)."
    			},
    			totalHolidaySalary: {
    				title: "total holiday salary",
    				descr: "Total yearly holiday salary for the purposes of holiday bonus."
    			},
    			yearlyLeaveDays: {
    				title: "yearly leave days",
    				descr: "Number of yearly leave days. This is for the purposes of calculating the Bonus,\nso this is yearly salary days, not the complete saldo (including StartSaldo)."
    			}
    		},
    		CalcHourlyData: {
    			title: "calc hourly data",
    			descr: "Holiday calculation based on hourly salary OR performance based salaries. May also contain other data related to working hours.",
    			isHourly: {
    				title: "is hourly",
    				descr: "If true, the hourly / performance based calculation is applied."
    			}
    		},
    		CalcInfo: {
    			title: "calc info",
    			descr: "Informational object that stores data about the salary calculation / payment that is not directly related to calculating the salary.",
    			backofficeNotes: {
    				title: "backoffice notes",
    				descr: "Notes related to the payment and other backoffice operations:\nFor example the justification why the sum is in calculation vs. Paytrail payment."
    			},
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			paymentChannel: {
    				title: "payment channel",
    				descr: "Channel that is used for payment."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "Payment reference number in payment service"
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "The Payroll Id, if this calculation is part of a Payroll list."
    			},
    			pensionPaymentDate: {
    				title: "pension payment date",
    				descr: "Calculated pension insurance payment date."
    			},
    			pensionPaymentRef: {
    				title: "pension payment ref",
    				descr: "Pension insurance reference number."
    			},
    			pensionPaymentSpecifier: {
    				title: "pension payment specifier",
    				descr: "Pension insurance payment specifier."
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "End date of the work"
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Start date of the work"
    			},
    			workerMessage: {
    				title: "worker message",
    				descr: "Message from worker to employer - when worker sends the salary calculation to the employer."
    			}
    		},
    		CalcMonthlyData: {
    			title: "calc monthly data",
    			descr: "Period recalculation parameters that are related to Monthly salary. Could theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested.",
    			absencesDaySalary: {
    				title: "absences day salary",
    				descr: "Price for a single absence day. Typically based on MonthlySalary, but may be modified."
    			},
    			isMonthly: {
    				title: "is monthly",
    				descr: "If true the Monthly (weekly) calculation is being applied to absences and holidays."
    			},
    			monthlySalary: {
    				title: "monthly salary",
    				descr: "Monthly salary in holiday and absences calculation.\nTypically, the same as the actual monthly calculation, but may be modified."
    			}
    		},
    		CalcRowConfig: {
    			title: "calc row config",
    			descr: {
    				title: "descr",
    				descr: "Description for the row type"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount input"
    			},
    			category: {
    				title: "category",
    				descr: "Categorization of the row types."
    			},
    			color: {
    				title: "color",
    				descr: "Color is based on the category"
    			},
    			favorite: {
    				title: "favorite",
    				descr: "Favorite row types and their sort order.\nThis is currently fixed, but may later be based on user profile (e.g. most used types)."
    			},
    			iconText: {
    				title: "icon text",
    				descr: "Icon short text is 1-5 characters suitable for icon.\nHas 5 characters only if very narrow, e.g. \"1\" and comma - typically 2-3 chars."
    			},
    			kind: {
    				title: "kind",
    				descr: "If the row has a kind selection (the main subtype), definition of the subtypes."
    			},
    			label: {
    				title: "label",
    				descr: "Label for the row type"
    			},
    			moreInfo: {
    				title: "more info",
    				descr: "More info HTML"
    			},
    			name: {
    				title: "name",
    				descr: "Name / unique identifier of the row type."
    			},
    			order: {
    				title: "order",
    				descr: "Order number in the lists"
    			},
    			price: {
    				title: "price",
    				descr: "Price input"
    			},
    			total: {
    				title: "total",
    				descr: "The total input or read-only field."
    			}
    		},
    		CalcTax: {
    			title: "calc tax",
    			descr: "Tax card information stored as part of the Calculation object",
    			estimatedTax: {
    				title: "estimated tax",
    				descr: "Calculated widthholding tax. Note that this is an estimation until the calculation is actually paid.\nAlso, if the salary is not paid as cash (e.g. fringe benefits), there may not be enough money to make the full deduction."
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Possibility to set a fixed widthholding tax amount for this calculation regardless of\nthe percentage etc. in the tax card."
    			},
    			hasNewSharedCard: {
    				title: "has new shared card",
    				descr: "A flag that during last recalculation, there was a new shared taxcard that was waiting approval."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "The API returns true if there is a valid taxcard for the PaidAt date for this calculation\nor if FixedTaxAmount is set for th calculation."
    			},
    			snapshot: {
    				title: "snapshot",
    				descr: "Snapshot of the tax card for salary slip etc. purposes.\nUpdated in Recalculate / Save based on TaxCardId"
    			},
    			taxPercentForVisualization: {
    				title: "tax percent for visualization",
    				descr: "Tax percent that is used in visualization. This is mainly used in Unit testing and anonymous calcultors.\nNot used in real paid calculations. This is percent. I.e. for 50% set 50, not 0.5.\nIf PaymentData or AccountId lead to a valid tax card type and percent, or if there is FixedTaxAmount, this value is not used."
    			},
    			taxcardId: {
    				title: "taxcard id",
    				descr: "Identifier for a stored tax card if one is applied to this calculation.\nSystem updates the TaxCardId automatically in Recalculate / Save if it finds a better candidate\nand also updates the Snapshot."
    			}
    		},
    		CalcWorker: {
    			title: "calc worker",
    			descr: "Worker object for the current calculation",
    			accountId: {
    				title: "account id",
    				descr: "Palkkaus.fi Account Id for the worker.\nAlternatively you can set the Payment data with the required infromation"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image and properties of the Worker.\nBased on either the AccountId or PaymentData"
    			},
    			dateOfBirth: {
    				title: "date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment Id for the worker. This is currently resolved using the workers AccountId."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the currently Account\nIf set to true, will set current Account as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			},
    			paymentData: {
    				title: "payment data",
    				descr: "Data that is needed for salary payment.\nThis object may be null if AccountId has been set.\nAlso, if AccountId has been set, these properties will be ignored:\nThe Worker will decide on payment properties herself."
    			},
    			tax: {
    				title: "tax",
    				descr: "New Widthholding tax logic that is based on separately stored tax cards."
    			}
    		},
    		CalcWorkerPaymentData: {
    			title: "calc worker payment data",
    			descr: "The required information for paying the Salary to Worker.",
    			email: {
    				title: "email",
    				descr: "Contact e-mail address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name of the person."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "IBAN number for the Bank account"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name / Surname of the person."
    			},
    			socialSecurityNumber: {
    				title: "social security number",
    				descr: "Gets the Social security number the way the user set it in input.\nHOWEVER: The getter will change the string to upper invariant/trim and if the last character is \"*\", it will be replaced by calculated checksum."
    			},
    			socialSecurityNumberValid: {
    				title: "social security number valid",
    				descr: "Social security number if valid or null"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		CalcWorkflow: {
    			title: "calc workflow",
    			descr: "Defines the properties of salary calculation workflow - typically the workflow from draft to Payment.",
    			paidAt: {
    				title: "paid at",
    				descr: "Time when this calculation was paid (if it was paid) to Palkkaus service."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			salaryPaidAt: {
    				title: "salary paid at",
    				descr: "Time when the salary was paid by Palkkaus service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from Draft to PaymentSucceeded"
    			}
    		},
    		CalcWorktime: {
    			title: "calc worktime",
    			descr: "Worktime data that is stored as part of Salary calculation: work days, absences, holidays and holiday accrual. In the future may also contain relevant part of timesheet data (work hours).",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of days of non-paid absences.\nThese are deducted from the Monthly salary, not from Hourly salary at the moment."
    			},
    			accruals: {
    				title: "accruals",
    				descr: "Accruals for each calendar month in this calculation."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Parameters related to Holiday Bonus."
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period that is updated from employment at each recalculation (as opposed to holiday specification)."
    			},
    			holidaySpec: {
    				title: "holiday spec",
    				descr: "Specification for the holiday calculation"
    			},
    			hourly: {
    				title: "hourly",
    				descr: "Parameters related to Hourly or Performance based salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			leavesDailySalary: {
    				title: "leaves daily salary",
    				descr: "Price for a single holiday day"
    			},
    			leavesDays: {
    				title: "leaves days",
    				descr: "Number of paid Leave days that is paid in this salary calculation / for this period."
    			},
    			monthly: {
    				title: "monthly",
    				descr: "Parameters related to Monthly salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			workDays: {
    				title: "work days",
    				descr: "Describes the working days in the period.\nCurrently, uses the DaysCount, but later the full days array will be taken to use.\nThis includes the absence days (divider in AbsencesDaySalary)."
    			}
    		},
    		Calculation: {
    			title: "calculation",
    			descr: "Calculation is the most important model in the Palkkaus.fi API. We suggest you open the separately provided documentation and especially the class diagram to understand the whole model in detail.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer",
    				descr: "The employer object for this calculation.\nTypically, you do not need to set this before the actual payment process."
    			},
    			framework: {
    				title: "framework",
    				descr: "DEPRECIATED: Usecase will be replacing this functionality in a next major release.\nThe Framework Agreement (TES) parameters that affect the framework based side costs etc. of this calculation"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "Information about the work that is not directly related to calculating salary and expenses."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "The results of the calculation. In the API, this is a read-only object."
    			},
    			rows: {
    				title: "rows",
    				descr: "Set of rows that make up the salary calculation:\nExpenses, Benefits, deductions etc. Note that side costs are not rows."
    			},
    			salary: {
    				title: "salary",
    				descr: "The main salary element.\nAlternatively the salary can be set in the Rows object - this will be the only way to set multiple salary rows (not yet supported)."
    			},
    			sharing: {
    				title: "sharing",
    				descr: "Sharing of the calculation. This object cannot be modified directly,\nuse specialized methods in Calculations service."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing salary calculations.\nIn practice, a usecase is either a user interface or a microservice that provides a salary calculation\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of calculation.\nNOTE: This functionality will replace Framework property in a next major release."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker (employee) that has performed the work."
    			},
    			workflow: {
    				title: "workflow",
    				descr: "Information about the workflow and state of the calculation."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "Readonly workflow data, which is exposed to API."
    			},
    			worktime: {
    				title: "worktime",
    				descr: "Worktime data that is stored as part of Salary calculation:\nwork days, absences, holidays and holiday accrual.\nIn the future may also contain relevant part of timesheet data (work hours)."
    			}
    		},
    		CalculationData: {
    			title: "calculation data",
    			descr: "Calculation specific business data for index.",
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "Payroll id."
    			},
    			pensionPayment: {
    				title: "pension payment",
    				descr: "The total pension payment for this calculation."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "A flag indicating if the requested date for the SalaryDate has been set.\nIf the date has been set, the value of the flag is 'SALA'."
    			},
    			socialSecurityPayment: {
    				title: "social security payment",
    				descr: "The social security payment- paid as part of taxation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from the end-user point-of-view."
    			},
    			taxPayment: {
    				title: "tax payment",
    				descr: "The taxes that were deducted from the worker salary a paid as part of taxation."
    			},
    			usecaseUri: {
    				title: "usecase uri",
    				descr: "Use case uri."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "Work end date."
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Work start date."
    			},
    			workerPayment: {
    				title: "worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			}
    		},
    		CalculationListItem: {
    			title: "calculation list item",
    			descr: "Represents a single item in a list of calculations in the user interface. It contains all essential information of the calculation and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		CalculationResult: {
    			title: "calculation result",
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			irRows: {
    				title: "ir rows",
    				descr: "Rows that are compatible with Incomes Register (tulorekisteri) process.\nThese are available only for calculations paid after 7.2.2019 (TODO: estimation as of writing)."
    			},
    			responsibilities: {
    				title: "responsibilities",
    				descr: "Responsibilities of Employer and parameters that affect those responsibilities."
    			},
    			rows: {
    				title: "rows",
    				descr: "The result rows. Note that this collection also contains the main Salary entry as one of the rows."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			validation: {
    				title: "validation",
    				descr: "The validation result for the current calculation.\nPlease note that this is not serialized into the data storage."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		CalculationSettings: {
    			title: "calculation settings",
    			descr: "Settings that affect the calculation result of the salaries.",
    			household: {
    				title: "household",
    				descr: "Settings that are specific to Households logic.\nThese do not apply to Companies and may be null for them"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Settings related to mandatory accident insurance."
    			},
    			marketing: {
    				title: "marketing",
    				descr: "Marketing settings, E.g. campaigns, customer groups etc."
    			},
    			pension: {
    				title: "pension",
    				descr: "Pension insurance related settings."
    			},
    			tax: {
    				title: "tax",
    				descr: "Settings related to taxation. Mainly widthholding tax, but potentially other taxes as well."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		CalculationSharing: {
    			title: "calculation sharing",
    			descr: "Defines the properties related to sharing and sending of the calculation.",
    			type: {
    				title: "type",
    				descr: "Type of sharing for this calculation."
    			},
    			uri: {
    				title: "uri",
    				descr: "Sharing uri."
    			}
    		},
    		CalculationSummary: {
    			title: "calculation summary",
    			descr: "Summarises from a large calculation object the relevant data for yearend feedback.",
    			benefitsInKindWithoutPhone: {
    				title: "benefits in kind without phone",
    				descr: "Benefits excluding phone benefit: Accomodation, Meals and other"
    			},
    			calculationId: {
    				title: "calculation id",
    				descr: "Identifier of the calculation or correction."
    			},
    			compensations: {
    				title: "compensations",
    				descr: "Compensations is daily allowance (including half days) plus meals compensation."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Total gross salary"
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes."
    			},
    			message: {
    				title: "message",
    				descr: "Message related to this row. Mainly for backoffice purposes."
    			},
    			milageCompensation: {
    				title: "milage compensation",
    				descr: "Milage compensation includes MilageOwnCar, Daily milage as well as other milage"
    			},
    			originalIsDeductible: {
    				title: "original is deductible",
    				descr: "If true the original calculation was marked as deductible (household deduction)."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "Date when the calculation was paid or where the correction should be set."
    			},
    			phoneBenefit: {
    				title: "phone benefit",
    				descr: "Phone benefit in euros"
    			},
    			salary: {
    				title: "salary",
    				descr: "Taxable salary of the calculation"
    			},
    			salaryType: {
    				title: "salary type",
    				descr: "Type of salary from Pension and Taxation point-of-view"
    			},
    			selectedIsDeductible: {
    				title: "selected is deductible",
    				descr: "If true the the calculation is currently marked as deductible (household deduction).\nDefault value is OriginalIsDeductible."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "The mandatory side costs for taxation purposes"
    			},
    			socialInsuranceDeductionsFromEmployee: {
    				title: "social insurance deductions from employee",
    				descr: "Pension and unemployment insurance deductions from the Worker"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "Categories for household deduction.\nUI implementors should force this enumeration being a single value although historically flags were supported."
    			},
    			type: {
    				title: "type",
    				descr: "Type of Calculation summary"
    			},
    			withholdingTax: {
    				title: "withholding tax",
    				descr: "Widthholding tax deducted from the Worker"
    			}
    		},
    		CalculationsDataRow: {
    			title: "calculations data row",
    			descr: "Data row for calculations data. This is typically used for imports and exports.",
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Tax if calculated"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the calculation"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End date of the work"
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start date of the work"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID for the worker."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			status: {
    				title: "status",
    				descr: "Read only calculation status"
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			}
    		},
    		CarBenefit: {
    			title: "car benefit",
    			ageGroupCode: {
    				title: "age group code"
    			},
    			carBenefitCode: {
    				title: "car benefit code"
    			},
    			kilometers: {
    				title: "kilometers"
    			}
    		},
    		CarBenefitUsecase: {
    			title: "car benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.CarBenefitUsecase to IrRows",
    			ageGroup: {
    				title: "age group",
    				descr: "Age group as in A,B,C or I for international."
    			},
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isKilometersBased: {
    				title: "is kilometers based",
    				descr: "If true, the calculation is kilometers based."
    			},
    			kilometers: {
    				title: "kilometers",
    				descr: "Number of kilometers is filled in only if that type of benefit calculation is used."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Certificate: {
    			title: {
    				title: "title",
    				descr: "End user description for the certificate."
    			},
    			descr: "User certificate.",
    			bytes: {
    				title: "bytes",
    				descr: "Certificate bytes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			password: {
    				title: "password",
    				descr: "Password for issuing a new certificate. Not persisted."
    			},
    			thumbprint: {
    				title: "thumbprint",
    				descr: "Certificate thumbprint."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		City: {
    			title: "city",
    			count: {
    				title: "count"
    			},
    			grouping: {
    				title: "grouping"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			nameLang2: {
    				title: "name lang2"
    			}
    		},
    		ClassDoc: {
    			title: "class doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			extendedBy: {
    				title: "extended by",
    				descr: "Types that extend this type (extendedBy)"
    			},
    			"extends": {
    				title: "extends",
    				descr: "Collection of reference types that this class extends (extendedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			implementedBy: {
    				title: "implemented by",
    				descr: "Types that iplement this interface (implementedBy)"
    			},
    			"implements": {
    				title: "implements",
    				descr: "Collection of interfaces that this class implements (implementedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			members: {
    				title: "members",
    				descr: "Public properties, fields and functions."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			usages: {
    				title: "usages",
    				descr: "Salaxy specific logic may add usages here.\nCurrently, components will add themselves to controllers."
    			}
    		},
    		ClassMemberDoc: {
    			title: "class member doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Display grouping for the member: E.g. accessors are considered properties here."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			parameters: {
    				title: "parameters",
    				descr: "For grouping type \"Method\", lists parameters\nTODO: Consider making a separate type for properties."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the member."
    			}
    		},
    		CompanyAccount: {
    			title: "company account",
    			descr: "A company account - note that this can also be an association etc. - something that has a Company ID.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			companyType: {
    				title: "company type",
    				descr: "High level company type"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Majority shareholders (over 25% stake) are stored in this field.\nOwnership is a list of Shareholder objects."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		CompanyInput: {
    			title: "company input",
    			descr: "Represents the input for creating a company account.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the company"
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a company account based on company data."
    			},
    			doYtjUpdate: {
    				title: "do ytj update",
    				descr: "If true, will fetch the data from the Finnish Business Infromation System (YTJ/BIS).\nFetching is done firstly based on Business ID, if that is not valid then based on name."
    			},
    			logo: {
    				title: "logo",
    				descr: "URL to a Logo picture (preferably PNG). This logo should be a square - icon type of logo for avatar."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Company ownership for KYC"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "Identifier at the partner site - typicelly an organization ID."
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given BusinessID exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			},
    			ytjDetails: {
    				title: "ytj details",
    				descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)"
    			},
    			ytjSearch: {
    				title: "ytj search",
    				descr: "A search to Finnish Business Infromation System (YTJ/BIS). This search is triggered if you set the RawInput field."
    			}
    		},
    		ComponentDoc: {
    			title: "component doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attributes: {
    				title: "attributes",
    				descr: "HTML attributes available to specify options to the component."
    			},
    			controller: {
    				title: "controller",
    				descr: "Name of the controller that implements the functionality of the component"
    			},
    			defaultTemplate: {
    				title: "default template",
    				descr: "The default template (view) used in the rendering of the component."
    			},
    			elem: {
    				title: "elem",
    				descr: "The component tag / element in HTML"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			require: {
    				title: "require",
    				descr: "Required components in the outside scope"
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			transclude: {
    				title: "transclude",
    				descr: "Transclude slots"
    			}
    		},
    		Config: {
    			title: "config",
    			descr: "Environment specific configuration for Salaxy API's and JavaScript in general",
    			apiServer: {
    				title: "api server",
    				descr: "The base address of the Salaxy http API server, e.g. https://secure.salaxy.com"
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "A flag indicating if the current configuration is for test environment only. If true, the site is for testing only."
    			},
    			useCookie: {
    				title: "use cookie",
    				descr: "A flag indicating if the site sets a Salaxy cookie which persists over web sessions. If true, the cookie will be created."
    			},
    			wwwServer: {
    				title: "www server",
    				descr: "The base address of the Palkkaus web site, e.g. https://www.palkkaus.fi"
    			}
    		},
    		Contact: {
    			title: "contact",
    			descr: "Common contact information object for Person, Company, Location etc.",
    			city: {
    				title: "city",
    				descr: "City or community (postitoimipaikka)"
    			},
    			countryCode: {
    				title: "country code",
    				descr: "This is the ISO code for country.\nCurrently, it should always be \"fi\"."
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail address. Always lower invariant, trimmed and empty string set to null."
    			},
    			postalCode: {
    				title: "postal code",
    				descr: "Postal code"
    			},
    			street: {
    				title: "street",
    				descr: "Street address"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		ContractParty: {
    			title: "contract party",
    			descr: "A signing party in a contract.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the Party.\nNote that this is not automatically fetched by all methods: Especially when IsSelf is true."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information - note that for most signatures, only E-mail and phone number are required."
    			},
    			contractPartyType: {
    				title: "contract party type",
    				descr: "Party type (Person or Prospect)"
    			},
    			iban: {
    				title: "iban",
    				descr: "Bank account IBAN number"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the party: Currently always an account, but may be an external ID later."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true, contact is an employer"
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nIf set to false AND was previously true, will set the Profile properties null (unless explicitly set to something else)"
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "True if the person has signed the contract."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official ID if needed in signature"
    			},
    			role: {
    				title: "role",
    				descr: "Role of the person - justification for authorization"
    			},
    			signatureUrl: {
    				title: "signature url",
    				descr: "Reserved for cases where there are several signees and each has a separate signature document."
    			}
    		},
    		ContractSalary: {
    			title: "contract salary",
    			descr: "Part of the Employment contract that is relevant to paying the salary",
    			collectiveContract: {
    				title: "collective contract"
    			},
    			endDate: {
    				title: "end date",
    				descr: "End date for the employment.\nOnly set if the contract is Fixed term."
    			},
    			payDay: {
    				title: "pay day"
    			},
    			salaryAmount: {
    				title: "salary amount"
    			},
    			salaryCompension: {
    				title: "salary compension"
    			},
    			salaryType: {
    				title: "salary type"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Starting date for the employment.\nShould always be set for a valid contract."
    			},
    			workHours: {
    				title: "work hours"
    			},
    			workType: {
    				title: "work type"
    			},
    			worksHoursType: {
    				title: "works hours type"
    			}
    		},
    		CoreTypeDoc: {
    			title: "core type doc",
    			descr: "H",
    			classes: {
    				title: "classes",
    				descr: "H"
    			},
    			enums: {
    				title: "enums",
    				descr: "H"
    			}
    		},
    		CustomerFundsSettings: {
    			title: "customer funds settings",
    			descr: "Settings related to payment methods that use the customer funds account (gross payment).",
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			}
    		},
    		DailyAllowanceUsecase: {
    			title: "daily allowance usecase",
    			descr: "Handles the Calculation of CalculationRowType.DailyAllowance to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		DataRow: {
    			title: "data row",
    			descr: "Interface for data row for exports and imports",
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		DateRange: {
    			title: "date range",
    			descr: "Describes the range of dates from start to end and the number of days in between.",
    			days: {
    				title: "days",
    				descr: "The collection of days contained in the DateRange if specified by the user.\nAlternative to DaysCount. If set, should also set DaysCount for backward compatibility."
    			},
    			daysCount: {
    				title: "days count",
    				descr: "Number of days in the range.\nDepending of the context, this may be mathematical (end - start) or\nexplicitly specified (typically working days).\nTypically an integer, and calculations and UI's may choose to round it."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			}
    		},
    		DeclarationReflection: {
    			title: "declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			}
    		},
    		EInvoiceAddress: {
    			title: "e invoice address",
    			descr: "Result item of the EInvoice address query.",
    			intermediator: {
    				title: "intermediator",
    				descr: "Intermediator id for this receiver address."
    			},
    			intermediatorName: {
    				title: "intermediator name",
    				descr: "Intermediator name for this receiver address."
    			},
    			receiver: {
    				title: "receiver",
    				descr: "Receiver EInvoice address of company."
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Name for the company."
    			},
    			receiverOfficialId: {
    				title: "receiver official id",
    				descr: "Official id of the company."
    			}
    		},
    		ESalaryPayment: {
    			title: "e salary payment",
    			descr: "E-salary / E-palkka is a way for a partner to direct an Employer to Salary payment without having access to Employer account. In first instance, E-salary is targeting Household Employers only.",
    			calc: {
    				title: "calc",
    				descr: "Calculation that should be paid by the Employer."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			household: {
    				title: "household",
    				descr: "Known data for the employer household that is paying the salary.\nIt is understood that the caller may not have all the information:\nPalkkaus.fi will ask the rest if necessary."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			request: {
    				title: "request",
    				descr: "Payment request and how to return to the calling service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the Payment.\nNOTE: You can only set states Created, Shared and Opened.\nOther states are set by the Salaxy service and cannot be modified throgh API."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides the validation of the ESalaryPayment object after the object has been saved."
    			}
    		},
    		EmailMessage: {
    			title: "email message",
    			descr: "API EmailMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (email address)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			subject: {
    				title: "subject",
    				descr: "Message subject"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (email address)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		EmployerCalculationDTO: {
    			title: "employer calculation d t o",
    			descr: "Models the business logic of Employer side of the calculation. For documentation, see the IEmployerCalculation interface.",
    			allSideCosts: {
    				title: "all side costs",
    				descr: "All side cost: The mandatory side costs + Palkkaus and partner fees"
    			},
    			deductionForeclosure: {
    				title: "deduction foreclosure",
    				descr: "Foreclosure that has already been paid by the employer."
    			},
    			deductionOtherDeductions: {
    				title: "deduction other deductions",
    				descr: "Other deductions that are deducted from the net salary."
    			},
    			deductionPensionSelfPayment: {
    				title: "deduction pension self payment",
    				descr: "Pension payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionSalaryAdvance: {
    				title: "deduction salary advance",
    				descr: "Salary advance - this part has already been paid."
    			},
    			deductionTaxAndSocialSecuritySelfPayment: {
    				title: "deduction tax and social security self payment",
    				descr: "Tax and social security payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnemploymentSelfPayment: {
    				title: "deduction unemployment self payment",
    				descr: "Unemployment insurance (TVR) payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnionPayment: {
    				title: "deduction union payment",
    				descr: "Union payment if it is deducted from the final figure"
    			},
    			deductionWorkerSelfPayment: {
    				title: "deduction worker self payment",
    				descr: "Worker salary and expense payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			finalCost: {
    				title: "final cost",
    				descr: "The final cost after household decuction from the tax authorities:\nAdd to the payment the deductions (paid by employer) and deduct House hold deduction (later potentially other such subsidies)"
    			},
    			foreclosureByPalkkaus: {
    				title: "foreclosure by palkkaus",
    				descr: "Foreclosure that Palkkaus.fi pays in behalf of employer."
    			},
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction that the tax man will reimburse in taxation.\nThis is an estimation: There is minimum and maximum limits etc."
    			},
    			mandatorySideCosts: {
    				title: "mandatory side costs",
    				descr: "The mandatory side costs are Pension (TyEL), Social secuirty and Unemployment insurance."
    			},
    			palkkaus: {
    				title: "palkkaus",
    				descr: "The Palkkaus.fi fee including potential partner fees."
    			},
    			pension: {
    				title: "pension",
    				descr: "The part of the Pension that Employer pays"
    			},
    			socialSecurity: {
    				title: "social security",
    				descr: "The social security payment- paid as part of taxation (ennakonpidätys)."
    			},
    			totalDeductions: {
    				title: "total deductions",
    				descr: "Total deductions that are made from the payment that is paid to Palkkaus and then to Worker"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment that is Paid to Palkkaus.fi"
    			},
    			totalPaymentLegacy: {
    				title: "total payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			totalSalaryCost: {
    				title: "total salary cost",
    				descr: "Total salary cost is the paid salary, deductions and side costs, but not expenses."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The part of the unemployment insurance that the employer pays"
    			}
    		},
    		EmployerGroups: {
    			title: "employer groups",
    			descr: "Employer groups that are set on Example calculations. These will be replaced by properties from Worker / Employer",
    			campaignPricing: {
    				title: "campaign pricing",
    				descr: "Campaign pricing may affect Palkkaus fees.\nThis field is set by the system when coming to the payment based on Employer properties (usually campaign code)."
    			},
    			employerType: {
    				title: "employer type",
    				descr: "Very small employers (households) do not required to report to the tax authorities."
    			},
    			isPrivatePerson: {
    				title: "is private person",
    				descr: "If true, the Employer is a private person.\nOtherwise Company or Unknown."
    			},
    			salaryPerMonth: {
    				title: "salary per month",
    				descr: "Very small salaries do not require Pension payments."
    			},
    			salaryPerYear: {
    				title: "salary per year",
    				descr: "Ranges for salary / year will affect the side costs."
    			}
    		},
    		EmployerInfo: {
    			title: "employer info",
    			descr: "Employer insurance information.",
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "The id of the insurance company."
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Insurance contract number."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "Indicates if the employer has an insurance."
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "Indicates if the insurance is provided by Palkkaus partnership insurance company."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "Indicates if the employer has a pension contract."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the mployer handles the pension himself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The id of the pension company."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Pension contract number."
    			}
    		},
    		EmploymentContract: {
    			title: "employment contract",
    			annualLeave: {
    				title: "annual leave"
    			},
    			benefitsException: {
    				title: "benefits exception"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isWorkAtHome: {
    				title: "is work at home"
    			},
    			mainWorkTasks: {
    				title: "main work tasks"
    			},
    			noticeExeption: {
    				title: "notice exeption"
    			},
    			otherPayDescription: {
    				title: "other pay description"
    			},
    			otherTerms: {
    				title: "other terms"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partTimeReason: {
    				title: "part time reason"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			salary: {
    				title: "salary"
    			},
    			testPeriod: {
    				title: "test period"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workPlace: {
    				title: "work place"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		EmploymentData: {
    			title: "employment data",
    			descr: "Employment specific business data for index.",
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			}
    		},
    		EmploymentListItem: {
    			title: "employment list item",
    			descr: "Represents a single item in a list of employments in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		EmploymentRelation: {
    			title: "employment relation",
    			descr: "Represents the Employment Relation: It is basically the properties of a Worker from the point-of-view of employer and as data owned by Employer whereas WorkerAccount is preliminarily owned by Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General information about the Employment relation."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		EmploymentRelationInfo: {
    			title: "employment relation info",
    			descr: "General information about the Employment relation.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of this employment relationship"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			workerAccountId: {
    				title: "worker account id",
    				descr: "The Account ID of the Worker if Worker"
    			}
    		},
    		EmploymentTerminationUsecase: {
    			title: "employment termination usecase",
    			descr: "Handles the Calculation of CalculationRowType.EmploymentTermination to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		EnumDoc: {
    			title: "enum doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			names: {
    				title: "names",
    				descr: "Member names"
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			values: {
    				title: "values",
    				descr: "Member values"
    			}
    		},
    		EnumMetadata: {
    			title: "enum metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Type name of the enumeration (UpperCamelCase)"
    			},
    			values: {
    				title: "values",
    				descr: "Values of the enumeration."
    			}
    		},
    		EnumValueMetadata: {
    			title: "enum value metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied.\nNOTE: Not all enumerations have description in the globalization files:\nThey are added only if necessary to the UI."
    			},
    			label: {
    				title: "label",
    				descr: "Label of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Text value of the enumeration (lowerCamelCase)"
    			},
    			order: {
    				title: "order",
    				descr: "Sorting order number of the value within the enum."
    			}
    		},
    		FileData: {
    			title: "file data",
    			descr: "Report specific business data for index.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "Number of calculations in the report."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id for the report."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "Number of workers in the report."
    			}
    		},
    		FileListItem: {
    			title: "file list item",
    			descr: "Represents a single item in a list of reports in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		FileParameter: {
    			title: "file parameter",
    			data: {
    				title: "data"
    			},
    			fileName: {
    				title: "file name"
    			}
    		},
    		FileType: {
    			title: "file type",
    			extension: {
    				title: "extension"
    			},
    			isImage: {
    				title: "is image"
    			},
    			isText: {
    				title: "is text"
    			},
    			mimeType: {
    				title: "mime type"
    			}
    		},
    		HealthCareHelttiProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer health care from the company Heltti",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by HelttiProductPackage."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			industry: {
    				title: "industry",
    				descr: "Industry of work"
    			},
    			"package": {
    				title: "package",
    				descr: "Defines the product package"
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		HolidayAccrual: {
    			title: "holiday accrual",
    			descr: "Yearly log for Holiday accrual.",
    			defaultAccrual: {
    				title: "default accrual",
    				descr: "Default accrual for months where the Accrual occurs.\nWhen Holiday year is recalculated, this is determined either from Employment Period or AccrualFixed if set."
    			},
    			endSaldo: {
    				title: "end saldo",
    				descr: "End saldo from the previous year. This is Total + Start Saldo."
    			},
    			months: {
    				title: "months",
    				descr: "Holiday accrual entries during the period."
    			},
    			period: {
    				title: "period",
    				descr: "The period for holiday accrual.\nThis is also the period of the holiday year that the Worker is Employed,\ni.e. it may be shorter than HolidayYear Period if the employment relation\nbegins or ends within the holiday year."
    			},
    			startSaldo: {
    				title: "start saldo",
    				descr: "Start saldo from the previous year."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the current year. This does not include Start saldo."
    			}
    		},
    		HolidayAccrualEntry: {
    			title: "holiday accrual entry",
    			descr: "Entry in the Yearly Holiday Accrual list.",
    			calcId: {
    				title: "calc id",
    				descr: "Identier of the calculation if Kind"
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Days added to holiday Accrual: Typically days per month."
    			},
    			index: {
    				title: "index",
    				descr: "Index of the month from 0-11 for easier iteration.\nThis is also the real identifier of the month when editing the entries through API.\nMonths 4-12 are index 0-8 and month 1-3 are index 9-11"
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			source: {
    				title: "source",
    				descr: "Source of the holiday accrual."
    			}
    		},
    		HolidayDate: {
    			title: "holiday date",
    			date: {
    				title: "date"
    			},
    			holiday: {
    				title: "holiday"
    			},
    			holidayGroups: {
    				title: "holiday groups"
    			},
    			localName: {
    				title: "local name"
    			},
    			name: {
    				title: "name"
    			}
    		},
    		HolidaySpecification: {
    			title: "holiday specification",
    			descr: "Specifies how the holidays are handled for this Worker.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			}
    		},
    		HolidaySpecificationForYear: {
    			title: "holiday specification for year",
    			descr: "Extends the Holiday specification with the properties of employment relation that are required for a specific year.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period for the purposes of holiday calculation (annual leave accrual).\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Wage basis for a Holiday Year.\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year.\nNOTE: This is not in HolidaySpecification because, the same value is stored in Employment relation."
    			}
    		},
    		HolidayYear: {
    			title: "holiday year",
    			descr: "Holiday year is in Finland from April to end of march. This is the time from which the holidays for the following summer and year are determined.",
    			accrual: {
    				title: "accrual",
    				descr: "Accrual of holidays (lomakirjanpito)."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation for the worker whose holidays are managed."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			leaves: {
    				title: "leaves",
    				descr: "Payments related to Annual Leaves (holidays)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			period: {
    				title: "period",
    				descr: "The holiday period for the year: Should always be 2.5.[Year]-30.4.[Year+1]"
    			},
    			spec: {
    				title: "spec",
    				descr: "Specifies how the holidays are handled.\nIf the Spec.Code is Undefined in Save, the contents is fetched from the current employment relation."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Salaxy ID of the worker. This value is required when inserting and cannot be edited later.\nWorkerSnapshot is automatically updated upon save."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			},
    			year: {
    				title: "year",
    				descr: "Year as integer. This value is required when inserting and cannot be edited later.\nPeriod will be set accordingly, or adjusted in case it is set below."
    			}
    		},
    		Holidays: {
    			title: "holidays",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		Holidays2: {
    			title: "holidays2",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		HouseholdSettings: {
    			title: "household settings",
    			descr: "Settings that are specific to Households. These do not apply to Companies.",
    			deductionInsuranceAmount: {
    				title: "deduction insurance amount",
    				descr: "Accident insurance amount reported for Household deduction purposes"
    			},
    			deductionSpouseName: {
    				title: "deduction spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			deductionSpousePersonalId: {
    				title: "deduction spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		HouseholdYearlyFeedback: {
    			title: "household yearly feedback",
    			descr: "Yearly feedback data that is specific to household customers. Main part is the information realted to household deduction and how it can be divided between spouses.",
    			insuranceAmount: {
    				title: "insurance amount",
    				descr: "Amount of accident insurance paid during this year as reported by the user\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			insuranceAmountFromAccount: {
    				title: "insurance amount from account",
    				descr: "Amount of accident insurance paid during this year from parner insurance (in Palkkaus.fi records).\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			spouseName: {
    				title: "spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			spousePersonalId: {
    				title: "spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		IEmployerResponsibilities: {
    			title: "i employer responsibilities",
    			descr: "Parameters that affect the calculation. EmployerResponsibilities is historical name - currentlty to object holds also other properties that are not well described as responsibilities. These based on the EmployerGroups selection or properties of the Employer and Worker snapshots as well as this Calculation",
    			calculationPaymentDate: {
    				title: "calculation payment date",
    				descr: "The assumed salary payment date from the worker perspective.\nThe system calculates the nearest possible salary date if no any salary date has been given by the\nemployer. If given, the system confirms the given date.\nThis date affects the yearly percents used in the salary calculation."
    			},
    			employerGroups: {
    				title: "employer groups",
    				descr: "Employer groups that are set on Example calculations.\nThese will be replaced by properties from Worker / Employer when these persons are set."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isPensionMonthlyContract: {
    				title: "is pension monthly contract",
    				descr: "If true, the employer has a Pension contract and makes the payments\nin a monthly schedule which means that Pension interest is calculated for the 20th of the next month.\nCurrently, this is only enabled to Etera - other companies will follow."
    			},
    			noAccidentInsurance: {
    				title: "no accident insurance",
    				descr: "If true, this employer is not required to get an accident insurance."
    			},
    			noPensionPayment: {
    				title: "no pension payment",
    				descr: "Pension is not paid if the salary to this Worker is less than a certain amount in a month (about 60€)\nor worker is less than 17 or more than 64 years old."
    			},
    			noSocialSecurity: {
    				title: "no social security",
    				descr: "If worker is less than 16 or more than 67, social secuiry payment is not paid"
    			},
    			noTaxResponsibility: {
    				title: "no tax responsibility",
    				descr: "If Employer pays to this Worker max 200 euro, he does not need to even make the reports to tax office.\nHe may do so though"
    			},
    			noTvr: {
    				title: "no tvr",
    				descr: "If Worker is less than 17 or more than 64, TVR is not paid.\nAlso if the Employer has no accident insurance."
    			},
    			noWithholdingTax: {
    				title: "no withholding tax",
    				descr: "If Employer pays to this Worker max 1500 euro, he does not need to deduct the witholding tax.\nHe may do so though"
    			},
    			pensionAgeRange: {
    				title: "pension age range",
    				descr: "The age range that is used in the business logic that determines the employer responsibilities.\nThis property replaces the EmployerGroups.AgeRange that was previous get/set.\nThis is based on WorkerDateOfBirth and CalculationPaymentDate."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Pension calculation type."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The name of the Pension company because this affects the Pension percentage used in the calculation (only in contract situation).\nIt also needs to be shown in the reports."
    			},
    			pensionOldAgePayment: {
    				title: "pension old age payment",
    				descr: "If the Worker is from 53 to 62 years old, more of the Pension payment\nis deducted from the Worker side of the salary (less for the Employer).\nNOTE: People from 63 to 67 years pay the same amount as 17-52 years old."
    			},
    			pensionRegularEmployer: {
    				title: "pension regular employer",
    				descr: "The employer should have a Pension contract (not a temporary employer).\nIt does not necessarily mean that the user has a contract yet.\nNOTE: At the moment, this is not used for calculation - only in the Side costs visualizer tool."
    			},
    			taxAndSocialSecuritySelfPayment: {
    				title: "tax and social security self payment",
    				descr: "Tax and social security payments are paid directly by the Employer - not by Palkkaus.fi"
    			},
    			unemploymentSelfPayment: {
    				title: "unemployment self payment",
    				descr: "If true, the employer pays the Unemployment insurance directly to TVR.\nAs of writing, this is true for all companies and with calculations starting 1.8.2018,\nfalse for households or calculations before the date."
    			},
    			workerDateOfBirth: {
    				title: "worker date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerDateOfBirthAccuracy: {
    				title: "worker date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerSelfPayment: {
    				title: "worker self payment",
    				descr: "Worker salary and expense payments are paid directly by the Employer - not by Palkkaus.fi"
    			}
    		},
    		IUserObjectIndex: {
    			title: "i user object index",
    			descr: "Generic index for all User Object Containers.",
    			additionalInfo: {
    				title: "additional info",
    				descr: "A short detail describing the transaction.\nShould contain the relevant information for administration purposes."
    			},
    			businessAttributes: {
    				title: "business attributes",
    				descr: "Business attributes to include further information of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			containerGuid: {
    				title: "container guid",
    				descr: "Guid of the container.\nThis is not always unique within the index.\nHowver, within the container this is unique."
    			},
    			containerType: {
    				title: "container type",
    				descr: "The container type that is the source of this index item."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "Created at time"
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id for the object in 3rd party system. E.g. Filing code in MoneyTransfer."
    			},
    			flags: {
    				title: "flags",
    				descr: "Classifications for the object. First implementation supports BusinessEventType enumeration for values."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			handledAt: {
    				title: "handled at",
    				descr: "This is the recorded handling date for the object. E.g. when the calculation was paid to worker."
    			},
    			icon: {
    				title: "icon",
    				descr: "Icon for the purposes of listings."
    			},
    			id: {
    				title: "id",
    				descr: "Unique key for the object. Unique within an Index.\nIn the case of UserObject, this is [Guid]_[ContainerType]_[Owner].\nFor non-user objects, it should typically be just a Guid."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Messages (Error, Warning, OnHold, Note) that are active in the system.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the object is also the Partition key.\n Mandatory in most storage scenarios."
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Latest partner information."
    			},
    			payloadType: {
    				title: "payload type",
    				descr: "The business object type that is the source of this event."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			reference: {
    				title: "reference",
    				descr: "Reference information. E.g. Payment or MoneyTransfer reference number."
    			},
    			revenue: {
    				title: "revenue",
    				descr: "Estimated revenue that the transaction generated to Palkkaus.fi. Only add here the revenue, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Updated at time: Note that in typical implementations, this is the UserUpdatedAt time."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		IVerifiedAccount: {
    			title: "i verified account",
    			descr: "An account that is verified. Typically this means a digitally signed contract. This means that the account also has an Identity object. However, as the Identity object contains confidential information, it is not necessarily present in all method calls.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Pricing partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated"
    			}
    		},
    		Identity: {
    			title: "identity",
    			descr: "Identity is the official identity of an actor. This part is not freely editable as changing it requires signature or some other means. Identity is also often fetched from external sources such as govenment registries.",
    			contract: {
    				title: "contract",
    				descr: "The latest authorization contract that is the bases for this Identity"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name OR company name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			isPep: {
    				title: "is pep",
    				descr: "Indicates whether the principal user of the customer is a politically exposed person or not."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name\nAvailable only to real persons (as opposed to companies)."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the actor:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)"
    			},
    			primaryContact: {
    				title: "primary contact",
    				descr: "Mainly for company accounts, but in special cases for Persons,\nthis is the main Contact party and usually the signer for the account."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			}
    		},
    		ImportBatchInfo: {
    			title: "import batch info",
    			descr: "Contains the basic details and the current status of the batch phase.\nTypically this data is created and updated during the import session.",
    			createdAt: {
    				title: "created at",
    				descr: "Date and time when the batch was created."
    			},
    			id: {
    				title: "id",
    				descr: "Unique id for the batch."
    			},
    			name: {
    				title: "name",
    				descr: "User friendly name for the batch."
    			},
    			phase: {
    				title: "phase",
    				descr: "Current phase of the batch:\n - ```new``` A new batch. Contains no data.\n - ```read``` The data file (e.g. csv) has been read from file or clipboard.\n - ```mapped``` The data has been mapped to a supported tabular format.\n - ```validated``` The data has been validated.\n - ```imported``` The data has been imported to backend.\n - ```rejected``` The batch has been rejected manually."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of the current phase:\n - ```info``` Typically the default status for the phase which has not yet been done.\n - ```success``` The current phase has been successfully finished.\n - ```warning``` The current phase has been finished with warnings.\n - ```error``` The current phase contains errors."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Date and time when the batch was last updated."
    			}
    		},
    		ImportResult: {
    			title: "import result",
    			descr: "Result for importing data (CSV, Excel copy-paste etc.)",
    			data: {
    				title: "data",
    				descr: "Result data is an array of objects with properties specified as headers"
    			},
    			errors: {
    				title: "errors",
    				descr: "Errors as validation errors."
    			},
    			headers: {
    				title: "headers",
    				descr: "Headers for the import: These are also the properties for the JSON"
    			},
    			ignoredLines: {
    				title: "ignored lines",
    				descr: "Number of ignored lines (empty line or rem/annotated) in the import."
    			}
    		},
    		ImportableCrudApi: {
    			title: "importable crud api",
    			descr: "Interface for APIs which implement import/export functionality.",
    			constructor: {
    				title: "constructor",
    				descr: "Constructor creates a new CRUD api with given Ajax-implementation"
    			},
    			ajax: {
    				title: "ajax"
    			},
    			baseUrl: {
    				title: "base url",
    				descr: "Deriving classes should define the baseURl for the method. E.g. \"/calculations\"\nAjax implementations will add the beginning (e.g. \"https://test-api.salaxy.com/api/v02\")"
    			},
    			"delete": {
    				title: "delete",
    				descr: "Deletes an single item from the sotrage\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			deleteWorkflowEvent: {
    				title: "delete workflow event",
    				descr: "Deletes the worklflow item or items from the item.\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			getAll: {
    				title: "get all",
    				descr: "Gets all the items of a given type.\n\n**Returns**: A Promise with result data array.\n"
    			},
    			getBaseUrl: {
    				title: "get base url",
    				descr: "Returns the base url for this api."
    			},
    			getBlank: {
    				title: "get blank",
    				descr: "Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.\nThe basic idea is that all the child object properties should be non-null to avoid null reference exceptions.\nIn special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.\nStrings, dates and booleans may be null (false if bool), but typically just undefined."
    			},
    			getListItemMappers: {
    				title: "get list item mappers",
    				descr: "Available list item mappers for import/export"
    			},
    			getOData: {
    				title: "get o data",
    				descr: "Makes an OData query to the service's main OData method."
    			},
    			getODataBase: {
    				title: "get o data base",
    				descr: "Makes an OData query to the the given service URL."
    			},
    			getODataUrl: {
    				title: "get o data url",
    				descr: "Gets the OData URL for v03 compatible services"
    			},
    			getObjectMappers: {
    				title: "get object mappers",
    				descr: "Available object mappers for import/export"
    			},
    			getSingle: {
    				title: "get single",
    				descr: "Gets a single item based on identier\n\n**Returns**: A Promise with result data.\n"
    			},
    			getValidationResult: {
    				title: "get validation result",
    				descr: "Returns the validation object for the validated object."
    			},
    			save: {
    				title: "save",
    				descr: "Saves an item to the storage.\nIf id is null, this is add/insert. If id exists, this is update.\n\n**Returns**: A Promise with result data as saved to the storage (contains id, createAt, owner etc.).\n"
    			},
    			saveWorkflowEvent: {
    				title: "save workflow event",
    				descr: "Adds/updates the workflow event for the item.\n\n**Returns**: Workflow data for the item.\n"
    			},
    			validate: {
    				title: "validate",
    				descr: "Validates the item using a server side call.\n\n**Returns**: A Promise with result data containing validation property.\n"
    			}
    		},
    		IncomeTypeMetadata: {
    			title: "income type metadata",
    			descr: "Describes the properties and behavior of the Income types in Incomes register.",
    			accidentInsurance: {
    				title: "accident insurance",
    				descr: "If true Accident insurance payments are made for this income type by default."
    			},
    			calcGrouping: {
    				title: "calc grouping",
    				descr: "Grouping that is used in reporting etc."
    			},
    			code: {
    				title: "code",
    				descr: "Code in incomes register. Language versioned."
    			},
    			description: {
    				title: "description",
    				descr: "Description of the row.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			healthInsurance: {
    				title: "health insurance",
    				descr: "If true Health insurance payments are made for this income type by default."
    			},
    			insuranceInformationAllowed: {
    				title: "insurance information allowed",
    				descr: "If true the default values for insurances may be overriden for a single row."
    			},
    			isNegativeSupported: {
    				title: "is negative supported",
    				descr: "If true, teh row may have a negative value."
    			},
    			isSupported: {
    				title: "is supported",
    				descr: "If false, the row type is not supported in this version"
    			},
    			label: {
    				title: "label",
    				descr: "Language versioned label for the income type."
    			},
    			notSupportedError: {
    				title: "not supported error",
    				descr: "When isSupported is false, there is a describtive text that tells why the type is not supported. Text is always in Finnish."
    			},
    			paymentDefault: {
    				title: "payment default",
    				descr: "Default behavior for payment through Salaxy Customer Funds account"
    			},
    			pensionInsurance: {
    				title: "pension insurance",
    				descr: "If true Pension payments are made for this income type by default."
    			},
    			taxAndSidecostsDescr: {
    				title: "tax and sidecosts descr",
    				descr: "Description of how the row is taken into widthholding tax\nand how insurance payments (pension, unemployment, accident and health care) are made.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			taxDefault: {
    				title: "tax default",
    				descr: "Describes the behavior of this row in calculating widthholding tax."
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "If true Unemployment insurance payments are made for this income type by default."
    			}
    		},
    		InsuranceProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer insurance. Currently, this is the mandatory employer insurance only.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPartnerInsurance."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "Company for the current insurance"
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		InsuranceSettings: {
    			title: "insurance settings",
    			company: {
    				title: "company",
    				descr: "Company for the current insurance"
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the insurance pension"
    			}
    		},
    		Invoice: {
    			title: "invoice",
    			descr: "Invoice for the pending payment in the business system. For example, a salary gross payment created for the salary calculation and delivered as an e-invoice to the employer. For example, a salary net payment, created for the employer which is self paying the net payment to the worker.",
    			content: {
    				title: "content",
    				descr: "Invoice data in XML or json format."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			header: {
    				title: "header",
    				descr: "Invoice header details."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Invoice payer details."
    			},
    			pdf: {
    				title: "pdf",
    				descr: "Invoice in pdf format."
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Invoice recipient details."
    			},
    			rows: {
    				title: "rows",
    				descr: "List of invoice rows."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		InvoiceData: {
    			title: "invoice data",
    			descr: "Invoice specific business data for index.",
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			}
    		},
    		InvoiceHeader: {
    			title: "invoice header",
    			descr: "Header for the invoice",
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business object ids (typically calculations) on which the payment is based on."
    			},
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			externalId: {
    				title: "external id",
    				descr: "The external ID is used for matching the executed payment from the 3rd party system\nto the pending payment in the Palkkaus system."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this invoice is done.\nFor salaries, this is salary date, for monthly / quarterly payments, this is the first day of period."
    			},
    			message: {
    				title: "message",
    				descr: "The free text that is displayed / sent with the invoice.\nMay have multiple lines."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Payment reference (e.g. RF0710010831).\nPlease note that in salary payment scenarios, this is not necessarily unique."
    			},
    			status: {
    				title: "status",
    				descr: "The current notified status of the invoice in external system."
    			},
    			total: {
    				title: "total",
    				descr: "Total payable amount for the invoice including VAT."
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Total amount for the invoice excluding VAT."
    			},
    			type: {
    				title: "type",
    				descr: "Invoice type."
    			}
    		},
    		InvoiceListItem: {
    			title: "invoice list item",
    			descr: "Represents a single item in a list of invoices in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		InvoiceParty: {
    			title: "invoice party",
    			descr: "Party in an Varma Invoice: Either Payer or Recipient.",
    			companyId: {
    				title: "company id",
    				descr: "Finnish Company ID (Y-tunnus)."
    			},
    			companyName: {
    				title: "company name",
    				descr: "Organisation name"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information"
    			},
    			contactPersonName: {
    				title: "contact person name",
    				descr: "Contact person Full name"
    			}
    		},
    		InvoicePayer: {
    			title: "invoice payer",
    			descr: "Payment payer party of the invoice.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the party."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the party."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official identifier of the party:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)."
    			},
    			partnerKey: {
    				title: "partner key",
    				descr: "Optional ID for identifying the party in the third party system.\nThis is the same partner key that is given as part of Single Sign-On process."
    			}
    		},
    		InvoiceRecipient: {
    			title: "invoice recipient",
    			descr: "Payment receiver party of the invoice.",
    			beneficiary: {
    				title: "beneficiary",
    				descr: "The ultimate beneficiary of the money:\nThis may be a Palkkaus.fi Worker Account (Avatar id is not null)\nor some other entity: Tax authorities, pension company etc. (Avatar id is null).\nTypically beneficiary is the same as Iban, OfficialId, FullName and Contact in this object,\nbut this is not true in edge cases (e.g. customer funds account)."
    			},
    			contact: {
    				title: "contact",
    				descr: "Optional contact info. Not available for all recipients."
    			},
    			fullName: {
    				title: "full name",
    				descr: "The full name of the payment receiver for payment purposes."
    			},
    			iban: {
    				title: "iban",
    				descr: "The International Bank Account Number of the receiver."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the party: Finnish Personal Id (HETU) for a Person or Company Id (Y-tunnus) for a company or association.\nWARNING: This is currently needed for Finvoice only (https://www.finanssiala.fi/finvoice/Sivut/default.aspx). As such we consider removing it from persons.\nPlease inform if you use this property!"
    			}
    		},
    		InvoiceRow: {
    			title: "invoice row",
    			descr: "Row for the invoice",
    			accountingAccountNumber: {
    				title: "accounting account number",
    				descr: "Suggested account number (RowNormalProposedAccountIdentifier)."
    			},
    			accountingCode: {
    				title: "accounting code",
    				descr: "Textual code for accounting mapping (RowProposedAccountText, Raportointikoodi).\nIn Finvoice: \"Accounting information agreed between seller and buyer.\""
    			},
    			articleId: {
    				title: "article id",
    				descr: "Article Identifier is given by seller, EAN etc.\nIn the case of Salaxy salaries, this is CalculationRowType."
    			},
    			articleName: {
    				title: "article name",
    				descr: "Name of the product or service.\nIn the case of Salaxy salaries, this is language versioned text of CalculationRowType."
    			},
    			businessObject: {
    				title: "business object",
    				descr: "Business object id (typically calculation) on which the row is based on."
    			},
    			count: {
    				title: "count",
    				descr: "Count (DeliveredQuantity in Finvoice) is the quantity of product or service."
    			},
    			message: {
    				title: "message",
    				descr: "Message (RowFreeText in Finvoice) is the message is mainly designed for accounting systems."
    			},
    			price: {
    				title: "price",
    				descr: "Price including VAT (UnitPriceVatIncludedAmount, Yksikköhinta verollinen)"
    			},
    			priceExVat: {
    				title: "price ex vat",
    				descr: "Price of one product or service without Vat (UnitPriceAmount, Yksikköhinta veroton).\nYksikköhinta voidaan ilmoittaa myös verollisena UnitPriceVatIncludedAmount- elementissä laskutuskäytännön ja verottajan ALV-ohjeistuksen mukaan."
    			},
    			total: {
    				title: "total",
    				descr: "Row total with VAT (RowAmount, Yhteensä verollinen)"
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Row total excluding VAT (RowVatExcludedAmount, Yhteensä veroton)"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Vat percent related to the product or service (RowVatRatePercent, Alv%)"
    			}
    		},
    		InvoiceSettings: {
    			title: "invoice settings",
    			descr: "Settings related to invoices that are sent to customers.",
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			}
    		},
    		InvoiceStatusNotification: {
    			title: "invoice status notification",
    			descr: "Contents for the invoice status notification message.",
    			amount: {
    				title: "amount",
    				descr: "Required realized amount of the transaction."
    			},
    			date: {
    				title: "date",
    				descr: "Required Date for the transaction (booking date)."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id can be set in the notification for the invoice for later messaging.\nIf the notification message contains an External Id AND regular Id, the External Id is updated in the Salaxy database.\nIf the notification contains External Id, but NO regular Id, an invoice is fetched based on External Id.\nIn the latter case, a non-existent Id or multiple invoices with the same External Id will throw an exception and the entire update process is canceled."
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID for the invoice in Salaxy system.\nIf omitted, the resolving to an invoice will be made using external id."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			receiverFullName: {
    				title: "receiver full name",
    				descr: "Optional full name of the payment receiver. Should be sent only if the user or process has the possibility of changing the name."
    			},
    			receiverIban: {
    				title: "receiver iban",
    				descr: "Optional bank account IBAN of the receiver. Should be sent only if the user or process has the possibility of changing the IBAN."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Optional payment reference number. Should be sent only if the user or process has the possibility of changing or removing the reference number.\nChanging the reference number will typically lead to an error in the back office process that needs to be manually handled."
    			},
    			status: {
    				title: "status",
    				descr: "New status for the invoice. Required main payload of the call."
    			}
    		},
    		IrDetails: {
    			title: "ir details",
    			descr: "Reporting details to Incomes registry (Tulorekisteri).",
    			carBenefit: {
    				title: "car benefit",
    				descr: "Provides the additional data needed for car benefit."
    			},
    			code: {
    				title: "code",
    				descr: "The number code in Incomes Register."
    			},
    			dailyAllowance: {
    				title: "daily allowance",
    				descr: "Daily allowance code."
    			},
    			flags: {
    				title: "flags",
    				descr: "Boolean flags that can be set on transaction."
    			},
    			insuranceExceptions: {
    				title: "insurance exceptions",
    				descr: "Defines the exceptions to Insurance / Sidecost defaults that are set by the row type."
    			},
    			mealBenefitIsTaxValue: {
    				title: "meal benefit is tax value",
    				descr: "Set this to true if meak benefit is of tax value - false if it si not.\nThe value should be null if the row is not mealBenefit."
    			}
    		},
    		IrIncomeTypeUsecase: {
    			title: "ir income type usecase",
    			descr: "Handles the Calculation of CalculationRowType.IrIncomeType to IrRows",
    			irData: {
    				title: "ir data",
    				descr: "Reporting details to Incomes registry (Tulorekisteri)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		IrRow: {
    			title: "ir row",
    			descr: "Business level modeling corresponding to WageReportsToIRTypes Transaction. Also contains additional data for accounting and integration purposes.",
    			accounting: {
    				title: "accounting",
    				descr: "Accounting related data for the row."
    			},
    			calcData: {
    				title: "calc data",
    				descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR)."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row. Default is one."
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			earningPeriod: {
    				title: "earning period",
    				descr: "Earnings period if differnet than the report period.\nNote that this property may be (and by default is) null."
    			},
    			irData: {
    				title: "ir data",
    				descr: "Details for the National Incomes registry"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row."
    			},
    			total: {
    				title: "total",
    				descr: "Simple multiplication: Count * Price - no other logic. Not used as Input."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the row - basically describes the source for this row:\nEither Manual (default) or Usecase logic, which may be legacy / API v02 or new / API v03."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. If specified as Undefined, set by the server-logic based on row type etc."
    			}
    		},
    		IrRowCalculationProperties: {
    			title: "ir row calculation properties",
    			descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR).",
    			behavior: {
    				title: "behavior",
    				descr: "Defines the behavior of the row in Salaxy calculation process."
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Grouping for reports etc."
    			}
    		},
    		JobProfile: {
    			title: "job profile",
    			backgroundImage: {
    				title: "background image",
    				descr: "Background image for the profile page"
    			},
    			cities: {
    				title: "cities",
    				descr: "A collection of known city IDs.\nQuery the interface for city IDs"
    			},
    			description: {
    				title: "description",
    				descr: "Long description: This field accepts MarkDown, potentially also HTML in the future (at the moment HTML may create an error)."
    			},
    			jobDetail: {
    				title: "job detail",
    				descr: "Detailed description of the Job"
    			},
    			jobTitle: {
    				title: "job title",
    				descr: "Title for the Job"
    			},
    			jobs: {
    				title: "jobs",
    				descr: "A collection of job ID's that this work or worker belongs"
    			},
    			links: {
    				title: "links",
    				descr: "External links to web sites, social media pages etc."
    			},
    			location: {
    				title: "location",
    				descr: "Free text for location of work or worker"
    			},
    			salaryMax: {
    				title: "salary max",
    				descr: "Max salary for the given salary period"
    			},
    			salaryMin: {
    				title: "salary min",
    				descr: "Minimum salary for the given salary period"
    			},
    			salaryPeriod: {
    				title: "salary period",
    				descr: "Period for which the Salary period is set."
    			}
    		},
    		LedgerAccount: {
    			title: "ledger account",
    			descr: "Represents a single accounting ledger.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			entries: {
    				title: "entries",
    				descr: "Legder entries"
    			},
    			saldo: {
    				title: "saldo",
    				descr: "Current saldo for the ledger."
    			}
    		},
    		LedgerEntry: {
    			title: "ledger entry",
    			descr: "Represents a single ledger entry.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			amount: {
    				title: "amount",
    				descr: "Posted amount."
    			},
    			description: {
    				title: "description",
    				descr: "Entry description."
    			},
    			dimension: {
    				title: "dimension",
    				descr: "Dimension data."
    			},
    			entryDate: {
    				title: "entry date",
    				descr: "Entry date."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the entry."
    			},
    			isDebit: {
    				title: "is debit",
    				descr: "Debit indicator."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "VAT percent."
    			}
    		},
    		"Map": {
    			title: "map",
    			descr: "Map between a hierarchical Salaxy object and a row in a table/dataset."
    		},
    		MapSelector: {
    			title: {
    				title: "title",
    				descr: "Title of the property. If null, path is used with reflection."
    			},
    			descr: {
    				title: "descr",
    				descr: "Description of the property. If null, path is used with reflection."
    			},
    			fromObject: {
    				title: "from object",
    				descr: "Function that defines how the row value(s) is derived from the object, typically in export.\nIf null, this is defined by path."
    			},
    			fromRow: {
    				title: "from row",
    				descr: "Function that sets the value from row back to object, typically in import.\nIf null, this is defined by path."
    			},
    			path: {
    				title: "path",
    				descr: "Defines source property as a simple object path if possible.\nIf a more complex logic is needed, you can define get and set functions below.\nThe property is also used in generating documentation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the property. If null, path is used with reflection."
    			}
    		},
    		MarketingSettings: {
    			title: "marketing settings",
    			descr: "Marketing related settings which may affect the pricing. E.g. campaigns, customer groups etc.",
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing."
    			}
    		},
    		MealBenefitUsecase: {
    			title: "meal benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.MealBenefit to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isTaxValue: {
    				title: "is tax value",
    				descr: "If true, the deduction from Worker salary corresponds to taxable value."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			taxablePrice: {
    				title: "taxable price",
    				descr: "The taxable price calculated by the server."
    			}
    		},
    		Measurement: {
    			title: "measurement",
    			descr: "Measurement value.",
    			tag: {
    				title: "tag",
    				descr: "Tag for the measurement."
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Time when the measurement was recorded."
    			},
    			value: {
    				title: "value",
    				descr: "Actual measurement value."
    			}
    		},
    		Message: {
    			title: "message",
    			descr: "A message within a messaging thread.",
    			content: {
    				title: "content",
    				descr: "Textual content: If message, this is the content text.\nIf other type, short title / description typically shown in the title bar."
    			},
    			contentData: {
    				title: "content data",
    				descr: "Other misc. data that is used in displaying the content in the immediate message view.\nDo not include the entire object here. Just the immediately available data."
    			},
    			contentId: {
    				title: "content id",
    				descr: "Identifier of the external content.\nBlobFile Id for attachments, Calculation ID for calculations etc."
    			},
    			fromParty: {
    				title: "from party",
    				descr: "The party / type of the message sender."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the message."
    			},
    			preview: {
    				title: "preview",
    				descr: "Content type specific text for preview.\nThis may be URI, ID or boolean.\nFor BlobFile, it is currently the BlobFile Guid (the same as ContentId) or null,\nbut in the future, the field may also contain an URL without that being a breaking change!"
    			},
    			readByOtherParty: {
    				title: "read by other party",
    				descr: "If set, specifies the time when the message has been read by the other party."
    			},
    			readByOwner: {
    				title: "read by owner",
    				descr: "If set, specifies the time when the message has been read by the thread owner."
    			},
    			time: {
    				title: "time",
    				descr: "Time for the message"
    			},
    			type: {
    				title: "type",
    				descr: "Content type of the message"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validates the object that is the payload of the message."
    			}
    		},
    		MessageData: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Message thread specific business data for index.",
    			otherPartyLastMessage: {
    				title: "other party last message",
    				descr: "Last message that was created by the other party"
    			},
    			otherPartyUnreadCount: {
    				title: "other party unread count",
    				descr: "Number of messages that have been read by the Other party and have not been read by the owner."
    			},
    			ownerLastMessage: {
    				title: "owner last message",
    				descr: "Last message that was was created by the owner"
    			},
    			ownerUnreadCount: {
    				title: "owner unread count",
    				descr: "Number of messages by Owner that have not been read by the other party"
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			}
    		},
    		MessageListItem: {
    			title: "message list item",
    			descr: "Represents a single item in a list of message threads in the user interface. It contains header data of the message thread and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		MessageThread: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Messages thread groups togehter a set of related messages and/or attachments.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			messages: {
    				title: "messages",
    				descr: "List of messages and attachments in this thread"
    			},
    			otherParty: {
    				title: "other party",
    				descr: "Palkkaus.fi id (IBAN) of the other party.\nThe other party is the second entity(besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party is the second entity (besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyWorkflow: {
    				title: "other party workflow",
    				descr: "The workflow from the other party point-of-view."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "The owner of the thread: The entity that has the right to delete or move the thread.\nThis may or may not be the originator the message thread.\nThe owner is typically Company or Household, in some cases Worker person.\nIt should NOT be partner or person from a company."
    			},
    			ownerWorkflow: {
    				title: "owner workflow",
    				descr: "The workflow from the Owner point-of-view."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		MessageThreadWorkflow: {
    			title: "message thread workflow",
    			descr: "Defines workflow actions for a message thread.",
    			lastMessageText: {
    				title: "last message text",
    				descr: "Text for the last message (written by this party). This is for preview purposes: May be shortened."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The salary date for the discussion.\nOwner's and other party's are typically the same, but they may differ:\ne.g. one may be the requested / estimated date and the other the first possible date if the requested is unrealistic."
    			},
    			unreadCount: {
    				title: "unread count",
    				descr: "Number of messages that this party has posted that have not been read by the other party."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Time of the last message posted by this party."
    			}
    		},
    		MonthlyHolidayAccrual: {
    			title: "monthly holiday accrual",
    			descr: "Holiday Accrual for a calendar month in releation to a calculation.",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of absence days used in the calculation."
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Holiday accrual for this month.\nNote that -1 is indication that accrual should not be updated for this particylar month.\nThere is no possiblity for negative accrual."
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			workDays: {
    				title: "work days",
    				descr: "Work days in determining the accrual for the month in 14 days rule."
    			},
    			workHours: {
    				title: "work hours",
    				descr: "Work hours in determining the accrual fot the month in 35 hour rule."
    			}
    		},
    		NonProfitOrgUsecase: {
    			title: "non profit org usecase",
    			descr: "Handles the Calculation of CalculationRowType.NonProfitOrg to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		OAuthMessage: {
    			title: "o auth message",
    			descr: "Based on standard https://tools.ietf.org/html/rfc6749 .",
    			access_token: {
    				title: "access_token",
    				descr: "The access token issued by the authorization server."
    			},
    			assertion: {
    				title: "assertion",
    				descr: "The value of the \"assertion\" parameter MUST contain a single JWT."
    			},
    			client_id: {
    				title: "client_id",
    				descr: "The client identifier issued to the client during the registration process."
    			},
    			client_secret: {
    				title: "client_secret",
    				descr: "The client secret. The client MAY omit the parameter if the client secret is an empty string."
    			},
    			code: {
    				title: "code",
    				descr: "The authorization code received from the authorization server."
    			},
    			connection: {
    				title: "connection",
    				descr: "Auth0 connection"
    			},
    			error: {
    				title: "error",
    				descr: "A single ASCII [USASCII] error code."
    			},
    			error_description: {
    				title: "error_description",
    				descr: "Human-readable ASCII [USASCII] text providing additional information, used to assist the client developer in understanding the error that occurred."
    			},
    			error_uri: {
    				title: "error_uri",
    				descr: "A URI identifying a human-readable web page with information about the error, used to provide the client developer with additional information about the error."
    			},
    			expires_in: {
    				title: "expires_in",
    				descr: "The lifetime in seconds of the access token.  For example, the value \"3600\" denotes that the access token will expire in one hour from the time the response was generated. If omitted, the authorization server SHOULD provide the expiration time via other means or document the default value."
    			},
    			grant_type: {
    				title: "grant_type",
    				descr: "Requested grant type."
    			},
    			id_token: {
    				title: "id_token",
    				descr: "The id token issued by the authorization server."
    			},
    			password: {
    				title: "password",
    				descr: "The resource owner password."
    			},
    			redirect_uri: {
    				title: "redirect_uri",
    				descr: "After completing its interaction with the resource owner, the authorization server directs the resource owner's user-agent back to the client."
    			},
    			refresh_token: {
    				title: "refresh_token",
    				descr: "The refresh token, which can be used to obtain new access tokens."
    			},
    			response_type: {
    				title: "response_type",
    				descr: "The value MUST be one of \"code\" for requesting an authorization code, \"token\" for  requesting an access token(implicit grant), or a registered extension value."
    			},
    			salaxy_language: {
    				title: "salaxy_language",
    				descr: "User interface language."
    			},
    			salaxy_onboarding_uri: {
    				title: "salaxy_onboarding_uri",
    				descr: "Salaxy specific onboarding uri."
    			},
    			salaxy_role: {
    				title: "salaxy_role",
    				descr: "Users role."
    			},
    			scope: {
    				title: "scope",
    				descr: "The value of the scope parameter is expressed as a list of space-delimited, case-sensitive strings.The strings are defined by the authorization server."
    			},
    			state: {
    				title: "state",
    				descr: "An opaque value used by the client to maintain state between the request and callback.The authorization server includes this value when redirecting the user-agent back to the client. The parameter SHOULD be used for preventing cross-site request forgery."
    			},
    			token_type: {
    				title: "token_type",
    				descr: "The type of the token issued."
    			},
    			username: {
    				title: "username",
    				descr: "The resource owner username."
    			}
    		},
    		ODataQueryOptions: {
    			title: "o data query options",
    			descr: "Defines the query options of OData",
    			$count: {
    				title: "$count",
    				descr: "Tells the server to include the total count of matching entities in the response. (Useful for server-side paging.)"
    			},
    			$expand: {
    				title: "$expand",
    				descr: "Expands related entities inline. At the moment, this typically not implemented."
    			},
    			$filter: {
    				title: "$filter",
    				descr: "Filters the results, based on a Boolean condition."
    			},
    			$orderby: {
    				title: "$orderby",
    				descr: "Sorts the results."
    			},
    			$search: {
    				title: "$search",
    				descr: "Performs free-text search using the in-built logic in the API."
    			},
    			$select: {
    				title: "$select",
    				descr: "Selects which properties to include in the response."
    			},
    			$skip: {
    				title: "$skip",
    				descr: "Skips the first n results."
    			},
    			$top: {
    				title: "$top",
    				descr: "Returns only the first n the results."
    			},
    			anon: {
    				title: "anon",
    				descr: "If true, perfoms the query also in anonymous mode\n(no token, token expired or invalid token).\nDefault is that the query is done only if user has a valid token."
    			}
    		},
    		ODataResult: {
    			title: "o data result",
    			descr: "Result from OData query",
    			count: {
    				title: "count",
    				descr: "Total count of the items (server-side/storage) if updated in the previous query.\nNote that this may be null if the count is not fetched."
    			},
    			items: {
    				title: "items",
    				descr: "Actual results array"
    			},
    			nextPageLink: {
    				title: "next page link",
    				descr: "Next page link"
    			}
    		},
    		Occupation: {
    			title: "occupation",
    			descr: "Occupation according to Statistics Finland.\nUsed for insurance pricing and reporting.",
    			code: {
    				title: "code",
    				descr: "Statistics Finland code. There may be duplicates with different id."
    			},
    			freeText: {
    				title: "free text",
    				descr: "Free text\nTODO: Do we need to expose this? Why?"
    			},
    			id: {
    				title: "id",
    				descr: "Salaxy identifier for the Occupation. Begins with code, but may have a suffix."
    			},
    			keyword: {
    				title: "keyword",
    				descr: "Keywords for searching etc."
    			},
    			label: {
    				title: "label",
    				descr: "Label"
    			},
    			lang: {
    				title: "lang",
    				descr: "Language for the entry"
    			},
    			search: {
    				title: "search",
    				descr: "Lower case text that contains the id, label and potentially other text that should match in searches."
    			}
    		},
    		Onboarding: {
    			title: "onboarding",
    			descr: "Provides the data for onboarding process for a company. Onboarding is the process where Salaxy gathers sufficient information to at least create an acccount, often even more so that the user can pay a salaary with the account.",
    			accountType: {
    				title: "account type",
    				descr: "Type of the account to be created."
    			},
    			appState: {
    				title: "app state",
    				descr: "Custom state object that is guaranteed to survive the round-trip in the onboarding process.\nCalling application to the onboarding may pass here any data that it needs after the onboarding process is ready and user has been authenticated."
    			},
    			company: {
    				title: "company",
    				descr: "Input that is needed for creating a company account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			credentials: {
    				title: "credentials"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			person: {
    				title: "person",
    				descr: "Input that is needed for creating a personal account.\nThis may be\n1) the principal account if the account is created for 1a) Worker or 1b) Household or\n2) creator owner if the account is created for a Company."
    			},
    			products: {
    				title: "products",
    				descr: "Possibility to define products.\nThese can only be defined to one account as part of the onboarding process:\nIf company account is created, the products are always for Company, not for the Person =&gt;\nPerson will have default products in that case and they need to be edited in a separate process once the accounts have been created."
    			},
    			signature: {
    				title: "signature",
    				descr: "Input and post data for digital signature."
    			},
    			status: {
    				title: "status",
    				descr: "The server-side status of the onboarding process.\nNOTE that there is a more relaxed user interface-level state in ui.state and ui.stateFullText\nthat can be used in onboarding user interfaces."
    			},
    			ui: {
    				title: "ui",
    				descr: "User interface related data: Mainly the state of the UI."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation status and errors."
    			}
    		},
    		OnboardingContactVerification: {
    			title: "onboarding contact verification",
    			descr: "Contact verification object for single message type.",
    			pin: {
    				title: "pin",
    				descr: "Given pin code."
    			}
    		},
    		OnboardingContactVerifications: {
    			title: "onboarding contact verifications",
    			descr: "Contact verification object for onboarding.",
    			email: {
    				title: "email",
    				descr: "Verification for email contact."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Verification for telephone contact."
    			}
    		},
    		OnboardingCredentials: {
    			title: "onboarding credentials",
    			descr: "Stores information about the credentials that were used in the Onboarding process.",
    			credentialId: {
    				title: "credential id",
    				descr: "Identifier of the credentials (UserAccount Guid in the database)."
    			},
    			partnerId: {
    				title: "partner id",
    				descr: "The partner that initiated the Onboarding process."
    			}
    		},
    		OnboardingUIState: {
    			title: "onboarding u i state",
    			descr: "Stores data for the onboarding user interface.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "URL where the user should be redirected back to if the process is canceled or there is an urecoverable error."
    			},
    			invitationKey: {
    				title: "invitation key",
    				descr: "Salaxy worker invitation key."
    			},
    			oAuthResponseType: {
    				title: "o auth response type",
    				descr: "Response type to OAuth -request"
    			},
    			state: {
    				title: "state",
    				descr: "State of the user interface. Typically an enumeration that stores where the UI was. E.g. Wizard step."
    			},
    			stateFullText: {
    				title: "state full text",
    				descr: "Full description of the user interface state for admin and analyses purposes."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "URL where the user should be redirected back to when the onboarding process is complete."
    			}
    		},
    		OtherCompensationUsecase: {
    			title: "other compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.OtherCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Ownership: {
    			title: "ownership",
    			isRequiredTracking: {
    				title: "is required tracking",
    				descr: "If true, authorities require tracking the shareholders.\nIn Finland, with current legistlation, this is if a shareholder has over 25% of the shares."
    			},
    			shareHolders: {
    				title: "share holders",
    				descr: "List of shareholders for the company"
    			}
    		},
    		PageResultOfApiListItem: {
    			title: "page result of api list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfArticle: {
    			title: "page result of article",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfCalculationListItem: {
    			title: "page result of calculation list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfEmploymentListItem: {
    			title: "page result of employment list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfFileListItem: {
    			title: "page result of file list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfInvoiceListItem: {
    			title: "page result of invoice list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfMessageListItem: {
    			title: "page result of message list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfPayrollListItem: {
    			title: "page result of payroll list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfSampleListItem: {
    			title: "page result of sample list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfTaxcardListItem: {
    			title: "page result of taxcard list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfWorkerListItem: {
    			title: "page result of worker list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PartnerAccounting: {
    			title: "partner accounting",
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, partner settings override company settings."
    			},
    			targetIds: {
    				title: "target ids",
    				descr: "Available accounting targets."
    			}
    		},
    		PartnerBranding: {
    			title: "partner branding",
    			descr: "Partner branding settings: logo, css etc.",
    			css: {
    				title: "css",
    				descr: "Css path for the primary partner."
    			},
    			logo: {
    				title: "logo",
    				descr: "Url to partner logo."
    			},
    			text: {
    				title: "text",
    				descr: "Long marketing text."
    			},
    			url: {
    				title: "url",
    				descr: "Marketing url."
    			}
    		},
    		PartnerCompanyAccountInfo: {
    			title: "partner company account info",
    			descr: "Information about a Customer account for a Partner",
    			accountId: {
    				title: "account id",
    				descr: "Account Id"
    			},
    			accountIdFormatted: {
    				title: "account id formatted",
    				descr: "Account Id fomratted for dispaly to end user"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for display information"
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If true, the current user (the partner) is authorized to access the account"
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has a verified owner."
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The partner account ID to which the access is currently restricted."
    			}
    		},
    		PartnerMessaging: {
    			title: "partner messaging",
    			descr: "Messaging settings for the partner.",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the messaging is switched on for this partner."
    			}
    		},
    		PartnerService: {
    			title: "partner service",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the calculation data delivery by forms is switched on for this partner."
    			}
    		},
    		PartnerWebSite: {
    			title: "partner web site",
    			descr: "Stores information about the web site that is used for customer (employer) to access the customer web site.",
    			url: {
    				title: "url",
    				descr: "The main URL that is displayed.\nDefault is https://www.palkkaus.fi/company"
    			}
    		},
    		Payer: {
    			title: "payer",
    			descr: "The paying party",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		Payment: {
    			title: "payment",
    			descr: "Payment object requested to be paid for completing the underlying business objects.",
    			amount: {
    				title: "amount",
    				descr: "Amount requested to be paid."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business objects (calculations etc.) on which the payment is based on."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payment."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			message: {
    				title: "message",
    				descr: "Message for the payment."
    			},
    			method: {
    				title: "method",
    				descr: "Payment method."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer as it would be on the invoice"
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Recipient as it would be on the invoice"
    			},
    			reference: {
    				title: "reference",
    				descr: "Bank reference number/string. Will be generated by the system automatically."
    			},
    			request: {
    				title: "request",
    				descr: "Request specific payment parameters."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the payment."
    			},
    			statusText: {
    				title: "status text",
    				descr: "Status text giving more information about the status."
    			},
    			type: {
    				title: "type",
    				descr: "Payment type."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PaymentRequestParameters: {
    			title: "payment request parameters",
    			descr: "Contains request specific parameters for payment.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "Return url for failed / cancelled payment."
    			},
    			paymentStartedUrl: {
    				title: "payment started url",
    				descr: "If this URL is set, the payment allows aynchronous payment methods\nsuch as payment with invoice (viitelasku).\nThis means that the payment UI returns in status InProgress and\npayment is finalized later (may take hours, days or even be canceled)."
    			},
    			sendEmails: {
    				title: "send emails",
    				descr: "Indicates whether to send emails according to workflow after payment.\nTODO: This should be retrieved from the partner profile etc."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "Return url for successful payment."
    			}
    		},
    		PaymentSettings: {
    			title: "payment settings",
    			descr: "Payment related settings",
    			customerFunds: {
    				title: "customer funds",
    				descr: "Settings related to payment methods that use the customer funds account (gross payment)."
    			},
    			invoice: {
    				title: "invoice",
    				descr: "Settings related to invoices: Either salary payment or other invoicing."
    			},
    			pricing: {
    				title: "pricing",
    				descr: "The pricing partner (accounant or software) that defines the pricing of the service.\nThis data is typically read-only in normal settings view."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		Payroll03Info: {
    			title: "payroll03 info",
    			descr: "Provides Read-only information about the Payroll In the case of non-paid Payrolls, this is the latest save/validation. In the case of paid Payrolls, this is the status of the Payroll when it was paid.",
    			calcCount: {
    				title: "calc count",
    				descr: "Count of calculations in this Payroll"
    			},
    			date: {
    				title: "date",
    				descr: "Date that the Info was last updated."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payroll. Date when the payment should be made to achieve SalaryDate."
    			},
    			fees: {
    				title: "fees",
    				descr: "Total Palkkaus.fi fees for the Payroll"
    			},
    			isReadyForPayment: {
    				title: "is ready for payment",
    				descr: "If true, all calculations have been validated and they are valid for payment.\nIf false, validation has not been done or some calculations are not valid for payment."
    			},
    			paymentDate: {
    				title: "payment date",
    				descr: "Paid at date for the payroll. (Date of the employer's payment)."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "The reference number of the payment after payment is made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			status: {
    				title: "status",
    				descr: "Status for the payroll and its calculations in the payment Workflow."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment for the Payroll."
    			}
    		},
    		PayrollDetails: {
    			title: "payroll details",
    			descr: "New version 03 of Payroll. Payroll (Palkkalista) is a list of employees who receive salary or wages from a particular organization. Typical usecase is that a a company has e.g. a monthly salary list that is paid at the end of month. For next month, a copy is then made from the latest list and the copy is potentially modified with the changes of that particular month.",
    			calcs: {
    				title: "calcs",
    				descr: "Snapshots of the calculations based on the ID's specified in Info"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing Payroll lists.\nIn practice, a usecase is either a user interface or a microservice that provides a Payroll processing\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of the Payroll."
    			},
    			validations: {
    				title: "validations",
    				descr: "When saved or validated, provides a validation object for each calculation:\nWhether the calculations are ready for payment."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		PayrollInput: {
    			title: {
    				title: "title",
    				descr: "Title that can be specified by the end user."
    			},
    			descr: "Input that user defines about the Payroll",
    			calculations: {
    				title: "calculations",
    				descr: "List of calculations that are attached to this calculation."
    			},
    			noUpdateFromEmployment: {
    				title: "no update from employment",
    				descr: "If true, the calculation rows are not updated from employment when the Payroll is copied.\nDefault is that the rows and other data is updated in copy / create new etc."
    			},
    			period: {
    				title: "period",
    				descr: "The Default Salary period for the Payroll.\nThis may be overriden for an individual calculation."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "What types of wages are paid with this payroll.\nMay affect the rendering of the user interface (e.g. for     holidays, hours and absences).\nAlso affects which Workers are show in the add / link dialog by default."
    			}
    		},
    		PayrollListItem: {
    			title: "payroll list item",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PensionEntrepreneurProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by TempContractMarker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			tempContractMarker: {
    				title: "temp contract marker",
    				descr: "If true, user wants to order a YEL-contract."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Pension for Workers: The payments and monthly reporting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPensionSelfHandling."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isPendingContract: {
    				title: "is pending contract",
    				descr: "If true, there is a new pending contract."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The pension company where the user has a contract."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Number for the contract."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionSettings: {
    			title: "pension settings",
    			company: {
    				title: "company",
    				descr: "The pension company where the user has a contract."
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Number for the contract."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the pension insurance."
    			}
    		},
    		PeriodRecalculationData: {
    			title: "period recalculation data",
    			descr: "Data that is needed for determening PeriodRecalculationParams and thus at the end recalculating holidays and absences to a calculation.",
    			absences: {
    				title: "absences",
    				descr: "Absences that should be deducted from monthly salary.\nIf not provided, absences are not calculated."
    			},
    			calc: {
    				title: "calc",
    				descr: "The salary calculation that should be recalculated."
    			},
    			holidayYear: {
    				title: "holiday year",
    				descr: "Holiday year with holidays that should be added as salary rows.\nIf not provided, holidays are not calculated."
    			}
    		},
    		PersonAccount: {
    			title: "person account",
    			descr: "An account for a Person - can be both Employer and Worker",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			contactVerificationPin: {
    				title: "contact verification pin",
    				descr: "Contact verification pin for securing contact address changes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerInfo: {
    				title: "employer info",
    				descr: "The employer information in case the person is an employer"
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			lastLoginAs: {
    				title: "last login as",
    				descr: "Type of the last login - the user role"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "The worker information in case the person is a Worker"
    			}
    		},
    		PersonInput: {
    			title: "person input",
    			bankAccountIban: {
    				title: "bank account iban",
    				descr: "Bank account number for salary payment"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the user / personal account"
    			},
    			contactVerification: {
    				title: "contact verification",
    				descr: "Verification objects for contact verification."
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a personal account based on this data.\nOtherwise this is just user data for Company."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Personal ID (Henkilötunnus /HETU)"
    			},
    			picture: {
    				title: "picture",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given PersonalId exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			}
    		},
    		PricingPartner: {
    			title: "pricing partner",
    			descr: "Defines the partner, service and pricing model for the customer.",
    			accountId: {
    				title: "account id",
    				descr: "Partner account id."
    			},
    			fixedFee: {
    				title: "fixed fee",
    				descr: "Fee for one calculation, which is visible and paid by customer.\nThis is not relevant if the PricingModel is PalkkausFee"
    			},
    			pricingModel: {
    				title: "pricing model",
    				descr: "Pricing model by the partner."
    			},
    			serviceModel: {
    				title: "service model",
    				descr: "Service model by the partner."
    			}
    		},
    		PricingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Defines the pricing partner, service model and pricing model.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically if partner set and partner specific pricing model."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			pricingPartner: {
    				title: "pricing partner",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of pricing."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PrimaryPartner: {
    			title: "primary partner",
    			descr: "Primary partner settings.",
    			accounting: {
    				title: "accounting"
    			},
    			branding: {
    				title: "branding"
    			},
    			description: {
    				title: "description",
    				descr: "Description of the primary account."
    			},
    			id: {
    				title: "id",
    				descr: "Primary account id."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "Boolean indicating if this partner has been authorized to act on behalf the current account."
    			},
    			isPrimary: {
    				title: "is primary",
    				descr: "Boolean indicating if this partner is the primary partner of the current account."
    			},
    			key: {
    				title: "key",
    				descr: "Human-readable, human decided, recognizable short unique key for the partner:\nAll lower-case, a-z, 0-9, \"-\" and \"_\". Used in URL's etc."
    			},
    			messaging: {
    				title: "messaging"
    			},
    			name: {
    				title: "name",
    				descr: "Display name for the account."
    			},
    			service: {
    				title: "service"
    			},
    			site: {
    				title: "site"
    			}
    		},
    		Product: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Base inteface for Products.\nThe common properties for all Products: BaseSalaryProduct, TaxProduct, UnemploymentProduct etc.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user. Restrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product.\nE.g. Raksaloki is available for households only and this is a Company account =&gt; Should not show the product."
    			}
    		},
    		Profile: {
    			title: "profile",
    			descr: "Profile object for either Worker or Employer",
    			accountId: {
    				title: "account id",
    				descr: "Account if known"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "The avatar object: Picture name and other such basic info."
    			},
    			contact: {
    				title: "contact",
    				descr: "Other contact information for this profile."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "The legal entity type of the profile: Person or Company"
    			},
    			expiresAt: {
    				title: "expires at",
    				descr: "Expiration date for the profile will be unpublished."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id (e.g. member number)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isPublic: {
    				title: "is public",
    				descr: "If true, the profile is set to public."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			job: {
    				title: "job"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			sortableName: {
    				title: "sortable name"
    			},
    			source: {
    				title: "source",
    				descr: "The source of this profile: Palkkaus section or partner site.\nThis is a strongly typed object that may be undefined - see SourceId in that case."
    			},
    			sourceId: {
    				title: "source id",
    				descr: "The identifier string of the source. This will always return a distinct value."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			userUpdatedAt: {
    				title: "user updated at",
    				descr: "Last time user updated the profile"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		ProfileJob: {
    			title: "profile job",
    			descr: "A job for a Public Profile (Osaajaprofiili) or later also for Job ads.",
    			category: {
    				title: "category"
    			},
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Job"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			picture: {
    				title: "picture"
    			},
    			synonyms: {
    				title: "synonyms"
    			}
    		},
    		ProfileJobCategory: {
    			title: "profile job category",
    			descr: "Category of Profile Jobs",
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Category"
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID of the Category"
    			},
    			name: {
    				title: "name",
    				descr: "Name / title of the Category"
    			},
    			picture: {
    				title: "picture",
    				descr: "Backround picture for the Category"
    			}
    		},
    		Recipient: {
    			title: "recipient",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		RemunerationUsecase: {
    			title: "remuneration usecase",
    			descr: "Handles the Calculation of CalculationRowType.Remuneration to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Report: {
    			title: {
    				title: "title",
    				descr: "Short title for the report that can be shown to the end user."
    			},
    			descr: "Describes a logical generated report. This is a report instance e.g. a monthly report for January 2017. It may contain several files of different types: PDF, preview PNG, HTML etc.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			htmlUrl: {
    				title: "html url",
    				descr: "HTML for the report. Use salaxy-rpt.css for formatting.\nFor preview and demo, this is the format to show to the user - PDF is not generated."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isDemoData: {
    				title: "is demo data",
    				descr: "If true, the report is created using demo data - should be used only in the beginning when the user has not paid any salaries.\nThis may be e.g. calculations in draft stage or completely invented demo data such as example workers.\nDemo data should not be confused to test database (test-api.palkkaus.fi / test-api.salaxy.com):\nPaid calculations in test database are production data."
    			},
    			isPreview: {
    				title: "is preview",
    				descr: "If true, the report is in preview state, typically in the middle of the month.\nThis means that the data itself is production, its just not final."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			jsonUrl: {
    				title: "json url",
    				descr: "Some reports are also available in JSON format, from which you can create lists, CSV, Excel files etc."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pdfUrl: {
    				title: "pdf url",
    				descr: "PDF report for viewing and printing. This is typically the main report.\nFor preview and demo, there is typically only HTML version - no PDF."
    			},
    			reportDate: {
    				title: "report date",
    				descr: "Report date: For monthly and yearly reports this is the first day of the year/month.\nFor ad-hoc reports, this is the start date of the period."
    			},
    			thumbnail: {
    				title: "thumbnail",
    				descr: "Thumbnail image for the report.\nFor preview and demo data this is typically a fixed image - the thumbnail is only generated when PDF is generated."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the report"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			urlsExpireAt: {
    				title: "urls expire at",
    				descr: "Time when the PDF, HTML or JSON URL's expire IF they are generated in a way that\nthey are callable without authentication for a specified time period.\nThis property is null if the URL requires authentication."
    			}
    		},
    		ReportingArea: {
    			title: "reporting area",
    			bottom: {
    				title: "bottom",
    				descr: "Bottom margin."
    			},
    			left: {
    				title: "left",
    				descr: "Left margin."
    			},
    			right: {
    				title: "right",
    				descr: "Right margin."
    			},
    			top: {
    				title: "top",
    				descr: "Top margin."
    			}
    		},
    		ReportingSettings: {
    			title: "reporting settings",
    			descr: "Settings related to reporting",
    			css: {
    				title: "css",
    				descr: "Additional CSS that is used to customize the layout of the report."
    			},
    			footerTemplate: {
    				title: "footer template",
    				descr: "HTML template for the print footer.\nUse the following handlebars tags to inject values into the footer:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			headerTemplate: {
    				title: "header template",
    				descr: "HTML template for the print header.\nUse the following handlebars tags to inject values into the header:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			img2: {
    				title: "img2",
    				descr: "Secondary image that can be used in header / footer as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			logo: {
    				title: "logo",
    				descr: "Main logo image as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			margins: {
    				title: "margins",
    				descr: "Defines the margins of the content area"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		ResultRow: {
    			title: "result row",
    			descr: "Defines a Calculation row in the calculation results",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowSource: {
    				title: "row source",
    				descr: "Source of the calculation row - affects how the row is handled in editing / recalculation scenarios."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is kept as-is in the Salaxy system."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the row - always Price * Count"
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "Base salary (Peruspalkka) is the monthly salary or total salary without addiotions (overtime, holidays etc.)"
    			},
    			totalDeduction: {
    				title: "total deduction",
    				descr: "Deductions from the salary: Union payment, Advance, Foreclosure"
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Taxfree expenses compensation"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary (Bruttopalkka) is \"real\" salary including the additions (overtime, holidays etc.) but not benefits."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "Social Security Base (Sosiaaliturvan alainen ansio) is typically the same as TotalTaxable,\nbut e.g. child care subsidy (yksityisen hoidon tuki) is handled differently."
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable income (Ennakonpidätyksen alainen ansio) is the salary from taxation point-of-view. It is basically the Gross Salary plus benefits."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row"
    			},
    			userRowIndex: {
    				title: "user row index",
    				descr: "Row index that matches this row to a user defined row (which is zero-based).\nIf the row is generated by calculation business logic (e.g. based on framework agreement), the index is -1."
    			}
    		},
    		RootDeclarationReflection: {
    			title: "root declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			}
    		},
    		RowAccountingData: {
    			title: "row accounting data",
    			descr: "Provides row specific data for accounting.",
    			accountNumberFlags: {
    				title: "account number flags",
    				descr: "Special flags for account number mapping."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percent for VAT calculation expressed as decimal: 24% is 0.24.\nThe price in the root level is assumed being with tax."
    			}
    		},
    		Salary: {
    			title: "salary",
    			descr: "Defines the main salary that is the bases for the calculation",
    			amount: {
    				title: "amount",
    				descr: "Amount of e.g. hours, months for the salary. Default is 1."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If set to true, will calculate the household deduction. Applies only to natural persons - as oppsed to companies."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the salary being calculated"
    			},
    			message: {
    				title: "message",
    				descr: "Optional message for the salary row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price of one unit - e.g. hour, month."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			unit: {
    				title: "unit",
    				descr: "E.g. hour, month. Typically this parameter can be left empty/null and the unit is set according to type."
    			}
    		},
    		SampleApiObject: {
    			title: "sample api object",
    			descr: "Sample object for creating v03 controller",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SampleData: {
    			title: "sample data",
    			descr: "Sample custom data in search index",
    			test: {
    				title: "test",
    				descr: "A test string"
    			}
    		},
    		SampleListItem: {
    			title: "sample list item",
    			descr: "Available Search index data of the Sample object",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		SessionUserCredential: {
    			title: "session user credential",
    			descr: "Represents a user credential in session: This credential is typically based on Claims and is availble without going to database.",
    			authenticationMethod: {
    				title: "authentication method",
    				descr: "The ultimate authentication provider that authenticated the user, e.g. Facebook, Google."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object that contains the visual reresentation of the User account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail address if provided by the authentication provider."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identitier for the user ID coming from the Auth0 database:\n'[MethodText]|[ProviderId]', e.g. 'facebook|12345678901234567'\n(Number may contain more characters - not sure for the exact cahracters count)"
    			}
    		},
    		SharedTaxcardExistsResult: {
    			title: "shared taxcard exists result",
    			descr: "Provides a a result to query of whether a taxcard exists for a personal ID (no personal information).",
    			isError: {
    				title: "is error",
    				descr: "If true there was an error fetching the information.\nTypically, this would be that there were too many attempts to fetch Personal ID's and request was classified as phishing.\nIn this case, you should currently create the employment relation and fetch the taxcard infor after that.\nWe may later introduce a \"Are you robot\" type of interface to go around this (if necessary)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard. Undefined if none is found."
    			},
    			ownedExists: {
    				title: "owned exists",
    				descr: "Returns true if there is already an owned taxcard for the employer."
    			},
    			ownedTaxcard: {
    				title: "owned taxcard",
    				descr: "If Owned taxcard exists, it is returned here."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID that was used in fetching.\nIt may be validated and formatted, or just the original string if there were issues."
    			},
    			sharedExists: {
    				title: "shared exists",
    				descr: "Returns true if a shared taxcard exists."
    			},
    			uri: {
    				title: "uri",
    				descr: "Uri for the Shared taxcard"
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity of the taxcard. Note that this property is null when Exists is false."
    			}
    		},
    		Shareholder: {
    			title: "shareholder",
    			descr: "Defines a shareholder. At the moment, this is only a name and it is used only if the person has more than 25% of the shares. Expect this object to be developed with, Ownership percent or number of shares, Id or other such link to the actual person etc.",
    			name: {
    				title: "name",
    				descr: "Name of the shareholder."
    			}
    		},
    		SignatureData: {
    			title: "signature data",
    			digitalSignature: {
    				title: "digital signature"
    			},
    			email: {
    				title: "email"
    			},
    			isPep: {
    				title: "is pep"
    			},
    			isProcura: {
    				title: "is procura",
    				descr: "The account is authorized to sign on behalf of the actual account."
    			},
    			personName: {
    				title: "person name",
    				descr: "Name of the signing person.\nBy default, this is Onboarding Person, but it may be a completely different person."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID of the person, who has the right to sign."
    			},
    			telephone: {
    				title: "telephone"
    			}
    		},
    		SmsMessage: {
    			title: "sms message",
    			descr: "API SmsMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (characters or numbers)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (phone)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SubsidisedCommuteUsecase: {
    			title: "subsidised commute usecase",
    			descr: "Handles the Calculation of CalculationRowType.SubsidisedCommute to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			periodDivider: {
    				title: "period divider",
    				descr: "Number of periods if Kind is PeriodicalDeduction.\nOtherwise this value is null."
    			}
    		},
    		TaxCardIncome: {
    			title: "tax card income",
    			descr: "Defines income that is recorded in a Taxcard: A paid salary calulation, income from previous employers or other salaries that affect the income calculation.",
    			description: {
    				title: "description",
    				descr: "A short description of the paid calculation.\nThis is a mandatory field if the Id is null."
    			},
    			diff: {
    				title: "diff",
    				descr: "When loading calculations from the database (in taxcard Save and FindBy with diff),\nthis property identifies the differences in the entry."
    			},
    			endDate: {
    				title: "end date",
    				descr: "End of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkEndDate, but may be different in edge cases."
    			},
    			id: {
    				title: "id",
    				descr: "Calculation id in the Salaxy database.\nIf the calculation is not in the Salaxy Database, leave this null."
    			},
    			income: {
    				title: "income",
    				descr: "Taxable income from this calculation"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the row, typically the employer that paid the salary.\nNull meaning that it is the owner of the original taxcard."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "The paidAt date which tells mainly to which year the payment should be attributed.\nThis is reliable only if the salary is paid by Salaxy (TaxCardIncomeType.SalaxyCalculation).\nIn other cases it is just the reporting date."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Beginning of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkStartDate, but may be different in edge cases."
    			},
    			tax: {
    				title: "tax",
    				descr: "Widthholding tax in euro"
    			},
    			type: {
    				title: "type",
    				descr: "Type of income"
    			}
    		},
    		TaxProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Service related to integration, reporting and payments to tax authorities.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		TaxSettings: {
    			title: "tax settings",
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			}
    		},
    		Taxcard: {
    			title: "taxcard",
    			descr: "Full tax card that is stored in database and that is used in tax card percentage calculations. Also contains the salary history in the income log.",
    			card: {
    				title: "card",
    				descr: "The input that is coming from the Worker / Tax authorities"
    			},
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"[Server]/[PayloadType]/[Owner]/[Id]\", e.g. \"https://test-secure.salaxy.com/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment for which the tax card is created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			incomeLog: {
    				title: "income log",
    				descr: "Calculations that have been paid to this tax card."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "Read-only calculation results and other logic values on the Taxcard."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker, whose salaries are paid based on this tax card."
    			}
    		},
    		TaxcardData: {
    			title: "taxcard data",
    			descr: "Tax card specific business data for index.",
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"/[PayloadType]/[Owner]/[Id]\", e.g. \"/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the taxcard validity. This may be null on NoTaxcard types."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "The type of the tax card."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the taxcard validity"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income paid to the taxcard during the year (forYear)."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity resolved as enumeration in relation to payment date given in the query.\nThis is not available in direct database OData queries as it is relevant to current / given date."
    			}
    		},
    		TaxcardInput: {
    			title: "taxcard input",
    			descr: "Input from user for a tax card: What is written in the card plus the state infomration that is directly editable by the user.",
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			state: {
    				title: "state",
    				descr: "Tax card approval state"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardListItem: {
    			title: "taxcard list item",
    			descr: "List item of taxcard searches.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		TaxcardResult: {
    			title: "taxcard result",
    			descr: "Provides the results of the tax card calculation and business logic.",
    			tax: {
    				title: "tax",
    				descr: "Provides a decimal number for a recommended widthholding tax IF it is possible to calculate that based on given information.\nNull, if the information is not sufficient to give a recommendation."
    			},
    			yearlyPreviousIncome: {
    				title: "yearly previous income",
    				descr: "Sum of income for this year based on the items marked on the tax card."
    			}
    		},
    		TaxcardResults: {
    			title: "taxcard results",
    			descr: "Read-only calculations and other logic methods on a Taxcard.",
    			payment: {
    				title: "payment",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for the actual year, which is used for income limit.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			paymentJanuary: {
    				title: "payment january",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income for the actual year, which is used for income limit.\nNote that this does not include January income that may be paid for previous year taxcard."
    			},
    			totalIncomeJanuary: {
    				title: "total income january",
    				descr: "Total income for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January."
    			},
    			validityToday: {
    				title: "validity today",
    				descr: "Gets the validity of the taxcard today. This value reflects purely the date value - it does not take into account approval or replacement."
    			}
    		},
    		TaxcardSnapshot: {
    			title: "taxcard snapshot",
    			descr: "Basic information about the taxcard: This object has the infromation printed in SalarySlip etc. but not the full salaries paid to the taxcard etc.",
    			cumulativeIncome: {
    				title: "cumulative income",
    				descr: "Cumulative income on the taxcard - including this calculation.\nNote that with shared taxcards, this may include income from other employers."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit.\nThis is percent. I.e. for 50% set 50, not 0.5."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardUiInfo: {
    			title: "taxcard ui info",
    			descr: "Display information and language versioned texts that make sense to\nend-users about the status of taxcard.",
    			kind: {
    				title: "kind",
    				descr: "Kind that is quaranteed to be non-null (undefined). This is about the existing taxcard,\nthere may still be a new card \"waiting\" and that may have another kind specified."
    			},
    			percent: {
    				title: "percent",
    				descr: "The tax percent if it is relevant for the taxcard.\nNumber is 0-60, not decimal e.g. 0.60. Null if not relevant."
    			},
    			shortText: {
    				title: "short text",
    				descr: "Shorter version of the descriptive text to the end user."
    			},
    			status: {
    				title: "status",
    				descr: "Status as in\n\"ok\": Valid taxcard,\n\"missing\": No taxcard, employer should add a new one.,\n\"waiting\": A shared taxcard is waiting for approval. Existing may be valid or not.\nor null: No calculation or employment relation specified."
    			},
    			text: {
    				title: "text",
    				descr: "Descriptive text to the end user containing the percent if relevant."
    			}
    		},
    		TaxcardWaitingApproval: {
    			title: "taxcard waiting approval",
    			descr: "Informs the employer that there is a Worker taxcard waiting for approval potentially without revealing personal information.",
    			createdAt: {
    				title: "created at",
    				descr: "Time when the taxcard was added."
    			},
    			fullCard: {
    				title: "full card",
    				descr: "If there is an owner instance of taxcard for the employer, the copy is added here.\nInstance exists only if there is a confirmed employment relation or if employer explicitly requests the taxcard."
    			},
    			isRelationUnconfirmed: {
    				title: "is relation unconfirmed",
    				descr: "If true, the relation has not yet been confirmed:\nI.e. the employer has just input the Finnish Personal Id (HETU) of the Worker.\nThe employment relation is confirmed when salaries are paid or salary request is received from Worker."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner account that added the taxcard."
    			},
    			state: {
    				title: "state",
    				descr: "Status of the taxcard"
    			},
    			uri: {
    				title: "uri",
    				descr: "URI for the shared card: This is the taxcard owned by the Worker and to view it\nyou need to request your own copy (would be in FullCard)."
    			}
    		},
    		TaxcardsDataRow: {
    			title: "taxcards data row",
    			descr: "Data row for taxcards data. This is typically used for imports and exports.",
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the taxcard"
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used. Any income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tax authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		TesParameters: {
    			title: "tes parameters",
    			descr: "Captures the collection of parameters that different Framework agreements (TES) have.",
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses amount"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Travel expences per day"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "The amount of kilometers travelled by day - used to define the amount of travel expenses in aomse framework agreements."
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "Marker that the payments defined by framework agreement are included in the salary (will not be added by the calculation)."
    			},
    			isYksityisenHoidonTuki: {
    				title: "is yksityisen hoidon tuki",
    				descr: "If true will calculate the Finnish child care subsidy"
    			},
    			numberOfDays: {
    				title: "number of days",
    				descr: "Number of days may affect calculation of different payments calculated based on a Framework agreement"
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype when one framework has several options"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the work framework"
    			},
    			yksityisenHoidonTukiAmount: {
    				title: "yksityisen hoidon tuki amount",
    				descr: "Amount of Finnish child care subsidy"
    			}
    		},
    		TestAccountInfo: {
    			title: "test account info",
    			descr: "Object containing essential details of the test account",
    			token: {
    				title: "token",
    				descr: "Bearer token for authentication."
    			},
    			userCredential: {
    				title: "user credential",
    				descr: "The current user credentials."
    			}
    		},
    		TestValues: {
    			title: "test values",
    			descr: "Test object with value properties.",
    			arrayValue: {
    				title: "array value",
    				descr: "Test array"
    			},
    			boolValue: {
    				title: "bool value",
    				descr: "Test bool."
    			},
    			dateTimeValue: {
    				title: "date time value",
    				descr: "Test DateTime."
    			},
    			dateValue: {
    				title: "date value",
    				descr: "Test Date."
    			},
    			decimalValue: {
    				title: "decimal value",
    				descr: "Test decimal."
    			},
    			dictValue: {
    				title: "dict value",
    				descr: "Free values dictionary."
    			},
    			doubleValue: {
    				title: "double value",
    				descr: "Test double."
    			},
    			enumValue: {
    				title: "enum value",
    				descr: "Test enum"
    			},
    			intValue: {
    				title: "int value",
    				descr: "Test integer."
    			},
    			objectValue: {
    				title: "object value",
    				descr: "Test object."
    			},
    			stringValue: {
    				title: "string value",
    				descr: "Test string."
    			}
    		},
    		TotalCalculationDTO: {
    			title: "total calculation d t o",
    			descr: "Calculates the total numbers - the ones that are common to Employer and worker.",
    			pension: {
    				title: "pension",
    				descr: "The total pension payment for this calculation"
    			},
    			total: {
    				title: "total",
    				descr: "The total of all calculation rows"
    			},
    			totalAccidentInsuranceBase: {
    				title: "total accident insurance base",
    				descr: "Base income for calculating the Accident insurance contributions."
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "The base salary without overtime and other such additions that are part of gross salary."
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Tax free expenses."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary is the salary including additions like overtime, but not benefits or expenses"
    			},
    			totalHealthInsuranceBase: {
    				title: "total health insurance base",
    				descr: "Base income for calculating the Health insurance contributions."
    			},
    			totalPayable: {
    				title: "total payable",
    				descr: "Total sum of rows that are paid through Salaxy Customer Funds account.\nThis is the Gross sum before side costs, deductions etc., but it is not Gross salary as it includes expenses etc."
    			},
    			totalPensionInsuranceBase: {
    				title: "total pension insurance base",
    				descr: "Base income for calculating the Pension insurance contributions."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "DEPRICATED: This is now the same as TotalPensionInsuranceBase. For other social insurances, you may want to use\nTotalUnemploymentInsuranceBase, TotalHealthInsuranceBase or TotalAccidentInsuranceBase,"
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable salary is the salary from the taxation point of view - includes some benefits."
    			},
    			totalUnemploymentInsuranceBase: {
    				title: "total unemployment insurance base",
    				descr: "Base income for calculating the Unemployment insurance contributions."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The total unemployment payment sum."
    			}
    		},
    		TypeDocType: {
    			title: "type doc type",
    			descr: "Simplified read-only interface for typedoc type.",
    			elementType: {
    				title: "element type",
    				descr: "Element type in array"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier in reference"
    			},
    			name: {
    				title: "name",
    				descr: "Name of type in intrinsic and typeParameter"
    			},
    			type: {
    				title: "type",
    				descr: "Type of type"
    			},
    			typeArguments: {
    				title: "type arguments",
    				descr: "Type arguments in reference"
    			},
    			types: {
    				title: "types",
    				descr: "Collection of types in union"
    			},
    			value: {
    				title: "value",
    				descr: "Value if literal"
    			}
    		},
    		UnemploymentProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Product that helps employer to handle the mandatory unemployment insurance reports and payments.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isNotificationsSelfHandling: {
    				title: "is notifications self handling",
    				descr: "If true, the account himself handles the unemployment insurance notifications."
    			},
    			isPrepayments: {
    				title: "is prepayments",
    				descr: "If true, the account has made prepayments or is registered to make prepayments.\nPalkkaus needs to contact the account."
    			},
    			isSalariesPaidThisYear: {
    				title: "is salaries paid this year",
    				descr: "If true, the account has paid salaries this year outside Palkkaus.fi.\nThis affects the yearly reporting - Palkkaus needs to contact the account.\nNote that this is a shared option with the taxation product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		UnionPaymentUsecase: {
    			title: "union payment usecase",
    			descr: "Handles the Calculation of CalculationRowType.UnionPayment to IrRows",
    			isDeductedAtBackoffice: {
    				title: "is deducted at backoffice",
    				descr: "If true, deduction is done in Salaxy backoffice, not directly by employer.\nAs of writing, this is only done for Rakennusliitto / Construction."
    			},
    			isFixedSum: {
    				title: "is fixed sum",
    				descr: "If true, the value is a fixed sum: the price should not be updated based on salary and\nthe count should not be updated based on percentage."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Usecase: {
    			title: "usecase",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UsecaseChildCare: {
    			title: "usecase child care",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isChildcareSubsidy: {
    				title: "is childcare subsidy",
    				descr: "If true, the salary calculation includes child care subsidy paid by the government / community."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isSunday: {
    				title: "is sunday",
    				descr: "If true, the work is done on Sunday or public holiday.\nThis means that the salary is increased by 100% according to Finnish law"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			subsidyAmount: {
    				title: "subsidy amount",
    				descr: "Amount for the child care subsidy paid by the government / community."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			useCustomPrice: {
    				title: "use custom price",
    				descr: "If true, a custom price is used instead of the recommended price.\nCurrently, this only affects the MLL case as the normal childCare price is always set by the user."
    			}
    		},
    		UsecaseCleaning: {
    			title: "usecase cleaning",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isContractLessThanYear: {
    				title: "is contract less than year",
    				descr: "If salaries are compensated (when holidays are not kept, isFullTime=false),\nthe percentage is different depending on whether the person has been working for a full year or not."
    			},
    			isFullTime: {
    				title: "is full time",
    				descr: "If true, this is a full time contract in the respect of holidays:\nHolidays are kept during summer and salary is paid for these holidays."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseConstruction: {
    			title: "usecase construction",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses (työkalukorvaus) if isExpensesCustom=true"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Daily travel expenses if isExpensesCustom=true"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "Daily distance (km) from worker home to construction site."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isExpensesCustom: {
    				title: "is expenses custom",
    				descr: "If true, the expenses are agreed in a bispoken way.\nDefault is that expenses are based on Framework agreement (TES) tables."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "If true, the TES additions are included in the agreed salary."
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseData: {
    			title: "usecase data",
    			descr: "A usecase for creating and editing salary calculations and other . In practice, a usecase is either a user interface or a microservice that provides the bases for the object. Provides the possibility to store useacase specific data as part of object. NOTE: For Calculations, this functionality will replace TesParameters in the 3.0 version.",
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			description: {
    				title: "description",
    				descr: "Optional description of the use case for the end user.\nShown in detail views when explaining the logic of the usecase."
    			},
    			label: {
    				title: "label",
    				descr: "A short label of that can be shown to the end user."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this may resolve to a user interface, microservice or a to manifest registering one.\nFor now, just use a unique string starting with a domain you own."
    			}
    		},
    		UsecaseGroup: {
    			title: "usecase group",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case group (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG)"
    			},
    			children: {
    				title: "children",
    				descr: "Collection of use cases - in the future, may also support sub groups."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG )"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the group"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UserDefinedRow: {
    			title: "user defined row",
    			descr: "This is a user defined calculation row as opposed to CalculationRow that may be either based on a user defined row or generated by the system based on e.g. TES parameters.",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowIndex: {
    				title: "row index",
    				descr: "Zero based row index that can be used to match the row to result rows or when deleting.\nNOTE: The Row ID is given values on the fly so setting this will not have any effect at the moment.\nIt may reorder the entries in later implementations."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is passed to the result calculations."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. Guessed based on the RowType and count, but you are better of setting it to be sure."
    			}
    		},
    		UserSession: {
    			title: "user session",
    			descr: "Passes the data about the authorization that is the bases for callin the services that require user authorization.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar that should be shown for the current session.\nTypically CurrentAccount.Avatar, but could theoretically be something different."
    			},
    			currentAccount: {
    				title: "current account",
    				descr: "The current Palkkaus.fi-account that the user acting as"
    			},
    			currentCredential: {
    				title: "current credential",
    				descr: "The current user credentials."
    			},
    			currentPartner: {
    				title: "current partner",
    				descr: "The primary partner of the current account."
    			},
    			hasCredential: {
    				title: "has credential",
    				descr: "If false, the current user does not have a credential in the system =&gt; need to redirect to login"
    			},
    			hasCurrentAccount: {
    				title: "has current account",
    				descr: "If false, the current user does not have a current account.\nIf HasCredential is true, this means that the user should be redirected to Account creation wizard."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, the current user needs to be authenticated or authenticated user does not have acces to the system."
    			},
    			requestingPartnerSite: {
    				title: "requesting partner site",
    				descr: "The site from which the API call is being made."
    			},
    			status: {
    				title: "status",
    				descr: "Authorization status"
    			},
    			systemAlertHtml: {
    				title: "system alert html",
    				descr: "Contains html for system alert."
    			}
    		},
    		VarmaPensionOrder: {
    			title: "varma pension order",
    			descr: "Contains order data for Varma pensions (TyEL, YEL) and required operations.",
    			action: {
    				title: "action",
    				descr: "Action for the order."
    			},
    			orderer: {
    				title: "orderer",
    				descr: "Orderer information."
    			},
    			tyel: {
    				title: "tyel",
    				descr: "TyEL data for the order"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation information."
    			},
    			yel: {
    				title: "yel",
    				descr: "YEL data for the order"
    			}
    		},
    		VarmaPensionOrderTyel: {
    			title: "varma pension order tyel",
    			descr: "Varma pension order TyEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startMonth: {
    				title: "start month",
    				descr: "Start month for the new order."
    			},
    			startYear: {
    				title: "start year",
    				descr: "Start year for the new order."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yerly income."
    			}
    		},
    		VarmaPensionOrderYel: {
    			title: "varma pension order yel",
    			descr: "Varma pension order YEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			isYearlyIncomeChange: {
    				title: "is yearly income change",
    				descr: "Indicates if the existing yearly income estimate will be changed."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer definition."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date for the new pension."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yearly income."
    			}
    		},
    		VismaSignPostData: {
    			title: "visma sign post data",
    			descr: "Data for building the Visma Sign API request form. This is the legacy API v0.2: https://allekirjoitus.visma.fi/download/visma_sign-api.pdf",
    			auth_service: {
    				title: "auth_service",
    				descr: "Specifies the Tupas authentication service (e.g. 'osuuspankki', 'nordea').\nIf not specified, Visma Sign will show its own selection screen."
    			},
    			customer: {
    				title: "customer",
    				descr: "Palkkaus.fi Customer code to Visma Sign-system"
    			},
    			data: {
    				title: "data",
    				descr: "Rest of the data as encrypted JSON."
    			},
    			documentUrl: {
    				title: "document url",
    				descr: "Document URL for preview purposes.\nTo Visma Sign, this is sent within the encrypted data field."
    			},
    			iv: {
    				title: "iv",
    				descr: "Encryption Initialization vector"
    			},
    			return_failure: {
    				title: "return_failure",
    				descr: "Failure address - needs to be non-encrypted, because the error may be in the encryption"
    			}
    		},
    		WelcomeDataWorker: {
    			title: "welcome data worker",
    			descr: "Data that is used for displaying the welcome page for the Worker account.",
    			activeTaxcard: {
    				title: "active taxcard",
    				descr: "Active taxcard added by the Worker or null, if none is added (or none is valid for today)."
    			},
    			invitation: {
    				title: "invitation",
    				descr: "Invitation for the worker to store a taxcard for salary payment."
    			},
    			timeline: {
    				title: "timeline",
    				descr: "The welcome page timeline cards."
    			}
    		},
    		WorkDescription: {
    			title: "work description",
    			descr: "Describes the work that is beign performed - mainly for Employment relation purposes.",
    			description: {
    				title: "description",
    				descr: "Free text description of the work that is being performed."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			salaryDefaults: {
    				title: "salary defaults",
    				descr: "Contains default calculation rows for the worker.\nTypically used for setting for example monthly salary or hourly salary price for workers."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Default bases for wage."
    			}
    		},
    		WorkerAbsencePeriod: {
    			title: "worker absence period",
    			descr: "Absence Period with Worker information.",
    			absencesId: {
    				title: "absences id",
    				descr: "WorkerAbsences storage object that contains the period"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			},
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker that owns the Absence period"
    			}
    		},
    		WorkerAbsences: {
    			title: "worker absences",
    			descr: "Absences for a single Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation id for the worker whose absences are recorded."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			periods: {
    				title: "periods",
    				descr: "Collection of absence periods."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Account ID for the worker whose absences are recorded."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			}
    		},
    		WorkerAccount: {
    			title: "worker account",
    			descr: "Account data that is relevant for a Worker.",
    			ageRange: {
    				title: "age range",
    				descr: "Age range that is relevant to workers salary calculation and/or employment.\nSet this especially if OfficialPersonId is not set.\nIf OfficialPersonId is valid, this property is always set based on it."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "If created (and still owned) by an employer, this is the AccountOwnerId of the employer who created this account."
    			},
    			employment: {
    				title: "employment",
    				descr: "Current Employment Relation: The data that is owned by the Employer about the Worker\neven if the Worker has registered and created an account.\nIf employer has several Employment Relations, this is the one that is marked as Primary."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The unique id for the employment. This does not change within one employment."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			officialPersonId: {
    				title: "official person id",
    				descr: "The Person ID of the worker if known."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides validation errors currently only in Save operations,\nbut potentially in the future also in detail/list views."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "Information needed when salary is paid to this account and other such stuff about the account as a worker."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		WorkerAccountEmployment: {
    			title: "worker account employment",
    			descr: "Employment Relation as it is shown as part of WorkerAccount.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isAccountOwner: {
    				title: "is account owner",
    				descr: "If true, the worker is the same person as the actual signed owner of the account."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		WorkerCalculationDTO: {
    			title: "worker calculation d t o",
    			descr: "The Worker side of the calculation. SalarySlip is largely written based on this calculation.",
    			benefits: {
    				title: "benefits",
    				descr: "The benefits as valued by the tax authority."
    			},
    			deductions: {
    				title: "deductions",
    				descr: "All the items that are deducted from the salary (this is also the order if there is not enough payable money):\nSalaryAdvance + Tax + Pension + UnemploymentInsurance + UnionPayment + OtherDeductions"
    			},
    			foreclosure: {
    				title: "foreclosure",
    				descr: "The foreclosure that been deducted from the salary by the employer."
    			},
    			fullOtherDeductions: {
    				title: "full other deductions",
    				descr: "Other deductions that are deducted from the net salary.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullPension: {
    				title: "full pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullPrepaidExpenses: {
    				title: "full prepaid expenses",
    				descr: "Prepaid expenses that are deducted from the expenses to be paid to the worker.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullSalaryAdvance: {
    				title: "full salary advance",
    				descr: "The salary advance that was paid to the Worker\nThe original calulated version - there might not be enough money to finally pay this."
    			},
    			fullTax: {
    				title: "full tax",
    				descr: "Widthholding tax: The original calulated version - there might not be enough money to finally pay this."
    			},
    			fullUnemploymentInsurance: {
    				title: "full unemployment insurance",
    				descr: "Workers part of the unemployment insurance.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullUnionPayment: {
    				title: "full union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			otherDeductions: {
    				title: "other deductions",
    				descr: "Part of the other deductions that are deducted from the net salary."
    			},
    			pension: {
    				title: "pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary."
    			},
    			prepaidExpenses: {
    				title: "prepaid expenses",
    				descr: "Part of the prepaid expenses that are deducted from the expenses."
    			},
    			salaryAdvance: {
    				title: "salary advance",
    				descr: "The salary advance that was paid to the Worker"
    			},
    			salaryAfterTax: {
    				title: "salary after tax",
    				descr: "Net salary after tax - before worker foreclosure or deductions of the side costs"
    			},
    			salaryAfterTaxAndForeclosure: {
    				title: "salary after tax and foreclosure",
    				descr: "Net salary after tax and foreclosure."
    			},
    			salaryPayment: {
    				title: "salary payment",
    				descr: "The payment that is made to the Worker based on salary after all deductions.\nDoes not include expenses."
    			},
    			tax: {
    				title: "tax",
    				descr: "Ennakonpidätys: The taxes that were deducted from the worker salary."
    			},
    			totalWorkerPayment: {
    				title: "total worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			},
    			totalWorkerPaymentLegacy: {
    				title: "total worker payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "Workers part of the unemployment insurance."
    			},
    			unionPayment: {
    				title: "union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			workerSideCosts: {
    				title: "worker side costs",
    				descr: "The side costs that are deducted from the Worker salary according to law:\nPension, Unemployment and Union payment. Tax is not included in this number.\nThis is not an official number, really used in the charts etc."
    			}
    		},
    		WorkerCurrentTaxcards: {
    			title: "worker current taxcards",
    			descr: "Contains the status and necessary data with Workers current taxcards.",
    			active: {
    				title: "active",
    				descr: "The currently active taxcard if one exists. This property is null by default."
    			},
    			employers: {
    				title: "employers",
    				descr: "Employer taxcards for the Worker account: One for each employment relationship."
    			},
    			employment: {
    				title: "employment",
    				descr: "Employment relation if the search is done based on employment relation ID and not Current worker account."
    			},
    			list: {
    				title: "list",
    				descr: "All the relevant taxcards as list items."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID that is used for taxcard binding. This always exists as opposed to Employment."
    			},
    			waitingApproval: {
    				title: "waiting approval",
    				descr: "A shared taxcard waiting for approval if one exists. This property is null by default."
    			}
    		},
    		WorkerData: {
    			title: "worker data",
    			descr: "Worker specific business data for index.",
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation."
    			}
    		},
    		WorkerInfo: {
    			title: "worker info",
    			descr: "Information directly related to Employment (as Worker)",
    			defaultTaxCard: {
    				title: "default tax card",
    				descr: "The Tax card that is used by default."
    			}
    		},
    		WorkerListItem: {
    			title: "worker list item",
    			descr: "Represents a single item in a list of workers in the user interface. It contains all essential information of the worker and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		WorkerProfileProperties: {
    			title: "worker profile properties",
    			dateOfBirth: {
    				title: "date of birth"
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy"
    			},
    			gender: {
    				title: "gender"
    			},
    			hasDriversLisence: {
    				title: "has drivers lisence"
    			},
    			hasTaxCard: {
    				title: "has tax card"
    			}
    		},
    		WorkerSummary: {
    			title: "worker summary",
    			descr: "Year end summary for one Worker account.",
    			calculations: {
    				title: "calculations",
    				descr: "Calculations that have been paid through Salaxy system."
    			},
    			corrections: {
    				title: "corrections",
    				descr: "Corrections that are made to existing calculations as well as\nsalary calculations that are paid outside Salaxy."
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes when set on worker level, not for each calculation separately."
    			},
    			summary: {
    				title: "summary",
    				descr: "Total of Calculations and Corrections as calculatied by the server side logic."
    			},
    			worker: {
    				title: "worker",
    				descr: "Worker account to which the salaries have been paid."
    			}
    		},
    		WorkersDataRow: {
    			title: "workers data row",
    			descr: "Data row for employments data. This is typically used for imports and exports.",
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the worker"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		WorkflowData: {
    			title: "workflow data",
    			descr: "Defines the data that IWorkflowItem must provide.",
    			events: {
    				title: "events",
    				descr: "Workflow events for the item."
    			}
    		},
    		WorkflowEvent: {
    			title: "workflow event",
    			descr: "Event in the workflow",
    			assignedTo: {
    				title: "assigned to",
    				descr: "Credential of the user which is responsible of the event."
    			},
    			features: {
    				title: "features",
    				descr: "Features for this event."
    			},
    			id: {
    				title: "id",
    				descr: "Event-provider specific identifier for the event.\nFor deletable events, this should identify , but do NOT trust this to be 100% unique."
    			},
    			message: {
    				title: "message",
    				descr: "Message for backoffice purposes from the User or system that created the event."
    			},
    			time: {
    				title: "time",
    				descr: "UTC time for when the event occured."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the event. This typically maps to parent-object specific enumeration."
    			},
    			ui: {
    				title: "ui",
    				descr: "User Interface styling of the event: Indicates the severity for the end-user."
    			},
    			user: {
    				title: "user",
    				descr: "User or system that created the event."
    			}
    		},
    		WorkingTimeCompensationUsecase: {
    			title: "working time compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.WorkingTimeCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		YearlyChangingNumbers: {
    			title: "yearly changing numbers",
    			descr: "Parameters that are changing yearly. This class is made for the change of the year and it is typically updated in December. It is not for recording longer history of numbers.",
    			holidays: {
    				title: "holidays",
    				descr: "Returns holidays for the year."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "Returns yearly sidecosts."
    			},
    			year: {
    				title: "year",
    				descr: "Year for numbers."
    			}
    		},
    		YearlyFeedback: {
    			title: "yearly feedback",
    			descr: "The feedback from the account holder at the end-of-year. Possibility to feed in changes, corrections and salaries that are paid outside the Salaxy system.",
    			adminNotes: {
    				title: "admin notes",
    				descr: "Full description of yearly process status for backoffice pusposes. Shown in backoffice detail views."
    			},
    			adminShortNote: {
    				title: "admin short note",
    				descr: "Short notice for backoffice purposes. This is the text visible in the lists."
    			},
    			adminStatus: {
    				title: "admin status",
    				descr: "Status for administrator check."
    			},
    			bankIban: {
    				title: "bank iban",
    				descr: "Bank IBAN number for yearly refunding of extra customer assets.\nREADONLY: Note that once set to a valid IBAN, this number is read-only and must be changed through customer service or onboarding (requires authentication)."
    			},
    			employerAvatar: {
    				title: "employer avatar",
    				descr: "Avatar for the employer of display purposes in reports etc."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "Account ID for the employer."
    			},
    			externalWorkerInsuranceWork: {
    				title: "external worker insurance work",
    				descr: "Temporary user interface helper for external workers (insurance) =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			externalWorkerSalaries: {
    				title: "external worker salaries",
    				descr: "Temporary user interface helper for external workers =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			household: {
    				title: "household",
    				descr: "Yearly feedback data that is specific to household customers.\nMain part is the information realted to household deduction and how it can be divided between spouses."
    			},
    			sendInsuranceReport: {
    				title: "send insurance report",
    				descr: "If true, Salaxy should report the salaries to Insurance company.\nCurrently, this is available only for Companies and for If insurance company only."
    			},
    			userFeedbackText: {
    				title: "user feedback text",
    				descr: "Free text from the account holder to Salaxy backoffice."
    			},
    			userFeedbackTime: {
    				title: "user feedback time",
    				descr: "Time when the feedback was given."
    			},
    			userStatus: {
    				title: "user status",
    				descr: "The satus that the account holder gives to yearly material."
    			},
    			workerSummaries: {
    				title: "worker summaries",
    				descr: "List of Workers for which salaries have been paid during the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the feedback"
    			}
    		},
    		YearlyHolidays: {
    			title: "yearly holidays",
    			descr: "Holidays for the entire year.",
    			holidays: {
    				title: "holidays",
    				descr: "Holidays for the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the holidays."
    			}
    		},
    		YearlySideCosts: {
    			title: "yearly side costs",
    			descr: "Encapsulates yearly side costs into one class.",
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction"
    			},
    			illnessInsurancePercent: {
    				title: "illness insurance percent",
    				descr: "Illness Insurance care percent that is withhold from the Worker. (vakuutetun sairausvakuutusmaksu: päivärahamaksu + sairaanhoitomaksu"
    			},
    			palkkausVatPercent: {
    				title: "palkkaus vat percent",
    				descr: "ALV-% for the fee."
    			},
    			socSecPercent: {
    				title: "soc sec percent",
    				descr: "Social Security payment for employer (SAVA-maksu)"
    			},
    			taxFreeDailyAllowance: {
    				title: "tax free daily allowance",
    				descr: "Tax free daily allowance for full day approved by tax authorities."
    			},
    			taxFreeDailyHalfAllowance: {
    				title: "tax free daily half allowance",
    				descr: "Tax free daily allowance for half day approved by tax authorities."
    			},
    			taxFreeKmAllowance: {
    				title: "tax free km allowance",
    				descr: "Tax free milage allowance approved by tax authorities."
    			},
    			taxFreeMealAllowance: {
    				title: "tax free meal allowance",
    				descr: "Tax free meal allowance approved by tax authorities."
    			},
    			tyel53Percent: {
    				title: "tyel53 percent",
    				descr: "Pension payment base percentage - deducted from 53-62 year old Worker"
    			},
    			tyelBasePercent: {
    				title: "tyel base percent",
    				descr: "Pension payment base percentage"
    			},
    			tyelContractEmployerLimit: {
    				title: "tyel contract employer limit",
    				descr: "Pension contract employer six month limit"
    			},
    			tyelIncomeLimit: {
    				title: "tyel income limit",
    				descr: "Pension payment income limit per month"
    			},
    			tyelWorkerPercent: {
    				title: "tyel worker percent",
    				descr: "Pension payment base percentage - deducted from Worker salary"
    			},
    			unemploymentEmployerPercent: {
    				title: "unemployment employer percent",
    				descr: "Unemployment insurance payment percent - Employer"
    			},
    			unemploymentPartialOwnerWorkerPercent: {
    				title: "unemployment partial owner worker percent",
    				descr: "Unemployment insurance payment percent for partial owner worker - deducted from Worker"
    			},
    			unemploymentWorkerPercent: {
    				title: "unemployment worker percent",
    				descr: "Unemployment insurance payment percent - deducted from Worker"
    			},
    			unionPaymentRaksaA: {
    				title: "union payment raksa a",
    				descr: "Union payment: Rakennusliitto, A (typical)"
    			},
    			unionPaymentRaksaAoTa: {
    				title: "union payment raksa ao ta",
    				descr: "Union payment: Rakennusliitto, AO and TA"
    			}
    		},
    		YtjCompany: {
    			title: "ytj company",
    			descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			companyTypeText: {
    				title: "company type text",
    				descr: "YTJ's legal type of the company as a string. This is only to be used for display purposes.\nSource: YritysHaku.YritysMuoto"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the"
    			},
    			contactPerson: {
    				title: "contact person",
    				descr: "Names are not necessarily avaiable. If they are, they may not be reliable."
    			},
    			contactPersonPersonalId: {
    				title: "contact person personal id",
    				descr: "Social security number of the contact person in the company, if available."
    			},
    			contactPhone: {
    				title: "contact phone",
    				descr: "Mobile for or other company phone.\nDifference between Contact.Phone and ContactPhone is that\nthis (ContactPhone) is first mobile number, then company general phone if the former was null.\nContact.Phone is fisrt general phone number and mobile number only if not available."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company.\nSource: Tiedostonimi.Tiedostonimi"
    			},
    			sectorOfBusinessCode: {
    				title: "sector of business code",
    				descr: "The sector where the company is active. (Toimialatieto.Koodi)"
    			},
    			sectorOfBusinessText: {
    				title: "sector of business text",
    				descr: "Text of the sector where company operates."
    			},
    			www: {
    				title: "www",
    				descr: "Web site address"
    			},
    			ytjCompanyType: {
    				title: "ytj company type",
    				descr: "The internal YTJ code number for the company's legal type."
    			}
    		},
    		YtjSearchResult: {
    			title: "ytj search result",
    			descr: "Search result for free text search from the Finnish Business Infromation System (YTJ/BIS). Returns a list of companies without details.",
    			counter: {
    				title: "counter",
    				descr: "API quota counter"
    			},
    			isSuccess: {
    				title: "is success",
    				descr: "If true, search is a success. This may mean that no results were found though."
    			},
    			isTooGeneric: {
    				title: "is too generic",
    				descr: "If true, the searh is too generic.\nTypically either there was more than 200 results or less than 3 characters in the search string.\nBoth of these return an error in the API."
    			},
    			rawInput: {
    				title: "raw input",
    				descr: "The original search string - without trimming etc. that are done in the actual search."
    			},
    			result: {
    				title: "result",
    				descr: "Result of the search."
    			},
    			status: {
    				title: "status",
    				descr: "Technical status - for developers / admin."
    			},
    			userMessage: {
    				title: "user message",
    				descr: "A message that can be shown to the end user."
    			}
    		},
    		YtjSearchResultCompany: {
    			title: "ytj search result company",
    			descr: "Search result list level of detail of companies coming from YTJ.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			id: {
    				title: "id",
    				descr: "Numeric ID (\"Yritysavain\")"
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the company. This is a free text type. Not necessary parseable to company type enumeration in details."
    			}
    		}
    	}
    };

    var SALAXY$7 = {
    	UI_TERMS: {
    		add: "Add",
    		addNew: "Add new",
    		addRow: "Add row",
    		areYouSure: "Are you sure?",
    		back: "Back",
    		calculate: "Calculate",
    		calculations: "Calculations",
    		cancel: "Cancel",
    		cancelEditing: "Cancel editing",
    		choose: "Choose…",
    		chooseWorkDateRange: "",
    		chooseWorkDuration: "",
    		clear: "Clear",
    		close: "Close",
    		confirm: "Confirm",
    		copy: "Copy",
    		copyAsNew: "Copy as new",
    		copyAsRequest: "Copy as Request",
    		copyToClipboard: "",
    		dateFormats: {
    			dm: "d.m."
    		},
    		dateNotChosen: "date has not been chosen",
    		"delete": "Delete",
    		demo: "DEMO",
    		dismiss: "Dismiss",
    		done: "Done",
    		edit: "Edit",
    		gotIt: "Got it",
    		isDeleting: "Deleting...",
    		isSaving: "Saving...",
    		learnMore: "Learn more",
    		loading: "Loading...",
    		login: "Log in",
    		logout: "Log out",
    		logoutAlt: "End current session",
    		makeOrder: "",
    		minCharacters: "Start with at least 3 letters...",
    		monthHalf: "Half a month",
    		monthShort0: "Dec",
    		monthShort1: "Jan",
    		monthShort10: "Oct",
    		monthShort11: "Nov",
    		monthShort12: "Dec",
    		monthShort2: "Feb",
    		monthShort3: "Mar",
    		monthShort4: "Apr",
    		monthShort5: "May",
    		monthShort6: "Jun",
    		monthShort7: "Jul",
    		monthShort8: "Aug",
    		monthShort9: "Sep",
    		moreInfo: "More info",
    		navigation: "Navigation",
    		newCalculation: "New calculation",
    		next: "Next",
    		no: "No",
    		noteRequiredField: "* Required Field",
    		noThanks: "No thanks",
    		ok: "OK",
    		oldVersion: "old version",
    		orChoose: "",
    		otherPeriod: "Other period",
    		pay: "Pay",
    		pleaseWait: "Please wait...",
    		postpone: "Not now",
    		previous: "Previous",
    		printable: "Printable",
    		recalculate: "Recalculate",
    		register: "Register",
    		save: "Save",
    		saveEdited: "Save changes",
    		search: "Search...",
    		searchByName: "Search by name...",
    		select: "Select",
    		selectAlt: "Select...",
    		sendEmail: "Send an email",
    		show: "Show",
    		skip: "Skip",
    		sureToDeleteRecord: "Are you sure to delete this record.",
    		today: "Today",
    		updateCalculation: "Update calculation",
    		updatingCalculation: "Updating calculation...",
    		use: "",
    		week2: "2 weeks",
    		yes: "Yes"
    	}
    };

    var SALAXY$6 = {
    	VALIDATION: {
    	}
    };

    var SALAXY$5 = {
    	ENUM: {
    	}
    };

    var SALAXY$4 = {
    	ENUM: {
    		AbcSection: {
    			association: {
    				description: "",
    				label: "Föreningar"
    			},
    			blog: {
    				description: "",
    				label: "Blogg"
    			},
    			businessOwner: {
    				description: "",
    				label: "Företagare"
    			},
    			documentTemplates: {
    				description: "",
    				label: "Dokument och formulär"
    			},
    			doNotShow: {
    				description: "",
    				label: "Ej visad till användare (endast Admin)"
    			},
    			employee: {
    				description: "",
    				label: "Anställd"
    			},
    			employment: {
    				description: "",
    				label: "Anställningsförhållande och arbetsavtal"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Företag eller samfund"
    			},
    			examples: {
    				description: "",
    				label: "Exempel av anställningssituationer"
    			},
    			householdEmployer: {
    				description: "",
    				label: "Hushåll"
    			},
    			instructionsAndExamples: {
    				description: "",
    				label: "Bruksanvisning"
    			},
    			insurance: {
    				description: "",
    				label: "Försäkringar och pension"
    			},
    			palkkausGeneral: {
    				description: "",
    				label: "Information om Palkkaus.fi-tjänsten"
    			},
    			palkkausInstructions: {
    				description: "",
    				label: "Bruksavisningar"
    			},
    			personEmployer: {
    				description: "",
    				label: "Hushållsarbetsgivare"
    			},
    			press: {
    				description: "",
    				label: "Meddelanden och material"
    			},
    			productLongDescription: {
    				description: "",
    				label: "Produktbeskrivning"
    			},
    			salary: {
    				description: "",
    				label: "Löneräkning och lönebetalning"
    			},
    			"undefined": {
    				description: "",
    				label: "Nya artiklar / odefinierat"
    			},
    			worker: {
    				description: "",
    				label: "Arbetsgivare"
    			}
    		},
    		AbsenceCauseCode: {
    			accruedHoliday: {
    				label: "Pekkasledighet"
    			},
    			annualLeave: {
    				label: "Semester"
    			},
    			childCareLeave: {
    				label: "Vårdledighet"
    			},
    			childIllness: {
    				label: "Sjukt barn eller tvingande familjeskäl"
    			},
    			illness: {
    				label: "Sjukdom"
    			},
    			industrialAction: {
    				label: "Strejk eller lockout"
    			},
    			interruptionInWorkProvision: {
    				label: "Avbrott i utbud av arbete"
    			},
    			jobAlternationLeave: {
    				label: "Alterneringsledighet"
    			},
    			layOff: {
    				label: "Permittering"
    			},
    			leaveOfAbsence: {
    				label: "Tjänstledighet"
    			},
    			midWeekHoliday: {
    				label: "Söckenhelg"
    			},
    			militaryRefresherTraining: {
    				label: "Repetitionsövning"
    			},
    			militaryService: {
    				label: "Militär- eller civiltjänst"
    			},
    			occupationalAccident: {
    				label: "Arbetsolycka"
    			},
    			other: {
    				label: "Annan orsak"
    			},
    			parentalLeave: {
    				label: "Moderskaps-, faderskaps- och föräldraledighet"
    			},
    			partTimeAbsenceDueToRehabilitation: {
    				label: "Partiell frånvaro på grund av rehabilitering"
    			},
    			partTimeChildCareLeave: {
    				label: "Partiell vårdledighet"
    			},
    			partTimeSickLeave: {
    				label: "Partiell sjukfrånvaro"
    			},
    			personalReason: {
    				label: ""
    			},
    			rehabilitation: {
    				label: "Rehabilitering"
    			},
    			specialMaternityLeave: {
    				label: "Särskild moderskapsledighet"
    			},
    			studyLeave: {
    				label: "Studieledighet"
    			},
    			training: {
    				label: "Utbildning"
    			},
    			unpaidLeave: {
    				label: ""
    			}
    		},
    		AgeGroupCode: {
    			a: {
    				label: ""
    			},
    			b: {
    				label: ""
    			},
    			c: {
    				label: ""
    			},
    			u: {
    				label: ""
    			}
    		},
    		AgeRange: {
    			age_15: {
    				description: "",
    				label: "15 år"
    			},
    			age16: {
    				description: "",
    				label: "16 år"
    			},
    			age17: {
    				description: "",
    				label: "17 år"
    			},
    			age18_52: {
    				description: "",
    				label: "18-52 år"
    			},
    			age53_62: {
    				description: "",
    				label: "53-62 år"
    			},
    			age63_64: {
    				description: "",
    				label: "63-64 år"
    			},
    			age65_67: {
    				description: "",
    				label: "65-67 år"
    			},
    			age68AndOVer: {
    				description: "",
    				label: "68 år och över"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			}
    		},
    		AllowanceCode: {
    			expensesWorkingAbroad: {
    				label: ""
    			},
    			fullDailyAllowance: {
    				label: ""
    			},
    			internationalDailyAllowance: {
    				label: ""
    			},
    			mealAllowance: {
    				label: ""
    			},
    			partialDailyAllowance: {
    				label: ""
    			},
    			taxExemptReimbursementsAbroad: {
    				description: "",
    				label: "TaxExemptReimbursementsAbroad"
    			}
    		},
    		AnnualLeavePaymentKind: {
    			draftCalc: {
    				description: "",
    				label: ""
    			},
    			endSaldo: {
    				description: "",
    				label: ""
    			},
    			info: {
    				description: "",
    				label: ""
    			},
    			manualBonus: {
    				description: "",
    				label: ""
    			},
    			manualCompensation: {
    				description: "",
    				label: ""
    			},
    			manualSalary: {
    				description: "",
    				label: ""
    			},
    			paidCalc: {
    				description: "",
    				label: ""
    			},
    			startSaldo: {
    				description: "",
    				label: ""
    			},
    			total: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			planned: {
    				description: "",
    				label: "Planned"
    			}
    		},
    		ApiTestErrorType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			userFriendly: {
    				description: "",
    				label: "Användarvänlig"
    			}
    		},
    		ApiValidationErrorType: {
    			general: {
    				label: "Allmän"
    			},
    			invalid: {
    				label: "Ogiltig"
    			},
    			required: {
    				label: "Krävs"
    			},
    			warning: {
    				label: ""
    			}
    		},
    		AuthenticationMethod: {
    			auth0Database: {
    				description: "",
    				label: "E-post-konto"
    			},
    			emailPwdLocal: {
    				description: "",
    				label: "Databas"
    			},
    			facebook: {
    				description: "",
    				label: "Facebook-konto"
    			},
    			google: {
    				description: "",
    				label: "Google eller Gmail-konto"
    			},
    			internalLocalhost: {
    				description: "",
    				label: ""
    			},
    			linkedIn: {
    				description: "",
    				label: "LinkedIn-konto"
    			},
    			microsoft: {
    				description: "",
    				label: "Microsoft-konto"
    			},
    			salaxy: {
    				description: "",
    				label: ""
    			},
    			test: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Okänd"
    			},
    			x509: {
    				description: "",
    				label: ""
    			}
    		},
    		AuthorizationStatus: {
    			accessDenied: {
    				description: "",
    				label: "Tillträde förbjudet"
    			},
    			expiredOauthTokdescription: "",
    			expiredOauthToklabel: "Session föråldrad",
    			invalidOAuthTokdescription: "",
    			invalidOAuthToklabel: "Fel i autentisering",
    			noOauthTokdescription: "",
    			noOauthToklabel: "Fel i autentisering (ingen token)",
    			oK: {
    				description: "",
    				label: "OK"
    			},
    			"undefined": {
    				description: "",
    				label: "Odefinierad"
    			},
    			expiredOauthToken: {
    				description: "",
    				label: "ExpiredOauthToken"
    			},
    			invalidOAuthToken: {
    				description: "",
    				label: "InvalidOAuthToken"
    			},
    			noOauthToken: {
    				description: "",
    				label: "NoOauthToken"
    			}
    		},
    		AuthorizationType: {
    			companyContract: {
    				description: "",
    				label: "Arbetsgivarkontrakt"
    			},
    			employerAuthorization: {
    				description: "",
    				label: "Fullmakt"
    			},
    			manual: {
    				description: "",
    				label: "Fullmakt inlämnat skilt"
    			},
    			none: {
    				description: "",
    				label: "Ej identifierad"
    			},
    			temporary: {
    				description: "",
    				label: "Tillfällig identifiering"
    			},
    			workerContract: {
    				description: "",
    				label: "Arbetstagarkontrakt"
    			}
    		},
    		AvatarPictureType: {
    			gravatar: {
    				description: "",
    				label: "Gravatar"
    			},
    			icon: {
    				description: "",
    				label: "Ikon"
    			},
    			uploaded: {
    				description: "",
    				label: "Bild från egen dator"
    			}
    		},
    		BlobFileType: {
    			authorizationPdf: {
    				description: "",
    				label: "Fullmakt"
    			},
    			avatar: {
    				description: "",
    				label: "Ikon"
    			},
    			calcReport: {
    				description: "",
    				label: ""
    			},
    			expensesReceipt: {
    				description: "",
    				label: "Kvitto för utlägg"
    			},
    			monthlyReport: {
    				description: "",
    				label: "Månatlig rapport"
    			},
    			raksaloki: {
    				description: "",
    				label: "Kvitto för utlägg"
    			},
    			taxCard: {
    				description: "",
    				label: "Skattekort"
    			},
    			template: {
    				description: "",
    				label: "Mall"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			},
    			yearlyReport: {
    				description: "",
    				label: "Årlig rapport"
    			},
    			eInvoice: {
    				description: "",
    				label: "EInvoice"
    			},
    			messages: {
    				description: "",
    				label: "Messages"
    			},
    			settings: {
    				description: "",
    				label: "Settings"
    			},
    			temporary: {
    				description: "",
    				label: "Temporary"
    			}
    		},
    		BlobRepository: {
    			cdnImages: {
    				description: "",
    				label: ""
    			},
    			fileSystemContent: {
    				description: "",
    				label: ""
    			},
    			gitContent: {
    				description: "",
    				label: ""
    			},
    			payload: {
    				description: "",
    				label: ""
    			},
    			reports: {
    				description: "",
    				label: ""
    			},
    			systemFiles: {
    				description: "",
    				label: ""
    			},
    			userFiles: {
    				description: "",
    				label: ""
    			},
    			versionHistory: {
    				description: "",
    				label: ""
    			}
    		},
    		BoardKind: {
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		CalculationFlag: {
    			accidentInsurance: {
    				label: ""
    			},
    			cfDeduction: {
    				label: ""
    			},
    			cfDeductionAtSalaxy: {
    				label: ""
    			},
    			cfNoPayment: {
    				label: ""
    			},
    			cfPayment: {
    				label: ""
    			},
    			exclude: {
    				label: ""
    			},
    			healthInsurance: {
    				label: ""
    			},
    			noTax: {
    				label: ""
    			},
    			pensionInsurance: {
    				label: ""
    			},
    			tax: {
    				label: ""
    			},
    			taxDeduction: {
    				label: ""
    			},
    			unemploymentInsurance: {
    				label: ""
    			},
    			insurancesDeduction: {
    				description: "",
    				label: "InsurancesDeduction"
    			}
    		},
    		CalculationRowSource: {
    			manualRow: {
    				description: "",
    				label: "Tilläggsrad"
    			},
    			salaryPage: {
    				description: "",
    				label: "Huvudlön"
    			},
    			tesSelection: {
    				description: "",
    				label: "Användarfall"
    			}
    		},
    		CalculationRowType: {
    			accomodationBenefit: {
    				amountLabel: "",
    				description: "Beskattningsbar boendeförmån.",
    				iconText: "",
    				label: "Boendetillägg",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			advance: {
    				amountLabel: "",
    				description: "Avdrag pga. löneförskott som betalats ut tidigare.",
    				iconText: "",
    				label: "Löneförskott",
    				moreInfo: "",
    				priceLabel: "Förskottsbetalning"
    			},
    			board: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			carBenefit: {
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			chainsawReduction: {
    				amountLabel: "",
    				description: "Avdrag för bruk av motorsåg, i specialfall (förskottsbetalningslag § 15, förskottsbetalnings 15§).",
    				iconText: "",
    				label: "Välj…",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			childCareSubsidy: {
    				amountLabel: "",
    				description: "FPA stöd för kommunala arbetare för barnvård. Påverkar sidokostnader.",
    				iconText: "FPA",
    				label: "Barnvårdsunderstöd",
    				moreInfo: "",
    				priceLabel: "FPA betalar"
    			},
    			compensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Arbetskompensation",
    				moreInfo: "",
    				priceLabel: "Arbetsersättning"
    			},
    			dailyAllowance: {
    				amountLabel: "Dagar",
    				description: "Dagsersättning, 40 euro, om arbetsresan är över 10 timmar eller 15 km.",
    				iconText: "40",
    				label: "Dagsersättning",
    				moreInfo: "",
    				priceLabel: "Euro / dag"
    			},
    			dailyAllowanceHalf: {
    				amountLabel: "Dagar",
    				description: "Halvdagsersättning, 19 euro, om arbetsresan är över 6 timmar eller 15 km.",
    				iconText: 19,
    				label: "Halvdagsersättning",
    				moreInfo: "",
    				priceLabel: "Euro / dag"
    			},
    			employmentTermination: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			eveningAddition: {
    				amountLabel: "Timmar",
    				description: "Ersättning för kvällsarbete enligt kollektivavtal.",
    				iconText: "KVÄLL",
    				label: "Kvällstillägg",
    				moreInfo: "",
    				priceLabel: "Tillägg (per timme)"
    			},
    			expenses: {
    				amountLabel: "",
    				description: "Ersättning för utlägg som arbetstagaren gjort för arbetsgivarens del (resekostnader, kontorsmaterial etc.).",
    				iconText: "kostnad",
    				label: "Ersättning för utlägg",
    				moreInfo: "",
    				priceLabel: "Erssättning för kostnader"
    			},
    			foreclosure: {
    				amountLabel: "",
    				description: "Löneerhållning, betalt rakt till befallningsmannen.",
    				iconText: "",
    				label: "Löneerhållning",
    				moreInfo: "",
    				priceLabel: "Summa"
    			},
    			foreclosureByPalkkaus: {
    				amountLabel: "",
    				description: "Löneerhållning gjort av Palkkaus.fi, betalt rakt till befallningsmannen.",
    				iconText: "",
    				label: "Löneerhållning av Palkkaus.fi",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			holidayBonus: {
    				amountLabel: "Procent",
    				description: "Semesterersättning enligt rådande kollektivavtal.",
    				iconText: "Kollektivavtal",
    				label: "Semesterersättning",
    				moreInfo: "",
    				priceLabel: "Lön för semestertid"
    			},
    			holidayCompensation: {
    				amountLabel: "Procent",
    				description: "Ersättning som betalas ut ifall man inte håller sin årliga semester.",
    				iconText: "11,5%",
    				label: "Semestertillägg",
    				moreInfo: "",
    				priceLabel: "Lönesumma"
    			},
    			holidaySalary: {
    				amountLabel: "",
    				description: "Lön för årlig semester.",
    				iconText: "semester",
    				label: "Semesterlön",
    				moreInfo: "",
    				priceLabel: "Lön för semestertid"
    			},
    			hourlySalary: {
    				amountLabel: "",
    				description: "Lönebetalning baserat på insatta timmar.",
    				iconText: "timme",
    				label: "Timlön",
    				moreInfo: "",
    				priceLabel: "Timlön"
    			},
    			irIncomeType: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			mealBenefit: {
    				amountLabel: "",
    				description: "Beskattningsbar måltidsförmån.",
    				iconText: "mat",
    				label: "Måltidsförmån",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			mealCompensation: {
    				amountLabel: "Dagar",
    				description: "Måltidsersättning när dagsersättning inte kan betalas.",
    				iconText: "måltid",
    				label: "Måltidsersättning",
    				moreInfo: "",
    				priceLabel: "Ersättning / dag"
    			},
    			milageDaily: {
    				amountLabel: "Dagar",
    				description: "Kilometerersättning enligt rådande kollektivavtal.",
    				iconText: "km",
    				label: "Daglig kilometerersättning",
    				moreInfo: "",
    				priceLabel: "Ersättning / dag"
    			},
    			milageOther: {
    				amountLabel: "Dagar",
    				description: "Övrig/annan kilometerersättning.",
    				iconText: "km",
    				label: "Annan kilometerersättning",
    				moreInfo: "",
    				priceLabel: "Ersättning / dag"
    			},
    			milageOwnCar: {
    				amountLabel: "",
    				description: "Kilometerersättning för bruk av egen bil.",
    				iconText: "0,43€",
    				label: "Kilometerersättning",
    				moreInfo: "",
    				priceLabel: "Ersättning / km"
    			},
    			monthlySalary: {
    				amountLabel: "",
    				description: "Fast månadslön.",
    				iconText: "månad",
    				label: "Månadslön",
    				moreInfo: "",
    				priceLabel: "Månatlig lön"
    			},
    			nightimeAddition: {
    				amountLabel: "Timmar",
    				description: "Ersättning för nattararbete enligt kollektivavtal.",
    				iconText: "NAT",
    				label: "Nattarbetstillägg",
    				moreInfo: "",
    				priceLabel: "Tillägg (per timme)"
    			},
    			nonProfitOrg: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherAdditions: {
    				amountLabel: "",
    				description: "Övriga tillägg enligt kollektivavtal.",
    				iconText: "tillägg",
    				label: "Övriga tillägg",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherBenefit: {
    				amountLabel: "",
    				description: "Övriga beskattningsbara förmåner.",
    				iconText: "Förmån",
    				label: "Övriga förmåner",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherCompensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			otherDeductions: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			overtime: {
    				amountLabel: "Procent",
    				description: "Övertidsersättningar.",
    				iconText: "över",
    				label: "Övertidskompensation",
    				moreInfo: "",
    				priceLabel: "Lönesumma"
    			},
    			phoneBenefit: {
    				amountLabel: "Månader",
    				description: "Beskattningsbar mobiltelefonförmån.",
    				iconText: 20,
    				label: "Mobiltelefonförmån",
    				moreInfo: "",
    				priceLabel: "Telefonförmån / mån"
    			},
    			prepaidExpenses: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			remuneration: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			salary: {
    				amountLabel: "",
    				description: "Klumpsumma för insatt arbete.",
    				iconText: "D",
    				label: "Lön eller provision",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			saturdayAddition: {
    				amountLabel: "Procent",
    				description: "Tillägg för lördagsarbete enligt kollektivavtal.",
    				iconText: "LÖR",
    				label: "Lördagstillägg",
    				moreInfo: "",
    				priceLabel: "Lönesumma"
    			},
    			subsidisedCommute: {
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			sundayWork: {
    				amountLabel: "",
    				description: "Tillägg för söndagsarbete enligt kollektivavtal.",
    				iconText: "SÖN",
    				label: "Söndags- och helgdagstillägg",
    				moreInfo: "",
    				priceLabel: "Lönesumma"
    			},
    			tesWorktimeShortening: {
    				amountLabel: "Procent",
    				description: "Skild lönedel som är definierad enligt kollektivavtal. Även känt som \"pekkasdagar\" på finska.",
    				iconText: "Kollektivavtal",
    				label: "Ersättning för arbetstidsutjämning",
    				moreInfo: "",
    				priceLabel: "Lönesumma"
    			},
    			toolCompensation: {
    				amountLabel: "Dagar",
    				description: "Ersättning för använding av egna arbetsrverktyg. Ersättningen enligt rådande kollektivavtal.",
    				iconText: "",
    				label: "Arbetsredskapsersättning",
    				moreInfo: "",
    				priceLabel: "Ersättning / dag"
    			},
    			totalEmployerPayment: {
    				amountLabel: "",
    				description: "Uträkning av totalkostnader: vad lönebetalningen kostar för arbetsgivaren.",
    				iconText: "sum",
    				label: "Total kostnad",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totals: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "Totalt",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			totalWorkerPayment: {
    				amountLabel: "",
    				description: "Uträkning av nettolönen: vad som blir kvar i handen efter skatter och andra avdrag.",
    				iconText: "net",
    				label: "Nettolön",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			unionPayment: {
    				amountLabel: "",
    				description: "Fackföreningsavgift betalt rakt till fackföreningen, om arbetstagaren så önskar.",
    				iconText: "Fackförening",
    				label: "Fackföreningsmedlemskap",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			unknown: {
    				amountLabel: "",
    				description: "Radtyp odefinierad.",
    				iconText: "?",
    				label: "Välj…",
    				moreInfo: "",
    				priceLabel: ""
    			},
    			workingTimeCompensation: {
    				amountLabel: "",
    				description: "",
    				iconText: "",
    				label: "",
    				moreInfo: "",
    				priceLabel: ""
    			}
    		},
    		CalculationRowUnit: {
    			count: {
    				description: "",
    				label: "st"
    			},
    			days: {
    				description: "",
    				label: "dagar"
    			},
    			kilometers: {
    				description: "",
    				label: "km"
    			},
    			percent: {
    				description: "",
    				label: "%"
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			}
    		},
    		CalculationStatus: {
    			draft: {
    				description: "",
    				label: "Utkast"
    			},
    			externalPaymentCanceled: {
    				description: "",
    				label: "Utbetalning avbruten utanför Salaxy-tjänsten"
    			},
    			externalPaymentError: {
    				description: "",
    				label: "Fel (utanför Salaxy-tjänsten)"
    			},
    			externalPaymentStarted: {
    				description: "",
    				label: "Utbetalning påbörjad utnaför Salaxy-tjänsten"
    			},
    			externalPaymentSucceeded: {
    				description: "",
    				label: "Utbetalning genomförd utanför Salaxy-tjänsten"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "Utbetalning avbruten"
    			},
    			paymentError: {
    				description: "",
    				label: "Fel i utbetalning"
    			},
    			paymentRefunded: {
    				description: "",
    				label: "Utbetalning avbruten och returnerad"
    			},
    			paymentStarted: {
    				description: "",
    				label: "Utbetalning påbörjad"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "Utbetalning genomförd"
    			},
    			paymentWorkerCopy: {
    				description: "",
    				label: "Arbetstagarens kvitto"
    			},
    			payrollDraft: {
    				description: "",
    				label: ""
    			},
    			proDraft: {
    				label: ""
    			},
    			sharedApproved: {
    				description: "",
    				label: ""
    			},
    			sharedRejected: {
    				description: "",
    				label: ""
    			},
    			sharedWaiting: {
    				description: "",
    				label: ""
    			},
    			template: {
    				description: "",
    				label: ""
    			},
    			workerRequestAccepted: {
    				description: "",
    				label: "Lönespecifikation gjord av arbetstagaren godkänd"
    			},
    			workerRequestDeclined: {
    				description: "",
    				label: "Lönespecifikation gjord av arbetstagaren avslagen"
    			},
    			workerRequested: {
    				description: "",
    				label: "Lönespecifikation gjord av arbetstagaren"
    			}
    		},
    		CarBenefitCode: {
    			fullCarBenefit: {
    				label: ""
    			},
    			limitedCarBenefit: {
    				label: ""
    			}
    		},
    		CarBenefitKind: {
    			fullCarBenefit: {
    				description: "",
    				label: ""
    			},
    			limitedCarBenefit: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		CompanyType: {
    			fiOy: {
    				description: "",
    				label: "Aktiebolag, även bostadsaktiebolag och publika aktiebolag"
    			},
    			fiRy: {
    				description: "",
    				label: "Föreningar"
    			},
    			fiTm: {
    				description: "",
    				label: "Privat näringsidkare eller jordbrukare"
    			},
    			fiYy: {
    				description: "",
    				label: "Annan, t.ex. Kommanditbolag, öppna bolag etc."
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		ContractPartyType: {
    			customContact: {
    				description: "",
    				label: ""
    			},
    			person: {
    				description: "",
    				label: ""
    			}
    		},
    		DailyAllowanceKind: {
    			fullDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			internationalDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			mealAllowance: {
    				description: "",
    				label: ""
    			},
    			partialDailyAllowance: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		DateOfBirthAccuracy: {
    			ageBased: {
    				description: "",
    				label: "Åldersbaserad"
    			},
    			ageGroupBased: {
    				description: "",
    				label: "Åldersgruppbaserad"
    			},
    			assumption: {
    				description: "",
    				label: "Antagande"
    			},
    			exact: {
    				description: "",
    				label: "Exakt datum"
    			},
    			monthCorrect: {
    				description: "",
    				label: "Rätt månad"
    			},
    			verified: {
    				description: "",
    				label: "Bekräftat"
    			}
    		},
    		EmployerType: {
    			fullTimeEmployer: {
    				description: "",
    				label: "Fortlöpande kontrakt eller över 8178 € / 6 månader"
    			},
    			miniEmployer: {
    				description: "",
    				label: "Under 1300 € / år"
    			},
    			smallEmployer: {
    				description: "",
    				label: "Minst 1300 € / år"
    			},
    			tyelOwnContractEmployer: {
    				description: "",
    				label: "ArPL-årskontrakt eller omskött av kunden"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			}
    		},
    		EmploymentContract: {
    			annualLeave: {
    				label: "Annual leave"
    			},
    			benefitsException: {
    				label: "Luontaisedut"
    			},
    			noticeException: {
    				label: "Term of notice"
    			},
    			otherTerms: {
    				label: "Other terms"
    			}
    		},
    		EmploymentRelationType: {
    			athlete: {
    				description: "",
    				label: ""
    			},
    			boardMember: {
    				description: "",
    				label: ""
    			},
    			compensation: {
    				description: "",
    				label: ""
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: ""
    			},
    			entrepreneur: {
    				description: "",
    				label: ""
    			},
    			entrepreneurNew: {
    				description: "",
    				label: ""
    			},
    			farmer: {
    				description: "",
    				label: ""
    			},
    			hourlySalary: {
    				description: "",
    				label: ""
    			},
    			keyEmployee: {
    				description: "",
    				label: ""
    			},
    			monthlySalary: {
    				description: "",
    				label: ""
    			},
    			performingArtist: {
    				description: "",
    				label: ""
    			},
    			salary: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		EmploymentSalaryType: {
    			hourlyPay: {
    				description: "",
    				label: "Timlön",
    				metricAbbrev: ""
    			},
    			montlyPay: {
    				description: "",
    				label: "Månadslön",
    				metricAbbrev: ""
    			},
    			otherPay: {
    				description: "",
    				label: "Annan"
    			},
    			unknown: {
    				description: "",
    				label: "Välj…"
    			}
    		},
    		EmploymentTerminationKind: {
    			monetaryWorkingTimeBankCompensation: {
    				description: "",
    				label: ""
    			},
    			noticePeriodCompensation: {
    				description: "",
    				label: ""
    			},
    			pensionPaidByEmployer: {
    				description: "",
    				label: ""
    			},
    			terminationAndLayOffDamages: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			voluntaryTerminationCompensation: {
    				description: "",
    				label: ""
    			}
    		},
    		EmploymentType: {
    			fullTime: {
    				description: "",
    				label: "Tills vidare anställning"
    			},
    			job: {
    				description: "",
    				label: "Engångsarbete"
    			},
    			partTime: {
    				description: "",
    				label: "Visstidsanställd"
    			},
    			unknown: {
    				description: "",
    				label: "Välj…"
    			}
    		},
    		EmploymentWorkHoursType: {
    			hourPerThreeWeeks: {
    				description: "",
    				label: "Timmar/tre veckor"
    			},
    			hourPerTwoWeeks: {
    				description: "",
    				label: "Timmar/två veckor"
    			},
    			hoursPerDay: {
    				description: "",
    				label: "Timmar/dygn"
    			},
    			hoursPerWeek: {
    				description: "",
    				label: "Timmar/vecka"
    			}
    		},
    		FrameworkAgreement: {
    			childCare: {
    				description: "Tills vidare barnskötare och där tillhörande stöd.",
    				label: "Övrig barnomsorg"
    			},
    			cleaning: {
    				description: "Städ- och fastighetsarbete.",
    				label: "Städ- och fastighetsbranschen"
    			},
    			construction: {
    				description: "Arbete enligt kollektivavtalet för byggnadsbranschen.",
    				label: "Byggnads- och renoveringsarbete"
    			},
    			entrepreneur: {
    				description: "Företagarens egen lön.",
    				label: "Företagare"
    			},
    			mll: {
    				description: "Lön för MLL:s barnskötare.",
    				label: "MLL rekommendationer"
    			},
    			notDefined: {
    				description: "Kollektivavtal eller rekommendation odefinierat.",
    				label: "Odefinierad"
    			},
    			other: {
    				description: "Annat arbete eller inget kollektivavtal.",
    				label: "Övrigt arbete"
    			},
    			santaClaus: {
    				description: "Hembesök av jultomten.",
    				label: "Jultomte"
    			}
    		},
    		Gender: {
    			female: {
    				description: "",
    				label: "Kvinna"
    			},
    			male: {
    				description: "",
    				label: "Man"
    			},
    			unknown: {
    				description: "",
    				label: ""
    			}
    		},
    		HelttiIndustry: {
    			i: {
    				description: "",
    				label: "Boende- och näringsverksamhet"
    			},
    			j: {
    				description: "",
    				label: "Informationssystem och kommunikation"
    			},
    			k: {
    				description: "",
    				label: "Finans- och försäkringsverksamhet"
    			},
    			m: {
    				description: "",
    				label: "Professional, scientific and technical activities"
    			},
    			n: {
    				description: "",
    				label: "Administrations- och stödverksamhet"
    			},
    			notDefined: {
    				description: "",
    				label: "Välj…"
    			},
    			o: {
    				description: "",
    				label: "Offentlig förvaltning och nationellt försvar"
    			},
    			other: {
    				description: "",
    				label: "Övriga industrier"
    			},
    			q: {
    				description: "",
    				label: "Hälso- och socialverksamhet"
    			},
    			r: {
    				description: "",
    				label: "Konst, nöje och rekration"
    			},
    			s: {
    				description: "",
    				label: "Övrig serviceverksamhet"
    			},
    			p: {
    				description: "",
    				label: "P"
    			}
    		},
    		HelttiProductPackage: {
    			large: {
    				description: "",
    				label: "Hifi"
    			},
    			medium: {
    				description: "",
    				label: "Tillväxt"
    			},
    			notDefined: {
    				description: "",
    				label: "Inget kontrakt"
    			},
    			small: {
    				description: "",
    				label: "Start"
    			}
    		},
    		HolidayAccrualKind: {
    			calculation: {
    				label: ""
    			},
    			holiday: {
    				label: ""
    			},
    			holidayCompensated: {
    				label: ""
    			},
    			manual: {
    				label: ""
    			},
    			saldo: {
    				label: ""
    			}
    		},
    		HolidayBonusPaymentMethod: {
    			none: {
    				label: ""
    			},
    			pay24Days: {
    				label: ""
    			},
    			payAllBonus: {
    				label: ""
    			},
    			payForHolidaySalary: {
    				label: ""
    			},
    			paySummerBonus: {
    				label: ""
    			}
    		},
    		HolidayCode: {
    			holidayCompensation: {
    				description: "",
    				label: ""
    			},
    			holidayCompensationIncluded: {
    				description: "",
    				label: ""
    			},
    			noHolidays: {
    				description: "",
    				label: ""
    			},
    			permanent14Days: {
    				description: "",
    				label: ""
    			},
    			permanent35Hours: {
    				description: "",
    				label: ""
    			},
    			temporaryTimeOff: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		HolidayYearInitType: {
    			createEmpty: {
    				label: ""
    			},
    			createFromCalcs: {
    				label: ""
    			},
    			createMonths: {
    				label: ""
    			},
    			ignore: {
    				label: ""
    			}
    		},
    		HouseHoldUseCaseTree: {
    			childCare: {
    				description: "",
    				kela: {
    					description: "",
    					label: ""
    				},
    				label: "",
    				mll: {
    					description: "",
    					label: ""
    				},
    				other: {
    					description: "",
    					label: ""
    				}
    			},
    			cleaning: {
    				"51530": {
    					description: "",
    					label: ""
    				},
    				"53221": {
    					description: "",
    					label: ""
    				},
    				"61132": {
    					description: "",
    					label: ""
    				},
    				"91110": {
    					description: "",
    					label: ""
    				},
    				description: "",
    				label: "",
    				other: {
    					description: "",
    					label: ""
    				}
    			},
    			construction: {
    				carpenter: {
    					description: "",
    					label: ""
    				},
    				description: "",
    				floor: {
    					description: "",
    					label: ""
    				},
    				label: "",
    				"new": {
    					description: "",
    					label: ""
    				},
    				painter: {
    					description: "",
    					label: ""
    				},
    				renovation: {
    					description: "",
    					label: ""
    				},
    				roof: {
    					description: "",
    					label: ""
    				}
    			},
    			other: {
    				"51530": {
    					description: "",
    					label: "Fastighetsskötare"
    				},
    				"53112": {
    					description: "",
    					label: "Familjedagvårdare"
    				},
    				"53212": {
    					description: "",
    					label: "Vårdare av utvecklingshämmade"
    				},
    				"53213": {
    					description: "",
    					label: "Vårdare inom socialsektorn"
    				},
    				"53221": {
    					description: "",
    					label: "Hemvårdspersonal"
    				},
    				"53222": {
    					description: "",
    					label: "Personliga assistenter, närståendevårdare m.fl."
    				},
    				"61112": {
    					description: "",
    					label: "Arbetsledare och arbetare inom åkerodling"
    				},
    				"61132": {
    					description: "",
    					label: "Arbetsledare och anställda inom trädgårds- och växthusodling"
    				},
    				"62100": {
    					description: "",
    					label: "Skogsbrukare och skogsarbetare"
    				},
    				"71110": {
    					description: "",
    					label: "Byggnadsarbetare"
    				},
    				"71150": {
    					description: "",
    					label: "Timmermän och byggnadssnickare"
    				},
    				"71210": {
    					description: "",
    					label: "Takmontörer och takreparatörer"
    				},
    				"71310": {
    					description: "",
    					label: "Byggnadsmålare m.fl."
    				},
    				"91110": {
    					description: "",
    					label: "Hembiträden och städare"
    				},
    				description: "",
    				label: "Annan",
    				"undefined": {
    					description: "",
    					label: "Annan"
    				}
    			}
    		},
    		InsuranceCompany: {
    			aktia: {
    				description: "",
    				label: "Aktia"
    			},
    			alandia: {
    				description: "",
    				label: "Alandia"
    			},
    			aVakuutus: {
    				description: "",
    				label: "A-Vakuutus"
    			},
    			fennia: {
    				description: "",
    				label: "Fennia"
    			},
    			folksam: {
    				description: "",
    				label: "Folksam"
    			},
    			"if": {
    				description: "",
    				label: "If"
    			},
    			"lähiTapiola": {
    				description: "",
    				label: "LähiTapiola"
    			},
    			none: {
    				description: "",
    				label: "(Välj försäkringsbolag)"
    			},
    			other: {
    				description: "",
    				label: "Annat bolag"
    			},
    			"pohjantähti": {
    				description: "",
    				label: "Pohjantähti"
    			},
    			pohjola: {
    				description: "",
    				label: "OP Vakuutus"
    			},
    			redarnas: {
    				description: "",
    				label: "Redarnas"
    			},
    			tryg: {
    				description: "",
    				label: "Tryg"
    			},
    			turva: {
    				description: "",
    				label: "Turva"
    			},
    			"ålands": {
    				description: "",
    				label: "Ålands"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			}
    		},
    		InvoiceStatus: {
    			canceled: {
    				description: "",
    				label: ""
    			},
    			error: {
    				description: "",
    				label: ""
    			},
    			forecast: {
    				description: "",
    				label: ""
    			},
    			paid: {
    				description: "",
    				label: ""
    			},
    			paymentStarted: {
    				description: "",
    				label: ""
    			},
    			preview: {
    				description: "",
    				label: ""
    			},
    			read: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			unread: {
    				description: "",
    				label: ""
    			},
    			waitingConfirmation: {
    				description: "",
    				label: ""
    			},
    			waitingPalkkaus: {
    				description: "",
    				label: ""
    			}
    		},
    		IrFlags: {
    			noMoney: {
    				label: ""
    			},
    			oneOff: {
    				label: ""
    			},
    			unjustEnrichment: {
    				label: ""
    			}
    		},
    		IrInsuranceExceptions: {
    			excludeAccidentInsurance: {
    				label: ""
    			},
    			excludeAll: {
    				label: ""
    			},
    			excludeHealthInsurance: {
    				label: ""
    			},
    			excludePension: {
    				label: ""
    			},
    			excludeUnemployment: {
    				label: ""
    			},
    			includeAccidentInsurance: {
    				label: ""
    			},
    			includeAll: {
    				label: ""
    			},
    			includeHealthInsurance: {
    				label: ""
    			},
    			includePension: {
    				label: ""
    			},
    			includeUnemployment: {
    				label: ""
    			}
    		},
    		LegalEntityType: {
    			company: {
    				description: "",
    				label: "FO-nummer"
    			},
    			partner: {
    				description: "",
    				label: "Partner"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			personCreatedByEmployer: {
    				description: "",
    				label: "Arbetstagare"
    			},
    			"undefined": {
    				description: "",
    				label: "Okänd"
    			}
    		},
    		MealBenefitKind: {
    			cateringContract: {
    				description: "",
    				label: ""
    			},
    			institute: {
    				description: "",
    				label: ""
    			},
    			mealAllowance: {
    				description: "",
    				label: ""
    			},
    			mealTicket: {
    				description: "",
    				label: ""
    			},
    			restaurantWorker: {
    				description: "",
    				label: ""
    			},
    			taxableAmount: {
    				description: "",
    				label: ""
    			},
    			teacher: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		MessageType: {
    			email: {
    				description: "",
    				label: "E-post"
    			},
    			sms: {
    				description: "",
    				label: "Textmeddelande"
    			}
    		},
    		NonProfitOrgKind: {
    			accomodationAllowance: {
    				description: "",
    				label: ""
    			},
    			dailyAllowance: {
    				description: "",
    				label: ""
    			},
    			kilometreAllowance: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		OnboardingStatus: {
    			created: {
    				description: "",
    				label: "Skapad"
    			},
    			done: {
    				description: "",
    				label: "Slutförd"
    			},
    			opdescription: "",
    			oplabel: "Pågående",
    			expired: {
    				description: "",
    				label: "Expired"
    			},
    			open: {
    				description: "",
    				label: "Open"
    			}
    		},
    		OtherCompensationKind: {
    			accruedTimeOffCompensation: {
    				description: "",
    				label: ""
    			},
    			capitalIncomePayment: {
    				description: "",
    				label: ""
    			},
    			employeeInventionCompensation: {
    				description: "",
    				label: ""
    			},
    			employeeStockOption: {
    				description: "",
    				label: ""
    			},
    			employeeStockOptionWithLowerPrice: {
    				description: "",
    				label: ""
    			},
    			lectureFee: {
    				description: "",
    				label: ""
    			},
    			meetingFee: {
    				description: "",
    				label: ""
    			},
    			membershipOfGoverningBodyCompensation: {
    				description: "",
    				label: ""
    			},
    			monetaryGiftForEmployees: {
    				description: "",
    				label: ""
    			},
    			otherTaxableIncomeAsEarnedIncome: {
    				description: "",
    				label: ""
    			},
    			positionOfTrustCompensation: {
    				description: "",
    				label: ""
    			},
    			stockOptionsAndGrants: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			useCompensationAsCapitalIncome: {
    				description: "",
    				label: ""
    			},
    			useCompensationAsEarnedIncome: {
    				description: "",
    				label: ""
    			},
    			workEffortBasedDividendsAsNonWage: {
    				description: "",
    				label: ""
    			},
    			workEffortBasedDividendsAsWage: {
    				description: "",
    				label: ""
    			}
    		},
    		PalkkausCampaignPricing: {
    			campaignAfterFirstUse: {
    				description: "",
    				label: ""
    			},
    			campaignFirstUse: {
    				description: "",
    				label: ""
    			},
    			companyFreeTrial: {
    				description: "",
    				label: ""
    			},
    			none: {
    				description: "",
    				label: ""
    			}
    		},
    		PartnerSite: {
    			careDotCom: {
    				description: "",
    				label: ""
    			},
    			dev: {
    				description: "",
    				label: ""
    			},
    			duunitori: {
    				description: "",
    				label: ""
    			},
    			example: {
    				description: "",
    				label: ""
    			},
    			mol: {
    				description: "",
    				label: ""
    			},
    			palkkaapakolaindescription: "",
    			palkkaapakolainlabel: "",
    			palkkaus: {
    				description: "",
    				label: ""
    			},
    			palkkausEmployerFillIn: {
    				description: "",
    				label: ""
    			},
    			palkkausIdentified: {
    				description: "",
    				label: ""
    			},
    			raksa: {
    				description: "",
    				label: ""
    			},
    			refugeeJobs: {
    				description: "",
    				label: ""
    			},
    			talkkarit: {
    				description: "",
    				label: ""
    			},
    			unknown: {
    				description: "",
    				label: ""
    			},
    			palkkaapakolainen: {
    				description: "",
    				label: "Palkkaapakolainen"
    			}
    		},
    		PaymentChannel: {
    			accountorGo: {
    				description: "",
    				label: ""
    			},
    			finagoSolo: {
    				description: "",
    				label: ""
    			},
    			holviCfa: {
    				description: "",
    				label: ""
    			},
    			kevytyrittaja: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaFinvoice: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaPaytrail: {
    				description: "",
    				label: ""
    			},
    			palkkausCfaReference: {
    				description: "",
    				label: ""
    			},
    			palkkausManual: {
    				description: "",
    				label: ""
    			},
    			palkkausWS: {
    				description: "",
    				label: ""
    			},
    			palkkausPersonal: {
    				description: "",
    				label: ""
    			},
    			procountor: {
    				description: "",
    				label: ""
    			},
    			talenomCfa: {
    				description: "",
    				label: ""
    			},
    			talenomOnline: {
    				description: "",
    				label: ""
    			},
    			test: {
    				description: "",
    				label: ""
    			},
    			zeroPayment: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentStatus: {
    			bankDelivered: {
    				description: "",
    				label: "Godkänt av banken"
    			},
    			bankError: {
    				description: "",
    				label: "Fel hos banken"
    			},
    			bankPartialError: {
    				description: "",
    				label: "Fel hos banken"
    			},
    			bankTechApproval: {
    				description: "",
    				label: "Mottaget av banken"
    			},
    			cancelled: {
    				description: "",
    				label: "Betalning avrbuten"
    			},
    			"new": {
    				description: "",
    				label: ""
    			},
    			paid: {
    				description: "",
    				label: "Betalning genomförd"
    			},
    			sentToBank: {
    				description: "",
    				label: "Skickat till banken"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			}
    		},
    		PayrollStatus: {
    			draft: {
    				label: "Utkast"
    			},
    			paymentCanceled: {
    				label: "Utbetalning avbruten"
    			},
    			paymentError: {
    				label: "Fel i utbetalning"
    			},
    			paymentStarted: {
    				label: "Utbetalning påbörjad"
    			},
    			paymentSucceeded: {
    				label: "Utbetalning genomförd"
    			},
    			template: {
    				label: ""
    			}
    		},
    		PensionCalculation: {
    			athlete: {
    				description: "",
    				label: ""
    			},
    			boardRemuneration: {
    				description: "",
    				label: ""
    			},
    			compensation: {
    				description: "",
    				label: ""
    			},
    			employee: {
    				description: "",
    				label: ""
    			},
    			entrepreneur: {
    				description: "",
    				label: ""
    			},
    			farmer: {
    				description: "",
    				label: ""
    			},
    			partialOwner: {
    				description: "",
    				label: ""
    			},
    			smallEntrepreneur: {
    				description: "",
    				label: ""
    			},
    			smallFarmer: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		PensionCompany: {
    			apteekkien: {
    				description: "",
    				label: "Apteekkien Eläkekassa"
    			},
    			elo: {
    				description: "",
    				label: "Elo"
    			},
    			etera: {
    				description: "",
    				label: "Etera"
    			},
    			ilmarindescription: "",
    			ilmarinen: {
    				label: "Ilmarinen"
    			},
    			none: {
    				description: "",
    				label: "Välj…"
    			},
    			other: {
    				description: "",
    				label: "Annat bolag"
    			},
    			pensionsAlandia: {
    				description: "",
    				label: "Alandia"
    			},
    			varma: {
    				description: "",
    				label: "Varma"
    			},
    			veritas: {
    				description: "",
    				label: "Veritas"
    			},
    			verso: {
    				description: "",
    				label: "Verso"
    			}
    		},
    		PeriodType: {
    			custom: {
    				label: ""
    			},
    			month: {
    				label: ""
    			},
    			quarter: {
    				label: ""
    			},
    			year: {
    				label: ""
    			}
    		},
    		PricingModel: {
    			fixedFee: {
    				label: ""
    			},
    			noFee: {
    				label: ""
    			},
    			palkkausFee: {
    				label: ""
    			}
    		},
    		Product: {
    			accounting: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			baseService: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			healthCareHeltti: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			insurance: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			pension: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			pensionEntrepreneur: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			tax: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			taxCard: {
    				description: "",
    				label: "",
    				status: ""
    			},
    			unemployment: {
    				description: "",
    				label: "",
    				status: "",
    				statusTVR: ""
    			}
    		},
    		ProductListFilter: {
    			all: {
    				description: "",
    				label: ""
    			},
    			available: {
    				description: "",
    				label: ""
    			}
    		},
    		Profession: {
    			"0": {
    				label: "Militärer"
    			},
    			"1": {
    				label: "Chefer"
    			},
    			"2": {
    				label: "Specialister"
    			},
    			"3": {
    				label: "Experter"
    			},
    			"4": {
    				label: "Kontors- och kundtjänstpersonal"
    			},
    			"5": {
    				label: "Service- och försäljningspersonal"
    			},
    			"6": {
    				label: "Jordbrukare, skogsarbetare m.fl."
    			},
    			"7": {
    				label: "Byggnads-, reparations- och tillverkningsarbetare"
    			},
    			"8": {
    				label: "Process- och transportarbetare"
    			},
    			"9": {
    				label: "Övriga arbetstagare"
    			},
    			"11": {
    				label: "Chefer, högre tjänstemän och organisationschefer"
    			},
    			"12": {
    				label: "Administrativa och kommersiella chefer"
    			},
    			"13": {
    				label: "Chefer inom tillverkning, utbildning, vård m.m."
    			},
    			"14": {
    				label: "Chefer inom hotell och restaurangbranschen, detaljhandel och andra servicenäringar"
    			},
    			"21": {
    				label: "Specialister inom naturvetenskap och teknik"
    			},
    			"22": {
    				label: "Hälso- och sjukvårdsspecialister"
    			},
    			"23": {
    				label: "Lärare och andra specialister inom undervisningsbranschen"
    			},
    			"24": {
    				label: "Specialister inom företagsekonomi och administration"
    			},
    			"25": {
    				label: "Specialister inom informations- och kommunikationsteknologi"
    			},
    			"26": {
    				label: "Specialister inom juridik samt inom social- och kultursektorn"
    			},
    			"31": {
    				label: "Experter inom naturvetenskap och teknik"
    			},
    			"32": {
    				label: "Hälso- och sjukvårdsexperter"
    			},
    			"33": {
    				label: "Experter inom företagsekonomi och administration"
    			},
    			"34": {
    				label: "Juridiska assistenter samt experter inom social- och kultursektorn"
    			},
    			"35": {
    				label: "Experter inom informations- och kommunikationsteknologi"
    			},
    			"41": {
    				label: "Kontorspersonal"
    			},
    			"42": {
    				label: "Kundtjänstpersonal"
    			},
    			"43": {
    				label: "Kontorspersonal inom ekonomi- och lagerförvaltning"
    			},
    			"44": {
    				label: "Annan kontors- och kundtjänstpersonal"
    			},
    			"51": {
    				label: "Servicepersonal"
    			},
    			"52": {
    				label: "Försäljare, butiksinnehavare m.fl."
    			},
    			"53": {
    				label: "Vård- och omsorgspersonal"
    			},
    			"54": {
    				label: "Säkerhets- och bevakningspersonal"
    			},
    			"61": {
    				label: "Jordbrukare och djuruppfödare m.fl."
    			},
    			"62": {
    				label: "Arbetare inom skogsbruk och fiske"
    			},
    			"63": {
    				label: "Växtodlare, fiskare och jägare (självhushållning)"
    			},
    			"71": {
    				label: "Byggnadsarbetare m.fl. (utom elmontörer)"
    			},
    			"72": {
    				label: "Verkstads- och gjuteriarbetare samt montörer och reparatörer"
    			},
    			"73": {
    				label: "Konsthantverkare, finmekaniker och tryckeripersonal"
    			},
    			"74": {
    				label: "Personal inom el- och elektronikbranschen"
    			},
    			"75": {
    				label: "Personal inom livsmedel, trä, textil m.m."
    			},
    			"81": {
    				label: "Processarbetare"
    			},
    			"82": {
    				label: "Hopsättare, industriprodukter"
    			},
    			"83": {
    				label: "Transportarbetare"
    			},
    			"91": {
    				label: "Städare, hembiträden och andra rengöringsarbetare"
    			},
    			"92": {
    				label: "Medhjälpare inom jordbruk, skogsbruk och fiske"
    			},
    			"93": {
    				label: "Medhjälpare inom industrin och byggverksamhet m.fl."
    			},
    			"94": {
    				label: "Köks- och restaurangbiträden"
    			},
    			"95": {
    				label: "Gatuförsäljare, skoputsare m.fl."
    			},
    			"96": {
    				label: "Gaturenhållningsarbetare och avfallshanterare m.fl."
    			},
    			"111": {
    				label: "Högre tjänstemän och organisationschefer"
    			},
    			"112": {
    				label: "Verkställande direktörer och verkschefer"
    			},
    			"121": {
    				label: "Chefer inom företagstjänster och administration"
    			},
    			"122": {
    				label: "Försäljnings-, marknadsförings- och utvecklingschefer"
    			},
    			"131": {
    				label: "Chefer inom jordbruk, skogsbruk och fiske"
    			},
    			"132": {
    				label: "Produktionschefer inom industri, gruvdrift, byggverksamhet och distribution"
    			},
    			"133": {
    				label: "Chefer inom informations- och kommunikationsteknologi (IKT)"
    			},
    			"134": {
    				label: "Chefer inom utbildning, vård, omsorg och finansiell verksamhet"
    			},
    			"141": {
    				label: "Hotell- och restaurangchefer"
    			},
    			"142": {
    				label: "Chefer inom detalj- och partihandel"
    			},
    			"143": {
    				label: "Chefer inom andra servicenäringar"
    			},
    			"211": {
    				label: "Specialister inom naturvetenskap och geovetenskap"
    			},
    			"212": {
    				label: "Specialister inom matematik och statistik"
    			},
    			"213": {
    				label: "Specialister inom biovetenskap"
    			},
    			"214": {
    				label: "Specialister inom teknik (utom elektroteknik)"
    			},
    			"215": {
    				label: "Specialister inom elektroteknik"
    			},
    			"216": {
    				label: "Arkitekter, planerare och lantmätare"
    			},
    			"221": {
    				label: "Läkare"
    			},
    			"222": {
    				label: "Specialister inom vårdarbete"
    			},
    			"223": {
    				label: "Specialister inom traditionella och alternativa vårdformer"
    			},
    			"224": {
    				label: "Producenter av andra hälsovårdstjänster"
    			},
    			"225": {
    				label: "Veterinärer"
    			},
    			"226": {
    				label: "Andra hälso- och sjukvårdsspecialister"
    			},
    			"231": {
    				label: "Universitets- och högskollärare"
    			},
    			"232": {
    				label: "Lärare i yrkesämnen"
    			},
    			"233": {
    				label: "Gymnasie- och högstadielärare"
    			},
    			"234": {
    				label: "Lågstadie-och barnträdgårdslärare"
    			},
    			"235": {
    				label: "Andra specialister inom undervisning"
    			},
    			"241": {
    				label: "Specialister inom finanssektorn"
    			},
    			"242": {
    				label: "Specialister inom administration"
    			},
    			"243": {
    				label: "Specialister inom försäljning, marknadsföring och information"
    			},
    			"251": {
    				label: "Specialister inom systemarbete"
    			},
    			"252": {
    				label: "Specialister, databaser, -nätverk och systemprogram"
    			},
    			"261": {
    				label: "Juridiska specialister"
    			},
    			"262": {
    				label: "Bibliotekarier, arkivarier och specialister inom museisektorn"
    			},
    			"263": {
    				label: "Specialister inom samhälls- och socialsektorn samt inom religion"
    			},
    			"264": {
    				label: "Journalister, författare och språkvetare"
    			},
    			"265": {
    				label: "Konstnärer"
    			},
    			"311": {
    				label: "Experter inom fysik, kemi och teknik"
    			},
    			"312": {
    				label: "Arbetsledare inom gruvdrift, industri och byggverksamhet"
    			},
    			"313": {
    				label: "Experter inom processövervakning"
    			},
    			"314": {
    				label: "Experter inom biovetenskap"
    			},
    			"315": {
    				label: "Fartygs-, flyg- och hamntrafikbefäl och -ledare"
    			},
    			"321": {
    				label: "Tekniska experter inom hälso- och sjukvården"
    			},
    			"322": {
    				label: "Sjukskötare, barnmorskor m.fl."
    			},
    			"323": {
    				label: "Utövare av naturmedicin och alternativ medicin"
    			},
    			"324": {
    				label: "Seminörer m.fl."
    			},
    			"325": {
    				label: "Andra hälso- och sjukvårdsexperter"
    			},
    			"331": {
    				label: "Experter inom finansiering, försäkring och redovisning"
    			},
    			"332": {
    				label: "Inköps- och försäljningsagenter"
    			},
    			"333": {
    				label: "Förmedlare av företagstjänster"
    			},
    			"334": {
    				label: "Administrativa och specialiserade sekreterare"
    			},
    			"335": {
    				label: "Tjänstemän inom beredning och bevakning i den offentliga förvaltningen"
    			},
    			"341": {
    				label: "Juridiska assistenter samt personal inom socialsektorn och församlingar"
    			},
    			"342": {
    				label: "Idrottare, idrottstränare och -instruktörer m.fl."
    			},
    			"343": {
    				label: "Experter inom konst och kultur samt kökschefer"
    			},
    			"351": {
    				label: "IKT-tekniker samt användarstöd"
    			},
    			"352": {
    				label: "Teletekniker samt radio- och tv-tekniker"
    			},
    			"411": {
    				label: "Kontorsassistenter"
    			},
    			"412": {
    				label: "Kontorssekreterare"
    			},
    			"413": {
    				label: "Textbehandlare och dataregistrerare"
    			},
    			"421": {
    				label: "Kassapersonal m.fl."
    			},
    			"422": {
    				label: "Annan kundtjänstpersonal"
    			},
    			"431": {
    				label: "Löneräknare, försäkringshandläggare m.fl."
    			},
    			"432": {
    				label: "Kontorspersonal inom transport och lager"
    			},
    			"441": {
    				label: "Annan kontors- och kundtjänstpersonal"
    			},
    			"511": {
    				label: "Personal inom resetjänster, konduktörer och guider"
    			},
    			"512": {
    				label: "Restaurang- och storhushållspersonal"
    			},
    			"513": {
    				label: "Serveringspersonal"
    			},
    			"514": {
    				label: "Frisörer, kosmetologer m.fl."
    			},
    			"515": {
    				label: "Arbetsledare inom städning och fastighetsskötsel"
    			},
    			"516": {
    				label: "Annan servicepersonal, personliga tjänster"
    			},
    			"521": {
    				label: "Gatu- och torgförsäljare"
    			},
    			"522": {
    				label: "Försäljare och butiksinnehavare"
    			},
    			"523": {
    				label: "Butikskassörer och biljettförsäljare"
    			},
    			"524": {
    				label: "Annan försäljningspersonal"
    			},
    			"531": {
    				label: "Barnskötare och skolgångsbiträden"
    			},
    			"532": {
    				label: "Närvårdare, annan vårdpersonal och hemvårdare"
    			},
    			"541": {
    				label: "Säkerhets- och bevakningspersonal"
    			},
    			"611": {
    				label: "Åker- och trädgårdsodlare"
    			},
    			"612": {
    				label: "Djuruppfödare"
    			},
    			"613": {
    				label: "Växtodlare och djuruppfödare, blandad drift"
    			},
    			"621": {
    				label: "Skogsbrukare och skogsarbetare"
    			},
    			"622": {
    				label: "Fiskodlare, fiskare och jägare"
    			},
    			"631": {
    				label: "Växtodlare (självhushållning)"
    			},
    			"632": {
    				label: "Boskapsuppfödare (självhushållning)"
    			},
    			"633": {
    				label: "Växtodlare och boskapsuppfödare (självhushållning)"
    			},
    			"634": {
    				label: "Fiskare och jägare (självhushållning)"
    			},
    			"711": {
    				label: "Byggnadsarbetare m.fl."
    			},
    			"712": {
    				label: "Byggnadshantverkare"
    			},
    			"713": {
    				label: "Målare och saneringsarbetare"
    			},
    			"721": {
    				label: "Gjuteriarbetare, svetsare, plåtslagare m.fl."
    			},
    			"722": {
    				label: "Smeder, verktygsmakare och maskinställare"
    			},
    			"723": {
    				label: "Maskinmontörer och reparatörer"
    			},
    			"731": {
    				label: "Konsthantverkare och finmekaniker"
    			},
    			"732": {
    				label: "Tryckeripersonal"
    			},
    			"741": {
    				label: "Elmontörer och -reparatörer"
    			},
    			"742": {
    				label: "Montörer och reparatörer inom elektronik och teleteknik"
    			},
    			"751": {
    				label: "Styckare, bagare, mejerister m.fl."
    			},
    			"752": {
    				label: "Trävaruhanterare, snickare m.fl."
    			},
    			"753": {
    				label: "Personal inom konfektionsbranschen m.fl."
    			},
    			"754": {
    				label: "Annan personal, oklassificerad under andra moment i huvudgrupp 7"
    			},
    			"811": {
    				label: "Maskinoperatörer i gruv- och stenbrottsarbete"
    			},
    			"812": {
    				label: "Processoperatörer och efterbehandlare, metallindustri"
    			},
    			"813": {
    				label: "Processoperatörer, kemisk industri och fotografiska produkter"
    			},
    			"814": {
    				label: "Processoperatörer, gummi-, plast- och pappersvaruindustri"
    			},
    			"815": {
    				label: "Processoperatörer, textil-, skinn- och läderindustri"
    			},
    			"816": {
    				label: "Processoperatörer, livsmedelsindustri"
    			},
    			"817": {
    				label: "Processoperatörer, sågvaror samt pappers- och kartongindustri"
    			},
    			"818": {
    				label: "Andra processoperatörer"
    			},
    			"821": {
    				label: "Hopsättare, industriprodukter"
    			},
    			"831": {
    				label: "Spårtrafikförare m.fl."
    			},
    			"832": {
    				label: "Person- och paketbilsförare samt motorcykelförare"
    			},
    			"833": {
    				label: "Förare av tunga motorfordon"
    			},
    			"834": {
    				label: "Arbetsmaskinförare"
    			},
    			"835": {
    				label: "Däckmanskap m.fl. sjötrafikarbetare"
    			},
    			"911": {
    				label: "Hem-, hotell- och kontorsstädare m.fl."
    			},
    			"912": {
    				label: "Fordonstvättare och fönsterputsare m.fl."
    			},
    			"921": {
    				label: "Medhjälpare inom jordbruk, skogsbruk och fiske"
    			},
    			"931": {
    				label: "Medhjälpare inom gruvdrift och byggverksamhet"
    			},
    			"932": {
    				label: "Medhjälpare inom tillverkning"
    			},
    			"933": {
    				label: "Godshanterare, lagerarbetare m.fl."
    			},
    			"941": {
    				label: "Köks- och restaurangbiträden"
    			},
    			"951": {
    				label: "Reklamutdelare, skoputsare m.fl."
    			},
    			"952": {
    				label: "Gatuförsäljare (utom livsmedel)"
    			},
    			"961": {
    				label: "Avfallshanterare"
    			},
    			"962": {
    				label: "Tidningsutdelare, bud m.fl."
    			},
    			"1111": {
    				label: "Lagstiftare"
    			},
    			"1112": {
    				label: "Högre tjänstemän inom offentlig förvaltning"
    			},
    			"1113": {
    				label: "Stam- och byhövdingar"
    			},
    			"1114": {
    				label: "Ledare i organisationssektorn"
    			},
    			"1120": {
    				label: "Verkställande direktörer och verkschefer"
    			},
    			"1211": {
    				label: "Ekonomichefer"
    			},
    			"1212": {
    				label: "Personalchefer"
    			},
    			"1213": {
    				label: "Politik- och planeringschefer"
    			},
    			"1219": {
    				label: "Övriga chefer inom företagstjänster och administration"
    			},
    			"1221": {
    				label: "Försäljnings- och marknadsföringschefer"
    			},
    			"1222": {
    				label: "Reklam- och kommunikationschefer, PR-chefer"
    			},
    			"1223": {
    				label: "Forsknings- och utvecklingschefer"
    			},
    			"1311": {
    				label: "Chefer inom jord- och skogsbruk"
    			},
    			"1312": {
    				label: "Chefer inom vattenbruk och fiske"
    			},
    			"1321": {
    				label: "Produktionschefer inom industri"
    			},
    			"1322": {
    				label: "Produktionschefer inom gruvdrift"
    			},
    			"1323": {
    				label: "Produktionschefer inom byggverksamhet"
    			},
    			"1324": {
    				label: "Inköps- och distributionschefer m.fl."
    			},
    			"1330": {
    				label: "Chefer inom informations- och kommunikationsteknik (IKT)"
    			},
    			"1341": {
    				label: "Chefer inom barnomsorg"
    			},
    			"1342": {
    				label: "Chefer inom hälso- och sjukvård"
    			},
    			"1343": {
    				label: "Chefer inom äldrevård"
    			},
    			"1344": {
    				label: "Chefer inom socialvård"
    			},
    			"1345": {
    				label: "Chefer inom utbildning"
    			},
    			"1346": {
    				label: "Chefer inom bank- och försäkringstjänster"
    			},
    			"1349": {
    				label: "Övriga chefer inom samhälleliga tjänster"
    			},
    			"1411": {
    				label: "Hotellchefer"
    			},
    			"1412": {
    				label: "Restaurangchefer"
    			},
    			"1420": {
    				label: "Chefer inom detalj- och partihandel"
    			},
    			"1431": {
    				label: "Chefer för sport-, fritids- och kulturcenter"
    			},
    			"1439": {
    				label: "Övriga chefer inom servicenäringar"
    			},
    			"2111": {
    				label: "Fysiker och astronomer"
    			},
    			"2112": {
    				label: "Meteorologer"
    			},
    			"2113": {
    				label: "Kemister"
    			},
    			"2114": {
    				label: "Geologer och geofysiker"
    			},
    			"2120": {
    				label: "Matematiker, aktuarier och statistiker"
    			},
    			"2131": {
    				label: "Biologer, botanister, zoologer m.fl."
    			},
    			"2132": {
    				label: "Specialister inom jordbruk, skogsbruk och fiske"
    			},
    			"2133": {
    				label: "Specialister inom miljö- och naturskydd"
    			},
    			"2141": {
    				label: "Specialister inom industriell produktion och produktionsteknik"
    			},
    			"2142": {
    				label: "Specialister inom anläggningsarbeten"
    			},
    			"2143": {
    				label: "Specialister inom miljöteknik"
    			},
    			"2144": {
    				label: "Specialister inom maskinteknik"
    			},
    			"2145": {
    				label: "Specialister inom träförädling och kemisk processteknik"
    			},
    			"2146": {
    				label: "Specialister inom gruvindustri, metallurgi m.fl."
    			},
    			"2149": {
    				label: "Övriga specialister inom teknik"
    			},
    			"2151": {
    				label: "Specialister inom elteknik"
    			},
    			"2152": {
    				label: "Specialister inom elektronik"
    			},
    			"2153": {
    				label: "Specialister inom IKT-branschen"
    			},
    			"2161": {
    				label: "Arkitekter inom husbyggande"
    			},
    			"2162": {
    				label: "Landskapsarkitekter"
    			},
    			"2163": {
    				label: "Produkt- och modedesigner"
    			},
    			"2164": {
    				label: "Samhälls- och trafikplanerare"
    			},
    			"2165": {
    				label: "Specialister inom kartläggning och lantmäteri"
    			},
    			"2166": {
    				label: "Grafiska designer och multimediadesigner"
    			},
    			"2211": {
    				label: "Allmänläkare"
    			},
    			"2212": {
    				label: "Överläkare och specialistläkare"
    			},
    			"2221": {
    				label: "Överskötare och avdelningsskötare"
    			},
    			"2222": {
    				label: "Ledande barnmorskor"
    			},
    			"2230": {
    				label: "Specialister inom traditionella och alternativa vårdformer"
    			},
    			"2240": {
    				label: "Producenter av andra hälsotjänster"
    			},
    			"2250": {
    				label: "Veterinärer"
    			},
    			"2261": {
    				label: "Tandläkare"
    			},
    			"2262": {
    				label: "Provisorer"
    			},
    			"2263": {
    				label: "Specialister inom miljöhälsa och arbetarskydd"
    			},
    			"2264": {
    				label: "Specialister inom fysioterapi"
    			},
    			"2265": {
    				label: "Specialister inom näringslära"
    			},
    			"2266": {
    				label: "Audiologer och logopeder"
    			},
    			"2267": {
    				label: "Specialister inom optometri"
    			},
    			"2269": {
    				label: "Övriga specialister inom hälso- och sjukvården utan annan klassificering"
    			},
    			"2310": {
    				label: "Universitets- och yrkeshögskolelärare"
    			},
    			"2320": {
    				label: "Lärare i yrkesämnen"
    			},
    			"2330": {
    				label: "Gymnasielärare och högstadielärare"
    			},
    			"2341": {
    				label: "Lågstadielärare"
    			},
    			"2342": {
    				label: "Barnträdgårdslärare"
    			},
    			"2351": {
    				label: "Specialister i pedagogiska metoder"
    			},
    			"2352": {
    				label: "Speciallärare"
    			},
    			"2353": {
    				label: "Övriga språklärare"
    			},
    			"2354": {
    				label: "Övriga musiklärare"
    			},
    			"2355": {
    				label: "Övriga lärare i konstnärliga ämnen"
    			},
    			"2356": {
    				label: "Övriga lärare och utbildare inom informationsteknik"
    			},
    			"2359": {
    				label: "Studiehandledare och övriga specialister inom undervisning"
    			},
    			"2411": {
    				label: "Specialister inom redovisning och revision"
    			},
    			"2412": {
    				label: "Finans- och investeringsrådgivare"
    			},
    			"2413": {
    				label: "Finansanalytiker"
    			},
    			"2421": {
    				label: "Specialister inom ledarskap och organisationer"
    			},
    			"2422": {
    				label: "Specialister inom utveckling av administration och näringar"
    			},
    			"2423": {
    				label: "Specialister inom personaladministration och karriärplanering"
    			},
    			"2424": {
    				label: "Specialister inom personalutveckling och utbildning"
    			},
    			"2431": {
    				label: "Specialister inom reklam och marknadsföring"
    			},
    			"2432": {
    				label: "Informatörer"
    			},
    			"2433": {
    				label: "Försäljningsingenjörer och läkemedelsrepresentanter (utom IKT)"
    			},
    			"2434": {
    				label: "Specialister inom försäljning av informations- och kommunikationsteknik (IKT)"
    			},
    			"2511": {
    				label: "Specialister inom applikationsarkitektur"
    			},
    			"2512": {
    				label: "Applikationsplanerare"
    			},
    			"2513": {
    				label: "Webb- och multimediautvecklare"
    			},
    			"2514": {
    				label: "Applikationsprogrammerare"
    			},
    			"2519": {
    				label: "Övriga program- och applikationsutvecklare"
    			},
    			"2521": {
    				label: "Databasutvecklare och databasansvariga"
    			},
    			"2522": {
    				label: "Systemadministratörer"
    			},
    			"2523": {
    				label: "Nätverksspecialister"
    			},
    			"2529": {
    				label: "Övriga databas- och nätverksspecialister"
    			},
    			"2611": {
    				label: "Advokater"
    			},
    			"2612": {
    				label: "Domare"
    			},
    			"2619": {
    				label: "Övriga specialister inom juridik"
    			},
    			"2621": {
    				label: "Arkivarier och specialister inom museisektorn"
    			},
    			"2622": {
    				label: "Bibliotekarier, informatiker m.fl."
    			},
    			"2631": {
    				label: "Nationalekonomer"
    			},
    			"2632": {
    				label: "Samhälls- och kulturforskare"
    			},
    			"2633": {
    				label: "Historiker, statsvetare och filosofer"
    			},
    			"2634": {
    				label: "Psykologer"
    			},
    			"2635": {
    				label: "Specialister inom socialt arbete"
    			},
    			"2636": {
    				label: "Präster och övriga specialister inom religion"
    			},
    			"2641": {
    				label: "Författare m.fl."
    			},
    			"2642": {
    				label: "Journalister"
    			},
    			"2643": {
    				label: "Översättare, tolkar och andra språkvetare"
    			},
    			"2651": {
    				label: "Bildkonstnärer"
    			},
    			"2652": {
    				label: "Musiker, sångare och kompositörer"
    			},
    			"2653": {
    				label: "Danskonstnärer och koreografer"
    			},
    			"2654": {
    				label: "Regissörer och producenter"
    			},
    			"2655": {
    				label: "Skådespelare"
    			},
    			"2656": {
    				label: "Presentatörer, konferencierer m.fl."
    			},
    			"2659": {
    				label: "Övriga konstnärer"
    			},
    			"3111": {
    				label: "Tekniska experter inom naturvetenskap"
    			},
    			"3112": {
    				label: "Experter inom byggande"
    			},
    			"3113": {
    				label: "Experter inom elteknik"
    			},
    			"3114": {
    				label: "Experter inom elektronik"
    			},
    			"3115": {
    				label: "Experter inom maskinteknik"
    			},
    			"3116": {
    				label: "Experter inom kemisk processteknik"
    			},
    			"3117": {
    				label: "Experter inom gruvindustri och metallurgi"
    			},
    			"3118": {
    				label: "Tekniska ritare"
    			},
    			"3119": {
    				label: "Övriga experter inom fysik, kemi och teknik"
    			},
    			"3121": {
    				label: "Arbetsledare inom gruvdrift"
    			},
    			"3122": {
    				label: "Arbetsledare inom industri"
    			},
    			"3123": {
    				label: "Arbetsledare inom byggverksamhet"
    			},
    			"3131": {
    				label: "Processövervakare, kraftverk"
    			},
    			"3132": {
    				label: "Processövervakare, förbränningsstationer och vattenreningsverk"
    			},
    			"3133": {
    				label: "Processövervakare, kemisk industri"
    			},
    			"3134": {
    				label: "Processövervakare, raffinering av petroleum och naturgas"
    			},
    			"3135": {
    				label: "Processövervakare, metallframställning"
    			},
    			"3139": {
    				label: "Övriga experter inom processövervakning"
    			},
    			"3141": {
    				label: "Laboranter m.fl."
    			},
    			"3142": {
    				label: "Jordbruks- och fiskeritekniker"
    			},
    			"3143": {
    				label: "Skogsbrukstekniker"
    			},
    			"3151": {
    				label: "Maskinchefer och -mästare på fartyg"
    			},
    			"3152": {
    				label: "Befäl och styrmän inom sjötrafik"
    			},
    			"3153": {
    				label: "Flygkaptener och -styrmän"
    			},
    			"3154": {
    				label: "Flygledare"
    			},
    			"3155": {
    				label: "Tekniska experter inom flygövervakning"
    			},
    			"3211": {
    				label: "Experter, bilddiagnostik och medicinteknisk utrustning"
    			},
    			"3212": {
    				label: "Bioanalytiker (hälsovården)"
    			},
    			"3213": {
    				label: "Farmaceuter"
    			},
    			"3214": {
    				label: "Tand- och hjälpmedelstekniker"
    			},
    			"3221": {
    				label: "Sjukskötare m.fl."
    			},
    			"3222": {
    				label: "Barnmorskor"
    			},
    			"3230": {
    				label: "Utövare av naturmedicin och alternativ medicin"
    			},
    			"3240": {
    				label: "Seminörer m.fl."
    			},
    			"3251": {
    				label: "Munhygienister"
    			},
    			"3252": {
    				label: "Patientadministratörer"
    			},
    			"3253": {
    				label: "Hälsorådgivare"
    			},
    			"3254": {
    				label: "Optiker"
    			},
    			"3255": {
    				label: "Fysioterapeuter m.fl."
    			},
    			"3256": {
    				label: "Sjukvårdsassistenter"
    			},
    			"3257": {
    				label: "Hälso- och arbetarskyddsinspektörer"
    			},
    			"3258": {
    				label: "Akutvårdare inom sjuktransporten"
    			},
    			"3259": {
    				label: "Övriga experter inom hälso- och sjukvården utan annan klassificering"
    			},
    			"3311": {
    				label: "Värdepappers- och valutamäklare"
    			},
    			"3312": {
    				label: "Kredit- och låneexperter"
    			},
    			"3313": {
    				label: "Experter inom bokföring och redovisning"
    			},
    			"3314": {
    				label: "Experter inom statistik och matematik"
    			},
    			"3315": {
    				label: "Värderare och skadeinspektörer"
    			},
    			"3321": {
    				label: "Försäljare av försäkringstjänster"
    			},
    			"3322": {
    				label: "Försäljningsrepresentanter"
    			},
    			"3323": {
    				label: "Inköpare"
    			},
    			"3324": {
    				label: "Affärsförmedlare"
    			},
    			"3331": {
    				label: "Speditörer, tull- och fartygsklarerare"
    			},
    			"3332": {
    				label: "Konferens- och evenemangsarrangörer"
    			},
    			"3333": {
    				label: "Arbetsförmedlare"
    			},
    			"3334": {
    				label: "Fastighetsmäklare och disponenter"
    			},
    			"3339": {
    				label: "Övriga experter inom affärslivet"
    			},
    			"3341": {
    				label: "Arbetsledare för kontorspersonal"
    			},
    			"3342": {
    				label: "Juristsekreterare"
    			},
    			"3343": {
    				label: "Ledningsassistenter och avdelningssekreterare"
    			},
    			"3344": {
    				label: "Byråsekreterare, hälso- och sjukvård"
    			},
    			"3351": {
    				label: "Tull- och gränstjänstemän"
    			},
    			"3352": {
    				label: "Skatteberedare och -inspektörer"
    			},
    			"3353": {
    				label: "Handläggare av socialskyddsförmåner"
    			},
    			"3354": {
    				label: "Tillståndstjänstemän"
    			},
    			"3355": {
    				label: "Kommissarier och överkonstaplar"
    			},
    			"3359": {
    				label: "Övriga tjänstemän inom beredning och tillsyn inom offentlig förvaltning"
    			},
    			"3411": {
    				label: "Juridiska assistenter och experter inom intresseorganisationer"
    			},
    			"3412": {
    				label: "Handledare och rådgivare inom socialsektorn"
    			},
    			"3413": {
    				label: "Församlingspersonal"
    			},
    			"3421": {
    				label: "Idrottare"
    			},
    			"3422": {
    				label: "Idrottstränare och funktionärer"
    			},
    			"3423": {
    				label: "Idrottsinstruktörer och fritidsledare"
    			},
    			"3431": {
    				label: "Fotografer"
    			},
    			"3432": {
    				label: "Inredningsplanerare m.fl."
    			},
    			"3433": {
    				label: "Galleri-, musei- och bibliotekstekniker"
    			},
    			"3434": {
    				label: "Kökschefer"
    			},
    			"3435": {
    				label: "Övriga experter inom konst och kultur"
    			},
    			"3511": {
    				label: "Driftoperatörer"
    			},
    			"3512": {
    				label: "Datasupportpersonal"
    			},
    			"3513": {
    				label: "Nätverkstekniker"
    			},
    			"3514": {
    				label: "Webbmaster och webbtekniker"
    			},
    			"3521": {
    				label: "Sändnings- och AV-tekniker"
    			},
    			"3522": {
    				label: "Tekniska experter inom telekommunikation"
    			},
    			"4110": {
    				label: "Kontorsassistenter"
    			},
    			"4120": {
    				label: "Kontorssekreterare"
    			},
    			"4131": {
    				label: "Textbehandlare"
    			},
    			"4132": {
    				label: "Dataregistrerare"
    			},
    			"4211": {
    				label: "Banktjänstemän m.fl."
    			},
    			"4212": {
    				label: "Vadförmedlare, croupierer m.fl."
    			},
    			"4213": {
    				label: "Pantlånare"
    			},
    			"4214": {
    				label: "Inkasserare"
    			},
    			"4221": {
    				label: "Resebyråtjänstemän"
    			},
    			"4222": {
    				label: "Telefonservicerådgivare"
    			},
    			"4223": {
    				label: "Växeltelefonister"
    			},
    			"4224": {
    				label: "Hotellreceptionister"
    			},
    			"4225": {
    				label: "Kundrådgivare vid informationsdisk"
    			},
    			"4226": {
    				label: "Receptions- och rådgivningspersonal"
    			},
    			"4227": {
    				label: "Undersöknings- och marknadsundersökningsintervjuare"
    			},
    			"4229": {
    				label: "Övrig kundtjänstpersonal utan annan klassificering"
    			},
    			"4311": {
    				label: "Kontorspersonal inom ekonomiförvaltning"
    			},
    			"4312": {
    				label: "Kontorspersonal inom statistik-, finansierings- och försäkringssektorn"
    			},
    			"4313": {
    				label: "Löneräknare"
    			},
    			"4321": {
    				label: "Lagerförvaltare m.fl."
    			},
    			"4322": {
    				label: "Produktionsassistenter"
    			},
    			"4323": {
    				label: "Kontorspersonal inom transport"
    			},
    			"4411": {
    				label: "Bibliotekspersonal"
    			},
    			"4412": {
    				label: "Brevbärare och postsorterare"
    			},
    			"4413": {
    				label: "Kodare, korrekturläsare m.fl."
    			},
    			"4414": {
    				label: "Skrivare m.fl."
    			},
    			"4415": {
    				label: "Arkivanställda"
    			},
    			"4416": {
    				label: "Assisterande kontorspersonal inom personalförvaltning"
    			},
    			"4419": {
    				label: "Övrig kontors- och kundtjänstpersonal utan annan klassificering"
    			},
    			"5111": {
    				label: "Flygvärdinnor, purser m.fl."
    			},
    			"5112": {
    				label: "Konduktörer, biljettkontrollanter m.fl."
    			},
    			"5113": {
    				label: "Guider"
    			},
    			"5120": {
    				label: "Restaurang- och storhushållspersonal"
    			},
    			"5131": {
    				label: "Servitörer"
    			},
    			"5132": {
    				label: "Bartendrar"
    			},
    			"5141": {
    				label: "Frisörer"
    			},
    			"5142": {
    				label: "Kosmetologer m.fl."
    			},
    			"5151": {
    				label: "Arbetsledare inom städning på kontor, hotell m.fl."
    			},
    			"5152": {
    				label: "Hushållerskor i privata hem"
    			},
    			"5153": {
    				label: "Fastighetsskötare"
    			},
    			"5161": {
    				label: "Astrologer, siare m.fl."
    			},
    			"5162": {
    				label: "Personliga betjänter"
    			},
    			"5163": {
    				label: "Personal inom begravningstjänster"
    			},
    			"5164": {
    				label: "Djurskötare och djurfrisörer"
    			},
    			"5165": {
    				label: "Trafiklärare"
    			},
    			"5169": {
    				label: "Övrig servicepersonal inom personliga tjänster utan annan klassificering"
    			},
    			"5211": {
    				label: "Kiosk- och torgförsäljare"
    			},
    			"5212": {
    				label: "Gatuförsäljare (livsmedel)"
    			},
    			"5221": {
    				label: "Butiksinnehavare (småföretagare)"
    			},
    			"5222": {
    				label: "Butiksföreståndare"
    			},
    			"5223": {
    				label: "Försäljare"
    			},
    			"5230": {
    				label: "Butikskassörer, biljettförsäljare m.fl."
    			},
    			"5241": {
    				label: "Modeller"
    			},
    			"5242": {
    				label: "Varudemonstratörer"
    			},
    			"5243": {
    				label: "Direktförsäljare"
    			},
    			"5244": {
    				label: "Försäljare, telefonförsäljning och kontaktcenter"
    			},
    			"5245": {
    				label: "Bensinstationspersonal"
    			},
    			"5246": {
    				label: "Försäljare, kaféer och barer"
    			},
    			"5249": {
    				label: "Övrig försäljningspersonal utan annan klassificering"
    			},
    			"5311": {
    				label: "Personal inom barnomsorgen"
    			},
    			"5312": {
    				label: "Skolgångsbiträden"
    			},
    			"5321": {
    				label: "Närvårdare"
    			},
    			"5322": {
    				label: "Hemvårdare (hemvårdstjänster)"
    			},
    			"5329": {
    				label: "Övrig personal inom hälso- och sjukvården"
    			},
    			"5411": {
    				label: "Brandmän"
    			},
    			"5412": {
    				label: "Poliser"
    			},
    			"5413": {
    				label: "Fångvaktare"
    			},
    			"5414": {
    				label: "Väktare"
    			},
    			"5419": {
    				label: "Övrig säkerhetspersonal"
    			},
    			"6111": {
    				label: "Odlare, åkerbruk och frilandsodling"
    			},
    			"6112": {
    				label: "Odlare av fruktträd och buskar m.fl."
    			},
    			"6113": {
    				label: "Odlare och arbetare inom trädgårds- och växthusodling"
    			},
    			"6114": {
    				label: "Odlare av blandade växtslag"
    			},
    			"6121": {
    				label: "Uppfödare av kött- och mjölkboskap och andra husdjur"
    			},
    			"6122": {
    				label: "Fjäderfäuppfödare"
    			},
    			"6123": {
    				label: "Biodlare m.fl."
    			},
    			"6129": {
    				label: "Övriga djuruppfödare och djurskötare"
    			},
    			"6130": {
    				label: "Växtodlare och djuruppfödare, blandad drift"
    			},
    			"6210": {
    				label: "Skogsbrukare och skogsarbetare"
    			},
    			"6221": {
    				label: "Fiskodlare och fiskodlingspersonal"
    			},
    			"6222": {
    				label: "Fiskare"
    			},
    			"6223": {
    				label: "Djuphavsfiskare"
    			},
    			"6224": {
    				label: "Viltvårdare och jägare"
    			},
    			"6310": {
    				label: "Växtodlare (självhushållning)"
    			},
    			"6320": {
    				label: "Djuruppfödare (självhushållning)"
    			},
    			"6330": {
    				label: "Växtodlare och djuruppfödare (självhushållning)"
    			},
    			"6340": {
    				label: "Fiskare och jägare (självhushållning)"
    			},
    			"7111": {
    				label: "Byggnadsarbetare"
    			},
    			"7112": {
    				label: "Murare m.fl."
    			},
    			"7113": {
    				label: "Stenhuggare m.fl."
    			},
    			"7114": {
    				label: "Betongarbetare och armerare"
    			},
    			"7115": {
    				label: "Timmermän och byggnadssnickare"
    			},
    			"7119": {
    				label: "Övriga byggnadsarbetare"
    			},
    			"7121": {
    				label: "Takmontörer och takreparatörer"
    			},
    			"7122": {
    				label: "Golvläggare"
    			},
    			"7123": {
    				label: "Rappare"
    			},
    			"7124": {
    				label: "Isoleringsmontörer"
    			},
    			"7125": {
    				label: "Glasmästare"
    			},
    			"7126": {
    				label: "Rörläggare"
    			},
    			"7127": {
    				label: "Ventilations- och kylmontörer"
    			},
    			"7131": {
    				label: "Byggnadsmålare m.fl."
    			},
    			"7132": {
    				label: "Sprutlackerare"
    			},
    			"7133": {
    				label: "Saneringsarbetare och sotare"
    			},
    			"7211": {
    				label: "Formare och kärnmakare"
    			},
    			"7212": {
    				label: "Svetsare och gasskärare"
    			},
    			"7213": {
    				label: "Tunnplåtslagare"
    			},
    			"7214": {
    				label: "Grovplåtslagare och stålkonstruktionsmontörer"
    			},
    			"7215": {
    				label: "Kabelläggare och riggare"
    			},
    			"7221": {
    				label: "Smeder"
    			},
    			"7222": {
    				label: "Verktygsmakare och låssmeder"
    			},
    			"7223": {
    				label: "Maskinställare och verkstadsmekaniker"
    			},
    			"7224": {
    				label: "Maskinslipare, -polerare och -vässare"
    			},
    			"7231": {
    				label: "Motorfordonsmontörer och -reparatörer"
    			},
    			"7232": {
    				label: "Flygmontörer och flygreparatörer"
    			},
    			"7233": {
    				label: "Maskinmontörer och -reparatörer (jordbruks- och industrimaskiner)"
    			},
    			"7234": {
    				label: "Cykelreparatörer m.fl."
    			},
    			"7311": {
    				label: "Urmakare och övriga tillverkare av finmekaniska instrument"
    			},
    			"7312": {
    				label: "Musikinstrumentmakare och -stämmare"
    			},
    			"7313": {
    				label: "Juvel-, guld- och silversmeder"
    			},
    			"7314": {
    				label: "Ler- och tegelgjutare, drejare"
    			},
    			"7315": {
    				label: "Glasblåsare, -skärare, -slipare och -efterbehandlare"
    			},
    			"7316": {
    				label: "Glasgravörer, glasetsare och skyltmålare"
    			},
    			"7317": {
    				label: "Konsthantverkare i trä m.m."
    			},
    			"7318": {
    				label: "Konsthantverkare i textil, läder m.m."
    			},
    			"7319": {
    				label: "Övriga konsthantverkare"
    			},
    			"7321": {
    				label: "Prepressoperatörer"
    			},
    			"7322": {
    				label: "Tryckare"
    			},
    			"7323": {
    				label: "Efterbehandlare och bokbinderiarbetare"
    			},
    			"7411": {
    				label: "Byggnadselmontörer"
    			},
    			"7412": {
    				label: "Övriga elmontörer"
    			},
    			"7413": {
    				label: "Linjemontörer och -reparatörer"
    			},
    			"7421": {
    				label: "Montörer och reparatörer, elektronik och automation"
    			},
    			"7422": {
    				label: "Montörer och reparatörer, IKT"
    			},
    			"7511": {
    				label: "Slaktare, fiskberedare m.fl."
    			},
    			"7512": {
    				label: "Bagare och konditorer"
    			},
    			"7513": {
    				label: "Mejerister, ostmästare m.fl."
    			},
    			"7514": {
    				label: "Tillverkare av frukt- och grönsaksprodukter"
    			},
    			"7515": {
    				label: "Kvalitetsbedömare (mat och dryck)"
    			},
    			"7516": {
    				label: "Tillverkare av tobaksprodukter"
    			},
    			"7521": {
    				label: "Trävaruhanterare"
    			},
    			"7522": {
    				label: "Möbelsnickare m.fl."
    			},
    			"7523": {
    				label: "Maskinsnickare"
    			},
    			"7531": {
    				label: "Skräddare, ateljésömmerskor, körsnärer och modister"
    			},
    			"7532": {
    				label: "Tillskärare och modellmästare"
    			},
    			"7533": {
    				label: "Broderare och andra textilsömmare"
    			},
    			"7534": {
    				label: "Tapetserare"
    			},
    			"7535": {
    				label: "Garvare och skinnberedare"
    			},
    			"7536": {
    				label: "Skomakare m.fl."
    			},
    			"7541": {
    				label: "Dykare"
    			},
    			"7542": {
    				label: "Laddare och sprängare"
    			},
    			"7543": {
    				label: "Klassificerare och kvalitetskontrollanter (utom drycker och livsmedel)"
    			},
    			"7544": {
    				label: "Röksanerare, skadedjurs- och ogrässanerare"
    			},
    			"7549": {
    				label: "Övrig personal, oklassificerad under andra moment i huvudgrupp 7"
    			},
    			"8111": {
    				label: "Gruv- och stenbrottsarbetare"
    			},
    			"8112": {
    				label: "Anrikningsarbetare"
    			},
    			"8113": {
    				label: "Slagborrare och djupborrare"
    			},
    			"8114": {
    				label: "Processoperatörer inom betongindustri m.fl."
    			},
    			"8121": {
    				label: "Processoperatörer inom metallindustri"
    			},
    			"8122": {
    				label: "Ytbeläggare och efterbehandlare inom metallindustri"
    			},
    			"8131": {
    				label: "Processoperatörer, kemisk industri m.fl."
    			},
    			"8132": {
    				label: "Processoperatörer, fotografiska produkter"
    			},
    			"8141": {
    				label: "Processoperatörer, gummiindustri"
    			},
    			"8142": {
    				label: "Processoperatörer, plastindustri"
    			},
    			"8143": {
    				label: "Processoperatörer, pappersvaruindustri"
    			},
    			"8151": {
    				label: "Maskinoperatörer, fibertillverkning, spinning och spolning"
    			},
    			"8152": {
    				label: "Maskinoperatörer, vävning och stickning"
    			},
    			"8153": {
    				label: "Symaskinsoperatörer"
    			},
    			"8154": {
    				label: "Maskinoperatörer, tvättning, blekning och färgning"
    			},
    			"8155": {
    				label: "Maskinoperatörer, pälsskinnsberedning, färgning"
    			},
    			"8156": {
    				label: "Processoperatörer, sko- och väskindustri"
    			},
    			"8157": {
    				label: "Tvätteripersonal"
    			},
    			"8159": {
    				label: "Övriga processoperatörer inom textil-, skinn- och läderindustri"
    			},
    			"8160": {
    				label: "Processoperatörer, livsmedelsindustri"
    			},
    			"8171": {
    				label: "Processoperatörer, pappers-, kartong- och massaindustri"
    			},
    			"8172": {
    				label: "Processoperatörer, trä- och sågvaror"
    			},
    			"8181": {
    				label: "Ugnsoperatörer, glas- och keramikindustri"
    			},
    			"8182": {
    				label: "Operatörer och eldare m.fl., ångmaskiner och ångpannor"
    			},
    			"8183": {
    				label: "Maskinoperatörer, packning, påfyllning och märkning"
    			},
    			"8189": {
    				label: "Övriga processoperatörer utan annan klassificering"
    			},
    			"8211": {
    				label: "Hopsättare, verkstads- och metallprodukter"
    			},
    			"8212": {
    				label: "Hopsättare, elektrisk och elektronisk utrustning"
    			},
    			"8219": {
    				label: "Övriga hopsättare av industriprodukter"
    			},
    			"8311": {
    				label: "Lokförare"
    			},
    			"8312": {
    				label: "Bangårdspersonal m.fl."
    			},
    			"8321": {
    				label: "Motorcykelbud m.fl."
    			},
    			"8322": {
    				label: "Person-, taxi- och paketbilsförare"
    			},
    			"8331": {
    				label: "Buss- och spårvagnsförare"
    			},
    			"8332": {
    				label: "Lastbils- och specialfordonsförare"
    			},
    			"8341": {
    				label: "Förare av jordbruks- och skogsmaskiner"
    			},
    			"8342": {
    				label: "Anläggningsmaskinförare m.fl."
    			},
    			"8343": {
    				label: "Kranförare"
    			},
    			"8344": {
    				label: "Stuveriarbetare och truckförare m.fl."
    			},
    			"8350": {
    				label: "Däckmanskap, sjötrafikarbetare m.fl."
    			},
    			"9111": {
    				label: "Hembiträden och städare"
    			},
    			"9112": {
    				label: "Kontors- och hotellstädare m.fl."
    			},
    			"9121": {
    				label: "Rengörare och pressare"
    			},
    			"9122": {
    				label: "Fordonstvättare"
    			},
    			"9123": {
    				label: "Fönsterputsare"
    			},
    			"9129": {
    				label: "Övriga rengöringsarbetare"
    			},
    			"9211": {
    				label: "Medhjälpare, jordbruk"
    			},
    			"9212": {
    				label: "Medhjälpare, djuruppfödning"
    			},
    			"9213": {
    				label: "Medhjälpare, växtodling och djuruppfödning, blandad drift"
    			},
    			"9214": {
    				label: "Medhjälpare, trädgårdsodling"
    			},
    			"9215": {
    				label: "Medhjälpare, skogsbruk"
    			},
    			"9216": {
    				label: "Medhjälpare, fiske och vattenbruk"
    			},
    			"9311": {
    				label: "Medhjälpare, gruvor"
    			},
    			"9312": {
    				label: "Medhjälpare, anläggningsverksamhet"
    			},
    			"9313": {
    				label: "Medhjälpare, byggnadsverksamhet"
    			},
    			"9321": {
    				label: "Handpaketerare"
    			},
    			"9329": {
    				label: "Övriga medhjälpare inom industrin"
    			},
    			"9331": {
    				label: "Förare av hand- och pedaldrivna bilar"
    			},
    			"9332": {
    				label: "Förare av djurdragna fordon"
    			},
    			"9333": {
    				label: "Godshanterare, lagerarbetare m.fl."
    			},
    			"9334": {
    				label: "Varupåfyllare"
    			},
    			"9411": {
    				label: "Personal inom snabbmatssektorn"
    			},
    			"9412": {
    				label: "Köksbiträden"
    			},
    			"9510": {
    				label: "Reklamutdelare, skoputsare m.fl."
    			},
    			"9520": {
    				label: "Gatuförsäljare (utom livsmedel)"
    			},
    			"9611": {
    				label: "Avfallsinsamlare"
    			},
    			"9612": {
    				label: "Avfallssorterare"
    			},
    			"9613": {
    				label: "Gatsopare m.fl."
    			},
    			"9621": {
    				label: "Tidnings- och reklamutdelare, bud"
    			},
    			"9622": {
    				label: "Diversearbetare"
    			},
    			"9623": {
    				label: "Mätaravläsare m.fl."
    			},
    			"9624": {
    				label: "Vattenhämtare och vedinsamlare"
    			},
    			"9629": {
    				label: "Övrig personal utan annan klassificering"
    			},
    			"11121": {
    				label: "Direktörer inom statens centralförvaltning"
    			},
    			"11122": {
    				label: "Direktörer och högre tjänstemän inom statlig distrikts- och lokalförvaltning"
    			},
    			"11141": {
    				label: "Chefer i arbetsmarknads- och näringslivsorganisationer"
    			},
    			"11142": {
    				label: "Chefer i andra organisationer"
    			},
    			"21321": {
    				label: "Specialister inom jordbruk och fiske"
    			},
    			"21322": {
    				label: "Specialister inom skogsbruk"
    			},
    			"21531": {
    				label: "Forskare inom teleteknik"
    			},
    			"21532": {
    				label: "Forskare inom informationsteknik"
    			},
    			"22121": {
    				label: "Överläkare"
    			},
    			"22122": {
    				label: "Specialistläkare"
    			},
    			"22211": {
    				label: "Överskötare"
    			},
    			"22212": {
    				label: "Avdelningsskötare"
    			},
    			"23101": {
    				label: "Professorer"
    			},
    			"23102": {
    				label: "Lektorer och överassistenter (universitet)"
    			},
    			"23103": {
    				label: "Assistenter och timlärare (universitet)"
    			},
    			"23104": {
    				label: "Överlärare (yrkeshögskolor)"
    			},
    			"23105": {
    				label: "Lektorer (yrkeshögskolor)"
    			},
    			"23106": {
    				label: "Timlärare m.fl. (yrkeshögskolor)"
    			},
    			"23301": {
    				label: "Lärare i matematiska ämnen"
    			},
    			"23302": {
    				label: "Lärare i modersmål och litteratur"
    			},
    			"23303": {
    				label: "Lärare i konst- och färdighetsämnen"
    			},
    			"23304": {
    				label: "Övriga högstadie- och gymnasielärare"
    			},
    			"23411": {
    				label: "Klasslärare"
    			},
    			"23412": {
    				label: "Ämneslärare (lågstadiet)"
    			},
    			"23591": {
    				label: "Studiehandledare"
    			},
    			"23592": {
    				label: "Övriga specialister inom undervisning"
    			},
    			"26211": {
    				label: "Arkivarier"
    			},
    			"26212": {
    				label: "Specialister inom museisektorn"
    			},
    			"26351": {
    				label: "Socialarbetare m.fl."
    			},
    			"26352": {
    				label: "Planerare inom socialsektorn m.fl."
    			},
    			"26421": {
    				label: "Redaktionschefer och redaktionssekreterare, programchefer"
    			},
    			"26422": {
    				label: "Redaktörer vid tidningar m.fl."
    			},
    			"26423": {
    				label: "Radio- och TV-redaktörer"
    			},
    			"31121": {
    				label: "Experter inom husbyggnad"
    			},
    			"31122": {
    				label: "Experter inom anläggningsarbeten"
    			},
    			"31123": {
    				label: "Experter inom kartläggning och lantmäteri"
    			},
    			"31521": {
    				label: "Fartygsbefäl och styrmän (stora fartyg)"
    			},
    			"31522": {
    				label: "Befäl (mindre fartyg)"
    			},
    			"31523": {
    				label: "Hamntrafikledare och hamnkaptener"
    			},
    			"32141": {
    				label: "Tandtekniker"
    			},
    			"32142": {
    				label: "Hjälpmedelstekniker"
    			},
    			"32211": {
    				label: "Sjukskötare"
    			},
    			"32212": {
    				label: "Hälsovårdare"
    			},
    			"32591": {
    				label: "Ergoterapeuter"
    			},
    			"32592": {
    				label: "Övriga terapeuter"
    			},
    			"33341": {
    				label: "Fastighetsmäklare"
    			},
    			"33342": {
    				label: "Disponenter"
    			},
    			"34111": {
    				label: "Rättegångsombud och utsökningsmän"
    			},
    			"34112": {
    				label: "Ombud, funktionärer m.fl. experter inom organisationssektorn"
    			},
    			"34113": {
    				label: "Konsumentrådgivare m.fl."
    			},
    			"34121": {
    				label: "Handledare inom socialsektorn"
    			},
    			"34122": {
    				label: "Ungdomsledare (ej församl.)"
    			},
    			"34123": {
    				label: "Handledare och sysselsättningsledare"
    			},
    			"34131": {
    				label: "Diakoner och diakonissor"
    			},
    			"34139": {
    				label: "Övriga församlingsarbetare"
    			},
    			"34351": {
    				label: "Scriptor och övriga experter inom scenteknik"
    			},
    			"34359": {
    				label: "Övriga experter inom konst- och konstindustrisektorn"
    			},
    			"42291": {
    				label: "Nödcentraloperatörer"
    			},
    			"42299": {
    				label: "Övrig kundtjänstpersonal utan annan klassificering"
    			},
    			"43231": {
    				label: "Spårtrafikledare, trafiksamordnare m.fl."
    			},
    			"43239": {
    				label: "Övriga kontorstjänstemän inom transport och spedition"
    			},
    			"44121": {
    				label: "Brevbärare"
    			},
    			"44122": {
    				label: "Kontorsvaktmästare"
    			},
    			"51201": {
    				label: "Kockar, kokerskor och kallskänkor"
    			},
    			"51202": {
    				label: "Restaurangföreståndare och skiftledare"
    			},
    			"51631": {
    				label: "Begravningsbyråföreståndare m.fl."
    			},
    			"51632": {
    				label: "Övrig begravningspersonal"
    			},
    			"53111": {
    				label: "Barnskötare, daghem och andra institutioner m.fl."
    			},
    			"53112": {
    				label: "Familjedagvårdare"
    			},
    			"53113": {
    				label: "Barnklubbsledare m.fl."
    			},
    			"53211": {
    				label: "Mentalhälsovårdare"
    			},
    			"53212": {
    				label: "Vårdare av utvecklingshämmade"
    			},
    			"53213": {
    				label: "Vårdare inom socialsektorn"
    			},
    			"53219": {
    				label: "Övriga närvårdare"
    			},
    			"53221": {
    				label: "Hemvårdspersonal"
    			},
    			"53222": {
    				label: "Personliga assistenter, närståendevårdare m.fl."
    			},
    			"53291": {
    				label: "Tandskötare"
    			},
    			"53292": {
    				label: "Instrumentvårdare"
    			},
    			"53293": {
    				label: "Läkemedelsarbetare på apotek"
    			},
    			"53294": {
    				label: "Massörer och konditionsskötare"
    			},
    			"61111": {
    				label: "Åkerodlare"
    			},
    			"61112": {
    				label: "Arbetsledare och arbetare inom åkerodling"
    			},
    			"61131": {
    				label: "Trädgårds- och växthusodlare"
    			},
    			"61132": {
    				label: "Arbetsledare och anställda inom trädgårds- och växthusodling"
    			},
    			"61211": {
    				label: "Boskapsuppfödare m.fl."
    			},
    			"61212": {
    				label: "Boskapsskötare m.fl."
    			},
    			"61213": {
    				label: "Sällskapsdjursuppfödare"
    			},
    			"61214": {
    				label: "Lantbruksavbytare"
    			},
    			"61291": {
    				label: "Pälsdjurs- och renskötare"
    			},
    			"61299": {
    				label: "Övriga djurskötare utan annan klassificering"
    			},
    			"62211": {
    				label: "Fiskodlingsföretagare"
    			},
    			"62212": {
    				label: "Arbetsledare och arbetare inom fiskodling"
    			},
    			"74211": {
    				label: "Elektronikmontörer och -reparatörer"
    			},
    			"74212": {
    				label: "Automationsmontörer och -reparatörer"
    			},
    			"81821": {
    				label: "Maskinreparatörer, kraftverk och avfallsbehandlingsanläggningar"
    			},
    			"81822": {
    				label: "Övriga maskinreparatörer m.fl."
    			},
    			"83441": {
    				label: "Stuveriarbetare"
    			},
    			"83442": {
    				label: "Truckförare m.fl."
    			},
    			"91121": {
    				label: "Kontorsstädare m.fl."
    			},
    			"91122": {
    				label: "Hotellstädare"
    			},
    			"91123": {
    				label: "Sjukhus- och vårdbiträden"
    			},
    			"91124": {
    				label: "Daghemsbiträden"
    			},
    			"91129": {
    				label: "Övriga städare utan annan klassificering"
    			},
    			"01": {
    				label: "Officerare"
    			},
    			"011": {
    				label: "Officerare"
    			},
    			"0110": {
    				label: "Officerare"
    			},
    			"02": {
    				label: "Underofficerare"
    			},
    			"021": {
    				label: "Underofficerare"
    			},
    			"0210": {
    				label: "Underofficerare"
    			},
    			"03": {
    				label: "Yrkesutbildad personal inom militären"
    			},
    			"031": {
    				label: "Yrkesutbildad personal inom militären"
    			},
    			"0310": {
    				label: "Yrkesutbildad personal inom militären"
    			},
    			"#X": {
    				label: "Okänd"
    			},
    			"#XX": {
    				label: "Okänd"
    			},
    			"#XXX": {
    				label: "Okänd"
    			},
    			"#XXXX": {
    				label: "Okänd"
    			},
    			"#XXXXX": {
    				label: "Okänd"
    			}
    		},
    		RemunerationKind: {
    			bonusPay: {
    				description: "",
    				label: ""
    			},
    			commission: {
    				description: "",
    				label: ""
    			},
    			initiativeFee: {
    				description: "",
    				label: ""
    			},
    			performanceBonus: {
    				description: "",
    				label: ""
    			},
    			profitSharingBonus: {
    				description: "",
    				label: ""
    			},
    			shareIssueForEmployees: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		ReportCategory: {
    			all: {
    				description: "",
    				label: ""
    			},
    			monthly: {
    				description: "",
    				label: ""
    			},
    			yearly: {
    				description: "",
    				label: ""
    			}
    		},
    		ReportType: {
    			employerReport: {
    				description: "",
    				label: "Arbetstagarrapport"
    			},
    			employmentContract: {
    				description: "",
    				label: "Arbetskontrakt"
    			},
    			example: {
    				description: "",
    				label: "Exempel"
    			},
    			householdDeduction: {
    				description: "",
    				label: "Hushållsavdrag"
    			},
    			insurance: {
    				description: "",
    				label: "Försäkring"
    			},
    			monthlyDetails: {
    				description: "",
    				label: "Månatliga detlajer"
    			},
    			monthlyLiikekirjuri: {
    				description: "",
    				label: ""
    			},
    			monthlyPension: {
    				description: "",
    				label: "Månatlig pension"
    			},
    			monthlyRapko: {
    				description: "",
    				label: ""
    			},
    			paymentReport: {
    				description: "",
    				label: "Betalningsrapport"
    			},
    			salarySlip: {
    				description: "",
    				label: "Lönekvitto"
    			},
    			salarySlipPaid: {
    				description: "",
    				label: "Lönekvitto betalt"
    			},
    			taxHouseholdDeduction14B: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseA: {
    				description: "",
    				label: ""
    			},
    			taxHouseholdDeduction14BSpouseB: {
    				description: "",
    				label: ""
    			},
    			taxMonthly4001: {
    				description: "",
    				label: "Månatlig skatt"
    			},
    			taxYearly7801: {
    				description: "",
    				label: "Årlig skatt"
    			},
    			"undefined": {
    				description: "",
    				label: "Odefinierad"
    			},
    			unemployment: {
    				description: "",
    				label: "Arbetslöshet"
    			},
    			yearEndReport: {
    				description: "",
    				label: "Årsslutsrapport"
    			},
    			yearlyDetails: {
    				description: "",
    				label: "Årliga uppgifter"
    			},
    			yearlyWorkerSummary: {
    				description: "",
    				label: ""
    			},
    			earningsPaymentReport: {
    				description: "",
    				label: "EarningsPaymentReport"
    			},
    			employersSeparateReport: {
    				description: "",
    				label: "EmployersSeparateReport"
    			},
    			monthlyAccounting: {
    				description: "",
    				label: "MonthlyAccounting"
    			},
    			salarySlipCopy: {
    				description: "",
    				label: "SalarySlipCopy"
    			}
    		},
    		SalaryKind: {
    			compensation: {
    				description: "",
    				label: "Kompensation",
    				labels: {
    					household: ""
    				}
    			},
    			fixedSalary: {
    				description: "",
    				label: "Fast lön"
    			},
    			hourlySalary: {
    				description: "",
    				label: "Timlön"
    			},
    			monthlySalary: {
    				description: "",
    				label: "Månadslön"
    			},
    			salary: {
    				description: "",
    				label: ""
    			},
    			totalEmployerPayment: {
    				description: "",
    				label: ""
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: "Odefinierad"
    			}
    		},
    		SalaryPeriod: {
    			fixed: {
    				description: "",
    				label: ""
    			},
    			hourly: {
    				description: "",
    				label: ""
    			},
    			monthly: {
    				description: "",
    				label: ""
    			},
    			notDefined: {
    				description: "",
    				label: ""
    			}
    		},
    		SalaryPerMonthRange: {
    			noPensionRequired: {
    				description: "",
    				label: "Under 57,10 €"
    			},
    			normal: {
    				description: "",
    				label: "57,10 € eller över"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			}
    		},
    		SalaryPerYearRange: {
    			noMonthlyReporting: {
    				description: "",
    				label: "203 - 1500 €"
    			},
    			noReportingNecessary: {
    				description: "",
    				label: "3 - 200 €"
    			},
    			normal: {
    				description: "",
    				label: "Över 1500 €"
    			},
    			zeroSalary: {
    				description: "",
    				label: 2
    			}
    		},
    		ServiceModel: {
    			partnerOnly: {
    				label: ""
    			},
    			shared: {
    				label: ""
    			}
    		},
    		SubsidisedCommuteKind: {
    			noDeduction: {
    				description: "",
    				label: ""
    			},
    			periodicalDeduction: {
    				description: "",
    				label: ""
    			},
    			singleDeduction: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		TaxCardIncomeType: {
    			externalSalaries: {
    				description: "",
    				label: "Utomstående löner"
    			},
    			previousEmployerSalaries: {
    				description: "Lön som personen förtjänat tidigare under samma år. Markera dem i euro i en klumpsumma.",
    				label: "Lön betalt av tidigare arbetsgivare"
    			},
    			salaxyCalculation: {
    				description: "",
    				label: "Salaxy uträkning"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd"
    			},
    			diff: {
    				description: "",
    				label: "Diff"
    			},
    			sharedCardExtSum: {
    				description: "",
    				label: "SharedCardExtSum"
    			}
    		},
    		TaxcardKind: {
    			defaultYearly: {
    				label: ""
    			},
    			historical: {
    				label: ""
    			},
    			noTaxCard: {
    				label: ""
    			},
    			noWithholdingHousehold: {
    				label: ""
    			},
    			others: {
    				label: ""
    			},
    			replacement: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			}
    		},
    		TaxcardState: {
    			employerAdded: {
    				label: ""
    			},
    			expired: {
    				label: ""
    			},
    			"new": {
    				label: "Ny"
    			},
    			replaced: {
    				label: ""
    			},
    			shared: {
    				label: "Ny"
    			},
    			sharedApproved: {
    				label: ""
    			},
    			sharedRejected: {
    				label: "Avslagen"
    			},
    			sharedRejectedWithoutOpen: {
    				label: ""
    			},
    			sharedWaiting: {
    				label: "Väntar"
    			},
    			verifiedVero: {
    				label: ""
    			},
    			approved: {
    				description: "",
    				label: "Approved"
    			}
    		},
    		TaxcardValidity: {
    			expired: {
    				label: ""
    			},
    			future: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			},
    			valid: {
    				label: ""
    			},
    			validJanuary: {
    				label: ""
    			}
    		},
    		TaxDeductionWorkCategories: {
    			carework: {
    				description: "",
    				label: "Omvårdnadsarbete"
    			},
    			homeImprovement: {
    				description: "",
    				label: "Hemförbättringsarbete"
    			},
    			householdwork: {
    				description: "",
    				label: "Hushållsarbete"
    			},
    			none: {
    				description: "",
    				label: "Välj…"
    			},
    			ownPropety: {
    				description: "",
    				label: "Fritidshus i eget bruk"
    			},
    			relativesProperty: {
    				description: "",
    				label: "Föräldrars eller släktingars fritidshus"
    			}
    		},
    		TesSubtype: {
    			constructionCarpenter: {
    				description: "",
    				label: ""
    			},
    			constructionFloor: {
    				description: "",
    				label: ""
    			},
    			constructionFreeContract: {
    				description: "",
    				label: ""
    			},
    			constructionOther: {
    				description: "",
    				label: ""
    			},
    			notSelected: {
    				description: "",
    				label: ""
    			}
    		},
    		TestEnum: {
    			valueEight: {
    				description: "",
    				label: ""
    			},
    			valueFive: {
    				description: "",
    				label: ""
    			},
    			valueFour: {
    				description: "",
    				label: ""
    			},
    			valueNine: {
    				description: "",
    				label: ""
    			},
    			valueOne: {
    				description: "",
    				label: ""
    			},
    			valueSevdescription: "",
    			valueSevlabel: "",
    			valueSix: {
    				description: "",
    				label: ""
    			},
    			valueTdescription: "",
    			valueThree: {
    				description: "",
    				label: ""
    			},
    			valueTlabel: "",
    			valueTwo: {
    				description: "",
    				label: ""
    			},
    			valueZero: {
    				description: "",
    				label: ""
    			},
    			valueSeven: {
    				description: "",
    				label: "ValueSeven"
    			},
    			valueTen: {
    				description: "",
    				label: "ValueTen"
    			}
    		},
    		UnionPaymentKind: {
    			other: {
    				description: "",
    				label: ""
    			},
    			raksaNormal: {
    				description: "",
    				label: ""
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			}
    		},
    		UnionPaymentType: {
    			notSelected: {
    				description: "",
    				label: "Inget medlemskap"
    			},
    			other: {
    				description: "",
    				label: "Annan fackförening"
    			},
    			raksaNormal: {
    				description: "",
    				label: ""
    			},
    			raksaUnemploymentOnly: {
    				description: "",
    				label: ""
    			}
    		},
    		WageBasis: {
    			hourly: {
    				label: ""
    			},
    			monthly: {
    				label: ""
    			},
    			other: {
    				label: ""
    			},
    			performanceBased: {
    				label: ""
    			},
    			"undefined": {
    				label: ""
    			}
    		},
    		WebSiteUserRole: {
    			company: {
    				description: "",
    				label: "Företagare"
    			},
    			household: {
    				description: "",
    				label: "Arbetsgivare"
    			},
    			none: {
    				description: "",
    				label: "Välj…"
    			},
    			worker: {
    				description: "",
    				label: "Arbetstagare"
    			}
    		},
    		WorkingTimeCompensationKind: {
    			emergencyWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			eveningShiftAllowance: {
    				description: "",
    				label: ""
    			},
    			eveningWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			extraWorkPremium: {
    				description: "",
    				label: ""
    			},
    			nightShiftCompensation: {
    				description: "",
    				label: ""
    			},
    			nightWorkAllowance: {
    				description: "",
    				label: ""
    			},
    			otherCompensation: {
    				description: "",
    				label: ""
    			},
    			overtimeCompensation: {
    				description: "",
    				label: ""
    			},
    			saturdayPay: {
    				description: "",
    				label: ""
    			},
    			standByCompensation: {
    				description: "",
    				label: ""
    			},
    			sundayWorkCompensation: {
    				description: "",
    				label: ""
    			},
    			"undefined": {
    				description: "",
    				label: ""
    			},
    			waitingTimeCompensation: {
    				description: "",
    				label: ""
    			},
    			weeklyRestCompensation: {
    				description: "",
    				label: ""
    			}
    		},
    		YtjCompanyType: {
    			alandFederation: {
    				description: "",
    				label: "Åländskt företag"
    			},
    			asoAssociation: {
    				description: "",
    				label: "Bostadsrättsförening"
    			},
    			association: {
    				description: "",
    				label: "Idelogisk förening"
    			},
    			condominium: {
    				description: "",
    				label: "Fastighets aktiebolag"
    			},
    			cooperativeBank: {
    				description: "",
    				label: "Andelsbank"
    			},
    			europeanCooperative: {
    				description: "",
    				label: "Europeiskt andelslag"
    			},
    			europeanCooperativeBank: {
    				description: "",
    				label: "Europeisk andelsbank"
    			},
    			federationOfMunicipalitiesEstablishment: {
    				description: "",
    				label: "Federal inrättning"
    			},
    			financialAssociation: {
    				description: "",
    				label: "Ekonomisk förening"
    			},
    			forestCareAssociation: {
    				description: "",
    				label: "Skogsbruksförening"
    			},
    			foundation: {
    				description: "",
    				label: "Fond"
    			},
    			housingCooperative: {
    				description: "",
    				label: "Bostadsandelslag"
    			},
    			hypoAssociation: {
    				description: "",
    				label: "Hypoteksförening"
    			},
    			insuranceAssociation: {
    				description: "",
    				label: "Försäkringsförening"
    			},
    			ky: {
    				description: "",
    				label: "Kommanditbolag"
    			},
    			municipalEstablishment: {
    				description: "",
    				label: "Kommunal inrättning"
    			},
    			mutualInsuranceAssociation: {
    				description: "",
    				label: "Ömsesidig skadeförsäkringsförening"
    			},
    			noCompanyType: {
    				description: "",
    				label: "Ingen företagsform"
    			},
    			openCompany: {
    				description: "",
    				label: "Öppet bolag"
    			},
    			osuuskunta: {
    				description: "",
    				label: "Andelslag"
    			},
    			otherAssociation: {
    				description: "",
    				label: "Annan förening"
    			},
    			otherFinancialAssociation: {
    				description: "",
    				label: "Annan finansiell förening"
    			},
    			oy: {
    				description: "",
    				label: "Aktiebolag"
    			},
    			privateEntrepreneur: {
    				description: "",
    				label: "Privat näringsidkare"
    			},
    			reindeerHerdingCooperative: {
    				description: "",
    				label: "Renbetes association"
    			},
    			savingsBank: {
    				description: "",
    				label: "Sparbank"
    			},
    			specialPurposeAssociation: {
    				description: "",
    				label: "Förening grundad på särskild lagstiftning"
    			},
    			stateEstablishment: {
    				description: "",
    				label: "Statligt företag"
    			},
    			unknown: {
    				description: "",
    				label: "Okänd företagsform"
    			},
    			mutualRealEstateCompany: {
    				description: "",
    				label: "MutualRealEstateCompany"
    			},
    			taxableGrouping: {
    				description: "",
    				label: "TaxableGrouping"
    			}
    		},
    		AccountantType: {
    			primaryPartner: {
    				description: "",
    				label: "PrimaryPartner"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			unlinkedAccountingOnly: {
    				description: "",
    				label: "UnlinkedAccountingOnly"
    			},
    			unlinkedPrimaryPartner: {
    				description: "",
    				label: "UnlinkedPrimaryPartner"
    			}
    		},
    		AccountingPeriodClosingOption: {
    			"default": {
    				description: "",
    				label: "Default"
    			}
    		},
    		AccountingReportRowType: {
    			childRow: {
    				description: "",
    				label: "ChildRow"
    			},
    			credit: {
    				description: "",
    				label: "Credit"
    			},
    			debit: {
    				description: "",
    				label: "Debit"
    			},
    			groupHeader: {
    				description: "",
    				label: "GroupHeader"
    			},
    			groupTotal: {
    				description: "",
    				label: "GroupTotal"
    			},
    			total: {
    				description: "",
    				label: "Total"
    			}
    		},
    		AccountingReportTableType: {
    			classic: {
    				description: "",
    				label: "Classic"
    			},
    			mapped: {
    				description: "",
    				label: "Mapped"
    			},
    			simple: {
    				description: "",
    				label: "Simple"
    			}
    		},
    		ApiListItemType: {
    			accountBase: {
    				description: "",
    				label: "AccountBase"
    			},
    			accountProducts: {
    				description: "",
    				label: "AccountProducts"
    			},
    			article: {
    				description: "",
    				label: "Article"
    			},
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			calculationPaid: {
    				description: "",
    				label: "CalculationPaid"
    			},
    			certificate: {
    				description: "",
    				label: "Certificate"
    			},
    			companyAccount: {
    				description: "",
    				label: "CompanyAccount"
    			},
    			eSalaryPayment: {
    				description: "",
    				label: "ESalaryPayment"
    			},
    			earningsPayment: {
    				description: "",
    				label: "EarningsPayment"
    			},
    			emailMessage: {
    				description: "",
    				label: "EmailMessage"
    			},
    			employment: {
    				description: "",
    				label: "Employment"
    			},
    			historical: {
    				description: "",
    				label: "Historical"
    			},
    			holidayYear: {
    				description: "",
    				label: "HolidayYear"
    			},
    			ifInsuranceOrder: {
    				description: "",
    				label: "IfInsuranceOrder"
    			},
    			invoice: {
    				description: "",
    				label: "Invoice"
    			},
    			messageThread: {
    				description: "",
    				label: "MessageThread"
    			},
    			onboarding: {
    				description: "",
    				label: "Onboarding"
    			},
    			payment: {
    				description: "",
    				label: "Payment"
    			},
    			payrollDetails: {
    				description: "",
    				label: "PayrollDetails"
    			},
    			personAccount: {
    				description: "",
    				label: "PersonAccount"
    			},
    			profile: {
    				description: "",
    				label: "Profile"
    			},
    			report: {
    				description: "",
    				label: "Report"
    			},
    			sessionUserCredential: {
    				description: "",
    				label: "SessionUserCredential"
    			},
    			taxcard: {
    				description: "",
    				label: "Taxcard"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			varmaPensionOrder: {
    				description: "",
    				label: "VarmaPensionOrder"
    			},
    			workerAbsences: {
    				description: "",
    				label: "WorkerAbsences"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			},
    			workerInvitation: {
    				description: "",
    				label: "WorkerInvitation"
    			}
    		},
    		BankPaymentType: {
    			bankFees: {
    				description: "",
    				label: "BankFees"
    			},
    			error: {
    				description: "",
    				label: "Error"
    			},
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			insuranceGrossPayment: {
    				description: "",
    				label: "InsuranceGrossPayment"
    			},
    			insuranceLahiTapiola: {
    				description: "",
    				label: "InsuranceLahiTapiola"
    			},
    			netSalary: {
    				description: "",
    				label: "NetSalary"
    			},
    			netvisorPayment: {
    				description: "",
    				label: "NetvisorPayment"
    			},
    			palkkausFees: {
    				description: "",
    				label: "PalkkausFees"
    			},
    			partialRefundPayment: {
    				description: "",
    				label: "PartialRefundPayment"
    			},
    			paytrailBatch: {
    				description: "",
    				label: "PaytrailBatch"
    			},
    			paytrailFees: {
    				description: "",
    				label: "PaytrailFees"
    			},
    			refundPayment: {
    				description: "",
    				label: "RefundPayment"
    			},
    			salaryAdvance: {
    				description: "",
    				label: "SalaryAdvance"
    			},
    			salaryGrossPayment: {
    				description: "",
    				label: "SalaryGrossPayment"
    			},
    			saldoYearBegin: {
    				description: "",
    				label: "SaldoYearBegin"
    			},
    			saldoYearEnd: {
    				description: "",
    				label: "SaldoYearEnd"
    			},
    			taxAccount: {
    				description: "",
    				label: "TaxAccount"
    			},
    			tvr: {
    				description: "",
    				label: "Tvr"
    			},
    			tvrRfnd: {
    				description: "",
    				label: "TvrRfnd"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			tyelEloContract: {
    				description: "",
    				label: "TyelEloContract"
    			},
    			tyelEloInitialImport: {
    				description: "",
    				label: "TyelEloInitialImport"
    			},
    			tyelEloTemp: {
    				description: "",
    				label: "TyelEloTemp"
    			},
    			tyelEteraContract: {
    				description: "",
    				label: "TyelEteraContract"
    			},
    			tyelEteraTemp: {
    				description: "",
    				label: "TyelEteraTemp"
    			},
    			tyelEteraUnknown: {
    				description: "",
    				label: "TyelEteraUnknown"
    			},
    			tyelIlmarinenContract: {
    				description: "",
    				label: "TyelIlmarinenContract"
    			},
    			tyelIlmarinenTemp: {
    				description: "",
    				label: "TyelIlmarinenTemp"
    			},
    			tyelVarmaContract: {
    				description: "",
    				label: "TyelVarmaContract"
    			},
    			tyelVarmaTemp: {
    				description: "",
    				label: "TyelVarmaTemp"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			},
    			unionRaksa: {
    				description: "",
    				label: "UnionRaksa"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		BenefitCode: {
    			accommodationBenefit: {
    				description: "",
    				label: "AccommodationBenefit"
    			},
    			mealBenefit: {
    				description: "",
    				label: "MealBenefit"
    			},
    			otherBenefits: {
    				description: "",
    				label: "OtherBenefits"
    			},
    			telephoneBenefit: {
    				description: "",
    				label: "TelephoneBenefit"
    			}
    		},
    		CalcGroup: {
    			baseSalary: {
    				description: "",
    				label: "BaseSalary"
    			},
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			disabled: {
    				description: "",
    				label: "Disabled"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			otherNoPayment: {
    				description: "",
    				label: "OtherNoPayment"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			totals: {
    				description: "",
    				label: "Totals"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationResultRowType: {
    			totalBenefits: {
    				description: "",
    				label: "TotalBenefits"
    			},
    			totalExpenses: {
    				description: "",
    				label: "TotalExpenses"
    			},
    			totalPalkkaus: {
    				description: "",
    				label: "TotalPalkkaus"
    			},
    			totalPayment: {
    				description: "",
    				label: "TotalPayment"
    			},
    			totalPension: {
    				description: "",
    				label: "TotalPension"
    			},
    			totalPensionEmployer: {
    				description: "",
    				label: "TotalPensionEmployer"
    			},
    			totalPensionEmployerDebt: {
    				description: "",
    				label: "TotalPensionEmployerDebt"
    			},
    			totalPensionPayment: {
    				description: "",
    				label: "TotalPensionPayment"
    			},
    			totalPensionWorker: {
    				description: "",
    				label: "TotalPensionWorker"
    			},
    			totalSalary: {
    				description: "",
    				label: "TotalSalary"
    			},
    			totalSocialSecurityEmployer: {
    				description: "",
    				label: "TotalSocialSecurityEmployer"
    			},
    			totalSocialSecurityEmployerDebt: {
    				description: "",
    				label: "TotalSocialSecurityEmployerDebt"
    			},
    			totalSocialSecurityEmployerPayment: {
    				description: "",
    				label: "TotalSocialSecurityEmployerPayment"
    			},
    			totalTax: {
    				description: "",
    				label: "TotalTax"
    			},
    			totalTaxPayment: {
    				description: "",
    				label: "TotalTaxPayment"
    			},
    			totalUnemployment: {
    				description: "",
    				label: "TotalUnemployment"
    			},
    			totalUnemploymentEmployer: {
    				description: "",
    				label: "TotalUnemploymentEmployer"
    			},
    			totalUnemploymentEmployerDebt: {
    				description: "",
    				label: "TotalUnemploymentEmployerDebt"
    			},
    			totalUnemploymentPayment: {
    				description: "",
    				label: "TotalUnemploymentPayment"
    			},
    			totalUnemploymentWorker: {
    				description: "",
    				label: "TotalUnemploymentWorker"
    			},
    			totalWorkerPayment: {
    				description: "",
    				label: "TotalWorkerPayment"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		CalculationRowCategory: {
    			benefits: {
    				description: "",
    				label: "Benefits"
    			},
    			deductions: {
    				description: "",
    				label: "Deductions"
    			},
    			expenses: {
    				description: "",
    				label: "Expenses"
    			},
    			holidays: {
    				description: "",
    				label: "Holidays"
    			},
    			other: {
    				description: "",
    				label: "Other"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			},
    			salaryAdditions: {
    				description: "",
    				label: "SalaryAdditions"
    			},
    			salaryCalc: {
    				description: "",
    				label: "SalaryCalc"
    			}
    		},
    		CalculationStatusCategory: {
    			all: {
    				description: "",
    				label: "All"
    			},
    			editable: {
    				description: "",
    				label: "Editable"
    			},
    			readonly: {
    				description: "",
    				label: "Readonly"
    			},
    			received: {
    				description: "",
    				label: "Received"
    			},
    			sent: {
    				description: "",
    				label: "Sent"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			}
    		},
    		ESalaryPaymentStatus: {
    			calculationCreated: {
    				description: "",
    				label: "CalculationCreated"
    			},
    			created: {
    				description: "",
    				label: "Created"
    			},
    			employerResolved: {
    				description: "",
    				label: "EmployerResolved"
    			},
    			employmentResolved: {
    				description: "",
    				label: "EmploymentResolved"
    			},
    			opened: {
    				description: "",
    				label: "Opened"
    			},
    			paymentCanceled: {
    				description: "",
    				label: "PaymentCanceled"
    			},
    			paymentMethodResolved: {
    				description: "",
    				label: "PaymentMethodResolved"
    			},
    			paymentStarted: {
    				description: "",
    				label: "PaymentStarted"
    			},
    			paymentSucceeded: {
    				description: "",
    				label: "PaymentSucceeded"
    			},
    			shared: {
    				description: "",
    				label: "Shared"
    			},
    			taxcardResolved: {
    				description: "",
    				label: "TaxcardResolved"
    			}
    		},
    		EmploymentRelationStatus: {
    			archived: {
    				description: "",
    				label: "Archived"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			secondaryCurrent: {
    				description: "",
    				label: "SecondaryCurrent"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		Holiday: {
    			allSaintsDay: {
    				description: "",
    				label: "AllSaintsDay"
    			},
    			ascensionDay: {
    				description: "",
    				label: "AscensionDay"
    			},
    			christmasDay: {
    				description: "",
    				label: "ChristmasDay"
    			},
    			christmasEve: {
    				description: "",
    				label: "ChristmasEve"
    			},
    			easterMonday: {
    				description: "",
    				label: "EasterMonday"
    			},
    			easterSaturday: {
    				description: "",
    				label: "EasterSaturday"
    			},
    			easterSunday: {
    				description: "",
    				label: "EasterSunday"
    			},
    			epiphany: {
    				description: "",
    				label: "Epiphany"
    			},
    			goodFriday: {
    				description: "",
    				label: "GoodFriday"
    			},
    			independenceDay: {
    				description: "",
    				label: "IndependenceDay"
    			},
    			mayDay: {
    				description: "",
    				label: "MayDay"
    			},
    			midsummerDay: {
    				description: "",
    				label: "MidsummerDay"
    			},
    			midsummerEve: {
    				description: "",
    				label: "MidsummerEve"
    			},
    			newYearsDay: {
    				description: "",
    				label: "NewYearsDay"
    			},
    			pentecost: {
    				description: "",
    				label: "Pentecost"
    			},
    			stStephensDay: {
    				description: "",
    				label: "StStephensDay"
    			}
    		},
    		HolidayAccrualSource: {
    			calcDraft: {
    				description: "",
    				label: "CalcDraft"
    			},
    			calcPaid: {
    				description: "",
    				label: "CalcPaid"
    			},
    			initial: {
    				description: "",
    				label: "Initial"
    			},
    			manual: {
    				description: "",
    				label: "Manual"
    			}
    		},
    		HolidayGroup: {
    			holiday: {
    				description: "",
    				label: "Holiday"
    			},
    			nonBanking: {
    				description: "",
    				label: "NonBanking"
    			},
    			official: {
    				description: "",
    				label: "Official"
    			}
    		},
    		IncomeEarnerType: {
    			athlete: {
    				description: "",
    				label: "Athlete"
    			},
    			employedByStateEmploymentFund: {
    				description: "",
    				label: "EmployedByStateEmploymentFund"
    			},
    			entrepreneurOrFarmerNoPensionRequired: {
    				description: "",
    				label: "EntrepreneurOrFarmerNoPensionRequired"
    			},
    			jointOwnerWithPayer: {
    				description: "",
    				label: "JointOwnerWithPayer"
    			},
    			keyEmployee: {
    				description: "",
    				label: "KeyEmployee"
    			},
    			leasedEmployeeLivingAbroad: {
    				description: "",
    				label: "LeasedEmployeeLivingAbroad"
    			},
    			netOfTaxContract: {
    				description: "",
    				label: "NetOfTaxContract"
    			},
    			organization: {
    				description: "",
    				label: "Organization"
    			},
    			partialOwner: {
    				description: "",
    				label: "PartialOwner"
    			},
    			performingArtist: {
    				description: "",
    				label: "PerformingArtist"
    			},
    			personWorkingAbroad: {
    				description: "",
    				label: "PersonWorkingAbroad"
    			},
    			personWorkingInFrontierDistrict: {
    				description: "",
    				label: "PersonWorkingInFrontierDistrict"
    			},
    			personWorkingOnAlandFerry: {
    				description: "",
    				label: "PersonWorkingOnAlandFerry"
    			},
    			restrictedPeriodInFinland: {
    				description: "",
    				label: "RestrictedPeriodInFinland"
    			}
    		},
    		IncomeLogDiff: {
    			changed: {
    				description: "",
    				label: "Changed"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			"new": {
    				description: "",
    				label: "New"
    			},
    			removed: {
    				description: "",
    				label: "Removed"
    			}
    		},
    		InvoicePaymentType: {
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			salary: {
    				description: "",
    				label: "Salary"
    			}
    		},
    		InvoiceType: {
    			foreclosure: {
    				description: "",
    				label: "Foreclosure"
    			},
    			gross: {
    				description: "",
    				label: "Gross"
    			},
    			net: {
    				description: "",
    				label: "Net"
    			},
    			pension: {
    				description: "",
    				label: "Pension"
    			},
    			tax: {
    				description: "",
    				label: "Tax"
    			},
    			taxSocialSecurity: {
    				description: "",
    				label: "TaxSocialSecurity"
    			},
    			taxWithholding: {
    				description: "",
    				label: "TaxWithholding"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			unemployment: {
    				description: "",
    				label: "Unemployment"
    			},
    			union: {
    				description: "",
    				label: "Union"
    			}
    		},
    		IrRowSourceType: {
    			manual: {
    				description: "",
    				label: "Manual"
    			},
    			usecase: {
    				description: "",
    				label: "Usecase"
    			},
    			usecaseV02: {
    				description: "",
    				label: "UsecaseV02"
    			}
    		},
    		LegacyTaxcardType: {
    			example: {
    				description: "",
    				label: "Example"
    			},
    			freelancer: {
    				description: "",
    				label: "Freelancer"
    			},
    			noTaxCard: {
    				description: "",
    				label: "NoTaxCard"
    			},
    			noTaxPrepayment: {
    				description: "",
    				label: "NoTaxPrepayment"
    			},
    			others: {
    				description: "",
    				label: "Others"
    			},
    			sideIncome: {
    				description: "",
    				label: "SideIncome"
    			},
    			steps: {
    				description: "",
    				label: "Steps"
    			},
    			taxCardA: {
    				description: "",
    				label: "TaxCardA"
    			},
    			taxCardB: {
    				description: "",
    				label: "TaxCardB"
    			}
    		},
    		MessageFrom: {
    			otherParty: {
    				description: "",
    				label: "OtherParty"
    			},
    			owner: {
    				description: "",
    				label: "Owner"
    			},
    			system: {
    				description: "",
    				label: "System"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthError: {
    			"<any>access_denied": {
    				description: "",
    				label: "Access_denied"
    			},
    			"<any>invalid_request": {
    				description: "",
    				label: "Invalid_request"
    			},
    			"<any>invalid_scope": {
    				description: "",
    				label: "Invalid_scope"
    			},
    			"<any>server_error": {
    				description: "",
    				label: "Server_error"
    			},
    			"<any>temporarily_unavailable": {
    				description: "",
    				label: "Temporarily_unavailable"
    			},
    			"<any>unauthorized_client": {
    				description: "",
    				label: "Unauthorized_client"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			"<any>unsupported_response_type": {
    				description: "",
    				label: "Unsupported_response_type"
    			}
    		},
    		OAuthGrantType: {
    			"<any>authorization_code": {
    				description: "",
    				label: "Authorization_code"
    			},
    			"<any>client_credentials": {
    				description: "",
    				label: "Client_credentials"
    			},
    			"<any>jwtBearer": {
    				description: "",
    				label: "JwtBearer"
    			},
    			"<any>password": {
    				description: "",
    				label: "Password"
    			},
    			"<any>refresh_token": {
    				description: "",
    				label: "Refresh_token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthResponseType: {
    			"<any>code": {
    				description: "",
    				label: "Code"
    			},
    			"<any>token": {
    				description: "",
    				label: "Token"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthSalaxyAuthorizeMode: {
    			"<any>sign_in": {
    				description: "",
    				label: "Sign_in"
    			},
    			"<any>sign_up": {
    				description: "",
    				label: "Sign_up"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		OAuthTokenType: {
    			"<any>bearer": {
    				description: "",
    				label: "Bearer"
    			},
    			"<any>mac": {
    				description: "",
    				label: "Mac"
    			},
    			"<any>undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PaymentMethod: {
    			sepa: {
    				description: "",
    				label: "Sepa"
    			},
    			siirto: {
    				description: "",
    				label: "Siirto"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		PeriodDateKind: {
    			paidAtDate: {
    				description: "",
    				label: "PaidAtDate"
    			},
    			salaryDate: {
    				description: "",
    				label: "SalaryDate"
    			}
    		},
    		Role: {
    			accountant: {
    				description: "",
    				label: "Accountant"
    			},
    			accountantCandidate: {
    				description: "",
    				label: "AccountantCandidate"
    			},
    			pricingPartner: {
    				description: "",
    				label: "PricingPartner"
    			},
    			trustedPartner: {
    				description: "",
    				label: "TrustedPartner"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SalaryType: {
    			compensation: {
    				description: "",
    				label: "Compensation"
    			},
    			tyel: {
    				description: "",
    				label: "Tyel"
    			},
    			yel: {
    				description: "",
    				label: "Yel"
    			}
    		},
    		SettingsStatus: {
    			checked: {
    				description: "",
    				label: "Checked"
    			},
    			pending: {
    				description: "",
    				label: "Pending"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		SharingUriType: {
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			url: {
    				description: "",
    				label: "Url"
    			}
    		},
    		SystemRole: {
    			anon: {
    				description: "",
    				label: "Anon"
    			},
    			any: {
    				description: "",
    				label: "Any"
    			},
    			auth: {
    				description: "",
    				label: "Auth"
    			},
    			company: {
    				description: "",
    				label: "Company"
    			},
    			employer: {
    				description: "",
    				label: "Employer"
    			},
    			fiOy: {
    				description: "",
    				label: "FiOy"
    			},
    			fiRy: {
    				description: "",
    				label: "FiRy"
    			},
    			fiTm: {
    				description: "",
    				label: "FiTm"
    			},
    			hidden: {
    				description: "",
    				label: "Hidden"
    			},
    			household: {
    				description: "",
    				label: "Household"
    			},
    			init: {
    				description: "",
    				label: "Init"
    			},
    			nonVerified: {
    				description: "",
    				label: "NonVerified"
    			},
    			partnerMessaging: {
    				description: "",
    				label: "PartnerMessaging"
    			},
    			partnerService: {
    				description: "",
    				label: "PartnerService"
    			},
    			person: {
    				description: "",
    				label: "Person"
    			},
    			prod: {
    				description: "",
    				label: "Prod"
    			},
    			test: {
    				description: "",
    				label: "Test"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			},
    			worker: {
    				description: "",
    				label: "Worker"
    			}
    		},
    		TableFormat: {
    			csv: {
    				description: "",
    				label: "Csv"
    			},
    			txt: {
    				description: "",
    				label: "Txt"
    			},
    			xlsx: {
    				description: "",
    				label: "Xlsx"
    			}
    		},
    		TaxReportHandling: {
    			always: {
    				description: "",
    				label: "Always"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			noMonthlyReport: {
    				description: "",
    				label: "NoMonthlyReport"
    			},
    			noZeroSalaryReport: {
    				description: "",
    				label: "NoZeroSalaryReport"
    			}
    		},
    		TaxcardApprovalMethod: {
    			approve: {
    				description: "",
    				label: "Approve"
    			},
    			assureWaiting: {
    				description: "",
    				label: "AssureWaiting"
    			},
    			reject: {
    				description: "",
    				label: "Reject"
    			}
    		},
    		TaxcardCalcDiffCheck: {
    			commitDiff: {
    				description: "",
    				label: "CommitDiff"
    			},
    			getDiff: {
    				description: "",
    				label: "GetDiff"
    			},
    			noCheck: {
    				description: "",
    				label: "NoCheck"
    			}
    		},
    		ThreadedMessageType: {
    			blobFile: {
    				description: "",
    				label: "BlobFile"
    			},
    			calculation: {
    				description: "",
    				label: "Calculation"
    			},
    			email: {
    				description: "",
    				label: "Email"
    			},
    			payroll: {
    				description: "",
    				label: "Payroll"
    			},
    			sms: {
    				description: "",
    				label: "Sms"
    			},
    			text: {
    				description: "",
    				label: "Text"
    			},
    			workerAccount: {
    				description: "",
    				label: "WorkerAccount"
    			}
    		},
    		Unit: {
    			count: {
    				description: "",
    				label: "Count"
    			},
    			days: {
    				description: "",
    				label: "Days"
    			},
    			euro: {
    				description: "",
    				label: "Euro"
    			},
    			hours: {
    				description: "",
    				label: "Hours"
    			},
    			kilometers: {
    				description: "",
    				label: "Kilometers"
    			},
    			one: {
    				description: "",
    				label: "One"
    			},
    			percent: {
    				description: "",
    				label: "Percent"
    			},
    			period: {
    				description: "",
    				label: "Period"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			},
    			weeks: {
    				description: "",
    				label: "Weeks"
    			}
    		},
    		VarmaPensionOrderAction: {
    			move: {
    				description: "",
    				label: "Move"
    			},
    			newTyel: {
    				description: "",
    				label: "NewTyel"
    			},
    			newYel: {
    				description: "",
    				label: "NewYel"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		VarmaPensionOrderYelPayer: {
    			company: {
    				description: "",
    				label: "Company"
    			},
    			entrepreneur: {
    				description: "",
    				label: "Entrepreneur"
    			},
    			"undefined": {
    				description: "",
    				label: "Undefined"
    			}
    		},
    		WorkflowEventFeatures: {
    			isActive: {
    				description: "",
    				label: "IsActive"
    			},
    			isEditable: {
    				description: "",
    				label: "IsEditable"
    			},
    			isOnHold: {
    				description: "",
    				label: "IsOnHold"
    			},
    			isRemovable: {
    				description: "",
    				label: "IsRemovable"
    			},
    			none: {
    				description: "",
    				label: "None"
    			}
    		},
    		WorkflowEventUi: {
    			danger: {
    				description: "",
    				label: "Danger"
    			},
    			"default": {
    				description: "",
    				label: "Default"
    			},
    			info: {
    				description: "",
    				label: "Info"
    			},
    			primary: {
    				description: "",
    				label: "Primary"
    			},
    			success: {
    				description: "",
    				label: "Success"
    			},
    			warning: {
    				description: "",
    				label: "Warning"
    			}
    		},
    		YearEndAdminCheck: {
    			errors: {
    				description: "",
    				label: "Errors"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithWarning: {
    				description: "",
    				label: "OkWithWarning"
    			}
    		},
    		YearEndFeedbackCalculationType: {
    			correction: {
    				description: "",
    				label: "Correction"
    			},
    			external: {
    				description: "",
    				label: "External"
    			},
    			salaxyPayment: {
    				description: "",
    				label: "SalaxyPayment"
    			},
    			unknown: {
    				description: "",
    				label: "Unknown"
    			}
    		},
    		YearEndUserFeedback: {
    			noYearlyReports: {
    				description: "",
    				label: "NoYearlyReports"
    			},
    			notChecked: {
    				description: "",
    				label: "NotChecked"
    			},
    			ok: {
    				description: "",
    				label: "Ok"
    			},
    			okWithModifications: {
    				description: "",
    				label: "OkWithModifications"
    			},
    			willHandleMyself: {
    				description: "",
    				label: "WillHandleMyself"
    			}
    		}
    	}
    };

    var SALAXY$3 = {
    	MODEL: {
    	}
    };

    var SALAXY$2 = {
    	MODEL: {
    		AbsencePeriod: {
    			title: "absence period",
    			descr: "Period of absence",
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			}
    		},
    		AccountBase: {
    			title: "account base",
    			descr: "Account - either Person or Company - is a juridical entity Paying or receiving salaries, making contracts etc. It may be related to zero, one or multiple UserAccounts/Credentials.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AccountInIndex: {
    			title: "account in index",
    			descr: "Object that represents a Salaxy account.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			}
    		},
    		AccountProducts: {
    			title: "account products",
    			descr: "Provides an overview and the possibility to edit products that are enabled to this account",
    			accounting: {
    				title: "accounting",
    				descr: "Bookkeeping reports for salarypayments"
    			},
    			baseService: {
    				title: "base service",
    				descr: "The basic services for Salary payment."
    			},
    			healthCareHeltti: {
    				title: "health care heltti",
    				descr: "Employer health care from the company Heltti"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Reports and payments for mandatory employment insurances."
    			},
    			pension: {
    				title: "pension",
    				descr: "Reports ant payments to Pension companies."
    			},
    			pensionEntrepreneur: {
    				title: "pension entrepreneur",
    				descr: "Pension payments for an entrepreneur"
    			},
    			pricing: {
    				title: "pricing",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			tax: {
    				title: "tax",
    				descr: "Reports and payments to tax authorities."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "Reports and payments to TVR (the unemployment insurance fund of Finland)"
    			}
    		},
    		AccountValidation: {
    			title: "account validation",
    			descr: "Validation errors for the onboarding process",
    			canStartSign: {
    				title: "can start sign",
    				descr: "TODO: NOt implemented yet - should be implemented\nIf true, the onboarding object has sufficient data to start the Digital Signature process (Visma Sign).\nDepending on the use case this may or may not be required for committing the data as account data."
    			},
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			existingAccount: {
    				title: "existing account",
    				descr: "Existing account based on official id (Personal ID / Business ID)."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, none of the accounts in ExistingAccounts cannot be accessed by the current user."
    			},
    			isCompleted: {
    				title: "is completed",
    				descr: "If true, the validation process has been completed. The IsValid may be true or false."
    			},
    			isOfficialIdValid: {
    				title: "is official id valid",
    				descr: "If true the official ID is technically valid.\nOfficial ID is either Finnish Personal ID (HETU, Henkilötunnus) or Business ID (Y-tunnus)."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		Accountant: {
    			title: "accountant",
    			descr: "The person or account reponsible for accounting",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the account"
    			},
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			type: {
    				title: "type",
    				descr: "If true, the accountant is a registered Palkkaus.fi account who typically has access to this account."
    			}
    		},
    		AccountingData: {
    			title: "accounting data",
    			descr: "Accounting data for reports.",
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			ledgerAccounts: {
    				title: "ledger accounts",
    				descr: "All ledgers."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			}
    		},
    		AccountingDataSummary: {
    			title: "accounting data summary",
    			descr: "Summary of the accounting data. Contains main figures: totals and workers and employer total figures.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary of the calculations included in the report."
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment of the calculations included in the report."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "The total number of distinct workers included in the report."
    			}
    		},
    		AccountingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			accountantEmail: {
    				title: "accountant email"
    			},
    			accountantName: {
    				title: "accountant name"
    			},
    			accountantPhone: {
    				title: "accountant phone"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by AccountantName, AccountantEmail and AccountantPhone."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		AccountingReportRow: {
    			title: "accounting report row",
    			descr: "Accounting report table row.",
    			accountNumber: {
    				title: "account number",
    				descr: "Proposed account number for the row."
    			},
    			accountText: {
    				title: "account text",
    				descr: "Proposed account text for the row."
    			},
    			additionalInformation: {
    				title: "additional information",
    				descr: "Additional information for the row."
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount for the row: May be positive or negative(depending on the depet/credit and account type)."
    			},
    			count: {
    				title: "count",
    				descr: "Count of units. Default is 1."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the unit."
    			},
    			productCode: {
    				title: "product code",
    				descr: "Returns product code."
    			},
    			productName: {
    				title: "product name",
    				descr: "Returns product name. (Code or ResultCode in this order)."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Row type. Default is the Booking."
    			},
    			styles: {
    				title: "styles",
    				descr: "Space separated set of class names for CSS styling purposes."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the booking entry / header."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit identifier;"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percentage of the value added tax that is included in the amount."
    			}
    		},
    		AccountingReportTable: {
    			title: "accounting report table",
    			descr: "Contains accounting rows for one or many calculations.",
    			businessId: {
    				title: "business id",
    				descr: "Finnish Business Identifier (Y-tunnus) for the company."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the report. Typically the employer, but may be something else.\nBy desing, the contact information is not filled in for Private Person employers."
    			},
    			employer: {
    				title: "employer",
    				descr: "Employer avatar"
    			},
    			period: {
    				title: "period",
    				descr: "Period for the report."
    			},
    			rows: {
    				title: "rows",
    				descr: "All rows in the report table."
    			},
    			summary: {
    				title: "summary",
    				descr: "Summary containing relevant numbers for the report. For example,\nnumber of calculations, gross and net amounts, payment amounts and Palkkaus fee."
    			},
    			tableType: {
    				title: "table type",
    				descr: "Accounting report table type."
    			}
    		},
    		AccountingReportTableSummary: {
    			title: "accounting report table summary",
    			descr: "Summary of the accounting report. Contains main figures: totals and workers and employer total figures.",
    			count: {
    				title: "count",
    				descr: "The total number of calculations included in the report."
    			},
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		AccountingRuleSet: {
    			title: "accounting rule set",
    			descr: "Ruleset containing required tabular logic and accounts for creating accounting entries.",
    			accounts: {
    				title: "accounts",
    				descr: "Accounts which are used in the tabular logic for creating accounting entries."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the ruleset."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the rule set is read only."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the ruleset."
    			},
    			rows: {
    				title: "rows",
    				descr: "Tabular logic for creating accounting entries."
    			},
    			templateId: {
    				title: "template id",
    				descr: "Template to which this is based on."
    			}
    		},
    		AccountingRuleSetAccount: {
    			title: "accounting rule set account",
    			descr: "Single account with number, grouping and default text.",
    			account: {
    				title: "account",
    				descr: "Account number."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the account."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the account is non-removable template."
    			},
    			text: {
    				title: "text",
    				descr: "Text for the account."
    			}
    		},
    		AccountingRuleSetRow: {
    			title: "accounting rule set row",
    			descr: "Contains a single row of the tabular logic for creating accounting entries.",
    			creditGroupingId: {
    				title: "credit grouping id",
    				descr: "Additional flags for directing the logic for creating accounting entry based on given entries."
    			},
    			creditGroupingText: {
    				title: "credit grouping text"
    			},
    			creditId: {
    				title: "credit id"
    			},
    			debitGroupingId: {
    				title: "debit grouping id",
    				descr: "Id of the credit account."
    			},
    			debitGroupingText: {
    				title: "debit grouping text"
    			},
    			debitId: {
    				title: "debit id",
    				descr: "Id of the debit account."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if the row is in use."
    			},
    			entryCode: {
    				title: "entry code",
    				descr: "A single code in the code group: single income type code or total row type."
    			},
    			entryCodeGroup: {
    				title: "entry code group",
    				descr: "Entry group: income type or total type."
    			},
    			flags: {
    				title: "flags"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the row."
    			},
    			isIncluded: {
    				title: "is included",
    				descr: "Boolean indicating whether the system should make an accounting entry based on this row."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Boolean indicating if the row is non-removable template."
    			}
    		},
    		AccountingSettings: {
    			title: "accounting settings",
    			descr: "Settings related to accounting reports.",
    			accountant: {
    				title: "accountant",
    				descr: "The person or company responsible for accounting."
    			},
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			ruleSets: {
    				title: "rule sets",
    				descr: "Rulesets containing required tabular logic and accounts for creating accounting entries."
    			},
    			software: {
    				title: "software",
    				descr: "Integrated accounting software."
    			},
    			targets: {
    				title: "targets",
    				descr: "Accounting target systems and the rulesets assigned to them."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		AccountingSoftwareSettings: {
    			title: "accounting software settings",
    			descr: "Setting related to accounting software"
    		},
    		AccountingTarget: {
    			title: "accounting target",
    			descr: "Accounting target systems and the rulesets assigned to them.",
    			enabled: {
    				title: "enabled",
    				descr: "Boolean indicating if this target is in use"
    			},
    			id: {
    				title: "id",
    				descr: "Id of the target definition."
    			},
    			image: {
    				title: "image",
    				descr: "Illustrative image for the target system."
    			},
    			name: {
    				title: "name",
    				descr: "Name for the target system."
    			},
    			ruleSetId: {
    				title: "rule set id",
    				descr: "Assigned ruleset for this target system."
    			}
    		},
    		Ajax: {
    			title: "ajax",
    			descr: "Abstracts the interface for Http / Ajax functionality that is required by Salaxy core classes.\nThis interface is implemented in this project for jQuery and Node.\nAdditional implementations exists for Angular.js (salaxy-lib-ng1) and Angular.io / ngx (salaxy-lib-ng).\nYou may also create your own implementation for the framework or http component that you want to support.",
    			serverAddress: {
    				title: "server address",
    				descr: "The server address - root of the server. This is settable field.\nWill probably be changed to a configuration object in the final version."
    			},
    			useCredentials: {
    				title: "use credentials",
    				descr: "By default credentials are not used in http-calls.\nEnable credentials with this flag (force disabled when the token is set)."
    			},
    			getApiAddress: {
    				title: "get api address",
    				descr: "Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api'"
    			},
    			getCurrentToken: {
    				title: "get current token",
    				descr: "Gets the current Salaxy Bearer token"
    			},
    			getHTML: {
    				title: "get h t m l",
    				descr: "Gets a HTML-message from server using the API\n\n**Returns**: A Promise with result html. Standard Promise rejection to be used for error handling.\n"
    			},
    			getJSON: {
    				title: "get j s o n",
    				descr: "Gets a JSON-message from server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			getServerAddress: {
    				title: "get server address",
    				descr: "Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com'"
    			},
    			postHTML: {
    				title: "post h t m l",
    				descr: "POSTS data to server and receives back HTML.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			postJSON: {
    				title: "post j s o n",
    				descr: "POSTS data to server and receives back a JSON-message.\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			remove: {
    				title: "remove",
    				descr: "Sends a DELETE-message to server using the API\n\n**Returns**: A Promise with result data. Standard Promise rejection to be used for error handling.\n"
    			},
    			setCurrentToken: {
    				title: "set current token",
    				descr: "Sets Salaxy Bearer token for this http context."
    			}
    		},
    		AnnualLeave: {
    			title: "annual leave",
    			descr: "A planned annual Leave - not sure whether it will kept or is it just planned",
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeavePayment: {
    			title: "annual leave payment",
    			descr: "Payment related to annual leave: holiday salary, bonus and compensation",
    			calcId: {
    				title: "calc id",
    				descr: "If the payment is defined by a calculation, stores the calculation ID."
    			},
    			date: {
    				title: "date",
    				descr: "The reference date: Typically the Paid at date.\nThe items are typically sorted based on this."
    			},
    			holidayBonus: {
    				title: "holiday bonus",
    				descr: "Bonus for either Holiday Salary or Compensation."
    			},
    			holidayCompensation: {
    				title: "holiday compensation",
    				descr: "Compensation that is paid for holidays not kept."
    			},
    			holidayDays: {
    				title: "holiday days",
    				descr: "Days of holidays that are paid or compensated"
    			},
    			holidaySalary: {
    				title: "holiday salary",
    				descr: "Salary that is paid for holidays"
    			},
    			id: {
    				title: "id",
    				descr: "Identifies a single leave."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the payment entry"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "The planned holiday period."
    			}
    		},
    		AnnualLeaves: {
    			title: "annual leaves",
    			descr: "Data about the annual leaves planned and used during the holiday year.",
    			paid: {
    				title: "paid",
    				descr: "Paid or otherwise handled annual leaves.\nPRELIMINARY: This property is under revision - it may be moved or renamed without being a breaking change."
    			},
    			planned: {
    				title: "planned",
    				descr: "Planned annual leaves."
    			}
    		},
    		ApiCrudObject: {
    			title: "api crud object",
    			descr: "Base interface for CRUD objects in the Salaxy API.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Partner service that has be used in modifying the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated. Typically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		ApiListItem: {
    			title: "api list item",
    			descr: "The line item class for search and listing results. Use this class for building user interfaces.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			listType: {
    				title: "list type",
    				descr: "List item type of the item if the item supports a custom OData interface."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			type: {
    				title: "type",
    				descr: "The payload type of the item."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		ApiListItemValidation: {
    			title: "api list item validation",
    			descr: "ApiValidation object that is specialized for list items. Provides if and index properties for connecting a validation to a specific item in the list",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the item this validation is for.\nIdentifiers are not always available (if the item has not been saved), so index is a more generic approach."
    			},
    			index: {
    				title: "index",
    				descr: "Zero based index of the item this validation is for."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiProfileSearchResult: {
    			title: "api profile search result",
    			descr: "Profile search result for the API",
    			cities: {
    				title: "cities"
    			},
    			facets: {
    				title: "facets",
    				descr: "Dynamic for search facets if faceting is being used.\nNOTE: Facet names are lower camel case - using the JavaScript naming conventions as in the original JSON."
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			},
    			results: {
    				title: "results",
    				descr: "Actual search result - list of items"
    			},
    			searchText: {
    				title: "search text",
    				descr: "Search text that was input by the user"
    			},
    			searchTextTechnical: {
    				title: "search text technical",
    				descr: "Search text as it is sent to the search provider:\nContains provider specific wildcards etc."
    			},
    			sort: {
    				title: "sort",
    				descr: "Sort string"
    			},
    			totalCount: {
    				title: "total count",
    				descr: "Total count returned by the search service.\nThis may be more than the count of Result list"
    			}
    		},
    		ApiProfileSearchResultProfile: {
    			title: "api profile search result profile",
    			avatar: {
    				title: "avatar"
    			},
    			cities: {
    				title: "cities"
    			},
    			id: {
    				title: "id"
    			},
    			jobCategories: {
    				title: "job categories"
    			},
    			jobs: {
    				title: "jobs"
    			}
    		},
    		ApiValidation: {
    			title: "api validation",
    			descr: "Common base class / interface for data validation.",
    			errors: {
    				title: "errors",
    				descr: "Validation errors on invalid field values.\nNote that required fields missing error messages are not here. Only the invalid values."
    			},
    			hasAllRequiredFields: {
    				title: "has all required fields",
    				descr: "If true, has required fields missing data."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "If true, the data is valid - no errors."
    			}
    		},
    		ApiValidationError: {
    			title: "api validation error",
    			descr: "Validation errors used in API output.",
    			code: {
    				title: "code",
    				descr: "Some error providers may have a unique technical code for error or validator"
    			},
    			key: {
    				title: "key",
    				descr: "Full path to Name of the property / field (Member name in JSON).\nThis may be null/empty for type General."
    			},
    			msg: {
    				title: "msg",
    				descr: "Validation error message in the requested language."
    			},
    			type: {
    				title: "type",
    				descr: "High level type: Specifies the relation to object and its Member."
    			}
    		},
    		AppStatus: {
    			title: "app status",
    			descr: "Supports the UserSession object by providing additional information about the status\nof fetching the session, data, expiration etc.",
    			clientRoles: {
    				title: "client roles",
    				descr: "Application specific roles that are set on the client (browser code)\nas opposed to normal server-side defined roles."
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "Indicates that we are running in test environment with test data.\nThis information is typically available already before the session is available and may be used in login / register.\nNull indicates that the information is not available. This should typically be considered false for security."
    			},
    			sessionCheckInProgress: {
    				title: "session check in progress",
    				descr: "If true, the session check from server is in progress"
    			},
    			sessionExpired: {
    				title: "session expired",
    				descr: "PLACEHOLDER: Indicates that the session has expired (as opposed to not checked yet).\nThis property has not yet been implemented."
    			}
    		},
    		Article: {
    			title: {
    				title: "title",
    				descr: "Title: Shown in te beginning of the text, in the browser window header, in the lists etc."
    			},
    			descr: "Read-only Article object for the API",
    			author: {
    				title: "author",
    				descr: "Author of the article."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			date: {
    				title: "date",
    				descr: "Publish date"
    			},
    			file: {
    				title: "file",
    				descr: "File path as in CDN"
    			},
    			html: {
    				title: "html",
    				descr: "Article text and/or markup.\nThis is not serialized to XML, but read/written separately to blob storage - typically using MarkDown."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			longSummary: {
    				title: "long summary",
    				descr: "A slightly longer summary, shown in Facebook, Google+\nand LinkedIn likes/shares."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			picture: {
    				title: "picture",
    				descr: "URL to a picture that should be used in social media sharing"
    			},
    			section: {
    				title: "section",
    				descr: "The section where the article should appear"
    			},
    			shortSummary: {
    				title: "short summary",
    				descr: "Shorter summary (max 140 chars), shown as default\nin Tweets."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		AssureCompanyAccountRequest: {
    			title: "assure company account request",
    			descr: "Request to assure that a company account exists: Either an account is returned or an account stub (non-verified) will be created.",
    			contactFirstName: {
    				title: "contact first name",
    				descr: "First name of the contact person"
    			},
    			contactLastName: {
    				title: "contact last name",
    				descr: "Last name of the contact person"
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Company number in the the trade register (Y-tunnus).\nFormat is \"1234567-8\" - 7 numbers, one control character, with or without the dash.\nAlso, the international VAT format is allowed: FI12345678"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The account ID at the partner site.\nIf present, this ID is used to restrict the authorization to one account at partner site only."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number for the contact person"
    			}
    		},
    		AttributeDoc: {
    			title: "attribute doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attr: {
    				title: "attr",
    				descr: "HTML attribute name e.g. my-prop for myProp."
    			},
    			binding: {
    				title: "binding",
    				descr: "Angular JS binding type"
    			},
    			controller: {
    				title: "controller",
    				descr: "Controller that implements the property"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the memeber."
    			}
    		},
    		Author: {
    			title: "author",
    			descr: "Author of an article in the CMS",
    			companyDescription: {
    				title: "company description",
    				descr: "Short description text / label for the organization."
    			},
    			companyLogo: {
    				title: "company logo",
    				descr: "Company or organization logo - expect that this may be null."
    			},
    			companyUrl: {
    				title: "company url",
    				descr: "Link to a web site - typically of the organization that the author represents"
    			},
    			description: {
    				title: "description",
    				descr: "Short description text"
    			},
    			guid: {
    				title: "guid",
    				descr: "Unique key for the object"
    			},
    			image: {
    				title: "image",
    				descr: "Public image for the author"
    			},
    			name: {
    				title: "name",
    				descr: "Person name"
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Exact time of last technical save: Not editable by user and will change in imports/exports etc."
    			},
    			type: {
    				title: "type",
    				descr: "Supports the Cosmos DB migration by serializing the type of the current object as a short type string for querying."
    			}
    		},
    		Authority: {
    			title: "authority",
    			descr: "A person or other party that authorized the current person.",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account ID. This is optional and should not be set if the ID is the same as the target account / Person."
    			},
    			authorizedAt: {
    				title: "authorized at",
    				descr: "Date for the authorization."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail of the person. This is currently not used, but is expected to be used in the future.\nSo fill it in as it was required unless you have very compelling reasons."
    			},
    			name: {
    				title: "name",
    				descr: "Name of the authority."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Person ID is the key to signature. This is currently always required."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number of the person. This is currently optional.\nIdea is to send SMS invitations in some scenarios."
    			}
    		},
    		AuthorizationContract: {
    			title: "authorization contract",
    			descr: "Contract / authorization document that is the bases for Salaxy acting in behalf of this account",
    			authorizationType: {
    				title: "authorization type",
    				descr: "Type of the authorization"
    			},
    			authorizedBy: {
    				title: "authorized by",
    				descr: "A person or other party that authorized the current person."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "If true, there is a valid authorization - the account has been validated."
    			},
    			signedAt: {
    				title: "signed at",
    				descr: "Time when the contract was signed"
    			}
    		},
    		Avatar: {
    			title: "avatar",
    			descr: "Defines an avatar for an account, profile etc. - mainly the image that should be shown, names and short description",
    			color: {
    				title: "color",
    				descr: "Color - currently only used by type Icon"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			description: {
    				title: "description",
    				descr: "Short description of the user.\nThis may be overriden by a context specific value by the business logic."
    			},
    			displayName: {
    				title: "display name",
    				descr: "Display name. For a person this is 'FirstName LastName' (auto-created)."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type: person/company"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			initials: {
    				title: "initials",
    				descr: "Initials - currently only used by type Icon"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pictureType: {
    				title: "picture type",
    				descr: "Type of the Avatar picture."
    			},
    			sortableName: {
    				title: "sortable name",
    				descr: "Sortable name for ordered lists etc. For a person this is 'LastName, FirstName' (auto-created).\nDepending on the data storage, this may be all-lowercase for easier sorting and search."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			url: {
    				title: "url",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			}
    		},
    		BaseSalaryProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "First version of the product that integrates the Palkkaus.fi-salary payments to employers accounting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		BlobFile: {
    			title: "blob file",
    			descr: "The BlobFile stores metadata about miscellaneous BLOB-files that are stored to the system. For example Receipts and Authorizations.",
    			blobUri: {
    				title: "blob uri",
    				descr: "Full unique URI in the storage"
    			},
    			container: {
    				title: "container",
    				descr: "The logical container - first level folder for the file."
    			},
    			filePath: {
    				title: "file path",
    				descr: "Technical file name in storage.  In practical terms, this is the relative path\nexcluding the container part \"/[container]/\".\nI.e. this property is without the starting slash \"/\".\nIt must be unique and is used for Uri.\nFor user files this is constructed \"[Username]/[Type]/[UniqueFileName].[xxx]\""
    			},
    			fileType: {
    				title: "file type",
    				descr: "File type (mime type etc.) based on the ending of the file name.\nNull for folders."
    			},
    			id: {
    				title: "id"
    			},
    			isCollection: {
    				title: "is collection",
    				descr: "If true, this is a collection (folder in file system)"
    			},
    			length: {
    				title: "length",
    				descr: "The length of the file in bytes."
    			},
    			metadata: {
    				title: "metadata"
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "If created, contains uri to a PNG image for preview purposes."
    			},
    			type: {
    				title: "type",
    				descr: "Strongly typed logical type of the file. For user files, this is a sub folder."
    			},
    			updatedAt: {
    				title: "updated at"
    			},
    			userFriendlyName: {
    				title: "user friendly name",
    				descr: "A user friendly name that can be used in listings.\nIn the upload scenarios this is usually the original file name when uploaded.\nIt is not necessarily unique. No path information"
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		BlobFileMetadata: {
    			title: "blob file metadata",
    			descr: "Metadata related to blob files as a strongly typed object.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object related to the report.\nFor example for the salary payments this is the other party (employer or worker)"
    			},
    			data: {
    				title: "data",
    				descr: "Arbitrary metadata."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Identifier of the original calculation etc. business object."
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "Whether the document is digitally signed (or manually checked as being signed).\nAt the time of writing, this is used only for AuthorizationPdf documents."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "Logical date for the file. E.g. first of the month for a monthly report."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype is internal typing within a BlobFileType. This may or may not be a sub-folder.\nThere is no enumeration on File level - there may be one under services specific to one BlobFileType."
    			}
    		},
    		BoardUsecase: {
    			title: "board usecase",
    			descr: "Handles the Calculation of CalculationRowType.Board to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		BusinessObject: {
    			title: "business object",
    			descr: "Business object in the Salaxy repository",
    			id: {
    				title: "id",
    				descr: "Id of the business object"
    			},
    			name: {
    				title: "name",
    				descr: "Optional name for the business object."
    			},
    			object: {
    				title: "object",
    				descr: "Optional property for object instance."
    			},
    			owner: {
    				title: "owner",
    				descr: "Partition key of the business object"
    			},
    			type: {
    				title: "type",
    				descr: "Business object type"
    			}
    		},
    		CalcEmployer: {
    			title: "calc employer",
    			descr: "Employer of a calculation",
    			accountId: {
    				title: "account id",
    				descr: "The Palkkaus.fi Account identifier for the Employer."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image information and type of employer.\nNOTE: Recalculate-method retains the values for display purposes if AccountId or IsSelf is not set.\nThis is mainly for demo and testing in anonymous mode.\nHowever, in storage methods, the value is reset based on values of AccountId or IsSelf\neven if they are not set: Anything stored in Avatar will be ignored."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			}
    		},
    		CalcHolidayBonus: {
    			title: "calc holiday bonus",
    			descr: "Holiday Bonus data that is stored into a Salary calculation.",
    			alreadyPaid: {
    				title: "already paid",
    				descr: "The sum of holiday bonus that has already been paid this year.\nThe payment is never so big that this would become more than BonusTotalHolidaySalary\n(by this calculation - the limit can be exceeded manually)."
    			},
    			totalHolidaySalary: {
    				title: "total holiday salary",
    				descr: "Total yearly holiday salary for the purposes of holiday bonus."
    			},
    			yearlyLeaveDays: {
    				title: "yearly leave days",
    				descr: "Number of yearly leave days. This is for the purposes of calculating the Bonus,\nso this is yearly salary days, not the complete saldo (including StartSaldo)."
    			}
    		},
    		CalcHourlyData: {
    			title: "calc hourly data",
    			descr: "Holiday calculation based on hourly salary OR performance based salaries. May also contain other data related to working hours.",
    			isHourly: {
    				title: "is hourly",
    				descr: "If true, the hourly / performance based calculation is applied."
    			}
    		},
    		CalcInfo: {
    			title: "calc info",
    			descr: "Informational object that stores data about the salary calculation / payment that is not directly related to calculating the salary.",
    			backofficeNotes: {
    				title: "backoffice notes",
    				descr: "Notes related to the payment and other backoffice operations:\nFor example the justification why the sum is in calculation vs. Paytrail payment."
    			},
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			paymentChannel: {
    				title: "payment channel",
    				descr: "Channel that is used for payment."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "Payment reference number in payment service"
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "The Payroll Id, if this calculation is part of a Payroll list."
    			},
    			pensionPaymentDate: {
    				title: "pension payment date",
    				descr: "Calculated pension insurance payment date."
    			},
    			pensionPaymentRef: {
    				title: "pension payment ref",
    				descr: "Pension insurance reference number."
    			},
    			pensionPaymentSpecifier: {
    				title: "pension payment specifier",
    				descr: "Pension insurance payment specifier."
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "End date of the work"
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Start date of the work"
    			},
    			workerMessage: {
    				title: "worker message",
    				descr: "Message from worker to employer - when worker sends the salary calculation to the employer."
    			}
    		},
    		CalcMonthlyData: {
    			title: "calc monthly data",
    			descr: "Period recalculation parameters that are related to Monthly salary. Could theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested.",
    			absencesDaySalary: {
    				title: "absences day salary",
    				descr: "Price for a single absence day. Typically based on MonthlySalary, but may be modified."
    			},
    			isMonthly: {
    				title: "is monthly",
    				descr: "If true the Monthly (weekly) calculation is being applied to absences and holidays."
    			},
    			monthlySalary: {
    				title: "monthly salary",
    				descr: "Monthly salary in holiday and absences calculation.\nTypically, the same as the actual monthly calculation, but may be modified."
    			}
    		},
    		CalcRowConfig: {
    			title: "calc row config",
    			descr: {
    				title: "descr",
    				descr: "Description for the row type"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount input"
    			},
    			category: {
    				title: "category",
    				descr: "Categorization of the row types."
    			},
    			color: {
    				title: "color",
    				descr: "Color is based on the category"
    			},
    			favorite: {
    				title: "favorite",
    				descr: "Favorite row types and their sort order.\nThis is currently fixed, but may later be based on user profile (e.g. most used types)."
    			},
    			iconText: {
    				title: "icon text",
    				descr: "Icon short text is 1-5 characters suitable for icon.\nHas 5 characters only if very narrow, e.g. \"1\" and comma - typically 2-3 chars."
    			},
    			kind: {
    				title: "kind",
    				descr: "If the row has a kind selection (the main subtype), definition of the subtypes."
    			},
    			label: {
    				title: "label",
    				descr: "Label for the row type"
    			},
    			moreInfo: {
    				title: "more info",
    				descr: "More info HTML"
    			},
    			name: {
    				title: "name",
    				descr: "Name / unique identifier of the row type."
    			},
    			order: {
    				title: "order",
    				descr: "Order number in the lists"
    			},
    			price: {
    				title: "price",
    				descr: "Price input"
    			},
    			total: {
    				title: "total",
    				descr: "The total input or read-only field."
    			}
    		},
    		CalcTax: {
    			title: "calc tax",
    			descr: "Tax card information stored as part of the Calculation object",
    			estimatedTax: {
    				title: "estimated tax",
    				descr: "Calculated widthholding tax. Note that this is an estimation until the calculation is actually paid.\nAlso, if the salary is not paid as cash (e.g. fringe benefits), there may not be enough money to make the full deduction."
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Possibility to set a fixed widthholding tax amount for this calculation regardless of\nthe percentage etc. in the tax card."
    			},
    			hasNewSharedCard: {
    				title: "has new shared card",
    				descr: "A flag that during last recalculation, there was a new shared taxcard that was waiting approval."
    			},
    			isValid: {
    				title: "is valid",
    				descr: "The API returns true if there is a valid taxcard for the PaidAt date for this calculation\nor if FixedTaxAmount is set for th calculation."
    			},
    			snapshot: {
    				title: "snapshot",
    				descr: "Snapshot of the tax card for salary slip etc. purposes.\nUpdated in Recalculate / Save based on TaxCardId"
    			},
    			taxPercentForVisualization: {
    				title: "tax percent for visualization",
    				descr: "Tax percent that is used in visualization. This is mainly used in Unit testing and anonymous calcultors.\nNot used in real paid calculations. This is percent. I.e. for 50% set 50, not 0.5.\nIf PaymentData or AccountId lead to a valid tax card type and percent, or if there is FixedTaxAmount, this value is not used."
    			},
    			taxcardId: {
    				title: "taxcard id",
    				descr: "Identifier for a stored tax card if one is applied to this calculation.\nSystem updates the TaxCardId automatically in Recalculate / Save if it finds a better candidate\nand also updates the Snapshot."
    			}
    		},
    		CalcWorker: {
    			title: "calc worker",
    			descr: "Worker object for the current calculation",
    			accountId: {
    				title: "account id",
    				descr: "Palkkaus.fi Account Id for the worker.\nAlternatively you can set the Payment data with the required infromation"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Display image and properties of the Worker.\nBased on either the AccountId or PaymentData"
    			},
    			dateOfBirth: {
    				title: "date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf PaymentData or AccountId lead to a valid Social security number, it will override any age set here."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment Id for the worker. This is currently resolved using the workers AccountId."
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the currently Account\nIf set to true, will set current Account as this contract party.\nThis overrides any value in AccountId, so be sure to switch this back to false if this is switchable.\nSetting to False does not have any immediate effect."
    			},
    			paymentData: {
    				title: "payment data",
    				descr: "Data that is needed for salary payment.\nThis object may be null if AccountId has been set.\nAlso, if AccountId has been set, these properties will be ignored:\nThe Worker will decide on payment properties herself."
    			},
    			tax: {
    				title: "tax",
    				descr: "New Widthholding tax logic that is based on separately stored tax cards."
    			}
    		},
    		CalcWorkerPaymentData: {
    			title: "calc worker payment data",
    			descr: "The required information for paying the Salary to Worker.",
    			email: {
    				title: "email",
    				descr: "Contact e-mail address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name of the person."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "IBAN number for the Bank account"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name / Surname of the person."
    			},
    			socialSecurityNumber: {
    				title: "social security number",
    				descr: "Gets the Social security number the way the user set it in input.\nHOWEVER: The getter will change the string to upper invariant/trim and if the last character is \"*\", it will be replaced by calculated checksum."
    			},
    			socialSecurityNumberValid: {
    				title: "social security number valid",
    				descr: "Social security number if valid or null"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		CalcWorkflow: {
    			title: "calc workflow",
    			descr: "Defines the properties of salary calculation workflow - typically the workflow from draft to Payment.",
    			paidAt: {
    				title: "paid at",
    				descr: "Time when this calculation was paid (if it was paid) to Palkkaus service."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			salaryPaidAt: {
    				title: "salary paid at",
    				descr: "Time when the salary was paid by Palkkaus service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from Draft to PaymentSucceeded"
    			}
    		},
    		CalcWorktime: {
    			title: "calc worktime",
    			descr: "Worktime data that is stored as part of Salary calculation: work days, absences, holidays and holiday accrual. In the future may also contain relevant part of timesheet data (work hours).",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of days of non-paid absences.\nThese are deducted from the Monthly salary, not from Hourly salary at the moment."
    			},
    			accruals: {
    				title: "accruals",
    				descr: "Accruals for each calendar month in this calculation."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Parameters related to Holiday Bonus."
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period that is updated from employment at each recalculation (as opposed to holiday specification)."
    			},
    			holidaySpec: {
    				title: "holiday spec",
    				descr: "Specification for the holiday calculation"
    			},
    			hourly: {
    				title: "hourly",
    				descr: "Parameters related to Hourly or Performance based salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			leavesDailySalary: {
    				title: "leaves daily salary",
    				descr: "Price for a single holiday day"
    			},
    			leavesDays: {
    				title: "leaves days",
    				descr: "Number of paid Leave days that is paid in this salary calculation / for this period."
    			},
    			monthly: {
    				title: "monthly",
    				descr: "Parameters related to Monthly salary.\nCould theoretically be used for other long periods, namely Weekly salary, but that is not actively supported or tested."
    			},
    			workDays: {
    				title: "work days",
    				descr: "Describes the working days in the period.\nCurrently, uses the DaysCount, but later the full days array will be taken to use.\nThis includes the absence days (divider in AbsencesDaySalary)."
    			}
    		},
    		Calculation: {
    			title: "calculation",
    			descr: "Calculation is the most important model in the Palkkaus.fi API. We suggest you open the separately provided documentation and especially the class diagram to understand the whole model in detail.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer",
    				descr: "The employer object for this calculation.\nTypically, you do not need to set this before the actual payment process."
    			},
    			framework: {
    				title: "framework",
    				descr: "DEPRECIATED: Usecase will be replacing this functionality in a next major release.\nThe Framework Agreement (TES) parameters that affect the framework based side costs etc. of this calculation"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "Information about the work that is not directly related to calculating salary and expenses."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "The results of the calculation. In the API, this is a read-only object."
    			},
    			rows: {
    				title: "rows",
    				descr: "Set of rows that make up the salary calculation:\nExpenses, Benefits, deductions etc. Note that side costs are not rows."
    			},
    			salary: {
    				title: "salary",
    				descr: "The main salary element.\nAlternatively the salary can be set in the Rows object - this will be the only way to set multiple salary rows (not yet supported)."
    			},
    			sharing: {
    				title: "sharing",
    				descr: "Sharing of the calculation. This object cannot be modified directly,\nuse specialized methods in Calculations service."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing salary calculations.\nIn practice, a usecase is either a user interface or a microservice that provides a salary calculation\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of calculation.\nNOTE: This functionality will replace Framework property in a next major release."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker (employee) that has performed the work."
    			},
    			workflow: {
    				title: "workflow",
    				descr: "Information about the workflow and state of the calculation."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "Readonly workflow data, which is exposed to API."
    			},
    			worktime: {
    				title: "worktime",
    				descr: "Worktime data that is stored as part of Salary calculation:\nwork days, absences, holidays and holiday accrual.\nIn the future may also contain relevant part of timesheet data (work hours)."
    			}
    		},
    		CalculationData: {
    			title: "calculation data",
    			descr: "Calculation specific business data for index.",
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			payrollId: {
    				title: "payroll id",
    				descr: "Payroll id."
    			},
    			pensionPayment: {
    				title: "pension payment",
    				descr: "The total pension payment for this calculation."
    			},
    			requestedSalaryDate: {
    				title: "requested salary date",
    				descr: "A flag indicating if the requested date for the SalaryDate has been set.\nIf the date has been set, the value of the flag is 'SALA'."
    			},
    			socialSecurityPayment: {
    				title: "social security payment",
    				descr: "The social security payment- paid as part of taxation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the calculation from the end-user point-of-view."
    			},
    			taxPayment: {
    				title: "tax payment",
    				descr: "The taxes that were deducted from the worker salary a paid as part of taxation."
    			},
    			usecaseUri: {
    				title: "usecase uri",
    				descr: "Use case uri."
    			},
    			workEndDate: {
    				title: "work end date",
    				descr: "Work end date."
    			},
    			workStartDate: {
    				title: "work start date",
    				descr: "Work start date."
    			},
    			workerPayment: {
    				title: "worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			}
    		},
    		CalculationListItem: {
    			title: "calculation list item",
    			descr: "Represents a single item in a list of calculations in the user interface. It contains all essential information of the calculation and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		CalculationResult: {
    			title: "calculation result",
    			employerCalc: {
    				title: "employer calc",
    				descr: "The calculation from the Employer point-of-view"
    			},
    			irRows: {
    				title: "ir rows",
    				descr: "Rows that are compatible with Incomes Register (tulorekisteri) process.\nThese are available only for calculations paid after 7.2.2019 (TODO: estimation as of writing)."
    			},
    			responsibilities: {
    				title: "responsibilities",
    				descr: "Responsibilities of Employer and parameters that affect those responsibilities."
    			},
    			rows: {
    				title: "rows",
    				descr: "The result rows. Note that this collection also contains the main Salary entry as one of the rows."
    			},
    			totals: {
    				title: "totals",
    				descr: "Calculation totals that are common to both Worker and Employer."
    			},
    			validation: {
    				title: "validation",
    				descr: "The validation result for the current calculation.\nPlease note that this is not serialized into the data storage."
    			},
    			workerCalc: {
    				title: "worker calc",
    				descr: "The calculation from the Worker point-of-view"
    			}
    		},
    		CalculationSettings: {
    			title: "calculation settings",
    			descr: "Settings that affect the calculation result of the salaries.",
    			household: {
    				title: "household",
    				descr: "Settings that are specific to Households logic.\nThese do not apply to Companies and may be null for them"
    			},
    			insurance: {
    				title: "insurance",
    				descr: "Settings related to mandatory accident insurance."
    			},
    			marketing: {
    				title: "marketing",
    				descr: "Marketing settings, E.g. campaigns, customer groups etc."
    			},
    			pension: {
    				title: "pension",
    				descr: "Pension insurance related settings."
    			},
    			tax: {
    				title: "tax",
    				descr: "Settings related to taxation. Mainly widthholding tax, but potentially other taxes as well."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		CalculationSharing: {
    			title: "calculation sharing",
    			descr: "Defines the properties related to sharing and sending of the calculation.",
    			type: {
    				title: "type",
    				descr: "Type of sharing for this calculation."
    			},
    			uri: {
    				title: "uri",
    				descr: "Sharing uri."
    			}
    		},
    		CalculationSummary: {
    			title: "calculation summary",
    			descr: "Summarises from a large calculation object the relevant data for yearend feedback.",
    			benefitsInKindWithoutPhone: {
    				title: "benefits in kind without phone",
    				descr: "Benefits excluding phone benefit: Accomodation, Meals and other"
    			},
    			calculationId: {
    				title: "calculation id",
    				descr: "Identifier of the calculation or correction."
    			},
    			compensations: {
    				title: "compensations",
    				descr: "Compensations is daily allowance (including half days) plus meals compensation."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Total gross salary"
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes."
    			},
    			message: {
    				title: "message",
    				descr: "Message related to this row. Mainly for backoffice purposes."
    			},
    			milageCompensation: {
    				title: "milage compensation",
    				descr: "Milage compensation includes MilageOwnCar, Daily milage as well as other milage"
    			},
    			originalIsDeductible: {
    				title: "original is deductible",
    				descr: "If true the original calculation was marked as deductible (household deduction)."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "Date when the calculation was paid or where the correction should be set."
    			},
    			phoneBenefit: {
    				title: "phone benefit",
    				descr: "Phone benefit in euros"
    			},
    			salary: {
    				title: "salary",
    				descr: "Taxable salary of the calculation"
    			},
    			salaryType: {
    				title: "salary type",
    				descr: "Type of salary from Pension and Taxation point-of-view"
    			},
    			selectedIsDeductible: {
    				title: "selected is deductible",
    				descr: "If true the the calculation is currently marked as deductible (household deduction).\nDefault value is OriginalIsDeductible."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "The mandatory side costs for taxation purposes"
    			},
    			socialInsuranceDeductionsFromEmployee: {
    				title: "social insurance deductions from employee",
    				descr: "Pension and unemployment insurance deductions from the Worker"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "Categories for household deduction.\nUI implementors should force this enumeration being a single value although historically flags were supported."
    			},
    			type: {
    				title: "type",
    				descr: "Type of Calculation summary"
    			},
    			withholdingTax: {
    				title: "withholding tax",
    				descr: "Widthholding tax deducted from the Worker"
    			}
    		},
    		CalculationsDataRow: {
    			title: "calculations data row",
    			descr: "Data row for calculations data. This is typically used for imports and exports.",
    			costCenter: {
    				title: "cost center",
    				descr: "Cost center for reporting purposes."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			fixedTaxAmount: {
    				title: "fixed tax amount",
    				descr: "Tax if calculated"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the calculation"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End date of the work"
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start date of the work"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID for the worker."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			projectNumber: {
    				title: "project number",
    				descr: "Project number for reporting purposes."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			salarySlipMessage: {
    				title: "salary slip message",
    				descr: "Message which will be displayed in the payslip."
    			},
    			status: {
    				title: "status",
    				descr: "Read only calculation status"
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			},
    			workDescription: {
    				title: "work description",
    				descr: "Description of the work for reporting purposes. Max 32 chars."
    			}
    		},
    		CarBenefit: {
    			title: "car benefit",
    			ageGroupCode: {
    				title: "age group code"
    			},
    			carBenefitCode: {
    				title: "car benefit code"
    			},
    			kilometers: {
    				title: "kilometers"
    			}
    		},
    		CarBenefitUsecase: {
    			title: "car benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.CarBenefitUsecase to IrRows",
    			ageGroup: {
    				title: "age group",
    				descr: "Age group as in A,B,C or I for international."
    			},
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isKilometersBased: {
    				title: "is kilometers based",
    				descr: "If true, the calculation is kilometers based."
    			},
    			kilometers: {
    				title: "kilometers",
    				descr: "Number of kilometers is filled in only if that type of benefit calculation is used."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Certificate: {
    			title: {
    				title: "title",
    				descr: "End user description for the certificate."
    			},
    			descr: "User certificate.",
    			bytes: {
    				title: "bytes",
    				descr: "Certificate bytes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			password: {
    				title: "password",
    				descr: "Password for issuing a new certificate. Not persisted."
    			},
    			thumbprint: {
    				title: "thumbprint",
    				descr: "Certificate thumbprint."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		City: {
    			title: "city",
    			count: {
    				title: "count"
    			},
    			grouping: {
    				title: "grouping"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			nameLang2: {
    				title: "name lang2"
    			}
    		},
    		ClassDoc: {
    			title: "class doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			extendedBy: {
    				title: "extended by",
    				descr: "Types that extend this type (extendedBy)"
    			},
    			"extends": {
    				title: "extends",
    				descr: "Collection of reference types that this class extends (extendedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			implementedBy: {
    				title: "implemented by",
    				descr: "Types that iplement this interface (implementedBy)"
    			},
    			"implements": {
    				title: "implements",
    				descr: "Collection of interfaces that this class implements (implementedTypes).\nNote that string may also contain type parameters as in CrudControllerBase<YearlyFeedback>."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			members: {
    				title: "members",
    				descr: "Public properties, fields and functions."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			usages: {
    				title: "usages",
    				descr: "Salaxy specific logic may add usages here.\nCurrently, components will add themselves to controllers."
    			}
    		},
    		ClassMemberDoc: {
    			title: "class member doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Display grouping for the member: E.g. accessors are considered properties here."
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			parameters: {
    				title: "parameters",
    				descr: "For grouping type \"Method\", lists parameters\nTODO: Consider making a separate type for properties."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			type: {
    				title: "type",
    				descr: "Typedoc reflection type of the member."
    			}
    		},
    		CompanyAccount: {
    			title: "company account",
    			descr: "A company account - note that this can also be an association etc. - something that has a Company ID.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			companyType: {
    				title: "company type",
    				descr: "High level company type"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Majority shareholders (over 25% stake) are stored in this field.\nOwnership is a list of Shareholder objects."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		CompanyInput: {
    			title: "company input",
    			descr: "Represents the input for creating a company account.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the company"
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a company account based on company data."
    			},
    			doYtjUpdate: {
    				title: "do ytj update",
    				descr: "If true, will fetch the data from the Finnish Business Infromation System (YTJ/BIS).\nFetching is done firstly based on Business ID, if that is not valid then based on name."
    			},
    			logo: {
    				title: "logo",
    				descr: "URL to a Logo picture (preferably PNG). This logo should be a square - icon type of logo for avatar."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company"
    			},
    			ownership: {
    				title: "ownership",
    				descr: "Company ownership for KYC"
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "Identifier at the partner site - typicelly an organization ID."
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given BusinessID exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			},
    			ytjDetails: {
    				title: "ytj details",
    				descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)"
    			},
    			ytjSearch: {
    				title: "ytj search",
    				descr: "A search to Finnish Business Infromation System (YTJ/BIS). This search is triggered if you set the RawInput field."
    			}
    		},
    		ComponentDoc: {
    			title: "component doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			attributes: {
    				title: "attributes",
    				descr: "HTML attributes available to specify options to the component."
    			},
    			controller: {
    				title: "controller",
    				descr: "Name of the controller that implements the functionality of the component"
    			},
    			defaultTemplate: {
    				title: "default template",
    				descr: "The default template (view) used in the rendering of the component."
    			},
    			elem: {
    				title: "elem",
    				descr: "The component tag / element in HTML"
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			require: {
    				title: "require",
    				descr: "Required components in the outside scope"
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			transclude: {
    				title: "transclude",
    				descr: "Transclude slots"
    			}
    		},
    		Config: {
    			title: "config",
    			descr: "Environment specific configuration for Salaxy API's and JavaScript in general",
    			apiServer: {
    				title: "api server",
    				descr: "The base address of the Salaxy http API server, e.g. https://secure.salaxy.com"
    			},
    			isTestData: {
    				title: "is test data",
    				descr: "A flag indicating if the current configuration is for test environment only. If true, the site is for testing only."
    			},
    			useCookie: {
    				title: "use cookie",
    				descr: "A flag indicating if the site sets a Salaxy cookie which persists over web sessions. If true, the cookie will be created."
    			},
    			wwwServer: {
    				title: "www server",
    				descr: "The base address of the Palkkaus web site, e.g. https://www.palkkaus.fi"
    			}
    		},
    		Contact: {
    			title: "contact",
    			descr: "Common contact information object for Person, Company, Location etc.",
    			city: {
    				title: "city",
    				descr: "City or community (postitoimipaikka)"
    			},
    			countryCode: {
    				title: "country code",
    				descr: "This is the ISO code for country.\nCurrently, it should always be \"fi\"."
    			},
    			email: {
    				title: "email",
    				descr: "Contact e-mail address. Always lower invariant, trimmed and empty string set to null."
    			},
    			postalCode: {
    				title: "postal code",
    				descr: "Postal code"
    			},
    			street: {
    				title: "street",
    				descr: "Street address"
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number"
    			}
    		},
    		ContractParty: {
    			title: "contract party",
    			descr: "A signing party in a contract.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for the Party.\nNote that this is not automatically fetched by all methods: Especially when IsSelf is true."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information - note that for most signatures, only E-mail and phone number are required."
    			},
    			contractPartyType: {
    				title: "contract party type",
    				descr: "Party type (Person or Prospect)"
    			},
    			iban: {
    				title: "iban",
    				descr: "Bank account IBAN number"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the party: Currently always an account, but may be an external ID later."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true, contact is an employer"
    			},
    			isSelf: {
    				title: "is self",
    				descr: "True if Profile is the current Authorization.\nIf set to true, will set current Authorization as this contract party.\nIf set to false AND was previously true, will set the Profile properties null (unless explicitly set to something else)"
    			},
    			isSigned: {
    				title: "is signed",
    				descr: "True if the person has signed the contract."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official ID if needed in signature"
    			},
    			role: {
    				title: "role",
    				descr: "Role of the person - justification for authorization"
    			},
    			signatureUrl: {
    				title: "signature url",
    				descr: "Reserved for cases where there are several signees and each has a separate signature document."
    			}
    		},
    		ContractSalary: {
    			title: "contract salary",
    			descr: "Part of the Employment contract that is relevant to paying the salary",
    			collectiveContract: {
    				title: "collective contract"
    			},
    			endDate: {
    				title: "end date",
    				descr: "End date for the employment.\nOnly set if the contract is Fixed term."
    			},
    			payDay: {
    				title: "pay day"
    			},
    			salaryAmount: {
    				title: "salary amount"
    			},
    			salaryCompension: {
    				title: "salary compension"
    			},
    			salaryType: {
    				title: "salary type"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Starting date for the employment.\nShould always be set for a valid contract."
    			},
    			workHours: {
    				title: "work hours"
    			},
    			workType: {
    				title: "work type"
    			},
    			worksHoursType: {
    				title: "works hours type"
    			}
    		},
    		CoreTypeDoc: {
    			title: "core type doc",
    			descr: "H",
    			classes: {
    				title: "classes",
    				descr: "H"
    			},
    			enums: {
    				title: "enums",
    				descr: "H"
    			}
    		},
    		CustomerFundsSettings: {
    			title: "customer funds settings",
    			descr: "Settings related to payment methods that use the customer funds account (gross payment).",
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			isWorkerSelfHandling: {
    				title: "is worker self handling",
    				descr: "If true, worker salary and expense payments are paid directly by the employer."
    			}
    		},
    		DailyAllowanceUsecase: {
    			title: "daily allowance usecase",
    			descr: "Handles the Calculation of CalculationRowType.DailyAllowance to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		DataRow: {
    			title: "data row",
    			descr: "Interface for data row for exports and imports",
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		DateRange: {
    			title: "date range",
    			descr: "Describes the range of dates from start to end and the number of days in between.",
    			days: {
    				title: "days",
    				descr: "The collection of days contained in the DateRange if specified by the user.\nAlternative to DaysCount. If set, should also set DaysCount for backward compatibility."
    			},
    			daysCount: {
    				title: "days count",
    				descr: "Number of days in the range.\nDepending of the context, this may be mathematical (end - start) or\nexplicitly specified (typically working days).\nTypically an integer, and calculations and UI's may choose to round it."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			}
    		},
    		DeclarationReflection: {
    			title: "declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			}
    		},
    		EInvoiceAddress: {
    			title: "e invoice address",
    			descr: "Result item of the EInvoice address query.",
    			intermediator: {
    				title: "intermediator",
    				descr: "Intermediator id for this receiver address."
    			},
    			intermediatorName: {
    				title: "intermediator name",
    				descr: "Intermediator name for this receiver address."
    			},
    			receiver: {
    				title: "receiver",
    				descr: "Receiver EInvoice address of company."
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Name for the company."
    			},
    			receiverOfficialId: {
    				title: "receiver official id",
    				descr: "Official id of the company."
    			}
    		},
    		ESalaryPayment: {
    			title: "e salary payment",
    			descr: "E-salary / E-palkka is a way for a partner to direct an Employer to Salary payment without having access to Employer account. In first instance, E-salary is targeting Household Employers only.",
    			calc: {
    				title: "calc",
    				descr: "Calculation that should be paid by the Employer."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			household: {
    				title: "household",
    				descr: "Known data for the employer household that is paying the salary.\nIt is understood that the caller may not have all the information:\nPalkkaus.fi will ask the rest if necessary."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			request: {
    				title: "request",
    				descr: "Payment request and how to return to the calling service."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the Payment.\nNOTE: You can only set states Created, Shared and Opened.\nOther states are set by the Salaxy service and cannot be modified throgh API."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides the validation of the ESalaryPayment object after the object has been saved."
    			}
    		},
    		EmailMessage: {
    			title: "email message",
    			descr: "API EmailMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (email address)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			subject: {
    				title: "subject",
    				descr: "Message subject"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (email address)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		EmployerCalculationDTO: {
    			title: "employer calculation d t o",
    			descr: "Models the business logic of Employer side of the calculation. For documentation, see the IEmployerCalculation interface.",
    			allSideCosts: {
    				title: "all side costs",
    				descr: "All side cost: The mandatory side costs + Palkkaus and partner fees"
    			},
    			deductionForeclosure: {
    				title: "deduction foreclosure",
    				descr: "Foreclosure that has already been paid by the employer."
    			},
    			deductionOtherDeductions: {
    				title: "deduction other deductions",
    				descr: "Other deductions that are deducted from the net salary."
    			},
    			deductionPensionSelfPayment: {
    				title: "deduction pension self payment",
    				descr: "Pension payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionSalaryAdvance: {
    				title: "deduction salary advance",
    				descr: "Salary advance - this part has already been paid."
    			},
    			deductionTaxAndSocialSecuritySelfPayment: {
    				title: "deduction tax and social security self payment",
    				descr: "Tax and social security payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnemploymentSelfPayment: {
    				title: "deduction unemployment self payment",
    				descr: "Unemployment insurance (TVR) payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			deductionUnionPayment: {
    				title: "deduction union payment",
    				descr: "Union payment if it is deducted from the final figure"
    			},
    			deductionWorkerSelfPayment: {
    				title: "deduction worker self payment",
    				descr: "Worker salary and expense payments, when paid directly by the Employer - not by Palkkaus.fi"
    			},
    			finalCost: {
    				title: "final cost",
    				descr: "The final cost after household decuction from the tax authorities:\nAdd to the payment the deductions (paid by employer) and deduct House hold deduction (later potentially other such subsidies)"
    			},
    			foreclosureByPalkkaus: {
    				title: "foreclosure by palkkaus",
    				descr: "Foreclosure that Palkkaus.fi pays in behalf of employer."
    			},
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction that the tax man will reimburse in taxation.\nThis is an estimation: There is minimum and maximum limits etc."
    			},
    			mandatorySideCosts: {
    				title: "mandatory side costs",
    				descr: "The mandatory side costs are Pension (TyEL), Social secuirty and Unemployment insurance."
    			},
    			palkkaus: {
    				title: "palkkaus",
    				descr: "The Palkkaus.fi fee including potential partner fees."
    			},
    			pension: {
    				title: "pension",
    				descr: "The part of the Pension that Employer pays"
    			},
    			socialSecurity: {
    				title: "social security",
    				descr: "The social security payment- paid as part of taxation (ennakonpidätys)."
    			},
    			totalDeductions: {
    				title: "total deductions",
    				descr: "Total deductions that are made from the payment that is paid to Palkkaus and then to Worker"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment that is Paid to Palkkaus.fi"
    			},
    			totalPaymentLegacy: {
    				title: "total payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			totalSalaryCost: {
    				title: "total salary cost",
    				descr: "Total salary cost is the paid salary, deductions and side costs, but not expenses."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The part of the unemployment insurance that the employer pays"
    			}
    		},
    		EmployerGroups: {
    			title: "employer groups",
    			descr: "Employer groups that are set on Example calculations. These will be replaced by properties from Worker / Employer",
    			campaignPricing: {
    				title: "campaign pricing",
    				descr: "Campaign pricing may affect Palkkaus fees.\nThis field is set by the system when coming to the payment based on Employer properties (usually campaign code)."
    			},
    			employerType: {
    				title: "employer type",
    				descr: "Very small employers (households) do not required to report to the tax authorities."
    			},
    			isPrivatePerson: {
    				title: "is private person",
    				descr: "If true, the Employer is a private person.\nOtherwise Company or Unknown."
    			},
    			salaryPerMonth: {
    				title: "salary per month",
    				descr: "Very small salaries do not require Pension payments."
    			},
    			salaryPerYear: {
    				title: "salary per year",
    				descr: "Ranges for salary / year will affect the side costs."
    			}
    		},
    		EmployerInfo: {
    			title: "employer info",
    			descr: "Employer insurance information.",
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "The id of the insurance company."
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Insurance contract number."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "Indicates if the employer has an insurance."
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "Indicates if the insurance is provided by Palkkaus partnership insurance company."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "Indicates if the employer has a pension contract."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the mployer handles the pension himself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The id of the pension company."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Pension contract number."
    			}
    		},
    		EmploymentContract: {
    			title: "employment contract",
    			annualLeave: {
    				title: "annual leave"
    			},
    			benefitsException: {
    				title: "benefits exception"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employer: {
    				title: "employer"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isWorkAtHome: {
    				title: "is work at home"
    			},
    			mainWorkTasks: {
    				title: "main work tasks"
    			},
    			noticeExeption: {
    				title: "notice exeption"
    			},
    			otherPayDescription: {
    				title: "other pay description"
    			},
    			otherTerms: {
    				title: "other terms"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partTimeReason: {
    				title: "part time reason"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			salary: {
    				title: "salary"
    			},
    			testPeriod: {
    				title: "test period"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workPlace: {
    				title: "work place"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		EmploymentData: {
    			title: "employment data",
    			descr: "Employment specific business data for index.",
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			}
    		},
    		EmploymentListItem: {
    			title: "employment list item",
    			descr: "Represents a single item in a list of employments in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		EmploymentRelation: {
    			title: "employment relation",
    			descr: "Represents the Employment Relation: It is basically the properties of a Worker from the point-of-view of employer and as data owned by Employer whereas WorkerAccount is preliminarily owned by Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General information about the Employment relation."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		EmploymentRelationInfo: {
    			title: "employment relation info",
    			descr: "General information about the Employment relation.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			status: {
    				title: "status",
    				descr: "Status of this employment relationship"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			workerAccountId: {
    				title: "worker account id",
    				descr: "The Account ID of the Worker if Worker"
    			}
    		},
    		EmploymentTerminationUsecase: {
    			title: "employment termination usecase",
    			descr: "Handles the Calculation of CalculationRowType.EmploymentTermination to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		EnumDoc: {
    			title: "enum doc",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			names: {
    				title: "names",
    				descr: "Member names"
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			},
    			values: {
    				title: "values",
    				descr: "Member values"
    			}
    		},
    		EnumMetadata: {
    			title: "enum metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Type name of the enumeration (UpperCamelCase)"
    			},
    			values: {
    				title: "values",
    				descr: "Values of the enumeration."
    			}
    		},
    		EnumValueMetadata: {
    			title: "enum value metadata",
    			descr: {
    				title: "descr",
    				descr: "Description of the enumeration if the translations have been applied.\nNOTE: Not all enumerations have description in the globalization files:\nThey are added only if necessary to the UI."
    			},
    			label: {
    				title: "label",
    				descr: "Label of the enumeration if the translations have been applied."
    			},
    			name: {
    				title: "name",
    				descr: "Text value of the enumeration (lowerCamelCase)"
    			},
    			order: {
    				title: "order",
    				descr: "Sorting order number of the value within the enum."
    			}
    		},
    		FileData: {
    			title: "file data",
    			descr: "Report specific business data for index.",
    			calculationCount: {
    				title: "calculation count",
    				descr: "Number of calculations in the report."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id for the report."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			workerCount: {
    				title: "worker count",
    				descr: "Number of workers in the report."
    			}
    		},
    		FileListItem: {
    			title: "file list item",
    			descr: "Represents a single item in a list of reports in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		FileParameter: {
    			title: "file parameter",
    			data: {
    				title: "data"
    			},
    			fileName: {
    				title: "file name"
    			}
    		},
    		FileType: {
    			title: "file type",
    			extension: {
    				title: "extension"
    			},
    			isImage: {
    				title: "is image"
    			},
    			isText: {
    				title: "is text"
    			},
    			mimeType: {
    				title: "mime type"
    			}
    		},
    		HealthCareHelttiProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer health care from the company Heltti",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by HelttiProductPackage."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			industry: {
    				title: "industry",
    				descr: "Industry of work"
    			},
    			"package": {
    				title: "package",
    				descr: "Defines the product package"
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		HolidayAccrual: {
    			title: "holiday accrual",
    			descr: "Yearly log for Holiday accrual.",
    			defaultAccrual: {
    				title: "default accrual",
    				descr: "Default accrual for months where the Accrual occurs.\nWhen Holiday year is recalculated, this is determined either from Employment Period or AccrualFixed if set."
    			},
    			endSaldo: {
    				title: "end saldo",
    				descr: "End saldo from the previous year. This is Total + Start Saldo."
    			},
    			months: {
    				title: "months",
    				descr: "Holiday accrual entries during the period."
    			},
    			period: {
    				title: "period",
    				descr: "The period for holiday accrual.\nThis is also the period of the holiday year that the Worker is Employed,\ni.e. it may be shorter than HolidayYear Period if the employment relation\nbegins or ends within the holiday year."
    			},
    			startSaldo: {
    				title: "start saldo",
    				descr: "Start saldo from the previous year."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the current year. This does not include Start saldo."
    			}
    		},
    		HolidayAccrualEntry: {
    			title: "holiday accrual entry",
    			descr: "Entry in the Yearly Holiday Accrual list.",
    			calcId: {
    				title: "calc id",
    				descr: "Identier of the calculation if Kind"
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Days added to holiday Accrual: Typically days per month."
    			},
    			index: {
    				title: "index",
    				descr: "Index of the month from 0-11 for easier iteration.\nThis is also the real identifier of the month when editing the entries through API.\nMonths 4-12 are index 0-8 and month 1-3 are index 9-11"
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			source: {
    				title: "source",
    				descr: "Source of the holiday accrual."
    			}
    		},
    		HolidayDate: {
    			title: "holiday date",
    			date: {
    				title: "date"
    			},
    			holiday: {
    				title: "holiday"
    			},
    			holidayGroups: {
    				title: "holiday groups"
    			},
    			localName: {
    				title: "local name"
    			},
    			name: {
    				title: "name"
    			}
    		},
    		HolidaySpecification: {
    			title: "holiday specification",
    			descr: "Specifies how the holidays are handled for this Worker.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			}
    		},
    		HolidaySpecificationForYear: {
    			title: "holiday specification for year",
    			descr: "Extends the Holiday specification with the properties of employment relation that are required for a specific year.",
    			accrualFixed: {
    				title: "accrual fixed",
    				descr: "Defines the number of holidays that the worker is entitled per month worked.\nIf the value is set to 0 (default), the number is 2 for first year and 2,5 thereafter as defined in the law.\nTo actually se the daysPerMonth to zero, use code value other than Permanent14Days or Permanent35Hours."
    			},
    			bonus: {
    				title: "bonus",
    				descr: "Holiday bonus for the worker (lomaraha, lomaltapaluuraha).\nTypically 0.5 for 50%."
    			},
    			bonusMethod: {
    				title: "bonus method",
    				descr: "Method for paying holiday bonus in this salary calculation."
    			},
    			code: {
    				title: "code",
    				descr: "Defines how the holidays are handled for the worker."
    			},
    			compensation: {
    				title: "compensation",
    				descr: "Holiday compensation as percent of salary (11.5% is 0.115)"
    			},
    			employmentPeriod: {
    				title: "employment period",
    				descr: "Employment period for the purposes of holiday calculation (annual leave accrual).\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Wage basis for a Holiday Year.\nWhen this is changed, you need to make a rerun to the previous calculations in the holiday year.\nNOTE: This is not in HolidaySpecification because, the same value is stored in Employment relation."
    			}
    		},
    		HolidayYear: {
    			title: "holiday year",
    			descr: "Holiday year is in Finland from April to end of march. This is the time from which the holidays for the following summer and year are determined.",
    			accrual: {
    				title: "accrual",
    				descr: "Accrual of holidays (lomakirjanpito)."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation for the worker whose holidays are managed."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			leaves: {
    				title: "leaves",
    				descr: "Payments related to Annual Leaves (holidays)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			period: {
    				title: "period",
    				descr: "The holiday period for the year: Should always be 2.5.[Year]-30.4.[Year+1]"
    			},
    			spec: {
    				title: "spec",
    				descr: "Specifies how the holidays are handled.\nIf the Spec.Code is Undefined in Save, the contents is fetched from the current employment relation."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Salaxy ID of the worker. This value is required when inserting and cannot be edited later.\nWorkerSnapshot is automatically updated upon save."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			},
    			year: {
    				title: "year",
    				descr: "Year as integer. This value is required when inserting and cannot be edited later.\nPeriod will be set accordingly, or adjusted in case it is set below."
    			}
    		},
    		Holidays: {
    			title: "holidays",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		Holidays2: {
    			title: "holidays2",
    			allSaintsDay: {
    				title: "all saints day"
    			},
    			ascensionDay: {
    				title: "ascension day"
    			},
    			christmasDay: {
    				title: "christmas day"
    			},
    			christmasEve: {
    				title: "christmas eve"
    			},
    			easterMonday: {
    				title: "easter monday"
    			},
    			easterSaturday: {
    				title: "easter saturday"
    			},
    			easterSunday: {
    				title: "easter sunday"
    			},
    			epiphany: {
    				title: "epiphany"
    			},
    			goodFriday: {
    				title: "good friday"
    			},
    			independenceDay: {
    				title: "independence day"
    			},
    			mayDay: {
    				title: "may day"
    			},
    			midsummerDay: {
    				title: "midsummer day"
    			},
    			midsummerEve: {
    				title: "midsummer eve"
    			},
    			newYearsDay: {
    				title: "new years day"
    			},
    			pentecost: {
    				title: "pentecost"
    			},
    			stStephensDay: {
    				title: "st stephens day"
    			}
    		},
    		HouseholdSettings: {
    			title: "household settings",
    			descr: "Settings that are specific to Households. These do not apply to Companies.",
    			deductionInsuranceAmount: {
    				title: "deduction insurance amount",
    				descr: "Accident insurance amount reported for Household deduction purposes"
    			},
    			deductionSpouseName: {
    				title: "deduction spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			deductionSpousePersonalId: {
    				title: "deduction spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		HouseholdYearlyFeedback: {
    			title: "household yearly feedback",
    			descr: "Yearly feedback data that is specific to household customers. Main part is the information realted to household deduction and how it can be divided between spouses.",
    			insuranceAmount: {
    				title: "insurance amount",
    				descr: "Amount of accident insurance paid during this year as reported by the user\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			insuranceAmountFromAccount: {
    				title: "insurance amount from account",
    				descr: "Amount of accident insurance paid during this year from parner insurance (in Palkkaus.fi records).\nThis is relevant for households for Household deduction:\nThis amount can be deducted from the yearly salaries and needs to be reported to tax authorities."
    			},
    			spouseName: {
    				title: "spouse name",
    				descr: "Name of the spouse with who the household deduction is split"
    			},
    			spousePersonalId: {
    				title: "spouse personal id",
    				descr: "Finnish Personal ID of the spouse with who the household deduction is split"
    			}
    		},
    		IEmployerResponsibilities: {
    			title: "i employer responsibilities",
    			descr: "Parameters that affect the calculation. EmployerResponsibilities is historical name - currentlty to object holds also other properties that are not well described as responsibilities. These based on the EmployerGroups selection or properties of the Employer and Worker snapshots as well as this Calculation",
    			calculationPaymentDate: {
    				title: "calculation payment date",
    				descr: "The assumed salary payment date from the worker perspective.\nThe system calculates the nearest possible salary date if no any salary date has been given by the\nemployer. If given, the system confirms the given date.\nThis date affects the yearly percents used in the salary calculation."
    			},
    			employerGroups: {
    				title: "employer groups",
    				descr: "Employer groups that are set on Example calculations.\nThese will be replaced by properties from Worker / Employer when these persons are set."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isPensionMonthlyContract: {
    				title: "is pension monthly contract",
    				descr: "If true, the employer has a Pension contract and makes the payments\nin a monthly schedule which means that Pension interest is calculated for the 20th of the next month.\nCurrently, this is only enabled to Etera - other companies will follow."
    			},
    			noAccidentInsurance: {
    				title: "no accident insurance",
    				descr: "If true, this employer is not required to get an accident insurance."
    			},
    			noPensionPayment: {
    				title: "no pension payment",
    				descr: "Pension is not paid if the salary to this Worker is less than a certain amount in a month (about 60€)\nor worker is less than 17 or more than 64 years old."
    			},
    			noSocialSecurity: {
    				title: "no social security",
    				descr: "If worker is less than 16 or more than 67, social secuiry payment is not paid"
    			},
    			noTaxResponsibility: {
    				title: "no tax responsibility",
    				descr: "If Employer pays to this Worker max 200 euro, he does not need to even make the reports to tax office.\nHe may do so though"
    			},
    			noTvr: {
    				title: "no tvr",
    				descr: "If Worker is less than 17 or more than 64, TVR is not paid.\nAlso if the Employer has no accident insurance."
    			},
    			noWithholdingTax: {
    				title: "no withholding tax",
    				descr: "If Employer pays to this Worker max 1500 euro, he does not need to deduct the witholding tax.\nHe may do so though"
    			},
    			pensionAgeRange: {
    				title: "pension age range",
    				descr: "The age range that is used in the business logic that determines the employer responsibilities.\nThis property replaces the EmployerGroups.AgeRange that was previous get/set.\nThis is based on WorkerDateOfBirth and CalculationPaymentDate."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Pension calculation type."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The name of the Pension company because this affects the Pension percentage used in the calculation (only in contract situation).\nIt also needs to be shown in the reports."
    			},
    			pensionOldAgePayment: {
    				title: "pension old age payment",
    				descr: "If the Worker is from 53 to 62 years old, more of the Pension payment\nis deducted from the Worker side of the salary (less for the Employer).\nNOTE: People from 63 to 67 years pay the same amount as 17-52 years old."
    			},
    			pensionRegularEmployer: {
    				title: "pension regular employer",
    				descr: "The employer should have a Pension contract (not a temporary employer).\nIt does not necessarily mean that the user has a contract yet.\nNOTE: At the moment, this is not used for calculation - only in the Side costs visualizer tool."
    			},
    			taxAndSocialSecuritySelfPayment: {
    				title: "tax and social security self payment",
    				descr: "Tax and social security payments are paid directly by the Employer - not by Palkkaus.fi"
    			},
    			unemploymentSelfPayment: {
    				title: "unemployment self payment",
    				descr: "If true, the employer pays the Unemployment insurance directly to TVR.\nAs of writing, this is true for all companies and with calculations starting 1.8.2018,\nfalse for households or calculations before the date."
    			},
    			workerDateOfBirth: {
    				title: "worker date of birth",
    				descr: "Date of birth for the Worker - this affects the side costs.\nPlease also set the DateOfBirthAccuracy, if you set this property.\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerDateOfBirthAccuracy: {
    				title: "worker date of birth accuracy",
    				descr: "Accuracy of the date of birth\nIf SocialSecurityNumber is set, it will override any value set here."
    			},
    			workerSelfPayment: {
    				title: "worker self payment",
    				descr: "Worker salary and expense payments are paid directly by the Employer - not by Palkkaus.fi"
    			}
    		},
    		IUserObjectIndex: {
    			title: "i user object index",
    			descr: "Generic index for all User Object Containers.",
    			additionalInfo: {
    				title: "additional info",
    				descr: "A short detail describing the transaction.\nShould contain the relevant information for administration purposes."
    			},
    			businessAttributes: {
    				title: "business attributes",
    				descr: "Business attributes to include further information of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			containerGuid: {
    				title: "container guid",
    				descr: "Guid of the container.\nThis is not always unique within the index.\nHowver, within the container this is unique."
    			},
    			containerType: {
    				title: "container type",
    				descr: "The container type that is the source of this index item."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "Created at time"
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id for the object in 3rd party system. E.g. Filing code in MoneyTransfer."
    			},
    			flags: {
    				title: "flags",
    				descr: "Classifications for the object. First implementation supports BusinessEventType enumeration for values."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			handledAt: {
    				title: "handled at",
    				descr: "This is the recorded handling date for the object. E.g. when the calculation was paid to worker."
    			},
    			icon: {
    				title: "icon",
    				descr: "Icon for the purposes of listings."
    			},
    			id: {
    				title: "id",
    				descr: "Unique key for the object. Unique within an Index.\nIn the case of UserObject, this is [Guid]_[ContainerType]_[Owner].\nFor non-user objects, it should typically be just a Guid."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Messages (Error, Warning, OnHold, Note) that are active in the system.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the object is also the Partition key.\n Mandatory in most storage scenarios."
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Latest partner information."
    			},
    			payloadType: {
    				title: "payload type",
    				descr: "The business object type that is the source of this event."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			reference: {
    				title: "reference",
    				descr: "Reference information. E.g. Payment or MoneyTransfer reference number."
    			},
    			revenue: {
    				title: "revenue",
    				descr: "Estimated revenue that the transaction generated to Palkkaus.fi. Only add here the revenue, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Updated at time: Note that in typical implementations, this is the UserUpdatedAt time."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		IVerifiedAccount: {
    			title: "i verified account",
    			descr: "An account that is verified. Typically this means a digitally signed contract. This means that the account also has an Identity object. However, as the Identity object contains confidential information, it is not necessarily present in all method calls.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Pricing partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated"
    			}
    		},
    		Identity: {
    			title: "identity",
    			descr: "Identity is the official identity of an actor. This part is not freely editable as changing it requires signature or some other means. Identity is also often fetched from external sources such as govenment registries.",
    			contract: {
    				title: "contract",
    				descr: "The latest authorization contract that is the bases for this Identity"
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name OR company name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			isPep: {
    				title: "is pep",
    				descr: "Indicates whether the principal user of the customer is a politically exposed person or not."
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name\nAvailable only to real persons (as opposed to companies)."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the actor:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)"
    			},
    			primaryContact: {
    				title: "primary contact",
    				descr: "Mainly for company accounts, but in special cases for Persons,\nthis is the main Contact party and usually the signer for the account."
    			},
    			roles: {
    				title: "roles",
    				descr: "List of roles which this account is member of."
    			}
    		},
    		ImportBatchInfo: {
    			title: "import batch info",
    			descr: "Contains the basic details and the current status of the batch phase.\nTypically this data is created and updated during the import session.",
    			createdAt: {
    				title: "created at",
    				descr: "Date and time when the batch was created."
    			},
    			id: {
    				title: "id",
    				descr: "Unique id for the batch."
    			},
    			name: {
    				title: "name",
    				descr: "User friendly name for the batch."
    			},
    			phase: {
    				title: "phase",
    				descr: "Current phase of the batch:\n - ```new``` A new batch. Contains no data.\n - ```read``` The data file (e.g. csv) has been read from file or clipboard.\n - ```mapped``` The data has been mapped to a supported tabular format.\n - ```validated``` The data has been validated.\n - ```imported``` The data has been imported to backend.\n - ```rejected``` The batch has been rejected manually."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of the current phase:\n - ```info``` Typically the default status for the phase which has not yet been done.\n - ```success``` The current phase has been successfully finished.\n - ```warning``` The current phase has been finished with warnings.\n - ```error``` The current phase contains errors."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Date and time when the batch was last updated."
    			}
    		},
    		ImportResult: {
    			title: "import result",
    			descr: "Result for importing data (CSV, Excel copy-paste etc.)",
    			data: {
    				title: "data",
    				descr: "Result data is an array of objects with properties specified as headers"
    			},
    			errors: {
    				title: "errors",
    				descr: "Errors as validation errors."
    			},
    			headers: {
    				title: "headers",
    				descr: "Headers for the import: These are also the properties for the JSON"
    			},
    			ignoredLines: {
    				title: "ignored lines",
    				descr: "Number of ignored lines (empty line or rem/annotated) in the import."
    			}
    		},
    		ImportableCrudApi: {
    			title: "importable crud api",
    			descr: "Interface for APIs which implement import/export functionality.",
    			constructor: {
    				title: "constructor",
    				descr: "Constructor creates a new CRUD api with given Ajax-implementation"
    			},
    			ajax: {
    				title: "ajax"
    			},
    			baseUrl: {
    				title: "base url",
    				descr: "Deriving classes should define the baseURl for the method. E.g. \"/calculations\"\nAjax implementations will add the beginning (e.g. \"https://test-api.salaxy.com/api/v02\")"
    			},
    			"delete": {
    				title: "delete",
    				descr: "Deletes an single item from the sotrage\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			deleteWorkflowEvent: {
    				title: "delete workflow event",
    				descr: "Deletes the worklflow item or items from the item.\n\n**Returns**: A Promise with result data \"Object deleted\".\n"
    			},
    			getAll: {
    				title: "get all",
    				descr: "Gets all the items of a given type.\n\n**Returns**: A Promise with result data array.\n"
    			},
    			getBaseUrl: {
    				title: "get base url",
    				descr: "Returns the base url for this api."
    			},
    			getBlank: {
    				title: "get blank",
    				descr: "Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.\nThe basic idea is that all the child object properties should be non-null to avoid null reference exceptions.\nIn special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.\nStrings, dates and booleans may be null (false if bool), but typically just undefined."
    			},
    			getListItemMappers: {
    				title: "get list item mappers",
    				descr: "Available list item mappers for import/export"
    			},
    			getOData: {
    				title: "get o data",
    				descr: "Makes an OData query to the service's main OData method."
    			},
    			getODataBase: {
    				title: "get o data base",
    				descr: "Makes an OData query to the the given service URL."
    			},
    			getODataUrl: {
    				title: "get o data url",
    				descr: "Gets the OData URL for v03 compatible services"
    			},
    			getObjectMappers: {
    				title: "get object mappers",
    				descr: "Available object mappers for import/export"
    			},
    			getSingle: {
    				title: "get single",
    				descr: "Gets a single item based on identier\n\n**Returns**: A Promise with result data.\n"
    			},
    			getValidationResult: {
    				title: "get validation result",
    				descr: "Returns the validation object for the validated object."
    			},
    			save: {
    				title: "save",
    				descr: "Saves an item to the storage.\nIf id is null, this is add/insert. If id exists, this is update.\n\n**Returns**: A Promise with result data as saved to the storage (contains id, createAt, owner etc.).\n"
    			},
    			saveWorkflowEvent: {
    				title: "save workflow event",
    				descr: "Adds/updates the workflow event for the item.\n\n**Returns**: Workflow data for the item.\n"
    			},
    			validate: {
    				title: "validate",
    				descr: "Validates the item using a server side call.\n\n**Returns**: A Promise with result data containing validation property.\n"
    			}
    		},
    		IncomeTypeMetadata: {
    			title: "income type metadata",
    			descr: "Describes the properties and behavior of the Income types in Incomes register.",
    			accidentInsurance: {
    				title: "accident insurance",
    				descr: "If true Accident insurance payments are made for this income type by default."
    			},
    			calcGrouping: {
    				title: "calc grouping",
    				descr: "Grouping that is used in reporting etc."
    			},
    			code: {
    				title: "code",
    				descr: "Code in incomes register. Language versioned."
    			},
    			description: {
    				title: "description",
    				descr: "Description of the row.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			healthInsurance: {
    				title: "health insurance",
    				descr: "If true Health insurance payments are made for this income type by default."
    			},
    			insuranceInformationAllowed: {
    				title: "insurance information allowed",
    				descr: "If true the default values for insurances may be overriden for a single row."
    			},
    			isNegativeSupported: {
    				title: "is negative supported",
    				descr: "If true, teh row may have a negative value."
    			},
    			isSupported: {
    				title: "is supported",
    				descr: "If false, the row type is not supported in this version"
    			},
    			label: {
    				title: "label",
    				descr: "Language versioned label for the income type."
    			},
    			notSupportedError: {
    				title: "not supported error",
    				descr: "When isSupported is false, there is a describtive text that tells why the type is not supported. Text is always in Finnish."
    			},
    			paymentDefault: {
    				title: "payment default",
    				descr: "Default behavior for payment through Salaxy Customer Funds account"
    			},
    			pensionInsurance: {
    				title: "pension insurance",
    				descr: "If true Pension payments are made for this income type by default."
    			},
    			taxAndSidecostsDescr: {
    				title: "tax and sidecosts descr",
    				descr: "Description of how the row is taken into widthholding tax\nand how insurance payments (pension, unemployment, accident and health care) are made.\nNote that this is long text with line feeds / Markdown\nText is currently always in Finnish."
    			},
    			taxDefault: {
    				title: "tax default",
    				descr: "Describes the behavior of this row in calculating widthholding tax."
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "If true Unemployment insurance payments are made for this income type by default."
    			}
    		},
    		InsuranceProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Employer insurance. Currently, this is the mandatory employer insurance only.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPartnerInsurance."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			insuranceCompany: {
    				title: "insurance company",
    				descr: "Company for the current insurance"
    			},
    			insuranceContractNumber: {
    				title: "insurance contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isInsuranceContractDone: {
    				title: "is insurance contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		InsuranceSettings: {
    			title: "insurance settings",
    			company: {
    				title: "company",
    				descr: "Company for the current insurance"
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Contract number for the current insurance."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "If true, the user indicates that he/she has already made an insurance contract.\nShould also fill in the InsuranceCompany and InsuranceContractNumber"
    			},
    			isPartnerInsurance: {
    				title: "is partner insurance",
    				descr: "If set to true, the insurance is set to a partner insurance."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the insurance pension"
    			}
    		},
    		Invoice: {
    			title: "invoice",
    			descr: "Invoice for the pending payment in the business system. For example, a salary gross payment created for the salary calculation and delivered as an e-invoice to the employer. For example, a salary net payment, created for the employer which is self paying the net payment to the worker.",
    			content: {
    				title: "content",
    				descr: "Invoice data in XML or json format."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			header: {
    				title: "header",
    				descr: "Invoice header details."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Invoice payer details."
    			},
    			pdf: {
    				title: "pdf",
    				descr: "Invoice in pdf format."
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Invoice recipient details."
    			},
    			rows: {
    				title: "rows",
    				descr: "List of invoice rows."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		InvoiceData: {
    			title: "invoice data",
    			descr: "Invoice specific business data for index.",
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			periodEnd: {
    				title: "period end",
    				descr: "End of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			periodStart: {
    				title: "period start",
    				descr: "Start of period for the Invoice: Work period, month / quarter of sidecosts etc."
    			}
    		},
    		InvoiceHeader: {
    			title: "invoice header",
    			descr: "Header for the invoice",
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business object ids (typically calculations) on which the payment is based on."
    			},
    			channel: {
    				title: "channel",
    				descr: "Channel that is used for paying the invoice."
    			},
    			date: {
    				title: "date",
    				descr: "Invoice Date."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the invoice."
    			},
    			externalId: {
    				title: "external id",
    				descr: "The external ID is used for matching the executed payment from the 3rd party system\nto the pending payment in the Palkkaus system."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this invoice is done.\nFor salaries, this is salary date, for monthly / quarterly payments, this is the first day of period."
    			},
    			message: {
    				title: "message",
    				descr: "The free text that is displayed / sent with the invoice.\nMay have multiple lines."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the Invoice: Work period, month / quarter of sidecosts etc."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Payment reference (e.g. RF0710010831).\nPlease note that in salary payment scenarios, this is not necessarily unique."
    			},
    			status: {
    				title: "status",
    				descr: "The current notified status of the invoice in external system."
    			},
    			total: {
    				title: "total",
    				descr: "Total payable amount for the invoice including VAT."
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Total amount for the invoice excluding VAT."
    			},
    			type: {
    				title: "type",
    				descr: "Invoice type."
    			}
    		},
    		InvoiceListItem: {
    			title: "invoice list item",
    			descr: "Represents a single item in a list of invoices in the user interface. It contains all essential information of the report and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		InvoiceParty: {
    			title: "invoice party",
    			descr: "Party in an Varma Invoice: Either Payer or Recipient.",
    			companyId: {
    				title: "company id",
    				descr: "Finnish Company ID (Y-tunnus)."
    			},
    			companyName: {
    				title: "company name",
    				descr: "Organisation name"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information"
    			},
    			contactPersonName: {
    				title: "contact person name",
    				descr: "Contact person Full name"
    			}
    		},
    		InvoicePayer: {
    			title: "invoice payer",
    			descr: "Payment payer party of the invoice.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the party."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the party."
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official identifier of the party:\nFinnish HETU (Soc. sec. number) for a Person or Y-tunnus (Company ID for a company or association)."
    			},
    			partnerKey: {
    				title: "partner key",
    				descr: "Optional ID for identifying the party in the third party system.\nThis is the same partner key that is given as part of Single Sign-On process."
    			}
    		},
    		InvoiceRecipient: {
    			title: "invoice recipient",
    			descr: "Payment receiver party of the invoice.",
    			beneficiary: {
    				title: "beneficiary",
    				descr: "The ultimate beneficiary of the money:\nThis may be a Palkkaus.fi Worker Account (Avatar id is not null)\nor some other entity: Tax authorities, pension company etc. (Avatar id is null).\nTypically beneficiary is the same as Iban, OfficialId, FullName and Contact in this object,\nbut this is not true in edge cases (e.g. customer funds account)."
    			},
    			contact: {
    				title: "contact",
    				descr: "Optional contact info. Not available for all recipients."
    			},
    			fullName: {
    				title: "full name",
    				descr: "The full name of the payment receiver for payment purposes."
    			},
    			iban: {
    				title: "iban",
    				descr: "The International Bank Account Number of the receiver."
    			},
    			officialId: {
    				title: "official id",
    				descr: "This is an official identifier of the party: Finnish Personal Id (HETU) for a Person or Company Id (Y-tunnus) for a company or association.\nWARNING: This is currently needed for Finvoice only (https://www.finanssiala.fi/finvoice/Sivut/default.aspx). As such we consider removing it from persons.\nPlease inform if you use this property!"
    			}
    		},
    		InvoiceRow: {
    			title: "invoice row",
    			descr: "Row for the invoice",
    			accountingAccountNumber: {
    				title: "accounting account number",
    				descr: "Suggested account number (RowNormalProposedAccountIdentifier)."
    			},
    			accountingCode: {
    				title: "accounting code",
    				descr: "Textual code for accounting mapping (RowProposedAccountText, Raportointikoodi).\nIn Finvoice: \"Accounting information agreed between seller and buyer.\""
    			},
    			articleId: {
    				title: "article id",
    				descr: "Article Identifier is given by seller, EAN etc.\nIn the case of Salaxy salaries, this is CalculationRowType."
    			},
    			articleName: {
    				title: "article name",
    				descr: "Name of the product or service.\nIn the case of Salaxy salaries, this is language versioned text of CalculationRowType."
    			},
    			businessObject: {
    				title: "business object",
    				descr: "Business object id (typically calculation) on which the row is based on."
    			},
    			count: {
    				title: "count",
    				descr: "Count (DeliveredQuantity in Finvoice) is the quantity of product or service."
    			},
    			message: {
    				title: "message",
    				descr: "Message (RowFreeText in Finvoice) is the message is mainly designed for accounting systems."
    			},
    			price: {
    				title: "price",
    				descr: "Price including VAT (UnitPriceVatIncludedAmount, Yksikköhinta verollinen)"
    			},
    			priceExVat: {
    				title: "price ex vat",
    				descr: "Price of one product or service without Vat (UnitPriceAmount, Yksikköhinta veroton).\nYksikköhinta voidaan ilmoittaa myös verollisena UnitPriceVatIncludedAmount- elementissä laskutuskäytännön ja verottajan ALV-ohjeistuksen mukaan."
    			},
    			total: {
    				title: "total",
    				descr: "Row total with VAT (RowAmount, Yhteensä verollinen)"
    			},
    			totalExVat: {
    				title: "total ex vat",
    				descr: "Row total excluding VAT (RowVatExcludedAmount, Yhteensä veroton)"
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Vat percent related to the product or service (RowVatRatePercent, Alv%)"
    			}
    		},
    		InvoiceSettings: {
    			title: "invoice settings",
    			descr: "Settings related to invoices that are sent to customers.",
    			eInvoiceIntermediator: {
    				title: "e invoice intermediator",
    				descr: "Receiver's operator for the e-invoice."
    			},
    			eInvoiceReceiver: {
    				title: "e invoice receiver",
    				descr: "Receiver's address for the e-invoice."
    			}
    		},
    		InvoiceStatusNotification: {
    			title: "invoice status notification",
    			descr: "Contents for the invoice status notification message.",
    			amount: {
    				title: "amount",
    				descr: "Required realized amount of the transaction."
    			},
    			date: {
    				title: "date",
    				descr: "Required Date for the transaction (booking date)."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id can be set in the notification for the invoice for later messaging.\nIf the notification message contains an External Id AND regular Id, the External Id is updated in the Salaxy database.\nIf the notification contains External Id, but NO regular Id, an invoice is fetched based on External Id.\nIn the latter case, a non-existent Id or multiple invoices with the same External Id will throw an exception and the entire update process is canceled."
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID for the invoice in Salaxy system.\nIf omitted, the resolving to an invoice will be made using external id."
    			},
    			paymentType: {
    				title: "payment type",
    				descr: "The payment type of the invoice. If the payment channel does not support the requested payment type (e.g. Salary),\nit should set here the payment type that it used (e.g. Default).\nPayment channel should also expect new types to be introduced, without that being a breaking change.\nCurrently, we expect Default always be supported."
    			},
    			receiverFullName: {
    				title: "receiver full name",
    				descr: "Optional full name of the payment receiver. Should be sent only if the user or process has the possibility of changing the name."
    			},
    			receiverIban: {
    				title: "receiver iban",
    				descr: "Optional bank account IBAN of the receiver. Should be sent only if the user or process has the possibility of changing the IBAN."
    			},
    			referenceNumber: {
    				title: "reference number",
    				descr: "Optional payment reference number. Should be sent only if the user or process has the possibility of changing or removing the reference number.\nChanging the reference number will typically lead to an error in the back office process that needs to be manually handled."
    			},
    			status: {
    				title: "status",
    				descr: "New status for the invoice. Required main payload of the call."
    			}
    		},
    		IrDetails: {
    			title: "ir details",
    			descr: "Reporting details to Incomes registry (Tulorekisteri).",
    			carBenefit: {
    				title: "car benefit",
    				descr: "Provides the additional data needed for car benefit."
    			},
    			code: {
    				title: "code",
    				descr: "The number code in Incomes Register."
    			},
    			dailyAllowance: {
    				title: "daily allowance",
    				descr: "Daily allowance code."
    			},
    			flags: {
    				title: "flags",
    				descr: "Boolean flags that can be set on transaction."
    			},
    			insuranceExceptions: {
    				title: "insurance exceptions",
    				descr: "Defines the exceptions to Insurance / Sidecost defaults that are set by the row type."
    			},
    			mealBenefitIsTaxValue: {
    				title: "meal benefit is tax value",
    				descr: "Set this to true if meak benefit is of tax value - false if it si not.\nThe value should be null if the row is not mealBenefit."
    			}
    		},
    		IrIncomeTypeUsecase: {
    			title: "ir income type usecase",
    			descr: "Handles the Calculation of CalculationRowType.IrIncomeType to IrRows",
    			irData: {
    				title: "ir data",
    				descr: "Reporting details to Incomes registry (Tulorekisteri)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		IrRow: {
    			title: "ir row",
    			descr: "Business level modeling corresponding to WageReportsToIRTypes Transaction. Also contains additional data for accounting and integration purposes.",
    			accounting: {
    				title: "accounting",
    				descr: "Accounting related data for the row."
    			},
    			calcData: {
    				title: "calc data",
    				descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR)."
    			},
    			count: {
    				title: "count",
    				descr: "Count for the row. Default is one."
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			earningPeriod: {
    				title: "earning period",
    				descr: "Earnings period if differnet than the report period.\nNote that this property may be (and by default is) null."
    			},
    			irData: {
    				title: "ir data",
    				descr: "Details for the National Incomes registry"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row."
    			},
    			total: {
    				title: "total",
    				descr: "Simple multiplication: Count * Price - no other logic. Not used as Input."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the row - basically describes the source for this row:\nEither Manual (default) or Usecase logic, which may be legacy / API v02 or new / API v03."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. If specified as Undefined, set by the server-logic based on row type etc."
    			}
    		},
    		IrRowCalculationProperties: {
    			title: "ir row calculation properties",
    			descr: "Details from Salaxy internal calulcation point-of-view (logic not in IR).",
    			behavior: {
    				title: "behavior",
    				descr: "Defines the behavior of the row in Salaxy calculation process."
    			},
    			grouping: {
    				title: "grouping",
    				descr: "Grouping for reports etc."
    			}
    		},
    		JobProfile: {
    			title: "job profile",
    			backgroundImage: {
    				title: "background image",
    				descr: "Background image for the profile page"
    			},
    			cities: {
    				title: "cities",
    				descr: "A collection of known city IDs.\nQuery the interface for city IDs"
    			},
    			description: {
    				title: "description",
    				descr: "Long description: This field accepts MarkDown, potentially also HTML in the future (at the moment HTML may create an error)."
    			},
    			jobDetail: {
    				title: "job detail",
    				descr: "Detailed description of the Job"
    			},
    			jobTitle: {
    				title: "job title",
    				descr: "Title for the Job"
    			},
    			jobs: {
    				title: "jobs",
    				descr: "A collection of job ID's that this work or worker belongs"
    			},
    			links: {
    				title: "links",
    				descr: "External links to web sites, social media pages etc."
    			},
    			location: {
    				title: "location",
    				descr: "Free text for location of work or worker"
    			},
    			salaryMax: {
    				title: "salary max",
    				descr: "Max salary for the given salary period"
    			},
    			salaryMin: {
    				title: "salary min",
    				descr: "Minimum salary for the given salary period"
    			},
    			salaryPeriod: {
    				title: "salary period",
    				descr: "Period for which the Salary period is set."
    			}
    		},
    		LedgerAccount: {
    			title: "ledger account",
    			descr: "Represents a single accounting ledger.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			entries: {
    				title: "entries",
    				descr: "Legder entries"
    			},
    			saldo: {
    				title: "saldo",
    				descr: "Current saldo for the ledger."
    			}
    		},
    		LedgerEntry: {
    			title: "ledger entry",
    			descr: "Represents a single ledger entry.",
    			accountGrouping: {
    				title: "account grouping",
    				descr: "Account grouping."
    			},
    			accountName: {
    				title: "account name",
    				descr: "Account name."
    			},
    			accountNumber: {
    				title: "account number",
    				descr: "Account number."
    			},
    			amount: {
    				title: "amount",
    				descr: "Posted amount."
    			},
    			description: {
    				title: "description",
    				descr: "Entry description."
    			},
    			dimension: {
    				title: "dimension",
    				descr: "Dimension data."
    			},
    			entryDate: {
    				title: "entry date",
    				descr: "Entry date."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the entry."
    			},
    			isDebit: {
    				title: "is debit",
    				descr: "Debit indicator."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "VAT percent."
    			}
    		},
    		"Map": {
    			title: "map",
    			descr: "Map between a hierarchical Salaxy object and a row in a table/dataset."
    		},
    		MapSelector: {
    			title: {
    				title: "title",
    				descr: "Title of the property. If null, path is used with reflection."
    			},
    			descr: {
    				title: "descr",
    				descr: "Description of the property. If null, path is used with reflection."
    			},
    			fromObject: {
    				title: "from object",
    				descr: "Function that defines how the row value(s) is derived from the object, typically in export.\nIf null, this is defined by path."
    			},
    			fromRow: {
    				title: "from row",
    				descr: "Function that sets the value from row back to object, typically in import.\nIf null, this is defined by path."
    			},
    			path: {
    				title: "path",
    				descr: "Defines source property as a simple object path if possible.\nIf a more complex logic is needed, you can define get and set functions below.\nThe property is also used in generating documentation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the property. If null, path is used with reflection."
    			}
    		},
    		MarketingSettings: {
    			title: "marketing settings",
    			descr: "Marketing related settings which may affect the pricing. E.g. campaigns, customer groups etc.",
    			campaignCode: {
    				title: "campaign code",
    				descr: "A campaign code that may affect product pricing."
    			}
    		},
    		MealBenefitUsecase: {
    			title: "meal benefit usecase",
    			descr: "Handles the Calculation of CalculationRowType.MealBenefit to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			isTaxValue: {
    				title: "is tax value",
    				descr: "If true, the deduction from Worker salary corresponds to taxable value."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			taxablePrice: {
    				title: "taxable price",
    				descr: "The taxable price calculated by the server."
    			}
    		},
    		Measurement: {
    			title: "measurement",
    			descr: "Measurement value.",
    			tag: {
    				title: "tag",
    				descr: "Tag for the measurement."
    			},
    			timestamp: {
    				title: "timestamp",
    				descr: "Time when the measurement was recorded."
    			},
    			value: {
    				title: "value",
    				descr: "Actual measurement value."
    			}
    		},
    		Message: {
    			title: "message",
    			descr: "A message within a messaging thread.",
    			content: {
    				title: "content",
    				descr: "Textual content: If message, this is the content text.\nIf other type, short title / description typically shown in the title bar."
    			},
    			contentData: {
    				title: "content data",
    				descr: "Other misc. data that is used in displaying the content in the immediate message view.\nDo not include the entire object here. Just the immediately available data."
    			},
    			contentId: {
    				title: "content id",
    				descr: "Identifier of the external content.\nBlobFile Id for attachments, Calculation ID for calculations etc."
    			},
    			fromParty: {
    				title: "from party",
    				descr: "The party / type of the message sender."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the message."
    			},
    			preview: {
    				title: "preview",
    				descr: "Content type specific text for preview.\nThis may be URI, ID or boolean.\nFor BlobFile, it is currently the BlobFile Guid (the same as ContentId) or null,\nbut in the future, the field may also contain an URL without that being a breaking change!"
    			},
    			readByOtherParty: {
    				title: "read by other party",
    				descr: "If set, specifies the time when the message has been read by the other party."
    			},
    			readByOwner: {
    				title: "read by owner",
    				descr: "If set, specifies the time when the message has been read by the thread owner."
    			},
    			time: {
    				title: "time",
    				descr: "Time for the message"
    			},
    			type: {
    				title: "type",
    				descr: "Content type of the message"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validates the object that is the payload of the message."
    			}
    		},
    		MessageData: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Message thread specific business data for index.",
    			otherPartyLastMessage: {
    				title: "other party last message",
    				descr: "Last message that was created by the other party"
    			},
    			otherPartyUnreadCount: {
    				title: "other party unread count",
    				descr: "Number of messages that have been read by the Other party and have not been read by the owner."
    			},
    			ownerLastMessage: {
    				title: "owner last message",
    				descr: "Last message that was was created by the owner"
    			},
    			ownerUnreadCount: {
    				title: "owner unread count",
    				descr: "Number of messages by Owner that have not been read by the other party"
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			}
    		},
    		MessageListItem: {
    			title: "message list item",
    			descr: "Represents a single item in a list of message threads in the user interface. It contains header data of the message thread and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		MessageThread: {
    			title: {
    				title: "title",
    				descr: "Title for the message"
    			},
    			descr: "Messages thread groups togehter a set of related messages and/or attachments.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			messages: {
    				title: "messages",
    				descr: "List of messages and attachments in this thread"
    			},
    			otherParty: {
    				title: "other party",
    				descr: "Palkkaus.fi id (IBAN) of the other party.\nThe other party is the second entity(besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party is the second entity (besides the owner) in that is juridically in the message.\nThis may or may not be the originator the message thread.\nIt is typically the Partner or Palkkaus.fi, but may also be the Worker in the future."
    			},
    			otherPartyWorkflow: {
    				title: "other party workflow",
    				descr: "The workflow from the other party point-of-view."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "The owner of the thread: The entity that has the right to delete or move the thread.\nThis may or may not be the originator the message thread.\nThe owner is typically Company or Household, in some cases Worker person.\nIt should NOT be partner or person from a company."
    			},
    			ownerWorkflow: {
    				title: "owner workflow",
    				descr: "The workflow from the Owner point-of-view."
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		MessageThreadWorkflow: {
    			title: "message thread workflow",
    			descr: "Defines workflow actions for a message thread.",
    			lastMessageText: {
    				title: "last message text",
    				descr: "Text for the last message (written by this party). This is for preview purposes: May be shortened."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The salary date for the discussion.\nOwner's and other party's are typically the same, but they may differ:\ne.g. one may be the requested / estimated date and the other the first possible date if the requested is unrealistic."
    			},
    			unreadCount: {
    				title: "unread count",
    				descr: "Number of messages that this party has posted that have not been read by the other party."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "Time of the last message posted by this party."
    			}
    		},
    		MonthlyHolidayAccrual: {
    			title: "monthly holiday accrual",
    			descr: "Holiday Accrual for a calendar month in releation to a calculation.",
    			absencesDays: {
    				title: "absences days",
    				descr: "Number of absence days used in the calculation."
    			},
    			daysAccrued: {
    				title: "days accrued",
    				descr: "Holiday accrual for this month.\nNote that -1 is indication that accrual should not be updated for this particylar month.\nThere is no possiblity for negative accrual."
    			},
    			month: {
    				title: "month",
    				descr: "First day of month for the holiday month"
    			},
    			workDays: {
    				title: "work days",
    				descr: "Work days in determining the accrual for the month in 14 days rule."
    			},
    			workHours: {
    				title: "work hours",
    				descr: "Work hours in determining the accrual fot the month in 35 hour rule."
    			}
    		},
    		NonProfitOrgUsecase: {
    			title: "non profit org usecase",
    			descr: "Handles the Calculation of CalculationRowType.NonProfitOrg to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		OAuthMessage: {
    			title: "o auth message",
    			descr: "Based on standard https://tools.ietf.org/html/rfc6749 .",
    			access_token: {
    				title: "access_token",
    				descr: "The access token issued by the authorization server."
    			},
    			assertion: {
    				title: "assertion",
    				descr: "The value of the \"assertion\" parameter MUST contain a single JWT."
    			},
    			client_id: {
    				title: "client_id",
    				descr: "The client identifier issued to the client during the registration process."
    			},
    			client_secret: {
    				title: "client_secret",
    				descr: "The client secret. The client MAY omit the parameter if the client secret is an empty string."
    			},
    			code: {
    				title: "code",
    				descr: "The authorization code received from the authorization server."
    			},
    			connection: {
    				title: "connection",
    				descr: "Auth0 connection"
    			},
    			error: {
    				title: "error",
    				descr: "A single ASCII [USASCII] error code."
    			},
    			error_description: {
    				title: "error_description",
    				descr: "Human-readable ASCII [USASCII] text providing additional information, used to assist the client developer in understanding the error that occurred."
    			},
    			error_uri: {
    				title: "error_uri",
    				descr: "A URI identifying a human-readable web page with information about the error, used to provide the client developer with additional information about the error."
    			},
    			expires_in: {
    				title: "expires_in",
    				descr: "The lifetime in seconds of the access token.  For example, the value \"3600\" denotes that the access token will expire in one hour from the time the response was generated. If omitted, the authorization server SHOULD provide the expiration time via other means or document the default value."
    			},
    			grant_type: {
    				title: "grant_type",
    				descr: "Requested grant type."
    			},
    			id_token: {
    				title: "id_token",
    				descr: "The id token issued by the authorization server."
    			},
    			password: {
    				title: "password",
    				descr: "The resource owner password."
    			},
    			redirect_uri: {
    				title: "redirect_uri",
    				descr: "After completing its interaction with the resource owner, the authorization server directs the resource owner's user-agent back to the client."
    			},
    			refresh_token: {
    				title: "refresh_token",
    				descr: "The refresh token, which can be used to obtain new access tokens."
    			},
    			response_type: {
    				title: "response_type",
    				descr: "The value MUST be one of \"code\" for requesting an authorization code, \"token\" for  requesting an access token(implicit grant), or a registered extension value."
    			},
    			salaxy_language: {
    				title: "salaxy_language",
    				descr: "User interface language."
    			},
    			salaxy_onboarding_uri: {
    				title: "salaxy_onboarding_uri",
    				descr: "Salaxy specific onboarding uri."
    			},
    			salaxy_role: {
    				title: "salaxy_role",
    				descr: "Users role."
    			},
    			scope: {
    				title: "scope",
    				descr: "The value of the scope parameter is expressed as a list of space-delimited, case-sensitive strings.The strings are defined by the authorization server."
    			},
    			state: {
    				title: "state",
    				descr: "An opaque value used by the client to maintain state between the request and callback.The authorization server includes this value when redirecting the user-agent back to the client. The parameter SHOULD be used for preventing cross-site request forgery."
    			},
    			token_type: {
    				title: "token_type",
    				descr: "The type of the token issued."
    			},
    			username: {
    				title: "username",
    				descr: "The resource owner username."
    			}
    		},
    		ODataQueryOptions: {
    			title: "o data query options",
    			descr: "Defines the query options of OData",
    			$count: {
    				title: "$count",
    				descr: "Tells the server to include the total count of matching entities in the response. (Useful for server-side paging.)"
    			},
    			$expand: {
    				title: "$expand",
    				descr: "Expands related entities inline. At the moment, this typically not implemented."
    			},
    			$filter: {
    				title: "$filter",
    				descr: "Filters the results, based on a Boolean condition."
    			},
    			$orderby: {
    				title: "$orderby",
    				descr: "Sorts the results."
    			},
    			$search: {
    				title: "$search",
    				descr: "Performs free-text search using the in-built logic in the API."
    			},
    			$select: {
    				title: "$select",
    				descr: "Selects which properties to include in the response."
    			},
    			$skip: {
    				title: "$skip",
    				descr: "Skips the first n results."
    			},
    			$top: {
    				title: "$top",
    				descr: "Returns only the first n the results."
    			},
    			anon: {
    				title: "anon",
    				descr: "If true, perfoms the query also in anonymous mode\n(no token, token expired or invalid token).\nDefault is that the query is done only if user has a valid token."
    			}
    		},
    		ODataResult: {
    			title: "o data result",
    			descr: "Result from OData query",
    			count: {
    				title: "count",
    				descr: "Total count of the items (server-side/storage) if updated in the previous query.\nNote that this may be null if the count is not fetched."
    			},
    			items: {
    				title: "items",
    				descr: "Actual results array"
    			},
    			nextPageLink: {
    				title: "next page link",
    				descr: "Next page link"
    			}
    		},
    		Occupation: {
    			title: "occupation",
    			descr: "Occupation according to Statistics Finland.\nUsed for insurance pricing and reporting.",
    			code: {
    				title: "code",
    				descr: "Statistics Finland code. There may be duplicates with different id."
    			},
    			freeText: {
    				title: "free text",
    				descr: "Free text\nTODO: Do we need to expose this? Why?"
    			},
    			id: {
    				title: "id",
    				descr: "Salaxy identifier for the Occupation. Begins with code, but may have a suffix."
    			},
    			keyword: {
    				title: "keyword",
    				descr: "Keywords for searching etc."
    			},
    			label: {
    				title: "label",
    				descr: "Label"
    			},
    			lang: {
    				title: "lang",
    				descr: "Language for the entry"
    			},
    			search: {
    				title: "search",
    				descr: "Lower case text that contains the id, label and potentially other text that should match in searches."
    			}
    		},
    		Onboarding: {
    			title: "onboarding",
    			descr: "Provides the data for onboarding process for a company. Onboarding is the process where Salaxy gathers sufficient information to at least create an acccount, often even more so that the user can pay a salaary with the account.",
    			accountType: {
    				title: "account type",
    				descr: "Type of the account to be created."
    			},
    			appState: {
    				title: "app state",
    				descr: "Custom state object that is guaranteed to survive the round-trip in the onboarding process.\nCalling application to the onboarding may pass here any data that it needs after the onboarding process is ready and user has been authenticated."
    			},
    			company: {
    				title: "company",
    				descr: "Input that is needed for creating a company account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			credentials: {
    				title: "credentials"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			person: {
    				title: "person",
    				descr: "Input that is needed for creating a personal account.\nThis may be\n1) the principal account if the account is created for 1a) Worker or 1b) Household or\n2) creator owner if the account is created for a Company."
    			},
    			products: {
    				title: "products",
    				descr: "Possibility to define products.\nThese can only be defined to one account as part of the onboarding process:\nIf company account is created, the products are always for Company, not for the Person =&gt;\nPerson will have default products in that case and they need to be edited in a separate process once the accounts have been created."
    			},
    			signature: {
    				title: "signature",
    				descr: "Input and post data for digital signature."
    			},
    			status: {
    				title: "status",
    				descr: "The server-side status of the onboarding process.\nNOTE that there is a more relaxed user interface-level state in ui.state and ui.stateFullText\nthat can be used in onboarding user interfaces."
    			},
    			ui: {
    				title: "ui",
    				descr: "User interface related data: Mainly the state of the UI."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation status and errors."
    			}
    		},
    		OnboardingContactVerification: {
    			title: "onboarding contact verification",
    			descr: "Contact verification object for single message type.",
    			pin: {
    				title: "pin",
    				descr: "Given pin code."
    			}
    		},
    		OnboardingContactVerifications: {
    			title: "onboarding contact verifications",
    			descr: "Contact verification object for onboarding.",
    			email: {
    				title: "email",
    				descr: "Verification for email contact."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Verification for telephone contact."
    			}
    		},
    		OnboardingCredentials: {
    			title: "onboarding credentials",
    			descr: "Stores information about the credentials that were used in the Onboarding process.",
    			credentialId: {
    				title: "credential id",
    				descr: "Identifier of the credentials (UserAccount Guid in the database)."
    			},
    			partnerId: {
    				title: "partner id",
    				descr: "The partner that initiated the Onboarding process."
    			}
    		},
    		OnboardingUIState: {
    			title: "onboarding u i state",
    			descr: "Stores data for the onboarding user interface.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "URL where the user should be redirected back to if the process is canceled or there is an urecoverable error."
    			},
    			invitationKey: {
    				title: "invitation key",
    				descr: "Salaxy worker invitation key."
    			},
    			oAuthResponseType: {
    				title: "o auth response type",
    				descr: "Response type to OAuth -request"
    			},
    			state: {
    				title: "state",
    				descr: "State of the user interface. Typically an enumeration that stores where the UI was. E.g. Wizard step."
    			},
    			stateFullText: {
    				title: "state full text",
    				descr: "Full description of the user interface state for admin and analyses purposes."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "URL where the user should be redirected back to when the onboarding process is complete."
    			}
    		},
    		OtherCompensationUsecase: {
    			title: "other compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.OtherCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Ownership: {
    			title: "ownership",
    			isRequiredTracking: {
    				title: "is required tracking",
    				descr: "If true, authorities require tracking the shareholders.\nIn Finland, with current legistlation, this is if a shareholder has over 25% of the shares."
    			},
    			shareHolders: {
    				title: "share holders",
    				descr: "List of shareholders for the company"
    			}
    		},
    		PageResultOfApiListItem: {
    			title: "page result of api list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfArticle: {
    			title: "page result of article",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfCalculationListItem: {
    			title: "page result of calculation list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfEmploymentListItem: {
    			title: "page result of employment list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfFileListItem: {
    			title: "page result of file list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfInvoiceListItem: {
    			title: "page result of invoice list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfMessageListItem: {
    			title: "page result of message list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfPayrollListItem: {
    			title: "page result of payroll list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfSampleListItem: {
    			title: "page result of sample list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfTaxcardListItem: {
    			title: "page result of taxcard list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PageResultOfWorkerListItem: {
    			title: "page result of worker list item",
    			count: {
    				title: "count"
    			},
    			items: {
    				title: "items"
    			},
    			nextPageLink: {
    				title: "next page link"
    			}
    		},
    		PartnerAccounting: {
    			title: "partner accounting",
    			defaultTargetId: {
    				title: "default target id",
    				descr: "Default accounting target."
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, partner settings override company settings."
    			},
    			targetIds: {
    				title: "target ids",
    				descr: "Available accounting targets."
    			}
    		},
    		PartnerBranding: {
    			title: "partner branding",
    			descr: "Partner branding settings: logo, css etc.",
    			css: {
    				title: "css",
    				descr: "Css path for the primary partner."
    			},
    			logo: {
    				title: "logo",
    				descr: "Url to partner logo."
    			},
    			text: {
    				title: "text",
    				descr: "Long marketing text."
    			},
    			url: {
    				title: "url",
    				descr: "Marketing url."
    			}
    		},
    		PartnerCompanyAccountInfo: {
    			title: "partner company account info",
    			descr: "Information about a Customer account for a Partner",
    			accountId: {
    				title: "account id",
    				descr: "Account Id"
    			},
    			accountIdFormatted: {
    				title: "account id formatted",
    				descr: "Account Id fomratted for dispaly to end user"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar for display information"
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If true, the current user (the partner) is authorized to access the account"
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has a verified owner."
    			},
    			partnerAccountId: {
    				title: "partner account id",
    				descr: "The partner account ID to which the access is currently restricted."
    			}
    		},
    		PartnerMessaging: {
    			title: "partner messaging",
    			descr: "Messaging settings for the partner.",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the messaging is switched on for this partner."
    			}
    		},
    		PartnerService: {
    			title: "partner service",
    			enabled: {
    				title: "enabled",
    				descr: "If true, the calculation data delivery by forms is switched on for this partner."
    			}
    		},
    		PartnerWebSite: {
    			title: "partner web site",
    			descr: "Stores information about the web site that is used for customer (employer) to access the customer web site.",
    			url: {
    				title: "url",
    				descr: "The main URL that is displayed.\nDefault is https://www.palkkaus.fi/company"
    			}
    		},
    		Payer: {
    			title: "payer",
    			descr: "The paying party",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		Payment: {
    			title: "payment",
    			descr: "Payment object requested to be paid for completing the underlying business objects.",
    			amount: {
    				title: "amount",
    				descr: "Amount requested to be paid."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "List of business objects (calculations etc.) on which the payment is based on."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payment."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			message: {
    				title: "message",
    				descr: "Message for the payment."
    			},
    			method: {
    				title: "method",
    				descr: "Payment method."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer as it would be on the invoice"
    			},
    			recipient: {
    				title: "recipient",
    				descr: "Recipient as it would be on the invoice"
    			},
    			reference: {
    				title: "reference",
    				descr: "Bank reference number/string. Will be generated by the system automatically."
    			},
    			request: {
    				title: "request",
    				descr: "Request specific payment parameters."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the payment."
    			},
    			statusText: {
    				title: "status text",
    				descr: "Status text giving more information about the status."
    			},
    			type: {
    				title: "type",
    				descr: "Payment type."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PaymentRequestParameters: {
    			title: "payment request parameters",
    			descr: "Contains request specific parameters for payment.",
    			cancelUrl: {
    				title: "cancel url",
    				descr: "Return url for failed / cancelled payment."
    			},
    			paymentStartedUrl: {
    				title: "payment started url",
    				descr: "If this URL is set, the payment allows aynchronous payment methods\nsuch as payment with invoice (viitelasku).\nThis means that the payment UI returns in status InProgress and\npayment is finalized later (may take hours, days or even be canceled)."
    			},
    			sendEmails: {
    				title: "send emails",
    				descr: "Indicates whether to send emails according to workflow after payment.\nTODO: This should be retrieved from the partner profile etc."
    			},
    			successUrl: {
    				title: "success url",
    				descr: "Return url for successful payment."
    			}
    		},
    		PaymentSettings: {
    			title: "payment settings",
    			descr: "Payment related settings",
    			customerFunds: {
    				title: "customer funds",
    				descr: "Settings related to payment methods that use the customer funds account (gross payment)."
    			},
    			invoice: {
    				title: "invoice",
    				descr: "Settings related to invoices: Either salary payment or other invoicing."
    			},
    			pricing: {
    				title: "pricing",
    				descr: "The pricing partner (accounant or software) that defines the pricing of the service.\nThis data is typically read-only in normal settings view."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		Payroll03Info: {
    			title: "payroll03 info",
    			descr: "Provides Read-only information about the Payroll In the case of non-paid Payrolls, this is the latest save/validation. In the case of paid Payrolls, this is the status of the Payroll when it was paid.",
    			calcCount: {
    				title: "calc count",
    				descr: "Count of calculations in this Payroll"
    			},
    			date: {
    				title: "date",
    				descr: "Date that the Info was last updated."
    			},
    			dueDate: {
    				title: "due date",
    				descr: "Due date for the payroll. Date when the payment should be made to achieve SalaryDate."
    			},
    			fees: {
    				title: "fees",
    				descr: "Total Palkkaus.fi fees for the Payroll"
    			},
    			isReadyForPayment: {
    				title: "is ready for payment",
    				descr: "If true, all calculations have been validated and they are valid for payment.\nIf false, validation has not been done or some calculations are not valid for payment."
    			},
    			paymentDate: {
    				title: "payment date",
    				descr: "Paid at date for the payroll. (Date of the employer's payment)."
    			},
    			paymentId: {
    				title: "payment id",
    				descr: "The reference number of the payment after payment is made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The estimated date when the salary is withdrawable by the worker."
    			},
    			status: {
    				title: "status",
    				descr: "Status for the payroll and its calculations in the payment Workflow."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Total gross salary"
    			},
    			totalPayment: {
    				title: "total payment",
    				descr: "Total payment for the Payroll."
    			}
    		},
    		PayrollDetails: {
    			title: "payroll details",
    			descr: "New version 03 of Payroll. Payroll (Palkkalista) is a list of employees who receive salary or wages from a particular organization. Typical usecase is that a a company has e.g. a monthly salary list that is paid at the end of month. For next month, a copy is then made from the latest list and the copy is potentially modified with the changes of that particular month.",
    			calcs: {
    				title: "calcs",
    				descr: "Snapshots of the calculations based on the ID's specified in Info"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			usecase: {
    				title: "usecase",
    				descr: "A usecase for creating and editing Payroll lists.\nIn practice, a usecase is either a user interface or a microservice that provides a Payroll processing\nand it may be implemented outside Salaxy.\nProvides the possibility to store useacase specific data as part of the Payroll."
    			},
    			validations: {
    				title: "validations",
    				descr: "When saved or validated, provides a validation object for each calculation:\nWhether the calculations are ready for payment."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		PayrollInput: {
    			title: {
    				title: "title",
    				descr: "Title that can be specified by the end user."
    			},
    			descr: "Input that user defines about the Payroll",
    			calculations: {
    				title: "calculations",
    				descr: "List of calculations that are attached to this calculation."
    			},
    			noUpdateFromEmployment: {
    				title: "no update from employment",
    				descr: "If true, the calculation rows are not updated from employment when the Payroll is copied.\nDefault is that the rows and other data is updated in copy / create new etc."
    			},
    			period: {
    				title: "period",
    				descr: "The Default Salary period for the Payroll.\nThis may be overriden for an individual calculation."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "The requested date for the SalaryDate from the employer."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "What types of wages are paid with this payroll.\nMay affect the rendering of the user interface (e.g. for     holidays, hours and absences).\nAlso affects which Workers are show in the add / link dialog by default."
    			}
    		},
    		PayrollListItem: {
    			title: "payroll list item",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			info: {
    				title: "info",
    				descr: "General summary information about the Payroll and when it was las processed."
    			},
    			input: {
    				title: "input",
    				descr: "Input that defines the Payroll."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		PensionEntrepreneurProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by TempContractMarker."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status"
    			},
    			tempContractMarker: {
    				title: "temp contract marker",
    				descr: "If true, user wants to order a YEL-contract."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Pension for Workers: The payments and monthly reporting.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically by IsPensionSelfHandling."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isPendingContract: {
    				title: "is pending contract",
    				descr: "If true, there is a new pending contract."
    			},
    			isPensionContractDone: {
    				title: "is pension contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			isPensionSelfHandling: {
    				title: "is pension self handling",
    				descr: "If true, the user handles the reporting and payments himself/herself."
    			},
    			pensionCompany: {
    				title: "pension company",
    				descr: "The pension company where the user has a contract."
    			},
    			pensionContractNumber: {
    				title: "pension contract number",
    				descr: "Number for the contract."
    			},
    			status: {
    				title: "status"
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PensionSettings: {
    			title: "pension settings",
    			company: {
    				title: "company",
    				descr: "The pension company where the user has a contract."
    			},
    			contractNumber: {
    				title: "contract number",
    				descr: "Number for the contract."
    			},
    			isContractDone: {
    				title: "is contract done",
    				descr: "The use has a pension contract done (as opposed to acting as a temporary employer that pays the pension without a acontract)."
    			},
    			status: {
    				title: "status",
    				descr: "Status of the pension insurance."
    			}
    		},
    		PeriodRecalculationData: {
    			title: "period recalculation data",
    			descr: "Data that is needed for determening PeriodRecalculationParams and thus at the end recalculating holidays and absences to a calculation.",
    			absences: {
    				title: "absences",
    				descr: "Absences that should be deducted from monthly salary.\nIf not provided, absences are not calculated."
    			},
    			calc: {
    				title: "calc",
    				descr: "The salary calculation that should be recalculated."
    			},
    			holidayYear: {
    				title: "holiday year",
    				descr: "Holiday year with holidays that should be added as salary rows.\nIf not provided, holidays are not calculated."
    			}
    		},
    		PersonAccount: {
    			title: "person account",
    			descr: "An account for a Person - can be both Employer and Worker",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			contactVerificationPin: {
    				title: "contact verification pin",
    				descr: "Contact verification pin for securing contact address changes."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerInfo: {
    				title: "employer info",
    				descr: "The employer information in case the person is an employer"
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			identity: {
    				title: "identity",
    				descr: "Information about the verification of the account.\nNOTE that this object may not be present in many method calls - this is because it contains confidential information."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			lastLoginAs: {
    				title: "last login as",
    				descr: "Type of the last login - the user role"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "The worker information in case the person is a Worker"
    			}
    		},
    		PersonInput: {
    			title: "person input",
    			bankAccountIban: {
    				title: "bank account iban",
    				descr: "Bank account number for salary payment"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the user / personal account"
    			},
    			contactVerification: {
    				title: "contact verification",
    				descr: "Verification objects for contact verification."
    			},
    			doCreate: {
    				title: "do create",
    				descr: "If true, creates a personal account based on this data.\nOtherwise this is just user data for Company."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name or company name"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name, for companies, this should be null"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Finnish Personal ID (Henkilötunnus /HETU)"
    			},
    			picture: {
    				title: "picture",
    				descr: "URL of the picture if specified as picture (null in a case of type Icon)"
    			},
    			resolvedId: {
    				title: "resolved id",
    				descr: "If an object with the given PersonalId exists, it is resolved here: The unique ID is set here by the onboarding service. This field cannot be set."
    			}
    		},
    		PricingPartner: {
    			title: "pricing partner",
    			descr: "Defines the partner, service and pricing model for the customer.",
    			accountId: {
    				title: "account id",
    				descr: "Partner account id."
    			},
    			fixedFee: {
    				title: "fixed fee",
    				descr: "Fee for one calculation, which is visible and paid by customer.\nThis is not relevant if the PricingModel is PalkkausFee"
    			},
    			pricingModel: {
    				title: "pricing model",
    				descr: "Pricing model by the partner."
    			},
    			serviceModel: {
    				title: "service model",
    				descr: "Service model by the partner."
    			}
    		},
    		PricingProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Defines the pricing partner, service model and pricing model.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "Enabled property set automatically if partner set and partner specific pricing model."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			pricingPartner: {
    				title: "pricing partner",
    				descr: "Defines the pricing partner, service model and pricing model."
    			},
    			status: {
    				title: "status",
    				descr: "Current status of pricing."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		PrimaryPartner: {
    			title: "primary partner",
    			descr: "Primary partner settings.",
    			accounting: {
    				title: "accounting"
    			},
    			branding: {
    				title: "branding"
    			},
    			description: {
    				title: "description",
    				descr: "Description of the primary account."
    			},
    			id: {
    				title: "id",
    				descr: "Primary account id."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "Boolean indicating if this partner has been authorized to act on behalf the current account."
    			},
    			isPrimary: {
    				title: "is primary",
    				descr: "Boolean indicating if this partner is the primary partner of the current account."
    			},
    			key: {
    				title: "key",
    				descr: "Human-readable, human decided, recognizable short unique key for the partner:\nAll lower-case, a-z, 0-9, \"-\" and \"_\". Used in URL's etc."
    			},
    			messaging: {
    				title: "messaging"
    			},
    			name: {
    				title: "name",
    				descr: "Display name for the account."
    			},
    			service: {
    				title: "service"
    			},
    			site: {
    				title: "site"
    			}
    		},
    		Product: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Base inteface for Products.\nThe common properties for all Products: BaseSalaryProduct, TaxProduct, UnemploymentProduct etc.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user. Restrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product.\nE.g. Raksaloki is available for households only and this is a Company account =&gt; Should not show the product."
    			}
    		},
    		Profile: {
    			title: "profile",
    			descr: "Profile object for either Worker or Employer",
    			accountId: {
    				title: "account id",
    				descr: "Account if known"
    			},
    			avatar: {
    				title: "avatar",
    				descr: "The avatar object: Picture name and other such basic info."
    			},
    			contact: {
    				title: "contact",
    				descr: "Other contact information for this profile."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "The legal entity type of the profile: Person or Company"
    			},
    			expiresAt: {
    				title: "expires at",
    				descr: "Expiration date for the profile will be unpublished."
    			},
    			externalId: {
    				title: "external id",
    				descr: "External id (e.g. member number)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isPublic: {
    				title: "is public",
    				descr: "If true, the profile is set to public."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			job: {
    				title: "job"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			sortableName: {
    				title: "sortable name"
    			},
    			source: {
    				title: "source",
    				descr: "The source of this profile: Palkkaus section or partner site.\nThis is a strongly typed object that may be undefined - see SourceId in that case."
    			},
    			sourceId: {
    				title: "source id",
    				descr: "The identifier string of the source. This will always return a distinct value."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			userUpdatedAt: {
    				title: "user updated at",
    				descr: "Last time user updated the profile"
    			},
    			worker: {
    				title: "worker"
    			}
    		},
    		ProfileJob: {
    			title: "profile job",
    			descr: "A job for a Public Profile (Osaajaprofiili) or later also for Job ads.",
    			category: {
    				title: "category"
    			},
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Job"
    			},
    			id: {
    				title: "id"
    			},
    			name: {
    				title: "name"
    			},
    			picture: {
    				title: "picture"
    			},
    			synonyms: {
    				title: "synonyms"
    			}
    		},
    		ProfileJobCategory: {
    			title: "profile job category",
    			descr: "Category of Profile Jobs",
    			count: {
    				title: "count",
    				descr: "Context-specific count that can show e.g. Profiles found for this Category"
    			},
    			id: {
    				title: "id",
    				descr: "Unique ID of the Category"
    			},
    			name: {
    				title: "name",
    				descr: "Name / title of the Category"
    			},
    			picture: {
    				title: "picture",
    				descr: "Backround picture for the Category"
    			}
    		},
    		Recipient: {
    			title: "recipient",
    			accountIBAN: {
    				title: "account i b a n"
    			},
    			avatar: {
    				title: "avatar"
    			},
    			bankPartyIdentification: {
    				title: "bank party identification",
    				descr: "This field is mandatory for SEPA payment scenario."
    			},
    			bic: {
    				title: "bic"
    			},
    			contactInfo: {
    				title: "contact info"
    			},
    			fullName: {
    				title: "full name"
    			}
    		},
    		RemunerationUsecase: {
    			title: "remuneration usecase",
    			descr: "Handles the Calculation of CalculationRowType.Remuneration to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Report: {
    			title: {
    				title: "title",
    				descr: "Short title for the report that can be shown to the end user."
    			},
    			descr: "Describes a logical generated report. This is a report instance e.g. a monthly report for January 2017. It may contain several files of different types: PDF, preview PNG, HTML etc.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			htmlUrl: {
    				title: "html url",
    				descr: "HTML for the report. Use salaxy-rpt.css for formatting.\nFor preview and demo, this is the format to show to the user - PDF is not generated."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isDemoData: {
    				title: "is demo data",
    				descr: "If true, the report is created using demo data - should be used only in the beginning when the user has not paid any salaries.\nThis may be e.g. calculations in draft stage or completely invented demo data such as example workers.\nDemo data should not be confused to test database (test-api.palkkaus.fi / test-api.salaxy.com):\nPaid calculations in test database are production data."
    			},
    			isPreview: {
    				title: "is preview",
    				descr: "If true, the report is in preview state, typically in the middle of the month.\nThis means that the data itself is production, its just not final."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			jsonUrl: {
    				title: "json url",
    				descr: "Some reports are also available in JSON format, from which you can create lists, CSV, Excel files etc."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			pdfUrl: {
    				title: "pdf url",
    				descr: "PDF report for viewing and printing. This is typically the main report.\nFor preview and demo, there is typically only HTML version - no PDF."
    			},
    			reportDate: {
    				title: "report date",
    				descr: "Report date: For monthly and yearly reports this is the first day of the year/month.\nFor ad-hoc reports, this is the start date of the period."
    			},
    			thumbnail: {
    				title: "thumbnail",
    				descr: "Thumbnail image for the report.\nFor preview and demo data this is typically a fixed image - the thumbnail is only generated when PDF is generated."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the report"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			urlsExpireAt: {
    				title: "urls expire at",
    				descr: "Time when the PDF, HTML or JSON URL's expire IF they are generated in a way that\nthey are callable without authentication for a specified time period.\nThis property is null if the URL requires authentication."
    			}
    		},
    		ReportingArea: {
    			title: "reporting area",
    			bottom: {
    				title: "bottom",
    				descr: "Bottom margin."
    			},
    			left: {
    				title: "left",
    				descr: "Left margin."
    			},
    			right: {
    				title: "right",
    				descr: "Right margin."
    			},
    			top: {
    				title: "top",
    				descr: "Top margin."
    			}
    		},
    		ReportingSettings: {
    			title: "reporting settings",
    			descr: "Settings related to reporting",
    			css: {
    				title: "css",
    				descr: "Additional CSS that is used to customize the layout of the report."
    			},
    			footerTemplate: {
    				title: "footer template",
    				descr: "HTML template for the print footer.\nUse the following handlebars tags to inject values into the footer:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			headerTemplate: {
    				title: "header template",
    				descr: "HTML template for the print header.\nUse the following handlebars tags to inject values into the header:\n\n- `{{ date }}` Logical report date, e.g.Salary date for salary slip.\n- `{ { today } }` Printing date of the report.\n- `{{ title }}` Report specific title, e.g. \"Palkkalaskelma\".\n- `{{ subTitle}}` Report specific sub title, e.g. \"Palkkapäivä 1.6.2020\".\n- `{{&gt; pageNumber }}` Current page number.Rendered as SPAN element.\n- `{ {&gt; totalPages } }` total pages in the document.Rendered as SPAN element.\n- `{ {&gt; logo } }` The main logo for the page as uploaded to settings.Rendered as SVG or IMG element.\n- `{ {&gt; img2 } }` Secondary image as uploaded to settings.Rendered as SVG or IMG element."
    			},
    			img2: {
    				title: "img2",
    				descr: "Secondary image that can be used in header / footer as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			logo: {
    				title: "logo",
    				descr: "Main logo image as embedded SVG or image (HTML).\nThis should preferably be an SVG or alternatively an image tag with data uri."
    			},
    			margins: {
    				title: "margins",
    				descr: "Defines the margins of the content area"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The date when the settings section was created or last modified."
    			}
    		},
    		ResultRow: {
    			title: "result row",
    			descr: "Defines a Calculation row in the calculation results",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowSource: {
    				title: "row source",
    				descr: "Source of the calculation row - affects how the row is handled in editing / recalculation scenarios."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is kept as-is in the Salaxy system."
    			},
    			total: {
    				title: "total",
    				descr: "Total for the row - always Price * Count"
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "Base salary (Peruspalkka) is the monthly salary or total salary without addiotions (overtime, holidays etc.)"
    			},
    			totalDeduction: {
    				title: "total deduction",
    				descr: "Deductions from the salary: Union payment, Advance, Foreclosure"
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Taxfree expenses compensation"
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary (Bruttopalkka) is \"real\" salary including the additions (overtime, holidays etc.) but not benefits."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "Social Security Base (Sosiaaliturvan alainen ansio) is typically the same as TotalTaxable,\nbut e.g. child care subsidy (yksityisen hoidon tuki) is handled differently."
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable income (Ennakonpidätyksen alainen ansio) is the salary from taxation point-of-view. It is basically the Gross Salary plus benefits."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row"
    			},
    			userRowIndex: {
    				title: "user row index",
    				descr: "Row index that matches this row to a user defined row (which is zero-based).\nIf the row is generated by calculation business logic (e.g. based on framework agreement), the index is -1."
    			}
    		},
    		RootDeclarationReflection: {
    			title: "root declaration reflection",
    			descr: {
    				title: "descr",
    				descr: "Full description: shortText + longer text field."
    			},
    			example: {
    				title: "example",
    				descr: "Short example(s) (inline in documentation)"
    			},
    			exampleUrls: {
    				title: "example urls",
    				descr: "AngularJS views (templates) that contain more comprehensive example pages."
    			},
    			group: {
    				title: "group",
    				descr: "Primary grouping depending on the library"
    			},
    			id: {
    				title: "id",
    				descr: "Reflection ID for the library. Unique within one library."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of item: \"class\", \"interface\" and \"enum\" are root level types,\n\"controller\", \"controllerBase\" and \"component\" are specialized types classes."
    			},
    			lib: {
    				title: "lib",
    				descr: "The library that defines the item"
    			},
    			name: {
    				title: "name",
    				descr: "Class name: Name of the component, controller etc."
    			},
    			rawKind: {
    				title: "raw kind",
    				descr: "Raw typedoc level type of reflection."
    			},
    			screenShots: {
    				title: "screen shots",
    				descr: "Paths to screen shot pictures."
    			},
    			shortText: {
    				title: "short text",
    				descr: "The short text: First paragraph of the documentation or if not available (e.g. parameters), then the text."
    			},
    			source: {
    				title: "source",
    				descr: "The source file that defines the component"
    			}
    		},
    		RowAccountingData: {
    			title: "row accounting data",
    			descr: "Provides row specific data for accounting.",
    			accountNumberFlags: {
    				title: "account number flags",
    				descr: "Special flags for account number mapping."
    			},
    			vatPercent: {
    				title: "vat percent",
    				descr: "Percent for VAT calculation expressed as decimal: 24% is 0.24.\nThe price in the root level is assumed being with tax."
    			}
    		},
    		Salary: {
    			title: "salary",
    			descr: "Defines the main salary that is the bases for the calculation",
    			amount: {
    				title: "amount",
    				descr: "Amount of e.g. hours, months for the salary. Default is 1."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If set to true, will calculate the household deduction. Applies only to natural persons - as oppsed to companies."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the salary being calculated"
    			},
    			message: {
    				title: "message",
    				descr: "Optional message for the salary row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price of one unit - e.g. hour, month."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			unit: {
    				title: "unit",
    				descr: "E.g. hour, month. Typically this parameter can be left empty/null and the unit is set according to type."
    			}
    		},
    		SampleApiObject: {
    			title: "sample api object",
    			descr: "Sample object for creating v03 controller",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SampleData: {
    			title: "sample data",
    			descr: "Sample custom data in search index",
    			test: {
    				title: "test",
    				descr: "A test string"
    			}
    		},
    		SampleListItem: {
    			title: "sample list item",
    			descr: "Available Search index data of the Sample object",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		SessionUserCredential: {
    			title: "session user credential",
    			descr: "Represents a user credential in session: This credential is typically based on Claims and is availble without going to database.",
    			authenticationMethod: {
    				title: "authentication method",
    				descr: "The ultimate authentication provider that authenticated the user, e.g. Facebook, Google."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar object that contains the visual reresentation of the User account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			email: {
    				title: "email",
    				descr: "E-mail address if provided by the authentication provider."
    			},
    			id: {
    				title: "id",
    				descr: "Unique identitier for the user ID coming from the Auth0 database:\n'[MethodText]|[ProviderId]', e.g. 'facebook|12345678901234567'\n(Number may contain more characters - not sure for the exact cahracters count)"
    			}
    		},
    		SharedTaxcardExistsResult: {
    			title: "shared taxcard exists result",
    			descr: "Provides a a result to query of whether a taxcard exists for a personal ID (no personal information).",
    			isError: {
    				title: "is error",
    				descr: "If true there was an error fetching the information.\nTypically, this would be that there were too many attempts to fetch Personal ID's and request was classified as phishing.\nIn this case, you should currently create the employment relation and fetch the taxcard infor after that.\nWe may later introduce a \"Are you robot\" type of interface to go around this (if necessary)."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard. Undefined if none is found."
    			},
    			ownedExists: {
    				title: "owned exists",
    				descr: "Returns true if there is already an owned taxcard for the employer."
    			},
    			ownedTaxcard: {
    				title: "owned taxcard",
    				descr: "If Owned taxcard exists, it is returned here."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID that was used in fetching.\nIt may be validated and formatted, or just the original string if there were issues."
    			},
    			sharedExists: {
    				title: "shared exists",
    				descr: "Returns true if a shared taxcard exists."
    			},
    			uri: {
    				title: "uri",
    				descr: "Uri for the Shared taxcard"
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity of the taxcard. Note that this property is null when Exists is false."
    			}
    		},
    		Shareholder: {
    			title: "shareholder",
    			descr: "Defines a shareholder. At the moment, this is only a name and it is used only if the person has more than 25% of the shares. Expect this object to be developed with, Ownership percent or number of shares, Id or other such link to the actual person etc.",
    			name: {
    				title: "name",
    				descr: "Name of the shareholder."
    			}
    		},
    		SignatureData: {
    			title: "signature data",
    			digitalSignature: {
    				title: "digital signature"
    			},
    			email: {
    				title: "email"
    			},
    			isPep: {
    				title: "is pep"
    			},
    			isProcura: {
    				title: "is procura",
    				descr: "The account is authorized to sign on behalf of the actual account."
    			},
    			personName: {
    				title: "person name",
    				descr: "Name of the signing person.\nBy default, this is Onboarding Person, but it may be a completely different person."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID of the person, who has the right to sign."
    			},
    			telephone: {
    				title: "telephone"
    			}
    		},
    		SmsMessage: {
    			title: "sms message",
    			descr: "API SmsMessage",
    			body: {
    				title: "body",
    				descr: "Message body"
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			from: {
    				title: "from",
    				descr: "Message From (characters or numbers)"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isReceivedMessage: {
    				title: "is received message",
    				descr: "Receiver's message"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			receivedByUser: {
    				title: "received by user",
    				descr: "Message To (ID)"
    			},
    			receiverName: {
    				title: "receiver name",
    				descr: "Receiver name"
    			},
    			senderName: {
    				title: "sender name",
    				descr: "Sender name"
    			},
    			sentAt: {
    				title: "sent at",
    				descr: "Message sent"
    			},
    			sentByUser: {
    				title: "sent by user",
    				descr: "Message Sender (ID)"
    			},
    			to: {
    				title: "to",
    				descr: "Message To (phone)"
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			}
    		},
    		SubsidisedCommuteUsecase: {
    			title: "subsidised commute usecase",
    			descr: "Handles the Calculation of CalculationRowType.SubsidisedCommute to IrRows",
    			deduction: {
    				title: "deduction",
    				descr: "Deduction from salary"
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			},
    			periodDivider: {
    				title: "period divider",
    				descr: "Number of periods if Kind is PeriodicalDeduction.\nOtherwise this value is null."
    			}
    		},
    		TaxCardIncome: {
    			title: "tax card income",
    			descr: "Defines income that is recorded in a Taxcard: A paid salary calulation, income from previous employers or other salaries that affect the income calculation.",
    			description: {
    				title: "description",
    				descr: "A short description of the paid calculation.\nThis is a mandatory field if the Id is null."
    			},
    			diff: {
    				title: "diff",
    				descr: "When loading calculations from the database (in taxcard Save and FindBy with diff),\nthis property identifies the differences in the entry."
    			},
    			endDate: {
    				title: "end date",
    				descr: "End of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkEndDate, but may be different in edge cases."
    			},
    			id: {
    				title: "id",
    				descr: "Calculation id in the Salaxy database.\nIf the calculation is not in the Salaxy Database, leave this null."
    			},
    			income: {
    				title: "income",
    				descr: "Taxable income from this calculation"
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner of the row, typically the employer that paid the salary.\nNull meaning that it is the owner of the original taxcard."
    			},
    			paidAt: {
    				title: "paid at",
    				descr: "The paidAt date which tells mainly to which year the payment should be attributed.\nThis is reliable only if the salary is paid by Salaxy (TaxCardIncomeType.SalaxyCalculation).\nIn other cases it is just the reporting date."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Beginning of the salary payment period from taxation point-of-view.\nThis is typically the same as CalcInfo.WorkStartDate, but may be different in edge cases."
    			},
    			tax: {
    				title: "tax",
    				descr: "Widthholding tax in euro"
    			},
    			type: {
    				title: "type",
    				descr: "Type of income"
    			}
    		},
    		TaxProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Service related to integration, reporting and payments to tax authorities.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			isTaxAndSocialSecuritySelfHandling: {
    				title: "is tax and social security self handling",
    				descr: "If true, tax and social security payments are paid directly by the employer."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		TaxSettings: {
    			title: "tax settings",
    			irReporting: {
    				title: "ir reporting",
    				descr: "Defines how the reporting is done for Incomes registry (Tulorekisteri)."
    			},
    			paymentReferenceNumber: {
    				title: "payment reference number",
    				descr: "The reference number for the tax authorities"
    			}
    		},
    		Taxcard: {
    			title: "taxcard",
    			descr: "Full tax card that is stored in database and that is used in tax card percentage calculations. Also contains the salary history in the income log.",
    			card: {
    				title: "card",
    				descr: "The input that is coming from the Worker / Tax authorities"
    			},
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"[Server]/[PayloadType]/[Owner]/[Id]\", e.g. \"https://test-secure.salaxy.com/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment for which the tax card is created."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			incomeLog: {
    				title: "income log",
    				descr: "Calculations that have been paid to this tax card."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			result: {
    				title: "result",
    				descr: "Read-only calculation results and other logic values on the Taxcard."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker, whose salaries are paid based on this tax card."
    			}
    		},
    		TaxcardData: {
    			title: "taxcard data",
    			descr: "Tax card specific business data for index.",
    			copyOfUri: {
    				title: "copy of uri",
    				descr: "If the item is an employer instance of a shared taxcard, contains the original identifier\nas URI (\"/[PayloadType]/[Owner]/[Id]\", e.g. \"/CalculationDTO/FI86POYL0009461858/16ea052e-f23f-4b72-83a9-b8f6590f0dc9\");\nThis item cannot be set in Save - only in Approve method."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "Employment id."
    			},
    			end: {
    				title: "end",
    				descr: "End date of the taxcard validity. This may be null on NoTaxcard types."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "The type of the tax card."
    			},
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the taxcard validity"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income paid to the taxcard during the year (forYear)."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity resolved as enumeration in relation to payment date given in the query.\nThis is not available in direct database OData queries as it is relevant to current / given date."
    			}
    		},
    		TaxcardInput: {
    			title: "taxcard input",
    			descr: "Input from user for a tax card: What is written in the card plus the state infomration that is directly editable by the user.",
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			state: {
    				title: "state",
    				descr: "Tax card approval state"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardListItem: {
    			title: "taxcard list item",
    			descr: "List item of taxcard searches.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		TaxcardResult: {
    			title: "taxcard result",
    			descr: "Provides the results of the tax card calculation and business logic.",
    			tax: {
    				title: "tax",
    				descr: "Provides a decimal number for a recommended widthholding tax IF it is possible to calculate that based on given information.\nNull, if the information is not sufficient to give a recommendation."
    			},
    			yearlyPreviousIncome: {
    				title: "yearly previous income",
    				descr: "Sum of income for this year based on the items marked on the tax card."
    			}
    		},
    		TaxcardResults: {
    			title: "taxcard results",
    			descr: "Read-only calculations and other logic methods on a Taxcard.",
    			payment: {
    				title: "payment",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for the actual year, which is used for income limit.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			paymentJanuary: {
    				title: "payment january",
    				descr: "Payments of the salary calculations that were made using Palkkaus.fi system (as opposed to external salaries etc)\nand that were marked to this card. These are for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January.\nFor Shared card, all employers are considered, for Owner copies of shared card, only salaries paid by owner is considered."
    			},
    			totalIncome: {
    				title: "total income",
    				descr: "Total income for the actual year, which is used for income limit.\nNote that this does not include January income that may be paid for previous year taxcard."
    			},
    			totalIncomeJanuary: {
    				title: "total income january",
    				descr: "Total income for January of the following year that is paid for previous year taxcard (this taxcard) if there is no new taxcard for January."
    			},
    			validityToday: {
    				title: "validity today",
    				descr: "Gets the validity of the taxcard today. This value reflects purely the date value - it does not take into account approval or replacement."
    			}
    		},
    		TaxcardSnapshot: {
    			title: "taxcard snapshot",
    			descr: "Basic information about the taxcard: This object has the infromation printed in SalarySlip etc. but not the full salaries paid to the taxcard etc.",
    			cumulativeIncome: {
    				title: "cumulative income",
    				descr: "Cumulative income on the taxcard - including this calculation.\nNote that with shared taxcards, this may include income from other employers."
    			},
    			fileUri: {
    				title: "file uri",
    				descr: "Uri to the file copy of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used.\nAny income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tazx authorities."
    			},
    			previewUri: {
    				title: "preview uri",
    				descr: "Uri to the preview image of the tax card.\nSet by the system as part of the upload process, do not change in the API though it may be technically possible. This may have unintended results."
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit.\nThis is percent. I.e. for 50% set 50, not 0.5."
    			},
    			validity: {
    				title: "validity",
    				descr: "Validity for the taxcard as expressed in the card.\nNote that the end date may not be reliable if new taxcard has replaced this one."
    			}
    		},
    		TaxcardUiInfo: {
    			title: "taxcard ui info",
    			descr: "Display information and language versioned texts that make sense to\nend-users about the status of taxcard.",
    			kind: {
    				title: "kind",
    				descr: "Kind that is quaranteed to be non-null (undefined). This is about the existing taxcard,\nthere may still be a new card \"waiting\" and that may have another kind specified."
    			},
    			percent: {
    				title: "percent",
    				descr: "The tax percent if it is relevant for the taxcard.\nNumber is 0-60, not decimal e.g. 0.60. Null if not relevant."
    			},
    			shortText: {
    				title: "short text",
    				descr: "Shorter version of the descriptive text to the end user."
    			},
    			status: {
    				title: "status",
    				descr: "Status as in\n\"ok\": Valid taxcard,\n\"missing\": No taxcard, employer should add a new one.,\n\"waiting\": A shared taxcard is waiting for approval. Existing may be valid or not.\nor null: No calculation or employment relation specified."
    			},
    			text: {
    				title: "text",
    				descr: "Descriptive text to the end user containing the percent if relevant."
    			}
    		},
    		TaxcardWaitingApproval: {
    			title: "taxcard waiting approval",
    			descr: "Informs the employer that there is a Worker taxcard waiting for approval potentially without revealing personal information.",
    			createdAt: {
    				title: "created at",
    				descr: "Time when the taxcard was added."
    			},
    			fullCard: {
    				title: "full card",
    				descr: "If there is an owner instance of taxcard for the employer, the copy is added here.\nInstance exists only if there is a confirmed employment relation or if employer explicitly requests the taxcard."
    			},
    			isRelationUnconfirmed: {
    				title: "is relation unconfirmed",
    				descr: "If true, the relation has not yet been confirmed:\nI.e. the employer has just input the Finnish Personal Id (HETU) of the Worker.\nThe employment relation is confirmed when salaries are paid or salary request is received from Worker."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner account that added the taxcard."
    			},
    			state: {
    				title: "state",
    				descr: "Status of the taxcard"
    			},
    			uri: {
    				title: "uri",
    				descr: "URI for the shared card: This is the taxcard owned by the Worker and to view it\nyou need to request your own copy (would be in FullCard)."
    			}
    		},
    		TaxcardsDataRow: {
    			title: "taxcards data row",
    			descr: "Data row for taxcards data. This is typically used for imports and exports.",
    			end: {
    				title: "end",
    				descr: "End date of the period."
    			},
    			forYear: {
    				title: "for year",
    				descr: "Year that the tax card is valid for. Typically this means from February of this year to January of the following."
    			},
    			id: {
    				title: "id",
    				descr: "Id for the taxcard"
    			},
    			incomeLimit: {
    				title: "income limit",
    				descr: "Income up to which the TaxPercent can be used. Any income above the limit is taxed with ExcessTaxPercent."
    			},
    			kind: {
    				title: "kind",
    				descr: "Type of the taxcard as of 2019, when the taxcards logic was simplified by the Finnish tax authorities."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "The personal ID as written in the tax card."
    			},
    			start: {
    				title: "start",
    				descr: "Start date of the period"
    			},
    			taxPercent: {
    				title: "tax percent",
    				descr: "Tax percent as percent. I.e. for 50% set 50, not 0.5."
    			},
    			taxPercent2: {
    				title: "tax percent2",
    				descr: "Tax percentage that is used in calculating the widthholding tax for the part of income above the IncomeLimit."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		TesParameters: {
    			title: "tes parameters",
    			descr: "Captures the collection of parameters that different Framework agreements (TES) have.",
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses amount"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Travel expences per day"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "The amount of kilometers travelled by day - used to define the amount of travel expenses in aomse framework agreements."
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "Marker that the payments defined by framework agreement are included in the salary (will not be added by the calculation)."
    			},
    			isYksityisenHoidonTuki: {
    				title: "is yksityisen hoidon tuki",
    				descr: "If true will calculate the Finnish child care subsidy"
    			},
    			numberOfDays: {
    				title: "number of days",
    				descr: "Number of days may affect calculation of different payments calculated based on a Framework agreement"
    			},
    			subType: {
    				title: "sub type",
    				descr: "Subtype when one framework has several options"
    			},
    			type: {
    				title: "type",
    				descr: "Type of the work framework"
    			},
    			yksityisenHoidonTukiAmount: {
    				title: "yksityisen hoidon tuki amount",
    				descr: "Amount of Finnish child care subsidy"
    			}
    		},
    		TestAccountInfo: {
    			title: "test account info",
    			descr: "Object containing essential details of the test account",
    			token: {
    				title: "token",
    				descr: "Bearer token for authentication."
    			},
    			userCredential: {
    				title: "user credential",
    				descr: "The current user credentials."
    			}
    		},
    		TestValues: {
    			title: "test values",
    			descr: "Test object with value properties.",
    			arrayValue: {
    				title: "array value",
    				descr: "Test array"
    			},
    			boolValue: {
    				title: "bool value",
    				descr: "Test bool."
    			},
    			dateTimeValue: {
    				title: "date time value",
    				descr: "Test DateTime."
    			},
    			dateValue: {
    				title: "date value",
    				descr: "Test Date."
    			},
    			decimalValue: {
    				title: "decimal value",
    				descr: "Test decimal."
    			},
    			dictValue: {
    				title: "dict value",
    				descr: "Free values dictionary."
    			},
    			doubleValue: {
    				title: "double value",
    				descr: "Test double."
    			},
    			enumValue: {
    				title: "enum value",
    				descr: "Test enum"
    			},
    			intValue: {
    				title: "int value",
    				descr: "Test integer."
    			},
    			objectValue: {
    				title: "object value",
    				descr: "Test object."
    			},
    			stringValue: {
    				title: "string value",
    				descr: "Test string."
    			}
    		},
    		TotalCalculationDTO: {
    			title: "total calculation d t o",
    			descr: "Calculates the total numbers - the ones that are common to Employer and worker.",
    			pension: {
    				title: "pension",
    				descr: "The total pension payment for this calculation"
    			},
    			total: {
    				title: "total",
    				descr: "The total of all calculation rows"
    			},
    			totalAccidentInsuranceBase: {
    				title: "total accident insurance base",
    				descr: "Base income for calculating the Accident insurance contributions."
    			},
    			totalBaseSalary: {
    				title: "total base salary",
    				descr: "The base salary without overtime and other such additions that are part of gross salary."
    			},
    			totalExpenses: {
    				title: "total expenses",
    				descr: "Tax free expenses."
    			},
    			totalGrossSalary: {
    				title: "total gross salary",
    				descr: "Gross salary is the salary including additions like overtime, but not benefits or expenses"
    			},
    			totalHealthInsuranceBase: {
    				title: "total health insurance base",
    				descr: "Base income for calculating the Health insurance contributions."
    			},
    			totalPayable: {
    				title: "total payable",
    				descr: "Total sum of rows that are paid through Salaxy Customer Funds account.\nThis is the Gross sum before side costs, deductions etc., but it is not Gross salary as it includes expenses etc."
    			},
    			totalPensionInsuranceBase: {
    				title: "total pension insurance base",
    				descr: "Base income for calculating the Pension insurance contributions."
    			},
    			totalSocialSecurityBase: {
    				title: "total social security base",
    				descr: "DEPRICATED: This is now the same as TotalPensionInsuranceBase. For other social insurances, you may want to use\nTotalUnemploymentInsuranceBase, TotalHealthInsuranceBase or TotalAccidentInsuranceBase,"
    			},
    			totalTaxable: {
    				title: "total taxable",
    				descr: "Taxable salary is the salary from the taxation point of view - includes some benefits."
    			},
    			totalUnemploymentInsuranceBase: {
    				title: "total unemployment insurance base",
    				descr: "Base income for calculating the Unemployment insurance contributions."
    			},
    			unemployment: {
    				title: "unemployment",
    				descr: "The total unemployment payment sum."
    			}
    		},
    		TypeDocType: {
    			title: "type doc type",
    			descr: "Simplified read-only interface for typedoc type.",
    			elementType: {
    				title: "element type",
    				descr: "Element type in array"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier in reference"
    			},
    			name: {
    				title: "name",
    				descr: "Name of type in intrinsic and typeParameter"
    			},
    			type: {
    				title: "type",
    				descr: "Type of type"
    			},
    			typeArguments: {
    				title: "type arguments",
    				descr: "Type arguments in reference"
    			},
    			types: {
    				title: "types",
    				descr: "Collection of types in union"
    			},
    			value: {
    				title: "value",
    				descr: "Value if literal"
    			}
    		},
    		UnemploymentProduct: {
    			title: {
    				title: "title",
    				descr: "The main short title for the product / service"
    			},
    			descr: "Product that helps employer to handle the mandatory unemployment insurance reports and payments.",
    			articleId: {
    				title: "article id",
    				descr: "Identifier for the main product desription article in the Palkkaus.fi CMS"
    			},
    			description: {
    				title: "description",
    				descr: "One paragraph description text"
    			},
    			enabled: {
    				title: "enabled",
    				descr: "If true, the product is enabled for the current user"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the product"
    			},
    			img: {
    				title: "img",
    				descr: "The logo image for the product."
    			},
    			isNotificationsSelfHandling: {
    				title: "is notifications self handling",
    				descr: "If true, the account himself handles the unemployment insurance notifications."
    			},
    			isPrepayments: {
    				title: "is prepayments",
    				descr: "If true, the account has made prepayments or is registered to make prepayments.\nPalkkaus needs to contact the account."
    			},
    			isSalariesPaidThisYear: {
    				title: "is salaries paid this year",
    				descr: "If true, the account has paid salaries this year outside Palkkaus.fi.\nThis affects the yearly reporting - Palkkaus needs to contact the account.\nNote that this is a shared option with the taxation product."
    			},
    			status: {
    				title: "status",
    				descr: "The status text that should give the user.\nRestrict the length to 56 characters."
    			},
    			visible: {
    				title: "visible",
    				descr: "If false, it is not bossible to enable this product for the current account / environment.\nTypically, this means that the UI should not show this product."
    			},
    			visited: {
    				title: "visited",
    				descr: "Indicates (true-value) if the end user has confirmed the product options.\nIf false, the end user has not confirmed the option."
    			}
    		},
    		UnionPaymentUsecase: {
    			title: "union payment usecase",
    			descr: "Handles the Calculation of CalculationRowType.UnionPayment to IrRows",
    			isDeductedAtBackoffice: {
    				title: "is deducted at backoffice",
    				descr: "If true, deduction is done in Salaxy backoffice, not directly by employer.\nAs of writing, this is only done for Rakennusliitto / Construction."
    			},
    			isFixedSum: {
    				title: "is fixed sum",
    				descr: "If true, the value is a fixed sum: the price should not be updated based on salary and\nthe count should not be updated based on percentage."
    			},
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		Usecase: {
    			title: "usecase",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UsecaseChildCare: {
    			title: "usecase child care",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isChildcareSubsidy: {
    				title: "is childcare subsidy",
    				descr: "If true, the salary calculation includes child care subsidy paid by the government / community."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isSunday: {
    				title: "is sunday",
    				descr: "If true, the work is done on Sunday or public holiday.\nThis means that the salary is increased by 100% according to Finnish law"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			subsidyAmount: {
    				title: "subsidy amount",
    				descr: "Amount for the child care subsidy paid by the government / community."
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			useCustomPrice: {
    				title: "use custom price",
    				descr: "If true, a custom price is used instead of the recommended price.\nCurrently, this only affects the MLL case as the normal childCare price is always set by the user."
    			}
    		},
    		UsecaseCleaning: {
    			title: "usecase cleaning",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isContractLessThanYear: {
    				title: "is contract less than year",
    				descr: "If salaries are compensated (when holidays are not kept, isFullTime=false),\nthe percentage is different depending on whether the person has been working for a full year or not."
    			},
    			isFullTime: {
    				title: "is full time",
    				descr: "If true, this is a full time contract in the respect of holidays:\nHolidays are kept during summer and salary is paid for these holidays."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseConstruction: {
    			title: "usecase construction",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG), if null the default may be set by the group"
    			},
    			dailyExpenses: {
    				title: "daily expenses",
    				descr: "Daily expenses (työkalukorvaus) if isExpensesCustom=true"
    			},
    			dailyTravelExpenses: {
    				title: "daily travel expenses",
    				descr: "Daily travel expenses if isExpensesCustom=true"
    			},
    			dailyTravelExpensesKm: {
    				title: "daily travel expenses km",
    				descr: "Daily distance (km) from worker home to construction site."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG ), if null the default may be set by the group"
    			},
    			id: {
    				title: "id",
    				descr: "Unique identifier for the item in this user interface tree"
    			},
    			isExpensesCustom: {
    				title: "is expenses custom",
    				descr: "If true, the expenses are agreed in a bispoken way.\nDefault is that expenses are based on Framework agreement (TES) tables."
    			},
    			isHouseholdDeductible: {
    				title: "is household deductible",
    				descr: "If true, the selection is deductible in household taxation"
    			},
    			isTesIncludedInSalary: {
    				title: "is tes included in salary",
    				descr: "If true, the TES additions are included in the agreed salary."
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			occupation: {
    				title: "occupation",
    				descr: "Occupation code for insurance purposes."
    			},
    			subType: {
    				title: "sub type",
    				descr: "Salaxy Framework subtype"
    			},
    			taxDeductionCategories: {
    				title: "tax deduction categories",
    				descr: "If tax IsHouseholdDeductible is set to true, please also select the category (categories).\nNote that \"OwnPropety\" is a default, it does not need to be set: Only set RelativesProperty if necessary."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			},
    			workTypeOtherMessage: {
    				title: "work type other message",
    				descr: "If worktype is \"other\", the user should describe the work.\nThese descriptions may be used in insurance reports generation."
    			}
    		},
    		UsecaseData: {
    			title: "usecase data",
    			descr: "A usecase for creating and editing salary calculations and other . In practice, a usecase is either a user interface or a microservice that provides the bases for the object. Provides the possibility to store useacase specific data as part of object. NOTE: For Calculations, this functionality will replace TesParameters in the 3.0 version.",
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			description: {
    				title: "description",
    				descr: "Optional description of the use case for the end user.\nShown in detail views when explaining the logic of the usecase."
    			},
    			label: {
    				title: "label",
    				descr: "A short label of that can be shown to the end user."
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this may resolve to a user interface, microservice or a to manifest registering one.\nFor now, just use a unique string starting with a domain you own."
    			}
    		},
    		UsecaseGroup: {
    			title: "usecase group",
    			descr: {
    				title: "descr",
    				descr: "Description of the use case group (mainly for the lists)"
    			},
    			badge: {
    				title: "badge",
    				descr: "Path to a sub icon (typically a PNG)"
    			},
    			children: {
    				title: "children",
    				descr: "Collection of use cases - in the future, may also support sub groups."
    			},
    			icon: {
    				title: "icon",
    				descr: "Path to the main icon (typically an SVG )"
    			},
    			id: {
    				title: "id",
    				descr: "Identifier for the group"
    			},
    			label: {
    				title: "label",
    				descr: "Label in the list / tree selection"
    			},
    			uri: {
    				title: "uri",
    				descr: "Key for the usecase based on which it is resolved.\nLater, this will resolve to a microservice."
    			}
    		},
    		UserDefinedRow: {
    			title: "user defined row",
    			descr: "This is a user defined calculation row as opposed to CalculationRow that may be either based on a user defined row or generated by the system based on e.g. TES parameters.",
    			count: {
    				title: "count",
    				descr: "Count for the row - default is one"
    			},
    			data: {
    				title: "data",
    				descr: "Usecase specific data"
    			},
    			message: {
    				title: "message",
    				descr: "Description text of the row that is shown in reports. If null, will be set according to type."
    			},
    			price: {
    				title: "price",
    				descr: "Price for the row"
    			},
    			rowIndex: {
    				title: "row index",
    				descr: "Zero based row index that can be used to match the row to result rows or when deleting.\nNOTE: The Row ID is given values on the fly so setting this will not have any effect at the moment.\nIt may reorder the entries in later implementations."
    			},
    			rowType: {
    				title: "row type",
    				descr: "Logical type of the row"
    			},
    			sourceId: {
    				title: "source id",
    				descr: "Identifier in the source system is a key defined by a source system / partner system.\nThis is a pass-through string that is passed to the result calculations."
    			},
    			unit: {
    				title: "unit",
    				descr: "Unit for the row. Guessed based on the RowType and count, but you are better of setting it to be sure."
    			}
    		},
    		UserSession: {
    			title: "user session",
    			descr: "Passes the data about the authorization that is the bases for callin the services that require user authorization.",
    			avatar: {
    				title: "avatar",
    				descr: "Avatar that should be shown for the current session.\nTypically CurrentAccount.Avatar, but could theoretically be something different."
    			},
    			currentAccount: {
    				title: "current account",
    				descr: "The current Palkkaus.fi-account that the user acting as"
    			},
    			currentCredential: {
    				title: "current credential",
    				descr: "The current user credentials."
    			},
    			currentPartner: {
    				title: "current partner",
    				descr: "The primary partner of the current account."
    			},
    			hasCredential: {
    				title: "has credential",
    				descr: "If false, the current user does not have a credential in the system =&gt; need to redirect to login"
    			},
    			hasCurrentAccount: {
    				title: "has current account",
    				descr: "If false, the current user does not have a current account.\nIf HasCredential is true, this means that the user should be redirected to Account creation wizard."
    			},
    			isAuthorized: {
    				title: "is authorized",
    				descr: "If false, the current user needs to be authenticated or authenticated user does not have acces to the system."
    			},
    			requestingPartnerSite: {
    				title: "requesting partner site",
    				descr: "The site from which the API call is being made."
    			},
    			status: {
    				title: "status",
    				descr: "Authorization status"
    			},
    			systemAlertHtml: {
    				title: "system alert html",
    				descr: "Contains html for system alert."
    			}
    		},
    		VarmaPensionOrder: {
    			title: "varma pension order",
    			descr: "Contains order data for Varma pensions (TyEL, YEL) and required operations.",
    			action: {
    				title: "action",
    				descr: "Action for the order."
    			},
    			orderer: {
    				title: "orderer",
    				descr: "Orderer information."
    			},
    			tyel: {
    				title: "tyel",
    				descr: "TyEL data for the order"
    			},
    			validation: {
    				title: "validation",
    				descr: "Validation information."
    			},
    			yel: {
    				title: "yel",
    				descr: "YEL data for the order"
    			}
    		},
    		VarmaPensionOrderTyel: {
    			title: "varma pension order tyel",
    			descr: "Varma pension order TyEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startMonth: {
    				title: "start month",
    				descr: "Start month for the new order."
    			},
    			startYear: {
    				title: "start year",
    				descr: "Start year for the new order."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yerly income."
    			}
    		},
    		VarmaPensionOrderYel: {
    			title: "varma pension order yel",
    			descr: "Varma pension order YEL details.",
    			isMove: {
    				title: "is move",
    				descr: "If true, indicates that existing contract will be moved to Varma."
    			},
    			isYearlyIncomeChange: {
    				title: "is yearly income change",
    				descr: "Indicates if the existing yearly income estimate will be changed."
    			},
    			moveMoment: {
    				title: "move moment",
    				descr: "Movement timing. (Q1,Q2,Q3 or Q4, or as soon as possible)"
    			},
    			payer: {
    				title: "payer",
    				descr: "Payer definition."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal id number."
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date for the new pension."
    			},
    			yearlyIncome: {
    				title: "yearly income",
    				descr: "Estimated yearly income."
    			}
    		},
    		VismaSignPostData: {
    			title: "visma sign post data",
    			descr: "Data for building the Visma Sign API request form. This is the legacy API v0.2: https://allekirjoitus.visma.fi/download/visma_sign-api.pdf",
    			auth_service: {
    				title: "auth_service",
    				descr: "Specifies the Tupas authentication service (e.g. 'osuuspankki', 'nordea').\nIf not specified, Visma Sign will show its own selection screen."
    			},
    			customer: {
    				title: "customer",
    				descr: "Palkkaus.fi Customer code to Visma Sign-system"
    			},
    			data: {
    				title: "data",
    				descr: "Rest of the data as encrypted JSON."
    			},
    			documentUrl: {
    				title: "document url",
    				descr: "Document URL for preview purposes.\nTo Visma Sign, this is sent within the encrypted data field."
    			},
    			iv: {
    				title: "iv",
    				descr: "Encryption Initialization vector"
    			},
    			return_failure: {
    				title: "return_failure",
    				descr: "Failure address - needs to be non-encrypted, because the error may be in the encryption"
    			}
    		},
    		WelcomeDataWorker: {
    			title: "welcome data worker",
    			descr: "Data that is used for displaying the welcome page for the Worker account.",
    			activeTaxcard: {
    				title: "active taxcard",
    				descr: "Active taxcard added by the Worker or null, if none is added (or none is valid for today)."
    			},
    			invitation: {
    				title: "invitation",
    				descr: "Invitation for the worker to store a taxcard for salary payment."
    			},
    			timeline: {
    				title: "timeline",
    				descr: "The welcome page timeline cards."
    			}
    		},
    		WorkDescription: {
    			title: "work description",
    			descr: "Describes the work that is beign performed - mainly for Employment relation purposes.",
    			description: {
    				title: "description",
    				descr: "Free text description of the work that is being performed."
    			},
    			occupationCode: {
    				title: "occupation code",
    				descr: "Occupation classification, used at least for Accident insurance purposes, but may be used for other reporting.\nFor Finnish Salaries use the Statistics Finland\n\"Classification of Occupations 2010\" (TK10): https://www.stat.fi/meta/luokitukset/ammatti/017-2018-05-15/index_en.html"
    			},
    			salaryDefaults: {
    				title: "salary defaults",
    				descr: "Contains default calculation rows for the worker.\nTypically used for setting for example monthly salary or hourly salary price for workers."
    			},
    			wageBasis: {
    				title: "wage basis",
    				descr: "Default bases for wage."
    			}
    		},
    		WorkerAbsencePeriod: {
    			title: "worker absence period",
    			descr: "Absence Period with Worker information.",
    			absencesId: {
    				title: "absences id",
    				descr: "WorkerAbsences storage object that contains the period"
    			},
    			amount: {
    				title: "amount",
    				descr: "Amount of salary that is paid for the absense.\nCurrently not in use: Would be used in Incomes Register integration."
    			},
    			causeCode: {
    				title: "cause code",
    				descr: "Cause / type of the absence"
    			},
    			isHolidayAccrual: {
    				title: "is holiday accrual",
    				descr: "If true, annual leaves are accrued from the absence. If false, these days are deducted from the accrual.\nBy default should always follow the IsPaid (in the UI / calling logic), but in exceptional cases, user can set this specifically true or false."
    			},
    			isPaid: {
    				title: "is paid",
    				descr: "If true, the absence is paid by the employer."
    			},
    			notes: {
    				title: "notes",
    				descr: "Additional information as recorded by the Employer."
    			},
    			period: {
    				title: "period",
    				descr: "Period for the absance."
    			},
    			worker: {
    				title: "worker",
    				descr: "The Worker that owns the Absence period"
    			}
    		},
    		WorkerAbsences: {
    			title: "worker absences",
    			descr: "Absences for a single Worker.",
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The employment relation id for the worker whose absences are recorded."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			periods: {
    				title: "periods",
    				descr: "Collection of absence periods."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			workerId: {
    				title: "worker id",
    				descr: "Account ID for the worker whose absences are recorded."
    			},
    			workerSnapshot: {
    				title: "worker snapshot",
    				descr: "Basic information for the Worker.\nAutomatically updated upon save based on WorkerId.\nMainly for the listings etc."
    			}
    		},
    		WorkerAccount: {
    			title: "worker account",
    			descr: "Account data that is relevant for a Worker.",
    			ageRange: {
    				title: "age range",
    				descr: "Age range that is relevant to workers salary calculation and/or employment.\nSet this especially if OfficialPersonId is not set.\nIf OfficialPersonId is valid, this property is always set based on it."
    			},
    			avatar: {
    				title: "avatar",
    				descr: "Avatar is the visual representation of the Account."
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the Account."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "If created (and still owned) by an employer, this is the AccountOwnerId of the employer who created this account."
    			},
    			employment: {
    				title: "employment",
    				descr: "Current Employment Relation: The data that is owned by the Employer about the Worker\neven if the Worker has registered and created an account.\nIf employer has several Employment Relations, this is the one that is marked as Primary."
    			},
    			employmentId: {
    				title: "employment id",
    				descr: "The unique id for the employment. This does not change within one employment."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Entity type of the account.\nType PersonCreatedByEmployer means that the Worker has not yet signed into the system and there may be multiple instances of the same logical person (Official ID) - one for each employer."
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Tha bank account number of the Worker - in Iban format."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isEmployer: {
    				title: "is employer",
    				descr: "If true the object Account is and Employer account - it has the EmployerInfo object"
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			isVerified: {
    				title: "is verified",
    				descr: "If true, the account has been verified. Typically this means a digitally signed contract.\nThis means that the account also has an Identity object.\nHowever, as the Identity object contains confidential information, it is not necessarily present in all method calls."
    			},
    			isWorker: {
    				title: "is worker",
    				descr: "If true the object Account is and Worker account - it has the WorkerInfo object"
    			},
    			officialPersonId: {
    				title: "official person id",
    				descr: "The Person ID of the worker if known."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			validation: {
    				title: "validation",
    				descr: "Provides validation errors currently only in Save operations,\nbut potentially in the future also in detail/list views."
    			},
    			workerInfo: {
    				title: "worker info",
    				descr: "Information needed when salary is paid to this account and other such stuff about the account as a worker."
    			},
    			workflowData: {
    				title: "workflow data",
    				descr: "The workflow data for manual editing and approval purposes."
    			}
    		},
    		WorkerAccountEmployment: {
    			title: "worker account employment",
    			descr: "Employment Relation as it is shown as part of WorkerAccount.",
    			endDate: {
    				title: "end date",
    				descr: "End date of the Employment Relation."
    			},
    			holidays: {
    				title: "holidays",
    				descr: "Defines how the holidays are specified to this Worker."
    			},
    			irIncomeEarnerTypes: {
    				title: "ir income earner types",
    				descr: "Type of additional income earner data by National Incomer Register classification . If several types apply, all of them should be selected."
    			},
    			isAccountOwner: {
    				title: "is account owner",
    				descr: "If true, the worker is the same person as the actual signed owner of the account."
    			},
    			isFixedTerm: {
    				title: "is fixed term",
    				descr: "Duration of the employment. If true, the employment relation is for a fixed term.\nOtherwise (which is the default) the employment relation continues until further notice."
    			},
    			isTerminated: {
    				title: "is terminated",
    				descr: "Indicates that the employment relation has been terminated.\nSaving with this status also makes the employment relation Archived."
    			},
    			pensionCalculation: {
    				title: "pension calculation"
    			},
    			startDate: {
    				title: "start date",
    				descr: "Start date of the Employment Relation."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation"
    			},
    			usecase: {
    				title: "usecase",
    				descr: "Usecase is data related to the source and processing of the Employment data outside the Salaxy backend service / system.\nTypically, Usecase data would be processed by an external system that has originally created the\nEmployment Relation (Worker / Employee in many systems). E.g. expenses system, hourly reporting system."
    			},
    			work: {
    				title: "work",
    				descr: "Describes the work that is being performed in this Employment Relation."
    			}
    		},
    		WorkerCalculationDTO: {
    			title: "worker calculation d t o",
    			descr: "The Worker side of the calculation. SalarySlip is largely written based on this calculation.",
    			benefits: {
    				title: "benefits",
    				descr: "The benefits as valued by the tax authority."
    			},
    			deductions: {
    				title: "deductions",
    				descr: "All the items that are deducted from the salary (this is also the order if there is not enough payable money):\nSalaryAdvance + Tax + Pension + UnemploymentInsurance + UnionPayment + OtherDeductions"
    			},
    			foreclosure: {
    				title: "foreclosure",
    				descr: "The foreclosure that been deducted from the salary by the employer."
    			},
    			fullOtherDeductions: {
    				title: "full other deductions",
    				descr: "Other deductions that are deducted from the net salary.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullPension: {
    				title: "full pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullPrepaidExpenses: {
    				title: "full prepaid expenses",
    				descr: "Prepaid expenses that are deducted from the expenses to be paid to the worker.\nThe original calulated version - there might not be enough money to finally deduct this."
    			},
    			fullSalaryAdvance: {
    				title: "full salary advance",
    				descr: "The salary advance that was paid to the Worker\nThe original calulated version - there might not be enough money to finally pay this."
    			},
    			fullTax: {
    				title: "full tax",
    				descr: "Widthholding tax: The original calulated version - there might not be enough money to finally pay this."
    			},
    			fullUnemploymentInsurance: {
    				title: "full unemployment insurance",
    				descr: "Workers part of the unemployment insurance.\nThe original calulated version - there might not be anough money to finally pay this."
    			},
    			fullUnionPayment: {
    				title: "full union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			otherDeductions: {
    				title: "other deductions",
    				descr: "Part of the other deductions that are deducted from the net salary."
    			},
    			pension: {
    				title: "pension",
    				descr: "Part of the Pension that is taken from the Worker side of the salary."
    			},
    			prepaidExpenses: {
    				title: "prepaid expenses",
    				descr: "Part of the prepaid expenses that are deducted from the expenses."
    			},
    			salaryAdvance: {
    				title: "salary advance",
    				descr: "The salary advance that was paid to the Worker"
    			},
    			salaryAfterTax: {
    				title: "salary after tax",
    				descr: "Net salary after tax - before worker foreclosure or deductions of the side costs"
    			},
    			salaryAfterTaxAndForeclosure: {
    				title: "salary after tax and foreclosure",
    				descr: "Net salary after tax and foreclosure."
    			},
    			salaryPayment: {
    				title: "salary payment",
    				descr: "The payment that is made to the Worker based on salary after all deductions.\nDoes not include expenses."
    			},
    			tax: {
    				title: "tax",
    				descr: "Ennakonpidätys: The taxes that were deducted from the worker salary."
    			},
    			totalWorkerPayment: {
    				title: "total worker payment",
    				descr: "The sum that is at the end paid to the Worker:\nSalary payment + expenses"
    			},
    			totalWorkerPaymentLegacy: {
    				title: "total worker payment legacy",
    				descr: "Total payment using pre Insiders register calculation logic.\nDEPRICATED: This is provided for special scenarios and may be removed without warning"
    			},
    			unemploymentInsurance: {
    				title: "unemployment insurance",
    				descr: "Workers part of the unemployment insurance."
    			},
    			unionPayment: {
    				title: "union payment",
    				descr: "The union payment that is deducted from the Worker salarya s per workers request."
    			},
    			workerSideCosts: {
    				title: "worker side costs",
    				descr: "The side costs that are deducted from the Worker salary according to law:\nPension, Unemployment and Union payment. Tax is not included in this number.\nThis is not an official number, really used in the charts etc."
    			}
    		},
    		WorkerCurrentTaxcards: {
    			title: "worker current taxcards",
    			descr: "Contains the status and necessary data with Workers current taxcards.",
    			active: {
    				title: "active",
    				descr: "The currently active taxcard if one exists. This property is null by default."
    			},
    			employers: {
    				title: "employers",
    				descr: "Employer taxcards for the Worker account: One for each employment relationship."
    			},
    			employment: {
    				title: "employment",
    				descr: "Employment relation if the search is done based on employment relation ID and not Current worker account."
    			},
    			list: {
    				title: "list",
    				descr: "All the relevant taxcards as list items."
    			},
    			personalId: {
    				title: "personal id",
    				descr: "Personal ID that is used for taxcard binding. This always exists as opposed to Employment."
    			},
    			waitingApproval: {
    				title: "waiting approval",
    				descr: "A shared taxcard waiting for approval if one exists. This property is null by default."
    			}
    		},
    		WorkerData: {
    			title: "worker data",
    			descr: "Worker specific business data for index.",
    			partnerMessageAssignedTo: {
    				title: "partner message assigned to",
    				descr: "Contains the email or id of the responsible for the partner workflow event."
    			},
    			pensionCalculation: {
    				title: "pension calculation",
    				descr: "Type for the pension insurance."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the employment relation."
    			}
    		},
    		WorkerInfo: {
    			title: "worker info",
    			descr: "Information directly related to Employment (as Worker)",
    			defaultTaxCard: {
    				title: "default tax card",
    				descr: "The Tax card that is used by default."
    			}
    		},
    		WorkerListItem: {
    			title: "worker list item",
    			descr: "Represents a single item in a list of workers in the user interface. It contains all essential information of the worker and is basis for searching, filtering and sorting.",
    			backOfficeStatus: {
    				title: "back office status",
    				descr: "The back office status depending on the type of the object."
    			},
    			businessObjects: {
    				title: "business objects",
    				descr: "Business object ids related to this object. E.g. calculations and payrolls in the payment."
    			},
    			createdAt: {
    				title: "created at",
    				descr: "The date when the object was created."
    			},
    			data: {
    				title: "data",
    				descr: "Business data to include further information of the object."
    			},
    			endAt: {
    				title: "end at",
    				descr: "This is the end date of the event.\nTypically, it is the UserUpdatedAt date, but it may be something else - e.g PaidAt for the calculation."
    			},
    			entityType: {
    				title: "entity type",
    				descr: "Sub category for the payload. E.g. Payment Category, MoneyTransfer Source."
    			},
    			fee: {
    				title: "fee",
    				descr: "Estimated fee of the transaction to Palkkaus.fi."
    			},
    			flags: {
    				title: "flags",
    				descr: "Workflow flags for the object.\nOnly workflow events with API supported message types are listed."
    			},
    			grossSalary: {
    				title: "gross salary",
    				descr: "Gross salary if that is relevant to the transaction."
    			},
    			id: {
    				title: "id",
    				descr: "Identifier of the object."
    			},
    			isReadOnly: {
    				title: "is read only",
    				descr: "Indication that for the currently logged-in account, the data is generally read-only."
    			},
    			logicalDate: {
    				title: "logical date",
    				descr: "The date for the actual period for which this object is done."
    			},
    			messages: {
    				title: "messages",
    				descr: "Workflow messages for the object.\nOnly workflow events with API supported message types are listed.\nShown in format \"[MessageType]:[Message] ([User] at [UTC-time])\""
    			},
    			otherId: {
    				title: "other id",
    				descr: "The GUID for the other party. Currently, this is always the PersonID."
    			},
    			otherPartyInfo: {
    				title: "other party info",
    				descr: "The other party (usually a Person) that is involved in the event (e.g Worker if this is a Salary payment by Employer)."
    			},
    			owner: {
    				title: "owner",
    				descr: "Owner ID for this data"
    			},
    			ownerId: {
    				title: "owner id",
    				descr: "Person GUID for the owner of the object."
    			},
    			ownerInfo: {
    				title: "owner info",
    				descr: "Metadata for the owner"
    			},
    			partner: {
    				title: "partner",
    				descr: "Primary partner information."
    			},
    			payment: {
    				title: "payment",
    				descr: "This is the payment from the Owner point-of-view:\nTotal payment for the Employer and Net salary for the Worker in the case of a calculation.\nOnly add here the payment, if the payment is really made."
    			},
    			salaryDate: {
    				title: "salary date",
    				descr: "This is valid for calculations only. The estimated date of salary in worker."
    			},
    			shortText: {
    				title: "short text",
    				descr: "A very short description describing the object as an event. E.g. \"Paid salary\""
    			},
    			startAt: {
    				title: "start at",
    				descr: "When the event started. Typically, this is the CreatedAt date, but it may be something else."
    			},
    			status: {
    				title: "status",
    				descr: "The main status depending on the type of the object."
    			},
    			updatedAt: {
    				title: "updated at",
    				descr: "The time when the object was last updated.\nTypically this should be a logical update by user (UserUpdatedAt in DTO), not technical updates."
    			},
    			uri: {
    				title: "uri",
    				descr: "Salaxy uri of the resource."
    			},
    			versionNumber: {
    				title: "version number",
    				descr: "Version number. May be used in conflicts"
    			}
    		},
    		WorkerProfileProperties: {
    			title: "worker profile properties",
    			dateOfBirth: {
    				title: "date of birth"
    			},
    			dateOfBirthAccuracy: {
    				title: "date of birth accuracy"
    			},
    			gender: {
    				title: "gender"
    			},
    			hasDriversLisence: {
    				title: "has drivers lisence"
    			},
    			hasTaxCard: {
    				title: "has tax card"
    			}
    		},
    		WorkerSummary: {
    			title: "worker summary",
    			descr: "Year end summary for one Worker account.",
    			calculations: {
    				title: "calculations",
    				descr: "Calculations that have been paid through Salaxy system."
    			},
    			corrections: {
    				title: "corrections",
    				descr: "Corrections that are made to existing calculations as well as\nsalary calculations that are paid outside Salaxy."
    			},
    			insuranceWork: {
    				title: "insurance work",
    				descr: "Type of work for insurance purposes when set on worker level, not for each calculation separately."
    			},
    			summary: {
    				title: "summary",
    				descr: "Total of Calculations and Corrections as calculatied by the server side logic."
    			},
    			worker: {
    				title: "worker",
    				descr: "Worker account to which the salaries have been paid."
    			}
    		},
    		WorkersDataRow: {
    			title: "workers data row",
    			descr: "Data row for employments data. This is typically used for imports and exports.",
    			email: {
    				title: "email",
    				descr: "Email address."
    			},
    			firstName: {
    				title: "first name",
    				descr: "First name"
    			},
    			ibanNumber: {
    				title: "iban number",
    				descr: "Bank account number (IBAN)"
    			},
    			id: {
    				title: "id",
    				descr: "Id for the worker"
    			},
    			lastName: {
    				title: "last name",
    				descr: "Last name"
    			},
    			officialId: {
    				title: "official id",
    				descr: "Official id of the person or company."
    			},
    			telephone: {
    				title: "telephone",
    				descr: "Telephone number."
    			},
    			validation: {
    				title: "validation",
    				descr: "Row specific validation."
    			}
    		},
    		WorkflowData: {
    			title: "workflow data",
    			descr: "Defines the data that IWorkflowItem must provide.",
    			events: {
    				title: "events",
    				descr: "Workflow events for the item."
    			}
    		},
    		WorkflowEvent: {
    			title: "workflow event",
    			descr: "Event in the workflow",
    			assignedTo: {
    				title: "assigned to",
    				descr: "Credential of the user which is responsible of the event."
    			},
    			features: {
    				title: "features",
    				descr: "Features for this event."
    			},
    			id: {
    				title: "id",
    				descr: "Event-provider specific identifier for the event.\nFor deletable events, this should identify , but do NOT trust this to be 100% unique."
    			},
    			message: {
    				title: "message",
    				descr: "Message for backoffice purposes from the User or system that created the event."
    			},
    			time: {
    				title: "time",
    				descr: "UTC time for when the event occured."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the event. This typically maps to parent-object specific enumeration."
    			},
    			ui: {
    				title: "ui",
    				descr: "User Interface styling of the event: Indicates the severity for the end-user."
    			},
    			user: {
    				title: "user",
    				descr: "User or system that created the event."
    			}
    		},
    		WorkingTimeCompensationUsecase: {
    			title: "working time compensation usecase",
    			descr: "Handles the Calculation of CalculationRowType.WorkingTimeCompensation to IrRows",
    			kind: {
    				title: "kind",
    				descr: "Primary subtyping of the usecase."
    			}
    		},
    		YearlyChangingNumbers: {
    			title: "yearly changing numbers",
    			descr: "Parameters that are changing yearly. This class is made for the change of the year and it is typically updated in December. It is not for recording longer history of numbers.",
    			holidays: {
    				title: "holidays",
    				descr: "Returns holidays for the year."
    			},
    			sideCosts: {
    				title: "side costs",
    				descr: "Returns yearly sidecosts."
    			},
    			year: {
    				title: "year",
    				descr: "Year for numbers."
    			}
    		},
    		YearlyFeedback: {
    			title: "yearly feedback",
    			descr: "The feedback from the account holder at the end-of-year. Possibility to feed in changes, corrections and salaries that are paid outside the Salaxy system.",
    			adminNotes: {
    				title: "admin notes",
    				descr: "Full description of yearly process status for backoffice pusposes. Shown in backoffice detail views."
    			},
    			adminShortNote: {
    				title: "admin short note",
    				descr: "Short notice for backoffice purposes. This is the text visible in the lists."
    			},
    			adminStatus: {
    				title: "admin status",
    				descr: "Status for administrator check."
    			},
    			bankIban: {
    				title: "bank iban",
    				descr: "Bank IBAN number for yearly refunding of extra customer assets.\nREADONLY: Note that once set to a valid IBAN, this number is read-only and must be changed through customer service or onboarding (requires authentication)."
    			},
    			employerAvatar: {
    				title: "employer avatar",
    				descr: "Avatar for the employer of display purposes in reports etc."
    			},
    			employerId: {
    				title: "employer id",
    				descr: "Account ID for the employer."
    			},
    			externalWorkerInsuranceWork: {
    				title: "external worker insurance work",
    				descr: "Temporary user interface helper for external workers (insurance) =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			externalWorkerSalaries: {
    				title: "external worker salaries",
    				descr: "Temporary user interface helper for external workers =&gt; The final implementation will add workers to WorkerSummaries."
    			},
    			household: {
    				title: "household",
    				descr: "Yearly feedback data that is specific to household customers.\nMain part is the information realted to household deduction and how it can be divided between spouses."
    			},
    			sendInsuranceReport: {
    				title: "send insurance report",
    				descr: "If true, Salaxy should report the salaries to Insurance company.\nCurrently, this is available only for Companies and for If insurance company only."
    			},
    			userFeedbackText: {
    				title: "user feedback text",
    				descr: "Free text from the account holder to Salaxy backoffice."
    			},
    			userFeedbackTime: {
    				title: "user feedback time",
    				descr: "Time when the feedback was given."
    			},
    			userStatus: {
    				title: "user status",
    				descr: "The satus that the account holder gives to yearly material."
    			},
    			workerSummaries: {
    				title: "worker summaries",
    				descr: "List of Workers for which salaries have been paid during the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the feedback"
    			}
    		},
    		YearlyHolidays: {
    			title: "yearly holidays",
    			descr: "Holidays for the entire year.",
    			holidays: {
    				title: "holidays",
    				descr: "Holidays for the year."
    			},
    			year: {
    				title: "year",
    				descr: "Year for the holidays."
    			}
    		},
    		YearlySideCosts: {
    			title: "yearly side costs",
    			descr: "Encapsulates yearly side costs into one class.",
    			householdDeduction: {
    				title: "household deduction",
    				descr: "Household deduction"
    			},
    			illnessInsurancePercent: {
    				title: "illness insurance percent",
    				descr: "Illness Insurance care percent that is withhold from the Worker. (vakuutetun sairausvakuutusmaksu: päivärahamaksu + sairaanhoitomaksu"
    			},
    			palkkausVatPercent: {
    				title: "palkkaus vat percent",
    				descr: "ALV-% for the fee."
    			},
    			socSecPercent: {
    				title: "soc sec percent",
    				descr: "Social Security payment for employer (SAVA-maksu)"
    			},
    			taxFreeDailyAllowance: {
    				title: "tax free daily allowance",
    				descr: "Tax free daily allowance for full day approved by tax authorities."
    			},
    			taxFreeDailyHalfAllowance: {
    				title: "tax free daily half allowance",
    				descr: "Tax free daily allowance for half day approved by tax authorities."
    			},
    			taxFreeKmAllowance: {
    				title: "tax free km allowance",
    				descr: "Tax free milage allowance approved by tax authorities."
    			},
    			taxFreeMealAllowance: {
    				title: "tax free meal allowance",
    				descr: "Tax free meal allowance approved by tax authorities."
    			},
    			tyel53Percent: {
    				title: "tyel53 percent",
    				descr: "Pension payment base percentage - deducted from 53-62 year old Worker"
    			},
    			tyelBasePercent: {
    				title: "tyel base percent",
    				descr: "Pension payment base percentage"
    			},
    			tyelContractEmployerLimit: {
    				title: "tyel contract employer limit",
    				descr: "Pension contract employer six month limit"
    			},
    			tyelIncomeLimit: {
    				title: "tyel income limit",
    				descr: "Pension payment income limit per month"
    			},
    			tyelWorkerPercent: {
    				title: "tyel worker percent",
    				descr: "Pension payment base percentage - deducted from Worker salary"
    			},
    			unemploymentEmployerPercent: {
    				title: "unemployment employer percent",
    				descr: "Unemployment insurance payment percent - Employer"
    			},
    			unemploymentPartialOwnerWorkerPercent: {
    				title: "unemployment partial owner worker percent",
    				descr: "Unemployment insurance payment percent for partial owner worker - deducted from Worker"
    			},
    			unemploymentWorkerPercent: {
    				title: "unemployment worker percent",
    				descr: "Unemployment insurance payment percent - deducted from Worker"
    			},
    			unionPaymentRaksaA: {
    				title: "union payment raksa a",
    				descr: "Union payment: Rakennusliitto, A (typical)"
    			},
    			unionPaymentRaksaAoTa: {
    				title: "union payment raksa ao ta",
    				descr: "Union payment: Rakennusliitto, AO and TA"
    			}
    		},
    		YtjCompany: {
    			title: "ytj company",
    			descr: "The details of company data as provided by the Finnish Business Infromation System (YTJ/BIS)",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			companyType: {
    				title: "company type",
    				descr: "The legal type of the company, as categorized by Palkkaus / used in generating IBAN numbers."
    			},
    			companyTypeText: {
    				title: "company type text",
    				descr: "YTJ's legal type of the company as a string. This is only to be used for display purposes.\nSource: YritysHaku.YritysMuoto"
    			},
    			contact: {
    				title: "contact",
    				descr: "Contact information for the"
    			},
    			contactPerson: {
    				title: "contact person",
    				descr: "Names are not necessarily avaiable. If they are, they may not be reliable."
    			},
    			contactPersonPersonalId: {
    				title: "contact person personal id",
    				descr: "Social security number of the contact person in the company, if available."
    			},
    			contactPhone: {
    				title: "contact phone",
    				descr: "Mobile for or other company phone.\nDifference between Contact.Phone and ContactPhone is that\nthis (ContactPhone) is first mobile number, then company general phone if the former was null.\nContact.Phone is fisrt general phone number and mobile number only if not available."
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company.\nSource: Tiedostonimi.Tiedostonimi"
    			},
    			sectorOfBusinessCode: {
    				title: "sector of business code",
    				descr: "The sector where the company is active. (Toimialatieto.Koodi)"
    			},
    			sectorOfBusinessText: {
    				title: "sector of business text",
    				descr: "Text of the sector where company operates."
    			},
    			www: {
    				title: "www",
    				descr: "Web site address"
    			},
    			ytjCompanyType: {
    				title: "ytj company type",
    				descr: "The internal YTJ code number for the company's legal type."
    			}
    		},
    		YtjSearchResult: {
    			title: "ytj search result",
    			descr: "Search result for free text search from the Finnish Business Infromation System (YTJ/BIS). Returns a list of companies without details.",
    			counter: {
    				title: "counter",
    				descr: "API quota counter"
    			},
    			isSuccess: {
    				title: "is success",
    				descr: "If true, search is a success. This may mean that no results were found though."
    			},
    			isTooGeneric: {
    				title: "is too generic",
    				descr: "If true, the searh is too generic.\nTypically either there was more than 200 results or less than 3 characters in the search string.\nBoth of these return an error in the API."
    			},
    			rawInput: {
    				title: "raw input",
    				descr: "The original search string - without trimming etc. that are done in the actual search."
    			},
    			result: {
    				title: "result",
    				descr: "Result of the search."
    			},
    			status: {
    				title: "status",
    				descr: "Technical status - for developers / admin."
    			},
    			userMessage: {
    				title: "user message",
    				descr: "A message that can be shown to the end user."
    			}
    		},
    		YtjSearchResultCompany: {
    			title: "ytj search result company",
    			descr: "Search result list level of detail of companies coming from YTJ.",
    			businessId: {
    				title: "business id",
    				descr: "The Business ID (Y-tunnus / Business Identity Code) is a code given to businesses and organizations by the PRH or the Tax Administration.\nIt consists of seven digits, a dash and a control mark, for example 1234567-8.\nAlternative accepted formats incule international VAT number \"FI12345678\" or number without the dash: \"12345678\"."
    			},
    			id: {
    				title: "id",
    				descr: "Numeric ID (\"Yritysavain\")"
    			},
    			name: {
    				title: "name",
    				descr: "Official name of the company."
    			},
    			type: {
    				title: "type",
    				descr: "Type of the company. This is a free text type. Not necessary parseable to company type enumeration in details."
    			}
    		}
    	}
    };

    var SALAXY$1 = {
    	UI_TERMS: {
    		add: "Lägg till",
    		addNew: "",
    		addRow: "Lägg till rad",
    		areYouSure: "",
    		back: "Tillbaka",
    		calculate: "Beräkna",
    		calculations: "Beräkningar",
    		cancel: "Avbryt",
    		cancelEditing: "",
    		choose: "Välj…",
    		chooseWorkDateRange: "",
    		chooseWorkDuration: "",
    		clear: "Avmarkera",
    		close: "Stäng",
    		confirm: "",
    		copy: "",
    		copyAsNew: "Kopiera som ny",
    		copyAsRequest: "",
    		copyToClipboard: "",
    		dateFormats: {
    			dm: ""
    		},
    		dateNotChosen: "",
    		"delete": "Ta bort",
    		demo: "",
    		dismiss: "Avslå",
    		done: "Färdig",
    		edit: "Ändra",
    		gotIt: "Jag förstår",
    		isDeleting: "",
    		isSaving: "",
    		learnMore: "Läs mer",
    		loading: "",
    		login: "Logga in",
    		logout: "Logga ut",
    		logoutAlt: "",
    		makeOrder: "",
    		minCharacters: "Skriv minst 3 tecken…",
    		monthHalf: "",
    		monthShort0: "",
    		monthShort1: "",
    		monthShort10: "",
    		monthShort11: "",
    		monthShort12: "",
    		monthShort2: "",
    		monthShort3: "",
    		monthShort4: "",
    		monthShort5: "",
    		monthShort6: "",
    		monthShort7: "",
    		monthShort8: "",
    		monthShort9: "",
    		moreInfo: "Tilläggsuppgifter",
    		navigation: "Navigering",
    		newCalculation: "Ny beräkning",
    		next: "Nästa",
    		no: "",
    		noteRequiredField: "",
    		noThanks: "Nej tack",
    		ok: "OK",
    		oldVersion: "",
    		orChoose: "",
    		otherPeriod: "",
    		pay: "Betala",
    		pleaseWait: "",
    		postpone: "Inte nu",
    		previous: "",
    		printable: "Utskrift",
    		recalculate: "Uppdatera uträkning",
    		register: "Registrera dig",
    		save: "Spara",
    		saveEdited: "Spara ändringarna",
    		search: "",
    		searchByName: "",
    		select: "Välj",
    		selectAlt: "Välj...",
    		sendEmail: "Skicka e-postmeddelande",
    		show: "Visa",
    		skip: "Hoppa över",
    		sureToDeleteRecord: "",
    		today: "I dag",
    		updateCalculation: "",
    		updatingCalculation: "",
    		use: "Ta i bruk",
    		week2: "",
    		yes: "Ja"
    	}
    };

    var SALAXY = {
    	VALIDATION: {
    	}
    };

    /**
     * Links together the dictionary files.
     * NOTE: With typing (as any => CoreDictionaryLang) we try to make sure the Typedoc and other reflection methods
     * do not spend time in going through the json structures.
     */
    const dictionaryAll = {
        en: {
            SALAXY: {
                ENUM: Object.assign(Object.assign({}, SALAXY$a.ENUM), SALAXY$b.ENUM),
                MODEL: Object.assign(Object.assign({}, SALAXY$8.MODEL), SALAXY$9.MODEL),
                UI_TERMS: SALAXY$7.UI_TERMS,
                VALIDATION: SALAXY$6.VALIDATION,
            },
        },
        fi: {
            SALAXY: {
                ENUM: Object.assign(Object.assign({}, SALAXY$g.ENUM), SALAXY$h.ENUM),
                MODEL: Object.assign(Object.assign({}, SALAXY$e.MODEL), SALAXY$f.MODEL),
                UI_TERMS: SALAXY$d.UI_TERMS,
                VALIDATION: SALAXY$c.VALIDATION,
            },
        },
        sv: {
            SALAXY: {
                ENUM: Object.assign(Object.assign({}, SALAXY$4.ENUM), SALAXY$5.ENUM),
                MODEL: Object.assign(Object.assign({}, SALAXY$2.MODEL), SALAXY$3.MODEL),
                UI_TERMS: SALAXY$1.UI_TERMS,
                VALIDATION: SALAXY.VALIDATION,
            },
        },
        sx: {
            SALAXY: {
                // TODO: Go through the structure: Should enum and enum-ir be separate or not.
                ENUM: Object.assign(Object.assign({}, SALAXY$m.ENUM), SALAXY$n.ENUM),
                MODEL: Object.assign(Object.assign({}, SALAXY$k.MODEL), SALAXY$l.MODEL),
                UI_TERMS: SALAXY$j.UI_TERMS,
                VALIDATION: SALAXY$i.VALIDATION,
            },
        },
    };

    /**
     * Provides universal i18n methods:
     * The core JavaScript code uses this to determine the UI texts and potentially
     * formatting. The frameworks on top of core (Angular, NG1) may have their own
     * frameworks and/or they may integrate to this core i18n.
     */
    class Translations {
        /**
         * Translates given key to the current language.
         * @param key - Key for the text.
         * @param interpolateParams - Variables for the interpolation.
         * @returns The translated value in current language or if none is found the key.
         */
        static get(key, interpolateParams) {
            const template = this.getWithDefault(key, null);
            if (!template) {
                return key;
            }
            // interpolate
            const regex = /\{\{\s*(\S+)\s*\}\}/g;
            // dotted paths not supported yet.
            // array references not supported yet.
            return template.replace(regex, (_unused, varName) => {
                if (!interpolateParams) {
                    return "";
                }
                return interpolateParams[varName] || "";
            });
        }
        /**
         * Translates given key to the current language with default value.
         * @param key - Key for the text.
         * @param defaultValue - The default value that is returned if the key is not found. Null is the default.
         * @returns The translated value in current language or if none is found the key.
         */
        static getWithDefault(key, defaultValue = null) {
            if (!key) {
                return null;
            }
            // TODO: We could check here that the value is actually a string and not a dictionary.
            // But some functionality may already be relying that the function returns also dictionaries.
            return this.getAny(key) || defaultValue;
        }
        /**
         * Gets any object (string or dictionary) with the given property path.
         * @param key Key for the text.
         * @returns The object if found using the key, null if none is found.
         */
        static getAny(key) {
            if (!key) {
                return null;
            }
            const subKeys = key.split(".");
            let value = this.getCurrentDictionary()[this.lang];
            for (const subKey of subKeys) {
                if (Objects.has(value, subKey)) {
                    value = value[subKey];
                }
                else {
                    return null;
                }
            }
            return value;
        }
        /**
         * Adds a dictionary.
         * @param lang - ISO language code for the dictionary to be added: fi, en or sv.
         * @param translations - A translations object (json).
         */
        static addDictionary(lang, translations) {
            const newDictionary = {};
            newDictionary[lang] = translations;
            this.dictionary = this.mergeDeep(this.dictionary, newDictionary);
        }
        /**
         * Add all dictionaries from a single language object. This object must have language codes as first level keys.
         * @param translations - A translations object (json).
         */
        static addDictionaries(translations) {
            for (const key in translations) {
                if (Objects.has(translations, key)) {
                    this.addDictionary(key, translations[key]);
                }
            }
        }
        /**
         * Sets the Salaxy user interface language.
         * @param lang - One of the supported languages: "fi", "en" or "sv".
         */
        static setLanguage(lang) {
            Translations.lang = lang;
        }
        /**
         * Gets the current user interface language
         * @returns One of the supported languages: "fi", "en" or "sv".
         */
        static getLanguage() {
            return Translations.lang;
        }
        /** Gets the current core dictionary (all languages). */
        static getCurrentDictionary() {
            return Translations.dictionary;
        }
        /**
         * Merges objects.
         * @param objects - Objects to merge.
         */
        static mergeDeep(...objects) {
            const isObject = (obj) => obj && typeof obj === "object";
            return objects.reduce((result, current) => {
                Object.keys(current).forEach((key) => {
                    const currentVal = current[key];
                    const resultVal = result[key];
                    if (Array.isArray(resultVal) && Array.isArray(currentVal)) {
                        result[key] = resultVal.concat(...currentVal);
                    }
                    else if (isObject(resultVal) && isObject(currentVal)) {
                        result[key] = this.mergeDeep(resultVal, currentVal);
                    }
                    else {
                        result[key] = currentVal;
                    }
                });
                return result;
            }, {});
        }
    }
    /** Current language. */
    Translations.lang = "fi";
    /** Assign the dictionary to static property that may be updated by the environment. */
    Translations.dictionary = dictionaryAll;

    /**
     * Date/Time parsing, formatting and other operations within the Salaxy framework.
     * We use ISO 8601 as our Date/Time format even inside JavaScript objects.
     * This is because of the following reasons:
     *
     * 1. Generally, we do not want to use JavaScript Date because it presents so many problems.
     * 2. However, MomentJS is scheduled to be factored out to anothe JavaScript library
     * 3. We store dates in JSON as ISO 8601 date strings - this is universally the best practice
     * 4. ISO 8601 provides a way to present a date without time and most importantly timezone information
     * 5. ISO 8601 date also works best for most UI components (because of similar reasons as above).
     *
     * IMPORTANT: Use Date/Time only where TIME and/or timezone is relevant.
     * Otherwise use Date (Dates library) that does not have time or timezone.
     *
     * Implementation uses MomentJS, but we should avoid using MomentJS outside this library:
     * It will be written out in a near future!
     */
    class DateTimes {
        /**
         * Formats a datetime ISO string (or other dately object)
         * as time with texts "Today", "Yesterday", "D.M. HH:mm" for this year
         * and "D.M.YYYY" for other years (if necessary, add option for adding time to previous years).
         *
         * @param time Dately object to format
         */
        static format(time) {
            const moment = Dates.getMoment(time);
            if (!moment) {
                return "-";
            }
            if (Dates.getTodayMoment().isSame(moment, "d")) {
                return Translations.getWithDefault("SALAXY.UI_TERMS.timeToday", "Today") + " " + moment.format("HH:mm");
            }
            if (Dates.getTodayMoment().add(-1, "d").isSame(moment, "d")) {
                return Translations.getWithDefault("SALAXY.UI_TERMS.timeYesterday", "Yesterday") + " " + moment.format("HH:mm");
            }
            if (Dates.getTodayMoment().isSame(moment, "y")) {
                return moment.format("D.M. HH:mm");
            }
            return moment.format("D.M.YYYY");
        }
        /**
         * Formats a date/time range to an end-user friendly duration text.
         * @param start Start of the range as ISO string.
         * @param end End of the range as ISO string.
         * @param nullValue Value if start or end is missing or cannot be parsed.
         */
        static formatRangeToDuration(start, end, nullValue = "-") {
            if (!start || !end) {
                return nullValue;
            }
            if (end < start) {
                return this.formatDurationFromMoment(Dates.getDuration(end, start), nullValue) + " ennen";
            }
            return this.formatDurationFromMoment(Dates.getDuration(start, end), nullValue);
        }
        /**
         * Formats a date/time range to an end-user friendly duration text.
         * @param duration Duration as ISO duration string (e.g. P3Y6M4DT12H30M5S) or
         * .Net timespan (e.g. '7.23:59:59.999').
         * @param nullValue Value if duration is missing or cannot be parsed.
         */
        static formatDuration(duration, nullValue = "-") {
            if (!duration) {
                return nullValue;
            }
            let postfix = "";
            if (duration.startsWith("-")) {
                postfix = " ennen";
                duration = duration.substr(1);
            }
            return this.formatDurationFromMoment(moment.duration(duration), nullValue) + postfix;
        }
        static formatDurationFromMoment(duration, nullValue) {
            if (!duration.isValid) {
                return nullValue;
            }
            let result = "";
            if (duration.years() !== 0) {
                result += `${duration.years()} vuotta `;
            }
            if (duration.months() !== 0) {
                result += `${duration.months()} kk `;
            }
            if (duration.days() !== 0) {
                result += `${duration.days()} pv `;
            }
            if (duration.hours() !== 0) {
                result += `${duration.hours()} h `;
            }
            if (duration.minutes() !== 0) {
                result += `${duration.minutes()} min `;
            }
            return result.trim();
        }
    }

    /**
     * Helpers for handling Iban bank account numbers
     */
    class Iban {
        /**
         * Format an Iban number in 4 digits segments and trim out other than alphanum characters.
         * @param value - String value to format
         * @returns string value
         */
        static formatIban(value) {
            // Remove spaces.
            const trimmed = value.toUpperCase().replace(/ /g, "");
            // Format into four characters segments
            const numbers = [];
            for (let i = 0; i < trimmed.length; i += 4) {
                numbers.push(trimmed.substr(i, 4));
            }
            return numbers.join(" ");
        }
        /**
         * Returns true if the value is a valid IBAN number.
         * Checks for the country specific length as well as checksum.
         * @param value IBAN number number candidate.
         */
        static isIban(value) {
            // Source: https://stackoverflow.com/questions/21928083/iban-validation-check
            const CODE_LENGTHS = {
                AD: 24, AE: 23, AT: 20, AZ: 28, BA: 20, BE: 16, BG: 22, BH: 22, BR: 29,
                CH: 21, CR: 21, CY: 28, CZ: 24, DE: 22, DK: 18, DO: 28, EE: 20, ES: 24,
                FI: 18, FO: 18, FR: 27, GB: 22, GI: 23, GL: 18, GR: 27, GT: 28, HR: 21,
                HU: 28, IE: 22, IL: 23, IS: 26, IT: 27, JO: 30, KW: 30, KZ: 20, LB: 28,
                LI: 21, LT: 20, LU: 20, LV: 21, MC: 27, MD: 24, ME: 22, MK: 19, MR: 27,
                MT: 31, MU: 30, NL: 18, NO: 15, PK: 24, PL: 28, PS: 29, PT: 25, QA: 29,
                RO: 24, RS: 22, SA: 24, SE: 24, SI: 19, SK: 24, SM: 27, TN: 24, TR: 26,
            };
            const iban = value.toUpperCase().replace(/ /g, ""); // replace only spaces, no other white space, they are illegal
            /* eslint-disable-next-line @typescript-eslint/prefer-regexp-exec */
            const code = iban.match(/^([A-Z]{2})(\d{2})([A-Z\d]+)$/); // match and capture (1) the country code, (2) the check digits, and (3) the rest
            // check syntax and length
            if (!code || iban.length !== CODE_LENGTHS[code[1]]) {
                return false;
            }
            // rearrange country code and check digits, and convert chars to ints
            const digits = (code[3] + code[1] + code[2]).replace(/[A-Z]/g, (letter) => {
                return (letter.charCodeAt(0) - 55);
            });
            // final check
            return Iban.mod97(digits) === 1;
        }
        static mod97(value) {
            let checksum = value.slice(0, 2);
            for (let offset = 2; offset < value.length; offset += 7) {
                const fragment = String(checksum) + value.substring(offset, offset + 7);
                checksum = parseInt(fragment, 10) % 97;
            }
            return checksum;
        }
    }

    /**
     * Helpers for handling numbers: Mathematical operations, parsing and formatting numbers in our supported languages etc.
     */
    class Numeric {
        /**
         * Parses a user input to a float taking into consideration the Finnish culture.
         * Main case is that decimal separator is usually a comma (,).
         * If there is both comma (,) and point (.) in the same string, NaN is returned.
         * Whitespaces are removed and if string contains commas they are replaced with points before conversion to Number.
         * Null, undefined return original value, empty strings (or whitespace only) return null.
         *
         * @param numCandidate - text that should be converted to a number if possible.
         */
        static parseNumber(numCandidate) {
            if (numCandidate == null || typeof (numCandidate) === "number") {
                return numCandidate;
            }
            if (typeof (numCandidate) === "string") {
                if (numCandidate.indexOf(",") >= 0 && numCandidate.indexOf(".") >= 0) {
                    return Number.NaN;
                }
                numCandidate = numCandidate.replace(/\s/, "");
                if (numCandidate === "") {
                    return null;
                }
                return Number(numCandidate.replace(",", "."));
            }
            return Number(numCandidate);
        }
        /**
         * Formats a number as a euro price. Uses comma as decimal separator.
         * @param value - Numeric value to format
         * @param addSuffix - If true (default), adds the €-suffix.
         * @param nullChar The character or string for null or Nan value, the default is '-'.
         */
        static formatPrice(value, addSuffix = true, nullChar = "-") {
            // TODO: Add localization for English language (dot instead of comma as decimal separator).
            // TODO: Add thousand separator?
            return Numeric.formatNumber(value, 2, (addSuffix ? "€" : ""), nullChar);
        }
        /**
         * Rounds a JavaScript number in a reliable way.
         * @param value Number that is rounded. If null given, returns null.
         * @param decimals Number of decimals after the decimal separator. Default is 2 as in money.
         * @returns Number properly rounded, Null/undefined as null, Non-numeric values return Number.NaN
         */
        static round(value, decimals = 2) {
            if (value == null) { // '==' handles both null and undefined values, do not use here '==='
                return value;
            }
            if (value < 0) {
                // Round away from zero
                return -1 * Numeric.round(-1 * value, decimals);
            }
            let sValue = `${value}`;
            if (sValue.indexOf("e") >= 0) {
                sValue = `${value.toFixed(15)}`;
            }
            return Number(Math.round(`${sValue}e${decimals}`).toString() + `e-${decimals}`);
        }
        /**
         * Fixes floating point problems in typical number.toString() scenarios.
         * E.g. `console.debug((0.07 * 100).toString());` returns "7.000000000000001".
         * Null/undefined returns empty string, numbers fixed and otherwise toString() is called.
         * @param value Number to show as string.
         * @returns Number rounded to specified number of decimals. Null/undefined returns empty string.
         * @example
         * console.debug(Numeric.toString(0.07 * 100)); // returns "7"
         * console.debug((0.07 * 100).toString());      // returns "7.000000000000001"
         */
        static toString(value) {
            if (value == null) {
                return "";
            }
            return value.toFixed ? parseFloat(value.toFixed(15)).toString() : value.toString();
        }
        /**
         * Fixes floating point problems in typical number.toFixed() scenarios.
         * E.g. `console.debug((1.005).toFixed(2));` returns "1.00"
         * @param value Number to show as string. Supports null/undefined, but otherwise should be a number.
         * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.
         * @returns Number rounded to specified number of decimals. Null/undefined returns empty string.
         * @example
         * console.debug(Numeric.toFixed(1.005, 2)) // returns "1.01"
         * console.debug((1.005).toFixed(2))        // returns 1.00
         */
        static toFixed(value, fractionDigits = 0) {
            value = Numeric.round(value, fractionDigits);
            return value == null ? "" : value.toFixed(fractionDigits);
        }
        /**
         * Rounds a JavaScript number in a reliable way in percentage format.
         *
         * @param value Number that is rounded.
         * @param decimals Number of decimals after the decimal separator. Default is 2 as in money.
         * If decimals is null, no rounding is done.
         * @param addSuffix If true, adds '%' sign at the end.
         * @param nullChar The character or string for null or Nan value, the default is '-'.
         */
        static formatPercent(value, decimals = 2, addSuffix = true, nullChar = "-") {
            return Numeric.formatNumber(value, decimals, (addSuffix ? "%" : ""), nullChar);
        }
        /**
         * Rounds a JavaScript number and formats it using comma as a separator and optional suffix.
         *
         * @param value Number that is rounded.
         * @param decimals Number of decimals after the decimal separator. Default is 2 as in money. If decimals is null, no rounding is done.
         * @param suffix The suffix to add to the number.
         * @param nullChar The character or string for null or Nan value, the default is '-'.
         */
        static formatNumber(value, decimals = 2, suffix = "", nullChar = "-") {
            const asFloat = parseFloat(value);
            if (isNaN(asFloat)) {
                return nullChar;
            }
            const asFixed = Numeric.toFixed(asFloat, decimals);
            return asFixed.replace(/\./, ",") + suffix;
        }
        /**
         * Checks if the given object is a number.
         * @param numberCandidate Object to check
         */
        static isNumber(numberCandidate) {
            return !isNaN(parseFloat(numberCandidate)) && isFinite(numberCandidate);
        }
    }

    /**
     *  base64.ts
     *
     *  Licensed under the BSD 3-Clause License.
     *    http://opensource.org/licenses/BSD-3-Clause
     *
     *  References:
     *    http://en.wikipedia.org/wiki/Base64
     *
     * @author Dan Kogai (https://github.com/dankogai)
     */
    const version = '3.6.1';
    /**
     * @deprecated use lowercase `version`.
     */
    const VERSION = version;
    const _hasatob = typeof atob === 'function';
    const _hasbtoa = typeof btoa === 'function';
    const _hasBuffer = typeof Buffer === 'function';
    const _TD = typeof TextDecoder === 'function' ? new TextDecoder() : undefined;
    const _TE = typeof TextEncoder === 'function' ? new TextEncoder() : undefined;
    const b64ch = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
    const b64chs = [...b64ch];
    const b64tab = ((a) => {
        let tab = {};
        a.forEach((c, i) => tab[c] = i);
        return tab;
    })(b64chs);
    const b64re = /^(?:[A-Za-z\d+\/]{4})*?(?:[A-Za-z\d+\/]{2}(?:==)?|[A-Za-z\d+\/]{3}=?)?$/;
    const _fromCC = String.fromCharCode.bind(String);
    const _U8Afrom = typeof Uint8Array.from === 'function'
        ? Uint8Array.from.bind(Uint8Array)
        : (it, fn = (x) => x) => new Uint8Array(Array.prototype.slice.call(it, 0).map(fn));
    const _mkUriSafe = (src) => src
        .replace(/[+\/]/g, (m0) => m0 == '+' ? '-' : '_')
        .replace(/=+$/m, '');
    const _tidyB64 = (s) => s.replace(/[^A-Za-z0-9\+\/]/g, '');
    /**
     * polyfill version of `btoa`
     */
    const btoaPolyfill = (bin) => {
        // console.log('polyfilled');
        let u32, c0, c1, c2, asc = '';
        const pad = bin.length % 3;
        for (let i = 0; i < bin.length;) {
            if ((c0 = bin.charCodeAt(i++)) > 255 ||
                (c1 = bin.charCodeAt(i++)) > 255 ||
                (c2 = bin.charCodeAt(i++)) > 255)
                throw new TypeError('invalid character found');
            u32 = (c0 << 16) | (c1 << 8) | c2;
            asc += b64chs[u32 >> 18 & 63]
                + b64chs[u32 >> 12 & 63]
                + b64chs[u32 >> 6 & 63]
                + b64chs[u32 & 63];
        }
        return pad ? asc.slice(0, pad - 3) + "===".substring(pad) : asc;
    };
    /**
     * does what `window.btoa` of web browsers do.
     * @param {String} bin binary string
     * @returns {string} Base64-encoded string
     */
    const _btoa = _hasbtoa ? (bin) => btoa(bin)
        : _hasBuffer ? (bin) => Buffer.from(bin, 'binary').toString('base64')
            : btoaPolyfill;
    const _fromUint8Array = _hasBuffer
        ? (u8a) => Buffer.from(u8a).toString('base64')
        : (u8a) => {
            // cf. https://stackoverflow.com/questions/12710001/how-to-convert-uint8-array-to-base64-encoded-string/12713326#12713326
            const maxargs = 0x1000;
            let strs = [];
            for (let i = 0, l = u8a.length; i < l; i += maxargs) {
                strs.push(_fromCC.apply(null, u8a.subarray(i, i + maxargs)));
            }
            return _btoa(strs.join(''));
        };
    /**
     * converts a Uint8Array to a Base64 string.
     * @param {boolean} [urlsafe] URL-and-filename-safe a la RFC4648 §5
     * @returns {string} Base64 string
     */
    const fromUint8Array = (u8a, urlsafe = false) => urlsafe ? _mkUriSafe(_fromUint8Array(u8a)) : _fromUint8Array(u8a);
    // This trick is found broken https://github.com/dankogai/js-base64/issues/130
    // const utob = (src: string) => unescape(encodeURIComponent(src));
    // reverting good old fationed regexp
    const cb_utob = (c) => {
        if (c.length < 2) {
            var cc = c.charCodeAt(0);
            return cc < 0x80 ? c
                : cc < 0x800 ? (_fromCC(0xc0 | (cc >>> 6))
                    + _fromCC(0x80 | (cc & 0x3f)))
                    : (_fromCC(0xe0 | ((cc >>> 12) & 0x0f))
                        + _fromCC(0x80 | ((cc >>> 6) & 0x3f))
                        + _fromCC(0x80 | (cc & 0x3f)));
        }
        else {
            var cc = 0x10000
                + (c.charCodeAt(0) - 0xD800) * 0x400
                + (c.charCodeAt(1) - 0xDC00);
            return (_fromCC(0xf0 | ((cc >>> 18) & 0x07))
                + _fromCC(0x80 | ((cc >>> 12) & 0x3f))
                + _fromCC(0x80 | ((cc >>> 6) & 0x3f))
                + _fromCC(0x80 | (cc & 0x3f)));
        }
    };
    const re_utob = /[\uD800-\uDBFF][\uDC00-\uDFFFF]|[^\x00-\x7F]/g;
    /**
     * @deprecated should have been internal use only.
     * @param {string} src UTF-8 string
     * @returns {string} UTF-16 string
     */
    const utob = (u) => u.replace(re_utob, cb_utob);
    //
    const _encode = _hasBuffer
        ? (s) => Buffer.from(s, 'utf8').toString('base64')
        : _TE
            ? (s) => _fromUint8Array(_TE.encode(s))
            : (s) => _btoa(utob(s));
    /**
     * converts a UTF-8-encoded string to a Base64 string.
     * @param {boolean} [urlsafe] if `true` make the result URL-safe
     * @returns {string} Base64 string
     */
    const encode = (src, urlsafe = false) => urlsafe
        ? _mkUriSafe(_encode(src))
        : _encode(src);
    /**
     * converts a UTF-8-encoded string to URL-safe Base64 RFC4648 §5.
     * @returns {string} Base64 string
     */
    const encodeURI = (src) => encode(src, true);
    // This trick is found broken https://github.com/dankogai/js-base64/issues/130
    // const btou = (src: string) => decodeURIComponent(escape(src));
    // reverting good old fationed regexp
    const re_btou = /[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3}/g;
    const cb_btou = (cccc) => {
        switch (cccc.length) {
            case 4:
                var cp = ((0x07 & cccc.charCodeAt(0)) << 18)
                    | ((0x3f & cccc.charCodeAt(1)) << 12)
                    | ((0x3f & cccc.charCodeAt(2)) << 6)
                    | (0x3f & cccc.charCodeAt(3)), offset = cp - 0x10000;
                return (_fromCC((offset >>> 10) + 0xD800)
                    + _fromCC((offset & 0x3FF) + 0xDC00));
            case 3:
                return _fromCC(((0x0f & cccc.charCodeAt(0)) << 12)
                    | ((0x3f & cccc.charCodeAt(1)) << 6)
                    | (0x3f & cccc.charCodeAt(2)));
            default:
                return _fromCC(((0x1f & cccc.charCodeAt(0)) << 6)
                    | (0x3f & cccc.charCodeAt(1)));
        }
    };
    /**
     * @deprecated should have been internal use only.
     * @param {string} src UTF-16 string
     * @returns {string} UTF-8 string
     */
    const btou = (b) => b.replace(re_btou, cb_btou);
    /**
     * polyfill version of `atob`
     */
    const atobPolyfill = (asc) => {
        // console.log('polyfilled');
        asc = asc.replace(/\s+/g, '');
        if (!b64re.test(asc))
            throw new TypeError('malformed base64.');
        asc += '=='.slice(2 - (asc.length & 3));
        let u24, bin = '', r1, r2;
        for (let i = 0; i < asc.length;) {
            u24 = b64tab[asc.charAt(i++)] << 18
                | b64tab[asc.charAt(i++)] << 12
                | (r1 = b64tab[asc.charAt(i++)]) << 6
                | (r2 = b64tab[asc.charAt(i++)]);
            bin += r1 === 64 ? _fromCC(u24 >> 16 & 255)
                : r2 === 64 ? _fromCC(u24 >> 16 & 255, u24 >> 8 & 255)
                    : _fromCC(u24 >> 16 & 255, u24 >> 8 & 255, u24 & 255);
        }
        return bin;
    };
    /**
     * does what `window.atob` of web browsers do.
     * @param {String} asc Base64-encoded string
     * @returns {string} binary string
     */
    const _atob = _hasatob ? (asc) => atob(_tidyB64(asc))
        : _hasBuffer ? (asc) => Buffer.from(asc, 'base64').toString('binary')
            : atobPolyfill;
    //
    const _toUint8Array = _hasBuffer
        ? (a) => _U8Afrom(Buffer.from(a, 'base64'))
        : (a) => _U8Afrom(_atob(a), c => c.charCodeAt(0));
    /**
     * converts a Base64 string to a Uint8Array.
     */
    const toUint8Array = (a) => _toUint8Array(_unURI(a));
    //
    const _decode = _hasBuffer
        ? (a) => Buffer.from(a, 'base64').toString('utf8')
        : _TD
            ? (a) => _TD.decode(_toUint8Array(a))
            : (a) => btou(_atob(a));
    const _unURI = (a) => _tidyB64(a.replace(/[-_]/g, (m0) => m0 == '-' ? '+' : '/'));
    /**
     * converts a Base64 string to a UTF-8 string.
     * @param {String} src Base64 string.  Both normal and URL-safe are supported
     * @returns {string} UTF-8 string
     */
    const decode = (src) => _decode(_unURI(src));
    /**
     * check if a value is a valid Base64 string
     * @param {String} src a value to check
      */
    const isValid = (src) => {
        if (typeof src !== 'string')
            return false;
        const s = src.replace(/\s+/g, '').replace(/=+$/, '');
        return !/[^\s0-9a-zA-Z\+/]/.test(s) || !/[^\s0-9a-zA-Z\-_]/.test(s);
    };
    //
    const _noEnum = (v) => {
        return {
            value: v, enumerable: false, writable: true, configurable: true
        };
    };
    /**
     * extend String.prototype with relevant methods
     */
    const extendString = function () {
        const _add = (name, body) => Object.defineProperty(String.prototype, name, _noEnum(body));
        _add('fromBase64', function () { return decode(this); });
        _add('toBase64', function (urlsafe) { return encode(this, urlsafe); });
        _add('toBase64URI', function () { return encode(this, true); });
        _add('toBase64URL', function () { return encode(this, true); });
        _add('toUint8Array', function () { return toUint8Array(this); });
    };
    /**
     * extend Uint8Array.prototype with relevant methods
     */
    const extendUint8Array = function () {
        const _add = (name, body) => Object.defineProperty(Uint8Array.prototype, name, _noEnum(body));
        _add('toBase64', function (urlsafe) { return fromUint8Array(this, urlsafe); });
        _add('toBase64URI', function () { return fromUint8Array(this, true); });
        _add('toBase64URL', function () { return fromUint8Array(this, true); });
    };
    /**
     * extend Builtin prototypes with relevant methods
     */
    const extendBuiltins = () => {
        extendString();
        extendUint8Array();
    };
    const gBase64 = {
        version: version,
        VERSION: VERSION,
        atob: _atob,
        atobPolyfill: atobPolyfill,
        btoa: _btoa,
        btoaPolyfill: btoaPolyfill,
        fromBase64: decode,
        toBase64: encode,
        encode: encode,
        encodeURI: encodeURI,
        encodeURL: encodeURI,
        utob: utob,
        btou: btou,
        decode: decode,
        isValid: isValid,
        fromUint8Array: fromUint8Array,
        toUint8Array: toUint8Array,
        extendString: extendString,
        extendUint8Array: extendUint8Array,
        extendBuiltins: extendBuiltins,
    };

    /**
     * Helper class for JWT tokens.
     */
    class Token {
        /**
         * Validates the JWT token. Please note that the method does not check the signature.
         * Returns:
         * noToken - the token is null,
         * invalid - the token has invalid format, or some payload values (e.g. exp) are missing,
         * expired - the token has expired,
         * ok - token is valid.
         */
        static validate(token) {
            if (!token) {
                return "noToken";
            }
            const payload = this.parsePayload(token);
            if (!payload || !payload.exp) {
                return "invalid";
            }
            if (payload.exp * 1000 <= new Date().getTime()) {
                return "expired";
            }
            return "ok";
        }
        /**
         * Decodes and parses the payload from the string formatted token.
         * @param token Token as a string.
         * @returns Payload of the token as JSON object.
         */
        static parsePayload(token) {
            const tokenParts = token.split(".");
            if (tokenParts.length === 3) {
                let payload = tokenParts[1];
                payload = gBase64.decode(unescape(payload));
                const data = JSON.parse(payload);
                return data;
            }
            return null;
        }
        /**
         * Decodes and parses the header from the string formatted token.
         * @param token Token as a string.
         * @returns Payload of the token as JSON object.
         */
        static parseHeader(token) {
            const tokenParts = token.split(".");
            if (tokenParts.length === 3) {
                let payload = tokenParts[0];
                payload = gBase64.decode(unescape(payload));
                const data = JSON.parse(payload);
                return data;
            }
            return null;
        }
    }

    /** Provides helpers for handling OData queries */
    class OData {
        /**
         * Makes an OData query to the the given service URL.
         * @param baseUrl The service URL that is used, e.g. "/v03-rc/api/taxcards/current".
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         * @param ajax The Ajax implementation that is used for HTTP get.
         */
        static getOData(baseUrl, query, ajax) {
            let fullPath;
            if (typeof query === "string") {
                query = query.trim();
                if (query.length > 0 && query.substr(0, 1) === "?") {
                    query = query.substr(1);
                }
                fullPath = (query.length === 0) ? baseUrl : (baseUrl + "?" + query);
            }
            else {
                query = query || {};
                if (Token.validate(ajax.getCurrentToken()) !== "ok" && !query.anon) {
                    throw new Error("OData call without token and anon option not set to true.");
                }
                fullPath = OData.getUrl(baseUrl, query);
            }
            return ajax.getJSON(fullPath).then((data) => {
                return OData.getODataResult(data);
            });
        }
        /**
         * Gets the result from Ajax result (json) to strongly typed result.
         * @param data Result data coming from the ajax call.
         * This may be either an OData object with count and next page link or just an array depending on the call.
         */
        static getODataResult(data) {
            const result = {
                items: [],
            };
            if (data.items) {
                data = data;
                result.items = data.items;
                result.count = data.count || null;
                result.nextPageLink = data.nextPageLink || null;
            }
            else if (Array.isArray(data)) {
                result.items = data;
                result.count = null;
                result.nextPageLink = null;
            }
            else {
                throw Error("Unexpected result: " + typeof (data));
            }
            return result;
        }
        /**
         * Escapes the Lucene search language special characters in search text values.
         * Special characters in Lucene Full syntax are: "+-&|!(){}[]^"~*?:\/"
         * and in simple syntax: "-*+|"()'\"
         * @param searchValue Search text value to escape for Lucene.
         * @param isSimpleSyntax If true, only escapes the necessary characters for simple syntax.
         */
        static escapeSearchValue(searchValue, isSimpleSyntax = false) {
            // TODO: Escapes here are probably really useless, but would need testing.
            if (isSimpleSyntax) {
                // eslint-disable-next-line no-useless-escape
                return searchValue.replace(/[\-\*\+\|\(\)\'\\]/g, (substring) => "\\" + substring);
            }
            // eslint-disable-next-line no-useless-escape
            return searchValue.replace(/[\+\-\&\|\!\(\)\{\}\[\]\^\"\~\*\?\:\\\/]/g, (substring) => "\\" + substring);
        }
        /**
         * Gets the URL including the query string using the provided OData query options.
         * @param baseUrl Base URL of the service to which the query is added.
         * If null, the returned URL is the query string part starting with the question mark
         * ...or if none of the options is set, returns just an empty string.
         * @param queryOptions options based on which the query is constructed.
         */
        static getUrl(baseUrl, queryOptions) {
            queryOptions = queryOptions || {};
            baseUrl = (baseUrl || "").trim();
            const queries = [];
            if (queryOptions.$count) {
                queries.push("$count=true");
            }
            if (queryOptions.$expand) {
                queries.push("$expand=" + queryOptions.$expand);
            }
            if (queryOptions.$filter) {
                queries.push("$filter=" + queryOptions.$filter);
            }
            if (queryOptions.$orderby) {
                queries.push("$orderby=" + queryOptions.$orderby);
            }
            if (queryOptions.$select) {
                queries.push("$select=" + queryOptions.$select);
            }
            if (queryOptions.$skip) {
                queries.push(`$skip=${queryOptions.$skip}`);
            }
            if (queryOptions.$top) {
                queries.push(`$top=${queryOptions.$top}`);
            }
            if (queryOptions.fullSearch) {
                queries.push("$search=" + encodeURIComponent(queryOptions.fullSearch));
            }
            else if (queryOptions.$search) {
                let search = queryOptions.$search;
                if (search.indexOf("\"") >= 0) {
                    // phrase search
                    queries.push("$search=" + encodeURIComponent(`${search}`));
                }
                else {
                    search = this.escapeSearchValue(queryOptions.$search, true);
                    // HACK: This is probably not necessary: The searches should work simply by escaping, but
                    //       we must test the behavior of "@" (email) and "-" (official id) before removing this.
                    //       Also note that $search may be depracted in favor of fullSearch.
                    // search by phrase, term and wildcard
                    search = search.endsWith("*") ? search.substring(0, search.length - 1) : search;
                    queries.push("$search=" + encodeURIComponent(`"${search}" | ${search} | ${search}*`));
                }
            }
            return baseUrl + (queries.length === 0 ? "" : (baseUrl.indexOf("?") >= 0 ? "&" : "?") + queries.join("&"));
        }
    }

    /**
     * Helpers for formatting and encodin / decoding texts.
     */
    class Texts {
        /**
         * Escapes HTML
         * @param html HTML text to escape
         * @returns HTML escaped (&<>!'/)
         */
        static escapeHtml(html) {
            const entityMap = {
                "&": "&amp;",
                "<": "&lt;",
                ">": "&gt;",
                '"': '&quot;',
                "'": '&#39;',
                "/": '&#x2F;'
            };
            // TODO: The escape is probably useless, but needs testing
            // eslint-disable-next-line no-useless-escape
            return String(html).replace(/[&<>"'\/]/g, (s) => {
                return entityMap[s];
            });
        }
        /**
         * Replaces characters in strings that are illegal/unsafe for filenames.
         * Unsafe characters are either removed or replaced by a substitute set
         * in the optional `options` object.
         *
         * Illegal Characters on Various Operating Systems
         * / ? < > \ : * | "
         * https://kb.acronis.com/content/39790
         *
         * Unicode Control codes
         * C0 0x00-0x1f & C1 (0x80-0x9f)
         * http://en.wikipedia.org/wiki/C0_and_C1_control_codes
         *
         * Reserved filenames on Unix-based systems (".", "..")
         * Reserved filenames in Windows ("CON", "PRN", "AUX", "NUL", "COM1",
         * "COM2", "COM3", "COM4", "COM5", "COM6", "COM7", "COM8", "COM9",
         * "LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", and
         * "LPT9") case-insesitively and with or without filename extensions.
         *
         * Capped at 255 characters in length.
         * http://unix.stackexchange.com/questions/32795/what-is-the-maximum-allowed-filename-and-folder-size-with-ecryptfs
         *
         * @param name File name to escape.
         */
        static escapeFileName(name) {
            // External code: We do not want to fix and retest it. At least the second line is probably justified. The 1. and 3. perhaps not.
            const illegal = /[\/\?<>\\:\*\|"]/g; // eslint-disable-line
            const control = /[\x00-\x1f\x80-\x9f]/g; // eslint-disable-line
            const reserved = /^\.+$/;
            const windowsReserved = /^(con|prn|aux|nul|com[0-9]|lpt[0-9])(\..*)?$/i;
            const windowsTrailing = /[\. ]+$/; // eslint-disable-line
            const r = "_";
            if (!name) {
                return name;
            }
            return name
                .replace(illegal, r)
                .replace(control, r)
                .replace(reserved, r)
                .replace(windowsReserved, r)
                .replace(windowsTrailing, r);
        }
    }

    /**
     * Provides base methods for validation.
     * These are typically used in user interface validator components.
     */
    class Validation {
        /**
         * Returns true if the given value is a valid Finnish Personal ID (HETU / SSN).
         * Checks the format and checksum.
         * @param value Candidate for Finnish Personal ID.
         */
        static isPersonalIdFi(value) {
            if (!value) {
                return true;
            }
            // TODO: The escape is probably useless, but needs testing
            // eslint-disable-next-line no-useless-escape
            const pattern = /^(\d{6})([\+\-A])(\d{3})([a-zA-Z0-9\*])$/;
            const groups = pattern.exec(value);
            if (!groups) {
                return false;
            }
            const date = groups[1];
            const centenry = groups[2];
            const person = groups[3];
            const checksum = groups[4];
            const cc = centenry === "-" ? "19" : "20";
            if (!Dates.isValidDateTime(date.substr(0, 4) + cc + date.substr(4, 2), true, "DDMMYYYY")) {
                return false;
            }
            if (checksum === "*") {
                return true;
            }
            const checksumLetters = "0123456789ABCDEFHJKLMNPRSTUVWXY";
            const calculatedChecksum = checksumLetters[parseInt(date + person, 10) % 31];
            return calculatedChecksum === checksum.toUpperCase();
        }
        /**
         * Returns true if the value is a valid e-mail.
         * Currently uses a simple regex (source?).
         * @param value E-mail candidate.
         */
        static isEmail(value) {
            // TODO: The escape is probably useless, but needs testing
            // eslint-disable-next-line no-useless-escape
            const regExp = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid mobile phone number: 5-16 numbers and spaces (5-18 if there is a "+" in the beginning).
         * Currently uses a naive regex, but we could extend this to take into account the regional differences.
         * @param value Mobile phone number candidate.
         */
        static isMobilePhone(value) {
            // TODO: Check lengths.
            const regExp = /^(\+[\d ]{1,3}[\d ]{4,15})$|^([\d ]{5,16})$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid currency amount with maximum two decimals.
         * @param value Currency amount candidate
         */
        static isCurrency(value) {
            const regExp = /^\d+(([.]|[,])\d{1,2})?$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid number.
         * @param value Number candidate
         */
        static isNumber(value) {
            const regExp = /^\d+(([.]|[,])\d+)?$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid integer.
         * @param value Number candidate
         */
        static isInteger(value) {
            const regExp = /^\d+$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid tax percent amount with maximum one decimal.
         * @param value Tax percent candidate
         */
        static isTaxPercent(value) {
            const regExp = /^\d{1,2}(([.]|[,])\d{1})?$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid SMS verification code.
         * SMS verification code is 6 digits.
         * @param value Verification code candidate.
         */
        static isSmsVerificationCode(value) {
            const regExp = /^\d{6}$/;
            return regExp.test(value);
        }
        /** Returns true if the value is a valid Finnish Postal code: 5 numbers. */
        static isPostalCodeFi(value) {
            const regExp = /^\d{5}$/;
            return regExp.test(value);
        }
        /**
         * Returns true if the value is a valid Finnish Company ID. If the format is not correct, will return false.
         * @param companyIdCandidate String to validate / format
         * @param allowStarAsChecksum If true, allows the checksum character (the last char) to be an asterix (*).
         * @returns boolean
         */
        static formatCompanyIdFi(companyIdCandidate, allowStarAsChecksum = true) {
            if (!companyIdCandidate) {
                return false;
            }
            companyIdCandidate = companyIdCandidate.toUpperCase().trim().replace("-", "").replace("–", "").replace("—", ""); // dash, n-dash, m-dash
            if (companyIdCandidate.substr(0, 2) === "FI") {
                companyIdCandidate = companyIdCandidate.substr(2);
            }
            if (companyIdCandidate.length === 7) {
                // Some old companies may have 6+1 numbers
                companyIdCandidate = "0" + companyIdCandidate;
            }
            if (companyIdCandidate.length !== 8) {
                return false;
            }
            const multipliers = [7, 9, 10, 5, 8, 4, 2];
            let total = 0;
            for (let i = 0; i < 7; i++) {
                const num = Number(companyIdCandidate.charAt(i));
                if (num < 0 || num > 9) {
                    return false;
                }
                total += num * multipliers[i];
            }
            const modulo = total % 11;
            if (modulo === 1) {
                return false;
            }
            const checksum = modulo === 0 ? 0 : (11 - modulo);
            return (allowStarAsChecksum && companyIdCandidate.charAt(7) === "*") || Number(companyIdCandidate[7]) === checksum;
        }
        /**
         * Returns true if the value is a valid IBAN number.
         * Checks for the country specific length as well as checksum.
         * @param value IBAN number number candidate.
         */
        static isIban(value) {
            return Iban.isIban(value);
        }
        /**
         * Returns PensionCompany for the given pension contract number.
         * @param value - Pension contract number.
         */
        static isPensionContractNumber(value) {
            if (value == null) {
                return null;
            }
            let regExp = null;
            // TODO: The escape is probably useless, but needs testing
            if (value.startsWith("7") || value.startsWith("8")) {
                // eslint-disable-next-line no-useless-escape
                regExp = /^(\d{5})\-?(\d{4})([a-zA-Z0-9\*])$/;
            }
            else {
                // eslint-disable-next-line no-useless-escape
                regExp = /^(\d{2})\-?(\d{7})([a-zA-Z0-9\*])$/;
            }
            const result = regExp.exec(value);
            if (!result) {
                return null;
            }
            const company = result[1];
            const contract = result[2];
            const check = result[3];
            const checksumLetters = "0123456789ABCDEFHJKLMNPRSTUVWXY";
            const calculatedChecksum = checksumLetters[parseInt(company + contract, 10) % 31];
            if (calculatedChecksum !== check) {
                return null;
            }
            switch (company) {
                case "80009":
                    return exports.PensionCompany.Apteekkien;
                case "80008":
                    return exports.PensionCompany.Verso;
                case "54":
                    return exports.PensionCompany.Elo;
                case "90":
                    return exports.PensionCompany.Etera;
                case "46":
                    return exports.PensionCompany.Ilmarinen;
                case "40":
                    return exports.PensionCompany.PensionsAlandia;
                case "55":
                    return exports.PensionCompany.Varma;
                case "56":
                    return exports.PensionCompany.Veritas;
            }
            return null;
        }
        /**
         * Returns PensionCompany for the given *temporary* pension contract number.
         * @param value - TemporaryPension contract number.
         */
        static isTemporaryPensionContractNumber(value) {
            if (value == null) {
                return null;
            }
            switch (value) {
                case "55-3999990D":
                    return exports.PensionCompany.Varma;
                case "46-3000000V":
                    return exports.PensionCompany.Ilmarinen;
                case "54-0000000U":
                    return exports.PensionCompany.Elo;
                case "56-0049990P":
                    return exports.PensionCompany.Veritas;
            }
            return null;
        }
    }

    /**
     * Helper for yearly changing sidecost percentages and holidays.
     */
    class Years {
        /**
         * Gets salary calculation parameters that are changing yearly and holidays of the year.
         * The method is designed for the end-of-year and as such it only supports 2 years:
         * the current / previous year (from Jan to Nov) OR current / next year (in approx. December).
         *
         * @param forYear - Dately object (JavaScript date, ISO string formatted date, or moment) for which the numbers are fetched.
         *
         * @returns The yearly changing numbers if year supported. Error if year not supported.
         */
        static getYearlyChangingNumbers(forYear) {
            const year = Dates.getYear(forYear);
            const numbers = yearlyChangingNumbersYears.find((x) => x.year === year);
            if (!numbers) {
                throw new Error(`Year ${year} not supported.`);
            }
            numbers.holidays = this.getYearlyHolidays(forYear);
            return numbers;
        }
        /**
         * Gets holidays for the given year.
         *
         * @param forYear - Dately object (JavaScript date, ISO string formatted date, or moment) for which the holidays are fetched.
         *
         * @returns The holidays for the given year.
         */
        static getYearlyHolidays(forYear) {
            // Implementation is moved to Dates to avoid circular dependencies.
            return Dates.getYearlyHolidays(forYear);
        }
    }

    /** Base class for CRUD services. */
    class CrudApiBase {
        /** Constructor creates a new CRUD api with given Ajax-implementation */
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Makes an OData query to the service's main OData method.
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         */
        getOData(query) {
            // TODO: Make result strongly typed: A second Generic parameter of the class.
            const baseUrl = this.getODataUrl();
            return OData.getOData(baseUrl, query, this.ajax);
        }
        /**
         * Makes an OData query to the the given service URL.
         * @param baseUrl The service URL that is used, e.g. "/v03-rc/api/taxcards/current".
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         * @deprecated You should use the OData.getOData() static util method instead.
         */
        getODataBase(baseUrl, query) {
            // TODO: Check that it is not used and remove.
            return OData.getOData(baseUrl, query, this.ajax);
        }
        /**
         * Gets all the items of a given type.
         * @returns A Promise with result data array.
         */
        getAll() {
            return this.ajax.getJSON(this.baseUrl);
        }
        /**
         * Gets a single item based on identier
         * @param id - Unique identifier for the object
         * @returns A Promise with result data.
         */
        getSingle(id) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}`);
        }
        /**
         * Deletes an single item from the sotrage
         * @param id - Unique identifier for the object
         * @returns A Promise with result data "Object deleted".
         */
        delete(id) {
            return this.ajax.remove(`${this.baseUrl}/${id}`);
        }
        /**
         * Saves an item to the storage.
         * If id is null, this is add/insert. If id exists, this is update.
         * @param itemToSave - The item that is updated/inserted.
         * @returns A Promise with result data as saved to the storage (contains id, createAt, owner etc.).
         */
        save(itemToSave) {
            return this.ajax.postJSON(`${this.baseUrl}/`, itemToSave);
        }
        /** Gets the OData URL for v03 compatible services */
        getODataUrl() {
            if (this.baseUrl.startsWith("/v03")) {
                // e.g. "/v03-rc/api/messages"
                return this.baseUrl;
            }
            // Backporting the OData features to v02 version API's
            switch (this.baseUrl) {
                case "/accounts/workers":
                    // This API now really returns employment relations, not workers. Will probably not change this in v03 scope
                    return "/v03-rc/api/accounts/workers";
                case "/calculations":
                    return "/v03-rc/api/calculations/all";
                case "/taxcards/owned":
                    return "/v03-rc/api/taxcards";
                case "/payments":
                case "/accounts/certificate": // TODO: Account based - probably implement
                case "/accounts/credential":
                case "/accounts/authorizedAccount":
                    throw new Error("getODataUrl() not implemented for: " + this.baseUrl);
                case "/absences":
                case "/holidays/all":
                    throw new Error("getODataUrl() is only implemented in Version 03 for: " + this.baseUrl);
                case "/yearEnd/feedback":
                    throw new Error("Year end is depricated and OData querying does not make sense for YearEnd in any case (because there is only on per year).");
            }
            throw new Error(`Service ${this.baseUrl} does not support OData interface.`);
        }
        /** Returns the base url for this api. */
        getBaseUrl() {
            return this.baseUrl;
        }
        /**
         * Adds/updates the workflow event for the item.
         *
         * @param item - Item for which to set the workflow event.
         * @param wfEvent - Event to add or update.
         * @returns Workflow data for the item.
         */
        saveWorkflowEvent(item, wfEvent) {
            const id = item.id;
            return this.ajax.postJSON(`${this.baseUrl}/${id}/workflows`, wfEvent);
        }
        /**
         * Deletes the worklflow item or items from the item.
         * @param item - Item for which to delete workflow events.
         * @param wfIdOrType - Event id or type.
         * @returns A Promise with result data "Object deleted".
         */
        deleteWorkflowEvent(item, wfIdOrType) {
            const id = item.id;
            return this.ajax.remove(`${this.baseUrl}/${id}/workflows/${wfIdOrType}`);
        }
        /**
         * Lists versions for the object.
         * @param id - Unique identifier for the object
         * @returns A Promise with result data.
         */
        getVersions(id) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}/versions`);
        }
        /**
         * Get the given version for the object.
         * @param id - Unique identifier for the object
         * @param versionId - Version identifier
         * @returns A Promise with result data.
         */
        getVersion(id, versionId) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}/versions/${versionId}`);
        }
    }

    /**
     * Methods for oauth2 communication.
     */
    class OAuth2 {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Token endpoint.
         *
         * @param msg - OAuth 2.0 authorization request.
         *
         * @returns A Promise with OAuthMessage.
         */
        token(msg) {
            return this.ajax.postJSON(this.ajax.getServerAddress() + "/oauth2/token", msg);
        }
        /**
         * Gets a federation message for the currently signed-in account as OpenID Connect UserInfo Response
         * (http://openid.net/specs/openid-connect-core-1_0.html#UserInfo).
         * This message can be used by trusted authentication providers (mainly Banks) to federate authentication and authorization information to Salaxy.
         * In such a flow this service end-point provides an example that could be used in testing.
         *
         * @returns A Promise with federation message {OpenIdUserInfoSupportedClaims}.
         */
        getFederationMessage() {
            return this.ajax.getJSON(this.ajax.getServerAddress() + "/oauth2/federationMessage");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    OAuth2.$inject = ["AjaxNg1"];

    /**
     * Base class for APIs that return only one item (not a list as in CRUD).
     */
    class SingleItemApiBase {
        /** Constructor creates a new CRUD api with given Ajax-implementation */
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Gets a single settings object.
         * @returns A Promise with settings object.
         */
        get() {
            return this.ajax.getJSON(this.baseUrl);
        }
        /**
         * Saves the settings to the storage.
         * @param settings - The settings that is saved.
         * @returns A Promise with settings object.
         */
        save(settings) {
            return this.ajax.postJSON(`${this.baseUrl}/`, settings);
        }
    }

    /**
     * Provides CRUD access for Absences of a Worker
     */
    class Absences extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/absences";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank(workerId) {
            return {
                employmentId: null,
                workerId,
                periods: [],
            };
        }
        /**
         * Gets all the Worker absences objects of specified year.
         * @returns A Promise with result data: The full absences objects.
         */
        getLatest() {
            return this.ajax.getJSON(`${this.baseUrl}/latest`);
        }
        /**
         * Gets the Absences for a specific employment relation.
         * @param employmentId Identifier for the Employment relation.
         * @returns A Promise with Absences.
         */
        getForEmployment(employmentId) {
            return this.ajax.getJSON(`${this.baseUrl}/employment/${employmentId}`);
        }
        /**
         * Gets the Absences for the given employment relations.
         * @param employmentIds Identifiers for the Employment relations.
         * @returns A Promise with result data array.
         */
        getForEmployments(employmentIds) {
            if (employmentIds == null || employmentIds.length === 0) {
                return Promise.resolve([]);
            }
            let method = `${this.baseUrl}/employment/set?`;
            employmentIds.forEach((x) => { method += "employmentIds=" + encodeURIComponent("" + x) + "&"; });
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Absences.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for Accounting Targets
     */
    class AccountingTargets extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/accounting-target";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                info: {
                    availableMethods: [],
                    target: {
                        avatar: {},
                    },
                    data: {},
                },
                currentAccount: {
                    availableMethods: [],
                    data: {},
                },
                ruleSet: {
                    accounts: [],
                    rows: [],
                    targetCoA: [],
                },
                workflowData: {
                    events: [],
                },
            };
        }
        /**
         * IN TEST ONLY: Regenerates the base data for the current account.
         * This method will be removed after development has ended.
         */
        testRecreateBaseData() {
            return this.ajax.postJSON(`${this.baseUrl}/regenerate`, {});
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    AccountingTargets.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own authorized accounts
     */
    class AuthorizedAccounts extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/accounts/authorizedAccount";
        }
        /**
         * Returns an empty avatar without any values.
         */
        getBlank() {
            return {};
        }
        /**
         * Lists all accounts on behalf of which this account has been authorized to act.
         *
         * @returns A Promise with result data (list of account objects).
         */
        /* eslint-disable-next-line */
        getAuthorizingAccounts() {
            const method = "/accounts/authorizingAccount";
            return this.ajax.getJSON(method);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    AuthorizedAccounts.$inject = ["AjaxNg1"];

    /** Methods for importing data into Salaxy system. */
    class ImportLogic {
        /**
         * Converts a tab-separated values text to raw JSON.
         * Used to converting data copy-pasted from Excel to JSON.
         * @param inputText The TSV string that should be imported.
         * @returns ImportResult object with the data and other realted information.
         */
        static tsvToJson(inputText) {
            const result = {
                ignoredLines: 0,
                data: [],
                headers: [],
                errors: [],
            };
            if (!inputText) {
                return result;
            }
            const inputLines = inputText.split(/\r\n|\n/);
            if (inputLines.length === 0) {
                return result;
            }
            result.headers = inputLines[0].split(/\t/);
            for (let i = 1; i < inputLines.length; i++) {
                const data = inputLines[i].split(/\t/);
                const row = {};
                let isEmptyLine = true;
                for (let j = 0; j < result.headers.length; j++) {
                    if (j < data.length) {
                        row[result.headers[j]] = data[j];
                        if (data[j]) {
                            isEmptyLine = false;
                        }
                    }
                }
                if (isEmptyLine) {
                    result.ignoredLines++;
                }
                else {
                    if (data.length !== result.headers.length) {
                        row.validation = {
                            isValid: false,
                            errors: [{
                                    key: null,
                                    type: exports.ApiValidationErrorType.General,
                                    code: "InvalidColAmount",
                                    msg: `PARSE ERROR: Row ${i} has ${data.length} columns whereas the header row has ${result.headers.length}.`,
                                }],
                        };
                    }
                    result.data.push(row);
                }
            }
            result.errors = result.data.filter((x) => x.validation && x.validation.errors).map((x) => x.validation.errors).reduce((all, x) => all.concat(x), []);
            return result;
        }
    }

    /**
     * Logic for table mappers to/from Salaxy core model.
     * TObject is a hierarchical Salaxy object and the TRow is a row in table/dataset.
     */
    class Mapper {
        /**
         * Creates a new mapper object for mapping objects between hierarchial and tabular format.
         * @param map Map containing selectors or paths for fields.
         * @param defaultObject Default object when mapping data from tabular data to hierarchial object. This is optional.
         */
        constructor(map, defaultObject = null) {
            this.map = map;
            this.defaultObject = defaultObject;
            /** Type for the mapper data row */
            this.rowType = null;
            /** Description for the mapper. */
            this.description = null;
            /** If true, this is a list item mapper. */
            this.isListItemMapper = false;
            /** Keys for the mapper */
            this._keys = null;
        }
        /** Returns the keys of the current map. */
        get keys() {
            if (!this._keys) {
                const result = [];
                for (const key in this.map) {
                    if (Objects.has(this.map, key)) {
                        const selector = this.map[key];
                        if (this.isString(selector)) {
                            //  path
                            result.push(key);
                        }
                        else if (this.isMapSelector(selector)) {
                            // MapSelector
                            result.push(key);
                        }
                        else if (this.isSubMap(selector)) {
                            const subMap = selector[Object.keys(selector)[0]];
                            const subMapper = new Mapper(subMap, null);
                            result.push(...subMapper.keys);
                        }
                        else if (this.isKeysArray(selector)) {
                            continue;
                        }
                    }
                }
                this._keys = result;
            }
            return this._keys;
        }
        /**
         * Maps from objects to data rows (typically export).
         * @param objects The objects to map to new rows.
         */
        fromObjects(objects) {
            const rows = [];
            if (!objects || !objects.length) {
                return rows;
            }
            for (const object of objects) {
                const row = {};
                let subRows = null;
                for (const key in this.map) {
                    if (Objects.has(this.map, key)) {
                        const selector = this.map[key];
                        const propKey = key;
                        if (this.isString(selector)) {
                            // path
                            row[propKey] = Objects.getProperty(object, selector);
                        }
                        else if (this.isMapSelector(selector)) {
                            // MapSelector
                            if (selector.fromObject) {
                                selector.fromObject(object, row);
                            }
                            else {
                                // selector path
                                row[propKey] = Objects.getProperty(object, selector.path);
                            }
                        }
                        else if (this.isSubMap(selector)) {
                            const items = Objects.getProperty(object, Object.keys(selector)[0]) || [];
                            const subMap = selector[Object.keys(selector)[0]];
                            const subMapper = new Mapper(subMap, null);
                            subRows = subMapper.fromObjects(items);
                            if (!subRows.length) {
                                for (const subKey in subMap) {
                                    if (Objects.has(subMap, subKey)) {
                                        const subPropKey = subKey;
                                        row[subPropKey] = null;
                                    }
                                }
                            }
                        }
                        else if (this.isKeysArray(selector)) {
                            continue;
                        }
                    }
                }
                if (subRows && subRows.length) {
                    for (const subRow of subRows) {
                        const newRow = Objects.copy(row);
                        for (const subKey in subRow) {
                            if (Objects.has(subRow, subKey)) {
                                const subPropKey = subKey;
                                newRow[subPropKey] = subRow[subPropKey];
                            }
                        }
                        rows.push(newRow);
                    }
                }
                else {
                    rows.push(row);
                }
            }
            return rows;
        }
        /**
         * Maps rows to objects (typically import)
         * @param rows The rows to map to new objects.
         */
        fromRows(rows) {
            const objects = [];
            if (!rows || !rows.length) {
                return objects;
            }
            let rowGroups = null;
            const groupBy = (all, keys) => {
                const objectArray = all.reduce((groups, item) => {
                    const itemId = keys.map((k) => item[k]).join(".") || null;
                    let group = groups.find((x) => x.id === itemId);
                    if (!group) {
                        group = { id: itemId, rows: [] };
                        groups.push(group);
                    }
                    group.rows.push(item);
                    return groups;
                }, []);
                return objectArray.map((x) => x.rows);
            };
            const keysKey = Object.keys(this.map).find((x) => this.isKeysArray(this.map[x]));
            const keys = keysKey ? this.map[keysKey] : null;
            if (keys) {
                rowGroups = groupBy(rows, keys);
            }
            else {
                rowGroups = rows.map((x) => [x]);
            }
            for (const rowGroup of rowGroups) {
                const object = this.defaultObject ? Objects.copy(this.defaultObject) : {};
                objects.push(object);
                const row = rowGroup[0];
                for (const key in this.map) {
                    if (Objects.has(this.map, key)) {
                        const propKey = key;
                        const selector = this.map[key];
                        if (this.isString(selector)) {
                            //  path
                            Objects.setProperty(object, selector, row[propKey]);
                        }
                        else if (this.isMapSelector(selector)) {
                            // MapSelector
                            if (selector.fromRow) {
                                selector.fromRow(row, object);
                            }
                            else {
                                // selector path
                                Objects.setProperty(object, selector.path, row[propKey]);
                            }
                        }
                        else if (this.isSubMap(selector)) {
                            const subMap = selector[Object.keys(selector)[0]];
                            const subMapper = new Mapper(subMap, null);
                            const items = subMapper.fromRows(rowGroup);
                            Objects.setProperty(object, Object.keys(selector)[0], items);
                        }
                        else if (this.isKeysArray(selector)) {
                            continue;
                        }
                    }
                }
            }
            return objects;
        }
        isString(selector) {
            return typeof (selector) === "string";
        }
        isKeysArray(selector) {
            return Array.isArray(selector);
        }
        isMapSelector(selector) {
            return typeof (selector) === "object" && (!!(selector || {}).fromObject ||
                !!(selector || {}).fromRow ||
                !!(selector || {}).path);
        }
        isSubMap(selector) {
            return typeof (selector) === "object" &&
                !this.isMapSelector(selector) &&
                Object.keys(selector).length === 1 &&
                typeof (selector[Object.keys(selector)[0]]) === "object";
        }
    }

    /** Utility class for creating mappers */
    class Mappers {
        /** Returns a new path only mapper. Field names are built from the paths (separated by dots or slashes) using underscores between the fields.  */
        static pathMapper(paths, defaultObject = null) {
            const map = {};
            paths.forEach((path) => {
                const dotPath = this.getDotPath(path);
                map[this.getFriendlyFieldName(dotPath)] = dotPath;
            });
            return new Mapper(map, defaultObject);
        }
        static getDotPath(path) {
            return path.replace(/\//g, ".");
        }
        static getFriendlyFieldName(path) {
            const fields = path.split(".");
            const field = fields[fields.length - 1];
            return field;
        }
    }

    /** File format for a table. */
    exports.TableFormat = void 0;
    (function (TableFormat) {
        /** Tab separated table. */
        TableFormat["Txt"] = "txt";
        /** Comma separated table. */
        TableFormat["Csv"] = "csv";
        /** Excel formatted table. */
        TableFormat["Xlsx"] = "xlsx";
    })(exports.TableFormat || (exports.TableFormat = {}));

    /** Utility class for processing tables. Eg. exporting and importing table formatted data. */
    class Tables {
        /**
         * Exports table data as in defined format. The default is csv formatted table.
         * @param rows  Rows as an array of objects with properties specified as headers.
         * @param format File format for a table export.
         * @param localeId Locale for data export. Defaults to Finnish (fi). Affects  separator (; vs ,), formatting of date (D.M.YYYY vs YYYY-MM-DD) and numeric fields (, vs . as decimal separator).
         */
        static export(rows, format = exports.TableFormat.Csv, localeId = "fi") {
            if (!rows || rows.length === 0) {
                return null;
            }
            const data = this.exportAsArrays(rows);
            switch (format) {
                case exports.TableFormat.Csv:
                    return this.objectToSeparated(data, localeId, (localeId === "fi" ? ";" : ","));
                case exports.TableFormat.Txt:
                    return this.objectToSeparated(data, localeId, "\t");
                default:
                    throw new Error(`Table format ${format} not supported.`);
            }
        }
        /**
         * Exports data as an array of rows, each of which consists of an array of cell values.
         * @param rows Array of row objects.
         */
        static exportAsArrays(rows) {
            const data = [];
            const headers = Object.getOwnPropertyNames(rows[0]).filter((x) => x !== "validation");
            data.push(headers);
            for (const row of rows) {
                const rowData = [];
                for (const header of headers) {
                    rowData.push(row[header]);
                }
                data.push(rowData);
            }
            return data;
        }
        /**
         * Imports table from the given data and format.
         * @param data Table data.
         * @param format Format for the table data.
         * @param localeId Locale for data import. Defaults to Finnish (fi). Affects formatting of date (D.M.YYYY vs YYYY-MM-DD) and numeric fields (, vs . as decimal separator).
         */
        static import(data, format = exports.TableFormat.Csv, localeId = "fi") {
            switch (format) {
                case exports.TableFormat.Csv:
                    return this.objectFromSeparated(data, localeId, (localeId === "fi" ? ";" : ","));
                case exports.TableFormat.Txt:
                    return this.objectFromSeparated(data, localeId, "\t");
                default:
                    throw new Error(`Table format ${format} not supported.`);
            }
        }
        static objectToSeparated(data, localeId, separator) {
            const toRow = this.toRow(localeId, separator);
            const rows = data.map(toRow);
            return (separator !== "\t" ? "\uFEFF" : "") + rows.join("\r\n");
        }
        static toRow(localeId, separator, addQuotes = true) {
            const toField = this.toField(localeId, addQuotes);
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return (row, index, array) => {
                return row.map(toField).join(separator);
            };
        }
        static toField(localeId, addQuotes = true) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return (value, index, array) => {
                if (value == null) {
                    return "";
                }
                if (typeof (value) === "number") {
                    if (localeId === "fi") {
                        return value.toString().replace(".", ",");
                    }
                    return value;
                }
                if (typeof (value) === "boolean") {
                    return (value ? "TRUE" : "FALSE");
                }
                if (typeof (value) === "string") {
                    // try read date, always in ISO
                    if (Dates.isValidDateTime(value, true, "YYYY-MM-DD")) {
                        if (localeId === "fi") {
                            return Dates.format(value, "D.M.YYYY");
                        }
                        else {
                            return Dates.format(value, "YYYY-MM-DD");
                        }
                    }
                    if (addQuotes) {
                        return '"' + value.replace(/"/g, '""') + '"';
                    }
                    return value;
                }
                if (typeof (value) === "object") {
                    if (value instanceof Date) {
                        if (localeId === "fi") {
                            return Dates.format(value, "D.M.YYYY");
                        }
                        else {
                            return Dates.format(value, "YYYY-MM-DD");
                        }
                    }
                    else {
                        if (addQuotes) {
                            return '"' + JSON.stringify(value).replace(/"/g, '""') + '"';
                        }
                        return JSON.stringify(value);
                    }
                }
                return JSON.stringify(value);
            };
        }
        static objectFromSeparated(data, localeId, separator) {
            const result = {
                ignoredLines: 0,
                data: [],
                headers: [],
                errors: [],
            };
            if (!data) {
                return result;
            }
            const rows = data.split(/\r\n|\n/);
            if (rows.length === 0) {
                return result;
            }
            const headerRow = rows[0].split(separator);
            result.headers = headerRow.map(this.fromHeaderField());
            if (result.headers.some((x) => !x)) {
                result.errors.push({
                    key: null,
                    type: exports.ApiValidationErrorType.General,
                    code: "InvalidHeader",
                    msg: `Some header field contains no value. This is not allowed.`,
                });
                return result;
            }
            if (rows.length === 1) {
                return result;
            }
            const dataRows = rows.slice(1);
            const fromRow = this.fromRow(result, localeId, separator);
            const rawData = dataRows.map(fromRow);
            result.data = rawData.filter((x) => x != null);
            result.ignoredLines = rawData.filter((x) => x === null).length;
            result.data.filter((x) => x.validation && x.validation.errors).forEach((x) => {
                result.errors.push(...x.validation.errors);
            });
            return result;
        }
        static fromRow(result, localeId, separator) {
            const fromField = this.fromField(localeId);
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return (row, index, array) => {
                const rowData = row.split(separator);
                let rowObject = {};
                if (rowData.length !== result.headers.length) {
                    if (rowData.length === 1 && rowData[0] === "") {
                        rowObject = null;
                    }
                    else {
                        rowObject.validation = rowObject.validation || { isValid: false, errors: [] };
                        rowObject.validation.errors.push({
                            key: null,
                            type: exports.ApiValidationErrorType.General,
                            code: "InvalidColAmount",
                            msg: `PARSE ERROR: Row ${index} has ${rowData.length} columns whereas the header row has ${result.headers.length}.`,
                        });
                    }
                }
                else {
                    const typedRowData = rowData.map(fromField);
                    if (typedRowData.every((x) => !x)) {
                        rowObject = null;
                    }
                    else {
                        for (let i = 0; i < result.headers.length; i++) {
                            rowObject[result.headers[i]] = typedRowData[i];
                        }
                    }
                }
                return rowObject;
            };
        }
        static fromHeaderField() {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return (value, index, array) => {
                if (!value) {
                    return value;
                }
                // trim
                value = value.trim();
                // remove double quotes
                value = value.replace(/""/g, '"');
                // remove leading and trailing quotes
                value = value.replace(/^"+/, "");
                value = value.replace(/"+$/, "");
                return value;
            };
        }
        static fromField(localeId) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return (value, index, array) => {
                if (!value) {
                    return null;
                }
                // trim
                value = value.trim();
                // remove double quotes
                value = value.replace(/""/g, '"');
                // remove leading and trailing quotes
                value = value.replace(/^"+/, "");
                value = value.replace(/"+$/, "");
                // try booleans
                if (value.toLowerCase() === "true") {
                    return true;
                }
                if (value.toLowerCase() === "false") {
                    return false;
                }
                // try date
                const dateFormat = localeId === "fi" ? "D.M.YYYY" : "YYYY-MM-DD";
                if (Dates.isValidDateTime(value, true, dateFormat)) {
                    return Dates.parseDate(value, dateFormat);
                }
                // try number
                const sNumber = localeId === "fi" ? value.replace(",", ".") : value;
                if (Numeric.isNumber(sNumber)) {
                    return Numeric.parseNumber(sNumber);
                }
                // let it be string
                return value;
            };
        }
    }

    /**
     * Logic for the batch import of data from sequence files.
     * TObject is a hierarchical Salaxy object and the TRow is a row in table/dataset.
     *
     */
    class ImportBatch {
        /**
         * Creates a new batch for the data import.
         * @param storage Storage for storing data for the batch.
         * @param batchId Id for the batch. If null, a new batch id will be generated.
         * @param mapper Mapper object for table to objects transformations.
         * @param api api object for validting and posting data to API.
         */
        constructor(storage, batchId, mapper, api) {
            this.storage = storage;
            this.mapper = mapper;
            this.api = api;
            this.id = batchId;
        }
        /** Chages the mapper for the batch. */
        changeMapper(mapper) {
            this.mapper = mapper;
        }
        /**
         * Reloads the data from the storage for existing batch.
         */
        open() {
            if (this.storage && this.id) {
                const batches = JSON.parse(this.storage.getItem(ImportBatch.BATCH_INFO_KEY) || "[]");
                const batchInfo = batches.find((x) => x.id === this.id);
                if (batchInfo) {
                    this.name = batchInfo.name;
                    this.createdAt = batchInfo.createdAt;
                    this.updatedAt = batchInfo.updatedAt;
                    this.phase = batchInfo.phase;
                    this.status = batchInfo.status;
                    this.rawData = JSON.parse(this.storage.getItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-raw`) || "null");
                    this.unmappedData = JSON.parse(this.storage.getItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-unmapped`) || "null");
                    this.data = JSON.parse(this.storage.getItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-data`) || "null");
                }
            }
            return Promise.resolve(this);
        }
        /** Saves the current batch data to storage. */
        save(phase = null, status = null) {
            const timeStamp = Dates.asMoment(new Date()).toISOString(false);
            this.id = this.id || this.newGuid();
            this.phase = phase || this.phase || "new";
            this.status = status || this.status || "info";
            this.createdAt = this.createdAt || timeStamp;
            this.updatedAt = timeStamp;
            if (this.storage) {
                const batches = JSON.parse(this.storage.getItem(ImportBatch.BATCH_INFO_KEY) || "[]");
                let batchInfo = batches.find((x) => x.id === this.id);
                if (!batchInfo) {
                    batchInfo = {
                        id: this.id,
                    };
                    batches.push(batchInfo);
                }
                batchInfo.name = this.name;
                batchInfo.createdAt = this.createdAt;
                batchInfo.updatedAt = this.updatedAt;
                batchInfo.phase = this.phase;
                batchInfo.status = this.status;
                this.storage.setItem(ImportBatch.BATCH_INFO_KEY, JSON.stringify(batches));
                this.storage.setItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-raw`, JSON.stringify(this.rawData || null));
                this.storage.setItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-unmapped`, JSON.stringify(this.unmappedData || null));
                this.storage.setItem(`${ImportBatch.BATCH_KEY_PREFIX}-${this.id}-data`, JSON.stringify(this.data || null));
            }
            return Promise.resolve(this);
        }
        /**
         * Read data into batch from the file content.
         * @param rawData File content, e.g. csv or tab separated data.
         * @param tableFormat Table format (csv or tab separated).
         * @param localeId If 'fi', the number and date formats are parsed using Finnish formats.
         */
        read(rawData = null, tableFormat = exports.TableFormat.Csv, localeId = "fi") {
            this.rawData = rawData || this.rawData;
            this.unmappedData = Tables.import(this.rawData, tableFormat, localeId);
            this.sourceTargetMap = this.getSourceTargetMap(this.unmappedData.data);
            const status = this.unmappedData.errors.length ? "error" :
                !this.unmappedData.data.length || this.unmappedData.ignoredLines ? "warning" : "success";
            return this.save("read", status);
        }
        /**
         * Map the read data to the supported tabular format.
         * @param sourceTargetMap Object containing source headers as keys with target values.
         */
        map(sourceTargetMap = null) {
            if (!this.mapper) {
                return Promise.resolve(this);
            }
            this.sourceTargetMap = sourceTargetMap || this.sourceTargetMap;
            const targetSourceMap = {};
            for (const key of this.mapper.keys) {
                const sourceProperty = Object.getOwnPropertyNames(this.sourceTargetMap).find((p) => this.sourceTargetMap[p] === key);
                if (sourceProperty) {
                    targetSourceMap[key] = sourceProperty;
                }
            }
            const targetProperties = Object.getOwnPropertyNames(targetSourceMap);
            this.data = this.unmappedData.data.map((unmappedItem) => {
                const dataItem = {};
                for (const targetProperty of targetProperties) {
                    const prop = targetProperty;
                    dataItem[prop] = unmappedItem[targetSourceMap[targetProperty]];
                }
                return dataItem;
            });
            return this.save("mapped", "success");
        }
        /** Returns true if the import data is valid */
        get isDataValid() {
            return this.data && this.data.every((x) => x.validation && x.validation.isValid);
        }
        /** Returns true if the import data contains validation errors */
        get hasDataValidationErrors() {
            return this.data && this.data.some((x) => x.validation && x.validation.errors && !!x.validation.errors.length);
        }
        /** Clears validation from the rows */
        clearValidation() {
            for (const row of this.data) {
                row.validation = null;
            }
        }
        /**
         * Validates the batch data.
         * @param notify Function for notifying progress to caller.
         */
        validate(notify = null) {
            const tryValidate = (obj) => {
                try {
                    return this.api.validate(obj).then((apiResult) => {
                        return this.api.getValidationResult(apiResult);
                    });
                }
                catch (error) {
                    return Promise.resolve({
                        isValid: false,
                        errors: [{
                                key: null,
                                type: exports.ApiValidationErrorType.General,
                                code: "Server error",
                                msg: JSON.stringify(error),
                            }],
                    });
                }
            };
            this.clearValidation();
            let status = "success";
            const objects = this.mapper.fromRows(this.data);
            let index = 0;
            const next = () => {
                if (index === objects.length) {
                    return Promise.resolve();
                }
                const obj = objects[index];
                return tryValidate(obj)
                    .then((validationResult) => {
                    if (notify) {
                        notify("validate", index, objects.length, validationResult);
                    }
                    this.updateValidation(index, objects, validationResult);
                    if (!validationResult.isValid) {
                        status = "error";
                    }
                    index++;
                    return next();
                });
            };
            return next().then(() => {
                return this.save("validated", status);
            });
        }
        /**
         * Imports the batch data.
         * @param notify Function for notifying progress to caller.
         */
        import(notify = null) {
            const trySave = (obj) => {
                try {
                    return this.api.save(obj).then((apiResult) => {
                        return { object: apiResult, validation: { isValid: true } };
                    });
                }
                catch (error) {
                    return Promise.resolve({
                        object: obj,
                        validation: {
                            isValid: false,
                            errors: [{
                                    key: null,
                                    type: exports.ApiValidationErrorType.General,
                                    code: "Server error",
                                    msg: JSON.stringify(error),
                                }],
                        },
                    });
                }
            };
            this.clearValidation();
            let status = "success";
            const objects = this.mapper.fromRows(this.data);
            let index = 0;
            const next = () => {
                if (index === objects.length) {
                    return Promise.resolve();
                }
                const obj = objects[index];
                return trySave(obj)
                    .then((tryResult) => {
                    if (notify) {
                        notify("import", index, objects.length, tryResult.validation);
                    }
                    if (tryResult.validation.isValid) {
                        this.updateSave(index, objects, tryResult.object);
                    }
                    this.updateValidation(index, objects, tryResult.validation);
                    if (!tryResult.validation.isValid) {
                        status = "error";
                    }
                    index++;
                    return next();
                });
            };
            return next().then(() => {
                return this.save("imported", status);
            });
        }
        newGuid() {
            let dt = new Date().getTime();
            const uuid = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, (c) => {
                /* eslint-disable-next-line no-bitwise */
                const r = (dt + Math.random() * 16) % 16 | 0;
                dt = Math.floor(dt / 16);
                /* eslint-disable-next-line no-bitwise */
                return (c === "x" ? r : (r & 0x3 | 0x8)).toString(16);
            });
            return uuid;
        }
        updateValidation(objectIndex, objects, validation) {
            const objectRowCount = this.mapper.fromObjects([objects[objectIndex]]).length;
            const previousObjectsCount = this.mapper.fromObjects(objects.slice(0, objectIndex)).length;
            for (let i = previousObjectsCount; i < previousObjectsCount + objectRowCount; i++) {
                this.data[i].validation = validation;
            }
        }
        updateSave(objectIndex, objects, object) {
            objects[objectIndex] = object;
        }
        /**
         * Returns source target map from sample data.
         * @param unmappedRows Rows to map
         */
        getSourceTargetMap(unmappedRows) {
            const sourceTargetMap = {};
            const sampleRow = unmappedRows && unmappedRows.length ? unmappedRows[0] : {};
            const sourceProperties = Object.getOwnPropertyNames(sampleRow);
            for (const sourceProperty of sourceProperties) {
                const targetKey = this.mapper ? this.mapper.keys.find((k) => k === sourceProperty) : null;
                if (targetKey) {
                    sourceTargetMap[sourceProperty] = targetKey;
                }
                else {
                    sourceTargetMap[sourceProperty] = null;
                }
            }
            return sourceTargetMap;
        }
    }
    /** Storage key for  summary object of all batches. */
    ImportBatch.BATCH_INFO_KEY = "salaxy-batch-info";
    /** Storage key prefix for a single batch. */
    ImportBatch.BATCH_KEY_PREFIX = "salaxy-batch";

    /** Utility class for listing and deleting batch imports */
    class ImportBatches {
        /**
         * Returns all batches from the storage.
         * @param storage Storage for storing data for the batch.
         */
        static getAll(storage) {
            const batches = JSON.parse((storage.getItem(ImportBatch.BATCH_INFO_KEY) || "[]"));
            return Promise.resolve(batches);
        }
        /**
         * Deletes one batch from the storage.
         * @param storage Storage for storing data for the batch.
         * @param batchId Id for the batch to remove.
         */
        static delete(storage, batchId) {
            if (!storage || !batchId) {
                return Promise.resolve();
            }
            return this.getAll(storage).then((all) => {
                if (all.some((x) => x.id === batchId)) {
                    const batches = all.filter((x) => x.id !== batchId);
                    storage.setItem(ImportBatch.BATCH_INFO_KEY, JSON.stringify(batches));
                    storage.removeItem(`${ImportBatch.BATCH_KEY_PREFIX}-${batchId}-raw`);
                    storage.removeItem(`${ImportBatch.BATCH_KEY_PREFIX}-${batchId}-unmapped`);
                    storage.removeItem(`${ImportBatch.BATCH_KEY_PREFIX}-${batchId}-data`);
                }
            });
        }
        /**
         * Opens an existing batch or creates a new batch.
         * @param storage Storage for storing data for the batch.
         * @param batchId Id for the batch. If null, a new batch id will be generated.
         * @param mapper Mapper object for table to objects transformations.
         * @param api api object for validting and posting data to API.
         */
        static open(storage, batchId, mapper, api) {
            const batch = new ImportBatch(storage, batchId, mapper, api);
            return batch.open();
        }
    }

    /**
     * Primary categorization of Calculation row types.
     */
    exports.CalculationRowCategory = void 0;
    (function (CalculationRowCategory) {
        /** Base salary: hourly, monthly or compensation. */
        CalculationRowCategory["Salary"] = "salary";
        /** Additions to base salary: overtime, evenings, saturdays etc. */
        CalculationRowCategory["SalaryAdditions"] = "salaryAdditions";
        /** Advanced salary calcualtions: Net total or gross payment total. */
        CalculationRowCategory["SalaryCalc"] = "salaryCalc";
        /** Holidays */
        CalculationRowCategory["Holidays"] = "holidays";
        /** Benefits: car, accomodotaion etc.  */
        CalculationRowCategory["Benefits"] = "benefits";
        /** Expenses, typically tax-free. */
        CalculationRowCategory["Expenses"] = "expenses";
        /** Deductions at different levels. */
        CalculationRowCategory["Deductions"] = "deductions";
        /** Other salary items. */
        CalculationRowCategory["Other"] = "other";
    })(exports.CalculationRowCategory || (exports.CalculationRowCategory = {}));

    /**
     * Calculation rows configuration: categories, UI elements etc.
     * The texts are stored in translations.
     */
    const calcRowsMetadata = [
        {
            order: 1,
            name: "salary",
            category: exports.CalculationRowCategory.Salary,
            color: "#4a92e9",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 2,
            name: "hourlySalary",
            category: exports.CalculationRowCategory.Salary,
            color: "#4a92e9",
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 3,
            name: "monthlySalary",
            category: exports.CalculationRowCategory.Salary,
            color: "#4a92e9",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 4,
            name: "compensation",
            category: exports.CalculationRowCategory.Salary,
            color: "#4a92e9",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 5,
            name: "totalWorkerPayment",
            category: exports.CalculationRowCategory.SalaryCalc,
            color: "#4a92e9",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 6,
            name: "totalEmployerPayment",
            category: exports.CalculationRowCategory.SalaryCalc,
            color: "#4a92e9",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 7,
            name: "board",
            category: exports.CalculationRowCategory.Salary,
            kind: { name: "BoardKind", values: null },
            color: "#4a92e9",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 9,
            name: "hourlySalaryWithWorkingTimeCompensation",
            category: exports.CalculationRowCategory.SalaryAdditions,
            kind: { name: "WorkingTimeCompensationKind", values: null },
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 10,
            name: "overtime",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            amount: {
                default: 0.5,
                unit: exports.CalculationRowUnit.Percent,
            },
            total: {},
            price: {},
        },
        {
            order: 11,
            name: "tesWorktimeShortening",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            favorite: 90,
            amount: {
                default: 0.077,
                unit: exports.CalculationRowUnit.Percent,
            },
            total: {},
            price: {},
        },
        {
            order: 12,
            name: "eveningAddition",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            amount: {
                unit: exports.CalculationRowUnit.Hours,
            },
            total: {},
            price: {},
        },
        {
            order: 13,
            name: "nightimeAddition",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            amount: {
                unit: exports.CalculationRowUnit.Hours,
            },
            total: {},
            price: {},
        },
        {
            order: 14,
            name: "saturdayAddition",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            amount: {
                default: 0.5,
                unit: exports.CalculationRowUnit.Percent,
            },
            total: {},
            price: {},
        },
        {
            order: 15,
            name: "sundayWork",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            favorite: 10,
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 19,
            name: "otherAdditions",
            category: exports.CalculationRowCategory.SalaryAdditions,
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 20,
            name: "remuneration",
            category: exports.CalculationRowCategory.SalaryAdditions,
            kind: { name: "RemunerationKind", values: null },
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 21,
            name: "otherCompensation",
            category: exports.CalculationRowCategory.SalaryAdditions,
            kind: { name: "OtherCompensationKind", values: null },
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 22,
            name: "workingTimeCompensation",
            category: exports.CalculationRowCategory.SalaryAdditions,
            kind: { name: "WorkingTimeCompensationKind", values: null },
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 23,
            name: "employmentTermination",
            category: exports.CalculationRowCategory.SalaryAdditions,
            kind: { name: "EmploymentTerminationKind", values: null },
            color: "#15535d",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 101,
            name: "accomodationBenefit",
            category: exports.CalculationRowCategory.Benefits,
            color: "#4fcdc4",
            amount: { default: 1 },
            total: {},
            price: {},
        },
        {
            order: 102,
            name: "mealBenefit",
            category: exports.CalculationRowCategory.Benefits,
            kind: { name: "MealBenefitKind", values: null },
            color: "#4fcdc4",
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 103,
            name: "phoneBenefit",
            category: exports.CalculationRowCategory.Benefits,
            color: "#4fcdc4",
            amount: { default: 1 },
            price: { default: 20 },
            total: {},
        },
        {
            order: 104,
            name: "carBenefit",
            category: exports.CalculationRowCategory.Benefits,
            kind: { name: "CarBenefitKind", values: null },
            color: "#4fcdc4",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: {},
            price: {},
        },
        {
            order: 199,
            name: "otherBenefit",
            category: exports.CalculationRowCategory.Benefits,
            color: "#4fcdc4",
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 201,
            name: "holidayCompensation",
            category: exports.CalculationRowCategory.Holidays,
            color: "#ffe66d",
            favorite: 80,
            amount: {
                default: 0.115,
                unit: exports.CalculationRowUnit.Percent,
            },
            total: {},
            price: {},
        },
        {
            order: 202,
            name: "holidayBonus",
            category: exports.CalculationRowCategory.Holidays,
            color: "#ffe66d",
            amount: {
                unit: exports.CalculationRowUnit.Percent,
            },
            total: {},
            price: {},
        },
        {
            order: 203,
            name: "holidaySalary",
            category: exports.CalculationRowCategory.Holidays,
            color: "#ffe66d",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: {},
            price: {},
        },
        {
            order: 301,
            name: "dailyAllowance",
            category: exports.CalculationRowCategory.Expenses,
            kind: { name: "DailyAllowanceKind", values: null },
            color: "#ffa900",
            favorite: 100,
            price: { default: 44 },
            amount: {},
            total: {},
        },
        {
            order: 302,
            name: "dailyAllowanceHalf",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            price: { default: 20 },
            amount: { unit: exports.CalculationRowUnit.Days },
            total: {},
        },
        {
            order: 303,
            name: "mealCompensation",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            price: { default: 11.00 },
            amount: { unit: exports.CalculationRowUnit.Days },
            total: {},
        },
        {
            order: 304,
            name: "milageOwnCar",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            favorite: 20,
            price: { default: 0.44 },
            amount: { unit: exports.CalculationRowUnit.Kilometers },
            total: {},
        },
        {
            order: 305,
            name: "toolCompensation",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            favorite: 60,
            amount: { unit: exports.CalculationRowUnit.Days },
            total: {},
            price: {},
        },
        {
            order: 306,
            name: "expenses",
            category: exports.CalculationRowCategory.Expenses,
            kind: { name: "ExpensesKind", values: null },
            color: "#ffa900",
            favorite: 70,
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 307,
            name: "milageDaily",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            favorite: 30,
            amount: { unit: exports.CalculationRowUnit.Days },
            total: {},
            price: {},
        },
        {
            order: 308,
            name: "milageOther",
            category: exports.CalculationRowCategory.Expenses,
            color: "#ffa900",
            amount: { unit: exports.CalculationRowUnit.Days },
            total: {},
            price: {},
        },
        {
            order: 401,
            name: "unionPayment",
            category: exports.CalculationRowCategory.Deductions,
            kind: { name: "UnionPaymentKind", values: null },
            color: "#ff6738",
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: { input: "hidden" },
        },
        {
            order: 402,
            name: "foreclosure",
            category: exports.CalculationRowCategory.Deductions,
            kind: { name: "ForeclosureKind", values: null },
            color: "#ff6738",
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: { input: "hidden" },
        },
        {
            order: 403,
            name: "advance",
            category: exports.CalculationRowCategory.Deductions,
            color: "#ff6738",
            favorite: 40,
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 405,
            name: "prepaidExpenses",
            category: exports.CalculationRowCategory.Deductions,
            color: "#ff6738",
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 406,
            name: "otherDeductions",
            category: exports.CalculationRowCategory.Deductions,
            color: "#ff6738",
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 901,
            name: "childCareSubsidy",
            category: exports.CalculationRowCategory.Deductions,
            color: "#4fcdc4",
            favorite: 50,
            amount: { input: "hidden" },
            total: { input: "hidden" },
            price: {},
        },
        {
            order: 1010,
            name: "nonProfitOrg",
            category: exports.CalculationRowCategory.Expenses,
            kind: { name: "NonProfitOrgKind", values: null },
            color: "#ffa900",
            amount: {},
            total: {},
            price: {},
        },
        {
            order: 1020,
            name: "subsidisedCommute",
            category: exports.CalculationRowCategory.Benefits,
            kind: { name: "SubsidisedCommuteKind", values: null },
            color: "#4fcdc4",
            amount: {
                default: 1,
                input: "hidden",
            },
            total: {},
            price: {},
        },
        {
            order: 9999,
            name: "irIncomeType",
            category: exports.CalculationRowCategory.Salary,
            kind: { name: "TransactionCode", values: null },
            color: "#4a92e9",
            amount: { default: 1 },
            total: {},
            price: {},
        },
    ];

    /** Categorization / grouping for CalculationRowType */
    class CalculationRowCategories {
    }
    /** Base salary types: the ones that you define directly */
    CalculationRowCategories.baseSalary = [
        exports.CalculationRowType.Salary,
        exports.CalculationRowType.HourlySalary,
        exports.CalculationRowType.MonthlySalary,
        exports.CalculationRowType.TotalWorkerPayment,
        exports.CalculationRowType.TotalEmployerPayment,
        exports.CalculationRowType.Compensation,
        // CalculationRowType.IrIncomeType,
        exports.CalculationRowType.Board,
    ];
    /** Additions to base salary - typically a percentage of base salary */
    CalculationRowCategories.salaryAdditions = [
        exports.CalculationRowType.Overtime,
        exports.CalculationRowType.TesWorktimeShortening,
        exports.CalculationRowType.EveningAddition,
        exports.CalculationRowType.NightimeAddition,
        exports.CalculationRowType.SaturdayAddition,
        exports.CalculationRowType.SundayWork,
        exports.CalculationRowType.HolidayCompensation,
        exports.CalculationRowType.HolidayBonus,
        exports.CalculationRowType.HolidaySalary,
        exports.CalculationRowType.OtherAdditions,
        // CalculationRowType.Remuneration,
        // CalculationRowType.OtherCompensation,
        // CalculationRowType.WorkingTimeCompensation,
        // CalculationRowType.EmploymentTermination,
    ];
    /** All salary types: baseSalary and salaryAdditions together */
    CalculationRowCategories.salary = CalculationRowCategories.baseSalary.concat(CalculationRowCategories.salaryAdditions);
    /** Benefits: Not paid as money but taxable income */
    CalculationRowCategories.benefits = [
        exports.CalculationRowType.AccomodationBenefit,
        exports.CalculationRowType.MealBenefit,
        exports.CalculationRowType.PhoneBenefit,
        exports.CalculationRowType.OtherBenefit,
    ];
    /** Expenses and other non-taxed compensations */
    CalculationRowCategories.expenses = [
        exports.CalculationRowType.Expenses,
        exports.CalculationRowType.MilageOwnCar,
        exports.CalculationRowType.DailyAllowance,
        exports.CalculationRowType.DailyAllowanceHalf,
        exports.CalculationRowType.MealCompensation,
        exports.CalculationRowType.ToolCompensation,
        exports.CalculationRowType.MilageDaily,
        exports.CalculationRowType.MilageOther,
        exports.CalculationRowType.ChildCareSubsidy,
    ];
    /** Deductions */
    CalculationRowCategories.deductions = [
        exports.CalculationRowType.Advance,
        exports.CalculationRowType.Foreclosure,
        exports.CalculationRowType.OtherDeductions,
        exports.CalculationRowType.PrepaidExpenses,
        exports.CalculationRowType.UnionPayment,
    ];
    /** Holidays */
    CalculationRowCategories.holidays = [
        exports.CalculationRowType.HolidayCompensation,
        exports.CalculationRowType.HolidayBonus,
        exports.CalculationRowType.HolidaySalary,
        // CalculationRowType.HolidayDays
    ];
    /**
     * Items in the section called "expenses". Besides expenses, contains also benefits.
     */
    CalculationRowCategories.expensesSection = CalculationRowCategories.expenses.concat(CalculationRowCategories.benefits, CalculationRowCategories.deductions);

    /**
     * Data that is sent to external dialog and received back from it.
     * Used in cross-site communication.
     */
    class ExternalDialogData {
    }

    /**
     * Enumerates the system roles:
     * The roles that are not returned from server as account roles but
     * rather as derived from properties of the current session.
     */
    exports.SystemRole = void 0;
    (function (SystemRole) {
        /** Session check is in progress: The role of the user is unknown. */
        SystemRole["Unknown"] = "unknown";
        /** User is anonymous: The session has been checked and there is no token, token has expired or the token is invalid. */
        SystemRole["Anon"] = "anon";
        /** User is authenticated either as Company or Person */
        SystemRole["Auth"] = "auth";
        /**
         * Initialization is in progress:
         * At the moment, this is the same as Session check period (SystemRole Unknown),
         * but there may later be other server-checks in the initialization.
         */
        SystemRole["Init"] = "init";
        /** User is in Worker role.  */
        SystemRole["Worker"] = "worker";
        /** User is in Employer role: Either Household or Company. */
        SystemRole["Employer"] = "employer";
        /** User is Household (Employer) */
        SystemRole["Household"] = "household";
        /** User is Company (Employer) */
        SystemRole["Company"] = "company";
        /** User is a Person: Employer and/or Worker (not Company) */
        SystemRole["Person"] = "person";
        /** Hidden always returns false. Used to hide items in navigation. */
        SystemRole["Hidden"] = "hidden";
        /** System is running with test data */
        SystemRole["Test"] = "test";
        /** System is runnin with production data */
        SystemRole["Prod"] = "prod";
        /** Finnish personal company ("Yksityinen elinkeinonharjoittaja"). */
        SystemRole["FiTm"] = "fiTm";
        /** Finnish Limited liability company ("Osakeyhtiö"). */
        SystemRole["FiOy"] = "fiOy";
        /** Finnish Association ("yhdistys") */
        SystemRole["FiRy"] = "fiRy";
        /**
         * Any user ("any" or "*") always returns true.
         * Note that in most scenarios also no roles ([], "" or null) are equivalent to Any.
         */
        SystemRole["Any"] = "any";
        /**
         * System role for account that is not verified.
         * Typically this means that the onboarding object should be shown e.g. before payment or on the fornt page.
         * The role is not set while account is not there (anonymous or session check in progress).
         */
        SystemRole["NonVerified"] = "nonVerified";
        /**
         * Roles which are coming from the primary partner.
         * Indicates that the partner messaging is enabled.
         */
        SystemRole["PartnerMessaging"] = "partnerMessaging";
        /**
         * Roles which are coming from the primary partner.
         * Indicates that the partner services are enabled.
         */
        SystemRole["PartnerService"] = "partnerService";
    })(exports.SystemRole || (exports.SystemRole = {}));

    /**
     * Provides metadata and other helpers for enumerations.
     */
    class EnumerationsLogic {
        /**
         * Gets a label text for an enumeration value.
         * @param enumType Name of the Type of the enumeration.
         * @param memberName Name of the member for which the label is fetched.
         * Also supports the value of the member if there is only difference in casing (which is the normal convention).
         * @param defaultValue The default value if the translation is not found.
         * Supports special keys "#name" and "name" to return the member name either with or without hash in the beginning.
         * Default is "#name", which returns "#[MemberName]" if the translation is not found.
         * @returns Language specific text form translations or the memberName text if no translation is found.
         */
        static getEnumLabel(enumType, memberName, defaultValue = "#name") {
            if (!enumType || !memberName) {
                return `#err ${enumType || "NoEnumType"}.${memberName || "NoEnumValue"}`;
            }
            enumType = this.upperCamelCase(enumType);
            memberName = this.lowerCamelCase(memberName);
            if (defaultValue === "name") {
                defaultValue = this.upperCamelCase(memberName);
            }
            else if (defaultValue === "#name") {
                defaultValue = this.upperCamelCase(memberName);
            }
            return Translations.getWithDefault(`SALAXY.ENUM.${enumType}.${memberName}.label`) || defaultValue;
        }
        /**
         * Gets a description text for an enumeration value in current language.
         * @param enumType - Name of the Type of the enumeration.
         * @param memberName Name of the member for which the description is fetched.
         * Also supports the value of the member if there is only difference in casing (which is the normal convention).
         * @param defaultValue The default value if the translation is not found. Default is null.
         * Supports special keys "#name" and "name" to return the member name either with or without hash in the beginning.
         * @returns Language specific text form translations or null if none is found.
         */
        static getEnumDescr(enumType, memberName, defaultValue) {
            if (!enumType || !memberName) {
                return `#err ${enumType || "NoEnumType"}.${memberName || "NoEnumValue"}`;
            }
            enumType = this.upperCamelCase(enumType);
            memberName = this.lowerCamelCase(memberName);
            if (defaultValue === "name") {
                defaultValue = this.upperCamelCase(memberName);
            }
            else if (defaultValue === "#name") {
                defaultValue = this.upperCamelCase(memberName);
            }
            return Translations.getWithDefault(`SALAXY.ENUM.${enumType}.${memberName}.description`) || defaultValue;
        }
        /**
         * Gets a very short user interface text for an enumeration.
         * Currently only suppotrted for Unit, but we expect more of these.
         * If the texts will be language-versioned, that logic will be here as well.
         * @param enumType - Name of the Type of the enumeration.
         * @param enumValue - Value for which the text is fetched.
         */
        static getEnumShortText(enumType, enumValue) {
            if (!enumType || !enumValue) {
                return `#err ${enumType || "NoEnumType"}.${enumValue || "NoEnumValue"}`;
            }
            enumType = this.upperCamelCase(enumType);
            enumValue = this.lowerCamelCase(enumValue);
            switch (enumType) {
                case "Unit": {
                    const unitTexts = {
                        undefined: "kpl?",
                        hour: "h",
                        day: "pv",
                        week: "vko",
                        period: "jaksoa",
                        one: "",
                        count: "kpl",
                        percent: "%",
                        kilometers: "km",
                    };
                    return unitTexts[enumValue];
                }
                default: {
                    throw new Error(`Enum type ${enumType} is not supported in getEnumShortText().`);
                }
            }
        }
        /**
         * Gets the metadata (texts etc.) for an enumeration value.
         * @param enumType - Name of the Type of the enumeration.
         * @param enumValue - Value for which the label is fetched.
         */
        static getEnumValueMetadata(enumType, enumValue) {
            const typeMetaData = EnumerationsLogic.getEnumMetadata(enumType);
            if (!enumValue || !typeMetaData) {
                return null;
            }
            enumValue = this.lowerCamelCase(enumValue);
            const enumValueObj = typeMetaData.values.filter((x) => x.name === enumValue)[0];
            return enumValueObj;
        }
        /**
         * Gets the metadata (texts etc.) for an enumeration and its values.
         * @param enumType - Name of the Type of the enumeration.
         */
        static getEnumMetadata(enumType) {
            if (!enumType) {
                return null;
            }
            enumType = this.upperCamelCase(enumType);
            let enumObj = generatedEnumData.filter((x) => x.name === enumType)[0];
            const override = EnumerationsLogic.enumDataOverride.filter((x) => x.name === enumType)[0];
            enumObj = override || enumObj;
            if (!enumObj) {
                return null;
            }
            const result = {
                name: enumType,
                descr: enumObj.description,
                values: enumObj.values.map((x) => {
                    return {
                        name: x.name,
                        order: x.order,
                        label: this.getEnumLabel(enumType, x.name),
                        descr: this.getEnumDescr(enumType, x.name),
                    };
                }),
            };
            return result;
        }
        /** Gets the currently known enumerations. */
        static getEnums() {
            return generatedEnumData.map((x) => x.name);
        }
        static lowerCamelCase(text) {
            return text.substr(0, 1).toLowerCase() + text.substr(1);
        }
        static upperCamelCase(text) {
            return text.substr(0, 1).toUpperCase() + text.substr(1);
        }
    }
    /**
     * You can override values of generatedEnumData without generating the entire set of enums from server.
     * TODO: These enums (if any) should be moved to C# code server-side and removed from here.
     */
    EnumerationsLogic.enumDataOverride = [{
            "name": "NewEnumName",
            "description": null,
            "values": [
                { "order": 0, "name": "undefined" },
                { "order": 1, "name": "enum1" },
            ],
        }, {
            "name": "SummaryTransactionCode",
            "description": "Income type code for Payment Summary Report",
            "values": [
                { "order": 0, "name": "undefined" },
                { "order": 101, "name": "noWagesPayable" },
                { "order": 102, "name": "employersHealthInsuranceContribution" },
                { "order": 103, "name": "deductionsToBeMadeFromTheEmployersHealthInsuranceContribution" },
            ],
        }, {
            "name": "ProfessionType",
            "description": "Profession type for Earnings Payment Report",
            "values": [
                { "order": 1, "name": "statisticsFinland" },
                { "order": 2, "name": "keva" },
                { "order": 3, "name": "bankOfFinland" },
                { "order": 4, "name": "trafi" },
            ],
        },
        {
            name: "EarningsPaymentReportStatus",
            description: "Status codes for Incomes registry Earnings Payment Reports",
            values: [
                { order: 0, name: "new" },
                { order: 1, name: "succeeded" },
                { order: 2, name: "canceled" },
                { order: 3, name: "error" },
                { order: 4, name: "invalid" },
                { order: 5, name: "scheduled" },
            ],
        },
        {
            name: "PayerSummaryReportStatus",
            description: "Status codes for Incomes registry Payment Summary Reports",
            values: [
                { order: 0, name: "new" },
                { order: 1, name: "succeeded" },
                { order: 2, name: "canceled" },
                { order: 3, name: "error" },
                { order: 4, name: "invalid" },
                { order: 5, name: "scheduled" },
            ],
        },
    ];

    /**
     * Provides business logic for Calculation rows:
     * How to create user interfaces, reports etc. based on them and their categories.
     */
    class CalcRowsLogic {
        /**
         * Creates a new CalcRowsLogic helper wichh adapts to role logic (company/household).
         * @param role Role for which the rows are fetched.
         */
        constructor(role = "company") {
            this.role = role;
        }
        /**
         * Gets a single UI configuration for Calculation row.
         * Searches through all row types without role filtering.
         * @param type Type for which the configuration is fetched.
         */
        static getRowConfig(type) {
            if (!type) {
                return null;
            }
            if (type == exports.CalculationRowType.Unknown) {
                return this.getUnknownConfig();
            }
            const result = CalcRowsLogic.getAllRowConfigs().find((x) => x.name === type);
            if (result == null) {
                console.error("Unknown row type: " + type);
            }
            return result;
        }
        /**
         * Gets an indicator string (1-2 characters) for a unit.
         * @param unit Unit that should be described.
         * @returns Given unit indicator or null if none is matched (convert to empty string if necessary).
         */
        static getUnitIndicator(unit) {
            // TODO: Consider another location
            // TODO: After views fixed, remove non-plural strings.
            switch (unit) {
                case exports.Unit.Euro:
                case "eur":
                case "euros":
                case "€":
                    return "€";
                case exports.Unit.Days:
                case exports.CalculationRowUnit.Days:
                case "day":
                    return "pv";
                case exports.Unit.Hours:
                case exports.CalculationRowUnit.Hours:
                case "hour":
                    return "h";
                case exports.Unit.Kilometers:
                case exports.CalculationRowUnit.Kilometers:
                case "km":
                    return "km";
                case exports.Unit.Percent:
                case exports.CalculationRowUnit.Percent:
                case "%":
                    return "%";
                case exports.Unit.Count:
                case exports.CalculationRowUnit.Count:
                    return "kpl";
                case exports.Unit.Weeks:
                case "week":
                    return "vko";
                case exports.Unit.Period:
                    return "jakso";
                case exports.CalculationRowUnit.Undefined:
                case exports.Unit.Undefined:
                case exports.Unit.One:
                default:
                    return null;
            }
        }
        /** Gets the row configuration for row type "unknown". */
        static getUnknownConfig() {
            if (!this.unknownRowConfig) {
                this.unknownRowConfig = this.getRowConfigInternal({
                    order: 0,
                    name: "unknown",
                    category: exports.CalculationRowCategory.Other,
                    color: "#ffffff",
                    amount: {
                        default: 1,
                        input: "hidden",
                    },
                    total: { input: "hidden" },
                    price: { input: "hidden" },
                });
            }
            return this.unknownRowConfig;
        }
        /**
         * Gets the row types by category.
         * The instance methods filter types based on role.
         * @param category Category to fetch
         */
        getRowTypesByCategory(category) {
            return CalcRowsLogic.rowsByCategory[category] || [];
        }
        /**
         * Gets the row configurations by category(ies).
         * The instance methods filter types based on role.
         * @param categories List of categories to filter by.
         */
        getRowConfigsByCategory(categories) {
            const types = [];
            for (const cat of categories || []) {
                types.push(...this.getRowTypesByCategory(cat));
            }
            return this.getRowConfigsByType(types);
        }
        /**
         * Gets the row configurations based on a list of row types.
         * The instance methods filter types based on role.
         * @param types List of types to filter by.
         */
        getRowConfigsByType(types) {
            types = types || [];
            return this.getRowConfigs().filter((x) => types.indexOf(x.name) >= 0);
        }
        /**
         * Gets a single UI configuration for Calculation row.
         * This method filters for user role. Use static method to return any row configuration (also faster).
         * The instance methods filter types based on role.
         * @param type Type for which the configuration is fetched.
         */
        getRowConfig(type) {
            if (!type) {
                return null;
            }
            if (type == exports.CalculationRowType.Unknown) {
                return CalcRowsLogic.getUnknownConfig();
            }
            const result = this.getRowConfigs().find((x) => x.name === type);
            if (result == null) {
                console.error("Unknown row type: " + type);
            }
            return result;
        }
        /**
         * Gets all the UI configuration objects for calculation rows.
         * Caches the data based on role and language.
         */
        getRowConfigs() {
            const cacheKey = this.role + Translations.getLanguage();
            if (!CalcRowsLogic.rowsCache[cacheKey]) {
                if (this.role === "household") {
                    CalcRowsLogic.rowsCache[cacheKey] = CalcRowsLogic.getAllRowConfigs().filter((x) => x.name !== exports.CalculationRowType.Compensation
                        && x.name !== exports.CalculationRowType.Board
                        && x.name !== exports.CalculationRowType.IrIncomeType
                        && x.name !== exports.CalculationRowType.OtherCompensation
                        && x.name !== exports.CalculationRowType.NonProfitOrg);
                }
                else {
                    CalcRowsLogic.rowsCache[cacheKey] = CalcRowsLogic.getAllRowConfigs().filter((x) => x.name !== exports.CalculationRowType.ChildCareSubsidy);
                }
            }
            return CalcRowsLogic.rowsCache[cacheKey];
        }
        /** Gets all the row configs cached for each language if language changes. */
        static getAllRowConfigs() {
            const lang = Translations.getLanguage();
            if (!CalcRowsLogic.rowsCache[lang]) {
                CalcRowsLogic.rowsCache[lang] = calcRowsMetadata.map((x) => CalcRowsLogic.getRowConfigInternal(x));
            }
            return CalcRowsLogic.rowsCache[lang];
        }
        /** Converts a raw excel generated row configuration to language-versioned JSON object. */
        static getRowConfigInternal(rowConfig) {
            var _a, _b;
            if (!rowConfig) {
                return null;
            }
            let kind = ((_a = rowConfig.kind) === null || _a === void 0 ? void 0 : _a.name) ? EnumerationsLogic.getEnumMetadata(rowConfig.kind.name) : null;
            if (((_b = kind === null || kind === void 0 ? void 0 : kind.values) === null || _b === void 0 ? void 0 : _b.length) < 2) {
                kind = null;
            }
            return {
                name: rowConfig.name,
                order: rowConfig.order,
                kind,
                label: Translations.get("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".label"),
                descr: Translations.get("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".description"),
                moreInfo: Translations.getWithDefault("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".moreInfo", null),
                category: rowConfig.category,
                favorite: rowConfig.favorite || 0,
                color: rowConfig.color || "#eee",
                iconText: Translations.getWithDefault("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".iconText", null) || "NN",
                price: {
                    label: Translations.getWithDefault("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".priceLabel", null) || "Hinta",
                    input: rowConfig.price.input || "number",
                    default: rowConfig.price.default || null,
                },
                amount: {
                    label: Translations.getWithDefault("SALAXY.ENUM.CalculationRowType." + rowConfig.name + ".amountLabel", null) || "Määrä",
                    input: rowConfig.amount.input || "number",
                    default: rowConfig.amount.default || null,
                    unit: rowConfig.amount.unit || null,
                },
                total: {
                    input: rowConfig.total.input || "readOnly",
                },
            };
        }
    }
    /** Raw classification of row types to primary categories. */
    CalcRowsLogic.rowsByCategory = {
        /** Base salary types: the ones that you define directly */
        salary: [
            exports.CalculationRowType.Salary,
            exports.CalculationRowType.HourlySalary,
            exports.CalculationRowType.MonthlySalary,
            exports.CalculationRowType.Compensation,
            exports.CalculationRowType.IrIncomeType,
            exports.CalculationRowType.Board,
        ],
        /** Additions to base salary - typically a percentage of base salary */
        salaryAdditions: [
            exports.CalculationRowType.Overtime,
            exports.CalculationRowType.TesWorktimeShortening,
            exports.CalculationRowType.EveningAddition,
            exports.CalculationRowType.NightimeAddition,
            exports.CalculationRowType.SaturdayAddition,
            exports.CalculationRowType.SundayWork,
            exports.CalculationRowType.OtherAdditions,
            exports.CalculationRowType.Remuneration,
            exports.CalculationRowType.OtherCompensation,
            exports.CalculationRowType.WorkingTimeCompensation,
            exports.CalculationRowType.EmploymentTermination,
            exports.CalculationRowType.HourlySalaryWithWorkingTimeCompensation,
        ],
        /** Benefits: Not paid as money but taxable income */
        benefits: [
            exports.CalculationRowType.MealBenefit,
            exports.CalculationRowType.PhoneBenefit,
            exports.CalculationRowType.SubsidisedCommute,
            exports.CalculationRowType.CarBenefit,
            exports.CalculationRowType.AccomodationBenefit,
            exports.CalculationRowType.OtherBenefit,
            exports.CalculationRowType.ChildCareSubsidy,
        ],
        /** Expenses and other non-taxed compensations */
        expenses: [
            exports.CalculationRowType.DailyAllowance,
            exports.CalculationRowType.Expenses,
            exports.CalculationRowType.MilageOwnCar,
            exports.CalculationRowType.MealCompensation,
            exports.CalculationRowType.ToolCompensation,
            exports.CalculationRowType.MilageDaily,
            exports.CalculationRowType.MilageOther,
            exports.CalculationRowType.NonProfitOrg,
            exports.CalculationRowType.DailyAllowanceHalf,
        ],
        /** Holidays */
        holidays: [
            exports.CalculationRowType.HolidayCompensation,
            exports.CalculationRowType.HolidayBonus,
            exports.CalculationRowType.HolidaySalary,
        ],
        /** Deductions */
        deductions: [
            exports.CalculationRowType.UnionPayment,
            exports.CalculationRowType.Advance,
            exports.CalculationRowType.Foreclosure,
            exports.CalculationRowType.OtherDeductions,
            exports.CalculationRowType.PrepaidExpenses,
        ],
        /** Types that are used only as temporary / input types in Calculations - not in the results. */
        salaryCalc: [
            exports.CalculationRowType.TotalWorkerPayment,
            exports.CalculationRowType.TotalEmployerPayment,
        ],
        /** Other row types */
        other: [
        // CalculationRowType.ChildCareSubsidy,
        ],
    };
    CalcRowsLogic.rowsCache = {};

    /** Extension methods for CalculationResult */
    class CalculationResultLogic {
        /**
         * Adds Calculation Results together and returns a new CalculationResult
         * @param calculationResults CalculationResults to add.
         * @returns A new CalculationResult.
         */
        static add(calculationResults) {
            if (calculationResults == null || calculationResults.length === 0) {
                return this.addResults(null, null);
            }
            return calculationResults.reduce((sum, item) => this.addResults(sum, item), this.safeObj(null));
        }
        static addResults(c1, c2) {
            c1 = this.safeObj(c1);
            c2 = this.safeObj(c2);
            return {
                totals: this.addTotals(c1.totals, c2.totals),
                workerCalc: this.addWorkerCalc(c1.workerCalc, c2.workerCalc),
                employerCalc: this.addEmployerCalc(c1.employerCalc, c2.employerCalc),
                rows: this.safeArr(c1.rows).concat(this.safeArr(c2.rows)),
                responsibilities: this.safeObj(c1.responsibilities),
                irRows: this.safeArr(c1.irRows).concat(this.safeArr(c2.irRows)),
            };
        }
        static addTotals(c1, c2) {
            c1 = this.safeObj(c1);
            c2 = this.safeObj(c2);
            return {
                pension: this.safeNum(c1.pension) + this.safeNum(c2.pension),
                total: this.safeNum(c1.total) + this.safeNum(c2.total),
                totalBaseSalary: this.safeNum(c1.totalBaseSalary) + this.safeNum(c2.totalBaseSalary),
                totalExpenses: this.safeNum(c1.totalExpenses) + this.safeNum(c2.totalExpenses),
                totalGrossSalary: this.safeNum(c1.totalGrossSalary) + this.safeNum(c2.totalGrossSalary),
                totalSocialSecurityBase: this.safeNum(c1.totalSocialSecurityBase) + this.safeNum(c2.totalSocialSecurityBase),
                totalTaxable: this.safeNum(c1.totalTaxable) + this.safeNum(c2.totalTaxable),
                unemployment: this.safeNum(c1.unemployment) + this.safeNum(c2.unemployment),
                totalAccidentInsuranceBase: this.safeNum(c1.totalAccidentInsuranceBase) + this.safeNum(c2.totalAccidentInsuranceBase),
                totalHealthInsuranceBase: this.safeNum(c1.totalHealthInsuranceBase) + this.safeNum(c2.totalHealthInsuranceBase),
                totalPayable: this.safeNum(c1.totalPayable) + this.safeNum(c2.totalPayable),
                totalPensionInsuranceBase: this.safeNum(c1.totalPensionInsuranceBase) + this.safeNum(c2.totalPensionInsuranceBase),
                totalUnemploymentInsuranceBase: this.safeNum(c1.totalUnemploymentInsuranceBase) + this.safeNum(c2.totalUnemploymentInsuranceBase),
            };
        }
        static addWorkerCalc(c1, c2) {
            c1 = this.safeObj(c1);
            c2 = this.safeObj(c2);
            return {
                benefits: this.safeNum(c1.benefits) + this.safeNum(c2.benefits),
                deductions: this.safeNum(c1.deductions) + this.safeNum(c2.deductions),
                fullTax: this.safeNum(c1.fullTax) + this.safeNum(c2.fullTax),
                fullOtherDeductions: this.safeNum(c1.fullOtherDeductions) + this.safeNum(c2.fullOtherDeductions),
                fullPension: this.safeNum(c1.fullPension) + this.safeNum(c2.fullPension),
                fullPrepaidExpenses: this.safeNum(c1.fullPrepaidExpenses) + this.safeNum(c2.fullPrepaidExpenses),
                fullSalaryAdvance: this.safeNum(c1.fullSalaryAdvance) + this.safeNum(c2.fullSalaryAdvance),
                fullUnemploymentInsurance: this.safeNum(c1.fullUnemploymentInsurance) + this.safeNum(c2.fullUnemploymentInsurance),
                fullUnionPayment: this.safeNum(c1.fullUnionPayment) + this.safeNum(c2.fullUnionPayment),
                otherDeductions: this.safeNum(c1.otherDeductions) + this.safeNum(c2.otherDeductions),
                pension: this.safeNum(c1.pension) + this.safeNum(c2.pension),
                prepaidExpenses: this.safeNum(c1.prepaidExpenses) + this.safeNum(c2.prepaidExpenses),
                foreclosure: this.safeNum(c1.foreclosure) + this.safeNum(c2.foreclosure),
                salaryAdvance: this.safeNum(c1.salaryAdvance) + this.safeNum(c2.salaryAdvance),
                salaryAfterTax: this.safeNum(c1.salaryAfterTax) + this.safeNum(c2.salaryAfterTax),
                salaryAfterTaxAndForeclosure: this.safeNum(c1.salaryAfterTaxAndForeclosure) + this.safeNum(c2.salaryAfterTaxAndForeclosure),
                salaryPayment: this.safeNum(c1.salaryPayment) + this.safeNum(c2.salaryPayment),
                tax: this.safeNum(c1.tax) + this.safeNum(c2.tax),
                totalWorkerPayment: this.safeNum(c1.totalWorkerPayment) + this.safeNum(c2.totalWorkerPayment),
                unemploymentInsurance: this.safeNum(c1.unemploymentInsurance) + this.safeNum(c2.unemploymentInsurance),
                unionPayment: this.safeNum(c1.unionPayment) + this.safeNum(c2.unionPayment),
                workerSideCosts: this.safeNum(c1.workerSideCosts) + this.safeNum(c2.workerSideCosts),
                totalWorkerPaymentLegacy: this.safeNum(c1.totalWorkerPaymentLegacy) + this.safeNum(c2.totalWorkerPaymentLegacy),
            };
        }
        static addEmployerCalc(c1, c2) {
            c1 = this.safeObj(c1);
            c2 = this.safeObj(c2);
            const result = {
                allSideCosts: this.safeNum(c1.allSideCosts) + this.safeNum(c2.allSideCosts),
                deductionPensionSelfPayment: this.safeNum(c1.deductionPensionSelfPayment) + this.safeNum(c2.deductionPensionSelfPayment),
                deductionUnemploymentSelfPayment: this.safeNum(c1.deductionUnemploymentSelfPayment) + this.safeNum(c2.deductionUnemploymentSelfPayment),
                deductionSalaryAdvance: this.safeNum(c1.deductionSalaryAdvance) + this.safeNum(c2.deductionSalaryAdvance),
                deductionForeclosure: this.safeNum(c1.deductionForeclosure) + this.safeNum(c2.deductionForeclosure),
                foreclosureByPalkkaus: this.safeNum(c1.foreclosureByPalkkaus) + this.safeNum(c2.foreclosureByPalkkaus),
                deductionUnionPayment: this.safeNum(c1.deductionUnionPayment) + this.safeNum(c2.deductionUnionPayment),
                deductionOtherDeductions: this.safeNum(c1.deductionOtherDeductions) + this.safeNum(c2.deductionOtherDeductions),
                finalCost: this.safeNum(c1.finalCost) + this.safeNum(c2.finalCost),
                householdDeduction: this.safeNum(c1.householdDeduction) + this.safeNum(c2.householdDeduction),
                mandatorySideCosts: this.safeNum(c1.mandatorySideCosts) + this.safeNum(c2.mandatorySideCosts),
                palkkaus: this.safeNum(c1.palkkaus) + this.safeNum(c2.palkkaus),
                pension: this.safeNum(c1.pension) + this.safeNum(c2.pension),
                socialSecurity: this.safeNum(c1.socialSecurity) + this.safeNum(c2.socialSecurity),
                totalDeductions: this.safeNum(c1.totalDeductions) + this.safeNum(c2.totalDeductions),
                totalPayment: this.safeNum(c1.totalPayment) + this.safeNum(c2.totalPayment),
                totalSalaryCost: this.safeNum(c1.totalSalaryCost) + this.safeNum(c2.totalSalaryCost),
                unemployment: this.safeNum(c1.unemployment) + this.safeNum(c2.unemployment),
                totalPaymentLegacy: this.safeNum(c1.totalPaymentLegacy) + this.safeNum(c2.totalPaymentLegacy),
            };
            result.deductionTaxAndSocialSecuritySelfPayment = this.safeNum(c1.deductionTaxAndSocialSecuritySelfPayment) + this.safeNum(c2.deductionTaxAndSocialSecuritySelfPayment);
            result.deductionWorkerSelfPayment = this.safeNum(c1.deductionWorkerSelfPayment) + this.safeNum(c2.deductionWorkerSelfPayment);
            return result;
        }
        static safeNum(value) {
            return value || 0;
        }
        static safeArr(value) {
            return value || [];
        }
        static safeObj(value) {
            return value || {};
        }
    }

    /**
     * Provides business logic related to building a salary calculator.
     */
    class CalculatorLogic {
        /**
         * Gets a UI cache object that is used for storing UI properties
         * to the calculation until a roundtrip to server (technically calc.$ui).
         * When a round-trip to server occurs (typically recalculate or save),
         * then the property is set to null => empty object.
         */
        static getUiCache(calc) {
            if (!calc) {
                return null;
            }
            calc.$ui = calc.$ui || {};
            return calc.$ui;
        }
        /**
         * Gets the first row of the calculation assuring that it exists
         * AND that it is a known Salary row or Unknown.
         * This should be the main row of the calculation that replaces the Salary object.
         * @param calc Calculation from which the row is fetched.
         * @param defaultType Default type for the new item if new item needs to be added.
         */
        static getSalaryRow(calc, defaultType = exports.CalculationRowType.Unknown) {
            if (!calc) {
                return null;
            }
            const allowedTypes = [
                exports.CalculationRowType.Unknown,
                exports.CalculationRowType.Compensation,
                exports.CalculationRowType.Salary,
                exports.CalculationRowType.HourlySalary,
                exports.CalculationRowType.MonthlySalary,
                exports.CalculationRowType.TotalEmployerPayment,
                exports.CalculationRowType.TotalWorkerPayment,
            ];
            this.moveSalaryToRows(calc);
            if (!calc.rows.find((x) => allowedTypes.indexOf(x.rowType) >= 0)) {
                calc.rows.unshift({
                    rowType: defaultType,
                });
            }
            return calc.rows.find((x) => allowedTypes.indexOf(x.rowType) >= 0);
        }
        /**
         * Gets the first row of the calculation assuring that it exists.
         * This should be the main row of the calculation that replaces the Salary object.
         * @param calc Calculation from which the row is fetched.
         */
        static getFirstRow(calc) {
            // TODO: Consider generalizing this as the basis of getSalaryRow().
            if (!calc) {
                return null;
            }
            this.moveSalaryToRows(calc);
            if (calc.rows.length === 0) {
                calc.rows.push({
                    rowType: exports.CalculationRowType.Unknown,
                });
            }
            return calc.rows[0];
        }
        /**
         * Gets a new blank object with default values, suitable for UI binding.
         * This is a synchronous method - does not go to the server.
         */
        static getBlank() {
            const blank = {
                employer: {
                    avatar: {
                        entityType: exports.LegalEntityType.Undefined,
                        initials: "?",
                    },
                    isSelf: true,
                },
                framework: {
                    type: exports.FrameworkAgreement.NotDefined,
                },
                info: {},
                rows: [],
                salary: {
                    kind: null,
                    price: null,
                },
                worker: {
                    avatar: {
                        entityType: exports.LegalEntityType.Person,
                        initials: "?",
                    },
                    paymentData: {},
                    tax: {},
                },
                workflow: {
                    status: exports.CalculationStatus.Draft,
                },
                result: {
                    irRows: [],
                },
            };
            return blank;
        }
        /**
         * Returns true if calculation totals should be shown:
         * If there are any such rows that the calculation result makes sense.
         */
        static shouldShowTotals(calc) {
            if (!calc || !calc.result || !calc.result.totals) {
                return false;
            }
            return calc.result.totals.total > 0;
        }
        /**
         * Moves the salary defined in Salary object to rows collection.
         * This is needed in old calculations for backward compatibility until moving to v03.
         * NOTE: In v03 this will be done server-side: The Salary will be removed completely.
         * @param calc Calculation to modify
         */
        static moveSalaryToRows(calc) {
            const salary = calc.salary;
            if (salary && salary.price > 0) {
                let rowType = exports.CalculationRowType.Salary;
                switch (salary.kind) {
                    case exports.SalaryKind.Compensation:
                    case exports.SalaryKind.HourlySalary:
                    case exports.SalaryKind.MonthlySalary:
                        // The same underlying enum string
                        rowType = salary.kind;
                        break;
                    case exports.SalaryKind.FixedSalary: // One exception
                        rowType = exports.CalculationRowType.Salary;
                        break;
                    case exports.SalaryKind.TotalEmployerPayment:
                    case exports.SalaryKind.TotalWorkerPayment:
                    case exports.SalaryKind.Undefined:
                    default:
                        // Should never occur => Default to Unknown, which is probably an error.
                        rowType = exports.CalculationRowType.Unknown;
                        break;
                }
                if (rowType !== exports.CalculationRowType.Unknown) {
                    const newRow = {
                        count: salary.amount,
                        message: salary.message,
                        price: salary.price,
                        rowIndex: -1,
                        rowType,
                        unit: rowType === exports.CalculationRowType.HourlySalary ? exports.CalculationRowUnit.Hours : exports.CalculationRowUnit.Count,
                    };
                    calc.rows.push(newRow);
                    salary.price = 0;
                    salary.amount = 0;
                    salary.kind = exports.SalaryKind.Undefined;
                }
            }
        }
        /**
         * Gets the Avatar of other party in a calculation:
         * Employer or Worker
         */
        static getOtherParty(calc, currentAvatar) {
            if (currentAvatar) {
                const id = currentAvatar.id;
                if (calc.worker && calc.worker.accountId !== id) {
                    return calc.worker.avatar;
                }
                else if (calc.employer && calc.employer.accountId !== id) {
                    return calc.employer.avatar;
                }
            }
            return {
                id: null,
                displayName: "?",
                sortableName: "?",
                initials: "?",
                firstName: "n",
                lastName: "n",
                entityType: exports.LegalEntityType.Undefined,
            };
        }
        /**
         * Gets a total sum for a row category from calculation results.
         * @param calc - Calculation object
         * @param category - Row category (collection of row types)
         */
        static getTotalForCategory(calc, category) {
            let total = 0.0;
            this.getResultRowsByCategory(calc, category).forEach((row) => total += row.total);
            return total;
        }
        /**
         * Gets a total sum for a row category from calculation results.
         * @param calc - Calculation object
         * @param types - Array of row types that are included in the result
         */
        static getTotalForRowsByType(calc, ...types) {
            let total = 0.0;
            this.getResultRowsByType(calc, ...types).forEach((row) => total += row.total);
            return total;
        }
        /**
         * Filter the result rows of a calculation based on the type
         * @param calc - Calculation object
         * @param types - Array of row types that are included in the result
         */
        static getResultRowsByType(calc, ...types) {
            return (!calc.result || !calc.result.rows) ? [] : calc.result.rows.filter((value) => types.indexOf(value.rowType) !== -1);
        }
        /**
         * Filter the result rows of a calculation based on a row category
         * @param calc - Calculation object
         * @param category - Row category (collection of row types)
         */
        static getResultRowsByCategory(calc, category) {
            const types = CalculationRowCategories[category];
            return this.getResultRowsByType(calc, ...types);
        }
    }

    /**
     * Provides business logic related to displaying / editing Contact objects
     */
    class ContractPartyLogic {
        /**
         * Gets a new blank Contact with default values, suitable for UI binding.
         */
        static getBlank() {
            return {
                avatar: {
                    entityType: null,
                    lastName: null,
                    pictureType: exports.AvatarPictureType.Icon,
                    isReadOnly: false,
                },
                contact: {
                    countryCode: "fi",
                },
                isSelf: false,
                isSigned: false,
                contractPartyType: exports.ContractPartyType.CustomContact,
                isEmployer: false,
            };
        }
        /** Converts a WorkerAccount object to a ContractParty object. */
        static getFromWorker(worker) {
            return {
                avatar: worker.avatar,
                contact: worker.contact,
                contractPartyType: exports.ContractPartyType.CustomContact,
                iban: worker.ibanNumber,
                id: worker.id,
                isEmployer: false,
                isSelf: false,
                isSigned: false,
                officialId: worker.officialPersonId,
            };
        }
        /** Gets sample contacts for demonstration user interfaces. */
        static getSampleContacts() {
            return [
                {
                    id: "example-default",
                    avatar: {
                        entityType: exports.LegalEntityType.Person,
                        firstName: "Erkki",
                        lastName: "Esimerkki",
                        displayName: "Erkki Esimerkki",
                        sortableName: "Esimerkki, Erkki",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "lime",
                        initials: "EE",
                        url: null,
                        description: "Esimerkkityöntekijä, oletus: 18-52 vuotta, veroprosentti 25%.",
                        id: "example-default",
                    },
                    contact: {
                        email: "erkki@esimerkki.fi",
                        telephone: "555-5555555",
                        street: "Mallikatu 3",
                        postalCode: "333333",
                        city: "Jokupaikka",
                        countryCode: "fi",
                    },
                },
                {
                    id: "example-17",
                    avatar: {
                        entityType: exports.LegalEntityType.Person,
                        firstName: "Laura",
                        lastName: "Lukiolainen",
                        displayName: "Laura Lukiolainen",
                        sortableName: "Lukiolainen, Laura",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "pink",
                        initials: "LL",
                        url: null,
                        description: "Esimerkki nuoresta työntekijästä: 17 vuotias, veroprosentti 0.",
                        id: "example-17",
                    },
                    contact: {
                        email: "laura@esimerkki.fi",
                        telephone: "222-2222222",
                        street: "Lukiolaisenkatu 5",
                        postalCode: "222222",
                        city: "Jokupaikka",
                        countryCode: "fi",
                    },
                },
                {
                    id: "example-company",
                    avatar: {
                        entityType: exports.LegalEntityType.Company,
                        firstName: "Mallitakomo Oy",
                        lastName: null,
                        displayName: "Mallitakomo Oy",
                        sortableName: "Mallitakomo Oy",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "red",
                        initials: "MT",
                        url: null,
                        description: "Esimerkkiyritys Tampereelta",
                        id: "example-company",
                    },
                    contact: {
                        email: "takomo@esimerkki.fi",
                        telephone: "888-88888888",
                        street: "Takomokatu 8",
                        postalCode: "8888888",
                        city: "Tampere",
                        countryCode: "fi",
                    },
                },
                {
                    id: "example-pensioner",
                    avatar: {
                        entityType: exports.LegalEntityType.Person,
                        firstName: "Tanja",
                        lastName: "Eläkeläinen",
                        displayName: "Tanja Eläkeläinen",
                        sortableName: "Eläkeläinen, Tanja",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "blue",
                        initials: "TE",
                        url: null,
                        description: "Esimerkki eläkeläisestä: 65-67 vuotias, veroprosentti 10%.",
                        id: "example-pensioner",
                    },
                    contact: {
                        email: "tanja@esimerkki.fi",
                        telephone: "777-7777777",
                        street: "Eläkeläisenkatu 7",
                        postalCode: "111111",
                        city: "Jokupaikka",
                        countryCode: "fi",
                    },
                },
            ];
        }
    }

    /** Logic for Holiday years and other holiday functionality. */
    class HolidaysLogic {
        /**
         * Gets the multiplier that is used in Average Daily Salary calculation (keskipäiväpalkka).
         * @param holidays Number of holidays for the year.
         */
        static getDailySalaryMultiplier(holidays) {
            const multipliers = [0, 0, 1.8, 2.7, 3.6, 4.5, 5.4, 6.3, 7.2, 8.1, 9.0, 9.9, 10.8, 11.8, 12.7, 13.6, 14.5, 15.5, 16.4, 17.4, 18.3, 19.3, 20.3, 21.3, 22.2, 23.2, 24.1, 25.0, 25.9, 26.9, 27.8];
            if (holidays > 30) {
                return multipliers[30] + (holidays - 30) * 0.9;
            }
            return multipliers[holidays];
        }
        /**
         * Gets a new blank holiday year.
         * @param year Holiday accrual year (e.g. 2019) or 0 for current year and -1 for previous year (from -10 to 10).
         * Current year is always from 1.4. until 31.3. e.g. year 2019 is 1.4.2019 until 31.3.2020.
         * The main part of the holidays from holiday accrual year 2019 are kept on summer 2020.
         */
        static getBlank(year = 0) {
            if (year < 11) {
                const currentYear = Dates.getMonth(Dates.getToday()) < 4 ? (Dates.getYear("today") - 1) : Dates.getYear("today");
                year = currentYear + year;
            }
            const result = {
                employmentId: null,
                year,
                workerId: null,
                period: {
                    start: `{year}-04-01`,
                    end: `{year + 1}-03-31`,
                },
                accrual: {
                    months: [],
                },
                leaves: {
                    paid: [],
                    planned: [],
                },
                spec: {
                    code: exports.HolidayCode.Undefined,
                },
            };
            result.period.daysCount = Dates.getDuration(result.period.start, result.period.end).asDays();
            return result;
        }
        /** Gets the salary payable type or default type if both are possible. */
        static getSalaryPayableType(absenseCauseCode) {
            const defaults = {
                unpaidLeave: "unpaid",
                personalReason: "defaultUnpaid",
                illness: "defaultPaid",
                partTimeSickLeave: "defaultUnpaid",
                parentalLeave: "defaultUnpaid",
                specialMaternityLeave: "defaultUnpaid",
                rehabilitation: "defaultUnpaid",
                childIllness: "defaultPaid",
                partTimeChildCareLeave: "defaultUnpaid",
                training: "defaultPaid",
                jobAlternationLeave: "unpaid",
                studyLeave: "unpaid",
                industrialAction: "unpaid",
                interruptionInWorkProvision: "unpaid",
                leaveOfAbsence: "defaultUnpaid",
                militaryRefresherTraining: "defaultUnpaid",
                militaryService: "unpaid",
                layOff: "unpaid",
                childCareLeave: "unpaid",
                midWeekHoliday: "paid",
                accruedHoliday: "paid",
                occupationalAccident: "paid",
                annualLeave: "paid",
                partTimeAbsenceDueToRehabilitation: "paid",
                other: "both",
            };
            return defaults[absenseCauseCode];
        }
        /** Gets different total days calculation of planned leaves. */
        static getPlannedLeavesCount(holidayYear, type = "all") {
            // TODO: Split period if it starts summer and ands winter.
            const year = holidayYear.year;
            const list = holidayYear.leaves.planned;
            switch (type) {
                case "all":
                    return Arrays.sum(list, (x) => x.period.daysCount);
                case "summer":
                    return Arrays.sum(list.filter((x) => x.period.start < `${year}-10-1`), (x) => x.period.daysCount);
                case "winter":
                    return Arrays.sum(list.filter((x) => x.period.start >= `${year}-10-1`), (x) => x.period.daysCount);
                case "holidaysSaldoStart":
                    return Arrays.sum(holidayYear.accrual.months, (x) => x.daysAccrued) + holidayYear.accrual.startSaldo;
                case "holidaysSaldoEnd":
                    return HolidaysLogic.getPlannedLeavesCount(holidayYear, "holidaysSaldoStart") - HolidaysLogic.getPlannedLeavesCount(holidayYear, "all");
            }
            return null;
        }
        /** Gets a calculated total or saldo row for paid calculations */
        static getPaidHolidaysCalculation(holidayYear, type = "total", dailyHolidaySalary = null) {
            switch (type) {
                case "total":
                    return {
                        avatar: HolidaysLogic.getPaidCalculationAvatar("total", null),
                        holidayBonus: Arrays.sum(holidayYear.leaves.paid, (x) => x.holidayBonus),
                        holidayCompensation: Arrays.sum(holidayYear.leaves.paid, (x) => x.holidayCompensation),
                        holidayDays: Arrays.sum(holidayYear.leaves.paid, (x) => x.holidayDays),
                        holidaySalary: Arrays.sum(holidayYear.leaves.paid, (x) => x.holidaySalary),
                    };
                case "startSaldo": {
                    const holidaySalary = dailyHolidaySalary ? dailyHolidaySalary * holidayYear.accrual.endSaldo : null;
                    return {
                        avatar: HolidaysLogic.getPaidCalculationAvatar("startSaldo", null),
                        holidayBonus: (holidaySalary && holidayYear.spec.bonus) ? (holidaySalary * holidayYear.spec.bonus) : null,
                        holidayDays: holidayYear.accrual.endSaldo,
                        holidayCompensation: null,
                        holidaySalary,
                    };
                }
                case "endSaldo": {
                    const total = HolidaysLogic.getPaidHolidaysCalculation(holidayYear, "total", dailyHolidaySalary);
                    const startSaldo = HolidaysLogic.getPaidHolidaysCalculation(holidayYear, "startSaldo", dailyHolidaySalary);
                    return {
                        avatar: HolidaysLogic.getPaidCalculationAvatar("endSaldo", null),
                        holidayBonus: startSaldo.holidayBonus ? startSaldo.holidayBonus - total.holidayBonus : null,
                        holidayCompensation: startSaldo.holidaySalary ? startSaldo.holidaySalary - (total.holidaySalary + total.holidayCompensation) : null,
                        holidayDays: startSaldo.holidayDays - total.holidayDays,
                        holidaySalary: null,
                    };
                }
            }
            return null;
        }
        /** Gets a display avatar for a paid calculation */
        static getPaidCalculationAvatar(type = "row", row) {
            switch (type) {
                case "row":
                    return {
                        displayName: row.kind === exports.AnnualLeavePaymentKind.DraftCalc ? `Luonnos ${Dates.getFormattedDate(row.date)}` : `Maksettu ${Dates.getFormattedDate(row.date)}`,
                        description: `Palkkakausi ${Dates.getFormattedRange(row.period.start, row.period.end)}`,
                        color: "rgb(74, 146, 233)",
                        initials: row.kind === "draftCalc" ? "luonnos" : "maksettu",
                        entityType: exports.LegalEntityType.Company,
                    };
                case "startSaldo":
                    return {
                        displayName: "Käytettävissä olevat",
                        description: "Edellisenä vuonna kertyneet lomat",
                        color: "gray",
                        initials: "saldo",
                        entityType: exports.LegalEntityType.Company,
                    };
                case "total":
                    return {
                        displayName: "Yhteensä",
                        description: "Käytetyt/maksetut lomapäivät, -rahat ja korvaukset",
                        color: "rgb(74, 146, 233)",
                        initials: "=",
                        entityType: exports.LegalEntityType.Company,
                    };
                case "endSaldo":
                    return {
                        displayName: "Maksettavaa tai siirrettävää",
                        description: "Seuraavalle lomavuodelle tai maksettavaa",
                        color: "gray",
                        initials: "saldo",
                        entityType: exports.LegalEntityType.Company,
                    };
            }
        }
        /**
         * Gets the visualisation data for the a holiday year.
         * @param holidayYear The holiday year to visualize
         */
        static getYearVisualisation(holidayYear) {
            // TODO: Add typing for the visualization
            const year = holidayYear.year;
            const todayYear = Dates.getYear("today");
            const todayMonthIndex = Dates.getMonth("today") - 4;
            const currentMonthIx = todayYear < year ? todayMonthIndex : (todayYear > year ? todayMonthIndex + 24 : todayMonthIndex + 12);
            return {
                year,
                employmentId: holidayYear.employmentId,
                currentMonthIx,
                years: [
                    { text: year - 1, months: 9, style: "default" },
                    { text: year, months: 12, style: "primary" },
                    { text: year + 1, months: 4, style: "default" },
                ],
                months: [4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4],
                items: [
                    { text: `Lomapäivien kertymä: ${holidayYear.accrual.endSaldo} pv`, months: 12, start: 0, style: "default", view: "accrual" },
                    { text: `Kesälomat: ${HolidaysLogic.getPlannedLeavesCount(holidayYear, "summer")} pv`, months: 5, start: 13, style: "default", view: "holidays-summer" },
                    { text: `Talvilomat: ${HolidaysLogic.getPlannedLeavesCount(holidayYear, "winter")} pv`, months: 7, start: 18, style: "default", view: "holidays-winter" },
                    { text: "Lomapalkat ja -rahat", months: 12, start: 13, style: "default", view: "paidHolidays" },
                ],
            };
        }
    }

    /**
     * Provides business logic for Income types in Incomes Registry (Tulorekisteri).
     */
    class IncomeTypesLogic {
        /**
         * Searches the income type texts (supportedTypes) using the current language version.
         * Returns all types if the search string is less than 2 chars.
         * @param searchString Search text - will be used as case-insensitive. Will be trimmed. Minimum length 2.
         */
        static search(searchString) {
            if (!searchString || searchString.trim().length < 2) {
                return IncomeTypesLogic.supportedTypes;
            }
            searchString = searchString.trim().toLowerCase();
            return IncomeTypesLogic.supportedTypes.filter((x) => x.code.toString().indexOf(searchString) >= 0
                || x.label.toLowerCase().indexOf(searchString) >= 0);
        }
        /**
         * Gets all codes in Incomes register reporting.
         * This also includes the Salaxy internal codes eg. 1: "Ignored" / "Not reported"
         * For normal language-versioned behavior use cached allTypes field.
         * @param language Language for the label.
         */
        static getAll(language = null) {
            language = language || Translations.getLanguage() || "fi";
            const allCodes = [{
                    code: 1,
                    label: (language === "en" ? "No IR reporting" : (language === "sv" ? "Ingen IR reporting" : "Ei ilmoiteta tulorekisteriin")),
                    isNegativeSupported: true,
                    pensionInsurance: false,
                    accidentInsurance: false,
                    unemploymentInsurance: false,
                    healthInsurance: false,
                    insuranceInformationAllowed: true,
                    taxDefault: 0,
                    notSupportedError: null,
                    description: "Tätä riviä ei ilmoiteta tulorekisteriin. Tällaisia rivejä ovat esimerkiksi palkan yhteydessä.",
                    taxAndSidecostsDescr: "Rivin summasta ai vähennetä ennakonpidätystä eikä työnantajan sosiaalivakuutusmaksuja.",
                    isSupported: true,
                    paymentDefault: 1,
                    calcGrouping: "expenses",
                }].concat(this.getOriginal());
            return allCodes;
        }
        /**
         * Gets the original codes in Incomes register.
         * This does not include virtual codes, eg. 1: "Ignored" / "Not reported"
         * For normal language-versioned behavior use cached originalTypes field.
         * @param language Language for the label. Typically use null for automatic language versioning.
         */
        static getOriginal(language = null) {
            language = language || Translations.getLanguage() || "fi";
            return incomeTypeCodes.map((x) => ({
                code: x.code,
                label: (language === "en" ? x.nameEn : (language === "sv" ? x.nameSv : x.nameFi)),
                isNegativeSupported: x.isNegativeSupported,
                pensionInsurance: x.pensionInsurance,
                accidentInsurance: x.accidentInsurance,
                unemploymentInsurance: x.unemploymentInsurance,
                healthInsurance: x.healthInsurance,
                insuranceInformationAllowed: x.insuranceInformationAllowed,
                taxDefault: x.taxDefault,
                notSupportedError: x.notSupportedError,
                description: x.description,
                taxAndSidecostsDescr: x.taxAndSidecostsDescr,
                isSupported: x.isSupported,
                paymentDefault: x.paymentDefault,
                calcGrouping: x.calcGrouping,
                transactionCode: x.transactionCode,
            }));
        }
    }
    /** Cached language-versioned list of all types - cached version of getAll() */
    IncomeTypesLogic.allTypes = IncomeTypesLogic.getAll();
    /**
     * Cached language-versioned list of supported types.
     * This is the list of types that you typically use.
     */
    IncomeTypesLogic.supportedTypes = IncomeTypesLogic.getAll().filter((x) => x.isSupported);

    /** Common logic for invoices */
    class InvoicesLogic {
        /**
         * Custom type guard that determines whether the payment object is a payroll (based on whether it calcs collection).
         * @param paymentObject Payment object that is either PayrollDetails or Calculation OR does not have calcs / employer properites that is used to determine the type.
         */
        static isPayroll(paymentObject) {
            return paymentObject && paymentObject.calcs !== undefined;
        }
        /**
         * Custom type guard that determines whether the payment object is a calculation (based on whether it employer property).
         * @param paymentObject Payment object that is either PayrollDetails or Calculation OR does not have calcs / employer properites that is used to determine the type.
         */
        static isCalculation(paymentObject) {
            return paymentObject && paymentObject.employer !== undefined;
        }
        /**
         * Gets and avatar image for a payment channel
         * Enum name is id, label as displayName, descr as description and sort order as 6-char string in sortableName.
         * Picture is either color-initials combination or pic as usual.
         * @param channel The channel to fetch avatar for.
         */
        static getChannelAvatar(channel) {
            return this.getAllChannels().find((x) => x.id === channel);
        }
        /**
         * Gets all the supported payment channels defined in PaymentChannel as avatars.
         * Enum name is id, label as displayName, descr as description and sort order as 6-char string in sortableName.
         * Picture is either color-initials combination or pic as usual.
         */
        static getAllChannels() {
            if (!this.avatars) {
                this.avatars = EnumerationsLogic.getEnumMetadata("PaymentChannel").values
                    .map((e) => {
                    const picData = this.avatarPics[e.name] || this.avatarPics[exports.PaymentChannel.Undefined];
                    return {
                        id: e.name,
                        displayName: e.label,
                        firstName: e.label,
                        sortableName: ("000000" + e.order).slice(-6),
                        description: e.descr,
                        color: picData.color,
                        entityType: picData.entityType || exports.LegalEntityType.Company,
                        initials: picData.initials,
                        pictureType: picData.url === "default" ? exports.AvatarPictureType.Uploaded : picData.pictureType,
                        url: picData.url === "default" ? `https://cdn.salaxy.com/ng1/img/paymentmethod/${e.name}.png` : picData.url,
                    };
                });
            }
            return this.avatars;
        }
    }
    InvoicesLogic.avatarPics = {
        [exports.PaymentChannel.Undefined]: { color: "gray", initials: "?" },
        [exports.PaymentChannel.Test]: { url: "default" },
        [exports.PaymentChannel.ZeroPayment]: { url: "default" },
        [exports.PaymentChannel.External]: { url: "default" },
        [exports.PaymentChannel.PalkkausManual]: { url: "default" },
        [exports.PaymentChannel.PalkkausWS]: { color: "#005f81", initials: "WS" },
        [exports.PaymentChannel.PalkkausPersonal]: { url: "default", },
        [exports.PaymentChannel.PalkkausCfaPaytrail]: { url: "default" },
        [exports.PaymentChannel.PalkkausCfaReference]: { url: "default" },
        [exports.PaymentChannel.PalkkausCfaFinvoice]: { url: "default" },
        [exports.PaymentChannel.PalkkausCfaTest]: { url: "default" },
        [exports.PaymentChannel.AccountorGo]: { url: "default" },
        [exports.PaymentChannel.TalenomOnline]: { url: "default" },
        [exports.PaymentChannel.TalenomCfa]: { url: "default" },
        [exports.PaymentChannel.HolviCfa]: { url: "default" },
        [exports.PaymentChannel.FinagoSolo]: { url: "default" },
        [exports.PaymentChannel.Procountor]: { url: "default" },
        [exports.PaymentChannel.Kevytyrittaja]: { url: "default" },
        [exports.PaymentChannel.VismaNetvisor]: { url: "default" },
    };

    var list = [
    	{
    		id: "01100",
    		code: "01100",
    		labelFi: "Upseerit",
    		labelEn: "Commissioned armed forces officers",
    		labelSv: "Officerare",
    		searchFi: "01100 upseerit",
    		searchEn: "01100 commissioned armed forces officers",
    		searchSv: "01100 officerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "02100",
    		code: "02100",
    		labelFi: "Aliupseerit",
    		labelEn: "Non-commissioned armed forces officers",
    		labelSv: "Underofficerare",
    		searchFi: "02100 aliupseerit",
    		searchEn: "02100 non-commissioned armed forces officers",
    		searchSv: "02100 underofficerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "03100",
    		code: "03100",
    		labelFi: "Sotilasammattihenkilöstö",
    		labelEn: "Armed forces occupations, other ranks",
    		labelSv: "Yrkesutbildad personal inom militären",
    		searchFi: "03100 sotilasammattihenkilöstö",
    		searchEn: "03100 armed forces occupations, other ranks",
    		searchSv: "03100 yrkesutbildad personal inom militären",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11110",
    		code: "11110",
    		labelFi: "Lainsäätäjät",
    		labelEn: "Legislators",
    		labelSv: "Lagstiftare",
    		searchFi: "11110 lainsäätäjät",
    		searchEn: "11110 legislators",
    		searchSv: "11110 lagstiftare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11121",
    		code: "11121",
    		labelFi: "Valtion keskushallinnon johtajat",
    		labelEn: "Senior central government officials",
    		labelSv: "Direktörer inom statens centralförvaltning",
    		searchFi: "11121 valtion keskushallinnon johtajat",
    		searchEn: "11121 senior central government officials",
    		searchSv: "11121 direktörer inom statens centralförvaltning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11122",
    		code: "11122",
    		labelFi: "Alue- ja paikallishallinnon johtajat ja ylimmät virkamiehet",
    		labelEn: "Senior district and local government officials and senior officials",
    		labelSv: "Direktörer och högre tjänstemän inom statlig distrikts- och lokalförvaltning",
    		searchFi: "11122 alue- ja paikallishallinnon johtajat ja ylimmät virkamiehet",
    		searchEn: "11122 senior district and local government officials and senior officials",
    		searchSv: "11122 direktörer och högre tjänstemän inom statlig distrikts- och lokalförvaltning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11141",
    		code: "11141",
    		labelFi: "Työmarkkina- ja elinkeinoelämän järjestöjen johtajat",
    		labelEn: "Senior officials of employers', workers' and other economic-interest organisations",
    		labelSv: "Chefer i arbetsmarknads- och näringslivsorganisationer",
    		searchFi: "11141 työmarkkina- ja elinkeinoelämän järjestöjen johtajat",
    		searchEn: "11141 senior officials of employers', workers' and other economic-interest organisations",
    		searchSv: "11141 chefer i arbetsmarknads- och näringslivsorganisationer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11142",
    		code: "11142",
    		labelFi: "Muiden järjestöjen johtajat",
    		labelEn: "Senior officials of humanitarian and other special-interest organisations",
    		labelSv: "Chefer i andra organisationer",
    		searchFi: "11142 muiden järjestöjen johtajat",
    		searchEn: "11142 senior officials of humanitarian and other special-interest organisations",
    		searchSv: "11142 chefer i andra organisationer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "11200",
    		code: "11200",
    		labelFi: "Toimitusjohtajat ja pääjohtajat",
    		labelEn: "Managing directors and chief executives",
    		labelSv: "Verkställande direktörer och verkschefer",
    		searchFi: "11200 toimitusjohtajat ja pääjohtajat tj ceo",
    		searchEn: "11200 managing directors and chief executives",
    		searchSv: "11200 verkställande direktörer och verkschefer",
    		freeTextFi: "tj ceo",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "11200-bm",
    		code: "11200",
    		labelFi: "Hallituksen jäsen",
    		labelEn: "Board member",
    		labelSv: "Styrelsemedlem",
    		searchFi: "11200 hallituksen jäsen",
    		searchEn: "11200 board member",
    		searchSv: "11200 styrelsemedlem",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "12110",
    		code: "12110",
    		labelFi: "Talousjohtajat",
    		labelEn: "Finance managers",
    		labelSv: "Ekonomichefer",
    		searchFi: "12110 talousjohtajat",
    		searchEn: "12110 finance managers",
    		searchSv: "12110 ekonomichefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "12120",
    		code: "12120",
    		labelFi: "Henkilöstöjohtajat",
    		labelEn: "Human resource managers",
    		labelSv: "Personalchefer",
    		searchFi: "12120 henkilöstöjohtajat",
    		searchEn: "12120 human resource managers",
    		searchSv: "12120 personalchefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "12130",
    		code: "12130",
    		labelFi: "Politiikka- ja suunnittelujohtajat",
    		labelEn: "Policy and planning managers",
    		labelSv: "Politik- och planeringschefer",
    		searchFi: "12130 politiikka- ja suunnittelujohtajat",
    		searchEn: "12130 policy and planning managers",
    		searchSv: "12130 politik- och planeringschefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "12190",
    		code: "12190",
    		labelFi: "Muut hallintojohtajat ja kaupalliset johtajat",
    		labelEn: "Business services and administration managers not elsewhere classified",
    		labelSv: "Övriga chefer inom företagstjänster och administration",
    		searchFi: "12190 muut hallintojohtajat ja kaupalliset johtajat",
    		searchEn: "12190 business services and administration managers not elsewhere classified",
    		searchSv: "12190 övriga chefer inom företagstjänster och administration",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "12210",
    		code: "12210",
    		labelFi: "Myynti- ja markkinointijohtajat",
    		labelEn: "Sales and marketing managers",
    		labelSv: "Försäljnings- och marknadsföringschefer",
    		searchFi: "12210 myynti- ja markkinointijohtajat sales",
    		searchEn: "12210 sales and marketing managers",
    		searchSv: "12210 försäljnings- och marknadsföringschefer",
    		freeTextFi: "sales",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "12220",
    		code: "12220",
    		labelFi: "Mainos- ja tiedotusjohtajat",
    		labelEn: "Advertising and public relations managers",
    		labelSv: "Reklam- och kommunikationschefer, PR-chefer",
    		searchFi: "12220 mainos- ja tiedotusjohtajat",
    		searchEn: "12220 advertising and public relations managers",
    		searchSv: "12220 reklam- och kommunikationschefer, pr-chefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "12230",
    		code: "12230",
    		labelFi: "Tutkimus- ja kehitysjohtajat",
    		labelEn: "Research and development managers",
    		labelSv: "Forsknings- och utvecklingschefer",
    		searchFi: "12230 tutkimus- ja kehitysjohtajat",
    		searchEn: "12230 research and development managers",
    		searchSv: "12230 forsknings- och utvecklingschefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13110",
    		code: "13110",
    		labelFi: "Maa- ja metsätalouden johtajat",
    		labelEn: "Agricultural and forestry production managers",
    		labelSv: "Chefer inom jord- och skogsbruk",
    		searchFi: "13110 maa- ja metsätalouden johtajat",
    		searchEn: "13110 agricultural and forestry production managers",
    		searchSv: "13110 chefer inom jord- och skogsbruk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13120",
    		code: "13120",
    		labelFi: "Vesiviljely- ja kalatalouden johtajat",
    		labelEn: "Aquaculture and fisheries production managers",
    		labelSv: "Chefer inom vattenbruk och fiske",
    		searchFi: "13120 vesiviljely- ja kalatalouden johtajat",
    		searchEn: "13120 aquaculture and fisheries production managers",
    		searchSv: "13120 chefer inom vattenbruk och fiske",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13210",
    		code: "13210",
    		labelFi: "Teollisuuden tuotantojohtajat",
    		labelEn: "Manufacturing managers",
    		labelSv: "Produktionschefer inom industri",
    		searchFi: "13210 teollisuuden tuotantojohtajat",
    		searchEn: "13210 manufacturing managers",
    		searchSv: "13210 produktionschefer inom industri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13220",
    		code: "13220",
    		labelFi: "Kaivostoiminnan tuotantojohtajat",
    		labelEn: "Mining managers",
    		labelSv: "Produktionschefer inom gruvdrift",
    		searchFi: "13220 kaivostoiminnan tuotantojohtajat",
    		searchEn: "13220 mining managers",
    		searchSv: "13220 produktionschefer inom gruvdrift",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13230",
    		code: "13230",
    		labelFi: "Rakennustoiminnan tuotantojohtajat",
    		labelEn: "Construction managers",
    		labelSv: "Produktionschefer inom byggverksamhet",
    		searchFi: "13230 rakennustoiminnan tuotantojohtajat",
    		searchEn: "13230 construction managers",
    		searchSv: "13230 produktionschefer inom byggverksamhet",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13240",
    		code: "13240",
    		labelFi: "Hankinta- ja jakelujohtajat",
    		labelEn: "Supply, distribution and related managers",
    		labelSv: "Inköps- och distributionschefer m.fl.",
    		searchFi: "13240 hankinta- ja jakelujohtajat",
    		searchEn: "13240 supply, distribution and related managers",
    		searchSv: "13240 inköps- och distributionschefer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13300",
    		code: "13300",
    		labelFi: "Tieto- ja viestintäteknologiajohtajat",
    		labelEn: "Information and communications technology service managers",
    		labelSv: "Chefer inom informations- och kommunikationsteknik (IKT)",
    		searchFi: "13300 tieto- ja viestintäteknologiajohtajat",
    		searchEn: "13300 information and communications technology service managers",
    		searchSv: "13300 chefer inom informations- och kommunikationsteknik (ikt)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13410",
    		code: "13410",
    		labelFi: "Lastenhoidon johtajat",
    		labelEn: "Child care services managers",
    		labelSv: "Chefer inom barnomsorg",
    		searchFi: "13410 lastenhoidon johtajat",
    		searchEn: "13410 child care services managers",
    		searchSv: "13410 chefer inom barnomsorg",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13420",
    		code: "13420",
    		labelFi: "Terveydenhuollon johtajat",
    		labelEn: "Health services managers",
    		labelSv: "Chefer inom hälso- och sjukvård",
    		searchFi: "13420 terveydenhuollon johtajat",
    		searchEn: "13420 health services managers",
    		searchSv: "13420 chefer inom hälso- och sjukvård",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13430",
    		code: "13430",
    		labelFi: "Vanhustenhuollon johtajat",
    		labelEn: "Aged care services managers",
    		labelSv: "Chefer inom äldrevård",
    		searchFi: "13430 vanhustenhuollon johtajat",
    		searchEn: "13430 aged care services managers",
    		searchSv: "13430 chefer inom äldrevård",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13440",
    		code: "13440",
    		labelFi: "Sosiaalihuollon johtajat",
    		labelEn: "Social welfare managers",
    		labelSv: "Chefer inom socialvård",
    		searchFi: "13440 sosiaalihuollon johtajat",
    		searchEn: "13440 social welfare managers",
    		searchSv: "13440 chefer inom socialvård",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13450",
    		code: "13450",
    		labelFi: "Opetusalan johtajat",
    		labelEn: "Education managers",
    		labelSv: "Chefer inom utbildning",
    		searchFi: "13450 opetusalan johtajat",
    		searchEn: "13450 education managers",
    		searchSv: "13450 chefer inom utbildning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13460",
    		code: "13460",
    		labelFi: "Rahoitus- ja vakuutuspalvelujen johtajat",
    		labelEn: "Financial and insurance services branch managers",
    		labelSv: "Chefer inom bank- och försäkringstjänster",
    		searchFi: "13460 rahoitus- ja vakuutuspalvelujen johtajat",
    		searchEn: "13460 financial and insurance services branch managers",
    		searchSv: "13460 chefer inom bank- och försäkringstjänster",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "13490",
    		code: "13490",
    		labelFi: "Muut yhteiskunnan palvelujen johtajat",
    		labelEn: "Professional services managers not elsewhere classified",
    		labelSv: "Övriga chefer inom samhälleliga tjänster",
    		searchFi: "13490 muut yhteiskunnan palvelujen johtajat",
    		searchEn: "13490 professional services managers not elsewhere classified",
    		searchSv: "13490 övriga chefer inom samhälleliga tjänster",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "14110",
    		code: "14110",
    		labelFi: "Hotellinjohtajat",
    		labelEn: "Hotel managers",
    		labelSv: "Hotellchefer",
    		searchFi: "14110 hotellinjohtajat",
    		searchEn: "14110 hotel managers",
    		searchSv: "14110 hotellchefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "14120",
    		code: "14120",
    		labelFi: "Ravintolanjohtajat",
    		labelEn: "Restaurant managers",
    		labelSv: "Restaurangchefer",
    		searchFi: "14120 ravintolanjohtajat",
    		searchEn: "14120 restaurant managers",
    		searchSv: "14120 restaurangchefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "14200",
    		code: "14200",
    		labelFi: "Vähittäis- ja tukkukaupan johtajat",
    		labelEn: "Retail and wholesale trade managers",
    		labelSv: "Chefer inom detalj- och partihandel",
    		searchFi: "14200 vähittäis- ja tukkukaupan johtajat",
    		searchEn: "14200 retail and wholesale trade managers",
    		searchSv: "14200 chefer inom detalj- och partihandel",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "14310",
    		code: "14310",
    		labelFi: "Urheilu-, vapaa-aika- ja kulttuurikeskusten johtajat",
    		labelEn: "Sports, recreation and cultural centre managers",
    		labelSv: "Chefer för sport-, fritids- och kulturcenter",
    		searchFi: "14310 urheilu-, vapaa-aika- ja kulttuurikeskusten johtajat",
    		searchEn: "14310 sports, recreation and cultural centre managers",
    		searchSv: "14310 chefer för sport-, fritids- och kulturcenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "14390",
    		code: "14390",
    		labelFi: "Muut palvelualojen johtajat",
    		labelEn: "",
    		labelSv: "Övriga chefer inom servicenäringar",
    		searchFi: "14390 muut palvelualojen johtajat",
    		searchEn: "14390",
    		searchSv: "14390 övriga chefer inom servicenäringar",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21110",
    		code: "21110",
    		labelFi: "Fyysikot ja astronomit",
    		labelEn: "Physicists and astronomers",
    		labelSv: "Fysiker och astronomer",
    		searchFi: "21110 fyysikot ja astronomit",
    		searchEn: "21110 physicists and astronomers",
    		searchSv: "21110 fysiker och astronomer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21120",
    		code: "21120",
    		labelFi: "Meteorologit",
    		labelEn: "Meteorologists",
    		labelSv: "Meteorologer",
    		searchFi: "21120 meteorologit",
    		searchEn: "21120 meteorologists",
    		searchSv: "21120 meteorologer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21130",
    		code: "21130",
    		labelFi: "Kemistit",
    		labelEn: "Chemists",
    		labelSv: "Kemister",
    		searchFi: "21130 kemistit",
    		searchEn: "21130 chemists",
    		searchSv: "21130 kemister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21140",
    		code: "21140",
    		labelFi: "Geologit ja geofyysikot",
    		labelEn: "Geologists and geophysicists",
    		labelSv: "Geologer och geofysiker",
    		searchFi: "21140 geologit ja geofyysikot",
    		searchEn: "21140 geologists and geophysicists",
    		searchSv: "21140 geologer och geofysiker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21200",
    		code: "21200",
    		labelFi: "Matemaatikot, aktuaarit ja tilastotieteilijät",
    		labelEn: "Mathematicians, actuaries and statisticians",
    		labelSv: "Matematiker, aktuarier och statistiker",
    		searchFi: "21200 matemaatikot, aktuaarit ja tilastotieteilijät",
    		searchEn: "21200 mathematicians, actuaries and statisticians",
    		searchSv: "21200 matematiker, aktuarier och statistiker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21310",
    		code: "21310",
    		labelFi: "Biologit, kasvi- ja eläintieteilijät ym. erityisasiantuntijat",
    		labelEn: "Biologists, botanists, zoologists and related professionals",
    		labelSv: "Biologer, botanister, zoologer m.fl.",
    		searchFi: "21310 biologit, kasvi- ja eläintieteilijät ym. erityisasiantuntijat",
    		searchEn: "21310 biologists, botanists, zoologists and related professionals",
    		searchSv: "21310 biologer, botanister, zoologer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21321",
    		code: "21321",
    		labelFi: "Maa- ja kalatalouden erityisasiantuntijat",
    		labelEn: "Agronomists and fishery professionals",
    		labelSv: "Specialister inom jordbruk och fiske",
    		searchFi: "21321 maa- ja kalatalouden erityisasiantuntijat",
    		searchEn: "21321 agronomists and fishery professionals",
    		searchSv: "21321 specialister inom jordbruk och fiske",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21322",
    		code: "21322",
    		labelFi: "Metsätalouden erityisasiantuntijat",
    		labelEn: "Forestry professionals",
    		labelSv: "Specialister inom skogsbruk",
    		searchFi: "21322 metsätalouden erityisasiantuntijat",
    		searchEn: "21322 forestry professionals",
    		searchSv: "21322 specialister inom skogsbruk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21330",
    		code: "21330",
    		labelFi: "Ympäristön- ja luonnonsuojelun erityisasiantuntijat",
    		labelEn: "Environmental protection professionals",
    		labelSv: "Specialister inom miljö- och naturskydd",
    		searchFi: "21330 ympäristön- ja luonnonsuojelun erityisasiantuntijat",
    		searchEn: "21330 environmental protection professionals",
    		searchSv: "21330 specialister inom miljö- och naturskydd",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21410",
    		code: "21410",
    		labelFi: "Teollisen valmistuksen ja tuotantotekniikan erityisasiantuntijat",
    		labelEn: "Industrial and production engineers",
    		labelSv: "Specialister inom industriell produktion och produktionsteknik",
    		searchFi: "21410 teollisen valmistuksen ja tuotantotekniikan erityisasiantuntijat",
    		searchEn: "21410 industrial and production engineers",
    		searchSv: "21410 specialister inom industriell produktion och produktionsteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21420",
    		code: "21420",
    		labelFi: "Maa- ja vesirakentamisen erityisasiantuntijat",
    		labelEn: "Civil engineers",
    		labelSv: "Specialister inom anläggningsarbeten",
    		searchFi: "21420 maa- ja vesirakentamisen erityisasiantuntijat",
    		searchEn: "21420 civil engineers",
    		searchSv: "21420 specialister inom anläggningsarbeten",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21430",
    		code: "21430",
    		labelFi: "Ympäristötekniikan erityisasiantuntijat",
    		labelEn: "Environmental engineers",
    		labelSv: "Specialister inom miljöteknik",
    		searchFi: "21430 ympäristötekniikan erityisasiantuntijat",
    		searchEn: "21430 environmental engineers",
    		searchSv: "21430 specialister inom miljöteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21440",
    		code: "21440",
    		labelFi: "Konetekniikan erityisasiantuntijat",
    		labelEn: "Mechanical engineers",
    		labelSv: "Specialister inom maskinteknik",
    		searchFi: "21440 konetekniikan erityisasiantuntijat",
    		searchEn: "21440 mechanical engineers",
    		searchSv: "21440 specialister inom maskinteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21450",
    		code: "21450",
    		labelFi: "Puunjalostuksen ja kemian prosessitekniikan erityisasiantuntijat",
    		labelEn: "Chemical engineers",
    		labelSv: "Specialister inom träförädling och kemisk processteknik",
    		searchFi: "21450 puunjalostuksen ja kemian prosessitekniikan erityisasiantuntijat",
    		searchEn: "21450 chemical engineers",
    		searchSv: "21450 specialister inom träförädling och kemisk processteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21460",
    		code: "21460",
    		labelFi: "Kaivosteollisuuden, metallurgian ym. erityisasiantuntijat",
    		labelEn: "Mining engineers, metallurgists and related professionals",
    		labelSv: "Specialister inom gruvindustri, metallurgi m.fl.",
    		searchFi: "21460 kaivosteollisuuden, metallurgian ym. erityisasiantuntijat",
    		searchEn: "21460 mining engineers, metallurgists and related professionals",
    		searchSv: "21460 specialister inom gruvindustri, metallurgi m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21490",
    		code: "21490",
    		labelFi: "Muut tekniikan erityisasiantuntijat",
    		labelEn: "Engineering professionals not elsewhere classified",
    		labelSv: "Övriga specialister inom teknik",
    		searchFi: "21490 muut tekniikan erityisasiantuntijat",
    		searchEn: "21490 engineering professionals not elsewhere classified",
    		searchSv: "21490 övriga specialister inom teknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21510",
    		code: "21510",
    		labelFi: "Sähkötekniikan erityisasiantuntijat",
    		labelEn: "Electrical engineers",
    		labelSv: "Specialister inom elteknik",
    		searchFi: "21510 sähkötekniikan erityisasiantuntijat",
    		searchEn: "21510 electrical engineers",
    		searchSv: "21510 specialister inom elteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21520",
    		code: "21520",
    		labelFi: "Elektroniikan erityisasiantuntijat",
    		labelEn: "Electronics engineers",
    		labelSv: "Specialister inom elektronik",
    		searchFi: "21520 elektroniikan erityisasiantuntijat",
    		searchEn: "21520 electronics engineers",
    		searchSv: "21520 specialister inom elektronik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21531",
    		code: "21531",
    		labelFi: "Tietoliikenneteknologian tutkijat",
    		labelEn: "Telecommunications researchers",
    		labelSv: "Forskare inom teleteknik",
    		searchFi: "21531 tietoliikenneteknologian tutkijat",
    		searchEn: "21531 telecommunications researchers",
    		searchSv: "21531 forskare inom teleteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21532",
    		code: "21532",
    		labelFi: "Tietotekniikka-alan tutkijat",
    		labelEn: "Information technology researchers",
    		labelSv: "Forskare inom informationsteknik",
    		searchFi: "21532 tietotekniikka-alan tutkijat",
    		searchEn: "21532 information technology researchers",
    		searchSv: "21532 forskare inom informationsteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21610",
    		code: "21610",
    		labelFi: "Talonrakennuksen arkkitehdit",
    		labelEn: "Building architects",
    		labelSv: "Arkitekter inom husbyggande",
    		searchFi: "21610 talonrakennuksen arkkitehdit",
    		searchEn: "21610 building architects",
    		searchSv: "21610 arkitekter inom husbyggande",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21620",
    		code: "21620",
    		labelFi: "Maisema-arkkitehdit",
    		labelEn: "Landscape architects",
    		labelSv: "Landskapsarkitekter",
    		searchFi: "21620 maisema-arkkitehdit",
    		searchEn: "21620 landscape architects",
    		searchSv: "21620 landskapsarkitekter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21630",
    		code: "21630",
    		labelFi: "Tuote- ja vaatesuunnittelijat",
    		labelEn: "Product and garment designers",
    		labelSv: "Produkt- och modedesigner",
    		searchFi: "21630 tuote- ja vaatesuunnittelijat",
    		searchEn: "21630 product and garment designers",
    		searchSv: "21630 produkt- och modedesigner",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21640",
    		code: "21640",
    		labelFi: "Yhdyskunta- ja liikennesuunnittelijat",
    		labelEn: "Town and traffic planners",
    		labelSv: "Samhälls- och trafikplanerare",
    		searchFi: "21640 yhdyskunta- ja liikennesuunnittelijat",
    		searchEn: "21640 town and traffic planners",
    		searchSv: "21640 samhälls- och trafikplanerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21650",
    		code: "21650",
    		labelFi: "Kartoituksen ja maanmittauksen erityisasiantuntijat",
    		labelEn: "Cartographers and surveyors",
    		labelSv: "Specialister inom kartläggning och lantmäteri",
    		searchFi: "21650 kartoituksen ja maanmittauksen erityisasiantuntijat",
    		searchEn: "21650 cartographers and surveyors",
    		searchSv: "21650 specialister inom kartläggning och lantmäteri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "21660",
    		code: "21660",
    		labelFi: "Graafiset ja multimediasuunnittelijat",
    		labelEn: "Graphic and multimedia designers",
    		labelSv: "Grafiska designer och multimediadesigner",
    		searchFi: "21660 graafiset ja multimediasuunnittelijat",
    		searchEn: "21660 graphic and multimedia designers",
    		searchSv: "21660 grafiska designer och multimediadesigner",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22110",
    		code: "22110",
    		labelFi: "Yleislääkärit",
    		labelEn: "Generalist medical practitioners",
    		labelSv: "Allmänläkare",
    		searchFi: "22110 yleislääkärit",
    		searchEn: "22110 generalist medical practitioners",
    		searchSv: "22110 allmänläkare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22121",
    		code: "22121",
    		labelFi: "Ylilääkärit",
    		labelEn: "Senior physicians",
    		labelSv: "Överläkare",
    		searchFi: "22121 ylilääkärit",
    		searchEn: "22121 senior physicians",
    		searchSv: "22121 överläkare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22122",
    		code: "22122",
    		labelFi: "Erikoislääkärit",
    		labelEn: "Specialists",
    		labelSv: "Specialistläkare",
    		searchFi: "22122 erikoislääkärit",
    		searchEn: "22122 specialists",
    		searchSv: "22122 specialistläkare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22211",
    		code: "22211",
    		labelFi: "Ylihoitajat",
    		labelEn: "Matrons",
    		labelSv: "Överskötare",
    		searchFi: "22211 ylihoitajat",
    		searchEn: "22211 matrons",
    		searchSv: "22211 överskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22212",
    		code: "22212",
    		labelFi: "Osastonhoitajat",
    		labelEn: "Ward sisters",
    		labelSv: "Avdelningsskötare",
    		searchFi: "22212 osastonhoitajat",
    		searchEn: "22212 ward sisters",
    		searchSv: "22212 avdelningsskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22500",
    		code: "22500",
    		labelFi: "Eläinlääkärit",
    		labelEn: "Veterinarians",
    		labelSv: "Veterinärer",
    		searchFi: "22500 eläinlääkärit",
    		searchEn: "22500 veterinarians",
    		searchSv: "22500 veterinärer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22610",
    		code: "22610",
    		labelFi: "Hammaslääkärit",
    		labelEn: "Dentists",
    		labelSv: "Tandläkare",
    		searchFi: "22610 hammaslääkärit",
    		searchEn: "22610 dentists",
    		searchSv: "22610 tandläkare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22620",
    		code: "22620",
    		labelFi: "Proviisorit",
    		labelEn: "Pharmacists",
    		labelSv: "Provisorer",
    		searchFi: "22620 proviisorit",
    		searchEn: "22620 pharmacists",
    		searchSv: "22620 provisorer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22630",
    		code: "22630",
    		labelFi: "Ympäristöterveyden ja työsuojelun erityisasiantuntijat",
    		labelEn: "Environmental and occupational health and hygiene professionals",
    		labelSv: "Specialister inom miljöhälsa och arbetarskydd",
    		searchFi: "22630 ympäristöterveyden ja työsuojelun erityisasiantuntijat",
    		searchEn: "22630 environmental and occupational health and hygiene professionals",
    		searchSv: "22630 specialister inom miljöhälsa och arbetarskydd",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22650",
    		code: "22650",
    		labelFi: "Ravitsemusalan erityisasiantuntijat",
    		labelEn: "Dieticians and nutritionists",
    		labelSv: "Specialister inom näringslära",
    		searchFi: "22650 ravitsemusalan erityisasiantuntijat",
    		searchEn: "22650 dieticians and nutritionists",
    		searchSv: "22650 specialister inom näringslära",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22660",
    		code: "22660",
    		labelFi: "Kuulontutkijat ja puheterapeutit",
    		labelEn: "Audiologists and speech therapists",
    		labelSv: "Audiologer och logopeder",
    		searchFi: "22660 kuulontutkijat ja puheterapeutit",
    		searchEn: "22660 audiologists and speech therapists",
    		searchSv: "22660 audiologer och logopeder",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "22690",
    		code: "22690",
    		labelFi: "Muut muualla luokittelemattomat terveydenhuollon erityisasiantuntijat",
    		labelEn: "Health professionals not elsewhere classified",
    		labelSv: "Övriga specialister inom hälso- och sjukvården utan annan klassificering",
    		searchFi: "22690 muut muualla luokittelemattomat terveydenhuollon erityisasiantuntijat",
    		searchEn: "22690 health professionals not elsewhere classified",
    		searchSv: "22690 övriga specialister inom hälso- och sjukvården utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23101",
    		code: "23101",
    		labelFi: "Professorit",
    		labelEn: "Professors",
    		labelSv: "Professorer",
    		searchFi: "23101 professorit",
    		searchEn: "23101 professors",
    		searchSv: "23101 professorer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23102",
    		code: "23102",
    		labelFi: "Lehtorit ja yliassistentit (yliopisto)",
    		labelEn: "Lecturers and senior assistants (university)",
    		labelSv: "Lektorer och överassistenter (universitet)",
    		searchFi: "23102 lehtorit ja yliassistentit (yliopisto)",
    		searchEn: "23102 lecturers and senior assistants (university)",
    		searchSv: "23102 lektorer och överassistenter (universitet)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23103",
    		code: "23103",
    		labelFi: "Assistentit ja tuntiopettajat (yliopisto)",
    		labelEn: "Assistants and part-time lecturers (university)",
    		labelSv: "Assistenter och timlärare (universitet)",
    		searchFi: "23103 assistentit ja tuntiopettajat (yliopisto)",
    		searchEn: "23103 assistants and part-time lecturers (university)",
    		searchSv: "23103 assistenter och timlärare (universitet)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23104",
    		code: "23104",
    		labelFi: "Yliopettajat (AMK)",
    		labelEn: "Head teachers (polytechnics)",
    		labelSv: "Överlärare (yrkeshögskolor)",
    		searchFi: "23104 yliopettajat (amk)",
    		searchEn: "23104 head teachers (polytechnics)",
    		searchSv: "23104 överlärare (yrkeshögskolor)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23105",
    		code: "23105",
    		labelFi: "Lehtorit (AMK)",
    		labelEn: "Lecturers (polytechnics)",
    		labelSv: "Lektorer (yrkeshögskolor)",
    		searchFi: "23105 lehtorit (amk)",
    		searchEn: "23105 lecturers (polytechnics)",
    		searchSv: "23105 lektorer (yrkeshögskolor)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23106",
    		code: "23106",
    		labelFi: "Tuntiopettajat ym. (AMK)",
    		labelEn: "Part-time lecturers, etc. (polytechnics)",
    		labelSv: "Timlärare m.fl. (yrkeshögskolor)",
    		searchFi: "23106 tuntiopettajat ym. (amk)",
    		searchEn: "23106 part-time lecturers, etc. (polytechnics)",
    		searchSv: "23106 timlärare m.fl. (yrkeshögskolor)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23200",
    		code: "23200",
    		labelFi: "Ammatillisen koulutuksen opettajat",
    		labelEn: "Vocational education teachers",
    		labelSv: "Lärare i yrkesämnen",
    		searchFi: "23200 ammatillisen koulutuksen opettajat",
    		searchEn: "23200 vocational education teachers",
    		searchSv: "23200 lärare i yrkesämnen",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23301",
    		code: "23301",
    		labelFi: "Matemaattisten aineiden opettajat",
    		labelEn: "Teachers in mathematical subjects",
    		labelSv: "Lärare i matematiska ämnen",
    		searchFi: "23301 matemaattisten aineiden opettajat",
    		searchEn: "23301 teachers in mathematical subjects",
    		searchSv: "23301 lärare i matematiska ämnen",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23302",
    		code: "23302",
    		labelFi: "Äidinkielen ja kirjallisuuden opettajat",
    		labelEn: "Native language and literature teachers",
    		labelSv: "Lärare i modersmål och litteratur",
    		searchFi: "23302 äidinkielen ja kirjallisuuden opettajat",
    		searchEn: "23302 native language and literature teachers",
    		searchSv: "23302 lärare i modersmål och litteratur",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23303",
    		code: "23303",
    		labelFi: "Taito- ja taideaineiden opettajat",
    		labelEn: "Crafts and art teachers",
    		labelSv: "Lärare i konst- och färdighetsämnen",
    		searchFi: "23303 taito- ja taideaineiden opettajat",
    		searchEn: "23303 crafts and art teachers",
    		searchSv: "23303 lärare i konst- och färdighetsämnen",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23304",
    		code: "23304",
    		labelFi: "Muut peruskoulun yläluokkien ja lukion opettajat",
    		labelEn: "Other secondary education teaching professionals",
    		labelSv: "Övriga högstadie- och gymnasielärare",
    		searchFi: "23304 muut peruskoulun yläluokkien ja lukion opettajat",
    		searchEn: "23304 other secondary education teaching professionals",
    		searchSv: "23304 övriga högstadie- och gymnasielärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23411",
    		code: "23411",
    		labelFi: "Luokanopettajat",
    		labelEn: "Primary education teaching professionals",
    		labelSv: "Klasslärare",
    		searchFi: "23411 luokanopettajat",
    		searchEn: "23411 primary education teaching professionals",
    		searchSv: "23411 klasslärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23412",
    		code: "23412",
    		labelFi: "Aineenopettajat (peruskoulun alaluokat)",
    		labelEn: "Primary education teaching professionals",
    		labelSv: "Ämneslärare (lågstadiet)",
    		searchFi: "23412 aineenopettajat (peruskoulun alaluokat)",
    		searchEn: "23412 primary education teaching professionals",
    		searchSv: "23412 ämneslärare (lågstadiet)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23420",
    		code: "23420",
    		labelFi: "Lastentarhanopettajat",
    		labelEn: "Early childhood educators",
    		labelSv: "Barnträdgårdslärare",
    		searchFi: "23420 lastentarhanopettajat",
    		searchEn: "23420 early childhood educators",
    		searchSv: "23420 barnträdgårdslärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23510",
    		code: "23510",
    		labelFi: "Opetusmenetelmien erityisasiantuntijat",
    		labelEn: "Education methods specialists",
    		labelSv: "Specialister i pedagogiska metoder",
    		searchFi: "23510 opetusmenetelmien erityisasiantuntijat",
    		searchEn: "23510 education methods specialists",
    		searchSv: "23510 specialister i pedagogiska metoder",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23520",
    		code: "23520",
    		labelFi: "Erityisopettajat",
    		labelEn: "Special needs teachers",
    		labelSv: "Speciallärare",
    		searchFi: "23520 erityisopettajat",
    		searchEn: "23520 special needs teachers",
    		searchSv: "23520 speciallärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23530",
    		code: "23530",
    		labelFi: "Muut kieltenopettajat",
    		labelEn: "Other language teachers",
    		labelSv: "Övriga språklärare",
    		searchFi: "23530 muut kieltenopettajat",
    		searchEn: "23530 other language teachers",
    		searchSv: "23530 övriga språklärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23540",
    		code: "23540",
    		labelFi: "Muut musiikin opettajat",
    		labelEn: "Other music teachers",
    		labelSv: "Övriga musiklärare",
    		searchFi: "23540 muut musiikin opettajat",
    		searchEn: "23540 other music teachers",
    		searchSv: "23540 övriga musiklärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23550",
    		code: "23550",
    		labelFi: "Muut taideaineiden opettajat",
    		labelEn: "Other arts teachers",
    		labelSv: "Övriga lärare i konstnärliga ämnen",
    		searchFi: "23550 muut taideaineiden opettajat",
    		searchEn: "23550 other arts teachers",
    		searchSv: "23550 övriga lärare i konstnärliga ämnen",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23560",
    		code: "23560",
    		labelFi: "Muut tietotekniikan opettajat ja kouluttajat",
    		labelEn: "Information technology trainers",
    		labelSv: "Övriga lärare och utbildare inom informationsteknik",
    		searchFi: "23560 muut tietotekniikan opettajat ja kouluttajat",
    		searchEn: "23560 information technology trainers",
    		searchSv: "23560 övriga lärare och utbildare inom informationsteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23591",
    		code: "23591",
    		labelFi: "Opinto-ohjaajat",
    		labelEn: "Career counsellors",
    		labelSv: "Studiehandledare",
    		searchFi: "23591 opinto-ohjaajat",
    		searchEn: "23591 career counsellors",
    		searchSv: "23591 studiehandledare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "23592",
    		code: "23592",
    		labelFi: "Muut opetuksen erityisasiantuntijat",
    		labelEn: "Other teaching professionals",
    		labelSv: "Övriga specialister inom undervisning",
    		searchFi: "23592 muut opetuksen erityisasiantuntijat",
    		searchEn: "23592 other teaching professionals",
    		searchSv: "23592 övriga specialister inom undervisning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24110",
    		code: "24110",
    		labelFi: "Laskentatoimen erityisasiantuntijat ja tilintarkastajat",
    		labelEn: "Accountants",
    		labelSv: "Specialister inom redovisning och revision",
    		searchFi: "24110 laskentatoimen erityisasiantuntijat ja tilintarkastajat",
    		searchEn: "24110 accountants",
    		searchSv: "24110 specialister inom redovisning och revision",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24120",
    		code: "24120",
    		labelFi: "Rahoitus- ja sijoitusneuvojat",
    		labelEn: "Financial and investment advisers",
    		labelSv: "Finans- och investeringsrådgivare",
    		searchFi: "24120 rahoitus- ja sijoitusneuvojat",
    		searchEn: "24120 financial and investment advisers",
    		searchSv: "24120 finans- och investeringsrådgivare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24130",
    		code: "24130",
    		labelFi: "Rahoitusanalyytikot",
    		labelEn: "Financial analysts",
    		labelSv: "Finansanalytiker",
    		searchFi: "24130 rahoitusanalyytikot",
    		searchEn: "24130 financial analysts",
    		searchSv: "24130 finansanalytiker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24210",
    		code: "24210",
    		labelFi: "Johtamisen ja organisaatioiden erityisasiantuntijat",
    		labelEn: "Management and organization analysts",
    		labelSv: "Specialister inom ledarskap och organisationer",
    		searchFi: "24210 johtamisen ja organisaatioiden erityisasiantuntijat",
    		searchEn: "24210 management and organization analysts",
    		searchSv: "24210 specialister inom ledarskap och organisationer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24220",
    		code: "24220",
    		labelFi: "Hallinnon ja elinkeinojen kehittämisen erityisasiantuntijat",
    		labelEn: "Policy administration professionals",
    		labelSv: "Specialister inom utveckling av administration och näringar",
    		searchFi: "24220 hallinnon ja elinkeinojen kehittämisen erityisasiantuntijat",
    		searchEn: "24220 policy administration professionals",
    		searchSv: "24220 specialister inom utveckling av administration och näringar",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24230",
    		code: "24230",
    		labelFi: "Henkilöstöhallinnon erityisasiantuntijat ja urasuunnittelijat",
    		labelEn: "Personnel and careers professionals",
    		labelSv: "Specialister inom personaladministration och karriärplanering",
    		searchFi: "24230 henkilöstöhallinnon erityisasiantuntijat ja urasuunnittelijat",
    		searchEn: "24230 personnel and careers professionals",
    		searchSv: "24230 specialister inom personaladministration och karriärplanering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24240",
    		code: "24240",
    		labelFi: "Henkilöstön kehittämisen erityisasiantuntijat ja kouluttajat",
    		labelEn: "Training and staff development professionals",
    		labelSv: "Specialister inom personalutveckling och utbildning",
    		searchFi: "24240 henkilöstön kehittämisen erityisasiantuntijat ja kouluttajat",
    		searchEn: "24240 training and staff development professionals",
    		searchSv: "24240 specialister inom personalutveckling och utbildning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24310",
    		code: "24310",
    		labelFi: "Mainonnan ja markkinoinnin erityisasiantuntijat",
    		labelEn: "Advertising and marketing professionals",
    		labelSv: "Specialister inom reklam och marknadsföring",
    		searchFi: "24310 mainonnan ja markkinoinnin erityisasiantuntijat",
    		searchEn: "24310 advertising and marketing professionals",
    		searchSv: "24310 specialister inom reklam och marknadsföring",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24320",
    		code: "24320",
    		labelFi: "Tiedottajat",
    		labelEn: "Public relations professionals",
    		labelSv: "Informatörer",
    		searchFi: "24320 tiedottajat",
    		searchEn: "24320 public relations professionals",
    		searchSv: "24320 informatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24330",
    		code: "24330",
    		labelFi: "Myynti-insinöörit ja lääke-esittelijät (pl. tieto- ja viestintätekniikka)",
    		labelEn: "Technical and medical sales professionals (excluding ICT)",
    		labelSv: "Försäljningsingenjörer och läkemedelsrepresentanter (utom IKT)",
    		searchFi: "24330 myynti-insinöörit ja lääke-esittelijät (pl. tieto- ja viestintätekniikka)",
    		searchEn: "24330 technical and medical sales professionals (excluding ict)",
    		searchSv: "24330 försäljningsingenjörer och läkemedelsrepresentanter (utom ikt)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "24340",
    		code: "24340",
    		labelFi: "Tieto- ja viestintätekniikan myynnin erityisasiantuntijat",
    		labelEn: "Information and communications technology sales professionals",
    		labelSv: "Specialister inom försäljning av informations- och kommunikationsteknik (IKT)",
    		searchFi: "24340 tieto- ja viestintätekniikan myynnin erityisasiantuntijat",
    		searchEn: "24340 information and communications technology sales professionals",
    		searchSv: "24340 specialister inom försäljning av informations- och kommunikationsteknik (ikt)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25110",
    		code: "25110",
    		labelFi: "Sovellusarkkitehdit",
    		labelEn: "Systems analysts",
    		labelSv: "Specialister inom applikationsarkitektur",
    		searchFi: "25110 sovellusarkkitehdit",
    		searchEn: "25110 systems analysts",
    		searchSv: "25110 specialister inom applikationsarkitektur",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25120",
    		code: "25120",
    		labelFi: "Sovellussuunnittelijat",
    		labelEn: "Software developers",
    		labelSv: "Applikationsplanerare",
    		searchFi: "25120 sovellussuunnittelijat ohjelmoija koodaaja",
    		searchEn: "25120 software developers",
    		searchSv: "25120 applikationsplanerare",
    		freeTextFi: "ohjelmoija koodaaja",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "25130",
    		code: "25130",
    		labelFi: "Web- ja multimediakehittäjät",
    		labelEn: "Web and multimedia developers",
    		labelSv: "Webb- och multimediautvecklare",
    		searchFi: "25130 web- ja multimediakehittäjät www html java php",
    		searchEn: "25130 web and multimedia developers",
    		searchSv: "25130 webb- och multimediautvecklare",
    		freeTextFi: "www html java php",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "25140",
    		code: "25140",
    		labelFi: "Sovellusohjelmoijat",
    		labelEn: "Applications programmers",
    		labelSv: "Applikationsprogrammerare",
    		searchFi: "25140 sovellusohjelmoijat",
    		searchEn: "25140 applications programmers",
    		searchSv: "25140 applikationsprogrammerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25190",
    		code: "25190",
    		labelFi: "Muut ohjelmisto- ja sovelluskehittäjät",
    		labelEn: "Software and applications developers and analysts not elsewhere classified",
    		labelSv: "Övriga program- och applikationsutvecklare",
    		searchFi: "25190 muut ohjelmisto- ja sovelluskehittäjät",
    		searchEn: "25190 software and applications developers and analysts not elsewhere classified",
    		searchSv: "25190 övriga program- och applikationsutvecklare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25210",
    		code: "25210",
    		labelFi: "Tietokantasuunnittelijat ja -vastaavat",
    		labelEn: "Database designers and administrators",
    		labelSv: "Databasutvecklare och databasansvariga",
    		searchFi: "25210 tietokantasuunnittelijat ja -vastaavat",
    		searchEn: "25210 database designers and administrators",
    		searchSv: "25210 databasutvecklare och databasansvariga",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25220",
    		code: "25220",
    		labelFi: "Tietojärjestelmien ylläpitäjät",
    		labelEn: "Systems administrators",
    		labelSv: "Systemadministratörer",
    		searchFi: "25220 tietojärjestelmien ylläpitäjät",
    		searchEn: "25220 systems administrators",
    		searchSv: "25220 systemadministratörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25230",
    		code: "25230",
    		labelFi: "Tietoverkkojen erityisasiantuntijat",
    		labelEn: "Computer network professionals",
    		labelSv: "Nätverksspecialister",
    		searchFi: "25230 tietoverkkojen erityisasiantuntijat",
    		searchEn: "25230 computer network professionals",
    		searchSv: "25230 nätverksspecialister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "25290",
    		code: "25290",
    		labelFi: "Muut tietokanta- ja tietoverkkojen erityisasiantuntijat",
    		labelEn: "Database and network professionals not elsewhere classified",
    		labelSv: "Övriga databas- och nätverksspecialister",
    		searchFi: "25290 muut tietokanta- ja tietoverkkojen erityisasiantuntijat",
    		searchEn: "25290 database and network professionals not elsewhere classified",
    		searchSv: "25290 övriga databas- och nätverksspecialister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26110",
    		code: "26110",
    		labelFi: "Asianajajat",
    		labelEn: "Lawyers",
    		labelSv: "Advokater",
    		searchFi: "26110 asianajajat",
    		searchEn: "26110 lawyers",
    		searchSv: "26110 advokater",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26120",
    		code: "26120",
    		labelFi: "Tuomioistuinlakimiehet",
    		labelEn: "Judges",
    		labelSv: "Domare",
    		searchFi: "26120 tuomioistuinlakimiehet",
    		searchEn: "26120 judges",
    		searchSv: "26120 domare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26190",
    		code: "26190",
    		labelFi: "Muut lainopilliset erityisasiantuntijat",
    		labelEn: "Legal professionals not elsewhere classified",
    		labelSv: "Övriga specialister inom juridik",
    		searchFi: "26190 muut lainopilliset erityisasiantuntijat",
    		searchEn: "26190 legal professionals not elsewhere classified",
    		searchSv: "26190 övriga specialister inom juridik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26211",
    		code: "26211",
    		labelFi: "Arkistonhoitajat",
    		labelEn: "Archivists",
    		labelSv: "Arkivarier",
    		searchFi: "26211 arkistonhoitajat",
    		searchEn: "26211 archivists",
    		searchSv: "26211 arkivarier",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26212",
    		code: "26212",
    		labelFi: "Museoalan erityisasiantuntijat",
    		labelEn: "Curators",
    		labelSv: "Specialister inom museisektorn",
    		searchFi: "26212 museoalan erityisasiantuntijat",
    		searchEn: "26212 curators",
    		searchSv: "26212 specialister inom museisektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26220",
    		code: "26220",
    		labelFi: "Kirjastonhoitajat, informaatikot ym.",
    		labelEn: "Librarians and related information professionals",
    		labelSv: "Bibliotekarier, informatiker m.fl.",
    		searchFi: "26220 kirjastonhoitajat, informaatikot ym.",
    		searchEn: "26220 librarians and related information professionals",
    		searchSv: "26220 bibliotekarier, informatiker m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26310",
    		code: "26310",
    		labelFi: "Ekonomistit",
    		labelEn: "Economists",
    		labelSv: "Nationalekonomer",
    		searchFi: "26310 ekonomistit",
    		searchEn: "26310 economists",
    		searchSv: "26310 nationalekonomer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26320",
    		code: "26320",
    		labelFi: "Yhteiskunta- ja kulttuuritutkijat",
    		labelEn: "Sociologists, anthropologists and related professionals",
    		labelSv: "Samhälls- och kulturforskare",
    		searchFi: "26320 yhteiskunta- ja kulttuuritutkijat",
    		searchEn: "26320 sociologists, anthropologists and related professionals",
    		searchSv: "26320 samhälls- och kulturforskare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26330",
    		code: "26330",
    		labelFi: "Historioitsijat, politiikan tutkijat ja filosofit",
    		labelEn: "Philosophers, historians and political scientists",
    		labelSv: "Historiker, statsvetare och filosofer",
    		searchFi: "26330 historioitsijat, politiikan tutkijat ja filosofit",
    		searchEn: "26330 philosophers, historians and political scientists",
    		searchSv: "26330 historiker, statsvetare och filosofer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26340",
    		code: "26340",
    		labelFi: "Psykologit",
    		labelEn: "Psychologists",
    		labelSv: "Psykologer",
    		searchFi: "26340 psykologit",
    		searchEn: "26340 psychologists",
    		searchSv: "26340 psykologer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26351",
    		code: "26351",
    		labelFi: "Sosiaalityöntekijät ym.",
    		labelEn: "Social workers, etc.",
    		labelSv: "Socialarbetare m.fl.",
    		searchFi: "26351 sosiaalityöntekijät ym.",
    		searchEn: "26351 social workers, etc.",
    		searchSv: "26351 socialarbetare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26352",
    		code: "26352",
    		labelFi: "Sosiaalialan suunnittelijat ym.",
    		labelEn: "Social planners, etc.",
    		labelSv: "Planerare inom socialsektorn m.fl.",
    		searchFi: "26352 sosiaalialan suunnittelijat ym.",
    		searchEn: "26352 social planners, etc.",
    		searchSv: "26352 planerare inom socialsektorn m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26360",
    		code: "26360",
    		labelFi: "Papit ym. uskonnollisen elämän erityisasiantuntijat",
    		labelEn: "Religious professionals",
    		labelSv: "Präster och övriga specialister inom religion",
    		searchFi: "26360 papit ym. uskonnollisen elämän erityisasiantuntijat",
    		searchEn: "26360 religious professionals",
    		searchSv: "26360 präster och övriga specialister inom religion",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26410",
    		code: "26410",
    		labelFi: "Kirjailijat ym.",
    		labelEn: "Authors and related writers",
    		labelSv: "Författare m.fl.",
    		searchFi: "26410 kirjailijat ym.",
    		searchEn: "26410 authors and related writers",
    		searchSv: "26410 författare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26421",
    		code: "26421",
    		labelFi: "Toimituspäälliköt ja -sihteerit, ohjelmapäälliköt",
    		labelEn: "Managing editors and subeditors",
    		labelSv: "Redaktionschefer och redaktionssekreterare, programchefer",
    		searchFi: "26421 toimituspäälliköt ja -sihteerit, ohjelmapäälliköt",
    		searchEn: "26421 managing editors and subeditors",
    		searchSv: "26421 redaktionschefer och redaktionssekreterare, programchefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26422",
    		code: "26422",
    		labelFi: "Lehtien yms. toimittajat",
    		labelEn: "Journalists",
    		labelSv: "Redaktörer vid tidningar m.fl.",
    		searchFi: "26422 lehtien yms. toimittajat",
    		searchEn: "26422 journalists",
    		searchSv: "26422 redaktörer vid tidningar m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26423",
    		code: "26423",
    		labelFi: "Radio- ja tv-toimittajat",
    		labelEn: "Radio and television journalists",
    		labelSv: "Radio- och TV-redaktörer",
    		searchFi: "26423 radio- ja tv-toimittajat",
    		searchEn: "26423 radio and television journalists",
    		searchSv: "26423 radio- och tv-redaktörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26430",
    		code: "26430",
    		labelFi: "Kääntäjät, tulkit ja muut kielitieteilijät",
    		labelEn: "Translators, interpreters and other linguists",
    		labelSv: "Översättare, tolkar och andra språkvetare",
    		searchFi: "26430 kääntäjät, tulkit ja muut kielitieteilijät",
    		searchEn: "26430 translators, interpreters and other linguists",
    		searchSv: "26430 översättare, tolkar och andra språkvetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26510",
    		code: "26510",
    		labelFi: "Kuvataiteilijat",
    		labelEn: "Visual artists",
    		labelSv: "Bildkonstnärer",
    		searchFi: "26510 kuvataiteilijat",
    		searchEn: "26510 visual artists",
    		searchSv: "26510 bildkonstnärer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26520",
    		code: "26520",
    		labelFi: "Muusikot, laulajat ja säveltäjät",
    		labelEn: "Musicians, singers and composers",
    		labelSv: "Musiker, sångare och kompositörer",
    		searchFi: "26520 muusikot, laulajat ja säveltäjät",
    		searchEn: "26520 musicians, singers and composers",
    		searchSv: "26520 musiker, sångare och kompositörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26530",
    		code: "26530",
    		labelFi: "Tanssitaiteilijat ja koreografit",
    		labelEn: "Dancers and choreographers",
    		labelSv: "Danskonstnärer och koreografer",
    		searchFi: "26530 tanssitaiteilijat ja koreografit",
    		searchEn: "26530 dancers and choreographers",
    		searchSv: "26530 danskonstnärer och koreografer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26540",
    		code: "26540",
    		labelFi: "Ohjaajat ja tuottajat",
    		labelEn: "Film, stage and related directors and producers",
    		labelSv: "Regissörer och producenter",
    		searchFi: "26540 ohjaajat ja tuottajat",
    		searchEn: "26540 film, stage and related directors and producers",
    		searchSv: "26540 regissörer och producenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26550",
    		code: "26550",
    		labelFi: "Näyttelijät",
    		labelEn: "Actors",
    		labelSv: "Skådespelare",
    		searchFi: "26550 näyttelijät",
    		searchEn: "26550 actors",
    		searchSv: "26550 skådespelare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26560",
    		code: "26560",
    		labelFi: "Juontajat, kuuluttajat ym.",
    		labelEn: "Announcers on radio, television and other media",
    		labelSv: "Presentatörer, konferencierer m.fl.",
    		searchFi: "26560 juontajat, kuuluttajat ym.",
    		searchEn: "26560 announcers on radio, television and other media",
    		searchSv: "26560 presentatörer, konferencierer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "26590",
    		code: "26590",
    		labelFi: "Muut taiteilijat",
    		labelEn: "Creative and performing artists not elsewhere classified",
    		labelSv: "Övriga konstnärer",
    		searchFi: "26590 muut taiteilijat",
    		searchEn: "26590 creative and performing artists not elsewhere classified",
    		searchSv: "26590 övriga konstnärer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31110",
    		code: "31110",
    		labelFi: "Luonnontieteen tekniset asiantuntijat",
    		labelEn: "Chemical and physical science technicians",
    		labelSv: "Tekniska experter inom naturvetenskap",
    		searchFi: "31110 luonnontieteen tekniset asiantuntijat",
    		searchEn: "31110 chemical and physical science technicians",
    		searchSv: "31110 tekniska experter inom naturvetenskap",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31121",
    		code: "31121",
    		labelFi: "Talonrakentamisen asiantuntijat",
    		labelEn: "Building construction technicians",
    		labelSv: "Experter inom husbyggnad",
    		searchFi: "31121 talonrakentamisen asiantuntijat",
    		searchEn: "31121 building construction technicians",
    		searchSv: "31121 experter inom husbyggnad",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31122",
    		code: "31122",
    		labelFi: "Maa- ja vesirakentamisen asiantuntijat",
    		labelEn: "Civil engineering technicians",
    		labelSv: "Experter inom anläggningsarbeten",
    		searchFi: "31122 maa- ja vesirakentamisen asiantuntijat",
    		searchEn: "31122 civil engineering technicians",
    		searchSv: "31122 experter inom anläggningsarbeten",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31123",
    		code: "31123",
    		labelFi: "Kartoituksen ja maanmittauksen asiantuntijat",
    		labelEn: "Cartographic and surveying technicians",
    		labelSv: "Experter inom kartläggning och lantmäteri",
    		searchFi: "31123 kartoituksen ja maanmittauksen asiantuntijat",
    		searchEn: "31123 cartographic and surveying technicians",
    		searchSv: "31123 experter inom kartläggning och lantmäteri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31130",
    		code: "31130",
    		labelFi: "Sähkötekniikan asiantuntijat",
    		labelEn: "Electrical engineering technicians",
    		labelSv: "Experter inom elteknik",
    		searchFi: "31130 sähkötekniikan asiantuntijat",
    		searchEn: "31130 electrical engineering technicians",
    		searchSv: "31130 experter inom elteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31140",
    		code: "31140",
    		labelFi: "Elektroniikan asiantuntijat",
    		labelEn: "Electronics engineering technicians",
    		labelSv: "Experter inom elektronik",
    		searchFi: "31140 elektroniikan asiantuntijat",
    		searchEn: "31140 electronics engineering technicians",
    		searchSv: "31140 experter inom elektronik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31150",
    		code: "31150",
    		labelFi: "Konetekniikan asiantuntijat",
    		labelEn: "Mechanical engineering technicians",
    		labelSv: "Experter inom maskinteknik",
    		searchFi: "31150 konetekniikan asiantuntijat",
    		searchEn: "31150 mechanical engineering technicians",
    		searchSv: "31150 experter inom maskinteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31160",
    		code: "31160",
    		labelFi: "Kemian prosessitekniikan asiantuntijat",
    		labelEn: "Chemical engineering technicians",
    		labelSv: "Experter inom kemisk processteknik",
    		searchFi: "31160 kemian prosessitekniikan asiantuntijat",
    		searchEn: "31160 chemical engineering technicians",
    		searchSv: "31160 experter inom kemisk processteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31170",
    		code: "31170",
    		labelFi: "Kaivosteollisuuden ja metallurgian asiantuntijat",
    		labelEn: "Mining and metallurgical technicians",
    		labelSv: "Experter inom gruvindustri och metallurgi",
    		searchFi: "31170 kaivosteollisuuden ja metallurgian asiantuntijat",
    		searchEn: "31170 mining and metallurgical technicians",
    		searchSv: "31170 experter inom gruvindustri och metallurgi",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31180",
    		code: "31180",
    		labelFi: "Tekniset piirtäjät",
    		labelEn: "Draughtspersons",
    		labelSv: "Tekniska ritare",
    		searchFi: "31180 tekniset piirtäjät",
    		searchEn: "31180 draughtspersons",
    		searchSv: "31180 tekniska ritare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31190",
    		code: "31190",
    		labelFi: "Muut fysiikan, kemian ja teknisten alojen asiantuntijat",
    		labelEn: "Physical and engineering science technicians not elsewhere classified",
    		labelSv: "Övriga experter inom fysik, kemi och teknik",
    		searchFi: "31190 muut fysiikan, kemian ja teknisten alojen asiantuntijat",
    		searchEn: "31190 physical and engineering science technicians not elsewhere classified",
    		searchSv: "31190 övriga experter inom fysik, kemi och teknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31210",
    		code: "31210",
    		labelFi: "Kaivostyönjohtajat",
    		labelEn: "Mining supervisors",
    		labelSv: "Arbetsledare inom gruvdrift",
    		searchFi: "31210 kaivostyönjohtajat",
    		searchEn: "31210 mining supervisors",
    		searchSv: "31210 arbetsledare inom gruvdrift",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31220",
    		code: "31220",
    		labelFi: "Teollisuuden työnjohtajat",
    		labelEn: "Manufacturing supervisors",
    		labelSv: "Arbetsledare inom industri",
    		searchFi: "31220 teollisuuden työnjohtajat",
    		searchEn: "31220 manufacturing supervisors",
    		searchSv: "31220 arbetsledare inom industri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31230",
    		code: "31230",
    		labelFi: "Rakennusalan työnjohtajat",
    		labelEn: "Construction supervisors",
    		labelSv: "Arbetsledare inom byggverksamhet",
    		searchFi: "31230 rakennusalan työnjohtajat",
    		searchEn: "31230 construction supervisors",
    		searchSv: "31230 arbetsledare inom byggverksamhet",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31310",
    		code: "31310",
    		labelFi: "Voimalaitosten prosessinhoitajat",
    		labelEn: "Power production plant operators",
    		labelSv: "Processövervakare, kraftverk",
    		searchFi: "31310 voimalaitosten prosessinhoitajat",
    		searchEn: "31310 power production plant operators",
    		searchSv: "31310 processövervakare, kraftverk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31320",
    		code: "31320",
    		labelFi: "Jätteenpoltto- ja vedenpuhdistuslaitosten prosessinhoitajat",
    		labelEn: "Incinerator and water treatment plant operators",
    		labelSv: "Processövervakare, förbränningsstationer och vattenreningsverk",
    		searchFi: "31320 jätteenpoltto- ja vedenpuhdistuslaitosten prosessinhoitajat",
    		searchEn: "31320 incinerator and water treatment plant operators",
    		searchSv: "31320 processövervakare, förbränningsstationer och vattenreningsverk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31330",
    		code: "31330",
    		labelFi: "Kemianteollisuuden prosessinhoitajat",
    		labelEn: "Chemical processing plant controllers",
    		labelSv: "Processövervakare, kemisk industri",
    		searchFi: "31330 kemianteollisuuden prosessinhoitajat",
    		searchEn: "31330 chemical processing plant controllers",
    		searchSv: "31330 processövervakare, kemisk industri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31340",
    		code: "31340",
    		labelFi: "Öljy- ja maakaasujalostamon prosessinhoitajat",
    		labelEn: "Petroleum and natural gas refining plant operators",
    		labelSv: "Processövervakare, raffinering av petroleum och naturgas",
    		searchFi: "31340 öljy- ja maakaasujalostamon prosessinhoitajat",
    		searchEn: "31340 petroleum and natural gas refining plant operators",
    		searchSv: "31340 processövervakare, raffinering av petroleum och naturgas",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31350",
    		code: "31350",
    		labelFi: "Metallien jalostuksen prosessinhoitajat",
    		labelEn: "Metal production process controllers",
    		labelSv: "Processövervakare, metallframställning",
    		searchFi: "31350 metallien jalostuksen prosessinhoitajat",
    		searchEn: "31350 metal production process controllers",
    		searchSv: "31350 processövervakare, metallframställning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31390",
    		code: "31390",
    		labelFi: "Muut prosessinvalvonnan asiantuntijat",
    		labelEn: "Process control technicians not elsewhere classified",
    		labelSv: "Övriga experter inom processövervakning",
    		searchFi: "31390 muut prosessinvalvonnan asiantuntijat",
    		searchEn: "31390 process control technicians not elsewhere classified",
    		searchSv: "31390 övriga experter inom processövervakning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31410",
    		code: "31410",
    		labelFi: "Laborantit ym.",
    		labelEn: "Life science technicians (excluding medical)",
    		labelSv: "Laboranter m.fl.",
    		searchFi: "31410 laborantit ym.",
    		searchEn: "31410 life science technicians (excluding medical)",
    		searchSv: "31410 laboranter m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31420",
    		code: "31420",
    		labelFi: "Maa- ja kalatalousteknikot",
    		labelEn: "Agricultural technicians",
    		labelSv: "Jordbruks- och fiskeritekniker",
    		searchFi: "31420 maa- ja kalatalousteknikot",
    		searchEn: "31420 agricultural technicians",
    		searchSv: "31420 jordbruks- och fiskeritekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31430",
    		code: "31430",
    		labelFi: "Metsätalousteknikot",
    		labelEn: "Forestry technicians",
    		labelSv: "Skogsbrukstekniker",
    		searchFi: "31430 metsätalousteknikot",
    		searchEn: "31430 forestry technicians",
    		searchSv: "31430 skogsbrukstekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31510",
    		code: "31510",
    		labelFi: "Laivojen konepäälliköt ja -mestarit",
    		labelEn: "Ships' engineers",
    		labelSv: "Maskinchefer och -mästare på fartyg",
    		searchFi: "31510 laivojen konepäälliköt ja -mestarit",
    		searchEn: "31510 ships' engineers",
    		searchSv: "31510 maskinchefer och -mästare på fartyg",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31521",
    		code: "31521",
    		labelFi: "Isojen alusten päälliköt ja perämiehet",
    		labelEn: "Large ships' officers and mates",
    		labelSv: "Fartygsbefäl och styrmän (stora fartyg)",
    		searchFi: "31521 isojen alusten päälliköt ja perämiehet",
    		searchEn: "31521 large ships' officers and mates",
    		searchSv: "31521 fartygsbefäl och styrmän (stora fartyg)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31522",
    		code: "31522",
    		labelFi: "Pienten alusten päälliköt",
    		labelEn: "Small ships' officers",
    		labelSv: "Befäl (mindre fartyg)",
    		searchFi: "31522 pienten alusten päälliköt",
    		searchEn: "31522 small ships' officers",
    		searchSv: "31522 befäl (mindre fartyg)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31523",
    		code: "31523",
    		labelFi: "Satamaliikenteen ohjaajat ja satamakapteenit",
    		labelEn: "Harbour traffic controllers and harbour masters",
    		labelSv: "Hamntrafikledare och hamnkaptener",
    		searchFi: "31523 satamaliikenteen ohjaajat ja satamakapteenit",
    		searchEn: "31523 harbour traffic controllers and harbour masters",
    		searchSv: "31523 hamntrafikledare och hamnkaptener",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31530",
    		code: "31530",
    		labelFi: "Lentokapteenit ja -perämiehet",
    		labelEn: "Aircraft pilots and related associate professionals",
    		labelSv: "Flygkaptener och -styrmän",
    		searchFi: "31530 lentokapteenit ja -perämiehet",
    		searchEn: "31530 aircraft pilots and related associate professionals",
    		searchSv: "31530 flygkaptener och -styrmän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31540",
    		code: "31540",
    		labelFi: "Lennonjohtajat",
    		labelEn: "Air traffic controllers",
    		labelSv: "Flygledare",
    		searchFi: "31540 lennonjohtajat",
    		searchEn: "31540 air traffic controllers",
    		searchSv: "31540 flygledare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "31550",
    		code: "31550",
    		labelFi: "Lennonvalvonnan tekniset asiantuntijat",
    		labelEn: "Air traffic safety electronics technicians",
    		labelSv: "Tekniska experter inom flygövervakning",
    		searchFi: "31550 lennonvalvonnan tekniset asiantuntijat",
    		searchEn: "31550 air traffic safety electronics technicians",
    		searchSv: "31550 tekniska experter inom flygövervakning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32110",
    		code: "32110",
    		labelFi: "Lääketieteellisen kuvantamis- ja laitetekniikan asiantuntijat",
    		labelEn: "Medical imaging and therapeutic equipment technicians",
    		labelSv: "Experter, bilddiagnostik och medicinteknisk utrustning",
    		searchFi: "32110 lääketieteellisen kuvantamis- ja laitetekniikan asiantuntijat",
    		searchEn: "32110 medical imaging and therapeutic equipment technicians",
    		searchSv: "32110 experter, bilddiagnostik och medicinteknisk utrustning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32120",
    		code: "32120",
    		labelFi: "Bioanalyytikot (terveydenhuolto)",
    		labelEn: "Medical and pathology laboratory technicians",
    		labelSv: "Bioanalytiker (hälsovården)",
    		searchFi: "32120 bioanalyytikot (terveydenhuolto)",
    		searchEn: "32120 medical and pathology laboratory technicians",
    		searchSv: "32120 bioanalytiker (hälsovården)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32130",
    		code: "32130",
    		labelFi: "Farmaseutit",
    		labelEn: "Pharmaceutical technicians and assistants",
    		labelSv: "Farmaceuter",
    		searchFi: "32130 farmaseutit",
    		searchEn: "32130 pharmaceutical technicians and assistants",
    		searchSv: "32130 farmaceuter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32141",
    		code: "32141",
    		labelFi: "Hammasteknikot",
    		labelEn: "Dental technicians",
    		labelSv: "Tandtekniker",
    		searchFi: "32141 hammasteknikot",
    		searchEn: "32141 dental technicians",
    		searchSv: "32141 tandtekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32142",
    		code: "32142",
    		labelFi: "Apuvälineteknikot",
    		labelEn: "Prosthetist-orthotists",
    		labelSv: "Hjälpmedelstekniker",
    		searchFi: "32142 apuvälineteknikot",
    		searchEn: "32142 prosthetist-orthotists",
    		searchSv: "32142 hjälpmedelstekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32211",
    		code: "32211",
    		labelFi: "Sairaanhoitajat",
    		labelEn: "Nurses",
    		labelSv: "Sjukskötare",
    		searchFi: "32211 sairaanhoitajat",
    		searchEn: "32211 nurses",
    		searchSv: "32211 sjukskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32212",
    		code: "32212",
    		labelFi: "Terveydenhoitajat",
    		labelEn: "Public health nurses",
    		labelSv: "Hälsovårdare",
    		searchFi: "32212 terveydenhoitajat",
    		searchEn: "32212 public health nurses",
    		searchSv: "32212 hälsovårdare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32220",
    		code: "32220",
    		labelFi: "Kätilöt",
    		labelEn: "Midwifery associate professionals",
    		labelSv: "Barnmorskor",
    		searchFi: "32220 kätilöt",
    		searchEn: "32220 midwifery associate professionals",
    		searchSv: "32220 barnmorskor",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32300",
    		code: "32300",
    		labelFi: "Luontais- ja vaihtoehtohoitajat",
    		labelEn: "Traditional and complementary medicine associate professionals",
    		labelSv: "Utövare av naturmedicin och alternativ medicin",
    		searchFi: "32300 luontais- ja vaihtoehtohoitajat",
    		searchEn: "32300 traditional and complementary medicine associate professionals",
    		searchSv: "32300 utövare av naturmedicin och alternativ medicin",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32400",
    		code: "32400",
    		labelFi: "Seminologit ym.",
    		labelEn: "Veterinary technicians and assistants",
    		labelSv: "Seminörer m.fl.",
    		searchFi: "32400 seminologit ym.",
    		searchEn: "32400 veterinary technicians and assistants",
    		searchSv: "32400 seminörer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32510",
    		code: "32510",
    		labelFi: "Suuhygienistit",
    		labelEn: "Dental assistants and therapists",
    		labelSv: "Munhygienister",
    		searchFi: "32510 suuhygienistit",
    		searchEn: "32510 dental assistants and therapists",
    		searchSv: "32510 munhygienister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32540",
    		code: "32540",
    		labelFi: "Optikot",
    		labelEn: "Dispensing opticians",
    		labelSv: "Optiker",
    		searchFi: "32540 optikot",
    		searchEn: "32540 dispensing opticians",
    		searchSv: "32540 optiker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32550",
    		code: "32550",
    		labelFi: "Fysioterapeutit ym.",
    		labelEn: "Physiotherapy technicians and assistants",
    		labelSv: "Fysioterapeuter m.fl.",
    		searchFi: "32550 fysioterapeutit ym.",
    		searchEn: "32550 physiotherapy technicians and assistants",
    		searchSv: "32550 fysioterapeuter m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32570",
    		code: "32570",
    		labelFi: "Terveys- ja työsuojelutarkastajat",
    		labelEn: "Environmental and occupational health inspectors and associates",
    		labelSv: "Hälso- och arbetarskyddsinspektörer",
    		searchFi: "32570 terveys- ja työsuojelutarkastajat",
    		searchEn: "32570 environmental and occupational health inspectors and associates",
    		searchSv: "32570 hälso- och arbetarskyddsinspektörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32580",
    		code: "32580",
    		labelFi: "Sairaankuljetuksen ensihoitajat",
    		labelEn: "Ambulance workers",
    		labelSv: "Akutvårdare inom sjuktransporten",
    		searchFi: "32580 sairaankuljetuksen ensihoitajat",
    		searchEn: "32580 ambulance workers",
    		searchSv: "32580 akutvårdare inom sjuktransporten",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32591",
    		code: "32591",
    		labelFi: "Toimintaterapeutit",
    		labelEn: "Occupational therapists",
    		labelSv: "Ergoterapeuter",
    		searchFi: "32591 toimintaterapeutit",
    		searchEn: "32591 occupational therapists",
    		searchSv: "32591 ergoterapeuter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "32592",
    		code: "32592",
    		labelFi: "Muut terapeutit",
    		labelEn: "Other therapists",
    		labelSv: "Övriga terapeuter",
    		searchFi: "32592 muut terapeutit",
    		searchEn: "32592 other therapists",
    		searchSv: "32592 övriga terapeuter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33110",
    		code: "33110",
    		labelFi: "Arvopaperi- ja valuuttakauppiaat",
    		labelEn: "Securities and finance dealers and brokers",
    		labelSv: "Värdepappers- och valutamäklare",
    		searchFi: "33110 arvopaperi- ja valuuttakauppiaat",
    		searchEn: "33110 securities and finance dealers and brokers",
    		searchSv: "33110 värdepappers- och valutamäklare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33120",
    		code: "33120",
    		labelFi: "Luotto- ja laina-asiantuntijat",
    		labelEn: "Credit and loans officers",
    		labelSv: "Kredit- och låneexperter",
    		searchFi: "33120 luotto- ja laina-asiantuntijat",
    		searchEn: "33120 credit and loans officers",
    		searchSv: "33120 kredit- och låneexperter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33130",
    		code: "33130",
    		labelFi: "Kirjanpidon ja laskentatoimen asiantuntijat",
    		labelEn: "Accounting associate professionals",
    		labelSv: "Experter inom bokföring och redovisning",
    		searchFi: "33130 kirjanpidon ja laskentatoimen asiantuntijat",
    		searchEn: "33130 accounting associate professionals",
    		searchSv: "33130 experter inom bokföring och redovisning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33140",
    		code: "33140",
    		labelFi: "Tilastointi- ja matematiikka-asiantuntijat",
    		labelEn: "Statistical, mathematical and related associate professionals",
    		labelSv: "Experter inom statistik och matematik",
    		searchFi: "33140 tilastointi- ja matematiikka-asiantuntijat",
    		searchEn: "33140 statistical, mathematical and related associate professionals",
    		searchSv: "33140 experter inom statistik och matematik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33150",
    		code: "33150",
    		labelFi: "Arvioitsijat ja vahinkotarkastajat",
    		labelEn: "Valuers and loss assessors",
    		labelSv: "Värderare och skadeinspektörer",
    		searchFi: "33150 arvioitsijat ja vahinkotarkastajat",
    		searchEn: "33150 valuers and loss assessors",
    		searchSv: "33150 värderare och skadeinspektörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33210",
    		code: "33210",
    		labelFi: "Vakuutusalan palvelumyyjät",
    		labelEn: "Insurance representatives",
    		labelSv: "Försäljare av försäkringstjänster",
    		searchFi: "33210 vakuutusalan palvelumyyjät",
    		searchEn: "33210 insurance representatives",
    		searchSv: "33210 försäljare av försäkringstjänster",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33220",
    		code: "33220",
    		labelFi: "Myyntiedustajat",
    		labelEn: "Commercial sales representatives",
    		labelSv: "Försäljningsrepresentanter",
    		searchFi: "33220 myyntiedustajat sales",
    		searchEn: "33220 commercial sales representatives",
    		searchSv: "33220 försäljningsrepresentanter",
    		freeTextFi: "sales",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "33230",
    		code: "33230",
    		labelFi: "Sisäänostajat",
    		labelEn: "Buyers",
    		labelSv: "Inköpare",
    		searchFi: "33230 sisäänostajat",
    		searchEn: "33230 buyers",
    		searchSv: "33230 inköpare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33240",
    		code: "33240",
    		labelFi: "Kaupanvälittäjät",
    		labelEn: "Trade brokers",
    		labelSv: "Affärsförmedlare",
    		searchFi: "33240 kaupanvälittäjät",
    		searchEn: "33240 trade brokers",
    		searchSv: "33240 affärsförmedlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33310",
    		code: "33310",
    		labelFi: "Huolitsijat, tulli- ja laivanselvittäjät",
    		labelEn: "Clearing and forwarding agents",
    		labelSv: "Speditörer, tull- och fartygsklarerare",
    		searchFi: "33310 huolitsijat, tulli- ja laivanselvittäjät",
    		searchEn: "33310 clearing and forwarding agents",
    		searchSv: "33310 speditörer, tull- och fartygsklarerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33320",
    		code: "33320",
    		labelFi: "Konferenssi- ja tapahtumajärjestäjät",
    		labelEn: "Conference and event planners",
    		labelSv: "Konferens- och evenemangsarrangörer",
    		searchFi: "33320 konferenssi- ja tapahtumajärjestäjät",
    		searchEn: "33320 conference and event planners",
    		searchSv: "33320 konferens- och evenemangsarrangörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33330",
    		code: "33330",
    		labelFi: "Työnvälittäjät",
    		labelEn: "Employment agents and contractors",
    		labelSv: "Arbetsförmedlare",
    		searchFi: "33330 työnvälittäjät",
    		searchEn: "33330 employment agents and contractors",
    		searchSv: "33330 arbetsförmedlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33341",
    		code: "33341",
    		labelFi: "Kiinteistönvälittäjät",
    		labelEn: "Estate agents",
    		labelSv: "Fastighetsmäklare",
    		searchFi: "33341 kiinteistönvälittäjät",
    		searchEn: "33341 estate agents",
    		searchSv: "33341 fastighetsmäklare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33342",
    		code: "33342",
    		labelFi: "Isännöitsijät",
    		labelEn: "Property managers",
    		labelSv: "Disponenter",
    		searchFi: "33342 isännöitsijät",
    		searchEn: "33342 property managers",
    		searchSv: "33342 disponenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33390",
    		code: "33390",
    		labelFi: "Muut liike-elämän asiantuntijat",
    		labelEn: "Business services agents not elsewhere classified",
    		labelSv: "Övriga experter inom affärslivet",
    		searchFi: "33390 muut liike-elämän asiantuntijat",
    		searchEn: "33390 business services agents not elsewhere classified",
    		searchSv: "33390 övriga experter inom affärslivet",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33410",
    		code: "33410",
    		labelFi: "Toimistotyön esimiehet",
    		labelEn: "Office supervisors",
    		labelSv: "Arbetsledare för kontorspersonal",
    		searchFi: "33410 toimistotyön esimiehet",
    		searchEn: "33410 office supervisors",
    		searchSv: "33410 arbetsledare för kontorspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33420",
    		code: "33420",
    		labelFi: "Asianajosihteerit",
    		labelEn: "Legal secretaries",
    		labelSv: "Juristsekreterare",
    		searchFi: "33420 asianajosihteerit",
    		searchEn: "33420 legal secretaries",
    		searchSv: "33420 juristsekreterare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33430",
    		code: "33430",
    		labelFi: "Johdon sihteerit ja osastosihteerit",
    		labelEn: "Administrative and executive secretaries",
    		labelSv: "Ledningsassistenter och avdelningssekreterare",
    		searchFi: "33430 johdon sihteerit ja osastosihteerit",
    		searchEn: "33430 administrative and executive secretaries",
    		searchSv: "33430 ledningsassistenter och avdelningssekreterare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33440",
    		code: "33440",
    		labelFi: "Toimistosihteerit (terveydenhuolto)",
    		labelEn: "Medical secretaries",
    		labelSv: "Byråsekreterare, hälso- och sjukvård",
    		searchFi: "33440 toimistosihteerit (terveydenhuolto)",
    		searchEn: "33440 medical secretaries",
    		searchSv: "33440 byråsekreterare, hälso- och sjukvård",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33510",
    		code: "33510",
    		labelFi: "Tulli- ja rajavirkamiehet",
    		labelEn: "Customs and border inspectors",
    		labelSv: "Tull- och gränstjänstemän",
    		searchFi: "33510 tulli- ja rajavirkamiehet",
    		searchEn: "33510 customs and border inspectors",
    		searchSv: "33510 tull- och gränstjänstemän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33520",
    		code: "33520",
    		labelFi: "Verovalmistelijat ja -tarkastajat",
    		labelEn: "Government tax and excise officials",
    		labelSv: "Skatteberedare och -inspektörer",
    		searchFi: "33520 verovalmistelijat ja -tarkastajat",
    		searchEn: "33520 government tax and excise officials",
    		searchSv: "33520 skatteberedare och -inspektörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33530",
    		code: "33530",
    		labelFi: "Sosiaaliturvaetuuksien käsittelijät",
    		labelEn: "Government social benefits officials",
    		labelSv: "Handläggare av socialskyddsförmåner",
    		searchFi: "33530 sosiaaliturvaetuuksien käsittelijät",
    		searchEn: "33530 government social benefits officials",
    		searchSv: "33530 handläggare av socialskyddsförmåner",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33540",
    		code: "33540",
    		labelFi: "Lupavirkamiehet",
    		labelEn: "Government licensing officials",
    		labelSv: "Tillståndstjänstemän",
    		searchFi: "33540 lupavirkamiehet",
    		searchEn: "33540 government licensing officials",
    		searchSv: "33540 tillståndstjänstemän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33550",
    		code: "33550",
    		labelFi: "Komisariot ja ylikonstaapelit",
    		labelEn: "Police inspectors and detectives",
    		labelSv: "Kommissarier och överkonstaplar",
    		searchFi: "33550 komisariot ja ylikonstaapelit",
    		searchEn: "33550 police inspectors and detectives",
    		searchSv: "33550 kommissarier och överkonstaplar",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "33590",
    		code: "33590",
    		labelFi: "Muut julkishallinnon valmistelu- ja valvontavirkamiehet",
    		labelEn: "Regulatory government associate professionals not elsewhere classified",
    		labelSv: "Övriga tjänstemän inom beredning och tillsyn inom offentlig förvaltning",
    		searchFi: "33590 muut julkishallinnon valmistelu- ja valvontavirkamiehet",
    		searchEn: "33590 regulatory government associate professionals not elsewhere classified",
    		searchSv: "33590 övriga tjänstemän inom beredning och tillsyn inom offentlig förvaltning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34111",
    		code: "34111",
    		labelFi: "Oikeudenkäyntiasiamiehet ja ulosottomiehet",
    		labelEn: "Legal and related business associate professionals",
    		labelSv: "Rättegångsombud och utsökningsmän",
    		searchFi: "34111 oikeudenkäyntiasiamiehet ja ulosottomiehet",
    		searchEn: "34111 legal and related business associate professionals",
    		searchSv: "34111 rättegångsombud och utsökningsmän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34112",
    		code: "34112",
    		labelFi: "Asiamiehet, toimitsijat ym. järjestöalan asiantuntijat",
    		labelEn: "Agents, officials and other organisation associate professionals",
    		labelSv: "Ombud, funktionärer m.fl. experter inom organisationssektorn",
    		searchFi: "34112 asiamiehet, toimitsijat ym. järjestöalan asiantuntijat",
    		searchEn: "34112 agents, officials and other organisation associate professionals",
    		searchSv: "34112 ombud, funktionärer m.fl. experter inom organisationssektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34113",
    		code: "34113",
    		labelFi: "Kuluttajaneuvojat ym.",
    		labelEn: "Consumer advisers, etc.",
    		labelSv: "Konsumentrådgivare m.fl.",
    		searchFi: "34113 kuluttajaneuvojat ym.",
    		searchEn: "34113 consumer advisers, etc.",
    		searchSv: "34113 konsumentrådgivare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34121",
    		code: "34121",
    		labelFi: "Sosiaalialan ohjaajat",
    		labelEn: "Social instructors",
    		labelSv: "Handledare inom socialsektorn",
    		searchFi: "34121 sosiaalialan ohjaajat",
    		searchEn: "34121 social instructors",
    		searchSv: "34121 handledare inom socialsektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34122",
    		code: "34122",
    		labelFi: "Nuorisotyön ohjaajat (ei srk.)",
    		labelEn: "Youth leaders (not parishes)",
    		labelSv: "Ungdomsledare (ej församl.)",
    		searchFi: "34122 nuorisotyön ohjaajat (ei srk.)",
    		searchEn: "34122 youth leaders (not parishes)",
    		searchSv: "34122 ungdomsledare (ej församl.)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34123",
    		code: "34123",
    		labelFi: "Työn- ja askarteluohjaajat",
    		labelEn: "Work and craft leaders",
    		labelSv: "Handledare och sysselsättningsledare",
    		searchFi: "34123 työn- ja askarteluohjaajat",
    		searchEn: "34123 work and craft leaders",
    		searchSv: "34123 handledare och sysselsättningsledare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34131",
    		code: "34131",
    		labelFi: "Diakonit ja diakonissat",
    		labelEn: "Deacons and deaconesses",
    		labelSv: "Diakoner och diakonissor",
    		searchFi: "34131 diakonit ja diakonissat",
    		searchEn: "34131 deacons and deaconesses",
    		searchSv: "34131 diakoner och diakonissor",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34139",
    		code: "34139",
    		labelFi: "Muut seurakuntatyöntekijät",
    		labelEn: "Other religious associate professionals",
    		labelSv: "Övriga församlingsarbetare",
    		searchFi: "34139 muut seurakuntatyöntekijät",
    		searchEn: "34139 other religious associate professionals",
    		searchSv: "34139 övriga församlingsarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34210",
    		code: "34210",
    		labelFi: "Urheilijat",
    		labelEn: "Athletes and sports players",
    		labelSv: "Idrottare",
    		searchFi: "34210 urheilijat",
    		searchEn: "34210 athletes and sports players",
    		searchSv: "34210 idrottare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34220",
    		code: "34220",
    		labelFi: "Urheiluvalmentajat ja toimitsijat",
    		labelEn: "Sports coaches, instructors and officials",
    		labelSv: "Idrottstränare och funktionärer",
    		searchFi: "34220 urheiluvalmentajat ja toimitsijat",
    		searchEn: "34220 sports coaches, instructors and officials",
    		searchSv: "34220 idrottstränare och funktionärer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34230",
    		code: "34230",
    		labelFi: "Liikunnan ja vapaa-ajan ohjaajat",
    		labelEn: "Fitness and recreation instructors and program leaders",
    		labelSv: "Idrottsinstruktörer och fritidsledare",
    		searchFi: "34230 liikunnan ja vapaa-ajan ohjaajat",
    		searchEn: "34230 fitness and recreation instructors and program leaders",
    		searchSv: "34230 idrottsinstruktörer och fritidsledare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34310",
    		code: "34310",
    		labelFi: "Valokuvaajat",
    		labelEn: "Photographers",
    		labelSv: "Fotografer",
    		searchFi: "34310 valokuvaajat",
    		searchEn: "34310 photographers",
    		searchSv: "34310 fotografer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34320",
    		code: "34320",
    		labelFi: "Sisustussuunnittelijat ym.",
    		labelEn: "Interior designers and decorators",
    		labelSv: "Inredningsplanerare m.fl.",
    		searchFi: "34320 sisustussuunnittelijat ym.",
    		searchEn: "34320 interior designers and decorators",
    		searchSv: "34320 inredningsplanerare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34330",
    		code: "34330",
    		labelFi: "Gallerioiden, museoiden ja kirjastojen tekniset työntekijät",
    		labelEn: "Gallery, museum and library technicians",
    		labelSv: "Galleri-, musei- och bibliotekstekniker",
    		searchFi: "34330 gallerioiden, museoiden ja kirjastojen tekniset työntekijät",
    		searchEn: "34330 gallery, museum and library technicians",
    		searchSv: "34330 galleri-, musei- och bibliotekstekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34340",
    		code: "34340",
    		labelFi: "Keittiöpäälliköt",
    		labelEn: "Chefs",
    		labelSv: "Kökschefer",
    		searchFi: "34340 keittiöpäälliköt",
    		searchEn: "34340 chefs",
    		searchSv: "34340 kökschefer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34351",
    		code: "34351",
    		labelFi: "Kuvaussihteerit ja muut näyttämötekniset asiantuntijat",
    		labelEn: "Producer's assistants and other stagecraft associate professionals",
    		labelSv: "Scriptor och övriga experter inom scenteknik",
    		searchFi: "34351 kuvaussihteerit ja muut näyttämötekniset asiantuntijat",
    		searchEn: "34351 producer's assistants and other stagecraft associate professionals",
    		searchSv: "34351 scriptor och övriga experter inom scenteknik",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "34359",
    		code: "34359",
    		labelFi: "Muut taide- ja kulttuurialan tekniset asiantuntijat",
    		labelEn: "Other artistic and cultural associate professionals",
    		labelSv: "Övriga experter inom konst- och konstindustrisektorn",
    		searchFi: "34359 muut taide- ja kulttuurialan tekniset asiantuntijat",
    		searchEn: "34359 other artistic and cultural associate professionals",
    		searchSv: "34359 övriga experter inom konst- och konstindustrisektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35110",
    		code: "35110",
    		labelFi: "Käytön operaattorit",
    		labelEn: "Information and communications technology operations technicians",
    		labelSv: "Driftoperatörer",
    		searchFi: "35110 käytön operaattorit",
    		searchEn: "35110 information and communications technology operations technicians",
    		searchSv: "35110 driftoperatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35120",
    		code: "35120",
    		labelFi: "Käytön tukihenkilöt",
    		labelEn: "Information and communications technology user support technicians",
    		labelSv: "Datasupportpersonal",
    		searchFi: "35120 käytön tukihenkilöt",
    		searchEn: "35120 information and communications technology user support technicians",
    		searchSv: "35120 datasupportpersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35130",
    		code: "35130",
    		labelFi: "Tietoverkkoteknikot",
    		labelEn: "Computer network and systems technicians",
    		labelSv: "Nätverkstekniker",
    		searchFi: "35130 tietoverkkoteknikot",
    		searchEn: "35130 computer network and systems technicians",
    		searchSv: "35130 nätverkstekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35140",
    		code: "35140",
    		labelFi: "Webmasterit ja -teknikot",
    		labelEn: "Web technicians",
    		labelSv: "Webbmaster och webbtekniker",
    		searchFi: "35140 webmasterit ja -teknikot",
    		searchEn: "35140 web technicians",
    		searchSv: "35140 webbmaster och webbtekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35210",
    		code: "35210",
    		labelFi: "Lähetys- ja audiovisuaaliteknikot",
    		labelEn: "Broadcasting and audio-visual technicians",
    		labelSv: "Sändnings- och AV-tekniker",
    		searchFi: "35210 lähetys- ja audiovisuaaliteknikot",
    		searchEn: "35210 broadcasting and audio-visual technicians",
    		searchSv: "35210 sändnings- och av-tekniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "35220",
    		code: "35220",
    		labelFi: "Televiestinnän tekniset asiantuntijat",
    		labelEn: "Telecommunications engineering technicians",
    		labelSv: "Tekniska experter inom telekommunikation",
    		searchFi: "35220 televiestinnän tekniset asiantuntijat",
    		searchEn: "35220 telecommunications engineering technicians",
    		searchSv: "35220 tekniska experter inom telekommunikation",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "41100",
    		code: "41100",
    		labelFi: "Toimistoavustajat",
    		labelEn: "General office clerks",
    		labelSv: "Kontorsassistenter",
    		searchFi: "41100 toimistoavustajat",
    		searchEn: "41100 general office clerks",
    		searchSv: "41100 kontorsassistenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "41200",
    		code: "41200",
    		labelFi: "Yleissihteerit",
    		labelEn: "Secretaries (general)",
    		labelSv: "Kontorssekreterare",
    		searchFi: "41200 yleissihteerit",
    		searchEn: "41200 secretaries (general)",
    		searchSv: "41200 kontorssekreterare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "41310",
    		code: "41310",
    		labelFi: "Tekstinkäsittelijät",
    		labelEn: "Typists and word processing operators",
    		labelSv: "Textbehandlare",
    		searchFi: "41310 tekstinkäsittelijät",
    		searchEn: "41310 typists and word processing operators",
    		searchSv: "41310 textbehandlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "41320",
    		code: "41320",
    		labelFi: "Tallentajat",
    		labelEn: "Data entry clerks",
    		labelSv: "Dataregistrerare",
    		searchFi: "41320 tallentajat",
    		searchEn: "41320 data entry clerks",
    		searchSv: "41320 dataregistrerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42110",
    		code: "42110",
    		labelFi: "Pankki- ym. toimihenkilöt",
    		labelEn: "Bank tellers and related clerks",
    		labelSv: "Banktjänstemän m.fl.",
    		searchFi: "42110 pankki- ym. toimihenkilöt",
    		searchEn: "42110 bank tellers and related clerks",
    		searchSv: "42110 banktjänstemän m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42120",
    		code: "42120",
    		labelFi: "Vedonvälittäjät, bingo- ja kasinopelin hoitajat ym.",
    		labelEn: "Bookmakers, croupiers and related gaming workers",
    		labelSv: "Vadförmedlare, croupierer m.fl.",
    		searchFi: "42120 vedonvälittäjät, bingo- ja kasinopelin hoitajat ym.",
    		searchEn: "42120 bookmakers, croupiers and related gaming workers",
    		searchSv: "42120 vadförmedlare, croupierer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42130",
    		code: "42130",
    		labelFi: "Panttilainaajat",
    		labelEn: "Pawnbrokers and money-lenders",
    		labelSv: "Pantlånare",
    		searchFi: "42130 panttilainaajat",
    		searchEn: "42130 pawnbrokers and money-lenders",
    		searchSv: "42130 pantlånare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42140",
    		code: "42140",
    		labelFi: "Maksujenperijät",
    		labelEn: "Debt-collectors and related workers",
    		labelSv: "Inkasserare",
    		searchFi: "42140 maksujenperijät",
    		searchEn: "42140 debt-collectors and related workers",
    		searchSv: "42140 inkasserare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42210",
    		code: "42210",
    		labelFi: "Matkatoimistovirkailijat",
    		labelEn: "Travel consultants and clerks",
    		labelSv: "Resebyråtjänstemän",
    		searchFi: "42210 matkatoimistovirkailijat",
    		searchEn: "42210 travel consultants and clerks",
    		searchSv: "42210 resebyråtjänstemän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42220",
    		code: "42220",
    		labelFi: "Puhelinpalveluneuvojat",
    		labelEn: "Contact centre information clerks",
    		labelSv: "Telefonservicerådgivare",
    		searchFi: "42220 puhelinpalveluneuvojat",
    		searchEn: "42220 contact centre information clerks",
    		searchSv: "42220 telefonservicerådgivare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42230",
    		code: "42230",
    		labelFi: "Puhelinvaihteenhoitajat",
    		labelEn: "Telephone switchboard operators",
    		labelSv: "Växeltelefonister",
    		searchFi: "42230 puhelinvaihteenhoitajat",
    		searchEn: "42230 telephone switchboard operators",
    		searchSv: "42230 växeltelefonister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42240",
    		code: "42240",
    		labelFi: "Hotellin vastaanottovirkailijat",
    		labelEn: "Hotel receptionists",
    		labelSv: "Hotellreceptionister",
    		searchFi: "42240 hotellin vastaanottovirkailijat",
    		searchEn: "42240 hotel receptionists",
    		searchSv: "42240 hotellreceptionister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42250",
    		code: "42250",
    		labelFi: "Informaatiopisteen asiakasneuvojat",
    		labelEn: "Enquiry clerks",
    		labelSv: "Kundrådgivare vid informationsdisk",
    		searchFi: "42250 informaatiopisteen asiakasneuvojat",
    		searchEn: "42250 enquiry clerks",
    		searchSv: "42250 kundrådgivare vid informationsdisk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42260",
    		code: "42260",
    		labelFi: "Vastaanoton ja neuvonnan hoitajat",
    		labelEn: "Receptionists (general)",
    		labelSv: "Receptions- och rådgivningspersonal",
    		searchFi: "42260 vastaanoton ja neuvonnan hoitajat",
    		searchEn: "42260 receptionists (general)",
    		searchSv: "42260 receptions- och rådgivningspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42270",
    		code: "42270",
    		labelFi: "Tutkimus- ja markkinatutkimushaastattelijat",
    		labelEn: "Survey and market research interviewers",
    		labelSv: "Undersöknings- och marknadsundersökningsintervjuare",
    		searchFi: "42270 tutkimus- ja markkinatutkimushaastattelijat",
    		searchEn: "42270 survey and market research interviewers",
    		searchSv: "42270 undersöknings- och marknadsundersökningsintervjuare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42291",
    		code: "42291",
    		labelFi: "Hälytyspäivystäjät",
    		labelEn: "Emergency officers",
    		labelSv: "Nödcentraloperatörer",
    		searchFi: "42291 hälytyspäivystäjät",
    		searchEn: "42291 emergency officers",
    		searchSv: "42291 nödcentraloperatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "42299",
    		code: "42299",
    		labelFi: "Muut muualla luokittelemattomat asiakaspalvelutyöntekijät",
    		labelEn: "Other client information clerks not elsewhere classified",
    		labelSv: "Övrig kundtjänstpersonal utan annan klassificering",
    		searchFi: "42299 muut muualla luokittelemattomat asiakaspalvelutyöntekijät",
    		searchEn: "42299 other client information clerks not elsewhere classified",
    		searchSv: "42299 övrig kundtjänstpersonal utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43110",
    		code: "43110",
    		labelFi: "Taloushallinnon toimistotyöntekijät",
    		labelEn: "Accounting and bookkeeping clerks",
    		labelSv: "Kontorspersonal inom ekonomiförvaltning",
    		searchFi: "43110 taloushallinnon toimistotyöntekijät",
    		searchEn: "43110 accounting and bookkeeping clerks",
    		searchSv: "43110 kontorspersonal inom ekonomiförvaltning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43120",
    		code: "43120",
    		labelFi: "Tilasto-, rahoitus- ja vakuutusalan toimistotyöntekijät",
    		labelEn: "Statistical, finance and insurance clerks",
    		labelSv: "Kontorspersonal inom statistik-, finansierings- och försäkringssektorn",
    		searchFi: "43120 tilasto-, rahoitus- ja vakuutusalan toimistotyöntekijät",
    		searchEn: "43120 statistical, finance and insurance clerks",
    		searchSv: "43120 kontorspersonal inom statistik-, finansierings- och försäkringssektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43130",
    		code: "43130",
    		labelFi: "Palkanlaskijat",
    		labelEn: "Payroll clerks",
    		labelSv: "Löneräknare",
    		searchFi: "43130 palkanlaskijat",
    		searchEn: "43130 payroll clerks",
    		searchSv: "43130 löneräknare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43210",
    		code: "43210",
    		labelFi: "Varastonhoitajat ym.",
    		labelEn: "Stock clerks",
    		labelSv: "Lagerförvaltare m.fl.",
    		searchFi: "43210 varastonhoitajat ym.",
    		searchEn: "43210 stock clerks",
    		searchSv: "43210 lagerförvaltare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43220",
    		code: "43220",
    		labelFi: "Tuotannon valmistelijat",
    		labelEn: "Production clerks",
    		labelSv: "Produktionsassistenter",
    		searchFi: "43220 tuotannon valmistelijat",
    		searchEn: "43220 production clerks",
    		searchSv: "43220 produktionsassistenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43231",
    		code: "43231",
    		labelFi: "Raideliikenteen ohjaajat, ajojärjestelijät ym.",
    		labelEn: "Rail traffic controllers, traffic co-ordinators, etc.",
    		labelSv: "Spårtrafikledare, trafiksamordnare m.fl.",
    		searchFi: "43231 raideliikenteen ohjaajat, ajojärjestelijät ym.",
    		searchEn: "43231 rail traffic controllers, traffic co-ordinators, etc.",
    		searchSv: "43231 spårtrafikledare, trafiksamordnare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "43239",
    		code: "43239",
    		labelFi: "Muut kuljetuksen ja huolinnan toimistotyöntekijät",
    		labelEn: "Other transport clerks",
    		labelSv: "Övriga kontorstjänstemän inom transport och spedition",
    		searchFi: "43239 muut kuljetuksen ja huolinnan toimistotyöntekijät",
    		searchEn: "43239 other transport clerks",
    		searchSv: "43239 övriga kontorstjänstemän inom transport och spedition",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44110",
    		code: "44110",
    		labelFi: "Kirjastotyöntekijät",
    		labelEn: "Library clerks",
    		labelSv: "Bibliotekspersonal",
    		searchFi: "44110 kirjastotyöntekijät",
    		searchEn: "44110 library clerks",
    		searchSv: "44110 bibliotekspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44121",
    		code: "44121",
    		labelFi: "Postinkantajat",
    		labelEn: "Mail carriers",
    		labelSv: "Brevbärare",
    		searchFi: "44121 postinkantajat",
    		searchEn: "44121 mail carriers",
    		searchSv: "44121 brevbärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44122",
    		code: "44122",
    		labelFi: "Toimistovahtimestarit",
    		labelEn: "Porters",
    		labelSv: "Kontorsvaktmästare",
    		searchFi: "44122 toimistovahtimestarit",
    		searchEn: "44122 porters",
    		searchSv: "44122 kontorsvaktmästare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44130",
    		code: "44130",
    		labelFi: "Koodaajat, oikolukijat ym.",
    		labelEn: "Coding, proof-reading and related clerks",
    		labelSv: "Kodare, korrekturläsare m.fl.",
    		searchFi: "44130 koodaajat, oikolukijat ym.",
    		searchEn: "44130 coding, proof-reading and related clerks",
    		searchSv: "44130 kodare, korrekturläsare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44150",
    		code: "44150",
    		labelFi: "Arkistotyöntekijät",
    		labelEn: "Filing and copying clerks",
    		labelSv: "Arkivanställda",
    		searchFi: "44150 arkistotyöntekijät",
    		searchEn: "44150 filing and copying clerks",
    		searchSv: "44150 arkivanställda",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44160",
    		code: "44160",
    		labelFi: "Henkilöstöhallinnon avustavat toimistotyöntekijät",
    		labelEn: "Personnel clerks",
    		labelSv: "Assisterande kontorspersonal inom personalförvaltning",
    		searchFi: "44160 henkilöstöhallinnon avustavat toimistotyöntekijät",
    		searchEn: "44160 personnel clerks",
    		searchSv: "44160 assisterande kontorspersonal inom personalförvaltning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "44190",
    		code: "44190",
    		labelFi: "Muut muualla luokittelemattomat toimisto- ja asiakaspalvelutyöntekijät",
    		labelEn: "Clerical support workers not elsewhere classified",
    		labelSv: "Övrig kontors- och kundtjänstpersonal utan annan klassificering",
    		searchFi: "44190 muut muualla luokittelemattomat toimisto- ja asiakaspalvelutyöntekijät",
    		searchEn: "44190 clerical support workers not elsewhere classified",
    		searchSv: "44190 övrig kontors- och kundtjänstpersonal utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51110",
    		code: "51110",
    		labelFi: "Lentoemännät, purserit ym.",
    		labelEn: "Travel attendants and travel stewards",
    		labelSv: "Flygvärdinnor, purser m.fl.",
    		searchFi: "51110 lentoemännät, purserit ym.",
    		searchEn: "51110 travel attendants and travel stewards",
    		searchSv: "51110 flygvärdinnor, purser m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51120",
    		code: "51120",
    		labelFi: "Konduktöörit, lipuntarkastajat ym.",
    		labelEn: "Transport conductors",
    		labelSv: "Konduktörer, biljettkontrollanter m.fl.",
    		searchFi: "51120 konduktöörit, lipuntarkastajat ym.",
    		searchEn: "51120 transport conductors",
    		searchSv: "51120 konduktörer, biljettkontrollanter m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51130",
    		code: "51130",
    		labelFi: "Matkaoppaat",
    		labelEn: "Travel guides",
    		labelSv: "Guider",
    		searchFi: "51130 matkaoppaat",
    		searchEn: "51130 travel guides",
    		searchSv: "51130 guider",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51201",
    		code: "51201",
    		labelFi: "Kokit, keittäjät ja kylmäköt",
    		labelEn: "Cooks",
    		labelSv: "Kockar, kokerskor och kallskänkor",
    		searchFi: "51201 kokit, keittäjät ja kylmäköt ravintola kahvila",
    		searchEn: "51201 cooks",
    		searchSv: "51201 kockar, kokerskor och kallskänkor",
    		freeTextFi: "ravintola kahvila",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "51202",
    		code: "51202",
    		labelFi: "Ravintolaesimiehet ja vuoropäälliköt",
    		labelEn: "Restaurant services supervisors and shift managers",
    		labelSv: "Restaurangföreståndare och skiftledare",
    		searchFi: "51202 ravintolaesimiehet ja vuoropäälliköt ravintola kahvila",
    		searchEn: "51202 restaurant services supervisors and shift managers",
    		searchSv: "51202 restaurangföreståndare och skiftledare",
    		freeTextFi: "ravintola kahvila",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51310",
    		code: "51310",
    		labelFi: "Tarjoilijat",
    		labelEn: "Waiters",
    		labelSv: "Servitörer",
    		searchFi: "51310 tarjoilijat ravintola kahvila",
    		searchEn: "51310 waiters",
    		searchSv: "51310 servitörer",
    		freeTextFi: "ravintola kahvila",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "51320",
    		code: "51320",
    		labelFi: "Baarimestarit",
    		labelEn: "Bartenders",
    		labelSv: "Bartendrar",
    		searchFi: "51320 baarimestarit ravintola kahvila",
    		searchEn: "51320 bartenders",
    		searchSv: "51320 bartendrar",
    		freeTextFi: "ravintola kahvila",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51410",
    		code: "51410",
    		labelFi: "Kampaajat ja parturit",
    		labelEn: "Hairdressers",
    		labelSv: "Frisörer",
    		searchFi: "51410 kampaajat ja parturit",
    		searchEn: "51410 hairdressers",
    		searchSv: "51410 frisörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51420",
    		code: "51420",
    		labelFi: "Kosmetologit ym.",
    		labelEn: "Beauticians and related workers",
    		labelSv: "Kosmetologer m.fl.",
    		searchFi: "51420 kosmetologit ym.",
    		searchEn: "51420 beauticians and related workers",
    		searchSv: "51420 kosmetologer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51510",
    		code: "51510",
    		labelFi: "Siivoustyön esimiehet toimistoissa, hotelleissa ja muissa laitoksissa",
    		labelEn: "Cleaning and housekeeping supervisors in offices, hotels and other establishments",
    		labelSv: "Arbetsledare inom städning på kontor, hotell m.fl.",
    		searchFi: "51510 siivoustyön esimiehet toimistoissa, hotelleissa ja muissa laitoksissa",
    		searchEn: "51510 cleaning and housekeeping supervisors in offices, hotels and other establishments",
    		searchSv: "51510 arbetsledare inom städning på kontor, hotell m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51520",
    		code: "51520",
    		labelFi: "Yksityiskotien taloudenhoitajat",
    		labelEn: "Domestic housekeepers",
    		labelSv: "Hushållerskor i privata hem",
    		searchFi: "51520 yksityiskotien taloudenhoitajat",
    		searchEn: "51520 domestic housekeepers",
    		searchSv: "51520 hushållerskor i privata hem",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51530",
    		code: "51530",
    		labelFi: "Kiinteistöhuollon työntekijät",
    		labelEn: "Building caretakers",
    		labelSv: "Fastighetsskötare",
    		searchFi: "51530 kiinteistöhuollon työntekijät talonmies huolto siivous",
    		searchEn: "51530 building caretakers",
    		searchSv: "51530 fastighetsskötare",
    		freeTextFi: "talonmies huolto siivous",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company household"
    	},
    	{
    		id: "51610",
    		code: "51610",
    		labelFi: "Astrologit, ennustajat ym.",
    		labelEn: "Astrologers, fortune-tellers and related workers",
    		labelSv: "Astrologer, siare m.fl.",
    		searchFi: "51610 astrologit, ennustajat ym.",
    		searchEn: "51610 astrologers, fortune-tellers and related workers",
    		searchSv: "51610 astrologer, siare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51631",
    		code: "51631",
    		labelFi: "Hautaustoimistonhoitajat ym.",
    		labelEn: "Undertakers and related funeral workers",
    		labelSv: "Begravningsbyråföreståndare m.fl.",
    		searchFi: "51631 hautaustoimistonhoitajat ym.",
    		searchEn: "51631 undertakers and related funeral workers",
    		searchSv: "51631 begravningsbyråföreståndare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51632",
    		code: "51632",
    		labelFi: "Muut hautaustyöntekijät",
    		labelEn: "Other funeral workers",
    		labelSv: "Övrig begravningspersonal",
    		searchFi: "51632 muut hautaustyöntekijät",
    		searchEn: "51632 other funeral workers",
    		searchSv: "51632 övrig begravningspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51640",
    		code: "51640",
    		labelFi: "Eläintenhoitajat ja lemmikkieläinten trimmaajat",
    		labelEn: "Pet groomers and animal care workers",
    		labelSv: "Djurskötare och djurfrisörer",
    		searchFi: "51640 eläintenhoitajat ja lemmikkieläinten trimmaajat",
    		searchEn: "51640 pet groomers and animal care workers",
    		searchSv: "51640 djurskötare och djurfrisörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51650",
    		code: "51650",
    		labelFi: "Ajo-opettajat",
    		labelEn: "Driving instructors",
    		labelSv: "Trafiklärare",
    		searchFi: "51650 ajo-opettajat",
    		searchEn: "51650 driving instructors",
    		searchSv: "51650 trafiklärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "51690",
    		code: "51690",
    		labelFi: "Muualla luokittelemattomat henkilökohtaisen palvelun työntekijät",
    		labelEn: "Personal services workers not elsewhere classified",
    		labelSv: "Övrig servicepersonal inom personliga tjänster utan annan klassificering",
    		searchFi: "51690 muualla luokittelemattomat henkilökohtaisen palvelun työntekijät",
    		searchEn: "51690 personal services workers not elsewhere classified",
    		searchSv: "51690 övrig servicepersonal inom personliga tjänster utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52110",
    		code: "52110",
    		labelFi: "Kioski- ja torimyyjät",
    		labelEn: "Stall and market salespersons",
    		labelSv: "Kiosk- och torgförsäljare",
    		searchFi: "52110 kioski- ja torimyyjät",
    		searchEn: "52110 stall and market salespersons",
    		searchSv: "52110 kiosk- och torgförsäljare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52120",
    		code: "52120",
    		labelFi: "Katumyyjät (elintarvikkeet)",
    		labelEn: "Street food salespersons",
    		labelSv: "Gatuförsäljare (livsmedel)",
    		searchFi: "52120 katumyyjät (elintarvikkeet)",
    		searchEn: "52120 street food salespersons",
    		searchSv: "52120 gatuförsäljare (livsmedel)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52210",
    		code: "52210",
    		labelFi: "Kauppiaat (pienyrittäjät)",
    		labelEn: "Shop keepers",
    		labelSv: "Butiksinnehavare (småföretagare)",
    		searchFi: "52210 kauppiaat (pienyrittäjät) kauppa",
    		searchEn: "52210 shop keepers",
    		searchSv: "52210 butiksinnehavare (småföretagare)",
    		freeTextFi: "kauppa",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52220",
    		code: "52220",
    		labelFi: "Myymäläesimiehet",
    		labelEn: "Shop supervisors",
    		labelSv: "Butiksföreståndare",
    		searchFi: "52220 myymäläesimiehet kauppa",
    		searchEn: "52220 shop supervisors",
    		searchSv: "52220 butiksföreståndare",
    		freeTextFi: "kauppa",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52230",
    		code: "52230",
    		labelFi: "Myyjät",
    		labelEn: "Shop sales assistants",
    		labelSv: "Försäljare",
    		searchFi: "52230 myyjät kauppa",
    		searchEn: "52230 shop sales assistants",
    		searchSv: "52230 försäljare",
    		freeTextFi: "kauppa",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "52300",
    		code: "52300",
    		labelFi: "Kassanhoitajat ja lipunmyyjät",
    		labelEn: "Cashiers and ticket clerks",
    		labelSv: "Butikskassörer, biljettförsäljare m.fl.",
    		searchFi: "52300 kassanhoitajat ja lipunmyyjät kauppa",
    		searchEn: "52300 cashiers and ticket clerks",
    		searchSv: "52300 butikskassörer, biljettförsäljare m.fl.",
    		freeTextFi: "kauppa",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52410",
    		code: "52410",
    		labelFi: "Mallit",
    		labelEn: "Fashion and other models",
    		labelSv: "Modeller",
    		searchFi: "52410 mallit",
    		searchEn: "52410 fashion and other models",
    		searchSv: "52410 modeller",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52420",
    		code: "52420",
    		labelFi: "Tuote-esittelijät",
    		labelEn: "Sales demonstrators",
    		labelSv: "Varudemonstratörer",
    		searchFi: "52420 tuote-esittelijät",
    		searchEn: "52420 sales demonstrators",
    		searchSv: "52420 varudemonstratörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52430",
    		code: "52430",
    		labelFi: "Suoramyyjät",
    		labelEn: "Door to door salespersons",
    		labelSv: "Direktförsäljare",
    		searchFi: "52430 suoramyyjät",
    		searchEn: "52430 door to door salespersons",
    		searchSv: "52430 direktförsäljare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52440",
    		code: "52440",
    		labelFi: "Puhelin- ja asiakaspalvelukeskusten myyjät",
    		labelEn: "Contact centre salespersons",
    		labelSv: "Försäljare, telefonförsäljning och kontaktcenter",
    		searchFi: "52440 puhelin- ja asiakaspalvelukeskusten myyjät",
    		searchEn: "52440 contact centre salespersons",
    		searchSv: "52440 försäljare, telefonförsäljning och kontaktcenter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52450",
    		code: "52450",
    		labelFi: "Huoltamotyöntekijät",
    		labelEn: "Service station attendants",
    		labelSv: "Bensinstationspersonal",
    		searchFi: "52450 huoltamotyöntekijät",
    		searchEn: "52450 service station attendants",
    		searchSv: "52450 bensinstationspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52460",
    		code: "52460",
    		labelFi: "Kahvila- ja baarimyyjät",
    		labelEn: "Food service counter attendants",
    		labelSv: "Försäljare, kaféer och barer",
    		searchFi: "52460 kahvila- ja baarimyyjät",
    		searchEn: "52460 food service counter attendants",
    		searchSv: "52460 försäljare, kaféer och barer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "52490",
    		code: "52490",
    		labelFi: "Muut muualla luokittelemattomat myyntityöntekijät",
    		labelEn: "Sales workers not elsewhere classified",
    		labelSv: "Övrig försäljningspersonal utan annan klassificering",
    		searchFi: "52490 muut muualla luokittelemattomat myyntityöntekijät",
    		searchEn: "52490 sales workers not elsewhere classified",
    		searchSv: "52490 övrig försäljningspersonal utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53111",
    		code: "53111",
    		labelFi: "Päiväkotien ja muiden laitosten lastenhoitajat ym.",
    		labelEn: "Childminders in kindergartens and other institutions",
    		labelSv: "Barnskötare, daghem och andra institutioner m.fl.",
    		searchFi: "53111 päiväkotien ja muiden laitosten lastenhoitajat ym.",
    		searchEn: "53111 childminders in kindergartens and other institutions",
    		searchSv: "53111 barnskötare, daghem och andra institutioner m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53112",
    		code: "53112",
    		labelFi: "Perhepäivähoitajat",
    		labelEn: "Private childminders",
    		labelSv: "Familjedagvårdare",
    		searchFi: "53112 perhepäivähoitajat lastenhoito",
    		searchEn: "53112 private childminders",
    		searchSv: "53112 familjedagvårdare",
    		freeTextFi: "lastenhoito",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53112-temp",
    		code: "53112",
    		labelFi: "Tilapäinen lastenhoito",
    		labelEn: "Tilapäinen lastenhoito EN",
    		labelSv: "Tilapäinen lastenhoito SV",
    		searchFi: "53112 tilapäinen lastenhoito mll",
    		searchEn: "53112 tilapäinen lastenhoito en",
    		searchSv: "53112 tilapäinen lastenhoito sv",
    		freeTextFi: "mll",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53113",
    		code: "53113",
    		labelFi: "Lastenkerhojen ohjaajat ym.",
    		labelEn: "Children's club leaders",
    		labelSv: "Barnklubbsledare m.fl.",
    		searchFi: "53113 lastenkerhojen ohjaajat ym.",
    		searchEn: "53113 children's club leaders",
    		searchSv: "53113 barnklubbsledare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53120",
    		code: "53120",
    		labelFi: "Koulunkäyntiavustajat",
    		labelEn: "Teachers' aides",
    		labelSv: "Skolgångsbiträden",
    		searchFi: "53120 koulunkäyntiavustajat",
    		searchEn: "53120 teachers' aides",
    		searchSv: "53120 skolgångsbiträden",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53211",
    		code: "53211",
    		labelFi: "Mielenterveyshoitajat",
    		labelEn: "Practical mental nurses",
    		labelSv: "Mentalhälsovårdare",
    		searchFi: "53211 mielenterveyshoitajat",
    		searchEn: "53211 practical mental nurses",
    		searchSv: "53211 mentalhälsovårdare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53212",
    		code: "53212",
    		labelFi: "Kehitysvammaisten hoitajat",
    		labelEn: "Mental handicap nurses",
    		labelSv: "Vårdare av utvecklingshämmade",
    		searchFi: "53212 kehitysvammaisten hoitajat",
    		searchEn: "53212 mental handicap nurses",
    		searchSv: "53212 vårdare av utvecklingshämmade",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53213",
    		code: "53213",
    		labelFi: "Sosiaalialan hoitajat",
    		labelEn: "Social work assistants",
    		labelSv: "Vårdare inom socialsektorn",
    		searchFi: "53213 sosiaalialan hoitajat",
    		searchEn: "53213 social work assistants",
    		searchSv: "53213 vårdare inom socialsektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53219",
    		code: "53219",
    		labelFi: "Muut lähihoitajat",
    		labelEn: "Other practical nurses",
    		labelSv: "Övriga närvårdare",
    		searchFi: "53219 muut lähihoitajat",
    		searchEn: "53219 other practical nurses",
    		searchSv: "53219 övriga närvårdare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53221",
    		code: "53221",
    		labelFi: "Kotityöpalvelutyöntekijät",
    		labelEn: "Household service workers",
    		labelSv: "Hemvårdspersonal",
    		searchFi: "53221 kotityöpalvelutyöntekijät kodinhoitaja",
    		searchEn: "53221 household service workers",
    		searchSv: "53221 hemvårdspersonal",
    		freeTextFi: "kodinhoitaja",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53222",
    		code: "53222",
    		labelFi: "Henkilökohtaiset avustajat, omaishoitajat ym.",
    		labelEn: "Personal care workers, family carers, etc.",
    		labelSv: "Personliga assistenter, närståendevårdare m.fl.",
    		searchFi: "53222 henkilökohtaiset avustajat, omaishoitajat ym. kodinhoitaja",
    		searchEn: "53222 personal care workers, family carers, etc.",
    		searchSv: "53222 personliga assistenter, närståendevårdare m.fl.",
    		freeTextFi: "kodinhoitaja",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "53291",
    		code: "53291",
    		labelFi: "Hammashoitajat",
    		labelEn: "Dental assistants",
    		labelSv: "Tandskötare",
    		searchFi: "53291 hammashoitajat",
    		searchEn: "53291 dental assistants",
    		searchSv: "53291 tandskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53292",
    		code: "53292",
    		labelFi: "Välinehuoltajat",
    		labelEn: "Equipment maintenance assistants",
    		labelSv: "Instrumentvårdare",
    		searchFi: "53292 välinehuoltajat",
    		searchEn: "53292 equipment maintenance assistants",
    		searchSv: "53292 instrumentvårdare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53293",
    		code: "53293",
    		labelFi: "Apteekkien lääketyöntekijät",
    		labelEn: "Pharmaceutical assistants",
    		labelSv: "Läkemedelsarbetare på apotek",
    		searchFi: "53293 apteekkien lääketyöntekijät",
    		searchEn: "53293 pharmaceutical assistants",
    		searchSv: "53293 läkemedelsarbetare på apotek",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "53294",
    		code: "53294",
    		labelFi: "Hierojat ja kuntohoitajat",
    		labelEn: "Massage therapists and practical rehabilitation nurses",
    		labelSv: "Massörer och konditionsskötare",
    		searchFi: "53294 hierojat ja kuntohoitajat",
    		searchEn: "53294 massage therapists and practical rehabilitation nurses",
    		searchSv: "53294 massörer och konditionsskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "54110",
    		code: "54110",
    		labelFi: "Palomiehet",
    		labelEn: "Fire-fighters",
    		labelSv: "Brandmän",
    		searchFi: "54110 palomiehet",
    		searchEn: "54110 fire-fighters",
    		searchSv: "54110 brandmän",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "54120",
    		code: "54120",
    		labelFi: "Poliisit",
    		labelEn: "Police officers",
    		labelSv: "Poliser",
    		searchFi: "54120 poliisit",
    		searchEn: "54120 police officers",
    		searchSv: "54120 poliser",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "54130",
    		code: "54130",
    		labelFi: "Vanginvartijat",
    		labelEn: "Prison guards",
    		labelSv: "Fångvaktare",
    		searchFi: "54130 vanginvartijat",
    		searchEn: "54130 prison guards",
    		searchSv: "54130 fångvaktare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "54140",
    		code: "54140",
    		labelFi: "Vartijat",
    		labelEn: "Security guards",
    		labelSv: "Väktare",
    		searchFi: "54140 vartijat",
    		searchEn: "54140 security guards",
    		searchSv: "54140 väktare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "54190",
    		code: "54190",
    		labelFi: "Muut suojelu- ja vartiointityöntekijät",
    		labelEn: "Protective services workers not elsewhere classified",
    		labelSv: "Övrig säkerhetspersonal",
    		searchFi: "54190 muut suojelu- ja vartiointityöntekijät",
    		searchEn: "54190 protective services workers not elsewhere classified",
    		searchSv: "54190 övrig säkerhetspersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61111",
    		code: "61111",
    		labelFi: "Peltoviljelijät",
    		labelEn: "Crop growers",
    		labelSv: "Åkerodlare",
    		searchFi: "61111 peltoviljelijät",
    		searchEn: "61111 crop growers",
    		searchSv: "61111 åkerodlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61112",
    		code: "61112",
    		labelFi: "Peltoviljelytyönjohtajat ja -työntekijät",
    		labelEn: "Field crop supervisors and workers",
    		labelSv: "Arbetsledare och arbetare inom åkerodling",
    		searchFi: "61112 peltoviljelytyönjohtajat ja -työntekijät",
    		searchEn: "61112 field crop supervisors and workers",
    		searchSv: "61112 arbetsledare och arbetare inom åkerodling",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "61120",
    		code: "61120",
    		labelFi: "Hedelmäpuiden ja pensaiden yms. kasvattajat",
    		labelEn: "Tree and shrub crop growers",
    		labelSv: "Odlare av fruktträd och buskar m.fl.",
    		searchFi: "61120 hedelmäpuiden ja pensaiden yms. kasvattajat",
    		searchEn: "61120 tree and shrub crop growers",
    		searchSv: "61120 odlare av fruktträd och buskar m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61131",
    		code: "61131",
    		labelFi: "Puutarhurit ja kasvihuoneviljelijät",
    		labelEn: "Gardeners, horticultural and nursery growers",
    		labelSv: "Trädgårds- och växthusodlare",
    		searchFi: "61131 puutarhurit ja kasvihuoneviljelijät",
    		searchEn: "61131 gardeners, horticultural and nursery growers",
    		searchSv: "61131 trädgårds- och växthusodlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61132",
    		code: "61132",
    		labelFi: "Puutarha- ja kasvihuonetyönjohtajat ja - työntekijät",
    		labelEn: "Gardeners, horticultural and nursery growers and workers",
    		labelSv: "Arbetsledare och anställda inom trädgårds- och växthusodling",
    		searchFi: "61132 puutarha- ja kasvihuonetyönjohtajat ja - työntekijät ruohonleikkuu",
    		searchEn: "61132 gardeners, horticultural and nursery growers and workers",
    		searchSv: "61132 arbetsledare och anställda inom trädgårds- och växthusodling",
    		freeTextFi: "ruohonleikkuu",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "61140",
    		code: "61140",
    		labelFi: "Yhdistetyn maan- ja vihannesviljelyn tai puutarhanhoidon ym. harjoittajat",
    		labelEn: "Mixed crop growers",
    		labelSv: "Odlare av blandade växtslag",
    		searchFi: "61140 yhdistetyn maan- ja vihannesviljelyn tai puutarhanhoidon ym. harjoittajat",
    		searchEn: "61140 mixed crop growers",
    		searchSv: "61140 odlare av blandade växtslag",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61211",
    		code: "61211",
    		labelFi: "Karjankasvattajat ym.",
    		labelEn: "Dairy and livestock producers",
    		labelSv: "Boskapsuppfödare m.fl.",
    		searchFi: "61211 karjankasvattajat ym.",
    		searchEn: "61211 dairy and livestock producers",
    		searchSv: "61211 boskapsuppfödare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61212",
    		code: "61212",
    		labelFi: "Karjanhoitajat ym.",
    		labelEn: "Dairy and livestock workers",
    		labelSv: "Boskapsskötare m.fl.",
    		searchFi: "61212 karjanhoitajat ym.",
    		searchEn: "61212 dairy and livestock workers",
    		searchSv: "61212 boskapsskötare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61213",
    		code: "61213",
    		labelFi: "Lemmikkieläinten kasvattajat",
    		labelEn: "Pet producers",
    		labelSv: "Sällskapsdjursuppfödare",
    		searchFi: "61213 lemmikkieläinten kasvattajat",
    		searchEn: "61213 pet producers",
    		searchSv: "61213 sällskapsdjursuppfödare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61214",
    		code: "61214",
    		labelFi: "Maatalouslomittajat",
    		labelEn: "Farm relief workers",
    		labelSv: "Lantbruksavbytare",
    		searchFi: "61214 maatalouslomittajat",
    		searchEn: "61214 farm relief workers",
    		searchSv: "61214 lantbruksavbytare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61220",
    		code: "61220",
    		labelFi: "Siipikarjankasvattajat",
    		labelEn: "Poultry producers",
    		labelSv: "Fjäderfäuppfödare",
    		searchFi: "61220 siipikarjankasvattajat",
    		searchEn: "61220 poultry producers",
    		searchSv: "61220 fjäderfäuppfödare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61230",
    		code: "61230",
    		labelFi: "Mehiläistenhoitajat ym.",
    		labelEn: "Apiarists and sericulturists",
    		labelSv: "Biodlare m.fl.",
    		searchFi: "61230 mehiläistenhoitajat ym.",
    		searchEn: "61230 apiarists and sericulturists",
    		searchSv: "61230 biodlare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61291",
    		code: "61291",
    		labelFi: "Turkiseläinten ja porojen kasvattajat",
    		labelEn: "Fur and reindeer producers",
    		labelSv: "Pälsdjurs- och renskötare",
    		searchFi: "61291 turkiseläinten ja porojen kasvattajat",
    		searchEn: "61291 fur and reindeer producers",
    		searchSv: "61291 pälsdjurs- och renskötare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61299",
    		code: "61299",
    		labelFi: "Muualla luokittelemattomat eläintenhoitajat",
    		labelEn: "Animal workers not elsewhere classified",
    		labelSv: "Övriga djurskötare utan annan klassificering",
    		searchFi: "61299 muualla luokittelemattomat eläintenhoitajat",
    		searchEn: "61299 animal workers not elsewhere classified",
    		searchSv: "61299 övriga djurskötare utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "61300",
    		code: "61300",
    		labelFi: "Yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat",
    		labelEn: "Mixed crop and animal producers",
    		labelSv: "Växtodlare och djuruppfödare, blandad drift",
    		searchFi: "61300 yhdistetyn maanviljelyn ja eläintenkasvatuksen harjoittajat",
    		searchEn: "61300 mixed crop and animal producers",
    		searchSv: "61300 växtodlare och djuruppfödare, blandad drift",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "62100",
    		code: "62100",
    		labelFi: "Metsurit ja metsätyöntekijät",
    		labelEn: "Forestry and related workers",
    		labelSv: "Skogsbrukare och skogsarbetare",
    		searchFi: "62100 metsurit ja metsätyöntekijät",
    		searchEn: "62100 forestry and related workers",
    		searchSv: "62100 skogsbrukare och skogsarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "62211",
    		code: "62211",
    		labelFi: "Kalanviljely-yrittäjät",
    		labelEn: "Fish farmers",
    		labelSv: "Fiskodlingsföretagare",
    		searchFi: "62211 kalanviljely-yrittäjät",
    		searchEn: "62211 fish farmers",
    		searchSv: "62211 fiskodlingsföretagare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "62212",
    		code: "62212",
    		labelFi: "Kalanviljelytyönjohtajat ja -työntekijät",
    		labelEn: "Fishery supervisors and workers",
    		labelSv: "Arbetsledare och arbetare inom fiskodling",
    		searchFi: "62212 kalanviljelytyönjohtajat ja -työntekijät",
    		searchEn: "62212 fishery supervisors and workers",
    		searchSv: "62212 arbetsledare och arbetare inom fiskodling",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "62220",
    		code: "62220",
    		labelFi: "Kalastajat",
    		labelEn: "Inland and coastal waters fishery workers",
    		labelSv: "Fiskare",
    		searchFi: "62220 kalastajat",
    		searchEn: "62220 inland and coastal waters fishery workers",
    		searchSv: "62220 fiskare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "62240",
    		code: "62240",
    		labelFi: "Riistanhoitajat ja metsästäjät",
    		labelEn: "Hunters and trappers",
    		labelSv: "Viltvårdare och jägare",
    		searchFi: "62240 riistanhoitajat ja metsästäjät",
    		searchEn: "62240 hunters and trappers",
    		searchSv: "62240 viltvårdare och jägare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71110",
    		code: "71110",
    		labelFi: "Talonrakentajat",
    		labelEn: "House builders",
    		labelSv: "Byggnadsarbetare",
    		searchFi: "71110 talonrakentajat rakennus remontti",
    		searchEn: "71110 house builders",
    		searchSv: "71110 byggnadsarbetare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company household"
    	},
    	{
    		id: "71120",
    		code: "71120",
    		labelFi: "Muurarit ym.",
    		labelEn: "Bricklayers and related workers",
    		labelSv: "Murare m.fl.",
    		searchFi: "71120 muurarit ym. rakennus remontti",
    		searchEn: "71120 bricklayers and related workers",
    		searchSv: "71120 murare m.fl.",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71130",
    		code: "71130",
    		labelFi: "Kivenhakkaajat ja -leikkaajat ym.",
    		labelEn: "Stonemasons, stone cutters, splitters and carvers",
    		labelSv: "Stenhuggare m.fl.",
    		searchFi: "71130 kivenhakkaajat ja -leikkaajat ym.",
    		searchEn: "71130 stonemasons, stone cutters, splitters and carvers",
    		searchSv: "71130 stenhuggare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71140",
    		code: "71140",
    		labelFi: "Betonirakentajat ja raudoittajat",
    		labelEn: "Concrete placers, concrete finishers and related workers",
    		labelSv: "Betongarbetare och armerare",
    		searchFi: "71140 betonirakentajat ja raudoittajat rakennus remontti",
    		searchEn: "71140 concrete placers, concrete finishers and related workers",
    		searchSv: "71140 betongarbetare och armerare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71150",
    		code: "71150",
    		labelFi: "Kirvesmiehet ja rakennuspuusepät",
    		labelEn: "Carpenters and joiners",
    		labelSv: "Timmermän och byggnadssnickare",
    		searchFi: "71150 kirvesmiehet ja rakennuspuusepät rakennus remontti",
    		searchEn: "71150 carpenters and joiners",
    		searchSv: "71150 timmermän och byggnadssnickare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company household"
    	},
    	{
    		id: "71190",
    		code: "71190",
    		labelFi: "Muut rakennustyöntekijät",
    		labelEn: "Building frame and related trades workers not elsewhere classified",
    		labelSv: "Övriga byggnadsarbetare",
    		searchFi: "71190 muut rakennustyöntekijät rakennus remontti",
    		searchEn: "71190 building frame and related trades workers not elsewhere classified",
    		searchSv: "71190 övriga byggnadsarbetare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71210",
    		code: "71210",
    		labelFi: "Kattoasentajat ja -korjaajat",
    		labelEn: "Roofers",
    		labelSv: "Takmontörer och takreparatörer",
    		searchFi: "71210 kattoasentajat ja -korjaajat rakennus remontti",
    		searchEn: "71210 roofers",
    		searchSv: "71210 takmontörer och takreparatörer",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "71220",
    		code: "71220",
    		labelFi: "Lattianpäällystystyöntekijät",
    		labelEn: "Floor layers and tile setters",
    		labelSv: "Golvläggare",
    		searchFi: "71220 lattianpäällystystyöntekijät rakennus remontti",
    		searchEn: "71220 floor layers and tile setters",
    		searchSv: "71220 golvläggare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71230",
    		code: "71230",
    		labelFi: "Rappaajat",
    		labelEn: "Plasterers",
    		labelSv: "Rappare",
    		searchFi: "71230 rappaajat rakennus remontti",
    		searchEn: "71230 plasterers",
    		searchSv: "71230 rappare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71240",
    		code: "71240",
    		labelFi: "Eristäjät",
    		labelEn: "Insulation workers",
    		labelSv: "Isoleringsmontörer",
    		searchFi: "71240 eristäjät rakennus remontti",
    		searchEn: "71240 insulation workers",
    		searchSv: "71240 isoleringsmontörer",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71250",
    		code: "71250",
    		labelFi: "Lasinasentajat",
    		labelEn: "Glaziers",
    		labelSv: "Glasmästare",
    		searchFi: "71250 lasinasentajat rakennus remontti",
    		searchEn: "71250 glaziers",
    		searchSv: "71250 glasmästare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71260",
    		code: "71260",
    		labelFi: "Putkiasentajat",
    		labelEn: "Plumbers and pipe fitters",
    		labelSv: "Rörläggare",
    		searchFi: "71260 putkiasentajat rakennus remontti",
    		searchEn: "71260 plumbers and pipe fitters",
    		searchSv: "71260 rörläggare",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71270",
    		code: "71270",
    		labelFi: "Ilmastointi- ja jäähdytyslaiteasentajat",
    		labelEn: "Air conditioning and refrigeration mechanics",
    		labelSv: "Ventilations- och kylmontörer",
    		searchFi: "71270 ilmastointi- ja jäähdytyslaiteasentajat",
    		searchEn: "71270 air conditioning and refrigeration mechanics",
    		searchSv: "71270 ventilations- och kylmontörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71310",
    		code: "71310",
    		labelFi: "Rakennusmaalarit ym.",
    		labelEn: "Painters and related workers",
    		labelSv: "Byggnadsmålare m.fl.",
    		searchFi: "71310 rakennusmaalarit ym. rakennus remontti",
    		searchEn: "71310 painters and related workers",
    		searchSv: "71310 byggnadsmålare m.fl.",
    		freeTextFi: "rakennus remontti",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "71320",
    		code: "71320",
    		labelFi: "Ruiskumaalaajat ja -lakkaajat",
    		labelEn: "Spray painters and varnishers",
    		labelSv: "Sprutlackerare",
    		searchFi: "71320 ruiskumaalaajat ja -lakkaajat",
    		searchEn: "71320 spray painters and varnishers",
    		searchSv: "71320 sprutlackerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "71330",
    		code: "71330",
    		labelFi: "Rakennuspuhdistajat ja nuohoojat",
    		labelEn: "Building structure cleaners",
    		labelSv: "Saneringsarbetare och sotare",
    		searchFi: "71330 rakennuspuhdistajat ja nuohoojat",
    		searchEn: "71330 building structure cleaners",
    		searchSv: "71330 saneringsarbetare och sotare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72110",
    		code: "72110",
    		labelFi: "Muotin- ja keernantekijät",
    		labelEn: "Metal moulders and coremakers",
    		labelSv: "Formare och kärnmakare",
    		searchFi: "72110 muotin- ja keernantekijät",
    		searchEn: "72110 metal moulders and coremakers",
    		searchSv: "72110 formare och kärnmakare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72120",
    		code: "72120",
    		labelFi: "Hitsaajat ja kaasuleikkaajat",
    		labelEn: "Welders and flamecutters",
    		labelSv: "Svetsare och gasskärare",
    		searchFi: "72120 hitsaajat ja kaasuleikkaajat",
    		searchEn: "72120 welders and flamecutters",
    		searchSv: "72120 svetsare och gasskärare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72130",
    		code: "72130",
    		labelFi: "Ohutlevysepät",
    		labelEn: "Sheet-metal workers",
    		labelSv: "Tunnplåtslagare",
    		searchFi: "72130 ohutlevysepät",
    		searchEn: "72130 sheet-metal workers",
    		searchSv: "72130 tunnplåtslagare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72140",
    		code: "72140",
    		labelFi: "Paksulevysepät ja rautarakennetyöntekijät",
    		labelEn: "Structural-metal preparers and erectors",
    		labelSv: "Grovplåtslagare och stålkonstruktionsmontörer",
    		searchFi: "72140 paksulevysepät ja rautarakennetyöntekijät",
    		searchEn: "72140 structural-metal preparers and erectors",
    		searchSv: "72140 grovplåtslagare och stålkonstruktionsmontörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72150",
    		code: "72150",
    		labelFi: "Kaapelin- ja köysienasentajat",
    		labelEn: "Riggers and cable splicers",
    		labelSv: "Kabelläggare och riggare",
    		searchFi: "72150 kaapelin- ja köysienasentajat",
    		searchEn: "72150 riggers and cable splicers",
    		searchSv: "72150 kabelläggare och riggare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72210",
    		code: "72210",
    		labelFi: "Sepät",
    		labelEn: "Blacksmiths, hammersmiths and forging press workers",
    		labelSv: "Smeder",
    		searchFi: "72210 sepät",
    		searchEn: "72210 blacksmiths, hammersmiths and forging press workers",
    		searchSv: "72210 smeder",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72220",
    		code: "72220",
    		labelFi: "Työkaluntekijät ja lukkosepät",
    		labelEn: "Toolmakers and related workers",
    		labelSv: "Verktygsmakare och låssmeder",
    		searchFi: "72220 työkaluntekijät ja lukkosepät",
    		searchEn: "72220 toolmakers and related workers",
    		searchSv: "72220 verktygsmakare och låssmeder",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72230",
    		code: "72230",
    		labelFi: "Koneenasettajat ja koneistajat",
    		labelEn: "Metal working machine tool setters and operators",
    		labelSv: "Maskinställare och verkstadsmekaniker",
    		searchFi: "72230 koneenasettajat ja koneistajat",
    		searchEn: "72230 metal working machine tool setters and operators",
    		searchSv: "72230 maskinställare och verkstadsmekaniker",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72240",
    		code: "72240",
    		labelFi: "Konehiojat, kiillottajat ja teroittajat",
    		labelEn: "Metal polishers, wheel grinders and tool sharpeners",
    		labelSv: "Maskinslipare, -polerare och -vässare",
    		searchFi: "72240 konehiojat, kiillottajat ja teroittajat",
    		searchEn: "72240 metal polishers, wheel grinders and tool sharpeners",
    		searchSv: "72240 maskinslipare, -polerare och -vässare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72310",
    		code: "72310",
    		labelFi: "Moottoriajoneuvojen asentajat ja korjaajat",
    		labelEn: "Motor vehicle mechanics and repairers",
    		labelSv: "Motorfordonsmontörer och -reparatörer",
    		searchFi: "72310 moottoriajoneuvojen asentajat ja korjaajat korjaamo auto",
    		searchEn: "72310 motor vehicle mechanics and repairers",
    		searchSv: "72310 motorfordonsmontörer och -reparatörer",
    		freeTextFi: "korjaamo auto",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "72320",
    		code: "72320",
    		labelFi: "Lentokoneasentajat ja -korjaajat",
    		labelEn: "Aircraft engine mechanics and repairers",
    		labelSv: "Flygmontörer och flygreparatörer",
    		searchFi: "72320 lentokoneasentajat ja -korjaajat",
    		searchEn: "72320 aircraft engine mechanics and repairers",
    		searchSv: "72320 flygmontörer och flygreparatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72330",
    		code: "72330",
    		labelFi: "Maatalous- ja teollisuuskoneasentajat ja -korjaajat",
    		labelEn: "Agricultural and industrial machinery mechanics and repairers",
    		labelSv: "Maskinmontörer och -reparatörer (jordbruks- och industrimaskiner)",
    		searchFi: "72330 maatalous- ja teollisuuskoneasentajat ja -korjaajat",
    		searchEn: "72330 agricultural and industrial machinery mechanics and repairers",
    		searchSv: "72330 maskinmontörer och -reparatörer (jordbruks- och industrimaskiner)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "72340",
    		code: "72340",
    		labelFi: "Polkupyöränkorjaajat ym.",
    		labelEn: "Bicycle and related repairers",
    		labelSv: "Cykelreparatörer m.fl.",
    		searchFi: "72340 polkupyöränkorjaajat ym.",
    		searchEn: "72340 bicycle and related repairers",
    		searchSv: "72340 cykelreparatörer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73110",
    		code: "73110",
    		labelFi: "Kellosepät ja muut hienomekaanisten instrumenttien tekijät ja korjaajat",
    		labelEn: "Precision-instrument makers and repairers",
    		labelSv: "Urmakare och övriga tillverkare av finmekaniska instrument",
    		searchFi: "73110 kellosepät ja muut hienomekaanisten instrumenttien tekijät ja korjaajat",
    		searchEn: "73110 precision-instrument makers and repairers",
    		searchSv: "73110 urmakare och övriga tillverkare av finmekaniska instrument",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73120",
    		code: "73120",
    		labelFi: "Soittimien tekijät ja virittäjät",
    		labelEn: "Musical instrument makers and tuners",
    		labelSv: "Musikinstrumentmakare och -stämmare",
    		searchFi: "73120 soittimien tekijät ja virittäjät",
    		searchEn: "73120 musical instrument makers and tuners",
    		searchSv: "73120 musikinstrumentmakare och -stämmare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73130",
    		code: "73130",
    		labelFi: "Koru-, kulta- ja hopeasepät",
    		labelEn: "Jewellery and precious-metal workers",
    		labelSv: "Juvel-, guld- och silversmeder",
    		searchFi: "73130 koru-, kulta- ja hopeasepät",
    		searchEn: "73130 jewellery and precious-metal workers",
    		searchSv: "73130 juvel-, guld- och silversmeder",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73140",
    		code: "73140",
    		labelFi: "Saven- ja tiilenvalajat ja dreijaajat",
    		labelEn: "Potters and related workers",
    		labelSv: "Ler- och tegelgjutare, drejare",
    		searchFi: "73140 saven- ja tiilenvalajat ja dreijaajat",
    		searchEn: "73140 potters and related workers",
    		searchSv: "73140 ler- och tegelgjutare, drejare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73150",
    		code: "73150",
    		labelFi: "Lasinpuhaltajat, -leikkaajat, -hiojat ja -viimeistelijät",
    		labelEn: "Glass makers, cutters, grinders and finishers",
    		labelSv: "Glasblåsare, -skärare, -slipare och -efterbehandlare",
    		searchFi: "73150 lasinpuhaltajat, -leikkaajat, -hiojat ja -viimeistelijät",
    		searchEn: "73150 glass makers, cutters, grinders and finishers",
    		searchSv: "73150 glasblåsare, -skärare, -slipare och -efterbehandlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73160",
    		code: "73160",
    		labelFi: "Kaivertajat, etsaajat ja koristemaalarit",
    		labelEn: "Sign writers, decorative painters, engravers and etchers",
    		labelSv: "Glasgravörer, glasetsare och skyltmålare",
    		searchFi: "73160 kaivertajat, etsaajat ja koristemaalarit",
    		searchEn: "73160 sign writers, decorative painters, engravers and etchers",
    		searchSv: "73160 glasgravörer, glasetsare och skyltmålare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73170",
    		code: "73170",
    		labelFi: "Puu-, kori- yms. käsityötuotteiden tekijät",
    		labelEn: "Handicraft workers in wood, basketry and related materials",
    		labelSv: "Konsthantverkare i trä m.m.",
    		searchFi: "73170 puu-, kori- yms. käsityötuotteiden tekijät",
    		searchEn: "73170 handicraft workers in wood, basketry and related materials",
    		searchSv: "73170 konsthantverkare i trä m.m.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73180",
    		code: "73180",
    		labelFi: "Tekstiili-, nahka- yms. käsityötuotteiden tekijät",
    		labelEn: "Handicraft workers in textile, leather and related materials",
    		labelSv: "Konsthantverkare i textil, läder m.m.",
    		searchFi: "73180 tekstiili-, nahka- yms. käsityötuotteiden tekijät",
    		searchEn: "73180 handicraft workers in textile, leather and related materials",
    		searchSv: "73180 konsthantverkare i textil, läder m.m.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73190",
    		code: "73190",
    		labelFi: "Muut käsityöntekijät",
    		labelEn: "Handicraft workers not elsewhere classified",
    		labelSv: "Övriga konsthantverkare",
    		searchFi: "73190 muut käsityöntekijät",
    		searchEn: "73190 handicraft workers not elsewhere classified",
    		searchSv: "73190 övriga konsthantverkare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73210",
    		code: "73210",
    		labelFi: "Painopinnanvalmistajat",
    		labelEn: "Pre-press technicians",
    		labelSv: "Prepressoperatörer",
    		searchFi: "73210 painopinnanvalmistajat",
    		searchEn: "73210 pre-press technicians",
    		searchSv: "73210 prepressoperatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73220",
    		code: "73220",
    		labelFi: "Painajat",
    		labelEn: "Printers",
    		labelSv: "Tryckare",
    		searchFi: "73220 painajat",
    		searchEn: "73220 printers",
    		searchSv: "73220 tryckare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "73230",
    		code: "73230",
    		labelFi: "Jälkikäsittelijät ja sitomotyöntekijät",
    		labelEn: "Print finishing and binding workers",
    		labelSv: "Efterbehandlare och bokbinderiarbetare",
    		searchFi: "73230 jälkikäsittelijät ja sitomotyöntekijät",
    		searchEn: "73230 print finishing and binding workers",
    		searchSv: "73230 efterbehandlare och bokbinderiarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "74110",
    		code: "74110",
    		labelFi: "Rakennussähköasentajat",
    		labelEn: "Building and related electricians",
    		labelSv: "Byggnadselmontörer",
    		searchFi: "74110 rakennussähköasentajat",
    		searchEn: "74110 building and related electricians",
    		searchSv: "74110 byggnadselmontörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "74120",
    		code: "74120",
    		labelFi: "Muut sähköasentajat",
    		labelEn: "Electrical mechanics and fitters",
    		labelSv: "Övriga elmontörer",
    		searchFi: "74120 muut sähköasentajat",
    		searchEn: "74120 electrical mechanics and fitters",
    		searchSv: "74120 övriga elmontörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "74130",
    		code: "74130",
    		labelFi: "Linja-asentajat ja -korjaajat",
    		labelEn: "Electrical line installers and repairers",
    		labelSv: "Linjemontörer och -reparatörer",
    		searchFi: "74130 linja-asentajat ja -korjaajat",
    		searchEn: "74130 electrical line installers and repairers",
    		searchSv: "74130 linjemontörer och -reparatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "74211",
    		code: "74211",
    		labelFi: "Elektroniikka-asentajat ja -korjaajat",
    		labelEn: "Electronics installers and repairers",
    		labelSv: "Elektronikmontörer och -reparatörer",
    		searchFi: "74211 elektroniikka-asentajat ja -korjaajat tietokone kodinkone",
    		searchEn: "74211 electronics installers and repairers",
    		searchSv: "74211 elektronikmontörer och -reparatörer",
    		freeTextFi: "tietokone kodinkone",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "74212",
    		code: "74212",
    		labelFi: "Automaatioasentajat ja -korjaajat",
    		labelEn: "Automation installers and repairers",
    		labelSv: "Automationsmontörer och -reparatörer",
    		searchFi: "74212 automaatioasentajat ja -korjaajat",
    		searchEn: "74212 automation installers and repairers",
    		searchSv: "74212 automationsmontörer och -reparatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "74220",
    		code: "74220",
    		labelFi: "Tieto- ja viestintäteknologian asentajat ja korjaajat",
    		labelEn: "Information and communications technology installers and servicers",
    		labelSv: "Montörer och reparatörer, IKT",
    		searchFi: "74220 tieto- ja viestintäteknologian asentajat ja korjaajat tietokone kodinkone",
    		searchEn: "74220 information and communications technology installers and servicers",
    		searchSv: "74220 montörer och reparatörer, ikt",
    		freeTextFi: "tietokone kodinkone",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75110",
    		code: "75110",
    		labelFi: "Lihanleikkaajat, kalankäsittelijät ym.",
    		labelEn: "Butchers, fishmongers and related food preparers",
    		labelSv: "Slaktare, fiskberedare m.fl.",
    		searchFi: "75110 lihanleikkaajat, kalankäsittelijät ym.",
    		searchEn: "75110 butchers, fishmongers and related food preparers",
    		searchSv: "75110 slaktare, fiskberedare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75120",
    		code: "75120",
    		labelFi: "Leipurit ja kondiittorit",
    		labelEn: "Bakers, pastry-cooks and confectionery makers",
    		labelSv: "Bagare och konditorer",
    		searchFi: "75120 leipurit ja kondiittorit",
    		searchEn: "75120 bakers, pastry-cooks and confectionery makers",
    		searchSv: "75120 bagare och konditorer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75130",
    		code: "75130",
    		labelFi: "Meijeristit, juustomestarit ym.",
    		labelEn: "Dairy-products makers",
    		labelSv: "Mejerister, ostmästare m.fl.",
    		searchFi: "75130 meijeristit, juustomestarit ym.",
    		searchEn: "75130 dairy-products makers",
    		searchSv: "75130 mejerister, ostmästare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75140",
    		code: "75140",
    		labelFi: "Hedelmä- ja vihannestuotteiden valmistajat",
    		labelEn: "Fruit, vegetable and related preservers",
    		labelSv: "Tillverkare av frukt- och grönsaksprodukter",
    		searchFi: "75140 hedelmä- ja vihannestuotteiden valmistajat",
    		searchEn: "75140 fruit, vegetable and related preservers",
    		searchSv: "75140 tillverkare av frukt- och grönsaksprodukter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75150",
    		code: "75150",
    		labelFi: "Ruokien ja juomien laaduntarkkailijat",
    		labelEn: "Food and beverage tasters and graders",
    		labelSv: "Kvalitetsbedömare (mat och dryck)",
    		searchFi: "75150 ruokien ja juomien laaduntarkkailijat",
    		searchEn: "75150 food and beverage tasters and graders",
    		searchSv: "75150 kvalitetsbedömare (mat och dryck)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75160",
    		code: "75160",
    		labelFi: "Tupakkatuotteiden valmistajat",
    		labelEn: "Tobacco preparers and tobacco products makers",
    		labelSv: "Tillverkare av tobaksprodukter",
    		searchFi: "75160 tupakkatuotteiden valmistajat",
    		searchEn: "75160 tobacco preparers and tobacco products makers",
    		searchSv: "75160 tillverkare av tobaksprodukter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75210",
    		code: "75210",
    		labelFi: "Raakapuun käsittelijät",
    		labelEn: "Wood treaters",
    		labelSv: "Trävaruhanterare",
    		searchFi: "75210 raakapuun käsittelijät",
    		searchEn: "75210 wood treaters",
    		searchSv: "75210 trävaruhanterare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75220",
    		code: "75220",
    		labelFi: "Huonekalupuusepät ym.",
    		labelEn: "Cabinet-makers and related workers",
    		labelSv: "Möbelsnickare m.fl.",
    		searchFi: "75220 huonekalupuusepät ym.",
    		searchEn: "75220 cabinet-makers and related workers",
    		searchSv: "75220 möbelsnickare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75230",
    		code: "75230",
    		labelFi: "Konepuusepät",
    		labelEn: "Woodworking-machine tool setters and operators",
    		labelSv: "Maskinsnickare",
    		searchFi: "75230 konepuusepät",
    		searchEn: "75230 woodworking-machine tool setters and operators",
    		searchSv: "75230 maskinsnickare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75310",
    		code: "75310",
    		labelFi: "Vaatturit, pukuompelijat, turkkurit ja hatuntekijät",
    		labelEn: "Tailors, dressmakers, furriers and hatters",
    		labelSv: "Skräddare, ateljésömmerskor, körsnärer och modister",
    		searchFi: "75310 vaatturit, pukuompelijat, turkkurit ja hatuntekijät",
    		searchEn: "75310 tailors, dressmakers, furriers and hatters",
    		searchSv: "75310 skräddare, ateljésömmerskor, körsnärer och modister",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75320",
    		code: "75320",
    		labelFi: "Leikkaajat ja mallimestarit",
    		labelEn: "Garment and related pattern-makers and cutters",
    		labelSv: "Tillskärare och modellmästare",
    		searchFi: "75320 leikkaajat ja mallimestarit",
    		searchEn: "75320 garment and related pattern-makers and cutters",
    		searchSv: "75320 tillskärare och modellmästare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75330",
    		code: "75330",
    		labelFi: "Koru- ja muut tekstiiliompelijat",
    		labelEn: "Sewing, embroidery and related workers",
    		labelSv: "Broderare och andra textilsömmare",
    		searchFi: "75330 koru- ja muut tekstiiliompelijat",
    		searchEn: "75330 sewing, embroidery and related workers",
    		searchSv: "75330 broderare och andra textilsömmare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75340",
    		code: "75340",
    		labelFi: "Verhoilijat",
    		labelEn: "Upholsterers and related workers",
    		labelSv: "Tapetserare",
    		searchFi: "75340 verhoilijat",
    		searchEn: "75340 upholsterers and related workers",
    		searchSv: "75340 tapetserare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75350",
    		code: "75350",
    		labelFi: "Turkisten muokkaajat ja nahkurit",
    		labelEn: "Pelt dressers, tanners and fellmongers",
    		labelSv: "Garvare och skinnberedare",
    		searchFi: "75350 turkisten muokkaajat ja nahkurit",
    		searchEn: "75350 pelt dressers, tanners and fellmongers",
    		searchSv: "75350 garvare och skinnberedare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75360",
    		code: "75360",
    		labelFi: "Suutarit ym.",
    		labelEn: "Shoemakers and related workers",
    		labelSv: "Skomakare m.fl.",
    		searchFi: "75360 suutarit ym.",
    		searchEn: "75360 shoemakers and related workers",
    		searchSv: "75360 skomakare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75410",
    		code: "75410",
    		labelFi: "Vedenalaistyöntekijät",
    		labelEn: "Underwater divers",
    		labelSv: "Dykare",
    		searchFi: "75410 vedenalaistyöntekijät",
    		searchEn: "75410 underwater divers",
    		searchSv: "75410 dykare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75420",
    		code: "75420",
    		labelFi: "Panostajat ja räjäyttäjät",
    		labelEn: "Shotfirers and blasters",
    		labelSv: "Laddare och sprängare",
    		searchFi: "75420 panostajat ja räjäyttäjät",
    		searchEn: "75420 shotfirers and blasters",
    		searchSv: "75420 laddare och sprängare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75430",
    		code: "75430",
    		labelFi: "Luokittelijat ja laaduntarkkailijat (pl. ruoat ja juomat)",
    		labelEn: "Product graders and testers (excluding foods and beverages)",
    		labelSv: "Klassificerare och kvalitetskontrollanter (utom drycker och livsmedel)",
    		searchFi: "75430 luokittelijat ja laaduntarkkailijat (pl. ruoat ja juomat)",
    		searchEn: "75430 product graders and testers (excluding foods and beverages)",
    		searchSv: "75430 klassificerare och kvalitetskontrollanter (utom drycker och livsmedel)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75440",
    		code: "75440",
    		labelFi: "Savupuhdistajat, tuholais- ja rikkakasvintorjujat",
    		labelEn: "Fumigators and other pest and weed controllers",
    		labelSv: "Röksanerare, skadedjurs- och ogrässanerare",
    		searchFi: "75440 savupuhdistajat, tuholais- ja rikkakasvintorjujat",
    		searchEn: "75440 fumigators and other pest and weed controllers",
    		searchSv: "75440 röksanerare, skadedjurs- och ogrässanerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "75490",
    		code: "75490",
    		labelFi: "Muut pääluokkaan 7 muualla luokittelemattomat työntekijät",
    		labelEn: "Craft and related workers not elsewhere classified",
    		labelSv: "Övrig personal, oklassificerad under andra moment i huvudgrupp 7",
    		searchFi: "75490 muut pääluokkaan 7 muualla luokittelemattomat työntekijät",
    		searchEn: "75490 craft and related workers not elsewhere classified",
    		searchSv: "75490 övrig personal, oklassificerad under andra moment i huvudgrupp 7",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81110",
    		code: "81110",
    		labelFi: "Kaivos- ja louhostyöntekijät",
    		labelEn: "Miners and quarriers",
    		labelSv: "Gruv- och stenbrottsarbetare",
    		searchFi: "81110 kaivos- ja louhostyöntekijät",
    		searchEn: "81110 miners and quarriers",
    		searchSv: "81110 gruv- och stenbrottsarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81120",
    		code: "81120",
    		labelFi: "Rikastustyöntekijät",
    		labelEn: "Mineral and stone processing plant operators",
    		labelSv: "Anrikningsarbetare",
    		searchFi: "81120 rikastustyöntekijät",
    		searchEn: "81120 mineral and stone processing plant operators",
    		searchSv: "81120 anrikningsarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81130",
    		code: "81130",
    		labelFi: "Iskuporaajat ja syväkairaajat",
    		labelEn: "Well drillers and borers and related workers",
    		labelSv: "Slagborrare och djupborrare",
    		searchFi: "81130 iskuporaajat ja syväkairaajat",
    		searchEn: "81130 well drillers and borers and related workers",
    		searchSv: "81130 slagborrare och djupborrare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81140",
    		code: "81140",
    		labelFi: "Betonituote- ym. teollisuuden prosessityöntekijät",
    		labelEn: "Cement, stone and other mineral products machine operators",
    		labelSv: "Processoperatörer inom betongindustri m.fl.",
    		searchFi: "81140 betonituote- ym. teollisuuden prosessityöntekijät",
    		searchEn: "81140 cement, stone and other mineral products machine operators",
    		searchSv: "81140 processoperatörer inom betongindustri m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81210",
    		code: "81210",
    		labelFi: "Metalliteollisuuden prosessityöntekijät",
    		labelEn: "Metal processing plant operators",
    		labelSv: "Processoperatörer inom metallindustri",
    		searchFi: "81210 metalliteollisuuden prosessityöntekijät",
    		searchEn: "81210 metal processing plant operators",
    		searchSv: "81210 processoperatörer inom metallindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81220",
    		code: "81220",
    		labelFi: "Metallien teolliset päällystäjät ja viimeistelijät",
    		labelEn: "Metal finishing, plating and coating machine operators",
    		labelSv: "Ytbeläggare och efterbehandlare inom metallindustri",
    		searchFi: "81220 metallien teolliset päällystäjät ja viimeistelijät",
    		searchEn: "81220 metal finishing, plating and coating machine operators",
    		searchSv: "81220 ytbeläggare och efterbehandlare inom metallindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81310",
    		code: "81310",
    		labelFi: "Kemianteollisuuden prosessityöntekijät ym.",
    		labelEn: "Chemical products plant and machine operators",
    		labelSv: "Processoperatörer, kemisk industri m.fl.",
    		searchFi: "81310 kemianteollisuuden prosessityöntekijät ym.",
    		searchEn: "81310 chemical products plant and machine operators",
    		searchSv: "81310 processoperatörer, kemisk industri m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81320",
    		code: "81320",
    		labelFi: "Valokuvatuotteiden valmistuksen prosessityöntekijät",
    		labelEn: "Photographic products machine operators",
    		labelSv: "Processoperatörer, fotografiska produkter",
    		searchFi: "81320 valokuvatuotteiden valmistuksen prosessityöntekijät",
    		searchEn: "81320 photographic products machine operators",
    		searchSv: "81320 processoperatörer, fotografiska produkter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81410",
    		code: "81410",
    		labelFi: "Kumituoteteollisuuden prosessityöntekijät",
    		labelEn: "Rubber products machine operators",
    		labelSv: "Processoperatörer, gummiindustri",
    		searchFi: "81410 kumituoteteollisuuden prosessityöntekijät",
    		searchEn: "81410 rubber products machine operators",
    		searchSv: "81410 processoperatörer, gummiindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81420",
    		code: "81420",
    		labelFi: "Muovituoteteollisuuden prosessityöntekijät",
    		labelEn: "Plastic products machine operators",
    		labelSv: "Processoperatörer, plastindustri",
    		searchFi: "81420 muovituoteteollisuuden prosessityöntekijät",
    		searchEn: "81420 plastic products machine operators",
    		searchSv: "81420 processoperatörer, plastindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81430",
    		code: "81430",
    		labelFi: "Paperituoteteollisuuden prosessityöntekijät",
    		labelEn: "Paper products machine operators",
    		labelSv: "Processoperatörer, pappersvaruindustri",
    		searchFi: "81430 paperituoteteollisuuden prosessityöntekijät",
    		searchEn: "81430 paper products machine operators",
    		searchSv: "81430 processoperatörer, pappersvaruindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81510",
    		code: "81510",
    		labelFi: "Kuitujenkäsittely-, kehruu- ja puolauskoneiden hoitajat",
    		labelEn: "Fibre preparing, spinning and winding machine operators",
    		labelSv: "Maskinoperatörer, fibertillverkning, spinning och spolning",
    		searchFi: "81510 kuitujenkäsittely-, kehruu- ja puolauskoneiden hoitajat",
    		searchEn: "81510 fibre preparing, spinning and winding machine operators",
    		searchSv: "81510 maskinoperatörer, fibertillverkning, spinning och spolning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81520",
    		code: "81520",
    		labelFi: "Kutoma- ja neulekoneiden hoitajat",
    		labelEn: "Weaving and knitting machine operators",
    		labelSv: "Maskinoperatörer, vävning och stickning",
    		searchFi: "81520 kutoma- ja neulekoneiden hoitajat",
    		searchEn: "81520 weaving and knitting machine operators",
    		searchSv: "81520 maskinoperatörer, vävning och stickning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81530",
    		code: "81530",
    		labelFi: "Teollisuusompelijat",
    		labelEn: "Sewing machine operators",
    		labelSv: "Symaskinsoperatörer",
    		searchFi: "81530 teollisuusompelijat",
    		searchEn: "81530 sewing machine operators",
    		searchSv: "81530 symaskinsoperatörer",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81540",
    		code: "81540",
    		labelFi: "Valkaisu-, värjäys- ja puhdistuskoneiden hoitajat",
    		labelEn: "Bleaching, dyeing and fabric cleaning machine operators",
    		labelSv: "Maskinoperatörer, tvättning, blekning och färgning",
    		searchFi: "81540 valkaisu-, värjäys- ja puhdistuskoneiden hoitajat",
    		searchEn: "81540 bleaching, dyeing and fabric cleaning machine operators",
    		searchSv: "81540 maskinoperatörer, tvättning, blekning och färgning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81550",
    		code: "81550",
    		labelFi: "Turkisten ja nahkojen teolliset käsittelijät, värjääjät ym.",
    		labelEn: "Fur and leather preparing machine operators",
    		labelSv: "Maskinoperatörer, pälsskinnsberedning, färgning",
    		searchFi: "81550 turkisten ja nahkojen teolliset käsittelijät, värjääjät ym.",
    		searchEn: "81550 fur and leather preparing machine operators",
    		searchSv: "81550 maskinoperatörer, pälsskinnsberedning, färgning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81560",
    		code: "81560",
    		labelFi: "Jalkine- ja laukkuteollisuuden prosessityöntekijät",
    		labelEn: "Shoemaking and related machine operators",
    		labelSv: "Processoperatörer, sko- och väskindustri",
    		searchFi: "81560 jalkine- ja laukkuteollisuuden prosessityöntekijät",
    		searchEn: "81560 shoemaking and related machine operators",
    		searchSv: "81560 processoperatörer, sko- och väskindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81570",
    		code: "81570",
    		labelFi: "Pesulatyöntekijät",
    		labelEn: "Laundry machine operators",
    		labelSv: "Tvätteripersonal",
    		searchFi: "81570 pesulatyöntekijät",
    		searchEn: "81570 laundry machine operators",
    		searchSv: "81570 tvätteripersonal",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81590",
    		code: "81590",
    		labelFi: "Muut tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät",
    		labelEn: "Textile, fur and leather products machine operators not elsewhere classified",
    		labelSv: "Övriga processoperatörer inom textil-, skinn- och läderindustri",
    		searchFi: "81590 muut tekstiili-, turkis- ja nahkatuoteteollisuuden prosessityöntekijät",
    		searchEn: "81590 textile, fur and leather products machine operators not elsewhere classified",
    		searchSv: "81590 övriga processoperatörer inom textil-, skinn- och läderindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81600",
    		code: "81600",
    		labelFi: "Elintarviketeollisuuden prosessityöntekijät",
    		labelEn: "Food and related products machine operators",
    		labelSv: "Processoperatörer, livsmedelsindustri",
    		searchFi: "81600 elintarviketeollisuuden prosessityöntekijät",
    		searchEn: "81600 food and related products machine operators",
    		searchSv: "81600 processoperatörer, livsmedelsindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81710",
    		code: "81710",
    		labelFi: "Paperimassan sekä paperin ja kartongin valmistuksen prosessityöntekijät",
    		labelEn: "Pulp and papermaking plant operators",
    		labelSv: "Processoperatörer, pappers-, kartong- och massaindustri",
    		searchFi: "81710 paperimassan sekä paperin ja kartongin valmistuksen prosessityöntekijät",
    		searchEn: "81710 pulp and papermaking plant operators",
    		searchSv: "81710 processoperatörer, pappers-, kartong- och massaindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81720",
    		code: "81720",
    		labelFi: "Puu- ja sahatavaran prosessityöntekijät",
    		labelEn: "Wood processing plant operators",
    		labelSv: "Processoperatörer, trä- och sågvaror",
    		searchFi: "81720 puu- ja sahatavaran prosessityöntekijät",
    		searchEn: "81720 wood processing plant operators",
    		searchSv: "81720 processoperatörer, trä- och sågvaror",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81810",
    		code: "81810",
    		labelFi: "Lasi- ja keramiikkateollisuuden uunienhoitajat",
    		labelEn: "Glass and ceramics plant operators",
    		labelSv: "Ugnsoperatörer, glas- och keramikindustri",
    		searchFi: "81810 lasi- ja keramiikkateollisuuden uunienhoitajat",
    		searchEn: "81810 glass and ceramics plant operators",
    		searchSv: "81810 ugnsoperatörer, glas- och keramikindustri",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81821",
    		code: "81821",
    		labelFi: "Voima- ja jätteenkäsittelylaitosten laitosmiehet",
    		labelEn: "Power-production and waste treatment plant workers",
    		labelSv: "Maskinreparatörer, kraftverk och avfallsbehandlingsanläggningar",
    		searchFi: "81821 voima- ja jätteenkäsittelylaitosten laitosmiehet",
    		searchEn: "81821 power-production and waste treatment plant workers",
    		searchSv: "81821 maskinreparatörer, kraftverk och avfallsbehandlingsanläggningar",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81822",
    		code: "81822",
    		labelFi: "Muut laitosmiehet ym.",
    		labelEn: "Other plant workers, etc.",
    		labelSv: "Övriga maskinreparatörer m.fl.",
    		searchFi: "81822 muut laitosmiehet ym.",
    		searchEn: "81822 other plant workers, etc.",
    		searchSv: "81822 övriga maskinreparatörer m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81830",
    		code: "81830",
    		labelFi: "Pakkaus-, pullotus- ja etiköintikoneiden hoitajat",
    		labelEn: "Packing, bottling and labelling machine operators",
    		labelSv: "Maskinoperatörer, packning, påfyllning och märkning",
    		searchFi: "81830 pakkaus-, pullotus- ja etiköintikoneiden hoitajat",
    		searchEn: "81830 packing, bottling and labelling machine operators",
    		searchSv: "81830 maskinoperatörer, packning, påfyllning och märkning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "81890",
    		code: "81890",
    		labelFi: "Muut muualla luokittelemattomat prosessityöntekijät",
    		labelEn: "Stationary plant and machine operators not elsewhere classified",
    		labelSv: "Övriga processoperatörer utan annan klassificering",
    		searchFi: "81890 muut muualla luokittelemattomat prosessityöntekijät",
    		searchEn: "81890 stationary plant and machine operators not elsewhere classified",
    		searchSv: "81890 övriga processoperatörer utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "82110",
    		code: "82110",
    		labelFi: "Konepaja- ja metallituotteiden kokoonpanijat",
    		labelEn: "Mechanical machinery assemblers",
    		labelSv: "Hopsättare, verkstads- och metallprodukter",
    		searchFi: "82110 konepaja- ja metallituotteiden kokoonpanijat",
    		searchEn: "82110 mechanical machinery assemblers",
    		searchSv: "82110 hopsättare, verkstads- och metallprodukter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "82120",
    		code: "82120",
    		labelFi: "Sähkö- ja elektroniikkalaitteiden kokoonpanijat",
    		labelEn: "Electrical and electronic equipment assemblers",
    		labelSv: "Hopsättare, elektrisk och elektronisk utrustning",
    		searchFi: "82120 sähkö- ja elektroniikkalaitteiden kokoonpanijat",
    		searchEn: "82120 electrical and electronic equipment assemblers",
    		searchSv: "82120 hopsättare, elektrisk och elektronisk utrustning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "82190",
    		code: "82190",
    		labelFi: "Muut teollisuustuotteiden kokoonpanijat",
    		labelEn: "Assemblers not elsewhere classified",
    		labelSv: "Övriga hopsättare av industriprodukter",
    		searchFi: "82190 muut teollisuustuotteiden kokoonpanijat",
    		searchEn: "82190 assemblers not elsewhere classified",
    		searchSv: "82190 övriga hopsättare av industriprodukter",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83110",
    		code: "83110",
    		labelFi: "Veturinkuljettajat",
    		labelEn: "Locomotive engine drivers",
    		labelSv: "Lokförare",
    		searchFi: "83110 veturinkuljettajat",
    		searchEn: "83110 locomotive engine drivers",
    		searchSv: "83110 lokförare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83120",
    		code: "83120",
    		labelFi: "Jarru-, turvalaite- ja vaihdetyöntekijät",
    		labelEn: "Railway brake, signal and switch operators",
    		labelSv: "Bangårdspersonal m.fl.",
    		searchFi: "83120 jarru-, turvalaite- ja vaihdetyöntekijät",
    		searchEn: "83120 railway brake, signal and switch operators",
    		searchSv: "83120 bangårdspersonal m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83210",
    		code: "83210",
    		labelFi: "Moottoripyörälähetit yms.",
    		labelEn: "Motorcycle drivers",
    		labelSv: "Motorcykelbud m.fl.",
    		searchFi: "83210 moottoripyörälähetit yms.",
    		searchEn: "83210 motorcycle drivers",
    		searchSv: "83210 motorcykelbud m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83220",
    		code: "83220",
    		labelFi: "Henkilö-, taksi- ja pakettiautonkuljettajat",
    		labelEn: "Car, taxi and van drivers",
    		labelSv: "Person-, taxi- och paketbilsförare",
    		searchFi: "83220 henkilö-, taksi- ja pakettiautonkuljettajat taxi",
    		searchEn: "83220 car, taxi and van drivers",
    		searchSv: "83220 person-, taxi- och paketbilsförare",
    		freeTextFi: "taxi",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "83310",
    		code: "83310",
    		labelFi: "Linja-auton- ja raitiovaununkuljettajat",
    		labelEn: "Bus and tram drivers",
    		labelSv: "Buss- och spårvagnsförare",
    		searchFi: "83310 linja-auton- ja raitiovaununkuljettajat",
    		searchEn: "83310 bus and tram drivers",
    		searchSv: "83310 buss- och spårvagnsförare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83320",
    		code: "83320",
    		labelFi: "Kuorma-auton ja erikoisajoneuvojen kuljettajat",
    		labelEn: "Heavy truck and lorry drivers",
    		labelSv: "Lastbils- och specialfordonsförare",
    		searchFi: "83320 kuorma-auton ja erikoisajoneuvojen kuljettajat",
    		searchEn: "83320 heavy truck and lorry drivers",
    		searchSv: "83320 lastbils- och specialfordonsförare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "83410",
    		code: "83410",
    		labelFi: "Maa- ja metsätaloustyökoneiden kuljettajat",
    		labelEn: "Mobile farm and forestry plant operators",
    		labelSv: "Förare av jordbruks- och skogsmaskiner",
    		searchFi: "83410 maa- ja metsätaloustyökoneiden kuljettajat",
    		searchEn: "83410 mobile farm and forestry plant operators",
    		searchSv: "83410 förare av jordbruks- och skogsmaskiner",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "83420",
    		code: "83420",
    		labelFi: "Maansiirtokoneiden ym. kuljettajat",
    		labelEn: "Earthmoving and related plant operators",
    		labelSv: "Anläggningsmaskinförare m.fl.",
    		searchFi: "83420 maansiirtokoneiden ym. kuljettajat",
    		searchEn: "83420 earthmoving and related plant operators",
    		searchSv: "83420 anläggningsmaskinförare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83430",
    		code: "83430",
    		labelFi: "Nosturinkuljettajat",
    		labelEn: "Crane, hoist and related plant operators",
    		labelSv: "Kranförare",
    		searchFi: "83430 nosturinkuljettajat",
    		searchEn: "83430 crane, hoist and related plant operators",
    		searchSv: "83430 kranförare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83441",
    		code: "83441",
    		labelFi: "Ahtaajat",
    		labelEn: "Stevedores",
    		labelSv: "Stuveriarbetare",
    		searchFi: "83441 ahtaajat",
    		searchEn: "83441 stevedores",
    		searchSv: "83441 stuveriarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83442",
    		code: "83442",
    		labelFi: "Trukinkuljettajat ym.",
    		labelEn: "Lifting-truck operators, etc.",
    		labelSv: "Truckförare m.fl.",
    		searchFi: "83442 trukinkuljettajat ym.",
    		searchEn: "83442 lifting-truck operators, etc.",
    		searchSv: "83442 truckförare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "83500",
    		code: "83500",
    		labelFi: "Kansimiehistö ym. vesiliikenteen työntekijät",
    		labelEn: "Ships' deck crews and related workers",
    		labelSv: "Däckmanskap, sjötrafikarbetare m.fl.",
    		searchFi: "83500 kansimiehistö ym. vesiliikenteen työntekijät",
    		searchEn: "83500 ships' deck crews and related workers",
    		searchSv: "83500 däckmanskap, sjötrafikarbetare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91110",
    		code: "91110",
    		labelFi: "Kotiapulaiset ja -siivoojat",
    		labelEn: "Domestic cleaners and helpers",
    		labelSv: "Hembiträden och städare",
    		searchFi: "91110 kotiapulaiset ja -siivoojat",
    		searchEn: "91110 domestic cleaners and helpers",
    		searchSv: "91110 hembiträden och städare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "household"
    	},
    	{
    		id: "91121",
    		code: "91121",
    		labelFi: "Toimistosiivoojat ym.",
    		labelEn: "Office cleaners, etc.",
    		labelSv: "Kontorsstädare m.fl.",
    		searchFi: "91121 toimistosiivoojat ym.",
    		searchEn: "91121 office cleaners, etc.",
    		searchSv: "91121 kontorsstädare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "91122",
    		code: "91122",
    		labelFi: "Hotellisiivoojat",
    		labelEn: "Hotel cleaners",
    		labelSv: "Hotellstädare",
    		searchFi: "91122 hotellisiivoojat",
    		searchEn: "91122 hotel cleaners",
    		searchSv: "91122 hotellstädare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91123",
    		code: "91123",
    		labelFi: "Sairaala- ja laitosapulaiset",
    		labelEn: "Hospital and institutional helpers",
    		labelSv: "Sjukhus- och vårdbiträden",
    		searchFi: "91123 sairaala- ja laitosapulaiset",
    		searchEn: "91123 hospital and institutional helpers",
    		searchSv: "91123 sjukhus- och vårdbiträden",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91124",
    		code: "91124",
    		labelFi: "Päiväkotiapulaiset",
    		labelEn: "Kindergarten assistants",
    		labelSv: "Daghemsbiträden",
    		searchFi: "91124 päiväkotiapulaiset",
    		searchEn: "91124 kindergarten assistants",
    		searchSv: "91124 daghemsbiträden",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91129",
    		code: "91129",
    		labelFi: "Muut muualla luokittelemattomat siivoojat",
    		labelEn: "Other cleaners not elsewhere classified",
    		labelSv: "Övriga städare utan annan klassificering",
    		searchFi: "91129 muut muualla luokittelemattomat siivoojat",
    		searchEn: "91129 other cleaners not elsewhere classified",
    		searchSv: "91129 övriga städare utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91210",
    		code: "91210",
    		labelFi: "Puhdistajat ja prässääjät",
    		labelEn: "Hand launderers and pressers",
    		labelSv: "Rengörare och pressare",
    		searchFi: "91210 puhdistajat ja prässääjät",
    		searchEn: "91210 hand launderers and pressers",
    		searchSv: "91210 rengörare och pressare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91220",
    		code: "91220",
    		labelFi: "Ajoneuvojen pesijät",
    		labelEn: "Vehicle cleaners",
    		labelSv: "Fordonstvättare",
    		searchFi: "91220 ajoneuvojen pesijät",
    		searchEn: "91220 vehicle cleaners",
    		searchSv: "91220 fordonstvättare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91230",
    		code: "91230",
    		labelFi: "Ikkunanpesijät",
    		labelEn: "Window cleaners",
    		labelSv: "Fönsterputsare",
    		searchFi: "91230 ikkunanpesijät",
    		searchEn: "91230 window cleaners",
    		searchSv: "91230 fönsterputsare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "91290",
    		code: "91290",
    		labelFi: "Muut puhdistustyöntekijät",
    		labelEn: "Other cleaning workers",
    		labelSv: "Övriga rengöringsarbetare",
    		searchFi: "91290 muut puhdistustyöntekijät",
    		searchEn: "91290 other cleaning workers",
    		searchSv: "91290 övriga rengöringsarbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92110",
    		code: "92110",
    		labelFi: "Maanviljelyn avustavat työntekijät",
    		labelEn: "Crop farm labourers",
    		labelSv: "Medhjälpare, jordbruk",
    		searchFi: "92110 maanviljelyn avustavat työntekijät",
    		searchEn: "92110 crop farm labourers",
    		searchSv: "92110 medhjälpare, jordbruk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92120",
    		code: "92120",
    		labelFi: "Karjankasvatuksen avustavat työntekijät",
    		labelEn: "Livestock farm labourers",
    		labelSv: "Medhjälpare, djuruppfödning",
    		searchFi: "92120 karjankasvatuksen avustavat työntekijät",
    		searchEn: "92120 livestock farm labourers",
    		searchSv: "92120 medhjälpare, djuruppfödning",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92130",
    		code: "92130",
    		labelFi: "Yhdistetyn maanviljelyn ja karjankasvatuksen avustavat työntekijät",
    		labelEn: "Mixed crop and livestock farm labourers",
    		labelSv: "Medhjälpare, växtodling och djuruppfödning, blandad drift",
    		searchFi: "92130 yhdistetyn maanviljelyn ja karjankasvatuksen avustavat työntekijät",
    		searchEn: "92130 mixed crop and livestock farm labourers",
    		searchSv: "92130 medhjälpare, växtodling och djuruppfödning, blandad drift",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92140",
    		code: "92140",
    		labelFi: "Avustavat puutarhatyöntekijät",
    		labelEn: "Garden and horticultural labourers",
    		labelSv: "Medhjälpare, trädgårdsodling",
    		searchFi: "92140 avustavat puutarhatyöntekijät",
    		searchEn: "92140 garden and horticultural labourers",
    		searchSv: "92140 medhjälpare, trädgårdsodling",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92150",
    		code: "92150",
    		labelFi: "Metsätalouden avustavat työntekijät",
    		labelEn: "Forestry labourers",
    		labelSv: "Medhjälpare, skogsbruk",
    		searchFi: "92150 metsätalouden avustavat työntekijät",
    		searchEn: "92150 forestry labourers",
    		searchSv: "92150 medhjälpare, skogsbruk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "92160",
    		code: "92160",
    		labelFi: "Kalatalouden ja vesiviljelyn avustavat työntekijät",
    		labelEn: "Fishery and aquaculture labourers",
    		labelSv: "Medhjälpare, fiske och vattenbruk",
    		searchFi: "92160 kalatalouden ja vesiviljelyn avustavat työntekijät",
    		searchEn: "92160 fishery and aquaculture labourers",
    		searchSv: "92160 medhjälpare, fiske och vattenbruk",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93110",
    		code: "93110",
    		labelFi: "Kaivosten avustavat työntekijät",
    		labelEn: "Mining and quarrying labourers",
    		labelSv: "Medhjälpare, gruvor",
    		searchFi: "93110 kaivosten avustavat työntekijät",
    		searchEn: "93110 mining and quarrying labourers",
    		searchSv: "93110 medhjälpare, gruvor",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93120",
    		code: "93120",
    		labelFi: "Maa- ja vesirakentamisen avustavat työntekijät",
    		labelEn: "Civil engineering labourers",
    		labelSv: "Medhjälpare, anläggningsverksamhet",
    		searchFi: "93120 maa- ja vesirakentamisen avustavat työntekijät",
    		searchEn: "93120 civil engineering labourers",
    		searchSv: "93120 medhjälpare, anläggningsverksamhet",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93130",
    		code: "93130",
    		labelFi: "Rakennusalan avustavat työntekijät",
    		labelEn: "Building construction labourers",
    		labelSv: "Medhjälpare, byggnadsverksamhet",
    		searchFi: "93130 rakennusalan avustavat työntekijät",
    		searchEn: "93130 building construction labourers",
    		searchSv: "93130 medhjälpare, byggnadsverksamhet",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93210",
    		code: "93210",
    		labelFi: "Käsinpakkaajat",
    		labelEn: "Hand packers",
    		labelSv: "Handpaketerare",
    		searchFi: "93210 käsinpakkaajat",
    		searchEn: "93210 hand packers",
    		searchSv: "93210 handpaketerare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93290",
    		code: "93290",
    		labelFi: "Muut valmistusalan avustavat työntekijät",
    		labelEn: "Manufacturing labourers not elsewhere classified",
    		labelSv: "Övriga medhjälpare inom industrin",
    		searchFi: "93290 muut valmistusalan avustavat työntekijät",
    		searchEn: "93290 manufacturing labourers not elsewhere classified",
    		searchSv: "93290 övriga medhjälpare inom industrin",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93310",
    		code: "93310",
    		labelFi: "Käsivetoisten- ja poljinkulkuneuvojen kuljettajat",
    		labelEn: "Hand and pedal vehicle drivers",
    		labelSv: "Förare av hand- och pedaldrivna bilar",
    		searchFi: "93310 käsivetoisten- ja poljinkulkuneuvojen kuljettajat",
    		searchEn: "93310 hand and pedal vehicle drivers",
    		searchSv: "93310 förare av hand- och pedaldrivna bilar",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93320",
    		code: "93320",
    		labelFi: "Eläinvetoisten kulkuneuvojen kuljettajat",
    		labelEn: "Drivers of animal-drawn vehicles and machinery",
    		labelSv: "Förare av djurdragna fordon",
    		searchFi: "93320 eläinvetoisten kulkuneuvojen kuljettajat",
    		searchEn: "93320 drivers of animal-drawn vehicles and machinery",
    		searchSv: "93320 förare av djurdragna fordon",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "93330",
    		code: "93330",
    		labelFi: "Rahdinkäsittelijät, varastotyöntekijät ym.",
    		labelEn: "Freight handlers",
    		labelSv: "Godshanterare, lagerarbetare m.fl.",
    		searchFi: "93330 rahdinkäsittelijät, varastotyöntekijät ym.",
    		searchEn: "93330 freight handlers",
    		searchSv: "93330 godshanterare, lagerarbetare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "93340",
    		code: "93340",
    		labelFi: "Hyllyjen täyttäjät",
    		labelEn: "Shelf fillers",
    		labelSv: "Varupåfyllare",
    		searchFi: "93340 hyllyjen täyttäjät",
    		searchEn: "93340 shelf fillers",
    		searchSv: "93340 varupåfyllare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "94110",
    		code: "94110",
    		labelFi: "Pikaruokatyöntekijät",
    		labelEn: "Fast food preparers",
    		labelSv: "Personal inom snabbmatssektorn",
    		searchFi: "94110 pikaruokatyöntekijät",
    		searchEn: "94110 fast food preparers",
    		searchSv: "94110 personal inom snabbmatssektorn",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "94120",
    		code: "94120",
    		labelFi: "Avustavat keittiötyöntekijät",
    		labelEn: "Kitchen helpers",
    		labelSv: "Köksbiträden",
    		searchFi: "94120 avustavat keittiötyöntekijät",
    		searchEn: "94120 kitchen helpers",
    		searchSv: "94120 köksbiträden",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: "company"
    	},
    	{
    		id: "95100",
    		code: "95100",
    		labelFi: "Mainosten jakajat, kengänkiillottajat ym.",
    		labelEn: "Street and related service workers",
    		labelSv: "Reklamutdelare, skoputsare m.fl.",
    		searchFi: "95100 mainosten jakajat, kengänkiillottajat ym.",
    		searchEn: "95100 street and related service workers",
    		searchSv: "95100 reklamutdelare, skoputsare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "95200",
    		code: "95200",
    		labelFi: "Katumyyjät (pl. elintarvikkeet)",
    		labelEn: "Street vendors (excluding food)",
    		labelSv: "Gatuförsäljare (utom livsmedel)",
    		searchFi: "95200 katumyyjät (pl. elintarvikkeet)",
    		searchEn: "95200 street vendors (excluding food)",
    		searchSv: "95200 gatuförsäljare (utom livsmedel)",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96110",
    		code: "96110",
    		labelFi: "Jätteiden kerääjät",
    		labelEn: "Garbage and recycling collectors",
    		labelSv: "Avfallsinsamlare",
    		searchFi: "96110 jätteiden kerääjät",
    		searchEn: "96110 garbage and recycling collectors",
    		searchSv: "96110 avfallsinsamlare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96120",
    		code: "96120",
    		labelFi: "Jätteiden lajittelijat",
    		labelEn: "Refuse sorters",
    		labelSv: "Avfallssorterare",
    		searchFi: "96120 jätteiden lajittelijat",
    		searchEn: "96120 refuse sorters",
    		searchSv: "96120 avfallssorterare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96130",
    		code: "96130",
    		labelFi: "Kadunlakaisijat ym.",
    		labelEn: "Sweepers and related labourers",
    		labelSv: "Gatsopare m.fl.",
    		searchFi: "96130 kadunlakaisijat ym.",
    		searchEn: "96130 sweepers and related labourers",
    		searchSv: "96130 gatsopare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96210",
    		code: "96210",
    		labelFi: "Sanomalehtien jakajat, lähetit ja kantajat",
    		labelEn: "Messengers, package deliverers and luggage porters",
    		labelSv: "Tidnings- och reklamutdelare, bud",
    		searchFi: "96210 sanomalehtien jakajat, lähetit ja kantajat",
    		searchEn: "96210 messengers, package deliverers and luggage porters",
    		searchSv: "96210 tidnings- och reklamutdelare, bud",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96220",
    		code: "96220",
    		labelFi: "Satunnaistöiden tekijät",
    		labelEn: "Odd job persons",
    		labelSv: "Diversearbetare",
    		searchFi: "96220 satunnaistöiden tekijät",
    		searchEn: "96220 odd job persons",
    		searchSv: "96220 diversearbetare",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96230",
    		code: "96230",
    		labelFi: "Mittareiden lukijat ym.",
    		labelEn: "Meter readers and vending-machine collectors",
    		labelSv: "Mätaravläsare m.fl.",
    		searchFi: "96230 mittareiden lukijat ym.",
    		searchEn: "96230 meter readers and vending-machine collectors",
    		searchSv: "96230 mätaravläsare m.fl.",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	},
    	{
    		id: "96290",
    		code: "96290",
    		labelFi: "Muut muualla luokittelemattomat työntekijät",
    		labelEn: "Elementary workers not elsewhere classified",
    		labelSv: "Övrig personal utan annan klassificering",
    		searchFi: "96290 muut muualla luokittelemattomat työntekijät",
    		searchEn: "96290 elementary workers not elsewhere classified",
    		searchSv: "96290 övrig personal utan annan klassificering",
    		freeTextFi: "",
    		freeTextEn: "",
    		freeTextSv: "",
    		keyword: ""
    	}
    ];

    /**
     * Helper for Statistics Finland Occupation codes and texts related to them.
     * Occupation is used for insurance pricing and reporting..
     */
    class Occupations {
        /** Gets all occupations for all languages. */
        static getAllForAllLanguages() {
            if (this.data.length === 0) {
                const occupations = [];
                // JSON import does not seem to support directly importing an array => Need to put it inside an object.
                list.forEach((x) => {
                    occupations.push({
                        id: x.id,
                        code: x.code,
                        label: x.labelFi,
                        freeText: x.freeTextFi,
                        search: x.searchFi,
                        keyword: x.keyword,
                        lang: "fi",
                    });
                    occupations.push({
                        id: x.id,
                        code: x.code,
                        label: x.labelSv,
                        freeText: x.freeTextSv,
                        search: x.searchSv,
                        keyword: x.keyword,
                        lang: "sv",
                    });
                    occupations.push({
                        id: x.id,
                        code: x.code,
                        label: x.labelEn,
                        freeText: x.freeTextEn,
                        search: x.searchEn,
                        keyword: x.keyword,
                        lang: "en",
                    });
                });
                this.data = occupations;
            }
            return this.data;
        }
        /**
         * Gets all the items of a given type.
         * @returns All occupation entries
         */
        static getAll(language) {
            language = language || Translations.getLanguage() || "fi";
            return this.getAllForAllLanguages().filter((x) => x.lang === language);
        }
        /**
         * Searches the occupations.
         * @param searchString String to search for. Typically user input.
         * @param language Language to search for.
         */
        static search(searchString, language) {
            searchString = (searchString || "").toLowerCase();
            if (searchString === "" || null) {
                this.getAll();
            }
            return this.getAll(language).filter((x) => (x.search || "").indexOf(searchString) >= 0);
        }
        /**
         * Gets a single occupation based on the original Statistics Finland code.
         * @param code - Statistics finland 5-number code.
         * Note that this is different than the Salaxy ID stored in Calculation.
         * Salaxy ID may specify additional suffix separated by dash. For Salaxy ID, use getById() instead.
         * @returns Occupation metadata or null if not found.
         */
        static getByCode(code) {
            return this.getAll().find((x) => x.code === code);
        }
        /**
         * Gets a single occupation based on the its Salaxy ID, which is derived from the original  Statistics Finland code.
         * @param id - Salaxy ID for the occupation.
         * This is the id stored in the calculation, but not the one that is reported to Incomes Register.
         * @returns Occupation metadata or null if not found.
         */
        static getById(id) {
            if (!id) {
                return null;
            }
            return this.getAll().find((x) => x.id === id);
        }
        /**
         * Fetches a list of occupations either based on list of IDs or a known keyword.
         * @param idListOrKeyword Either a comma separated list of ID's
         * or a known keyword: "household" and "company" currently supported.
         */
        static getByIds(idListOrKeyword) {
            if (!idListOrKeyword) {
                return [];
            }
            const splittedIdList = idListOrKeyword.split(",").map((x) => x.trim());
            const hasNumber = /^\d{5}/;
            if (idListOrKeyword.search(hasNumber) > -1) {
                return this.getAll().filter((x) => splittedIdList.indexOf(x.id) > -1);
            }
            else {
                return this.getAll().filter((x) => splittedIdList.some((k) => x.keyword.indexOf(k.trim()) > -1));
            }
        }
    }
    Occupations.data = [];

    /**
     * Business logic relating to Onboarding process and Digital signature.
     */
    class OnboardingLogic {
        /** Gets the authentication methods for digital signature. */
        static getTupasMethods() {
            const cdn = "https://cdn.salaxy.com/img";
            const imageOverrides = [
                "op",
                "nordea",
                "danske",
                "spankki",
                "handelsbanken",
                "mobiilivarmenne",
                "aktia",
                "pop",
                "sp",
                "omasp",
                "alandsbanken",
            ];
            const result = vismaSignOriginalMethods.map((v, i) => {
                return {
                    id: v.identifier,
                    title: v.name,
                    img: imageOverrides[i] ? `${cdn}/salaxy/sign/${imageOverrides[i]}.png` : v.image,
                    isPopular: i < 3,
                };
            });
            return result;
        }
    }

    /**
     * Business logic relating to payments.
     */
    class PaymentLogic {
        /**
         * OBSOLETE
         * @deprecated This is not any more updated and should not be used.
         */
        static getEInvoiceIntermediators() {
            const result = {
                "HELSFIHH": "Aktia - HELSFIHH",
                "003723327487": "Apix Messaging Oy - 003723327487",
                "BAWCFI22": "Basware Oyj - BAWCFI22",
                "003703575029": "CGI - 003703575029",
                "DABAFIHH": "Danske Bank - DABAFIHH",
                "DNBAFIHX": "DNB - DNBAFIHX",
                "HANDFIHH": "Handelsbanken - HANDFIHH",
                "885790000000418": "HighJump AS - 885790000000418",
                "INEXCHANGE": "InExchange Factorum AB - INEXCHANGE",
                "EXPSYS": "Lexmark Expert Systems AB - EXPSYS",
                "003708599126": "Liaison Technologies Oy - 003708599126",
                "003721291126": "Maventa - 003721291126",
                "003726044706": "Netbox Finland Oy - 003726044706",
                "NDEAFIHH": "Nordea Pankki - NDEAFIHH",
                "E204503": "OpusCapita Solutions Oy - E204503",
                "OKOYFIHH": "Osuuspankit ja Pohjola Pankki - OKOYFIHH",
                "PAGERO": "Pagero - PAGERO",
                "PALETTE": "Palette Software - PALETTE",
                "POPFFI22": "POP Pankki  - POPFFI22",
                "003710948874": "Posti Messaging Oy - 003710948874",
                "003701150617": "PostNord Strålfors Oy - 003701150617",
                "003714377140": "Ropo Capital Oy - 003714377140",
                "SBANFIHH": "S-Pankki - SBANFIHH",
                "TAPIFI22": "S-Pankki (ent. LähiTapiola Pankin tunnus toistaiseksi käytössä) - TAPIFI22",
                "ITELFIHH": "Säästöpankit - ITELFIHH",
                "003701011385": "Tieto Oyj - 003701011385",
                "885060259470028": "Tradeshift - 885060259470028",
                "AABAFI22": "Ålandsbanken - AABAFI22",
            };
            return result;
        }
    }

    /**
     * Framework independendent business logic methods and helpers for Reporting.
     */
    class ReportsLogic {
        /**
         * Gets a link URL for a yearly report. This is a relative link from the servr root (e.g. "/ReportHtml/YearEndReportJson?year=2017").
         * Typically the environment specific implementation needs to figure out the server address and token.
         * @param type - Type of the report must be one of the yearly reports
         * @param year - Year for the report
         * @param token - Token for authentication. Adds "access_token" to the query string.
         * This is typically required from the environment specific implementation,
         * but in exceptional cases you may use some other authentication mechanism and thus leave this null.
         * @param id - Worker ID for those reports that are specific to one Worker.
         * @param id2 - Second Worker ID for those reports that have two Workers in one report
         */
        static getYearlyReportUrl(type, year, token, id, id2) {
            let url;
            switch (type) {
                case exports.ReportType.YearEndReport:
                    url = `/ReportHtml/YearEndReportJson?year=${year}`;
                    break;
                case exports.ReportType.YearlyDetails:
                case exports.ReportType.Insurance:
                case exports.ReportType.HouseholdDeduction:
                    url = `/ReportHtml/${type}?year=${year}`;
                    break;
                case exports.ReportType.YearlyWorkerSummary:
                    url = `/ReportHtml/${type}/${id}?year=${year}`;
                    break;
                case exports.ReportType.Unemployment:
                    url = `/ReportPdf/${type}?year=${year}`;
                    break;
                case exports.ReportType.TaxYearly7801:
                    url = `/ReportPdf/${type}/${id}?year=${year}`;
                    break;
                case exports.ReportType.TaxHouseholdDeduction14B:
                case exports.ReportType.TaxHouseholdDeduction14BSpouseA:
                case exports.ReportType.TaxHouseholdDeduction14BSpouseB:
                    url = `/ReportPdf/${type}/${id}?id2=${id2 || ""}&year=${year}`;
                    break;
                default:
                    throw new Error("Not a supported year end report: " + type);
            }
            if (token) {
                url = url + "&access_token=" + token;
            }
            return url;
        }
        /**
         * Gets a URL for a calculation report. This is a relative link from the servr root (e.g. "/ReportHtml/SalarySlip/CALC_ID_HERE").
         * Typically the environment specific implementation needs to figure out the server address and token.
         * @param reportType - Type of report
         * @param calcId - Identifier of the calculation. This method requires that the calculation has been saved.
         * @param token - Token for authentication. Adds "access_token" to the query string.
         * This is typically required from the environment specific implementation,
         * but in exceptional cases you may use some other authentication mechanism and thus leave this null.
         * @param cumulative If true, fetches the report as cumulative.
         * Currently only supported by salarySlip
         */
        static getCalcReportUrl(reportType, calcId, token, cumulative = false) {
            if (!calcId) {
                throw new Error("Calculation ID is required by getCalcReportUrl");
            }
            switch (reportType) {
                case "salarySlip":
                case "paymentReport":
                case "employerReport": {
                    let url = `/ReportHtml/${reportType}/${calcId}?cumulative=${cumulative}`;
                    if (token) {
                        url += "&access_token=" + token;
                    }
                    return url;
                }
                default:
                    throw new Error("Unknown type in getCalcReportUrl: " + reportType);
            }
        }
        /**
         * Gets a URL for a calculation pdf file. This is a relative link from the servr root (e.g. "/ReportPdf/SalarySlip/CALC_ID_HERE").
         * Typically the environment specific implementation needs to figure out the server address and token.
         * @param reportType - Type of report
         * @param calcId - Identifier of the calculation. This method requires that the calculation has been saved.
         * @param inline - If true, the Content-Disposition header is returned with inline parameter.
         * @param token - Token for authentication. Adds "access_token" to the query string.
         * This is typically required from the environment specific implementation,
         * but in exceptional cases you may use some other authentication mechanism and thus leave this null.
         */
        static getCalcPdfUrl(reportType, calcId, inline = false, token) {
            if (!calcId) {
                throw new Error("Calculation ID is required by getCalcPdfUrl");
            }
            switch (reportType) {
                case "salarySlip":
                case "paymentReport":
                case "employerReport": {
                    let url = `/ReportPdf/${reportType}/${calcId}?inline=${inline ? "true" : "false"}`;
                    if (token) {
                        url += "&access_token=" + token;
                    }
                    return url;
                }
                default:
                    throw new Error("Unknown type in getCalcPdfUrl: " + reportType);
            }
        }
        /** Gets metadata about the supported report types */
        static getReportTypes() {
            // TODO: Merge this with salaxy.moveToCore.enumerations in palkkaus-web-worker project. Type the structure.
            const reportTypes = [
                {
                    id: "undefined",
                    title: "Valitse...",
                    description: "Ei valintaa tai tuntematon tyyppi. ",
                    roles: "",
                    category: "tech",
                },
                {
                    id: "example",
                    title: "Esimerkki",
                    description: "Esimerkki raportti kehityskäyttöön. Ei dataa, ei parametrejä.",
                    roles: "",
                    category: "tech",
                },
                {
                    id: "monthlyDetails",
                    title: "Monthly details",
                    description: "The most detailed of the employer monthly reports.",
                    roles: "employer",
                    category: "monthly",
                },
                {
                    id: "taxMonthly4001",
                    title: "Monthly tax report (4001)",
                    description: "Monthly tax report",
                    roles: "employer",
                    category: "monthly",
                },
                {
                    id: "monthlyPension",
                    title: "Monthly pension report",
                    description: "Monthly pension report",
                    roles: "employer",
                    category: "monthly",
                },
                {
                    id: "monthlyLiikekirjuri",
                    title: "Accounting report (Liikekirjuri)",
                    description: "'Liikekirjuri' - a historical monthly report that was used for 2016 and 2017. Will be replaced by Rapko.",
                    roles: "company",
                    category: "monthly",
                },
                {
                    id: "monthlyRapko",
                    title: "Accounting report (Rapko)",
                    description: "Rapko / Raportointikoodisto is the Finnish standard for accounting.",
                    roles: "company",
                    category: "monthly",
                },
                {
                    id: "yearlyDetails",
                    title: "Yearly details",
                    description: "Detailed list for employer - this is a general report.",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "yearEndReport",
                    title: "Year-end report",
                    description: "The year-end report with customer feedback",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "yearlyWorkerSummary",
                    title: "Yearly worker summary",
                    description: "Detailed report - similar to salary slip - from employer to worker. This report must be given by employer according to Finnish law.",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "taxYearly7801",
                    title: "Yearly tax report (7801)",
                    description: "Form 7801: Yearly report for Finnish tax authorities",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "unemployment",
                    title: "Unemployment insurance report",
                    description: "Report for Finnish Unemployment insurance TVR",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "insurance",
                    title: "Accident insurance report",
                    description: "Report for mandatory accident insurance company",
                    roles: "employer",
                    category: "yearly",
                },
                {
                    id: "householdDeduction",
                    title: "Household deduction yearly report",
                    description: "General (internal) household deduction yearly report.",
                    roles: "household",
                    category: "yearly",
                },
                {
                    id: "taxHouseholdDeduction14B",
                    title: "Household deduction report to authorities (14B)",
                    description: "Form 14B: Household deduction report for Finnish tax authorities. All deductible salaries included.",
                    roles: "household",
                    category: "yearly",
                },
                {
                    id: "taxHouseholdDeduction14BSpouseA",
                    title: "Household deduction 14B, self only",
                    description: "Form 14B: Household deduction report for Finnish tax authorities. Contains only deductible salaries which are attributed to self.",
                    roles: "household",
                    category: "yearly",
                },
                {
                    id: "taxHouseholdDeduction14BSpouseB",
                    title: "Household deduction 14B, spouse only",
                    description: "Form 14B: Household deduction report for Finnish tax authorities. Contains only deductible salaries which are split to spouse of customer.",
                    roles: "household",
                    category: "yearly",
                },
                {
                    id: "salarySlipCopy",
                    title: "Salary slip (worker copy)",
                    description: "Copy of the salary slip for the worker.",
                    roles: "worker",
                    category: "calc",
                },
                {
                    id: "salarySlipPaid",
                    title: "Salary slip (paid by employer)",
                    description: "Salary slip paid by the employer",
                    roles: "employer",
                    category: "calc",
                },
                {
                    id: "employerReport",
                    title: "Employer report of salary",
                    description: "Report of a single calculation from the Employer point-of-view",
                    roles: "employer",
                    category: "calc",
                },
                {
                    id: "paymentReport",
                    title: "Payment report of salary",
                    description: "Report of a single calculation from the payments / society point-of-view.",
                    roles: "employer",
                    category: "calc",
                },
                {
                    id: "employmentContract",
                    title: "Employment contract",
                    description: "Employment contract",
                    roles: "all",
                    category: "contract",
                },
            ];
            for (const reportType of reportTypes) {
                reportType.title = Translations.get("SALAXY.ENUM.ReportType." + reportType.id + ".label");
                reportType.description = Translations.get("SALAXY.ENUM.ReportType." + reportType.id + ".description");
            }
            return reportTypes;
        }
    }

    /**
     * Provides business logic for role membership rules.
     */
    class RoleLogic {
        /**
         * Checks whether the user is in a given role
         * @param session - Session of the current account.
         * @param appStatus - Additional information about the session and application status.
         * @param role - One of the known roles or role from server.
         * You can also use exclamation mark for negative (e.g.)
         * @returns Boolean whether user is in the given role
         */
        static isInRole(session, appStatus, role) {
            if (!role) {
                return true;
            }
            role = role.trim();
            const isNegative = role.startsWith("!");
            if (isNegative) {
                role = role.substr(1);
            }
            role = (role || "any").trim();
            role = role.substr(0, 1).toLowerCase() + role.substr(1); // Tolerate Capital first letter
            let primaryRole = null;
            const currentAccount = session ? session.currentAccount : null;
            if (currentAccount) {
                if (currentAccount.entityType === exports.LegalEntityType.Company) {
                    primaryRole = exports.SystemRole.Company; // Company is clear
                }
                else {
                    const personAccount = currentAccount;
                    if (personAccount.lastLoginAs === exports.WebSiteUserRole.Worker || (!personAccount.isEmployer && personAccount.isWorker)) {
                        primaryRole = exports.SystemRole.Worker; // LastLoginAs is clear, but we cannot rely on isWorker directly as some users are both isEmployer=true ans isWorker=true.
                    }
                    else {
                        primaryRole = exports.SystemRole.Household; // Default to household if both and no lastLogin.
                    }
                }
            }
            const checkRole = (role) => {
                var _a, _b, _c, _d, _e, _f, _g, _h;
                switch (role) {
                    // No session required
                    case exports.SystemRole.Unknown:
                        return appStatus.sessionCheckInProgress;
                    case exports.SystemRole.Anon:
                        return !appStatus.sessionCheckInProgress && (!session || !session.isAuthorized);
                    case exports.SystemRole.Init:
                        return appStatus.sessionCheckInProgress;
                    case exports.SystemRole.Hidden:
                        return false;
                    case exports.SystemRole.Test:
                        return !!appStatus.isTestData;
                    case exports.SystemRole.Prod:
                        return !appStatus.isTestData;
                    case exports.SystemRole.Any:
                    case "*":
                        return true;
                    // Session required (may be checked from primaryRole or currentAccount shortcuts)
                    case exports.SystemRole.Auth:
                        return session && session.isAuthorized;
                    case exports.SystemRole.Worker:
                        return primaryRole === exports.SystemRole.Worker;
                    case exports.SystemRole.Employer:
                        return primaryRole === exports.SystemRole.Company || primaryRole === exports.SystemRole.Household;
                    case exports.SystemRole.Household:
                        return primaryRole === exports.SystemRole.Household;
                    case exports.SystemRole.Company:
                        return primaryRole === exports.SystemRole.Company;
                    case exports.SystemRole.Person:
                        return primaryRole === exports.SystemRole.Worker || primaryRole === exports.SystemRole.Household;
                    case exports.SystemRole.NonVerified:
                        return currentAccount && !currentAccount.isVerified;
                    case exports.SystemRole.FiTm:
                    case exports.SystemRole.FiOy:
                    case exports.SystemRole.FiRy:
                        return primaryRole === exports.SystemRole.Company && session.currentAccount.companyType === role;
                    case exports.SystemRole.PartnerMessaging:
                        return (_d = (_c = (_b = (_a = session === null || session === void 0 ? void 0 : session.settings) === null || _a === void 0 ? void 0 : _a.partner) === null || _b === void 0 ? void 0 : _b.serviceModel) === null || _c === void 0 ? void 0 : _c.features) === null || _d === void 0 ? void 0 : _d.messaging;
                    case exports.SystemRole.PartnerService:
                        return appStatus.isTestData && ((_h = (_g = (_f = (_e = session === null || session === void 0 ? void 0 : session.settings) === null || _e === void 0 ? void 0 : _e.partner) === null || _f === void 0 ? void 0 : _f.serviceModel) === null || _g === void 0 ? void 0 : _g.features) === null || _h === void 0 ? void 0 : _h.forms);
                    default:
                        return (currentAccount && session.currentAccount.identity.roles.indexOf(role) >= 0)
                            || appStatus.clientRoles.indexOf(role) >= 0;
                }
            };
            return isNegative ? !checkRole(role) : checkRole(role);
        }
        /**
         * Returns true if the current account is a member of all of the given roles.
         * Returns true if the roles is null or an empty array or and an empty string.
         * @param session - Session of the current account.
         * @param appStatus - Additional information about the session and application status.
         * @param roles - Array of role values.
         */
        static isInAllRoles(session, appStatus, roles) {
            if (!roles || roles.length === 0) {
                return true;
            }
            return roles.every((x) => this.isInRole(session, appStatus, x));
        }
        /**
         * Returns true if the current account is a member of some of the given roles.
         * Returns true if the roles is null or an empty array or an empty string.
         * @param session Session of the current account.
         * @param appStatus State of the current application (i.e. login, session check etc.)
         * @param roles Array of role values.
         */
        static isInSomeRole(session, appStatus, roles) {
            if (!roles || roles.length === 0) {
                return true;
            }
            return roles.some((x) => this.isInRole(session, appStatus, x));
        }
    }

    /**
     * Provides business logic for calculating salary date.
     */
    class SalaryDateLogic {
        /**
         * Returns true, if the given date is a valid Salary date for given gross salary payment date.
         *
         * @param salaryDate - Salary date (the date when the salary is in worker).
         * @param iban - Bank account number of the worker.
         * @param paidAt - Date for the gross salary payment.
         * @param isRequestedSalaryDate - Boolean indicating if the salary date is the requested salary date. The default is true.
         * @param channel - Payment channel.
         * @returns - True if the given dates are not conflicting (salary date is possible for given paid at date).
         */
        static isValidSalaryDate(salaryDate, iban = null, paidAt = null, isRequestedSalaryDate = null, channel = null) {
            isRequestedSalaryDate = isRequestedSalaryDate === null ? true : isRequestedSalaryDate;
            const calculatedSalaryDate = this.CalculateSalaryDate(iban, paidAt, isRequestedSalaryDate, channel);
            return Dates.isWorkday(salaryDate) && Dates.getWorkdays(calculatedSalaryDate, salaryDate).length >= 1;
        }
        /**
         * Calculates the estimated salary date (the date when the salary is in worker) for the given gross salary payment date.
         *
         * @param iban  - The iban number of the worker.
         * @param paidAt - Date for the salary gross payment.
         * @param isRequestedSalaryDateSet - Flag to indicate, if the requested salary date is set.
         * @param channel - Payment channel.
         */
        static CalculateSalaryDate(iban = null, paidAt = null, isRequestedSalaryDateSet = null, channel = null) {
            let requiredTime = 0;
            let latePaymentDelay = 0;
            let paidAtMoment = Dates.getMoment(paidAt);
            if (!paidAtMoment) {
                const d = new Date();
                // time shift needed for backend compatibility
                const unshiftedPaidAtMoment = Dates.getMoment(d);
                paidAtMoment = unshiftedPaidAtMoment.clone().add(-1 * unshiftedPaidAtMoment.utcOffset(), "m");
                if (Dates.isWorkday(paidAtMoment)) {
                    if (paidAtMoment.date() === unshiftedPaidAtMoment.date()) {
                        if (unshiftedPaidAtMoment.hours() >= 16) {
                            latePaymentDelay = this.daysLatePaymentDelay;
                        }
                    }
                    else {
                        latePaymentDelay = this.daysLatePaymentDelay;
                    }
                }
            }
            else {
                if (Dates.isWorkday(paidAtMoment)) {
                    if (paidAtMoment.date() === paidAtMoment.local().date()) {
                        if (paidAtMoment.local().hours() >= 16) {
                            latePaymentDelay = this.daysLatePaymentDelay;
                        }
                    }
                    else {
                        latePaymentDelay = this.daysLatePaymentDelay;
                    }
                }
            }
            const paidAtWorkday = this.getWorkday(paidAtMoment);
            const paymentChannel = channel || exports.PaymentChannel.Undefined;
            const hasPaymentChannel = !(paymentChannel === exports.PaymentChannel.Undefined ||
                paymentChannel === exports.PaymentChannel.PalkkausCfaReference ||
                paymentChannel === exports.PaymentChannel.PalkkausCfaFinvoice ||
                paymentChannel === exports.PaymentChannel.PalkkausCfaPaytrail ||
                paymentChannel === exports.PaymentChannel.HolviCfa ||
                paymentChannel === exports.PaymentChannel.TalenomCfa);
            if (!hasPaymentChannel) {
                requiredTime += this.daysCustomerServiceProcessing;
                if (isRequestedSalaryDateSet) {
                    requiredTime += 2;
                }
                else {
                    requiredTime += latePaymentDelay;
                    const smallBankDelay = this.isBigBank(iban) ? 0 : this.daysNonBigBankPaymentDelay;
                    requiredTime += smallBankDelay;
                }
            }
            else {
                requiredTime += latePaymentDelay + 1;
            }
            const salaryDate = Dates.addWorkdays(paidAtWorkday, requiredTime);
            return Dates.asDate(salaryDate);
        }
        static getWorkday(startDate) {
            const newMoment = Dates.getMoment(Dates.asDate(startDate));
            const temp = true;
            while (temp) {
                if (Dates.isWorkday(newMoment)) {
                    return Dates.asDate(newMoment);
                }
                newMoment.add(1, "days");
            }
        }
        static isBigBank(iban) {
            if (iban && iban.length > 6) {
                const id = iban.substr(4, 3);
                if (id.startsWith("1") ||
                    id.startsWith("2") ||
                    id.startsWith("34") ||
                    id.startsWith("8") ||
                    id.startsWith("5")) {
                    return true;
                }
                return false;
                /*
                        1nn = Nordea Bank(Nordea)
                        2nn = Nordea Bank(Nordea)
                        31n = Handelsbanken(SHB)
                        33n = Skandinaviska Enskilda Banken(SEB)
                        34n = Danske Bank
                        36n = S - Bank
                        37n = DnB NOR Bank(DnB NOR)
                        38n = Swedbank
                        39n = S - Bank
                        4nn = Savings Banks(Sp), local co-operative banks and Aktia Bank
                        5nn = Co - operative banks(Op) and OKO Bank
                        6nn = Bank of Åland
                        713 = Citibank
                        8nn = Danske Bank
                  */
            }
        }
    }
    /**
     * Required days to process the incoming payment to worker net payment.
     */
    SalaryDateLogic.daysCustomerServiceProcessing = 1;
    /**
     * Payment delay if the bank is not a big bank.
     */
    SalaryDateLogic.daysNonBigBankPaymentDelay = 1;
    /**
     * Payment delay if the  payment takes place outside bank hours.
     */
    SalaryDateLogic.daysLatePaymentDelay = 1;

    /** Logic related to settings: Insurances, pension etc. */
    class SettingsLogic {
        /**
         * Returns pension number for test account
         * @param company The company enumerations.
         * Only the most common pension companies are supported.
         */
        static getPensionNumberForTest(company) {
            switch (company) {
                case exports.PensionCompany.Varma:
                    return "55-1111111S";
                case exports.PensionCompany.Elo:
                    return "54-11111114";
                case exports.PensionCompany.Ilmarinen:
                    return "46-1111111Y";
                case exports.PensionCompany.Veritas:
                    return "56-1111111D";
                case exports.PensionCompany.Apteekkien:
                    return "80009-1111N";
                case exports.PensionCompany.Verso:
                    return "80008-11113";
                default:
                    return null;
            }
        }
    }

    /**
     * Provides business logic related to TaxCards:
     * The Finnish way of determining the widtholding tax that Employer should deduct from Worker salary.
     * Taxcard handling was changed by tax authorities significantly for year 2019, hence the naming.
     */
    class TaxCard2019Logic {
        /**
         * Gets a new blank taxcard with default values, suitable for UI binding.
         *
         * @param personalId The personal ID that is used to connect the Taxcard to a person. This is required in save.
         * @param kind If specified, sets the type and defaults according to the type
         * @param year Specifies the year. This is used in December-January when there may be cards from both years.
         * Current year is the default.
         * @param today Allows changing the today date in unit testing etc. Default is actual today.
         */
        static getBlank(personalId, kind = null, year, today) {
            today = today || Dates.getToday();
            year = year || Dates.getYear("today");
            const newCard = {
                worker: null,
                card: {
                    personalId,
                    forYear: year,
                    taxPercent: null,
                    incomeLimit: null,
                    taxPercent2: null,
                    kind: exports.TaxcardKind.Undefined,
                    fileUri: null,
                    previewUri: null,
                    validity: {},
                },
                incomeLog: [],
                id: null,
                createdAt: today,
                updatedAt: today,
                isReadOnly: null,
            };
            if (kind) {
                newCard.card.kind = kind;
                switch (kind) {
                    case exports.TaxcardKind.Undefined:
                        newCard.card.taxPercent = 23;
                        break;
                    case exports.TaxcardKind.NoWithholdingHousehold:
                        newCard.card.taxPercent = 0;
                        newCard.card.validity.start = today;
                        break;
                    case exports.TaxcardKind.NoTaxCard:
                        newCard.card.taxPercent = 60;
                        newCard.card.validity.start = today;
                        break;
                    case exports.TaxcardKind.DefaultYearly:
                        newCard.card.validity.start = `${year}-02-01`;
                        newCard.card.validity.end = `${year}-12-31`;
                        break;
                    case exports.TaxcardKind.Replacement:
                    case exports.TaxcardKind.Others:
                    case exports.TaxcardKind.Historical:
                        newCard.card.validity.start = Dates.getYear(today) < year ? `${year}-01-01`
                            : (Dates.getYear(today) > year ? `${year}-12-31` : today);
                        newCard.card.validity.end = `${year}-12-31`;
                        break;
                }
            }
            return newCard;
        }
        /**
         * Gets display information and texts for taxcard within a calculation.
         * Helper for showing taxcards UI's for calculations.
         * NOTE: The data is cached to the calculation ($ui) until round-trip to server.
         * @param calc Calculation for which the data is resolved.
         */
        static getCalcTaxcardInfo(calc) {
            const result = {
                status: null,
                kind: exports.TaxcardKind.Undefined,
                text: null,
                shortText: null,
                percent: null,
            };
            if (!calc) {
                return result;
            }
            const $ui = CalculatorLogic.getUiCache(calc);
            if ($ui.tax) {
                return $ui.tax;
            }
            if (!calc.worker || !calc.worker.tax) {
                $ui.tax = result;
                return result;
            }
            const tax = calc.worker.tax;
            result.kind = (tax.snapshot ? tax.snapshot.kind : null) || exports.TaxcardKind.Undefined;
            result.status = tax.hasNewSharedCard ? "waiting" :
                (result.kind === exports.TaxcardKind.Undefined ? "missing" : "ok");
            // TODO: Translation
            if (result.status === "waiting") {
                result.text = "Tarkista uusi verokortti";
                result.shortText = "Uusi verokortti";
            }
            else {
                switch (result.kind) {
                    case exports.TaxcardKind.Undefined:
                        result.text = EnumerationsLogic.getEnumLabel("TaxcardKind", result.kind);
                        result.shortText = "Ei verokorttia";
                        break;
                    case exports.TaxcardKind.NoTaxCard:
                        result.text = EnumerationsLogic.getEnumLabel("TaxcardKind", result.kind);
                        result.shortText = result.text;
                        result.percent = 60;
                        break;
                    case exports.TaxcardKind.NoWithholdingHousehold:
                        result.text = EnumerationsLogic.getEnumLabel("TaxcardKind", result.kind);
                        result.shortText = result.text;
                        result.percent = 0;
                        break;
                    default:
                        result.percent = calc.worker.tax.snapshot.taxPercent;
                        if (calc.result.totals.totalTaxable > 0) {
                            result.percent = (calc.worker.tax.estimatedTax / calc.result.totals.totalTaxable) * 100;
                        }
                        result.text = EnumerationsLogic.getEnumLabel("TaxcardKind", result.kind) + ": " + Numeric.formatPercent(result.percent);
                        result.shortText = "Ennakonpidätys" + ": " + Numeric.formatPercent(result.percent);
                        break;
                }
            }
            $ui.tax = result;
            return result;
        }
        /**
         * Checks if the taxcard is valid for the salary date.
         * This is used when in test environment the salaryDate may be changed in UX.
         * Typically, you should get this information from the API methods.
         * @param card Taxcard to check.
         * @param salaryDate Salary date. Default is today.
         */
        static isTaxcardValid(card, salaryDate = "today") {
            if (!card || !card.card || !card.card.validity) {
                return false;
            }
            salaryDate = Dates.asDate(salaryDate) || Dates.getToday();
            if (!(card.card.validity.start > salaryDate) && !(card.card.validity.end < salaryDate)) {
                return true;
            }
            if (Dates.getMonth(salaryDate) === 1 && card.card.forYear === Dates.getYear(salaryDate) - 1) {
                return true;
            }
            return false;
        }
        /**
         * TODO: This is currently used for charting only. As such it could probably be simplified
         * and / or combined with getMainChart() / getPieChart().
         * Gets information about the the income limit and real percent.
         * Note that this method calculates the real percent i.e.
         * for examples and no tax cards 60%. If you wish to see the example percent here (23%),
         * catch it separately.
         * @param taxcard Card to analyze
         */
        static getIncomeLimitInfo(taxcard) {
            const result = {
                type: exports.TaxcardKind.Undefined,
                percent: 60,
                hasLimit: false,
                fractionOfLimit: 0,
            };
            if (!taxcard || !taxcard.card || !taxcard.card.kind) {
                return result;
            }
            const card = taxcard.card;
            result.percent = card.taxPercent === 0 ? 0 : card.taxPercent || 60;
            result.type = card.kind;
            switch (result.type) {
                // May or may not have limits
                case exports.TaxcardKind.DefaultYearly:
                case exports.TaxcardKind.Replacement:
                case exports.TaxcardKind.Others:
                    result.hasLimit = !!(card.incomeLimit && card.taxPercent2);
                    if (result.hasLimit) {
                        result.fractionOfLimit = taxcard.result.totalIncome / card.incomeLimit;
                        if (taxcard.result.totalIncome > card.incomeLimit && card.taxPercent2) {
                            result.percent = card.taxPercent2;
                        }
                    }
                    return result;
                // No limit
                case exports.TaxcardKind.NoTaxCard:
                case exports.TaxcardKind.NoWithholdingHousehold:
                case exports.TaxcardKind.Undefined:
                default:
                    return result;
            }
        }
        /** Gets the main chart for taxcards */
        static getMainChart(taxcard) {
            const limits = TaxCard2019Logic.getIncomeLimitInfo(taxcard);
            const taxCardIncomeLimit = taxcard.card.incomeLimit;
            const totalIncome = taxcard.result.totalIncome;
            const taxPercent = taxcard.card.taxPercent;
            const taxPercent2 = taxcard.card.taxPercent2;
            let incomeSum = 0;
            let incomes = taxcard.incomeLog.sort((x, y) => x.paidAt < y.paidAt ? -1 : (x.paidAt > y.paidAt ? 1 : 0)).map((value) => {
                incomeSum += value.income;
                return { x: value.paidAt, y: incomeSum };
            });
            if (incomes.length === 0) {
                incomes = [
                    { x: taxcard.card.validity.start, y: 0 },
                ];
            }
            const validity = [
                { x: taxcard.card.validity.start, y: 0 },
                { x: taxcard.card.validity.end, y: taxcard.card.incomeLimit },
            ];
            const colorPrimary = Brand.getBrandColor("primary", "rgba");
            const colorDanger = Brand.getBrandColor("danger", "rgba");
            const colorWarning = Brand.getBrandColor("warning", "rgba");
            const result = {
                limits,
                data: [
                    incomes,
                    validity,
                ],
                labels: null,
                datasets: [
                    {
                        label: "Palkanmaksu",
                        fill: false,
                        index: 2,
                        lineTension: 0,
                        backgroundColor: colorPrimary,
                        pointBackgroundColor: colorPrimary,
                        pointHoverBackgroundColor: colorPrimary,
                        borderColor: colorPrimary,
                        pointBorderColor: colorPrimary,
                        pointHoverBorderColor: colorPrimary,
                    },
                    {
                        label: "Tuloraja",
                        backgroundColor: colorWarning,
                        pointBackgroundColor: colorWarning,
                        pointHoverBackgroundColor: colorWarning,
                        borderColor: colorWarning,
                        pointBorderColor: colorWarning,
                        pointHoverBorderColor: colorWarning,
                        fill: false,
                        borderDash: [5, 5],
                        lineTension: 0,
                    },
                ],
                options: {
                    maintainAspectRatio: false,
                    legend: {
                        display: true,
                        position: "bottom",
                    },
                    title: {
                        display: true,
                        text: "Maksetut palkat ja vuosikohtainen tuloraja",
                    },
                    scales: {
                        yAxes: [{
                                display: true,
                                ticks: {
                                    suggestedMax: taxCardIncomeLimit * 1.5,
                                    // OR //
                                    beginAtZero: true, // minimum value will be 0.
                                },
                            }],
                        xAxes: [{
                                type: "time",
                                time: {
                                    displayFormats: {
                                        millisecond: "D.M.YYYY",
                                        second: "D.M.YYYY",
                                        minute: "D.M.YYYY",
                                        hour: "D.M.YYYY",
                                        day: "D.M.YYYY",
                                        week: "D.M.YYYY",
                                        month: "D.M.YYYY",
                                        year: "D.M.YYYY",
                                    },
                                    minUnit: "day",
                                    tooltipFormat: "D.M.YYYY",
                                },
                            }],
                    },
                    annotation: {
                        annotations: [{
                                type: "line",
                                mode: "horizontal",
                                scaleID: "y-axis-0",
                                value: taxCardIncomeLimit,
                                borderColor: colorDanger,
                                borderWidth: 2,
                                label: {
                                    position: "left",
                                    enabled: true,
                                    content: `${taxCardIncomeLimit} €`,
                                    fontColor: "#fff",
                                    fontSize: 11,
                                    backgroundColor: colorDanger,
                                    yAdjust: 0,
                                    xAdjust: 60,
                                },
                            },
                            {
                                type: "line",
                                mode: "horizontal",
                                scaleID: "y-axis-0",
                                value: taxCardIncomeLimit,
                                borderColor: "transparent",
                                borderWidth: 0,
                                label: {
                                    position: "left",
                                    enabled: true,
                                    content: `${taxPercent2} %`,
                                    fontSize: 16,
                                    fontColor: colorDanger,
                                    backgroundColor: "transparent",
                                    yAdjust: -11,
                                    xAdjust: 5,
                                },
                            },
                            {
                                type: "line",
                                mode: "horizontal",
                                scaleID: "y-axis-0",
                                value: taxCardIncomeLimit,
                                borderColor: "transparent",
                                borderWidth: 0,
                                label: {
                                    position: "left",
                                    enabled: true,
                                    content: `${taxPercent} %`,
                                    fontSize: 16,
                                    fontColor: colorWarning,
                                    backgroundColor: "transparent",
                                    yAdjust: 15,
                                    xAdjust: 5,
                                },
                            },
                            {
                                type: "line",
                                mode: "horizontal",
                                scaleID: "y-axis-0",
                                value: 0,
                                borderColor: "transparent",
                                borderWidth: 0,
                                label: {
                                    position: "right",
                                    enabled: true,
                                    content: `31.12.${taxcard.card.forYear}: ${Numeric.round(totalIncome, 0)} €`,
                                    fontSize: 12,
                                    fontColor: colorPrimary,
                                    backgroundColor: "#fff",
                                    yAdjust: -30,
                                    xAdjust: 10,
                                },
                            }],
                    },
                    tooltips: {
                        backgroundColor: "rgba(255,255,255,1)",
                        titleFontColor: "#000",
                        bodyFontColor: "#000",
                        borderColor: "#000",
                        displayColors: false,
                        intersect: false,
                        enabled: true,
                        mode: "single",
                        callbacks: {
                            label: (tooltipItem, data) => {
                                let label = data.datasets[tooltipItem.datasetIndex].label || "";
                                if (label) {
                                    label += ` ${tooltipItem.xLabel}: `;
                                }
                                label += Math.round(tooltipItem.yLabel * 100) / 100;
                                return label;
                            },
                        },
                    },
                    events: ["mousemove", "click"],
                },
            };
            return result;
        }
        /** Gets the pie chart for the taxcards. This is used for simplistic cases that do not have an income limit. */
        static getPieChartData(taxcard) {
            const limits = TaxCard2019Logic.getIncomeLimitInfo(taxcard);
            const netSalary = 100 - limits.percent;
            const label1Suffix = limits.fractionOfLimit < 1 ? "perusprosentti" : "lisäprosentti";
            const getColorPalette = (color) => {
                return {
                    backgroundColor: color,
                    pointBackgroundColor: color,
                    pointHoverBackgroundColor: color,
                    borderColor: color,
                    pointBorderColor: color,
                    pointHoverBorderColor: color,
                };
            };
            const result = {
                limits,
                data: [
                    limits.percent,
                    100 - limits.percent,
                ],
                datasets: [
                    {
                        label: ["Test", "test 2"],
                    },
                ],
                labels: [
                    `Ennakonpidätys (${label1Suffix}) ${Numeric.formatPercent(limits.percent)}`,
                    `Käteen jäävä osuus ${Numeric.formatPercent(netSalary)}`,
                ],
                colors: [
                    getColorPalette(limits.type === exports.TaxcardKind.NoTaxCard || limits.type === exports.TaxcardKind.NoWithholdingHousehold
                        ? Brand.getBrandColor("danger", "rgba")
                        : Brand.getBrandColor("warning", "rgba")),
                    getColorPalette(Brand.getBrandColor("primary", "rgba")),
                ],
                options: {
                    legend: {
                        display: true,
                        position: "bottom",
                        fullWidth: true,
                        reverse: true,
                    },
                    tooltips: {
                        enabled: true,
                        mode: "single",
                        bodySpacing: 6,
                        callbacks: {
                            label: (tooltipItem, data) => {
                                return data.labels[tooltipItem.index];
                            },
                        },
                    },
                    events: ["mousemove", "click"],
                },
            };
            return result;
        }
    }

    /**
     * Logic functions related to Typedoc parsing and interpretation.
     */
    class TypedocLogic {
        /**
         * Finds an interface based on the type name.
         * @param docs Array of classes and interfaces from which to find an interface.
         * @param typeName The type name of the interface to look for. Search is case-insensitive.
         */
        static findInterface(docs, typeName) {
            typeName = typeName.toLowerCase().trim();
            if (!typeName) {
                return null;
            }
            return docs.find((x) => x.kind === "interface" && x.name.toLowerCase() === typeName);
        }
        /**
         * Finds an enum based on the type name.
         * @param allEnums All enumerations from which to find the enum by name.
         * @param name The enum name to look for. Search is case-insensitive.
         */
        static findEnum(allEnums, name) {
            name = name.toLowerCase().trim();
            if (!name) {
                return null;
            }
            return allEnums.find((x) => x.name.toLowerCase() === name);
        }
        /**
         * Gets a property from an interface supports single property names or longer property paths.
         * @param type The root type from which the property is found.
         * @param path Property path starting with the root type.
         * @param interfaces The interfaces that is used in finding types.
         * Typically, this is the full colelction of all classes and interfaces.
         */
        static getProperty(type, path, interfaces) {
            var _a;
            if (!type || !(path === null || path === void 0 ? void 0 : path.trim())) {
                return null;
            }
            const pathArr = path.split(".");
            let member = null;
            while (pathArr.length) {
                const propName = pathArr.shift().trim();
                member = type.members.find((x) => x.name.toLowerCase() === propName.toLowerCase());
                if (pathArr.length) {
                    const nextTypeName = (_a = this.getTypeAsText(member === null || member === void 0 ? void 0 : member.type)) === null || _a === void 0 ? void 0 : _a.replace("?", "");
                    if (!nextTypeName) {
                        return null;
                    }
                    type = this.findInterface(interfaces, nextTypeName);
                }
            }
            return member === undefined ? null : member;
        }
        /**
         * Gets a typedoc type formatted as text.
         * @param type Type as typedoc object model.
         */
        static getTypeAsText(type) {
            if (type == null) {
                return null;
            }
            switch (type.type) {
                case "intrinsic":
                    return `${type.name}`;
                case "union":
                    if (type.types.length === 2 && type.types[1].name === "null") {
                        return TypedocLogic.getTypeAsText(type.types[0]) + "?";
                    }
                    return type.types.map((innerType) => TypedocLogic.getTypeAsText(innerType)).join(" | ");
                case "array":
                    return TypedocLogic.getTypeAsText(type.elementType) + "[]";
                case "stringLiteral":
                    return `"${type.value}"`;
                case "typeParameter":
                    return `<${type.name}>`;
                case "reference":
                    return type.name
                        + (type.typeArguments ? "<" + type.typeArguments.map((arg) => TypedocLogic.getTypeAsText(arg)).join(", ") + ">" : "");
                case "reflection":
                    return "Reflection not supported in quick docs.";
                default:
                    return `TODO: ${JSON.stringify(type)}`;
            }
        }
        /**
         * Gets a typedoc type formatted as HTML.
         * @param type Type as typedoc object model.
         * @param typeLinker Method that resolves a Typedoc type to link in the current environment.
         */
        static getTypeHtml(type, typeLinker) {
            if (type == null) {
                return null;
            }
            switch (type.type) {
                case "intrinsic":
                    return `<i>${type.name}</i>`;
                case "union":
                    if (type.types.length === 2 && type.types[1].name === "null") {
                        return TypedocLogic.getTypeHtml(type.types[0], typeLinker) + "?";
                    }
                    return type.types.map((innerType) => TypedocLogic.getTypeHtml(innerType, typeLinker)).join(" | ");
                case "array":
                    return TypedocLogic.getTypeHtml(type.elementType, typeLinker) + "[]";
                case "stringLiteral":
                    return `"${type.value}"`;
                case "typeParameter":
                    return `&lt;${type.name}&gt;`;
                case "reference": {
                    const typeLink = typeLinker && typeLinker(type);
                    let refHtml = typeLink ? `<a href="${typeLink}">${type.name}</a>` : type.name;
                    if (type.typeArguments) {
                        refHtml += "&lt;" + type.typeArguments.map((arg) => TypedocLogic.getTypeHtml(arg, typeLinker)).join(", ") + "&gt;";
                    }
                    return refHtml;
                }
                case "reflection":
                    return "<i>Reflection type not implemented in quick docs. See source for details.</i>";
                default:
                    return `<span>${type.type}: <code>${JSON.stringify(type)}</code></span>`;
            }
        }
        /** Gets the enumerations for the given library from raw TypeDoc json. */
        static getEnums(typedoc, lib) {
            const enums = typedoc.children
                .filter((x) => x.kindString === "Enumeration")
                .map((typedocEnum) => {
                const enumResult = TypedocLogic.getRootDeclarationReflection("enum", lib, typedocEnum);
                enumResult.names = (typedocEnum.children || []).map((td) => td.name);
                enumResult.values = (typedocEnum.children || []).map((td) => { var _a; return (_a = td.defaultValue) === null || _a === void 0 ? void 0 : _a.replace("\"", "").replace("\"", ""); });
                return enumResult;
            });
            return enums;
        }
        /** Gets the classes and interfaces strongly typed from raw TypeDoc json */
        static getClasses(typedoc, lib) {
            const classes = typedoc.children
                .filter((x) => x.kindString === "Class" || x.kindString === "Interface")
                .map((typedocClass) => {
                const classResult = this.getRootDeclarationReflection("class", lib, typedocClass);
                classResult.extendedBy = typedocClass.extendedBy || [];
                classResult.extends = typedocClass.extendedTypes || [];
                classResult.implementedBy = typedocClass.implementedBy || [];
                classResult.implements = typedocClass.implementedTypes || [];
                if (classResult.rawKind === "Interface") {
                    classResult.kind = "interface";
                }
                else if (classResult.implements.find((x) => x.name === "IController")) {
                    classResult.kind = "controllerBase";
                }
                // Registered components and controllers will be set to data later.
                classResult.group = this.getGroup(lib, classResult.source);
                classResult.members = (typedocClass.children || []).filter((x) => x.flags.isPublic || x.flags.isExported)
                    .map((tdMember) => {
                    const result = {
                        type: tdMember.type,
                        grouping: "Other",
                        parameters: [],
                    };
                    if (tdMember.kindString === "Method" && tdMember.signatures[0]) {
                        tdMember.signatures[0].sources = tdMember.sources;
                        this.setDeclrProperties(result, tdMember.signatures[0]);
                        result.rawKind = "Method";
                    }
                    else {
                        this.setDeclrProperties(result, tdMember);
                    }
                    switch (result.rawKind) {
                        case "Accessor":
                            // Alternative way:
                            // result.type = (td.getSignature || td.setSignature)[0].type;
                            result.type = (tdMember.setSignature && tdMember.setSignature[0]) ? tdMember.setSignature[0].parameters[0].type : tdMember.getSignature[0].type;
                            // Add read-only info here if necessary
                            result.grouping = "Property";
                            break;
                        case "Property":
                            result.grouping = "Property";
                            break;
                        case "Method":
                            result.grouping = "Method";
                            if (tdMember.signatures[0]) {
                                if (tdMember.signatures.length > 1) {
                                    result.shortText += "\n\nNOTE: method has more than one signature. See source for all signatures.";
                                }
                                const signature = tdMember.signatures[0];
                                result.type = signature.type;
                                if (signature.parameters) {
                                    result.parameters = signature.parameters.map((paramSrc) => {
                                        const paramResult = {
                                            grouping: "Property",
                                            type: paramSrc.type,
                                        };
                                        this.setDeclrProperties(paramResult, paramSrc);
                                        return paramResult;
                                    });
                                }
                            }
                            else {
                                result.shortText += "\nERROR: No signature was found!";
                            }
                            break;
                    }
                    return result;
                });
                return classResult;
            });
            return classes;
        }
        /**
         * Gets the common ROOT declaration reflection types from the Typedoc JSON: class, interface and enum
         * and their sub-kinds in AngularJS: controller, controllerBase and component.
         */
        static getRootDeclarationReflection(kind, lib, typedocSource) {
            var _a, _b, _c, _d;
            const result = {
                id: typedocSource.id,
                kind,
                lib,
            };
            if (typedocSource.sources && typedocSource.sources[0]) {
                result.source = typedocSource.sources[0].fileName;
            }
            result.group = this.getGroup(lib, result.source);
            this.setDeclrProperties(result, typedocSource);
            result.exampleUrls = ((_b = (_a = typedocSource.comment) === null || _a === void 0 ? void 0 : _a.tags) === null || _b === void 0 ? void 0 : _b.filter((t) => t.tag === "exampleurl").map((e) => {
                const url = e.text.trim();
                return url.toLowerCase() === "true" ? `/ng1/examples/${result.group}/${result.name}.html` : url.trim();
            })) || [];
            result.screenShots = ((_d = (_c = typedocSource.comment) === null || _c === void 0 ? void 0 : _c.tags) === null || _d === void 0 ? void 0 : _d.filter((t) => t.tag === "screenshot").map((e) => {
                const url = e.text.trim();
                return url.toLowerCase() === "true" ? `/ng1/screen-shots/${result.group}/${result.name}.png` : url.trim();
            })) || [];
            return result;
        }
        /**
         * Sets the DeclarationReflection properties from source JSON
         * @param target Target data structure
         * @param tdSource The source object from raw typedoc JSON source.
         */
        static setDeclrProperties(target, tdSource) {
            if (!tdSource) {
                return;
            }
            target.name = tdSource.name;
            target.rawKind = tdSource.kindString;
            if (tdSource.comment) {
                if (tdSource.comment.shortText) {
                    target.shortText = tdSource.comment.shortText;
                    target.descr = tdSource.comment.shortText + (tdSource.comment.text ? "\n\n" + tdSource.comment.text : "");
                }
                else {
                    target.shortText = tdSource.comment.text || "Error: No description in comments.";
                    target.descr = tdSource.comment.text || "Error: No description in comments.";
                }
                if (tdSource.comment.returns) {
                    target.descr += "\n\n**Returns**: " + tdSource.comment.returns;
                }
                if (tdSource.comment.tags) {
                    const examples = tdSource.comment.tags.filter((t) => t.tag === "example");
                    if (examples.length > 0) {
                        target.example = examples.map((e) => e.text.trim()).join("\n\n------------\n\n");
                    }
                }
            }
        }
        /** Gets the grouping based on library and source file */
        static getGroup(lib, source) {
            // HACK: Probably make private.
            if (!source) {
                return "err:no-source";
            }
            const splitSrc = source.split("/");
            switch (lib) {
                case "@salaxy/core":
                    if (splitSrc[splitSrc.length - 1] === "") {
                        return "v01.ts";
                    }
                    return splitSrc[0];
                case "@salaxy/ng1":
                    // For some reason, this changed to: "salaxy-lib-ng1/src/ts/controllers/calc/CalcIrRowsController.ts"
                    if (splitSrc.length > 5 && (splitSrc[3] === "components" || splitSrc[3] === "controllers")) {
                        return splitSrc[4];
                    }
                    if (splitSrc.length > 4) {
                        return splitSrc[3];
                    }
                    if (splitSrc.length > 2 && (splitSrc[0] === "components" || splitSrc[0] === "controllers")) {
                        return splitSrc[1];
                    }
                    if (splitSrc.length > 2 && (splitSrc[0] === "components" || splitSrc[0] === "controllers")) {
                        return splitSrc[1];
                    }
                    return splitSrc[0];
            }
        }
        /** For an array of types, gets an array type strings  */
        static getTypeStrings(types) {
            if (!types) {
                return [];
            }
            return types.map((t) => TypedocLogic.getTypeAsText(t));
        }
    }

    /**
     * Provides business logic related to Worker accounts: Workers as Employers see them.
     */
    class WorkerLogic {
        /**
         * Gets a new blank worker account with default values, suitable for UI binding.
         */
        static getBlank() {
            return {
                avatar: {
                    pictureType: exports.AvatarPictureType.Icon,
                },
                employment: {
                    irIncomeEarnerTypes: [],
                    holidays: {},
                    usecase: {
                        data: {},
                    },
                    work: {
                        salaryDefaults: [],
                    },
                },
                contact: {},
                ageRange: exports.AgeRange.Age18_52,
                entityType: exports.LegalEntityType.PersonCreatedByEmployer,
                id: null,
                createdAt: Dates.getToday(),
                updatedAt: Dates.getToday(),
                isReadOnly: null,
            };
        }
        /**
         * Creates a new avatar object based on firstName and lastName.
         * This simulates how server will eventually create an avatar object for a worker.
         * @param firstName First name of the person.
         * @param lastName Last name of the person.
         * @param color Avatar color. If not set, this will later be random generated (in this implementation, a static grayish color).
         */
        static createAvatar(firstName, lastName, color) {
            firstName = WorkerLogic.capitalizeFirstLetter((firstName || "").trim()) || " ";
            lastName = WorkerLogic.capitalizeFirstLetter((lastName || "").trim()) || " ";
            color = color || "rgb(21, 83, 93)";
            return {
                color,
                displayName: (firstName + " " + lastName).trim() || "?",
                firstName,
                entityType: exports.LegalEntityType.PersonCreatedByEmployer,
                initials: (firstName.substr(0, 1) + lastName.substr(0, 1)) || "?",
                lastName,
                pictureType: exports.AvatarPictureType.Icon,
                sortableName: ((lastName + ", " + firstName).replace(/,/g, "").trim()) || "?",
            };
        }
        /**
         * Capitalizes the first character of a string.
         * The method is null-safe: Returns null if null, empty if empty.
         * @param text Text for which the first letter is capitalized.
         */
        static capitalizeFirstLetter(text) {
            if (!text) {
                return text;
            }
            return text.charAt(0).toUpperCase() + text.slice(1);
        }
        /**
         * Gets a description text for a WorkerAccount that may be either
         * IWorkerListItem or WorkerAccount.
         * @param worker Worker account to describe. Either account or list item.
         */
        static getDescription(worker) {
            const emptyDescription = "-";
            const describe = (w) => {
                if (w.employment && w.employment.type && w.employment.type !== exports.EmploymentRelationType.Undefined) {
                    let descr = EnumerationsLogic.getEnumLabel("EmploymentRelationType", w.employment.type);
                    switch (w.employment.type) {
                        case exports.EmploymentRelationType.Entrepreneur:
                            if (w.employment.pensionCalculation === exports.PensionCalculation.Entrepreneur) {
                                descr += " (YEL)";
                            }
                            else if (w.employment.pensionCalculation === exports.PensionCalculation.PartialOwner) {
                                descr += " (YEL, osaomistaja)";
                            }
                            else if (w.employment.pensionCalculation === exports.PensionCalculation.SmallEntrepreneur) {
                                descr += " (YEL, ei vakuuttamisvelvollisuutta)";
                            }
                            else if (w.employment.pensionCalculation === exports.PensionCalculation.Employee) {
                                descr += " (TyEL)";
                            }
                            break;
                        case exports.EmploymentRelationType.Farmer:
                            if (w.employment.pensionCalculation === exports.PensionCalculation.Farmer) {
                                descr += " (MYEL)";
                            }
                            else if (w.employment.pensionCalculation === exports.PensionCalculation.SmallFarmer) {
                                descr += " (MYEL, ei vakuuttamisvelvollisuutta)";
                            }
                            else if (w.employment.pensionCalculation === exports.PensionCalculation.Employee) {
                                descr += " (TyEL)";
                            }
                            break;
                    }
                    return descr;
                }
                return w.contact.email || emptyDescription;
            };
            if (!worker) {
                return emptyDescription;
            }
            if (worker.employment) {
                return describe(worker);
            }
            const workerListItem = worker;
            if (!workerListItem.otherPartyInfo) {
                return emptyDescription;
            }
            return describe({
                avatar: workerListItem.otherPartyInfo.avatar,
                contact: {
                    email: workerListItem.otherPartyInfo.email,
                    telephone: workerListItem.otherPartyInfo.telephone,
                },
                employment: {
                    type: workerListItem.data.type,
                    pensionCalculation: workerListItem.data.pensionCalculation,
                },
                entityType: workerListItem.otherPartyInfo.avatar.entityType,
                ibanNumber: workerListItem.otherPartyInfo.ibanNumber,
                id: workerListItem.otherId,
                officialPersonId: workerListItem.otherPartyInfo.officialId,
            });
        }
        /** Sample workers for anonymous demo user interfaces. */
        static getSampleWorkers() {
            return [
                {
                    avatar: {
                        entityType: exports.LegalEntityType.PersonCreatedByEmployer,
                        firstName: "Erkki",
                        lastName: "Esimerkki",
                        displayName: "Erkki Esimerkki",
                        sortableName: "Esimerkki, Erkki",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "lime",
                        initials: "EE",
                        url: null,
                        description: "Esimerkkityöntekijä, oletus: 18-52 vuotta, veroprosentti 23%.",
                        id: "example-default",
                    },
                    contact: {
                        email: "erkki.esimerkki@palkkaus.fi",
                        telephone: "09 425 50603",
                    },
                    id: "example-default",
                },
                {
                    avatar: {
                        entityType: exports.LegalEntityType.PersonCreatedByEmployer,
                        firstName: "Laura",
                        lastName: "Lukiolainen",
                        displayName: "Laura Lukiolainen",
                        sortableName: "Lukiolainen, Laura",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "pink",
                        initials: "LL",
                        url: null,
                        description: "Esimerkki nuoresta työntekijästä: 17 vuotias",
                        id: "example-17",
                    },
                    contact: {
                        email: "laura.lukiolainen@palkkaus.fi",
                        telephone: "09 425 50603",
                    },
                    id: "example-17",
                },
                {
                    avatar: {
                        entityType: exports.LegalEntityType.PersonCreatedByEmployer,
                        firstName: "Tanja",
                        lastName: "Eläkeläinen",
                        displayName: "Tanja Eläkeläinen",
                        sortableName: "Eläkeläinen, Tanja",
                        pictureType: exports.AvatarPictureType.Icon,
                        color: "blue",
                        initials: "TE",
                        url: null,
                        description: "Esimerkki eläkeläisestä: 65-67 vuotias",
                        id: "example-pensioner",
                    },
                    contact: {
                        email: "tanja.elakelainen@palkkaus.fi",
                        telephone: "09 425 50603",
                    },
                    id: "example-pensioner",
                },
            ];
        }
        /**
         * Sets default values for the worker based on employment type.
         * @param worker - Worker to which default values are set.
         */
        static setEmploymentDefaultValues(worker) {
            if (!worker) {
                return;
            }
            worker.employment.work.description = null;
            worker.employment.work.occupationCode = null;
            switch (worker.employment.type) {
                case exports.EmploymentRelationType.MonthlySalary:
                case exports.EmploymentRelationType.HourlySalary:
                case exports.EmploymentRelationType.Salary:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Employee;
                    break;
                case exports.EmploymentRelationType.EmployedByStateEmploymentFund:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Employee;
                    break;
                case exports.EmploymentRelationType.PerformingArtist:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Employee;
                    break;
                case exports.EmploymentRelationType.KeyEmployee:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Employee;
                    break;
                case exports.EmploymentRelationType.Entrepreneur:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Entrepreneur;
                    break;
                case exports.EmploymentRelationType.Farmer:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Farmer;
                    break;
                case exports.EmploymentRelationType.Compensation:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Compensation;
                    break;
                case exports.EmploymentRelationType.BoardMember:
                    worker.employment.pensionCalculation = exports.PensionCalculation.BoardRemuneration;
                    worker.employment.work.description = "Hallituksen jäsen";
                    break;
                case exports.EmploymentRelationType.Athlete:
                    worker.employment.pensionCalculation = exports.PensionCalculation.Athlete;
                    worker.employment.work.occupationCode = "34210";
                    worker.employment.work.description = "Urheilija";
                    break;
            }
            worker.employment.work.salaryDefaults = [];
            const defaultRow = {
                rowIndex: 0,
                rowType: null,
                count: null,
                price: null,
                message: null,
            };
            worker.employment.work.salaryDefaults.push(defaultRow);
            defaultRow.message = null;
            switch (worker.employment.type) {
                case exports.EmploymentRelationType.MonthlySalary:
                    defaultRow.rowType = exports.CalculationRowType.MonthlySalary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.HourlySalary:
                    defaultRow.rowType = exports.CalculationRowType.HourlySalary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = exports.CalculationRowUnit.Hours;
                    break;
                case exports.EmploymentRelationType.Salary:
                    defaultRow.rowType = exports.CalculationRowType.Salary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.EmployedByStateEmploymentFund:
                    defaultRow.rowType = exports.CalculationRowType.HourlySalary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = exports.CalculationRowUnit.Hours;
                    break;
                case exports.EmploymentRelationType.PerformingArtist:
                    defaultRow.rowType = exports.CalculationRowType.Salary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.KeyEmployee:
                    defaultRow.rowType = exports.CalculationRowType.MonthlySalary;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.Entrepreneur:
                case exports.EmploymentRelationType.Farmer:
                    break;
                case exports.EmploymentRelationType.Compensation:
                    defaultRow.rowType = exports.CalculationRowType.Compensation;
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.BoardMember:
                    defaultRow.rowType = exports.CalculationRowType.Board;
                    defaultRow.message = "Hallituspalkkio";
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
                case exports.EmploymentRelationType.Athlete:
                    defaultRow.rowType = exports.CalculationRowType.Salary;
                    defaultRow.message = "Urheilijan palkka";
                    defaultRow.count = 1;
                    defaultRow.price = 0;
                    defaultRow.unit = null;
                    break;
            }
        }
        /**
         * Returns options object for PensionCalculation selection by employment relation type.
         * @param employmentRelationType Employment relation type of the worker.
         */
        static getPensionCalculationOptions(employmentRelationType) {
            const types = [];
            switch (employmentRelationType) {
                case exports.EmploymentRelationType.HourlySalary:
                case exports.EmploymentRelationType.MonthlySalary:
                case exports.EmploymentRelationType.Salary:
                case undefined:
                case null:
                    types.push(exports.PensionCalculation.Employee);
                    break;
                case "employedByStateEmploymentFund": // EmploymentRelationType.EmployedByStateEmploymentFund
                    types.push(exports.PensionCalculation.Employee);
                    break;
                case "performingArtist": // EmploymentRelationType.PerformingArtist
                    types.push(...[exports.PensionCalculation.Employee, exports.PensionCalculation.Compensation]);
                    break;
                case "keyEmployee": // EmploymentRelationType.KeyEmployee
                    types.push(exports.PensionCalculation.Employee);
                    break;
                case exports.EmploymentRelationType.Entrepreneur:
                    types.push(...[exports.PensionCalculation.Entrepreneur, exports.PensionCalculation.SmallEntrepreneur, exports.PensionCalculation.PartialOwner, exports.PensionCalculation.Employee]);
                    break;
                case exports.EmploymentRelationType.Farmer:
                    types.push(...[exports.PensionCalculation.Farmer, exports.PensionCalculation.SmallFarmer, exports.PensionCalculation.Employee]);
                    break;
                case exports.EmploymentRelationType.Compensation:
                    types.push(exports.PensionCalculation.Compensation);
                    break;
                case exports.EmploymentRelationType.BoardMember:
                    types.push(exports.PensionCalculation.BoardRemuneration);
                    break;
                case exports.EmploymentRelationType.Athlete:
                    types.push(exports.PensionCalculation.Athlete);
                    break;
            }
            return types.map((type) => {
                const label = EnumerationsLogic.getEnumLabel("PensionCalculation", type);
                const descr = EnumerationsLogic.getEnumDescr("PensionCalculation", type);
                return { value: type, text: label, title: descr && descr !== type ? descr : null };
            });
        }
        /**
         * Populates given user with test data.
         * Sets values only if the firstName and lastName have been given.
         * @param worker - Worker to populate with test data.
         */
        static populateWorkerWithTestData(worker) {
            if (!worker || !worker.avatar || !worker.avatar.firstName || !worker.avatar.lastName) {
                return;
            }
            // email
            worker.contact.email = this.cleanupEmailPart(worker.avatar.firstName) + "." + this.cleanupEmailPart(worker.avatar.lastName) + "@palkkaus.fi";
            // phone
            worker.contact.telephone = "040 " + (1000000 + Math.round((Math.random() * 8999999))).toString();
            // hetu
            const birthYear = (new Date().getFullYear()) - (20 + Math.round(Math.random() * 40));
            const birthDate = new Date(new Date(birthYear, 0, 1).getTime() + Math.round(Math.random() * (new Date(birthYear + 1, 11, 31).getTime() - new Date(birthYear, 0, 1).getTime())));
            const separator = birthYear >= 2000 ? "A" : "-";
            const personNumber = 900 + Math.round(Math.random() * 99);
            const checksumLetters = "0123456789ABCDEFHJKLMNPRSTUVWXY";
            const sDate = Dates.format(birthDate, "DDMMYY");
            const idChecksum = checksumLetters[parseInt(sDate + personNumber.toString(), 10) % 31];
            worker.officialPersonId = sDate + separator + personNumber.toString() + idChecksum;
            // iban
            const bank = (100000 + Math.round((Math.random() * 99999)));
            const account = (100000 + Math.round((Math.random() * 99999)));
            const bankAccount = `${bank}00${account}`;
            const ibanValue = bankAccount + "15" + "18" + "00";
            const mod97 = (value) => {
                let checksum = value.slice(0, 2);
                for (let offset = 2; offset < value.length; offset += 7) {
                    const fragment = String(checksum) + value.substring(offset, offset + 7);
                    checksum = parseInt(fragment, 10) % 97;
                }
                return checksum;
            };
            const remainder = 98 - mod97(ibanValue);
            const ibanChecksum = remainder < 10 ? "0" + remainder.toString() : remainder.toString();
            worker.ibanNumber = Iban.formatIban("FI" + ibanChecksum + bankAccount);
        }
        static cleanupEmailPart(name) {
            if (!name) {
                return name;
            }
            // to lower case
            name = name.toLowerCase();
            // remove accents
            name = name.normalize("NFD").replace(/[\u0300-\u036f]/g, "");
            // leave only letters
            name = name.replace(/[^a-z]+/g, "");
            return name;
        }
    }
    /**
     * Application level single instance of the WorkerLogic.getSampleWorkers() methods.
     * These sample workers can be used and modified in the demo application.
     */
    WorkerLogic.sampleWorkersSingleton = WorkerLogic.getSampleWorkers();

    /**
     * Calculator is the main module for creating and modifying salary calculations.
     * The module can be used completely anonymously without any authentication.
     */
    class Calculator {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * OBSOLETE: use CalculatorLogic.getBlank() instead.
         * @deprecated Factory method to create a new calculation on server-side.
         * Typically, you do not need to call server-side factory methods in JavaScript.
         * Instead, you can use client-side factory methods under logic. This saves you one round-trip to server.
         * @ignore
         */
        createNew() {
            return this.ajax.getJSON("/calculator/new");
        }
        /**
         * Creates a new calucation with simple salary
         *
         * @param kind The type of the salary.
         * @param [amount=1] The amount of typically hours (could be months etc.). Defaults to 1 if left as null.
         * @param price The price of one unit of amount.
         * @param message Message text for end user - used in salary calculation
         * @param calc The calculation object that should be updated. This is optional - often null, but can be set if you are updating an existing calculation.
         *
         * @returns A Promise with the recalculated Calculation.
         */
        calculateSalary(kind, amount, price, message, calc = null) {
            const method = `/calculator/salary?kind=${kind}&amount=${Numeric.parseNumber(amount)}&price=${Numeric.parseNumber(price)}&message=${message}`;
            return this.ajax.postJSON(method, calc);
        }
        /**
         * Sets a household deduction to a calculation
         *
         * @param calc The calculation object to which household deduction is added
         * @param setHouseHoldDeductionTo Boolean to which household
         *
         * @returns A Promise with the recalculated Calculation.
         */
        setTaxDeduction(calc, setHouseHoldDeductionTo) {
            const method = `/calculator/taxdeduction?setHouseHoldDeduction=${setHouseHoldDeductionTo}`;
            return this.ajax.postJSON(method, calc);
        }
        /**
         * Takes in a calculation - typically with modified parameters - and recalculates it.
         *
         * @param calc The salary calculation that should be recalculated
         *
         * @returns A Promise with the recalculated Calculation.
         */
        recalculate(calc) {
            return this.ajax.postJSON("/calculator/recalculate", calc);
        }
        /**
         * Recalculates a calculation taking into account holiday year and absences from the worktime property.
         * This is an anonymous method that can be used for unit testing: Does not read / write from database.
         * @param calc The salary calculation that should be recalculated.
         * Worktime property should contian the bases for the recalculation.
         *
         * @returns A Promise with the recalculated Calculation.
         */
        recalculateWorktime(calc) {
            return this.ajax.postJSON("/calculator/recalculate-worktime", calc);
        }
        /**
         * Gets the worktime data for Period recalculation from raw calc, holidays andabsences data.
         * This is an anonymous method that can be used for unit testing: Does not read / write from database.
         *
         * @param calc The salary calculation that should be recalculated.
         * @param holidayYear Holiday year with holidays that should be added as salary rows. If not provided, holidays are not calculated.
         * @param absences Absences that should be deducted from monthly salary. If not provided, absences are not calculated.
         *
         * @returns A Promise with Worktime data.
         */
        getWorktimeData(calc, holidayYear, absences) {
            return this.ajax.postJSON("/calculator/worktime", {
                calc,
                holidayYear,
                absences,
            });
        }
        /**
         * Sets a unionPayment to a calculation
         *
         * @param calc The calculation object to which union payment is added
         * @param paymentType Type of union payment, options are 'notSelected', 'raksaNormal', 'raksaUnemploymentOnly' and 'other'
         * @param unionPaymentPercent How many percents the union payment is from the salary
         *
         * @returns A Promise with the recalculated Calculation.
         */
        setUnionPayment(calc, paymentType, unionPaymentPercent) {
            const method = `/calculator/setunionpayment?paymentType=${paymentType}&unionPaymentPercent=${Numeric.parseNumber(unionPaymentPercent)}`;
            return this.ajax.postJSON(method, calc);
        }
        /**
         * Adds a calculation row: An expenses, benefits or other such row and returns the recalculated calculation with full data.
         *
         * @param calc The salary calculation that should be modified.
         * @param rowType Type of the calculation row that should be added.
         * @param message Message text for end user - used in salary calculation
         * @param price Price for the row (e.g. €/h, €/km, €/month. Default is 0.
         * @param count Count of the items - default is 1.
         *
         * @returns A Promise with the recalculated Calculation.
         */
        addRow(calc, rowType, message, price, count) {
            const method = `/calculator/addrow?rowType=${rowType}&message=${message}&price=${Numeric.parseNumber(price)}&count=${Numeric.parseNumber(count)}`;
            return this.ajax.postJSON(method, calc);
        }
        /**
         * Deletes a calculation row: Expenses, benefits etc. item and returns the recalculated calculation with full data.
         *
         * @param calc The salary calculation that should be modified.
         * @param rowIndex Zero based row index of the row that should be deleted.
         *
         * @returns A Promise with the recalculated Calculation.
         */
        deleteRow(calc, rowIndex) {
            return this.ajax.postJSON(`/calculator/deleterow?rowIndex=${rowIndex}`, calc);
        }
        /**
         * Gets salary calculation parameters that are changing yearly.
         * The method is designed for the end-of-year and as such it only supports 2 years:
         * the current / previous year (from Jan to Nov) OR current / next year (in approx. December).
         *
         * @param forYear Date for which the numbers are fetched.
         *
         * @returns The yearly changing numbers if year supported. Error if year not supported.
         */
        getYearlyChangingNumbers(forYear) {
            return Years.getYearlyChangingNumbers(forYear);
        }
        /**
         * Experimental: Returns a new Finvoice object for the given calculations.
         * @param calculations Calculations for the Finvoice.
         * @param tableType Accounting table type for the generated Finvoice.
         * @ignore
         */
        getFinvoice(calculations, tableType = exports.AccountingReportTableType.Classic) {
            const method = `/calculator/finvoice/new?tableType=${encodeURIComponent("" + tableType)}`;
            return this.ajax.postJSON(method, calculations);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Calculator.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own calculations
     */
    class Calculations extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. (2) */
            this.baseUrl = "/calculations";
            this.calculator = new Calculator(ajax);
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return CalculatorLogic.getBlank();
        }
        /**
         * Takes in a calculation - typically with modified parameters - and recalculates it.
         * @param calc The salary calculation that should be recalculated
         * @returns A Promise with the recalculated Calculation.
         */
        recalculate(calc) {
            return this.calculator.recalculate(calc);
        }
        /**
         * Recalculates a calculation taking into account holiday year and absences from the worktime property.
         * This is an anonymous method that can be used for unit testing: Does not read / write from database.
         * @param calc The salary calculation that should be recalculated.
         * Worktime property should contian the bases for the recalculation.
         *
         * @returns A Promise with the recalculated Calculation.
         */
        recalculateWorktime(calc) {
            return this.calculator.recalculateWorktime(calc);
        }
        /**
         * Gets the worktime data for Period recalculation based on given calculation
         * but fetching absences and holidays from the database.
         *
         * @param calc The salary calculation that should be recalculated.
         * It may potentially have changes to compared to the one in database or the calc may not have even been saved.
         * The calculation MUST have emloymentId and period set for calculation to work properly.
         * @returns A Promise with Worktime data.
         */
        getWorktimeData(calc) {
            return this.ajax.postJSON(`${this.baseUrl}/worktime`, calc);
        }
        /**
         * Gets the Calculation objects for the given ids.
         * @param ids Identifiers for the Calculation objects.
         * @returns A Promise with result data array.
         */
        getMulti(ids) {
            if (ids == null || ids.length === 0) {
                return Promise.resolve([]);
            }
            let method = this.baseUrl + "/set?";
            ids.forEach((x) => { method += "ids=" + encodeURIComponent("" + x) + "&"; });
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
        /**
         * Gets the Calculations for a specific employment relation.
         * @param employmentId Identifier for the Employment relation.
         * @returns A Promise with Calculations.
         */
        getForEmployment(employmentId) {
            return this.ajax.getJSON(`${this.baseUrl}/employment/${employmentId}`);
        }
        /**
         * Gets the Calculations for the given employment relations.
         * @param employmentIds Identifiers for the Employment relations.
         * @returns A Promise with Calculations.
         */
        getForEmployments(employmentIds) {
            if (employmentIds == null || employmentIds.length === 0) {
                return Promise.resolve([]);
            }
            let method = `${this.baseUrl}/employment/set?`;
            employmentIds.forEach((x) => { method += "employmentIds=" + encodeURIComponent("" + x) + "&"; });
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
        /**
         * Updates a calculation from employment relation by deleting all the rows and re-setting the employment.
         * @param calc Calculation to update.
         * @param save If true, saves the calculation. If false, only recalculates. Returns the result in any case.
         * @param updateRows If true, updates also the default salary rows from employment data to calculation.
         * False updates only other employment data IBAN, phone, E-mail, pension insurance code message etc.
         * @returns The updated and saved or recalculated calculation
         */
        updateFromEmployment(calc, save, updateRows) {
            return this.ajax.postJSON(`${this.baseUrl}/update-from-employment?save=${save}&updateRows=${updateRows}`, calc);
        }
        /**
         * Updates (saves) the employment related information from Calculation to Employment relation.
         * Currently only updates rows, OccupationCode and WorkDescription.
         * @param calc Calculation to update.
         * @returns The updated and saves employment relation
         */
        updateToEmployment(calc) {
            return this.ajax.postJSON(`${this.baseUrl}/update-to-employment`, calc);
        }
        /**
         * Shares the calculation.
         * @param id Id for the calculation to share.
         * @param type Type of the sharing action or link.
         * @param employerId Id of the employer to share the calculation with.
         * @param to Email address of the employer.
         * @param cc Cc address for the email.
         * @param message Message to the employer.
         * @returns Sharing object of the calculation.
         */
        share(id, type = exports.SharingUriType.Undefined, employerId = null, to = null, cc = null, message = null) {
            let method = `${this.baseUrl}/sharing/${id}?type=${encodeURIComponent(type)}${employerId ? "&employerId=" + encodeURIComponent(employerId) : ""}`;
            method += `${to ? "&to=" + encodeURIComponent(to) : ""}${cc ? "&cc=" + encodeURIComponent(cc) : ""}${message ? "&message=" + encodeURIComponent(message) : ""}`;
            return this.ajax.postJSON(method, null);
        }
        /**
         * Notifies the employer from the shared calculation.
         * @param id Id of the shared calculation.
         * @param to Email address of the employer.
         * @param cc Cc address for the email.
         * @param message Message to the employer.
         * @returns True, if the send action was successful.
         */
        notifySharing(id, to = null, cc = null, message = null) {
            let method = `${this.baseUrl}/sharing/${id}/notification?`;
            method += `${to ? "&to=" + encodeURIComponent(to) : ""}${cc ? "&cc=" + encodeURIComponent(cc) : ""}${message ? "&message=" + encodeURIComponent(message) : ""}`;
            return this.ajax.postJSON(method, null);
        }
        /**
         * Cancels the invoice or payment processing for the given calculation.
         * Please note that this is possible only if the channel supports the cancel action.
         * @param id Calculation for which the cancellation is requested.
         * @returns Calculation which have been cancelled.
         */
        cancel(id) {
            const url = `${this.baseUrl}/cancel/${id}`;
            return this.ajax.getJSON(url);
        }
        /** Available object mappers for import/export */
        getObjectMappers() {
            const map = {
                keys: ["id", "personalId", "salaryDate"],
                id: "id",
                status: "workflow.status",
                personalId: "worker.paymentData.socialSecurityNumber",
                occupationCode: "info.occupationCode",
                periodStart: "info.workStartDate",
                periodEnd: "info.workEndDate",
                workDescription: "info.workDescription",
                salarySlipMessage: "info.salarySlipMessage",
                projectNumber: "info.projectNumber",
                costCenter: "info.costCenter",
                salaryDate: "workflow.salaryDate",
                fixedTaxAmount: "worker.tax.fixedTaxAmount",
                rows: {
                    "rows": {
                        rowType: "rowType",
                        message: "message",
                        count: "count",
                        price: "price",
                    },
                },
            };
            const defaultMapper = new Mapper(map, new Calculations(null).getBlank());
            defaultMapper.rowType = "CalculationsDataRow";
            defaultMapper.description = "Vakioaineisto";
            return [defaultMapper];
        }
        /** Available list item mappers for import/export */
        getListItemMappers() {
            const paths = [
                "status",
                "otherPartyInfo.avatar.displayName",
                "otherPartyInfo.officialId",
                "data.workStartDate",
                "data.workEndDate",
                "data.shortText",
                "salaryDate",
                "data.workerPayment",
                "data.taxPayment",
                "grossSalary"
            ];
            const defaultMapper = Mappers.pathMapper(paths);
            defaultMapper.rowType = "ApiListItem";
            defaultMapper.description = "Esikatselu";
            defaultMapper.isListItemMapper = true;
            return [defaultMapper];
        }
        /**
         * Validates the item using a server side call.
         *
         * @param itemToValidate - The item that is validated.
         * @returns A Promise with result data containing validation property.
         */
        validate(itemToValidate) {
            return Promise.resolve(itemToValidate);
        }
        /**
         * Returns the validation object for the validated object.
         * @param validatedItem The validated object with a validation result.
         */
        getValidationResult(validatedItem) {
            return { isValid: true };
        }
        /**
         * Saves the IR -related data of the calculation.
         * This operation can be done even if the calculation otherwise is in read only state.
         * @param calculation Calculation object to save.
         */
        saveIr(calculation) {
            return this.ajax.postJSON(`${this.baseUrl}/ir`, calculation);
        }
        /**
         * Converts the given calculation to an earnings payment report.
         * @param calculation - Calculation object.
         * @returns A Promise with earnings payment report data.
         */
        convertToEpr(calculation) {
            return this.ajax.postJSON(`${this.baseUrl}/epr`, calculation);
        }
        /**
         * Returns the earnings payment report for the given calculation.
         * @param id Id of the calculation for the earnings payment report.
         * @returns Earnings payment report for the calculation.
         */
        getEpr(id) {
            return this.ajax.getJSON(`${this.baseUrl}/epr/${id}`);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Calculations.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for Calendar events and other calendar methods.
     */
    class CalendarEvents extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/calendar/events";
            /** URL for top-level (non calendar event) methods. */
            this.calendarBaseUrl = "/v03-rc/api/calendar";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                event: {
                    isAllDay: true,
                    start: Dates.getToday(),
                    end: Dates.getToday(),
                    duration: "1",
                    actions: [],
                    attachments: [],
                    attendees: [],
                    categories: [],
                    recurrenceRules: [],
                },
                bo: {},
                workflowData: {
                    events: [],
                }
            };
        }
        /**
         * Recalculates the occurence info based on the event.
         * This is an anonymous method that does not commit anything to database.
         * @param event Calendar event to recalculate.
         * @returns A Promise with result data: The calendar event with occurenceInfo recalculated.
         */
        recalculate(event) {
            return this.ajax.postJSON(`${this.baseUrl}/recalculate`, event);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    CalendarEvents.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own Certificates
     */
    class Certificates extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/accounts/certificate";
        }
        /**
         * Returns an empty certificate without any values.
         */
        getBlank() {
            return {};
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Certificates.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access all credentials which can access this account.
     */
    class Credentials extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/accounts/credential";
        }
        /**
         * Returns an empty credential without any values.
         */
        getBlank() {
            return {};
        }
        /**
         * Returns the url where to post the avatar file
         */
        getAvatarUploadUrl(credentialId) {
            return this.ajax.getApiAddress()
                + this.baseUrl
                + (credentialId ? `/${credentialId}` : "")
                + "/avatar";
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Credentials.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access to employment relations.
     */
    class Employments extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/employments";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            throw new Error("Not supported.");
        }
        /**
         * Returns a list of all employing relations of existing employers for the current account.
         */
        getODataForEmployingRelations(options) {
            const fullPath = OData.getUrl(`${this.baseUrl}/employing`, options);
            return this.ajax.getJSON(fullPath).then((data) => {
                return OData.getODataResult(data);
            });
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Employments.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access to employment contracts.
     * NOTE: Currently, this API is read-only.
     */
    class EmploymentContracts extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/employment-contracts";
        }
        /**
         * Client-side (synchronous) method for getting a new blank item as bases for UI binding.
         * NOTE: Currently, this API is read-only => Method is not supported.
         */
        getBlank() {
            throw new Error("Not supported.");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    EmploymentContracts.$inject = ["AjaxNg1"];

    /**
     * Provides access to user files (of the authenticated user).
     * PLEASE NOTE: Currently the API is read-only.
     * The methods for delete save etc. will return a "Not supported." exceptions.
     */
    class Files extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/files";
        }
        /**
         * Gets a list of files - filtered by a type
         *
         * @param type Logical file type.
         *
         * @returns A Promise with result data (list of files).
         */
        filesOfType(type) {
            return this.ajax.getJSON(`${this.baseUrl}/type/${type}`);
        }
        /**
         * Returns the content url for the file id.
         * The returned does not contain access token.
         * @param id - Identifier of the file.
         */
        getContentUrl(id) {
            return `${this.ajax.getApiAddress()}/${this.baseUrl}/${id}/stream`;
        }
        /**
         * Returns the preview image url for the file id.
         * The returned url does not contain access token.
         * @param id - Identifier of the file.
         */
        getPreviewUrl(id) {
            return `${this.ajax.getApiAddress()}/${this.baseUrl}/${id}/preview`;
        }
        /**
         * Downloads the content as bytes.
         * @param id - The identifier of the file.
         */
        download(id) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}/content`);
        }
        /**
         * Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.
         * The basic idea is that all the child object properties should be non-null to avoid null reference exceptions.
         * In special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.
         * Strings, dates and booleans may be null (false if bool), but typically just undefined.
         */
        getBlank() {
            throw new Error("Not supported.");
        }
        /**
         * Gets all the items of a given type.
         * @throws Not supported
         */
        getAll() {
            throw new Error("Not supported.");
        }
        /**
         * Deletes an single item from the sotrage
         * @param id - Unique identifier for the object
         * @throws Not supported
         */
        delete(id) {
            throw new Error("Not supported.");
        }
        /**
         * Saves an item to the storage.
         * If id is null, this is add/insert. If id exists, this is update.
         * @param itemToSave - The item that is updated/inserted.
         * @throws Not supported
         */
        save(itemToSave) {
            throw new Error("Not supported.");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Files.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for Holiday Years as a raw list and
     * more typically as holiday years relate to an employment relation.
     */
    class HolidayYears extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/holidays";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return HolidaysLogic.getBlank();
        }
        /**
         * Gets all the Holiday Year objects of specified year.
         * @param year The holiday year to filter
         * @returns A Promise with result data: The full holiday year objects.
         */
        getForYear(year) {
            return this.ajax.getJSON(`${this.baseUrl}/year/${year.toFixed()}`);
        }
        /**
         * Gets a default holiday specification based on employment relation information.
         * This is presented to the user before calling initForEmployment().
         * @param employmentId The employment relation ID from which the defaults are fetched.
         * @returns Holiday specification that can be used for inititalization.
         */
        getDefaultSpecForEmployment(employmentId) {
            const url = `${this.baseUrl}/employment/${employmentId}/default-spec`;
            return this.ajax.getJSON(url);
        }
        /**
         * Initializes (or re-creates) the holiday years for an employment relation.
         * @param employmentId Identifier of the Employment relation that stores the holiday specification.
         * @param holidaySpec The holiday specification tht should be used for inititalization. The spec is also stored to the Employment relation.
         * @returns Holiday years for this employment relation - as initialized.
         */
        initForEmployment(employmentId, holidaySpec) {
            const url = `${this.baseUrl}/employment/${employmentId}`;
            return this.ajax.postJSON(url, holidaySpec);
        }
        /**
         * Gets the Holiday Years for a specific employment relation.
         * @param employmentId Identifier for the Employment relation.
         * @returns A Promise with Holiday Years.
         */
        getForEmployment(employmentId) {
            return this.ajax.getJSON(`${this.baseUrl}/employment/${employmentId}`);
        }
        /**
         * Gets the Holiday Years for the given employment relations.
         * @param employmentIds Identifiers for the Employment relations.
         * @returns A Promise with Holiday Years.
         */
        getForEmployments(employmentIds) {
            if (employmentIds == null || employmentIds.length === 0) {
                return Promise.resolve([]);
            }
            let method = "${this.baseUrl}/employment/set?";
            employmentIds.forEach((x) => { method += "employmentIds=" + encodeURIComponent("" + x) + "&"; });
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    HolidayYears.$inject = ["AjaxNg1"];

    /**
     * Provides read access for authenticated user to access his/her own invoices.
     */
    class Invoices extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/invoices";
        }
        /**
         * Client-side (synchronous) method for getting a new blank item as bases for UI binding.
         * NOTE: At the moment, this method is pretty useless as you cannot save invoice items:
         * You need to create them using createInvoice() method based on a calculation.
         */
        getBlank() {
            return {
                header: {},
                rows: [],
                payer: {},
                recipient: {},
            };
        }
        /**
         * Creates invoices to pay a salary calculation.
         * @param calc Calculation for which the invoices are created.
         * @param channel Payment channel to use.
         * @returns Invoices which have been created in the Invoices queue.
         */
        createInvoices(calc, channel) {
            if (!channel) {
                throw new Error("Payment channel must be specified.");
            }
            const url = `${this.baseUrl}/create/${channel}`;
            return this.ajax.postJSON(url, calc);
        }
        /**
         * Creates invoices to pay a salary calculation(s).
         * @param channel Payment channel to use.
         * @param calcIds One or several identifiers of the Calculation for which the invoices are created.
         * @returns Invoices which have been created in the Invoices queue.
         */
        createInvoicesById(channel, ...calcIds) {
            if (!channel || !calcIds || calcIds.length < 1) {
                throw new Error("One or more calculations Ids and Payment channel must be specified.");
            }
            const url = `${this.baseUrl}/create-by-id/${channel}`;
            return this.ajax.postJSON(url, calcIds);
        }
        /**
         * Creates invoices to pay a set of salary calculation.
         * @param channel Payment channel to use.
         * @param payrollId Identifier for the Payroll for which the invoices are created.
         * @returns Invoices which have been created in the Invoices queue.
         */
        createInvoicesForPayroll(channel, payrollId) {
            if (!channel || !payrollId) {
                throw new Error("The payroll ID and Payment channel must be specified.");
            }
            const url = `${this.baseUrl}/create-for-payroll/${channel}/${payrollId}`;
            return this.ajax.postJSON(url, null);
        }
        /**
         * Previews invoices to pay a salary calculation(s) and fetches the ones that have already been created.
         * Used for payment dialog preview: Does not yet save the invoices and does not do any other storage saves.
         * @param channel Payment channel to use.
         * @param calcs The calculations for which the invoices are created as preview (not saved to storage).
         * Specified as ID's or unsaved Calculations or Payroll detail items.
         * @returns Invoices (list item + metadata) which have been created for preview (not saved) or that were previously created.
         */
        previewInvoices(channel, calcs) {
            const itemsCount = (calcs.ids || []).length + (calcs.payrollIds || []).length + (calcs.calcs || []).length + (calcs.payrolls || []).length;
            if (!channel || itemsCount < 1) {
                throw new Error("One or more calculations/payroll/id and Payment channel must be specified.");
            }
            const url = `${this.baseUrl}/preview-and-created/${channel}`;
            return this.ajax.postJSON(url, calcs);
        }
        /**
         * Makes a request for updating the status of the invoice.
         * @param id Identifier for the invoice.
         * @param status The requested new status of the invoice.
         */
        requestStatusUpdate(id, status) {
            const url = `${this.baseUrl}/${id}/status/${status}`;
            return this.ajax.postJSON(url, null);
        }
        /**
         * Returns content (PDF) url for an invoice.
         * @param id The id of the invoice.
         * @param inline If true, returns the content as inline (disposition) in browser.
         * If false, the content is returned as an attachment (download).
         */
        getContentUrl(id, inline = false) {
            return this.ajax.getServerAddress()
                + `${this.baseUrl}/${id}/content?inline=${inline}`
                + `&access_token=${encodeURIComponent(this.ajax.getCurrentToken())}`;
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Invoices.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own Earnings Payments objects
     */
    class IrEarningsPayments extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v01/api/epr";
            this.baseUrl = `${ajax.getServerAddress()}/v01/api/epr`;
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                deliveryData: null
            };
        }
        /** Get odata base url */
        getODataUrl() {
            return this.baseUrl;
        }
        /** Validate */
        validate(earningsPayment) {
            return this.ajax.postJSON(`${this.baseUrl}/validate`, earningsPayment);
        }
        /** Send schedule item to Incomes register queue */
        sendSchedule(aso) {
            return this.ajax.postJSON(`${this.baseUrl}/schedule`, aso);
        }
        /** Get Service Bus queue item */
        getIrQueueItem(id) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}/schedule`);
        }
        /** Remove Service Bus queue item */
        removeIrQueueItem(id) {
            return this.ajax.remove(`${this.baseUrl}/${id}/schedule`);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    IrEarningsPayments.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own Earnings Payments objects
     */
    class IrPayerSummaries extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v01/api/psr";
            this.baseUrl = `${ajax.getServerAddress()}/v01/api/psr`;
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                deliveryData: null
            };
        }
        /** Get odata base url */
        getODataUrl() {
            return this.baseUrl;
        }
        /** Validate */
        validate(earningsPayment) {
            return this.ajax.postJSON(`${this.baseUrl}/validate`, earningsPayment);
        }
        /** Send schedule item to Incomes register queue */
        sendSchedule(aso) {
            return this.ajax.postJSON(`${this.baseUrl}/schedule`, aso);
        }
        /** Get Service Bus queue item */
        getIrQueueItem(id) {
            return this.ajax.getJSON(`${this.baseUrl}/${id}/schedule`);
        }
        /** Remove Service Bus queue item */
        removeIrQueueItem(id) {
            return this.ajax.remove(`${this.baseUrl}/${id}/schedule`);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    IrPayerSummaries.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for Message threads.
     */
    class MessageThreads extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/messages";
        }
        /** Gets a new blank message thread. */
        getBlank() {
            return {
                messages: [],
            };
        }
        /**
         * Adds a new message with text.
         * @param id Identifier of the messages thread.
         * @param message The message to add.
         */
        addMessage(id, message) {
            const method = `${this.baseUrl}/${id}/messages`;
            return this.ajax.postJSON(method, message);
        }
        /**
         * Gets the upload URL for attachment files of a message thread.
         * This URL is compatible with the HTML forms standard multipart/form-data upload.
         * @param id Identifier of the messages thread.
         * @returns The URL for multipart/form-data upload
         */
        getUploadUrl(id) {
            return `${this.ajax.getServerAddress()}${this.baseUrl}/${id}/files`;
        }
        /**
         * Marks all messages as read by a given party
         * @param id Identifier of the thread
         * @param readBy The party for which the messages are marked as read.
         * @returns The entire thread with the messages marked as read.
         */
        markAsRead(id, readBy) {
            const method = `${this.baseUrl}/${id}/read-by/${readBy}`;
            return this.ajax.postJSON(method, null);
        }
        /**
         * Sends an email notification to the owner of the message thread.
         * @param id Identifier of the thread
         * @param subject Subject of the notification.
         * @param body Body of the notification.
         * @returns A boolean indicating if the email was sent successfully.
         */
        sendNotification(id, subject, body) {
            const method = `${this.baseUrl}/${id}/notifications?subject=${encodeURIComponent(subject)}`;
            return this.ajax.postJSON(method, JSON.stringify(body));
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    MessageThreads.$inject = ["AjaxNg1"];

    /**
     * Methods for payment creation and processing.
     */
    class Payments extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/payments";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                payer: {},
                recipient: {},
                businessObjects: [],
                request: {},
            };
        }
        /**
         * List payments for the current user.
         *
         * @param status - Payment status filter. If null or empty, returns all statuses.
         *
         * @returns A Promise with result data (list of payments).
         */
        getPayments(status) {
            if (!status) {
                return this.getAll();
            }
            let method = "/payments?";
            if (status) {
                status.forEach((item) => { method += "status=" + encodeURIComponent("" + item) + "&"; });
            }
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
        /**
         * Gets a single payment object based on identifier.
         *
         * @param id - Unique identifier for the payment.
         *
         * @returns A Promise with result data (Payment).
         */
        getPayment(id) {
            return this.getSingle(id);
        }
        /**
         * Deletes a payment.
         *
         * @param id - Unique identifier for the payment.
         *
         * @returns A Promise with result data "Object deleted".
         */
        deletePayment(id) {
            return this.delete(id);
        }
        /**
         * OBSOLETE
         * @deprecated Creates a new SalaryGrossPayment (startPayment will replace this method).
         * @ignore
         *
         * @param calculationIds - Array of calculation ids.
         * @param successUrl - Return address after successful payment
         * @param cancelUrl - Return address after cancelled payment
         * @param sendEmails - Boolean indicating whether to send emails in the workflow
         *
         * @returns A Promise with result data (created Payment).
         */
        createSalaryGrossPayment(calculationIds, successUrl, cancelUrl, sendEmails) {
            let method = "/payments/salaryGrossPayment?";
            calculationIds.forEach((item) => { method += "calculationIds=" + encodeURIComponent("" + item) + "&"; });
            method += "successUrl=" + encodeURIComponent("" + successUrl) + "&";
            method += "cancelUrl=" + encodeURIComponent("" + cancelUrl) + "&";
            method += "sendEmails=" + encodeURIComponent(sendEmails);
            return this.ajax.postJSON(method, calculationIds);
        }
        /**
         * Creates/updates the SalaryGrossPayment.
         * If a payment for the same calculation ids exist, it will be updated and no new payment
         * is created.
         *
         * @param calculationIds - Array of calculation ids.
         * @param paymentMethod - Payment method.
         * @param successUrl - Return address after successful payment
         * @param cancelUrl - Return address after cancelled payment
         * @param sendEmails - Boolean indicating whether to send emails in the workflow
         *
         * @returns A Promise with result data (created Payment).
         */
        startPayment(calculationIds, paymentMethod, successUrl, cancelUrl, sendEmails) {
            let method = "/payments?";
            calculationIds.forEach((item) => { method += "calculationIds=" + encodeURIComponent("" + item) + "&"; });
            method += "paymentMethod=" + encodeURIComponent("" + paymentMethod) + "&";
            method += "successUrl=" + encodeURIComponent("" + successUrl) + "&";
            method += "cancelUrl=" + encodeURIComponent("" + cancelUrl) + "&";
            method += "sendEmails=" + encodeURIComponent(sendEmails);
            return this.ajax.postJSON(method, {});
        }
        /**
         * Experimental: Gets a new finvoice object for the given payment.
         * @param id - The payment id which the finvoice will be returned for.
         * @param tableType - Accounting table type for the generated Finvoice.
         * @ignore
         */
        getFinvoice(id, tableType = exports.AccountingReportTableType.Classic) {
            let method = `${this.baseUrl}/${id}/finvoice?`;
            method += "tableType=" + encodeURIComponent("" + tableType);
            return this.ajax.postJSON(method, "");
        }
        /**
         * Experimental: Sends the finvoice object to operator.
         * @param finvoice - Finvoice for the delivery.
         *
         * @returns Finvoice which was sent.
         * @ignore
         */
        sendFinvoice(finvoice) {
            const method = `${this.baseUrl}/finvoice`;
            return this.ajax.postJSON(method, finvoice);
        }
        /**
         * Returns the url for starting the payment process for the payroll.
         * @param calculationIds - Array of calculation ids.
         * @param paymentMethod - Payment method.
         * @param successUrl - Return address after successful payment
         * @param cancelUrl - Return address after cancelled payment
         * @param sendEmails - Boolean indicating whether to send emails in the workflow
         * @returns The url to the Paytrail payment page.
         */
        getPaytrailPaymentUrl(calculationIds, paymentMethod, successUrl, cancelUrl, sendEmails) {
            let url = this.ajax.getServerAddress() + "/SalaryPayment/PayCalculations?";
            calculationIds.forEach((item) => { url += "calculationIds=" + encodeURIComponent("" + item) + "&"; });
            url += "successUrl=" + encodeURIComponent("" + successUrl) + "&";
            url += "cancelUrl=" + encodeURIComponent("" + cancelUrl) + "&";
            url += "sendEmails=" + encodeURIComponent(sendEmails) + "&";
            url += "paymentMethod=" + encodeURIComponent("" + paymentMethod) + "&";
            url += "access_token=" + encodeURIComponent("" + this.ajax.getCurrentToken());
            return url;
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Payments.$inject = ["AjaxNg1"];

    /**
     * Payment channels are methods that hanlde salary payments:
     * Paytrail / Palkkaus.fi Customer funds account handling, Finvoice, Accounting software, Banks Web Services etc.
     * This controller is for those channels to fetch Invoices and other data as well as to update payment feedback back to Palkkaus.fi.
     * The services can only be used by registered payment channels: Ask us if you wish to register.
     */
    class PaymentChannelApi extends CrudApiBase {
        /** Creates a new API class for a given PaymentChannel */
        constructor(paymentChannel, ajax) {
            super(ajax);
            this.paymentChannel = paymentChannel;
            /** Base URL is set by the constructor because it is using the paymentChannel as part of URL. */
            this.baseUrl = null;
            this.baseUrl = `/v03-rc/api/payment-channel/${paymentChannel}`;
        }
        /**
         * NOTE: Direct Save is currently not supported.
         * Use createInvoices and updateStatus instead.
         * This method is only for compatibility with CrudApiBase and in expectation of future needs (non-salary invoices).
         * @param itemToSave - The item that is updated/inserted.
         * @throws Not supported
         */
        save(itemToSave) {
            throw new Error("Not supported.");
        }
        /**
         * Client-side (synchronous) method for getting a new blank item as bases for UI binding.
         * NOTE: At the moment, this method is pretty useless as you cannot save invoice items:
         * You need to create them using createInvoice() method based on a calculation.
         */
        getBlank() {
            const invoice = new Invoices(this.ajax).getBlank();
            invoice.header.channel = this.paymentChannel;
            return invoice;
        }
        /**
         * Gets invoices with their full payload - including PDF content - filtering by a status.
         * NOTE: This is designed mainly for long-running batch processes for reading unread Invoices.
         * They can read the items in one read. For browser process we suggest OData query (with list item)
         * and then fetching items one-by-one.
         * This is for more http-queries, but each taking a shorter time so you can use )
         * @param status Status filter for the query. Default is unread.
         * @param start Optional start date for the period to query the invoices. If not set, defaults to last 14 days.
         * @param end Optional end date for the period to query the invoices. If not set, today is used.
         * @returns Invoices which have been created in the Invoices queue.
         */
        getInvoicesByStatus(status = exports.InvoiceStatus.Unread, start = null, end = null) {
            if (this.paymentChannel) {
                start = Dates.asDate(start);
                end = Dates.asDate(end);
                const url = `${this.baseUrl}/status/${status}?start=${start || "null"}&end=${end || "null"}`;
                return this.ajax.getJSON(url);
            }
            else {
                return Promise.resolve([]);
            }
        }
        /**
         * Makes an OData query to the service's main OData method.
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         */
        getOData(query) {
            if (this.paymentChannel) {
                return super.getOData(query);
            }
            return Promise.resolve({ items: [], count: 0 });
        }
        /**
         * Gets all the items of a given type.
         * @throws Not supported
         */
        getAll() {
            throw new Error("Not supported.");
        }
        /**
         * Gets a single item based on identier
         * @param id - Unique identifier for the object
         * @returns A Promise with result data.
         */
        getSingle(id) {
            if (this.paymentChannel) {
                return super.getSingle(id);
            }
            return Promise.resolve(null);
        }
        /**
         * Updates the payment statuses of invoices.
         * @param data Array of invoices to update.
         */
        updateStatus(data) {
            const url = `${this.baseUrl}/update-status`;
            return this.ajax.postJSON(url, data);
        }
        /**
         * Returns content (PDF) url for an invoice.
         * @param id The id of the invoice.
         * @param inline If true, returns the content as inline (disposition) in browser.
         * If false, the content is returned as an attachment (download).
         */
        getContentUrl(id, inline = false) {
            return `${this.ajax.getServerAddress()}${this.baseUrl}/${id}/content?inline=${inline}`
                + `&access_token=${encodeURIComponent(this.ajax.getCurrentToken())}`;
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    PaymentChannelApi.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for authenticated user to access a his/her own PayrollDetails objects
     */
    class Payrolls extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/payroll";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                input: {
                    period: {},
                    calculations: [],
                },
                info: {},
                calcs: [],
                validations: [],
                workflowData: {},
            };
        }
        /** Because of historical reasons, Payrolls v03 has a non-standard endpoint for OData methods. */
        getODataUrl() {
            return this.baseUrl + "/odata";
        }
        /**
         * Validates a PayrollDetails object for payment.
         * @param payrollToValidate PayrollDetails that should be validated.
         * @returns The PayrollDetails object with Calculations added to the Result object and Validated.
         */
        validate(payrollToValidate) {
            return this.ajax.postJSON(`${this.baseUrl}/validate`, payrollToValidate);
        }
        /**
         * Starts the payment process for the payroll.
         * Creates and saves the calculations as well as the new payment object.
         * Updates the status of calculations and payroll to paymentStarted.
         * @param payroll PayrollDetails which is going to be paid.
         * @param paymentMethod - Payment method.
         * @param successUrl - Return address after successful payment
         * @param cancelUrl - Return address after cancelled payment
         * @param sendEmails - Boolean indicating whether to send emails in the workflow
         * @returns The PayrollDetails object with payment id.
         */
        startPayment(payroll, paymentMethod, successUrl, cancelUrl, sendEmails) {
            let method = `${this.baseUrl}/payment?`;
            method += "paymentMethod=" + encodeURIComponent("" + paymentMethod) + "&";
            method += "successUrl=" + encodeURIComponent("" + successUrl) + "&";
            method += "cancelUrl=" + encodeURIComponent("" + cancelUrl) + "&";
            method += "sendEmails=" + encodeURIComponent(sendEmails);
            return this.ajax.postJSON(method, payroll);
        }
        /**
         * Saves changes to to a Calculation that is part of a Payroll and returns the Paryoll validation data.
         * @param calc The calculation object that is updated.
         * @param payrollId Payroll ID where calculation belongs to.
         * Note, that the current implementation does not assure that the calculation is within a Payroll.
         * The ID is simply used for assuring corrrect data in Calculation.
         * @returns The Saved Payroll object with Calculations added to the Result object and Validated.
         */
        calculationSave(calc, payrollId) {
            const method = `${this.baseUrl}/${payrollId}/calcs`;
            return this.ajax.postJSON(method, calc);
        }
        /**
         * Adds existing calculations to the the payroll and saves the payroll object.
         * If calculation is read-only, a copy is made. If it is draft, the calculation is moved.
         * @param payroll The Payroll object to update and save.
         * @param calculationIds Array of calculation id's that are copied / moved to the Payroll list.
         * @returns The Saved Payroll object with Calculations added to the Result object and Validated.
         */
        addCalculations(payroll, calculationIds) {
            let method = `${this.baseUrl}/add-calc?`;
            method += "ids=" + calculationIds.join(",");
            return this.ajax.postJSON(method, payroll);
        }
        /**
         * Remove calculations from the payroll and saves the payroll object.
         * The calculations are completely deleted.
         * This can only be done if the calculations and the payroll have not been paid yet.
         * @param payroll The Payroll  object.
         * @param calculationIds Array of calculation id's that are deleted.
         * @returns The Saved Payroll object with Calculations removed from the Result object and Validated.
         */
        deleteCalculations(payroll, calculationIds) {
            let method = `${this.baseUrl}/delete-calc?`;
            method += "ids=" + calculationIds.join(",");
            return this.ajax.postJSON(method, payroll);
        }
        /**
         * Remove calculations from the payroll and saves the payroll object.
         * The calculations are unlinked and they become individual calculations in the draft list (and can then be processed individually).
         * This can only be done if the calculations and the payroll have not been paid yet.
         * @param payroll The Payroll  object.
         * @param calculationIds Array of calculation id's that are unlinked from the Payroll list.
         * @returns The Saved Payroll object with Calculations removed from the Result object and Validated.
         */
        unlinkCalculations(payroll, calculationIds) {
            let method = `${this.baseUrl}/unlink-calc?`;
            method += "ids=" + calculationIds.join(",");
            return this.ajax.postJSON(method, payroll);
        }
        /**
         * Adds new calculations to the the payroll and saves the payroll object.
         * The calculations are added based on employment relation ID's.
         * @param payroll The Payroll object to update and save.
         * @param employmentIds Array of employment relation id's for which the a new calculation is added to the Payroll list.
         * @returns The Saved Payroll object with Calculations added to the Result object and Validated.
         */
        addEmployments(payroll, employmentIds) {
            let method = `${this.baseUrl}/add-employment?`;
            method += "ids=" + employmentIds.join(",");
            return this.ajax.postJSON(method, payroll);
        }
        /**
         * Returns the url for starting the payment process for the payroll.
         * @param payrollId Payroll which is going to be paid.
         * @param paymentMethod - Payment method.
         * @param successUrl - Return address after successful payment
         * @param cancelUrl - Return address after cancelled payment
         * @param sendEmails - Boolean indicating whether to send emails in the workflow
         * @returns The url to the Paytrail payment page.
         */
        getPaytrailPaymentUrl(payrollId, paymentMethod, successUrl, cancelUrl, sendEmails) {
            let url = this.ajax.getServerAddress() + "/SalaryPayment/PayPayroll?";
            url += "payrollId=" + encodeURIComponent("" + payrollId) + "&";
            url += "successUrl=" + encodeURIComponent("" + successUrl) + "&";
            url += "cancelUrl=" + encodeURIComponent("" + cancelUrl) + "&";
            url += "sendEmails=" + encodeURIComponent(sendEmails) + "&";
            url += "paymentMethod=" + encodeURIComponent("" + paymentMethod) + "&";
            url += "access_token=" + encodeURIComponent("" + this.ajax.getCurrentToken());
            return url;
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Payrolls.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for accounting settings
     */
    class PrimaryPartners extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/settings/primary-partners";
        }
        /**
         * Makes a search to YTJ.
         * @param search Search text: company name or business id.
         */
        searchYtj(search) {
            return this.ajax.getJSON(`/v03-rc/api/settings/ytj-companies?search=${encodeURIComponent(search)}`);
        }
        /**
         * Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.
         * The basic idea is that all the child object properties should be non-null to avoid null reference exceptions.
         * In special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.
         * Strings, dates and booleans may be null (false if bool), but typically just undefined.
         */
        getBlank() {
            throw new Error("Not supported.");
        }
        /**
         * Gets all the items of a given type.
         * @throws Not supported
         */
        getAll() {
            throw new Error("Not supported.");
        }
        /**
         * Deletes an single item from the sotrage
         * @param id - Unique identifier for the object
         * @throws Not supported
         */
        delete(id) {
            throw new Error("Not supported.");
        }
        /**
         * Saves an item to the storage.
         * If id is null, this is add/insert. If id exists, this is update.
         * @param itemToSave - The item that is updated/inserted.
         * @throws Not supported
         */
        save(itemToSave) {
            throw new Error("Not supported.");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    PrimaryPartners.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD functionality as well as business logic for storing and using Finnish Taxcards and calculating Withholding tax.
     */
    class Taxcards extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/taxcards";
        }
        /**
         * Sets the auto-taxcard functionality on for the given personal ID.
         * Auto-taxcard fetches the widtholding data automatically from the tax authorities API every time the salary is paid.
         * NOTE: In current implementation, the income type is always Salary (00024: Palkkaa).
         * @param personalId The Personal ID for the person to whom to set the taxcard as auto.
         * @param firstSalaryDate First salary date if known: If this is close enought to the actual salary date we do not need to refetch the taxcard from the authorities.
         * @returns The saved new automatic taxcard.
         * If null, the tax authorities do not have a record for this personal id: It must be handled manually (cannot be set as shared).
         */
        setAuto(personalId, firstSalaryDate = null) {
            firstSalaryDate = Dates.asDate(firstSalaryDate);
            return this.ajax.postJSON(`${this.baseUrl}/auto-set?personalId=${personalId}&salaryDate=${firstSalaryDate}`, null);
        }
        /**
         * Checks automatic taxcards from the tax authorities. If the taxcard has been checked within 2 days, does nothing.
         * If there are no changes, updates the LastChecked date only. If there are changes, creates a new taxcard container and returns it instead.
         * @param taxcardIds List of identifiers of taxcards that should be checked.
         * @param salaryDate Salary date for which the taxcard is checked.
         * @param forceUpdate If true, skips the 2 day rule and always checks from the tax athorities API.
         * @returns The refreshed taxcards.
         */
        refreshAuto(taxcardIds, salaryDate = null, forceUpdate = false) {
            salaryDate = Dates.asDate(salaryDate);
            return this.ajax.postJSON(`${this.baseUrl}/auto-refresh?&taxcardIds=${taxcardIds || ""}&salaryDate=${salaryDate || ""}&forceUpdate=${forceUpdate}`, null);
        }
        /**
         * Client-side (synchronous) method for getting a new blank item as bases for UI binding.
         * You need to set the Personal ID to bind the taxcard to an account.
         */
        getBlank() {
            return TaxCard2019Logic.getBlank(null);
        }
        /**
         * Saves a taxcard to the storage.
         * Removes empty incomeLog entries because they may be added by data binding in UI.
         * Entries are currently considered empty if both id and income are falsy: Typically income should be there,
         * but taxable income may be zero on a calculation (entry with id) and it makes sense to keep the entry still.
         * @param itemToSave - The item that is updated/inserted.
         * @returns A Promise with result data as saved to the storage (contains id, createAt, owner etc.).
         */
        save(itemToSave) {
            itemToSave.incomeLog = itemToSave.incomeLog.filter((x) => x.id || x.income);
            return super.save(itemToSave);
        }
        /**
         * Commits the current DIFF to the saved item.
         * Diff is the difference of income log saved in the taxcard to real paid calculations in the storage..
         * It is returned in each Save / GetSingle.
         * Modifications to non-modified items are also saved, but modifications to diff items have no effect: Diff is fetched from server.
         * Removes empty incomeLog entries because they may be added by data binding in UI (see Save for details).
         * @param itemToSave - The item that is updated/inserted with Diff saved.
         * @returns A Promise with result data as saved to the storage (contains id, createAt, owner etc.).
         */
        commitDiff(itemToSave) {
            itemToSave.incomeLog = itemToSave.incomeLog.filter((x) => x.id || x.income);
            return this.ajax.postJSON(`${this.baseUrl}?calcDiffCheck=${exports.TaxcardCalcDiffCheck.CommitDiff}`, itemToSave);
        }
        /**
         * Sets the approval for a shared taxcard.
         * @param uri - Shared taxcard is owned by Worker, so you need to pass a URI which has the owner as part of the URI.
         * @param method Approval / Reject method.
         * @returns Employer owned copy of the taxcard after approval / reject.
         */
        approveShared(uri, method) {
            return this.ajax.postJSON(`${this.baseUrl}/approve/${method}?sharedUri=${encodeURIComponent("" + uri)}`, {});
        }
        /**
         * Sets the approval for the Owned copy of the taxcard.
         * @param id Id of the local copy owned by Employer / current account (Statuses SharedWaiting, SharedRejected or SharedRejectedWithoutOpen).
         * @param method Approval / Reject method.
         * @returns Employer owned copy with the new approval status and other changes.
         */
        approveOwned(id, method) {
            return this.ajax.postJSON(`${this.baseUrl}/approve/${method}?ownedId=${encodeURIComponent("" + id)}`, {});
        }
        /**
         * Calculates the widhholding tax based on a given tax card.
         *
         * @param id - Identifier of the tax card.
         * @param income - New salary or other income, for which the widthholding tax is calculated.
         *
         * @returns A Promise with result data (TaxCardResult).
         */
        calculateWithId(id, income) {
            const method = `${this.baseUrl}/calculate/${encodeURIComponent("" + id)}?income=${Numeric.parseNumber(income)}`;
            return this.ajax.getJSON(method);
        }
        /**
         * Calculates the widhholding tax based on a given tax card
         *
         * @param income - New salary or other income, for which the widthholding tax is calculated.
         * @param taxCard - Tax card based on which the widthholding tax is calculated. This method will not connect to database in any way so the taxcard may or may not be stored.
         *
         * @returns A Promise with result data (TaxCardResult).
         */
        calculate(income, taxCard) {
            const method = `${this.baseUrl}/calculate?income=${Numeric.parseNumber(income)}`;
            return this.ajax.postJSON(method, taxCard);
        }
        /**
         * Gets all the current taxcards for the current Worker account:
         * The cards that the account has uploaded as well as the cards created by all Employers.
         * This method can only be run for a personal account (not for a Company account).
         * @returns The cards that the account has uploaded as well as the cards created by all Employers.
         * Only current cards (not expired) are provided.
         */
        getMyTaxcards() {
            const method = `${this.baseUrl}/my`;
            return this.ajax.getJSON(method);
        }
        /**
         * Gets all the current taxcards for an employment relation:
         * All the cards by the Employer, but also the latest card uploaded by the Worker herself.
         * @param employmentId The employment id for which to search the taxcards.
         * @param paymentDate Payment date for which the validity (current card in use) is calculated. If null, Next workday is used.
         * This may be changed to next default payment date without that being a breaking change.
         * @returns The cards created by the Employer and also the latest card uploaded by the Worker herself.
         * Only current cards (not expired) are provided.
         */
        getEmploymentTaxcards(employmentId, paymentDate) {
            let method = `${this.baseUrl}/employment/${employmentId}`;
            const date = Dates.asDate(paymentDate);
            if (date) {
                method += "?paymentDate=" + date;
            }
            return this.ajax.getJSON(method);
        }
        /**
         * For an employment relationship that is not yet created, allows checking if there is a shared taxcard.
         * Returns true if there is a shared taxcard available for the given Personal ID.
         * NOTE: There are unspecified and potentially changing restrictions on how many times this interface can be called to remove the risk of Phishing.
         * If such restriciton hits, the IsError property will be true. In this case, you should currently create the employment relation and fetch the taxcard info after that.
         * We may later introduce a "Are you robot" type of interface to go around this (if necessary).
         * @param personalId Personal ID of the Worker that will be added.
         * @returns Anonymous information of the shared taxcard if found. Check exists-property.
         */
        getSharedTaxcardExists(personalId) {
            const method = `${this.baseUrl}/shared-exists/${personalId}`;
            return this.ajax.getJSON(method);
        }
        /**
         * Returns the url where to post the tax card file using form post ('multipart/form-data').
         */
        getUploadUrl() {
            return `${this.ajax.getServerAddress()}${this.baseUrl}/file`;
        }
        /**
         * Returns the url where to get the tax card file with authentication token.
         */
        getDownloadUrl(taxCard) {
            if (taxCard && taxCard.card && taxCard.card.fileUri) {
                return `${this.ajax.getServerAddress()}${this.baseUrl}/file/${taxCard.id}?access_token=${this.ajax.getCurrentToken()}`;
            }
            return null;
        }
        /**
         * Returns the url where to get the tax card preview image with authentication token.
         * @param taxcard Either an API object or list item for which the preview is shown.
         * For Workers, supports Employer owned taxcards by passing owner ID in addition to just ID.
         */
        getPreviewUrl(taxcard) {
            if ((taxcard.card || {}).previewUri || (taxcard.data || {}).previewUri) {
                return `${this.ajax.getServerAddress()}${this.baseUrl}/preview/${taxcard.id}@${taxcard.owner}?access_token=${this.ajax.getCurrentToken()}`;
            }
            return null;
        }
        /**
         * Gets a list of all non-terminated employments of the employer and resolves their current taxcards.
         * The current taxcards may be for the current year or for next year if the current date is either December or January.
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         * @param paymentDate Payment date for which the validity (current card in use) is calculated. If null, Next workday is used.
         * This may be changed to next default payment date without that being a breaking change.
         */
        getCurrentCards(query, paymentDate = null) {
            const date = Dates.asDate(paymentDate);
            const url = "/v03-rc/api/taxcards/current" + (date ? ("/" + date) : "");
            return OData.getOData(url, query, this.ajax);
        }
        /** Available object mappers for import/export */
        getObjectMappers() {
            const map = {
                id: "id",
                personalId: "card.personalId",
                forYear: "card.forYear",
                start: "card.validity.start",
                end: "card.validity.end",
                taxPercent: "card.taxPercent",
                incomeLimit: "card.incomeLimit",
                taxPercent2: "card.taxPercent2",
                kind: "card.kind",
            };
            const defaultMapper = new Mapper(map, new Taxcards(null).getBlank());
            defaultMapper.rowType = "TaxcardsDataRow";
            defaultMapper.description = "Vakioaineisto";
            return [defaultMapper];
        }
        /** Available list item mappers for import/export */
        getListItemMappers() {
            const paths = [
                "otherPartyInfo.avatar.displayName",
                "otherPartyInfo.officialId",
                "data.forYear",
                "data.start",
                "data.end",
                "data.taxPercent",
                "data.incomeLimit",
                "data.taxPercent2",
                "data.kind"
            ];
            const defaultMapper = Mappers.pathMapper(paths);
            defaultMapper.rowType = "ApiListItem";
            defaultMapper.description = "Esikatselu";
            defaultMapper.isListItemMapper = true;
            return [defaultMapper];
        }
        /**
         * Validates the item using a server side call.
         *
         * @param itemToValidate - The item that is validated.
         * @returns A Promise with result data containing validation property.
         */
        validate(itemToValidate) {
            return Promise.resolve(itemToValidate);
        }
        /**
         * Returns the validation object for the validated object.
         * @param validatedItem The validated object with a validation result.
         */
        getValidationResult(validatedItem) {
            return { isValid: true };
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Taxcards.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access to Workers (employees) created by the employer
     * or to whom the employer has paid salaries.
     */
    class Workers extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/accounts/workers";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return WorkerLogic.getBlank();
        }
        /** Saves the Worker account ot the storage */
        save(itemToSave) {
            return super.save(itemToSave);
        }
        /** Available object mappers for import/export */
        getObjectMappers() {
            const map = {
                id: "id",
                firstName: "avatar.firstName",
                lastName: "avatar.lastName",
                email: "contact.email",
                telephone: "contact.telephone",
                ibanNumber: "ibanNumber",
                officialId: "officialPersonId",
            };
            const defaultMapper = new Mapper(map, new Workers(null).getBlank());
            defaultMapper.rowType = "WorkersDataRow";
            defaultMapper.description = "Vakioaineisto";
            return [defaultMapper];
        }
        /** Available list item mappers for import/export */
        getListItemMappers() {
            const paths = [
                "otherPartyInfo.avatar.firstName",
                "otherPartyInfo.avatar.lastName",
                "otherPartyInfo.email",
                "otherPartyInfo.telephone",
                "otherPartyInfo.ibanNumber",
                "otherPartyInfo.officialId"
            ];
            const defaultMapper = Mappers.pathMapper(paths);
            defaultMapper.rowType = "ApiListItem";
            defaultMapper.description = "Esikatselu";
            defaultMapper.isListItemMapper = true;
            return [defaultMapper];
        }
        /**
         * Validates the item using a server side call.
         *
         * @param itemToValidate - The item that is validated.
         * @returns A Promise with result data containing validation property.
         */
        validate(itemToValidate) {
            return Promise.resolve(itemToValidate);
        }
        /**
         * Returns the validation object for the validated object.
         * @param validatedItem The validated object with a validation result.
         */
        getValidationResult(validatedItem) {
            return validatedItem.validation || { isValid: true };
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Workers.$inject = ["AjaxNg1"];

    /**
     * Methods for updating account information
     */
    class Accounts {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Gets the current company account if the current account is a company.
         * NOTE: Typically the account data is fetched in Session check => You do not need to fetch it separately.
         *
         * @returns A Promise with result data (Company account or null).
         */
        getCompany() {
            return this.ajax.getJSON("/accounts/company");
        }
        /**
         * Gets the current Person account:
         * Either the current account or the Primary contact person account if the current account is a company.
         * NOTE: Typically the current account data is fetched in Session check => You do not need to fetch it separately.
         * => Only use this to fetch the Primary contact person account if the current account is a company.
         *
         * @returns A Promise with result data (Personal account).
         */
        getPerson() {
            return this.ajax.getJSON("/accounts/person");
        }
        /**
         * Saves the avatar and contact for the current Person account:
         * Either the current account or the Primary contact person account if the current account is a company.
         *
         * @returns A Promise with result data (Personal account).
         */
        savePerson(personAccount) {
            return this.ajax.postJSON("/accounts/person", personAccount);
        }
        /**
         * Returns the url where to post the avatar file
         */
        getAvatarUploadUrl() {
            return `${this.ajax.getApiAddress()}/accounts/avatar`;
        }
        /**
         * Switches the current web site usage role.
         *
         * @param role - household or worker. You cannot change to company at the moment using this method.
         *
         * @returns A Promise with result data (new role).
         */
        switchRole(role) {
            return this.ajax.postJSON("/accounts/switchRole?role=" + role, {});
        }
        /**
         * Sends a verification.
         *
         * @param type - Verification type.
         * @param address - Verification address.
         *
         * @returns A Promise with result data (pin code in test environments).
         */
        sendVerification(type, address) {
            return this.ajax.postJSON("/accounts/verification/" + type, JSON.stringify(address));
        }
        /**
         * Confirms a verification.
         *
         * @param type - Verification type.
         * @param pin - Verification pin code.
         *
         * @returns A Promise with result data (true/false).
         */
        confirmVerification(type, pin) {
            return this.ajax.postJSON("/accounts/verification/" + type + "/confirm", JSON.stringify(pin));
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Accounts.$inject = ["AjaxNg1"];

    /**
     * Provides functionality for Onboarding:
     * The process where the user becomes a user of Salaxy service and an account is created for him / her / the company.
     */
    class Onboardings {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Gets a single Onboarding object based on identier
         *
         * @param id - Unique identifier for the object
         *
         * @returns A Promise with result data (Onboarding).
         */
        getOnboarding(id) {
            return this.ajax.getJSON("/onboarding/" + encodeURIComponent("" + id));
        }
        /**
         * Gets a single Onboarding object for authorizing account
         *
         * @param accountId - Account id of the authorizing account.
         *
         * @returns A Promise with result data (Onboarding).
         */
        getOnboardingForAccount(accountId) {
            return this.ajax.getJSON("/onboarding/account/" + encodeURIComponent("" + accountId));
        }
        /**
         * Creates or updates an Onboarding.
         *
         * @param onboarding - The Onboarding object that is updated.
         *
         * @returns A Promise with result data (saved Onboarding).
         */
        saveOnboarding(onboarding) {
            return this.ajax.postJSON("/onboarding", onboarding);
        }
        /**
         * Deletes an onboarding object from the storage
         *
         * @param id - Unique identifier for the object
         *
         * @returns A Promise with result data "Object deleted".
         */
        deleteOnboarding(id) {
            return this.ajax.remove("/onboarding/" + encodeURIComponent("" + id));
        }
        /**
         * Sends a new pin code to the telephone number to be verified in onboarding.
         *
         * @param onboarding - The Onboarding object which contains the telephone number.
         *
         * @returns A Promise with result data (true if sending succeeded).
         */
        sendSmsVerificationPin(onboarding) {
            return this.ajax.postJSON("/onboarding/verification/sms", onboarding);
        }
        /**
         * Commits onboarding and creates/updates the account.
         *
         * @param onboarding - The Onboarding object that is to be committed..
         *
         * @returns A Promise with result data (committed Onboarding).
         */
        commitOnboarding(onboarding) {
            return this.ajax.postJSON("/onboarding/commit", onboarding);
        }
        /**
         * Returns the url to start the onboarding for current account (server side).
         * @param id - Optional id for the existing onboarding object.
         */
        getLaunchUrl(id = null) {
            return `${this.ajax.getServerAddress()}/onboarding/launch${id ? "/" + id : ""}?access_token=${encodeURIComponent("" + this.ajax.getCurrentToken())}`;
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Onboardings.$inject = ["AjaxNg1"];

    /**
     * Provides access to HTML reports on calculations
     */
    class Reports {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Gets a list of generated HTML/PDF/JSON reports by type.
         *
         * @param reportType - Type of reports to fetch.
         *
         * @returns A Promise with result data (array of reports).
         */
        getReportsByType(reportType) {
            return this.ajax.getJSON("/reports/type/" + reportType);
        }
        /**
         * Gets a single generated HTML/PDF/JSON report by type and id.
         *
         * @param reportType - Type of the report to fetch.
         * @param id - Id of the report to fetch.
         * @param wait - Optional flag to indicate whether to wait for the report to be generated.
         *
         * @returns A Promise with result data (report).
         */
        getReport(reportType, id, wait) {
            const method = `/reports/type/${reportType}/encodeURIComponent(id)?wait=${wait}`;
            return this.ajax.getJSON(method);
        }
        /**
         * Gets the monthly / quarterly/ yearly accounting data for the current account.
         *
         * @param refDate Reference date for the period. Please note that even if the date is not the first day of the given period, the entire period is returned.
         * @param target Accounting target name : contains  default ruleset and other parameters for creating accounting entries.
         * @param periodType Month, quarter, year or a custom period. The custom period requires endDate. Default value is the month.
         * @param endDate End date for the period. Required only for the custom period.
         * @param periodDateKind Period date type: paid at date or salary date.
         * @param minStatus Minimum status level for the calculations to include in the data. The default level is PaymentSucceeded.
         * @param ruleSet Optional additional ruleset for the target to use for creating the accounting entries.
         * @returns A Promise with result data (Raw data for accounting purposes).
         */
        getAccountingDataForPeriod(refDate, target, periodType, endDate, periodDateKind, minStatus, ruleSet) {
            refDate = Dates.asDate(refDate) || Dates.getToday();
            const prmsArr = [];
            if (target) {
                prmsArr.push(`target=${encodeURIComponent(target)}`);
            }
            if (ruleSet) {
                prmsArr.push(`ruleSet=${encodeURIComponent(ruleSet)}`);
            }
            if (periodType) {
                prmsArr.push(`periodType=${periodType}`);
            }
            if (endDate) {
                prmsArr.push(`endDate=${endDate}`);
            }
            if (minStatus) {
                prmsArr.push(`minStatus=${minStatus}`);
            }
            if (periodDateKind) {
                prmsArr.push(`periodDateKind=${periodDateKind}`);
            }
            const prms = prmsArr.join("&");
            return this.ajax.getJSON(`/v03-rc/api/reports/accounting/${refDate}${prms ? "?" + prms : ""}`);
        }
        /**
         * Gets the accounting data based on given set of calculations.
         *
         * @param calculationIds - Calculations that are the bases for the report.
         * @param target  - Accounting target name : contains  default ruleset and other parameters for creating accounting entries.
         * @param ruleSet - Optional additional ruleset for the target to use for creating the accounting entries.
         * @returns A Promise with result data (Raw data for accounting purposes).
         */
        getAccountingDataForCalculationIds(calculationIds, target, ruleSet) {
            const ids = calculationIds.map((x) => `calculationIds=${x}`).join("&");
            const prmsArr = [];
            if (target) {
                prmsArr.push(`target=${encodeURIComponent(target)}`);
            }
            if (ruleSet) {
                prmsArr.push(`ruleSet=${encodeURIComponent(ruleSet)}`);
            }
            const prms = prmsArr.join("&");
            return this.ajax.getJSON(`/v03-rc/api/reports/accounting?${ids}${prms ? "&" + prms : ""}`);
        }
        /**
         * Gets the accounting data based on given report id.
         *
         * @param id - Saved report id containing accounting data.
         * @param target  - Accounting target name : contains  default ruleset and other parameters for creating accounting entries.
         * @param ruleSet - Optional additional ruleset for the target to use for creating the accounting entries.
         * @returns A Promise with result data (Raw data for accounting purposes).
         */
        getAccountingData(id, target, ruleSet) {
            const prmsArr = [];
            if (target) {
                prmsArr.push(`target=${encodeURIComponent(target)}`);
            }
            if (ruleSet) {
                prmsArr.push(`ruleSet=${encodeURIComponent(ruleSet)}`);
            }
            const prms = prmsArr.join("&");
            return this.ajax.getJSON(`/v03-rc/api/reports/accounting/${id}${prms ? "?" + prms : ""}`);
        }
        /**
         * Gets the accounting report based on given set of calculations.
         * This method can be used anonymously.
         *
         * @param calculations - Calculations that are the bases for the report.
         * @param target  - Accounting target name : contains  default ruleset and other parameters for creating accounting entries.
         * @param ruleSet - Optional additional ruleset for the target to use for creating the accounting entries.
         * @returns A Promise with result data (Raw data for accounting purposes).
         */
        getAccountingDataForCalculations(calculations, target, ruleSet) {
            const prmsArr = [];
            if (target) {
                prmsArr.push(`target=${encodeURIComponent(target)}`);
            }
            if (ruleSet) {
                prmsArr.push(`ruleSet=${encodeURIComponent(ruleSet)}`);
            }
            const prms = prmsArr.join("&");
            return this.ajax.postJSON(`/v03-rc/api/reports/accounting${prms ? "?" + prms : ""}`, calculations);
        }
        /**
         * Closes the accounting period for the given period, and generates a report for the period.
         * @param refDate - Period reference date.
         * @param option - Closing option. Currently only default -option supported.
         * @returns Report id for the period.
         */
        closeAccountingPeriod(refDate, option) {
            return this.ajax.postJSON(`/v03-rc/api/reports/accounting/${refDate}/close${option ? "?option=" + option : ""}`, null);
        }
        /**
         * Gets an HTML report based on Calculation ID.
         *
         * @param reportType - Type of the report to fetch. See the HtmlReportType enumeration for possible values.
         * @param calculationId - GUID for the calculation
         *
         * @returns A Promise with result HTML.
         */
        getReportHtmlById(reportType, calculationId) {
            let method;
            if (!reportType) {
                return;
            }
            switch (reportType.toLowerCase()) {
                case "employer":
                    method = "/reportpartials/employerCalculation/" + calculationId;
                    break;
                case "employerpage2":
                    method = "/reportpartials/employerCalculationAdditionalInfo/" + calculationId;
                    break;
                case "worker":
                    method = "/reportpartials/workerCalculation/" + calculationId;
                    break;
                case "workerpage2":
                    method = "/reportpartials/workerCalculationAdditionalInfo/" + calculationId;
                    break;
                case "payments":
                    method = "/reportpartials/paymentCalculation/" + calculationId;
                    break;
                default:
                    throw new Error("Unknown report partial type: " + reportType);
            }
            return this.ajax.getHTML(this.ajax.getServerAddress() + method);
        }
        /**
         * Gets the monthly / quarterly/ yearly accounting data for the current account.
         *
         * @param refDate - Reference date for the period. Please note that even if the date is not the first day of the given period, the entire period is returned.
         * @param tableType  - Accounting table type.
         * @param periodType - Month, quarter, year or a custom period. The custom period requires endDate. Default value is the month.
         * @param endDate - End date for the period. Required only for the custom period.
         * @returns A Promise with result data (Raw data for accounting purposes).
         */
        getAccountingReportTableForPeriod(refDate, tableType = exports.AccountingReportTableType.Classic, periodType = exports.PeriodType.Month, endDate = null) {
            refDate = Dates.asDate(refDate) || Dates.getToday();
            let method = "/reports/data/accounting/" + tableType + "/" + encodeURIComponent(refDate) + "?";
            method += "periodType=" + encodeURIComponent("" + periodType);
            if (endDate) {
                method += "&endDate=" + encodeURIComponent(endDate);
            }
            return this.ajax.getJSON(method);
        }
        /**
         * Experimental: Gets the accounting report based on given set of calculations.
         *
         * @param calculationIds - Calculations that are the bases for the report.
         * @param tableType  - Accounting table type.
         * @returns Account report based on the calculations.
         * @ignore
         */
        getAccountingReportTableForCalculationIds(calculationIds, tableType = exports.AccountingReportTableType.Classic) {
            let method = "/reports/data/accounting/" + tableType + "?";
            calculationIds.forEach((x) => { method += "calculationIds=" + encodeURIComponent("" + x) + "&"; });
            method = method.replace(/[?&]$/, "");
            return this.ajax.getJSON(method);
        }
        /**
         * Experimental: Gets the accounting report based on given set of calculations.
         *
         * @param calculations - Calculations that are the bases for the report.
         * @param tableType  - Accounting table type.
         * @returns Account report based on the calculations.
         * @ignore
         */
        getAccountingReportTableForCalculations(calculations, tableType = exports.AccountingReportTableType.Classic) {
            const method = "/reports/data/accounting/" + tableType;
            return this.ajax.postJSON(method, calculations);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Reports.$inject = ["AjaxNg1"];

    /**
     * Provides read-only access to the current user session
     */
    class Session {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Checks whether the current browser session is authorized.
         *
         * @returns A Promise with result data (boolean).
         */
        isAuthorized() {
            return this.ajax.getJSON("/session/isAuthorized");
        }
        /**
         * Gets the current session information or null if there is no active session.
         * UserSession data is the Authorization object that provides information about the current session. Null, if there is no session.
         *
         * @returns A Promise with result data (UserSession).
         */
        getSession() {
            return this.ajax.getJSON("/session/current");
        }
        /**
         * Returns server address
         */
        getServerAddress() {
            return this.ajax.getServerAddress();
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Session.$inject = ["AjaxNg1"];

    /**
     * Methods for testing the API.
     */
    class Test {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Basic hello world message. Echoes the given message back.
         *
         * @param message - Message.
         *
         * @returns A Promise with result data (message).
         */
        hello(message) {
            return this.ajax.getJSON("/test/hello?message=" + encodeURIComponent("" + message));
        }
        /**
         * Test a long lasting request.
         *
         * @param seconds - Delay in seconds.
         *
         * @returns A Promise with result data (Delay in seconds).
         */
        delay(seconds) {
            return this.ajax.getJSON(`/test/delay?seconds=${seconds}`);
        }
        /**
         * Test JSON serialization. Echoes a given object either in success callback or in the promise.
         *
         * @param testValues - TestValues object.
         *
         * @returns A Promise with result.
         */
        values(testValues) {
            return this.ajax.postJSON("/test/values", testValues);
        }
        /**
         * Creates a test account and returns test account details.
         *
         * @param onboarding - Onboarding object.
         *
         * @returns A Promise with result data (test accounts).
         */
        createTestAccount(onboarding) {
            return this.ajax.postJSON("/test/accounts", onboarding);
        }
        /**
         * Deletes a test account and related objects
         *
         * @param accountId - Account id to delete.
         *
         * @returns A Promise with result data "Object deleted".
         */
        deleteTestAccount(accountId) {
            return this.ajax.remove("/test/accounts/" + encodeURIComponent("" + accountId));
        }
        /**
         * Test methods that throws an exception on server side.
         *
         * @param errorType Type of error
         *
         * @returns A Promise if success parameter is null. For throwException,
         * the promise is always rejected after the error is thrown in the server.
         */
        throwException(errorType) {
            return this.ajax.getJSON("/test/exceptions/" + errorType);
        }
        /**
         * Delete the current account data and credentials (including Auth0 user).
         * Can be called only in a test environment.
         */
        deleteCurrentAccount() {
            return this.ajax.remove("/accounts/current");
        }
        /**
         * Remove all calculations, workers etc. user objects except products and signature from the account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountData() {
            return this.ajax.remove("/accounts/data/all");
        }
        /**
         * Remove all calculations, payrolls and payments from the account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountCalculations() {
            return this.ajax.remove("/accounts/calculation/all");
        }
        /**
         * Remove workers including calculations, employment contracts and tax cards from the account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountWorkers() {
            return this.ajax.remove("/accounts/worker/all");
        }
        /**
         * Remove all holiday year from all workers. Does not touch the default values of holidays in Worker Employment relation.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountHolidays() {
            return this.ajax.remove("/accounts/holidays/all");
        }
        /**
         * Remove pension and insurance from the account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountPensionAndInsurance() {
            return this.ajax.remove("/accounts/product/pensionAndInsurance");
        }
        /**
         * Remove the signature from the account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountSignature() {
            return this.ajax.remove("/accounts/signature");
        }
        /**
         * Delete all empty accounts (company or worker) created by this account.
         * Can be called only in a test environment.
         */
        deleteCurrentAccountAuthorizingAccounts() {
            return this.ajax.remove("/accounts/authorizingAccount/all");
        }
        /**
         * Sets the data for taxcards API for a personal Id for system-wide (all users).
         * @param testData The data that is stored as test data.
         * If the taxcard kind is Undefined, the data is cleared for that taxcard.
         * If the kind is NoTaxcard, the API wil return null.
         * Other kinds will be set as Auto.
         * @returns The data as it would be returnded now from the API
         */
        setTaxcardApi(testData) {
            return this.ajax.postJSON("/v03-rc/api/test-data/taxcard-api", testData);
        }
        /**
         * Gets taxcard information for a personal Id.
         * @param personalId Personal ID for which the data is fetched.
         * @returns Taxcard input or null if none is stored in the test data file.
         * Note if this APi returns null, system defaults to a fixed test taxcard.
         * If you want the system to return null, set kind to NoTaxcard.
         */
        getTaxcardApi(personalId) {
            return this.ajax.getJSON("/v03-rc/api/test-data/taxcard-api/" + (encodeURIComponent(personalId)));
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Test.$inject = ["AjaxNg1"];

    /**
     * Client to Salaxy API.
     * Abstracts the raw REST api to async methods that are easier to call from TypeScript / JavaScript.
     * Differnet platforms (NG1, NG, jQuery etc.) create derived Clients typically at least by injecting their own Ajax implementation,
     * potentially also by overriding other methods.
     */
    class Client {
        constructor(ajaxImpl) {
            this._ajax = ajaxImpl;
        }
        /** The platform (NG1, NG, jQuery etc.) dependent class that performs the actual Ajax calls */
        get ajax() {
            return this._ajax;
        }
        /** Gets the Accounts API: Access to current account, its products and workers */
        get accountsApi() {
            if (!this._accountsApi) {
                this._accountsApi = new Accounts(this._ajax);
            }
            return this._accountsApi;
        }
        /** Gets the Calculations API: CRUD operations on calculations */
        get calculationsApi() {
            if (!this._calculationsApi) {
                this._calculationsApi = new Calculations(this._ajax);
            }
            return this._calculationsApi;
        }
        /** Gets the Calculator API: Anonymous salary calculation methods */
        get calculatorApi() {
            if (!this._calculatorApi) {
                this._calculatorApi = new Calculator(this._ajax);
            }
            return this._calculatorApi;
        }
        /** Gets the Reports API: Viewing and generating reports in different formats */
        get reportsApi() {
            if (!this._reportsApi) {
                this._reportsApi = new Reports(this._ajax);
            }
            return this._reportsApi;
        }
        /** Gets the Files API: Files stored by the user, thumbnails of pictures etc. */
        get filesApi() {
            if (!this._filesApi) {
                this._filesApi = new Files(this._ajax);
            }
            return this._filesApi;
        }
        /** Gets the Session API: Access to current session, user and roles */
        get sessionApi() {
            if (!this._sessionApi) {
                this._sessionApi = new Session(this._ajax);
            }
            return this._sessionApi;
        }
        /** Gets the Payments API for paying salaries using different methods */
        get paymentsApi() {
            if (!this._paymentsApi) {
                this._paymentsApi = new Payments(this._ajax);
            }
            return this._paymentsApi;
        }
        /** Gets the OAuth2 API: Authentication methods */
        get oAuth2Api() {
            if (!this._oAuth2Api) {
                this._oAuth2Api = new OAuth2(this._ajax);
            }
            return this._oAuth2Api;
        }
        /** Gets the Test API: Testing different functionalities of the API and system in general. */
        get testApi() {
            if (!this._testApi) {
                this._testApi = new Test(this._ajax);
            }
            return this._testApi;
        }
        /** Gets the Tax cards API */
        get taxcardsApi() {
            if (!this._taxcardsApi) {
                this._taxcardsApi = new Taxcards(this._ajax);
            }
            return this._taxcardsApi;
        }
        /** Gets the Workers API */
        get workersApi() {
            if (!this._workersApi) {
                this._workersApi = new Workers(this._ajax);
            }
            return this._workersApi;
        }
        /** Gets the Onboarding API for creating a new user */
        get onboardingsApi() {
            if (!this._onboardingsApi) {
                this._onboardingsApi = new Onboardings(this._ajax);
            }
            return this._onboardingsApi;
        }
    }

    /**
     * Methods for getting and saving yearly feedback data.
     */
    class YearEnd extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/yearEnd/feedback";
        }
        /**
         * Client-side (synchronous) method for getting a new blank CRUD item as bases for UI binding.
         * The basic idea is that all the child object properties should be non-null to avoid null reference exceptions.
         * In special cases, when null is significant it may be used: e.g. Calculation.result is null untill the recalculation has been done.
         * Strings, dates and booleans may be null (false if bool), but typically just undefined.
         */
        getBlank() {
            throw new Error("Not implemented");
        }
        /**
         * Gets a single item based on identier
         * @param id - Unique identifier for the object
         * @returns A Promise with result data.
         */
        getSingle(id) {
            return super.getAll().then((all) => {
                if (all && all.length !== 0) {
                    return all[0];
                }
                return null;
            });
        }
        /**
         * Deletes an single item from the sotrage
         * @param id - Unique identifier for the object
         * @throws Not implemented
         */
        delete(id) {
            throw new Error("Not implemented");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    YearEnd.$inject = ["AjaxNg1"];

    /**
     * Provides read / update functionality for main account settings (employer/company)
     */
    class AccountSettings extends SingleItemApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/settings/company";
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    AccountSettings.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for beneficiary owners
     */
    class BeneficialOwnersApi extends SingleItemApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/beneficial-owners";
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    BeneficialOwnersApi.$inject = ["AjaxNg1"];

    /**
     * Provides CRUD access for Calendar occurences.
     * NOTE: Occurences are mainly created and updated through Calendar Events,
     * but limited modifications and delete (exception to recurrence rules) is allowed.
     */
    class CalendarOccurences extends CrudApiBase {
        constructor(ajax) {
            super(ajax);
            /** Base URL for details etc. */
            this.baseUrl = "/v03-rc/api/calendar/occurences";
        }
        /** Client-side (synchronous) method for getting a new blank item as bases for UI binding. */
        getBlank() {
            return {
                categories: [],
            };
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    CalendarOccurences.$inject = ["AjaxNg1"];

    /**
     * Methods for testing the API.
     */
    class Overview {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Makes an OData query over all objects within the current account.
         * @param query The options for the query: Filter, search sort etc.
         * This is either a strongly typed object or query string that is added directly.
         */
        getAllData(query) {
            return OData.getOData("/v03-rc/api/overview/all", query, this.ajax);
        }
        /**
         * Gets the welcome page data for a Worker account.
         */
        getWelcomeDataWorker() {
            return this.ajax.getJSON("/v03-rc/api/overview/welcome-worker");
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    Overview.$inject = ["AjaxNg1"];

    /**
     * API for partner services, like pension, insurance and health care.
     */
    class PartnerServices {
        constructor(ajax) {
            this.ajax = ajax;
        }
        /**
         * Returns a new Varma pension order with default values for the current account.
         */
        getNewVarmaPensionOrder() {
            return this.ajax.getJSON("/partner-services/varma/pension/new");
        }
        /**
         * Validate the given Varma pension order in the server.
         */
        validateVarmaPensionOrder(order) {
            return this.ajax.postJSON("/partner-services/varma/pension/validate", order);
        }
        /**
         * Send a new Varma pension order for later processing.
         */
        sendVarmaPensionOrder(order) {
            return this.ajax.postJSON("/partner-services/varma/pension/order", order);
        }
        /**
         * Returns a new Local Tapiola Insurance order with default values for the current account.
         */
        getNewLocalTapiolaInsuranceOrder() {
            return this.ajax.getJSON("/partner-services/local-tapiola/insurance/new");
        }
        /**
         * Validate the given Local Tapiola Insurance order in the server.
         */
        validateLocalTapiolaInsuranceOrder(order) {
            return this.ajax.postJSON("/partner-services/local-tapiola/insurance/validate", order);
        }
        /**
         * Send a new Local Tapiola Insurance order for later processing.
         */
        sendLocalTapiolaInsuranceOrder(order) {
            return this.ajax.postJSON("/partner-services/local-tapiola/insurance/order", order);
        }
    }
    /**
     * For NG1-dependency injection
     * @ignore
     */
    PartnerServices.$inject = ["AjaxNg1"];

    /**
     * The base usecase tree for household customers.
     * The labels and descriptions are in SALAXY.ENUM.HouseHoldUseCaseTree.
     */
    const householdUsecaseTree = [
        {
            id: "childCare",
            icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/childcare.svg",
            children: [
                {
                    id: "childCare/mll",
                    uri: "https://secure.salaxy.com/v02/api/usecases/mll",
                    isHouseholdDeductible: true,
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/childcare.svg",
                    badge: "https://cdn.salaxy.com/ng1/household-usecases/badge/mll.png",
                    occupation: "53112",
                },
                {
                    id: "childCare/kela",
                    uri: "https://secure.salaxy.com/v02/api/usecases/childCare",
                    isChildcareSubsidy: true,
                    isHouseholdDeductible: false,
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/childcare.svg",
                    badge: "https://cdn.salaxy.com/ng1/household-usecases/badge/kela.png",
                    occupation: "53112",
                },
                {
                    id: "childCare/other",
                    uri: "https://secure.salaxy.com/v02/api/usecases/childCare",
                    isHouseholdDeductible: true,
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/childcare.svg",
                    occupation: "53112",
                },
            ],
        },
        {
            id: "construction",
            icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/construction.svg",
            children: [
                {
                    id: "construction/carpenter",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/construction-carpenter.svg",
                    uri: "https://secure.salaxy.com/v02/api/usecases/construction",
                    occupation: "71150",
                },
                {
                    id: "construction/painter",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/construction-painter.svg",
                    uri: "https://secure.salaxy.com/v02/api/usecases/construction",
                    occupation: "71310",
                },
                {
                    id: "construction/renovation",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/construction-renovation.svg",
                    uri: "https://secure.salaxy.com/v02/api/usecases/construction",
                    isHouseholdDeductible: true,
                    occupation: "71110",
                },
                {
                    id: "construction/new",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/construction-new.svg",
                    uri: "https://secure.salaxy.com/v02/api/usecases/construction",
                    isHouseholdDeductible: false,
                    occupation: "71110",
                },
                {
                    id: "construction/other",
                    uri: "https://secure.salaxy.com/v02/api/usecases/other",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/other.svg",
                    occupation: "other",
                },
            ],
        },
        {
            id: "cleaning",
            uri: "https://secure.salaxy.com/v02/api/usecases/cleaning",
            icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/cleaning.svg",
            children: [
                {
                    id: "cleaning/91110",
                    isHouseholdDeductible: true,
                    occupation: "91110",
                    isContractLessThanYear: true
                },
                {
                    id: "cleaning/51530",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/cleaning-real-estate.svg",
                    isHouseholdDeductible: true,
                    occupation: "51530",
                    isContractLessThanYear: true
                },
                {
                    id: "cleaning/61132",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/cleaning-garden.svg",
                    isHouseholdDeductible: true,
                    occupation: "61132",
                    isContractLessThanYear: true
                },
                {
                    id: "cleaning/53221",
                    isHouseholdDeductible: true,
                    occupation: "53221",
                    isContractLessThanYear: true
                },
                {
                    id: "cleaning/other",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/cleaning-other.svg",
                    isHouseholdDeductible: true,
                    occupation: "other",
                    isContractLessThanYear: true
                },
            ],
        },
        {
            id: "other",
            uri: "https://secure.salaxy.com/v02/api/usecases/other",
            icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/other.svg",
            children: [
                {
                    id: "other/53222",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Carework,
                    occupation: "53222",
                },
                {
                    id: "other/61112",
                    occupation: "61112",
                },
                {
                    id: "other/91110",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Householdwork,
                    occupation: "91110",
                },
                {
                    id: "other/53213",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Carework,
                    occupation: "53213",
                },
                {
                    id: "other/53212",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Carework,
                    occupation: "53212",
                },
                {
                    id: "other/53112",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Carework,
                    occupation: "53112",
                },
                {
                    id: "other/53221",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Carework,
                    occupation: "53221",
                },
                {
                    id: "other/62100",
                    icon: "https://cdn.salaxy.com/ng1/household-usecases/icon/other.svg",
                    occupation: "62100",
                },
                {
                    id: "other/51530",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Householdwork,
                    occupation: "51530",
                },
                {
                    id: "other/61132",
                    isHouseholdDeductible: true,
                    taxDeductionCategories: exports.TaxDeductionWorkCategories.Householdwork,
                    occupation: "61132",
                },
                {
                    id: "other/undefined",
                    occupation: "other",
                },
            ],
        },
    ];

    /**
     * Provides business logic for Calculation usecases:
     * Salary framework contracts ("työehtosopimus") and other salary recommendations
     * that are used as bases for salary calculation.
     */
    class HouseholdUsecasesLogic {
        /**
         * Finds a parent group of a given usecase node
         * @param usecaseId - The usecase whose parent is fetched.
         * If you pass a group here, the function will return null which will typically mean the tree root in the user interface.
         *
         * @example
         * ```
         * // Creates a simple calculation
         * const usecase = UsecasesLogic.findUsecaseById("childCare/mll");
         * UsecasesLogic.setUsecase(this.context.calculations.current, usecase);
         * this.context.calculations.recalculateCurrent((calc) => {
         *     calc.worker.paymentData.telephone = "+358401234567";
         * });
         * ```
         */
        static findUsecaseById(usecaseId) {
            for (const group of HouseholdUsecasesLogic.getUsecaseTree()) {
                for (const usecase of group.children || []) {
                    if (usecase.id === usecaseId) {
                        return usecase;
                    }
                }
            }
            return null;
        }
        /**
         * Gets a tree of usecases (user interfaces for defining a framework) for a given role.
         */
        static getUsecaseTree() {
            const tree = householdUsecaseTree;
            for (const group of tree) {
                this.translate("HouseHoldUseCaseTree", group.id, group);
                for (const usecase of group.children) {
                    usecase.badge = usecase.badge || group.badge;
                    usecase.icon = usecase.icon || group.icon;
                    usecase.uri = usecase.uri || group.uri;
                    this.translate("HouseHoldUseCaseTree", usecase.id, usecase);
                }
            }
            return tree;
        }
        /**
         * Abstract the getting of the use case for the calculation.
         * We are expecting changes to the usecase implementation.
         * @param calc Calculation into which the usecase data is stored.
         */
        static getUsecaseData(calc) {
            var _a;
            if (!((_a = calc.usecase) === null || _a === void 0 ? void 0 : _a.uri)) {
                const usecase = HouseholdUsecasesLogic.findUsecaseById("other/undefined");
                calc.usecase = {
                    uri: usecase.uri,
                    label: usecase.label,
                    description: usecase.descr,
                    data: usecase,
                };
            }
            return calc.usecase.data;
        }
        /**
         * Sets calculation properties based on selected usecase.
         * @param calc Calculation to set
         * @param usecase Usecase to apply.
         * @example
         * ```
         * // Creates a simple calculation
         * const usecase = UsecasesLogic.findUsecaseById("childCare/mll");
         * UsecasesLogic.setUsecase(this.calc, usecase);
         * this.calcApi.recalculate(this.calc).then((data) => {
         *   angular.copy(data, this.calc);
         * });
         * ```
         */
        static setUsecase(calc, usecase) {
            calc.usecase = {
                uri: usecase.uri,
                label: usecase.label,
                description: usecase.descr,
                data: usecase,
            };
            this.applyUseCase(calc);
            return this.getUsecaseData(calc);
        }
        /**
         * Applies the usecase to a calculation, especially to the old Framework object.
         * Should be called before sending the calculation to the server (recalculate/save).
         * @param calc Calculation where usecase is set to calc.data.usecase.
         */
        static applyUseCase(calc) {
            const usecase = this.getUsecaseData(calc);
            switch (usecase.uri) {
                case HouseholdUsecasesLogic.knownUseCases.childCare:
                    this.applyUsecaseChildCare(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.mll:
                    this.applyUsecaseMll(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.cleaning:
                    this.applyUsecaseCleaning(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.construction:
                    this.applyUsecaseConstruction(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.santaClaus:
                    this.applyUsecaseSantaClaus(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.other:
                    this.applyCommonUsecaseProperties(calc);
                    break;
                case HouseholdUsecasesLogic.knownUseCases.notDefined:
                    // Nothing applied.
                    break;
            }
        }
        /**
         * Applies the constructions salary calculation usecase.
         * @param calc Calculation to which the usecase is applied.
         */
        static applyUsecaseConstruction(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc);
            switch (mainRow.rowType) {
                case exports.CalculationRowType.HourlySalary:
                case exports.CalculationRowType.MonthlySalary:
                case exports.CalculationRowType.Salary:
                    break;
                case exports.CalculationRowType.TotalEmployerPayment:
                    usecase.isTesIncludedInSalary = true;
                    usecase.isExpensesCustom = true;
                    usecase.dailyExpenses = null;
                    usecase.dailyTravelExpenses = null;
                    break;
                default:
                    mainRow.rowType = exports.CalculationRowType.Salary;
                    break;
            }
            usecase.taxDeductionCategories = exports.TaxDeductionWorkCategories.HomeImprovement;
            usecase.isTesIncludedInSalary = usecase.isTesIncludedInSalary || false;
            usecase.isExpensesCustom = usecase.isExpensesCustom || false;
            const numberOfDays = calc.framework.numberOfDays || 0;
            const totalBaseSalary = mainRow.count * mainRow.price;
            calc.rows = calc.rows.filter((row) => { var _a; return ((_a = row.data) === null || _a === void 0 ? void 0 : _a.usecase) != usecase.uri; });
            if (!usecase.isTesIncludedInSalary) {
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.HolidayCompensation, 0.185, totalBaseSalary, "Lomakorvaus");
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.TesWorktimeShortening, 0.077, totalBaseSalary, "Erillinen palkan osa");
            }
            if (usecase.isExpensesCustom) {
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.MilageOther, numberOfDays, usecase.dailyTravelExpenses, `Matkakustannusten korvaus ${usecase.dailyTravelExpenses || 0} €/päivä`);
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.ToolCompensation, numberOfDays, usecase.dailyExpenses, `Työkalukorvaus ${usecase.dailyExpenses || 0} €/päivä`);
            }
            else {
                const milage = HouseholdUsecasesLogic.raksaKmChart(usecase.dailyTravelExpensesKm);
                const toolCompensation = 1.68;
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.MilageOther, numberOfDays, milage, `Matkakustannusten korvaus ${milage || 0} €/päivä`);
                this.addUsecaseRow(calc, usecase.uri, exports.CalculationRowType.ToolCompensation, numberOfDays, toolCompensation, `Työkalukorvaus ${toolCompensation } €/päivä`);
            }
            this.applyCommonUsecaseProperties(calc);
        }
        /** Applies the childCare salary calculation usecase */
        static applyUsecaseChildCare(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc);
            // Salary settings
            if (mainRow.rowType === exports.CalculationRowType.HourlySalary) {
                mainRow.message = "Lastenhoitajan tuntipalkka";
            }
            else {
                mainRow.rowType = exports.CalculationRowType.MonthlySalary;
                mainRow.message = "Lastenhoitajan kuukausipalkka";
            }
            // Child care subsidy and tax deduction
            calc.rows = calc.rows
                .filter((x) => x.rowType !== exports.CalculationRowType.ChildCareSubsidy);
            if (usecase.isChildcareSubsidy) {
                calc.rows.push({
                    count: 1,
                    price: usecase.subsidyAmount,
                    rowType: exports.CalculationRowType.ChildCareSubsidy,
                });
            }
            else {
                usecase.subsidyAmount = null;
            }
            if (usecase.isChildcareSubsidy) {
                usecase.isHouseholdDeductible = false;
                usecase.taxDeductionCategories = null;
            }
            if (usecase.isHouseholdDeductible == null) {
                usecase.isHouseholdDeductible = true;
            }
            usecase.taxDeductionCategories = usecase.isHouseholdDeductible ? exports.TaxDeductionWorkCategories.Carework : null;
            this.applyCommonUsecaseProperties(calc);
        }
        /** Applies the cleaning salary calculation usecase */
        static applyUsecaseCleaning(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc);
            if (mainRow.rowType === exports.CalculationRowType.HourlySalary) ;
            else if (mainRow.rowType === exports.CalculationRowType.MonthlySalary) ;
            else {
                mainRow.rowType = exports.CalculationRowType.Salary;
            }
            calc.rows = calc.rows
                .filter((x) => x.rowType !== exports.CalculationRowType.HolidayCompensation);
            if (!usecase.isFullTime) {
                const salary = mainRow.count * mainRow.price;
                const percent = usecase.isContractLessThanYear ? 0.09 : 0.115;
                calc.rows.push({
                    count: percent,
                    price: salary,
                    rowType: exports.CalculationRowType.HolidayCompensation,
                });
            }
            if (usecase.isHouseholdDeductible == null) {
                usecase.isHouseholdDeductible = true;
            }
            usecase.taxDeductionCategories = exports.TaxDeductionWorkCategories.Householdwork;
            this.applyCommonUsecaseProperties(calc);
        }
        /** Applies the MLL salary calculation usecase */
        static applyUsecaseMll(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc, exports.CalculationRowType.HourlySalary);
            mainRow.rowType = exports.CalculationRowType.HourlySalary;
            mainRow.message = "Lastenhoitajan tuntipalkka";
            if (!mainRow.count || mainRow.count < 2) {
                mainRow.count = 2;
            }
            if (!usecase.useCustomPrice) {
                mainRow.price = 9.50;
            }
            calc.rows = calc.rows
                .filter((x) => x.rowType !== exports.CalculationRowType.SundayWork);
            if (usecase.isSunday) {
                calc.rows.push({
                    count: 1,
                    price: mainRow.count * mainRow.price,
                    rowType: exports.CalculationRowType.SundayWork,
                });
            }
            // Deductions
            calc.framework.isYksityisenHoidonTuki = false;
            usecase.isHouseholdDeductible = true;
            usecase.taxDeductionCategories = exports.TaxDeductionWorkCategories.Carework;
            // Defaults to Period
            if (!calc.info.workStartDate) {
                calc.info.workStartDate = Dates.getToday();
                calc.info.workEndDate = null; // workEndDate is no longer valid
                calc.framework.numberOfDays = null;
            }
            if (!calc.info.workEndDate) {
                calc.info.workEndDate = calc.info.workStartDate;
                calc.framework.numberOfDays = null;
            }
            if (!calc.framework.numberOfDays) {
                calc.framework.numberOfDays = Dates.getWorkdays(calc.info.workStartDate, calc.info.workEndDate).length;
            }
            this.applyCommonUsecaseProperties(calc);
        }
        static applyUsecaseSantaClaus(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc, exports.CalculationRowType.Salary);
            mainRow.rowType = exports.CalculationRowType.Salary;
            mainRow.message = "Joulupukin palkka";
            usecase.isHouseholdDeductible = false;
            usecase.taxDeductionCategories = null;
            const year = Dates.getMonth("today") < 6 ? Dates.getYear("today") - 1 : Dates.getYear("today");
            const nearestXmsEve = Dates.getDate(year, 12, 24);
            calc.info.workStartDate = nearestXmsEve;
            calc.info.workEndDate = nearestXmsEve;
            calc.framework.numberOfDays = 1;
            this.applyCommonUsecaseProperties(calc);
        }
        static applyCommonUsecaseProperties(calc) {
            const usecase = this.getUsecaseData(calc);
            const mainRow = CalculatorLogic.getSalaryRow(calc);
            calc.info.occupationCode = usecase.occupation;
            calc.salary.isHouseholdDeductible = usecase.isHouseholdDeductible || false;
            calc.salary.taxDeductionCategories = usecase.taxDeductionCategories;
            if (mainRow.rowType === exports.CalculationRowType.HourlySalary) {
                // For hourly salary, default to null so that users really sets the value.
                if (mainRow.count === 0) {
                    mainRow.count = null;
                }
            }
            else {
                // We currently do not support multiple months etc. in these user interfaces.
                // API does support them, but they would add unnecessary complexity to UX.
                mainRow.count = 1;
            }
            mainRow.rowType = mainRow.rowType || exports.CalculationRowType.Salary;
        }
        static translate(name, id, useCaseOrGroup) {
            useCaseOrGroup.label = Translations.get("SALAXY.ENUM." + name + "." + id.replace(/\//g, ".") + ".label") || useCaseOrGroup.label;
            useCaseOrGroup.descr = Translations.get("SALAXY.ENUM." + name + "." + id.replace(/\//g, ".") + ".description") || useCaseOrGroup.descr;
        }
        /** Adds a usecase row if price is not zero */
        static addUsecaseRow(calc, usecaseId, rowType, count, price, message) {
            if (price) {
                calc.rows.push({
                    rowType,
                    price,
                    message,
                    count,
                    data: { usecase: usecaseId }
                });
            }
        }
        /** Does the kilometers milage euros conversion */
        static raksaKmChart(km) {
            if ((km !== null && km !== void 0 ? km : 0) < 5)
                return 0;
            if (km < 10)
                return 1.94;
            if (km < 20)
                return 3.13;
            if (km < 30)
                return 5.63;
            if (km < 40)
                return 8.20;
            if (km < 50)
                return 10.10;
            if (km < 60)
                return 12.24;
            if (km < 70)
                return 16.08;
            if (km < 80)
                return 18.20;
            if (km < 90)
                return 20.68;
            if (km < 100)
                return 23.55;
            return 26.40;
        }
    }
    /**
     * Known use cases in the first implementation.
     * These implementations replace the olf Calc.Framework.Type -based solution that is not very extensible.
     * The naming is still based on the old system for easier compatibility.
     */
    HouseholdUsecasesLogic.knownUseCases = {
        /** Usecase has not been defined. Note that value is null, not "https://secure.salaxy.com/v02/api/usecases/notDefined" */
        notDefined: null,
        /** Generic child care usecase */
        childCare: "https://secure.salaxy.com/v02/api/usecases/childCare",
        /** Cleaning */
        cleaning: "https://secure.salaxy.com/v02/api/usecases/cleaning",
        /** Construction (Raksa) */
        construction: "https://secure.salaxy.com/v02/api/usecases/construction",
        /** Child care according to MLL recommendations. */
        mll: "https://secure.salaxy.com/v02/api/usecases/mll",
        /**
         * Santa Claus for Christmas campaigns: By default, this is not visible in the household UI:
         * It needs to be separately enabled for chirstmas campaigns.
         */
        santaClaus: "https://secure.salaxy.com/v02/api/usecases/santaClaus",
        /** Generic household salary payment usecase user interface. */
        other: "https://secure.salaxy.com/v02/api/usecases/other",
    };

    /** Client-side logic for BoardUsecase  */
    class BoardUsecaseLogic {
        /**
         * Updates the usecase values based on user input without going to the server.
         * @param row Row that contains the usecase data.
         */
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        update(row) {
            // Empty method stub
        }
        /** Gets validation message for BoardUsecaseLogic  */
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        getValidation(row) {
            return null;
        }
    }

    /** Client-side logic for EmploymentTerminationUsecase  */
    class EmploymentTerminationUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (codeData) {
                // TODO: check if this is the right way to set the message
                if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", row.rowType) || this.isIrIncomeTypeLabel(row.message)) {
                    row.message = codeData.label;
                }
            }
        }
        /** Gets validation message for IrIncomeTypeUsecase  */
        getValidation(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (!codeData) {
                return {
                    type: "error",
                    msg: "Tulolaji puuttuu.",
                };
            }
            return null;
        }
        getCodeData(kind) {
            if (kind) {
                let code = 0;
                switch (kind) {
                    case exports.EmploymentTerminationKind.NoticePeriodCompensation:
                        code = 208;
                        break;
                    case exports.EmploymentTerminationKind.MonetaryWorkingTimeBankCompensation:
                        code = 224;
                        break;
                    case exports.EmploymentTerminationKind.TerminationAndLayOffDamages:
                        code = 229;
                        break;
                    case exports.EmploymentTerminationKind.VoluntaryTerminationCompensation:
                        code = 231;
                        break;
                    case exports.EmploymentTerminationKind.PensionPaidByEmployer:
                        code = 338;
                        break;
                    case exports.EmploymentTerminationKind.Undefined:
                    default:
                        code = 0;
                        break;
                }
                return IncomeTypesLogic.getAll().find((x) => x.code === code);
            }
            return null;
        }
        isIrIncomeTypeLabel(label) {
            return IncomeTypesLogic.getAll().some((x) => x.label === label);
        }
    }

    /** Client-side logic for ForeclosureUsecase  */
    class ForeclosureUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         * @param calc Calculation.
         */
        update(row, calc) {
            var _a, _b, _c;
            const data = (_a = row.data) !== null && _a !== void 0 ? _a : {};
            if (!data.kind || data.kind === exports.ForeclosureKind.Undefined) {
                data.kind = exports.ForeclosureKind.Fixed;
            }
            if (data.kind !== exports.ForeclosureKind.Periodic && data.kind !== exports.ForeclosureKind.NonPeriodic) {
                return;
            }
            let salaryAfterTax = 0;
            if (calc && calc.result && calc.result.workerCalc) {
                salaryAfterTax = calc.result.workerCalc.salaryAfterTax - calc.result.workerCalc.pension - calc.result.workerCalc.unemploymentInsurance;
            }
            if (!data.protectedPortionBase) {
                const numbers = Years.getYearlyChangingNumbers((_c = (_b = calc === null || calc === void 0 ? void 0 : calc.workflow) === null || _b === void 0 ? void 0 : _b.salaryDate) !== null && _c !== void 0 ? _c : new Date());
                data.protectedPortionBase = numbers.sideCosts.protectedPortionBase;
                data.protectedPortionDependant = numbers.sideCosts.protectedPortionDependant;
            }
            let otherIncome = Numeric.round(data.incomeFromOtherSources || 0.0, 2);
            let leftToDebtor = Numeric.round(data.additionalAmountLeftToDebtor || 0.0, 2);
            let foreclosure = 0.0;
            if (salaryAfterTax > 0) {
                if (data.kind === exports.ForeclosureKind.NonPeriodic) {
                    foreclosure = salaryAfterTax / 3.0;
                }
                else {
                    data.periodLength = this.calculatePeriodLength(calc);
                    otherIncome = Numeric.round(data.periodLength * (otherIncome / 30.0), 2);
                    leftToDebtor = Numeric.round(data.periodLength * (leftToDebtor / 30.0), 2);
                    const protectedPortion = Numeric.round((data.periodLength || 0) *
                        ((data.protectedPortionBase || 0.0)
                            +
                                ((data.dependantsCount || 0) * (data.protectedPortionDependant || 0))), 2);
                    const totalSalaryAfterTax = salaryAfterTax + otherIncome;
                    if (protectedPortion > totalSalaryAfterTax) {
                        foreclosure = 0.0;
                    }
                    else if (totalSalaryAfterTax <= (protectedPortion * 2.0)) {
                        foreclosure = 2.0 * (totalSalaryAfterTax - protectedPortion) / 3.0;
                    }
                    else if (totalSalaryAfterTax <= (protectedPortion * 4.0)) {
                        foreclosure = totalSalaryAfterTax / 3.0;
                    }
                    else {
                        foreclosure = (protectedPortion * 4.0) / 3.0 +
                            4.0 * (totalSalaryAfterTax - protectedPortion * 4.0) / 5.0;
                        foreclosure = Math.min(foreclosure, totalSalaryAfterTax / 2.0);
                    }
                    foreclosure = Math.max(0.0, Math.min(foreclosure, salaryAfterTax) - leftToDebtor);
                }
            }
            row.count = 1;
            row.price = Numeric.round(foreclosure, 2);
        }
        /** Gets validation message for UnionPaymentUsecase  */
        getValidation(row) {
            const data = row.data || {};
            if (data.kind === exports.ForeclosureKind.NonPeriodic || data.kind === exports.ForeclosureKind.Periodic) {
                return {
                    type: "default",
                    msg: `Ulosottosumma päivitetään automaattisesti, työnantaja tilittää maksun itse.`,
                };
            }
            else {
                if (!row.price) {
                    return {
                        type: "error",
                        msg: "HUOM: Ulosoton summaa ei ole annettu.",
                    };
                }
            }
            return null;
        }
        calculatePeriodLength(calc) {
            const startDate = calc && calc.info && calc.info.workStartDate && calc.info.workEndDate ? calc.info.workStartDate : null;
            const endDate = calc && calc.info && calc.info.workStartDate && calc.info.workEndDate ? calc.info.workEndDate : null;
            if (startDate != null && endDate != null) {
                if (this.isOnceAMonthPeriod(startDate, endDate)) {
                    return 30;
                }
                else if (this.isEvery2WeeksPeriod(startDate, endDate)) {
                    return 14;
                }
                else if (this.isTwiceAMonthPeriod(startDate, endDate)) {
                    return 15;
                }
                else {
                    return Math.max(0, this.getDays(startDate, endDate));
                }
            }
            else {
                return 30;
            }
        }
        isOnceAMonthPeriod(startDate, endDate) {
            return endDate === Dates.asDate(Dates.getMoment(startDate).add(1, "month").subtract(1, "day"));
        }
        isTwiceAMonthPeriod(startDate, endDate) {
            return (Dates.getDuration(startDate, endDate).asDays() + 1 === 15)
                ||
                    (Dates.getMoment(startDate).date() === 15 && endDate === Dates.asDate(Dates.getMoment(startDate).endOf("month")))
                ||
                    (Dates.getMoment(startDate).date() === 16 && endDate === Dates.asDate(Dates.getMoment(startDate).endOf("month")));
        }
        isEvery2WeeksPeriod(startDate, endDate) {
            return endDate === Dates.asDate(Dates.getMoment(startDate).add(2, "week").subtract(1, "day"));
        }
        getDays(startDate, endDate) {
            return Dates.getDuration(startDate, endDate).asDays() + 1;
        }
    }

    /** Client-side logic for HourlySalaryWithWorkingTimeCompensationUsecaseLogic  */
    class HourlySalaryWithWorkingTimeCompensationUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            var _a, _b, _c, _d, _e;
            const data = (_a = row.data) !== null && _a !== void 0 ? _a : {};
            if (!data.compensationRows || data.compensationRows.length < 1) {
                data.compensationRows = [{}];
            }
            for (const cr of data.compensationRows) {
                cr.unit = !cr.unit || cr.unit === exports.CalculationRowUnit.Undefined ? exports.CalculationRowUnit.Percent : cr.unit;
                cr.count = (_b = cr.count) !== null && _b !== void 0 ? _b : 0;
                cr.price = cr.unit === exports.CalculationRowUnit.Percent ? Numeric.round(cr.count * ((_c = row.price) !== null && _c !== void 0 ? _c : 0), 2) : cr.price;
                cr.message = (_e = this.getCodeData((_d = cr.rowKind) !== null && _d !== void 0 ? _d : data.kind)) === null || _e === void 0 ? void 0 : _e.label;
            }
            if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", exports.CalculationRowType.HourlySalaryWithWorkingTimeCompensation)) {
                row.message = EnumerationsLogic.getEnumLabel("CalculationRowType", exports.CalculationRowType.HourlySalary);
            }
        }
        /** Gets validation message */
        getValidation(row) {
            var _a, _b;
            const data = row.data || {};
            if (!data.compensationRows || data.compensationRows.length < 1) {
                return;
            }
            for (const cr of data.compensationRows) {
                const codeData = this.getCodeData((_a = cr.rowKind) !== null && _a !== void 0 ? _a : data.kind);
                if (!codeData) {
                    return {
                        type: "error",
                        msg: "Korvauksen laji puuttuu.",
                    };
                }
            }
            const msgs = [];
            for (const cr of data.compensationRows) {
                const rowMsg = !cr.unit || cr.unit === exports.CalculationRowUnit.Undefined || cr.unit === exports.CalculationRowUnit.Percent ?
                    `Lisä ${Numeric.formatPercent(((_b = cr.count) !== null && _b !== void 0 ? _b : 0) * 100)}, ${Numeric.formatPrice(cr.price)} / tunti, lisä yhteensä ${Numeric.formatPrice(Numeric.round(row.count * cr.price, 2))}` :
                    `Lisä ${Numeric.formatPrice(cr.price)} / tunti, lisä yhteensä ${Numeric.formatPrice(Numeric.round(row.count * cr.price, 2))}`;
                msgs.push(rowMsg);
            }
            return { type: "default", msg: msgs.join("\n") };
        }
        getCodeData(kind) {
            if (kind) {
                let code = null;
                switch (kind) {
                    case exports.WorkingTimeCompensationKind.EmergencyWorkCompensation:
                        code = 205;
                        break;
                    case exports.WorkingTimeCompensationKind.EveningWorkCompensation:
                        code = 206;
                        break;
                    case exports.WorkingTimeCompensationKind.EveningShiftAllowance:
                        code = 207;
                        break;
                    case exports.WorkingTimeCompensationKind.SaturdayPay:
                        code = 211;
                        break;
                    case exports.WorkingTimeCompensationKind.ExtraWorkPremium:
                        code = 212;
                        break;
                    case exports.WorkingTimeCompensationKind.OtherCompensation:
                        code = 216;
                        break;
                    case exports.WorkingTimeCompensationKind.WaitingTimeCompensation:
                        code = 217;
                        break;
                    case exports.WorkingTimeCompensationKind.SundayWorkCompensation:
                        code = 221;
                        break;
                    case exports.WorkingTimeCompensationKind.StandByCompensation:
                        code = 230;
                        break;
                    case exports.WorkingTimeCompensationKind.WeeklyRestCompensation:
                        code = 232;
                        break;
                    case exports.WorkingTimeCompensationKind.OvertimeCompensation:
                        code = 235;
                        break;
                    case exports.WorkingTimeCompensationKind.NightWorkAllowance:
                        code = 236;
                        break;
                    case exports.WorkingTimeCompensationKind.NightShiftCompensation:
                        code = 237;
                        break;
                    case exports.WorkingTimeCompensationKind.Undefined:
                    default:
                        code = 0;
                        break;
                }
                return IncomeTypesLogic.getAll().find((x) => x.code === code);
            }
            return null;
        }
    }

    /** Client-side logic for IrIncomeTypeUsecase  */
    class IrIncomeTypeUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            var _a, _b, _c;
            const data = row.data;
            // Get code data by kind first (new version) and then using the old method.
            const codeData = (_b = this.getCodeDataByNumber((_a = data === null || data === void 0 ? void 0 : data.irData) === null || _a === void 0 ? void 0 : _a.code)) !== null && _b !== void 0 ? _b : this.getCodeDataByTransaction(data === null || data === void 0 ? void 0 : data.kind);
            if (codeData) {
                data.irData = (_c = data.irData) !== null && _c !== void 0 ? _c : {};
                data.irData.code = codeData.code;
                data.kind = codeData.transactionCode;
                if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", row.rowType) || this.isIrIncomeTypeLabel(row.message)) {
                    row.message = codeData.label;
                }
            }
        }
        /** Gets validation message for IrIncomeTypeUsecase  */
        getValidation(row) {
            var _a, _b;
            const data = row.data;
            const codeData = (_b = this.getCodeDataByNumber((_a = data === null || data === void 0 ? void 0 : data.irData) === null || _a === void 0 ? void 0 : _a.code)) !== null && _b !== void 0 ? _b : this.getCodeDataByTransaction(data === null || data === void 0 ? void 0 : data.kind);
            if (!codeData) {
                return {
                    type: "error",
                    msg: "Tulolaji puuttuu tai ei ole tuettu.",
                };
            }
            return null;
        }
        getCodeDataByNumber(code) {
            if (!code) {
                return null;
            }
            return IncomeTypesLogic.getAll().find((x) => x.code === code);
        }
        getCodeDataByTransaction(transactionCode) {
            if (!transactionCode ||
                transactionCode === exports.TransactionCode.Unknown ||
                transactionCode === "undefined") {
                return null;
            }
            return IncomeTypesLogic.getAll().find((x) => x.transactionCode === transactionCode);
        }
        isIrIncomeTypeLabel(label) {
            return IncomeTypesLogic.getAll().some((x) => x.label === label);
        }
    }

    /** Client-side logic for MealBenefitUsecase  */
    class MealBenefitUsecaseLogic {
        // TODO: Create interface and probably a base class that has contructor with row and the data propety casted.
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row, calc) {
            const getDate = () => {
                if (calc && calc.workflow && calc.workflow.salaryDate) {
                    return calc.workflow.salaryDate;
                }
                return new Date();
            };
            const data = row.data;
            data.taxablePrice = this.getTaxablePrice(data.kind, row.price, getDate());
            switch (data.kind) {
                case exports.MealBenefitKind.MealAllowance:
                    row.price = Years.getYearlyChangingNumbers(getDate()).sideCosts.taxFreeMealAllowance; // 10.75;
                    break;
                case exports.MealBenefitKind.CateringContract:
                case exports.MealBenefitKind.Institute:
                case exports.MealBenefitKind.RestaurantWorker:
                case exports.MealBenefitKind.Teacher:
                    row.price = this.getTaxablePrice(data.kind, 1, getDate());
                    break;
                case exports.MealBenefitKind.MealTicket:
                case exports.MealBenefitKind.TaxableAmount:
                case exports.MealBenefitKind.Undefined:
            }
        }
        /** Gets validation message for MealBenefitUsecase  */
        getValidation(row, calc) {
            const getDate = () => {
                if (calc && calc.workflow && calc.workflow.salaryDate) {
                    return calc.workflow.salaryDate;
                }
                return new Date();
            };
            const data = row.data || {};
            data.taxablePrice = this.getTaxablePrice(data.kind, row.price, getDate());
            switch (data.kind) {
                case exports.MealBenefitKind.MealAllowance:
                case exports.MealBenefitKind.CateringContract:
                case exports.MealBenefitKind.Institute:
                case exports.MealBenefitKind.MealTicket:
                case exports.MealBenefitKind.RestaurantWorker:
                case exports.MealBenefitKind.Teacher:
                case exports.MealBenefitKind.TaxableAmount:
                    return {
                        type: "default",
                        msg: `${EnumerationsLogic.getEnumLabel("MealBenefitKind", data.kind)}, verotusarvo ${Numeric.formatPrice(data.taxablePrice)}, vähennys palkasta ${Numeric.formatPrice(data.deduction || 0)}.`,
                    };
                case exports.MealBenefitKind.Undefined:
                default:
                    data.taxablePrice = row.price;
                    return {
                        type: "error",
                        msg: "HUOM: Ravintoedun tyyppiä ei ole määritetty. Oletus: verotusarvo, ei korvausta työntekijän palkasta.",
                    };
            }
        }
        /**
         * Gets the taxable price for the row.
         * https://www.vero.fi/syventavat-vero-ohjeet/paatokset/47380/verohallinnon-p%C3%A4%C3%A4t%C3%B6s-vuodelta-2020-toimitettavassa-verotuksessa-noudatettavista-luontoisetujen-laskentaperusteista/
         * @param kind meal benefit type
         * @param price Price of the row.
         */
        getTaxablePrice(kind, price, datelyObject) {
            switch (kind) {
                case exports.MealBenefitKind.MealAllowance:
                    return null;
                case exports.MealBenefitKind.CateringContract:
                    return Years.getYearlyChangingNumbers(datelyObject).sideCosts.mealBenefitCateringContract; // 6.80;
                case exports.MealBenefitKind.Institute:
                    return Years.getYearlyChangingNumbers(datelyObject).sideCosts.mealBenefitInstitute; // 5.10;
                case exports.MealBenefitKind.MealTicket:
                    return Numeric.round(0.75 * price, 3); // Calculate using one extra decimal to preserve decimals
                case exports.MealBenefitKind.RestaurantWorker:
                    return Years.getYearlyChangingNumbers(datelyObject).sideCosts.mealBenefitRestaurantWorker; // 5.78;
                case exports.MealBenefitKind.Teacher:
                    return Years.getYearlyChangingNumbers(datelyObject).sideCosts.mealBenefitTeacher; // 4.08;
                case exports.MealBenefitKind.TaxableAmount:
                    return price;
                case exports.MealBenefitKind.Undefined:
                default:
                    return price;
            }
        }
    }

    /** Client-side logic for OtherCompensationUsecase  */
    class OtherCompensationUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (codeData) {
                // TODO: check if this is the right way to set the message
                if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", row.rowType) || this.isIrIncomeTypeLabel(row.message)) {
                    row.message = codeData.label;
                }
            }
        }
        /** Gets validation message for IrIncomeTypeUsecase  */
        getValidation(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (!codeData) {
                return {
                    type: "error",
                    msg: "Tulolaji puuttuu.",
                };
            }
            return null;
        }
        getCodeData(kind) {
            if (kind) {
                let code = 0;
                switch (kind) {
                    case exports.OtherCompensationKind.MeetingFee:
                        code = 210;
                        break;
                    case exports.OtherCompensationKind.LectureFee:
                        code = 214;
                        break;
                    case exports.OtherCompensationKind.PositionOfTrustCompensation:
                        code = 215;
                        break;
                    case exports.OtherCompensationKind.AccruedTimeOffCompensation:
                        code = 225;
                        break;
                    case exports.OtherCompensationKind.MembershipOfGoverningBodyCompensation:
                        code = 308;
                        break;
                    case exports.OtherCompensationKind.MonetaryGiftForEmployees:
                        code = 310;
                        break;
                    case exports.OtherCompensationKind.UseCompensationAsEarnedIncome:
                        code = 313;
                        break;
                    case exports.OtherCompensationKind.UseCompensationAsCapitalIncome:
                        code = 314;
                        break;
                    case exports.OtherCompensationKind.OtherTaxableIncomeAsEarnedIncome:
                        code = 316;
                        break;
                    case exports.OtherCompensationKind.StockOptionsAndGrants:
                        code = 320;
                        break;
                    case exports.OtherCompensationKind.EmployeeInventionCompensation:
                        code = 326;
                        break;
                    case exports.OtherCompensationKind.CapitalIncomePayment:
                        code = 332;
                        break;
                    case exports.OtherCompensationKind.WorkEffortBasedDividendsAsWage:
                        code = 339;
                        break;
                    case exports.OtherCompensationKind.WorkEffortBasedDividendsAsNonWage:
                        code = 340;
                        break;
                    case exports.OtherCompensationKind.EmployeeStockOption:
                        code = 343;
                        break;
                    case exports.OtherCompensationKind.EmployeeStockOptionWithLowerPrice:
                        code = 361;
                        break;
                    case exports.OtherCompensationKind.Undefined:
                    default:
                        code = 0;
                        break;
                }
                return IncomeTypesLogic.getAll().find((x) => x.code === code);
            }
            return null;
        }
        isIrIncomeTypeLabel(label) {
            return IncomeTypesLogic.getAll().some((x) => x.label === label);
        }
    }

    /** Client-side logic for RemunerationUsecase  */
    class RemunerationUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (codeData) {
                // TODO: check if this is the right way to set the message
                if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", row.rowType) || this.isIrIncomeTypeLabel(row.message)) {
                    row.message = codeData.label;
                }
            }
        }
        /** Gets validation message for IrIncomeTypeUsecase  */
        getValidation(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (!codeData) {
                return {
                    type: "error",
                    msg: "Tulolaji puuttuu.",
                };
            }
            return null;
        }
        getCodeData(kind) {
            if (kind) {
                let code = 0;
                switch (kind) {
                    case exports.RemunerationKind.InitiativeFee:
                        code = 202;
                        break;
                    case exports.RemunerationKind.BonusPay:
                        code = 203;
                        break;
                    case exports.RemunerationKind.Commission:
                        code = 220;
                        break;
                    case exports.RemunerationKind.PerformanceBonus:
                        code = 223;
                        break;
                    case exports.RemunerationKind.ShareIssueForEmployees:
                        code = 226;
                        break;
                    case exports.RemunerationKind.ProfitSharingBonus:
                        code = 233;
                        break;
                    case exports.RemunerationKind.Undefined:
                    default:
                        code = 0;
                        break;
                }
                return IncomeTypesLogic.getAll().find((x) => x.code === code);
            }
            return null;
        }
        isIrIncomeTypeLabel(label) {
            return IncomeTypesLogic.getAll().some((x) => x.label === label);
        }
    }

    /** Client-side logic for UnionPaymentUsecase  */
    class UnionPaymentUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         * @param calc Calculation.
         */
        update(row, calc) {
            var _a, _b, _c, _d;
            const data = (_a = row.data) !== null && _a !== void 0 ? _a : {};
            data.kind = (data.kind || exports.UnionPaymentKind.Percentage);
            if (!data.kind || data.kind === exports.UnionPaymentKind.Undefined) {
                data.kind = exports.UnionPaymentKind.Percentage;
            }
            if (data.kind === exports.UnionPaymentKind.Other) {
                if (data.isFixedSum) {
                    data.kind = exports.UnionPaymentKind.Fixed;
                    row.price = row.price * ((_b = row.count) !== null && _b !== void 0 ? _b : 1);
                    row.count = 1;
                }
                else {
                    data.kind = exports.UnionPaymentKind.Percentage;
                }
            }
            const numbers = Years.getYearlyChangingNumbers((_d = (_c = calc === null || calc === void 0 ? void 0 : calc.workflow) === null || _c === void 0 ? void 0 : _c.salaryDate) !== null && _d !== void 0 ? _d : new Date());
            const raksaIban = "FI6115553000120981";
            const raksaName = "Rakennusliitto";
            switch (data.kind) {
                case exports.UnionPaymentKind.Fixed:
                    data.iban = data.iban === raksaIban ? null : data.iban;
                    data.recipientFullName = data.recipientFullName === raksaName ? null : data.recipientFullName;
                    row.count = 1;
                    if (calc && calc.result && calc.result.totals && row.price === calc.result.totals.totalTaxable) {
                        row.price = 0;
                    }
                    // row.price
                    break;
                case exports.UnionPaymentKind.Percentage:
                    data.iban = data.iban === raksaIban ? null : data.iban;
                    data.recipientFullName = data.recipientFullName === raksaName ? null : data.recipientFullName;
                    if (!row.count || row.count >= 1) {
                        row.count = 0;
                    }
                    if (calc && calc.result && calc.result.totals) {
                        // row.count
                        row.price = calc.result.totals.totalTaxable;
                    }
                    break;
                case exports.UnionPaymentKind.RaksaNormal:
                    data.iban = raksaIban;
                    data.recipientFullName = raksaName;
                    row.count = numbers.sideCosts.unionPaymentRaksaA;
                    if (calc && calc.result && calc.result.totals) {
                        row.price = calc.result.totals.totalTaxable;
                    }
                    break;
                case exports.UnionPaymentKind.RaksaUnemploymentOnly:
                    data.iban = raksaIban;
                    data.recipientFullName = raksaName;
                    row.count = numbers.sideCosts.unionPaymentRaksaAoTa;
                    if (calc && calc.result && calc.result.totals) {
                        row.price = calc.result.totals.totalTaxable;
                    }
                    break;
            }
        }
        /** Gets validation message for UnionPaymentUsecase  */
        getValidation(row) {
            var _a;
            const data = row.data || {};
            if (data.kind === exports.UnionPaymentKind.Other) {
                if (data.isFixedSum) {
                    data.kind = exports.UnionPaymentKind.Fixed;
                    row.price = row.price * ((_a = row.count) !== null && _a !== void 0 ? _a : 1);
                    row.count = 1;
                }
                else {
                    data.kind = exports.UnionPaymentKind.Percentage;
                }
            }
            switch (data.kind) {
                case exports.UnionPaymentKind.Fixed:
                    if (!row.price) {
                        return {
                            type: "error",
                            msg: "HUOM: Jäsenmaksua ei ole määritetty.",
                        };
                    }
                    return {
                        type: "default",
                        msg: "Kiinteä AY-jäsenmaksun määrä. Työnantaja tilittää maksun itse.",
                    };
                case exports.UnionPaymentKind.Percentage:
                    if (!row.count) {
                        return {
                            type: "error",
                            msg: "HUOM: Jäsenmaksu-%:ia ei ole määritetty.",
                        };
                    }
                    return {
                        type: "default",
                        msg: "AY-jäsenmaksu on %-osuus palkasta, palkkasumma päivitetään automaattisesti. Työnantaja tilittää maksun itse.",
                    };
                case exports.UnionPaymentKind.RaksaNormal:
                case exports.UnionPaymentKind.RaksaUnemploymentOnly:
                    if (!row.count) {
                        return {
                            type: "error",
                            msg: "HUOM: Jäsenmaksu-%:ia ei ole määritetty.",
                        };
                    }
                    return {
                        type: "default",
                        msg: "AY-jäsenmaksu on %-osuus palkasta, palkkasumma päivitetään automaattisesti. Työnantaja tilittää maksun itse.",
                    };
                default:
                    return {
                        type: "error",
                        msg: "Tuntematon AY-jäsenmaksun tyyppi.",
                    };
            }
        }
    }

    /** Client-side logic for WorkingTimeCompensationUsecase  */
    class WorkingTimeCompensationUsecaseLogic {
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (codeData) {
                // TODO: check if this is the right way to set the message
                if (!row.message || row.message === EnumerationsLogic.getEnumLabel("CalculationRowType", row.rowType) || this.isIrIncomeTypeLabel(row.message)) {
                    row.message = codeData.label;
                }
            }
        }
        /** Gets validation message for IrIncomeTypeUsecase  */
        getValidation(row) {
            const data = row.data || {};
            const codeData = this.getCodeData(data.kind);
            if (!codeData) {
                return {
                    type: "error",
                    msg: "Tulolaji puuttuu.",
                };
            }
            return null;
        }
        getCodeData(kind) {
            if (kind) {
                let code = 0;
                switch (kind) {
                    case exports.WorkingTimeCompensationKind.EmergencyWorkCompensation:
                        code = 205;
                        break;
                    case exports.WorkingTimeCompensationKind.EveningWorkCompensation:
                        code = 206;
                        break;
                    case exports.WorkingTimeCompensationKind.EveningShiftAllowance:
                        code = 207;
                        break;
                    case exports.WorkingTimeCompensationKind.SaturdayPay:
                        code = 211;
                        break;
                    case exports.WorkingTimeCompensationKind.ExtraWorkPremium:
                        code = 212;
                        break;
                    case exports.WorkingTimeCompensationKind.OtherCompensation:
                        code = 216;
                        break;
                    case exports.WorkingTimeCompensationKind.WaitingTimeCompensation:
                        code = 217;
                        break;
                    case exports.WorkingTimeCompensationKind.SundayWorkCompensation:
                        code = 221;
                        break;
                    case exports.WorkingTimeCompensationKind.StandByCompensation:
                        code = 230;
                        break;
                    case exports.WorkingTimeCompensationKind.WeeklyRestCompensation:
                        code = 232;
                        break;
                    case exports.WorkingTimeCompensationKind.OvertimeCompensation:
                        code = 235;
                        break;
                    case exports.WorkingTimeCompensationKind.NightWorkAllowance:
                        code = 236;
                        break;
                    case exports.WorkingTimeCompensationKind.NightShiftCompensation:
                        code = 237;
                        break;
                    case exports.WorkingTimeCompensationKind.Undefined:
                    default:
                        code = 0;
                        break;
                }
                return IncomeTypesLogic.getAll().find((x) => x.code === code);
            }
            return null;
        }
        isIrIncomeTypeLabel(label) {
            return IncomeTypesLogic.getAll().some((x) => x.label === label);
        }
    }

    /** Client-side logic for TotalWorkerPayment  */
    class TotalWorkerPaymentUsecaseLogic {
        /**
         * Updates the usecase values based on user input without going to the server.
         * @param row Row that contains the usecase data.
         */
        update(row) {
            const data = row.data || {};
            if (!data.kind || data.kind === exports.TotalWorkerPaymentKind.Undefined) {
                data.kind = exports.TotalWorkerPaymentKind.TotalWorkerPayment;
            }
            if (!data.rowType) {
                data.rowType = exports.CalculationRowType.Salary;
            }
        }
        /** Gets validation message for BoardUsecaseLogic  */
        getValidation(row) {
            const data = row.data || {};
            if (!data.kind || data.kind === exports.TotalWorkerPaymentKind.Undefined) {
                data.kind = exports.TotalWorkerPaymentKind.TotalWorkerPayment;
            }
            if (!data.rowType) {
                data.rowType = exports.CalculationRowType.Salary;
            }
            return {
                type: "default",
                msg: `${EnumerationsLogic.getEnumLabel("CalculationRowType", data.rowType)} lasketaan annetun summan ${Numeric.formatPrice(row.price)} perusteella. Summan sisältämät erät: ${EnumerationsLogic.getEnumLabel("TotalWorkerPaymentKind", data.kind)}.`,
            };
        }
    }

    /**
     * Provides logic for usecase user interfaces.
     */
    class UsecaseValidations {
        /**
         * Returns a validation message for a row or null if none is required.
         * @param row The row that is validated.
         * @param calc The calculation of the row.
         */
        static getValidation(row, calc) {
            if (row.price < 0) {
                if (!UsecaseValidations.canHaveNegativeRow(row.rowType)) {
                    return {
                        type: "error",
                        msg: "Hinta ei voi olla negatiivinen: Tulorekisteriin ei voi ilmoittaa miinusmerkkisiä arvoja.",
                    };
                }
            }
            if (row.count < 0) {
                return {
                    type: "error",
                    msg: "Määrä ei voi olla negatiivinen: Tulorekisteriin ei voi ilmoittaa miinusmerkkisiä arvoja.",
                };
            }
            switch (row.rowType) {
                case exports.CalculationRowType.Expenses:
                case exports.CalculationRowType.PrepaidExpenses:
                    return UsecaseValidations.getExpensesValidation(row);
                case exports.CalculationRowType.MealBenefit:
                    return new MealBenefitUsecaseLogic().getValidation(row, calc);
                case exports.CalculationRowType.DailyAllowance:
                    return UsecaseValidations.getDailyAllowanceValidation(row);
                case exports.CalculationRowType.NonProfitOrg:
                    return UsecaseValidations.getNonProfitOrgValidation(row);
                case exports.CalculationRowType.SubsidisedCommute:
                    return UsecaseValidations.getSubsidisedCommuteValidation(row);
                case exports.CalculationRowType.CarBenefit:
                    return UsecaseValidations.getCarBenefitValidation(row);
                case exports.CalculationRowType.UnionPayment:
                    return new UnionPaymentUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.IrIncomeType:
                    return new IrIncomeTypeUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.Board:
                    return new BoardUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.Remuneration:
                    return new RemunerationUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.OtherCompensation:
                    return new OtherCompensationUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.WorkingTimeCompensation:
                    return new WorkingTimeCompensationUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.EmploymentTermination:
                    return new EmploymentTerminationUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.HourlySalaryWithWorkingTimeCompensation:
                    return new HourlySalaryWithWorkingTimeCompensationUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.Foreclosure:
                    return new ForeclosureUsecaseLogic().getValidation(row);
                case exports.CalculationRowType.TotalWorkerPayment:
                    return new TotalWorkerPaymentUsecaseLogic().getValidation(row);
                default:
                    return null;
            }
        }
        /**
         * Returns true if the row type supports negative values.
         * Typically, this means that the sum of all rows must still be zero or positive.
         * @param rowType Row type to check
         */
        static canHaveNegativeRow(rowType) {
            // NOTE: This is also validated server-side, so you need to add it there as well.
            const supportsNegative = [
                // Note that there is a corresponding client-side method in UsecaseValidations.canHaveNegativeRow(), so you should update that as well.
                exports.CalculationRowType.MonthlySalary,
                exports.CalculationRowType.Salary,
                exports.CalculationRowType.Compensation,
                exports.CalculationRowType.Expenses,
            ];
            return !!supportsNegative.find((x) => x === rowType);
        }
        /** Gets validation message for DailyAllowanceUsecase  */
        static getDailyAllowanceValidation(row) {
            const data = row.data || {};
            let msg;
            switch (data.kind) {
                case exports.DailyAllowanceKind.PartialDailyAllowance:
                case exports.DailyAllowanceKind.InternationalDailyAllowance:
                case exports.DailyAllowanceKind.MealAllowance:
                    msg = EnumerationsLogic.getEnumLabel("DailyAllowanceKind", data.kind);
                    break;
                case exports.DailyAllowanceKind.Undefined:
                default:
                    msg = "Kotimaan kokopäiväraha";
                    break;
            }
            return { type: "default", msg };
        }
        /** Gets validation message for NonProfitOrgUsecase  */
        static getNonProfitOrgValidation(row) {
            const data = row.data || {};
            if (!data.kind) {
                return {
                    type: "error",
                    msg: "HUOM: Määritä korvauksen tyyppi. Oletuksena käytetään kilometrikorvausta.",
                };
            }
            return { type: "default", msg: `Yleishyödyllisen yhteisön ${EnumerationsLogic.getEnumLabel("NonProfitOrgKind", data.kind)}` };
        }
        /** Gets validation message for SubsidisedCommuteUsecase  */
        static getSubsidisedCommuteValidation(row) {
            const data = row.data || {};
            let msg;
            switch (data.kind) {
                case exports.SubsidisedCommuteKind.SingleDeduction:
                    msg = `Työsuhdematkalippu, vähennys palkasta: ${Numeric.formatPrice(data.deduction || 0)}`;
                    break;
                case exports.SubsidisedCommuteKind.PeriodicalDeduction:
                    msg = `Työsuhdematkalippu, vähennys palkasta: ${Numeric.formatPrice(data.deduction || 0)} jaettuna ${data.periodDivider || 12} osaan.`;
                    break;
                case exports.SubsidisedCommuteKind.NoDeduction:
                default:
                    msg = "Työsuhdematkalippu, ei vähennystä palkasta.";
                    break;
            }
            return { type: "default", msg };
        }
        /** Gets validation message for CarBenefitUsecase  */
        static getCarBenefitValidation(row) {
            const data = row.data || {};
            if (!data.kind || !data.ageGroup || (data.isKilometersBased && data.kilometers == null)) {
                return {
                    type: "error",
                    msg: "Pakollisia tietoja puuttuu! Autoetu lasketaan todennäköisesti väärin.",
                };
            }
            const msg = EnumerationsLogic.getEnumLabel("CarBenefitCode", data.kind)
                + ", "
                + EnumerationsLogic.getEnumLabel("AgeGroupCode", data.ageGroup)
                + (data.isKilometersBased ? `, kilometripohjainen laskenta: ${data.kilometers || ""}km` : "")
                + (data.deduction ? `. Vähennys: ${Numeric.formatPrice(data.deduction)}.` : ". Ei vähennystä palkasta.");
            return { type: "default", msg };
        }
        /** Gets validation message for Expenses  */
        static getExpensesValidation(row) {
            const data = row.data || {};
            let msg = "Arvonlisäveroa ei määritetty";
            if (data && data.accounting) {
                if (data.accounting.vatPercent === 0) {
                    msg = "Arvonlisävero 0 % - Nollaverokanta";
                }
                else if (data.accounting.vatPercent === 0.10) {
                    msg = "Arvonlisävero 10 % - Kirjat, kulttuuri, henkilökuljetus ja majoitus";
                }
                else if (data.accounting.vatPercent === 0.14) {
                    msg = "Arvonlisävero 14 % - Elintarvikkeet, ravintola- ja ateriapalvelut";
                }
                else if (data.accounting.vatPercent === 0.24) {
                    msg = "Arvonlisävero 24 % - Yleinen verokanta";
                }
                else if (!data.accounting.vatPercent) ;
                else {
                    msg = "Arvonlisävero " + (100 * data.accounting.vatPercent) + " %";
                }
            }
            return { type: "default", msg };
        }
    }

    /**
     * Provides logic for usecase user interfaces based on calculation rows.
     */
    class RowsUsecaseLogic {
        /**
         * Returns a validation message for a row or null if none is required.
         * @param row The row that is validated.
         */
        static getValidation(row, calc) {
            return UsecaseValidations.getValidation(row, calc);
        }
        /**
         * Updates the usecase property values without going to the server.
         * @param row Row to update.
         */
        static updateUsecase(row, calc) {
            // TODO: Move all these logic to usecase specific classes similar to MealBenefitUsecaseLogic.
            switch (row.rowType) {
                case exports.CalculationRowType.CarBenefit:
                    return RowsUsecaseLogic.updateCarBenefitUsecase(row);
                case exports.CalculationRowType.UnionPayment:
                    return new UnionPaymentUsecaseLogic().update(row, calc);
                case exports.CalculationRowType.DailyAllowance:
                    return RowsUsecaseLogic.updateDailyAllowanceUsecase(row, calc);
                case exports.CalculationRowType.MealBenefit:
                    return new MealBenefitUsecaseLogic().update(row, calc);
                case exports.CalculationRowType.NonProfitOrg:
                    return RowsUsecaseLogic.updateNonProfitOrgUsecase(row);
                case exports.CalculationRowType.IrIncomeType:
                    return new IrIncomeTypeUsecaseLogic().update(row);
                case exports.CalculationRowType.Board:
                    return new BoardUsecaseLogic().update(row);
                case exports.CalculationRowType.Remuneration:
                    return new RemunerationUsecaseLogic().update(row);
                case exports.CalculationRowType.OtherCompensation:
                    return new OtherCompensationUsecaseLogic().update(row);
                case exports.CalculationRowType.WorkingTimeCompensation:
                    return new WorkingTimeCompensationUsecaseLogic().update(row);
                case exports.CalculationRowType.EmploymentTermination:
                    return new EmploymentTerminationUsecaseLogic().update(row);
                case exports.CalculationRowType.HourlySalaryWithWorkingTimeCompensation:
                    return new HourlySalaryWithWorkingTimeCompensationUsecaseLogic().update(row);
                case exports.CalculationRowType.Foreclosure:
                    return new ForeclosureUsecaseLogic().update(row, calc);
                case exports.CalculationRowType.TotalWorkerPayment:
                    return new TotalWorkerPaymentUsecaseLogic().update(row);
            }
        }
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        static updateDailyAllowanceUsecase(row, calc) {
            const getDate = () => {
                if (calc && calc.workflow && calc.workflow.salaryDate) {
                    return calc.workflow.salaryDate;
                }
                return new Date();
            };
            const data = row.data;
            switch (data.kind) {
                case exports.DailyAllowanceKind.FullDailyAllowance:
                    row.price = Years.getYearlyChangingNumbers(getDate()).sideCosts.taxFreeDailyAllowance; // 43;
                    break;
                case exports.DailyAllowanceKind.PartialDailyAllowance:
                    row.price = Years.getYearlyChangingNumbers(getDate()).sideCosts.taxFreeDailyHalfAllowance; // 20;
                    break;
                case exports.DailyAllowanceKind.MealAllowance:
                    row.price = Years.getYearlyChangingNumbers(getDate()).sideCosts.taxFreeMealAllowance; // 10.75;
                    break;
                case exports.DailyAllowanceKind.InternationalDailyAllowance:
                case exports.DailyAllowanceKind.Undefined:
            }
        }
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        static updateCarBenefitUsecase(row) {
            const data = row.data;
            if (data.kind === exports.CarBenefitKind.LimitedCarBenefit) {
                row.message = "Auton käyttöetu, " + (data.ageGroup || "-").toUpperCase();
            }
            else if (data.kind === exports.CarBenefitKind.FullCarBenefit) {
                row.message = "Vapaa autoetu, " + (data.ageGroup || "-").toUpperCase();
            }
            else {
                row.message = "Autoetu, ei tyyppivalintaa, " + (data.ageGroup || "-").toUpperCase();
            }
            if (data.isKilometersBased) {
                row.message += `, ${data.kilometers}.`;
            }
        }
        /**
         * Updates the usecase values based onuser input without going to the server.
         * @param row Row that contains the usecase data.
         */
        static updateNonProfitOrgUsecase(row) {
            const data = row.data;
            switch (data.kind) {
                case exports.NonProfitOrgKind.AccomodationAllowance:
                    row.message = "Majoittumiskorvaus";
                    break;
                case exports.NonProfitOrgKind.KilometreAllowance:
                    row.message = "Kilometrikorvaus";
                    break;
                case exports.NonProfitOrgKind.DailyAllowance:
                    row.message = "Päiväraha";
                    break;
            }
        }
    }

    exports.Absences = Absences;
    exports.AccountSettings = AccountSettings;
    exports.AccountingTargets = AccountingTargets;
    exports.Accounts = Accounts;
    exports.AjaxJQuery = AjaxJQuery;
    exports.AjaxXHR = AjaxXHR;
    exports.Arrays = Arrays;
    exports.AuthorizedAccounts = AuthorizedAccounts;
    exports.Barcodes = Barcodes;
    exports.BeneficialOwnersApi = BeneficialOwnersApi;
    exports.Brand = Brand;
    exports.CalcRowsLogic = CalcRowsLogic;
    exports.CalculationResultLogic = CalculationResultLogic;
    exports.CalculationRowCategories = CalculationRowCategories;
    exports.Calculations = Calculations;
    exports.Calculator = Calculator;
    exports.CalculatorLogic = CalculatorLogic;
    exports.CalendarEvents = CalendarEvents;
    exports.CalendarOccurences = CalendarOccurences;
    exports.Certificates = Certificates;
    exports.Client = Client;
    exports.Configs = Configs;
    exports.ContractPartyLogic = ContractPartyLogic;
    exports.Cookies = Cookies;
    exports.Credentials = Credentials;
    exports.CrudApiBase = CrudApiBase;
    exports.DateTimes = DateTimes;
    exports.Dates = Dates;
    exports.EmploymentContracts = EmploymentContracts;
    exports.Employments = Employments;
    exports.EnumerationsLogic = EnumerationsLogic;
    exports.ExternalDialogData = ExternalDialogData;
    exports.Files = Files;
    exports.FormHelpers = FormHelpers;
    exports.HolidayYears = HolidayYears;
    exports.HolidaysLogic = HolidaysLogic;
    exports.HouseholdUsecasesLogic = HouseholdUsecasesLogic;
    exports.Iban = Iban;
    exports.ImportBatch = ImportBatch;
    exports.ImportBatches = ImportBatches;
    exports.ImportLogic = ImportLogic;
    exports.IncomeTypesLogic = IncomeTypesLogic;
    exports.Invoices = Invoices;
    exports.InvoicesLogic = InvoicesLogic;
    exports.IrEarningsPayments = IrEarningsPayments;
    exports.IrPayerSummaries = IrPayerSummaries;
    exports.JsonSchemaCache = JsonSchemaCache;
    exports.JsonSchemaUtils = JsonSchemaUtils;
    exports.Mapper = Mapper;
    exports.Mappers = Mappers;
    exports.MessageThreads = MessageThreads;
    exports.Numeric = Numeric;
    exports.OAuth2 = OAuth2;
    exports.OData = OData;
    exports.Objects = Objects;
    exports.Occupations = Occupations;
    exports.OnboardingLogic = OnboardingLogic;
    exports.Onboardings = Onboardings;
    exports.Overview = Overview;
    exports.PartnerServices = PartnerServices;
    exports.PaymentChannelApi = PaymentChannelApi;
    exports.PaymentLogic = PaymentLogic;
    exports.Payments = Payments;
    exports.Payrolls = Payrolls;
    exports.PrimaryPartners = PrimaryPartners;
    exports.Reports = Reports;
    exports.ReportsLogic = ReportsLogic;
    exports.RoleLogic = RoleLogic;
    exports.RowsUsecaseLogic = RowsUsecaseLogic;
    exports.SalaryDateLogic = SalaryDateLogic;
    exports.Session = Session;
    exports.SettingsLogic = SettingsLogic;
    exports.SingleItemApiBase = SingleItemApiBase;
    exports.Tables = Tables;
    exports.TaxCard2019Logic = TaxCard2019Logic;
    exports.Taxcards = Taxcards;
    exports.Test = Test;
    exports.Texts = Texts;
    exports.Token = Token;
    exports.Translations = Translations;
    exports.TypedocLogic = TypedocLogic;
    exports.Validation = Validation;
    exports.WorkerLogic = WorkerLogic;
    exports.Workers = Workers;
    exports.YearEnd = YearEnd;
    exports.Years = Years;
    exports.accountProductDefaults = accountProductDefaults;
    exports.calcRowsMetadata = calcRowsMetadata;
    exports.generatedEnumData = generatedEnumData;
    exports.incomeTypeCodes = incomeTypeCodes;
    exports.vismaSignOriginalMethods = vismaSignOriginalMethods;
    exports.yearlyChangingNumbersYears = yearlyChangingNumbersYears;
    exports.yearlyHolidaysYears = yearlyHolidaysYears;

    Object.defineProperty(exports, '__esModule', { value: true });

})));

(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@salaxy/core')) :
    typeof define === 'function' && define.amd ? define(['exports', '@salaxy/core'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.salaxy = global.salaxy || {}, global.salaxy.jquery = {}), global.salaxy.core));
}(this, (function (exports, core) { 'use strict';

    /**
     * Provides wrapper methods for communicating with the Palkkaus.fi API.
     * The raw Ajax-access to the server methods: GET, POST and DELETE
     * with different return types and authentication / error events.
     * This is the JQuery based ajax implementation.
     */
    class AjaxJQuery {
        /**
         * Creates a new instance of AjaxJQuery
         */
        constructor() {
            /**
             * By default (true) the token is set to salaxy-token -cookie.
             * Disable cookies with this flag.
             */
            this.useCookie = true;
            /**
             * By default credentials are not used in http-calls.
             * Enable credentials with this flag.
             */
            this.useCredentials = false;
            /**
             * The server address - root of the server. This is settable field.
             * Will probably be changed to a configuration object in the final version.
             */
            this.serverAddress = "https://test-api.salaxy.com";
            const config = core.Configs.current;
            if (config) {
                // apiServer
                if (config.apiServer) {
                    this.serverAddress = config.apiServer;
                }
                // useCredentials
                if (config.useCredentials != null) {
                    this.useCredentials = config.useCredentials;
                }
                // useCookie
                if (config.useCookie != null) {
                    this.useCookie = config.useCookie;
                }
            }
        }
        /** Gets the API address with version information. E.g. 'https://test-api.salaxy.com/v02/api' */
        getApiAddress() {
            return this.serverAddress + "/v02/api";
        }
        /** Gets the Server address that is used as bases to the HTML queries. E.g. 'https://test-api.salaxy.com' */
        getServerAddress() {
            return this.serverAddress;
        }
        /**
         * Gets a JSON-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        getJSON(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                dataType: "json",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Gets a HTML-message from server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result html. Standard Promise rejection to be used for error handling.
         */
        getHTML(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                dataType: "html",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * POSTS data to server and receives back a JSON-message.
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        postJSON(method, data) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getUrl(method),
                data,
                dataType: "json",
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * POSTS data to server and receives back HTML.
         *
         * @param method - The API method is the url starting from api version, e.g. '/v02/api'. E.g. '/calculator/new'
         * @param data - The data that is posted to the server.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        postHTML(method, data) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getUrl(method),
                data,
                dataType: "html",
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Sends a DELETE-message to server using the API
         *
         * @param method - The API method is the url path after the api version segments (e.g. '/v02/api')
         * and starts with a forward slash, e.g. '/calculator/new', or a full URL address.
         *
         * @returns A Promise with result data. Standard Promise rejection to be used for error handling.
         */
        remove(method) {
            const token = this.getCurrentToken();
            return this.getJQuery().ajax({
                type: "DELETE",
                url: this.getUrl(method),
                xhrFields: {
                    withCredentials: (token) ? false : this.useCredentials,
                },
                beforeSend: (request) => {
                    if (token) {
                        request.setRequestHeader("Authorization", "Bearer " + token);
                    }
                },
            });
        }
        /**
         * Gets the current token.
         * Will check the salaxy-token cookie if the token is persisted there
         */
        getCurrentToken() {
            if (!this.token && this.useCookie) {
                this.token = new core.Cookies().get("salaxy-token") || "";
            }
            return this.token;
        }
        /**
         * Sets the current token. The token is set to cookie called "salaxy-token" or
         * if the HTML page is running from local computer, it is set to local storage.
         *
         * @param token - the authentication token to store.
         */
        setCurrentToken(token) {
            if (this.useCookie) {
                new core.Cookies().setCookie("salaxy-token", token || "");
            }
            this.token = token;
        }
        /**
         * Implements the OAuth2 "Resource Owner Password Credentials Grant" flow (RFC6749 4.3).
         * This method is not typically used in production web sites - use Implicit Grant instead for client side JavScript (SPAs).
         * However, it is very useful in development, testing, trusted helpers and server-side scenarios.
         */
        resourceOwnerLogin(username, password) {
            return this.getJQuery().ajax({
                type: "POST",
                url: this.getServerAddress() + "/oauth2/token",
                data: {
                    grant_type: "password",
                    username,
                    password,
                },
                success: (data) => {
                    this.setCurrentToken(data.access_token);
                },
                dataType: "json",
            });
        }
        getJQuery() {
            const jQuery = core.Configs.global.jQuery;
            if (jQuery) {
                return jQuery;
            }
            else {
                throw new Error("ERROR: No jQuery. AjaxJQuery requires jQuery to be referenced.");
            }
        }
        /** If missing, append the API server address to the given url method string */
        getUrl(method) {
            if (!method || method.trim() === "") {
                return null;
            }
            if (method.toLowerCase().startsWith("http")) {
                return method;
            }
            if (method.toLowerCase().startsWith("/v")) {
                return this.getServerAddress() + method;
            }
            return this.getApiAddress() + method;
        }
    }

    exports.AjaxJQuery = AjaxJQuery;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
